USE [master]
GO

CREATE DATABASE [InconcertOnline] ON  PRIMARY 
( NAME = N'InconcertOnline', FILENAME = N'N:\Bases\InconcertOnline.mdf' , SIZE = 1024000KB , MAXSIZE = UNLIMITED, FILEGROWTH = 10%)
 LOG ON 
( NAME = N'InconcertOnline_log', FILENAME = N'N:\Bases\InconcertOnline_log.ldf' , SIZE = 512000KB , MAXSIZE = 2048GB , FILEGROWTH = 10%)
GO

USE [InconcertOnline]
GO

-- Verifico que se haya creado correctamente la base de datos
IF (SELECT DB_NAME()) <> 'InconcertOnline' BEGIN
    -- Si no, no sigo, para evitar que todo los siguiente se ejecute sobre la master
	RAISERROR('Ocurrió un error al crear archivos de la nueva base de datos. Se interrumpirá la ejecucuón el script.', 20, -1) WITH LOG
END

GO

DECLARE @DB VARCHAR(255)
SET @DB = DB_NAME()
EXEC dbo.sp_dbcmptlevel @DB, 130
EXEC('ALTER DATABASE [' + @DB + '] COLLATE SQL_Latin1_General_CP1_CI_AS')
EXEC('ALTER DATABASE [' + @DB + '] SET AUTO_CLOSE OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET AUTO_SHRINK OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET ALLOW_SNAPSHOT_ISOLATION OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET READ_COMMITTED_SNAPSHOT OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET RECOVERY FULL')
EXEC('ALTER DATABASE [' + @DB + '] SET PAGE_VERIFY CHECKSUM')
EXEC('ALTER DATABASE [' + @DB + '] SET AUTO_CREATE_STATISTICS ON')
EXEC('ALTER DATABASE [' + @DB + '] SET AUTO_UPDATE_STATISTICS ON')
EXEC('ALTER DATABASE [' + @DB + '] SET AUTO_UPDATE_STATISTICS_ASYNC OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET ANSI_NULL_DEFAULT OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET ANSI_NULLS OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET ANSI_PADDING OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET ANSI_WARNINGS OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET ARITHABORT OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET CONCAT_NULL_YIELDS_NULL OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET NUMERIC_ROUNDABORT OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET QUOTED_IDENTIFIER OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET RECURSIVE_TRIGGERS OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET CURSOR_CLOSE_ON_COMMIT OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET CURSOR_DEFAULT GLOBAL')
EXEC('ALTER DATABASE [' + @DB + '] SET TRUSTWORTHY OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET DB_CHAINING OFF')
EXEC('ALTER DATABASE [' + @DB + '] SET PARAMETERIZATION SIMPLE')
EXEC('ALTER DATABASE [' + @DB + '] SET DATE_CORRELATION_OPTIMIZATION OFF')
GO


GO


CREATE TABLE [dbo].[Ability] (
   [Instance] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Title] [varchar](100) NOT NULL ,
   [Detail] [varchar](500) NOT NULL ,
   [Type] [varchar](50) NULL ,
   [CalificationType] [varchar](50) NULL ,
   [ExecutionType] [varchar](25) NULL ,
   [Binary] [bit] NULL   ,
   [NotApply] [bit] NULL   ,
   [Distribute] [bit] NULL   ,
   [IsCritical] [bit] NULL   ,
   [AcceptComments] [bit] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [Enabled] [bit] NULL   ,
   [AnalysisType] [varchar](25) NULL ,
   [Query] [varchar](1000) NULL ,
   [QueryName] [varchar](50) NULL ,
   [Language] [varchar](50) NULL ,
   [IsNegative] [bit] NULL   

   ,CONSTRAINT [PK_Ability] PRIMARY KEY CLUSTERED ([Instance], [ID])
)


GO

CREATE TABLE [dbo].[AbilityEvaluationFormat] (
   [Instance] [varchar](50) NOT NULL ,
   [ID] [int] NOT NULL  
      IDENTITY (1,1) ,
   [IdEvaluationFormat] [varchar](50) NOT NULL ,
   [IdAbility] [varchar](50) NOT NULL ,
   [IdGroup] [varchar](50) NOT NULL ,
   [IdSubGroup] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [Penalizes] [varchar](3) NULL ,
   [Score] [numeric](5,2) NULL  ,
   [Order] [numeric](3,0) NULL  ,
   [Conditional] [varchar](50) NULL ,
   [HelpComment] [varchar](250) NULL 

   ,CONSTRAINT [PK_AbilityEvaluationFormat] PRIMARY KEY CLUSTERED ([Instance], [ID])
)


GO

CREATE TABLE [dbo].[AbilityQuery] (
   [ID] [varchar](50) NOT NULL ,
   [VCC] [varchar](50) NOT NULL ,
   [AbilityId] [varchar](50) NOT NULL ,
   [QueryId] [varchar](50) NOT NULL ,
   [QueryContent] [varchar](max) NULL ,
   [AcceptancePercentage] [decimal](18,2) NULL 
      CONSTRAINT [DF_AbilityQuery_AcceptancePercentage] DEFAULT ((0)) ,
   [Order] [int] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[AbilityQueryEntity] (
   [VCC] [varchar](50) NOT NULL ,
   [AbilityQueryId] [varchar](50) NOT NULL ,
   [EntityId] [varchar](50) NOT NULL ,
   [EntityName] [varchar](50) NULL ,
   [AcceptancePercentage] [decimal](18,2) NULL 
      CONSTRAINT [DF_AbilityQueryEntity_AcceptancePercentage] DEFAULT ((0)) ,
   [Order] [int] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[AbilityScoreCard] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [DegradeScoreCard] [bit] NULL   

   ,CONSTRAINT [PK__AbilityS__A6FE7C3DBDF63631] PRIMARY KEY CLUSTERED ([VCC], [ID])
)


GO

CREATE TABLE [dbo].[AdvancedAI] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](250) NULL ,
   [StatusCode] [varchar](50) NULL ,
   [CreatedByUserId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_AdvancedAI] PRIMARY KEY CLUSTERED ([VCC], [ID])
)


GO

CREATE TABLE [dbo].[AdvancedAIEngineToken] (
   [VCC] [varchar](50) NOT NULL ,
   [Token] [varchar](250) NOT NULL ,
   [CreatedDate] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[AdvancedAIExample] (
   [VCC] [varchar](50) NOT NULL ,
   [AppId] [varchar](50) NOT NULL ,
   [IntentId] [varchar](50) NOT NULL ,
   [Id] [varchar](50) NOT NULL ,
   [Text] [varchar](500) NOT NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_AdvancedAIExample] PRIMARY KEY CLUSTERED ([VCC], [AppId], [IntentId], [Id])
)


GO

CREATE TABLE [dbo].[AdvancedAIIntent] (
   [VCC] [varchar](50) NOT NULL ,
   [AppId] [varchar](50) NOT NULL ,
   [Id] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Prompt] [varchar](max) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_AdvancedAIIntent] PRIMARY KEY CLUSTERED ([VCC], [AppId], [Id])
)


GO

CREATE TABLE [dbo].[Agent] (
   [VCC] [varchar](50) NOT NULL ,
   [AgentId] [varchar](50) NOT NULL ,
   [Name] [varchar](250) NOT NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

)

CREATE  NONCLUSTERED INDEX [IX_BulkAudio] ON [dbo].[Agent] ([VCC], [AgentId])

GO

CREATE TABLE [dbo].[APIUserAllegro] (
   [VCC] [varchar](50) NOT NULL ,
   [User] [varchar](50) NULL ,
   [Password] [varchar](50) NULL 

   ,CONSTRAINT [PK_APIUserAllegro] PRIMARY KEY CLUSTERED ([VCC])
)


GO

CREATE TABLE [dbo].[APIUserOCC] (
   [VCC] [varchar](50) NOT NULL ,
   [User] [varchar](50) NULL ,
   [Password] [varchar](50) NULL 

   ,CONSTRAINT [PK_APIUserOCC] PRIMARY KEY CLUSTERED ([VCC])
)


GO

CREATE TABLE [dbo].[AsyncInteraction] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](250) NOT NULL ,
   [Folder] [varchar](50) NULL ,
   [ExportedContent] [varchar](250) NULL ,
   [Channel] [varchar](50) NULL ,
   [Campaign] [varchar](100) NULL ,
   [StartDate] [datetime] NULL   ,
   [EndDate] [datetime] NOT NULL   ,
   [Account] [varchar](250) NULL ,
   [Initiative] [varchar](50) NULL ,
   [ContactName] [varchar](300) NULL ,
   [FirstTakedAgent] [varchar](50) NULL ,
   [FirstTakedAgentName] [varchar](100) NULL ,
   [LastTakedAgent] [varchar](50) NULL ,
   [LastTakedAgentName] [varchar](100) NULL ,
   [IsTaken] [varchar](10) NULL ,
   [TotalMessages] [int] NULL   ,
   [DurationTime] [varchar](50) NULL ,
   [AttentionTime] [varchar](50) NULL ,
   [DispositionCode] [varchar](150) NULL ,
   [DispositionTreePath] [varchar](250) NULL ,
   [IsProcess] [bit] NULL   ,
   [IsCancel] [bit] NULL   ,
   [IsTranscribed] [bit] NULL   ,
   [FileName] [varchar](250) NULL ,
   [ViewFileName] [varchar](250) NULL ,
   [ErrorMessage] [varchar](max) NULL ,
   [RawChar] [int] NULL 
      CONSTRAINT [DF_Interaction_RawWords] DEFAULT ((0))  ,
   [ValidChar] [int] NULL 
      CONSTRAINT [DF_AsyncInteraction_ValidChar] DEFAULT ((0))  ,
   [ValidWords] [int] NULL 
      CONSTRAINT [DF_AsyncInteraction_ValidWords] DEFAULT ((0))  ,
   [TotalPhrases] [int] NULL 
      CONSTRAINT [DF_Interaction_ProcessedWords] DEFAULT ((0))  ,
   [CharsPerPhrase] [int] NULL   ,
   [NeedI6Update] [bit] NULL   ,
   [CreatedDate] [datetime] NULL 
      CONSTRAINT [DF_Interaction_CreatedDate] DEFAULT (getdate())  ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [ContactId] [varchar](50) NULL ,
   [ContactAddress] [varchar](300) NULL ,
   [Ender] [varchar](300) NULL ,
   [Ticket] [varchar](300) NULL 

   ,CONSTRAINT [PK_Interaction] PRIMARY KEY CLUSTERED ([VCC], [ID], [EndDate])
)

CREATE  NONCLUSTERED INDEX [BulkProgress_01] ON [dbo].[AsyncInteraction] ([VCC], [Channel], [Folder], [Campaign], [ID])
CREATE  NONCLUSTERED INDEX [GetI6AlreadyRegisteredTextInteractions_01] ON [dbo].[AsyncInteraction] ([VCC], [ID], [Folder])

GO

CREATE TABLE [dbo].[Audio] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](100) NOT NULL ,
   [Section] [numeric](5,0) NOT NULL  ,
   [Part] [numeric](5,0) NOT NULL  ,
   [Channel] [varchar](50) NULL 
      CONSTRAINT [DF_Audio_Channel] DEFAULT ('CALL'),
   [Number] [varchar](50) NULL ,
   [Gender] [varchar](50) NULL ,
   [Format] [varchar](50) NULL ,
   [SilenceTime] [numeric](12,3) NULL  ,
   [DurationTime] [numeric](12,3) NULL  ,
   [MediaInfoTime] [numeric](12,3) NULL  ,
   [DurationLinguisticTime] [numeric](12,3) NULL  ,
   [IsDiariced] [bit] NULL   ,
   [ChannelsChanged] [bit] NULL   ,
   [ValidChar] [int] NULL 
      CONSTRAINT [DF_Audio_Char] DEFAULT ((0))  ,
   [ValidWords] [int] NULL 
      CONSTRAINT [DF_Audio_Words] DEFAULT ((0))  ,
   [TotalPhrases] [int] NULL 
      CONSTRAINT [DF_Audio_TotalPhrases] DEFAULT ((0))  ,
   [CharsPerPhrase] [int] NULL 
      CONSTRAINT [DF_Audio_CharsPerPhrase] DEFAULT ((0))  ,
   [Category1] [varchar](100) NULL ,
   [Category2] [varchar](100) NULL ,
   [Category3] [varchar](100) NULL ,
   [Description] [varchar](100) NULL ,
   [Campaign] [varchar](100) NULL ,
   [Agent] [varchar](50) NULL ,
   [AgentName] [varchar](100) NULL ,
   [Type] [varchar](50) NULL ,
   [Init] [varchar](50) NULL ,
   [IsProcess] [bit] NULL   ,
   [IsCancel] [bit] NULL   ,
   [IsTranscribed] [int] NULL   ,
   [NeedI6Update] [bit] NULL   ,
   [StartDate] [datetime] NULL   ,
   [EndDate] [datetime] NULL   ,
   [DispositionCode] [varchar](50) NULL ,
   [RelativePath] [varchar](500) NULL ,
   [FileName] [varchar](200) NULL ,
   [ErrorMessage] [varchar](max) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [ContactId] [varchar](50) NULL ,
   [ContactName] [varchar](300) NULL ,
   [ContactAddress] [varchar](300) NULL ,
   [RelativeFileName] [varchar](1000) NULL ,
   [Ender] [varchar](300) NULL ,
   [Ticket] [varchar](300) NULL ,
   [SilenceInTime] [numeric](12,3) NULL  ,
   [SilenceOutTime] [numeric](12,3) NULL  

   ,CONSTRAINT [PK_Audio_1] PRIMARY KEY CLUSTERED ([VCC], [ID], [Section], [Part])
)

CREATE  NONCLUSTERED INDEX [GetI6AlreadyRegisteredCallInteractions_01] ON [dbo].[Audio] ([VCC], [ID])
CREATE  NONCLUSTERED INDEX [IX_GetProcess] ON [dbo].[Audio] ([ID], [NeedI6Update])
CREATE  NONCLUSTERED INDEX [IX_ProcessInteractionsTask] ON [dbo].[Audio] ([VCC], [IsProcess], [IsTranscribed], [IsCancel])

GO

CREATE TABLE [dbo].[AudioQuery] (
   [VCC] [varchar](50) NOT NULL ,
   [ProcessId] [varchar](50) NOT NULL ,
   [AudioId] [varchar](50) NOT NULL ,
   [Section] [numeric](5,0) NOT NULL  ,
   [Part] [numeric](5,0) NOT NULL  ,
   [QueryId] [varchar](50) NOT NULL ,
   [LastModifiedDate] [datetime] NULL   

   ,CONSTRAINT [PK_AudioQuery] PRIMARY KEY CLUSTERED ([VCC], [ProcessId], [AudioId], [Section], [Part], [QueryId])
)


GO

CREATE TABLE [dbo].[Auditor] (
   [Instance] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](100) NOT NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Auditor] PRIMARY KEY CLUSTERED ([Instance], [ID])
)


GO

CREATE TABLE [dbo].[AuditorEvaluationPlan] (
   [Instance] [varchar](50) NOT NULL ,
   [IdEvaluationPlan] [varchar](50) NOT NULL ,
   [IdAuditor] [varchar](50) NOT NULL ,
   [IdInteraction] [varchar](250) NOT NULL ,
   [Completed] [bit] NULL   ,
   [Score] [numeric](5,2) NULL  ,
   [AutomaticScore] [numeric](5,2) NULL  ,
   [ManualScore] [numeric](5,2) NULL  ,
   [IdCampaign] [varchar](100) NULL ,
   [IdAgent] [varchar](50) NOT NULL ,
   [InteractionDate] [datetime] NULL   ,
   [InteractionType] [varchar](50) NOT NULL ,
   [InteractionDuration] [numeric](8,0) NULL  ,
   [InteractionOrigin] [varchar](50) NULL ,
   [InteractionFormat] [varchar](50) NULL ,
   [InteractionDateEnd] [datetime] NULL   ,
   [ManagementResult] [varchar](500) NULL ,
   [ContactId] [varchar](100) NULL ,
   [ContactName] [varchar](300) NULL ,
   [ContactAddress] [varchar](300) NULL ,
   [EvaluationTime] [int] NULL   ,
   [Status] [bit] NULL 
      CONSTRAINT [DF_AuditorEvaluationPlan_IsDeleted] DEFAULT ((1))  ,
   [StatusDescription] [varchar](50) NULL ,
   [CreatedByUserId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_AuditorEvaluationPlan_1] PRIMARY KEY CLUSTERED ([Instance], [IdEvaluationPlan], [IdAuditor], [IdInteraction], [IdAgent], [InteractionType])
)

CREATE  NONCLUSTERED INDEX [GetEvaluationPlans_01] ON [dbo].[AuditorEvaluationPlan] ([Instance], [IdEvaluationPlan])
CREATE  NONCLUSTERED INDEX [GetEvaluationPlans_02] ON [dbo].[AuditorEvaluationPlan] ([IdInteraction])
CREATE  NONCLUSTERED INDEX [GetEvaluationPlans_03] ON [dbo].[AuditorEvaluationPlan] ([IdInteraction], [Completed])
CREATE  NONCLUSTERED INDEX [GetEvaluationPlans_04] ON [dbo].[AuditorEvaluationPlan] ([IdInteraction], [Status])
CREATE  NONCLUSTERED INDEX [GetNotEvaluatedInteractionsAuditor_01] ON [dbo].[AuditorEvaluationPlan] ([Instance], [InteractionDate], [Completed], [IdAuditor], [StatusDescription], [InteractionType])
CREATE  NONCLUSTERED INDEX [Index_IX_ReportByDetailAgent] ON [dbo].[AuditorEvaluationPlan] ([Instance], [IdEvaluationPlan], [IdInteraction], [IdAuditor])
CREATE  NONCLUSTERED INDEX [IX_AuditorEvaluationPlan] ON [dbo].[AuditorEvaluationPlan] ([Instance], [IdAuditor], [IdCampaign], [IdAgent], [InteractionType], [InteractionOrigin], [InteractionFormat], [ManagementResult])
CREATE  NONCLUSTERED INDEX [IX_AuditorEvaluationPlan_IdEvaluationPlan] ON [dbo].[AuditorEvaluationPlan] ([IdEvaluationPlan]) INCLUDE ([Instance], [IdAuditor], [IdInteraction], [IdAgent], [InteractionType])
CREATE  NONCLUSTERED INDEX [IX_ReportByDetailAgent_02] ON [dbo].[AuditorEvaluationPlan] ([Completed], [StatusDescription], [TimeStamp])

GO

CREATE TABLE [dbo].[AuthConfiguration] (
   [VCC] [varchar](50) NOT NULL ,
   [Type] [varchar](50) NOT NULL ,
   [Mode] [varchar](100) NULL ,
   [ClientId] [varchar](250) NOT NULL ,
   [SecretId] [varchar](250) NULL ,
   [CreatedDate] [datetime] NOT NULL   ,
   [CreatedByUserId] [varchar](50) NOT NULL ,
   [ModifiedDate] [datetime] NULL   ,
   [ModifiedByUserId] [varchar](50) NULL ,
   [Tiimestamp] [datetime] NOT NULL   
)


GO

CREATE TABLE [dbo].[AzureSubscription] (
   [VCC] [varchar](50) NOT NULL ,
   [AzureSubscription] [varchar](50) NOT NULL ,
   [ResourceGroup] [varchar](50) NULL ,
   [AccountName] [varchar](50) NULL 

   ,CONSTRAINT [PK_AzureSubscription] PRIMARY KEY CLUSTERED ([VCC], [AzureSubscription])
)


GO

CREATE TABLE [dbo].[AzureSubscriptionKey] (
   [VCC] [varchar](50) NOT NULL ,
   [SubscriptionKey] [varchar](50) NOT NULL ,
   [SubscriptionRegion] [varchar](50) NULL ,
   [IsDemo] [bit] NULL   ,
   [SubscriptionName] [varchar](50) NULL 

   ,CONSTRAINT [PK_AzureSubscriptionKey] PRIMARY KEY CLUSTERED ([VCC], [SubscriptionKey])
)


GO

CREATE TABLE [dbo].[Batch] (
   [ID] [varchar](100) NULL 
      CONSTRAINT [DF_Batch_ID] DEFAULT (newid()),
   [VCC] [varchar](50) NULL ,
   [BatchTypeName] [varchar](50) NOT NULL ,
   [BatchTypeId] [varchar](50) NULL ,
   [StartReported] [bit] NULL   ,
   [IsFinished] [bit] NULL 
      CONSTRAINT [DF_Batch_IsFinished] DEFAULT ((0))  ,
   [FinishReported] [bit] NULL 
      CONSTRAINT [DF_Batch_IsReported] DEFAULT ((0))  ,
   [CreatedDate] [datetime] NULL 
      CONSTRAINT [DF_Batch_CreatedDate] DEFAULT (getutcdate())  ,
   [ClosedDate] [datetime] NULL   ,
   [LastModifiedDate] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[BatchError] (
   [VCC] [varchar](50) NOT NULL ,
   [BatchType] [varchar](25) NOT NULL ,
   [BatchTypeId] [varchar](50) NOT NULL ,
   [BatchTypeName] [varchar](50) NOT NULL ,
   [Implementation] [varchar](100) NULL ,
   [InteractionId] [varchar](250) NULL ,
   [Query] [varchar](250) NULL ,
   [ErrorFunction] [varchar](100) NULL ,
   [ErrorMessage] [varchar](250) NULL ,
   [ErrorDetail] [varchar](max) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[BatchSchedule] (
   [VCC] [varchar](50) NOT NULL ,
   [Id] [varchar](50) NOT NULL ,
   [BatchType] [varchar](50) NOT NULL ,
   [Frequency] [varchar](50) NULL ,
   [Weekday] [int] NULL   ,
   [Monthday] [int] NULL   ,
   [DaysInterval] [int] NULL   ,
   [HoursInterval] [int] NULL   ,
   [DaysToExpire] [int] NULL   ,
   [StartDate] [datetime] NULL   ,
   [EndDate] [datetime] NULL   ,
   [FirstExecutionDate] [datetime] NULL   ,
   [LastExecutionDate] [datetime] NULL   ,
   [FinalExecutionDate] [datetime] NULL   ,
   [ExecutionDelay] [int] NULL 
      CONSTRAINT [DF_BatchSchedule_ExecutionDelay] DEFAULT ((0))  ,
   [BatchData] [varchar](max) NULL ,
   [ProcessFiltersId] [varchar](50) NULL ,
   [Status] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_BatchSchedule] PRIMARY KEY CLUSTERED ([VCC], [Id], [BatchType])
)


GO

CREATE TABLE [dbo].[BatchScheduleExecution] (
   [VCC] [varchar](50) NOT NULL ,
   [Id] [varchar](50) NOT NULL ,
   [BatchScheduleId] [varchar](50) NULL ,
   [BatchId] [varchar](50) NULL ,
   [ProcessId] [varchar](50) NULL ,
   [ExecutionDate] [datetime] NULL   ,
   [Order] [int] NULL   ,
   [Status] [varchar](50) NULL ,
   [CreatedByUserId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_BatchScheduleExecution] PRIMARY KEY CLUSTERED ([VCC], [Id])
)


GO

CREATE TABLE [dbo].[CampaignEvaluationFormat] (
   [Instance] [varchar](50) NOT NULL ,
   [IdEvaluationFormat] [varchar](50) NOT NULL ,
   [IdCampaign] [varchar](100) NOT NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_CampaignEvaluationFormat] PRIMARY KEY CLUSTERED ([Instance], [IdEvaluationFormat], [IdCampaign])
)


GO

CREATE TABLE [dbo].[Category] (
   [VCC] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Description] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Category_1] PRIMARY KEY CLUSTERED ([VCC], [Name])
)


GO

CREATE TABLE [dbo].[CategoryPermissions] (
   [VCC] [varchar](50) NOT NULL ,
   [Category] [varchar](50) NOT NULL ,
   [Position] [int] NULL   ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[ChangeLog] (
   [ID] [uniqueidentifier] NOT NULL 
      CONSTRAINT [DF__ChangeLog__ID__36470DEF] DEFAULT (newid())  ,
   [Instance] [varchar](50) NOT NULL ,
   [UserId] [varchar](50) NOT NULL ,
   [LogTimeStamp] [datetime] NOT NULL   ,
   [LogActionType] [char](6) NULL ,
   [LogType] [varchar](50) NOT NULL ,
   [ValueBefore] [text] NULL   ,
   [ValueAfter] [text] NULL   

   ,CONSTRAINT [CK__ChangeLog__LogAc__373B3228] CHECK  ([LogActionType]='DELETE' OR [LogActionType]='UPDATE' OR [LogActionType]='INSERT')
   ,CONSTRAINT [PK__ChangeLo__3214EC27260C2517] PRIMARY KEY CLUSTERED ([ID])
)


GO

CREATE TABLE [dbo].[CleanerSummary] (
   [Task] [varchar](15) NULL ,
   [Status] [bit] NULL   ,
   [Description] [varchar](250) NULL ,
   [CreatedAt] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[Client] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](50) NULL ,
   [Country] [varchar](50) NULL ,
   [Contact] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Client] PRIMARY KEY CLUSTERED ([VCC], [ID])
)


GO

CREATE TABLE [dbo].[CountryParser] (
   [Country] [varchar](50) NOT NULL ,
   [CountryId] [varchar](50) NULL 

   ,CONSTRAINT [PK_CountryParser] PRIMARY KEY CLUSTERED ([Country])
)


GO

CREATE TABLE [dbo].[CustomField] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](100) NOT NULL ,
   [Description] [varchar](250) NULL ,
   [Type] [varchar](50) NOT NULL ,
   [Label] [varchar](50) NULL ,
   [CreatedByUserId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL 
      CONSTRAINT [DF_CustomField_CreatedDate] DEFAULT (getutcdate())  ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[CustomFieldResult] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](100) NOT NULL ,
   [Order] [int] NULL   ,
   [CustomFieldId] [varchar](100) NULL ,
   [CreatedByUserId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[Dictionary] (
   [VCC] [varchar](50) NULL ,
   [Name] [varchar](50) NULL ,
   [Description] [varchar](250) NULL ,
   [Language] [varchar](5) NULL ,
   [IsSystemDictionary] [bit] NULL 
      CONSTRAINT [DF_Dictionary_IsSystemDictionary] DEFAULT ((0))  ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[DictionaryWord] (
   [VCC] [varchar](50) NOT NULL ,
   [DictionaryId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Word] [varchar](100) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[DictionaryWordSoundLike] (
   [VCC] [varchar](50) NULL ,
   [DictionaryWordId] [varchar](50) NULL ,
   [ID] [varchar](50) NULL ,
   [SoundLike] [varchar](100) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[Domain] (
   [VCC] [varchar](50) NOT NULL ,
   [Domain] [varchar](50) NOT NULL ,
   [Description] [varchar](200) NULL ,
   [Client] [varchar](50) NULL ,
   [HasHttps] [bit] NULL   ,
   [ContentPort] [varchar](50) NULL ,
   [Prefix] [varchar](100) NULL ,
   [IsMainDomain] [bit] NULL   ,
   [IsContentDomain] [bit] NULL   ,
   [ContentDefaultRedirect] [varchar](50) NULL ,
   [IsDistributedService] [bit] NULL   ,
   [AssociatedDistributedService] [varchar](50) NULL ,
   [CustomCode] [varchar](max) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Domain_1] PRIMARY KEY CLUSTERED ([VCC], [Domain])
)


GO

CREATE TABLE [dbo].[Entity] (
   [VCC] [varchar](50) NOT NULL ,
   [AppId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Type] [varchar](50) NOT NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [Status] [int] NULL 
      CONSTRAINT [DF__Entity__Status__4850AF91] DEFAULT ('1')  ,
   [StatusCode] [varchar](50) NULL 
      CONSTRAINT [DF__Entity__StatusCo__4944D3CA] DEFAULT ('AVAILABLE')
)


GO

CREATE TABLE [dbo].[Environment] (
   [VCC] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Type] [varchar](50) NULL ,
   [Description] [varchar](200) NULL ,
   [Bucket] [varchar](50) NULL ,
   [FTPDirectory] [varchar](max) NULL ,
   [ApiHost] [varchar](50) NULL ,
   [ApiPort] [varchar](50) NULL ,
   [ApiUser] [varchar](50) NULL ,
   [ApiPassword] [varchar](50) NULL ,
   [DbDisabled] [bit] NULL   ,
   [DbServer] [varchar](50) NULL ,
   [DbPort] [varchar](50) NULL ,
   [DbUser] [varchar](50) NULL ,
   [DbPassword] [varchar](50) NULL ,
   [PbxDisabled] [bit] NULL   ,
   [PbxServer] [varchar](50) NULL ,
   [PbxPort] [varchar](50) NULL ,
   [PbxUser] [varchar](50) NULL ,
   [PbxPassword] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Environment] PRIMARY KEY CLUSTERED ([VCC], [Name])
)


GO

CREATE TABLE [dbo].[EvaluationCustomFieldResult] (
   [VCC] [varchar](50) NOT NULL ,
   [EvaluationPlanId] [varchar](50) NOT NULL ,
   [InteractionId] [varchar](250) NOT NULL ,
   [AuditorId] [varchar](50) NOT NULL ,
   [AgentId] [varchar](50) NOT NULL ,
   [CustomFieldId] [varchar](100) NOT NULL ,
   [Result] [varchar](150) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[EvaluationFormat] (
   [Instance] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](100) NULL ,
   [type] [varchar](100) NULL ,
   [EnvironmentType] [varchar](50) NULL ,
   [ExecutionType] [varchar](25) NULL ,
   [MinAutomaticQualification] [numeric](5,2) NULL  ,
   [MinQualificationAcceptable] [numeric](5,2) NOT NULL  ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [ManagementFormURL] [varchar](1000) NULL ,
   [Enabled] [bit] NULL   ,
   [templateId] [varchar](50) NULL 

   ,CONSTRAINT [PK_EvaluationFormat] PRIMARY KEY CLUSTERED ([Instance], [ID])
)


GO

CREATE TABLE [dbo].[EvaluationFormatAbilityScoreTipification] (
   [VCC] [varchar](50) NOT NULL ,
   [EvaluationFormatId] [varchar](50) NOT NULL ,
   [ScoreId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](250) NOT NULL ,
   [Detail] [varchar](500) NULL ,
   [Order] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[EvaluationFormatAbilityScoreTipificationItem] (
   [VCC] [varchar](50) NOT NULL ,
   [EvaluationFormatId] [varchar](50) NOT NULL ,
   [ScoreId] [varchar](50) NOT NULL ,
   [TipificationId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [SecondLevelItemId] [varchar](50) NULL ,
   [Description] [varchar](250) NOT NULL ,
   [Order] [int] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[EvaluationFormatAbilityScoreTipificationQuery] (
   [VCC] [varchar](50) NOT NULL ,
   [EvaluationFormatId] [varchar](50) NOT NULL ,
   [ScoreId] [varchar](50) NOT NULL ,
   [TipificationId] [varchar](50) NOT NULL ,
   [QueryId] [varchar](50) NULL ,
   [QueryContent] [varchar](max) NULL ,
   [AcceptancePercentage] [decimal](18,2) NULL  ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[EvaluationFormatAttachedSignature] (
   [VCC] [varchar](50) NOT NULL ,
   [Description] [varchar](50) NULL ,
   [Order] [int] NULL   ,
   [EvaluationFormatId] [varchar](50) NULL ,
   [CreatedByUserId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[EvaluationFormatCustomField] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NULL ,
   [EvaluationFormatId] [varchar](50) NOT NULL ,
   [CustomFieldId] [varchar](100) NOT NULL ,
   [Order] [int] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_EvaluationFormatCustomField] PRIMARY KEY CLUSTERED ([VCC], [EvaluationFormatId], [CustomFieldId])
)


GO

CREATE TABLE [dbo].[EvaluationPlan] (
   [Instance] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Modality] [varchar](50) NOT NULL ,
   [DateFin] [datetime] NOT NULL   ,
   [IdEvaluationFormat] [varchar](50) NULL ,
   [TranscriptionLanguage] [varchar](25) NULL ,
   [EnvironmentId] [varchar](100) NULL ,
   [Description] [varchar](200) NULL ,
   [Enabled] [bit] NULL   ,
   [Total] [int] NULL   ,
   [Progress] [int] NULL   ,
   [Error] [int] NULL   ,
   [Percentage] [float] NULL   ,
   [StatusDescription] [varchar](max) NULL ,
   [Status] [numeric](2,0) NULL  ,
   [Evaluation_Mode] [varchar](1) NULL ,
   [FilterByContent] [bit] NULL   ,
   [FilterByContentPercentage] [numeric](3,0) NULL  ,
   [EvaluatePercentage] [numeric](3,0) NULL  ,
   [ProcessAndEvaluate] [bit] NULL   ,
   [FilterByContentQueryName] [varchar](50) NULL ,
   [FilterByContentQuery] [varchar](1000) NULL ,
   [EvaluationStarted] [bit] NULL   ,
   [ProcessId] [varchar](50) NULL ,
   [BatchStatus] [varchar](50) NULL ,
   [BatchScheduleId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_EvaluationPlan] PRIMARY KEY CLUSTERED ([Instance], [ID])
)

CREATE  NONCLUSTERED INDEX [IX_ReportByDetailAgent] ON [dbo].[EvaluationPlan] ([Instance], [ID])

GO

CREATE TABLE [dbo].[EvaluationResult] (
   [Instance] [varchar](50) NOT NULL ,
   [IdCampaign] [varchar](100) NULL ,
   [IdInteraction] [varchar](250) NOT NULL ,
   [IdAgent] [varchar](50) NOT NULL ,
   [IdEvaluationFormat] [varchar](50) NOT NULL ,
   [MinQualificationAcceptable] [numeric](5,2) NOT NULL  ,
   [IdGroup] [varchar](50) NULL ,
   [IdSubGroup] [varchar](50) NULL ,
   [IdAbility] [varchar](50) NOT NULL ,
   [Qualification] [numeric](5,2) NOT NULL  ,
   [OwnScore] [numeric](5,2) NULL 
      CONSTRAINT [DF_EvaluationResult_Ownscore] DEFAULT ((0)) ,
   [InheritedScore] [numeric](5,2) NULL 
      CONSTRAINT [DF_EvaluationResult_InheritedScore] DEFAULT ((0)) ,
   [IdEvaluator] [varchar](50) NOT NULL ,
   [EvaluationDate] [datetime] NULL   ,
   [MaxAbilityScore] [numeric](5,2) NOT NULL  ,
   [AbilityInnerScore] [numeric](5,2) NULL  ,
   [IdEvaluationPlan] [varchar](50) NOT NULL 
      CONSTRAINT [DF__Evaluatio__IdEva__3DE82FB7] DEFAULT ((0)),
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [Comment] [varchar](1000) NULL ,
   [NotApplies] [bit] NULL   ,
   [ScoreDescription] [varchar](100) NOT NULL 
      CONSTRAINT [DF__Evaluatio__Score__3EDC53F0] DEFAULT (''),
   [CampaignDescription] [varchar](100) NULL ,
   [AgentDescription] [varchar](100) NULL ,
   [InteractionDateini] [datetime] NULL   ,
   [InteractionDatefin] [datetime] NULL   ,
   [InteractionAddress] [varchar](200) NULL 
      CONSTRAINT [DF__Evaluatio__Inter__3FD07829] DEFAULT (''),
   [AbilityOrder] [numeric](4,0) NULL  ,
   [InteractionType] [varchar](50) NULL ,
   [IdAbilityEvaluationFormat] [varchar](50) NOT NULL ,
   [IdAuditor] [varchar](50) NOT NULL ,
   [Conditional] [varchar](50) NULL 

   ,CONSTRAINT [PK_EvaluationResult] PRIMARY KEY CLUSTERED ([Instance], [IdEvaluationPlan], [IdAuditor], [IdInteraction], [IdAgent], [IdAbilityEvaluationFormat])
)

CREATE  NONCLUSTERED INDEX [BulkEvaluationPlanAbility_01] ON [dbo].[EvaluationResult] ([Instance], [IdEvaluationPlan], [IdInteraction])
CREATE  NONCLUSTERED INDEX [BulkEvaluationPlanAbility_02] ON [dbo].[EvaluationResult] ([Instance], [IdEvaluationPlan], [IdAbility])
CREATE  NONCLUSTERED INDEX [IX_ReportByDetailAgent_01] ON [dbo].[EvaluationResult] ([Instance], [AgentDescription], [IdAuditor], [CampaignDescription])
CREATE  NONCLUSTERED INDEX [IX_ReportByDetailAgent_02] ON [dbo].[EvaluationResult] ([Instance], [IdEvaluationPlan], [IdEvaluationFormat])
CREATE  NONCLUSTERED INDEX [IX_ReportByDetailAgent_03] ON [dbo].[EvaluationResult] ([Instance], [IdEvaluationPlan], [IdAbility])
CREATE  NONCLUSTERED INDEX [IX_ReportByDetailAgent_04] ON [dbo].[EvaluationResult] ([Instance], [IdEvaluationPlan])
CREATE  NONCLUSTERED INDEX [IX_ReportByDetailAgent_05] ON [dbo].[EvaluationResult] ([Instance], [IdEvaluationPlan], [IdInteraction], [IdAuditor])
CREATE  NONCLUSTERED INDEX [IX_ReportByDetailAgent_06] ON [dbo].[EvaluationResult] ([Instance], [IdEvaluationPlan], [IdGroup])
CREATE  NONCLUSTERED INDEX [IX_ReportByDetailAgent_07] ON [dbo].[EvaluationResult] ([Instance], [IdEvaluationPlan], [IdSubGroup])
CREATE  NONCLUSTERED INDEX [ReportByDetailAgent_01] ON [dbo].[EvaluationResult] ([Instance], [AgentDescription], [IdAuditor], [CampaignDescription])

GO

CREATE TABLE [dbo].[EvaluationResultEfectivity] (
   [VCC] [varchar](50) NULL ,
   [EfectivityPlanId] [varchar](50) NULL ,
   [EvaluationPlanId] [varchar](50) NULL ,
   [InteractionId] [varchar](250) NULL ,
   [AgentId] [varchar](50) NULL ,
   [AbilityEvaluationFormatId] [varchar](50) NULL ,
   [AuditorId] [varchar](50) NULL ,
   [Efectivity] [varchar](50) NULL ,
   [EfectivityStatus] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [LastModifiedDate] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[EvaluationReview] (
   [VCC] [varchar](50) NOT NULL ,
   [EvaluationPlanId] [varchar](50) NOT NULL ,
   [InteractionId] [varchar](250) NOT NULL ,
   [SupervisorId] [varchar](50) NOT NULL ,
   [Review] [varchar](500) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedUserId] [varchar](50) NULL ,
   [Timestamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[EvaluationReviewCommitment] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL 
      CONSTRAINT [DF_EvaluationReviewCommitment_ID] DEFAULT (newid()),
   [LargeInteractionId] [varchar](255) NOT NULL ,
   [EvaluationPlanId] [varchar](50) NULL ,
   [InteractionId] [varchar](250) NULL ,
   [SupervisorId] [varchar](50) NULL ,
   [AgentId] [varchar](50) NOT NULL ,
   [Commitment] [varchar](250) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedUserId] [varchar](50) NULL ,
   [Timestamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[EvaluationTipification] (
   [VCC] [varchar](50) NOT NULL ,
   [AbilityId] [varchar](50) NOT NULL ,
   [ScoreId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [IsChecked] [bit] NULL   ,
   [Comment] [varchar](500) NULL ,
   [EvaluationPlanId] [varchar](50) NOT NULL ,
   [InteractionId] [varchar](250) NOT NULL ,
   [AuditorId] [varchar](50) NOT NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL 
)


GO

CREATE TABLE [dbo].[EvaluationTipificationItem] (
   [VCC] [varchar](50) NOT NULL ,
   [ScoreId] [varchar](50) NOT NULL ,
   [TipificationId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [IsChecked] [bit] NULL   ,
   [EvaluationPlanId] [varchar](50) NOT NULL ,
   [InteractionId] [varchar](250) NOT NULL ,
   [AuditorId] [varchar](50) NOT NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL 
)


GO

CREATE TABLE [dbo].[Exception] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](50) NOT NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Exception] PRIMARY KEY CLUSTERED ([VCC], [ID])
)


GO

CREATE TABLE [dbo].[Feeling] (
   [VCC] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Description] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Feeling_1] PRIMARY KEY CLUSTERED ([VCC], [Name])
)


GO

CREATE TABLE [dbo].[Generative] (
   [VCC] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [AppId] [varchar](50) NOT NULL ,
   [Description] [varchar](250) NULL ,
   [Language] [varchar](5) NULL ,
   [IsPublished] [bit] NULL   ,
   [NeedPublish] [bit] NULL   ,
   [LastPublishedDate] [datetime] NULL   ,
   [IsSystemApp] [bit] NULL   ,
   [StatusCode] [varchar](50) NULL ,
   [CreatedByUserId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Generative] PRIMARY KEY CLUSTERED ([VCC], [Name])
)


GO

CREATE TABLE [dbo].[GenerativeEngineToken] (
   [VCC] [varchar](50) NOT NULL ,
   [Token] [varchar](250) NOT NULL ,
   [CreatedDate] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[GenerativeEngineUsage] (
   [VCC] [varchar](50) NULL ,
   [EvaluationPlanId] [varchar](50) NULL ,
   [InteractionId] [varchar](250) NULL ,
   [AppId] [varchar](50) NULL ,
   [Tokens] [int] NULL   ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[GenerativeExample] (
   [VCC] [varchar](50) NOT NULL ,
   [AppId] [varchar](50) NOT NULL ,
   [IntentId] [varchar](50) NOT NULL ,
   [Id] [varchar](50) NOT NULL ,
   [Text] [varchar](500) NOT NULL ,
   [StatusCode] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_GenerativeExample] PRIMARY KEY CLUSTERED ([VCC], [AppId], [IntentId], [Id])
)


GO

CREATE TABLE [dbo].[GenerativeIntent] (
   [VCC] [varchar](50) NOT NULL ,
   [AppId] [varchar](50) NOT NULL ,
   [Id] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Description] [varchar](250) NULL ,
   [Color] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_GenerativeIntent] PRIMARY KEY CLUSTERED ([VCC], [Name], [Id], [AppId])
)


GO

CREATE TABLE [dbo].[GenericQuery] (
   [VCC] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Description] [varchar](500) NULL ,
   [Type] [varchar](25) NULL ,
   [Query] [varchar](max) NULL ,
   [RawData] [varchar](max) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [Version] [smallint] NULL 
      CONSTRAINT [DF__GenericQu__Versi__7834CCDD] DEFAULT ('1')  

   ,CONSTRAINT [PK_GenericQuery_1] PRIMARY KEY CLUSTERED ([VCC], [Name])
)


GO

CREATE TABLE [dbo].[GenericTemplateQuery] (
   [VCC] [varchar](50) NOT NULL ,
   [TemplateId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Query] [varchar](1000) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_GenericQuery] PRIMARY KEY CLUSTERED ([VCC], [TemplateId], [ID])
)


GO

CREATE TABLE [dbo].[Group] (
   [Instance] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](100) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [Enabled] [bit] NULL   

   ,CONSTRAINT [PK_Group] PRIMARY KEY CLUSTERED ([Instance], [ID])
)


GO

CREATE TABLE [dbo].[HistoricAbility] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Title] [varchar](100) NULL ,
   [Detail] [varchar](500) NULL ,
   [Type] [varchar](50) NULL ,
   [CalificationType] [varchar](50) NULL ,
   [ExecutionType] [varchar](25) NULL ,
   [Binary] [bit] NULL   ,
   [NotApply] [bit] NULL   ,
   [Distribute] [bit] NULL   ,
   [IsCritical] [bit] NULL   ,
   [AcceptComments] [bit] NULL   ,
   [Enabled] [bit] NULL   ,
   [AnalysisType] [varchar](25) NULL ,
   [Query] [varchar](1000) NULL ,
   [QueryName] [varchar](50) NULL ,
   [Language] [varchar](50) NULL ,
   [IsNegative] [bit] NULL   ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricAbility] ON [dbo].[HistoricAbility] ([Title], [Detail], [Type], [Query], [EvaluationPlanId], [VCC], [ID])
CREATE  NONCLUSTERED INDEX [Index_ReportByDetailAgent] ON [dbo].[HistoricAbility] ([VCC], [EvaluationPlanId], [ID])

GO

CREATE TABLE [dbo].[HistoricAbilityEvaluationFormat] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [int] NOT NULL   ,
   [IdEvaluationFormat] [varchar](50) NULL ,
   [IdAbility] [varchar](50) NULL ,
   [IdGroup] [varchar](50) NULL ,
   [IdSubGroup] [varchar](50) NULL ,
   [Penalizes] [varchar](3) NULL ,
   [Score] [numeric](5,2) NULL  ,
   [Order] [numeric](3,0) NULL  ,
   [Conditional] [varchar](50) NULL ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricAbilityEvaluationFormat] ON [dbo].[HistoricAbilityEvaluationFormat] ([IdEvaluationFormat], [IdAbility], [IdGroup], [IdSubGroup], [Score], [Order], [VCC], [ID])

GO

CREATE TABLE [dbo].[HistoricAbilityQuery] (
   [ID] [varchar](50) NOT NULL ,
   [VCC] [varchar](50) NOT NULL ,
   [AbilityId] [varchar](50) NULL ,
   [QueryId] [varchar](50) NULL ,
   [QueryContent] [varchar](max) NULL ,
   [AcceptancePercentage] [decimal](18,2) NULL 
      CONSTRAINT [DF__HistoricA__Accep__4BEC364B] DEFAULT ((0)) ,
   [Order] [int] NULL   ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricAbilityQuery] ON [dbo].[HistoricAbilityQuery] ([ID], [VCC], [AbilityId])

GO

CREATE TABLE [dbo].[HistoricAbilityQueryEntity] (
   [VCC] [varchar](50) NULL ,
   [AbilityQueryId] [varchar](50) NULL ,
   [EntityId] [varchar](50) NULL ,
   [EntityName] [varchar](50) NULL ,
   [AcceptancePercentage] [decimal](18,2) NULL 
      CONSTRAINT [DF__HistoricA__Accep__4CE05A84] DEFAULT ((0)) ,
   [Order] [int] NULL   ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricAbilityQueryEntity] ON [dbo].[HistoricAbilityQueryEntity] ([VCC], [AbilityQueryId], [EvaluationPlanId], [EntityId])

GO

CREATE TABLE [dbo].[HistoricAdvancedAI] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](250) NULL ,
   [StatusCode] [varchar](50) NULL ,
   [EvaluationPlanId] [varchar](50) NOT NULL 
)


GO

CREATE TABLE [dbo].[HistoricAdvancedAIExample] (
   [VCC] [varchar](50) NOT NULL ,
   [AppId] [varchar](50) NOT NULL ,
   [IntentId] [varchar](50) NOT NULL ,
   [Id] [varchar](50) NOT NULL ,
   [Text] [varchar](500) NOT NULL ,
   [EvaluationPlanId] [varchar](50) NOT NULL 
)


GO

CREATE TABLE [dbo].[HistoricAdvancedAIIntent] (
   [VCC] [varchar](50) NOT NULL ,
   [AppId] [varchar](50) NOT NULL ,
   [Id] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Prompt] [varchar](max) NULL ,
   [EvaluationPlanId] [varchar](50) NOT NULL 
)


GO

CREATE TABLE [dbo].[HistoricCampaignEvaluationFormat] (
   [VCC] [varchar](50) NOT NULL ,
   [IdEvaluationFormat] [varchar](50) NOT NULL ,
   [IdCampaign] [varchar](100) NULL ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricCampaignEvaluationFormat] ON [dbo].[HistoricCampaignEvaluationFormat] ([VCC], [IdEvaluationFormat], [IdCampaign])

GO

CREATE TABLE [dbo].[HistoricCustomField] (
   [VCC] [varchar](50) NULL ,
   [ID] [varchar](100) NULL ,
   [Description] [varchar](250) NULL ,
   [Type] [varchar](50) NULL ,
   [Label] [varchar](50) NULL ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricCustomField] ON [dbo].[HistoricCustomField] ([VCC], [ID], [Description])

GO

CREATE TABLE [dbo].[HistoricCustomFieldResult] (
   [VCC] [varchar](50) NULL ,
   [ID] [varchar](50) NULL ,
   [Description] [varchar](100) NULL ,
   [Order] [int] NULL   ,
   [CustomFieldId] [varchar](100) NULL ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricCustomFieldResult] ON [dbo].[HistoricCustomFieldResult] ([VCC], [ID], [Description], [CustomFieldId])

GO

CREATE TABLE [dbo].[HistoricEvaluationFormat] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](100) NULL ,
   [Type] [varchar](100) NULL ,
   [EnvironmentType] [varchar](100) NULL ,
   [ExecutionType] [varchar](25) NULL ,
   [MinAutomaticQualification] [numeric](5,2) NULL  ,
   [MinQualificationAcceptable] [numeric](5,2) NULL  ,
   [ManagementFormURL] [varchar](1000) NULL ,
   [Enabled] [bit] NULL   ,
   [TemplateId] [varchar](50) NULL ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricEvaluationFormat] ON [dbo].[HistoricEvaluationFormat] ([Description], [Type], [VCC], [ID])
CREATE  NONCLUSTERED INDEX [Index_ReportByDetailAgent] ON [dbo].[HistoricEvaluationFormat] ([VCC], [EvaluationPlanId], [ID])

GO

CREATE TABLE [dbo].[HistoricEvaluationFormatAbilityScoreTipification] (
   [VCC] [varchar](50) NOT NULL ,
   [EvaluationFormatId] [varchar](50) NOT NULL ,
   [ScoreId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](250) NOT NULL ,
   [Detail] [varchar](500) NULL ,
   [Order] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [EvaluationPlanId] [varchar](50) NOT NULL 
)


GO

CREATE TABLE [dbo].[HistoricEvaluationFormatAbilityScoreTipificationItem] (
   [VCC] [varchar](50) NOT NULL ,
   [EvaluationFormatId] [varchar](50) NOT NULL ,
   [ScoreId] [varchar](50) NOT NULL ,
   [TipificationId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [SecondLevelItemId] [varchar](50) NULL ,
   [Description] [varchar](250) NULL ,
   [Order] [int] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [EvaluationPlanId] [varchar](50) NOT NULL 
)


GO

CREATE TABLE [dbo].[HistoricEvaluationFormatAbilityScoreTipificationQuery] (
   [VCC] [varchar](50) NOT NULL ,
   [EvaluationFormatId] [varchar](50) NOT NULL ,
   [ScoreId] [varchar](50) NOT NULL ,
   [TipificationId] [varchar](50) NOT NULL ,
   [QueryId] [varchar](50) NULL ,
   [QueryContent] [varchar](max) NULL ,
   [AcceptancePercentage] [decimal](18,2) NULL  ,
   [CreatedDare] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [EvaluationPlanId] [varchar](50) NOT NULL 
)


GO

CREATE TABLE [dbo].[HistoricEvaluationFormatAttachedSignature] (
   [VCC] [varchar](50) NULL ,
   [Description] [varchar](50) NULL ,
   [Order] [int] NULL   ,
   [EvaluationFormatId] [varchar](50) NULL ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricEvaluationFormatAttachedSignature] ON [dbo].[HistoricEvaluationFormatAttachedSignature] ([VCC], [EvaluationFormatId], [Description], [EvaluationPlanId])

GO

CREATE TABLE [dbo].[HistoricEvaluationFormatCustomField] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NULL ,
   [EvaluationFormatId] [varchar](50) NOT NULL ,
   [CustomFieldId] [varchar](100) NOT NULL ,
   [Order] [int] NULL   ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricEvaluationFormatCustomField] ON [dbo].[HistoricEvaluationFormatCustomField] ([VCC], [EvaluationFormatId], [CustomFieldId])

GO

CREATE TABLE [dbo].[HistoricGenericTemplateQuery] (
   [VCC] [varchar](50) NOT NULL ,
   [TemplateId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Query] [varchar](max) NULL ,
   [StudyId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricGenericTemplateQuery] ON [dbo].[HistoricGenericTemplateQuery] ([StudyId], [VCC], [TemplateId], [ID])

GO

CREATE TABLE [dbo].[HistoricGroup] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](100) NULL ,
   [Enabled] [bit] NULL   ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricGroup] ON [dbo].[HistoricGroup] ([Description], [EvaluationPlanId], [VCC], [ID])
CREATE  NONCLUSTERED INDEX [Index_ReportByDetailAgent] ON [dbo].[HistoricGroup] ([VCC], [EvaluationPlanId], [ID])

GO

CREATE TABLE [dbo].[HistoricScore] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [IdAbility] [varchar](50) NULL ,
   [Description] [varchar](100) NULL ,
   [Value] [numeric](5,1) NULL  ,
   [EvaluationPlanId] [varchar](50) NOT NULL 
)


GO

CREATE TABLE [dbo].[HistoricSubGroup] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](100) NULL ,
   [Enabled] [bit] NULL   ,
   [EvaluationPlanId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricSubGroup] ON [dbo].[HistoricSubGroup] ([Description], [EvaluationPlanId], [VCC], [ID])
CREATE  NONCLUSTERED INDEX [Index_ReportByDetailAgent] ON [dbo].[HistoricSubGroup] ([VCC], [EvaluationPlanId], [ID])

GO

CREATE TABLE [dbo].[HistoricTemplate] (
   [VCC] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Type] [varchar](50) NULL ,
   [Description] [varchar](50) NULL ,
   [Template] [varchar](50) NULL ,
   [Category] [varchar](50) NULL ,
   [StudyId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricTemplate] ON [dbo].[HistoricTemplate] ([Type], [Description], [StudyId], [VCC], [Name])

GO

CREATE TABLE [dbo].[HistoricTemplateTheme] (
   [VCC] [varchar](50) NOT NULL ,
   [TemplateId] [varchar](50) NOT NULL ,
   [ID] [varchar](200) NOT NULL ,
   [Order] [int] NULL   ,
   [Description] [varchar](50) NULL ,
   [Category1] [varchar](50) NULL ,
   [Category2] [varchar](50) NULL ,
   [AnalysisType] [varchar](15) NULL ,
   [IsNegative] [bit] NULL 
      CONSTRAINT [DF__HistoricT__IsNeg__4DD47EBD] DEFAULT ((0))  ,
   [StudyId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricTemplateTheme] ON [dbo].[HistoricTemplateTheme] ([StudyId], [VCC], [TemplateId], [ID])

GO

CREATE TABLE [dbo].[HistoricTemplateThemeItem] (
   [VCC] [varchar](50) NOT NULL ,
   [TemplateId] [varchar](50) NOT NULL ,
   [TemplateThemeId] [varchar](200) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [AcceptancePercentage] [decimal](5,2) NULL 
      CONSTRAINT [DF__HistoricT__Accep__4EC8A2F6] DEFAULT ((0)) ,
   [StudyId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricTemplateThemeItem] ON [dbo].[HistoricTemplateThemeItem] ([StudyId], [VCC], [TemplateId], [TemplateThemeId], [ID])

GO

CREATE TABLE [dbo].[HistoricTemplateThemeItemEntity] (
   [VCC] [varchar](50) NULL ,
   [TemplateId] [varchar](50) NULL ,
   [TemplateThemeId] [varchar](50) NULL ,
   [TemplateThemeItemId] [varchar](50) NULL ,
   [EntityId] [varchar](50) NULL ,
   [EntityName] [varchar](50) NULL ,
   [AcceptancePercentage] [decimal](18,2) NULL  ,
   [Order] [int] NULL   ,
   [StudyId] [varchar](50) NULL 

)

CREATE  NONCLUSTERED INDEX [Index_HistoricTemplateThemeItemEntity] ON [dbo].[HistoricTemplateThemeItemEntity] ([TemplateThemeId], [TemplateThemeItemId], [EntityId], [EntityName], [StudyId])

GO

CREATE TABLE [dbo].[IdathaAuth] (
   [VCC] [varchar](50) NOT NULL ,
   [Realm] [varchar](100) NOT NULL ,
   [UserName] [varchar](50) NOT NULL ,
   [Password] [varchar](200) NOT NULL 

   ,CONSTRAINT [PK_IdathaAuth] PRIMARY KEY CLUSTERED ([VCC], [Realm])
)


GO

CREATE TABLE [dbo].[Intent] (
   [VCC] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [IntentId] [varchar](50) NOT NULL ,
   [AppId] [varchar](50) NOT NULL ,
   [Color] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [Status] [int] NULL 
      CONSTRAINT [DF__Intent__Status__4A38F803] DEFAULT ('1')  ,
   [StatusCode] [varchar](50) NULL 
      CONSTRAINT [DF__Intent__StatusCo__4B2D1C3C] DEFAULT ('AVAILABLE')

   ,CONSTRAINT [PK_Intent_1] PRIMARY KEY CLUSTERED ([VCC], [Name], [IntentId], [AppId])
)


GO

CREATE TABLE [dbo].[InteractionEvaluationPlan] (
   [Instance] [varchar](50) NOT NULL ,
   [IdEvaluationPlan] [varchar](50) NOT NULL ,
   [InteractionType] [varchar](25) NOT NULL ,
   [FormattedEndDate] [varchar](50) NULL ,
   [IdCampaign] [varchar](100) NULL ,
   [IdInteraction] [varchar](250) NOT NULL ,
   [InteractionSection] [numeric](5,0) NULL  ,
   [InteractionPart] [numeric](5,0) NULL  ,
   [IdAgent] [varchar](50) NULL ,
   [AgentName] [varchar](100) NULL ,
   [InteractionMessages] [int] NULL   ,
   [InteractionDateStart] [datetime] NULL   ,
   [InteractionDateEnd] [datetime] NULL 
      CONSTRAINT [DF_InteractionEvaluationPlan_InteractionDateEnd] DEFAULT ((0))  ,
   [InteractionDuration] [numeric](8,0) NULL 
      CONSTRAINT [DF_InteractionEvaluationPlan_InteractionDuration] DEFAULT ((0)) ,
   [InteractionOrigin] [varchar](10) NULL ,
   [InteractionFormat] [varchar](5) NULL ,
   [ManagementResult] [varchar](250) NULL ,
   [ExportedContent] [varchar](250) NULL ,
   [RelativePath] [varchar](500) NULL ,
   [FileName] [varchar](200) NULL ,
   [ContactId] [varchar](100) NULL ,
   [ContactName] [varchar](300) NULL ,
   [ContactAddress] [varchar](300) NULL ,
   [QueryPassed] [bit] NULL   ,
   [NeedI6Update] [bit] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [TimeStamp] [datetime] NULL   ,
   [RelativeFileName] [varchar](1000) NULL ,
   [Ender] [varchar](300) NULL ,
   [Ticket] [varchar](300) NULL 
)


GO

CREATE TABLE [dbo].[InteractionQueryResult] (
   [VCC] [varchar](50) NOT NULL ,
   [InteractionId] [varchar](250) NOT NULL ,
   [Section] [int] NULL   ,
   [Part] [int] NULL   ,
   [TranscriptionId] [varchar](250) NULL ,
   [InteractionType] [varchar](10) NULL ,
   [BatchType] [varchar](50) NOT NULL ,
   [BatchTypeId] [varchar](50) NOT NULL ,
   [BatchTypeName] [varchar](50) NOT NULL ,
   [AnalysisType] [varchar](25) NULL ,
   [QueryId] [varchar](50) NOT NULL ,
   [QueryName] [varchar](50) NOT NULL ,
   [QueryValue] [varchar](max) NOT NULL ,
   [QueryData] [varchar](max) NULL ,
   [Matches] [int] NULL   ,
   [Details] [varchar](max) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [TimeStamp] [datetime] NULL   

)

CREATE  NONCLUSTERED INDEX [IX_InteractionQueryResult_01] ON [dbo].[InteractionQueryResult] ([BatchTypeName], [InteractionId])
CREATE  NONCLUSTERED INDEX [IX_InteractionQueryResult_BatchType] ON [dbo].[InteractionQueryResult] ([BatchType])
CREATE  NONCLUSTERED INDEX [IX_InteractionQueryResult_BatchTypeId] ON [dbo].[InteractionQueryResult] ([BatchTypeId])

GO

CREATE TABLE [dbo].[InteractionType] (
   [Instance] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](100) NULL 

   ,CONSTRAINT [PK_InteractionType] PRIMARY KEY CLUSTERED ([Instance], [ID])
)


GO

CREATE TABLE [dbo].[NeuroLinguisticProgramming] (
   [VCC] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Description] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_NeuroLinguisticProgramming_1] PRIMARY KEY CLUSTERED ([VCC], [Name])
)


GO

CREATE TABLE [dbo].[NLU] (
   [VCC] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Culture] [varchar](50) NULL ,
   [Description] [varchar](50) NULL ,
   [AppId] [varchar](100) NULL ,
   [SubscriptionKey] [varchar](50) NULL ,
   [IsPublished] [bit] NULL   ,
   [LastPublishedDate] [datetime] NULL   ,
   [IsTrained] [bit] NULL   ,
   [NeedTrain] [bit] NULL 
      CONSTRAINT [DF_NLU_NeedTrain] DEFAULT ((0))  ,
   [LastTrainedDate] [datetime] NULL   ,
   [IsSystemApp] [bit] NULL 
      CONSTRAINT [DF_NLU_IsSystemApp] DEFAULT ((0))  ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [CognitiveEngine] [varchar](50) NULL ,
   [Status] [smallint] NULL 
      CONSTRAINT [DF__NLU__Status__11F49EE0] DEFAULT ('1')  ,
   [StatusCode] [varchar](50) NULL 
      CONSTRAINT [DF__NLU__StatusCode__12E8C319] DEFAULT ('AVAILABLE'),
   [NeedPublish] [bit] NULL   ,
   [TrainVersion] [varchar](250) NULL ,
   [TrainId] [varchar](250) NULL 

   ,CONSTRAINT [PK_NLU] PRIMARY KEY CLUSTERED ([VCC], [Name])
)


GO

CREATE TABLE [dbo].[Process] (
   [VCC] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Description] [varchar](50) NULL ,
   [ClientId] [varchar](50) NULL ,
   [Total] [int] NULL   ,
   [Progress] [int] NULL   ,
   [Error] [int] NULL   ,
   [Percentage] [float] NULL   ,
   [StatusDescription] [varchar](max) NULL ,
   [Status] [varchar](50) NULL ,
   [Type] [varchar](50) NULL ,
   [StartDate] [datetime] NULL   ,
   [EndDate] [datetime] NULL   ,
   [MinDurationTime] [numeric](18,2) NULL  ,
   [MaxDurationTime] [numeric](18,2) NULL  ,
   [Language] [varchar](25) NULL ,
   [DictionaryId] [varchar](50) NULL ,
   [Environment] [varchar](50) NULL ,
   [QueryId] [varchar](50) NULL ,
   [QueryDataId] [varchar](50) NULL ,
   [QueryDataIdDate] [datetime] NULL   ,
   [BatchScheduleId] [varchar](50) NULL 
      CONSTRAINT [DF_Process_IsScheduled] DEFAULT ((0)),
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [Retranscription] [bit] NULL   ,
   [Diarization] [bit] NULL   ,
   [ChannelChange] [bit] NULL   

   ,CONSTRAINT [PK_Process_1] PRIMARY KEY CLUSTERED ([VCC], [Name])
)


GO

CREATE TABLE [dbo].[ProcessAsyncInteraction] (
   [VCC] [varchar](50) NOT NULL ,
   [ProcessId] [varchar](50) NOT NULL ,
   [InteractionType] [varchar](50) NOT NULL ,
   [Campaign] [varchar](100) NULL ,
   [InteractionId] [varchar](250) NOT NULL ,
   [EndDate] [datetime] NOT NULL   ,
   [FormattedEndDate] [varchar](100) NULL ,
   [CreatedDate] [datetime] NULL 
      CONSTRAINT [DF_ProcessAsyncInteraction_CreatedDate] DEFAULT (getdate())  ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_ProcessAsyncInteraction_1] PRIMARY KEY CLUSTERED ([VCC], [ProcessId], [InteractionType], [InteractionId], [EndDate])
)

CREATE  NONCLUSTERED INDEX [IX_BulkEvaluationPlanAbility_01] ON [dbo].[ProcessAsyncInteraction] ([VCC], [ProcessId], [InteractionType], [FormattedEndDate], [Campaign], [InteractionId])

GO

CREATE TABLE [dbo].[ProcessAudio] (
   [VCC] [varchar](50) NOT NULL ,
   [ProcessId] [varchar](50) NOT NULL ,
   [AudioId] [varchar](100) NOT NULL ,
   [Section] [numeric](5,0) NOT NULL  ,
   [Part] [numeric](5,0) NOT NULL  ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_ProcessAudio] PRIMARY KEY CLUSTERED ([VCC], [ProcessId], [AudioId], [Section], [Part])
)

CREATE  NONCLUSTERED INDEX [IX_BulkProgress_01] ON [dbo].[ProcessAudio] ([VCC], [AudioId], [Section], [Part])
CREATE  NONCLUSTERED INDEX [IX_BulkProgress_02] ON [dbo].[ProcessAudio] ([VCC], [ProcessId])
CREATE  NONCLUSTERED INDEX [IX_ProcessAudio_AudioId_Section_Part] ON [dbo].[ProcessAudio] ([AudioId], [Section], [Part]) INCLUDE ([ProcessId])

GO

CREATE TABLE [dbo].[ProcessAudioSummary] (
   [VCC] [varchar](50) NOT NULL ,
   [ProcessId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL 
      CONSTRAINT [DF_ProcessAudioSummary_ID] DEFAULT (newid()),
   [Category1] [varchar](100) NULL ,
   [Category2] [varchar](100) NULL ,
   [Category3] [varchar](100) NULL ,
   [Description] [varchar](100) NULL ,
   [Total] [int] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_ProcessAudioSummary] PRIMARY KEY NONCLUSTERED ([VCC], [ProcessId], [ID])
)


GO

CREATE TABLE [dbo].[ProcessFilter] (
   [VCC] [varchar](50) NOT NULL ,
   [ProcessId] [varchar](50) NOT NULL ,
   [Key] [varchar](50) NOT NULL ,
   [Value] [varchar](250) NOT NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_ProcessFilter] PRIMARY KEY CLUSTERED ([VCC], [ProcessId], [Key], [Value])
)


GO

CREATE TABLE [dbo].[ProcessSummary] (
   [VCC] [varchar](50) NOT NULL ,
   [ProcessId] [varchar](50) NOT NULL ,
   [SilenceTime] [varchar](50) NULL ,
   [AverageSilenceTime] [varchar](50) NULL ,
   [DurationTime] [varchar](50) NULL ,
   [AverageDurationTime] [varchar](50) NULL ,
   [DurationTimeRate] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_ProcessSummary] PRIMARY KEY CLUSTERED ([VCC], [ProcessId])
)


GO

CREATE TABLE [dbo].[ProfileImage] (
   [VCC] [varchar](50) NOT NULL ,
   [Type] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [ContentType] [varchar](50) NULL ,
   [ContentData] [varchar](max) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK__ProfileImage] PRIMARY KEY CLUSTERED ([VCC], [Type], [ID])
)


GO

CREATE TABLE [dbo].[Progress] (
   [Key] [varchar](300) NULL ,
   [VCC] [varchar](50) NULL ,
   [ProcessId] [varchar](50) NULL ,
   [Type] [varchar](50) NULL ,
   [TypeId] [varchar](50) NULL ,
   [Total] [int] NULL   ,
   [Progress] [int] NULL   ,
   [Error] [int] NULL   ,
   [Percentage] [float] NULL   ,
   [StatusDescription] [varchar](max) NULL ,
   [TimeStamp] [datetime] NULL 
      CONSTRAINT [DF_Progress_TimeStamp] DEFAULT (getutcdate())  
)


GO

CREATE TABLE [dbo].[RealtimeEvaluationPlan] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [EvaluationFormatId] [varchar](50) NOT NULL ,
   [CampaignId] [varchar](100) NOT NULL ,
   [IsActive] [bit] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

)

CREATE  NONCLUSTERED INDEX [IX_RealtimeEvaluationPlan] ON [dbo].[RealtimeEvaluationPlan] ([VCC], [EvaluationFormatId], [CampaignId], [IsActive])

GO

CREATE TABLE [dbo].[Report] (
   [VCC] [varchar](100) NOT NULL ,
   [ID] [varchar](100) NOT NULL ,
   [Description] [varchar](100) NULL ,
   [Procedure] [varchar](50) NULL ,
   [StartDate] [datetime] NULL   ,
   [EndDate] [datetime] NULL   ,
   [Frecuency] [varchar](50) NULL ,
   [HourSpace] [int] NULL   ,
   [ExactTime] [varchar](50) NULL ,
   [Weekday] [varchar](50) NULL ,
   [DayMonth] [varchar](50) NULL ,
   [Summary] [varchar](300) NULL ,
   [Filter] [varchar](1000) NULL ,
   [Email] [varchar](1000) NULL ,
   [Type] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK__Report] PRIMARY KEY CLUSTERED ([VCC], [ID])
)


GO

CREATE TABLE [dbo].[ReportFieldAttribute] (
   [Field] [varchar](50) NULL ,
   [Label] [varchar](50) NULL ,
   [IsDisabled] [bit] NULL 
      CONSTRAINT [DF__ReportFie__IsDis__1590259A] DEFAULT ((0))  ,
   [IsDropdown] [bit] NULL 
      CONSTRAINT [DF__ReportFie__IsDro__168449D3] DEFAULT ((0))  ,
   [Procedure] [varchar](100) NULL ,
   [Arguments] [varchar](max) NULL ,
   [Variables] [varchar](max) NULL ,
   [CreatedAt] [datetime] NULL 
      CONSTRAINT [DF__ReportFie__Creat__17786E0C] DEFAULT (getutcdate())  ,
   [IsMultiple] [bit] NULL 
      CONSTRAINT [DF__ReportFie__IsMul__1D314762] DEFAULT ('0')  
)


GO

CREATE TABLE [dbo].[Role] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](50) NULL ,
   [SystemRole] [bit] NULL 
      CONSTRAINT [DF_Role_SystemRole] DEFAULT ((0))  ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [Type] [varchar](50) NULL 

   ,CONSTRAINT [PK_Role] PRIMARY KEY CLUSTERED ([VCC], [ID])
)


GO

CREATE TABLE [dbo].[RolePermission] (
   [VCC] [varchar](50) NOT NULL ,
   [RoleId] [varchar](50) NOT NULL ,
   [GroupId] [varchar](200) NOT NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_RolePermission] PRIMARY KEY CLUSTERED ([VCC], [RoleId], [GroupId])
)


GO

CREATE TABLE [dbo].[RolePermissionsGroup] (
   [VCC] [varchar](50) NOT NULL ,
   [GroupId] [varchar](200) NOT NULL ,
   [Category] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [Type] [varchar](50) NULL 

   ,CONSTRAINT [PK_RolePermissionsGroup] PRIMARY KEY CLUSTERED ([VCC], [GroupId])
)


GO

CREATE TABLE [dbo].[RoutePermission] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [GroupId] [varchar](200) NULL ,
   [Route] [varchar](250) NULL ,
   [Level] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_RoutePermission] PRIMARY KEY CLUSTERED ([VCC], [ID])
)


GO

CREATE TABLE [dbo].[Score] (
   [Instance] [varchar](50) NOT NULL ,
   [ID] [int] NOT NULL  
      IDENTITY (1,1) ,
   [IdAbility] [varchar](50) NOT NULL ,
   [Description] [varchar](100) NOT NULL ,
   [Value] [numeric](5,1) NOT NULL  ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Score] PRIMARY KEY CLUSTERED ([Instance], [ID])
)


GO

CREATE TABLE [dbo].[ScoreTipification] (
   [VCC] [varchar](50) NOT NULL ,
   [ScoreId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](250) NOT NULL ,
   [Detail] [varchar](500) NULL ,
   [Order] [int] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[ScoreTipificationItem] (
   [VCC] [varchar](50) NOT NULL ,
   [ScoreId] [varchar](50) NOT NULL ,
   [TipificationId] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [SecondLevelItemId] [varchar](50) NULL ,
   [Description] [varchar](250) NOT NULL ,
   [Order] [int] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[ScoreTipificationQuery] (
   [VCC] [varchar](50) NOT NULL ,
   [ScoreId] [varchar](50) NOT NULL ,
   [TipificationId] [varchar](50) NOT NULL ,
   [QueryId] [varchar](50) NULL ,
   [QueryContent] [varchar](max) NULL ,
   [AcceptancePercentage] [decimal](18,2) NULL  ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[SentimentAnalysisResult] (
   [VCC] [varchar](50) NOT NULL ,
   [InteractionId] [varchar](250) NOT NULL ,
   [InteractionSection] [numeric](5,0) NOT NULL  ,
   [InteractionPart] [numeric](5,0) NOT NULL  ,
   [InteractionType] [varchar](50) NOT NULL ,
   [InteractionInterval] [numeric](5,0) NOT NULL  ,
   [AnalysisType] [varchar](50) NOT NULL ,
   [SentimentId] [varchar](50) NOT NULL ,
   [AppId] [varchar](50) NOT NULL ,
   [Score] [numeric](10,5) NULL  ,
   [MaxScore] [bit] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [EndDate] [datetime] NULL   

   ,CONSTRAINT [PK__Sentimen__A53A5F5D580DBCC9] PRIMARY KEY CLUSTERED ([VCC], [InteractionId], [InteractionSection], [InteractionPart], [InteractionType], [InteractionInterval], [AnalysisType], [SentimentId], [AppId])
)


GO

CREATE TABLE [dbo].[SinglePermission] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [GroupId] [varchar](200) NULL ,
   [Type] [varchar](50) NULL ,
   [Element] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_SinglePermission] PRIMARY KEY CLUSTERED ([VCC], [ID])
)


GO

CREATE TABLE [dbo].[Study] (
   [VCC] [varchar](50) NOT NULL ,
   [ProcessId] [varchar](50) NOT NULL ,
   [TemplateId] [varchar](50) NOT NULL ,
   [Name] [varchar](200) NOT NULL ,
   [Description] [varchar](100) NULL ,
   [Total] [int] NULL   ,
   [Progress] [int] NULL   ,
   [Error] [int] NULL   ,
   [Percentage] [float] NULL   ,
   [StatusDescription] [varchar](max) NULL ,
   [Status] [float] NULL   ,
   [Type] [varchar](50) NULL ,
   [LastQueryId] [varchar](50) NULL ,
   [ErrorMessage] [varchar](800) NULL ,
   [BatchStatus] [varchar](50) NULL ,
   [BatchScheduleId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Study_1] PRIMARY KEY CLUSTERED ([VCC], [ProcessId], [TemplateId], [Name])
)


GO

CREATE TABLE [dbo].[StudyDetail] (
   [VCC] [varchar](50) NOT NULL ,
   [StudyId] [varchar](200) NOT NULL ,
   [TemplateThemeId] [varchar](200) NOT NULL ,
   [TemplateThemeItemId] [varchar](200) NOT NULL ,
   [TranscriptionId] [varchar](250) NOT NULL ,
   [Total] [int] NULL   ,
   [Duration] [float] NULL   ,
   [Silence] [float] NULL   ,
   [OtherDuration] [float] NULL   ,
   [OtherSilence] [float] NULL   ,
   [Confidence] [float] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [IsMatched] [bit] NULL 
      CONSTRAINT [DF__StudyDeta__IsMat__4FF1D159] DEFAULT ('0')  

   ,CONSTRAINT [PK_StudyDetail_1] PRIMARY KEY CLUSTERED ([VCC], [StudyId], [TemplateThemeId], [TemplateThemeItemId], [TranscriptionId])
)


GO

CREATE TABLE [dbo].[StudyResultSummary] (
   [VCC] [varchar](50) NOT NULL ,
   [StudyId] [varchar](200) NOT NULL ,
   [TemplateId] [varchar](200) NOT NULL ,
   [TemplateThemeId] [varchar](200) NOT NULL ,
   [ID] [varchar](250) NOT NULL ,
   [Description] [varchar](50) NULL ,
   [Category1] [varchar](50) NULL ,
   [Category2] [varchar](50) NULL ,
   [Total] [int] NULL   ,
   [Duration] [float] NULL   ,
   [Silence] [float] NULL   ,
   [OtherDuration] [float] NULL   ,
   [OtherSilence] [float] NULL   ,
   [Confidence] [float] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [IsMatched] [bit] NULL 
      CONSTRAINT [DF__StudyResu__IsMat__50E5F592] DEFAULT ('0')  

   ,CONSTRAINT [PK_StudyResultSummary] PRIMARY KEY CLUSTERED ([VCC], [StudyId], [TemplateId], [TemplateThemeId], [ID])
)


GO

CREATE TABLE [dbo].[StudyResultSummaryDetail] (
   [VCC] [varchar](50) NOT NULL ,
   [StudyId] [varchar](200) NOT NULL ,
   [TemplateThemeId] [varchar](200) NOT NULL ,
   [TemplateThemeItemId] [varchar](200) NOT NULL ,
   [TranscriptionId] [varchar](250) NOT NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [IsMatched] [bit] NULL 
      CONSTRAINT [DF__StudyResu__IsMat__51DA19CB] DEFAULT ('0')  

   ,CONSTRAINT [PK_StudyResultSummaryDetail_1] PRIMARY KEY CLUSTERED ([VCC], [StudyId], [TemplateThemeId], [TemplateThemeItemId], [TranscriptionId])
)


GO

CREATE TABLE [dbo].[SubGroup] (
   [Instance] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](100) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [Enabled] [bit] NULL   

   ,CONSTRAINT [PK_SubGroup] PRIMARY KEY CLUSTERED ([Instance], [ID])
)


GO

CREATE TABLE [dbo].[Tag] (
   [VCC] [varchar](50) NOT NULL ,
   [AudioId] [varchar](50) NOT NULL ,
   [Section] [numeric](5,0) NOT NULL  ,
   [Part] [numeric](5,0) NOT NULL  ,
   [Tag] [varchar](50) NOT NULL ,
   [CurrentTime] [float] NOT NULL   ,
   [Comment] [varchar](200) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Tag] PRIMARY KEY CLUSTERED ([VCC], [AudioId], [Section], [Part], [Tag], [CurrentTime])
)


GO

CREATE TABLE [dbo].[Template] (
   [VCC] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [Type] [varchar](50) NOT NULL ,
   [Description] [varchar](50) NULL ,
   [Template] [varchar](50) NULL ,
   [Category] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Template] PRIMARY KEY CLUSTERED ([VCC], [Name])
)


GO

CREATE TABLE [dbo].[TemplateTheme] (
   [VCC] [varchar](50) NOT NULL ,
   [TemplateId] [varchar](50) NOT NULL ,
   [ID] [varchar](200) NOT NULL ,
   [Order] [int] NULL   ,
   [Description] [varchar](50) NULL ,
   [Category1] [varchar](50) NULL ,
   [Category2] [varchar](50) NULL ,
   [AnalysisType] [varchar](15) NULL ,
   [IsNegative] [bit] NULL 
      CONSTRAINT [DF__TemplateT__isNeg__1C722D53] DEFAULT ('0')  ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_TemplateTheme_1] PRIMARY KEY CLUSTERED ([VCC], [TemplateId], [ID])
)


GO

CREATE TABLE [dbo].[TemplateThemeItem] (
   [VCC] [varchar](50) NOT NULL ,
   [TemplateId] [varchar](50) NOT NULL ,
   [TemplateThemeId] [varchar](200) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [AcceptancePercentage] [decimal](5,2) NULL 
      CONSTRAINT [DF__TemplateT__Accep__25FB978D] DEFAULT ('0') ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_TemplateThemeItem] PRIMARY KEY CLUSTERED ([VCC], [TemplateId], [TemplateThemeId], [ID])
)


GO

CREATE TABLE [dbo].[TemplateThemeItemEntity] (
   [VCC] [varchar](50) NOT NULL ,
   [TemplateId] [varchar](50) NOT NULL ,
   [TemplateThemeId] [varchar](50) NOT NULL ,
   [TemplateThemeItemId] [varchar](50) NOT NULL ,
   [EntityId] [varchar](50) NOT NULL ,
   [EntityName] [varchar](50) NULL ,
   [AcceptancePercentage] [decimal](18,2) NULL  ,
   [Order] [int] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[TenantVCC] (
   [VCC] [varchar](50) NOT NULL ,
   [TenantId] [varchar](50) NULL ,
   [ClientId] [varchar](50) NULL ,
   [ClientSecret] [varchar](50) NULL ,
   [Resource] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL 
      CONSTRAINT [DF_TenantVCC_CreatedDate] DEFAULT (getdate())  ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_TenantVCC] PRIMARY KEY CLUSTERED ([VCC])
)


GO

CREATE TABLE [dbo].[TimeZoneParser] (
   [TimeZone] [varchar](50) NOT NULL ,
   [TimeZoneId] [varchar](50) NULL 

   ,CONSTRAINT [PK_TimeZoneParser] PRIMARY KEY CLUSTERED ([TimeZone])
)


GO

CREATE TABLE [dbo].[Tipification] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Description] [varchar](250) NOT NULL ,
   [Detail] [varchar](500) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[TipificationItem] (
   [VCC] [varchar](50) NULL ,
   [TipificationId] [varchar](50) NULL ,
   [ID] [varchar](50) NULL ,
   [SecondLevelItemId] [varchar](50) NULL ,
   [Description] [varchar](250) NULL ,
   [Order] [int] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL 
)


GO

CREATE TABLE [dbo].[TipificationQuery] (
   [VCC] [varchar](50) NOT NULL ,
   [TipificationId] [varchar](50) NOT NULL ,
   [AnalysisType] [varchar](25) NULL ,
   [QueryId] [varchar](50) NULL ,
   [QueryContent] [varchar](max) NULL ,
   [AcceptancePercentage] [decimal](18,2) NULL  ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[Transcription] (
   [VCC] [varchar](50) NOT NULL ,
   [ProcessId] [varchar](50) NOT NULL ,
   [InteractionType] [varchar](50) NULL ,
   [Section] [numeric](5,0) NOT NULL  ,
   [Part] [numeric](5,0) NOT NULL  ,
   [ID] [varchar](50) NOT NULL ,
   [Campaign] [varchar](100) NULL ,
   [Folder] [varchar](50) NULL ,
   [Value] [int] NULL   ,
   [CreatedDate] [datetime] NULL 
      CONSTRAINT [DF_TranscriptionQualification_CreatedDate] DEFAULT (getutcdate())  ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL 

   ,CONSTRAINT [PK_Transcription] PRIMARY KEY CLUSTERED ([VCC], [Section], [Part], [ID])
)


GO

CREATE TABLE [dbo].[Underlined] (
   [VCC] [varchar](50) NOT NULL ,
   [AudioId] [varchar](50) NULL ,
   [Section] [numeric](5,0) NULL  ,
   [Part] [numeric](5,0) NULL  ,
   [Speaker] [varchar](3) NULL ,
   [AnchorOffset] [int] NULL   ,
   [FocusOffset] [int] NULL   ,
   [CurrentTime] [float] NULL   ,
   [EndCurrentTime] [float] NULL   ,
   [UtteranceId] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[UniqueTag] (
   [VCC] [varchar](50) NOT NULL ,
   [TagName] [varchar](50) NOT NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_UniqueTag] PRIMARY KEY CLUSTERED ([VCC], [TagName])
)


GO

CREATE TABLE [dbo].[User] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Email] [varchar](100) NULL ,
   [Firstname] [varchar](100) NULL ,
   [Lastname] [varchar](100) NULL ,
   [Password] [varchar](100) NULL ,
   [Role] [varchar](50) NULL ,
   [Country] [varchar](50) NULL ,
   [Language] [varchar](50) NULL ,
   [TimeZone] [varchar](50) NULL ,
   [Preferences] [varchar](200) NULL ,
   [IsSystemUser] [bit] NULL   ,
   [IsLocalSupervisor] [bit] NULL 
      CONSTRAINT [DF_User_IsLocalSupervisor] DEFAULT ((0))  ,
   [IsExternalSupervisor] [bit] NULL 
      CONSTRAINT [DF_User_IsExternalSupervisor] DEFAULT ((0))  ,
   [IsExternalAgent] [bit] NULL 
      CONSTRAINT [DF_User_IsExternalAgent] DEFAULT ((0))  ,
   [LimitClients] [bit] NULL   ,
   [API] [bit] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_User] PRIMARY KEY CLUSTERED ([VCC], [ID])
)


GO

CREATE TABLE [dbo].[UserAccessToken] (
   [VCC] [varchar](50) NOT NULL ,
   [Realm] [varchar](100) NOT NULL ,
   [UserName] [varchar](50) NOT NULL ,
   [AccessToken] [varchar](2000) NOT NULL ,
   [CreatedDate] [datetime] NOT NULL   

   ,CONSTRAINT [PK_UserAccessToken_1] PRIMARY KEY CLUSTERED ([VCC], [Realm])
)


GO

CREATE TABLE [dbo].[UserAPI] (
   [VCC] [varchar](50) NOT NULL ,
   [User] [varchar](50) NOT NULL ,
   [IsLocked] [bit] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_UserAPI] PRIMARY KEY CLUSTERED ([VCC], [User])
)


GO

CREATE TABLE [dbo].[UserClient] (
   [VCC] [varchar](50) NOT NULL ,
   [UserId] [varchar](50) NOT NULL ,
   [ClientId] [varchar](50) NOT NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK__UserClient] PRIMARY KEY CLUSTERED ([VCC], [UserId], [ClientId])
)


GO

CREATE TABLE [dbo].[UserHistory] (
   [ID] [int] NOT NULL  
      IDENTITY (1,1) ,
   [VCC] [varchar](50) NULL ,
   [UserId] [varchar](50) NULL ,
   [UserRole] [varchar](50) NULL ,
   [Event] [varchar](50) NULL ,
   [EventDate] [datetime] NULL   ,
   [EventDetail] [varchar](200) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_UserHistory] PRIMARY KEY CLUSTERED ([ID])
)


GO

CREATE TABLE [dbo].[Utterance] (
   [VCC] [varchar](50) NOT NULL ,
   [AppId] [varchar](50) NOT NULL ,
   [IntentId] [varchar](50) NULL ,
   [Id] [varchar](50) NOT NULL ,
   [Text] [varchar](500) NULL ,
   [CreatedDate] [datetime] NULL 
      CONSTRAINT [DF__Utterance__Creat__06B7F65E] DEFAULT (getutcdate())  ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [Status] [int] NULL 
      CONSTRAINT [DF__Utterance__Statu__4C214075] DEFAULT ('1')  ,
   [StatusCode] [varchar](50) NULL 
      CONSTRAINT [DF__Utterance__Statu__4D1564AE] DEFAULT ('AVAILABLE')
)


GO

CREATE TABLE [dbo].[UtteranceEntity] (
   [VCC] [varchar](50) NULL ,
   [UtteranceId] [varchar](50) NOT NULL ,
   [EntityName] [varchar](50) NOT NULL ,
   [Start] [int] NULL   ,
   [End] [int] NULL   ,
   [EndTokenIndex] [int] NULL   ,
   [StartTokenIndex] [int] NULL   ,
   [CreatedDate] [datetime] NULL   
)


GO

CREATE TABLE [dbo].[VCC] (
   [VCC] [varchar](50) NOT NULL ,
   [ID] [varchar](50) NOT NULL ,
   [Type] [varchar](50) NOT NULL 
      CONSTRAINT [DF_VCC_Type] DEFAULT ('ALLEGRO'),
   [Description] [varchar](50) NULL ,
   [Authentication] [varchar](50) NULL ,
   [TranscriptionDefaultLanguage] [varchar](25) NULL ,
   [CognitiveEngine] [varchar](25) NULL ,
   [EnableGenerativeEngine] [bit] NULL   ,
   [EnableAdvancedAI] [bit] NULL   ,
   [EnableRealTime] [bit] NULL   ,
   [Country] [varchar](50) NULL ,
   [Language] [varchar](50) NULL ,
   [TimeZone] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   ,
   [IsMigrating] [bit] NULL 
      CONSTRAINT [DF__VCC__IsMigrating__4E0988E7] DEFAULT ('0')  ,
   [DeleteDataDays] [int] NULL 
      CONSTRAINT [DF__VCC__DeleteDataD__705EA0EB] DEFAULT ('360')  

   ,CONSTRAINT [PK_VCC] PRIMARY KEY CLUSTERED ([VCC], [ID])
)


GO

CREATE TABLE [dbo].[VCCLicense] (
   [VCC] [varchar](50) NOT NULL ,
   [Product] [varchar](50) NOT NULL ,
   [Status] [bit] NULL 
      CONSTRAINT [DF__VCCLicens__Statu__592635D8] DEFAULT ((0))  ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL 

   ,CONSTRAINT [PK_VCCLicense_1] PRIMARY KEY CLUSTERED ([VCC], [Product])
)


GO

CREATE TABLE [dbo].[Version] (
   [Id] [varchar](50) NOT NULL ,
   [InstallationId] [varchar](50) NOT NULL ,
   [ServicePack] [varchar](100) NOT NULL ,
   [BuildVersion] [varchar](50) NULL ,
   [IsRegistered] [bit] NULL   ,
   [TimeStamp] [datetime] NULL   ,
   [ServicePackTimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Version] PRIMARY KEY CLUSTERED ([Id])
)


GO

CREATE TABLE [dbo].[Word] (
   [VCC] [varchar](50) NOT NULL ,
   [NeuroLinguisticProgrammingId] [varchar](50) NOT NULL ,
   [Name] [varchar](50) NOT NULL ,
   [ProcessId] [varchar](50) NOT NULL ,
   [Language] [varchar](50) NULL ,
   [Type] [varchar](50) NULL ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_Word_1] PRIMARY KEY CLUSTERED ([VCC], [NeuroLinguisticProgrammingId], [Name], [ProcessId])
)


GO

CREATE TABLE [dbo].[WordBlacklist] (
   [VCC] [varchar](50) NOT NULL ,
   [ProcessId] [varchar](50) NOT NULL ,
   [Word] [varchar](50) NOT NULL ,
   [CreatedDate] [datetime] NOT NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK_WordBlacklist] PRIMARY KEY CLUSTERED ([VCC], [ProcessId], [Word])
)


GO

CREATE TABLE [dbo].[WordsByLanguage] (
   [Label] [varchar](50) NULL ,
   [Language] [varchar](4) NULL ,
   [Word] [varchar](100) NULL 
)


GO

CREATE TABLE [dbo].[WorkGroupSupervisor] (
   [VCC] [varchar](50) NOT NULL ,
   [Description] [varchar](50) NOT NULL ,
   [SupervisorId] [varchar](250) NOT NULL ,
   [CampaignId] [varchar](250) NOT NULL ,
   [AgentId] [varchar](250) NOT NULL ,
   [IsHistoric] [bit] NULL   ,
   [CreatedDate] [datetime] NULL   ,
   [CreatedByUserId] [varchar](50) NULL ,
   [LastModifiedDate] [datetime] NULL   ,
   [LastModifiedByUserId] [varchar](50) NULL ,
   [TimeStamp] [datetime] NULL   

   ,CONSTRAINT [PK__tmp_ms_x__89284CC5064D97A4] PRIMARY KEY CLUSTERED ([VCC], [Description], [SupervisorId], [CampaignId], [AgentId])
)


GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[CheckQuery] 'query2019061201'
CREATE FUNCTION [dbo].[CheckQuery]
(
	@ID		VARCHAR(50)
)
RETURNS VARCHAR(50)
AS
BEGIN
	DECLARE @QueryId VARCHAR(50)
	SET @QueryId = (SELECT DISTINCT ID FROM TemplateThemeItem WITH (NOLOCK) WHERE ID = @ID)
	RETURN @QueryId
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- SELECT [dbo].[ConvertHourMin](12.333)
CREATE FUNCTION [dbo].[ConvertHourMin]
(
	@segundos				NUMERIC(12, 3)
)
RETURNS VARCHAR(8)
AS
BEGIN
	DECLARE @HraMin VARCHAR(8)
	SELECT @HraMin =	REPLICATE('0', 2-LEN(CAST(CAST(@segundos AS INT) / 3600 AS VARCHAR(2)))) + CAST(CAST(@segundos AS INT) / 3600 AS VARCHAR(2)) + ':'  
				+ REPLICATE('0', 2-LEN(CAST((CAST(@segundos AS INT) % 3600) / 60 AS VARCHAR(2)))) + CAST((CAST(@segundos AS INT) % 3600) / 60 AS VARCHAR(2)) + ':' 
				+ REPLICATE('0', 2-LEN(CAST((CAST(@segundos AS INT) % 60)  AS VARCHAR(2)))) + CAST((CAST(@segundos AS INT) % 60) AS VARCHAR(2))

	RETURN (@HraMin)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE FUNCTION [dbo].[GetInstanceAnalyticCodeInstance] (
     @Code Varchar(100) = ''
)
RETURNS Varchar(50)
AS
BEGIN
	
	RETURN ISNULL((SELECT TOP 1 [Instance] 
		FROM [AnalyticCodeInstance] (NOLOCK)
		WHERE [Code] = @Code
		AND ISNULL([Deleted], 0) = 0),'')

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE FUNCTION [dbo].[GetUserCampaigns] (
    @Instance Varchar(50) = '', @UserId Varchar(50) = ''
)
RETURNS @Campaigns TABLE (CampaignId Varchar(50))
AS
BEGIN

	INSERT INTO @Campaigns
	
	SELECT [ID] AS 'CampaignId'
	FROM [Campaign] (NOLOCK)
	WHERE [Instance] = @Instance
	AND [Client] IN (
		SELECT [ClientId]
		FROM [UserClient] (NOLOCK)
		WHERE [Instance] = @Instance
		AND [UserId] = @UserId
		AND [Deleted] = 0
	)
	AND [Deleted] = 0

    RETURN

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE FUNCTION [dbo].[IsEmpty](@Value Varchar(255), @VaueIfEmpty Varchar(255))
RETURNS Varchar(255)
AS
BEGIN
	IF ISNULL(@Value, '') = ''
		SET  @Value = @VaueIfEmpty

	RETURN (@Value)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE FUNCTION [dbo].[IsInteger](@Value Varchar(18))
RETURNS Bit
AS 
BEGIN
  
  RETURN ISNULL(
     (SELECT CASE WHEN CHARINDEX('.', @Value) > 0 
                  THEN CASE WHEN CONVERT(Int, PARSENAME(@Value, 1)) <> 0
                            THEN 0
                            ELSE 1
                            END
                  ELSE 1
                  END
      WHERE ISNUMERIC(@Value + 'e0') = 1), 0)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE FUNCTION [dbo].[split](
    @delimited NVARCHAR(MAX),
    @delimiter NVARCHAR(100)
) RETURNS @t TABLE (Id INT IDENTITY(1,1), Value NVARCHAR(MAX))
AS
BEGIN
    DECLARE @xml XML
    SET @xml = N'<t>' + REPLACE(@delimited,@delimiter,'</t><t>') + '</t>'

    INSERT INTO @t(Value)
    SELECT  r.value('.','varchar(MAX)') as item
    FROM  @xml.nodes('/t') as records(r)
    RETURN
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE FUNCTION [dbo].[SplitString] (
    @String Varchar(MAX) = '', @Delimiter Char(1) = 'Â§'
)
RETURNS @Parts TABLE (value Varchar(500))
AS
BEGIN

    IF @String IS NULL RETURN
    
    DECLARE	@Start INT,
    		@Pos INT
    		
    if SUBSTRING(@String, 1, 1) = @Delimiter 
		BEGIN
    	SET	@Start = 2
    	INSERT INTO @Parts
    	VALUES (NULL)
		END
    ELSE 
    	SET	@Start = 1
    	
    WHILE 1=1
		BEGIN
		
    	SET	@Pos = CHARINDEX(@Delimiter, @String, @Start)
    	
    	IF @Pos = 0
    		SET	@Pos = LEN(@String) + 1
    		
    	IF @Pos - @Start > 0			
    		INSERT INTO @Parts
    		VALUES (LTRIM(RTRIM(SUBSTRING(@String, @Start, @Pos - @Start))))
    		
    	ELSE
    		INSERT INTO @Parts
    		VALUES (NULL)
    		
    	SET	@Start = @Pos + 1
    	
    	IF @Start > LEN( @String ) 
    		BREAK
    		
		END
		
    RETURN

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE FUNCTION [dbo].[SplitStringXML] (
    @String Varchar(MAX) = '', @Delimiter Char(1) = 'Ã‚Â§'
)
RETURNS @Parts TABLE (value Varchar(500))
AS
BEGIN
	DECLARE @X xml
	SELECT @X = CONVERT(xml,'<root><s>'+ REPLACE(@String,@Delimiter,'</s> <s>')+ '</s></root>')
	
	insert into @Parts (value)
	SELECT T.c.value('.','varchar(500)')
	FROM @X.nodes('/root/s') T(c)
		
    RETURN

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkAbilityModel]
AS
BEGIN
	SET NOCOUNT ON
	
	/*- INSERTAMOS LA DATA DE LA TEMPORAL PARA DEPURAR EL SP MANUALMENTE
	IF OBJECT_ID(N'[dbo].[TMP_BULK_INTENT]', N'U') IS NULL
	BEGIN
		SELECT *
			INTO [dbo].[TMP_BULK_INTENT]
			FROM [dbo].[#AbilityModelTemp]

		/*
		SELECT * FROM [dbo].[TMP_BULK_INTENT] (NOLOCK)
		SELECT * 
			INTO [#AbilityModelTemp]
			FROM [dbo].[TMP_BULK_INTENT] (NOLOCK)
		SELECT * FROM [#AbilityModelTemp]
		*/
	END
	ELSE
	BEGIN
		INSERT INTO [dbo].[TMP_BULK_INTENT]
			SELECT * 
				FROM [dbo].[#AbilityModelTemp]
	END
	*/
	
	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS PLANES Y SU CORRESPONDIENTE VCC
	DECLARE MY_CURSOR CURSOR 
	LOCAL STATIC READ_ONLY FORWARD_ONLY -- SCROLL 
		FOR 
		-- CONSULTAMOS LOS ID DE PLANES EN LA TABLA TEMPORAL
		SELECT DISTINCT ACTMP.[VCC],
							ACTMP.[EvaluationPlanId] 'EVALUATION_PLAN_ID'
						FROM [#AbilityModelTemp] ACTMP (NOLOCK)
						WHERE ACTMP.[VCC] IS NOT NULL 
							AND ACTMP.[EvaluationPlanId] IS NOT NULL

	-- DECLARARAMOS VARIABLES PARA INDICAR EL PLAN Y VCC
	DECLARE @CURRENT_VCC VARCHAR(50) = ''
	DECLARE @CURRENT_EVALUATION_PLAN_ID VARCHAR(100) = ''

	-- ABRIMOS Y RECORREMOS EL ARRAY DE PLANES
	OPEN MY_CURSOR
	FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID
	WHILE @@FETCH_STATUS = 0
	BEGIN 
		-- EJECUTAMOS EL SP QUE INSERTA LOS RESULT BASE DE LOS PLANES QUE AUN NO LOS TIENEN
		EXEC [dbo].[SaveEvaluationBaseResult] @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID, 'AUTOMATIC'
			
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID
	END
	CLOSE MY_CURSOR
	DEALLOCATE MY_CURSOR
	
	-- ACTUALIZAMOS LOS EVALUATION RESULT DE LAS HABILIDADES EN LA TABLA TEMPORAL DEL BULK
	UPDATE ER
		SET ER.[Qualification] = (AEF.[Score] * S.[Value]) / 100,
			ER.[OwnScore] = (AEF.[Score] * S.[Value]) / 100,
			ER.[ScoreDescription] = S.[Description],
			ER.[AbilityInnerScore] = S.[Value]
		FROM [#AbilityModelTemp] ACTMP 
		INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
			ON EP.[ID] = ACTMP.[EvaluationPlanId]
		INNER JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
			ON AEF.[Instance] = EP.[Instance]
				AND AEF.[IdEvaluationFormat] = EP.[IdEvaluationFormat]
				AND AEF.[IdAbility] = ACTMP.[AbilityId]
		INNER JOIN EvaluationResult ER (NOLOCK)
			ON ER.[Instance] = EP.[Instance] 
				AND ER.[IdEvaluationFormat] = EP.[IdEvaluationFormat]
				AND ER.[IdInteraction] = ACTMP.[ID]
		INNER JOIN [dbo].[Ability] A (NOLOCK) 
	       ON A.[Instance] = ER.[Instance] 
	            AND A.[ID] = ER.[IdAbility]
		INNER JOIN [dbo].[Score] S (NOLOCK)
			ON S.[IdAbility] = ACTMP.[AbilityId]
				AND S.[Value] = IIF(A.[IsNegative] = 1, 0, 100)
		WHERE AEF.[IdAbility] = ER.[IdAbility]
			AND ER.[IdAuditor] = 'SPEECH'

	/* Script para actualizar la tabla AuditorEvaluationPlan a Completed y Suma de Scores de la evaluacion */
	UPDATE AEP
		SET --AEP.[Completed] = '1',
			AEP.[Completed] = CASE WHEN AEPR.[Evaluation_Mode] = 'X' AND AEPR.[MinAutomaticQualification] <= AEPR.[SCORE] THEN '0' ELSE '1' END,
			AEP.[StatusDescription] = CASE WHEN AEPR.[Evaluation_Mode] = 'X' AND AEPR.[MinAutomaticQualification] <= AEPR.[SCORE] THEN 'NEW' ELSE 'COMPLETED' END,
			AEP.[Score] = AEPR.[SCORE],
			AEP.[AutomaticScore] = AEPR.[SCORE],
			-- AEP.[ManualScore] = AEPR.[SCORE], -- UNA EVALUACION AUTOMATICA DEBE TENER 0 DE MANUALSCORE
			AEP.[ManualScore] = 0,
			AEP.[LastModifiedDate] = GETUTCDATE(),
			AEP.[TimeStamp] = GETUTCDATE()
		FROM AuditorEvaluationPlan AEP (NOLOCK)
		LEFT JOIN (
			SELECT AEP.[Instance],
				AEP.[IdEvaluationPlan],
				AEP.[IdInteraction],
				EP.[Evaluation_Mode],
				EF.[MinAutomaticQualification],
				SUM(ER.[Qualification]) AS SCORE
			FROM AuditorEvaluationPlan AEP (NOLOCK)
			LEFT JOIN EvaluationPlan EP (NOLOCK)
				ON EP.[Instance] = AEP.[Instance]
					AND EP.[ID] = AEP.[IdEvaluationPlan] 
			LEFT JOIN [dbo].[EvaluationFormat] EF (NOLOCK)
				ON EF.[Instance] = AEP.[Instance]
					AND EF.[ID] = EP.[IdEvaluationFormat]
			LEFT JOIN EvaluationResult ER (NOLOCK)
				ON ER.[Instance] = AEP.[Instance]
					AND ER.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND ER.[IdInteraction] = AEP.[IdInteraction]
			LEFT JOIN AbilityEvaluationFormat AEF (NOLOCK) 
				ON AEF.[Instance] = EP.[Instance]
					AND AEF.[IdEvaluationFormat] = EP.[IdEvaluationFormat]
					AND AEF.[IdAbility] = ER.[IdAbility]
			LEFT JOIN (SELECT DISTINCT ACTMP.[VCC],
									ACTMP.[EvaluationPlanId]
								FROM [#AbilityModelTemp] ACTMP (NOLOCK)
								WHERE ACTMP.[VCC] IS NOT NULL 
									AND ACTMP.[EvaluationPlanId] IS NOT NULL) TMP 
				ON TMP.[VCC] = AEP.[Instance]
					AND TMP.[EvaluationPlanId] = AEP.[IdEvaluationPlan]
			WHERE TMP.[EvaluationPlanId] = AEP.[IdEvaluationPlan]
				AND AEP.[IdAuditor] = 'SPEECH'
			GROUP BY AEP.[Instance], AEP.[IdEvaluationPlan], AEP.[IdInteraction], EP.[Evaluation_Mode], EF.[MinAutomaticQualification]
		) AEPR 
		ON AEPR.[Instance] = AEP.[Instance]
			AND AEPR.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
			AND AEPR.[IdInteraction] = AEP.[IdInteraction]
		WHERE AEPR.[Instance] = AEP.[Instance]
			AND AEPR.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
			AND AEPR.[IdInteraction] = AEP.[IdInteraction]
	
	;
	/****************************************************************************************/
	/*INICIO SECCION PARA ACTUALIZAR LA CALIFICACION TOTAL DE LA EVALUACION C/S PENALIZACION*/
	/****************************************************************************************/

	DECLARE @EvaluationPlans VARCHAR(MAX) = NULL
	SELECT @EvaluationPlans = ISNULL(@EvaluationPlans + ', ', '') + D.[EvaluationPlanId]
		FROM (SELECT DISTINCT [EvaluationPlanId] FROM [#AbilityModelTemp]) D
	
	-- EMULAMOS UNA NUEVA TABLA VARIABLE PARA DETERMINAR LAS INTERACCIONES QUE TIENEN HABILIDAD(ES) PENALIZADA(S) POR NOTA CERO
	--  ORDENANDO POR PRIORIDAD (INTERACCION 1, GRUPO 2 Y SUBGRUPO 3)
	;WITH CTEFP AS (
		SELECT DISTINCT 
				ER.[Instance],
				ER.[IdInteraction],
				ER.[IdEvaluationPlan],
				ER.[IdEvaluationFormat],
				ER.[IdAbility],
				CASE WHEN ER.[AbilityInnerScore] = 0 AND ER.[NotApplies] = 0 AND HAEF.[Penalizes] <> 'NO' THEN 1 ELSE 0 END 'Penalized',
				HAEF.[Penalizes],
				CASE HAEF.[Penalizes]
					WHEN 'INT' THEN ER.[IdInteraction] 
					WHEN 'GRP' THEN ER.[IdGroup] 
					WHEN 'SGR' THEN ER.[IdSubGroup] 
				END 'PenalizedOn',
				RN = ROW_NUMBER() OVER(PARTITION BY ER.[IdInteraction], CASE WHEN ER.[AbilityInnerScore] = 0 THEN 1 ELSE 0 END 
					ORDER BY CASE HAEF.[Penalizes]
					WHEN 'INT' THEN 1 
					WHEN 'GRP' THEN 2 
					WHEN 'SGR' THEN 3 
				END)
			FROM [dbo].[EvaluationResult] ER (NOLOCK) 
			LEFT JOIN [dbo].[HistoricAbilityEvaluationFormat] HAEF (NOLOCK) 
				ON HAEF.[VCC] = ER.[Instance]
					AND HAEF.[EvaluationPlanId] = ER.[IdEvaluationPlan]
					AND HAEF.[IdEvaluationFormat] = ER.[IdEvaluationFormat]
					AND HAEF.[IdAbility] = ER.[IdAbility]
			WHERE ER.[IdEvaluationPlan] IN (SELECT Value FROM [dbo].[split](@EvaluationPlans,',')) 
				AND ER.[AbilityInnerScore] = 0
				AND ER.[NotApplies] = 0
				AND HAEF.[Penalizes] <> 'NO'
			-- WHERE ER.[IdEvaluationPlan] IN (SELECT DISTINCT EPA.[EvaluationPlanId] FROM [DummyEvaluationPlanAbility] EPA (NOLOCK) WHERE EPA.[EvaluationPlanId] IS NOT NULL)
	),

	-- ALMACENAMOS EL RESULTADO AGRUPADO DE LAS EVALUACIONES EN UNA TABLA VARIABLE PARA HACER UN CRUCE DIRECTO CON
	--  LA TABLA AuditorEvaluationPlan
	CTEFS AS (
		SELECT ER.[Instance],
				ER.[IdEvaluationPlan],
				ER.[IdInteraction],
				ER.[IdAuditor],
				SUM(ER.[Qualification])															'TOTAL_SCORE',
				SUM(IIF(HA.[Type] = 'MANUAL_AUTOMATIC', ER.[Qualification], 0))					'TOTAL_AUTOMATIC_SCORE',
				SUM(IIF(HA.[Type] = 'MANUAL', ER.[Qualification], 0))							'TOTAL_MANUAL_SCORE',
				--SUM(ER.[Qualification] * CASE WHEN FP2.[Penalized] IS NULL THEN 1 ELSE 0 END) 'PENALIZED_SCORE',
				SUM(ER.[Qualification] * (
					CASE WHEN (
						CASE FP2.[Penalizes]
							WHEN 'INT' THEN ER.[IdInteraction] 
							WHEN 'GRP' THEN ER.[IdGroup] 
							WHEN 'SGR' THEN ER.[IdSubGroup] 
						END = FP2.[PenalizedOn]
						AND FP2.[Penalized] = 1) THEN 0 ELSE 1 END
				))																				'PENALIZED_SCORE',
				SUM(IIF(HA.[Type] = 'MANUAL_AUTOMATIC',
					ER.[Qualification] * (
					CASE WHEN (
						CASE FP2.[Penalizes]
							WHEN 'INT' THEN ER.[IdInteraction] 
							WHEN 'GRP' THEN ER.[IdGroup] 
							WHEN 'SGR' THEN ER.[IdSubGroup] 
						END = FP2.[PenalizedOn]
						AND FP2.[Penalized] = 1
					) THEN 0 ELSE 1 END),
					0) 
				)																				'PENALIZED_AUTOMATIC_SCORE',
				SUM(IIF(HA.[Type] = 'MANUAL',
					ER.[Qualification] * (
					CASE WHEN (
						CASE FP2.[Penalizes]
							WHEN 'INT' THEN ER.[IdInteraction] 
							WHEN 'GRP' THEN ER.[IdGroup] 
							WHEN 'SGR' THEN ER.[IdSubGroup] 
						END = FP2.[PenalizedOn]
						AND FP2.[Penalized] = 1
					) THEN 0 ELSE 1 END),
					0) 
				)																				'PENALIZED_MANUAL_SCORE'
			FROM [dbo].[EvaluationResult] ER (NOLOCK) 
			LEFT JOIN (
				-- HACEMOS UN CRUCE CON LA TABLA VARIABLE Y TOMAMOS SOLO UNA DE LAS PENALIZACIONES POR INTERACCION, EMPEZANDO 
				--  POR LA MAS PRIORITARIA (FP.[RN])
				SELECT DISTINCT FP.[Instance],
								FP.[IdEvaluationPlan], 
								FP.[IdInteraction], 
								FP.[Penalized],
								FP.[Penalizes], 
								FP.[PenalizedOn]
					FROM CTEFP FP (NOLOCK)
					WHERE FP.[Penalizes] <> 'NO'
						AND FP.[RN] = 1
			) FP2
				ON FP2.[Instance] = ER.[Instance]
					AND ER.[IdEvaluationPlan] = FP2.[IdEvaluationPlan]
					AND ER.[IdInteraction] = FP2.[IdInteraction]
					/*
					AND CASE FP2.[Penalizes]
							WHEN 'INT' THEN ER.[IdInteraction] 
							WHEN 'GRP' THEN ER.[IdGroup] 
							WHEN 'SGR' THEN ER.[IdSubGroup] 
						END = FP2.[PenalizedOn]
					AND FP2.[Penalized] = 1
					*/
			INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
				ON HA.[VCC] = ER.[Instance]
					AND HA.[EvaluationPlanId] = ER.[IdEvaluationPlan]
					AND HA.[ID] = ER.[IdAbility]
			--WHERE ER.[Instance] = 'multisalud'
			WHERE -- ER.[Instance] = FP2.[Instance] -- ESTA LINEA CAUSA QUE NO SE ACTUALICEN LOS SCORES LAS EVALUACIONES QUE NO PENALIZAN
				-- AND 
				ER.[IdEvaluationPlan] IN (SELECT Value FROM [dbo].[split](@EvaluationPlans,','))
				--AND ER.[IdEvaluationPlan] IN (SELECT DISTINCT EvaluationPlanId FROM DummyEvaluationPlanAbility)
			GROUP BY ER.[Instance],
				ER.[IdEvaluationPlan],
				ER.[IdInteraction],
				ER.[IdAuditor]
	)
	-- SELECT * FROM CTEFS

	-- ACTUALIZAMOS LOS REGISTROS DE EVALUACION CON LA NOTA PENALIZADA OBTENIDA EN LOS PASOS ANTERIORES
	UPDATE AEP 
		SET AEP.[Score] = FS.[PENALIZED_SCORE],
			AEP.[AutomaticScore] = FS.[PENALIZED_AUTOMATIC_SCORE],
			AEP.[ManualScore] = FS.[PENALIZED_MANUAL_SCORE],
			AEP.[LastModifiedDate] = GETUTCDATE(),
			AEP.[TimeStamp] = GETUTCDATE()
		FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
		LEFT JOIN CTEFS FS (NOLOCK) 
			ON FS.[Instance] = AEP.[Instance] 
				AND FS.[IdEvaluationPlan] = AEP.[IdEvaluationPlan] 
				AND FS.[IdInteraction] = AEP.[IdInteraction]
				AND FS.[IdAuditor] = AEP.[IdAuditor]
		WHERE FS.[Instance] IS NOT NULL
		
	/*************************************************************************************/
	/*FIN SECCION PARA ACTUALIZAR LA CALIFICACION TOTAL DE LA EVALUACION C/S PENALIZACION*/
	/*************************************************************************************/
     --insert into test(origin,content) values('BulkAbilityCognitive',(select *  from [dbo].[#AbilityModelTemp] FOR JSON AUTO))
	DROP TABLE [dbo].[#AbilityModelTemp];
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkAgent]
AS
BEGIN

	-- ACTUALIZAMOS LOS REGISTROS QUE VIENEN EN LA TEMPORAL Y YA EXISTEN EN LA TABLA AGENT
	UPDATE A
		SET A.[Name] = A_TMP.[Name],
			A.[LastModifiedDate] = GETUTCDATE(),
			A.[TimeStamp] = GETUTCDATE()
		FROM [dbo].[Agent] A (NOLOCK)
		INNER JOIN [#AgentTemp] A_TMP 
			ON A_TMP.[VCC] = A.[VCC]
				AND A_TMP.[AgentId] = A.[AgentId] 

	-- INSERTAMOS LA DATA DE LA TABLA TEMPORAL
	INSERT INTO [dbo].[Agent] ([VCC], [AgentId], [Name], [CreatedDate], [TimeStamp])
		SELECT A_TMP.[VCC], 
				A_TMP.[AgentId], 
				REPLACE(LTRIM(RTRIM(A_TMP.[Name])), '  ', ' '), 
				GETUTCDATE(), 
				GETUTCDATE()
			FROM [#AgentTemp] A_TMP (NOLOCK) 
			LEFT JOIN [dbo].[Agent] A (NOLOCK)
				ON A.[VCC] = A_TMP.[VCC]
					AND A.[AgentId] = A_TMP.[AgentId]
			WHERE A.[VCC] IS NULL

	-- ELIMINAMOS LA TABLA TEMPORAL 
	DROP TABLE [#AgentTemp]

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
 CREATE PROCEDURE [dbo].[BulkAsyncInteraction]
AS
BEGIN
	-- ESTE SP INSERTA REGISTROS UNICOS DE ASYNCINTERACTION PROVENIENTES DE LA BD EXTERNA EN LA TABLA ASYNCINTERACTION Y PROCESSASYNCINTERACTION

	--
	INSERT INTO [dbo].[AsyncInteraction] (
		[VCC], [ID], [Folder], [ExportedContent], [Channel]
		, [Campaign], [StartDate], [EndDate], [Account], [Initiative]
		, [ContactName], [FirstTakedAgent], [LastTakedAgent], [FirstTakedAgentName], [LastTakedAgentName]
		, [IsTaken], [TotalMessages], [DurationTime], [AttentionTime], [DispositionCode]
		, [DispositionTreePath], [RawChar], [CreatedDate], [CreatedByUserId], [TimeStamp] 
		, [ContactId], [ContactAddress], [NeedI6Update], [Ender], [Ticket] 
	)
		-- SELECCIONAMOS TODOS LOS REGISTROS DE LA TABLA TEMPORAL [#AsyncInteractionTemp] QUE NO EXISTEN EN [AsyncInteraction] POR LOS PK
		SELECT AIT.[VCC], AIT.[ID], AIT.[Folder], AIT.[ExportedContent], AIT.[Channel]
				, AIT.[Campaign], AIT.[StartDate], AIT.[EndDate], AIT.[Account], AIT.[Initiative]
				, AIT.[ContactName], AIT.[FirstTakedAgent], AIT.[LastTakedAgent], A1.[Name], A2.[Name]
				, AIT.[IsTaken], AIT.[TotalMessages], AIT.[DurationTime], AIT.[AttentionTime], AIT.[DispositionCode]
				, AIT.[DispositionTreePath], AIT.[RawChar], AIT.[CreatedDate], AIT.[CreatedByUserId], GETUTCDATE()
				, AIT.[ContactId], AIT.[contactAddress], 1, AIT.[Ender], AIT.[Ticket] 
			FROM [#AsyncInteractionTemp] AIT
			LEFT JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
				ON AI.[VCC] = AIT.[VCC]
					AND AI.[ID] = AIT.[ID]
					AND CONVERT(VARCHAR, AI.[EndDate], 23) = AIT.[Folder]
			LEFT JOIN [dbo].[Agent] A1 (NOLOCK)
				ON A1.[VCC] = AIT.[VCC]
					AND A1.[AgentId] = AIT.[FirstTakedAgent]
			LEFT JOIN [dbo].[Agent] A2 (NOLOCK)
				ON A2.[VCC] = AIT.[VCC]
					AND A2.[AgentId] = AIT.[LastTakedAgent]
			WHERE AI.[ID] IS NULL

	--
	INSERT INTO [dbo].[ProcessAsyncInteraction] (
		[VCC], [ProcessId], [InteractionType], [Campaign], [InteractionId]
		, [EndDate], [FormattedEndDate], [CreatedDate], [CreatedByUserId], [LastModifiedDate]
		, [LastModifiedByUserId], [TimeStamp]
	)
		-- SELECCIONAMOS TODOS LOS REGISTROS DE LA TABLA TEMPORAL [#ProcessAsyncInteractionTemp] QUE NO EXISTEN EN [ProcessAsyncInteraction] POR LOS PK
		SELECT PAIT.[VCC], PAIT.[ProcessId], PAIT.[InteractionType], PAIT.[Campaign], PAIT.[InteractionId] 
				, PAIT.[EndDate], PAIT.[FormattedEndDate], PAIT.[CreatedDate], PAIT.[CreatedByUserId], PAIT.[LastModifiedDate]
				, PAIT.[LastModifiedByUserId], GETUTCDATE()
			FROM [#ProcessAsyncInteractionTemp] PAIT
			LEFT JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
				ON PAI.[VCC] = PAIT.[VCC]
					AND PAI.[ProcessId] = PAIT.[ProcessId]
					AND PAI.[InteractionType] = PAIT.[InteractionType]
					AND PAI.[InteractionId] = PAIT.[InteractionId]
					--AND PAI.[EndDate] = PAIT.[EndDate]
					AND PAI.[FormattedEndDate] = PAIT.[FormattedEndDate]
			WHERE PAI.[InteractionId] IS NULL
	
	-- ELIMINAMOS LAS TABLAS TEMPORALES
	DROP TABLE [#AsyncInteractionTemp]
	DROP TABLE [#ProcessAsyncInteractionTemp]
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkAudio]
AS
BEGIN
	-- DECLARAMOS UNA VARIABLE PARA DEFINIR EL VCC
	DECLARE @VCC VARCHAR(50) = NULL
	DECLARE @VCC_TYPE VARCHAR(50) = NULL

	-- ASIGNAMOS A LA VARIABLE EL NOMBRE DEL VCC
	SELECT TOP 1 @VCC = [VCC] FROM #AudioTemp (NOLOCK)

	-- BUSCAMOS EL TIPO DE VCC
	SELECT @VCC_TYPE = V.[Type] FROM [dbo].[VCC] V (NOLOCK) WHERE V.[VCC] = @VCC

	-- CREAMOS UN INDICE PARA LA TEMPORAL
	CREATE NONCLUSTERED INDEX [TMP_AUDIO_01] ON #AudioTemp ([ID])
	CREATE NONCLUSTERED INDEX [TMP_AUDIO_02] ON #AudioTemp ([VCC], [Agent])
	CREATE NONCLUSTERED INDEX [TMP_AUDIO_03] ON #AudioTemp ([VCC], [ID], [Section], [Part])
	CREATE NONCLUSTERED INDEX [TMP_AUDIO_04] ON #AudioTemp ([VCC], [ProcessId], [Id],[Section], [Part])
	CREATE NONCLUSTERED INDEX [TMP_AUDIO_05] ON #AudioTemp ([VCC], [ProcessId], [Id],[Section], [Part], [CreatedByUserId])

	-- ACTUALIZAR RELATIVE PATH
	UPDATE A
		SET A.[RelativePath] = ATE.[RelativePath],
			A.[RelativeFileName] = ATE.[RelativeFileName]
		FROM dbo.[AUDIO] A
		INNER JOIN #AudioTemp ATE 
			ON A.[VCC] = ATE.[VCC] 
				AND A.[ID] = ATE.[ID]
				AND A.[Section] = ATE.[Section] 
				AND A.[Part] = ATE.[Part]
		WHERE A.[IsCancel] = 1

	-- EMULAMOS UNA NUEVA TABLA EN BASE A #AudioTemp DONDE SE REMOVERAN TODOS LOS REGISTROS DUPLICADOS PARA EL PK DE LA TABLA Audio
	;WITH CTE AS (
		SELECT A.[VCC], A.[ID], A.[Section], A.[Part], A.[Number],
				A.[Gender], A.[SilenceTime], A.[DurationTime], A.[Category1], A.[Category2],
				A.[Category3], A.[Description], A.[Campaign], A.[Agent], A.[AgentName],--ISNULL(A.[AgentName], AG.[Name]) 'AgentName',
				A.[Type], A.[Init], A.[StartDate], A.[EndDate], A.[DispositionCode], 
				A.[RelativePath], A.[FileName], A.[CreatedByUserId], A.[ContactId], A.[ContactName], 
				A.[ContactAddress], A.[NeedI6Update], A.[RelativeFileName], A.[Ender], A.[Ticket], 
				RN = ROW_NUMBER() OVER(PARTITION BY A.[VCC], A.[ID], A.[Section], A.[Part] ORDER BY A.[ID])
			FROM #AudioTemp A (NOLOCK)
	),
	FILTERED_CTE AS (
		SELECT CTE.*, A.[ID]    'Exists'
			FROM CTE 
			LEFT JOIN [dbo].[Audio] A (NOLOCK)
				ON A.[VCC] = CTE.[VCC]
					AND A.[ID] = CTE.[ID]
					AND A.[Section] = CTE.[Section]
					AND A.[Part] = CTE.[Part]
			WHERE RN = 1
	)-- SELECT * FROM FILTERED_CTE WHERE [Exists] IS NULL
	-- MOVEMOS LA DATA FILTRADA EN UNA TEMPORAL
	SELECT * 
		INTO [dbo].[#FilteredAudioTemp]
		FROM FILTERED_CTE

	-- LA TABLA TEMPORAL #AudioTemp LLEGA CON ALGUNOS REGISTROS DUPLICADOS DEBIDO A LOS CRUCES AL APLICAR LOR FILTROS, POR ELLO DEBEMOS 
	-- IDENTIFICAR LOS REPETIDOS, RN = ROW_NUMBER() OVER(PARTITION BY ...) <- NUMERA CADA REGISTRO SEGUN EL(LOS) VALOR(ES) DE ...

	-- EL CTE SOLO PUEDE SER INVOCADO UNA VEZ POR ESO LO PONEMOS EN EL SELECT FINAL DONDE EL CONTADOR(RN) ES 1
	--DELETE FROM CTE WHERE RN > 1
	
	PRINT CONCAT('INSERTING TO AUDIO TABLE ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	INSERT INTO dbo.Audio(
			[VCC]
			,[ID]
			,[Section]
			,[Part]
			,[Number]
			,[Gender]
			,[SilenceTime]
			,[DurationTime]
			,[Category1]
			,[Category2]
			,[Category3]
			,[Description]
			,[Campaign]
			,[Agent]
			,[AgentName]
			,[Type]
			,[Init]
			,[NeedI6Update]
			,[StartDate]
			,[EndDate]
			,[DispositionCode]
			,[RelativePath]
			,[FileName]
			,[CreatedDate]
			,[CreatedByUserId]
			,[TimeStamp]
			,[ContactId]			
			,[ContactName]
			,[ContactAddress]
			,[RelativeFileName]
			,[Ender]
			,[Ticket]
			) 
		SELECT
			 A.[VCC]
			,A.[ID]
			,A.[Section]
			,A.[Part]
			,A.[Number]
			,A.[Gender]
			,A.[SilenceTime]
			,A.[DurationTime]
			,A.[Category1]
			,A.[Category2]
			,A.[Category3]
			,A.[Description]
			,A.[Campaign]
			,A.[Agent]
			,A.[AgentName]
			,A.[Type]
			,A.[Init]
			,A.[NeedI6Update]
			,A.[StartDate]
			,A.[EndDate]
			,A.[DispositionCode]
			,A.[RelativePath]
			,A.[FileName]
			,GETUTCDATE()
			,A.[CreatedByUserId]
			,GETUTCDATE()
			,A.[ContactId]		
			,A.[ContactName]
			,A.[ContactAddress]
			,A.[RelativeFileName]
			,A.[Ender]
			,A.[Ticket]
		FROM [dbo].[#FilteredAudioTemp] A (NOLOCK)
		WHERE A.[Exists] IS NULL


	-- EMULAMOS UNA NUEVA TABLA EN BASE A #AudioTemp DONDE SE REMOVERAN TODOS LOS REGISTROS DUPLICADOS PARA EL PK DE LA TABLA Audio
	;WITH CTE_PATT AS (
		SELECT A_TMP.[VCC], A_TMP.[ProcessId], A_TMP.[ID], A_TMP.[Section], A_TMP.[Part],
				GETUTCDATE() 'CreatedDate', A_TMP.[CreatedByUserId], RN = ROW_NUMBER() OVER(PARTITION BY A_TMP.[VCC], A_TMP.[ProcessId], A_TMP.[Id], A_TMP.[Section], A_TMP.[Part] ORDER BY A_TMP.[ID])
			FROM #AudioTemp A_TMP (NOLOCK)
	),
	FILTERED_CTE_PATT AS (
		SELECT CTE_PATT.*, PA.[AudioId]    'Exists'
			FROM CTE_PATT 
			LEFT JOIN [dbo].[ProcessAudio] PA (NOLOCK)
				ON PA.[VCC] = CTE_PATT.[VCC]
					AND PA.[ProcessId] = CTE_PATT.[ProcessId]
					AND PA.[AudioId] = CTE_PATT.[ID]
					AND PA.[Section] = CTE_PATT.[Section]
					AND PA.[Part] = CTE_PATT.[Part]
			WHERE RN = 1
	)
	SELECT * 
		INTO [dbo].[#FilteredProcessAudioTemp]
		FROM FILTERED_CTE_PATT

	-- EL CTE_PAT SOLO PUEDE SER INVOCADO UNA VEZ POR ESO LO PONEMOS EN EL SELECT FINAL DONDE EL CONTADOR(RN) ES 1
	--DELETE FROM CTE_PAT WHERE RN > 1
		--select top 5 * from dbo.ProcessAudio
	-- Insertamos registros en la tabla ProcessAudio
	INSERT INTO dbo.ProcessAudio(
			[VCC]
			,[ProcessId]
			,[AudioId]
			,[Section]
			,[Part]
			,[CreatedDate]
			,[CreatedByUserId]
			,[TimeStamp]
		) 
		SELECT PAT.[VCC]
			,PAT.[ProcessId]
			,PAT.[ID]
			,PAT.[Section]
			,PAT.[Part]
			,GETUTCDATE()
			,PAT.[CreatedByUserId]
			,GETUTCDATE()
		FROM [dbo].[#FilteredProcessAudioTemp] PAT (NOLOCK)
		WHERE PAT.[Exists] IS NULL

	DROP TABLE [dbo].[#AudioTemp]
	DROP TABLE [dbo].[#FilteredAudioTemp]
	DROP TABLE [dbo].[#FilteredProcessAudioTemp]
		--DROP TABLE #ProcessAudioTemp

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkAuditorEvaluationPlan]
AS
BEGIN
	SET NOCOUNT ON

	/*
	IF OBJECT_ID(N'[dbo].[TMP_AUDITOR_EVALUATION_PLAN]', N'U') IS NULL
	BEGIN
		SELECT * 
			INTO [dbo].[TMP_AUDITOR_EVALUATION_PLAN]
			FROM [dbo].[#AuditorEvaluationPlanTemp]
		
		/*
		SELECT *
			INTO #AuditorEvaluationPlanTemp
			FROM TMP_AUDITOR_EVALUATION_PLAN
		*/

		-- SELECT * FROM TMP_AUDITOR_EVALUATION_PLAN
		-- SELECT * FROM dbo.[#AuditorEvaluationPlanTemp]
		--RAISERROR('DATABASE_ERROR_DEBUGGING', 16, 16)
	END
	*/
	
	-- ACTUALIZAMOS LOS REGISTROS QUE YA EXISTAN BD
	UPDATE AEP
		SET AEP.[InteractionOrigin] = TMP_AEP.[InteractionOrigin],
			AEP.[InteractionFormat] = TMP_AEP.[InteractionFormat],
			AEP.[ContactId] = TMP_AEP.[InteractionContactId],
			AEP.[LastModifiedDate] = GETUTCDATE(),
			AEP.[TimeStamp] = GETUTCDATE()
		FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
		INNER JOIN [#AuditorEvaluationPlanTemp] TMP_AEP
			ON AEP.[Instance] = TMP_AEP.[VCC]
				AND TMP_AEP.[EvaluationPlanId] = AEP.[IdEvaluationPlan]
				AND TMP_AEP.[AuditorId] = AEP.[IdAuditor]
				AND TMP_AEP.[InteractionId] = AEP.[IdInteraction]
				AND TMP_AEP.[AgentId] = AEP.[IdAgent]
				AND TMP_AEP.[InteractionType] = AEP.[InteractionType]

	-- VOLCAMOS LOS REGISTROS QUE NO EXISTAN EN LA BD
	;WITH CTE_NEW_AUDITOR_EVALUATION_PLAN AS (
		SELECT TMP_AEP.* 
			FROM [#AuditorEvaluationPlanTemp] TMP_AEP
			LEFT JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				ON AEP.[Instance] = TMP_AEP.[VCC]
					AND AEP.[IdEvaluationPlan] = TMP_AEP.[EvaluationPlanId]
					AND AEP.[IdAuditor] = TMP_AEP.[AuditorId]
					AND AEP.[IdInteraction] = TMP_AEP.[InteractionId]
					AND AEP.[IdAgent] = TMP_AEP.[AgentId]
					AND AEP.[InteractionType] = TMP_AEP.[InteractionType]
			WHERE AEP.[Instance] IS NULL 
	)
		-- INSERTMAOS LA DATA EN LA TABLA AEP
		INSERT INTO [dbo].[AuditorEvaluationPlan] (
				[Instance], [IdEvaluationPlan], [idCampaign], [IdAgent], [IdAuditor],
				[IdInteraction], [InteractionType], [InteractionDate], [InteractionDateEnd], [InteractionDuration],
				[ManagementResult], [InteractionOrigin], [InteractionFormat], [ContactId], [ContactAddress], 
				[ContactName], [Status], [StatusDescription], [Completed], [Score],
				[CreatedByUserId], [CreatedDate], [TimeStamp]
			)
			SELECT CTE_AEP.[VCC], CTE_AEP.[EvaluationPlanId], ISNULL(CTE_AEP.[CampaignId], ''), CTE_AEP.[AgentId], CTE_AEP.[AuditorId],
					CTE_AEP.[InteractionId], CTE_AEP.[InteractionType], CTE_AEP.[InteractionStartDate], CTE_AEP.[InteractionEndDate], CTE_AEP.[InteractionDuration],
					CTE_AEP.[InteractionManagementResult], CTE_AEP.[InteractionOrigin], CTE_AEP.[InteractionFormat], CTE_AEP.[InteractionContactId], CTE_AEP.[InteractionContactAddress],
					CTE_AEP.[InteractionContactName], 1, 'NEW', 0, 0,
					'system', GETUTCDATE(), GETUTCDATE()
				FROM CTE_NEW_AUDITOR_EVALUATION_PLAN CTE_AEP 


	-- ELIMINAMOS LA TABLA TEMPORAL 
	DROP TABLE [#AuditorEvaluationPlanTemp]
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkEvaluationPlanAbility]
AS
BEGIN
	SET NOCOUNT ON
	-- ESTE PROCEDURE RECIBE COMO RESULTADO LAS INTERACCIONES QUE COINCIDEN CON UN QUERY EN UN PLAN DE EVALUACION
	
	-- INSERTAMOS LA DATA DE LA TEMPORAL PARA DEPURAR EL SP MANUALMENTE
	/*
	SELECT *
		INTO [dbo].[TMP_BULK_MODEL]
		FROM [dbo].[#StudyModelTemp]
	*/

	-- INSERTAMOS LA DATA DE LA TEMPORAL PARA DEPURAR EL SP MANUALMENTE
	IF OBJECT_ID(N'[dbo].[TMP_BULK_EVALUATION_PLAN_INTENT]', N'U') IS NULL
	BEGIN
		SELECT *
			INTO [dbo].[TMP_BULK_EVALUATION_PLAN_INTENT]
			FROM [dbo].[#EvaluationPlanAbilityTemp]

		/*
		SELECT * FROM [dbo].[TMP_BULK_EVALUATION_PLAN_INTENT] (NOLOCK)
		SELECT * 
			INTO [#EvaluationPlanAbilityTemp]
			FROM [dbo].[TMP_BULK_EVALUATION_PLAN_INTENT] (NOLOCK)
		SELECT * FROM [#EvaluationPlanAbilityTemp]
		*/
	END
	ELSE
	BEGIN
		INSERT INTO [dbo].[TMP_BULK_EVALUATION_PLAN_INTENT]
			SELECT * 
				FROM [dbo].[#EvaluationPlanAbilityTemp]
	END

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS PLANES Y SU CORRESPONDIENTE VCC
	DECLARE MY_CURSOR CURSOR 
	LOCAL STATIC READ_ONLY FORWARD_ONLY -- SCROLL 
		FOR 
		-- CONSULTAMOS LOS ID DE PLANES EN LA TABLA TEMPORAL
		SELECT DISTINCT
				EPATMP.[VCC]						'VCC', 
				EPATMP.[EvaluationPlanId]			'EVALUATION_PLAN_ID'
			FROM [dbo].[#EvaluationPlanAbilityTemp] EPATMP (NOLOCK)

	-- DECLARARAMOS VARIABLES PARA INDICAR EL PLAN Y VCC
	DECLARE @CURRENT_VCC VARCHAR(50) = ''
	DECLARE @CURRENT_EVALUATION_PLAN_ID VARCHAR(100) = ''

	-- ABRIMOS Y RECORREMOS EL ARRAY DE CANALES
	OPEN MY_CURSOR
	FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID
	WHILE @@FETCH_STATUS = 0
	BEGIN 
		-- EJECUTAMOS EL SP QUE INSERTA LOS RESULT BASE DE LOS PLANES QUE AUN NO LOS TIENEN
		EXEC [dbo].[SaveEvaluationBaseResult] @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID, 'AUTOMATIC'

		-- INSERTAMOS LOS RESULTADOS DE LAS INTERACCIONES DE AUDIO QUE MATCHEARON CON LA QUERY
		INSERT INTO [dbo].[InteractionQueryResult] (
			[VCC],
			[InteractionId],
			[Section],
			[Part],
			[InteractionType],
			[TranscriptionId],
			[BatchType],
			[BatchTypeId],
			[BatchTypeName],
			[AnalysisType],
			[QueryId],
			[QueryName],
			[QueryValue],
			[QueryData],
			[Matches],
			[Details],
			[CreatedDate],
			[TimeStamp]
		)
			SELECT DISTINCT @CURRENT_VCC,
					PA.[AudioId],
					PA.[Section],
					PA.[Part],
					'CALL', --EPATMP.[Channel],
					CONCAT(PA.[AudioId], '_', ISNULL(PA.[Section], '0'), '_', ISNULL(PA.[Part], '0')),
					'EVALUATION_PLAN',
					@CURRENT_EVALUATION_PLAN_ID,
					EP.[Description],
					'SEMANTIC',
					GQ.[Name],
					GQ.[Name],
					GQ.[Query],
					GQ.[RawData],
					EPATMP.[Matches],
					EPATMP.[MatchesDetail],
					GETUTCDATE(),
					GETUTCDATE()
				FROM [#EvaluationPlanAbilityTemp] EPATMP (NOLOCK)
				INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
					ON EP.[Instance] = EPATMP.[VCC]
						AND EP.[ID] = EPATMP.[EvaluationPlanId]
				INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK)
					ON PA.[VCC] = EPATMP.[VCC]
						AND PA.[ProcessId] = EPATMP.[ProcessId]
						AND PA.[AudioId] = EPATMP.[AudioId]
						AND PA.[Section] = EPATMP.[Section]
						AND PA.[Part] = EPATMP.[Part]
				INNER JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
					ON AEF.[Instance] = EPATMP.[VCC]
						AND AEF.[IdEvaluationFormat] = EP.[IdEvaluationFormat]
				/*
				INNER JOIN [dbo].[AbilityQuery] AQ (NOLOCK)
					ON AQ.[VCC] = EPATMP.[VCC]
						AND AQ.[AbilityId] = AEF.[IdAbility] 
						AND AQ.[QueryId] = EPATMP.[QueryId]
				*/
				INNER JOIN [GenericQuery] GQ (NOLOCK)
					ON GQ.[VCC] = EPATMP.[VCC]
						AND GQ.[Name] = EPATMP.[QueryId]
				WHERE EPATMP.[VCC] = @CURRENT_VCC
					AND EPATMP.[EvaluationPlanId] = @CURRENT_EVALUATION_PLAN_ID

		-- INSERTAMOS LOS RESULTADOS DE LAS INTERACCIONES DE TEXTO QUE MATCHEARON CON LA QUERY
		INSERT INTO [dbo].[InteractionQueryResult] (
			[VCC],
			[InteractionId],
			[Section],
			[Part],
			[InteractionType],
			[TranscriptionId],
			[BatchType],
			[BatchTypeId],
			[BatchTypeName],
			[AnalysisType],
			[QueryId],
			[QueryName],
			[QueryValue],
			[QueryData],
			[Matches],
			[Details],
			[CreatedDate],
			[TimeStamp]
		)
			SELECT @CURRENT_VCC,
					PAI.[InteractionId],
					NULL,
					NULL,
					EPATMP.[Channel],
					EPATMP.[AudioId],
					'EVALUATION_PLAN',
					@CURRENT_EVALUATION_PLAN_ID,
					EP.[Description],
					'SEMANTIC',
					GQ.[Name],
					GQ.[Name],
					GQ.[Query],
					GQ.[RawData],
					EPATMP.[Matches],
					EPATMP.[MatchesDetail],
					GETUTCDATE(),
					GETUTCDATE()
				FROM [#EvaluationPlanAbilityTemp] EPATMP (NOLOCK)
				INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
					ON EP.[Instance] = EPATMP.[VCC]
						AND EP.[ID] = EPATMP.[EvaluationPlanId]
				INNER JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
					ON PAI.[VCC] = EPATMP.[VCC]
						AND PAI.[ProcessId] = EPATMP.[ProcessId]
						AND PAI.[InteractionType] = EPATMP.[Channel]
						AND PAI.[FormattedEndDate] = EPATMP.[EndDate]
						AND PAI.[Campaign] = EPATMP.[Campaign]
						/* LA TEMPORAL DE TYPESCRIPT REMOVIA EL PREFIJO, POR ELLO SE CAMBIO PARA MATCHEAR CON EL ID
						AND (
							CONCAT(
									PAI.[InteractionType], 
									'_', 
									PAI.[Campaign], 
									'_', 
									PAI.[FormattedEndDate], 
									'_', 
									REPLACE(PAI.[InteractionId], ':', '_')
							) = EPATMP.[AudioId]
						)
						*/
						AND REPLACE(PAI.[InteractionId], ':', '_') = EPATMP.[AudioId]
				INNER JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
					ON AEF.[Instance] = EPATMP.[VCC]
						AND AEF.[IdEvaluationFormat] = EP.[IdEvaluationFormat]
				/*
				INNER JOIN [dbo].[AbilityQuery] AQ (NOLOCK)
					ON AQ.[VCC] = EPATMP.[VCC]
						AND AQ.[AbilityId] = AEF.[IdAbility] 
						AND AQ.[QueryId] = EPATMP.[QueryId]
				*/
				INNER JOIN [GenericQuery] GQ (NOLOCK)
					ON GQ.[VCC] = EPATMP.[VCC]
						AND GQ.[Name] = EPATMP.[QueryId]
				WHERE EPATMP.[VCC] = @CURRENT_VCC
					AND EPATMP.[EvaluationPlanId] = @CURRENT_EVALUATION_PLAN_ID
		
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID
	END
	CLOSE MY_CURSOR
	DEALLOCATE MY_CURSOR
	-- FIN MEDIDA PARA PLANES FINALIZADOS QUE NO TIENE RESULTS REGISTRADOS

	
	
	
	--ACTUALIZAR LOS SCORES DE LAS HABILIDADES RETORNADAS EN LA TABLA TEMPORAL
	UPDATE ER 
	SET ER.[Qualification] = (S.[Value] * AEF.[Score] / 100), 
		ER.[OwnScore] = (AEF.[Score] * S.[Value]) / 100,
		ER.[ScoreDescription] = S.[Description],
		ER.[AbilityInnerScore] = S.[Value]
	/*
	SELECT (S.[Value] * AEF.[Score] / 100), 
			(AEF.[Score] * S.[Value]) / 100,
			S.[Description],
			S.[Value]
	*/
	FROM [dbo].[EvaluationResult] ER (NOLOCK)
	INNER JOIN [dbo].[#EvaluationPlanAbilityTemp] EPATMP (NOLOCK)
	--INNER JOIN [dbo].[DummyEvaluationPlanAbility] EPATMP (NOLOCK)
		ON ER.[Instance] = EPATMP.[VCC]
		AND ER.[IdEvaluationPlan] = EPATMP.[EvaluationPlanId]
		AND REPLACE(ER.[IdInteraction], ':', '_') = EPATMP.[AudioId] 
	LEFT JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
		ON AEF.[Instance] = ER.[Instance] 
		AND AEF.[IdEvaluationFormat] = ER.[IdEvaluationFormat]
		AND AEF.[IdAbility] = ER.[IdAbility]
	INNER JOIN [dbo].[Ability] A (NOLOCK) 
		ON A.[Instance] = ER.[Instance] 
		AND A.[ID] = ER.[IdAbility]
	INNER JOIN [dbo].[AbilityQuery] AB (NOLOCK)
		ON AB.[VCC] = ER.[Instance] 
			AND AB.[AbilityId] = AEF.[IdAbility]
			AND AB.[QueryId] = EPATMP.[QueryId]
	LEFT JOIN [dbo].[Score] S (NOLOCK) 
		ON S.[Instance] = ER.[Instance] 
		AND S.[IdAbility] = ER.[IdAbility]
		AND S.[Value] =  IIF(A.[IsNegative] = 1, 0, 100)

	;WITH 
	-- DECLARAMOS UNA CTE PARA DEPOSITAR LOS RESULTADOS DE LA SUMA DE HABILIDADES PARA LA EVALUACION
	CteEvaluationScoreResult AS (
		-- ESTE BLOQUE ACTUALIZA LAS NOTAS DE EVALUACIONES AUTOMATICAS SEGUN LOS PLANES CONTENIDOS EN LA TEMPORAL
		SELECT ER.[Instance], ER.[IdEvaluationPlan], ER.[IdEvaluator], ER.[IdInteraction], SUM(ER.[Qualification]) TOTAL
			FROM EvaluationResult ER 
			INNER JOIN (SELECT DISTINCT EvaluationPlanId FROM #EvaluationPlanAbilityTemp) EPATMP 
			--INNER JOIN (SELECT DISTINCT EvaluationPlanId FROM DummyEvaluationPlanAbility) EPATMP 
				ON EPATMP.[EvaluationPlanId] = ER.[IdEvaluationPlan] 
			LEFT JOIN AbilityEvaluationFormat AEF 
				ON AEF.[Instance] = ER.[Instance]
					AND AEF.[IdEvaluationFormat] = ER.[IdEvaluationFormat]
					AND AEF.[IdAbility] = ER.[IdAbility]
					AND AEF.[ID] = ER.[IdAbilityEvaluationFormat]
			WHERE ER.[IdAuditor] = 'SPEECH'
			GROUP BY ER.[Instance], ER.[IdEvaluationPlan], ER.[IdEvaluator], ER.[IdInteraction]
	)
	
	-- SE ACTUALIZA LAS EVALUACIONES QUE COINCIDEN CON LOS RESULTADOS
	UPDATE AEP 
		SET AEP.[Score] = ESR.[TOTAL],
			AEP.[AutomaticScore] =  ESR.[TOTAL],
			AEP.[ManualScore] = 0,
			AEP.[LastModifiedDate] = GETUTCDATE(),
			AEP.[TimeStamp] = GETUTCDATE()
		FROM AuditorEvaluationPlan AEP (NOLOCK)
		INNER JOIN CteEvaluationScoreResult ESR 
			ON AEP.[Instance] = ESR.[Instance]
				AND AEP.[IdEvaluationPlan] = ESR.[IdEvaluationPlan]
				AND AEP.[IdAuditor] = ESR.[IdEvaluator]
				AND AEP.[IdInteraction] = ESR.[IdInteraction]

	/****************************************************************************************/
	/*INICIO SECCION PARA ACTUALIZAR LA CALIFICACION TOTAL DE LA EVALUACION C/S PENALIZACION*/
	/****************************************************************************************/
	
	DECLARE @EVALUATION_PLANS VARCHAR(500) = NULL
	SELECT @EVALUATION_PLANS = ISNULL(@EVALUATION_PLANS + ', ', '') + D.[EvaluationPlanId]
		FROM (SELECT DISTINCT [EvaluationPlanId] FROM [dbo].[#EvaluationPlanAbilityTemp]) D

	-- EMULAMOS UNA NUEVA TABLA VARIABLE PARA DETERMINAR LAS INTERACCIONES QUE TIENEN HABILIDAD(ES) PENALIZADA(S) POR NOTA CERO
	--  ORDENANDO POR PRIORIDAD (INTERACCION 1, GRUPO 2 Y SUBGRUPO 3)
	;WITH CTEFP AS (
		SELECT DISTINCT 
				ER.[Instance],
				ER.[IdInteraction],
				ER.[IdEvaluationPlan],
				ER.[IdEvaluationFormat],
				ER.[IdAbility],
				CASE WHEN ER.[AbilityInnerScore] = 0 AND ER.[NotApplies] = 0 AND HAEF.[Penalizes] <> 'NO' THEN 1 ELSE 0 END 'Penalized',
				HAEF.[Penalizes],
				CASE HAEF.[Penalizes]
					WHEN 'INT' THEN ER.[IdInteraction] 
					WHEN 'GRP' THEN ER.[IdGroup] 
					WHEN 'SGR' THEN ER.[IdSubGroup] 
				END 'PenalizedOn',
				RN = ROW_NUMBER() OVER(PARTITION BY ER.[IdInteraction], CASE WHEN ER.[AbilityInnerScore] = 0 THEN 1 ELSE 0 END 
					ORDER BY CASE HAEF.[Penalizes]
					WHEN 'INT' THEN 1 
					WHEN 'GRP' THEN 2 
					WHEN 'SGR' THEN 3 
				END)
			FROM [dbo].[EvaluationResult] ER (NOLOCK) 
			LEFT JOIN [dbo].[HistoricAbilityEvaluationFormat] HAEF (NOLOCK) 
				ON HAEF.[VCC] = ER.[Instance]
					AND HAEF.[EvaluationPlanId] = ER.[IdEvaluationPlan]
					AND HAEF.[IdEvaluationFormat] = ER.[IdEvaluationFormat]
					AND HAEF.[IdAbility] = ER.[IdAbility]
			WHERE ER.[IdEvaluationPlan] IN (SELECT Value FROM [dbo].[split](@EVALUATION_PLANS,',')) 
			--WHERE ER.[IdEvaluationPlan] IN (SELECT DISTINCT EvaluationPlanId FROM DummyEvaluationPlanAbility) 
				AND ER.[AbilityInnerScore] = 0
                AND ER.[NotApplies] = 0
				AND HAEF.[Penalizes] <> 'NO'
			-- WHERE ER.[IdEvaluationPlan] IN (SELECT DISTINCT EPA.[EvaluationPlanId] FROM [DummyEvaluationPlanAbility] EPA (NOLOCK) WHERE EPA.[EvaluationPlanId] IS NOT NULL)
	),
	--SELECT * FROM CTEFP

	-- ALMACENAMOS EL RESULTADO AGRUPADO DE LAS EVALUACIONES EN UNA TABLA VARIABLE PARA HACER UN CRUCE DIRECTO CON
	--  LA TABLA AuditorEvaluationPlan
	CTEFS AS (
		SELECT ER.[Instance],
				ER.[IdEvaluationPlan],
				ER.[IdInteraction],
				ER.[IdAuditor],
				SUM(ER.[Qualification])															'TOTAL_SCORE',
				SUM(IIF(HA.[Type] = 'MANUAL_AUTOMATIC', ER.[Qualification], 0))					'TOTAL_AUTOMATIC_SCORE',
				SUM(IIF(HA.[Type] = 'MANUAL', ER.[Qualification], 0))							'TOTAL_MANUAL_SCORE',
				--SUM(ER.[Qualification] * CASE WHEN FP2.[Penalized] IS NULL THEN 1 ELSE 0 END) 'PENALIZED_SCORE',
				SUM(ER.[Qualification] * (
					CASE WHEN (
						CASE FP2.[Penalizes]
							WHEN 'INT' THEN ER.[IdInteraction] 
							WHEN 'GRP' THEN ER.[IdGroup] 
							WHEN 'SGR' THEN ER.[IdSubGroup] 
						END = FP2.[PenalizedOn]
						AND FP2.[Penalized] = 1) THEN 0 ELSE 1 END
				))																				'PENALIZED_SCORE',
				SUM(IIF(HA.[Type] = 'MANUAL_AUTOMATIC',
					ER.[Qualification] * (
					CASE WHEN (
						CASE FP2.[Penalizes]
							WHEN 'INT' THEN ER.[IdInteraction] 
							WHEN 'GRP' THEN ER.[IdGroup] 
							WHEN 'SGR' THEN ER.[IdSubGroup] 
						END = FP2.[PenalizedOn]
						AND FP2.[Penalized] = 1
					) THEN 0 ELSE 1 END),
					0) 
				)																				'PENALIZED_AUTOMATIC_SCORE',
				SUM(IIF(HA.[Type] = 'MANUAL',
					ER.[Qualification] * (
					CASE WHEN (
						CASE FP2.[Penalizes]
							WHEN 'INT' THEN ER.[IdInteraction] 
							WHEN 'GRP' THEN ER.[IdGroup] 
							WHEN 'SGR' THEN ER.[IdSubGroup] 
						END = FP2.[PenalizedOn]
						AND FP2.[Penalized] = 1
					) THEN 0 ELSE 1 END),
					0) 
				)																				'PENALIZED_MANUAL_SCORE'
			FROM [dbo].[EvaluationResult] ER (NOLOCK) 
			LEFT JOIN (
				-- HACEMOS UN CRUCE CON LA TABLA VARIABLE Y TOMAMOS SOLO UNA DE LAS PENALIZACIONES POR INTERACCION, EMPEZANDO 
				--  POR LA MAS PRIORITARIA (FP.[RN])
				SELECT DISTINCT FP.[Instance],
								FP.[IdEvaluationPlan], 
								FP.[IdInteraction], 
								FP.[Penalized],
								FP.[Penalizes], 
								FP.[PenalizedOn]
					FROM CTEFP FP (NOLOCK)
					WHERE FP.[Penalizes] <> 'NO'
						AND FP.[RN] = 1
			) FP2
				ON FP2.[Instance] = ER.[Instance]
					AND ER.[IdEvaluationPlan] = FP2.[IdEvaluationPlan]
					AND ER.[IdInteraction] = FP2.[IdInteraction]
					/*
					AND CASE FP2.[Penalizes]
							WHEN 'INT' THEN ER.[IdInteraction] 
							WHEN 'GRP' THEN ER.[IdGroup] 
							WHEN 'SGR' THEN ER.[IdSubGroup] 
						END = FP2.[PenalizedOn]
					AND FP2.[Penalized] = 1
					*/
			INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
				ON HA.[VCC] = ER.[Instance]
					AND HA.[EvaluationPlanId] = ER.[IdEvaluationPlan]
					AND HA.[ID] = ER.[IdAbility]
			--WHERE ER.[Instance] = 'multisalud'
			WHERE -- ER.[Instance] = FP2.[Instance] -- ESTA LINEA CAUSA QUE NO SE ACTUALICEN LOS SCORES LAS EVALUACIONES QUE NO PENALIZAN
				-- AND 
				ER.[IdEvaluationPlan] IN (SELECT Value FROM [dbo].[split](@EVALUATION_PLANS,','))
				--AND ER.[IdEvaluationPlan] IN (SELECT DISTINCT EvaluationPlanId FROM DummyEvaluationPlanAbility)
			GROUP BY ER.[Instance],
				ER.[IdEvaluationPlan],
				ER.[IdInteraction],
				ER.[IdAuditor]
	)
	--SELECT * FROM CTEFS

	-- ACTUALIZAMOS LOS REGISTROS DE EVALUACION CON LA NOTA PENALIZADA OBTENIDA EN LOS PASOS ANTERIORES
	UPDATE AEP 
		SET AEP.[Score] = FS.[PENALIZED_SCORE],
			AEP.[AutomaticScore] = FS.[PENALIZED_AUTOMATIC_SCORE],
			AEP.[ManualScore] = FS.[PENALIZED_MANUAL_SCORE],
			AEP.[LastModifiedDate] = GETUTCDATE(),
			AEP.[TimeStamp] = GETUTCDATE()
		FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
		LEFT JOIN CTEFS FS (NOLOCK) 
			ON FS.[Instance] = AEP.[Instance] 
				AND FS.[IdEvaluationPlan] = AEP.[IdEvaluationPlan] 
				AND FS.[IdInteraction] = AEP.[IdInteraction]
				AND FS.[IdAuditor] = AEP.[IdAuditor]
		WHERE FS.[Instance] IS NOT NULL
		
	/*************************************************************************************/
	/*FIN SECCION PARA ACTUALIZAR LA CALIFICACION TOTAL DE LA EVALUACION C/S PENALIZACION*/
	/*************************************************************************************/
	--insert into test(origin,content) values('BulkAbilityCognitive',(select *  from [dbo].[#AbilityCognitiveTemp] FOR JSON AUTO))

	-- ELIMINAMOS LA TABLA TEMPORAL
	DROP TABLE [dbo].[#EvaluationPlanAbilityTemp]
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkEvaluationPlanQuery]
AS
BEGIN
	SET NOCOUNT ON
	-- ESTE SP ESTA DESTINADO A ACTUALIZAR RESULTADOSD EL PASO 3 DEL PLAN DE EVALUACION

	/*
	CREATE TABLE TMP_EVALUATION_PLAN_QUERY(
		[VCC] VARCHAR(50)
		  ,[ProcessId] VARCHAR(50)
		  ,[EvaluationPlanId] VARCHAR(50)
		  ,[AudioId] VARCHAR(250)
		  ,[Section] INT
		  ,[Part] INT
		  ,[QueryId] VARCHAR(50)
		  ,[EndDate] VARCHAR(50)
		  ,[Channel] VARCHAR(50)
		  ,[Campaign] VARCHAR(50)
		  ,[CreatedDate] DATETIME
	)
	select * from TMP_EVALUATION_PLAN_QUERY ORDER BY CreatedDate DESC
	*/
	/*
	insert into TMP_EVALUATION_PLAN_QUERY (
			[VCC]
		  ,[ProcessId]
		  ,[EvaluationPlanId]
		  ,[AudioId]
		  ,[Section]
		  ,[Part]
		  ,[QueryId]
		  ,[EndDate]
		  ,[Channel]
		  ,[Campaign]
		  ,[CreatedDate]
		  )  
	SELECT [VCC]
		  ,[ProcessId]
		  ,[EvaluationPlanId]
		  ,[AudioId]
		  ,[Section]
		  ,[Part]
		  ,[QueryId]
		  ,[EndDate]
		  ,[Channel]
		  ,[Campaign]
		  ,CURRENT_TIMESTAMP
	  FROM [dbo].[#EvaluationPlanQueryTemp]
	  */
	  
	--select * from Dummy2 where AudioId = '60813629052019153349'
	--delete from Dummy2 where EvaluationPlanId <> '4f8baccd-c974-439c-b37a-0ac4ce34ada1'
	--truncate table Dummy2

	UPDATE IEP
	SET IEP.[QueryPassed] = 1
	FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
	LEFT JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
		ON EP.[Instance] = IEP.[Instance]
			AND EP.[ID] = IEP.[IdEvaluationPlan]
	LEFT JOIN [dbo].[#EvaluationPlanQueryTemp] IEPTMP (NOLOCK)
		ON IEPTMP.[VCC] = IEP.[Instance]
			AND IEPTMP.[EvaluationPlanId] = IEP.[IdEvaluationPlan]
			AND IEPTMP.[AudioId] = REPLACE(IEP.[IdInteraction], ':', '_')
	WHERE IEP.[Instance] = IEPTMP.[VCC]
		AND IEP.[IdEvaluationPlan] = IEPTMP.[EvaluationPlanId]
		AND EP.[ProcessId] = IEPTMP.[ProcessId]
		AND IEPTMP.[AudioId] IS NOT NULL;

	-- ELIMINAMOS LOS AUDIOS (INTERACCIONES) DE PROCESSAUDIO QUE NO PASARON LA VALIDACION DEL FILTRO POR CONTENIDO
	DELETE PA
	FROM [dbo].[ProcessAudio] PA
	INNER JOIN [dbo].[#EvaluationPlanQueryTemp] IEPTMP (NOLOCK)
		ON PA.[VCC] = IEPTMP.[VCC]
			AND PA.[ProcessId] = IEPTMP.[ProcessId]
			AND PA.[AudioId] = IEPTMP.[AudioId]
			AND PA.[Section] = IEPTMP.[Section]
			AND PA.[Part] = IEPTMP.[Part]
	WHERE PA.[VCC] = IEPTMP.[VCC]
		AND PA.[ProcessId] = IEPTMP.[ProcessId]
		AND PA.[AudioId] IN (
			SELECT IEP.[IdInteraction]
				FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				INNER JOIN [dbo].[#EvaluationPlanQueryTemp] IEPTMP (NOLOCK)
					ON IEP.[Instance] = IEPTMP.[vcc]
						AND IEP.[IdEvaluationPlan] = IEPTMP.[EvaluationPlanId]
				LEFT JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
					ON EP.[Instance] = IEP.[Instance]
						AND EP.[ID] = IEP.[IdEvaluationPlan]
				WHERE IEP.[Instance] = IEPTMP.[VCC]
					AND IEP.[IdEvaluationPlan]  = IEPTMP.[EvaluationPlanId]
					AND EP.[ProcessId] =  IEPTMP.[ProcessId]
					AND IEP.[QueryPassed] <> 1
		)

	DELETE PAI 
		FROM [dbo].[ProcessAsyncInteraction] PAI 
		INNER JOIN [dbo].[#EvaluationPlanQueryTemp] IEPTMP (NOLOCK)
			ON PAI.[VCC] = IEPTMP.[VCC]
				AND PAI.[ProcessId] = IEPTMP.[ProcessId]
				AND PAI.[InteractionId] = IEPTMP.[AudioId]
				AND PAI.[Campaign] = IEPTMP.[Campaign]
				AND PAI.[FormattedEndDate] = IEPTMP.[EndDate]
		WHERE PAI.[VCC] = IEPTMP.[VCC]
			AND PAI.[ProcessId] = IEPTMP.[ProcessId]
			AND REPLACE(PAI.[InteractionId], ':', '_' ) IN (
				SELECT IEP.[IdInteraction]
					FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
					INNER JOIN [dbo].[#EvaluationPlanQueryTemp] IEPTMP (NOLOCK)
						ON IEP.[Instance] = IEPTMP.[vcc]
							AND IEP.[IdEvaluationPlan] = IEPTMP.[EvaluationPlanId]
					LEFT JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
						ON EP.[Instance] = IEP.[Instance]
							AND EP.[ID] = IEP.[IdEvaluationPlan]
					WHERE IEP.[Instance] = IEPTMP.[VCC]
						AND IEP.[IdEvaluationPlan]  = IEPTMP.[EvaluationPlanId]
						AND EP.[ProcessId] =  IEPTMP.[ProcessId]
						AND IEP.[QueryPassed] <> 1
			)

	--UPDATE IEP SET IEP.[QueryPassed] = 0
	DELETE IEP
	FROM [dbo].[InteractionEvaluationPlan] IEP
	INNER JOIN [dbo].[#EvaluationPlanQueryTemp] IEPTMP (NOLOCK)
		ON IEP.[Instance] = IEPTMP.[VCC]
			AND IEP.[IdEvaluationPlan] = IEPTMP.[EvaluationPlanId]
	WHERE [Instance] = IEPTMP.[VCC]
		AND [IdEvaluationPlan] = IEPTMP.[EvaluationPlanId]
		AND [QueryPassed] IS NULL;

	-- INGRESAMOS UN REGISTRO EN AUDITOREVALUATIONPLAN POR CADA INTERACCION QUE PASO EL FILTRO
	INSERT INTO [AuditorEvaluationPlan] (
		[Instance], [IdEvaluationPlan], [IdAuditor],
		[IdInteraction], [InteractionType], [InteractionDate],
		[InteractionDateEnd], [ManagementResult], [InteractionOrigin], 
		[IdAgent], [idCampaign], [InteractionDuration],
		[InteractionFormat], [TimeStamp], [Completed],
		[Score], [StatusDescription], [LastModifiedByUserId]
		)
	SELECT DISTINCT IEP.[Instance], IEP.[IdEvaluationPlan], 'SPEECH', 
		IEP.[IdInteraction], IEP.[InteractionType], IEP.[InteractionDateStart],
		CASE IEP.[InteractionType] 
			WHEN 'CALL' THEN IEP.[InteractionDateEnd]
			ELSE (
				SELECT MAX(IEP_INNER.[InteractionDateEnd])
				  FROM [InteractionEvaluationPlan] IEP_INNER (NOLOCK)
				  WHERE IEP_INNER.[IdInteraction] = IEP.[IdInteraction]
					AND IEP_INNER.[IdEvaluationPlan] = IEP.[IdEvaluationPlan]
			) END,
		STUFF((
			SELECT ',' + IEP_INNER.[ManagementResult]
				FROM [InteractionEvaluationPlan] IEP_INNER (NOLOCK)
				WHERE IEP_INNER.[IdInteraction] = IEP.[IdInteraction]
					AND IEP_INNER.[IdEvaluationPlan] = IEP.[IdEvaluationPlan]
				FOR XML PATH(''), TYPE).value('.', 'NVARCHAR(MAX)'), 1, 1, ''),
		IEP.[InteractionOrigin],
		IEP.[IdAgent], IEP.[IdCampaign], IEP.[InteractionDuration], 
		IEP.[InteractionFormat], GETUTCDATE(), 0,
		0, 'NEW', 'SPEECH'
	FROM [InteractionEvaluationPlan] IEP (NOLOCK)
	INNER JOIN [dbo].[#EvaluationPlanQueryTemp] IEPTMP (NOLOCK)
		ON IEP.[Instance] = IEPTMP.[VCC]
			AND IEP.[IdEvaluationPlan] = IEPTMP.[EvaluationPlanId]
			AND IEP.[InteractionType] = IEPTMP.[Channel]
			AND (
					(
						IEP.[InteractionType] = 'CALL'
						AND IEP.[IdInteraction] = IEPTMP.[AudioId]
						AND IEP.[InteractionSection] = IEPTMP.[Section]
						AND IEP.[InteractionPart] = IEPTMP.[Part])
					OR
					(
						IEP.[InteractionType] <> 'CALL'
						AND REPLACE(IEP.[IdInteraction], ':', '_') = IEPTMP.[AudioId]
						AND IEP.[FormattedEndDate] = IEPTMP.[EndDate]
						AND IEP.[IdCampaign] = IEPTMP.[Campaign]
					)
				)
	LEFT JOIN [AuditorEvaluationPlan] AEP (NOLOCK)
		ON AEP.[Instance] = IEP.[Instance]
			AND AEP.[IdEvaluationPlan] = IEP.[IdEvaluationPlan]
			AND AEP.[IdInteraction] = IEP.[IdInteraction]
	WHERE IEP.[Instance] = IEPTMP.[VCC]
		AND IEP.[IdEvaluationPlan] = IEPTMP.[EvaluationPlanId]
		AND AEP.[Instance] IS NULL

	DROP TABLE [dbo].[#EvaluationPlanQueryTemp]
	--DROP TABLE [dbo].[#EvaluationPlanQueryTemp1]
	
	SET NOCOUNT OFF
END;
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkInteractionEvaluationPlan]
AS
BEGIN
	SET NOCOUNT ON

	-- DECLARAMOS UNA VARIABLE PARA DEFINIR EL VCC
	DECLARE @VCC VARCHAR(50) = NULL
	DECLARE @VCC_TYPE VARCHAR(50) = NULL
	DECLARE @NEED_I6_UPDATE BIT = 0 

	-- ASIGNAMOS A LA VARIABLE EL NOMBRE DEL VCC
	SELECT TOP 1 @VCC = [VCC] FROM [#InteractionEvaluationPlanTemp] (NOLOCK)

	-- BUSCAMOS EL TIPO DE VCC
	SELECT @VCC_TYPE = V.[Type] FROM [dbo].[VCC] V (NOLOCK) WHERE V.[VCC] = @VCC

	-- CAMBIAMOS EL VALOR DEL FLAG
	IF @VCC_TYPE = 'I6' SET @NEED_I6_UPDATE = 1
	
	-- ACTUALIZAMOS EL AGENT_NAME EN LA TABLA TEMPORAL
	UPDATE IEP_TMP 
		SET IEP_TMP.[AgentName] = IIF (ISNULL(IEP_TMP.[AgentName], '') = '', A.[Name], IEP_TMP.[AgentName])
 		FROM [#InteractionEvaluationPlanTemp] IEP_TMP
		INNER JOIN [dbo].[Agent] A (NOLOCK)
			ON A.[VCC] = IEP_TMP.[VCC]
				AND A.[AgentId] = IEP_TMP.[IdAgent]

	-- INSERTAMOS LA DATA DE LA TABLA TEMPORAL
	INSERT INTO [dbo].[InteractionEvaluationPlan] (
			[Instance], [IdEvaluationPlan], [IdAgent],
			[AgentName], [IdCampaign], [IdInteraction], 
			[FormattedEndDate], [InteractionSection], [InteractionPart], 
			[InteractionDateStart], [InteractionDateEnd], [InteractionType], 
			[InteractionMessages], [InteractionDuration], [InteractionOrigin], 
			[ManagementResult], [RelativePath], [FileName],
			[ContactId], [ContactAddress], [ContactName], 
			[InteractionFormat], [NeedI6Update], [RelativeFileName], 
			[Ender], [Ticket],
			[CreatedByUserId], [CreatedDate], [TimeStamp])
		SELECT IEP.[VCC], IEP.[IdEvaluationPlan], IEP.[IdAgent],
				IEP.[AgentName], IEP.[IdCampaign], IEP.[IdInteraction], 
				IEP.[InteractionFolder], IEP.[InteractionSection], IEP.[InteractionPart], 
				IEP.[InteractionDateStart], IEP.[InteractionDateEnd], IEP.[InteractionType], 
				IEP.[TotalMessages], IEP.[InteractionDuration], IEP.[InteractionOrigin], 
				IEP.[InteractionManagementResult], IEP.[RelativePath], IEP.[FileName],
				IEP.[InteractionContactId], IEP.[InteractionContactAddress], IEP.[InteractionContactName],
				IEP.[InteractionFormat], ISNULL(IEP.[NeedI6Update], 1),
				/*
				CASE WHEN @VCC_TYPE = 'OCC'
					THEN IIF(IEP.[InteractionType] = 'CALL', 0 , 1) 
					ELSE @NEED_I6_UPDATE
					END,
				*/
				[RelativeFileName],
				IEP.[Ender], IEP.[Ticket],
				IEP.[CreatedByUserId], GETUTCDATE(), GETUTCDATE()
			FROM [#InteractionEvaluationPlanTemp] IEP (NOLOCK)
			/*
			LEFT JOIN [dbo].[Agent] A (NOLOCK)
				ON A.[VCC] = IEP.[VCC]
					AND A.[AgentId] = IEP.[IdAgent]
			*/

	-- ELIMINAMOS LA TABLA TEMPORAL 
	DROP TABLE [#InteractionEvaluationPlanTemp]
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkInteractionProcess]
AS
BEGIN
	-- SE USA ESTE SP?
	UPDATE [dbo].[Audio] SET
		[dbo].[Audio].DurationLinguisticTime = [dbo].[#InteractionProcessTemp].DurationLinguisticTime,
		[dbo].[Audio].IsTranscribed = [dbo].[#InteractionProcessTemp].IsTranscribed,
		[dbo].[Audio].IsCancel = [dbo].[#InteractionProcessTemp].IsCancel,
		[dbo].[Audio].[FileName] = [dbo].[#InteractionProcessTemp].[FileName],
		[dbo].[Audio].[ErrorMessage] = [dbo].[#InteractionProcessTemp].[ErrorMessage],
		[dbo].[Audio].LastModifiedDate = GETUTCDATE()
	FROM [dbo].[#InteractionProcessTemp]
	WHERE
		[dbo].[Audio].VCC = [dbo].[#InteractionProcessTemp].VCC
		AND [dbo].[Audio].ID = [dbo].[#InteractionProcessTemp].ID
		AND [dbo].[Audio].Section = [dbo].[#InteractionProcessTemp].Section
		AND [dbo].[Audio].Part = [dbo].[#InteractionProcessTemp].Part

	  UPDATE Process
	  SET [Status] = CASE 
		WHEN total <= transcription + cancel AND cancel > transcription THEN 'FAILED' 
		WHEN total <= transcription + cancel THEN 'COMPLETE' 
		ELSE 'INPROCESS_TRANSCRIPTION' 
		END
	  FROM (
		  SELECT 
			COUNT(1)					total
			,COUNT(C.isCancel)			cancel
			,COUNT(C.IsTranscribed)		transcription 
			,B.ProcessId				processId
			,B.vcc						vcc
		  FROM [ProcessAudio] A (NOLOCK)
		  INNER JOIN (SELECT DISTINCT vcc, processId FROM [#InteractionProcessTemp]) B ON A.[vcc] = B.[vcc] AND A.ProcessId = B.ProcessId 
		  INNER JOIN Audio C (NOLOCK) ON A.VCC = C.VCC AND A.AudioId =C.ID AND ISNULL(A.Section,0) = ISNULL(C.Section,0) AND ISNULL(A.Part,0) = ISNULL(C.Part,0)
		  GROUP BY B.ProcessId, B.vcc
	  ) B
	  WHERE  B.VCC = [Process].VCC AND B.ProcessId = Process.[Name]

	 UPDATE EvaluationPlan
	  set [Status] = 5
	  --SELECT * 
	  FROM EvaluationPlan A (NOLOCK)
	  INNER JOIN Process B (NOLOCK) ON A.Instance = B.Vcc AND A.ProcessId =  B.[Name] AND B.[Status] = 'COMPLETE'
	  WHERE A.Evaluation_Mode = 'A' AND A.FilterByContentQueryName IS NULL AND A.[Status] < 5

	  UPDATE EvaluationPlan
	  set [Status] = 4
	  --SELECT * 
	  FROM EvaluationPlan A (NOLOCK)
	  INNER JOIN Process B (NOLOCK) ON A.Instance = B.Vcc AND A.ProcessId =  B.[Name] AND B.[Status] = 'COMPLETE'
	  WHERE A.Evaluation_Mode = 'A' AND A.FilterByContentQueryName IS NOT NULL AND A.[Status] < 4

	DROP TABLE [dbo].[#InteractionProcessTemp]
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkInteractionQuery]
AS
BEGIN
	SET NOCOUNT ON

	INSERT INTO [dbo].[AudioQuery]
		([VCC]
		  ,[ProcessId]
		  ,[AudioId]
		  ,[Section]
		  ,[Part]
		  ,[QueryId]
		  ,[LastModifiedDate]
		  )
	SELECT DISTINCT
		A.[VCC]
		,A.[ProcessId]
		,A.[AudioId]
		,A.[Section]
		,A.[Part]
		,A.[QueryId]
		,GETUTCDATE()
	FROM [dbo].[#InteractionQueryTemp] A
	LEFT JOIN [AudioQuery] B (NOLOCK) ON A.[VCC] = B.[VCC]
		AND A.[ProcessId] = B.[ProcessId] 
		AND A.[AudioId] = B.[AudioId]
		AND A.[Section] = B.[Section]
		AND A.[Part] = B.[Part]
		AND A.[QueryId] = B.[QueryId]
	INNER JOIN [dbo].[ProcessAudio] C ON A.[VCC] = C.[VCC]
		AND A.[ProcessId] = C.[ProcessId]
		AND A.[ProcessId] = C.[ProcessId] 
		AND A.[AudioId] = C.[AudioId]
		AND ISNULL(A.[Section],0) = C.[Section]
		AND ISNULL(A.[Part],0) = C.[Part]
	WHERE B.[ProcessId] IS NULL

	DROP TABLE [dbo].[#InteractionQueryTemp]

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkInteractionQueryStudy]
AS
BEGIN
	SET NOCOUNT ON
	-- SP PARA EL VOLCADO DE RESULTADOS DE ANALISIS SEMANTICO EN ESTUDIOS

	/*
	DROP TABLE TMP_INTERACTION_QUERY_STUDY
	CREATE TABLE TMP_INTERACTION_QUERY_STUDY (
		VCC VarChar(50),
		ProcessId VarChar(50),
		StudyId VarChar(200),
		TemplateId VarChar(200), 
		TemplateThemeItemId VarChar(200),
		TranscriptionId VARCHAR(250),
		Nlu VARCHAR(250),
		IntentName VARCHAR(250),
		Total Float,
		Duration Float,
		Silence Float,
		OtherDuration Float,
		OtherSilence Float,
		Confidence Float, 
		EndDate VarChar(50),
		Channel VarChar(50),
		Campaign VarChar(100),
		ValidChar Int,
		ValidWords Int
	)
	
	--SELECT * FROM TMP_INTERACTION_QUERY_STUDY
	INSERT INTO TMP_INTERACTION_QUERY_STUDY (
		VCC, ProcessId, StudyId, TemplateId, TemplateThemeItemId, 
		TranscriptionId, Total, Duration, Silence, OtherDuration,
		OtherSilence, Confidence, EndDate, Channel, Campaign, 
		ValidChar, ValidWords
	)
		SELECT VCC, ProcessId, StudyId, TemplateId, TemplateThemeItemId, 
				TranscriptionId, Total, Duration, Silence, OtherDuration,
				OtherSilence, Confidence, EndDate, Channel, Campaign, 
				ValidChar, ValidWords
			FROM #InteractionQueryStudyTemp
	*/
	
	-- INSERTAMOS LOS RESULTADOS DE LAS INTERACCIONES DE AUDIO QUE MATCHEARON CON LA QUERY
	INSERT INTO [dbo].[InteractionQueryResult] (
		[VCC],
		[InteractionId],
		[Section],
		[Part],
		[InteractionType],
		[TranscriptionId],
		[BatchType],
		[BatchTypeId],
		[BatchTypeName],
		[AnalysisType],
		[QueryId],
		[QueryName],
		[QueryValue],
		[QueryData],
		[Matches],
		[Details],
		[CreatedDate],
		[TimeStamp]
	)
		SELECT DISTINCT IQST.[VCC],
				PA.[AudioId],
				PA.[Section],
				PA.[Part],
				IQST.[Channel],
				IQST.[TranscriptionId],
				'STUDY',
				IQST.[StudyId],
				IQST.[StudyId],
				'SEMANTIC',
				GQ.[Name],
				GQ.[Name],
				GQ.[Query],
				GQ.[RawData],
				IQST.[Matches],
				IQST.[MatchesDetail],
				GETUTCDATE(),
				GETUTCDATE()
			FROM [#InteractionQueryStudyTemp] IQST (NOLOCK)
			INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK)
				ON PA.[VCC] = IQST.[VCC]
					AND PA.[ProcessId] = IQST.[ProcessId]
					AND CONCAT(PA.[AudioId], '_', ISNULL(PA.[Section], '0'), '_', ISNULL(PA.[Part], '0')) = IQST.[TranscriptionId]
			INNER JOIN [dbo].[GenericQuery] GQ (NOLOCK)
				ON GQ.[VCC] = IQST.[VCC]
					AND GQ.[Name] = IQST.[TemplateThemeItemId]
			WHERE IQST.[Channel] = 'CALL'

	-- INSERTAMOS LOS RESULTADOS DE LAS INTERACCIONES DE TEXTO QUE MATCHEARON CON LA QUERY
	INSERT INTO [dbo].[InteractionQueryResult] (
		[VCC],
		[InteractionId],
		[Section],
		[Part],
		[InteractionType],
		[TranscriptionId],
		[BatchType],
		[BatchTypeId],
		[BatchTypeName],
		[AnalysisType],
		[QueryId],
		[QueryName],
		[QueryValue],
		[QueryData],
		[Matches],
		[Details],
		[CreatedDate],
		[TimeStamp]
	)
		SELECT IQST.[VCC],
				PAI.[InteractionId],
				NULL,
				NULL,
				PAI.[InteractionType],
				IQST.[TranscriptionId],
				'STUDY',
				IQST.[StudyId],
				IQST.[StudyId],
				'SEMANTIC',
				GQ.[Name],
				GQ.[Name],
				GQ.[Query],
				GQ.[RawData],
				IQST.[Matches],
				IQST.[MatchesDetail],
				GETUTCDATE(),
				GETUTCDATE()
			FROM [#InteractionQueryStudyTemp] IQST (NOLOCK)
			INNER JOIN [ProcessAsyncInteraction] PAI (NOLOCK) 
				ON PAI.[VCC] = IQST.[VCC]
					AND PAI.[InteractionType] = IQST.[Channel]
					AND PAI.[FormattedEndDate] = IQST.[EndDate]
					AND PAI.[Campaign] = IQST.[Campaign]
					AND PAI.[ProcessId] = IQST.[ProcessId]
					AND CONCAT(PAI.[InteractionType], '_', PAI.[Campaign], '_', PAI.[FormattedEndDate], '_', REPLACE(PAI.[InteractionId], ':', '_')) = IQST.[TranscriptionId]
			INNER JOIN [dbo].[GenericQuery] GQ (NOLOCK)
				ON GQ.[VCC] = IQST.[VCC]
					AND GQ.[Name] = IQST.[TemplateThemeItemId]
			WHERE IQST.[Channel] <> 'CALL'
	
	-- INSERTAMOS LOS REGISTROS CALL A LA TABLA STUDYRESULTSUMMARYDETAIL
	INSERT INTO [StudyResultSummaryDetail]
		( [VCC]
		,[StudyId]
		,[TemplateThemeId]
		,[TemplateThemeItemId]
		,[TranscriptionId]
		,[IsMatched]
		,[CreatedDate]
		,[CreatedByUserId]
		,[TimeStamp]
		)
	SELECT
		A.[VCC]
		, A.[StudyId]
		, B.[TemplateThemeId]
		, A.[TemplateThemeItemId]
		, A.[TranscriptionId]
		, 1
		, GETUTCDATE()
		, 'system'
		, GETUTCDATE()
	FROM (
		SELECT DISTINCT
				[VCC]
				, [ProcessId]
				, [StudyId]
				, [TemplateId]
				, [TemplateThemeItemId]
				, [TranscriptionId]
			FROM #InteractionQueryStudyTemp
			WHERE [Channel] = 'CALL'
	) A
	LEFT JOIN [TemplateThemeItem] B (nolock) 
		ON A.[VCC] = B.[VCC]
			AND A.[TemplateId] = B.[TemplateId]
			AND A.[TemplateThemeItemId] = B.Id
	LEFT JOIN [StudyResultSummaryDetail] C (nolock) 
		ON A.[VCC] = C.[VCC]
			AND A.[StudyId] = C.[StudyId]
			AND B.[TemplateThemeId] = C.[TemplateThemeId]
			AND A.[TranscriptionId] = C.[TranscriptionId]
	INNER JOIN [ProcessAudio] D (nolock) 
		ON A.[VCC] = D.[VCC]
			AND A.ProcessId = D.ProcessId
			AND CONCAT(D.[AudioId],'_',ISNULL(D.[Section], '0'),'_',ISNULL(D.[Part], '0')) = A.[TranscriptionId]
	WHERE C.[TranscriptionId] is null 
		and B.[TemplateThemeId] is not null


	-- INSERTAMOS LOS REGISTROS DE CANALES ASINCRONOS A LA TABLA STUDYRESULTSUMMARYDETAIL
	INSERT INTO [StudyResultSummaryDetail]
		( [VCC]
		,[StudyId]
		,[TemplateThemeId]
		,[TemplateThemeItemId]
		,[TranscriptionId]
		,[IsMatched]
		,[CreatedDate]
		,[CreatedByUserId]
		,[TimeStamp]
		)
	SELECT
		TMP_IQS.[VCC]
		, TMP_IQS.[StudyId]
		, TTI.[TemplateThemeId]
		, TMP_IQS.[TemplateThemeItemId]
		, TMP_IQS.[TranscriptionId]
		, 1
		, GETUTCDATE()
		, 'system'
		, GETUTCDATE()
	FROM (
		SELECT DISTINCT
				[VCC]
				, [ProcessId]
				, [StudyId]
				, [TemplateId]
				, [TemplateThemeItemId]
				, [TranscriptionId]
				, [Campaign]
				, [EndDate]
				, [Channel]
			FROM #InteractionQueryStudyTemp
			-- FROM TMP_INTERACTION_QUERY_STUDY
			WHERE [Channel] <> 'CALL'
	) TMP_IQS
	LEFT JOIN [TemplateThemeItem] TTI (NOLOCK) 
		ON TMP_IQS.[VCC] = TTI.[VCC]
			AND TMP_IQS.[TemplateId] = TTI.[TemplateId]
			AND TMP_IQS.[TemplateThemeItemId] = TTI.Id
	LEFT JOIN [StudyResultSummaryDetail] SRSD (NOLOCK) 
		ON TMP_IQS.[VCC] = SRSD.[VCC]
			AND TMP_IQS.[StudyId] = SRSD.[StudyId]
			AND TTI.[TemplateThemeId] = SRSD.[TemplateThemeId]
			AND TMP_IQS.[TranscriptionId] = SRSD.[TranscriptionId]
	INNER JOIN [ProcessAsyncInteraction] PAI (NOLOCK) 
		ON TMP_IQS.[VCC] = PAI.[VCC]
			AND PAI.[InteractionType] = TMP_IQS.[Channel]
			AND PAI.[FormattedEndDate] = TMP_IQS.[EndDate]
			AND PAI.[Campaign] = TMP_IQS.[Campaign]
			AND PAI.[ProcessId] = TMP_IQS.ProcessId
			-- AND PAI.[InteractionId] = TMP_IQS.[TranscriptionId]
			AND CONCAT(PAI.[InteractionType], '_', PAI.[Campaign], '_', PAI.[FormattedEndDate], '_', REPLACE(PAI.[InteractionId], ':', '_')) = TMP_IQS.[TranscriptionId]
	WHERE SRSD.[TranscriptionId] IS NULL
		AND TTI.[TemplateThemeId] IS NOT NULL
		

	-- INSERTAMOS LOS REGISTROS CALL A LA TABLA STUDYDETAIL
	INSERT INTO [StudyDetail]
		( [VCC]
		,[StudyId]
		,[TemplateThemeId]
		,[TemplateThemeItemId]
		,[TranscriptionId]
		,[Total]
		,[Duration]
		,[Silence]
		,[OtherDuration]
		,[OtherSilence]
		,[Confidence]
		,[IsMatched]
		,[CreatedDate]
		,[CreatedByUserId]
		,[LastModifiedDate]
		,[LastModifiedByUserId]
		,[TimeStamp]
		)
	SELECT
		A.[VCC]
		, A.[StudyId]
		, B.[TemplateThemeId]
		, A.[TemplateThemeItemId]
		, A.[TranscriptionId]
		, A.[Total]
		, A.[Duration]
		, A.[Silence]
		, A.[OtherDuration]
		, A.[OtherSilence]
		, A.[Confidence]
		, 1
		, GETUTCDATE()
		, 'system'
		, GETUTCDATE()
		, 'system'
		, GETUTCDATE()
	FROM (
		SELECT DISTINCT
			[VCC]
			, [ProcessId]
			, [StudyId]
			, [TemplateId]
			, [TemplateThemeItemId]
			, [TranscriptionId]
			, [Total]
			, [Duration]
			, [Silence]
			, [OtherDuration]
			, [OtherSilence]
			, [Confidence]
		FROM #InteractionQueryStudyTemp
		WHERE [Channel] = 'CALL'
	) A 
	LEFT JOIN [TemplateThemeItem] B (nolock) 
		ON A.[VCC] = B.[VCC]
			AND A.[TemplateId] = B.[TemplateId]
			AND A.[TemplateThemeItemId] = B.Id
	LEFT JOIN [StudyDetail] C (nolock) 
		ON A.[VCC] = C.[VCC]
			AND A.[StudyId] = C.[StudyId]
			AND B.[TemplateThemeId] = C.[TemplateThemeId]
			AND A.[TranscriptionId] = C.[TranscriptionId]
	INNER JOIN [ProcessAudio] D (nolock) 
		ON A.[VCC] = D.[VCC]
			AND A.ProcessId = D.ProcessId
			AND CONCAT(D.[AudioId],'_',ISNULL(D.[Section], '0'),'_',ISNULL(D.[Part], '0')) = A.[TranscriptionId]
	WHERE C.[TranscriptionId] is null 
		and B.[TemplateThemeId] is not null


	-- INSERTAMOS LOS REGISTROS DE OTROS CANALES A LA TABLA STUDYRESULTSUMMARYDETAIL
	INSERT INTO [StudyDetail]
		( [VCC]
		,[StudyId]
		,[TemplateThemeId]
		,[TemplateThemeItemId]
		,[TranscriptionId]
		,[Total]
		,[Duration]
		,[Silence]
		,[OtherDuration]
		,[OtherSilence]
		,[Confidence]
		,[IsMatched]
		,[CreatedDate]
		,[CreatedByUserId]
		,[LastModifiedDate]
		,[LastModifiedByUserId]
		,[TimeStamp]
		)
	SELECT
		TMP_IQS.[VCC]
		, TMP_IQS.[StudyId]
		, TTI.[TemplateThemeId]
		, TMP_IQS.[TemplateThemeItemId]
		, TMP_IQS.[TranscriptionId]
		, TMP_IQS.[Total]
		, TMP_IQS.[Duration]
		, TMP_IQS.[Silence]
		, TMP_IQS.[OtherDuration]
		, TMP_IQS.[OtherSilence]
		, TMP_IQS.[Confidence]
		, 1
		, GETUTCDATE()
		, 'system'
		, GETUTCDATE()
		, 'system'
		, GETUTCDATE()
	FROM (
		SELECT DISTINCT
			[VCC]
			, [ProcessId]
			, [StudyId]
			, [TemplateId]
			, [TemplateThemeItemId]
			, [TranscriptionId]
			, [Total]
			, [Duration]
			, [Silence]
			, [OtherDuration]
			, [OtherSilence]
			, [Confidence]
			, [Campaign]
			, [EndDate]
			, [Channel]
		FROM #InteractionQueryStudyTemp
		WHERE [Channel] <> 'CALL'
	) TMP_IQS 
	LEFT JOIN [TemplateThemeItem] TTI (NOLOCK) 
		ON TMP_IQS.[VCC] = TTI.[VCC]
			AND TMP_IQS.[TemplateId] = TTI.[TemplateId]
			AND TMP_IQS.[TemplateThemeItemId] = TTI.Id
	LEFT JOIN [StudyDetail] SD (NOLOCK) 
		ON TMP_IQS.[VCC] = SD.[VCC]
			AND TMP_IQS.[StudyId] = SD.[StudyId]
			AND TTI.[TemplateThemeId] = SD.[TemplateThemeId]
			AND TMP_IQS.[TranscriptionId] = SD.[TranscriptionId]
	INNER JOIN [ProcessAsyncInteraction] PAI (NOLOCK) 
		ON TMP_IQS.[VCC] = PAI.[VCC]
			AND PAI.[InteractionType] = TMP_IQS.[Channel]
			AND PAI.[FormattedEndDate] = TMP_IQS.[EndDate]
			AND PAI.[Campaign] = TMP_IQS.[Campaign]
			AND PAI.[ProcessId] = TMP_IQS.ProcessId
			-- AND PAI.[InteractionId] = TMP_IQS.[TranscriptionId]
			AND CONCAT(PAI.[InteractionType], '_', PAI.[Campaign], '_', PAI.[FormattedEndDate], '_', REPLACE(PAI.[InteractionId], ':', '_')) = TMP_IQS.[TranscriptionId]
	WHERE SD.[TranscriptionId] IS NULL 
		AND TTI.[TemplateThemeId] IS NOT NULL

	
	-- INSERTAMOS LOS REGISTROS CALL A LA TABLA STUDYRESULTSUMMARY
	INSERT INTO [dbo].[StudyResultSummary]
		([VCC]
		,[StudyId]
		, [TemplateId]
		, [TemplateThemeId]
		, [ID]
		, [Description]
		, [Category1]
		, [Category2]
		, [Total]
		, [Duration]
		, [Silence]
		, [OtherDuration]
		, [OtherSilence]
		, [Confidence]
		, [IsMatched]
		, [CreatedDate]
		, [CreatedByUserId]
		, [LastModifiedDate]
		, [LastModifiedByUserId]
		, [TimeStamp]
		)
	SELECT
		A.VCC
		, A.StudyId
		, A.TemplateId
		, B.TemplateThemeId
		, CONVERT(VARCHAR(50), NEWID()) ID
		, D.[Description]
		, D.Category1
		, D.Category2
		, COUNT(A.Total) Total
		, SUM(A.Duration) Duration
		, SUM(A.Silence) Silence
		, SUM(A.OtherDuration) Duration
		, SUM(A.OtherSilence) Silence
		, AVG(A.Confidence) Confidence
		, 1
		, GETUTCDATE()
		, 'system'
		, GETUTCDATE()
		, 'system'
		, GETUTCDATE()
	FROM (SELECT DISTINCT
			[VCC]
			, [ProcessId]
			, [StudyId]
			, [TemplateId]
			, [TemplateThemeItemId]
			, [TranscriptionId]
			, [Total]
			, [Duration]
			, [Silence]
			, [OtherDuration]
			, [OtherSilence]
			, [Confidence]
		FROM #InteractionQueryStudyTemp
		WHERE [Channel] = 'CALL'
	) A
	LEFT JOIN [TemplateThemeItem] B (nolock) ON A.[VCC] = B.[VCC]
		AND A.[TemplateId] = B.[TemplateId]
		AND A.[TemplateThemeItemId] = B.Id
	LEFT JOIN [StudyResultSummary] C (nolock) ON A.[VCC] = C.[VCC]
		AND A.[StudyId] = C.[StudyId]
		AND B.[TemplateThemeId] = C.[TemplateThemeId]
	LEFT JOIN [TemplateTheme] D (nolock) ON A.[VCC] = D.[VCC]
		AND A.[TemplateId] = D.[TemplateId]
		AND B.[TemplateThemeId] = D.[ID]
	INNER JOIN [ProcessAudio] E (nolock) ON A.[VCC] = E.[VCC]
		AND A.ProcessId = E.ProcessId
		AND CONCAT(E.[AudioId],'_',ISNULL(E.[Section], '0'),'_',ISNULL(E.[Part], '0')) = A.[TranscriptionId]
	WHERE C.[TemplateThemeId] is null 
		and B.[TemplateThemeId] is not null
	GROUP BY A.VCC
	, A.StudyId
	, A.TemplateId
	, B.TemplateThemeId
	, D.[Description]
	, D.Category1
	, D.Category2


	-- INSERTAMOS LOS REGISTROS DE OTROS CANALES A LA TABLA STUDYRESULTSUMMARY
	INSERT INTO [dbo].[StudyResultSummary]
		([VCC]
		,[StudyId]
		, [TemplateId]
		, [TemplateThemeId]
		, [ID]
		, [Description]
		, [Category1]
		, [Category2]
		, [Total]
		, [Duration]
		, [Silence]
		, [OtherDuration]
		, [OtherSilence]
		, [Confidence]
		, [IsMatched]
		, [CreatedDate]
		, [CreatedByUserId]
		, [LastModifiedDate]
		, [LastModifiedByUserId]
		, [TimeStamp]
		)
	SELECT
		TMP_IQS.VCC
		, TMP_IQS.StudyId
		, TMP_IQS.TemplateId
		, TTI.TemplateThemeId
		, CONVERT(VARCHAR(50), NEWID()) ID
		, TT.[Description]
		, TT.Category1
		, TT.Category2
		, COUNT(TMP_IQS.Total) Total
		, SUM(TMP_IQS.Duration) Duration
		, SUM(TMP_IQS.Silence) Silence
		, SUM(TMP_IQS.OtherDuration) Duration
		, SUM(TMP_IQS.OtherSilence) Silence
		, AVG(TMP_IQS.Confidence) Confidence
		, 1
		, GETUTCDATE()
		, 'system'
		, GETUTCDATE()
		, 'system'
		, GETUTCDATE()
	FROM (SELECT DISTINCT
			[VCC]
			, [ProcessId]
			, [StudyId]
			, [TemplateId]
			, [TemplateThemeItemId]
			, [TranscriptionId]
			, [Total]
			, [Duration]
			, [Silence]
			, [OtherDuration]
			, [OtherSilence]
			, [Confidence]
			, [Campaign]
			, [EndDate]
			, [Channel]
		FROM #InteractionQueryStudyTemp
		-- FROM TMP_INTERACTION_QUERY_STUDY (NOLOCK)
		WHERE [Channel] <> 'CALL'
	) TMP_IQS 
	LEFT JOIN [TemplateThemeItem] TTI (nolock) 
		ON TMP_IQS.[VCC] = TTI.[VCC]
			AND TMP_IQS.[TemplateId] = TTI.[TemplateId]
			AND TMP_IQS.[TemplateThemeItemId] = TTI.Id
	LEFT JOIN [StudyResultSummary] SRS (nolock) 
		ON TMP_IQS.[VCC] = SRS.[VCC]
			AND TMP_IQS.[StudyId] = SRS.[StudyId]
			AND TTI.[TemplateThemeId] = SRS.[TemplateThemeId]
	LEFT JOIN [TemplateTheme] TT (nolock) 
		ON TMP_IQS.[VCC] = TT.[VCC]
			AND TMP_IQS.[TemplateId] = TT.[TemplateId]
			AND TTI.[TemplateThemeId] = TT.[ID]
	INNER JOIN [ProcessAsyncInteraction] PAI (NOLOCK) 
		ON TMP_IQS.[VCC] = PAI.[VCC]
			AND PAI.[InteractionType] = TMP_IQS.[Channel]
			AND PAI.[FormattedEndDate] = TMP_IQS.[EndDate]
			AND PAI.[Campaign] = TMP_IQS.[Campaign]
			AND PAI.[ProcessId] = TMP_IQS.ProcessId
			--AND PAI.[InteractionId] = TMP_IQS.[TranscriptionId]
			AND CONCAT(PAI.[InteractionType], '_', PAI.[Campaign], '_', PAI.[FormattedEndDate], '_', REPLACE(PAI.[InteractionId], ':', '_')) = TMP_IQS.[TranscriptionId]
	WHERE SRS.[TemplateThemeId] IS NULL 
		AND TTI.[TemplateThemeId] IS NOT NULL 
	GROUP BY TMP_IQS.VCC
	, TMP_IQS.StudyId
	, TMP_IQS.TemplateId
	, TTI.TemplateThemeId
	, TT.[Description]
	, TT.Category1
	, TT.Category2

	--select * from [ProcessAsyncInteraction] (NOLOCK)
	-- ACTUALIZAMOS LA DATA DE AUDIO(SYNCINTERACTION)
	UPDATE A
		SET A.[ValidChar] = TMP_IQS.[ValidChar],
			A.[ValidWords] = TMP_IQS.[ValidWords],
			A.[LastModifiedByUserId] = 'system',
			A.[LastModifiedDate] = GETUTCDATE(),
			A.[TimeStamp] = GETUTCDATE()
		FROM [dbo].[Audio] A
		INNER JOIN [dbo].[#InteractionQueryStudyTemp] TMP_IQS 
			ON CONCAT(A.[ID],'_',ISNULL(A.[Section], '0'),'_',ISNULL(A.[Part], '0')) = TMP_IQS.[TranscriptionId]
			

	-- ACTUALIZAMOS LA DATA DE ASYNCINTERACTION
	UPDATE  AI
		SET AI.[ValidChar] = TMP_IQS.[ValidChar],
			AI.[ValidWords] = TMP_IQS.[ValidWords],
			AI.[LastModifiedByUserId] = 'system',
			AI.[LastModifiedDate] = GETUTCDATE(),
			AI.[TimeStamp] = GETUTCDATE()
		FROM [dbo].[AsyncInteraction] AI
		INNER JOIN [dbo].[#InteractionQueryStudyTemp] TMP_IQS 
			ON TMP_IQS.[VCC] = AI.[VCC]
				AND AI.[Channel] = TMP_IQS.[Channel]
				AND AI.[Folder] = TMP_IQS.[EndDate]
				AND AI.[Campaign] = TMP_IQS.[Campaign]
				--AND AI.[ID] = TMP_IQS.[TranscriptionId]
				AND CONCAT(AI.[Channel], '_', AI.[Campaign], '_', AI.[Folder], '_', REPLACE(AI.[ID], ':', '_')) = TMP_IQS.[TranscriptionId]
				

	DROP TABLE [dbo].[#InteractionQueryStudyTemp]

	SELECT 1 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkProcessAudioSummary]
AS
BEGIN
		-- No aplicamos where not exists porque esta tabla tiene un ID autogenerado.
		INSERT INTO dbo.ProcessAudioSummary
			(VCC
			,ProcessId
			,Category1
			,Category2
			,Category3
			,[Description]
			,[Total]
			,CreatedDate
			,CreatedByUserId
			,LastModifiedDate
			,LastModifiedByUserId
			,[TimeStamp]
			) 
		SELECT
			 A.VCC
			,A.ProcessId
			,A.Category1
			,A.Category2
			,A.Category3
			,A.[Description]
			,A.[Total]
			,A.CreatedDate
			,A.CreatedByUserId
			,A.LastModifiedDate
			,A.LastModifiedByUserId
			,GETUTCDATE()
		FROM #ProcessAudioSummaryTemp A (NOLOCK)

		DROP TABLE #ProcessAudioSummaryTemp
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkProcessI6Update]
AS
BEGIN
	-- ACTUALIZAMOS LOS AUDIOS CON EL RESULTADO EN LA TABLA
	UPDATE [dbo].[Audio] 
		SET [dbo].[Audio].[IsProcess] = NULL, 
			[dbo].[Audio].[Format] = [dbo].[#ProcessI6UpdateTemp].[Format],
			[dbo].[Audio].[IsTranscribed] = [dbo].[#ProcessI6UpdateTemp].[IsTranscribed],
			[dbo].[Audio].[IsCancel] = [dbo].[#ProcessI6UpdateTemp].[IsCancel],
			[dbo].[Audio].[ErrorMessage] = [dbo].[#ProcessI6UpdateTemp].[ErrorMessage],
			[dbo].[Audio].[LastModifiedDate] = GETUTCDATE()
		FROM [dbo].[#ProcessI6UpdateTemp]
		WHERE [dbo].[Audio].[VCC] = [dbo].[#ProcessI6UpdateTemp].[VCC]
			AND [dbo].[Audio].[ID] = [dbo].[#ProcessI6UpdateTemp].[ID]
			AND [dbo].[Audio].[Section] = [dbo].[#ProcessI6UpdateTemp].[Section]
			AND [dbo].[Audio].[Part] = [dbo].[#ProcessI6UpdateTemp].[Part]
			AND [dbo].[#ProcessI6UpdateTemp].[Channel] = 'CALL'
			AND [dbo].[Audio].[NeedI6Update] = 1
			AND [dbo].[Audio].[IsProcess] = 1
			AND [dbo].[Audio].[IsCancel] IS NULL 
			AND [dbo].[Audio].[IsTranscribed] IS NULL

	DROP TABLE [dbo].[#ProcessI6UpdateTemp]
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkProcessInteraction]
AS
BEGIN
	/*
	-- ACTUALIZAMOS LOS AUDIOS CON EL RESULTADO EN LA TABLA
	UPDATE [dbo].[Audio] SET
		[dbo].[Audio].[Format] = [dbo].[#ProcessInteractionTemp].[Format],
		[dbo].[Audio].[MediaInfoTime] = [dbo].[#ProcessInteractionTemp].[MediaInfoTime],
		[dbo].[Audio].[DurationLinguisticTime] = [dbo].[#ProcessInteractionTemp].[DurationLinguisticTime],
		[dbo].[Audio].[IsDiariced] = [dbo].[#ProcessInteractionTemp].[IsDiariced],
		[dbo].[Audio].[ChannelsChanged] = [dbo].[#ProcessInteractionTemp].[ChannelsChanged],
		[dbo].[Audio].[ValidChar] = ISNULL([dbo].[#ProcessInteractionTemp].[ValidChar], [dbo].[Audio].[ValidChar]),
		[dbo].[Audio].[ValidWords] = ISNULL([dbo].[#ProcessInteractionTemp].[ValidWords], [dbo].[Audio].[ValidWords]),
		[dbo].[Audio].[TotalPhrases] = [dbo].[#ProcessInteractionTemp].[TotalPhrases],
		[dbo].[Audio].[CharsPerPhrase] = [dbo].[#ProcessInteractionTemp].[CharsPerPhrase],
		[dbo].[Audio].[IsTranscribed] = [dbo].[#ProcessInteractionTemp].[IsTranscribed],
		[dbo].[Audio].[IsCancel] = [dbo].[#ProcessInteractionTemp].[IsCancel],
		[dbo].[Audio].[FileName] = [dbo].[#ProcessInteractionTemp].[FileName],
		[dbo].[Audio].[ErrorMessage] = [dbo].[#ProcessInteractionTemp].[ErrorMessage],
		[dbo].[Audio].[LastModifiedDate] = GETUTCDATE()
	FROM [dbo].[#ProcessInteractionTemp]
	WHERE
		[dbo].[Audio].[VCC] = [dbo].[#ProcessInteractionTemp].[VCC]
		AND [dbo].[Audio].[ID] = [dbo].[#ProcessInteractionTemp].[ID]
		AND [dbo].[Audio].[Section] = [dbo].[#ProcessInteractionTemp].[Section]
		AND [dbo].[Audio].[Part] = [dbo].[#ProcessInteractionTemp].[Part]
		AND [dbo].[#ProcessInteractionTemp].[Channel] = 'CALL'
	*/

	-- CREAMOS UN INDICE PARA LA TABLA TEMPORAL
	CREATE NONCLUSTERED INDEX [TMP_PROCESS_INTERACTION_01] ON #ProcessInteractionTemp ([VCC], [ID], [Section], [Part])

	-- ACTUALIZAMOS LOS AUDIOS CON EL RESULTADO EN LA TABLA
	UPDATE A
		SET A.[Format] = PIT.[Format],
			A.[MediaInfoTime] = PIT.[MediaInfoTime],
			A.[DurationLinguisticTime] = PIT.[DurationLinguisticTime],
			A.[IsDiariced] = PIT.[IsDiariced],
			A.[ChannelsChanged] = PIT.[ChannelsChanged],
			A.[ValidChar] = ISNULL(PIT.[ValidChar], A.[ValidChar]),
			A.[ValidWords] = ISNULL(PIT.[ValidWords], A.[ValidWords]),
			A.[TotalPhrases] = PIT.[TotalPhrases],
			A.[CharsPerPhrase] = PIT.[CharsPerPhrase],
			A.[IsTranscribed] = PIT.[IsTranscribed],
			A.[IsCancel] = PIT.[IsCancel],
			A.[FileName] = PIT.[FileName],
			A.[ErrorMessage] = PIT.[ErrorMessage],
			A.[LastModifiedDate] = GETUTCDATE(),
			A.[SilenceInTime] = PIT.[SilenceInTime],
			A.[SilenceOutTime] = PIT.[SilenceOutTime],
			A.[SilenceTime] = PIT.[SilenceTime]
		FROM [dbo].[Audio] A
		INNER JOIN [dbo].[#ProcessInteractionTemp] PIT
			ON PIT.[VCC] = PIT.[VCC]
				AND A.[ID] = PIT.[ID]
				AND A.[Section] = PIT.[Section]
				AND A.[Part] = PIT.[Part]
		WHERE PIT.[Channel] = 'CALL'

	-- ACTUALIZAMOS LOS TEXT CON EL RESULTADO DE LA TABLA
	UPDATE [dbo].[AsyncInteraction] 
		SET [dbo].[AsyncInteraction].[ValidChar] = ISNULL([dbo].[#ProcessInteractionTemp].[ValidChar], [dbo].[AsyncInteraction].[ValidChar]),
			[dbo].[AsyncInteraction].[ValidWords] = ISNULL([dbo].[#ProcessInteractionTemp].[ValidWords], [dbo].[AsyncInteraction].[ValidWords]),
			[dbo].[AsyncInteraction].[TotalPhrases] = [dbo].[#ProcessInteractionTemp].[TotalPhrases],
			[dbo].[AsyncInteraction].[CharsPerPhrase] = [dbo].[#ProcessInteractionTemp].[CharsPerPhrase],
			[dbo].[AsyncInteraction].[IsTranscribed] = [dbo].[#ProcessInteractionTemp].[IsTranscribed],
			[dbo].[AsyncInteraction].[IsCancel] = [dbo].[#ProcessInteractionTemp].[IsCancel],
			[dbo].[AsyncInteraction].[FileName] = [dbo].[#ProcessInteractionTemp].[FileName],
			[dbo].[AsyncInteraction].[ViewFileName] = [dbo].[#ProcessInteractionTemp].[ViewFileName],
			[dbo].[AsyncInteraction].[ErrorMessage] = [dbo].[#ProcessInteractionTemp].[ErrorMessage],
			[dbo].[AsyncInteraction].[LastModifiedDate] = GETUTCDATE()
	FROM [dbo].[#ProcessInteractionTemp]
	WHERE [dbo].[AsyncInteraction].[VCC] = [dbo].[#ProcessInteractionTemp].[VCC]
		AND [dbo].[AsyncInteraction].[Campaign] = [dbo].[#ProcessInteractionTemp].[Campaign]
		AND [dbo].[AsyncInteraction].[Channel] = [dbo].[#ProcessInteractionTemp].[Channel]
		AND [dbo].[AsyncInteraction].[ID] = [dbo].[#ProcessInteractionTemp].[ID]
		AND [dbo].[AsyncInteraction].[Folder] = [dbo].[#ProcessInteractionTemp].[Folder]

	DROP TABLE [dbo].[#ProcessInteractionTemp]
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkProgress]
AS
BEGIN

	/*
	IF OBJECT_ID(N'[dbo].[TMP_BULK_PROGRESS]', N'U') IS NULL
	BEGIN
		SELECT *
			INTO [dbo].[TMP_BULK_PROGRESS]
			FROM [dbo].[#ProgressTemp]
	END
	ELSE
	BEGIN
		SELECT * 
			INTO #ProgressTemp
			FROM [dbo].[TMP_BULK_PROGRESS] (NOLOCK)
	END
	*/
	
	INSERT INTO [Progress] ([Key]
		  ,[VCC]
		  ,[ProcessId]
		  ,[Type]
		  ,[TypeId]
		  ,[Total]
		  ,[Progress]
		  ,[Error]
		  ,[Percentage]
		  ,[StatusDescription])  
	SELECT [Key]
		  ,[VCC]
		  ,[ProcessId]
		  ,[Type]
		  ,[TypeId]
		  ,[Total]
		  ,[Progress]
		  ,[Error]
		  ,[Percentage]
		  ,[StatusDescription]
	  FROM [dbo].[#ProgressTemp]

	



	------------INICIO DE ACTUALIZACION DE ESTADOS DE PROCESS I6UPDATE --------------

	-- ACTUALIZAMOS EL ESTADO GENERAL DE PROCESS SOBRE LA ACTUALIZACION DE INTERACCIONES CALL I6
	PRINT CONCAT('UPDATING I6_UPDATE STUFF ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE P 
		SET [Total] = PT.[Total]
			,[progress] = PT.[Progress]
			,[error] = PT.[Error]
			,[percentage] = CASE WHEN PT.[Total] > 0 THEN PT.[Percentage] ELSE 0 END
			,[StatusDescription] = CASE WHEN PT.[Total] >= 0 
												AND PT.[Percentage] = 1 
												AND  ISNULL(PT.[Error],0) > 0 
											THEN 'I6_UPDATE_COMPLETED_WITH_ERROR' 
										WHEN PT.[Total] >= 0 
												AND PT.[Percentage] = 1 
												AND  ISNULL(PT.[Error],0) = 0 
											THEN 'I6_UPDATE_COMPLETED' 
										ELSE  'IN_PROGRESS' END
			,[LastModifiedDate] = GETUTCDATE()
		FROM [dbo].[Process] P
		INNER JOIN [dbo].[#ProgressTemp] PT 
			ON P.[vcc] = PT.[vcc] 
				AND PT.ProcessId = P.[Name] 
		WHERE PT.[Type] = 'i6update'

	-- ACTUALIZAMOS EL ESTADO DEL PROCESS TERMINADO
	PRINT CONCAT('UPDATING I6_UPDATE: PROCESS_STATUS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE A
		SET [Status] = CASE 
				WHEN B.total >= 0 
						AND B.total <= B.[updated] + B.cancel 
						AND B.cancel > B.[updated] 
					THEN 'FAILED' 
				WHEN B.total >= 0 
						AND B.total <= B.[updated] + B.cancel 
					THEN 'COMPLETE' 
				ELSE 'INPROCESS_I6_UPDATE' 
				END
			,[LastModifiedDate] = GETUTCDATE()
		FROM [dbo].[Process] AS A
		INNER JOIN (
			SELECT SUM(UT.total)									'total',
					SUM(UT.cancel)									'cancel',
					SUM(UT.[updated])								'updated',
					UT.processId									'processId',
					UT.vcc											'vcc'
				FROM (
					SELECT COUNT(1)										'total'
							,COUNT(A.[isCancel])						'cancel'
							,COUNT(IIF(A.[NeedI6Update] = 0, 1, 0))		'updated' 
							,PT.[ProcessId]								'processId'
							,PT.[vcc]									'vcc'
						FROM [dbo].[ProcessAudio] PA (NOLOCK)
						INNER JOIN [#ProgressTemp] PT 
							ON PA.[vcc] = PT.[vcc] 
								AND PA.ProcessId = PT.ProcessId 
								AND PT.[Percentage] = 1
						INNER JOIN [dbo].[Audio] A (NOLOCK) 
							ON PA.[VCC] = A.VCC 
								AND PA.[AudioId] = A.ID 
						WHERE PT.[Type] = 'i6update'
						GROUP BY PT.ProcessId, PT.vcc
				) UT 
				GROUP BY UT.processId, UT.vcc
		) B ON A.[vcc] = B.[vcc] 
			AND A.[Name] = B.[ProcessId]

	------------FIN DE ACTUALIZACION DE ESTADOS DE PROCESS I6UPDATE --------------



	  
	------------INICIO DE ACTUALIZACION DE ESTADOS DE PROCESS --------------
	PRINT CONCAT('UPDATING PROCESS_UPDATE: STUFF ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE A
		SET [Total] = PT.[Total]
			,[progress] = PT.[Progress]
			,[error] = PT.[Error]
			,[percentage] = CASE WHEN PT.[Total] > 0 THEN PT.[Percentage] ELSE 0 END
			,[StatusDescription] = CASE WHEN PT.[Total] >= 0 
												AND PT.[Percentage] = 1 
												AND  ISNULL(PT.[Error], 0) > 0 
											THEN 'COMPLETED_WITH_ERROR' 
										WHEN PT.[Total] >= 0 
												AND PT.[Percentage] = 1 
												AND  ISNULL(PT.[Error], 0) = 0 
											THEN 'COMPLETED' 
										ELSE  'IN_PROGRESS' END
			,[LastModifiedDate] = GETUTCDATE()
		FROM Process AS A
		INNER JOIN [dbo].[#ProgressTemp] PT 
			ON A.[vcc] = PT.[vcc] 
				AND A.[Name] = PT.ProcessId 
		WHERE PT.[Type] IS NULL

	-- ACTUALIZAMOS EL ESTADO DEL PROCESS TERMINADO
	PRINT CONCAT('UPDATING PROCESS_UPDATE: PROCESS_STATUS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE A
	SET [Status] = CASE 
			WHEN B.total >= 0 AND B.total <= B.transcription + B.cancel AND B.cancel > B.transcription THEN 'FAILED' 
			WHEN B.total >= 0 AND B.total <= B.transcription + B.cancel THEN 'COMPLETE' 
			ELSE 'INPROCESS_TRANSCRIPTION' 
			END
		,[LastModifiedDate] = GETUTCDATE()
	FROM Process AS A
	INNER JOIN (
		SELECT ISNULL(SUM(UT.total), 0)					'total',
				ISNULL(SUM(UT.cancel), 0)				'cancel',
				ISNULL(SUM(UT.transcription), 0)		'transcription',
				UT.processId							'processId',
				UT.vcc									'vcc'
			FROM (
			SELECT COUNT(1)					total
					,COUNT(C.isCancel)			cancel
					,COUNT(C.IsTranscribed)		transcription 
					,B.ProcessId				processId
					,B.vcc						vcc
				FROM [ProcessAudio] A (NOLOCK)
				INNER JOIN [#ProgressTemp] B 
					ON A.[vcc] = B.[vcc] 
						AND A.ProcessId = B.ProcessId 
						-- AND B.[Percentage] = 1 -- ESTA LINEA NO PERMITE ACTUALIZAR LA COLUMNA STATUS CUANDO EL PROCESS AUN NO TERMINA
				INNER JOIN Audio C (NOLOCK) 
					ON A.VCC = C.VCC 
						AND A.AudioId =C.ID 
						AND ISNULL(A.Section,0) = ISNULL(C.Section,0) 
						AND ISNULL(A.Part,0) = ISNULL(C.Part,0)
				WHERE B.[Type] IS NULL
				GROUP BY B.ProcessId, B.vcc
			UNION
			SELECT 
				COUNT(1)						total
				,COUNT(AI.IsCancel)			cancel
				,COUNT(AI.IsTranscribed)		transcription
				,PT.[ProcessId]				processId
				,PT.[vcc]						vcc
				FROM [ProcessAsyncInteraction] PAI (NOLOCK)
				INNER JOIN [#ProgressTemp] PT
					ON PAI.[VCC] = PT.[vcc]
						AND PAI.[ProcessId] = PT.[ProcessId]
						-- AND PT.[Percentage] = 1 -- ESTA LINEA NO PERMITE ACTUALIZAR LA COLUMNA STATUS CUANDO EL PROCESS AUN NO TERMINA
				INNER JOIN [AsyncInteraction] AI (NOLOCK) 
					ON AI.[VCC] = PT.[vcc]
						AND AI.[Channel] = PAI.[InteractionType]
						AND AI.[Folder] = PAI.[FormattedEndDate]
						AND AI.[Campaign] = PAI.[Campaign]
						AND AI.[ID] = PAI.[InteractionId]
				WHERE PT.[Type] IS NULL
				GROUP BY PT.ProcessId, PT.vcc
		) UT GROUP BY UT.processId, UT.vcc

	  ) B ON A.[vcc] = B.[vcc] 
		AND A.[Name] = B.ProcessId

	-- ACTUALIZAMOS EL ESTADO DE OTROS PROCESOS QUE ESTAN COMO FALLIDOS, COLGADOS, ETC QUE COINCIDEN CON LAS INTERACCIONES DEL PROCESS TERMINADO
	PRINT CONCAT('UPDATING PROCESS_UPDATE: OTHER_PROCESS_STATUS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE P
		SET P.[Status] = CASE WHEN TMPP.[totalFailed] > TMPP.[totalSucced] THEN 'FAILED'
								ELSE 'COMPLETE' END,
			P.[StatusDescription] = CASE WHEN TMPP.[totalFailed] > 0 THEN 'COMPLETED_WITH_ERROR' 
											ELSE 'COMPLETED' END,
			P.[Percentage] = 1
			,P.[LastModifiedDate] = GETUTCDATE()
		FROM [dbo].[Process] P (NOLOCK)
		INNER JOIN (
			SELECT UT.[processId]							'processId',
					ISNULL(SUM(UT.[totalAudio]), 0)			'totalAudio',
					ISNULL(SUM(UT.[totalSucced]), 0)		'totalSucced',
					ISNULL(SUM(UT.[totalFailed]), 0)		'totalFailed'
				FROM (
					SELECT P.[Name] 'processId',
							COUNT(*) 'totalAudio',
							COUNT(CASE WHEN ISNULL(A.[IsTranscribed], 0) = 1 THEN 1 ELSE NULL END) 'totalSucced',
							COUNT(CASE WHEN ISNULL(A.[IsCancel], 0) = 1 THEN 1 ELSE NULL END) 'totalFailed'
						FROM [dbo].[Process] P (NOLOCK) 
						INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
							ON PA.[VCC] = P.[VCC]
								AND PA.[ProcessId] = P.[Name]
						INNER JOIN [dbo].[Audio] A (NOLOCK) 
							ON A.[VCC] = PA.[VCC]
								AND A.[ID] = PA.[AudioId]
								AND A.[Section] = PA.[Section]
								AND A.[Part] = PA.[Part]
						WHERE 
							(P.[StatusDescription] <> 'COMPLETED' AND P.[Status] <> 'COMPLETE')
							OR 
							(P.[StatusDescription] = 'COMPLETED' AND P.[Status] <> 'COMPLETE')
							OR 
							(P.[StatusDescription] = 'COMPLETED' AND P.[Status] IS NULL)
							OR 
							(P.[Total] = 0 AND P.[Percentage] = 0)
							-- AND P.[Status] = 'INPROCESS_TRANSCRIPTION'
							--AND P.[StatusDescription] = 'IN_PROGRESS'
						GROUP BY P.[Name]
			
					UNION

					SELECT P.[Name],
							COUNT(*) 'totalAudio',
							COUNT(CASE WHEN ISNULL(AI.[IsTranscribed], 0) = 1 THEN 1 ELSE NULL END) 'totalSucced',
							COUNT(CASE WHEN ISNULL(AI.[IsCancel], 0) = 1 THEN 1 ELSE NULL END) 'totalFailed'
						FROM [dbo].[Process] P (NOLOCK) 
						INNER JOIN [ProcessAsyncInteraction] PAI (NOLOCK)
							ON PAI.[VCC] = P.[vcc]
								AND PAI.[ProcessId] = P.[Name]
						INNER JOIN [AsyncInteraction] AI (NOLOCK) 
							ON AI.[VCC] = P.[vcc]
								AND AI.[Channel] = PAI.[InteractionType]
								AND AI.[Folder] = PAI.[FormattedEndDate]
								AND AI.[Campaign] = PAI.[Campaign]
								AND AI.[ID] = PAI.[InteractionId]
						WHERE 
							-- ESTO CAUSA PROBLEMAS PARA LA FINALIDAD DE ACTUALIZAR OTROS PROCESS
							-- P.[Name] IN (SELECT DISTINCT [ProcessId] FROM #ProgressTemp)
							-- AND (
								(P.[StatusDescription] <> 'COMPLETED' AND P.[Status] <> 'COMPLETE')
								OR 
								(P.[StatusDescription] = 'COMPLETED' AND P.[Status] <> 'COMPLETE')
								OR 
								(P.[StatusDescription] = 'COMPLETED' AND P.[Status] IS NULL)
								OR 
								(P.[Total] = 0 AND P.[Percentage] = 0)
							-- )
						GROUP BY P.[Name]
					) UT GROUP BY UT.[processId]
		) TMPP
			ON TMPP.[processId] = P.[Name]
				AND TMPP.[totalAudio] = (ISNULL(TMPP.[totalSucced], 0) + ISNULL(TMPP.[totalFailed], 0))


	-- REGISTROS DE [dbo].[#ProgressTemp] CON [Percentage] = 1 YA ESTAN TERMINADO (CON O SIN ERRORES)
	-- SI EL PROCESS SE MARCA COMO FAILED TUVO MUCHO FRACASOS, PERO AUN ASI TUVO EXITOS QUE DEBERERIA COBRARSE
	PRINT CONCAT('UPDATING PROCESS_UPDATE: BATCH_INSERT ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	INSERT INTO [dbo].[Batch] ([VCC], [BatchTypeName], [BatchTypeId], [IsFinished], [ClosedDate], [LastModifiedDate])
		SELECT PTMP.[vcc], 'TRANSCRIPTION', PTMP.[ProcessId], 1, GETUTCDATE(), GETUTCDATE()
			FROM [#ProgressTemp] PTMP
			INNER JOIN [dbo].[Process] P 
				ON P.[vcc] = PTMP.[vcc] 
					AND P.[Name] = PTMP.[ProcessId] 
					AND PTMP.[Type] IS NULL
			WHERE PTMP.[Percentage] = 1 
				AND PTMP.[Type] IS NULL -- EL NULL INDICA QUE ES PROCESS DE TRANSCRIPCION Y NO DE PLAN DE EVALUACION

	-- REGISTRAMOS LA(S) AGENDA(S) DE PLANTILLA PARA LOS PROCESS TERMINADO
	PRINT CONCAT('UPDATING PROCESS_UPDATE: TEMPLATE_SCHEDULE_INSERT ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	INSERT INTO [dbo].[BatchScheduleExecution] ([VCC], [Id], [BatchScheduleId], [ProcessId], [ExecutionDate], [Status], [CreatedByUserId], [CreatedDate], [TimeStamp])
		SELECT PTMP.[vcc],
				NEWID(),
				BS.[Id],
				PTMP.[ProcessId],
				GETUTCDATE(),
				'PENDING',
				'system',
				GETUTCDATE(),
				GETUTCDATE()
			FROM [#ProgressTemp] PTMP
			LEFT JOIN [dbo].[BatchSchedule] BS (NOLOCK)
				ON BS.[VCC] = PTMP.[vcc]
			WHERE PTMP.[Percentage] = 1 
				AND PTMP.[Type] IS NULL -- EL NULL INDICA QUE ES PROCESS DE TRANSCRIPCION Y NO DE PLAN DE EVALUACION
				AND BS.[BatchType] = 'TEMPLATE'
				AND GETUTCDATE() >= BS.[StartDate]
				AND BS.[Status] IN ('CREATED', 'RUNNING')
				AND (
					BS.[EndDate] IS NULL 
					OR
					GETUTCDATE() < BS.[EndDate]
				)

	------------FIN DE ACTUALIZACION DE ESTADOS DE PROCESS --------------





	------------INICIO DE ACTUALIZACION DE ESTADOS DE ESTUDIOS --------------
	PRINT CONCAT('UPDATING STUDY_UPDATE: STUFF ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE A
		SET [Total] = B.[Total]
			,[progress] = B.[Progress]
			,[error] = B.[Error]
			,[percentage] = B.[Percentage]
			,[StatusDescription] = CASE WHEN B.[Percentage] = 1 AND  ISNULL(B.[Error],0) > 0 THEN 'COMPLETED_WITH_ERROR' 
										WHEN B.[Percentage] = 1 AND  ISNULL(B.[Error],0) = 0 THEN 'COMPLETED' 
										ELSE  'IN_PROGRESS' END
			,[Status] = B.[Percentage]
			,[LastModifiedDate] = GETUTCDATE()
			--,[BatchStatus] = CASE WHEN B.[Percentage] = 1 AND  ISNULL(B.[Error],0) > 0 THEN 'FINISHED_WITH_ERROR' 
				--					WHEN B.[Percentage] = 1 AND  ISNULL(B.[Error],0) = 0 THEN 'FINISHED' 
					--				ELSE 'RUNNING' END
		FROM Study AS A
		INNER JOIN [dbo].[#ProgressTemp] B 
			ON A.[vcc] = B.[vcc] 
				AND A.[ProcessId] = B.ProcessId 
				AND A.[Name] = B.[TypeId] 
				AND B.[Type] IN ('study', 'template')
				
	--Eliminar StudyResultSummary
	PRINT CONCAT('UPDATING STUDY_UPDATE: DELETE_STUFF_01 ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	         DELETE SRS FROM [dbo].[StudyResultSummary] SRS 
	                    JOIN [dbo].[Study]  S 
	                         ON 
	                        (S.[VCC] = SRS.[VCC] 
	                       AND S.[Name] = SRS.[StudyId] 
	                       AND S.[TemplateId] = SRS.[TemplateId])
	                   JOIN [dbo].[TemplateTheme] TT
	                         ON 
	                           (TT.[VCC] = S.[VCC]
	                            AND TT.[TemplateId] = S.[TemplateId]
	                            AND TT.[ID] = SRS.[TemplateThemeId])
	                   JOIN [dbo].[#ProgressTemp] PT 
	                        ON  
	                       (S.[VCC] = PT.[vcc] 
							AND S.[ProcessId] = PT.ProcessId 
							AND S.[Name] = PT.[TypeId] 
							AND PT.[Type]  IN ('study', 'template'))
						--Si es negativo eliminar los que IsMatched = 1 
	                   WHERE SRS.[IsMatched] = IIF(TT.[IsNegative] = 1, 1, 0)
	                  
	           	        
	 -- Eliminar StudyResultSummaryDetail
	 PRINT CONCAT('UPDATING STUDY_UPDATE: DELETE_STUFF_02 ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	           	  DELETE SRSD FROM [dbo].[StudyResultSummaryDetail] SRSD 
	                    JOIN [dbo].[Study]  S 
	                         ON 
	                        (S.[VCC] = SRSD.[VCC] 
	                       AND S.[Name] = SRSD.[StudyId])
	                   JOIN [dbo].[TemplateTheme] TT
	                         ON 
	                           (TT.[VCC] = S.[VCC]
	                            AND TT.[TemplateId] = S.[TemplateId]
	                            AND TT.[ID] = SRSD.[TemplateThemeId])
	                   JOIN [dbo].[TemplateThemeItem] TTI 
	                          ON (TTI.[VCC] = S.[VCC]
	                              AND TTI.[TemplateThemeId] = TT.[ID]
	                              AND TTI.[ID] = SRSD.[TemplateThemeItemId])
	                   JOIN [dbo].[#ProgressTemp] PT 
	                        ON  
	                       (S.[VCC] = PT.[vcc] 
							AND S.[ProcessId] = PT.ProcessId 
							AND S.[Name] = PT.[TypeId] 
							AND PT.[Type]  IN ('study', 'template'))
						--Si es negativo eliminar los que IsMatched = 1 
	                   WHERE SRSD.[IsMatched] = IIF(TT.[IsNegative] = 1, 1, 0)
	
   	 -- Eliminar StudyResultSummaryDetail
	 PRINT CONCAT('UPDATING STUDY_UPDATE: DELETE_STUFF_03 ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	           	  DELETE SD FROM [dbo].[StudyDetail] SD 
	                    JOIN [dbo].[Study]  S 
	                         ON 
	                        (S.[VCC] = SD.[VCC] 
	                       AND S.[Name] = SD.[StudyId])
	                   JOIN [dbo].[TemplateTheme] TT
	                         ON 
	                           (TT.[VCC] = S.[VCC]
	                            AND TT.[TemplateId] = S.[TemplateId]
	                            AND TT.[ID] = SD.[TemplateThemeId])
	                   JOIN [dbo].[TemplateThemeItem] TTI 
	                          ON (TTI.[VCC] = S.[VCC]
	                              AND TTI.[TemplateThemeId] = TT.[ID]
	                              AND TTI.[ID] = SD.[TemplateThemeItemId])
	                   JOIN [dbo].[#ProgressTemp] PT 
	                        ON  
	                       (S.[VCC] = PT.[vcc] 
							AND S.[ProcessId] = PT.[ProcessId] 
							AND S.[Name] = PT.[TypeId] 
							AND PT.[Type] IN ('study', 'template'))
						--Si es negativo eliminar los que IsMatched = 1 
	                   WHERE SD.[IsMatched] = IIF(TT.[IsNegative] = 1, 1, 0)

	-- REGISTROS DE [dbo].[#ProgressTemp] CON [Percentage] = 1 YA ESTAN TERMINADO (CON O SIN ERRORES)
	PRINT CONCAT('UPDATING STUDY_UPDATE: BATCH_INSERT ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	INSERT INTO [dbo].[Batch] ([VCC], [BatchTypeName], [BatchTypeId], [IsFinished], [ClosedDate], [LastModifiedDate])
		SELECT PTMP.[vcc], UPPER(PTMP.[Type]), PTMP.[TypeId], 1, GETUTCDATE(), GETUTCDATE()
			FROM [#ProgressTemp] PTMP
			INNER JOIN [dbo].[Study] S 
				ON S.[vcc] = PTMP.[vcc] 
					AND S.[ProcessId] = PTMP.[ProcessId] 
					AND S.[Name] = PTMP.[TypeId]
					AND PTMP.[Type] IN ('study', 'template')
			WHERE PTMP.[Percentage] = 1 
				AND PTMP.[Type] IN ('study', 'template')

	-- ACTUALIZAMOS LA EJECUCION RELACIONADA AL BATCH TEMPLATE TERMINADO
	PRINT CONCAT('UPDATING STUDY_UPDATE: BATCH_SCHEDULE_UPDATE ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE BSE 
		SET BSE.[Status] = 'FINISHED',
			BSE.[LastModifiedByUserId] = 'system',
			BSE.[LastModifiedDate] = GETUTCDATE()
		FROM [#ProgressTemp] PTMP
		INNER JOIN [dbo].[BatchScheduleExecution] BSE 
			ON BSE.[vcc] = PTMP.[vcc] 
				AND BSE.[BatchId] = PTMP.[TypeId] 
		WHERE PTMP.[Percentage] = 1 
			AND PTMP.[Type] = 'template'

	------------FIN DE ACTUALIZACION DE ESTADOS DE ESTUDIOS --------------





	------------INICIO DE ACTUALIZACION DE ESTADOS DE PLAN DE EVALUACION --------------

	-- UPDATE PARA LA PARTE DE TRANSCRIPCION
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: STUFF ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE EP
		SET [Total] = B.[Total]
			,[progress] = B.[Progress]
			,[error] = B.[Error]
			,[percentage] = B.[Percentage]
			,[StatusDescription] = CASE WHEN B.[Percentage] = 1 AND  ISNULL(B.[Error],0) > 0 THEN 'COMPLETED_WITH_ERROR' 
										WHEN B.[Percentage] = 1 AND  ISNULL(B.[Error],0) = 0 THEN 'COMPLETED' 
										ELSE 'IN_PROGRESS' END
			,[Status] = CASE WHEN B.[Percentage] = 1 THEN 3 ELSE 2 END
			,[LastModifiedDate] = GETUTCDATE()
		FROM [dbo].[EvaluationPlan] EP (NOLOCK)
		INNER JOIN [dbo].[#ProgressTemp] B 
			ON EP.[Instance] = B.[vcc] 
				AND EP.[ProcessId] = B.ProcessId 
				AND B.[Type] IS NULL 
				AND EP.[FilterByContentQueryName] IS NOT NULL 
				AND EP.[FilterByContentQuery] IS NOT NULL

	-- UPDATE CONTEMPLADO PARA PLAN DE EVALUACION CON PASO 3 (FILTRO DE CONTENIDO)
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: STEP_03_STUFF ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE A
	SET [Total] = B.[Total]
		,[progress] = B.[Progress]
		,[error] = B.[Error]
		,[percentage] = B.[Percentage]
		,[StatusDescription] = CASE WHEN B.[Percentage] = 1 AND  ISNULL(B.[Error],0) > 0 THEN 'COMPLETED_WITH_ERROR' 
									WHEN B.[Percentage] = 1 AND  ISNULL(B.[Error],0) = 0 THEN 'COMPLETED' 
									ELSE  'IN_PROGRESS' END
		,[Status] = CASE WHEN B.[Percentage] = 1 THEN 5 ELSE 4 END
		,[LastModifiedDate] = GETUTCDATE()
	FROM EvaluationPlan AS A
	INNER JOIN [dbo].[#ProgressTemp] B 
		ON A.[Instance] = B.[vcc] 
			AND A.[ProcessId] = B.ProcessId 
			AND A.Id = B.[TypeId] 
			AND B.[Type] = 'evaluationPlan'
			
	-- UPDATE PARA INICIAR LA PARTE DE ANALISIS SEMANTICO/COGNITIVO CUANDO EL PROCESS TERMINOS
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: EVALUATION_PLAN_STATUS_UPDATE_01 ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE EP
		SET EP.[Total] = B.[Total]
			,EP.[progress] = B.[Progress]
			,EP.[error] = B.[Error]
			,EP.[percentage] = B.[Percentage]
			,EP.[StatusDescription] = CASE WHEN B.[Percentage] = 1 
												AND P.[Status] NOT LIKE '%I6_UPDATE%'
											--THEN CASE WHEN P.[StatusDescription] = 'FAILED' THEN 'CANCELLED_BY_FAILED_PROCESS'
											--		ELSE P.[StatusDescription]
											--		END
											THEN IIF (P.[Status] = 'FAILED', 'CANCELLED_BY_FAILED_PROCESS', P.[Status])
											ELSE 'IN_PROGRESS' END
			,EP.[Status] = CASE WHEN B.[Percentage] = 1
							--THEN CASE WHEN P.[StatusDescription] = 'FAILED' THEN 10 
							--		ELSE 5 END
								THEN IIF (P.[Status] = 'FAILED', 9, 5)
							ELSE 2 END
			,EP.[LastModifiedDate] = GETUTCDATE()
		FROM [dbo].[EvaluationPlan] EP (NOLOCK)
		INNER JOIN [dbo].[Process] P (NOLOCK)
			ON P.[VCC] = EP.[Instance]
				AND P.[Name] = EP.[ProcessId] 
		INNER JOIN [dbo].[#ProgressTemp] B 
			ON EP.[Instance] = B.[vcc] 
				AND EP.[ProcessId] = B.ProcessId 
		WHERE EP.[FilterByContentQueryName] IS NULL 
				AND EP.[FilterByContentQuery] IS NULL 
				AND EP.[Evaluation_Mode] IN ('A', 'X')
				AND B.[Type] IS NULL 

	/*
	--OBTENGO ESTADOS DE PROCESO DE AUDIOS
	;WITH AudioStatus AS (
		SELECT D.[VCC]										'instance',
				D.[ProcessId]								'processId',
				COUNT(*) 									'total',
				SUM(IIF(A.[IsTranscribed] = 1, 1, 0))		'transcribed',
				SUM(IIF(A.[IsCancel] = 1, 1, 0))			'cancelled'
			FROM [dbo].[Audio] A (NOLOCK)
			INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON A.[ID] = IEP.[IdInteraction]
					AND A.[Section] = IEP.[InteractionSection]
					AND A.[Part] = IEP.[InteractionPart]
			INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
				ON EP.[Instance] = IEP.[Instance]
					AND EP.[ID] = IEP.[IdEvaluationPlan]
			INNER JOIN [dbo].[#ProgressTemp] D (NOLOCK)
				ON D.[vcc] = EP.[Instance]
					AND D.[ProcessId] = EP.ProcessId
					AND  D.[TypeId] = EP.Id
					AND D.[Type] = 'ability'
			GROUP BY D.[VCC], D.[ProcessId]
	)
	*/

	-- ACTUALIZAMOS EL ESTADO DE LOS PLANES(FASE DE EVALUACION AUTOMATICA) DE LA TABLA TEMPORAL CUANDO EL PLAN ESTA EN STATUS 5 (ANALIZADO) O 6 (EVALUANDO)
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: EVALUATION_PLAN_STATUS_UPDATE_02 ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE EP
		SET EP.[Total] = B.[Total]
			,EP.[progress] = B.[Progress]
			,EP.[error] = B.[Error]
			,EP.[percentage] = B.[Percentage]
			,EP.[StatusDescription] = CASE WHEN B.[Percentage] = 1 AND  ISNULL(B.[Error], 0) > 0 THEN 'EVALUATION_COMPLETED_WITH_ERROR' 
										WHEN B.[Percentage] = 1 AND  ISNULL(B.[Error], 0) = 0 THEN 'EVALUATION_COMPLETED' 
										ELSE  'IN_PROGRESS' END
			,EP.[Status] = CASE WHEN B.[Percentage] = 1 THEN 11 ELSE 6 END
			,EP.[LastModifiedDate] = GETUTCDATE() 
		FROM [dbo].[EvaluationPlan] EP (NOLOCK)
		INNER JOIN [dbo].[#ProgressTemp] B 
			ON EP.[Instance] = B.[vcc] 
				AND EP.[ProcessId] = B.ProcessId 
				AND EP.Id = B.[TypeId] 
				AND B.[Type] = 'ability'
		WHERE EP.[Status] IN (5, 6)

	-- EN UNA TABLA ADICIONAL PODRIA INSERTAR LOS RP STATUS 11 PARA LUEGO ENVIARLOS A CERRAR EN EL SC
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: AUDITOR_EVALUATION_PLAN_STUFF ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE AEP 
		SET AEP.[Completed] = 1
			,[StatusDescription] = 'COMPLETED'
			,[LastModifiedByUserId] = 'system'
			,[LastModifiedDate] = GETUTCDATE()
			,[TimeStamp] = GETUTCDATE()
	FROM dbo.[AuditorEvaluationPlan] AEP 
	INNER JOIN dbo.[EvaluationPlan] EP 
		ON EP.[ID] = AEP.[IdEvaluationPlan]
	INNER JOIN [dbo].[#ProgressTemp] B 
		ON EP.[Instance] = B.[vcc] 
			AND EP.[ProcessId] = B.ProcessId 
			AND EP.Id = B.[TypeId] 
	WHERE EP.[Status] = 11
		AND AEP.[IdAuditor] = 'SPEECH' 
		AND (EP.[Evaluation_Mode] = 'A' OR EP.[Evaluation_Mode] = 'X') 
		AND B.[Type] = 'ability'



	/*************************************INICIO DE ACTUALIZACION DE TIPIFICACIONES***********************************************/
	
	-- ACTUALIZAMOS EL ESTADO DE LOS PLANES(FASE DE TIPIFICACIONES) DE LA TABLA TEMPORAL
	PRINT CONCAT('UPDATING TIPIFICATION_UPDATE: STUFF ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE EP
		SET EP.[percentage] = TMP_P.[Percentage]
			,EP.[StatusDescription] = CASE WHEN TMP_P.[Percentage] = 1 AND  ISNULL(TMP_P.[Error],0) > 0 THEN 'COMPLETED_WITH_ERROR' 
										WHEN TMP_P.[Percentage] = 1 AND  ISNULL(TMP_P.[Error],0) = 0 THEN 'COMPLETED' 
										ELSE  'IN_PROGRESS' END
			,EP.[Status] = CASE WHEN TMP_P.[Percentage] = 1 THEN 7 ELSE 12 END
			,EP.[LastModifiedDate] = GETUTCDATE() 
		FROM [dbo].[EvaluationPlan] EP
		INNER JOIN [dbo].[#ProgressTemp] TMP_P 
			ON EP.[Instance] = TMP_P.[vcc] 
				AND EP.[ProcessId] = TMP_P.ProcessId 
				AND EP.Id = TMP_P.[TypeId] 
		WHERE EP.[Status] IN (11, 12)
			AND TMP_P.[Type] = 'tipification'

	/*************************************FIN DE ACTUALIZACION DE TIPIFICACIONES***********************************************/

	-- REGISTROS DE [dbo].[#ProgressTemp] CON [Percentage] = 1 YA ESTAN TERMINADO (CON O SIN ERRORES)
	INSERT INTO [dbo].[Batch] ([VCC], [BatchTypeName], [BatchTypeId], [IsFinished], [ClosedDate], [LastModifiedDate])
		SELECT PTMP.[vcc], 'EVALUATION_PLAN', PTMP.[TypeId], 1, GETUTCDATE(), GETUTCDATE()
			FROM [#ProgressTemp] PTMP
			INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
				ON EP.[Instance] = PTMP.[vcc] 
					AND EP.[ProcessId] = PTMP.[ProcessId] 
					AND EP.[Id] = PTMP.[TypeId]
					AND PTMP.[Type] IN ('ability', 'evaluationPlan', 'tipification')
			WHERE EP.[Status] = 7
				AND PTMP.[Percentage] = 1 
				AND PTMP.[Type] IN ('ability', 'evaluationPlan', 'tipification')



	-- REGISTRAMOS LA(S) AGENDA(S) DE PLANTILLA PARA LOS PROCESS TERMINADO
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: TEMPLATE_SCHEDULE_INSERT ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	INSERT INTO [dbo].[BatchScheduleExecution] ([VCC], [Id], [BatchScheduleId], [ProcessId], [ExecutionDate], [Status], [CreatedByUserId], [CreatedDate], [TimeStamp])
		SELECT PTMP.[vcc],
				NEWID(),
				BS.[Id],
				PTMP.[ProcessId],
				GETUTCDATE(),
				'PENDING',
				'system',
				GETUTCDATE(),
				GETUTCDATE()
			FROM [#ProgressTemp] PTMP
			LEFT JOIN [dbo].[BatchSchedule] BS (NOLOCK)
				ON BS.[VCC] = PTMP.[vcc]
			WHERE PTMP.[Percentage] = 1 
				AND (PTMP.[Type] = 'ability' OR PTMP.[Type] = 'evaluationPlan') -- EL NULL INDICA QUE ES PROCESS DE PLAN DE EVALUACION
				AND BS.[BatchType] = 'TEMPLATE'
				AND GETUTCDATE() >= BS.[StartDate]
				AND BS.[Status] = 'RUNNING'
				AND (
					BS.[EndDate] IS NULL 
					OR
					GETUTCDATE() < BS.[EndDate]
				)

	-- INICIO MEDIDA PARA PLANES FINALIZADOS QUE NO TIENE RESULTS REGISTRADOS
	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS PLANES Y SU CORRESPONDIENTE VCC
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: PATCHING_MISSING_RESULTS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	DECLARE MY_CURSOR CURSOR 
	LOCAL STATIC READ_ONLY FORWARD_ONLY -- SCROLL 
		FOR 
		-- CONSULTAMOS LOS ID DE PLANES EN LA TABLA TEMPORAL
		SELECT PTMP.[vcc] 'VCC', 
				PTMP.[TypeId] 'EVALUATION_PLAN_ID'
			FROM [#ProgressTemp] PTMP
			INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
				ON EP.[Instance] = PTMP.[vcc] 
					AND EP.[ProcessId] = PTMP.[ProcessId] 
					AND EP.[Id] = PTMP.[TypeId]
			WHERE PTMP.[Percentage] = 1 
				AND (
					PTMP.[Type] = 'ability' 
					OR 
					PTMP.[Type] = 'evaluationPlan'
				)
			
	-- DECLARARAMOS VARIABLES PARA INDICAR EL PLAN Y VCC
	DECLARE @CURRENT_VCC VARCHAR(50) = ''
	DECLARE @CURRENT_EVALUATION_PLAN_ID VARCHAR(100) = ''
	DECLARE @CURRENT_EVALUATION_PLAN_MIN_AUTOMATIC_QUALIFICATION NUMERIC(5, 2) = 0

	-- ABRIMOS Y RECORREMOS EL ARRAY DE CANALES
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: PATCHING_MISSING_RESULTS_PER_PLAN ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	OPEN MY_CURSOR
	FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID
	WHILE @@FETCH_STATUS = 0
	BEGIN 
		-- BUSCAMOS SI EXISTE RESULTS PARA EL PLAN ACTUAL
		IF NOT EXISTS (
			SELECT 1 
				FROM [dbo].[EvaluationResult] ER (NOLOCK) 
				WHERE ER.[Instance] = @CURRENT_VCC
					AND ER.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
		)
		BEGIN
			-- EJECUTAMOS EL SP QUE INSERTA LOS RESULT BASE DE LOS PLANES QUE AUN NO LOS TIENEN
			EXEC [dbo].[SaveEvaluationBaseResult] @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID, 'AUTOMATIC'

			-- ACTUALIZAMOS A COMPLETED LOS REGISTROS AUDITOR_EVALUATION_PLAN DEL PLAN ACTUAL
			UPDATE AEP
				SET AEP.[Completed] = '1'
					,AEP.[StatusDescription] = 'COMPLETED'
					,AEP.[LastModifiedByUserId] = 'system'
					,AEP.[LastModifiedDate] = GETUTCDATE()
					,AEP.[TimeStamp] = GETUTCDATE()
				FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				WHERE AEP.[Instance] = @CURRENT_VCC
					AND AEP.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
		END

		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID
	END
	CLOSE MY_CURSOR
	DEALLOCATE MY_CURSOR
	-- FIN MEDIDA PARA PLANES FINALIZADOS QUE NO TIENE RESULTS REGISTRADOS





	-- INICIO DE ACTUALIZACION DE ESTADO PARA PLANES MIXTOS(TERMINADA LA FASE DE EVALUACION AUTOMATICA(ESTADO 11), PASA A LA MANUAL(ESTADO 8) Y ACABADA LA MANUAL AL ESTADO 7)

	DECLARE @CURRENT_AUDITOR_ID VARCHAR(50) = ''

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS PLANES Y SU CORRESPONDIENTE VCC
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: UPDATING_MIXED_EVALUATION_PLAN_STATUS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	DECLARE MY_CURSOR_2 CURSOR 
	LOCAL STATIC READ_ONLY FORWARD_ONLY -- SCROLL 
		FOR 
		-- CONSULTAMOS LOS ID DE PLANES EN LA TABLA TEMPORAL
		SELECT DISTINCT
				PT.[vcc]								'VCC', 
				PT.[TypeId]								'EVALUATION_PLAN_ID',
				EF.[MinAutomaticQualification]			'MIN_AUTOMATIC_QUALIFICATION'
			FROM [dbo].[EvaluationPlan] AS EP (NOLOCK)
			INNER JOIN [dbo].[#ProgressTemp] PT 
				ON EP.[Instance] = PT.[vcc] 
					AND EP.[ProcessId] = PT.[ProcessId] 
					AND EP.[Id] = PT.[TypeId] 
					AND PT.[Type] = 'ability'
			INNER JOIN [dbo].[EvaluationFormat] EF (NOLOCK)
				ON EF.[Instance] = EP.[Instance]
					AND EF.[ID] = EP.[IdEvaluationFormat]
			WHERE EP.[Status] = 11
				AND EP.[Evaluation_Mode] = 'X'

	-- ABRIMOS Y RECORREMOS EL ARRAY DE PLANES
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: UPDATING_MIXED_EVALUATION_PLAN_STATUS_PER_PLAN ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	OPEN MY_CURSOR_2
	FETCH NEXT FROM MY_CURSOR_2 INTO @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID, @CURRENT_EVALUATION_PLAN_MIN_AUTOMATIC_QUALIFICATION
	WHILE @@FETCH_STATUS = 0
	BEGIN 
		-- EVALUAMOS SI EL AUDITOR SPEECH CALIFICO CON NOTA MAYOR O IGUAL AL ACEPTABLE MINIMO PARA LA PARTE AUTOMATICA
		IF EXISTS (
			SELECT 1
				FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				WHERE AEP.[Instance] = @CURRENT_VCC
					AND AEP.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
					AND AEP.[IdAuditor] = 'SPEECH'
					AND AEP.[Score] >= @CURRENT_EVALUATION_PLAN_MIN_AUTOMATIC_QUALIFICATION
		)
		BEGIN
			-- ACTUALIZAMOS EL ESTADO DEL PLAN MIXTO PARA PASAR A LA PARTE MANUAL
			UPDATE EP 
				SET EP.[percentage] = 0
					,EP.[StatusDescription] = 'IN_PROGRESS'
					,EP.[Status] = 8
					,EP.[LastModifiedDate] = GETUTCDATE() 
				FROM [dbo].[EvaluationPlan] AS EP (NOLOCK)
				WHERE EP.[Instance] = @CURRENT_VCC
					AND EP.[ID] = @CURRENT_EVALUATION_PLAN_ID

			-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS ID DE AUDITORES CORRESPONDIENTES AL PLAN
			DECLARE MY_AUDITOR_CURSOR CURSOR 
			LOCAL STATIC READ_ONLY FORWARD_ONLY -- SCROLL 
				FOR 
				-- CONSULTAMOS LOS ID DE AUDITORES
				SELECT DISTINCT AEP.[IdAuditor]
					FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
					WHERE  AEP.[Instance] = @CURRENT_VCC
						AND AEP.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
						AND AEP.[IdAuditor] <> 'SPEECH'

			-- ABRIMOS Y RECORREMOS EL ARRAY DE PLANES
			OPEN MY_AUDITOR_CURSOR
			FETCH NEXT FROM MY_AUDITOR_CURSOR INTO @CURRENT_AUDITOR_ID
			WHILE @@FETCH_STATUS = 0
			BEGIN 
				-- INSERTARMOS LOS REGISTROS DE ER QUE TENIA EL AUDITOR SPEECH
				INSERT INTO [dbo].[EvaluationResult] (
					[Instance], [IdCampaign], [IdInteraction], [IdAgent], [IdEvaluationFormat],
					[MinQualificationAcceptable], [IdGroup], [IdSubGroup], [IdAbility], [Qualification],
					[OwnScore], [IdEvaluator], [EvaluationDate], [MaxAbilityScore], [IdEvaluationPlan], 
					[CreatedDate], [CreatedByUserId], [TimeStamp], [NotApplies], [ScoreDescription], 
					[CampaignDescription], [AgentDescription], [InteractionDateini], [InteractionDatefin], [InteractionAddress], 
					[AbilityOrder], [InteractionType], [IdAbilityEvaluationFormat], [IdAuditor], [Conditional]
				) 
					-- SELECCIONAMOS LOS RESULTADOS REGISTRADOS POR SPEECH
					SELECT ER.[Instance], ER.[IdCampaign], ER.[IdInteraction], ER.[IdAgent], ER.[IdEvaluationFormat],
							ER.[MinQualificationAcceptable], ER.[IdGroup], ER.[IdSubGroup], ER.[IdAbility], ER.[Qualification],
							ER.[OwnScore], ER.[IdEvaluator], ER.[EvaluationDate], ER.[MaxAbilityScore], ER.[IdEvaluationPlan], 
							ER.[CreatedDate], ER.[CreatedByUserId], GETUTCDATE(), ER.[NotApplies], ER.[ScoreDescription], 
							ER.[CampaignDescription], ER.[AgentDescription], ER.[InteractionDateini], ER.[InteractionDatefin], ER.[InteractionAddress], 
							ER.[AbilityOrder], ER.[InteractionType], ER.[IdAbilityEvaluationFormat], @CURRENT_AUDITOR_ID, ER.[Conditional]
						FROM [dbo].[EvaluationResult] ER (NOLOCK)
						INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
							ON AEP.[Instance] = ER.[Instance]
								AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]
								AND AEP.[IdAuditor] = ER.[IdAuditor]
								AND AEP.[IdInteraction] = ER.[IdInteraction]
								AND AEP.[Score] >= @CURRENT_EVALUATION_PLAN_MIN_AUTOMATIC_QUALIFICATION
						WHERE ER.[Instance] = @CURRENT_VCC
							AND ER.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
							AND ER.[IdAuditor] = 'SPEECH'

				-- REABRIMOS LAS EVALUACIONES DE LAS INTERACCIONES CON SCORE MAYOR O IGUAL AL UMBRAL
				UPDATE [dbo].[AuditorEvaluationPlan]
					SET [Completed] = 0,
						[AutomaticScore] = [Score],
						[ManualScore] = 0,
						[StatusDescription] = 'NEW',
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [Instance] = @CURRENT_VCC
						AND [IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
						AND [IdAuditor] = @CURRENT_AUDITOR_ID

				FETCH NEXT FROM MY_AUDITOR_CURSOR INTO @CURRENT_AUDITOR_ID
			END
			CLOSE MY_AUDITOR_CURSOR
			DEALLOCATE MY_AUDITOR_CURSOR

			-- COMPLETAMOS LOS REGISTROS DE ER PARA LAS HABILIDADES MANUALES <- LAS HAB. MANUALES DEBEN QUEDAN EN BLANCO PARA EL AUDITOR SPEECH
			-- EXEC [dbo].[SaveEvaluationBaseResult] @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID, 'MANUAL'

			-- REMOVEMOS LOS REGISTROS DE SPEECH (AEP Y ER) QUE FUERON ASIGNADOS A LOS AUDITORES
			DELETE ER
				FROM [dbo].[EvaluationResult] ER
				INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
					ON AEP.[Instance] = ER.[Instance]
						AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]
						AND AEP.[IdInteraction] = ER.[IdInteraction]
						AND AEP.[Score] >= @CURRENT_EVALUATION_PLAN_MIN_AUTOMATIC_QUALIFICATION
				WHERE ER.[Instance] = @CURRENT_VCC
					AND ER.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
					AND ER.[IdAuditor] = 'SPEECH'
					AND AEP.[Score] >= @CURRENT_EVALUATION_PLAN_MIN_AUTOMATIC_QUALIFICATION

			-- REMOVEMOS LOS REGISTROS DE SPEECH EN AEP QUE PASARON EL MINIMO PARA AUTOMATICO
			DELETE 
				FROM [dbo].[AuditorEvaluationPlan] 
				WHERE [Instance] = @CURRENT_VCC
					AND [IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
					AND [IdAuditor] = 'SPEECH'
					AND [Score] >= @CURRENT_EVALUATION_PLAN_MIN_AUTOMATIC_QUALIFICATION

			-- REMOVEMOS LOS REGISTROS DE LOS OTROS AUDITORES QUE NO PASARON EL MINIMO PARA AUTOMATICO
			DELETE 
				FROM [dbo].[AuditorEvaluationPlan] 
				WHERE [Instance] = @CURRENT_VCC
					AND [IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
					AND [IdAuditor] <> 'SPEECH'
					AND [IdInteraction] IN (
						SELECT [IdInteraction] 
							FROM [dbo].[AuditorEvaluationPlan] 
							WHERE [Instance] = @CURRENT_VCC
								AND [IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
								AND [IdAuditor] = 'SPEECH'
								AND [Score] < @CURRENT_EVALUATION_PLAN_MIN_AUTOMATIC_QUALIFICATION
					)

			-- ACTUALIZAMOS LOS CAMPOS SCORE DE LAS EVALUACIONES DE SPEECH
			UPDATE [dbo].[AuditorEvaluationPlan]
					SET [AutomaticScore] = [Score],
						[ManualScore] = 0,
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [Instance] = @CURRENT_VCC
						AND [IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
						AND [IdAuditor] = 'SPEECH'

		END
		ELSE
		BEGIN
			-- ELIMINAMOS LOS REGISTROS DE LOS OTROS AUDITORES (AEP)
			DELETE FROM [dbo].[AuditorEvaluationPlan] 
				WHERE  [Instance] = @CURRENT_VCC
					AND [IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN_ID
					AND [IdAuditor] <> 'SPEECH'

			-- COMPLETAMOS LOS REGISTROS DE ER PARA LAS HABILIDADES MANUALES <- LAS HAB. MANUALES DEBEN QUEDAN EN BLANCO PARA EL AUDITOR SPEECH
			-- EXEC [dbo].[SaveEvaluationBaseResult] @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID, 'MANUAL'

		END

		FETCH NEXT FROM MY_CURSOR_2 INTO @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID, @CURRENT_EVALUATION_PLAN_MIN_AUTOMATIC_QUALIFICATION
	END
	CLOSE MY_CURSOR_2
	DEALLOCATE MY_CURSOR_2

	/*-- INVALIDAR LAS AUDITORIAS CUYO AUDIO PRESENTAN ALGUN ERROR (DEPRECADO)
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: UPDATING_FAILED_INTERACTIONS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	UPDATE AEP
		SET AEP.[StatusDescription] = 'INVALID'
		FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
		INNER JOIN [dbo].[Audio] A (NOLOCK)
		    ON AEP.[Instance] = A.[VCC] 
		        AND AEP.[IdInteraction] = A.[ID]
		INNER JOIN [#ProgressTemp] PTMP (NOLOCK)
			ON AEP.[Instance] = PTMP.[vcc]
			    AND AEP.[IdEvaluationPlan] = PTMP.[TypeId]
		WHERE (
				A.[IsTranscribed] IS NULL 
				OR 
				A.[IsTranscribed]  <> 1
			)
		    AND (
				PTMP.[Type] = 'ability' 
				OR 
				PTMP.[Type] = 'evaluationPlan'
			)
		    -- AND A.[ErrorMessage] IN ('GET_AUDIO_REQUEST_FAILED','JSON_IDATHA_IS_NOT_FOUND','JSON_TRANSCRIPTION_CONTENT_IS_EMPTY')
	*/

	-- INVALIDAR LAS AUDITORIAS CUYO AUDIO PRESENTAN ALGUN ERROR
	PRINT CONCAT('UPDATING EVALUATION_PLAN_UPDATE: UPDATING_FAILED_INTERACTIONS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	;WITH CTE_SUCCEED_AUDIOS AS (
		SELECT DISTINCT [ID]
			FROM [dbo].[Audio] A (NOLOCK)
			INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				ON AEP.[Instance] = A.[VCC] 
		        	AND AEP.[IdInteraction] = A.[ID]
			INNER JOIN [#ProgressTemp] PTMP (NOLOCK)
				ON AEP.[Instance] = PTMP.[vcc]
					AND AEP.[IdEvaluationPlan] = PTMP.[TypeId]
			WHERE (
				PTMP.[Type] = 'ability' 
				OR 
				PTMP.[Type] = 'evaluationPlan'
			)
				AND A.[IsTranscribed] = 1
			
	)
	UPDATE AEP
		SET AEP.[StatusDescription] = 'INVALID'
		FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
		INNER JOIN [#ProgressTemp] PTMP (NOLOCK)
			ON AEP.[Instance] = PTMP.[vcc]
			    AND AEP.[IdEvaluationPlan] = PTMP.[TypeId]
		WHERE (
				PTMP.[Type] = 'ability' 
				OR 
				PTMP.[Type] = 'evaluationPlan'
			)
			AND AEP.[IdInteraction] NOT IN (
				SELECT [ID]
					FROM CTE_SUCCEED_AUDIOS CTE_SA (NOLOCK)
			)

	-- RECUPERAMOS LAS EVALUACIONES DE SPEECH DE LA TABLA AUDITOR EVALUATION PARA REPLICARLA

	/*
	-- ACTUALIZAMOS EL ESTADO DEL PLAN MIXTO PARA PASAR A LA PARTE MANUAL
	UPDATE EP 
		SET EP.[percentage] = 0
			,EP.[StatusDescription] = 'IN_PROGRESS'
			,EP.[Status] = 8
			,EP.[LastModifiedDate] = GETUTCDATE() 
		FROM [dbo].[EvaluationPlan] AS EP (NOLOCK)
		INNER JOIN [dbo].[#ProgressTemp] PT 
			ON EP.[Instance] = PT.[vcc] 
				AND EP.[ProcessId] = PT.[ProcessId] 
				AND EP.[Id] = PT.[TypeId] 
				AND PT.[Type] = 'ability'
		WHERE EP.[Status] = 7
			AND EP.[Evaluation_Mode] = 'X'
	*/
	

	-- FIN DE ACTUALIZACION DE ESTADO PARA PLANES MIXTOS





	-----------------------------------------------------

	DROP TABLE [dbo].[#ProgressTemp]

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkStudyModel]
AS
BEGIN
	SET NOCOUNT ON
	
	/* 
	SELECT *
		INTO [dbo].[TMP_BULK_MODEL]
		FROM [dbo].[#StudyModelTemp]
	*/

	/*
	-- INSERTAMOS LA DATA DE LA TEMPORAL PARA DEPURAR EL SP MANUALMENTE
	IF OBJECT_ID(N'[dbo].[TMP_BULK_INTENT]', N'U') IS NULL
	BEGIN
		SELECT *
			INTO [dbo].[TMP_BULK_INTENT]
			FROM [dbo].[#StudyModelTemp]

		/*
		SELECT * FROM [dbo].[TMP_BULK_INTENT] (NOLOCK)
		SELECT * 
			INTO [#StudyModelTemp]
			FROM [dbo].[TMP_BULK_INTENT] (NOLOCK)
		SELECT * FROM [#StudyModelTemp]
		*/
	END
	ELSE
	BEGIN
		INSERT INTO [dbo].[TMP_BULK_INTENT]
			SELECT * 
				FROM [dbo].[#StudyModelTemp]
	END
	*/

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS PLANES Y SU CORRESPONDIENTE VCC
	DECLARE MY_CURSOR CURSOR 
	LOCAL STATIC READ_ONLY FORWARD_ONLY -- SCROLL 
		FOR 
		-- CONSULTAMOS LOS ID DE PLANES EN LA TABLA TEMPORAL
		SELECT DISTINCT
				[VCC],
				[StudyId]		
			FROM [dbo].[#StudyModelTemp]  (NOLOCK)

	-- DECLARARAMOS VARIABLES PARA INDICAR EL PLAN Y VCC
	DECLARE @CURRENT_VCC VARCHAR(50) = ''
	DECLARE @CURRENT_STUDY_ID VARCHAR(100) = ''

	-- ABRIMOS Y RECORREMOS EL ARRAY DE CANALES
	OPEN MY_CURSOR
	FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC, @CURRENT_STUDY_ID
	WHILE @@FETCH_STATUS = 0
	BEGIN 
		-- EJECUTAMOS EL SP QUE INSERTA LOS RESULT BASE DE LOS ESTUDIOS QUE AUN NO LOS TIENEN
		EXEC [dbo].[SaveStudyBaseResult] @CURRENT_VCC, @CURRENT_STUDY_ID
		
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC, @CURRENT_STUDY_ID
	END
	CLOSE MY_CURSOR
	DEALLOCATE MY_CURSOR
	-- FIN MEDIDA PARA ESTUDIOS FINALIZADOS QUE NO TIENE RESULTS REGISTRADOS



	UPDATE SRSD 
	SET 
          SRSD.[IsMatched] = 1
	    FROM 
	 [StudyResultSummaryDetail] SRSD 
	 JOIN  (
			SELECT DISTINCT
					 [VCC]
					, [StudyId]
					, [TemplateThemeId]
					,[TemplateThemeItemId]
					, [TranscriptionId]
				FROM #StudyModelTemp
				--WHERE [Channel] = 'CALL'
		) ICT
		    ON (ICT.[VCC] = SRSD.[VCC]
			AND ICT.[StudyId] = SRSD.[StudyId]
			AND ICT.[TemplateThemeId] = SRSD.[TemplateThemeId]
			AND ICT.[TranscriptionId] = SRSD.[TranscriptionId]
			AND ICT.[TemplateThemeItemId] = SRSD.[TemplateThemeItemId]
			)
			
		 WHERE   ICT.[TemplateThemeId] IS NOT NULL 
		


	UPDATE SD 
	SET 
          SD.[IsMatched] = 1
	    FROM 
	 [StudyDetail] SD 
	 JOIN  (
			       SELECT DISTINCT
					 [VCC]
					, [StudyId]
					, [TemplateThemeId]
					,[TemplateThemeItemId]
					, [TranscriptionId]
				FROM #StudyModelTemp
				--WHERE [Channel] = 'CALL'
		) ICT
		    ON (ICT.[VCC] = SD.[VCC]
			AND ICT.[StudyId] = SD.[StudyId]
			AND ICT.[TemplateThemeId] = SD.[TemplateThemeId]
			AND ICT.[TranscriptionId] = SD.[TranscriptionId]
			AND ICT.[TemplateThemeItemId] = SD.[TemplateThemeItemId]
			)
			
		 WHERE   ICT.[TemplateThemeId] IS NOT NULL 



UPDATE SRS SET SRS.[IsMatched] = 1

FROM [dbo].[StudyResultSummary]  SRS 

JOIN  (SELECT DISTINCT
			 [VCC]
			, [StudyId]
			, [TemplateId]
			, [TemplateThemeId]

		FROM #StudyModelTemp
		--WHERE [Channel] = 'CALL'
	) ICT   ON (ICT.[VCC] = SRS.[VCC]
			AND ICT.[StudyId] = SRS.[StudyId]
			AND ICT.[TemplateId] = SRS.[TemplateId]
			AND ICT.[TemplateThemeId] = SRS.[TemplateThemeId])

	
	WHERE  ICT.[TemplateThemeId] IS NOT NULL
	

	
	-- ACTUALIZAMOS LA DATA DE ASYNCINTERACTION
	UPDATE [StudyResultSummary]
		SET [Total] = A.TOTAL
		FROM (
			SELECT A.[VCC]
					,A.[StudyId]
					,A.[TemplateId]
					,A.[TemplateThemeId]
					, COUNT(1) TOTAL
				FROM [StudyResultSummaryDetail] SRSD (NOLOCK)
				INNER JOIN (
					SELECT DISTINCT
							[VCC]
							,[StudyId]
							,[TemplateId]
							,[TemplateThemeId]
							,[TemplateThemeItemId]
						FROM #StudyModelTemp (NOLOCK)
				) A 
					ON SRSD.VCC = A.[VCC] 
						AND SRSD.StudyId = A.StudyId 
						AND SRSD.[TemplateThemeItemId] = A.[TemplateThemeItemId] 
			  JOIN [dbo].[TemplateTheme] TT (NOLOCK) 
			       ON TT.VCC = SRSD.VCC 
			          AND TT.[TemplateId] = A.[TemplateId]
			          AND TT.[ID] = SRSD.[TemplateThemeId]
			   --Si es negativo contara los que quedaron en 0 y los 1 si es positivo
			   WHERE SRSD.[IsMatched] = IIF(TT.[IsNegative] = 1, 0, 1)
				GROUP BY A.[VCC]
						,A.[StudyId]
						,A.[TemplateId]
						,A.[TemplateThemeId]
		) A
		WHERE StudyResultSummary.vcc = a.VCC
			AND StudyResultSummary.StudyId = A.StudyId 
			AND StudyResultSummary.[TemplateThemeId] = A.[TemplateThemeId]
	
	DROP TABLE #StudyModelTemp

	SELECT 1 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkTemplateTheme]
AS
BEGIN
		INSERT INTO dbo.TemplateTheme
			([VCC]
			,[TemplateId]
			,[ID]
			,[Order]
			,[Description]
			,[Category1]
			,[Category2]
			,[AnalysisType]
			,[CreatedDate]
			,[CreatedByUserId]
			,[LastModifiedDate]
			,[LastModifiedByUserId]
			,[TimeStamp]
			)
			SELECT DISTINCT A.[VCC]
							,A.[TemplateId]
							,A.[ID]
							,A.[Order]
							,A.[Description]
							,A.[Category1]
							,A.[Category2]
							,A.[AnalysisType]
							,A.[CreatedDate]
							,A.[CreatedByUserId]
							,A.[LastModifiedDate]
							,A.[LastModifiedByUserId]
							,GETUTCDATE()
				FROM #TemplateThemeTemp A (NOLOCK)
				LEFT JOIN [dbo].[TemplateTheme] B (NOLOCK) 
					ON A.[VCC] = B.[VCC] 
						AND A.[TemplateId] = B.[TemplateId]
				WHERE B.[ID] IS NULL
		
		DROP TABLE #TemplateThemeTemp
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkTemplateThemeItem]
AS
BEGIN
		INSERT INTO dbo.TemplateThemeItem
			(VCC
			,TemplateId
			,TemplateThemeId
			,ID
			,CreatedDate
			,CreatedByUserId
			,LastModifiedDate
			,LastModifiedByUserId
			,[TimeStamp]
			)
		SELECT DISTINCT
			 A.VCC
			,A.TemplateId
			,A.TemplateThemeId
			,A.[ID]
			,A.CreatedDate
			,A.CreatedByUserId
			,A.LastModifiedDate
			,A.LastModifiedByUserId
			,GETUTCDATE()
		FROM #TemplateThemeItemTemp A (NOLOCK)
		
		DROP TABLE #TemplateThemeItemTemp

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[BulkTipificationInteraction]
AS
BEGIN
	SET NOCOUNT ON
	
	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS PLANES Y SU CORRESPONDIENTE VCC
	DECLARE MY_CURSOR CURSOR 
	LOCAL STATIC READ_ONLY FORWARD_ONLY -- SCROLL 
		FOR 
		-- CONSULTAMOS LOS ID DE PLANES EN LA TABLA TEMPORAL
		SELECT DISTINCT TIT_TMP.[VCC],
						TIT_TMP.[EvaluationPlanId]		'EVALUATION_PLAN_ID'
			FROM [#TipificationInteractionTemp] TIT_TMP (NOLOCK)
			WHERE TIT_TMP.[VCC] IS NOT NULL 
				AND TIT_TMP.[EvaluationPlanId] IS NOT NULL

	-- DECLARARAMOS VARIABLES PARA INDICAR EL PLAN Y VCC
	DECLARE @CURRENT_VCC VARCHAR(50) = ''
	DECLARE @CURRENT_EVALUATION_PLAN_ID VARCHAR(100) = ''

	-- ABRIMOS Y RECORREMOS EL ARRAY DE PLANES
	OPEN MY_CURSOR
	FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID
	WHILE @@FETCH_STATUS = 0
	BEGIN 
		-- EJECUTAMOS EL SP QUE INSERTA LOS RESULT BASE DE LOS PLANES QUE AUN NO LOS TIENEN
		EXEC [dbo].[SaveEvaluationTipificationBaseResult] @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID
			
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_VCC, @CURRENT_EVALUATION_PLAN_ID
	END
	CLOSE MY_CURSOR
	DEALLOCATE MY_CURSOR
	
	-- ACTUALIZAMOS LOS REGISTROS CORRESPONDIENTES A LA TABLA TEMPORAL SOLO EN EL CAMPO ISCHECKED
	UPDATE ET
		SET ET.[IsChecked] = TIT_TMP.[IsChecked]
		FROM [EvaluationTipification] ET
		INNER JOIN  [#TipificationInteractionTemp] TIT_TMP (NOLOCK)
			ON TIT_TMP .[VCC] = ET.[VCC]
				AND TIT_TMP.[AbilityId] = ET.[AbilityId]
				AND TIT_TMP.[ScoreId] = ET.[ScoreId]
				AND TIT_TMP.[ID] = ET.[ID]
				AND TIT_TMP.[EvaluationPlanId] = ET.[EvaluationPlanId]
				AND TIT_TMP.[InteractionId] = ET.[InteractionId]
		
	-- ELIMINAMOS LA TABLA TEMPORAL
	DROP TABLE [dbo].[#TipificationInteractionTemp];
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[CheckCreatedIndexes]
AS
BEGIN
	SET NOCOUNT ON

	SELECT TableName = t.name,
			 IndexName = ind.name,
			 IndexId = ind.index_id,
			 ColumnId = ic.index_column_id,
			 ColumnName = col.name,
			 ind.*,
			 ic.*,
			 col.* 
		FROM sys.indexes ind 
		INNER JOIN 
			 sys.index_columns ic ON  ind.object_id = ic.object_id and ind.index_id = ic.index_id 
		INNER JOIN 
			 sys.columns col ON ic.object_id = col.object_id and ic.column_id = col.column_id 
		INNER JOIN 
			 sys.tables t ON ind.object_id = t.object_id 
		WHERE 
			 ind.is_primary_key = 0 
			 AND ind.is_unique = 0 
			 AND ind.is_unique_constraint = 0 
			 AND t.is_ms_shipped = 0 
			 --AND t.name in ('EvaluationResult', 'HistoricEvaluationFormat', 'HistoricAbility', 'EvaluationPlan', 'AuditorEvaluationPlan', 'HistoricGroup', 'HistoricSubGroup')
		ORDER BY 
			 t.name, ind.name, ind.index_id, ic.is_included_column, ic.key_ordinal;
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[CheckDatabaseStorageUsage]
AS
BEGIN
	SET NOCOUNT ON
	-- SP PARA EL VOLCADO DE RESULTADOS DE ANALISIS SEMANTICO EN ESTUDIOS

	SELECT a3.name + '.' + a2.name AS [name],
	  a1.rows AS [rows],
	  (a1.reserved + ISNULL(a4.reserved, 0)) * 8 AS [Reserved KB],
	  a1.data * 8 AS [Data KB],
	  (CASE
		   WHEN(a1.used + ISNULL(a4.used, 0)) > a1.data
		   THEN(a1.used + ISNULL(a4.used, 0)) - a1.data
		   ELSE 0
	   END) * 8 AS [Index_size KB],
	  (CASE
		   WHEN(a1.reserved + ISNULL(a4.reserved, 0)) > a1.used
		   THEN(a1.reserved + ISNULL(a4.reserved, 0)) - a1.used
		   ELSE 0
	   END) * 8 AS [unused KB],
	  CONVERT(DECIMAL(18, 2), (((a1.reserved + ISNULL(a4.reserved, 0)) * 8) - ((CASE
																					WHEN(a1.reserved + ISNULL(a4.reserved, 0)) > a1.used
																					THEN(a1.reserved + ISNULL(a4.reserved, 0)) - a1.used
																					ELSE 0
																				END) * 8)) / 1024.0 / 1024.0) AS [Table_used_Space GB]
	FROM
	(
	SELECT ps.object_id,
		  SUM(CASE
				  WHEN(ps.index_id < 2)
				  THEN row_count
				  ELSE 0
			  END) AS [rows],
		  SUM(ps.reserved_page_count) AS reserved,
		  SUM(CASE
				  WHEN(ps.index_id < 2)
				  THEN(ps.in_row_data_page_count + ps.lob_used_page_count + ps.row_overflow_used_page_count)
				  ELSE(ps.lob_used_page_count + ps.row_overflow_used_page_count)
			  END) AS data,
		  SUM(ps.used_page_count) AS used
	FROM sys.dm_db_partition_stats ps
	GROUP BY ps.object_id
	) AS a1
	LEFT OUTER JOIN
	(
	SELECT it.parent_id,
		  SUM(ps.reserved_page_count) AS reserved,
		  SUM(ps.used_page_count) AS used
	FROM sys.dm_db_partition_stats ps
		INNER JOIN sys.internal_tables it ON(it.object_id = ps.object_id)
	WHERE it.internal_type IN(202, 204)
	GROUP BY it.parent_id
	) AS a4 ON(a4.parent_id = a1.object_id)
	INNER JOIN sys.all_objects a2 ON(a1.object_id = a2.object_id)
	INNER JOIN sys.schemas a3 ON(a2.schema_id = a3.schema_id)
	WHERE a2.type != N'S'
	 AND a2.type != N'IT'
	ORDER BY [Table_used_Space GB] DESC, [rows] desc ;

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE procedure [dbo].[CleanFailedBatchData](
@idEvaluationPlan VARCHAR(50)
)
AS
BEGIN

DELETE FROM InteractionEvaluationPlan WHERE IdEvaluationPlan = @idEvaluationPlan
DELETE FROM AuditorEvaluationPlan WHERE IdEvaluationPlan = @idEvaluationPlan
DELETE FROM EvaluationPlan WHERE id = @idEvaluationPlan

--eliminar historicos generados

DELETE FROM [dbo].[HistoricEvaluationFormat] WHERE EvaluationPlanId = @idEvaluationPlan
DELETE FROM [dbo].[HistoricGroup] WHERE EvaluationPlanId = @idEvaluationPlan
DELETE FROM [dbo].[HistoricSubGroup] WHERE EvaluationPlanId = @idEvaluationPlan
DELETE FROM [dbo].[HistoricAbility] WHERE EvaluationPlanId = @idEvaluationPlan
DELETE FROM [dbo].[HistoricAbilityEvaluationFormat] WHERE EvaluationPlanId = @idEvaluationPlan
DELETE FROM [dbo].[HistoricAbilityQuery] WHERE EvaluationPlanId = @idEvaluationPlan
DELETE FROM [dbo].[HistoricAbilityQueryEntity] WHERE EvaluationPlanId = @idEvaluationPlan
DELETE FROM [dbo].[HistoricEvaluationFormatCustomField] WHERE EvaluationPlanId = @idEvaluationPlan
DELETE FROM [dbo].[HistoricCustomField] WHERE EvaluationPlanId = @idEvaluationPlan
DELETE FROM [dbo].[HistoricCustomFieldResult] WHERE EvaluationPlanId = @idEvaluationPlan
DELETE FROM [dbo].[HistoricEvaluationFormatAttachedSignature] WHERE EvaluationPlanId = @idEvaluationPlan
DELETE FROM [dbo].[HistoricCampaignEvaluationFormat] WHERE EvaluationPlanId = @idEvaluationPlan
				



END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE procedure [dbo].[CleanFailedEvaluationPlanData](
@VCC VARCHAR(50),
@idEvaluationPlan VARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

	-- Verifica que exista el proceso para eliminar.
	IF EXISTS(SELECT [ID] FROM [dbo].[EvaluationPlan] WHERE [Instance] = @VCC AND [ID] = @idEvaluationPlan)
	BEGIN
		-- ELIMINAMOS LA DATA DE LAS TABLAS PRINCIPALES
		DELETE FROM InteractionEvaluationPlan WHERE IdEvaluationPlan = @idEvaluationPlan AND Instance = @VCC
		DELETE FROM AuditorEvaluationPlan WHERE IdEvaluationPlan = @idEvaluationPlan AND Instance = @VCC
		DELETE FROM EvaluationPlan WHERE id = @idEvaluationPlan AND Instance = @VCC

		-- ELIMINAMOS LA DATA DE LAS TABLAS HISTORICAS
		DELETE FROM [dbo].[HistoricEvaluationFormat] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricAbilityEvaluationFormat] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricEvaluationFormatCustomField] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricCustomField] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricCustomFieldResult] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricEvaluationFormatAttachedSignature] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricCampaignEvaluationFormat] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricGroup] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricSubGroup] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricAbility] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricScore] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipification] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipificationQuery] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipificationItem] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricAbilityQuery] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
		DELETE FROM [dbo].[HistoricAbilityQueryEntity] WHERE [VCC] = @VCC AND [EvaluationPlanId] = @idEvaluationPlan 
	
		SELECT 1 AS 'result'
	END
	ELSE
	BEGIN
		SELECT 0 AS 'result'
	END		
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ClearDuplicateQueryResults] (
	@Type VARCHAR(25) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION
		DECLARE @BATCH_TYPE_ID VARCHAR(50) = @EvaluationPlanId

		-- AQUI EMPIEZA LA SECCION CON LOS INSERT/UPDATE/DELETE PARA LA NUEVA VERSION
		IF @Type = 'EVALUATION_PLAN'
		BEGIN 
			DECLARE RECURRENT_CURSOR CURSOR 
			LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- SELECCIONAMOS LOS VCC EXISTENTES
			SELECT [IdInteraction] 
				FROM [dbo].[InteractionEvaluationPlan] (NOLOCK) 
				WHERE [IdEvaluationPlan] = @BATCH_TYPE_ID
		END
		ELSE
		BEGIN
			DECLARE RECURRENT_CURSOR CURSOR 
			LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- SELECCIONAMOS LOS VCC EXISTENTES
			SELECT PA.[AudioId] 
				FROM [dbo].[ProcessAudio] PA (NOLOCK) 
				INNER JOIN [dbo].[Study] S (NOLOCK)
					ON S.[VCC] = PA.[VCC]
						AND S.[ProcessId] = PA.[ProcessId]
				WHERE S.[Name] = @BATCH_TYPE_ID
		END
		

		-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
		DECLARE @CURRENT_INTERACTION VARCHAR(50) = ''

		-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
		OPEN RECURRENT_CURSOR
		FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_INTERACTION
		WHILE @@FETCH_STATUS = 0
		BEGIN 

			-- PASAMOS LA DATA DE LA INTERACCION A LA TEMPORAL
			SELECT *
				INTO #TMP_IQR
				FROM [dbo].[InteractionQueryResult] (NOLOCK) 
				WHERE [BatchTypeName] = @BATCH_TYPE_ID
					AND [InteractionId] = @CURRENT_INTERACTION

			-- CONTAMOS LOS ITEMS TOTALES Y UNICOS
			DECLARE @TOTAL_RECORDS INT = 0
			DECLARE @UNIQUE_RECORDS INT = 0

			SELECT @TOTAL_RECORDS = COUNT(*) FROM #TMP_IQR
			SELECT @UNIQUE_RECORDS = COUNT(*) FROM (SELECT DISTINCT * FROM #TMP_IQR) AS DT

			IF @TOTAL_RECORDS <> @UNIQUE_RECORDS
			BEGIN
				PRINT CONCAT('INTERACCION ', @CURRENT_INTERACTION, ': REDUCIENDO DATA DE ', (CONVERT(VARCHAR(24), @TOTAL_RECORDS, 121)), ' A ', (CONVERT(VARCHAR(24), @UNIQUE_RECORDS, 121)))
				-- ELIMINAMOS LA DATA EN LA TABLA ORIGINAL
				DELETE FROM [dbo].[InteractionQueryResult] 
					WHERE [BatchTypeName] = @BATCH_TYPE_ID
						AND [InteractionId] = @CURRENT_INTERACTION

				PRINT('INSERTANDO DATA UNICA')
				-- REINSERTAMOS LA DATA UNICA EN LA TABLA ORIGINAL
				INSERT INTO [dbo].[InteractionQueryResult] 
					SELECT DISTINCT * FROM #TMP_IQR
			END

			-- ELIMINAMOS LA TABLA TEMPORAL
			PRINT('ELIMINAMOS LA TEMPORAL')
			DROP TABLE #TMP_IQR


			FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_INTERACTION
		END
		CLOSE RECURRENT_CURSOR
		DEALLOCATE RECURRENT_CURSOR

		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION
		PRINT('HACIENDO ROLLBACK')

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[CloseI6InteractionAsyncInteraction] (
	@VCC					VARCHAR(50) = NULL,
	@InteractionId			VARCHAR(250) = NULL,
	@Folder					VARCHAR(50) = NULL,
	@Channel				VARCHAR(25) = NULL,
	@Succed					BIT = 0,
	@ErrorMessage			VARCHAR(50) = NULL
) AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR UNA INTERACCION DE I6
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- ACTUALIZAMOS LA TABLA DE LA INTERACCION EN LA TABLA ASYNC_INTERACTION
		UPDATE [dbo].[AsyncInteraction]
			SET [IsProcess] = NULL,
				[IsCancel] = NULL,
				[NeedI6Update] = IIF (@Succed = 1, 0, 1),
				[ErrorMessage] = @ErrorMessage,
				[LastModifiedByUserId] = 'system',
				[LastModifiedDate] = GETUTCDATE(),
				[TimeStamp] = GETUTCDATE()
			WHERE [VCC] = @VCC
				AND [ID] = @InteractionId
				AND [Folder] = @Folder
				AND [NeedI6Update] = 1
				AND (
					[IsTranscribed] IS NULL 
					OR
					[IsTranscribed] <> 1
				)

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[CloseI6InteractionAudio] (
	@VCC					VARCHAR(50) = NULL,
	@InteractionId			VARCHAR(250) = NULL,
	@Succed					BIT = 0,
	@ErrorMessage			VARCHAR(50) = NULL
) AS 
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR UNA INTERACCION DE I6
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- ACTUALIZAMOS EL SECTION Y PART EN LA INTERACCION EN LA TABLA AUDIO
		UPDATE [dbo].[Audio] 
			SET [IsProcess] = NULL,
				[IsCancel] = NULL,
				[NeedI6Update] = IIF (@Succed = 1, 0, 1),
				[ErrorMessage] = IIF (@Succed = 1, NULL, @ErrorMessage),
				[LastModifiedByUserId] = 'system',
				[LastModifiedDate] = GETUTCDATE(),
				[TimeStamp] = GETUTCDATE()
			WHERE [VCC] = @VCC
				AND [ID] = @InteractionId
				AND [NeedI6Update] = 1
				AND (
					[IsTranscribed] IS NULL 
					OR
					[IsTranscribed] <> 1
				)
			
		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[CreateVcc] (
	@VCC Varchar(50) = NULL, 
	@ID Varchar(50) = NULL,
	@Description Varchar(50) = NULL,
	@Type VARCHAR(50) = NULL,
	@TranscriptionDefaultLanguage VARCHAR(25) = NULL,
	@CognitiveEngine VARCHAR(25) = NULL,
	@Country Varchar(50) = NULL,
	@Language Varchar(50) = NULL,
	@TimeZone Varchar(50) = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId   VARCHAR(50) = NULL,
	@DeleteDataDays         INT = 360,
	@UseRasaApi				BIT = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	BEGIN TRY
    BEGIN TRANSACTION


	INSERT INTO [dbo].[VCC] (
			[VCC], 
			[ID], 
			[Description], 
			[Type], 
			[Country], 
			[TranscriptionDefaultLanguage], 
			[CognitiveEngine], 
			[Language],
			[TimeZone], 
			[CreatedDate], 
			[CreatedByUserId], 
			[LastModifiedDate], 
			[LastModifiedByUserId], 
			[DeleteDataDays], 
			[TimeStamp]
			)
	VALUES (
			@VCC, 
			@ID, 
			@Description, 
			@Type, 
			@Country, 
			@TranscriptionDefaultLanguage, 
			@CognitiveEngine, 
			@Language, 
			@TimeZone, 
			@CreatedDate,  
			@CreatedByUserId, 
			@LastModifiedDate, 
			@LastModifiedByUserId, 
			@DeleteDataDays,
			GETUTCDATE()
		)
	
	DECLARE @VCC_SYSTEM       VARCHAR(50) = 'system'

    --INSERTAR ROLES
	
	DECLARE @ROLES TABLE(
    [ID]            VARCHAR(50),
    [Description]   VARCHAR(50),
	[CreatedDate]   DATETIME,
    [CreatedByUserId] VARCHAR(50),
    [LastModifiedDate] DATETIME,
    [LastModifiedByUserId] VARCHAR(50),
	[SystemRole]    BIT,
    [Type]                 VARCHAR(50)
	)
	INSERT INTO @ROLES EXEC [dbo].[GetRoles] @VCC_SYSTEM,'name',''

	IF(EXISTS(SELECT [ID] FROM @ROLES))
	BEGIN
			INSERT INTO [dbo].[Role] (
					[VCC],
					[ID], 
					[Description],
					[CreatedDate], 
					[CreatedByUserId], 
					[LastModifiedDate],
					[LastModifiedByUserId],
					[SystemRole],
					[Type],
					[TimeStamp]
					)
			SELECT @VCC,
				   [ID],
				   [Description],
				   GETUTCDATE(), 
				   [CreatedByUserId],
				   GETUTCDATE(),
				   [LastModifiedByUserId],
				   [SystemRole],
				   [Type],
				   GETUTCDATE()
			FROM @ROLES
     END
	 ELSE
	 BEGIN
	       RAISERROR('SERVER_ERROR_ROLES_NOT_FOUND', 16, 16)
	 END

	--INSERTAR PERMISOS PARA LOS ROLES

	DECLARE @CountLicense INT = 0
	DECLARE @License      VARCHAR(50)
	
	DECLARE @ROLE_PERMISSIONS TABLE(
	 [ID]      VARCHAR(50),
	 [GroupId] VARCHAR(200)
	)
	DECLARE @ROUTE_PERMISSIONS TABLE(
	 [ID]      VARCHAR(50),
	 [Route]   VARCHAR(250),
	 [Level]   VARCHAR(50),
	 [GroupId] VARCHAR(200)
	)

	SET @CountLicense = (SELECT COUNT(*) FROM [dbo].[VCCLicense] (NOLOCK) WHERE [VCC] = @VCC AND [Status] = 1)
	
	IF(@CountLicense>1)
	BEGIN
	    INSERT INTO @ROLE_PERMISSIONS EXEC [dbo].[GetRolePermission] @VCC_SYSTEM,''
		INSERT INTO @ROUTE_PERMISSIONS EXEC [dbo].[GetRoleRoutePermission] @VCC_SYSTEM
	END

	IF(@CountLicense=1)
	BEGIN
	    SET @License = (SELECT Product FROM [dbo].[VCCLicense] (NOLOCK) WHERE [VCC] = @VCC AND [Status] = 1)
	    INSERT INTO @ROLE_PERMISSIONS EXEC [dbo].[GetRolePermission] @VCC_SYSTEM,'', @License
		INSERT INTO @ROUTE_PERMISSIONS EXEC [dbo].[GetRoleRoutePermission] @VCC_SYSTEM,'', @License
	END
	
	INSERT INTO [dbo].[RolePermission] (
			    [VCC], 
				[RoleId], 
				[GroupId],
			    [TimeStamp]
			    ) 
				SELECT 
				@VCC,
				[ID],
				[GroupId],
			    GETUTCDATE()
				FROM @ROLE_PERMISSIONS

	INSERT INTO [dbo].[RoutePermission] (
			[VCC], 
			[ID], 
			[GroupId],
			[Route], 
			[Level], 
			[TimeStamp]
			)
			SELECT 
			@VCC,
			[ID],
			[GroupId],
			[Route],
			[Level],
			GETUTCDATE()
			FROM @ROUTE_PERMISSIONS


    --INSERTAR GRUPOS DE PERMISOS PARA LOS ROLES
	DECLARE @ROLE_PERMISSIONS_GROUP TABLE(
	 [Category] VARCHAR(50),
	 [GroupId]  VARCHAR(200),
	 [Type]     VARCHAR(50)
	)

	INSERT INTO @ROLE_PERMISSIONS_GROUP EXEC [dbo].[GetRolePermissionsGroups] @VCC_SYSTEM

	INSERT INTO [dbo].[RolePermissionsGroup] (
			                              [VCC], 
										  [GroupId], 
										  [Category], 
										  [Type], 
										  [TimeStamp]
										  )
		                                 SELECT 
										 @VCC, 
										 GroupId,
										 Category,
										 [Type], 
										 GETUTCDATE()	
										 FROM @ROLE_PERMISSIONS_GROUP

    --INSERTAR FEELINGS
	INSERT INTO [dbo].[Feeling] (
			                   [VCC], 
							   [Name], 
							   [Description],
							   [CreatedDate], 
							   [CreatedByUserId], 
							   [LastModifiedDate], 
							   [LastModifiedByUserId], 
							   [TimeStamp]
							   )
							   SELECT
							   @VCC,
							   [Name],
							   [Description],
							   GETUTCDATE(),
							   [CreatedByUserId],
							   GETUTCDATE(),
							   [LastModifiedByUserId],
							   GETUTCDATE()	
							   FROM [dbo].[Feeling] (NOLOCK)
							   WHERE [VCC] = @VCC_SYSTEM

    --INSERTAR PROGRAMACION NEUROLINGUISTICA
	INSERT INTO [dbo].[NeuroLinguisticProgramming] (
								[VCC],
								[Name],
								[Description],
								[CreatedDate],
								[CreatedByUserId],
								[LastModifiedDate], 
								[LastModifiedByUserId],
								[TimeStamp]
								)
								SELECT 
								@VCC,
								[Name],
								[Description],
								GETUTCDATE(),
								[CreatedByUserId],
								GETUTCDATE(),
								[LastModifiedByUserId],
								GETUTCDATE()
								FROM [dbo].[NeuroLinguisticProgramming] (NOLOCK)
								WHERE [VCC] = @VCC_SYSTEM
		
   --INSERTAR PALABRAS
   IF(EXISTS(SELECT [Name] FROM [dbo].[Word] (NOLOCK) WHERE [VCC] = @VCC_SYSTEM))
   BEGIN
		   INSERT INTO [dbo].[Word](
					[VCC],
					[NeuroLinguisticProgrammingId],
					[Name],
					[ProcessId],
					[Language],
					[Type],
					[CreatedDate],
					[CreatedByUserId],
					[LastModifiedDate],
					[LastModifiedByUserId],
					[TimeStamp]
					)
				SELECT
				@VCC,
				[NeuroLinguisticProgrammingId],
				[Name],
				[ProcessId],
				[Language],
				[Type],
				GETUTCDATE(),
				[CreatedByUserId],
				GETUTCDATE(),
				[LastModifiedByUserId],
				GETUTCDATE()
			   FROM [dbo].[Word] (NOLOCK)
			   WHERE [VCC] = @VCC_SYSTEM
   END
   ELSE
   BEGIN
	       RAISERROR('SERVER_ERROR_WORD_NOT_FOUND', 16, 16)
   END

   --INSERTAR CATEGORIAS
   INSERT INTO [dbo].[Category] (
			[VCC], 
			[Name], 
			[Description],
			[CreatedDate], 
			[CreatedByUserId], 
			[LastModifiedDate], 
			[LastModifiedByUserId], 
			[TimeStamp]
			)
		SELECT 
		@VCC,
		[Name],
		[Description],
		GETUTCDATE(),
		[CreatedByUserId],
		GETUTCDATE(),
		[LastModifiedByUserId],
		GETUTCDATE()
	FROM [dbo].[Category] (NOLOCK)
	WHERE [VCC] = @VCC_SYSTEM

	--INSERTAR AMBIENTES
	IF(@Type <> 'OTHER')
	BEGIN
		DECLARE @NAME VARCHAR(100)
		SELECT @NAME = [Word] FROM [dbo].[WordsByLanguage] (NOLOCK) WHERE [Language] = @Language AND [Label] = 'ENVIRONMENT' 
		
		-- DECLARAMOS UNA VARIABLE PARA SETTEAR EL TIPO DE AMBIENTE INTEGRADO PARA EL VCC
		DECLARE @VCC_TYPE VARCHAR(25) = NULL
		IF (@Type = 'ALLEGRO' OR @Type = 'OCC') SET @VCC_TYPE = 'allegro'
		ELSE IF @Type = 'I6' SET @VCC_TYPE = 'i6'
		
		INSERT INTO [dbo].[Environment] (
			[VCC], 
			[Name], 
			[Type],
			[Description],
			[FTPDirectory],
			[ApiHost],
			[ApiPort],
			[ApiUser],
			[DbDisabled],
			[DbServer],
			[DbPort],
			[DbUser], 
			[PbxDisabled],
			[PbxServer],
			[PbxPort],
			[PbxUser],
			[ApiPassword],
			[DbPassword],
			[PbxPassword],
			[CreatedDate],
			[CreatedByUserId],
			[TimeStamp]
		)
			SELECT @VCC,
					@NAME,
					[Type],
					@NAME,
					[FTPDirectory],
					[ApiHost],
					[ApiPort],
					[ApiUser],
					[DbDisabled],
					[DbServer],
					[DbPort],
					[DbUser],
					[PbxDisabled],
					[PbxServer]	,
					[PbxPort],
					[PbxUser],
					[ApiPassword],
					[DbPassword],
					[PbxPassword],
					GETUTCDATE(),
					[CreatedByUserId],
					GETUTCDATE()
				FROM [dbo].[Environment] (NOLOCK)
				WHERE [VCC] = @VCC_SYSTEM
					AND [Type] = @VCC_TYPE

	END


	--INSERTAR PERMISOS PARA CATEGORIAS
	INSERT INTO [dbo].[CategoryPermissions](
			[VCC], 
			[Category], 
			[Position], 
			[TimeStamp]
			)
			SELECT
			@VCC,
		    [Category],
			[Position],
			GETUTCDATE()
			FROM [dbo].[CategoryPermissions] (NOLOCK) 
			WHERE [VCC] = @VCC_SYSTEM
			GROUP BY [Category],[Position]
		  

   --INSERTAR TENANT
   DECLARE @TENANT TABLE(
    [TenantId]     VARCHAR(50),
    [ClientId]     VARCHAR(50),
    [ClientSecret] VARCHAR(50),
    [Resource]     VARCHAR(50)
	)
	
	INSERT INTO @TENANT EXEC [dbo].[GetTenantVCCByVCC] @VCC_SYSTEM
	IF(EXISTS(SELECT [TenantId] FROM @TENANT))
	BEGIN

		INSERT INTO [dbo].[TenantVCC](
		[VCC], 
		[TenantId], 
		[ClientId], 
		[ClientSecret], 
		[Resource], 
		[CreatedDate], 
		[CreatedByUserId], 
		[TimeStamp])
		SELECT 
		@VCC,
		[TenantId],
		[ClientId],
		[ClientSecret],
		[Resource],
		GETUTCDATE(),
		@CreatedByUserId,
		GETUTCDATE()
		FROM @TENANT

	END
	ELSE
	BEGIN
	   RAISERROR('SERVER_ERROR_TENANT_NOT_FOUND', 16, 16)
	END

	--INSERTAR NLU
	IF(EXISTS(SELECT [Name] FROM [dbo].[NLU] WHERE [IsSystemApp]=1 AND [VCC] = @VCC_SYSTEM))
	BEGIN
		INSERT INTO [dbo].[NLU] (
					[VCC], 
					[Name], 
					[Culture],
					[Description], 
					[AppId], 
					[NeedTrain],
					[SubscriptionKey],
					[IsSystemApp], 
					[CreatedDate], 
					[CreatedByUserId], 
					[LastModifiedDate], 
					[LastModifiedByUserId], 
					[TimeStamp],
					[CognitiveEngine]
		)
			SELECT @VCC,
					[Name], 
					[Culture],
					[Description], 
					CONCAT(@VCC,'::',[Name]), 
					0,
					[SubscriptionKey],
					[IsSystemApp], 
					GETUTCDATE(), 
					[CreatedByUserId], 
					GETUTCDATE(), 
					[LastModifiedByUserId], 
					GETUTCDATE(),
					CASE WHEN @UseRasaApi = 0 THEN 'RASA' ELSE 'RASAAPI' END
				FROM [dbo].[NLU] (NOLOCK)
				WHERE [IsSystemApp]=1 
					AND [VCC] = @VCC_SYSTEM
					AND [Name] NOT LIKE 'sentiment-%'
	    
		
		

		-- INSERTAMOS LOS REGISTROS DE MODELOS DE DIARIZACION
        INSERT INTO [dbo].[Intent](
			[VCC], 
			[Name],
			[IntentId],
			[AppId],
			[CreatedDate],
			[CreatedByUserId],
			[LastModifiedDate], 
			[LastModifiedByUserId], 
			[TimeStamp],
			[Status],
			[StatusCode]
		)
			SELECT @VCC,
					'Agent',
					'Agent',
					CONCAT(@VCC,'::',[Name]),
					GETUTCDATE(),
					@CreatedByUserId,
					GETUTCDATE(),
					@CreatedByUserId,
					GETUTCDATE(),
					1,
					'AVAILABLE'
			FROM [dbo].[NLU]
			WHERE [IsSystemApp]=1 
				AND [VCC] = @VCC_SYSTEM
				AND [Name] LIKE 'diarization-%'


		
	END
	ELSE
	BEGIN
	   RAISERROR('SERVER_ERROR_SYSTEM_APPS_SAVING_FAILED', 16, 16)
	END

	--INSERTAR DICCIONARIOS
	IF(EXISTS(SELECT [Name] FROM [dbo].[Dictionary] (NOLOCK) WHERE [VCC] = @VCC_SYSTEM))
	BEGIN 
	INSERT INTO [dbo].[Dictionary] (
			[VCC], 
			[Name], 
			[Description], 
			[Language], 
			[IsSystemDictionary], 
			[CreatedDate], 
			[CreatedByUserId], 
			[TimeStamp]
		)
		SELECT
		    @VCC,
		    [Name],
			[Description],
			[Language],
			[IsSystemDictionary],
			GETUTCDATE(),
			[CreatedByUserId],
			GETUTCDATE()
		FROM [dbo].[Dictionary] (NOLOCK)
		WHERE [VCC] = @VCC_SYSTEM
	END
	ELSE
	BEGIN
	    RAISERROR('SERVER_ERROR_SYSTEM_DICTIONARIES_SAVING_FAILED',16,16)
	END

	--INSERTAR TIPOS DE INTERACCIONES
	IF(EXISTS(SELECT [ID] FROM [dbo].[InteractionType] WHERE [Instance] = @VCC_SYSTEM ))
	BEGIN
	INSERT INTO [dbo].[InteractionType] (
			[Instance],
			[ID],
			[Description]
			)
			SELECT
			@VCC,
			[ID],
		    [Description]
			FROM InteractionType  (NOLOCK)
			WHERE [Instance] = @VCC_SYSTEM
	END
	ELSE
	BEGIN
	   RAISERROR('SERVER_ERROR_INTERACTION_TYPE_NOT_FOUND',16,16) 
	END

	--INSERTAR GENERIC QUERY
	IF(EXISTS(SELECT [Name] FROM [dbo].[GenericQuery] (NOLOCK) WHERE [VCC] = @VCC_SYSTEM))
	BEGIN
	INSERT INTO [dbo].[GenericQuery]
		(
			[VCC],
			[Name],
			[Description],
			[Query],
			[RawData],
			[CreatedDate],
			[CreatedByUserId],
			[LastModifiedDate], 
			[LastModifiedByUserId],
			[Version],
			[TimeStamp]
		)
		SELECT
		@VCC,
		[Name],--REPLACE([Name], @Language+'_', ''),
		[Description],
		[Query],
		[RawData],
		GETUTCDATE(),
		[CreatedByUserId],
		GETUTCDATE(),
		[LastModifiedByUserId],
		[Version],
		GETUTCDATE()
		FROM [dbo].[GenericQuery] (NOLOCK)
		WHERE [VCC] = @VCC_SYSTEM --AND [Name] LIKE @Language + '%'
	END
	ELSE
	BEGIN
	   RAISERROR('SERVER_ERROR_GENERIC_QUERY_NOT_FOUND',16,16) 
	END

	--INSERTAR  USUARIO AUDITOR SPEECH
	EXEC [dbo].[SaveSpeechAnalyticAuditorUser] @VCC

	--INSERTAR REPORTES
	IF(EXISTS(SELECT [ID] FROM  [dbo].[Report] (NOLOCK) WHERE [VCC] = @VCC_SYSTEM))
	BEGIN
	INSERT INTO [Report] (
			[VCC], 
			[ID], 
			[Description],
			[Type],
            [Procedure], 
			[StartDate],
			[EndDate],
            [Frecuency],
			[HourSpace],
			[ExactTime],
            [Weekday],
			[DayMonth],
			[Summary],
            [Filter], 
			[Email], 
			[CreatedDate],
            [CreatedByUserId],
			[LastModifiedDate],
			[LastModifiedByUserId],
            [TimeStamp]
        )
		SELECT
		        @VCC,
				[ID],
				[Description],
				[Type],
				[Procedure],
				[StartDate],
				[EndDate],
				[Frecuency],
				[HourSpace],
				[ExactTime],
				[Weekday],
				[DayMonth],
				[Summary],
				[Filter],
				[Email],
				GETUTCDATE(),
				@CreatedByUserId,
				GETUTCDATE(),
				@CreatedByUserId,
				GETUTCDATE()
			FROM [dbo].[Report] (NOLOCK)
			WHERE [VCC] = @VCC_SYSTEM

	END
	ELSE
	BEGIN
	   RAISERROR('SERVER_ERROR_REPORT_NOT_FOUND',16,16) 
	END

		-- INSERTAMOS LOS GRUPOS POR DEFECTO PARA COPC
		INSERT INTO [dbo].[Group] ([Instance], [ID], [Description], [CreatedDate], [CreatedByUserId], [TimeStamp], [Enabled])
			SELECT @VCC,
					G.[Description],
					WBL.[Word],
					GETUTCDATE(),
					'system',
					GETUTCDATE(),
					1
				FROM [dbo].[Group] G (NOLOCK)
				INNER JOIN [dbo].[WordsByLanguage] WBL (NOLOCK)
					ON WBL.[Label] = G.[Description]
				WHERE G.[Instance] = @VCC_SYSTEM
					AND WBL.[Language] = @Language

		-- INSERTAMOS LOS SUBGRUPOS POR DEFECTO PARA COPC
		INSERT INTO [dbo].[SubGroup] ([Instance], [ID], [Description], [CreatedDate], [CreatedByUserId], [TimeStamp], [Enabled])
			SELECT @VCC,
					SG.[Description],
					WBL.[Word],
					GETUTCDATE(),
					'system',
					GETUTCDATE(),
					1
				FROM [dbo].[SubGroup] SG (NOLOCK)
				INNER JOIN [dbo].[WordsByLanguage] WBL (NOLOCK)
					ON WBL.[Label] = SG.[Description]
				WHERE SG.[Instance] = @VCC_SYSTEM
					AND WBL.[Language] = @Language

	COMMIT TRANSACTION

    END TRY

    BEGIN CATCH

    IF @@TRANCOUNT > 0 
	BEGIN
		ROLLBACK TRANSACTION
		--ELIMINAR LOS REGISTROS SI HA FALLADO
		EXEC [dbo].[RollbackVcc] @VCC
	END

    DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
    DECLARE @Severity integer = ERROR_SEVERITY()
    DECLARE @State integer = ERROR_STATE()

    RAISERROR(@Message, @Severity, @State)
  
    END CATCH
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteAbilities] (
	@Instance Varchar(50) = NULL,
	@AbilityIDs Varchar(MAX) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
		
    DECLARE @_IDS TABLE(ID VARCHAR(50))

    INSERT INTO @_IDS SELECT [Value] FROM [dbo].[Split](@AbilityIDs,',')

    -- ELIMINAMOS LO ITEMS ASOCIADOS A LOS SCORES
    DECLARE RECURRENT_CURSOR CURSOR 
    LOCAL STATIC READ_ONLY FORWARD_ONLY
    FOR 
    -- SELECCIONAMOS LOS VCC EXISTENTES
    SELECT [ID]
        FROM [Score]
        WHERE [Instance] = @Instance
            AND  [IdAbility] IN (SELECT [ID] FROM @_IDS)
    
    -- DECLARARAMOS UNA VARIABLE PARA INDICAR EL SCORE ACTUAL QUE ESTAMOS ELIMINANDO
    DECLARE @CURRENT_SCORE_TO_REMOVE VARCHAR(50) = ''

    -- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
    OPEN RECURRENT_CURSOR

    FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_SCORE_TO_REMOVE
    WHILE @@FETCH_STATUS = 0
    BEGIN
        -- ELIMINAMOS LOS REGISTROS DE SCORETIPIFICATION
        DELETE [ScoreTipificationItem] 
            WHERE [VCC] = @Instance
                AND [ScoreId] = @CURRENT_SCORE_TO_REMOVE

        -- ELIMINAMOS LOS REGISTROS DE SCORETIPIFICATION
        DELETE [ScoreTipification] 
            WHERE [VCC] = @Instance
                AND [ScoreId] = @CURRENT_SCORE_TO_REMOVE

        -- ELIMINAMOS LOS REGISTROS DE SCORE
        DELETE [Score]
            WHERE [Instance] = @Instance
                AND  [IdAbility] IN (SELECT [ID] FROM @_IDS)
                
        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_SCORE_TO_REMOVE
    END
    CLOSE RECURRENT_CURSOR
    DEALLOCATE RECURRENT_CURSOR

    -- ELIMINAMOS LOS REGISTROS ABILITYQUERY
    DELETE [AbilityQuery] 
        WHERE [VCC] = @Instance 
            AND  [AbilityId] IN (SELECT [ID] FROM @_IDS)

    -- ELIMINAMOS EL REGISTRO ABILITY
    DELETE [Ability] 
        WHERE [Instance] = @Instance
            AND [ID] IN (SELECT [ID] FROM @_IDS)

    DELETE [AbilityScoreCard] 
        WHERE [VCC] = @Instance
            AND [ID] IN (SELECT [ID] FROM @_IDS)

    SELECT 1 AS 'result'
       
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[DeleteAbility] (
	@Instance Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION
			-- ELIMINAMOS LO ITEMS ASOCIADOS A LOS SCORES
			DECLARE RECURRENT_CURSOR CURSOR 
			LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- SELECCIONAMOS LOS VCC EXISTENTES
			SELECT [ID]
				FROM [Score]
				WHERE [Instance] = @Instance
					AND [IdAbility] = @ID 
			
			-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL SCORE ACTUAL QUE ESTAMOS ELIMINANDO
			DECLARE @CURRENT_SCORE_TO_REMOVE VARCHAR(50) = ''

			-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
			OPEN RECURRENT_CURSOR

			FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_SCORE_TO_REMOVE
			WHILE @@FETCH_STATUS = 0
			BEGIN
				-- ELIMINAMOS LOS REGISTROS DE SCORETIPIFICATION
				DELETE [ScoreTipificationItem] 
					WHERE [VCC] = @Instance
						AND [ScoreId] = @CURRENT_SCORE_TO_REMOVE

				-- ELIMINAMOS LOS REGISTROS DE SCORETIPIFICATION
				DELETE [ScoreTipification] 
					WHERE [VCC] = @Instance
						AND [ScoreId] = @CURRENT_SCORE_TO_REMOVE

				-- ELIMINAMOS LOS REGISTROS DE SCORE
				DELETE [Score]
					WHERE [Instance] = @Instance
						AND [IdAbility] = @ID 
						
				FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_SCORE_TO_REMOVE
			END
			CLOSE RECURRENT_CURSOR
			DEALLOCATE RECURRENT_CURSOR

			-- ELIMINAMOS LOS REGISTROS ABILITYQUERY
			DELETE [AbilityQuery] 
				WHERE [VCC] = @Instance 
					AND [AbilityId] = @ID

			-- ELIMINAMOS EL REGISTRO ABILITY
			DELETE [Ability] 
				WHERE [Instance] = @Instance
					AND [ID] = @ID

			DELETE [AbilityScoreCard] 
				WHERE [VCC] = @Instance
					AND [ID] = @ID

			SELECT 1 AS 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteAbilityQueries] (
	@VCC VARCHAR(50) = NULL, 
	@AbilityId VARCHAR(100) = NULL
) AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON;

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS VCC QUE ACTUALIZAREMOS
		DECLARE RECURRENT_CURSOR CURSOR 
		LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
			-- SELECCIONAMOS LOS ABILITY_QUERY RELACIONADOS AL ABILITY
			SELECT AQ.[ID]
				FROM [dbo].[AbilityQuery] AQ (NOLOCK) 
				WHERE AQ.[VCC] = @VCC 
					AND AQ.[AbilityId] = @AbilityId

		-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS ACTUALIZANDO
		DECLARE @CURRENT_ABILITY_ID VARCHAR(50) = ''

		-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
		OPEN RECURRENT_CURSOR
		FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_ABILITY_ID
		WHILE @@FETCH_STATUS = 0
		BEGIN 

			-- ELIMINAMOS LOS ABILITY_QUERY_ENTITY QUE COINCIDEN CON EL ABILITY_QUERY_ID ACTUAL
			DELETE FROM [dbo].[AbilityQueryEntity]
				WHERE [AbilityQueryId] = @CURRENT_ABILITY_ID

			FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_ABILITY_ID
		END
		CLOSE RECURRENT_CURSOR
		DEALLOCATE RECURRENT_CURSOR

		-- SP PARA ELIMINAR TODAS LAS ABILITY_QUERIES
		DELETE FROM [dbo].[AbilityQuery] 
			WHERE [VCC] = @VCC 
				AND [AbilityId] = @AbilityId
		
		SELECT 1 AS 'result'
    
		SET NOCOUNT OFF

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	SET NOCOUNT OFF

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
create PROCEDURE [dbo].[DeleteAdvancedAIExample] (
    @VCC VARCHAR(50) = NULL,
    @AppId VARCHAR(100) = NULL,
	@IntentId VARCHAR(50) = NULL,
    @Id VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON

    -- ELIMINAMOS LOS EXAMPLES REGISTRADOS CON EL APPID E INTENTID EN LA TABLA [dbo].[GenerativeExample]
    DELETE FROM [dbo].[AdvancedAIExample]
		WHERE [VCC] = @VCC
		  AND [AppId] = @AppId
		  AND [IntentId] = @IntentId
		  AND [Id] = @Id

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteAdvancedAIIntent] (
    @VCC VARCHAR(50) = NULL,
    @AppId VARCHAR(100) = NULL,
    @Id VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON

    -- ELIMINAMOS EL INTENT DE LA TABLA [dbo].[AdvancedAIIntent]
    DELETE FROM [dbo].[AdvancedAIIntent]
		WHERE [VCC] = @VCC
		  AND [AppId] = @AppId
		  AND [Id] = @Id

    -- ELIMINAMOS LOS EXAMPLES REGISTRADOS CON EL APPID E INTENTID EN LA TABLA [dbo].[AdvancedAIExample]
    DELETE FROM [dbo].[AdvancedAIExample]
		WHERE [VCC] = @VCC
		  AND [AppId] = @AppId
		  AND [IntentId] = @Id

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteApiUser] (
	@VCC Varchar(50) = NULL, @User Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
		
	DELETE FROM [UserAPI]
	WHERE [VCC] = @VCC
	AND [User] = @User
    
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[DeleteAuditorEvaluationPlan] (
	@Instance Varchar(50) = NULL, @IdEvaluationPlan Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	DELETE
	    [AuditorEvaluationPlan]
	WHERE
	    [Instance] = @Instance
	    AND [IdEvaluationPlan] = @IdEvaluationPlan
		AND ISNULL([Completed], 0) = 0

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTA FUNCION TIENE POR FINALIDAD ELIMINAR OTRAS EVALUACIONES PENDIENTES DE UNA INTERACCION CORRESPONDIENTES AL MISMO 
--  PLAN DE EVALUACION
CREATE PROCEDURE [dbo].[DeleteAuditorEvaluationPlanByInteraction] (
	@VCC Varchar(50) = NULL,
	@IdEvaluationPlan Varchar(50) = NULL,
	@IdInteraction Varchar(250) = NULL,
	@InteractionType Varchar(50) = NULL,
	@IsDraft BIT = NULL
) AS
BEGIN
	SET NOCOUNT ON
	DECLARE @EvaluationPlanMode Varchar(50) = NULL

	-- DECLARAMOS UNA VARIABLE PARA VERIFICAR EL TIPO DEL PLAN
	DECLARE @IS_MIXED_EVALUATION_PLAN BIT = 0

	-- CONSULTAMOS LA MODALIDAD DEL PLAN DE EVALUACION
	SELECT @EvaluationPlanMode = EP.[Modality],
			@IS_MIXED_EVALUATION_PLAN = IIF (EP.[Evaluation_Mode] = 'X', 1, 0)
		FROM [dbo].[EvaluationPlan] EP (NOLOCK)
		WHERE EP.[Instance] = @VCC
			AND EP.[ID] = @IdEvaluationPlan

	-- SI ES UN PLAN DE MODALIDAD COMPARTIDA(FREE) ELIMINAMOS LAS DEMAS EVALUACIONES DE LA MISMA INTERACCION QUE EXISTEN 
	--  PARA LOS OTROS AUDITORES
	IF (@EvaluationPlanMode = 'FREE')
	BEGIN 
		-- 
		IF @IsDraft = 1
		BEGIN
			-- SELECT * FROM [dbo].[AuditorEvaluationPlan] (NOLOCK)
			-- SI SE GUARDA COMO BORRADOR, INHABILITAMOS LOS DEMAS REGISTROS PARA AUDITORES
			UPDATE [dbo].[AuditorEvaluationPlan]
				SET [Status] = 0
				WHERE [Instance] = @VCC
					AND [IdEvaluationPlan] = @IdEvaluationPlan
					AND [IdInteraction] = @IdInteraction
					AND [InteractionType] = @InteractionType
					AND ISNULL([Completed], 0) = 0
					AND [StatusDescription] <> 'DRAFT'
		END
		ELSE 
		BEGIN
		    -- SI NO SE GUARDA COMO BORRADOR, ELIMINAMOS LOS DEMAS REGISTROS PARA OTROS AUDITORES
			DELETE [dbo].[AuditorEvaluationPlan]
				WHERE [Instance] = @VCC
					AND [IdEvaluationPlan] = @IdEvaluationPlan
					AND [IdInteraction] = @IdInteraction
					AND [InteractionType] = @InteractionType
					AND ISNULL([Completed], 0) = 0

			-- ELIMINAMOS LOS REGISTROS BORRADOR QUE SE GENERAN PARA PLANES MIXTOS
			IF @IS_MIXED_EVALUATION_PLAN = 1
				DELETE FROM [dbo].[EvaluationResult]
					WHERE [Instance] = @VCC
						AND [IdEvaluationPlan] = @IdEvaluationPlan
						AND [IdInteraction] =  @IdInteraction
						AND [InteractionType] = @InteractionType
						AND [IdAuditor] LIKE '%-DRAFT'
		END
	END 

	SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteBatchSchedule] (
	@VCC VARCHAR(50) = NULL, 
	@BatchType VARCHAR(50) = NULL,
	@ID VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- ELIMINAMOS LA AGENDA
	DELETE [dbo].[BatchSchedule] 
		WHERE [VCC] = @VCC
			AND [BatchType] = @BatchType
			AND [ID] = @ID

	-- ELIMINAMOS LAS EJECUCIONES
	DELETE [dbo].[BatchScheduleExecution] 
		WHERE [VCC] = @VCC
			AND [BatchScheduleId] = @ID
    
	-- RETORNAMOS EL RESULTADO
	SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[DeleteBatchScheduleExecution] (
	@VCC VARCHAR(50) = NULL, 
	@ID VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	-- ELIMINAMOS LA EJECUCION
	DELETE [BatchScheduleExecution] 
		WHERE [VCC] = @VCC
			AND [Id] = @ID

    -- RETORNAMOS EL RESULTADO
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--[DeleteCategory] 'inconcert', 'banca'
CREATE PROCEDURE [dbo].[DeleteCategory] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [Category]
	WHERE [VCC] = @VCC
	AND [Name] = @Name

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteClient] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [Client]
	WHERE [VCC] = @VCC
	AND [ID] = @ID

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[DeleteCustomField] (
	@VCC VARCHAR(50) = NULL, 				
	@CustomFieldId VARCHAR(100) = NULL
) AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON;

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS SI EL CUSTOMFIELD ESTA RELACIONADO A ALGUN FORMATO EN USO
		IF EXISTS (
			SELECT DISTINCT EFCF.[CustomFieldId] 'customFieldId'
				FROM [dbo].[EvaluationFormatCustomField] EFCF (NOLOCK)
				WHERE EFCF.[VCC] = @VCC 
					AND EFCF.[CustomFieldId] = @CustomFieldId
		)	
			RAISERROR('DATABASE_ERROR_CUSTOM_FIELD_ON_USE', 16, 16)

		-- ELIMINAMOS LOS RESULTS DEL CUSTOM FIELD
		DELETE FROM [dbo].[CustomFieldResult] 
			WHERE [VCC] = @VCC 
				AND [CustomFieldId] = @CustomFieldId

		-- ELIMINAMOS EL CUSTOM FIELD
		DELETE FROM [dbo].[CustomField]
			WHERE [VCC] = @VCC 
				AND [ID] = @CustomFieldId
		
		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteCustomFieldResults] (
	@VCC VARCHAR(50) = NULL, 
	@CustomFieldId VARCHAR(100) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- SP PARA ELIMINAR TODAS LAS CUSTOM_FIELD_RESULTS
	DELETE FROM [dbo].[CustomFieldResult] 
		WHERE [VCC] = @VCC 
			AND [CustomFieldId] = @CustomFieldId
		
	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE DeleteCustomFields 
	@VCC VARCHAR(50) = NULL, 				
	@CustomFieldIds VARCHAR(MAX) = NULL
AS
BEGIN
	SET NOCOUNT ON;

    DECLARE @_IDS TABLE(
	 ID VARCHAR(50)
	)

    INSERT INTO @_IDS SELECT [Value] FROM [dbo].[Split](@CustomFieldIds,',')

	-- ELIMINAMOS LOS RESULTS DEL CUSTOM FIELD
	DELETE FROM [dbo].[CustomFieldResult] 
		WHERE [VCC] = @VCC 
			AND [CustomFieldId] IN (SELECT [ID] FROM @_IDS)

	-- ELIMINAMOS EL CUSTOM FIELD
	DELETE FROM [dbo].[CustomField]
		WHERE [VCC] = @VCC 
			AND [ID]  IN (SELECT [ID] FROM @_IDS)
		
	-- RETORNAMOS RESULTADO 1
	SELECT 1 'result'
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteDictionaryWord] (
	@VCC VARCHAR(50) = NULL, 
	@DictionaryWordId VARCHAR(50) = NULL
) AS
BEGIN
	-- ELIMINAMOS LA PALABRA DE LA TABLA DICTIONARY WORD
	DELETE FROM [dbo].[DictionaryWord] WHERE [VCC] = @VCC AND [ID] = @DictionaryWordId

	-- ELIMINAMOS LAS PALABRAS SOUND-LIKE DE LA TABLA DICTIONARY WORD SOUND LIKE
	DELETE FROM [dbo].[DictionaryWordSoundLike] WHERE [VCC] = @VCC AND [DictionaryWordId] = @DictionaryWordId

	SELECT 1 'result'
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteDictionaryWordSoundLike] (
	@VCC VARCHAR(50) = NULL, 
	@DictionaryWordId VARCHAR(50) = NULL,
	@DictionaryWordSoundLikeId VARCHAR(50) = NULL
) AS
BEGIN
	-- ELIMINAMOS LAS PALABRAS SOUND-LIKE DE LA TABLA DICTIONARY WORD SOUND LIKE
	DELETE FROM [dbo].[DictionaryWordSoundLike] WHERE [VCC] = @VCC AND [DictionaryWordId] = @DictionaryWordId AND [ID] = @DictionaryWordSoundLikeId

	SELECT 1 'result'
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteDomain] (
	@VCC Varchar(50) = NULL, @Domain Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [Domain]
	WHERE [VCC] = @VCC
	AND [Domain] = @Domain

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteEntity] (
    @VCC VARCHAR(50) = NULL,
    @AppId VARCHAR(100) = NULL,
    @ID VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON

    -- ELIMINAMOS EL ENTITY DE LA TABLA
    DELETE FROM [dbo].[Entity]
    WHERE [VCC] = @VCC
      AND [AppId] = @AppId
      AND [ID] = @ID

    --Marco el NLU como pendiente de entrenamiento
    UPDATE
        [dbo].[NLU]
    SET
        [NeedTrain] = 1
    WHERE
            [VCC] = @VCC AND [AppId] = @AppId

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteEnvironment] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
			
	DELETE FROM [Environment]
	WHERE [VCC] = @VCC
	AND [Name] = @Name
    
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteEvaluationFormat] (
	@Instance Varchar(50) = NULL, 
	@ID Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION
			-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LA PLANTILLA
			DECLARE @TEMPLATE_ID VARCHAR(50) = NULL

			-- RECUPERAMOS EL TEMPLATE ASOCIADO AL FORMATO
			SELECT @TEMPLATE_ID = [templateId]
				FROM [dbo].[EvaluationFormat] (NOLOCK)
				WHERE [Instance] = @Instance
					AND [ID] = @ID
			
			-- ELIMINAMOS EL REGISTRO DEL FORMATO
			DELETE [dbo].[EvaluationFormat] 
				WHERE [Instance] = @Instance
					AND [ID] = @ID

			-- ELIMINAMOS EL CONTENIDO DEL FORMATO
			EXEC [dbo].[DeleteEvaluationFormatContent] @Instance, @ID, @TEMPLATE_ID

			SELECT 1 AS 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[DeleteEvaluationFormatContent] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationFormatId VARCHAR(50) = NULL,
	@TemplateId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION
			-- VERIFICAMOS SI TIENE CONTENIDO EL NOMBRE DE FORMATO
			IF @TemplateId IS NOT NULL AND @TemplateId <> ''
			BEGIN
				-- ELIMINAMOS EL TEMPLATE (SP DeleteTemplate)
				EXEC [dbo].[DeleteTemplate] @VCC, @TemplateId
			END

			-- ELIMINAMOS LAS CAMPAIGNS (REEMPLAZA SP DeleteCampainEvaluationFormat)
			DELETE [dbo].[CampaignEvaluationFormat]
				WHERE [Instance] = @VCC
					AND [IdEvaluationFormat] = @EvaluationFormatId
			
			-- ELIMINAMOS LOS ABILITYEVALUATIONFORMAT (REEMPLAZA SP DeleteAbilityEvaluationFormat)
			DELETE [AbilityEvaluationFormat] 
				WHERE [Instance] = @VCC
					AND [IdEvaluationFormat] = @EvaluationFormatId

			-- ELIMINAMOS LOS SCORETIPIFICATION
			DELETE [dbo].[EvaluationFormatAbilityScoreTipification]
				WHERE [VCC] = @VCC 
					AND [EvaluationFormatId] = @EvaluationFormatId

			-- ELIMINAMOS LOS SCORETIPIFICATIONITEM
			DELETE [dbo].[EvaluationFormatAbilityScoreTipificationItem]
				WHERE [VCC] = @VCC 
					AND [EvaluationFormatId] = @EvaluationFormatId

			-- ELIMINAMOS LOS SCORETIPIFICATIONQUERY
			DELETE [dbo].[EvaluationFormatAbilityScoreTipificationQuery]
				WHERE [VCC] = @VCC 
					AND [EvaluationFormatId] = @EvaluationFormatId

			-- ELIMINAMOS LOS CAMPOS PERSONALIZADOS DEL FORMATO (REEMPLAZA SP DeleteEvaluationFormatCustomFields)
			DELETE FROM [dbo].[EvaluationFormatCustomField] 
				WHERE [VCC] = @VCC 
					AND [EvaluationFormatId] = @EvaluationFormatId

			-- ELIMINAMOS LAS FIRMAS ADJUNTAS DEL FORMATO (REEMPLAZA SP DeleteEvaluationFormatAttachedSignatures)
			DELETE FROM [dbo].[EvaluationFormatAttachedSignature] 
				WHERE [VCC] = @VCC 
					AND [EvaluationFormatId] = @EvaluationFormatId

			-- AGREGAMOS UNA EXCEPCION EN CASO SEA FORMATO REALTIME
			DECLARE @IS_REALTIME_FORMAT BIT = 0
			SELECT @IS_REALTIME_FORMAT = 1
				FROM [dbo].[EvaluationFormat] (NOLOCK)
				WHERE [Instance] = @VCC 
					AND [ID] = @EvaluationFormatId
					AND [ExecutionType] = 'REALTIME'
			IF @IS_REALTIME_FORMAT = 1
				UPDATE [dbo].[RealtimeEvaluationPlan] 
					SET [IsActive] = 0,
						[LastModifiedDate] = GETUTCDATE(),
						[LastModifiedByUserId] = 'system',
						[TimeStamp] = GETUTCDATE()
					WHERE [VCC] = @VCC 
						AND [EvaluationFormatId] = @EvaluationFormatId
    
			SELECT 1 AS 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
    

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- PARA ELIMINAR UN PLAN DE EVALUACION SE SISTEMA, DEBEMOS ELIMINARLO DE LAS TABLAS EVALUATIONRESULT, 
--  AUDITOREVALUATIONPLAN, INTERACTIONSEVALUATIONPLAN Y EVALUATIONPLAN EN ESE ORDEN
CREATE PROCEDURE [dbo].[DeleteEvaluationPlan] (
	@Instance Varchar(50) = NULL, 
	@IdEvaluationPlan Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	-- DECLARAMOS UNA VARIABLE PARA DETERMINAR SI EXISTE EL PLAN DADO O NO
	DECLARE @EVALUATIONPLAN_EXISTS BIT = 0

	-- CONTAMOS CUANTOS PLANES DE EVALUACION EXISTEN CON EL ID Y VCC DADOS
	SELECT @EVALUATIONPLAN_EXISTS = COUNT(EP.[ID]) 
		FROM EvaluationPlan EP 
		WHERE EP.[Instance] = @Instance
			AND EP.[ID] = @IdEvaluationPlan
	
	-- SI EXISTE UN PLAN DE EVALUACION CON LOS PARAMETROS RECIBIDOS, ELIMINAMOS
	--  LOS REGISTROS CORRESPONDIENTES QUE SE ENCUENTRAN EN OTRAS TABLAS
	IF (@EVALUATIONPLAN_EXISTS = 1) 
	BEGIN
		-- ELIMANOS LOS REGISTROS DE RESULTADOS DE LAS EVALUACIONES DEL PLAN
		DELETE FROM [dbo].[EvaluationResult] 
			WHERE [Instance] = @Instance
				AND [IdEvaluationPlan] = @IdEvaluationPlan

		-- ELIMINAMOS LOS REGISTROS DE LOS RESULTADOS DE CAMPOS PERSONALIZADOS
		DELETE FROM [dbo].[EvaluationCustomFieldResult]
			WHERE [VCC] = @Instance
				AND [EvaluationPlanId] = @IdEvaluationPlan

		-- ELIMINAMOS LOS REGISTROS DE EVALUACION DEL PLAN
		DELETE FROM [dbo].[AuditorEvaluationPlan]
			WHERE [Instance] = @Instance
				AND [IdEvaluationPlan] = @IdEvaluationPlan

		-- ELIMINAMOS LOS REGISTROS DE INTERACCIONES RELACIONADAS AL PLAN
		DELETE FROM [dbo].[InteractionEvaluationPlan]
			WHERE [Instance] = @Instance
				AND [IdEvaluationPlan] = @IdEvaluationPlan

		-- ELIMINAR DATA HISTORICA
		EXEC [dbo].[CleanFailedEvaluationPlanData] @Instance, @IdEvaluationPlan

		-- ELIMINAMOS EL REGISTROS DEL PLAN DE EVALUACION
		DELETE [dbo].[EvaluationPlan]
			WHERE [Instance] = @Instance
				AND [ID] = @IdEvaluationPlan
		
		SELECT 1 AS 'result'

	END 
	ELSE
	BEGIN
		SELECT 0 AS 'result'
	END
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteEvaluationPlanInteractionByTypeAndId] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionType VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR UNA INTERACCION DE I6
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- ELIMINAMOS EL REGISTRO DE LA TABLA [dbo].[InteractionEvaluationPlan]
		DELETE FROM [dbo].[InteractionEvaluationPlan] 
			WHERE [Instance] = @VCC
				AND [IdEvaluationPlan] = @EvaluationPlanId
				AND [InteractionType] = @InteractionType
				AND [IdInteraction] = @InteractionId

		-- ELIMINAMOS EL REGISTRO DE LA TABLA [dbo].[AuditorEvaluationPlan]
		DELETE FROM [dbo].[AuditorEvaluationPlan] 
			WHERE [Instance] = @VCC
				AND [IdEvaluationPlan] = @EvaluationPlanId
				AND [InteractionType] = @InteractionType
				AND [IdInteraction] = @InteractionId

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteEvaluationResultDraftBackup] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@AuditorId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	-- EL PRESENTE SP TIENE LA FINALIDAD DE ELIMINAR LOS REGISTROS DE RESPALDO PARA BORRADOR

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- DECLARAMOS UNA VARIABLE PARA VERIFICAR EL TIPO DEL PLAN
		DECLARE @IS_MIXED_EVALUATION_PLAN BIT = 0

		-- VERIFICAMOS QUE EL PLAN DE EVALUACION ES DEL TIPO MIXTO
		SELECT @IS_MIXED_EVALUATION_PLAN = IIF (EP.[Evaluation_Mode] = 'X', 1, 0) 
			FROM [dbo].[EvaluationPlan] EP (NOLOCK) 
			WHERE EP.[Instance] = @VCC
				AND EP.[ID] = @EvaluationPlanId

		-- DECLARAMOS UNA VARIABLE PARA VERIFICAR SI LA AUDITORIA YA ESTA GUARDADA COMO BORRADOR
		DECLARE @EXISTS_DRAFT_DATA BIT = 0

		IF EXISTS (
			SELECT 1 
				FROM [dbo].[EvaluationResult] (NOLOCK)
				WHERE [Instance] = @VCC
					AND [IdEvaluationPlan] = @EvaluationPlanId
					AND [IdInteraction] = @InteractionId
					AND [IdEvaluator] = 'SPEECH'
					AND [IdAuditor] = @AuditorId + '-DRAFT'
		)
			SET @EXISTS_DRAFT_DATA = 1

		IF @IS_MIXED_EVALUATION_PLAN = 1 AND @EXISTS_DRAFT_DATA = 1
		BEGIN
			-- RESTAURAMOS LOS RESULTADOS DE LA PARTE AUTOMATICA
			DELETE [dbo].[EvaluationResult]
				WHERE [Instance] = @VCC
					AND [IdEvaluationPlan] = @EvaluationPlanId
					AND [IdInteraction] = @InteractionId
					AND [IdEvaluator] = 'SPEECH'
					AND [IdAuditor] = @AuditorId + '-DRAFT'
		END 

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteEvaluationResults] (
	@Instance Varchar(50) = NULL,
	@IdInteraction Varchar(250) = NULL,
	@InteractionType varchar(50) = NULL,
	@IdEvaluationPlan Varchar(50) = NULL,
	@IdAgent Varchar(50) = NULL,
	@IdAuditor Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	DELETE FROM
		[dbo].[EvaluationResult]
	WHERE
		[Instance] = @Instance
		AND [IdInteraction] = @IdInteraction
		AND [InteractionType] = @InteractionType
		AND [IdEvaluationPlan] = @IdEvaluationPlan
		AND [IdAgent] = @IdAgent
		AND [IdAuditor] = @IdAuditor

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteException] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	DELETE [Exception] 
	WHERE [VCC] = @VCC
    AND [ID] = @ID

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteGenerative] (
	@VCC VARCHAR(50) = NULL, 
	@Id VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- ELIMINAMOS LOS EJEMPLOS
	DELETE FROM [dbo].[GenerativeExample]
		WHERE [VCC] = @VCC
			AND [AppID] = @Id

	-- ELIMINAMOS LOS INTENTS
	DELETE FROM [dbo].[GenerativeIntent]
		WHERE [VCC] = @VCC
			AND [AppId] = @Id

	-- ELIMINAMOS EL MODELO DE LA TABLA NLU
	DELETE FROM [dbo].[Generative]
		WHERE [VCC] = @VCC
			AND [AppId] = @Id

	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteGenerativeExample] (
    @VCC VARCHAR(50) = NULL,
    @AppId VARCHAR(100) = NULL,
	@IntentId VARCHAR(50) = NULL,
    @Id VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON

    -- ELIMINAMOS LOS EXAMPLES REGISTRADOS CON EL APPID E INTENTID EN LA TABLA [dbo].[GenerativeExample]
    DELETE FROM [dbo].[GenerativeExample]
		WHERE [VCC] = @VCC
		  AND [AppId] = @AppId
		  AND [IntentId] = @IntentId
		  AND [Id] = @Id

    -- MARCAMOS EL MODELO CON ESTADO NEED_PUBLISH
    UPDATE [dbo].[Generative]
		SET [StatusCode] = 'NEED_PUBLISH',
			[NeedPublish] = 1
		WHERE [VCC] = @VCC 
			AND [AppId] = @AppId
			AND [StatusCode] <> 'NEED_PUBLISH'

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteGenerativeIntent] (
    @VCC VARCHAR(50) = NULL,
    @AppId VARCHAR(100) = NULL,
    @Id VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON

    -- ELIMINAMOS EL INTENT DE LA TABLA [dbo].[GenerativeIntent]
    DELETE FROM [dbo].[GenerativeIntent]
		WHERE [VCC] = @VCC
		  AND [AppId] = @AppId
		  AND [Id] = @Id

    -- ELIMINAMOS LOS EXAMPLES REGISTRADOS CON EL APPID E INTENTID EN LA TABLA [dbo].[GenerativeExample]
    DELETE FROM [dbo].[GenerativeExample]
		WHERE [VCC] = @VCC
		  AND [AppId] = @AppId
		  AND [IntentId] = @Id

    -- MARCAMOS EL MODELO CON ESTADO NEED_PUBLISH
    UPDATE [dbo].[Generative]
		SET [StatusCode] = 'NEED_PUBLISH',
			[NeedPublish] = 1
		WHERE [VCC] = @VCC 
			AND [AppId] = @AppId
			AND [StatusCode] <> 'NEED_PUBLISH'

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

-- [dbo].[DeleteGenericQuery] 'inconcert', 'Query1'
CREATE PROCEDURE [dbo].[DeleteGenericQuery]
(
	@VCC				VARCHAR(50),
	@Name				VARCHAR(50)
)
AS
BEGIN

	SET NOCOUNT ON

	DELETE FROM [dbo].[GenericQuery]
	WHERE [VCC] = @VCC AND [Name] = @Name

	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteGenericTemplate] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [GenericTemplateQuery]
	WHERE [VCC] = @VCC
	AND [TemplateId] = @ID
	
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[DeleteGroup] (
	@Instance Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [Group]
	WHERE [Instance] = @Instance
	AND [ID] = @ID

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[DeleteGroups] 
	@Instance VARCHAR(50) = NULL,
	@GroupIDs VARCHAR(MAX) = NULL
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

    DECLARE @_IDS TABLE(
	 ID VARCHAR(50)
	)

    INSERT INTO @_IDS SELECT [Value] FROM [dbo].[Split](@GroupIDs,',')

	DELETE FROM [Group]
		WHERE [Instance] = @Instance
		AND [ID] IN (SELECT [ID] FROM @_IDS)   
	SELECT 1 AS 'result'	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteHistoricEvaluationPlanData] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50)=NULL
) AS
BEGIN
	SET NOCOUNT ON

	--
	DELETE FROM [dbo].[HistoricEvaluationFormat] WHERE [EvaluationPlanId] = @EvaluationPlanId
	DELETE FROM [dbo].[HistoricAbilityEvaluationFormat]
	DELETE FROM [dbo].[HistoricEvaluationFormatCustomField]
	DELETE FROM [dbo].[HistoricCustomField]
	DELETE FROM [dbo].[HistoricCustomFieldResult]
	DELETE FROM [dbo].[HistoricEvaluationFormatAttachedSignature]
	DELETE FROM [dbo].[HistoricCampaignEvaluationFormat]
	DELETE FROM [dbo].[HistoricGroup]
	DELETE FROM [dbo].[HistoricSubGroup]
	DELETE FROM [dbo].[HistoricAbility]
	DELETE FROM [dbo].[HistoricScore]
	DELETE FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipification]
	DELETE FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipificationQuery]
	DELETE FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipificationItem]
	DELETE FROM [dbo].[HistoricAbilityQuery]
	DELETE FROM [dbo].[HistoricAbilityQueryEntity]

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteI6InteractionAudio] (
	@VCC					VARCHAR(50) = NULL,
	@InteractionId			VARCHAR(250) = NULL,
	@Section				INT = NULL,
	@Part					INT = NULL
) AS 
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR UNA INTERACCION DE I6
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION
		
		-- ACTUALIZAMOS EL SECTION Y PART EN LA INTERACCION EN LA TABLA AUDIO
		DELETE FROM [dbo].[Audio] 
			WHERE [VCC] = @VCC
				AND [ID] = @InteractionId
				AND [Section] = @Section
				AND [Part] = @Part

		-- ACTUALIZAMOS EL SECTION Y PART EN LA INTERACCION EN LA TABLA PROCESSAUDIO
		DELETE FROM [dbo].[ProcessAudio]
			WHERE [VCC] = @VCC
				AND [AudioId] = @InteractionId
				AND [Section] = @Section
				AND [Part] = @Part 

		-- ACTUALIZAMOS EL SECTION Y PART EN LA INTERACCION EN LA TABLA INTERACTIONEVALUATIONPLAN
		DELETE FROM [dbo].[InteractionEvaluationPlan] 
			WHERE [Instance] = @VCC
				AND [IdInteraction] = @InteractionId
				AND [InteractionSection] = @Section
				AND [InteractionPart] = @Part 

		/*
		-- ELIMINAMOS LA INTERACCION DE LA TABLA AUSITOREVALUATIONPLAN
		DELETE FROM [dbo].[AuditorEvaluationPlan]
			WHERE [Instance] = @VCC
				AND [IdInteraction] = @InteractionId
		*/

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteIntent] (
    @VCC Varchar(50) = NULL,
    @AppId VARCHAR(100) = NULL,
    @IntentId Varchar(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON

    -- ELIMINAMOS EL INTENT DE LA TABLA INTENT
    DELETE FROM [Intent]
    WHERE [VCC] = @VCC
      AND [AppId] = @AppId
      AND [IntentId] = @IntentId

    -- ELIMINAMOS LOS UTTERANCES REGISTRADOS CON EL APPID E INTENTID
    DELETE FROM [Utterance]
    WHERE [VCC] = @VCC
      AND [AppId] = @AppId
      AND [IntentId] = @IntentId
    --ELIMINAR UNDERLINED
    DELETE UL FROM [dbo].[Intent] I
                       JOIN [dbo].[Utterance] U
                            ON I.[VCC] = U.[VCC]
                                AND I.[AppId] = U.[AppId]
                       JOIN [dbo].[Underlined] UL
                            ON UL.[VCC] = U.[VCC]
                                AND UL.[UtteranceId]= U.[Id]
    WHERE I.[IntentId] = @IntentId AND I.[AppId] = @AppId AND I.[VCC] = @VCC
    --ELIMINAR UtteranceEntity
    DELETE UE
    FROM [dbo].[Intent] I
             JOIN [dbo].[Entity] E
                  ON I.[VCC] = E.[VCC]
                      AND I.[AppId] = E.[AppId]
             JOIN [dbo].[Utterance] U
                  ON I.[VCC] = U.[VCC]
                      AND I.[AppId] = U.[AppId]
             JOIN [dbo].[UtteranceEntity] UE
                  ON UE.[VCC] = U.[VCC]
                      AND UE.[UtteranceId]= U.[Id]
                      AND UE.[EntityName] = E.[Name]
    WHERE I.[IntentId] = @IntentId AND I.[AppId] = @AppId AND I.[VCC] = @VCC

    --Marco el NLU como pendiente de entrenamiento
    UPDATE
        [dbo].[NLU]
    SET
        [NeedTrain] = 1
    WHERE
            [VCC] = @VCC AND [AppId] = @AppId

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteInteraction] (
	@VCC     VARCHAR(50)  = NULL,
	@ID      VARCHAR(250) = NULL,
	@IsAudio BIT          = 0
)
AS
BEGIN
	SET NOCOUNT ON
	
	IF(@IsAudio = 1) --ELIMINAR AUDIOS
		BEGIN
			 DELETE FROM [dbo].[Audio] WHERE [VCC] = @VCC AND [ID] = @ID
		END
    ELSE --ELIMINAR INTERACCIONES ASINCRONAS
		BEGIN
			DELETE FROM [dbo].[AsyncInteraction] WHERE [VCC] = @VCC AND [ID] = @ID
		END

    SELECT 1 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[DeleteMultipleUtterances] (
	@VCC		VARCHAR(50) = NULL,
	@Ids		VARCHAR(MAX) = NULL,
	@DeleteByIntent BIT = 1
)
AS
BEGIN
	SET NOCOUNT ON
	DECLARE @_IDS TABLE(
	 ID VARCHAR(50)
	)
	DECLARE @UTTERANCES TABLE(
	ID VARCHAR(50)
	)
	DECLARE @APPIDS TABLE(
	 ID VARCHAR(50)
	)
	
	INSERT INTO @_IDS SELECT [Value] FROM [dbo].[Split](@Ids,',')
	
	IF(@DeleteByIntent = 1)
	BEGIN 
		INSERT INTO @UTTERANCES 
		SELECT 
		U.[Id] 
		FROM [dbo].[Utterance] U (NOLOCK)
		JOIN [dbo].[Underlined] UN (NOLOCK)
		ON U.[VCC] = UN.[VCC] AND U.[Id] = UN.[UtteranceId]
		WHERE U.[VCC] = @VCC AND U.[IntentId] IN(SELECT [ID] FROM @_IDS)
	
	   INSERT INTO @APPIDS 
	   SELECT [AppId] 
	   FROM [dbo].[Intent] 
	   WHERE [VCC] = @VCC 
	   AND [IntentId] IN(SELECT [ID] FROM @_IDS)
	   
	END
	ELSE
	BEGIN 
	
		INSERT INTO @UTTERANCES SELECT [ID] FROM @_IDS
		
		INSERT INTO @APPIDS 
		   SELECT [AppId] 
		   FROM [dbo].[Utterance] 
		   WHERE [VCC] = @VCC 
		   AND [Id] IN(SELECT [ID] FROM @_IDS)
	
	END

	
	
	
	
	DELETE [dbo].[Utterance] WHERE [VCC] = @VCC AND [Id] IN(SELECT [ID] FROM @UTTERANCES)
	
	DELETE [dbo].[UtteranceEntity] WHERE [VCC] = @VCC AND [UtteranceId] IN(SELECT [ID] FROM @UTTERANCES)
			
	DELETE FROM [dbo].[Underlined] WHERE [VCC] = @VCC AND  [UtteranceId] IN(SELECT [ID] FROM @UTTERANCES)

    UPDATE [dbo].[NLU] SET 
    [IsPublished] = 0, 
    [LastPublishedDate] = NULL, 
    [IsTrained] = 0, 
    [LastTrainedDate] = NULL
    WHERE [AppId] IN(SELECT [ID] FROM @APPIDS)
    
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteNeuroLinguisticProgramming] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL,	@ProcessId Varchar(50) = NULL,		@Language Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [Word]
	WHERE [VCC] = @VCC
	AND [NeuroLinguisticProgrammingId] = @Name
	AND [ProcessId] = @ProcessId
	AND [Language] = @Language
		
	DELETE FROM [NeuroLinguisticProgramming]
	WHERE [VCC] = @VCC
	AND [Name] = @Name
    
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


CREATE PROCEDURE [dbo].[DeleteNLU] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL ID DEL NLU
	DECLARE @APP_ID VARCHAR(100) = NULL
	
	-- ASIGNAMOS EL APPID A LA VARIABLE
	SELECT @APP_ID = N.[AppId]
		FROM [dbo].[NLU] N (NOLOCK)
		WHERE N.[VCC] = @VCC 
			AND N.[Name] = @Name

	-- ELIMINAMOS LOS ENTITY DE LA TABLA UTTERANCE-ENTITY
	DELETE UE 
		FROM [dbo].[NLU] N
		INNER JOIN [dbo].[Entity] E 
			ON E.[VCC] = N.[VCC]
				AND E.[AppId] = N.[AppId]
		INNER JOIN [dbo].[Utterance] U 
			ON N.[VCC] = U.[VCC]
				AND U.[AppId] = N.[AppId] 
		INNER JOIN [dbo].[UtteranceEntity] UE 
			ON UE.[VCC] = U.[VCC]
				AND UE.[UtteranceId]= U.[Id]
				AND UE.[EntityName] = E.[Name]
		WHERE N.[AppId] = @APP_ID

    --ELIMINAMOS LOS UNDERLINEDS DE LA TABLA UNDERLINED
    
    	DELETE UL 
		FROM [dbo].[NLU] N
		INNER JOIN [dbo].[Intent] I 
			ON I.[VCC] = N.[VCC]
				AND I.[AppId] = N.[AppId]
		INNER JOIN [dbo].[Utterance] U 
			ON N.[VCC] = U.[VCC]
				AND U.[AppId] = N.[AppId] 
		INNER JOIN [dbo].[Underlined] UL 
			ON UL.[VCC] = U.[VCC]
				AND UL.[UtteranceId]= U.[Id]
		WHERE N.[AppId] = @APP_ID
		
	-- ELIMINAMOS LOS UTTERANCES LA TABLA UTTERANCE
	DELETE FROM [dbo].[Utterance]
		WHERE [VCC] = @VCC
			AND [AppID] = @APP_ID

	-- ELIMINAMOS LOS INTENTS DE LA TABLA INTENT
	DELETE FROM [dbo].[Intent]
		WHERE [VCC] = @VCC
			AND [AppId] = @APP_ID

	-- ELIMINAMOS LOS ENTITY DE LA TABLA ENTITY
	DELETE FROM [dbo].[Entity]
		WHERE [VCC] = @VCC
			AND [AppId] = @APP_ID

	-- ELIMINAMOS EL MODELO DE LA TABLA NLU
	DELETE FROM [dbo].[NLU]
		WHERE [VCC] = @VCC
			AND [AppId] = @APP_ID

	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteProcess] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

			-- VERIFICAMOS SI EXISTE EL PROCESS A ELIMINAR
			IF EXISTS(
				SELECT [Name] 
					FROM [dbo].[Process] (NOLOCK)
					WHERE [VCC] = @VCC 
					AND [Name] = @Name
			)
			BEGIN
				-- ELIMINAMOS DE PROCESSSUMMARY
				DELETE FROM [dbo].[ProcessSummary]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS DE PROCESSAUDIO
				DELETE FROM [dbo].[ProcessAudio]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS DE PROCEESSFILTER
				DELETE FROM [dbo].[ProcessFilter]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS DE PROCESSAUDIOSUMMARY
				DELETE FROM [dbo].[ProcessAudioSummary]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS DE TRANSCRIPTION
				DELETE FROM [dbo].[Transcription]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS DE PROGRESS
				DELETE FROM [dbo].[Progress]
					WHERE [VCC] = @VCC
					AND [ProcessId] = @Name

				-- ELIMINAMOS LA DATA POSIBLE DATA DE UNA PLANTILLA AGENDADA (STUDYRESULTSUMMARYDETAIL)
				DELETE FROM [dbo].[StudyResultSummaryDetail] 
					WHERE [VCC] = @VCC 
						AND [StudyId] IN (
							SELECT S.[Name] 
								FROM [Study] S (NOLOCK) 
								WHERE S.[ProcessId] = @Name
									AND S.[Type] = 'TEMPLATE'
						)

				-- ELIMINAMOS LA DATA POSIBLE DATA DE UNA PLANTILLA AGENDADA (STUDYRESULTSUMMARY)
				DELETE FROM [dbo].[StudyResultSummary]
					WHERE [VCC] = @VCC 
						AND [StudyId] IN (
							SELECT S.[Name] 
								FROM [Study] S (NOLOCK) 
								WHERE S.[ProcessId] = @Name
									AND S.[Type] = 'TEMPLATE'
						)

				-- ELIMINAMOS LA DATA POSIBLE DATA DE UNA PLANTILLA AGENDADA (STUDY)
				DELETE FROM [Study]
					WHERE [ProcessId] = @Name
						AND [Type] = 'TEMPLATE'

				-- ACTUALIZAMOS LOS REGISTROS DE AGENDA
				UPDATE [dbo].[BatchScheduleExecution]
					SET [Status] = 'DISCARDED_BY_DELETED_PROCESS',
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [VCC] = @VCC 
						AND [ProcessId] = @NAME

				-- FINALIZAMOS ELIMINANDO DE PROCSS
				DELETE FROM [dbo].[Process]
					WHERE [VCC] = @VCC
					AND [Name] = @Name

				SELECT 1 AS 'result'
			END
			ELSE
			BEGIN
				SELECT 0 AS 'result'
			END

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteProcessAsync] (
	@VCC        VARCHAR(50)  = NULL,
	@ProcessId  VARCHAR(50)  = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [dbo].[ProcessAsyncInteraction] WHERE [ProcessId] = @ProcessId

    SELECT 1 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
/* Procedure para eliminar los processAudio relacionados a un process */
CREATE PROCEDURE [dbo].[DeleteProcessAudioByProcess] (
    @VCC Varchar(50) = NULL,
	@ProcessId Varchar(50) = NULL
)
AS
BEGIN
SET NOCOUNT ON
	DELETE 
		FROM [dbo].[ProcessAudio]
		WHERE [VCC] = @VCC
			AND [ProcessId] = @ProcessId;

SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

-- [dbo].[DeleteProcessAudioByProcessIdInteractionId] 'inconcert', 'test201905274', '{0eba8530-1dd2-11b2-96fe-96884e041c49}, {7a7d5b1e-1dd2-11b2-b539-cc5557f92e52}', 1
CREATE PROCEDURE [dbo].[DeleteProcessAudioByProcessIdInteractionId]
(
	@VCC					VARCHAR(50),
	@ProcessId				VARCHAR(50),
	@ID						VARCHAR(MAX),
	@IsUpdate				VARCHAR(1)
)
AS
BEGIN
	SET NOCOUNT ON
	DECLARE @SQL VARCHAR(MAX)
	SET @SQL = '
		DELETE FROM [dbo].[ProcessAudio]
		WHERE [VCC] = ''' + @VCC + '''
		AND [ProcessId] = ''' + @ProcessId + '''
	'
	IF(@IsUpdate = 0)
	BEGIN
		-- Se utiliza para la eliminar interacciones donde el ID tenga este formato {guid}_{section}_{part}
		SET @SQL += '
			AND CONCAT([AudioId], ''_'', [Section], ''_'', [Part]) NOT IN(SELECT value FROM dbo.SplitString(''' + @ID + ''', '',''))
		'
	END
	ELSE
	BEGIN
		-- Se utiliza para la eliminar interacciones donde el ID tenga este formato {guid}. Utilizado para UpdateLinguisticProcess.
		SET @SQL += '
			AND [AudioId] NOT IN(SELECT value FROM dbo.SplitString(''' + @ID + ''', '',''))
		'
	END
	-- PRINT @SQL
	EXEC(@SQL)

	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [DeleteProcessFilter] 'inconcert', 'test'
CREATE PROCEDURE [dbo].[DeleteProcessFilter] (
	@VCC Varchar(50) = NULL, @ProcessId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [ProcessFilter]
	WHERE [VCC] = @VCC
	AND [ProcessId] = @ProcessId

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteReport] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	DELETE FROM [Report]
	WHERE [VCC] = @VCC
	AND [ID] = @ID
    
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteRole] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	DELETE FROM [RolePermission]
	WHERE [VCC] = @VCC
	AND [RoleId] = @ID
	
	DELETE FROM [Role]
	WHERE [VCC] = @VCC
	AND [ID] = @ID

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteRolePermission] (
	@VCC Varchar(50) = NULL, @RoleId Varchar(50)
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [RolePermission]
	WHERE [VCC] = @VCC
	AND [RoleId] = @RoleId
    
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteScore] (
	@Instance Varchar(50) = NULL, @IdAbility Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	DELETE [Score] 
	WHERE [Instance] = @Instance
    AND [IdAbility] = @IdAbility

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteScoreTipification] (
	@VCC VARCHAR(50) = NULL, 
	@ScoreId VARCHAR(50) = NULL, 
	@ID VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

			-- ELIMINAMOS LA TIPIFICACION
			DELETE [ScoreTipification] 
				WHERE [VCC] = @VCC
					AND [ScoreId] = @ScoreId
					AND [ID] = @ID
					
			-- ELIMINAMOS LOS ITEMS DE LA TIPIFICACION
			DELETE [ScoreTipificationItem] 
				WHERE [VCC] = @VCC 
					AND [ScoreId] = @ScoreId
					AND [TipificationId] = @ID
    
			SELECT 1 AS 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteScoreTipificationItem] (
	@VCC VARCHAR(50) = NULL, 
	@ScoreId VARCHAR(50) = NULL, 
	@ID VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	DELETE [ScoreTipificationItem] 
		WHERE [VCC] = @VCC 
			AND [ScoreId] = @ScoreId
			AND [ID] = @ID
    
	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteScoreTipificationQuery] (
	@VCC VARCHAR(50) = NULL, 
	@ScoreId VARCHAR(50) = NULL, 
	@TipificationId VARCHAR(50) = NULL
) AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON;

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- SCRIPT PARA ELIMINAR EL REGISTRO DE TIPIFICATION_QUERY
		DELETE FROM [dbo].[ScoreTipificationQuery] 
			WHERE [VCC] = @VCC 
				AND [ScoreId] = @ScoreId
				AND [TipificationId] = @TipificationId
		
		SELECT 1 AS 'result'
    
		SET NOCOUNT OFF

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	SET NOCOUNT OFF

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteSinglePermission] (
	@VCC Varchar(50) = NULL, @Type Varchar(50) = NULL, @Element Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [RolePermission]
    WHERE [VCC] = @VCC
    AND [GroupId] = 'SINGLE_PERM_' + @Type + '_' + @Element
	
	DELETE FROM [RolePermissionsGroup]
    WHERE [VCC] = @VCC
    AND [GroupId] = 'SINGLE_PERM_' + @Type + '_' + @Element
    AND [Category] = 'CATEG_GROUP_' + @Type

	DELETE FROM [SinglePermission]
    WHERE [VCC] = @VCC
    AND [GroupId] = 'SINGLE_PERM_' + @Type + '_' + @Element
    AND [Type] = @Type
    AND [Element] = @Element

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteStudy] (
	@VCC Varchar(50) = NULL, @StudyId Varchar(200) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	DELETE FROM [Study]
	WHERE [VCC] = @VCC
	AND [Name] = @StudyId
	
	DELETE FROM [StudyDetail]
	WHERE [VCC] = @VCC
	AND [StudyId] = @StudyId

	DELETE FROM [StudyResultSummary]
	WHERE [VCC] = @VCC
	AND [StudyId] = @StudyId

	DELETE FROM [StudyResultSummaryDetail]
	WHERE [VCC] = @VCC
	AND [StudyId] = @StudyId
	
	--ELIMINAR DATA HISTORICA
	DELETE FROM [dbo].[HistoricTemplate] WHERE [VCC] = @VCC AND [StudyId] = @StudyId
	DELETE FROM [dbo].[HistoricTemplateTheme] WHERE [VCC] = @VCC AND [StudyId] = @StudyId
	DELETE FROM [dbo].[HistoricTemplateThemeItem] WHERE [VCC] = @VCC AND [StudyId] = @StudyId
	DELETE FROM [dbo].[HistoricTemplateThemeItemEntity] WHERE [VCC] = @VCC AND [StudyId] = @StudyId
	DELETE FROM [dbo].[HistoricGenericTemplateQuery] WHERE [VCC] = @VCC AND [StudyId] = @StudyId

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[DeleteSubGroup] (
	@Instance Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [SubGroup]
	WHERE [Instance] = @Instance
	AND [ID] = @ID

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[DeleteSubGroups] 
	@Instance VARCHAR(50) = NULL,
	@SubGroupIDs VARCHAR(MAX) = NULL
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

    DECLARE @_IDS TABLE(
	 ID VARCHAR(50)
	)

    INSERT INTO @_IDS SELECT [Value] FROM [dbo].[Split](@SubGroupIDs,',')

	DELETE FROM [SubGroup]
		WHERE [Instance] = @Instance
		AND [ID] IN (SELECT [ID] FROM @_IDS)
	SELECT 1 AS 'result'	           
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

-- [dbo].[DeleteTag] 'default', '{00c7211a-973a-11e8-87f3-be2929e88805}', 0, 0, 'tag'
CREATE PROCEDURE [dbo].[DeleteTag]
(
	@VCC						VARCHAR(50),
	@AudioId					VARCHAR(50),
	@Section					NUMERIC(5, 0),
	@Part						NUMERIC(5, 0),
	@Tag						VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT OFF

	DELETE FROM [dbo].[Tag]
	WHERE [VCC] = @VCC
	AND [AudioId] = @AudioId
	AND [Section] = @Section
	AND [Part] = @Part
	AND [Tag] = @Tag

    SELECT 1 AS 'result'

	SET NOCOUNT ON
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteTemplate] (
	@VCC Varchar(50) = NULL, 
	@Name Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION
			DELETE FROM [Template]
				WHERE [VCC] = @VCC
					AND [Name] = @Name
	
			DELETE FROM [TemplateTheme]
				WHERE [VCC] = @VCC
					AND [TemplateId] = @Name

			DELETE FROM [TemplateThemeItem]
				WHERE [VCC] = @VCC
					AND [TemplateId] = @Name
	
			SELECT 1 AS 'result'
		
		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteTemplateTheme] (
	@VCC Varchar(50) = NULL, @Name Varchar(200) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [TemplateTheme]
		WHERE [VCC] = @VCC
			AND [TemplateId] = @Name

	DELETE FROM [TemplateThemeItem]
		WHERE [VCC] = @VCC
			AND [TemplateId] = @Name
	
    DELETE FROM [TemplateThemeItemEntity]
		WHERE [VCC] = @VCC
			AND [TemplateId] = @Name
	
	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteTipification] (
	@VCC VARCHAR(50) = NULL, 
	@ID VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

			-- ELIMINAMOS LA TIPIFICACION
			DELETE [Tipification] 
				WHERE [VCC] = @VCC
					AND [ID] = @ID

			-- ELIMINAMOS LA QUERY TIPIFICACION
			DELETE [TipificationQuery] 
				WHERE [VCC] = @VCC
					AND [TipificationId] = @ID

			-- ELIMINAMOS LOS ITEMS DE LA TIPIFICACION
			DELETE [TipificationItem] 
				WHERE [VCC] = @VCC 
					AND [TipificationId] = @ID
    
			SELECT 1 AS 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteTipificationItem] (
	@VCC VARCHAR(50) = NULL, 
	@ID VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	DELETE [TipificationItem] 
		WHERE [VCC] = @VCC 
			AND [ID] = @ID
    
	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteTipificationQuery] (
	@VCC VARCHAR(50) = NULL, 
	@TipificationId VARCHAR(50) = NULL
) AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON;

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- SCRIPT PARA ELIMINAR EL REGISTRO DE TIPIFICATION_QUERY
		DELETE FROM [dbo].[TipificationQuery] 
			WHERE [VCC] = @VCC 
				AND [TipificationId] = @TipificationId
		
		SELECT 1 AS 'result'
    
		SET NOCOUNT OFF

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	SET NOCOUNT OFF

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteUser] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	DELETE FROM [UserClient]
	WHERE [VCC] = @VCC
	AND [UserId] = @ID
	
	DELETE FROM [User]
	WHERE [VCC] = @VCC
	AND [ID] = @ID
    
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteUtterance] (
	@VCC			VARCHAR(50) = NULL,
	@AppId			VARCHAR(50) = NULL,
	@IntentId		VARCHAR(50) = NULL,
	@Id				VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
    -- DECLARE @ENGINE VARCHAR(15)  = (SELECT [CognitiveEngine] FROM [dbo].[VCC] (NOLOCK) WHERE [VCC] = @VCC)
	-- DECLARE @ID     VARCHAR(50) = @UtteranceId
	-- DECLARE @TempId VARCHAR(50) = NULL
	DECLARE @CognitiveEngine  varchar(50)
	SELECT @CognitiveEngine = CognitiveEngine FROM VCC WHERE VCC = @VCC
	IF @CognitiveEngine = 'LUIS' OR @CognitiveEngine ='RASA'
	SELECT @IntentId = IntentId from [dbo].[Intent] WHERE [Name] = @IntentId and [VCC] = @VCC and [AppId] = @AppId

	DELETE [Utterance] 
		WHERE [VCC] = @VCC
			AND [AppId] = @AppId
			AND [IntentId] = @IntentId
			AND [Id] = @Id
	
	DELETE [UtteranceEntity] 
		WHERE [VCC] = @VCC
			AND [UtteranceId] = @Id
			
			
	DELETE FROM [dbo].[Underlined] WHERE [VCC] = @VCC AND [UtteranceId] = @Id

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteUtteranceEntity] (
	@VCC				VARCHAR(50) = NULL,
    @UtteranceId		VARCHAR(50) = NULL,
    @EntityName			VARCHAR(50) = NULL,
    @Start				INT = NULL,
    @End				INT = NULL,
    @EndTokenIndex		INT = NULL,
    @StartTokenIndex	INT = NULL
)
AS
BEGIN
	SET NOCOUNT ON
    
	-- ELIMINAMOS EL UTTERANCE ENTITY QUE COINCIDE CON LOS PARAMETROS INGRESADOS
	DELETE FROM [dbo].[UtteranceEntity] 
		WHERE [VCC] = @VCC
			AND [UtteranceId] = @UtteranceId 
			AND [EntityName] = @EntityName 
			AND [Start] = @Start
			AND [End] = @End
	
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteVCC] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [VCC]
	WHERE [VCC] = @VCC
	AND [ID] = @ID

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[DeleteWorkGroupSupervisor] (
	@VCC Varchar(50) = NULL,
	@Description Varchar(50) = NULL,
	@SupervisorId Varchar(250) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	DELETE FROM [WorkGroupSupervisor]
	WHERE [VCC] = @VCC
	AND [Description] = @Description
    AND [SupervisorId] = @SupervisorId

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[EvaluateProcessStatus]
(
	@VCC						VARCHAR(50),
	@ProcessId					VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON
	
	  UPDATE Process
	  SET [Status] = CASE 
		WHEN B.total <= transcription + cancel AND cancel > transcription THEN 'FAILED' 
		WHEN B.total <= transcription + cancel THEN 'COMPLETE' 
		ELSE 'INPROCESS_TRANSCRIPTION' 
		END
	  FROM (
		  SELECT 
			COUNT(1)					total
			,COUNT(B.isCancel)			cancel
			,COUNT(B.IsTranscribed)		transcription 
			,A.ProcessId				processId
			,A.vcc						vcc
		  FROM [ProcessAudio] A (NOLOCK)
		  INNER JOIN Audio B (NOLOCK) ON A.VCC = B.VCC AND A.AudioId =B.ID AND ISNULL(A.Section,0) = ISNULL(B.Section,0) AND ISNULL(A.Part,0) = ISNULL(B.Part,0)
		  WHERE A.VCC = @VCC AND A.ProcessId = @ProcessId
		  GROUP BY A.ProcessId, A.vcc
	  ) B
	  WHERE  B.VCC = [Process].VCC AND B.ProcessId = Process.[Name]

	UPDATE EvaluationPlan
	  set [Status] = 5
	  --SELECT * 
	  FROM EvaluationPlan A (NOLOCK)
	  INNER JOIN Process B (NOLOCK) ON A.Instance = B.Vcc AND A.ProcessId =  B.[Name] AND B.[Status] = 'COMPLETE'
	  WHERE
	  A.Instance = @VCC 
	  AND A.Evaluation_Mode = 'A'
	  AND A.ProcessId = @ProcessId
	  AND A.[Status] < 5

	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[FinishedBatchClose](
	@VCC VARCHAR(50) = NULL,
	@BatchTypeName VARCHAR(50) = NULL,
	@BatchTypeId VARCHAR(50) = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET NOCOUNT ON

	-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
	IF @VCC IS NULL OR @BatchTypeName IS NULL OR @BatchTypeId IS NULL
		-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
		RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

	-- DECLARAMOS UNA VARIABLE PARA EL ID DE AGENDA
	DECLARE @SCHEDULE_BATCH_ID VARCHAR(50)

	-- DECLARAMOS UNA VARIABLE PARA CONTABILIZAR LAS EJECUCIONES AUN NO TERMINADAS
	DECLARE @SCHEDULE_EXECUTIONS INT = 0

	--SELECT * FROM [dbo].[Batch] 
	UPDATE [dbo].[Batch] 
		SET [FinishReported] = 1
			,[LastModifiedDate] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [BatchTypeName] = @BatchTypeName 
			AND [BatchTypeId] = @BatchTypeId

	-- SEGUN EL BATCHTYPENAME ACTUALIZAMOS EL STATUS DE LA TABLA DEL BATCH
	IF @BatchTypeName = 'TRANSCRIPTION'
	BEGIN
		UPDATE [dbo].[Process] 
			SET [Status] = CASE WHEN ([Total] - ISNULL([Error], 0)) > ISNULL([Error], 0) THEN 'COMPLETE'
							WHEN ([Total] - ISNULL([Error], 0)) < ISNULL([Error], 0) THEN 'FAILED' END
				,[StatusDescription] = CASE WHEN ISNULL([Error], 0) > 0 THEN 'COMPLETED_WITH_ERROR' 
										WHEN ISNULL([Error], 0) = 0 THEN 'COMPLETED' END
			WHERE [VCC] = @VCC
				AND [Name] = @BatchTypeId
				AND [Percentage] = 1

		-- OBTENEMOS EL ID DE LA AGENDA EN CASO EXISTA
		SELECT @SCHEDULE_BATCH_ID = P.[BatchScheduleId]
			FROM [dbo].[Process] P (NOLOCK)
			WHERE P.[VCC] =  @VCC
				AND P.[Name] = @BatchTypeId

		-- SI EXISTE UN ID DE AGENDA
		IF @SCHEDULE_BATCH_ID IS NOT NULL
		BEGIN
			-- ACTUALIZAMOS LA EJECUCION EN CASO SEA UN PROCESS AGENDADO
			UPDATE BSE 
				SET BSE.[Status] = P.[StatusDescription],
					BSE.[LastModifiedByUserId] = 'system',
					BSE.[LastModifiedDate] = GETUTCDATE()
				FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				INNER JOIN [dbo].[Process] P (NOLOCK)
					ON P.[VCC] = BSE.[VCC]
						AND P.[Name] = BSE.[BatchId]
				WHERE BSE.[VCC] = @VCC
					AND P.[Name] = @BatchTypeId
					AND P.[BatchScheduleId] IS NOT NULL
		END
	END
	ELSE IF @BatchTypeName IN ('STUDY', 'TEMPLATE')
	BEGIN 
		UPDATE [dbo].[Study] 
			SET [StatusDescription] = CASE WHEN [Percentage] = 1 AND  ISNULL([Error],0) > 0 THEN 'COMPLETED_WITH_ERROR' 
										WHEN [Percentage] = 1 AND  ISNULL([Error],0) = 0 THEN 'COMPLETED' END
			WHERE [VCC] = @VCC
				AND [Name] = @BatchTypeId
				AND [Percentage] = 1

		-- OBTENEMOS EL ID DE LA AGENDA EN CASO EXISTA
		SELECT @SCHEDULE_BATCH_ID = S.[BatchScheduleId]
			FROM [dbo].[Study] S (NOLOCK)
			WHERE S.[VCC] =  @VCC
				AND S.[Name] = @BatchTypeId

		IF @SCHEDULE_BATCH_ID IS NOT NULL
		BEGIN
			-- ACTUALIZAMOS LA EJECUCION EN CASO SEA UN TEMPLATE AGENDADO 
			UPDATE BSE 
				SET BSE.[Status] = S.[StatusDescription],
					BSE.[LastModifiedByUserId] = 'system',
					BSE.[LastModifiedDate] = GETUTCDATE()
				FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				INNER JOIN [dbo].[Study] S (NOLOCK)
					ON S.[VCC] = BSE.[VCC]
						AND S.[Name] = BSE.[BatchId]
				WHERE BSE.[VCC] = @VCC
					AND S.[Name] = @BatchTypeId
					AND S.[BatchScheduleId] IS NOT NULL
		END
	END
	ELSE IF @BatchTypeName = 'EVALUATION_PLAN'
	BEGIN 
		UPDATE [dbo].[EvaluationPlan] 
			SET [StatusDescription] = CASE WHEN [Percentage] = 1 AND  ISNULL([Error], 0) > 0 THEN 'COMPLETED_WITH_ERROR' 
									WHEN [Percentage] = 1 AND  ISNULL([Error], 0) = 0 THEN 'COMPLETED' END 
			WHERE [Instance] = @VCC
				AND [ID] = @BatchTypeId
				AND [Percentage] = 1
				AND ([Status] = 3 OR [Status] = 5 OR [Status] = 7)

		-- OBTENEMOS EL ID DE LA AGENDA EN CASO EXISTA
		SELECT @SCHEDULE_BATCH_ID = EP.[BatchScheduleId]
			FROM [dbo].[EvaluationPlan] EP (NOLOCK)
			WHERE EP.[Instance] =  @VCC
				AND EP.[ID] = @BatchTypeId

		IF @SCHEDULE_BATCH_ID IS NOT NULL
		BEGIN
			-- ACTUALIZAMOS LA EJECUCION EN CASO SEA UN PLAN DE EVALUACION AGENDADO 
			UPDATE BSE 
				SET BSE.[Status] = EP.[StatusDescription],
					BSE.[LastModifiedByUserId] = 'system',
					BSE.[LastModifiedDate] = GETUTCDATE()
				FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
					ON EP.[Instance] = BSE.[VCC]
						AND EP.[ID] = BSE.[BatchId]
				WHERE BSE.[VCC] = @VCC
					AND EP.[ID] = @BatchTypeId
					AND EP.[BatchScheduleId] IS NOT NULL
		END
	END

	-- SI EXISTE UN ID DE AGENDA
	IF @SCHEDULE_BATCH_ID IS NOT NULL
	BEGIN
		-- CONTAMOS LA EJECUCIONES PARA ESA AGENDA
		SELECT @SCHEDULE_EXECUTIONS = COUNT(BSE.[Id]) 
			FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
			INNER JOIN [dbo].[BatchSchedule] BS (NOLOCK)
				ON BS.[VCC] = BSE.[VCC]
					AND BS.[Id] = BSE.[BatchScheduleId]
			WHERE BSE.[VCC] = @VCC
				AND BS.[BatchType] = @BatchTypeName
				AND BSE.[BatchScheduleId] = @SCHEDULE_BATCH_ID
				AND BSE.[Status] IN ('PENDING', 'REGISTERED_ON_REDIS', 'BATCH_CREATED', 'BATCH_STARTED')

		-- SI TENEMOS 0 EJECUCIONES NO TERMINADAS, DAMOS POR FINALIZADA LA AGENDA
		IF @SCHEDULE_EXECUTIONS = 0
			UPDATE [dbo].[BatchSchedule]
				SET [Status] = 'FINISHED',
					[LastModifiedByUserId] = 'system',
					[LastModifiedDate] = GETUTCDATE()
				WHERE [VCC] = @VCC 
					AND [ID] = @SCHEDULE_BATCH_ID
	END

	SELECT 1 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAbilitiesUsage] 
	@Instance VARCHAR(50) = NULL,
	@AbilityIDs VARCHAR(MAX) = NULL
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

    DECLARE @_IDS TABLE(
	 ID VARCHAR(50)
	)

    INSERT INTO @_IDS SELECT [Value] FROM [dbo].[Split](@AbilityIDs,',')

	SELECT [IdAbility] 'idAbility'
        FROM [dbo].[AbilityEvaluationFormat] 
		WHERE Instance = @Instance 
			AND IdAbility  IN (SELECT [ID] FROM @_IDS)
    
           
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAbility] (
	@Instance Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT A.[ID]								'id',
			A.[Title]							'title',
			A.[Detail]							'detail',
			N.[Culture]							'culture',
			A.[Type]							'type',
			A.[CalificationType]				'calificationType',
			A.[ExecutionType]					'executionType',
			A.[AnalysisType]					'analysisType',
			A.[Binary]							'binary',
			A.[NotApply]						'notApply',
			A.[Distribute]						'distribute',
			A.[IsCritical]						'isCritical',
			A.[AcceptComments]					'acceptComments',
			A.[CreatedDate]						'createdDate',
			A.[CreatedByUserId]					'createdByUserId',
			A.[LastModifiedDate]				'lastModifiedDate',
			A.[LastModifiedByUserId]			'lastModifiedByUserId',
			A.[Enabled]							'enabled',
			A.[Query]							'query',
			A.[QueryName]						'queryName',
			A.[Language]						'language',
			A.[IsNegative]						'isNegative',
            AC.[DegradeScoreCard]				'degradeScoreCard'
		FROM [dbo].[Ability] A (NOLOCK)
		LEFT JOIN [dbo].[NLU] N (NOLOCK)
			ON N.[VCC] = A.[Instance]
				AND N.[Name] = SUBSTRING(A.[Query], 0, CHARINDEX('|', A.[Query]))
        LEFT JOIN [dbo].[AbilityScoreCard] AC 
            ON AC.[VCC] = A.[Instance] 
                AND AC.[ID] = A.[ID]       
		WHERE A.[Instance] = @Instance
			AND A.[ID] = @ID

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAbilityDetail] (
	@VCC Varchar(50) = NULL,
	@AbilityId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
	AQ.QueryId      'queryId',
	AQ.QueryContent 'queryContent',
	A.[AnalysisType] 'analysisType',
	(
	SELECT [AQE].[EntityName] 'entityName' 
	FROM [dbo].[AbilityQueryEntity] AQE (NOLOCK)
	WHERE [AQE].[VCC] = @VCC AND [AQE].[AbilityQueryId] IN(AQ.ID) FOR JSON AUTO
	)              'entitiesRaw'
	
	FROM [dbo].[AbilityQuery] AQ (NOLOCK)
	JOIN [dbo].[Ability] A (NOLOCK)
	ON A.[Instance] = AQ.[VCC] AND A.[ID] = AQ.[AbilityId]
	WHERE AQ.[VCC]=@VCC AND 
	      AQ.AbilityId= @AbilityId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAbilityEvaluationFormat] (
	@Instance Varchar(50) = NULL, @IdEvaluationFormat Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]							'id',
		[IdEvaluationFormat]			'idEvaluationFormat',
		[IdAbility]						'idAbility',
		[IdGroup]						'idGroup',
		[IdSubGroup]					'idSubGroup',
		[Penalizes]						'penalizes',
		[Score]							'score',
		[Order]							'order',
		[Conditional]					'conditional'
	FROM [AbilityEvaluationFormat] (NOLOCK)
	WHERE [Instance] = @Instance
	AND [IdEvaluationFormat] = @IdEvaluationFormat
	ORDER BY [Order] ASC

	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAbilityEvaluationFormats] (
	@Instance           Varchar(50) = NULL, 
	@IdEvaluationFormat Varchar(50) = NULL,
	@EvaluationPlanId   VARCHAR(50) = NULL -- DEPRECATED: A ELIMINAR
) AS
BEGIN
	SET NOCOUNT ON

	SELECT DISTINCT AEF.[ID]						'id',
					AEF.[IdEvaluationFormat]		'idEvaluationFormat',
					AEF.[IdAbility]					'idAbility',
					A.[Title]						'titleAbility',
					A.[Detail]						'detailAbility',
					A.[Type]						'abilityType',
					A.[AnalysisType]				'abilityAnalysisType',
					-- A.[CalificationType]			'abilityCalificationType',
					AEF.[IdGroup]					'idGroup',
					G.[Description]                 'descriptionGroup',
					AEF.[IdSubGroup]				'idSubGroup',
					SG.[Description]                'descriptionSubGroup',
					AEF.[Penalizes]					'penalizes',
					AEF.[Score]						'score',
					AEF.[Order]						'order',
					A.[NotApply]					'acceptNotApplies',
					A.[Distribute]					'distribute',
					A.[AcceptComments]				'acceptComments',
					A.[Detail]						'descriptionAbility',
					AEF.[Conditional]				'conditional',
					A.[Query]						'abilityQuery',
					A.QueryName						'query',
					A.[IsNegative]                  'isNegative',
					AEF.[HelpComment]				'helpComment'
		FROM [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
		INNER JOIN [dbo].[Group] G (NOLOCK)
			ON G.[Instance] = AEF.[Instance]
				AND G.[ID] = AEF.[IdGroup]
		INNER JOIN [dbo].[SubGroup] SG (NOLOCK)
			ON SG.[Instance] = AEF.[Instance]
				AND SG.[ID] = AEF.[IdSubGroup]
		INNER JOIN [dbo].[Ability] A (NOLOCK)
			ON A.[Instance] = AEF.[Instance]
				AND A.[ID] = AEF.[IdAbility]
		WHERE AEF.[Instance] = @Instance
			AND AEF.[IdEvaluationFormat]= @IdEvaluationFormat
		ORDER BY [Order] ASC


	
	-- CONSULTAMOS LOS SCORES RELACIONADOS AL FORMATO
	SELECT S.[Id]									'id',
			S.[IdAbility]							'abilityId',
			S.[Description]							'description',
			S.[Value]								'value'
		FROM [dbo].[Ability] A (NOLOCK)
		INNER JOIN [dbo].[Score] S (NOLOCK)
			ON S.[Instance] = A.[Instance]
				AND S.[IdAbility] = A.[ID]
		INNER JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK) 
			ON AEF.[Instance] = A.[Instance]
				AND AEF.[IdAbility] = A.[ID]
		WHERE AEF.[Instance] = @Instance
			AND AEF.[IdEvaluationFormat] = @IdEvaluationFormat
		ORDER BY S.[Value] DESC



	-- CONSULTAMOS LAS TIPIFICACIONES DE LAS HABILIDADES RELACIONADAS AL FORMATO
	SELECT CAST(EFAST.[ScoreId] AS INT)				'scoreId',
			EFAST.[ID]								'id',
			EFAST.[Description]						'description',
			EFAST.[Detail]							'detail',
			EFAST.[Order]							'order'
		FROM [dbo].[EvaluationFormatAbilityScoreTipification] EFAST (NOLOCK)
		WHERE EFAST.[VCC] = @Instance
			AND EFAST.[EvaluationFormatId] = @IdEvaluationFormat
		ORDER BY EFAST.[Order] ASC



	-- CONSULTAMOS LAS QUERY DE TIPIFICACION DE LAS HABILIDADES RELACIONADAS AL FORMATO
	SELECT CAST(EFASTQ.[ScoreId] AS INT)			'scoreId',
			EFASTQ.[TipificationId]					'tipificationId',
			EFASTQ.[QueryId]						'queryId',
			EFASTQ.[QueryContent]					'queryContent',
			EFASTQ.[AcceptancePercentage]			'acceptancePercentage',
			EFASTQ.[AcceptancePercentage] * 100		'acceptanceInteger'
		FROM [dbo].[EvaluationFormatAbilityScoreTipificationQuery] EFASTQ (NOLOCK)
		WHERE EFASTQ.[VCC] = @Instance
			AND EFASTQ.[EvaluationFormatId] = @IdEvaluationFormat	



	-- CONSULTAMOS LOS ITEMS DE TIPIFICACIONES DE LAS HABILIDADES RELACIONADAS AL FORMATO
	SELECT CAST(EFASTI.[ScoreId] AS INT)			'scoreId',
			EFASTI.[TipificationId]					'tipificationId',
			EFASTI.[ID]								'id',
			EFASTI.[SecondLevelItemId]				'secondLevelItemId',
			EFASTI.[Description]					'description',
			EFASTI.[Order]							'order'
		FROM [dbo].[EvaluationFormatAbilityScoreTipificationItem] EFASTI (NOLOCK)
		WHERE EFASTI.[VCC] = @Instance
			AND EFASTI.[EvaluationFormatId] = @IdEvaluationFormat
		ORDER BY EFASTI.[Order] ASC
	


	/*
	DECLARE @QUERY VARCHAR(MAX) = ''

	SET @QUERY += '	SELECT DISTINCT
					AEF.[ID]							''id'',
					AEF.[IdEvaluationFormat]		''idEvaluationFormat'',
					AEF.[IdAbility]					''idAbility'',
					A.[Title]						''titleAbility'',
					A.[Detail]						''detailAbility'',
					A.[Type]						''abilityType'',
					A.[AnalysisType]				''abilityAnalysisType'',
					AEF.[IdGroup]					''idGroup'',
					G.[Description]                 ''descriptionGroup'',
					AEF.[IdSubGroup]				''idSubGroup'',
					SG.[Description]                ''descriptionSubGroup'',
					AEF.[Penalizes]					''penalizes'',
					AEF.[Score]						''score'',
					AEF.[Order]						''order'',
					A.[NotApply]					''acceptNotApplies'',
					A.[Distribute]					''distribute'',
					A.[AcceptComments]				''acceptComments'',
					A.[Detail]						''descriptionAbility'',
					AEF.[Conditional]				''conditional'',
					A.[Query]						''abilityQuery'',
					A.QueryName						''query'',
					A.[IsNegative]                  ''isNegative'''

	 IF (@EvaluationPlanId IS NOT NULL AND @EvaluationPlanId <> '')
	 BEGIN 

			SET @QUERY +='	FROM [dbo].[HistoricAbilityEvaluationFormat] AEF (NOLOCK)
				INNER JOIN [dbo].[HistoricGroup] G (NOLOCK)
					ON G.[VCC] = AEF.[VCC]
						AND G.[ID] = AEF.[IdGroup] AND G.[EvaluationPlanId] = AEF.[EvaluationPlanId]
				INNER JOIN [dbo].[HistoricSubGroup] SG (NOLOCK)
					ON SG.[VCC] = AEF.[VCC]
						AND SG.[ID] = AEF.[IdSubGroup]
						AND SG.[EvaluationPlanId] = AEF.[EvaluationPlanId]
				INNER JOIN [dbo].[HistoricAbility] A (NOLOCK)
					ON A.[VCC] = AEF.[VCC]
						AND A.[ID] = AEF.[IdAbility]
						AND A.[EvaluationPlanId] = AEF.[EvaluationPlanId]
				WHERE AEF.[VCC] = '''+@Instance+'''
					AND AEF.[IdEvaluationFormat]= '''+@IdEvaluationFormat+'''
				    AND AEF.[EvaluationPlanId] = '''+@EvaluationPlanId+'''
				ORDER BY [Order] ASC'
	
	 END
	 ELSE
	 BEGIN 

		   SET @QUERY += '	FROM [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
				INNER JOIN [dbo].[Group] G (NOLOCK)
					ON G.[Instance] = AEF.[Instance]
						AND G.[ID] = AEF.[IdGroup]
				INNER JOIN [dbo].[SubGroup] SG (NOLOCK)
					ON SG.[Instance] = AEF.[Instance]
						AND SG.[ID] = AEF.[IdSubGroup]
				INNER JOIN [dbo].[Ability] A (NOLOCK)
					ON A.[Instance] = AEF.[Instance]
						AND A.[ID] = AEF.[IdAbility]
				WHERE AEF.[Instance] = '''+ @Instance+'''
					AND AEF.[IdEvaluationFormat]= '''+@IdEvaluationFormat+'''
				ORDER BY [Order] ASC'
			
	END

	EXEC(@QUERY)
	*/
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAbilityQueries] (
	@VCC              VARCHAR(50) = NULL, 
	@AbilityId        VARCHAR(100) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
    
    DECLARE @QUERY VARCHAR(MAX) = ''
    
    SET @QUERY += 'SELECT [AQ].[ID]											''id'',
					[AQ].[QueryId]											''queryId'',
					[AQ].[QueryContent]										''queryContent'',
					[AQ].[AcceptancePercentage]								''acceptancePercentage'',
					ISNULL([AQ].[AcceptancePercentage], 0) * 100			''acceptanceInteger'',
					[AQ].[Order]										    ''order'' FROM '
    IF(@EvaluationPlanId IS NOT NULL AND @EvaluationPlanId <> '')
    BEGIN
	    SET @QUERY += ' [dbo].[HistoricAbilityQuery] AQ (NOLOCK) WHERE [AQ].[VCC] = '''+@VCC+
					  '''AND [AQ].[AbilityId] ='''+ @AbilityId+''' AND AQ.[EvaluationPlanId] = '''+@EvaluationPlanId+''''		
    END
    ELSE
    BEGIN	
	    SET @QUERY += ' [dbo].[AbilityQuery] AQ (NOLOCK) WHERE [AQ].[VCC] = '''+@VCC+
					  '''AND [AQ].[AbilityId] ='''+ @AbilityId+''''
    END

	SET @QUERY += ' ORDER BY [AQ].[Order] ASC'
	EXEC(@QUERY)
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAbilityQueryEntities] (
	@VCC VARCHAR(50) = NULL, 
	@AbilityQueryId VARCHAR(100) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL
) AS
BEGIN
		SET NOCOUNT ON

		DECLARE @QUERY VARCHAR(MAX) = ''
		SET @QUERY +=' SELECT [AQE].[EntityId]								''entityId'',
					   [AQE].[EntityName]									''entityName'',
					   [AQE].[AcceptancePercentage]						    ''acceptancePercentage'',
					   ISNULL([AQE].[AcceptancePercentage], 0) * 100		''acceptanceInteger'',
					   [AQE].[Order]										''order''
					   FROM '
			   
		IF(@EvaluationPlanId IS NOT NULL AND @EvaluationPlanId <> '')
		BEGIN
			SET @QUERY += ' [dbo].[HistoricAbilityQueryEntity] AQE (NOLOCK) 
							WHERE AQE.[EvaluationPlanId] = '''+@EvaluationPlanId+''' AND '	
		END 
		ELSE 
		BEGIN 
			SET @QUERY += '	[dbo].[AbilityQueryEntity] AQE (NOLOCK) WHERE '
		END
		SET @QUERY += ' [AQE].[VCC] ='''+ @VCC +''''
		SET @QUERY += ' AND [AQE].[AbilityQueryId] ='''+ @AbilityQueryId+''''
		SET @QUERY += ' ORDER BY [AQE].[Order] ASC'
		EXEC(@QUERY)

		SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE  PROCEDURE [dbo].[GetAbilitys] (
	@Instance VARCHAR(50) = NULL, 
	@SearchCriteria VARCHAR(50) = NULL, 
	@SearchExpression VARCHAR(50) = NULL, 
	@ExecutionType VARCHAR(25) = NULL,
	@Disabled BIT = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT A.[ID]								'id',
			A.[Title]							'title',
			A.[Detail]							'detail',
			A.[Type]							'type',
			A.[CalificationType]				'calificationType',
			A.[AnalysisType]					'analysisType',
			A.[ExecutionType]					'executionType',
			A.[Binary]							'binary',
			A.[NotApply]						'notApply',
			A.[IsCritical]						'isCritical',
			A.[AcceptComments]					'acceptComments',
			A.[Enabled]							'enabled',
			A.[Query]							'query',
			A.[Queryname]						'queryName',
			N.[Culture]							'culture',
			A.[CreatedDate]						'createdDate',
			A.[CreatedByUserId]					'createdByUserId',
			A.[LastModifiedDate]  				'lastModifiedDate',
			A.[LastModifiedByUserId]			'lastModifiedByUserId'
		FROM Ability a (NOLOCK)
		LEFT JOIN [dbo].[NLU] N (NOLOCK)
			ON N.[VCC] = A.[Instance]
				AND N.[Name] = SUBSTRING(A.[Query], 0, CHARINDEX('|', A.[Query]))
		WHERE a.Instance = @Instance
			AND ISNULL(a.ID, '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND ISNULL(a.Title, '') LIKE CASE WHEN @SearchCriteria = 'title' THEN '%' + @SearchExpression + '%' COLLATE Latin1_General_CI_AI ELSE '%' END
			AND ISNULL(a.Detail, '') LIKE CASE WHEN @SearchCriteria = 'detail' THEN '%' + @SearchExpression + '%' COLLATE Latin1_General_CI_AI ELSE '%' END
			AND A.[ExecutionType] = @ExecutionType
			AND (ISNULL(@Disabled, 0) = 1 OR (ISNULL(@Disabled, 0) = 0 AND ISNULL(@Disabled, 0) <> [Enabled]))
		ORDER BY a.[Title]

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAbilityTitle] (
	@Instance Varchar(50) = NULL, @Title Varchar(100) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]							'id',
		[Title]							'title',
		[Detail]						'detail',
		[Binary]						'binary',
		[NotApply]						'notApply',
		[AcceptComments]				'acceptComments',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId',
		[Query]							'query',
		[QueryName]                     'queryName'
	FROM [Ability] (NOLOCK)
	WHERE [Instance] = @Instance
	AND [Title] = @Title

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAdvancedAI] (
    @VCC VARCHAR(50) = NULL,
    @SearchCriteria VARCHAR(50) = NULL,
    @SearchExpression VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
   
   -- BUSCAMOS EL MODELO POR EL DATO RECIBIDO
	SELECT [VCC]					'vcc',
			[ID]					'id',
			[Description]			'description',
			[StatusCode]			'status',
			[CreatedDate]			'createdDate',
			[CreatedByUserId]		'createdByUserId',
			[LastModifiedDate]		'lastModifiedDate',
			[LastModifiedByUserId]	'lastModifiedByUserId'
		FROM [dbo].[AdvancedAI] (NOLOCK)
		WHERE [VCC] = @VCC
			AND [ID] = CASE WHEN @SearchCriteria = 'id' THEN @SearchExpression ELSE [ID] END
			--AND [Prompt] LIKE CASE WHEN @SearchCriteria = 'name' THEN '%' + @SearchExpression + '%' ELSE [Prompt] END

	-- SI LA BUSQUEDA SE HIZO POR ID, BUSCAMOS LOS INTENTS Y EXAMPLES
	IF @SearchCriteria = 'id'
	BEGIN
		EXEC [dbo].[GetAdvancedAIIntents] @VCC, 'appId', @SearchExpression, NULL
		EXEC [dbo].[GetAdvancedAIExamples] @VCC, 'appId', @SearchExpression
	END

	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAdvancedAIEngineToken] (
	@VCC VARCHAR(50) = NULL
) AS
BEGIN
	-- OBTENEMOS EL TOKEN USANDO EL VCC
	SELECT [Token]				'token'
		FROM [dbo].[AdvancedAIEngineToken] (NOLOCK)
		WHERE [VCC] = @VCC

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAdvancedAIExamples] (
    @VCC VARCHAR(50) = NULL,
    @SearchCriteria VARCHAR(50) = NULL,
    @SearchExpression VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
   
	-- HACEMOS LA BUSQUEDA SEGUN EL CRITERIO RECIBIDO
	SELECT [Id]						'id',
			[AppId]					'appId',
			[IntentId]				'intentId',
			[Text]					'text'
		FROM [dbo].[AdvancedAIExample] (NOLOCK)
		WHERE [VCC] = @VCC
			AND [AppId] = CASE WHEN @SearchCriteria = 'appId' THEN @SearchExpression ELSE [AppId] END
		ORDER BY [text] ASC

	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAdvancedAIIntentItems] (
    @VCC VARCHAR(50) = NULL,
    @SearchCriteria VARCHAR(50) = NULL,
    @SearchExpression VARCHAR(50) = NULL,
    @IntentIds NVARCHAR(MAX) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- HACEMOS LA BUSQUEDA SEGUN EL CRITERIO RECIBIDO
	SELECT AI.[Id]						'appId',
			AI.[Id]						'appName',
			AII.[Id]					'id',
			AII.[Name]					'name'
		FROM [dbo].[AdvancedAIIntent] AII (NOLOCK)
		INNER JOIN [dbo].[AdvancedAI] AI (NOLOCK)
			ON AI.[VCC] = AII.[VCC]
				AND AI.[Id] = AII.[AppId]
		WHERE AII.[VCC] = @VCC
			AND AII.[AppId] = CASE WHEN @SearchCriteria = 'appId' THEN @SearchExpression ELSE AII.[AppId] END
			AND AII.[Id] = CASE WHEN @SearchCriteria = 'intentId' THEN @SearchExpression ELSE AII.[Id] END
		ORDER BY AI.[Id] ASC, AII.[Name] ASC
		
	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAdvancedAIIntents] (
    @VCC VARCHAR(50) = NULL,
    @SearchCriteria VARCHAR(50) = NULL,
    @SearchExpression VARCHAR(50) = NULL,
    @IntentIds NVARCHAR(MAX)
)
AS
BEGIN
	SET NOCOUNT ON

	/*
	-- HACEMOS LA BUSQUEDA SEGUN EL CRITERIO RECIBIDO
	SELECT [AppId]					'appId',
			[Id]					'id',
			[Name]					'name',
			[Description]			'description'
		FROM [dbo].[GenerativeIntent] (NOLOCK)
		WHERE [VCC] = @VCC
			AND ISNULL([AppId], '''') = CASE WHEN @SearchCriteria = 'appId' THEN @SearchExpression ELSE [AppId] END
			AND ISNULL([Name], '''') = CASE WHEN @SearchCriteria = 'name' THEN @SearchExpression ELSE [Name] END
		ORDER BY [LastModifiedDate] ASC, [Name] DESC

	SET NOCOUNT OFF
	*/

	
	DECLARE @SQL NVARCHAR(MAX)
	SET @SQL = '
	SET NOCOUNT ON
	SELECT [AppId]								''appId'',
			[Id]								''id'',
			[Name]								''name'',
	        [prompt]							''prompt''
		FROM [dbo].[AdvancedAIIntent] (NOLOCK) 
		WHERE [VCC] = ''' + @VCC + ''' '

	-- AGREGAMOS LOS FILTROS SEGUN LOS CRITERIOS
	IF @SearchCriteria IS NOT NULL AND @SearchExpression IS NOT NULL
	BEGIN
		IF @SearchCriteria = 'appId' 
			SET @SQL += ' 
			AND ISNULL([AppId], '''') = ''' + @SearchExpression + ''' '
		IF @SearchCriteria = 'name' 
			SET @SQL += ' 
			AND ISNULL([Name], '''') = ''' + @SearchExpression + ''' '
	END

	-- BUSCAMOS POR IDS EN CASO EXISTE ESE INPUT
	IF ISNULL(@IntentIds, '') <> ''
		SET @SQL += ' 
			AND [ID] IN (SELECT Value FROM [dbo].[split](''' + @IntentIds + ''','',''))'

	SET @SQL += '
		ORDER BY [AppId]'

	SET @SQL += ' 
	SET NOCOUNT OFF'
	
	--print @SQL
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAdvancedAIs] (
    @VCC VARCHAR(50) = NULL,
    @SearchCriteria VARCHAR(50) = NULL,
    @SearchExpression VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
   
	DECLARE  @SQL              NVARCHAR(MAX)
	DECLARE  @FILTER           VARCHAR(MAX) = ''
    DECLARE  @UNION_SQL        VARCHAR(MAX) = ''
    DECLARE  @APPS_FILTER      VARCHAR(MAX) = ''
    DECLARE  @STATUS_FILTER    VARCHAR(MAX) = ''
    DECLARE  @ORDER_BY         VARCHAR(MAX) = ''
    
	SET @SQL = '
    SELECT [VCC]					''vcc'',
			[ID]					''id'',
			[Description]			''description'',
			[CreatedDate]			''createdDate'',
			[CreatedByUserId]		''createdByUserId'',
			[LastModifiedDate]		''lastModifiedDate'',
			[LastModifiedByUserId]	''lastModifiedByUserId''
		FROM [dbo].[AdvancedAI] (NOLOCK)
		WHERE [VCC] = ''' + @VCC + ''' '

	IF (
		@SearchCriteria IS NOT NULL AND @SearchCriteria <> '' 
		AND 
		@SearchExpression IS NOT NULL AND @SearchExpression <> ''
	)
    BEGIN
        SET @FILTER += '
			AND ISNULL([ID], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
			AND ISNULL([Description], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''description'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END'
    END

    SET @ORDER_BY = ' ORDER BY [LastModifiedDate] ASC, [ID] DESC'

    SET @SQL += @FILTER + @APPS_FILTER + @STATUS_FILTER + @UNION_SQL +  @ORDER_BY
    
	EXEC(@SQL)

	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAgentQualificationAverage] (
	@Instance Varchar(50) = NULL,
	@IdAgent Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		ERAQS.[EvaluationResultAgentQualificationSum] / ERAQ.[EvaluationResultAgentQuantity]				'AgentQualificationAverage'
	FROM
		(
			-- EvaluationResultAgentQualificationSum
			SELECT
				SUM([QualificationSum]) EvaluationResultAgentQualificationSum
			FROM
				(
					SELECT SUM([Score])			'QualificationSum'
						FROM [dbo].[AuditorEvaluationPlan] (NOLOCK) 
						WHERE [Instance] = @Instance
							AND [IdAgent]= @IdAgent
							AND [Completed] = 1 
						GROUP BY
							[Instance],
							[IdEvaluationPlan],
							[IdAuditor],
							[IdAgent],
							[IdInteraction]
					/*
					SELECT
						SUM([Qualification]) QualificationSum
					FROM
						EvaluationResult
					WHERE
						[Instance] = @Instance
						AND [IdAgent]= @IdAgent
					GROUP BY
						[Instance],
						[IdEvaluationPlan],
						[IdAuditor],
						[IdAgent],
						[IdInteraction]
					*/
				) EvaluationResultAgent
		) ERAQS,
		(
			-- EvaluationResultAgentQuantity
			SELECT
				COUNT(*) EvaluationResultAgentQuantity
			FROM
				(
					SELECT COUNT(*)						'EvaluationResultAgentQuantity'
						FROM [dbo].[AuditorEvaluationPlan] (NOLOCK) 
						WHERE [Instance] = @Instance
							AND [IdAgent]= @IdAgent
							AND [Completed] = 1 
						GROUP BY
							[Instance],
							[IdEvaluationPlan],
							[IdAuditor],
							[IdAgent],
							[IdInteraction]

			/*
					SELECT
						[IdAgent]
					FROM
						EvaluationResult
					WHERE
						[Instance] = @Instance
						AND [IdAgent]= @IdAgent
					GROUP BY
						[Instance],
						[IdEvaluationPlan],
						[IdAuditor],
						[IdAgent],
						[IdInteraction]
			*/
				) EvaluationResultAgent
		) ERAQ

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAgents] (
	@VCC VARCHAR(50) = NULL,
	@AgentIds VARCHAR(MAX) = NULL
) AS 
BEGIN 
    SET NOCOUNT ON

	-- DECLARAMOS LA VARIABLES PARA LA QUERY A EJECUTAR
	DECLARE @SQL_SELECT VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''
	
	-- ARMAMOS LA CONSULTA SIMPLE
	SET @SQL_SELECT += '
		SELECT A.[AgentId]												''agentId'',
				A.[Name]												''name'' '

	-- AGREGAMOS LA TABLA DE BUSQUEDA
	SET @SQL_FROM += '
			FROM [dbo].[Agent] A (NOLOCK) '

	-- 
	SET @SQL_FILTER += '
			WHERE A.[VCC] = ''' + @VCC + ''' '

	-- AGREGAMOS LOS FILTROS
	IF (@AgentIds IS NOT NULL AND @AgentIds != '') 
		-- FILTRAMOS LOS AGENTES QUE SE MODIFICARON HACE MAS DE 7 DIAS PARA ACTUALIZARLOS EN CASO SEA NECESARIO
		SET @SQL_FILTER += '
				AND A.[AgentId] IN (SELECT Value FROM [dbo].[split](''' + @AgentIds + ''','',''))
				AND DATEDIFF(SECOND, A.[TimeStamp], GETUTCDATE()) <= 5 '

	-- JUNTAMOS LAS PARTES DE LA CONSULTA
	SET @SQL_SELECT += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA

	-- EJECUTAMOS LA CONSULTA
	EXEC (@SQL_SELECT)
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTE SP RETORNA LOS AGENTES ASOCIADOS A EVALUACIONES
CREATE PROCEDURE [dbo].[GetAgentsFromEvaluation] 
(
	@VCC					VARCHAR(50) = NULL,
	@UserId					VARCHAR(50) = NULL,
	@Campaigns				VARCHAR(MAX) = NULL,
	@SearchType				VARCHAR(50) = NULL,
	@EfectivityMode			BIT = 0 
) 
AS
BEGIN
	-- DECLARAMOS UN CADENA PARA EL QUERY FINAL
	DECLARE @SQL_SELECT VARCHAR(MAX) = '';
	DECLARE @SQL_FROM VARCHAR(MAX) = '';
	DECLARE @SQL_FROM_EFECTIVITY VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER_EFECTIVITY VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA VARCHAR(MAX) = '';

	-- ARMAMOS EL QUERY PARA HACER LA CONSULTA
	SET @SQL_SELECT += '
		SELECT DISTINCT IEP.[Instance]							''vcc'', 
						IEP.[IdAgent]							''userId''
						-- IEP.[IdCampaign]						''campaignId'' -- NO SE USA APARENTEMENTE (PROBLEMA SAT V.3.7.0) '

	-- ARMAMOS LA PARTE DE TABLAS 
	SET @SQL_FROM += '
			FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
			INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = AEP.[Instance]
					AND IEP.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND IEP.[IdInteraction] = AEP.[IdInteraction] '
	SET @SQL_FROM_EFECTIVITY += '
			FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
			INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = AEP.[Instance]
					AND IEP.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND IEP.[IdInteraction] = AEP.[IdInteraction] 
			INNER JOIN (
				SELECT DISTINCT [VCC],
								[InteractionId],
								[AgentId],
								[EvaluationPlanId],
								[AuditorId] 
					FROM [dbo].[EvaluationResultEfectivity] (NOLOCK)
					WHERE [VCC] = ''' + @VCC + ' '' 
						AND [EfectivityStatus] IN (''' + 'CREATED' + '''' + ',' + '''' + 'DRAFT' + ''''+ ')' + '
			) AS ERE ON 
				(AEP.[Instance] = ERE.[VCC] 
					AND AEP.[IdEvaluationPlan] = ERE.[EvaluationPlanId]
					AND AEP.[IdInteraction] = ERE.[InteractionId])'	

	-- ARMAMOS LA PARTE DE FILTROS
	SET @SQL_FILTER += '
			WHERE AEP.[Instance] = ''' + @VCC + ''' 
				AND LEN(IEP.[IdAgent]) > 0 '
	SET @SQL_FILTER_EFECTIVITY += '
			WHERE AEP.[Instance] = ''' + @VCC + ''' 
				AND LEN(IEP.[IdAgent]) > 0 '

	-- ARMAMOS LA PARTE DE EXTRAS
	SET @SQL_EXTRA = '
			ORDER BY IEP.[IdAgent] ASC '

	-- FILTRAMOS SEGUN EL TIPO DE BUSQUEDA
	IF @SearchType = 'PENDING'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES QUE NO HAN SIDO EVALUADAS
		SET @SQL_FILTER += '
			AND AEP.[Completed] = 0 '
	END
	ELSE IF @SearchType = 'HISTORY'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES QUE YA HAN SIDO EVALUADAS
		SET @SQL_FILTER += '
			AND AEP.[Completed] = 1 '
	END

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL ROLE DEL USUARIO
	DECLARE @USER_ROLE VARCHAR(50) 
	SELECT TOP 1 @USER_ROLE = U.[Role] 
		FROM [dbo].[User] U (NOLOCK) 
		WHERE U.[VCC] = @VCC
			AND U.[ID] = @UserId

	-- FILTRAMOS SEGUN EL ROLE DEL USUARIO
	-- IF @RoleId = 'administrator, quality_admin'
	IF @USER_ROLE = 'quality_auditor'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES AUDITADAS POR EL USUARIO
		SET @SQL_FILTER += '
			AND AEP.[IdAuditor] = ''' + @UserId + ''' '
		SET @SQL_FILTER_EFECTIVITY += '
			AND ERE.[AuditorId] = ''' + @UserId + ''' '
	END
	ELSE IF @USER_ROLE = 'quality_agent'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES HECHAS POR EL USUARIO
		SET @SQL_FILTER += '
			AND AEP.[IdAgent] = ''' + @UserId + ''' '
	END

	-- FILTRAMOS SEGUN EL VALOR DE CAMPAIGNS
	IF @Campaigns IS NOT NULL AND @Campaigns <> ''
	BEGIN
		SET @SQL_FILTER += '
			AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','','')) '
        SET @SQL_FILTER_EFECTIVITY += '
			AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','','')) '
	END 

	-- UNIMOS LAS PARTES DEL QUERY
	IF (@EfectivityMode =1)
	SET @SQL_SELECT += @SQL_FROM_EFECTIVITY + @SQL_FILTER_EFECTIVITY + @SQL_EXTRA
	ELSE
	SET @SQL_SELECT += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA
	
	-- EJECUTAMOS LA CONSULTA
	--PRINT @SQL_SELECT
	EXEC (@SQL_SELECT)
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAgentsFromHistoricEvaluationPlans] (
	@VCC VARCHAR(50)
	)
AS
BEGIN
	SET NOCOUNT ON
 	
 	SELECT DISTINCT ER.[IdAgent] 'idAgent'
 	FROM [dbo].[EvaluationResult] ER (NOLOCK)
 	JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
 	ON ER.[Instance] = AEP.[Instance] AND ER.IdEvaluationPlan = AEP.IdEvaluationPlan
 	WHERE ER.[Instance] = @VCC AND AEP.Completed = 1

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAllegroAgent]
(
	@IP				VARCHAR(50),
	@VCC			VARCHAR(50),
	@UserId			VARCHAR(50),
	@Type			INT
)
AS
BEGIN

	DECLARE @SQL VARCHAR(MAX)
	IF(@Type = 0)
	BEGIN
		SET @SQL = '
		SELECT [VCC], [UserId]
		FROM
			[' + @Ip + ']' + '.mmprodat.dbo.[Agent] WITH (NOLOCK)
		WHERE
			[VCC] = ''' + @VCC + '''
			AND [UserId] = ''' + @UserId + '''
		'
	END
	ELSE
	BEGIN
		SET @SQL = '
		SELECT [VCC], [UserId]
		FROM
			mmprodat.dbo.[Agent] WITH (NOLOCK)
		WHERE
		[VCC] = ''' + @VCC + '''
		AND [UserId] = ''' + @UserId + '''
		'
	END
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAllegroAgents] (
	@Ip VARCHAR(50), 
	@VCC VARCHAR(50), 
	@Type INT, 
	@Campaigns VARCHAR(MAX) = NULL
) AS
BEGIN
	-- ESTE SP RETORNA LA LISTA DE AGENTES DEL VCC SEGUN LAS CAMPANAS

	-- DECLARAMOS VARIABLES PARA ARMAR LA CONSULTA
	DECLARE @SQL VARCHAR(MAX) = NULL
	DECLARE @TABLE_MMPRODAT VARCHAR(100) = NULL

	-- EL TYPE 0 INDICA QUE LAS TABLAS SE ENCUENTRAN EN UN SERVIDOR EXTERNO
	IF @Type = 0
	BEGIN  
		SET @TABLE_MMPRODAT = '[' + @Ip + ']' + '.[MMProDat].[dbo].'
	END
	ELSE IF(@Type = 1) 
	BEGIN
		SET @TABLE_MMPRODAT = '[MMProDat].[dbo].'
	END

	-- ARMAMOS LA QUERY optimizada para agentes sin linea asignada
	SET @Sql = '
		SELECT U.[VCC]								''vcc'',
				UC.[CampaignId]						''campaignId'',
				U.[Id]								''userId'',
				U.[Name]							''userName'',
				A.[WorkgroupId]						''workGroupId''
			FROM ' + @TABLE_MMPRODAT + '[User] U (NOLOCK)
			LEFT JOIN ' + @TABLE_MMPRODAT + '[GroupApplication] GA (NOLOCK)
				ON (
						GA.[VCC] = U.[VCC]
						OR
						GA.[VCC] = ''system''
					)
					AND GA.[GroupId] = U.[UserGroup]
			LEFT JOIN ' + @TABLE_MMPRODAT + '[Agent] A (NOLOCK)
				ON A.[VCC] = U.[VCC]
					AND A.[UserId] = U.[Id] 
			LEFT JOIN ' + @TABLE_MMPRODAT + '[UserCampaign] UC (NOLOCK) 
				ON UC.VCC = U.[VCC] 
					AND UC.[UserId] = U.[Id] 
			WHERE U.[VCC] = ''' + @VCC + '''
				AND GA.ApplicationId = ''baragent'' '

	/*
	-- ARMAMOS LA QUERY
	SET @Sql = '
		SET NOCOUNT ON
		SELECT UC.[VCC]							''vcc'',
				UC.[CampaignId]						''campaignId'',
				UC.[UserId]							''userId'',
				U.[Name]							''userName'',
				A.[WorkgroupId]						''workGroupId''
			FROM ' + @TABLE_MMPRODAT + '[Agent]  A (NOLOCK)
			LEFT JOIN ' + @TABLE_MMPRODAT + '[User] U (NOLOCK)
				ON U.[VCC] = A.[VCC]
					AND U.[Id] = A.[UserId]
			LEFT JOIN ' + @TABLE_MMPRODAT + '[UserCampaign] UC (NOLOCK) 
				ON UC.VCC = A.VCC 
					AND UC.UserId = A.UserId
			WHERE UC.[VCC] = ''' + @VCC + ''''
	*/

	IF @Campaigns IS NOT NULL AND @Campaigns <> ''
		SET @Sql += 'AND UC.[CampaignId] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','','')) '

	SET @Sql += ' 
		SET NOCOUNT OFF'

	-- select @Sql
	EXEC (@Sql)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTE SP RETORNA UN RESUMEN DE LA BUSQUEDA DE INTERACCIONES DE ALLEGRO PARA INDETIFICAR LOS YA TRANSCRITOS O INPROCESS SEGUN LOS FILTROS RECIBIDOS 
-- FUNCIONA IGUAL QUE LA FUNCION EN KNEX
CREATE PROCEDURE [dbo].[GetAllegroBillingInteractions] (
   	@VCC VARCHAR(50) = NULL,
	@IP VARCHAR(50), 
	@Type INT,
	@SecondsPerRequest DECIMAL(18, 5),
   	@StartDate DATETIME = '1980-01-01T00:00:00',
    @EndDate DATETIME = '2050-01-01T23:59:59',
	@Campaigns VARCHAR(MAX) = '',
 	@Agents VARCHAR(MAX) = '',
   	@Origins VARCHAR(100) = NULL,
	@StartHour VARCHAR(50) = NULL,
	@EndHour VARCHAR(50) = NULL,
	@MinDuration INT = 0,
    @MaxDuration INT = 100000,
    @Results VARCHAR(MAX) = NULL,
	@Formats VARCHAR(50) = NULL
)
AS 
BEGIN 
	-- SETTEAMOS VALORES MINIMOS PARA LAS HORAS DE INICIO Y FIN
	IF (@StartHour IS NOT NULL AND @StartHour = '') SET @StartHour = '00:00:00'
	IF (@EndHour IS NOT NULL AND @EndHour = '') SET @EndHour = '23:59:59'

	-- DECLARAMOASUN CADENA PARA EL QUERY FINAL
	DECLARE @SQL_SELECT_0 VARCHAR(MAX) = ''
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''
	
	-- DECLARAMOS VARIABLES PARA NOS NOMBRES DE LAS TABLAS
	DECLARE @INTERACTION_DETAIL_TABLE NVARCHAR(100)
	DECLARE @INTERACTION_ACTOR_DETAIL_TABLE NVARCHAR(100)
	DECLARE @INTERACTION_MULTIMEDIA_PARTS_TABLE NVARCHAR(100)
	DECLARE @CAMPAIGN_GOAL_RESULT_MANAGEMENT_TABLE NVARCHAR(100)

	-- SETTEAMOS LOS VALORES DE DURACION MINIMA Y MAXIMA A UNO VALIDO
	IF @MinDuration = @MaxDuration AND @MaxDuration = 0 SET @MaxDuration = 100000
	IF @MinDuration IS NULL SET @MinDuration = 0
	IF @MaxDuration IS NULL SET @MaxDuration = 100000

	-- EL PARAMETRO TYPE DEFINE SI LA BD ALLEGRO SE ENCUENTRA LOCAL O REMOTO
	IF (@Type = 0)  
	BEGIN  
		SET @INTERACTION_DETAIL_TABLE = '[' + @Ip + ']' + '.HistoricalData.dbo.[InteractionDetail]'
		SET @INTERACTION_ACTOR_DETAIL_TABLE = '[' + @Ip + ']' + '.HistoricalData.dbo.[InteractionActorDetail]'
		SET @INTERACTION_MULTIMEDIA_PARTS_TABLE = '[' + @Ip + ']' + '.HistoricalData.dbo.[InteractionMultimediaParts]'
		SET @CAMPAIGN_GOAL_RESULT_MANAGEMENT_TABLE = '[' + @Ip + ']' + '.mmprodat.dbo.[CampaignGoalResultManagement]'
	END
	ELSE IF(@Type = 1) 
	BEGIN
		SET @INTERACTION_DETAIL_TABLE = 'HistoricalData.dbo.[InteractionDetail]';
		SET @INTERACTION_ACTOR_DETAIL_TABLE = 'HistoricalData.dbo.[InteractionActorDetail]'
		SET @INTERACTION_MULTIMEDIA_PARTS_TABLE = 'HistoricalData.dbo.[InteractionMultimediaParts]';
		SET @CAMPAIGN_GOAL_RESULT_MANAGEMENT_TABLE = 'mmprodat.dbo.[CampaignGoalResultManagement]';
	END  

	-- ARMAMOS EL QUERY PARA HACER LA CONSULTA
	SET @SQL_SELECT_1 += '
		SELECT '

	SET @SQL_SELECT_1 += '
				ID.[Id] + ''_'' + CONVERT(VARCHAR(10), IMP.[Section]) + ''_'' + CONVERT(VARCHAR(10), IMP.Part)			''ID'',
				ID.[Id]																									''InteractionId'',
				A.[IsProcess],
				A.[IsCancel],
				A.[IsTranscribed],
				A.[TotalPhrases],
				ID.[AttentionTime],
				RN = ROW_NUMBER() OVER(PARTITION BY (ID.[Id] + ''_'' + CONVERT(VARCHAR(10), IMP.[Section]) + ''_'' + CONVERT(VARCHAR(10), IMP.Part)) ORDER BY ID.[Id])
			'
				
	-- ARMAMOS LA PARTE DE TABLAS 
	SET @SQL_FROM += '
			FROM ' + @INTERACTION_MULTIMEDIA_PARTS_TABLE + ' IMP (NOLOCK)
			LEFT JOIN ' + @INTERACTION_DETAIL_TABLE + ' ID (NOLOCK) 
				ON ID.[Id] = IMP.[Id]
			LEFT JOIN ' + @INTERACTION_ACTOR_DETAIL_TABLE + ' IAD (NOLOCK) 
				ON IAD.[Id] = ID.[id]
			LEFT JOIN (
				SELECT VCC, CampaignID, [Order], [Code], 
								[Description], Category1, Category2, Category3 
					FROM ' + @CAMPAIGN_GOAL_RESULT_MANAGEMENT_TABLE + ' (NOLOCK)
					WHERE VCC = ''' + @VCC + '''
			) CGRM  
				ON CGRM.VCC = ID.VirtualCC 
					AND CGRM.CampaignId = ID.Campaign 
					AND CGRM.[Description] = ID.ManagementResult
			LEFT JOIN [dbo].[Audio] A (NOLOCK) 
				ON A.[ID] = ID.[Id]
					AND A.[Section] = IMP.[Section]
					AND A.[Part] = IMP.[Part] 
			'

	-- ARMAMOS LA PARTE DE FILTROS
	SET @SQL_FILTER += '
			WHERE ID.[VirtualCC] = ''' + @VCC + '''
				AND ID.[Id] IS NOT NULL 
				AND ID.[EndDate] BETWEEN ''' + (SELECT CONVERT(VARCHAR(21), @StartDate, 127)) + ''' AND ''' + (SELECT CONVERT(VARCHAR(21), @EndDate, 127)) + ''' 
				AND ID.[AttentionTime] BETWEEN ' + (SELECT CONVERT(VARCHAR(10), @MinDuration)) + ' AND ' + (SELECT CONVERT(VARCHAR(10), @MaxDuration)) + ' 
				'
	
	-- CONTINUAMOS CON LOS DEMAS FILTROS RECIBIDOS
	IF (@Campaigns IS NOT NULL AND @Campaigns != '') SET @SQL_FILTER += 'AND ID.[Campaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','','')) '
	IF (@Agents IS NOT NULL AND @Agents != '') SET @SQL_FILTER += 'AND IAD.[Actor] IN (SELECT Value FROM [dbo].[split](''' + @Agents + ''','','')) '
	IF (@Origins IS NOT NULL AND @Origins != '') SET @SQL_FILTER += 'AND ID.[Direction] IN (SELECT Value FROM [dbo].[split](''' + @Origins + ''','','')) '
	IF (@Results IS NOT NULL AND @Results != '') 
		/*SET @SQL_FILTER += 'AND (
								(ID.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @Results + ''','','')))
								OR 
								() 
							) '*/
		SET @SQL_FILTER += 'AND (
									(CONCAT(ID.Campaign, ''_'', ISNULL(CONVERT(VARCHAR(4), CGRM.[Code]), '''')) IN (SELECT Value FROM [dbo].[split](''' + @Results + ''','','')))
									OR
									(ID.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @Results + ''','',''))) 
								) '
	IF (@Formats IS NOT NULL AND @Formats != '') SET @SQL_FILTER += 'AND IMP.[Format] IN (SELECT Value FROM [dbo].[split](''' + @Formats + ''','','')) ';
	
	-- PARA EL CASO EN QUE HORA DE INICIO Y FIN NO SEAN NULL
	IF @StartHour IS NOT NULL AND @EndHour IS NOT NULL
	BEGIN
		IF (CAST(@StartHour AS TIME) <= CAST(@EndHour AS TIME)) 
			SET @SQL_FILTER += 'AND CAST(ID.[EndDate] AS TIME) BETWEEN ''' + @StartHour + ''' AND ''' + @EndHour + ''' '
		IF (CAST(@StartHour AS TIME) > CAST(@EndHour AS TIME)) 
			SET @SQL_FILTER += 'AND CAST(ID.[EndDate] AS TIME) NOT BETWEEN ''' + @EndHour + ''' AND ''' + @StartHour + ''' '
	END
	-- PARA EL CASO EN QUE HORA DE INICIO NO SEA NULL Y FIN SEA NULL
	ELSE IF @StartHour IS NOT NULL AND @EndHour IS NULL
	BEGIN
		SET @SQL_FILTER += 'AND CAST(ID.[EndDate] AS TIME) > ''' + @StartHour + ''' '
	END
	-- PARA EL CASO EN QUE HORA DE INICIO SEA NULL Y FIN NO SEA NULL
	ELSE IF @StartHour IS NULL AND @EndHour IS NOT NULL
	BEGIN
		SET @SQL_FILTER += 'AND CAST(ID.[EndDate] AS TIME) < ''' + @EndHour + ''' '
	END
	
	-- UNIMOS LAS PARTES DEL QUERY
	SET @SQL_SELECT_1 += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA

	-- VERIFICAMOS SI EXISTE LA TABLA TEMPORAL #TMP_HD_INTERACTIONS
	IF OBJECT_ID('TEMPDB..#TMP_HD_BILLING_INTERACTIONS') IS NULL
	BEGIN
		-- CREAMOS UNA TABLA TEMPORAL PARA INSERTAR LOS REGISTROS DE LA HDI6
		CREATE TABLE #TMP_HD_BILLING_INTERACTIONS
		(
			[ID] VARCHAR(150),
			[InteractionId] VARCHAR(100),
			[IsProcess] BIT,
			[IsCancel] BIT,
			[IsTranscribed] BIT,
			[TotalPhrases] DECIMAL(18, 5),
			[AttentionTime] DECIMAL(18, 5),
			RN INT
		)
	END

	-- EJECUTAMOS EL QUERY Y DEPOSITAMOS EL RESULTADO EN UNA TABLA TEMPORAL
	INSERT INTO #TMP_HD_BILLING_INTERACTIONS
		-- EJECUTAMOS LA CONSULTA
		EXEC (@SQL_SELECT_1)

	-- select * from #TMP_HD_BILLING_INTERACTIONS
		
	-- HACEMOS EL SELECT A LA TABLA TEMPORAL
	SELECT SUM(CASE WHEN A.[IsTranscribed] = 1 THEN 1 ELSE 0 END)									'alreadyTranscribedAudio',
			SUM(CASE WHEN A.[IsTranscribed] = 1 THEN A.[AttentionTime] ELSE 0 END)					'alreadyTranscribedDurationTime',
			SUM(CASE WHEN (A.[IsProcess] = 1 
							AND A.[IsTranscribed] IS NULL 
							AND A.[IsCancel] IS NULL) 
						THEN 1 ELSE 0 END)															'inProcessAudio',
			SUM(CASE WHEN (A.[IsProcess] = 1 
							AND A.[IsTranscribed] IS NULL 
							AND A.[IsCancel] IS NULL) 
						THEN A.[AttentionTime] ELSE 0 END)											'inProcessDurationTime',
			SUM(CASE WHEN (A.[IsProcess] IS NULL 
							OR (A.[IsProcess] = 1 
								AND A.[IsCancel] = 1)) 
						THEN 1 ELSE 0 END)															'notTranscribedAudio',
			SUM(CASE WHEN (A.[IsProcess] IS NULL 
							OR (A.[IsProcess] = 1 
								AND A.[IsCancel] = 1)) 
						THEN A.[AttentionTime] ELSE 0 END)											'notTranscribedDurationTime',
			SUM(CASE WHEN A.[IsTranscribed] = 1 
							AND A.[TotalPhrases] IS NOT NULL
						THEN A.[TotalPhrases] ELSE 0 END)											'totalPhrases',
			SUM(CASE WHEN (A.[IsProcess] IS NULL 
							OR (A.[IsProcess] = 1 
								AND A.[IsCancel] = 1)
							OR (A.[TotalPhrases] IS NULL)) 
						THEN (A.[AttentionTime] / @SecondsPerRequest) ELSE 0 END)					'totalRawPhrases',
			SUM(A.[AttentionTime] / @SecondsPerRequest)												'averageModelRequest' 
		FROM #TMP_HD_BILLING_INTERACTIONS A
		WHERE A.RN = 1

	-- ELIMINAMOS LA TABLA TEMPORARL
	DROP TABLE #TMP_HD_BILLING_INTERACTIONS

	-- EJECUTAMOS LA CONSULTA
	-- EXEC (@SQL_SELECT_1)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTE SCRIPT TIENE POR FINALIDAD RETORNAR UN RESUMEN DE LOS AUDIOS EN EL PROCESSID INPUT QUE YA EXISTEN EN UN PROCESSID ANTERIOR
-- DEBIDO A QUE PLANES CREAN EL PROCESS Y EL EVALUATION_PLAN EN DIFERENTES ENDPOINTS, DEJARA DE SER NECESARIO SI SE INTEGRA EL 
-- GUARDADO DE PLAN DE EVALUACION
CREATE PROCEDURE [dbo].[GetAllegroBillingInteractionsForEvaluationPlan] (
	@VCC VARCHAR(50) = NULL,
	@ProcessId VARCHAR(50) = NULL,
	@SecondsPerRequest DECIMAL(18, 5) = NULL
)
AS 
BEGIN 
	-- CREAMOS UNA VARIEBLE PARA ALMACENAR LA FECHA DE CREACION DEL PROCESS INPUT
	--DECLARE @vcc VARCHAR(50) = 'allegro'
	--DECLARE @ProcessId VARCHAR(50) = '3EU91680-UB6U-3GT5-8T50-73474D13WVB8'
	DECLARE @PROCESS_CREATED_DATE DATETIME = NULL

	-- RECUPERAMOS LA FECHA DE CREACION DEL PROCESS INPUT
	SELECT @PROCESS_CREATED_DATE = P.[CreatedDate] 
		FROM [dbo].[Process] P (NOLOCK) 
		WHERE P.[VCC] = @vcc
			AND P.[Name] = @processId;

	-- CREAMOS LAS CTE QUE USAREMOS PARA RECUPERAR EL HISTORICO DE LOS AUDIOS DEL PROCESS INPUT
	WITH CTE_PROCESS_AUDIO AS (
		-- RECUPERAMOS LOS AUDIOS DEL PROCESS INPUT
		SELECT PA.[AudioId],
				PA.[Section],
				PA.[Part]
			FROM [dbo].[ProcessAudio] PA (NOLOCK) 
			WHERE PA.[VCC] = @vcc
				AND PA.[ProcessId] = @processId
	), CTE_PROCESS_AUDIO_2 AS (
		-- CON LA PRIMERA CTE, RECUPERAMOSEL DETALLE DE LOS AUDIOS Y LA CANTIDAD DE VECES QUE SE REPITE EN PROCESS ANTERIORES
		SELECT PA.[AudioId],
				PA.[Section],
				PA.[Part],
				COUNT(*) 'Times' 
			FROM CTE_PROCESS_AUDIO CPA
			LEFT JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
				ON PA.[AudioId] = CPA.[AudioId]
					AND PA.[Section] = CPA.[Section]
					AND PA.[Part] = CPA.[Part]
			WHERE PA.[VCC] = @vcc
				AND PA.[CreatedDate] < @PROCESS_CREATED_DATE
			GROUP BY PA.[AudioId], PA.[Section], PA.[Part]
	), CTE_PROCESS_AUDIO_3 AS (
		-- ARMAMOS UNA CTE CON TODOS LOS AUDIOS QUE SE REPITEN Y EL DETALLE DE CADA AUDIO
		SELECT CPA2.[AudioId],
				CPA2.[Section],
				CPA2.[Part],
				A.[DurationTime],
				A.[DurationLinguisticTime],
				A.[IsProcess],
				A.[IsCancel],
				A.[IsTranscribed],
				A.[TotalPhrases]
			FROM CTE_PROCESS_AUDIO_2 CPA2
			LEFT JOIN [dbo].[Audio] A
				ON A.[ID] = CPA2.[AudioId]
					AND A.[Section] = CPA2.[Section]
					AND A.[Part] = CPA2.[Part]
			WHERE A.[VCC] = @vcc
				AND CPA2.[Times] > 0
	)

	SELECT SUM(CASE WHEN PA3.[IsTranscribed] = 1 THEN 1 ELSE 0 END)							'alreadyTranscribedAudio', 
			SUM(CASE WHEN PA3.[IsTranscribed] = 1 THEN PA3.[DurationTime] ELSE 0 END)		'alreadyTranscribedDurationTime',
			SUM(CASE WHEN PA3.[IsTranscribed] IS NULL 
							AND PA3.[IsCancel] IS NULL 
						THEN 1 ELSE 0 END)													'inProcessAudio', 
			SUM(CASE WHEN PA3.[IsTranscribed] IS NULL 
							AND PA3.[IsCancel] IS NULL 
						THEN PA3.[DurationTime] ELSE 0 END)									'inProcessDurationTime', 
			SUM(CASE WHEN PA3.[IsCancel] = 1 THEN 1 ELSE 0 END)								'canceledAudio', 
			SUM(CASE WHEN PA3.[IsCancel] = 1 THEN PA3.[DurationTime] ELSE 0 END)			'canceledDurationTime',
			SUM(CASE WHEN PA3.[IsTranscribed] = 1 
							AND PA3.[TotalPhrases] IS NOT NULL
						THEN PA3.[TotalPhrases] ELSE 0 END)									'totalPhrases',
			SUM(CASE WHEN PA3.[TotalPhrases] IS NULL 
						THEN (PA3.[DurationTime] / @SecondsPerRequest) ELSE 0 END)			'totalRawPhrases' 
		FROM CTE_PROCESS_AUDIO_3 PA3 
	--SELECT * FROM CTE_PROCESS_AUDIO_4
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAllegroCampaignMembers]
(
	@IP				VARCHAR(50),
	@VCC			VARCHAR(50),
	@Campaigns		VARCHAR(MAX),
	@Type			BIT
)
AS
BEGIN
	-- ESTE SP RETORNA LA LISTA DE SUPERVISORES DE UN VCC DE TIPO ALLEGRO/OCC CON SUS WORKGROUPS DE AGENTES
	
	-- DECLARAMOS VARIABLES PARA ARMAR LA CONSULTA
	DECLARE @SQL VARCHAR(MAX) = NULL
	DECLARE @TABLE_MMPRODAT VARCHAR(100) = NULL

	-- EL TYPE 0 INDICA QUE LAS TABLAS SE ENCUENTRAN EN UN SERVIDOR EXTERNO
	IF @Type = 0
	BEGIN  
		SET @TABLE_MMPRODAT = '[' + @Ip + ']' + '.[MMProDat].[dbo].'
	END
	ELSE IF(@Type = 1) 
	BEGIN
		SET @TABLE_MMPRODAT = '[MMProDat].[dbo].'
	END
	
	-- ARMAMOS LA CONSULTA PRINCIPAL
	/*
	SET @SQL = '
		SELECT DISTINCT UC.[VCC]					''vcc'',
						UC.[CampaignId]				''campaign'',
						UC.[Id]						''userId''
			FROM ' + @TABLE_MMPRODAT + '[UserCampaign] UC (NOLOCK)
			WHERE UC.[VCC] = ''' + @VCC + ''' 
				AND UC.[CampaigniD] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','','')) '
	*/

	SELECT DISTINCT UC.[VCC]									'vcc',
					UC.[CampaignId]								'campaignId',
					UC.[UserId]									'userId',
					IIF(
						(
							SWG.[SupervisorId] IS NOT NULL
							OR 
							U.[UserGroup] = 'supervisor'
						), 1, 0)	'isSupervisor',
					IIF(A.[UserId] IS NOT NULL, 1, 0)			'isAgent',
					SWG.[WorkgroupId]							'supervisorWorkgroup',
					A.[WorkgroupId]								'agentWorkgroup'
		FROM [MMProDat].[dbo].[UserCampaign] UC (NOLOCK)
		LEFT JOIN [MMProDat].[dbo].[Agent] A (NOLOCK)
			ON A.[VCC] = UC.[VCC]
				AND A.[UserId] = UC.[UserId]
		LEFT JOIN [MMProDat].[dbo].[SupervisorWorkgroup] SWG (NOLOCK)
			ON SWG.[VCC] = UC.[VCC]
				AND SWG.[CampaignId] = UC.[CampaignId]
				AND SWG.[SupervisorId] = UC.[UserId]
		LEFT JOIN [MMProDat].[dbo].[User] U (NOLOCK)
			ON U.[VCC] = UC.[VCC]
				AND U.[Id] = UC.[UserId]
 		WHERE UC.[VCC] = @VCC 
			AND UC.[CampaignId] IN (SELECT Value FROM [dbo].[split](@Campaigns,','))

	-- EJECUTAMOS LA CONSULTA
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetAllegroCampaigns] '38.127.113.216', 'icperu', 1
CREATE PROCEDURE [dbo].[GetAllegroCampaigns]
(
@Ip Varchar(50), @VCC Varchar(50), @Type Int
)
AS
BEGIN

	DECLARE @Sql NVARCHAR(MAX);

	IF(@Type = 0)  
	BEGIN  
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[Id]							''id'',
			[Description]					''description''
		FROM  [' + @Ip + ']' + '.mmprodat.dbo.[Campaign] WITH (NOLOCK)
		WHERE [VCC] = ''' + @VCC + ''''
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE IF(@Type = 1) 
	BEGIN
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[Id]							''id'',
			[Description]					''description''
		FROM mmprodat.dbo.[Campaign] (NOLOCK)
		WHERE [VCC] = ''' + @VCC + ''''
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE  
	BEGIN
		SET @Sql = 'SELECT 1 AS ''result'''  
	END

	EXEC (@Sql)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAllegroInteractions] (
	@Ip						VARCHAR(50),
	@VCC					VARCHAR(50),
	@StartDate				DATETIME = NULL,
	@EndDate				DATETIME = NULL,
	@StartHour				VARCHAR(50),
	@EndHour				VARCHAR(50),
	@Campaign				VARCHAR(MAX),
	@Agent					VARCHAR(MAX),
	@ResultCode				VARCHAR(MAX),
	@Type					INT,
	@Origin					VARCHAR(MAX),
	@MinDuration			INT,
	@MaxDuration			INT,
	@Format					VARCHAR(50) = NULL, 
	@InteractionIds         VARCHAR(MAX) = NULL, 
	@Evaluated              VARCHAR(MAX) = NULL,
	-- @UniqueIds				BIT = 0,							-- ESTE FLAG TIENE COMO PROPOSITO RETORNAR LA LISTA CONSOLIDAD POR IDS O EL TOTAL DEL LISTADO (AUDIOS/INTERACTIONS)
	@ResultType				VARCHAR(50) = NULL,						-- VARIABLE PARA DETERMINAR SI LA BUSQUEDA SERA SOBRE SOBRE DETALLES O ID (DETAILED, SUMMARIZED)
	@ContactAddress			VARCHAR(300) = NULL,
	@LimitIntByAgent		INT = 0
) AS
BEGIN
	-- SETTEAMOS UN VALOR POR DEFECTO AL FORMAT
	IF @Format IS NULL OR @Format = '' SET @Format = 'mp3,wav,ogg'

	-- SETTEAMOS VALORES MINIMOS PARA LAS HORAS DE INICIO Y FIN
	IF (@StartHour IS NOT NULL AND @StartHour = '') SET @StartHour = '00:00:00'
	IF (@EndHour IS NOT NULL AND @EndHour = '') SET @EndHour = '23:59:59'

	-- DECLARAMOS UNA VARIABLE PARA IDENTIFICAR SI EL SP ES LLAMADO DEL SISTEMA O POR OTRO SP
	DECLARE @IS_NESTED BIT = 0

	-- CREAMOS LAS VARIABLES QUE HARAN REFERENCIA A LAS TABLAS QUE PUEDEN SER EXTERNAS
	DECLARE @TABLE_HISTORICAL_DATA VARCHAR(50) = ''
	DECLARE @TABLE_MMPRODAT VARCHAR(50) = ''

	-- DECLARAMOS CADENAS PARA EL QUERY FINAL
	DECLARE @SQL_TMP VARCHAR(MAX) = ''
	DECLARE @SQL_CTE VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_SELECT VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FROM_FIXED VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FROM_OPTIONAL VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_FIXED VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_OPTIONAL VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_EXCLUDING VARCHAR(MAX) = ''
	
	DECLARE @Sql NVARCHAR(MAX)

	-- EL TYPE 0 INDICA QUE LAS TABLAS SE ENCUENTRAN EN UN SERVIDOR EXTERNO
	IF(@Type = 0)  
	BEGIN  
		SET @TABLE_HISTORICAL_DATA = '[' + @Ip + ']' + '.[HistoricalData].[dbo].'
		SET @TABLE_MMPRODAT = '[' + @Ip + ']' + '.[MMProDat].[dbo].'
	END
	ELSE IF(@Type = 1) 
	BEGIN
		SET @TABLE_HISTORICAL_DATA = '[HistoricalData].[dbo].'
		SET @TABLE_MMPRODAT = '[MMProDat].[dbo].'
	END
	
	IF(@Type = 0 OR @Type = 1)  
	BEGIN  
		-- VERIFICAMOS SI EXISTE LA TABLA TEMPORAL #TMP_HD_INTERACTIONS
		IF OBJECT_ID('TEMPDB..#TMP_HD_INTERACTIONS') IS NULL
		BEGIN
			-- CREAMOS UNA TABLA TEMPORAL PARA INSERTAR LOS REGISTROS DE LA HDI6
			CREATE TABLE #TMP_HD_INTERACTIONS
			(
				[ID] VARCHAR(150),
				[InteractionId] VARCHAR(100),
				[Section] INT,
				[Part] INT,
				[Channel] VARCHAR(50),
				[Number] VARCHAR(100),
				[Gender] VARCHAR(50),
				[SilenceTime] INT,
				[DurationTime] INT,
				[AttentionTime] INT,
				[Category1] VARCHAR(100),
				[Category2] VARCHAR(100),
				[Category3] VARCHAR(100),
				[Description] VARCHAR(150),
				[DispositionCode] INT,
				[Campaign] VARCHAR(150),
				[Agent] VARCHAR(100),
				[AgentName] VARCHAR(100),
				[Type] VARCHAR(100),
				[StartDate] VARCHAR(100),
				[EndDate] VARCHAR(100),
				[Format] VARCHAR(50),
				[ContactAddress] VARCHAR(300),
				[ContactId]  VARCHAR(50),
				[ContactName] VARCHAR(300),
				[RelativeFileName] VARCHAR(1000),
				[RNA] INT
			)

			-- REAFIRMAMOS QUE EL SP NO ESTA ANIDADO EN OTRO
			SET @IS_NESTED = 0
		END
		ELSE
		BEGIN
			-- SI LA TABLA TEMPORAL YA EXISTIA, EL SP ESTA ANIDADO EN OTRO (P.E.: [dbo].[GetAsyncBillingInteractions])
			SET @IS_NESTED = 1
		END 

		-- INICIAMOS ARMANDO LOS FILTROS PARA LA TABLA CTE PRINCIPAL
		SET @SQL_CTE_FILTERS_FIXED = '
				WHERE ID.[VirtualCC] = ''' + @VCC + '''
					AND ID.[EndDate] BETWEEN ''' + (SELECT CONVERT(VARCHAR(21), @StartDate, 127)) + ''' 
					AND ''' + (SELECT CONVERT(VARCHAR(21), @EndDate, 127)) + ''' '

		-- AGREGAMOS EL SCRIPT PARA LOS FILREOS DE HORA MAXIMA Y HORA MINIMA LAS CUALES TIENEN UN COMPORTAMIENTO DIFERENTES SEGUN CUAL ES MAYOR
		IF (CAST(@StartHour AS TIME) <= CAST(@EndHour AS TIME)) 
			SET @SQL_CTE_FILTERS_OPTIONAL += '
					AND CAST(ID.[EndDate] AS TIME) BETWEEN ''' + @StartHour + ''' AND ''' + @EndHour + ''' '
		IF (CAST(@StartHour AS TIME) > CAST(@EndHour AS TIME)) 
			SET @SQL_CTE_FILTERS_OPTIONAL += '
					AND CAST(ID.[EndDate] AS TIME) NOT BETWEEN ''' + @EndHour + ''' AND ''' + @StartHour + ''' '

		-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE CAMPAIGN
		IF (@Campaign IS NOT NULL AND @Campaign <> '') 
		BEGIN
			-- DEBEMOS PASAR LA LISTA DE CAMPAIGNS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
			SET @SQL_TMP += '
			SELECT * 
				INTO #FILTER_CAMPAIGNS
				FROM (
					SELECT Value FROM [dbo].[split](''' + @Campaign + ''','','')
				) A '

			-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
			SET @SQL_CTE_FROM_OPTIONAL += '  
				INNER JOIN #FILTER_CAMPAIGNS FC 
					ON FC.[Value] = ID.[Campaign] ' 
		END

		-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE AGENT
		IF (@Agent IS NOT NULL AND @Agent <> '') 
		BEGIN
			-- DEBEMOS PASAR LA LISTA DE AGENTS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
			SET @SQL_TMP += '
			SELECT * 
				INTO #FILTER_AGENTS
				FROM (
					SELECT Value FROM [dbo].[split](''' + @Agent + ''','','')
				) A '

			-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
			SET @SQL_CTE_FROM_OPTIONAL += '  
				INNER JOIN #FILTER_AGENTS FA 
					ON FA.[Value] = ISNULL(ID.[LastAgent], ID.[FirstAgent]) '
		END

		-- AGREGAMOS EL SCRPT PARA EL FILTRO ORIGEN
		IF (@Origin IS NOT NULL AND @Origin <> '') 
			SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND ID.[Direction] IN (SELECT Value FROM [dbo].[split](''' + @Origin + ''','','')) '

		-- AGREGAMOS EL SCRIPT PARA LOS FILTROS DE DURACION MAXIMA Y HORA MINIMA LAS CUALES TIENEN UN COMPORTAMIENTO DIFERENTES SEGUN CUAL ES MAYOR
		IF (@MinDuration IS NOT NULL) 
			SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND ID.[AttentionTime] >= ' + (SELECT CONVERT(varchar(10), @MinDuration)) + ' '
		IF (@MaxDuration IS NOT NULL) 
			SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND ID.[AttentionTime] <= ' + (SELECT CONVERT(varchar(10), @MaxDuration)) + ' '

		-- AGREGAMOS EL SCRIPT PARA EL FILTRO CONTACT
		IF (@ContactAddress IS NOT NULL)
			SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND ID.[ContactId] LIKE ''%' + @ContactAddress + '%'' '

		-- AGREGAMOS EL SCRIPT PARA INTERACTION_ID
		IF @InteractionIds IS NOT NULL AND @InteractionIds <> ''
		BEGIN
			SET @SQL_TMP = ''
			SET @SQL_CTE_FROM_OPTIONAL = ''
			SET @SQL_CTE_FILTERS_OPTIONAL = ''
			
			-- DEBEMOS PASAR LA LISTA DE CAMPAIGNS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
			SET @SQL_TMP += '
			SELECT * 
				INTO #FILTER_INTERACTIONS_IDS
				FROM (
					SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')
				) A '
			-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
			SET @SQL_CTE_FROM_OPTIONAL += '  
				INNER JOIN #FILTER_INTERACTIONS_IDS FID 
					ON FID.[Value] = ID.[Id] ' 
			-- SET @SQL_CTE_FILTERS_EXCLUDING = ' AND ID.[Id] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) '
		END 


		-- ARMAMOS EL QUERY PARA EL CONTENIDO DE LA CTE
		SET @SQL_CTE_SELECT = '
			;WITH CTE_INTERACTION AS (
				SELECT ID.[VirtualCC]																						''VCC'',
						ID.[Id]																								''InteractionId'',
						''CALL''																							''Channel'',
						''''																								''Gender'',
						ID.[HoldTime]																						''SilenceTime'',
						ID.[AttentionTime]																					''DurationTime'', -- ESTO SE DEBE CORREGIR 
						ID.[AttentionTime]																					''AttentionTime'', -- SE AGREGO CAMPO PARA AMBOS EXISTAN
						ID.[Campaign]																						''Campaign'',
						ISNULL(ID.[LastAgent], ID.[FirstAgent])																''Agent'',
						ID.[ManagementResult]																				''ManagementResult'',
						ID.[Direction]																						''Type'',
						CONVERT(VARCHAR, ID.[StartDate], 120)																''StartDate'',
						CONVERT(VARCHAR, ID.[EndDate], 120)																	''EndDate'', 
						ID.[ContactName]																					''ContactName'',
						ID.[Direction]																						''Direction''
					FROM [HistoricalData].[dbo].[InteractionDetail] ID (NOLOCK) '
					+ @SQL_CTE_FROM_OPTIONAL
					+ @SQL_CTE_FILTERS_FIXED
					+ @SQL_CTE_FILTERS_OPTIONAL
					+ @SQL_CTE_FILTERS_EXCLUDING + '
			)
			,CTE_WITH_CID AS (
				SELECT CTE_I.*,
						CASE WHEN CTE_I.[Direction] = ''E'' THEN CID.[CallerId] ELSE CID.[CalledId] END						''Number'',
						CASE WHEN CTE_I.[Direction] = ''E'' THEN CID.[CallerId] ELSE CID.[CalledId] END						''ContactAddress'',
						CASE WHEN CTE_I.[Direction] = ''E'' THEN CID.[CallerId] ELSE CID.[CalledId] END						''ContactId'',
						CASE WHEN CID.[SourceHangup] = 0 THEN ''Client'' ELSE ''Agent'' END      							''Ender''
					FROM CTE_INTERACTION CTE_I (NOLOCK)
					INNER JOIN [HistoricalData].[dbo].[CallInteractionDetail] CID (NOLOCK)        
						ON CTE_I.[InteractionId] = CID.[Id]   
			)
			,CTE_WITH_IMP AS (
				SELECT CTE_CID.*,
						CTE_CID.[InteractionId] + ''_'' + CONVERT(VARCHAR(10), IMP.[Section]) + ''_'' + CONVERT(VARCHAR(10), IMP.[Part])		''ID'',
						IMP.[Section]																						''Section'',
						IMP.[Part]																							''Part'',     
						IMP.[Format]																						''Format'', 
						IMP.[RelativePath] + ''/'' + IMP.[FileName]															''RelativeFileName''
					FROM CTE_WITH_CID CTE_CID (NOLOCK)
					INNER JOIN [HistoricalData].[dbo].[InteractionMultimediaParts] IMP (NOLOCK)      
						ON IMP.[Id] = CTE_CID.[InteractionId]
					WHERE IMP.[Format] IN (SELECT Value FROM [dbo].[split](''' + @Format + ''','',''))
			)
			,CTE_WITH_IAD AS (
				SELECT CTE_IMP.*,
						IAD.[ActorName]											 ''agentName'' 
					FROM CTE_WITH_IMP CTE_IMP
					LEFT JOIN [HistoricalData].[dbo].[InteractionActorDetail] IAD (NOLOCK)       
						ON IAD.[VirtualCC] = CTE_IMP.[VCC]
							AND IAD.[Id] = CTE_IMP.[InteractionId]        
							AND IAD.[Section] = CTE_IMP.[Section]        
							AND IAD.[Part] = CTE_IMP.[Part]
			)
			,CTE_WITH_CGRM AS (	
				SELECT CTE_C.*,
						ISNULL(CGRM.[Category1], '''')								''Category1'',       
						ISNULL(CGRM.[Category2], '''')								''Category2'',       
						ISNULL(CGRM.[Category3], '''')								''Category3'',       
						ISNULL(CGRM.[Description], '''')							''Description'',       
						ISNULL(CONVERT(VARCHAR(4), CGRM.[Code]), '''')				''DispositionCode'', 
						RNA = ROW_NUMBER() OVER(PARTITION BY (CTE_C.[Id] + ''_'' + CONVERT(VARCHAR(10), CTE_C.[Section]) + ''_'' + CONVERT(VARCHAR(10), CTE_C.Part)) ORDER BY CTE_C.[Id]) -- ROW NUMBER AUDIO
				FROM CTE_WITH_IAD CTE_C 
				LEFT JOIN (       
					SELECT VCC, CampaignID, [Order], [Code],            
							[Description], Category1, Category2, Category3         
						FROM [MMProDat].[dbo].[CampaignGoalResultManagement] (NOLOCK)
						WHERE VCC = ''' + @VCC + '''
				) CGRM         
					ON CGRM.VCC = CTE_C.[VCC]         
						AND CGRM.CampaignId = CTE_C.[Campaign]
						AND CGRM.[Description] = CTE_C.[ManagementResult]'

		-- AGREGAMOS EL SCRIPT PARA RESULTADO DE GESTION
		IF (@ResultCode IS NOT NULL AND @ResultCode <> '') 
		BEGIN
			-- DEBEMOS PASAR LA LISTA DE CAMPAIGNS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
			SET @SQL_TMP += '
			SELECT * 
				INTO #FILTER_RESULT_CODES
				FROM (
					SELECT Value FROM [dbo].[split](''' + @ResultCode + ''','','')
				) A '

			-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
			SET @SQL_CTE_SELECT += '  
				INNER JOIN #FILTER_RESULT_CODES FRC 
					ON (
						FRC.[Value] = CONCAT(CTE_C.[Campaign], ''_'', ISNULL(CONVERT(VARCHAR(4), CGRM.[Code]), ''''))
						OR
						FRC.[Value] = CTE_C.[ManagementResult] 
					) '
		END

		SET @SQL_CTE_SELECT += '
			) -- SELECT * FROM CTE_WITH_CGRM'

		-- ARMAMOS EL QUERY PRINCIPAL
		SET @SQL_CTE = @SQL_TMP + @SQL_CTE_SELECT

		-- SEGUN SI EL SP ES ANIDADO O NO INSERTAMOS EN LA TEMPORAL EXISTENTE O LA CREAMOS
		IF @IS_NESTED = 1 
			SET @SQL_CTE += '
			INSERT INTO #TMP_HD_INTERACTIONS'	
		SET @SQL_CTE += '
			SELECT CTE_C.[ID],
					CTE_C.[InteractionId],
					CTE_C.[Section],
					CTE_C.[Part],
					CTE_C.[Channel],
					CTE_C.[Number],
					CTE_C.[Gender],
					CTE_C.[SilenceTime],
					CTE_C.[DurationTime],
					CTE_C.[AttentionTime],
					CTE_C.[Category1],
					CTE_C.[Category2],
					CTE_C.[Category3],
					CTE_C.[Description],
					CTE_C.[DispositionCode],
					CTE_C.[Campaign],
					CTE_C.[Agent],
					CTE_C.[AgentName],
					CTE_C.[Type],
					CTE_C.[StartDate],
					CTE_C.[EndDate],
					CTE_C.[Format],
					CTE_C.[ContactAddress],
					CTE_C.[ContactId],
					CTE_C.[ContactName],
					CTE_C.[RelativeFileName],
					CTE_C.[Ender],
					CTE_C.[RNA] '
		IF @IS_NESTED = 0 
			SET @SQL_CTE += '
				INTO #TMP_HD_INTERACTIONS '
		SET @SQL_CTE += '
				FROM CTE_WITH_CGRM CTE_C
			
			SELECT ROW_NUMBER() OVER (PARTITION BY Agent ORDER BY StartDate) AS RowNumber,
					HDI.*
				INTO #TMP_HD_INTERACTIONS_LIMIT
				FROM #TMP_HD_INTERACTIONS HDI
				WHERE [RNA] = 1

			SELECT DISTINCT [Agent] 
				INTO #AGENTS
				FROM #TMP_HD_INTERACTIONS HDI
				WHERE [RNA] = 1

			DROP TABLE IF EXISTS #FILTER_CAMPAIGNS 
			DROP TABLE IF EXISTS #FILTER_AGENTS
			DROP TABLE IF EXISTS #FILTER_RESULT_CODES
			DROP TABLE IF EXISTS #FILTER_INTERACTIONS_IDS '

		-- EJECUTAMOS EL SQL QUE NOS DEJA LA TEMPORAL #TMP_HD_INTERACTIONS CON REGISTROS
		--PRINT @SQL_CTE
		--EXEC (@SQL_CTE)
		--SELECT * FROM #TMP_HD_INTERACTIONS

		-- VARIABLES PARA TRABAJAR CON LA TABLA TEMPORAL AL FINAL DEL SP
		DECLARE @QUERY_SUMMARY VARCHAR(MAX) = ''
		DECLARE @QUERY_DETAIL VARCHAR(MAX) = ''
		DECLARE @QUERY VARCHAR(MAX) = ''
		DECLARE @QUERY_FILTER VARCHAR(MAX) = ''
		DECLARE @QUERY_JOIN VARCHAR(MAX) = ''
	
		-- FILTRO POR INTERACCIONES EVALUADAS O NO
		IF(@Evaluated IS NOT NULL AND @Evaluated <> '')
		BEGIN 
			DECLARE @EvaluatedTable TABLE (Val BIT)
			INSERT INTO @EvaluatedTable SELECT Value FROM [dbo].[split]( @Evaluated ,',')
			-- SOLAMENTE SI ES SI O NO
			IF((SELECT COUNT(Val) FROM @EvaluatedTable) = 1 )
			BEGIN
				IF((SELECT Val FROM @EvaluatedTable WHERE Val = 1) IS NOT NULL) -- Si  es Si
				BEGIN 
					SET @QUERY_JOIN = ' JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) ON (AEP.[IdInteraction] = HDIL.[InteractionId] AND AEP.[Completed] = 1) '
				END
				ELSE  --Si es No
				BEGIN
					SET @QUERY_JOIN += ' LEFT JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) ON (AEP.[IdInteraction] = HDIL.[InteractionId] AND AEP.[Completed] = 1) '
					SET @QUERY_FILTER = ' AND AEP.[IdInteraction] IS NULL '
				END
			END
		END
		-- FIN FILTRO INTERACCIONES EVALUADAS

		-- SI EL SP NO ESTA ANIDADO, HACEMOS SELECT AL RESULTADO A MOSTRAR
		IF @IS_NESTED = 0
		BEGIN
			-- EN AMBOS CASOS SE REQUIERE EL RESUMEN
			SET @QUERY_SUMMARY = @SQL_CTE + '
				SELECT HDIL.[channel]										AS ''interactionType'',
						COUNT(DISTINCT (HDIL.[InteractionId]))				AS ''interactions'',
						COUNT(DISTINCT (HDIL.[ID]))							AS ''details'',
						SUM(HDIL.[AttentionTime])							AS ''attentionTime''
					FROM #TMP_HD_INTERACTIONS_LIMIT HDIL '
			SET @QUERY +=  @QUERY_SUMMARY + @QUERY_JOIN
			IF @LimitIntByAgent > 0
			BEGIN
				SET @QUERY += ' WHERE HDIL.[RowNumber] <= '+CAST(@LimitIntByAgent AS VARCHAR) + @QUERY_FILTER + ' GROUP BY HDIL.[channel] '
			END
			ELSE
			BEGIN
				SET @QUERY += @QUERY_FILTER + ' GROUP BY HDIL.[channel] '
			END

			-- HACEMOS EL SELECT SEGUN EL TIPO DE BUSQUEDA SETTEADO
			--IF @ResultType = 'SUMMARIZED'
			--BEGIN
			-- END
			-- ELSE 
			IF @ResultType = 'DETAILED'
			BEGIN
				SET @QUERY_DETAIL = '
					SELECT HDIL.* 
						FROM #TMP_HD_INTERACTIONS_LIMIT HDIL ' 
				SET @QUERY += @QUERY_DETAIL + @QUERY_JOIN 
				IF (@LimitIntByAgent > 0)
				BEGIN
					SET @QUERY += ' WHERE HDIL.[RowNumber] <= '+CAST(@LimitIntByAgent AS VARCHAR) + @QUERY_FILTER + ' '
				END
				ELSE
				BEGIN
					SET @QUERY +=  @QUERY_FILTER + ' '
				END
			END

			DECLARE @QUERY_AGENTS VARCHAR(MAX)
			SET @QUERY_AGENTS = ' SELECT Agent as agent FROM #AGENTS'

			SET @QUERY+=  @QUERY_AGENTS

			-- ELIMINAMOS LAS TEMPORALES DE LA CONSULTA PRINCPAL
			SET @SQL_CTE += '
				DROP TABLE IF EXISTS #TMP_HD_INTERACTIONS_LIMIT
				DROP TABLE IF EXISTS #TMP_HD_INTERACTIONS
				DROP TABLE IF EXISTS #AGENTS '

			-- EJECUTAMOS LA CONSULTA
			-- PRINT @QUERY
			EXEC (@QUERY)
    
			-- ELIMINAMOS LA TABLA TEMPORARL
			DROP TABLE #TMP_HD_INTERACTIONS
		END

	END
	ELSE
	BEGIN
		SET @Sql = 'SELECT 1 AS ''result'''	
		EXEC (@Sql)
	END

	

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAllegroInteractionVideos]
(
	@Ip						VARCHAR(50) = NULL,
	@VCC					VARCHAR(50) = NULL,
	@Type					INT = 0,
	@InteractionIds         VARCHAR(250) = NULL -- EN CASO SE CAMBIE LA BUSQUEDA A VARIOS IDS, CAMBIAR A VARCHAR(MAX)
)
AS
BEGIN
	DECLARE @SQL NVARCHAR(MAX)

	-- SETTEAMOS UN VALOR POR DEFECTO AL FORMAT
	DECLARE @VIDEO_FORMATS VARCHAR(25) = 'mp4'

	-- CREAMOS LAS VARIABLES QUE HARAN REFERENCIA A LAS TABLAS QUE PUEDEN SER EXTERNAS
	DECLARE @HISTORICALDATA_TABLE_REF VARCHAR(50) = NULL
    
	-- EL TYPE 0 INDICA QUE LAS TABLAS SE ENCUENTRAN EN UN SERVIDOR EXTERNO
	IF(@Type = 0)  
	BEGIN  
		SET @HISTORICALDATA_TABLE_REF = '[' + @Ip + ']' + '.[HistoricalData].[dbo].'
	END
	ELSE IF(@Type = 1) 
	BEGIN
		SET @HISTORICALDATA_TABLE_REF = '[HistoricalData].[dbo].'
	END

	-- SE ARMA EL QUERY PRINCIPAL PARA LA BUSQUEDA DE INTERACCIONES Y TIEMPO TOTAL
	SET @SQL = '
		SELECT ID.[VirtualCC]														''vcc'', 
				ID.[Id]																''id'',
				IMP.[Section]														''section'',
				IMP.[Part]															''part'',
				IMP.[Format]														''format''
			FROM ' + @HISTORICALDATA_TABLE_REF + '[InteractionDetail] ID (NOLOCK)
			INNER JOIN ' + @HISTORICALDATA_TABLE_REF + '[InteractionMultimediaParts] IMP (NOLOCK)
				ON IMP.[Id] = ID.[Id]
			WHERE ID.[VirtualCC] = ''' + @VCC + '''
				AND ID.[Id] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','',''))
				AND IMP.[Format] IN (SELECT Value FROM [dbo].[split](''' + @VIDEO_FORMATS + ''','','')) '

	-- EJECUTAMOS LA CONSULTA
	EXEC (@SQL)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetAllegroManagementResults] '38.127.113.216', 'inconcert', 0
CREATE PROCEDURE [dbo].[GetAllegroManagementResults]
(
@Ip Varchar(50), @VCC Varchar(50), @Type Int
)
AS
BEGIN

	DECLARE @Sql NVARCHAR(MAX);

	IF(@Type = 0)  
	BEGIN  
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[VCC]							''vcc'',
			[CampaignId]					''campaignId'',
			[Code]							''code'',
			[Description]					''description''
		FROM  [' + @Ip + ']' + '.mmprodat.dbo.[CampaignGoalResultManagement] WITH (NOLOCK)
		WHERE [VCC] = ''' + @VCC + ''''
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE IF(@Type = 1) 
	BEGIN
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[VCC]							''vcc'',
			[CampaignId]					''campaignId'',
			[Code]							''code'',
			[Description]					''description''
		FROM mmprodat.dbo.[CampaignGoalResultManagement] (NOLOCK)
		WHERE [VCC] = ''' + @VCC + ''''
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE  
	BEGIN
		SET @Sql = 'SELECT 1 AS ''result'''  
	END

	EXEC (@Sql)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAllegroRole]
(
	@IP				VARCHAR(50),
	@VCC			VARCHAR(50),
	@RoleId			VARCHAR(50),
	@Type			INT
)
AS
BEGIN
	DECLARE @SQL VARCHAR(MAX)
	
	IF(@Type = 0)
	BEGIN
		SET @SQL = '
		SELECT TOP 1 ''' + @VCC + '''				''vcc'',
					 [Id]
		FROM [' + @Ip + ']' + '.mmprodat.dbo.[Group] (NOLOCK)
		WHERE (
				[VCC] = ''' + @VCC + '''
				OR
				[VCC] = ''system''
			)
			AND [Id] = ''' + @RoleId + '''
		'
	END
	ELSE
	BEGIN
		SET @SQL = '
		SELECT TOP 1 ''' + @VCC + '''				''vcc'',
					 [Id]
		FROM mmprodat.dbo.[Group] (NOLOCK)
		WHERE
			(
				[VCC] = ''' + @VCC + '''
				OR
				[VCC] = ''system''
			)
			AND [Id] = ''' + @RoleId + '''
		'
	END
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAllegroSupervisor]
(
	@IP				VARCHAR(50),
	@VCC			VARCHAR(50),
	@UserId			VARCHAR(50),
	@Type			BIT
)
AS
BEGIN
	-- ESTE SP RETORNA LA LISTA DE SUPERVISORES DE UN VCC DE TIPO ALLEGRO/OCC CON SUS WORKGROUPS DE AGENTES
	
	-- DECLARAMOS VARIABLES PARA ARMAR LA CONSULTA
	DECLARE @VCC_TYPE VARCHAR(25) = NULL
	DECLARE @SQL VARCHAR(MAX) = NULL
	DECLARE @TABLE_MMPRODAT VARCHAR(100) = NULL

	-- INDETIFICAMOS EL TIPO DE VCC
	SELECT @VCC_TYPE = V.[Type] 
		FROM [dbo].[VCC] V (NOLOCK) 
		WHERE V.[VCC] = @VCC

	-- EL TYPE 0 INDICA QUE LAS TABLAS SE ENCUENTRAN EN UN SERVIDOR EXTERNO
	IF @Type = 0
	BEGIN  
		SET @TABLE_MMPRODAT = '[' + @Ip + ']' + '.[MMProDat].[dbo].'
	END
	ELSE IF(@Type = 1) 
	BEGIN
		SET @TABLE_MMPRODAT = '[MMProDat].[dbo].'
	END
	
	--
	IF @VCC_TYPE = 'ALLEGRO'
	BEGIN
		-- ARMAMOS LA CONSULTA PRINCIPAL
		/*
		SET @SQL = '
			SELECT DISTINCT U.[VCC]					''vcc'',
							U.[Id]					''userId''
				FROM ' + @TABLE_MMPRODAT + '[SupervisorWorkgroup] SWG (NOLOCK)
				INNER JOIN ' + @TABLE_MMPRODAT + '[User] U (NOLOCK)
					ON U.[VCC] = SWG.[VCC]
						AND U.[Id] = SWG.[SupervisorId]
				WHERE SWG.[VCC] = ''' + @VCC + ''' '
		*/
		SET @SQL = '
			SELECT U.[VCC]					''vcc'',
					U.[Id]					''userId''
				FROM ' + @TABLE_MMPRODAT + '[User] U (NOLOCK)
				LEFT JOIN ' + @TABLE_MMPRODAT + '[SupervisorWorkgroup] SWG (NOLOCK)
					ON SWG.[VCC] = U.[VCC] 
						AND SWG.[SupervisorId] = U.[Id] 
				WHERE U.[VCC] = ''' + @VCC + ''' 
					AND U.[Id] = ''' + @UserId + '''
					AND U.[UserGroup] = ''supervisor'' '
	END
	ELSE IF @VCC_TYPE = 'OCC'
	BEGIN
		SET @SQL = '
			SELECT DISTINCT U.[VCC]					''vcc'',
							U.[Id]					''userId''
				FROM ' + @TABLE_MMPRODAT + '[User] U (NOLOCK) 
				INNER JOIN ' + @TABLE_MMPRODAT + '[GroupApplication] GA (NOLOCK)
					ON GA.[VCC] = U.[VCC]
						AND GA.[GroupId] = U.[UserGroup] 
				LEFT JOIN ' + @TABLE_MMPRODAT + '[UserCampaign] UC (NOLOCK) 
					ON UC.[VCC] = UC.[VCC]
						AND UC.[UserId] = U.[Id]
				LEFT JOIN ' + @TABLE_MMPRODAT + '[SupervisorWorkgroup] SW (NOLOCK)
					ON SW.[VCC] = U.[VCC]
						AND SW.[SupervisorId] = U.[Id]
						AND SW.[CampaignId] = UC.[CampaignId]
				WHERE GA.[VCC] = ''' + @VCC + '''
					AND GA.[ApplicationId] = ''supervisor''
					AND U.[Id] = ''' + @UserId + '''
				ORDER BY U.[Id] ASC, UC.[CampaignId] ASC 
		'
	END
	
	-- EJECUTAMOS LA CONSULTA
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAllegroSupervisors]
(
	@IP				VARCHAR(50),
	@VCC			VARCHAR(50),
	@Type			BIT
)
AS
BEGIN
	-- ESTE SP RETORNA LA LISTA DE SUPERVISORES DE UN VCC DE TIPO ALLEGRO/OCC CON SUS WORKGROUPS DE AGENTES

	-- DECLARAMOS VARIABLES PARA ARMAR LA CONSULTA
	DECLARE @VCC_TYPE VARCHAR(25) = NULL
	DECLARE @SQL VARCHAR(MAX) = NULL
	DECLARE @TABLE_MMPRODAT VARCHAR(100) = NULL

	-- INDETIFICAMOS EL TIPO DE VCC
	SELECT @VCC_TYPE = V.[Type] 
		FROM [dbo].[VCC] V (NOLOCK) 
		WHERE V.[VCC] = @VCC

	-- EL TYPE 0 INDICA QUE LAS TABLAS SE ENCUENTRAN EN UN SERVIDOR EXTERNO
	IF @Type = 0
	BEGIN  
		SET @TABLE_MMPRODAT = '[' + @Ip + ']' + '.[MMProDat].[dbo].'
	END
	ELSE IF(@Type = 1) 
	BEGIN
		SET @TABLE_MMPRODAT = '[MMProDat].[dbo].'
	END
	
	--
	IF @VCC_TYPE = 'ALLEGRO'
	BEGIN
		-- ARMAMOS LA CONSULTA PRINCIPAL
		/*
		SET @SQL = '
			SELECT U.[VCC]					''vcc'',
					U.[Id]					''userId'',
					U.[Name]				''userName'',
					SWG.[CampaignId]		''campaignId'',
					SWG.[WorkgroupId]		''workGroupId''
				FROM ' + @TABLE_MMPRODAT + '[SupervisorWorkgroup] SWG (NOLOCK)
				INNER JOIN ' + @TABLE_MMPRODAT + '[User] U (NOLOCK)
					ON U.[VCC] = SWG.[VCC]
						AND U.[Id] = SWG.[SupervisorId]
				WHERE SWG.[VCC] = ''' + @VCC + ''' '
		*/
		SET @SQL = '
			SELECT U.[VCC]					''vcc'',
					U.[Id]					''userId'',
					U.[Name]				''userName'',
					UC.[CampaignId]			''campaignId'',
					SWG.[WorkgroupId]		''workGroupId''
				FROM ' + @TABLE_MMPRODAT + '[User] U (NOLOCK)
				LEFT JOIN ' + @TABLE_MMPRODAT + '[UserCampaign] UC (NOLOCK)
					ON UC.[VCC] = U.[VCC]
						AND UC.[UserId] = U.[Id]
				LEFT JOIN ' + @TABLE_MMPRODAT + '[SupervisorWorkgroup] SWG (NOLOCK)
					ON SWG.[VCC] = U.[VCC] 
						AND SWG.[SupervisorId] = U.[Id] 
						AND SWG.[CampaignId] = UC.[CampaignId]
				WHERE U.[VCC] = ''' + @VCC + ''' 
					AND U.[UserGroup] = ''supervisor'' '
	END
	ELSE IF @VCC_TYPE = 'OCC'
	BEGIN
		SET @SQL = '
			SELECT U.[VCC]					''vcc'',
					U.[Id]					''userId'',
					U.[Name]				''userName'',
					--U.[UserGroup],
					UC.[CampaignId]			''campaignId'',
					SW.[WorkgroupId]		''workGroupId''
				FROM ' + @TABLE_MMPRODAT + '[User] U (NOLOCK) 
				INNER JOIN ' + @TABLE_MMPRODAT + '[GroupApplication] GA (NOLOCK)
					ON GA.[VCC] = U.[VCC]
						AND GA.[GroupId] = U.[UserGroup] 
				LEFT JOIN ' + @TABLE_MMPRODAT + '[UserCampaign] UC (NOLOCK) 
					ON UC.[VCC] = UC.[VCC]
						AND UC.[UserId] = U.[Id]
				LEFT JOIN ' + @TABLE_MMPRODAT + '[SupervisorWorkgroup] SW (NOLOCK)
					ON SW.[VCC] = U.[VCC]
						AND SW.[SupervisorId] = U.[Id]
						AND SW.[CampaignId] = UC.[CampaignId]
				WHERE GA.[VCC] = ''' + @VCC + '''
					AND GA.[ApplicationId] = ''supervisor''
				ORDER BY U.[Id] ASC, UC.[CampaignId] ASC 
		'
	END
	
	-- EJECUTAMOS LA CONSULTA
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAllTags]
(
	@VCC						VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		distinct 		[Tag]	'tag'
	FROM
		[dbo].[Tag]
	WHERE
		[VCC] = @VCC

	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


CREATE PROCEDURE [dbo].[GetAllTagsInteraction]
(
	@VCC						VARCHAR(50),
	@AudioId					VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]												'vcc',
		[AudioId]											'audioId',
		[Section]											'section',
		[Part]												'part',
		CONVERT(decimal(18, 15), [CurrentTime])				'currentTime',
		[Tag]												'tag',
		[Comment]											'comment',
		[CreatedDate]										'createdDate',
		[CreatedByUserId]									'createdByUserId',
		[LastModifiedDate]									'lastModifiedDate',
		[LastModifiedByUserId]								'lastModifiedByUserId',
		[TimeStamp]											'timeStamp'
	FROM
		[dbo].[Tag]
	WHERE
		[VCC] = @VCC AND
		[AudioId] = @AudioId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAPIUser] (
	@VCC Varchar(50) = NULL, @User Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]								'vcc',
		[User] 								'user',
		[IsLocked] 						    'isLocked',
		[CreatedDate]						'createdDate',
		[CreatedByUserId]					'createdByUserId',
		[LastModifiedDate]					'lastModifiedDate',
		[LastModifiedByUserId]				'lastModifiedByUserId'
	FROM [UserAPI] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [User] = @User
	
	
	SET NOCOUNT OFF
END 
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetAPIUserAllegro] 'inconcert'
CREATE PROCEDURE [dbo].[GetAPIUserAllegro](
@VCC Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'vcc',
		[User] 							'user',
		[Password] 						'password'
	FROM [APIUserAllegro] (NOLOCK)
	WHERE [VCC] = @VCC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec GetAPIUserOCC
CREATE PROCEDURE [dbo].[GetAPIUserOCC](
@VCC Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'vcc',
		[User] 							'user',
		[Password] 						'password'
	FROM [APIUserOCC] (NOLOCK)
	WHERE [VCC] = @VCC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAPIUsers](
@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'vcc',
		[User] 							'user',
		[IsLocked]						'isLocked',
		[CreatedDate] 					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [UserAPI] (NOLOCK)
	WHERE [VCC] = @VCC
		AND ISNULL([User], '') LIKE CASE WHEN @SearchCriteria = 'user' THEN '%' + @SearchExpression + '%' ELSE '%' END
	SET NOCOUNT OFF
END



GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec GetAPIUsersAllegro
CREATE PROCEDURE [dbo].[GetAPIUsersAllegro]
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'vcc',
		[User] 							'user',
		[Password] 						'password'
	FROM [APIUserAllegro] (NOLOCK)

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec GetAPIUsersOCC
CREATE PROCEDURE [dbo].[GetAPIUsersOCC]
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'vcc',
		[User] 							'user',
		[Password] 						'password'
	FROM [APIUserOCC] (NOLOCK)

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTE SP RETORNA UN RESUMEN DE LA BUSQUEDA DE INTERACCIONES DE ALLEGRO PARA INDETIFICAR LOS YA TRANSCRITOS O INPROCESS SEGUN LOS FILTROS RECIBIDOS 
-- FUNCIONA IGUAL QUE LA FUNCION EN KNEX
CREATE PROCEDURE [dbo].[GetAsyncBillingInteractions] (
   	@VCC					VARCHAR(50) = NULL,
	@IP						VARCHAR(50),						-- IP DEL SERVIDOR PARA LA BASE I6
	@Type					INT,								-- FLAG PARA INDICAR SI EL SERVIDOR I6 SE ENCUENTRA LOCAL O REMOTO
	@InteractionType		VARCHAR(50) = NULL,
	@StartDate				DATETIME = '1980-01-01T00:00:00',
    @EndDate				DATETIME = '2050-01-01T23:59:59',
	@StartHour				VARCHAR(50) = NULL,
	@EndHour				VARCHAR(50) = NULL,
	@Campaigns				VARCHAR(MAX) = NULL,
 	@Agents					VARCHAR(MAX) = NULL,
   	@DispositionCodes		VARCHAR(MAX) = NULL,
   	@Origins				VARCHAR(100) = NULL,
	@InteractionIds         VARCHAR(MAX) = NULL,
	@CharsPerPhrase			INT = 0
	
)
AS 
BEGIN 
	-- SETTEAMOS VALORES MINIMOS PARA LAS HORAS DE INICIO Y FIN
	IF (@StartHour IS NOT NULL AND @StartHour = '') SET @StartHour = '00:00:00'
	IF (@EndHour IS NOT NULL AND @EndHour = '') SET @EndHour = '23:59:59'

	-- CREAMOS UNA TABLA TEMPORAL PARA INSERTAR LOS REGISTROS DE LA HDI6
	CREATE TABLE #TMP_HD_INTERACTIONS
	(
		vcc VARCHAR(50), 
		id VARCHAR(250), 
		folder VARCHAR(50),
		exportedContent VARCHAR(250),  
		channel VARCHAR(50), 
		campaign VARCHAR(50), 
		startDate DATETIME,
		endDate DATETIME,
		account VARCHAR(50), 
		initiative VARCHAR(50), 
		contactName VARCHAR(250), 
		firstTakedAgent VARCHAR(50), 
		lastTakedAgent VARCHAR(50), 
		isTaken VARCHAR(50), 
		totalMessages INT,
		durationTime VARCHAR(50),  
		attentionTime VARCHAR(50), 
		dispositionCode VARCHAR(250), 
		dispositionTreePath VARCHAR(250), 
		rawChar INT,
		RN INT
	)

	-- EJECUTAMOS EL SP DE CONSULTA QUE LLENARA LA TABLA TEMPORAL
	EXEC [dbo].[GetI6AsyncInteractions]
		@Ip = @IP, @VCC = @VCC, @Type = @Type, @InteractionType = @InteractionType, @StartDate = @StartDate, 
		@EndDate = @EndDate, @StartHour = @StartHour, @EndHour = @EndHour, @Campaigns = @Campaigns, @Agents = @Agents,
		@DispositionCodes = @DispositionCodes, @Origins = @Origins,  @InteractionIds = @InteractionIds, @UniqueIds = 0, @ResultType = 'DETAILED'

	-- DECLARAMOS UNA VARIABLE PARA PARA EL TOTALDE RESULTADOS
	DECLARE @TOTAL_INTERACTIONS INT = 0

	-- SETTEAMOS EL NUMERO DE REGISTROS A LA VARIABLE DE TOTAL
	SELECT @TOTAL_INTERACTIONS = COUNT(*) FROM #TMP_HD_INTERACTIONS
	
	IF @TOTAL_INTERACTIONS = 0 
	BEGIN
		-- SI NO EXISTEN RESULTADOS EN HD LLENAMOSUN RESULTADOS CON 0
		SELECT @InteractionType												'interactionType',
				0															'totalInteractionDetails',
				0															'alreadyTranscribedInteraction',
				0															'inProcessInteraction',
				0															'notTranscribedInteraction',
				0															'totalPhrases',
				0															'totalRawChars',
				0															'totalRawPhrases'
	END 
	ELSE
	BEGIN
		-- CRUZAMOS LA TABLA TEMPORAL DE INTERACCIONES ASINCRONAS CON LA TABLA LOCAL INTERACTION PARA LAS INTERACCIONES ASINCRONAS YA REGISTRADAS
		SELECT HDI.[channel]												'interactionType',
				COUNT(*)													'totalInteractionDetails',
				SUM(CASE WHEN (AI.[VCC] IS NOT NULL 
					AND AI.[IsTranscribed] = 1) THEN 1 ELSE 0 END)			'alreadyTranscribedInteraction',
				0															'inProcessInteraction',
				SUM(CASE WHEN (AI.[VCC] IS NULL 
					OR AI.[IsTranscribed] <> 1) THEN 1 ELSE 0 END)			'notTranscribedInteraction',
				SUM(ISNULL(AI.[TotalPhrases], 0))							'totalPhrases',
				-- CUANDO TENEMOS VALIDPHRASES, YA NO CONTAMOS LAS RAWCHAR 
				SUM(
					CASE WHEN (AI.[TotalPhrases] IS NULL 
							OR AI.[TotalPhrases] = 0)
					THEN (
						ISNULL(AI.[RawChar], ISNULL(HDI.[rawChar], 0))
					)
					ELSE 0 END
				)															'totalRawChars',
				SUM(
					CASE WHEN (AI.[TotalPhrases] IS NULL 
							OR AI.[TotalPhrases] = 0)
					THEN (
						-- SI RAWCHAR ES NULL, SE DEBE VALUAR COMO MINIMO 1 FRASE(@CharsPerPhrase)
						-- CEILING(ISNULL(AI.[RawChar], ISNULL(HDI.[rawChar], @CharsPerPhrase)) * 1.00 / @CharsPerPhrase)
						CEILING(ISNULL(AI.[RawChar], ISNULL(HDI.[rawChar], 0)) * 1.00 / @CharsPerPhrase)
					)
					ELSE 0 END
				)															'totalRawPhrases'
			FROM #TMP_HD_INTERACTIONS HDI (NOLOCK)
			LEFT JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
				ON HDI.[vcc] = AI.[VCC]
					AND HDI.[channel] = AI.[Channel]
					AND HDI.[campaign] = AI.[Campaign]
					AND HDI.[id] = AI.[ID]
					AND HDI.[endDate] = AI.[EndDate]
			GROUP BY HDI.[vcc],
						HDI.[channel]
			--ORDER BY AI.[TotalPhrases], AI.[RawChar] DESC, HDI.[RawChar] DESC

	END

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAudio] (
	@VCC Varchar(50) = NULL,
	@ProcessId Varchar(50) = NULL,
	@ID Varchar(100) = NULL,
	@Section NUMERIC(5, 0) NULL,
	@Part NUMERIC(5, 0) NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT TOP 1
		A.[ID]						'id',
		A.[Section]					'section',
		A.[Part]					'part',
		PA.[ProcessId]				'processId',
		A.[Number]					'number',
		A.[Gender]					'gender',
		A.[SilenceTime]				'silenceTime',
		A.[DurationTime]			'durationTime',
		A.[DurationLinguisticTime]	'durationLinguisticTime',
		A.[Category1]				'category1',
		A.[Category2]				'category2',
		A.[Category3]				'category3',
		A.[Description]				'description',
		A.[Campaign]				'campaign',
		A.[Agent]					'agent',
		A.[Type]					'type',
		A.[Init]					'init',
		A.[ChannelsChanged]			'channelsChanged',
		A.[IsTranscribed]			'isTranscribed',
		A.[RelativePath]			'relativePath',
		A.[FileName]				'fileName',
		P.[Retranscription]			'retranscription',
		A.[RelativeFileName]		'relativeFileName'
	FROM [Audio] A (NOLOCK)
	LEFT JOIN [ProcessAudio] PA (NOLOCK)
		ON PA.[VCC] = A.[VCC]
			AND A.ID = PA.AudioId 
			AND A.Section = PA.Section 
			AND A.Part = PA.Part
	LEFT JOIN [dbo].[Process] P (NOLOCK)
		ON P.[VCC] = A.[VCC]
			AND PA.[ProcessId] = P.[Name]
	WHERE A.[VCC] = @VCC
		AND (
			(@ProcessId IS NOT NULL AND P.[Name] = @ProcessId)
			OR 
			(@ProcessId IS NULL)
		)
		AND A.[ID] = @ID
		AND A.[Section] = @Section
		AND A.[Part] = @Part
		AND A.[IsTranscribed] = 1
		AND ISNULL(A.[IsCancel], 0) = 0
		AND P.[Name] IS NOT NULL
	ORDER BY A.CreatedDate DESC, PA.[CreatedDate] DESC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAudioNavigatorInteractions]
(
	@VCC					VARCHAR(50),
	@ProcessId				VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON
	
		SELECT 
				CONCAT('CALL/',@ProcessId,'/',A.[ID],'/',A.[Section],'/',A.[Part]) 'url',
				A.[ID]														'id'
			FROM [dbo].[Audio] A (NOLOCK)
			INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
				ON A.[VCC] = PA.[VCC]
					AND A.ID = PA.AudioId 
					AND A.Section = PA.Section 
					AND A.Part = PA.Part
			INNER JOIN [dbo].[Process] P (NOLOCK) 
				ON P.[VCC] = PA.[VCC]
					AND PA.[ProcessId] = P.[Name]
			WHERE A.[VCC] = @VCC
				AND PA.[ProcessId] = @ProcessId
				AND P.[Type] = 'SPEECH'
				AND A.[ErrorMessage] IS NULL
				AND A.[IsTranscribed]=1

		UNION

		SELECT 
				CONCAT(AI.[Channel],'/',@ProcessId,'/',AI.[ID],'/0/0')			    'url',
				AI.[ID]																'id'
			FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
			INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
				ON AI.[VCC] = PAI.[VCC]
					AND AI.[Channel] = PAI.[InteractionType]
					AND AI.[ID] = PAI.[InteractionId]
					AND AI.[Folder] = PAI.[FormattedEndDate]
			INNER JOIN [dbo].[Process] P (NOLOCK) 
				ON P.[VCC] = PAI.[VCC]
					AND P.[Name] = PAI.[ProcessId]
			WHERE PAI.[VCC] = @VCC
				AND PAI.[ProcessId] = @ProcessId
				AND P.[Type] = 'SPEECH'
				AND AI.[ErrorMessage] IS NULL 
				AND AI.[IsTranscribed]=1
				

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--[dbo].[GetAudios] 'inconcert', 'gss_retencion_abd'
CREATE PROCEDURE [dbo].[GetAudios]
(
	@VCC								VARCHAR(50),
	@ProcessId							VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		A.[VCC]							'vcc',
		A.[ID]							'id',
		A.[Section]						'section',
		A.[Part]						'part',
		PA.[ProcessId]					'processId',
		[Number]						'number',
		[Gender]						'gender',
		[SilenceTime]					'silenceTime',
		[DurationTime]					'durationTime',
		[DurationLinguisticTime]		'durationLinguisticTime',
		[Category1]						'category1',
		[Category2]						'category2',
		[Category3]						'category3',
		[Description]					'description',
		[Campaign]						'campaign',
		[Agent]							'agent',
		[Type]							'type',
		[Init]							'init',
		[IsTranscribed]					'isTranscribed',
		PA.[CreatedDate]				'createDate',
		PA.[CreatedByUserId]			'createdByUserId',
		PA.[LastModifiedDate]			'lastModifiedDate',
		PA.[LastModifiedByUserId]		'lastModifiedByUserId'
	FROM [dbo].[Audio] A (NOLOCK)
	LEFT JOIN [ProcessAudio] PA (NOLOCK) ON A.ID = PA.AudioId AND A.Section = PA.Section AND A.Part = PA.Part
	WHERE
	A.[VCC] = @VCC
	AND PA.[ProcessId] = @ProcessId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAudiosByProcessId]
(
	@VCC		VARCHAR(50),
	@ProcessId	VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	BEGIN
		-- Actualiza en el primer momento que se consulta a estos registros, con el valor 1 en el campo IsProcess. Indica que se encuentra utilizado por un proceso.
		UPDATE [dbo].[Audio] SET
			[dbo].[Audio].[IsProcess] = 1	-- Indicar que se esta tomando como procesando para que ningun otro procesamiento en parelelo pueda utilizar el registro.
		-- Retornamos todas las columnas a listar.
		OUTPUT INSERTED.[VCC]							'vcc',
				PA.[ProcessId]							'processId',
				INSERTED.[ID]							'id',
				INSERTED.[Section]						'section',
				INSERTED.[Part]							'part',
				INSERTED.[StartDate]					'startDate',
				INSERTED.[EndDate]						'endDate',
				INSERTED.[Campaign]						'campaign',
				INSERTED.[DispositionCode]				'dispositionCode',
				INSERTED.[Agent]						'agent',
				INSERTED.[Type]							'origin',
				E.[Name]								'name',
				E.[Type]								'environmentType',
				E.[Description]							'description',
				E.[FTPDirectory]						'ftpDirectory',
				E.[ApiHost]								'apiHost',
				E.[ApiPort]								'apiPort',
				E.[ApiUser]								'apiUser',
				E.[ApiPassword]							'apiPassword',
				E.[DbDisabled]							'dbDisabled',
				E.[DbServer]							'dbServer',
				E.[DbPort]								'dbPort',
				E.[DbUser]								'dbUser',
				E.[DbPassword]							'dbPassword',
				E.[PbxDisabled]							'pbxDisabled',
				E.[PbxServer]							'pbxServer',
				E.[PbxPort]								'pbxPort',
				E.[PbxUser]								'pbxUser',
				E.[PbxPassword]							'pbxPassword'
		FROM
			[dbo].[ProcessAudio] PA WITH (NOLOCK)
			INNER JOIN [dbo].[Audio] A WITH (NOLOCK) ON CONCAT(A.[ID], '_', A.[Section], '_', A.[Part]) = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])
			INNER JOIN [dbo].[Process] P WITH (NOLOCK) ON PA.ProcessId = P.[Name]
			INNER JOIN [dbo].[Environment] E WITH (NOLOCK) ON P.[Environment] = E.[Name]

		WHERE
			A.[VCC] = @VCC
			AND PA.[ProcessId] = @ProcessId
			AND ISNULL(A.[IsTranscribed], 0) = 0	-- Retornar sOlo las interacciones que no hayan sido transcritas.
			AND ISNULL(A.[IsProcess], 0) = 0		-- Retornar sOlo las interacciones que no hayan sido tomadas por otro procesamiento.
	END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAudiosProcess]
(
	@PageSize				INT,
	@PageNumber				INT,
	@VCC					VARCHAR(50),
	@ProcessId				VARCHAR(50),
	@ID						VARCHAR(250),
	@StartDate				DATETIME = NULL,
	@EndDate				DATETIME = NULL,
	@StartHour				INT,
	@StartMinute			INT,
	@EndHour				INT,
	@EndMinute				INT,
	@Campaign				VARCHAR(MAX),
	@Agent					VARCHAR(MAX),
	@ResultCode				VARCHAR(MAX),
	@Type					VARCHAR(50),
	@IdathaResults			NVARCHAR(MAX),
	@TotalIdathaResults		INT,
	@InteractionType		VARCHAR(50),
	@InteractionIds         VARCHAR(MAX) = NULL,
	@Folder					VARCHAR(50),
	@ContactAddress			VARCHAR(300) = NULL,
	@SentimentIds			VARCHAR(MAX) = NULL
	
)
AS
BEGIN
	SET NOCOUNT ON
	  
	
	-- DEFINIMOS UNA TABLE TEMPORAL PARA DEPOSITAR LA TODOS LAS INTERACCIONES DEL PROCESS
	CREATE TABLE #PROCESS_INTERACTIONS(
		vcc VARCHAR(50),
		interactionType VARCHAR(100),
		id VARCHAR(250),
		section VARCHAR(5),
		part VARCHAR(5),
		processId VARCHAR(100),
		number VARCHAR(100),
		name VARCHAR(100),
		silenceTime INT,
		durationTime INT,
		durationLinguisticTime INT,
		category1 VARCHAR(100),
		category2 VARCHAR(100),
		category3 VARCHAR(100),
		agent VARCHAR(100),
		agentName VARCHAR(150),
		description VARCHAR(100),
		environment VARCHAR(100),
		isTranscribed VARCHAR(100),
		errorMessage VARCHAR(MAX),
		createdDate DATETIME,
		endDate DATETIME,
		campaign VARCHAR(50),
		dispositionCode VARCHAR(250),
		folder VARCHAR(50),
		contactAddress varchar(300),
		sentimentResult varchar(50),
		ender varchar(300),
		ticket varchar(300)
	)

	-- INSERTAMOS LOS REGISTROS CALL
	INSERT INTO #PROCESS_INTERACTIONS
		SELECT A.[VCC]														'vcc',
				'CALL'														'interactionType',
				A.[ID]														'id',
				A.[Section]													'section',
				A.[Part]													'part',
				PA.[ProcessId]												'processId',
				A.[Number]													'number',
				P.[Name]													'name',
				A.[SilenceTime]												'silenceTime',
				A.[DurationTime]											'durationTime',
				A.[DurationLinguisticTime]									'durationLinguisticTime',
				A.[Category1]												'category1',
				A.[Category2]												'category2',
				A.[Category3]												'category3',
				A.[Agent]													'agent',
				A.[AgentName]												'agentName',
				A.[Description]												'description',
				P.[Environment]												'environment',
				A.[IsTranscribed]											'isTranscribed',
				A.[ErrorMessage]											'errorMessage',
				A.[CreatedDate]												'createdDate',
				A.[EndDate]													'endDate',
				A.[Campaign]												'campaign',
				A.[DispositionCode]											'dispositionCode',
				NULL														'folder',
				A.[ContactAddress]											'contactAddress',
				SAR.SentimentId												'sentimentResult',
				A.[Ender]                                                   'ender',
				A.[Ticket]                                                  'ticket'  
			FROM [dbo].[Audio] A (NOLOCK)
			INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
				ON A.[VCC] = PA.[VCC]
					AND A.ID = PA.AudioId 
					AND A.Section = PA.Section 
					AND A.Part = PA.Part
			INNER JOIN [dbo].[Process] P (NOLOCK) 
				ON P.[VCC] = PA.[VCC]
					AND PA.[ProcessId] = P.[Name]
			LEFT JOIN  SentimentAnalysisResult SAR 
			ON A.[VCC] = SAR.[VCC]
					AND A.ID = SAR.InteractionId 
					AND A.Section = SAR.InteractionSection 
					AND A.Part = SAR.InteractionPart	
					AND ISNULL(A.EndDate, '') = ISNULL(SAR.EndDate,'')
					AND SAR.AnalysisType = 'general'
					AND SAR.MaxScore = 1
			WHERE A.[VCC] = @VCC
				AND PA.[ProcessId] = @ProcessId
				-- AND P.[Type] = 'SPEECH' -- GENERA CONFLICTO CUANDO SE QUIERE VER EL DETALLE DE UN AUDIO DE PROCESS QUALITY DESDE EL TREEMAP
				
	-- INSERTAMOS LOS REGISTROS DE INTERACCIONES DE OTROS CANALES
	INSERT INTO #PROCESS_INTERACTIONS
		SELECT AI.[VCC]																'vcc',
				AI.[Channel]														'interactionType',
				AI.[ID]																'id',
				NULL																'section',
				NULL																'part',
				PAI.[ProcessId]														'processId',
				AI.[ContactName]													'number',
				PAI.[ProcessId]														'name',
				NULL																'silenceTime',
				CASE WHEN TRY_CAST(AI.[DurationTime] AS INT) IS NULL THEN
					CAST(SUBSTRING(AI.[DurationTime], 1, CHARINDEX(':', AI.[DurationTime], 1) - 1) AS INT) * 3600
					+
					CAST(SUBSTRING(AI.[DurationTime], CHARINDEX(':', AI.[DurationTime], 1) + 1, 2) AS INT) * 60
					+
					CAST(SUBSTRING(AI.[DurationTime], CHARINDEX(':', AI.[DurationTime], CHARINDEX(':', AI.[DurationTime], 1) + 1) + 1, 2) AS INT)
				ELSE AI.[DurationTime]
				END																	'durationTime',
				NULL																'durationLinguisticTime',
				NULL																'category1',
				NULL																'category2',
				NULL																'category3',
				AI.[FirstTakedAgent]												'agent',
				ISNULL(AI.[LastTakedAgentName], AI.[LastTakedAgentName])			'agentName',
				AI.[DispositionCode]												'description',
				'I6'																'environment',
				AI.[IsTranscribed]													'isTranscribed',
				AI.[ErrorMessage]													'errorMessage',
				AI.[CreatedDate]													'createdDate',
				AI.[EndDate]														'endDate',
				AI.[Campaign]														'campaign',
				AI.[DispositionTreePath]											'dispositionCode',
				AI.[Folder]															'folder',
				AI.[ContactAddress]													'contactAddress',
				NULL																'sentimentResult',
				AI.[Ender]                                                    		'ender',
				AI.[Ticket]                                                  		'ticket'
			FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
			INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
				ON AI.[VCC] = PAI.[VCC]
					--AND AI.[Campaign] = PAI.[Campaign]
					AND AI.[Channel] = PAI.[InteractionType]
					AND AI.[ID] = PAI.[InteractionId]
					AND AI.[Folder] = PAI.[FormattedEndDate]
			INNER JOIN [dbo].[Process] P (NOLOCK) 
				ON P.[VCC] = PAI.[VCC]
					AND P.[Name] = PAI.[ProcessId]
			WHERE PAI.[VCC] = @VCC
				AND PAI.[ProcessId] = @ProcessId
				AND P.[Type] = 'SPEECH'
		
	-- DECLARAMOS VARIABLE PARA LA CONSULTA ACUMULADA QUE LANZAREMOS
	DECLARE @SQL NVARCHAR(MAX) = ''
	DECLARE @SQL_WHERE_IDATHA NVARCHAR(MAX) = ''
    DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = ''
	DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX )= ''
	DECLARE @FILTERS_OPTIONAL_SENTIMENT VARCHAR(MAX) = ''
	
	
	SET @SQL_WHERE_IDATHA = ''

	-- HACEMOS EL PRIMER QUERY PARA SACAR EL NUMERO DE REGISTROS DEL PROCESS
	SET @SQL = ' SET NOCOUNT ON
	DECLARE @rows INT
	DECLARE @SYNC_INTERACTIONS INT
	DECLARE @ASYNC_INTERACTIONS INT
	DECLARE @maxDurationTime Numeric(12,3)

	SELECT @SYNC_INTERACTIONS = COUNT(*) FROM #PROCESS_INTERACTIONS PI WHERE PI.[interactionType] = ''CALL''
	SELECT @ASYNC_INTERACTIONS = COUNT(*) FROM #PROCESS_INTERACTIONS PI WHERE PI.[interactionType] <> ''CALL''
	SELECT @maxDurationTime = MAX([durationTime]) FROM #PROCESS_INTERACTIONS 
	'

	SET @SQL += '
		SET @rows = (
			SELECT COUNT(PI.[id])
				FROM #PROCESS_INTERACTIONS PI (NOLOCK)
				WHERE PI.[vcc] = ''' + @VCC + ''' 
					'
	IF (@ProcessId IS NOT NULL) 
			SET @SQL += 'AND PI.[ProcessId]= ''' + @ProcessId + ''' 
						'

	IF(@Type = 'query')
	BEGIN
		SET @SQL_WHERE_IDATHA += 'AND ((CONCAT(PI.[id], ''_'', PI.[section], ''_'', PI.[part]) IN (''' + @IdathaResults + '''))
						OR (PI.[id] IN (''' + @IdathaResults + ''')))
					'
	
		SET @SQL += @SQL_WHERE_IDATHA

	END
	ELSE IF(@Type = 'allegro' OR @Type = 'i6' OR @Type = 'ftp' OR @Type = 'sftp' OR @Type = 'fileSystem')
	BEGIN
		IF (@ID IS NOT NULL) 
			SET @FILTERS_OPTIONAL_PART += 'AND ((CONCAT(PI.[id], ''_'', PI.[section], ''_'', PI.[part]) LIKE ''%' + @ID + '%'')
						OR (PI.[id] = ''' + @ID + '''))
					'
		
		IF (@Campaign IS NOT NULL AND LEN(@Campaign) > 0) 
			SET @FILTERS_OPTIONAL_PART += 'AND PI.[campaign] IN (''' + @Campaign + ''')
						'
		
		IF (@Agent IS NOT NULL AND LEN(@Agent) > 0) 
			SET @FILTERS_OPTIONAL_PART += 'AND (
									(PI.[agent] IN (''' + @Agent + '''))
									OR 
									(PI.[agent] IS NULL)
								)
						'
		
		--CORREGIR
		-- (CONCAT(PI.campaign, ''_'', ISNULL(CONVERT(VARCHAR(4), PI.[dispositionCode]), '''') IN (''' + @ResultCode + '''))
		IF (@ResultCode IS NOT NULL AND LEN(@ResultCode) > 0) 
			SET @FILTERS_OPTIONAL_PART += 'AND (
									(CONCAT(PI.campaign, ''_'', ISNULL(CONVERT(VARCHAR(4), PI.[dispositionCode]), ''X'')) IN (''' + @ResultCode + '''))
									OR 
									--(CONCAT(PI.campaign, ''_'', ISNULL(PI.[dispositionCode], ''X'')) IN (''' + @ResultCode + '''))
									(CONCAT(PI.campaign, ISNULL(REPLACE(PI.[dispositionCode], ''/'', ''_''), ''_'')) IN (''' + @ResultCode + '''))
									) '
		
		IF (@InteractionType IS NOT NULL AND LEN(@InteractionType) > 0) 
			SET @FILTERS_OPTIONAL_PART += 'AND PI.[interactionType] = ''' + @InteractionType + ''' 
						'

		IF (@Folder IS NOT NULL AND LEN(@Folder) > 0) 
			SET @FILTERS_OPTIONAL_PART += 'AND PI.[folder] = ''' + @Folder + ''' 
						'

		IF (@Folder IS NOT NULL AND LEN(@Folder) > 0) 

		IF (@StartDate IS NOT NULL AND @EndDate IS NOT NULL) 
			SET @FILTERS_OPTIONAL_PART += 'AND PI.[endDate] BETWEEN ''' + CONVERT(VARCHAR(10), @StartDate, 120) + ''' 
						AND ''' + CONVERT(VARCHAR(10), @EndDate, 120) + '''
						'
		
		IF (@StartHour IS NOT NULL AND @EndHour IS NOT NULL) 
			SET @FILTERS_OPTIONAL_PART += 'AND DATEPART(HOUR, PI.[endDate]) >= ' + CONVERT(VARCHAR, @StartHour) + ' 
						AND DATEPART(HOUR, PI.[endDate]) <= ' + CONVERT(VARCHAR, @EndHour) + '
						'
		
		IF (@StartMinute IS NOT NULL AND @EndMinute IS NOT NULL) 
			SET @FILTERS_OPTIONAL_PART += 'AND DATEPART(MINUTE, PI.[endDate]) >= ' + CONVERT(VARCHAR, @StartMinute) + ' 
						AND DATEPART(MINUTE, PI.[endDate]) <= ' + CONVERT(VARCHAR, @EndMinute) + '
						'

		IF(@ContactAddress IS NOT NULL AND @ContactAddress <> '')
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND ContactAddress LIKE ''%' + @ContactAddress + '%''
			'
		END	
		
		IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
        BEGIN
			SET @FILTERS_OPTIONAL_PART = ''
			-- SET @FILTERS_EXCLUDING_PART = ' AND PI.[id] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) 
			--			'
			SET @FILTERS_EXCLUDING_PART = ' AND (
						(CONCAT(PI.[id], ''_'', PI.[section], ''_'', PI.[part]) IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')))
						OR 
						(PI.[id] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')))
						)
						'
	    END

		IF (@SentimentIds IS NOT NULL AND @SentimentIds <> '')
        BEGIN		
			
			SET @FILTERS_OPTIONAL_PART += ' AND 
						 PI.[sentimentResult] IN (SELECT Value FROM [dbo].[split](''' + @SentimentIds + ''','',''))						
						'
	    END
		
	END

	SET @SQL += @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART
	SET @SQL += ') '
	SET @SQL += 'SELECT @rows AS total, 
						@maxDurationTime AS maxDurationTime,
						@ASYNC_INTERACTIONS AS asyncDetails, 
						@SYNC_INTERACTIONS AS audios '

	-- SECCION DE QUERY PARA RESUMEN DE CANALES
	SET @SQL += '
		SELECT PI.[interactionType]			''interactionType'',
				COUNT(*)					''interactions'' 
			FROM #PROCESS_INTERACTIONS PI 
			WHERE PI.[vcc] = ''' + @VCC + ''' 
	'

	IF(@ProcessId IS NOT NULL) 
		SET @SQL += '
				AND PI.[processId] = ''' + @ProcessId + ''' '
		
	SET @SQL += '
		GROUP BY  PI.[interactionType] '

	-- SECCION DE QUERY PARA RESUMEN DE CAMPANAS
	SET @SQL += '
		SELECT PI.[campaign]			''campaign''
			FROM #PROCESS_INTERACTIONS PI 
			WHERE PI.[vcc] = ''' + @VCC + ''' 
	'

	IF(@ProcessId IS NOT NULL) 
		SET @SQL += '
				AND PI.[processId] = ''' + @ProcessId + ''' '
		
	SET @SQL += '
		GROUP BY  PI.[campaign] '

	-- SECCION DE QUERY PARA LISTADO DE DETALLES(AUDIOS/TEXTOS)
	SET @SQL += '
		SELECT PI.[interactionType],
				PI.[id],
				PI.[section],
				PI.[part],
				PI.[processId],
				PI.[folder],
				PI.[campaign],
				PI.[number],
				PI.[name],
				PI.[silenceTime],
				PI.[durationTime],
				PI.[durationLinguisticTime],
				PI.[category1],
				PI.[category2],
				PI.[category3],
				PI.[agent],
				PI.[agentName],
				PI.[description],
				PI.[environment],
				PI.[isTranscribed],
				PI.[errorMessage],
				PI.[createdDate],
				PI.[endDate],
				PI.[ender],
				PI.[ticket]
			FROM #PROCESS_INTERACTIONS PI
			WHERE PI.[vcc] = ''' + @VCC + ''' 
				'

	IF(@ProcessId IS NOT NULL) SET @SQL += 'AND PI.[processId] = ''' + @ProcessId + ''' '
			
	SET @SQL += ' '

	SET @SQL += @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART
	SET @SQL += @SQL_WHERE_IDATHA
	SET @SQL += 'ORDER BY PI.[ID], 
						PI.[ProcessId], 
						PI.[Agent], 
						PI.[AgentName], 
						PI.[Description], 
						PI.[Environment], 
						PI.[DurationTime], 
						PI.[CreatedDate]
			OFFSET ' + CONVERT(VARCHAR(50), @PageSize) + ' * (' + CONVERT(VARCHAR(50), @PageNumber) + ' - 1) ROWS
			FETCH NEXT ' + CONVERT(VARCHAR(50), @PageSize) + ' ROWS ONLY
			'

	SET @SQL += ' SET NOCOUNT OFF'
	EXEC(@SQL)
	
	DROP TABLE #PROCESS_INTERACTIONS
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAudiosTranscriptions]
( 
	@PageSize				INT,
	@PageNumber				INT,
	@VCC					VARCHAR(50),
	@ProcessId				VARCHAR(50),
	@ID						VARCHAR(50),
	@StartDate				DATETIME = NULL,
	@EndDate				DATETIME = NULL,
	@StartHour				INT,
	@StartMinute			INT,
	@EndHour				INT,
	@EndMinute				INT,
	@Campaign				VARCHAR(MAX),
	@Agent					VARCHAR(MAX),
	@ResultCode				VARCHAR(MAX),
    @InteractionIds         VARCHAR(MAX) = NULL,
	@ContactAddress		    VARCHAR(300) =	NULL,
	@SentimentIds			VARCHAR(MAX) = NULL
)
AS
BEGIN

 
	DECLARE @SQL                    NVARCHAR(MAX);
    DECLARE @FILTERS_OPTIONAL_PART  VARCHAR(MAX) = ''
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_FIXED_PART     VARCHAR(MAX) = ''

	SET @SQL = 'SET NOCOUNT ON
	DECLARE @rows INT
	SET @rows = (SELECT COUNT(A.ID)
		FROM [dbo].[Audio] A (NOLOCK)
		INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
			ON PA.[VCC] = A.[VCC]
				AND A.ID = PA.AudioId 
				AND A.Section = PA.Section 
				AND A.Part = PA.Part
		LEFT JOIN  SentimentAnalysisResult SAR 
			ON A.[VCC] = SAR.[VCC]
					AND A.[ID] = SAR.InteractionId 
					AND A.[Section] = SAR.InteractionSection 
					AND A.[Part] = SAR.InteractionPart
					AND ISNULL(A.[EndDate],0) = ISNULL(SAR.[EndDate],0)
					AND SAR.[AnalysisType] =' +''''+'general'+''''+ '
					AND SAR.[MaxScore] = 1
		LEFT JOIN [dbo].[Process] P (NOLOCK)
			-- ON P.[Description] = PA.[ProcessId]
			ON P.[VCC] = A.[VCC]
				AND P.[Name] = PA.[ProcessId]
		WHERE A.[VCC] = ''' + @VCC + ''' 
			AND A.[IsTranscribed] = 1 
			AND P.[TYPE] = ''SPEECH'' '
			IF(@ProcessId IS NOT NULL)
			BEGIN
				SET @SQL += '
				AND PA.[ProcessId]= ''' + @ProcessId + ''' '
			END
	
	BEGIN
		IF(@ID IS NOT NULL)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND CONCAT(A.[ID], ''_'', A.[Section], ''_'', A.[Part]) LIKE ''%' + @ID + '%''
			'
		END
		IF(@Campaign IS NOT NULL AND LEN(@Campaign) > 0)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND A.[Campaign] IN(''' + @Campaign + ''')
			'
		END
		IF(@Agent IS NOT NULL AND LEN(@Agent) > 0)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND A.[Agent] IN(''' + @Agent + ''')
			'
		END
		IF(@ResultCode IS NOT NULL AND LEN(@ResultCode) > 0)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND CONCAT(A.Campaign, ''_'', ISNULL(CONVERT(VARCHAR(4), A.[DispositionCode]), '''')) IN (''' + @ResultCode + ''')
			'
		END
		IF(@StartDate IS NOT NULL AND @EndDate IS NOT NULL)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND A.[EndDate] BETWEEN ''' + CONVERT(VARCHAR(10), @StartDate, 120) + ''' AND ''' + CONVERT(VARCHAR(10), @EndDate, 120) + '''
			'
		END
		IF(@StartHour IS NOT NULL AND @EndHour IS NOT NULL)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND DATEPART(HOUR, A.[EndDate]) >= ' + CONVERT(VARCHAR, @StartHour) + ' AND DATEPART(HOUR, A.[EndDate]) <= ' + CONVERT(VARCHAR, @EndHour) + '
			'
		END
		IF(@StartMinute IS NOT NULL AND @EndMinute IS NOT NULL)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND DATEPART(MINUTE, A.[EndDate]) >= ' + CONVERT(VARCHAR, @StartMinute) + ' AND DATEPART(MINUTE, A.[EndDate]) <= ' + CONVERT(VARCHAR, @EndMinute) + '
			'
		END
		IF(@ContactAddress IS NOT NULL)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND A.ContactAddress LIKE ''%' + @ContactAddress + '%''
			'
		END

	
		IF (@SentimentIds IS NOT NULL AND @SentimentIds <> '')
        BEGIN		
			
			SET @FILTERS_OPTIONAL_PART += ' AND 
						 SAR.[SentimentId] IN ('+''''+ @SentimentIds +''''+ ')						
						'
	    END
		
		IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
        BEGIN 
            SET @FILTERS_OPTIONAL_PART = '';
	        SET @FILTERS_EXCLUDING_PART=' AND A.[ID] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) '
        END

		
	
	END
	SET @SQL += @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART
	SET @SQL += ') '
	SET @SQL += ' SELECT @rows AS total '
	
	SET @SQL += '
		SELECT
			A.[ID]						''id'',
			A.[Section]					''section'',
			A.[Part]					''part'',
			PA.[ProcessId]				''processId'',
			A.[Number]					''number'',
			B.[Name]					''name'',
			A.[SilenceTime]				''silenceTime'',
			A.[DurationTime]			''durationTime'',
			A.[DurationLinguisticTime]	''durationLinguisticTime'',
			A.[Category1]				''category1'',
			A.[Category2]				''category2'',
			A.[Category3]				''category3'',
			A.[Agent]					''agent'',
			A.[AgentName]				''agentName'',
			A.[Description]				''description'',
			B.[Environment]				''environment'',
			E.[Type]					''environmentType'',
			A.[IsTranscribed]			''isTranscribed'',
			COUNT(UT.[Text])		    ''isSubrayed'',
			A.[CreatedDate]				''createdDate'',
			A.[EndDate]					''endDate'',
			SAR.[SentimentId]			''sentimentResult''
		FROM [dbo].[Audio] A (NOLOCK)
		INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
			ON PA.[VCC] = A.[VCC]
				AND A.ID = PA.AudioId 
				AND A.Section = PA.Section 
				AND A.Part = PA.Part
		LEFT JOIN  SentimentAnalysisResult SAR 
			ON A.[VCC] = SAR.[VCC]
					AND A.[ID] = SAR.InteractionId 
					AND A.[Section] = SAR.InteractionSection 
					AND A.[Part] = SAR.InteractionPart
					AND ISNULL(A.[EndDate],0) = ISNULL(SAR.[EndDate],0)
					AND SAR.[AnalysisType] =' +''''+'general'+''''+ '
					AND SAR.[MaxScore] = 1
		INNER JOIN [dbo].[Process] B (NOLOCK) 
			ON B.[VCC] = PA.[VCC]
				AND PA.[ProcessId] = B.[Name]
		INNER JOIN [dbo].[Environment] E (NOLOCK) 
			ON E.[VCC] = B.[VCC]
				AND B.[Environment] = E.[Name] 
				AND B.VCC = E.VCC
		LEFT JOIN [dbo].[Underlined] U (NOLOCK) 
			ON U.[VCC] = A.[VCC]
				AND A.ID = U.AudioId 
				AND A.Section = U.Section 
				AND A.Part = U.Part
	    LEFT JOIN [dbo].[Utterance] UT (NOLOCK)
	        ON UT.[VCC] = U.[VCC]
	           AND U.[UtteranceId] = UT.[Id]
		WHERE
			A.[VCC] = ''' + @VCC + ''' AND A.[IsTranscribed] = 1 '
			IF(@ProcessId IS NOT NULL)
			BEGIN
				SET @SQL += '
				AND PA.[ProcessId] = ''' + @ProcessId + ''' '
			END
			SET @SQL += '
	'

	SET @SQL += @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART
	
	SET @SQL += '
		AND B.[TYPE] = ''SPEECH''
		GROUP BY A.[ID], A.[Section], A.[Part], PA.[ProcessId], A.[Number],
				B.[Name], A.[SilenceTime], A.[DurationTime], A.[DurationLinguisticTime], A.[Category1],     
				A.[Category2], A.[Category3], A.[Agent], A.[AgentName], A.[Description], 
				B.[Environment], E.[Type], A.[IsTranscribed], A.[CreatedDate], A.[EndDate], SAR.[SentimentId]
		ORDER BY A.[ID], PA.[ProcessId], A.[Agent], A.[Description], B.[Environment], A.[DurationTime], A.[CreatedDate]
		OFFSET ' + CONVERT(VARCHAR(50), @PageSize) + ' * (' + CONVERT(VARCHAR(50), @PageNumber) + ' - 1) ROWS
		FETCH NEXT ' + CONVERT(VARCHAR(50), @PageSize) + ' ROWS ONLY;
	'
	
	SET @SQL += ' SET NOCOUNT OFF'	
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAuditorEvaluationPlanEvaluation] (
	@Instance Varchar(50) = NULL,
	@AuditorId Varchar(50) = NULL,
	@EvaluationPlanId Varchar(50) = NULL,
	@InteractionId Varchar(250) = NULL,
	@InteractionType Varchar(25) = NULL,
	@EfectivityPlanId Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	IF ISNULL(@EfectivityPlanId, '') = '' 
	BEGIN
		-- 
		SELECT DISTINCT AEP.[IdEvaluationPlan]		'evaluationPlanId', 
				EP.[Evaluation_Mode]				'evaluation_mode',
				AEP.[IdAuditor]						'auditorId', 
				IEP.[IdAgent]						'agentId',
				IEP.[AgentName]						'agentName',
				AEP.[IdInteraction]					'interactionId', 
				AEP.[Completed]						'evaluationCompleted', 
				EP.[Description]					'evaluationPlanDescription',
				EP.[IdEvaluationFormat]				'evaluationFormatId',
				AEP.[Completed]						'completed',
				AEP.[Status]						'status',
				AEP.[StatusDescription]				'statusDescription',
				AEP.[EvaluationTime]				'evaluationTime'
			FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
				ON EP.[Instance] = AEP.[Instance]
					AND EP.[ID] = AEP.[IdEvaluationPlan]
			INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = AEP.[Instance]
					AND IEP.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND IEP.[IdInteraction] = AEP.[IdInteraction]
					AND IEP.[InteractionType] = AEP.[InteractionType]
					AND (
						IEP.[IdAgent] = AEP.[IdAgent]
						OR
						ISNULL(IEP.[IdAgent], '') = ISNULL(AEP.[IdAgent], '')
					)
			WHERE AEP.[Instance] = @Instance 
				AND AEP.[IdEvaluationPlan] = @EvaluationPlanId 
				AND AEP.[IdAuditor] = @AuditorId 
				AND AEP.[IdInteraction] = @InteractionId
				AND AEP.[InteractionType] = @InteractionType
	END
	ELSE
	BEGIN
		SELECT DISTINCT AEP.[IdEvaluationPlan]		'evaluationPlanId', 
				ERE.[EfectivityPlanId]				'efectivityPlanId',
				EP.[Evaluation_Mode]				'evaluation_mode',
				ERE.[AuditorId]					    'auditorId', 
				IEP.[IdAgent]						'agentId',
				IEP.[AgentName]						'agentName',
				AEP.[IdInteraction]					'interactionId', 
				AEP.[Completed]						'evaluationCompleted', 
				EP.[Description]					'evaluationPlanDescription',
				EP.[IdEvaluationFormat]				'evaluationFormatId',
				AEP.[Completed]						'completed',
				AEP.[Status]						'status',
				AEP.[StatusDescription]				'statusDescription'
			FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
				ON EP.[Instance] = AEP.[Instance]
					AND EP.ID = AEP.IdEvaluationPlan
			INNER JOIN (SELECT DISTINCT [VCC],[EfectivityPlanId],[EvaluationPlanId],[InteractionId],[AuditorId] FROM dbo.[EvaluationResultEfectivity]
						WHERE EfectivityPlanId = @EfectivityPlanId AND AuditorId = @AuditorId
			) AS ERE
			ON AEP.[Instance] = ERE.[VCC]
			AND AEP.[IdEvaluationPlan] = ERE.[EvaluationPlanId]
			AND AEP.[IdInteraction] = [ERE].[InteractionId]			

			INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = AEP.[Instance]
					AND IEP.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND IEP.[IdInteraction] = AEP.[IdInteraction]
					AND IEP.[InteractionType] = AEP.[InteractionType]
					AND (
						IEP.[IdAgent] = AEP.[IdAgent]
						OR
						ISNULL(IEP.[IdAgent], '') = ISNULL(AEP.[IdAgent], '')
					)
			WHERE AEP.[Instance] = @Instance 
				AND AEP.[IdEvaluationPlan] = @EvaluationPlanId 
				AND ERE.[AuditorId] = @AuditorId 
				AND AEP.[IdInteraction] = @InteractionId
				AND AEP.[InteractionType] = @InteractionType;

		END


	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


CREATE PROCEDURE [dbo].[GetAuditors] (
	@Instance Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
 	SELECT T.ID								'id',
			U.lastname + ' ' + U.Firstname	'name',
			U.Firstname						'firstname',
			U.lastname						'lastname',
			U.email							'email',
			U.[IsLocalSupervisor]			'isLocalSupervisor',
			U.[IsExternalSupervisor]		'isExternalSupervisor',
			U.[IsExternalAgent]				'isExternalAgent',
			T.CreatedDate					'createdDate',
			T.CreatedByUserId				'createdByUserId',
			T.LastModifiedDate				'lastModifiedDate',
			T.LastModifiedByUserId			'lastModifiedByUserId'
		FROM [dbo].[Auditor] T (NOLOCK)
		INNER JOIN [dbo].[User] U (NOLOCK)
			ON U.[VCC] = T.[Instance]
				AND U.[ID] = T.[ID]
		WHERE T.Instance = @Instance
			AND ISNULL(t.ID, '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAuditorsEvaluationPlan] (
	@Instance Varchar(50) = NULL,
	@idEvaluationPlan Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		AEP.[IdEvaluationPlan] 'idEvaluationPlan',
		AEP.[IdAuditor] 'idAuditor',
		A.[Description] 'descriptionAuditor',
		COUNT(AEP.IdInteraction) 'numberInteractions'
	FROM
		[AuditorEvaluationPlan] (NOLOCK) AEP
		,[Auditor] (NOLOCK) A
	WHERE
		AEP.[IdAuditor] = A.[ID]
		AND AEP.[Instance] = @Instance
		AND AEP.[IdEvaluationPlan] = @idEvaluationPlan
	GROUP BY
		AEP.[IdEvaluationPlan],
		AEP.[IdAuditor],
		A.[Description]

	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAuditorsFromEvaluationPlans] (
	@VCC VARCHAR(50)
	)
AS
BEGIN
	SET NOCOUNT ON

 	SELECT DISTINCT ER.[IdAuditor] 'idAuditor'
 	FROM [dbo].[EvaluationResult] ER (NOLOCK)
 	JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
 	ON ER.[Instance] = AEP.[Instance] AND ER.IdEvaluationPlan = AEP.IdEvaluationPlan
 	WHERE ER.[Instance] = @VCC AND AEP.Completed = 1

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAuditorsInteractions] (
	@Instance Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

 	SELECT t.ID								'id',
			u.lastname + ' ' + u.Firstname		'name',
			u.Firstname							'firstname',
			u.lastname							'lastname',
			u.email								'email',
			U.[IsLocalSupervisor]				'isLocalSupervisor',
			U.[IsExternalSupervisor]			'isExternalSupervisor',
			U.[IsExternalAgent]					'isExternalAgent',
			t.CreatedDate						'createdDate',
			t.CreatedByUserId					'createdByUserId',
			t.LastModifiedDate					'lastModifiedDate',
			t.LastModifiedByUserId				'lastModifiedByUserId',
			count(1) - sum(CASE WHEN a.Completed=0 THEN 0 ELSE 1 END) 'uncompleteInteractions',
			ISNULL(sum(a.InteractionDuration),0)	'interactionTimeSum'
		FROM [dbo].[Auditor] t (NOLOCK)
		INNER JOIN [dbo].[user] u (NOLOCK)
			ON t.Instance = u.VCC 
				AND t.id = u.ID
		FULL OUTER JOIN [dbo].[AuditorEvaluationPlan] a (NOLOCK)
			ON ( 
				t.Instance = a.Instance 
				AND 
				t.ID = a.IdAuditor
			)
		WHERE t.Instance = @Instance
			AND ISNULL(t.ID, '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END

			-- SE AGREGA LA VERIFICACION DE QUE EL USER NO SEA NULL PORQUE AL NO EXISTIR EL USUARIO EN LA TABLA USER MUESTRA FILAS CON NULL. SE PRUEBA EN LOS ENDPOINTS: SAVE_USER, DELETE_USER Y EN GET_AUDITORS_INTERACTIONS.
			AND u.[Firstname] IS NOT NULL
			AND u.[LastName] IS NOT NULL
		GROUP BY
			t.ID,
			u.lastname + ' ' + u.Firstname,
			u.Firstname,
			u.lastname,
			u.email,
			U.[IsLocalSupervisor],
			U.[IsExternalSupervisor],
			U.[IsExternalAgent],
			t.CreatedDate,
			t.CreatedByUserId,
			t.LastModifiedDate,
			t.LastModifiedByUserId

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAverageGroupRating] (
	@Instance VARCHAR(50) = NULL,
	@InitialDate DATETIME = NULL,
	@EndDate DATETIME = NULL,
	@IdCampaign VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdEvaluationFormat VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdAgent VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdWorkGroup VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdAttentionLevel VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdEvaluationPlan VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdGroup VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdSubGroup VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdAbility VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdAuditor VARCHAR(MAX) = 'GetAverageGroupRating'
)
AS
BEGIN
	SET NOCOUNT ON;

	SELECT
		AverageGroupRating.*,
		(100 * AverageGroupRating.AverageRating / AverageGroupRating.MaxAbilityScoreAverage) AverageRatingNorm
	FROM
		(
			-- Grupo, grupo y subgrupo o grupo, subgrupo y habilidad con su promedio
			SELECT
				IdGroup
				,[Group].Description
				,CASE WHEN @IdGroup = 'GetAverageGroupRating' THEN '' ELSE IdSubGroup END AS IdSubGroup
				,[SubGroup].Description SubGroupDescription
				,CASE WHEN (@IdGroup = 'GetAverageGroupRating' OR @IdSubGroup = 'GetAverageGroupRating') THEN '' ELSE IdAbility END AS IdAbility
				,[Ability].Title AbilityTitle
				,AVG(InteractionTotalFilterQualification) AverageRating
				,AVG(MaxAbilityScore) MaxAbilityScoreAverage
			FROM
				(
					-- Llamadas evaluadas con la suma de los puntajes agrupados por grupo, grupo y subgrupo o grupo, subgrupo y habilidad
					SELECT
						ER.IdInteraction,
						ER.IdGroup,
						CASE WHEN @IdGroup = 'GetAverageGroupRating' THEN '' ELSE ER.IdSubGroup END AS IdSubGroup,
						CASE WHEN (@IdGroup = 'GetAverageGroupRating' OR @IdSubGroup = 'GetAverageGroupRating') THEN '' ELSE ER.IdAbility END AS IdAbility,
						SUM(ER.Qualification) InteractionTotalFilterQualification,
						SUM(ER.MaxAbilityScore) MaxAbilityScore
					FROM
						EvaluationResult ER (NOLOCK),
						(
							-- Llamadas evaluadas con su ultima fecha de evaluacion
							SELECT
								IdInteraction,
								MAX(EvaluationDate) LastEvaluationDate
							FROM 
								EvaluationResult (NOLOCK)
							WHERE
								EvaluationResult.Instance = @Instance
								AND EvaluationDate BETWEEN @InitialDate AND @EndDate
								AND CASE WHEN @IdAgent = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdAgent END IN (SELECT Value FROM [dbo].[split](@IdAgent,','))
								AND CASE WHEN @IdCampaign = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdCampaign END IN (SELECT Value FROM [dbo].[split](@IdCampaign,','))
								AND CASE WHEN @IdEvaluationFormat = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdEvaluationFormat END IN (SELECT Value FROM [dbo].[split](@IdEvaluationFormat,','))
								AND CASE WHEN @IdEvaluationPlan = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdEvaluationPlan END IN (SELECT Value FROM [dbo].[split](@IdEvaluationPlan,','))
							    AND IdAuditor LIKE CASE WHEN @IdAuditor = 'GetAverageGroupRating' THEN '%' ELSE 'SPEECH' END 
								AND IdAuditor <> CASE WHEN @IdAuditor = 'GetAverageGroupRating' THEN 'SPEECH' ELSE 'B_I_G_C_H_E_E_S_E' END
							GROUP BY IdInteraction
						) LastEvaluation
					WHERE
						ER.IdInteraction = LastEvaluation.IdInteraction
						AND ER.EvaluationDate = LastEvaluation.LastEvaluationDate
						AND CASE WHEN @IdGroup = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdGroup END IN (SELECT Value FROM [dbo].[split](@IdGroup,','))
						AND CASE WHEN @IdSubGroup = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdSubGroup END IN (SELECT Value FROM [dbo].[split](@IdSubGroup,','))
						AND CASE WHEN @IdAbility = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdAbility END IN (SELECT Value FROM [dbo].[split](@IdAbility,','))
					    AND IdAuditor LIKE CASE WHEN @IdAuditor = 'GetAverageGroupRating' THEN '%' ELSE 'SPEECH' END AND IdAuditor <> CASE WHEN @IdAuditor = 'GetAverageGroupRating' THEN 'SPEECH' ELSE 'B_I_G_C_H_E_E_S_E' END
					GROUP BY
						ER.IdGroup,
						CASE WHEN @IdGroup = 'GetAverageGroupRating' THEN '' ELSE ER.IdSubGroup END,
						CASE WHEN (@IdGroup = 'GetAverageGroupRating' OR @IdSubGroup = 'GetAverageGroupRating') THEN '' ELSE ER.IdAbility END,
						ER.IdInteraction
				) InteractionTotalFilterQualification
				LEFT JOIN [Group] (NOLOCK) ON InteractionTotalFilterQualification.IdGroup = [Group].ID
				LEFT JOIN [SubGroup] (NOLOCK) ON InteractionTotalFilterQualification.IdSubGroup = [SubGroup].ID
				LEFT JOIN [Ability] (NOLOCK) ON InteractionTotalFilterQualification.IdAbility = [Ability].ID
			GROUP BY
				IdGroup
				,[Group].Description
				,CASE WHEN @IdGroup = 'GetAverageGroupRating' THEN '' ELSE IdSubGroup END
				,[SubGroup].Description
				,CASE WHEN (@IdGroup = 'GetAverageGroupRating' OR @IdSubGroup = 'GetAverageGroupRating') THEN '' ELSE IdAbility END
				,[Ability].Title
			) AverageGroupRating

	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAzureSubscription] (
	@VCC VARCHAR(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	SELECT [AS].[VCC]								'vcc',
			[AS].[AzureSubscription]				'azureSubscription',
			[AS].[ResourceGroup]					'resourceGroup',
			[AS].[AccountName]						'accountName'
	FROM [dbo].[AzureSubscription] [AS] (NOLOCK)
	WHERE [AS].[VCC] = @VCC

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

create PROCEDURE [dbo].[GetAzureSubscriptionKeyByKey] (
	@VCC VARCHAR(50) = NULL,
	@SubscriptionKey VARCHAR(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
 	SELECT ASK.[SubscriptionKey]						'subscriptionKey',
			ASK.[SubscriptionName]						'subscriptionName',
			ASK.[SubscriptionRegion]					'subscriptionRegion',
			ASK.[IsDemo]								'isDemo'
	FROM [dbo].[AzureSubscriptionKey] ASK (NOLOCK)
	WHERE ASK.[VCC] = @VCC
		AND ASK.[SubscriptionKey] = @SubscriptionKey

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

Create PROCEDURE [dbo].[GetAzureSubscriptionKeyByName] (
	@VCC VARCHAR(50) = NULL,
	@SubscriptionName VARCHAR(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
 	SELECT ASK.[SubscriptionKey]						'subscriptionKey',
			ASK.[SubscriptionName]						'subscriptionName',
			ASK.[SubscriptionRegion]					'subscriptionRegion',
			ASK.[IsDemo]								'isDemo'
	FROM [dbo].[AzureSubscriptionKey] ASK (NOLOCK)
	WHERE ASK.[VCC] = @VCC
		AND ASK.[SubscriptionName] = @SubscriptionName

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAzureSubscriptionKeys] (
	@Instance Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
 	SELECT
		s.SubscriptionKey						'subscriptionKey',
		s.SubscriptionName						'subscriptionName',
		s.SubscriptionRegion					'subscriptionRegion',
		s.IsDemo								'isDemo'
	FROM AzureSubscriptionKey s (NOLOCK)
	WHERE s.VCC = @Instance
	ORDER BY s.[IsDemo] DESC, s.[SubscriptionName] ASC

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchDetail](
	@VCC VARCHAR(50) = NULL,
	@BatchTypeName VARCHAR(50) = NULL,
	@BatchTypeId VARCHAR(50) = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
	IF @VCC IS NULL OR @BatchTypeName IS NULL OR @BatchTypeId IS NULL 
		-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
		RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

	-- VERIFICAMOS QUE EL BATCH TYPE SEA UN TIPO CONOCIDO DEL SISTEMA
	IF @BatchTypeName NOT IN ('TRANSCRIPTION', 'STUDY', 'TEMPLATE', 'EVALUATION_PLAN')
		-- LANZAMOS MENSAJE AVISANDO QUE HAN SOLICITADO UNA OPERACION DESCONOCIDA
		RAISERROR('DATABASE_ERROR_UNKNOWN_BATCH_TYPENAME', 16, 16)

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		
		-- DEPENDIENDO DEL BATCHTYPENAME CONSULTAMOS A UNA TABLA DIFERENTE
		IF @BatchTypeName = 'TRANSCRIPTION'
		BEGIN
			-- HACEMOS LA BUSQUEDA EN LA TABLA CORRESPONDIENTE SEGUN EL TIPO DE BATCH
			SELECT P.[VCC]						'vcc',
					@BatchTypeName				'batchTypeName',
					P.[Name]					'batchTypeId',
					P.[Description]				'description',
					p.[Percentage]				'percentage',
					P.[Status]					'status',
					P.[StatusDescription]		'statusDescription',
					B.[IsFinished]				'isFinished',
					B.[FinishReported]			'isReported'
				FROM [dbo].[Process] P (NOLOCK)
				LEFT JOIN [dbo].[Batch] B (NOLOCK)
					ON B.[VCC] = P.[VCC]
						AND B.BatchTypeId = P.[Name]
				WHERE P.[VCC] = @VCC 
					AND P.[Name] = @BatchTypeId
		END 
		ELSE IF @BatchTypeName IN ('STUDY', 'TEMPLATE')
		BEGIN
			SELECT S.[VCC]						'vcc',
					@BatchTypeName				'batchTypeName',
					S.[Name]					'batchTypeId',
					S.[Description]				'description',
					S.[Percentage]				'percentage',
					S.[Status]					'status',
					S.[StatusDescription]		'statusDescription' ,
					S.[BatchStatus]				'batchStatus',
					B.[IsFinished]				'isFinished',
					B.[FinishReported]			'isReported'
				FROM [dbo].[Study] S (NOLOCK)
				LEFT JOIN [dbo].[Batch] B (NOLOCK)
					ON B.[VCC] = S.[VCC]
						AND B.BatchTypeId = S.[Name]
				WHERE S.[VCC] = @VCC 
					AND S.[Name] = @BatchTypeId
		END 
		ELSE IF @BatchTypeName = 'EVALUATION_PLAN'
		BEGIN
			SELECT EP.[Instance]				'vcc',
					@BatchTypeName				'batchTypeName',
					EP.[ID]						'batchTypeId',
					EP.[Description]			'description',
					EP.[Percentage]				'percentage',
					EP.[Status]					'status',
					EP.[StatusDescription]		'statusDescription' ,
					EP.[BatchStatus]			'batchStatus',
					B.[IsFinished]				'isFinished',
					B.[FinishReported]			'isReported'
				FROM [dbo].[EvaluationPlan] EP (NOLOCK)
				LEFT JOIN [dbo].[Batch] B (NOLOCK)
					ON B.[VCC] = EP.[Instance]
						AND B.BatchTypeId = EP.[ID]
				WHERE EP.[Instance] = @VCC 
					AND EP.[ID] = @BatchTypeId
		END 

	END TRY
    BEGIN CATCH
        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchErrors] (
	@VCC VARCHAR(50) = NULL,
	@BatchType VARCHAR(25) = NULL,
	@BatchTypeId VARCHAR(50) = NULL
) AS 
BEGIN 
    SET NOCOUNT ON

	--
	SELECT BE.[BatchType]					'batchType',
			BE.[BatchTypeId]				'batchTypeId',
			BE.[BatchTypeName]				'batchTypeName',
			BE.[Implementation]				'implementation',
			BE.[InteractionId]				'interactionId',
			BE.[Query]						'query',
			BE.[ErrorFunction]				'errorFunction',
			BE.[ErrorMessage]				'errorMessage',
			BE.[ErrorDetail]				'errorDetail'
		FROM [dbo].[BatchError] BE (NOLOCK)
		WHERE BE.[VCC] = @VCC 
			AND BE.[BatchType] = @BatchType
			AND BE.[BatchTypeId] = @BatchTypeId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchEvaluationFormatsFromHistoricEvaluationPlans] (
	@VCC VARCHAR(50)
) AS
BEGIN
	SET NOCOUNT ON

 	SELECT DISTINCT HEF.[ID]			'id',
					HEF.[Description]	'description' 
		FROM [dbo].[EvaluationResult] ER (NOLOCK)
		INNER JOIN [dbo].[HistoricEvaluationFormat] HEF (NOLOCK)
			ON HEF.[EvaluationPlanId] = ER.[IdEvaluationPlan]
				AND HEF.[VCC] = ER.[Instance] 
				AND HEF.[ID] = ER.[IdEvaluationFormat]
		INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			ON AEP.[Instance] = ER.[Instance]
				AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]
		WHERE HEF.[VCC] = @VCC 
			AND HEF.[ExecutionType] = 'BATCH'
			AND AEP.[Completed] = 1
		ORDER BY HEF.[Description] ASC

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchEvaluationPlansSummary] (
	@VCC Varchar(50) = NULL,
	@IdAuditor Varchar(50) = NULL,
	@IdAgent Varchar(50) = NULL,
	@Campaigns VARCHAR(MAX) = NULL,
	@EndDate Varchar(50) = NULL,
	@EvaluationFormats Varchar(MAX) = NULL,
	@Modalities Varchar(MAX) = NULL,
	@Completed Bit = 1
)
AS
BEGIN
	SET NOCOUNT ON
	
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = '';
	DECLARE @SQL_FROM VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA VARCHAR(MAX) = '';
	DECLARE @SQL_SELECT_2 VARCHAR(MAX) = '';
	DECLARE @SQL_FROM_2 VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER_2 VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA_2 VARCHAR(MAX) = '';

	SET @SQL_SELECT_1 += '
		SELECT EP.[ID]																	''id'',
				EP.[Modality]															''modality'',
				EP.[DateFin]															''dateFin'',
				EP.[CreatedDate]														''createdDate'',
				EP.[IdEvaluationFormat]													''idEvaluationFormat'',
				EP.[Description]														''description'',
				COUNT(CASE AEP.Completed WHEN 0 THEN 1 ELSE NULL END)					''pendingInteractionCount'',
				COUNT(AEP.Completed)													''completedCount'',
				SUM(AEP.InteractionDuration)											''interactionTimeSum'',
				SUM(CASE AEP.Completed WHEN 0 THEN AEP.InteractionDuration ELSE 0 END)	''pendingInteractionTimeSum'' ';

	SET @SQL_FROM += '
			FROM [EvaluationPlan] EP (NOLOCK)
			INNER JOIN [dbo].[EvaluationFormat] EF (NOLOCK)
				ON EF.[Instance] = EP.[Instance]
					AND EF.[ID] = EP.[IdEvaluationFormat]
			LEFT JOIN [AuditorEvaluationPlan] AEP (NOLOCK)
				ON AEP.[Instance] = EP.[Instance]
					AND AEP.[IdEvaluationPlan] = EP.[ID] '

	IF (@IdAuditor IS NOT NULL AND @IdAuditor <> '') SET @SQL_FROM += 'AND AEP.[IdAuditor] = ''' + @IdAuditor + ''' '

	IF (@IdAgent IS NOT NULL AND @IdAgent <> '') SET @SQL_FROM += 'AND AEP.[IdAgent] = ''' + @IdAgent + ''' '

	SET @SQL_FILTER += '
			WHERE EP.[Instance] = ''' + @VCC + '''
				AND AEP.[IdEvaluationPlan] = EP.[ID]
				AND EP.[Enabled] = 1 
				AND EF.[ExecutionType] = ''BATCH'' ';
				
				
    IF (@EndDate IS NOT NULL AND @EndDate <> '')
    BEGIN 
       SET @SQL_FILTER +=' AND EP.[DateFin] <= ''' + @EndDate + ''' '
    END

	IF (@Campaigns IS NOT NULL AND @Campaigns <> '')
	BEGIN
		SET @SQL_FILTER += '
				AND EP.[IdEvaluationFormat] IN ( 
					SELECT CEF.[IdEvaluationFormat] FROM CampaignEvaluationFormat CEF (NOLOCK)
						WHERE CEF.[Instance] = ''' + @VCC + '''
							AND CEF.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','',''))) '
	END;

	IF (@EvaluationFormats IS NOT NULL AND @EvaluationFormats <> '') SET @SQL_FILTER += 'AND EP.[IdEvaluationFormat] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationFormats + ''','','')) ';
	IF (@Modalities IS NOT NULL AND @Modalities <> '') SET @SQL_FILTER += 'AND EP.[Modality] IN (SELECT Value FROM [dbo].[split](''' + @Modalities + ''','','')) ';
	
	IF (@Completed = 1)
	BEGIN
		SET @SQL_FILTER += 'AND AEP.[Completed] = 1 ';
		SET @SQL_FILTER_2 = ' ';	
	END
	ELSE 
	BEGIN
		SET @SQL_FILTER += 'AND AEP.[Completed] = 0 ';
		SET @SQL_FILTER_2 = 'WHERE X.[pendingInteractionCount] > 0 ';
	END;
	

	SET @SQL_EXTRA += '
			GROUP BY EP.[ID],
						EP.[Modality],
						EP.[DateFin],
						EP.[CreatedDate],
						EP.[IdEvaluationFormat],
						EP.[Description]';

	SET @SQL_SELECT_1 += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA;

	--SET @SQL_SELECT_1 = 'SELECT * FROM (' + @SQL_SELECT_1 + ') AS X WHERE X.[pendingInteractionCount] > 0 ORDER BY X.[Description] ASC; ';
	SET @SQL_SELECT_2 = 'SELECT * ';
	SET @SQL_FROM_2 = 'FROM (' + @SQL_SELECT_1 + ') AS X ';
	-- @SQL_FILTER_2 YA FUE DEFINIDO SEGUN SE BUSCAN LAS YA EVALUADAS O NO EVALUADAS
	SET @SQL_EXTRA_2 = 'ORDER BY X.[Description] ASC;';

	SET @SQL_SELECT_2 += @SQL_FROM_2 + @SQL_FILTER_2 + @SQL_EXTRA_2;


	EXEC (@SQL_SELECT_2);
	
	SET NOCOUNT OFF
	
END;
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchScheduleById] (
	@VCC VARCHAR(50) = NULL, 
	@Id VARCHAR(50) = NULL
	--@Type VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT BS.[Id]								'id',
			BS.[BatchType]						'batchType',
			BS.[Frequency]						'frequency',
			BS.[Weekday]						'weekday',
			BS.[Monthday]						'monthday',
			BS.[DaysInterval]					'daysInterval',
			BS.[HoursInterval]					'hoursInterval',
			BS.[ExecutionDelay]					'executionDelay',
			BS.[StartDate]						'startDate',
			BS.[EndDate]						'endDate',
			BS.[FirstExecutionDate]				'firstExecutionDate',
			BS.[LastExecutionDate]				'lastExecutionDate',
			BS.[FinalExecutionDate]				'finalExecutionDate',
			BS.[Status]							'status',
			BS.[CreatedDate]					'createdDate',
			BS.[CreatedByUserId]				'createdByUserId',
			BS.[LastModifiedDate]				'lastModifiedDate',
			BS.[LastModifiedByUserId]			'lastModifiedByUserId'
		FROM [dbo].[BatchSchedule] BS (NOLOCK)
		WHERE BS.[VCC] = @VCC
			AND BS.[Id] = @Id
			--AND BS.[BatchType] = @Type

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchScheduleByName] (
	@VCC VARCHAR(50) = NULL, 
	@Name VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT BS.[Id]								'id',
			BS.[BatchType]						'batchType',
			BS.[Frequency]						'frequency',
			BS.[Weekday]						'weekday',
			BS.[Monthday]						'monthday',
			BS.[DaysInterval]					'daysInterval',
			BS.[HoursInterval]					'hoursInterval',
			BS.[StartDate]						'startDate',
			BS.[EndDate]						'endDate',
			BS.[FirstExecutionDate]				'firstExecutionDate',
			BS.[LastExecutionDate]				'lastExecutionDate',
			BS.[FinalExecutionDate]				'finalExecutionDate',
			BS.[CreatedDate]					'createdDate',
			BS.[CreatedByUserId]				'createdByUserId',
			BS.[LastModifiedDate]				'lastModifiedDate',
			BS.[LastModifiedByUserId]			'lastModifiedByUserId'
		FROM [dbo].[BatchSchedule] BS (NOLOCK)
		WHERE BS.[VCC] = @VCC
			AND BS.[Id] = @Name

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchScheduledTemplateItemResults]
(
	@VCC					VARCHAR(50) = NULL,
	@BatchScheduledId		VARCHAR(50) = NULL,
	@StartDate				DATETIME = NULL, -- FILTRO PARA LOS RESULATDOS RECIENTES
	@EndDate				DATETIME = NULL, -- FILTRO PARA LOS RESULATDOS RECIENTES
	@Category1				VARCHAR(50) = NULL,
	@Category2				VARCHAR(50) = NULL,
	@Description			VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- OBTENEMOS LOS RESULTADOS RELACIONADOS CON EL ITEM DE LA AGENDA
	;WITH CTE AS (
		SELECT --SRD.[VCC]								'vcc',
				S.[BatchScheduleId]						'batchScheduleId',
				HTT.[Description]						'description',
				SRD.[StudyId]							'studyId',
				S.[ProcessId]							'processId',
				SRD.[TranscriptionId]					'transcriptionId',
				IIF (PAI.[InteractionType] IS NULL, 
						'CALL', 
						PAI.[InteractionType]
				)										'interactionType',
				IIF (PAI.[InteractionType] IS NULL, 
						PA.[AudioId],
						PAI.[InteractionId]
				)										'id',
				PA.[Section]							'section',
				PA.[Part]								'part',
				PAI.[Campaign]							'campaign',
				PAI.[FormattedEndDate]					'folder',
				-- RN = ROW_NUMBER() OVER(PARTITION BY IIF (PAI.[InteractionType] IS NULL, PA.[AudioId], PAI.[InteractionId]), PA.[Section], PA.[Part], PAI.[Campaign], PAI.[FormattedEndDate] ORDER BY S.[CreatedDate] desc)
				RN = ROW_NUMBER() OVER(PARTITION BY SRD.[TranscriptionId] ORDER BY S.[CreatedDate] desc)
			FROM [dbo].[StudyResultSummaryDetail] SRD (NOLOCK)
			INNER JOIN [dbo].[Study] S (nolock)
				ON S.[VCC] = SRD.[VCC]
					AND S.[Name] = SRD.[StudyId]
			LEFT JOIN [dbo].[ProcessAudio] PA (NOLOCK)
				ON PA.[VCC] = SRD.[VCC] 
					AND PA.[ProcessId] = S.[ProcessId] 
					AND CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part]) = SRD.[TranscriptionId]
			LEFT JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
				ON PAI.[VCC] = SRD.[VCC]
					AND PAI.[ProcessId] = S.[ProcessId]
					AND CONCAT(PAI.[InteractionType], '_', PAI.[Campaign], '_', PAI.[FormattedEndDate], '_', PAI.[InteractionId]) = SRD.[TranscriptionId]
			INNER JOIN [dbo].[HistoricTemplateTheme] HTT (NOLOCK)
				ON HTT.[VCC] = SRD.[VCC]
					AND HTT.[StudyId] = SRD.[StudyId]
					AND HTT.[ID] = SRD.[TemplateThemeId]
			WHERE S.[VCC] = @VCC
				AND S.[BatchScheduleId] = @BatchScheduledId
				AND SRD.[CreatedDate] BETWEEN @StartDate AND @EndDate
				AND (
					HTT.[Category1] = @Category1
					OR (
						HTT.[Category1] IS NULL
						AND 
						@Category1 IS NULL
					)
				)
				AND (
					HTT.[Category2] = @Category2
					OR (
						HTT.[Category2] IS NULL
						AND
						@Category2 IS NULL
					)
				)
				AND HTT.[Description] = @Description
			-- GROUP BY SRD.[VCC], SRD.[StudyId], S.[ProcessId], SRD.[TranscriptionId], PAI.[InteractionType], PA.[AudioId], PAI.[InteractionId]
			
	)
	SELECT * FROM CTE WHERE [RN] = 1 ORDER BY [TranscriptionId]

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchScheduledTemplateResults]
(
	@VCC					VARCHAR(50) = NULL,
	@BatchScheduledIds		VARCHAR(MAX) = NULL,
	@StartDate				DATETIME = NULL, -- FILTRO PARA LOS RESULATDOS RECIENTES
	@EndDate				DATETIME = NULL -- FILTRO PARA LOS RESULATDOS RECIENTES
)
AS
BEGIN
	SET NOCOUNT ON

	--DECLARE @VCC VARCHAR(50) = NULL, @BatchScheduleId VARCHAR(50) = NULL
	--SET @VCC = 'vccocc'
	--SET @BatchScheduleId = '20211230_niveles'

	-- OBTENEMOS LOS RESULTADOS RELACIONADOS CON LA AGENDA
	SELECT SRD.[VCC]								'vcc',
			S.[BatchScheduleId]						'batchScheduleId',
			--SRD.[StudyId]							'studyId',
			HTT.[TemplateId]						'templateId',
			SRD.[TemplateThemeId]					'templateThemeId',
			HTT.[Category1]							'category1',
			HTT.[Category2]							'category2',
			HTT.[Description]						'description',
			SRD.[TemplateThemeItemId]				'templateThemeItemId',
			SRD.[TranscriptionId]					'transcriptionId',
			COUNT(*)								'ocurrencies',
			COUNT(
				CASE WHEN SRD.[CreatedDate] >= @StartDate THEN 1 ELSE NULL END
			)										'latestOcurrencies'
		FROM [dbo].[StudyResultSummaryDetail] SRD (NOLOCK)
		INNER JOIN [dbo].[Study] S (nolock)
			ON S.[VCC] = SRD.[VCC]
				AND S.[Name] = SRD.[StudyId]
		INNER JOIN [dbo].[HistoricTemplateTheme] HTT (NOLOCK)
			ON HTT.[VCC] = SRD.[VCC]
				AND HTT.[StudyId] = SRD.[StudyId]
				AND HTT.[ID] = SRD.[TemplateThemeId]
		WHERE S.[VCC] = @VCC
			AND SRD.[CreatedDate] BETWEEN @StartDate AND @EndDate
			AND (
				(
					@BatchScheduledIds <> ''
					AND
					S.[BatchScheduleId] IN (SELECT VALUE FROM [dbo].[split](@BatchScheduledIds, ','))
				)
				OR 
				(
					@BatchScheduledIds = ''
					AND 
					S.[BatchScheduleId] IS NOT NULL
				)
			)
		GROUP BY SRD.[VCC],
					S.[BatchScheduleId],
					--SRD.[StudyId],
					HTT.[TemplateId],
					SRD.[TemplateThemeId],
					HTT.[Category1],
					HTT.[Category2],
					HTT.[Description],
					SRD.[TemplateThemeItemId],
					SRD.[TranscriptionId]
	
	/*
	-- OBTENEMOS LOS RESULTADOS RECIENTES
	SELECT SRD.[VCC]								'vcc',
			S.[BatchScheduleId]						'batchScheduleId',
			SRD.[StudyId]							'studyId',
			HTT.[TemplateId]						'templateId',
			SRD.[TemplateThemeId]					'templateThemeId',
			HTT.[Category1]							'category1',
			HTT.[Category2]							'category2',
			HTT.[Description]						'description',
			SRD.[TemplateThemeItemId]				'templateThemeItemId',
			SRD.[TranscriptionId]					'transcriptionId',
			SRD.[CreatedDate]						'createdDate'
		FROM [dbo].[StudyResultSummaryDetail] SRD (NOLOCK)
		INNER JOIN [dbo].[Study] S (nolock)
			ON S.[VCC] = SRD.[VCC]
				AND S.[Name] = SRD.[StudyId]
		INNER JOIN [dbo].[HistoricTemplateTheme] HTT (NOLOCK)
			ON HTT.[VCC] = SRD.[VCC]
				AND HTT.[StudyId] = SRD.[StudyId]
				AND HTT.[ID] = SRD.[TemplateThemeId]
		WHERE S.[VCC] = @VCC
			AND (
				(
					@BatchScheduleId IS NOT NULL
					AND
					S.[BatchScheduleId] = @BatchScheduleId 
				)
				OR 
				(
					@BatchScheduleId IS NULL
					AND 
					S.[BatchScheduleId] IS NOT NULL
				)
			)
			AND SRD.[CreatedDate] >= @StartDate
	*/

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchScheduleExecutionsByBatchType] (
	@VCC VARCHAR(50) = NULL, 
	@BatchScheduleId VARCHAR(50) = NULL,
	@BatchType VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- SI NO RECIBIMOS EL TIPO, LOS CONSULTAMOS A LA TABLA DE AGENDAS
	IF @BatchType IS NULL
		SELECT @BatchType  = BS.[BatchType] 
			FROM [dbo].[BatchSchedule] BS (NOLOCK)
			WHERE BS.[VCC] = @VCC 
				AND BS.[Id] = @BatchScheduleId

	-- EJECUTAMOS LA CONSULTA SEGUN EL TIPO DE BATCH
	IF @BatchType = 'TRANSCRIPTION'
	BEGIN
		-- HACEMOS LA CONSULTA DE BATCHES CORRESPONDIENTES A LA AGENDA
		SELECT BSE.[VCC]										'vcc',
					BSE.[Id]									'id', 
					BSE.[BatchScheduleId]						'batchScheduleId', 
					BSE.[BatchId]								'batchId', 
					BSE.[ProcessId]								'processId',
					BSE.[ExecutionDate]							'executionDate', 
					BS.[ExecutionDelay]							'executionDelay',
					BSE.[Order]									'order', 
					BSE.[Status]								'status', 
					--BSE.[CreatedDate]							'createdDate',
					--BSE.[CreatedByUserId]						'createdByUserId',
					--BSE.[LastModifiedDate]						'lastModifiedDate',
					--BSE.[LastModifiedByUserId]					'lastModifiedByUserId',
					P.[Name]									'name',
					P.[Description]								'description',
					P.[ClientId]								'clientId',
					P.[Percentage]								'percentage',
					P.[StatusDescription]						'statusDescription',
					(
						CASE WHEN P.[Status] IS NULL 
							THEN 'UNKNOWN' 
							ELSE P.[Status] END
					)											'batchStatus',
					P.[StartDate]								'startDate',
					P.[EndDate]									'endDate',
					P.[MinDurationTime]							'minDurationTime',
					P.[MaxDurationTime]							'maxDurationTime',
					P.[Language]								'language',
					P.[Environment]								'environment',
					P.[CreatedDate]								'createdDate',
					P.[CreatedByUserId]							'createdByUserId',
					P.[LastModifiedDate]						'lastModifiedDate',
					P.[LastModifiedByUserId]					'lastModifiedByUserId'
			FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
			INNER JOIN [dbo].[BatchSchedule] BS (NOLOCK)
				ON BS.[VCC] = BSE.[VCC]
					AND BS.[Id] = BSE.[BatchScheduleId]
			LEFT JOIN [dbo].[Process] P (NOLOCK)
				ON P.[VCC] = BSE.[VCC]
					AND P.[Name] = BSE.[BatchId]
			WHERE BSE.[VCC] = @VCC
				AND BSE.[BatchScheduleId] = @BatchScheduleId
				AND BSE.[Status] <> 'PENDING'
			ORDER BY BSE.[ExecutionDate] DESC
	END
	ELSE IF @BatchType = 'TEMPLATE'
	BEGIN 
		-- HACEMOS LA CONSULTA DE BATCHES CORRESPONDIENTES A LA AGENDA
		SELECT BSE.[VCC]										'vcc',
					BSE.[Id]									'id', 
					BSE.[BatchScheduleId]						'batchScheduleId', 
					BSE.[BatchId]								'batchId', 
					BS.[ProcessFiltersId]						'templateId',
					BSE.[ProcessId]								'processId',
					BSE.[ExecutionDate]							'executionDate', 
					BS.[ExecutionDelay]							'executionDelay',
					BSE.[Order]									'order', 
					BSE.[Status]								'status', 
					--BSE.[CreatedDate]							'createdDate',
					--BSE.[CreatedByUserId]						'createdByUserId',
					--BSE.[LastModifiedDate]						'lastModifiedDate',
					--BSE.[LastModifiedByUserId]					'lastModifiedByUserId',
					S.[Name]									'name',
					S.[Description]								'description',
					S.[Percentage]								'percentage',
					S.[StatusDescription]						'statusDescription',
					(
						CASE WHEN S.[Status] IS NULL 
							THEN 'UNKNOWN' 
							WHEN S.[Status] = 1
							THEN 
								'COMPLETE'
							ELSE 'IN_PROCESS'
							END
					)											'batchStatus',
					S.[CreatedDate]								'createdDate',
					S.[CreatedByUserId]							'createdByUserId',
					S.[LastModifiedDate]						'lastModifiedDate',
					S.[LastModifiedByUserId]					'lastModifiedByUserId'
			FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
			INNER JOIN [dbo].[BatchSchedule] BS (NOLOCK)
				ON BS.[VCC] = BSE.[VCC]
					AND BS.[Id] = BSE.[BatchScheduleId]
			LEFT JOIN [dbo].[Study] S (NOLOCK)
				ON S.[VCC] = BSE.[VCC]
					AND S.[Name] = BSE.[BatchId]
			WHERE BSE.[VCC] = @VCC
				AND BSE.[BatchScheduleId] = @BatchScheduleId
				AND BSE.[Status] <> 'PENDING'
			ORDER BY BSE.[ExecutionDate] DESC
	END
	ELSE IF @BatchType = 'EVALUATION_PLAN'
	BEGIN
		-- HACEMOS LA CONSULTA DE BATCHES CORRESPONDIENTES A LA AGENDA
		SELECT BSE.[VCC]										'vcc',
					BSE.[Id]									'id', 
					BSE.[BatchScheduleId]						'batchScheduleId', 
					BSE.[BatchId]								'batchId', 
					BS.[ProcessFiltersId]						'templateId',
					BSE.[ProcessId]								'processId',
					BSE.[ExecutionDate]							'executionDate', 
					BS.[ExecutionDelay]							'executionDelay',
					BSE.[Order]									'order', 
					BSE.[Status]								'status', 
					--BSE.[CreatedDate]							'createdDate',
					--BSE.[CreatedByUserId]						'createdByUserId',
					--BSE.[LastModifiedDate]						'lastModifiedDate',
					--BSE.[LastModifiedByUserId]					'lastModifiedByUserId',
					EP.[ID]										'evaluationPlanId',
					EP.[Description]							'description',
					EP.[Percentage]								'percentage',
					EP.[Evaluation_Mode]						'batchMode',
					EP.[StatusDescription]						'statusDescription',
					(
						CASE WHEN EP.[Status] IS NULL 
							THEN 'UNKNOWN' 
							WHEN EP.[Status] IN (7, 9)
							THEN 
								'COMPLETE'
							ELSE 'IN_PROCESS'
							END
					)											'batchStatus',
					EP.[CreatedDate]							'createdDate',
					EP.[CreatedByUserId]						'createdByUserId',
					EP.[LastModifiedDate]						'lastModifiedDate',
					EP.[LastModifiedByUserId]					'lastModifiedByUserId'
			FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
			INNER JOIN [dbo].[BatchSchedule] BS (NOLOCK)
				ON BS.[VCC] = BSE.[VCC]
					AND BS.[Id] = BSE.[BatchScheduleId]
			LEFT JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
				ON EP.[Instance] = BSE.[VCC]
					AND EP.[ID] = BSE.[BatchId]
			WHERE BSE.[VCC] = @VCC
				AND BSE.[BatchScheduleId] = @BatchScheduleId
				AND BSE.[Status] <> 'PENDING'
			ORDER BY BSE.[ExecutionDate] DESC
	END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchScheduleExecutionsByCriteria] (
	@VCC VARCHAR(50) = NULL, 
	@SearchCriteria VARCHAR(50) = NULL,
	@SearchExpression VARCHAR(500) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	DECLARE @SQL VARCHAR(MAX) = ''
	-- DECLARAMOS CADENAS PARA EL QUERY FINAL
	DECLARE @SQL_SELECT VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''
    DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_FIXED_PART     VARCHAR(MAX) = ''

	SET @SQL_SELECT = '
		SELECT BSE.[VCC]									''vcc'',
				BSE.[Id]									''id'', 
				BSE.[BatchScheduleId]						''batchScheduleId'', 
				BSE.[BatchId]								''batchId'', 
				BSE.[ProcessId]								''processId'', 
				BSE.[ExecutionDate]							''executionDate'', 
				BS.[DaysToExpire]							''daysToExpire'',
				BS.[ExecutionDelay]							''executionDelay'',
				BSE.[Order]									''order'', 
				BSE.[Status]								''status'', 
				BSE.[CreatedDate]							''createdDate'',
				BSE.[CreatedByUserId]						''createdByUserId'',
				BSE.[LastModifiedDate]						''lastModifiedDate'',
				BSE.[LastModifiedByUserId]					''lastModifiedByUserId'',
				RN = ROW_NUMBER() OVER(PARTITION BY BSE.[VCC],
													BSE.[BatchScheduleId]
										ORDER BY BSE.[ExecutionDate] DESC) '

	-- ARMAMOS LA PARTE DE TABLAS 
	SET @SQL_FROM += '
			FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
			INNER JOIN [dbo].[BatchSchedule] BS (NOLOCK)
				ON BS.[VCC] = BSE.[VCC]
					AND BS.[Id] = BSE.[BatchScheduleId] '

	-- ARMAMOS LA PARTE BASE DE FILTROS
	SET @FILTERS_FIXED_PART = '
			WHERE BSE.[Id] IS NOT NULL '

	-- 
	IF @VCC IS NOT NULL
		SET @FILTERS_OPTIONAL_PART += '
				AND BSE.[VCC] = ''' + @VCC + ''' '

	/* DEPRECATED
	-- CUANDO SE HACE BUSQUEDA POR ULTIMA EJECUCIONES
	IF @SearchCriteria = 'ALREADY_EXECUTED_BY_BS_ID'
		SET @FILTERS_OPTIONAL_PART += '
				AND BSE.[BatchScheduleId] = ''' + @SearchExpression + ''' 
				AND BSE.[Status] <> ''PENDING'' '
	*/
	
	-- CUANDO SE HACE BUSQUEDA POR NO EJECUTADAS
	IF @SearchCriteria = 'NO_EXECUTED_BY_SCHEDULE_ID'
	SET @FILTERS_OPTIONAL_PART += '
				AND BSE.[BatchScheduleId] IN (SELECT Value FROM [dbo].[split](''' + @SearchExpression + ''','','')) 
				AND BSE.[Status] IN (''PENDING'', ''REGISTERED_ON_REDIS'') '

	-- CUANDO SE HACE BUSQUEDA MULTIPLE POR IDS
	IF @SearchCriteria = 'NOT_CREATED_BY_BS_ID'
		SET @FILTERS_OPTIONAL_PART += '
				AND BSE.[BatchScheduleId] IN (SELECT Value FROM [dbo].[split](''' + @SearchExpression + ''','',''))
				AND BSE.[BatchId] IS NULL 
				AND BSE.[Status] NOT LIKE ''CANCELLED_%'' '

	-- CUANDO SE HACE BUSQUEDA POR ULTIMA EJECUCIONES
	IF @SearchCriteria = 'LAST_CREATED_EXECUTIONS'
	SET @FILTERS_OPTIONAL_PART += '
				AND BSE.[BatchScheduleId] IN (SELECT Value FROM [dbo].[split](''' + @SearchExpression + ''','','')) '
	
	-- EL FILTRO POR DEFECTO
	IF @SQL_EXTRA IS NULL OR @SQL_EXTRA  = ''
		SET @SQL_EXTRA += '
			ORDER BY BSE.[ExecutionDate] ASC, BSE.[BatchScheduleId] '

	
	
	-- CUANDO SE HACE BUSQUEDA POR ULTIMA EJECUCIONES
	IF @SearchCriteria = 'LAST_CREATED_EXECUTIONS'
	BEGIN
		-- UNIMOS LAS PARTES DEL QUERY
		SET @SQL_SELECT += @SQL_FROM + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART
		
		--
		SET @SQL = ';WITH CTE AS (' + @SQL_SELECT + ')'

	
		SET @SQL += 'SELECT * FROM CTE BSE WHERE RN = 1 ORDER BY BSE.[ExecutionDate] DESC, BSE.[BatchScheduleId] '
	END
	ELSE
	BEGIN
		-- UNIMOS LAS PARTES DEL QUERY
		SET @SQL_SELECT += @SQL_FROM + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART + @SQL_EXTRA
		SET @SQL += @SQL_SELECT
	END

	-- EJECUTAMOS LA QUERY
	EXEC (@SQL)

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchSchedules] (
	@VCC VARCHAR(50) = NULL, 
	@SearchCriteria VARCHAR(50) = NULL,
	@SearchExpression VARCHAR(500) = NULL,
	@ExtendedContent BIT = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	DECLARE @SQL NVARCHAR(MAX)
	SET @SQL = '
		SET NOCOUNT ON
			
		SELECT BS.[VCC]									''vcc'',
				BS.[Id]									''id'',
				BS.[BatchType]							''batchType'',
				BS.[Frequency]							''frequency'',
				BS.[Weekday]							''weekDay'',
				BS.[Monthday]							''monthDay'',
				BS.[DaysInterval]						''daysInterval'',
				BS.[HoursInterval]						''hoursInterval'',
				BS.[DaysToExpire]						''daysToExpire'',
				BS.[StartDate]							''startDate'',
				BS.[EndDate]							''endDate'',
				BS.[FirstExecutionDate]					''firstExecutionDate'',
				BS.[LastExecutionDate]					''lastExecutionDate'',
				BS.[FinalExecutionDate]					''finalExecutionDate'', '

	IF @ExtendedContent = 1
		SET @SQL += '
				-- BS.[ExecutionList]						''executionList'',
				BS.[BatchData]							''batchData'', 
				BS.[ProcessFiltersId]					''processFiltersId'', '
				
	SET @SQL += '
				BS.[CreatedDate]						''createdDate'',
				BS.[CreatedByUserId]					''createdByUserId'',
				BS.[LastModifiedDate]					''lastModifiedDate'',
				BS.[LastModifiedByUserId]				''lastModifiedByUserId''
			FROM [dbo].[BatchSchedule] BS (NOLOCK)
			WHERE BS.[Id] IS NOT NULL '

	IF @VCC IS NOT NULL
		SET @SQL += '
				AND BS.[VCC] = ''' + @VCC + ''' '

	IF @SearchCriteria = 'NAME'
		SET @SQL += '
				AND BS.[Id] = ''' + @SearchExpression + ''' '

	IF @SearchCriteria = 'STATUS'
		SET @SQL += '
				AND BS.[Status] IN (SELECT Value FROM [dbo].[split](''' + @SearchExpression + ''','','')) '


	--select * FROM [dbo].[BatchSchedule] BS (NOLOCK)
	SET @SQL += '
			ORDER BY BS.[CreatedDate] DESC, BS.[Id]

		SET NOCOUNT OFF'

	-- EJECUTAMOS LA QUERY
	EXEC (@SQL)

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBatchSchedulesSummary] (
	@VCC VARCHAR(50) = NULL, 
	@SearchCriteria VARCHAR(50) = NULL,
	@SearchExpression VARCHAR(500) = NULL,
	@BatchType VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- CREAMOS UNA TABLA TEMPORAL PARA LA ULTIMA EJECUCION DE CADA AGENDA
	CREATE TABLE #TMP_SCHEDULE_LAST_EXECUTION (
		[VCC] VARCHAR(50),
		[BatchScheduleId] VARCHAR(50),
		[BatchId] VARCHAR(50),
		[LastExecutionDate] DATETIME,
		[Status] VARCHAR(50)
	)
	-- BUSCAMOS LAS ULTIMAS EJECUCIONES DE CADA AGENDA
	;WITH CTE_LAST_VALID_EXECUTIONS AS (
		SELECT BSE.[VCC],
				BSE.[BatchScheduleId],
				BSE.[BatchId],
				BSE.[ExecutionDate],
				BSE.[Status],
				RN = ROW_NUMBER() OVER(PARTITION BY BSE.[VCC],
													BSE.[BatchScheduleId]
										ORDER BY BSE.[ExecutionDate] DESC)
			FROM [dbo].[BatchSchedule] BS (NOLOCK)
			INNER JOIN [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				ON BSE.[VCC] = BS.[VCC]
					AND BSE.[BatchScheduleId] = BS.[Id]
			WHERE BS.[VCC] = @VCC
				AND BS.[BatchType] = @BatchType
				AND BS.[Status] IN ('CREATED', 'RUNNING', 'PAUSED', 'FINISHED')
				AND BSE.[Status] NOT IN ('PENDING', 'REGISTERED_ON_REDIS', 'READY_TO_EXECUTE', 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
	)
	INSERT INTO #TMP_SCHEDULE_LAST_EXECUTION
		SELECT [VCC],
				[BatchScheduleId],
				[BatchId],
				[ExecutionDate],
				[Status]
			FROM CTE_LAST_VALID_EXECUTIONS 
			WHERE RN = 1

			--SELECT * FROM [dbo].[BatchScheduleExecution] (NOLOCK) WHERE [BatchScheduleId] = '20211030_0141_prueba'

	-- CREAMOS UNA TABLA TEMPORAL PARA LA PROXIMA EJECUCION DE CADA AGENDA
	CREATE TABLE #TMP_SCHEDULE_NEXT_EXECUTION (
		VCC VARCHAR(50),
		BatchScheduleId VARCHAR(50),
		NextExecutionDate DATETIME
	)
	-- BUSCAMOS LAS ULTIMAS EJECUCIONES DE CADA AGENDA
	;WITH CTE_NEXT_EXECUTIONS AS (
		SELECT BSE.[VCC],
				BSE.[BatchScheduleId],
				BSE.[BatchId],
				BSE.[ExecutionDate],
				BSE.[Status],
				RN = ROW_NUMBER() OVER(PARTITION BY BSE.[VCC],
													BSE.[BatchScheduleId]
										ORDER BY BSE.[ExecutionDate] ASC)
			FROM [dbo].[BatchSchedule] BS (NOLOCK)
			INNER JOIN [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				ON BSE.[VCC] = BS.[VCC]
					AND BSE.[BatchScheduleId] = BS.[Id]
			WHERE BS.[VCC] = @VCC
				AND BS.[BatchType] = @BatchType
				AND BSE.[Status] IN ('PENDING', 'REGISTERED_ON_REDIS')
	)
	INSERT INTO #TMP_SCHEDULE_NEXT_EXECUTION
		SELECT [VCC],
				[BatchScheduleId],
				[ExecutionDate]
			FROM CTE_NEXT_EXECUTIONS 
			WHERE RN = 1



	-- BUSCAMOS LOS RESUMENES DE LAS EJECUCIONES
	;WITH CTE_SUMMARY AS (
		SELECT BS.[VCC], 
				BS.[Id]																							'BatchScheduleId',	
				COUNT(BSE.[Id])																					'Total',
				COUNT(IIF(BSE.[STATUS] LIKE 'REGISTERED_%', 1, NULL))											'Preparing',
				COUNT(IIF(BSE.[STATUS] LIKE '%_CREATED' OR BSE.[STATUS] LIKE '%_STARTED', 1, NULL))				'Running',
				COUNT(IIF(BSE.[STATUS] LIKE 'CANCELLED_%', 1, NULL))											'Cancelled',
				COUNT(IIF(BSE.[STATUS] LIKE 'COMPLETED%' OR BSE.[STATUS] = 'FAILED', 1, NULL))					'Finished',
				COUNT(IIF(BSE.[STATUS] = 'COMPLETED', 1, NULL))													'Succeed',
				COUNT(IIF(BSE.[STATUS] LIKE '%_WITH_ERRORS', 1, NULL))											'Error',
				COUNT(IIF(BSE.[STATUS] LIKE '%FAILED%', 1, NULL))												'Failed'
			FROM [dbo].[BatchSchedule] BS (NOLOCK)
			LEFT JOIN [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				ON BSE.[VCC] = BS.[VCC]
					AND BSE.[BatchScheduleId] = BS.[Id]
			WHERE BS.[VCC] = @VCC
				AND BS.[BatchType] = @BatchType
				AND BS.[Status] IN ('CREATED', 'RUNNING', 'PAUSED', 'FINISHED')
				AND BSE.[Status] <> 'PENDING'
			GROUP BY BS.[VCC], BS.[Id]
	) -- Select * FROM [dbo].[BatchSchedule] BS (NOLOCK)
		SELECT BS.[VCC]																	'vcc',
				BS.[Id]																	'id',
				BS.[BatchType]															'batchType',
				CASE WHEN BS.[BatchType] IN ('TEMPLATE', 'EVALUATION_PLAN') THEN BS.[BatchData]
					ELSE '' END															'batchData',
				BS.[Frequency]															'frequency',
				-- BS.[Weekday]															'weekday',
				-- BS.[Monthday]														'monthday',
				BS.[DaysInterval]														'daysInterval',
				BS.[HoursInterval]														'hoursInterval',
				BS.[StartDate]															'startDate',
				BS.[EndDate]															'endDate',
				BS.[FirstExecutionDate]													'firstExecutionDate',
				BS.[Status]																'status',
				-- BS.[LastExecutionDate]												'lastExecutionDate',
				-- BS.[FinalExecutionDate]												'finalExecutionDate', 
				ISNULL(CTE_S.[Total], 0)												'total',
				ISNULL(CTE_S.[Preparing], 0)											'preparing',
				ISNULL(CTE_S.[Running], 0)												'running',
				ISNULL(CTE_S.[Cancelled], 0)											'cancelled',
				ISNULL(CTE_S.[Finished], 0)												'finished',
				ISNULL(CTE_S.[Succeed], 0)												'succeed',
				ISNULL(CTE_S.[Error], 0)												'error',
				ISNULL(CTE_S.[Failed], 0)												'failed',
				TMP_LE.[LastExecutionDate]												'lastExecutionDate',
				TMP_LE.[Status]															'lastExecutionStatus',
				TMP_NE.[NextExecutionDate]												'nextExecutionDate',
				BS.[CreatedDate]														'createdDate',
				BS.[CreatedByUserId]													'createdByUserId',
				BS.[LastModifiedDate]													'lastModifiedDate',
				BS.[LastModifiedByUserId]												'lastModifiedByUserId'
			FROM [dbo].[BatchSchedule] BS (NOLOCK)
			LEFT JOIN CTE_SUMMARY CTE_S
				ON CTE_S.[VCC] = BS.[VCC]
					AND CTE_S.[BatchScheduleId] = BS.[Id]
			LEFT JOIN #TMP_SCHEDULE_LAST_EXECUTION TMP_LE 
				ON TMP_LE.[VCC] = BS.[VCC]
					AND TMP_LE.[BatchScheduleId] = BS.[Id]
			LEFT JOIN #TMP_SCHEDULE_NEXT_EXECUTION TMP_NE 
				ON TMP_NE.[VCC] = BS.[VCC]
					AND TMP_NE.[BatchScheduleId] = BS.[Id]
			WHERE BS.[VCC] = @VCC
				AND BS.[BatchType] = @BatchType
				AND ISNULL(BS.[Id], '') LIKE CASE WHEN @SearchCriteria = 'NAME' THEN '%' + @SearchExpression + '%' ELSE '%' END

	-- ELIMINAMOS LAS TABLAS TEMPORALES
	IF OBJECT_ID('tempdb..#TMP_SCHEDULE_LAST_EXECUTION') IS NOT NULL DROP TABLE #TMP_SCHEDULE_LAST_EXECUTION
	IF OBJECT_ID('tempdb..#TMP_SCHEDULE_NEXT_EXECUTION') IS NOT NULL DROP TABLE #TMP_SCHEDULE_NEXT_EXECUTION
		
	

	/*
	-- BUSCAMOS LAS ULTIMAS EJECUCIONES DE CADA AGENDA
	;WITH CTE_LAST_VALID_EXECUTIONS AS (
		SELECT BSE.[VCC],
				BSE.[BatchScheduleId],
				BSE.[BatchId],
				BSE.[ExecutionDate],
				BSE.[Status],
				RN = ROW_NUMBER() OVER(PARTITION BY BSE.[VCC],
													BSE.[BatchScheduleId]
										ORDER BY BSE.[ExecutionDate] DESC)
			FROM [dbo].[BatchSchedule] BS (NOLOCK)
			INNER JOIN [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				ON BSE.[VCC] = BS.[VCC]
					AND BSE.[BatchScheduleId] = BS.[Id]
			WHERE BS.[VCC] = @VCC
				AND BS.[Status] IN ('CREATED', 'RUNNING', 'PAUSED')
				AND BSE.[Status] NOT IN ('PENDING', 'REGISTERED_ON_REDIS', 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
				-- AND BSE.[Status] = 'COMPLETED'
	),
	CTE_LAST_EXECUTIONS AS (
		SELECT CTE_VE.[VCC], 
				CTE_VE.[BatchScheduleId], 
				CTE_VE.[BatchId], 
				CASE WHEN CTE_VE.[Status] = 'REGISTERED_ON_REDIS' THEN NULL ELSE CTE_VE.[ExecutionDate] END		'LastExecutionDate', 
				CTE_VE.[Status]
			FROM CTE_LAST_VALID_EXECUTIONS CTE_VE
			WHERE CTE_VE.[RN] = 1
	),
	CTE_NEXT_VALID_EXECUTIONS AS (
		SELECT BSE.[VCC],
				BSE.[BatchScheduleId],
				BSE.[BatchId],
				BSE.[ExecutionDate],
				BSE.[Status],
				CTE_LA.[LastExecutionDate],
				RN = ROW_NUMBER() OVER(PARTITION BY BSE.[VCC],
													BSE.[BatchScheduleId]
										ORDER BY BSE.[ExecutionDate] ASC)
			FROM [dbo].[BatchSchedule] BS (NOLOCK)
			INNER JOIN [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				ON BSE.[VCC] = BS.[VCC]
					AND BSE.[BatchScheduleId] = BS.[Id]
			INNER JOIN CTE_LAST_EXECUTIONS CTE_LA
				ON CTE_LA.[VCC] = BS.[VCC]
					AND CTE_LA.[BatchScheduleId] = BS.[Id]
			WHERE BS.[VCC] = @VCC
				AND BSE.[Status] = 'PENDING'
	),
	CTE_NEXT_EXECUTIONS AS (
		SELECT CTE_VE.[VCC], 
				CTE_VE.[BatchScheduleId], 
				CTE_VE.[BatchId], 
				CTE_VE.[ExecutionDate]																			'NextExecutionDate', 
				CTE_VE.[LastExecutionDate]																		'LastExecutionDate', 
				CTE_VE.[Status]
			FROM CTE_NEXT_VALID_EXECUTIONS CTE_VE
			WHERE CTE_VE.[RN] = 1
	),
	CTE_SUMMARY AS (
		SELECT BS.[VCC], 
				BS.[Id]																							'BatchScheduleId',	
				CTE_NE.[BatchId], 
				CTE_NE.[NextExecutionDate], 
				CTE_NE.[LastExecutionDate], 
				CTE_NE.[Status],
				COUNT(BSE.[Id])																					'Total',
				COUNT(IIF(BSE.[STATUS] LIKE 'REGISTERED_%', 1, NULL))											'Preparing',
				COUNT(IIF(BSE.[STATUS] LIKE '%_CREATED' OR BSE.[STATUS] LIKE '%_STARTED', 1, NULL))				'Running',
				COUNT(IIF(BSE.[STATUS] LIKE 'CANCELLED_%', 1, NULL))											'Cancelled',
				COUNT(IIF(BSE.[STATUS] LIKE 'COMPLETED%' OR BSE.[STATUS] = 'FAILED', 1, NULL))					'Finished',
				COUNT(IIF(BSE.[STATUS] = 'COMPLETED', 1, NULL))													'Succeed',
				COUNT(IIF(BSE.[STATUS] LIKE '%_WITH_ERRORS', 1, NULL))											'Error',
				COUNT(IIF(BSE.[STATUS] LIKE '%FAILED%', 1, NULL))												'Failed'
			FROM [dbo].[BatchSchedule] BS (NOLOCK)
			LEFT JOIN [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				ON BSE.[VCC] = BS.[VCC]
					AND BSE.[BatchScheduleId] = BS.[Id]
			LEFT JOIN CTE_NEXT_EXECUTIONS CTE_NE 
				ON CTE_NE.[VCC] = BS.[VCC]
					AND CTE_NE.[BatchScheduleId] = BS.[Id]
			WHERE BS.[VCC] = @VCC
				AND BS.[Status] IN ('CREATED', 'RUNNING', 'PAUSED')
				AND BSE.[Status] <> 'PENDING'
			GROUP BY BS.[VCC], BS.[Id], CTE_NE.[BatchId], CTE_NE.[NextExecutionDate], CTE_NE.[LastExecutionDate], CTE_NE.[Status]
	)-- select * from CTE_SUMMARY
		SELECT BS.[VCC]																	'vcc',
				BS.[Id]																	'id',
				BS.[BatchType]															'batchType',
				BS.[Frequency]															'frequency',
				--BS.[Weekday]															'weekday',
				--BS.[Monthday]															'monthday',
				BS.[DaysInterval]														'daysInterval',
				BS.[HoursInterval]														'hoursInterval',
				BS.[StartDate]															'startDate',
				BS.[EndDate]															'endDate',
				BS.[FirstExecutionDate]													'firstExecutionDate',
				BS.[Status]																'status',
				--BS.[LastExecutionDate]													'lastExecutionDate',
				-- BS.[FinalExecutionDate]													'finalExecutionDate', 
				ISNULL(CTE_S.[Total], 0)												'total',
				ISNULL(CTE_S.[Preparing], 0)											'preparing',
				ISNULL(CTE_S.[Running], 0)												'running',
				ISNULL(CTE_S.[Cancelled], 0)											'cancelled',
				ISNULL(CTE_S.[Finished], 0)												'finished',
				ISNULL(CTE_S.[Succeed], 0)												'succeed',
				ISNULL(CTE_S.[Error], 0)												'error',
				ISNULL(CTE_S.[Failed], 0)												'failed',
				CTE_S.[LastExecutionDate]												'lastExecutionDate',
				CTE_S.[Status]															'lastExecutionStatus',
				CTE_S.[NextExecutionDate]												'nextExecutionDate',
				BS.[CreatedDate]														'createdDate',
				BS.[CreatedByUserId]													'createdByUserId',
				BS.[LastModifiedDate]													'lastModifiedDate',
				BS.[LastModifiedByUserId]												'lastModifiedByUserId'
			FROM [dbo].[BatchSchedule] BS (NOLOCK)
			LEFT JOIN CTE_SUMMARY CTE_S
				ON CTE_S.[VCC] = BS.[VCC]
					AND CTE_S.[BatchScheduleId] = BS.[Id]
			LEFT JOIN [dbo].[Process] P (NOLOCK)
				ON BS.[BatchType] = 'TRANSCRIPTION'
					AND P.[VCC] = CTE_S.[VCC]
					AND P.[Name] = CTE_S.[BatchId]
			LEFT JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
				ON BS.[BatchType] = 'EVALUATION_PLAN'
					AND EP.[Instance] = CTE_S.[VCC]
					AND EP.[ID] = CTE_S.[BatchId]
			WHERE BS.[VCC] = @VCC
				AND ISNULL(BS.[Id], '') LIKE CASE WHEN @SearchCriteria = 'NAME' THEN '%' + @SearchExpression + '%' ELSE '%' END
	*/

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetBillingInteractions] (
	@VCC					VARCHAR(50) = NULL,
	@SecondsPerRequest		DECIMAL(18, 5) = 0,
	@CharsPerPhrase			DECIMAL(18, 5) = 0,
	@Channel				VARCHAR(25) = NULL,
	@Retranscription		BIT = 0
	
)
AS
BEGIN
	--select * into [TMP_PROCESS_HISTORY] from [#InteractionsTemp]
	--select * into [#InteractionsTemp] from [TMP_PROCESS_HISTORY]
	
	-- SEGUN EL TIPO DE CANAL DEFINIMOS LA FUNCION
	IF @Channel = 'CALL'
	BEGIN
		-- DECLARAMOASUN CADENA PARA EL QUERY FINAL
		DECLARE @VCC_TYPE VARCHAR(50) = ''
		
		-- RECUPERAMOS EL TIPO DE VCC
		SELECT @VCC_TYPE = V.[Type] FROM [dbo].[VCC] V (NOLOCK) WHERE V.[VCC] = @VCC

		-- CREAMOS UNA TEMPORAL PARA AUDIO CON SU DEFINICION
		CREATE TABLE #INNER_AUDIO (
			[VCC] VARCHAR(50),
			[ID] VARCHAR(100),
			[Section] NUMERIC(5,0),
			[Part] NUMERIC(5,0), 
			[DurationTime] NUMERIC(12,0), 
			[TotalPhrases] INT,
			[IsProcess] BIT,
			[IsTranscribed] BIT,
			[IsCancel] BIT
		)
		
		-- SEGUN EL TIPO DE VCC COMPLETAMOS EL FROM
		IF @VCC_TYPE <> 'I6' -- I6
		BEGIN
			--OBTENEMOS TODOS AUDIOS RELACIONADOS A LA BUSQUEDA
			INSERT INTO #INNER_AUDIO
				SELECT DISTINCT A.[VCC],
								A.[ID],
								A.[Section],
								A.[Part],
								A.[DurationTime],
								A.[TotalPhrases],
								A.[IsProcess],
								A.[IsTranscribed],
								A.[IsCancel]
					FROM [dbo].[Audio] A (NOLOCK)
					INNER JOIN [#InteractionsTemp] IT 
						ON IT.[VCC] = A.[VCC]
							AND IT.[InteractionId] = A.[ID]
					WHERE A.[VCC] = @VCC
		END
		ELSE
	    BEGIN
			-- GENERAMOS EL 1ER RESUMEN DE AUDIOS
			;WITH CTE_AUDIO AS (
				SELECT DISTINCT A.[VCC],
								A.[ID],
								A.[Section],
								A.[Part],
								A.[DurationTime],
								A.[TotalPhrases],
								A.[IsProcess],
								A.[IsTranscribed],
								A.[IsCancel],
								RN = ROW_NUMBER() OVER(PARTITION BY A.[ID] ORDER BY A.[EndDate] DESC)
					FROM [dbo].[Audio] A (NOLOCK)
					INNER JOIN [#InteractionsTemp] IT 
						ON IT.[VCC] = A.[VCC]
							AND IT.[InteractionId] = A.[ID]
							AND IT.[Section] = A.[Section]
							AND IT.[Part] = A.[Part]
					WHERE A.[VCC] = @VCC
			)

			--OBTENEMOS TODOS AUDIOS RELACIONADOS A LA BUSQUEDA
			INSERT INTO #INNER_AUDIO
				SELECT CTE.[VCC],
						CTE.[ID],
						CTE.[Section],
						CTE.[Part],
						CTE.[DurationTime],
						CTE.[TotalPhrases],
						CTE.[IsProcess],
						CTE.[IsTranscribed],
						CTE.[IsCancel] 
					FROM CTE_AUDIO CTE
					WHERE CTE.[RN] = 1
				
		END
		
		-- HACEMOS LA BUSQUEDA
		SELECT SUM(CASE WHEN A.[IsTranscribed] = 1 
								AND @Retranscription = 0 
							THEN 1 ELSE 0 END)															'alreadyTranscribedAudio',
				SUM(CASE WHEN A.[IsTranscribed] = 1 
								AND @Retranscription = 0 
							THEN A.[DurationTime] ELSE 0 END)											'alreadyTranscribedDurationTime',
				SUM(CASE WHEN (A.[IsProcess] = 1 
								AND A.[IsTranscribed] IS NULL 
								AND A.[IsCancel] IS NULL) 
							THEN 1 ELSE 0 END)															'inProcessAudio',
				SUM(CASE WHEN (A.[IsProcess] = 1 
								AND A.[IsTranscribed] IS NULL 
								AND A.[IsCancel] IS NULL) 
							THEN A.[DurationTime] ELSE 0 END)											'inProcessDurationTime',
				'CALL'																				'interactionType',
				SUM(CASE WHEN (
					A.[IsProcess] IS NULL 
					OR
					@Retranscription = 1
					OR 
					(
						A.[IsProcess] = 1 
						AND 
						A.[IsCancel] = 1
					)
				) THEN 1 ELSE 0 END)																	'notTranscribedAudio',
				SUM(CASE WHEN (
					A.[IsProcess] IS NULL 
					OR
					@Retranscription = 1
					OR (
						A.[IsProcess] = 1 
						AND 
						A.[IsCancel] = 1
					)
				--) THEN A.[DurationTime] ELSE 0 END)														'notTranscribedDurationTime',
				) THEN IT.[AttentionTime] ELSE 0 END)														'notTranscribedDurationTime',
				SUM(CASE WHEN A.[IsTranscribed] = 1  
								AND A.[TotalPhrases] IS NOT NULL
							THEN A.[TotalPhrases] ELSE 0 END)											'totalPhrases',
				SUM(CASE WHEN (A.[IsProcess] IS NULL 
								OR (A.[IsProcess] = 1 
									AND A.[IsCancel] = 1)
								OR (A.[TotalPhrases] IS NULL)) 
							THEN (IT.[AttentionTime] / @SecondsPerRequest) ELSE 0 END)					'totalRawPhrases',
				SUM(IT.[AttentionTime] / @SecondsPerRequest)											'averageModelRequest' 
			FROM [#InteractionsTemp] IT
			INNER JOIN [dbo].[VCC] V (NOLOCK)
				ON V.[VCC] = IT.[VCC]
			LEFT JOIN [#INNER_AUDIO] A 
				ON A.[VCC] = IT.[VCC]
					AND  A.[ID] = IT.[InteractionId]
					AND (
						(
							V.[Type] <> 'I6'
							AND 
							A.[Section] = IT.[Section]
							AND 
							A.[Part] = IT.[Part]
						)
						OR 
						(
							V.[Type] = 'I6'
						)
					)
			WHERE IT.[VCC] = @VCC

		-- ELIMINAMOS LA TABLA TEMPORAL
		DROP TABLE IF EXISTS #INNER_AUDIO
	END
	ELSE
	BEGIN
		-- DECLARAMOS UNA VARIABLE PARA PARA EL TOTALDE RESULTADOS
		DECLARE @TOTAL_INTERACTIONS INT = 0

		-- SETTEAMOS EL NUMERO DE REGISTROS A LA VARIABLE DE TOTAL
		SELECT @TOTAL_INTERACTIONS = COUNT(*) FROM [#InteractionsTemp]

		IF @TOTAL_INTERACTIONS = 0 
		BEGIN
			-- SI NO EXISTEN RESULTADOS EN HD LLENAMOSUN RESULTADOS CON 0
			SELECT @Channel																					'interactionType',
					0																						'totalInteractionDetails',
					0																						'alreadyTranscribedInteraction',
					0																						'inProcessInteraction',
					0																						'notTranscribedInteraction',
					0																						'totalPhrases',
					0																						'totalRawChars',
					0																						'totalRawPhrases'
		END 
		ELSE
		BEGIN
			-- CRUZAMOS LA TABLA TEMPORAL DE INTERACCIONES ASINCRONAS CON LA TABLA LOCAL INTERACTION PARA LAS INTERACCIONES ASINCRONAS YA REGISTRADAS
			SELECT ISNULL(IT.[Channel], @Channel)															'interactionType',
					COUNT(*)																				'totalInteractionDetails',
					SUM(CASE WHEN (AI.[VCC] IS NOT NULL 
						AND AI.[IsTranscribed] = 1) THEN 1 ELSE 0 END)										'alreadyTranscribedInteraction',
					0																						'inProcessInteraction',
					SUM(CASE WHEN (AI.[VCC] IS NULL 
						OR AI.[IsTranscribed] <> 1) THEN 1 ELSE 0 END)										'notTranscribedInteraction',
					SUM(ISNULL(AI.[TotalPhrases], 0))														'totalPhrases',
					-- CUANDO TENEMOS VALIDPHRASES, YA NO CONTAMOS LAS RAWCHAR 
					SUM(
						CASE WHEN (AI.[TotalPhrases] IS NULL 
								OR AI.[TotalPhrases] = 0)
						THEN (
							ISNULL(AI.[RawChar], ISNULL(IT.[rawChar], 0))
						)
						ELSE 0 END
					)																						'totalRawChars',
					SUM(
						CASE WHEN (AI.[TotalPhrases] IS NULL 
								OR AI.[TotalPhrases] = 0)
						THEN (
							-- SI RAWCHAR ES NULL, SE DEBE VALUAR COMO MINIMO 1 FRASE(@CharsPerPhrase)
							-- CEILING(ISNULL(AI.[RawChar], ISNULL(HDI.[rawChar], @CharsPerPhrase)) * 1.00 / @CharsPerPhrase)
							CEILING(ISNULL(AI.[RawChar], ISNULL(IT.[RawChar], 0)) * 1.00 / @CharsPerPhrase)
						)
						ELSE 0 END
					)																						'totalRawPhrases' 
				FROM [#InteractionsTemp] IT
				LEFT JOIN [dbo].[AsyncInteraction] AI 
					ON IT.[VCC] = AI.[VCC]
						AND IT.[Channel] = AI.[Channel]
						--AND IT.[Campaign] = AI.[Campaign]
						AND IT.[InteractionId] = AI.[ID]
						AND IT.[EndDate] = AI.[EndDate]
					--WHERE AI.[VCC] = @VCC
				GROUP BY IT.[VCC],
							IT.[Channel]

		END
	END

	-- ELIMINAMOS LA TABLA TEMPORAL
	DROP TABLE [#InteractionsTemp] 
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetCampaignEvaluationFormats] (
	@Instance Varchar(50) = NULL, 
	@IdEvaluationFormat Varchar(50) = NULL,
	@EvaluationPlanId   VARCHAR(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DECLARE @QUERY VARCHAR(MAX) = ''
	SET @QUERY += 'SELECT
				CE.[IdEvaluationFormat]			''idEvaluationFormat'',
				CE.[IdCampaign]					''idCampain'',
				CE.[IdCampaign]                 ''descriptionCampain'' FROM '

	IF(@EvaluationPlanId IS NOT NULL AND @EvaluationPlanId <> '')
	BEGIN
		 SET @QUERY +=' [HistoricCampaignEvaluationFormat] CE (NOLOCK)
			WHERE CE.[EvaluationPlanId] = '''+@EvaluationPlanId+''' AND  CE.[VCC] = '''+@Instance+''''
	END
	ELSE
	BEGIN
         SET @QUERY += ' [CampaignEvaluationFormat] CE (NOLOCK) WHERE  CE.[Instance] = '''+@Instance+'''' 
	END

	SET @QUERY += ' AND CE.[IdEvaluationFormat]='''+ @IdEvaluationFormat +''''

    EXEC(@QUERY)
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetCampaignFulfillment] (
	@Instance VARCHAR(50) = NULL,
	@InitialDate DATETIME = NULL,
	@EndDate DATETIME = NULL,
	@IdCampaign VARCHAR(MAX) = 'GetCampaignFulfillment',
	@IdEvaluationFormat VARCHAR(MAX) = 'GetCampaignFulfillment',
	@IdAgent VARCHAR(MAX) = 'GetCampaignFulfillment',
	@IdWorkGroup VARCHAR(MAX) = 'GetCampaignFulfillment',
	@IdAttentionLevel VARCHAR(MAX) = 'GetCampaignFulfillment',
	@IdEvaluationPlan VARCHAR(MAX) = 'GetCampaignFulfillment',
	@IdGroup VARCHAR(MAX) = 'GetCampaignFulfillment',
	@IdSubGroup VARCHAR(MAX) = 'GetCampaignFulfillment',
	@IdAbility VARCHAR(MAX) = 'GetCampaignFulfillment',
	@IdAuditor VARCHAR(MAX) = 'GetCampaignFulfillment'
)
AS
BEGIN
	DECLARE @INTERACTION_TOTAL INT = 0
	DECLARE @INTERACTION_APPROVED INT = 0
	DECLARE @INTERACTION_DISAPPROVED INT = 0
	DECLARE @INTERACTION_RATIO INT = 0
		
	
	SET NOCOUNT ON;

	WITH 

		-- DECLARAMOS UNA CTE PARA DEPOSITAR LAS LLAMADAS EVALUADAS CON SU ULTIMA FECHA DE EVALUACION
		CteTotalInteractionsEvaluated AS (
			SELECT
				ER.[IdCampaign] 
				, ER.[IdEvaluationPlan]
				, ER.[IdInteraction] 
				, ER.[EvaluationDate] LastEvaluationDate 
				, ER.[IdAuditor]
				, AEP.[Score]
				, ER.[MinQualificationAcceptable]
				, RN = ROW_NUMBER() OVER(PARTITION BY ER.[IdCampaign], ER.[IdEvaluationPlan], ER.[IdInteraction], ER.[IdAuditor] ORDER BY ER.[EvaluationDate] DESC)
			FROM
				EvaluationResult ER
			LEFT JOIN AuditorEvaluationPlan AEP 
				ON AEP.[Instance] = ER.[Instance] 
					AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan] 
					AND AEP.[IdInteraction] = ER.[IdInteraction]
					AND AEP.[IdAuditor] = ER.[IdAuditor]
			WHERE
				ER.Instance = @Instance
				AND ER.EvaluationDate BETWEEN @InitialDate AND @EndDate
				AND CASE WHEN @IdAgent = 'GetCampaignFulfillment' THEN 'GetCampaignFulfillment' ELSE ER.IdAgent END IN (SELECT Value FROM [dbo].[split](@IdAgent,','))
				AND CASE WHEN @IdCampaign = 'GetCampaignFulfillment' THEN 'GetCampaignFulfillment' ELSE ER.IdCampaign END IN (SELECT Value FROM [dbo].[split](@IdCampaign,','))
				AND CASE WHEN @IdEvaluationFormat = 'GetCampaignFulfillment' THEN 'GetCampaignFulfillment' ELSE ER.IdEvaluationFormat END IN (SELECT Value FROM [dbo].[split](@IdEvaluationFormat,','))
				AND CASE WHEN @IdEvaluationPlan = 'GetCampaignFulfillment' THEN 'GetCampaignFulfillment' ELSE ER.IdEvaluationPlan END IN (SELECT Value FROM [dbo].[split](@IdEvaluationPlan,','))
				AND ER.[IdAuditor] LIKE CASE WHEN @IdAuditor = 'GetCampaignFulfillment' THEN '%' ELSE 'SPEECH' END AND ER.IdAuditor <> CASE WHEN @IdAuditor = 'GetCampaignFulfillment' THEN 'SPEECH' ELSE 'B_I_G_C_H_E_E_S_E' END
		)
	
	SELECT 
		@INTERACTION_TOTAL = (
			SELECT COUNT(TEI.[IdInteraction])
			FROM CteTotalInteractionsEvaluated TEI 
			WHERE TEI.[RN] = 1
		),
		@INTERACTION_APPROVED = (
			SELECT COUNT(TEI.[IdInteraction])
				FROM CteTotalInteractionsEvaluated TEI 
				WHERE TEI.[RN] = 1
					AND TEI.[Score] >= TEI.[MinQualificationAcceptable]
			
		)
	
	
	IF (@INTERACTION_TOTAL > 0)
	BEGIN
		SET @INTERACTION_RATIO = 100 * @INTERACTION_APPROVED / @INTERACTION_TOTAL 
		SET @INTERACTION_DISAPPROVED = @INTERACTION_TOTAL - @INTERACTION_APPROVED

		SELECT @INTERACTION_DISAPPROVED AS 'disapproved',
			@INTERACTION_TOTAL AS 'evaluated', 
			@INTERACTION_APPROVED AS 'approved',
			@INTERACTION_RATIO AS 'ApprovalPercentage'
	END
	ELSE
	BEGIN
		SET @INTERACTION_TOTAL = 0
		SET @INTERACTION_RATIO = 0
		SET @INTERACTION_APPROVED = 0
		SET @INTERACTION_DISAPPROVED = 0

	END
		
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetCampaignQualificationAverage] (
	@Instance Varchar(50) = NULL,
	@IdCampaign Varchar(100) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		ERCQS.[EvaluationResultCampaignQualificationSum] / ERCQ.[EvaluationResultCampaignQuantity]			'CampaignQualificationAverage'
	FROM
		(
			-- EvaluationResultCampaignQualificationSum
			SELECT
				SUM([QualificationSum]) EvaluationResultCampaignQualificationSum
			FROM
				(

					SELECT SUM([Score])						'QualificationSum'
						FROM [dbo].[AuditorEvaluationPlan] (NOLOCK)
						WHERE [Instance] = @Instance
							AND [IdCampaign]= @IdCampaign
							AND [Completed] = 1 
						GROUP BY
							[Instance],
							[IdEvaluationPlan],
							[IdAuditor],
							[IdAgent],
							[IdInteraction]
					/*
					SELECT
						SUM([Qualification]) QualificationSum
					FROM
						EvaluationResult
					WHERE
						[Instance] = @Instance
						AND [IdCampaign]= @IdCampaign
					GROUP BY
						[Instance],
						[IdEvaluationPlan],
						[IdAuditor],
						[IdAgent],
						[IdInteraction]
					*/
				) EvaluationResultCampaign
		) ERCQS,
		(
			-- EvaluationResultCampaignQuantity
			SELECT
				COUNT(*) EvaluationResultCampaignQuantity
			FROM
				(
					SELECT COUNT(*)							'EvaluationResultCampaingQuantity'
						FROM [dbo].[AuditorEvaluationPlan] (NOLOCK) 
						WHERE [Instance] = @Instance
							AND [IdCampaign]= @IdCampaign
							AND [Completed] = 1 
						GROUP BY
							[Instance],
							[IdEvaluationPlan],
							[IdAuditor],
							[IdAgent],
							[IdInteraction]
					/*
					SELECT
						[IdCampaign]
					FROM
						EvaluationResult
					WHERE
						[Instance] = @Instance
						AND [IdCampaign]= @IdCampaign
					GROUP BY
						[Instance],
						[IdEvaluationPlan],
						[IdAuditor],
						[IdAgent],
						[IdInteraction],
						[IdCampaign]
					*/
				) EvaluationResultCampaign
		) ERCQ

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetCampaignsEvaluationFormat] (
	@Instance Varchar(50) = NULL, @IdEvaluationFormat Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
 	SELECT
		ce.IdCampaign							'id',
		ce.IdCampaign					'description'
	FROM CampaignEvaluationFormat ce  (NOLOCK)
	WHERE
	ce.Instance = @Instance
	AND ce.IdEvaluationFormat=@IdEvaluationFormat
 	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTE SP RETORNA LAS CAMPANAS ASOCIADAS A EVALUACIONES
CREATE PROCEDURE [dbo].[GetCampaignsFromEvaluation] 
(
	@VCC					VARCHAR(50) = NULL,
	@UserId					VARCHAR(50) = NULL,
	@SearchType				VARCHAR(50) = 'HISTORY',
	@EfectivityMode			BIT = 0
) 
AS
BEGIN
	-- DECLARAMOS UN CADENA PARA EL QUERY FINAL
	DECLARE @SQL_SELECT VARCHAR(MAX) = '';
	DECLARE @SQL_FROM VARCHAR(MAX) = '';
	DECLARE @SQL_FROM_EFECTIVITY VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER_EFECTIVITY VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA VARCHAR(MAX) = '';

	-- ARMAMOS EL QUERY PARA HACER LA CONSULTA
	SET @SQL_SELECT += '
		SELECT DISTINCT IEP.[Instance]							''vcc'', 
						IEP.[IdCampaign]						''id'',
						IEP.[IdCampaign]						''description'' '

	-- ARMAMOS LA PARTE DE TABLAS 
	SET @SQL_FROM += '
			FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
			INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = AEP.[Instance]
					AND IEP.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND IEP.[IdInteraction] = AEP.[IdInteraction] '
	SET @SQL_FROM_EFECTIVITY +=	'
			FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
			INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = AEP.[Instance]
					AND IEP.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND IEP.[IdInteraction] = AEP.[IdInteraction]
			INNER JOIN ( SELECT DISTINCT [VCC],[InteractionId],[AgentId],[EvaluationPlanId],[AuditorId] FROM [dbo].[EvaluationResultEfectivity] 
						 WHERE [EfectivityStatus] IN( ''' + 'CREATED'+''''+','+ ''''+ 'DRAFT'+''''+ ')' +

			')AS ERE ON 
				(AEP.[Instance] = ERE.[VCC] 
					AND AEP.[IdEvaluationPlan] = ERE.[EvaluationPlanId]
					AND AEP.[IdInteraction] = ERE.[InteractionId])'	

	-- ARMAMOS LA PARTE DE FILTROS
	SET @SQL_FILTER += '
			WHERE AEP.[Instance] = ''' + @VCC + ''' 
				AND IEP.[IdCampaign] IS NOT NULL '
	SET @SQL_FILTER_EFECTIVITY += '
			WHERE AEP.[Instance] = ''' + @VCC + ''' 
				AND IEP.[IdCampaign] IS NOT NULL '

	-- ARMAMOS LA PARTE DE EXTRAS
	SET @SQL_EXTRA = '
			ORDER BY IEP.[IdCampaign] ASC '

	-- FILTRAMOS SEGUN EL TIPO DE BUSQUEDA
	IF @SearchType = 'PENDING'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES QUE NO HAN SIDO EVALUADAS
		SET @SQL_FILTER += '
			AND AEP.[Completed] = 0 '
	END
	ELSE IF @SearchType = 'HISTORY'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES QUE YA HAN SIDO EVALUADAS
		SET @SQL_FILTER += '
			AND AEP.[Completed] = 1 '
	END

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL ROLE DEL USUARIO
	DECLARE @USER_ROLE VARCHAR(50) 
	SELECT TOP 1 @USER_ROLE = U.[Role] 
		FROM [dbo].[User] U (NOLOCK) 
		WHERE U.[VCC] = @VCC
			AND U.[ID] = @UserId

	-- FILTRAMOS SEGUN EL ROLE DEL USUARIO
	-- IF @RoleId = 'administrator, quality_admin'
	IF @USER_ROLE = 'quality_auditor'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES AUDITADAS POR EL USUARIO
		SET @SQL_FILTER += '
			AND AEP.[IdAuditor] = ''' + @UserId + ''' '
		SET @SQL_FILTER_EFECTIVITY += '
			AND ERE.[AuditorId] = ''' + @UserId + ''' '
		
	END
	ELSE IF @USER_ROLE = 'quality_agent'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES HECHAS POR EL USUARIO
		SET @SQL_FILTER += '
			AND AEP.[IdAgent] = ''' + @UserId + ''' '
	END

	-- UNIMOS LAS PARTES DEL QUERY
	IF (@EfectivityMode =1)
	SET @SQL_SELECT += @SQL_FROM_EFECTIVITY + @SQL_FILTER_EFECTIVITY + @SQL_EXTRA
	ELSE
	SET @SQL_SELECT += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA
	
	-- EJECUTAMOS LA CONSULTA
	--PRINT @SQL_SELECT
	EXEC (@SQL_SELECT)
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetCategories] 'default','',''
CREATE PROCEDURE [dbo].[GetCategories] (
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]							'name',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Category] (NOLOCK)
	WHERE [VCC] = @VCC
	AND ISNULL([Name], '') LIKE CASE WHEN @SearchCriteria = 'name' THEN '%' + @SearchExpression + '%' ELSE '%' END
	AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
	ORDER BY [Name] asc, [CreatedDate] desc

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetCategory] 'inconcert','banca'
CREATE PROCEDURE [dbo].[GetCategory] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]						    'name',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Category] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [Name] = @Name
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetCategoryByName] 'inconcert','Banca'
CREATE PROCEDURE [dbo].[GetCategoryByName] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]						    'name',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Category] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [Name] = @Name
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetCategoryPermissions] (
	@VCC Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT [Category]							'category',
			[Position] 							'position'
		FROM [CategoryPermissions] (NOLOCK)
		GROUP BY [Category],[Position]
		ORDER BY [Position] ASC
	
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


CREATE PROCEDURE [dbo].[GetCleanerData] (
@VCC VARCHAR(50),
@DAYS INT
)

AS
BEGIN
	SET NOCOUNT ON
      
				      --BUSCA REGISTROS PARA ELIMINARLOS DE LA BASE DE DATOS POR CADA VCC
				      
				
					   DECLARE @NOW VARCHAR(15) = CONVERT(VARCHAR,GETUTCDATE(),23)
				
					   CREATE TABLE #ProcessToDelete(
					   [Name] VARCHAR(50)
					   )
				       
				       CREATE TABLE #ProcessAsyncToDelete(
				       [ProcessId]  VARCHAR(50)
				       )
				       
				       DECLARE @FindInEvaluationPlan TABLE (
				        [Name] VARCHAR(50)
				       )
				       
				       DECLARE @FindInStudy TABLE (
				        [Name] VARCHAR(50)
				       )	
				       
					   DECLARE @EvaluationPlan TABLE(
					   [ID]      VARCHAR(50)
					   )
					   DECLARE @Study TABLE(
					   [Name]    VARCHAR(50)
					   )

					   CREATE TABLE #AudioToDelete(
					   [ID]       VARCHAR(100),
					   [FileName] VARCHAR(700)
					   )
					   
					   CREATE TABLE #AsyncInteractionToDelete(
					   [ID]       VARCHAR(250)
					   )
					   
					 --BUSCAR PLANES
					 
					  INSERT INTO @EvaluationPlan
					  SELECT 
							[ID]
					  FROM  [dbo].[EvaluationPlan] (NOLOCK) 
					  WHERE [Instance] = @VCC 
							AND DATEDIFF(DAY,CONVERT(VARCHAR,[CreatedDate],23),@NOW) > @DAYS

					--BUSCAR ESTUDIOS
					
					INSERT INTO @Study
					SELECT 
					    
						  [Name]
				    FROM  [dbo].[Study] (NOLOCK)
					WHERE [VCC] = @VCC
					      AND DATEDIFF(DAY,CONVERT(VARCHAR,[CreatedDate],23),@NOW) > @DAYS
					      
					  
					--BUSCAR PROCESOS DE PLANES
					
			       INSERT INTO #ProcessToDelete 
				   SELECT 
					     P.[Name]
				   FROM  [dbo].[Process] P (NOLOCK) 
				   JOIN  [dbo].[EvaluationPlan] EP (NOLOCK)
				         ON (P.[VCC] = EP.[Instance] AND P.[Name] = EP.[ProcessId] )
				   WHERE P.[VCC] = @VCC 
				         AND EP.[ID] IN (SELECT [ID] FROM @EvaluationPlan)
				         
				         
				    --BUSCAR SI SE ESTA UTILIZANDO EN PLANES MAS RECIENTES
				    
				    INSERT INTO @FindInEvaluationPlan
				    SELECT 
				         DISTINCT [ProcessId]
				    FROM [dbo].[EvaluationPlan] (NOLOCK)
				    WHERE [Instance] = @VCC 
				         -- AND [ProcessId] IN (SELECT [Name] FROM #ProcessToDelete)
				          AND DATEDIFF(DAY,CONVERT(VARCHAR,[CreatedDate],23),@NOW) <= @DAYS
				          
	

					--BUSCAR PROCESOS DE ESTUDIOS
					
			       INSERT INTO #ProcessToDelete 
				   SELECT 
					     P.[Name]
				   FROM  [dbo].[Process] P (NOLOCK) 
				   JOIN  [dbo].[Study] S (NOLOCK)
				         ON (P.[VCC] = S.[VCC] AND P.[Name] = S.[ProcessId] )
				   WHERE P.[VCC] = @VCC 
				         AND S.[Name] IN (SELECT [Name] FROM @Study)
				         

				    --BUSCAR SI SE ESTA UTILIZANDO EN ESTUDIOS MAS RECIENTES
				    
				    INSERT INTO @FindInStudy
				    SELECT 
				         DISTINCT [ProcessId]
				    FROM [dbo].[Study] (NOLOCK)
				    WHERE [VCC] = @VCC 
				          --AND [ProcessId] IN (SELECT [Name] FROM #ProcessToDelete)
				          AND DATEDIFF(DAY,CONVERT(VARCHAR,[CreatedDate],23),@NOW) <= @DAYS
				          
				     
				          
 					--BUSCAR PROCESOS QUE NO ESTEN ASIGNADOS
					
			       INSERT INTO #ProcessToDelete 
				   SELECT 
					     [Name]
				   FROM  [dbo].[Process] (NOLOCK) 
				   WHERE [VCC] = @VCC 
				         AND [Name] NOT IN (SELECT [Name] FROM #ProcessToDelete)
				         AND DATEDIFF(DAY,CONVERT(VARCHAR,[CreatedDate],23),@NOW) > @DAYS
				         
				         
				    --ELIMINAR DE #ProcessToDelete LOS PROCESOS QUE SE ASIGNARON MAS DE UNA VEZ Y RECIENTEMENTE
				    
				     DELETE FROM #ProcessToDelete WHERE [Name] IN (SELECT [Name] FROM @FindInEvaluationPlan)
				     DELETE FROM #ProcessToDelete WHERE [Name] IN (SELECT [Name] FROM @FindInStudy)  
				     

                    --BUSCAR AUDIOS DE LOS PROCESOS QUE QUEDARON PARA SER ELIMINADOS
                    
                    INSERT INTO #AudioToDelete
                    SELECT A.[ID],A.[FileName] FROM [dbo].[Audio] A (NOLOCK)
                    JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
                    ON (A.[VCC] = PA.[VCC] AND A.[ID] = PA.[AudioId])
                    WHERE A.[VCC] = @VCC 
                          AND  PA.[ProcessId] IN (SELECT [Name] FROM #ProcessToDelete)
                    
                    --BUSCAR Y QUITAR LOS AUDIOS QUE FUERON ASIGNADOS A UN NUEVO PROCESO
                    
                     DELETE FROM #AudioToDelete 
                     WHERE [ID] IN (
                     SELECT [ID] FROM [dbo].[Audio] A (NOLOCK) 
                     JOIN [dbo].[ProcessAudio] PA (NOLOCK)
                     ON (A.[VCC] = PA.[VCC] AND A.[ID] = PA.[AudioId])
                     WHERE A.[VCC] = @VCC
                          AND  PA.[ProcessId] NOT IN (SELECT [Name] FROM #ProcessToDelete)
                          AND DATEDIFF(DAY,CONVERT(VARCHAR,PA.[CreatedDate],23),@NOW) <= @DAYS
                     )
                     
                    --BUSCAR AUDIOS QUE NO TIENEN RELACION CON PROCESOS
                    
                      INSERT INTO #AudioToDelete
                      SELECT A.[ID],A.[FileName] FROM [dbo].[Audio] A (NOLOCK) 
                      LEFT JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
                      ON (A.[VCC] = PA.[VCC] AND A.[ID] = PA.[AudioId])
                      WHERE A.[VCC] = @VCC  AND PA.[AudioId] IS NULL
                            AND DATEDIFF(DAY,CONVERT(VARCHAR,A.[CreatedDate],23),@NOW) > @DAYS
                       
                    --RESETEAR VARIABLES TEMPORALES PARA UTILIZARLAS CON PROCESOS ASINCRONOS
                    
                    DELETE FROM @FindInEvaluationPlan
                    DELETE FROM @FindInStudy
                    
                    
                    --BUSCAR PROCESOS ASINCRONOS DE PLANES
                     INSERT INTO #ProcessAsyncToDelete
                     SELECT 
					     P.[ProcessId]
				     FROM  [dbo].[ProcessAsyncInteraction] P (NOLOCK) 
				     JOIN  [dbo].[EvaluationPlan] EP (NOLOCK)
				         ON (P.[VCC] = EP.[Instance] AND P.[ProcessId] = EP.[ProcessId] )
				     WHERE P.[VCC] = @VCC 
				         AND EP.[ID] IN (SELECT [ID] FROM @EvaluationPlan)    
				         
				         
				    --BUSCAR PROCESOS ASINCRONOS SI SE ESTA UTILIZANDO EN PLANES MAS RECIENTES
				    
				    INSERT INTO @FindInEvaluationPlan
				    SELECT 
				         [ProcessId]
				    FROM  [dbo].[EvaluationPlan] (NOLOCK)
				    WHERE [Instance] = @VCC 
				          --AND [ProcessId] IN (SELECT [ProcessId] FROM #ProcessAsyncToDelete)
				          AND DATEDIFF(DAY,CONVERT(VARCHAR,[CreatedDate],23),@NOW) <= @DAYS
				          

					--BUSCAR PROCESOS ASINCRONOS DE ESTUDIOS
					
			       INSERT INTO #ProcessAsyncToDelete 
				   SELECT 
					     P.[ProcessId]
				   FROM  [dbo].[ProcessAsyncInteraction] P (NOLOCK) 
				   JOIN  [dbo].[Study] S (NOLOCK)
				         ON (P.[VCC] = S.[VCC] AND P.[ProcessId] = S.[ProcessId] )
				   WHERE P.[VCC] = @VCC 
				         AND S.[Name] IN (SELECT [Name] FROM @Study)
				         
				         
				    --BUSCAR PROCESOS ASINCRONOS SI SE ESTA UTILIZANDO EN ESTUDIOS MAS RECIENTES
				    
				    INSERT INTO @FindInStudy
				    SELECT 
				         DISTINCT [ProcessId]
				    FROM [dbo].[Study]  (NOLOCK)
				    WHERE [VCC] = @VCC 
				          --AND [ProcessId] IN (SELECT [ProcessId] FROM #ProcessAsyncToDelete)
				          AND DATEDIFF(DAY,CONVERT(VARCHAR,[CreatedDate],23),@NOW) <= @DAYS
				          
				     
 					--BUSCAR PROCESOS ASINCRONOS QUE NO ESTEN ASIGNADOS A PLANES NI ESTUDIOS
					
			       INSERT INTO #ProcessAsyncToDelete 
				   SELECT 
					     [ProcessId]
				   FROM  [dbo].[ProcessAsyncInteraction] (NOLOCK) 
				   WHERE [VCC] = @VCC 
				         AND [ProcessId] NOT IN (SELECT [ProcessId] FROM #ProcessAsyncToDelete)
				         AND DATEDIFF(DAY,CONVERT(VARCHAR,[CreatedDate],23),@NOW) > @DAYS
				         
				    --ELIMINAR DE #ProcessAsyncToDelete LOS PROCESOS QUE SE ASIGNARON MAS DE UNA VEZ Y RECIENTEMENTE
				    
				     DELETE FROM #ProcessAsyncToDelete WHERE [ProcessId] IN (SELECT [ProcessId] FROM @FindInEvaluationPlan)
				     DELETE FROM #ProcessAsyncToDelete WHERE [ProcessId] IN (SELECT [ProcessId] FROM @FindInStudy)  
				     

                    --BUSCAR INTERACCIONES ASINCRONAS DE LOS PROCESOS QUE QUEDARON PARA SER ELIMINADOS
                    
                    INSERT INTO #AsyncInteractionToDelete
                    SELECT 
                          A.[ID]
                    FROM [dbo].[AsyncInteraction] A (NOLOCK)
                    JOIN [dbo].[ProcessAsyncInteraction] PA (NOLOCK) 
                    ON (A.[VCC] = PA.[VCC] AND A.[ID] = PA.[InteractionId])
                    WHERE A.[VCC] = @VCC 
                          AND  PA.[ProcessId] IN (SELECT [ProcessId] FROM #ProcessAsyncToDelete)
                    
                    --BUSCAR Y QUITAR LAS INTERACCIONES ASINCRONAS QUE FUERON ASIGNADAS A UN NUEVO PROCESO
                    
                     DELETE FROM #AsyncInteractionToDelete 
                     WHERE [ID] IN (
                     SELECT [ID] FROM [dbo].[AsyncInteraction] A (NOLOCK) 
                     JOIN [dbo].[ProcessAsyncInteraction] PA (NOLOCK)
                     ON (A.[VCC] = PA.[VCC] AND A.[ID] = PA.[InteractionId])
                     WHERE A.[VCC] = @VCC
                          AND  PA.[ProcessId] NOT IN (SELECT [ProcessId] FROM #ProcessAsyncToDelete)
                          AND DATEDIFF(DAY,CONVERT(VARCHAR,PA.[CreatedDate],23),@NOW) <= @DAYS
                     )
                     
                    --BUSCAR  INTERACCIONES ASINCRONAS QUE NO TIENEN RELACION CON PROCESOS
                    
                      INSERT INTO #AsyncInteractionToDelete
                      SELECT A.[ID] 
                      FROM [dbo].[AsyncInteraction] A (NOLOCK) 
                      LEFT JOIN [dbo].[ProcessAsyncInteraction] PA (NOLOCK) 
                      ON (A.[VCC] = PA.[VCC] AND A.[ID] = PA.[InteractionId])
                      WHERE A.[VCC] = @VCC  AND PA.[InteractionId] IS NULL
                            AND DATEDIFF(DAY,CONVERT(VARCHAR,A.[CreatedDate],23),@NOW) > @DAYS
                            
                    --LISTAR LOS RESULTADOS FINALES
                    
                    SELECT [ID] 'id'     FROM @EvaluationPlan
                    
                    SELECT [Name] 'name' FROM @Study
                  
					SELECT DISTINCT [Name] 'name' FROM #ProcessToDelete
					
					SELECT DISTINCT [ProcessId] 'processId' FROM #ProcessAsyncToDelete
					
					SELECT DISTINCT [ID] 'id',[FileName] 'fileName' FROM #AudioToDelete
					
					SELECT DISTINCT [ID] 'id' FROM #AsyncInteractionToDelete
					
					
					--ELIMINAR LAS TABLAS TEMPORALES
					
					DROP TABLE #ProcessToDelete
					DROP TABLE #AudioToDelete
					DROP TABLE #ProcessAsyncToDelete
					DROP TABLE #AsyncInteractionToDelete
	  
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetClient] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]							'id',
		[Description]					'description',
		[Country]						'country',
		[Contact]						'contact',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Client] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [ID] = @ID
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetClients] (
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]							'id',
		[Description]					'description',
		[Country]						'country',
		[Contact]						'contact',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Client] (NOLOCK)
	WHERE [VCC] = @VCC
	AND ISNULL([ID], '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
	AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
	AND ISNULL([Contact], '') LIKE CASE WHEN @SearchCriteria = 'contact' THEN '%' + @SearchExpression + '%' ELSE '%' END
	AND ISNULL([Country], '') LIKE CASE WHEN @SearchCriteria = 'country' THEN '%' + @SearchExpression + '%' ELSE '%' END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE  PROCEDURE [dbo].[GetCognitiveAnalysisProcessValidDurationTime](
	@VCC VARCHAR(50) = NULL,
	@ProcessId VARCHAR(50)
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE RETORNAR LA CANTIDAD DE SEGUNDOS DE TRANSCRIPCION DE AUDIO CORRESPONDIENTE A UN PROCESS
	SET XACT_ABORT, NOCOUNT ON
	
	SELECT P.[Name] 'name',
			SUM(A.[DurationLinguisticTime]) 'validDurationTime'
		FROM Process P (NOLOCK)
		LEFT JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
			ON PA.[ProcessId] = P.[Name]
		LEFT JOIN [dbo].[Audio] A (NOLOCK) 
			ON A.[ID] = PA.[AudioId]
				AND A.[Section] = PA.[Section]
				AND A.[Part] = PA.[Part]
		WHERE P.[VCC] = @VCC 
			AND P.[Name] = @ProcessId
		GROUP BY P.[Name] 

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetCognitiveInteractionIntents]
(
@IdInteraction VARCHAR(100),
@Section VARCHAR(50),
@Part VARCHAR(50),
@Vcc VARCHAR(50),
@TypeProcessResult VARCHAR(50),
@ProcessResult VARCHAR(50)
)
AS
BEGIN

	IF (@TypeProcessResult = 'EvaluationPlan')
	BEGIN
		SELECT DISTINCT
		NI.[VCC]				'vcc',
		NI.[Name]				'intentName',
		NI.[IntentId]			'intentId',
		NI.[AppId]				'appId',
		NI.[CognitiveEngine]	'cognitiveEngine',
		NI.[nluId]				'modelName'

		FROM InteractionEvaluationPlan  IEP 
		INNER JOIN EvaluationResult ER ON IEP.Instance = ER.Instance AND IEP.IdInteraction = ER.IdInteraction
		INNER JOIN EvaluationPlan EP ON IEP.IdEvaluationPlan = EP.ID
		INNER JOIN EvaluationFormat EF ON EP.IdEvaluationFormat = EF.ID
		INNER JOIN AbilityEvaluationFormat AEF ON EF.ID = AEF.IdEvaluationFormat
		INNER JOIN AbilityQuery AQ ON AEF.IdAbility = AQ.AbilityId
		INNER JOIN (
		SELECT DISTINCT
						INNER_N.[VCC]								'vcc'
						,INNER_N.[Name] + '|'+ INNER_I.[Name]		'cognitiveId'
						,INNER_N.[Name]								'nluId'
						,INNER_N.[AppId]							'appId'
						,INNER_I.[IntentId]							'intentId'
						,INNER_I.[Name]
						,INNER_N.[CognitiveEngine]					'cognitiveEngine'	

					FROM [dbo].[NLU] INNER_N (NOLOCK)
					INNER JOIN [dbo].[Intent] INNER_I (NOLOCK) 
					ON INNER_N.[VCC] = INNER_I.[VCC]
							AND INNER_N.AppId = INNER_I.[AppId] 
					WHERE INNER_N.[VCC] = @Vcc
		) NI ON NI.vcc =AQ.VCC AND NI.cognitiveId =AQ.QueryContent

		WHERE IEP.IdInteraction = @IdInteraction and
		IEP.InteractionSection =  @Section and
		IEP.InteractionPart = @Part and 
		IEP.instance = @Vcc and 
		IEP.IdEvaluationPlan = @ProcessResult

	END

	ELSE --Es un estudio
	BEGIN
	SELECT DISTINCT	
	            TTI.[VCC]											'vcc',	
				NI.[Name]											'intentName',
				NI.[intentId]										'intentId',
				TTI.[VCC]+'::'+ NI.[nluId]							'appId',			
				NI.[cognitiveEngine]								'cognitiveEngine',
				NI.[nluId]											'modelName'	
				
				FROM StudyResultSummary SR 
				INNER JOIN StudyResultSummaryDetail SRD ON SR.StudyId =SRD.StudyId
				INNER JOIN TemplateTheme TT ON SR.TemplateId = TT.TemplateId
				INNER JOIN TemplateThemeItem TTI ON TT.TemplateId = TTI.TemplateId
				INNER JOIN (
				SELECT DISTINCT
						INNER_N.[VCC]								'vcc'
						,INNER_N.[Name] + '|'+ INNER_I.[Name]		'cognitiveId'
						,INNER_N.[Name]								'nluId'
						,INNER_N.[AppId]							'appId'
						,INNER_I.[IntentId]							'intentId'
						,INNER_I.[Name]
						,INNER_N.[CognitiveEngine]					'cognitiveEngine'	

					FROM [dbo].[NLU] INNER_N (NOLOCK)
					INNER JOIN [dbo].[Intent] INNER_I (NOLOCK) 
					ON INNER_N.[VCC] = INNER_I.[VCC]
							AND INNER_N.AppId = INNER_I.[AppId] 
					WHERE INNER_N.[VCC] = @Vcc
				) NI ON NI.[vcc] = TT.[VCC] AND NI.[cognitiveId] = TTI.[ID]
				WHERE 
				SR.StudyId = @ProcessResult AND
				SR.VCC = @Vcc AND
				SRD.TranscriptionId = @IdInteraction +'_'+@Section+'_'+@part	
		
	END

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetColumns] '38.127.113.216', 'GyT', 'Agencias', 1
CREATE PROCEDURE [dbo].[GetColumns]
(
@Ip Varchar(50), @Database Varchar(50), @Table Varchar(50), @Type Int
)
AS
BEGIN

	DECLARE @Sql NVARCHAR(MAX);

	IF(@Type = 0)  
	BEGIN  
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[TABLE_CATALOG]						''table_catalog'',
			[TABLE_NAME]						''table_name'',
			[COLUMN_NAME]						''column_name'',
			[DATA_TYPE]							''data_type''
		FROM [' + @Ip + ']' + '.' + @Database + '.INFORMATION_SCHEMA.COLUMNS WITH (NOLOCK)
		WHERE TABLE_NAME = ''' + @Table + ''''
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE IF(@Type = 1) 
	BEGIN
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[TABLE_CATALOG]						''table_catalog'',
			[TABLE_NAME]						''table_name'',
			[COLUMN_NAME]						''column_name'',
			[DATA_TYPE]							''data_type''
		FROM ' + @Database + '.INFORMATION_SCHEMA.COLUMNS (NOLOCK)
		WHERE TABLE_NAME = ''' + @Table + ''''
		SET @Sql += '
		SET NOCOUNT OFF'
	END  
	ELSE  
	BEGIN  
		SET @Sql = 'SELECT 1 AS ''result'''  
	END

	EXEC (@Sql)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTE SP RETORNA UN RESUMEN DE LA BUSQUEDA DE INTERACCIONES DE CSV CRUZANDO CON ALLEGRO PARA INDETIFICAR LOS YA TRANSCRITOS O INPROCESS SEGUN LOS FILTROS RECIBIDOS 
-- FUNCIONA IGUAL QUE LA FUNCION EN KNEX
CREATE PROCEDURE [dbo].[GetCSVBillingInteractions] (
	@VCC VARCHAR(50) = NULL,
	@SecondsPerRequest DECIMAL(18, 5),
	@StartDate DATETIME = '1980-01-01T00:00:00',
    @EndDate DATETIME = '2050-01-01T23:59:59',
	@Campaigns VARCHAR(MAX) = '',
 	@Agents VARCHAR(MAX) = '',
   	@Origins VARCHAR(100) = NULL,
	@StartHour VARCHAR(50) = NULL,
	@EndHour VARCHAR(50) = NULL,
	@MinDuration INT = 0,
    @MaxDuration INT = 100000,
    @Results VARCHAR(MAX) = NULL,
	@InteractionIds VARCHAR(MAX) = NULL
) 
AS
BEGIN
	-- SETTEAMOS VALORES MINIMOS PARA LAS HORAS DE INICIO Y FIN
	IF (@StartHour IS NOT NULL AND @StartHour = '') SET @StartHour = '00:00:00'
	IF (@EndHour IS NOT NULL AND @EndHour = '') SET @EndHour = '23:59:59'

	-- DECLARAMOS UN CADENA PARA EL QUERY FINAL
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = '';
	DECLARE @SQL_FROM VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA VARCHAR(MAX) = '';
    DECLARE @FILTERS_OPTIONAL_PART  VARCHAR(MAX) = '';
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = '';
    DECLARE @FILTERS_FIXED_PART     VARCHAR(MAX) = '';
	
	-- DECLARAMOS VARIABLES PARA NOS NOMBRES DE LAS TABLAS
	DECLARE @INTERACTION_DETAIL_TABLE NVARCHAR(100)
	DECLARE @INTERACTION_ACTOR_DETAIL_TABLE NVARCHAR(100)
	DECLARE @INTERACTION_MULTIMEDIA_PARTS_TABLE NVARCHAR(100)
	DECLARE @CAMPAIGN_GOAL_RESULT_MANAGEMENT_TABLE NVARCHAR(100)

	-- SETTEAMOS LOS VALORES DE DURACION MINIMA Y MAXIMA A UNO VALIDO
	IF @MinDuration = @MaxDuration AND @MaxDuration = 0 SET @MaxDuration = 100000
	IF @MinDuration IS NULL SET @MinDuration = 0
	IF @MaxDuration IS NULL SET @MaxDuration = 100000

	SET @SQL_SELECT_1 += '
		SELECT DISTINCT '

	SET @SQL_SELECT_1 += '
				''CALL''																	''interactionType'',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN 1 ELSE 0 END)						''alreadyTranscribedAudio'',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN PCT.[DurationTime] ELSE 0 END)		''alreadyTranscribedDurationTime'',
				SUM(CASE WHEN (A.[IsProcess] = 1 
								AND A.[IsTranscribed] IS NULL 
								AND A.[IsCancel] IS NULL
								) THEN 1 ELSE 0 END)										''inProcessAudio'',
				SUM(CASE WHEN (A.[IsProcess] = 1
								AND A.[IsTranscribed] IS NULL 
								AND A.[IsCancel] IS NULL
								) THEN PCT.[DurationTime] ELSE 0 END)						''inProcessDurationTime'',
				SUM(CASE WHEN A.[IsCancel] = 1 THEN 1 ELSE 0 END)							''canceledAudio'', 
				SUM(CASE WHEN A.[IsCancel] = 1 THEN A.[DurationTime] ELSE 0 END)			''canceledDurationTime'',
				SUM(CASE WHEN (A.[IsProcess] IS NULL
								OR (A.[IsProcess] = 1 AND A.[IsCancel] = 1)
								) THEN 1 ELSE 0 END)										''notTranscribedAudio'',
				SUM(CASE WHEN (A.[IsProcess] IS NULL 
								OR (A.[IsProcess] = 1 AND A.[IsCancel] = 1)
								) THEN PCT.[DurationTime] ELSE 0 END)						''notTranscribedDurationTime'', 
				SUM(CASE WHEN A.[IsTranscribed] = 1 
							AND A.[TotalPhrases] IS NOT NULL
						THEN A.[TotalPhrases] ELSE 0 END)									''totalPhrases'',
				SUM(CASE WHEN A.[TotalPhrases] IS NULL 
						THEN (PCT.[DurationTime] / ' + (SELECT CONVERT(varchar(10), @SecondsPerRequest)) + ') ELSE 0 END)			''totalRawPhrases'' ' 
				--SUM(PCT.[DurationTime] / ' + (SELECT CONVERT(varchar(10), @SecondsPerRequest)) + ') ''averageModelResquest'' '
	
	-- ARMAMOS LA PARTE DE TABLAS 
	SET @SQL_FROM += '
			FROM [#ProcessCsvTemp] PCT
			LEFT JOIN [dbo].[Audio] A (NOLOCK)
				-- ON CONCAT(A.[ID], ''_'', A.[Section], ''_'', A.[Part]) = PCT.[ID]
				ON A.[ID] = PCT.[ID]
					AND A.[VCC] = ''' + @VCC + ''' '

	-- ARMAMOS LA PARTE DE FILTROS
	SET @FILTERS_FIXED_PART = ' WHERE PCT.[ID] IS NOT NULL ';
    SET @FILTERS_OPTIONAL_PART = ' AND PCT.[EndDate] BETWEEN ''' + (SELECT CONVERT(VARCHAR(21), @StartDate, 127)) + ''' AND ''' + (SELECT CONVERT(VARCHAR(21), @EndDate, 127)) + ''' 
				AND PCT.[DurationTime] BETWEEN ' + (SELECT CONVERT(VARCHAR(10), @MinDuration)) + ' AND ' + (SELECT CONVERT(VARCHAR(10), @MaxDuration)) + ' '
	
    -- CONTINUAMOS CON LOS DEMAS FILTROS RECIBIDOS
	IF (@Campaigns IS NOT NULL AND @Campaigns != '') 
		SET @FILTERS_OPTIONAL_PART += 'AND PCT.[Campaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','','')) '
	
	IF (@Agents IS NOT NULL AND @Agents != '') 
		SET @FILTERS_OPTIONAL_PART += 'AND PCT.[Agent] IN (SELECT Value FROM [dbo].[split](''' + @Agents + ''','','')) '
	
	IF (@Origins IS NOT NULL AND @Origins != '') 
		SET @FILTERS_OPTIONAL_PART += 'AND PCT.[Direction] IN (SELECT Value FROM [dbo].[split](''' + @Origins + ''','','')) '
	
	IF (@Results IS NOT NULL AND @Results != '') 
		SET @FILTERS_OPTIONAL_PART += ' 
			AND (
				(CONCAT(PCT.[Campaign], ''_'', ISNULL(CONVERT(VARCHAR(4), PCT.[DispositionCode]), '''')) IN (SELECT Value FROM [dbo].[split](''' + @Results + ''','',''))) 
				OR
				(PCT.[Description] IN (SELECT Value FROM [dbo].[split](''' + @Results + ''','','')))
			) '

	-- PARA EL CASO EN QUE HORA DE INICIO Y FIN NO SEAN NULL
	IF @StartHour IS NOT NULL AND @EndHour IS NOT NULL
	BEGIN
		IF (CAST(@StartHour AS TIME) <= CAST(@EndHour AS TIME)) 
			SET @FILTERS_OPTIONAL_PART += 'AND CAST(PCT.[EndDate] AS TIME) BETWEEN ''' + @StartHour + ''' AND ''' + @EndHour + ''' '
		IF (CAST(@StartHour AS TIME) > CAST(@EndHour AS TIME)) 
			SET @FILTERS_OPTIONAL_PART += 'AND CAST(PCT.[EndDate] AS TIME) NOT BETWEEN ''' + @EndHour + ''' AND ''' + @StartHour + ''' '
	END
	-- PARA EL CASO EN QUE HORA DE INICIO NO SEA NULL Y FIN SEA NULL
	ELSE IF @StartHour IS NOT NULL AND @EndHour IS NULL
	BEGIN
		SET @FILTERS_OPTIONAL_PART += 'AND CAST(PCT.[EndDate] AS TIME) > ''' + @StartHour + ''' '
	END
	-- PARA EL CASO EN QUE HORA DE INICIO SEA NULL Y FIN NO SEA NULL
	ELSE IF @StartHour IS NULL AND @EndHour IS NOT NULL
	BEGIN
		SET @FILTERS_OPTIONAL_PART += 'AND CAST(PCT.[EndDate] AS TIME) < ''' + @EndHour + ''' '
	END

    IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
    BEGIN
         SET @FILTERS_OPTIONAL_PART = '';
	     SET @FILTERS_EXCLUDING_PART =' AND PCT.[ID] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) ';
    END

	-- ELIMINAMOS LA TABLA TEMPORAL DESPUES DE EJECUTAR EL QUERY
	SET @SQL_EXTRA += 'DROP TABLE [#ProcessCsvTemp]'
	
	-- UNIMOS LAS PARTES DEL QUERY
	SET @SQL_SELECT_1 += @SQL_FROM + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART + @SQL_EXTRA

	-- EJECUTAMOS LA CONSULTA
	EXEC (@SQL_SELECT_1)
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetCSVFiltersInteractions] 
AS
BEGIN
	-- BUSCAMOS LAS CAMPAIGNS DE LA TABLA
	SELECT DISTINCT PCT.[Campaign]									AS 'id',
					PCT.[Campaign]									AS 'description' 
		FROM [#ProcessCsvTemp] PCT
		ORDER BY PCT.[Campaign] ASC

	-- BUSCAMOS LOS AGENTES DE LA TABLA
	SELECT DISTINCT PCT.[Agent]										AS 'userId' 
		FROM [#ProcessCsvTemp] PCT
		ORDER BY PCT.[Agent] ASC

	-- BUSCAMOS LOS DISPOSITIONCODE DE LA TABLA
	SELECT DISTINCT PCT.[Campaign]									AS 'campaignId' ,
					PCT.[Category1]									AS 'description',
					PCT.[Category2]									AS 'description2',
					PCT.[Category3]									AS 'description3',
					PCT.[Description]								AS 'description4',
					PCT.[DispositionCode]							AS 'code'
		FROM [#ProcessCsvTemp] PCT
		WHERE PCT.[Description] IS NOT NULL
			AND PCT.[DispositionCode] IS NOT NULL
		ORDER BY PCT.[Category1] ASC, 
					PCT.[Category2] ASC, 
					PCT.[Category3] ASC,  
					PCT.[Description] ASC

	-- ELIMINAMOSLA TABLA TEMPORAL
	DROP TABLE [#ProcessCsvTemp]
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetCSVSummaryInteractions] 
(
	@VCC					VARCHAR(50) = NULL,
	@StartDate				DATETIME = '1980-01-01T00:00:00',
    @EndDate				DATETIME = '2050-01-01T23:59:59',
	@StartHour				VARCHAR(50) = NULL,
	@EndHour				VARCHAR(50) = NULL,
	@MinDuration			INT = 0,
    @MaxDuration			INT = 100000,
    @Campaigns				VARCHAR(MAX) = NULL,
 	@Agents					VARCHAR(MAX) = NULL,
   	@Origins				VARCHAR(100) = NULL,
	@DispositionCodes		VARCHAR(MAX) = NULL,
	@InteractionIds         VARCHAR(MAX) = NULL,
	--@Evaluated              VARCHAR(MAX) = NULL,
	@IsSummary				BIT = 0	,							-- ESTE FLAG TIENE COMO PROPOSITO RETORNAR LA LISTA CONSOLIDAD POR IDS O EL TOTAL DEL LISTADO
	@LimitIntByAgent		INT = 0
) 
AS
BEGIN
	
	
	DECLARE @SQL_SELECT_AGENTS VARCHAR(MAX) = '';
	IF @LimitIntByAgent > 0
	BEGIN 
		CREATE TABLE #Agents(
		 [agent] VARCHAR(200)
		)

		INSERT INTO #Agents
		SELECT DISTINCT agent FROM #ProcessCsvTemp

		SET @SQL_SELECT_AGENTS+= 'SELECT [agent] FROM [#Agents] '

		CREATE TABLE #EnumProcessCsvTemp
		(
			[AgentNumber] INT,
			[ID] VarChar(100),
			[Number] VarChar(50),
			[Gender] VarChar(50),
			[SilenceTime] Int,
			[DurationTime] Int,
			[Category1] VarChar(50),
			[Category2] VarChar(50),
			[Category3] VarChar(50),
			[Description] VarChar(100),
			[Campaign] VarChar(100),
			[Agent] VarChar(50),
			[Direction] VarChar(50),
			[StartDate] DateTime,
			[EndDate] DateTime,
			[DispositionCode] VarChar(50),
			[RelativePath] VarChar(500),
			[RelativeFileName] VarChar(200)
		)

		

		INSERT INTO #EnumProcessCsvTemp
		SELECT
			ROW_NUMBER() OVER (PARTITION BY Agent ORDER BY StartDate) AS RowNumber,
			ID,
			Number,
			Gender,
			SilenceTime,
			DurationTime,
			Category1,
			Category2,
			Category3,
			Description,
			Campaign,
			Agent,
			Direction,
			StartDate,
			EndDate,
			DispositionCode,
			RelativePath,
			RelativeFileName
		FROM
			#ProcessCsvTemp

	

	END

	IF OBJECT_ID(N'[dbo].[TMP_PROCESS_CSV]', N'U') IS NULL
	BEGIN
		SELECT * 
			INTO [dbo].[TMP_PROCESS_CSV]
			FROM [dbo].[#ProcessCsvTemp]
	END
	

	
	-- SETTEAMOS VALORES MINIMOS PARA LAS HORAS DE INICIO Y FIN
	IF (@StartHour IS NULL OR @StartHour = '') SET @StartHour = '00:00:00'
	IF (@EndHour IS NULL OR @EndHour = '') SET @EndHour = '23:59:59'

	-- SETTEAMOS VALORES MINIMOS PARA LAS HORAS DE MINIMO Y MAXIMO
	IF (@MinDuration IS NULL OR @MinDuration = '') SET @MinDuration = 0
	IF (@MaxDuration IS NULL OR @MaxDuration = '') SET @MaxDuration = 100000
	
	-- DECLARAMOS UN CADENA PARA EL QUERY FINAL
	DECLARE @SQL_SELECT VARCHAR(MAX) = '';
	DECLARE @SQL_SELECT_0 VARCHAR(MAX) = '';
	
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = '';
	DECLARE @SQL_FROM VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA VARCHAR(MAX) = '';
    DECLARE @FILTERS_OPTIONAL_PART  VARCHAR(MAX) = '';
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = '';
    DECLARE @FILTERS_FIXED_PART     VARCHAR(MAX) = '';
   -- DECLARE @QUERY_JOIN            VARCHAR(MAX) = '';

	-- HACEMOS EL SELECT SEGUN EL TIPO DE BUSQUEDA SETTEADO
	IF @IsSummary = 1 
	BEGIN
		-- ARMAMOS EL QUERY PARA HACER LA CONSULTA
		SET @SQL_SELECT_0 += '
			SELECT ''CALL''											AS ''interactionType'', 
					COUNT(DISTINCT PCT.[ID])						AS ''totalInteractions'',
					COUNT(*)										AS ''details'',
					ISNULL(SUM(PCT.[DurationTime]), 0)				AS ''totalAttentionTime''
					'

			
		 
		

		-- SETTEAMOS EL SELECT A LA VARIABLE FINAL
		SET @SQL_SELECT = @SQL_SELECT_0
	END
	ELSE
	BEGIN
		-- ARMAMOS EL QUERY PARA HACER LA CONSULTA
		SET @SQL_SELECT_1 += '
			SELECT PCT.[ID] + ''_0_0''													''ID'',
					PCT.[ID]															''InteractionId'',
					0																	''Section'',
					0																	''Part'',
					''CALL''															''Channel'',
					PCT.[Number]														''Number'',
					PCT.[Gender]														''Gender'',
					PCT.[SilenceTime]													''SilenceTime'',
					PCT.[DurationTime]													''DurationTime'',
					PCT.[DurationTime]													''AttentionTime'',
					PCT.[Category1]														''Category1'',
					PCT.[Category2]														''Category2'',
					PCT.[Category3]														''Category3'',
					PCT.[Description]													''Description'',
					PCT.[DispositionCode]												''DispositionCode'',
					PCT.[Campaign]														''Campaign'',
					PCT.[Agent]															''Agent'',
					NULL																''agentName'',
					PCT.[Direction]														''Direction'', -- CAMBIAR POR TYPE
					CONVERT(VARCHAR, PCT.[StartDate], 120)								''StartDate'',
					CONVERT(VARCHAR, PCT.[EndDate], 120)								''EndDate'',
					NULL																''Format'',
					PCT.[RelativePath]													''RelativePath'',
					PCT.[RelativeFileName]												''RelativeFileName''
		'

		-- SETTEAMOS EL SELECT A LA VARIABLE FINAL
		SET @SQL_SELECT = @SQL_SELECT_1
	END
	
	-- ARMAMOS LA PARTE DE TABLAS 
	IF @LimitIntByAgent > 0
	SET @SQL_FROM += ' FROM [#EnumProcessCsvTemp] PCT ' 
	ELSE
	SET @SQL_FROM += ' FROM [#ProcessCsvTemp] PCT '

	-- FILTRO POR INTERACCIONES EVALUADAS O NO
	/*IF(@Evaluated IS NOT NULL AND @Evaluated <> '')
	  BEGIN 
	   DECLARE @EvaluatedTable TABLE (Val BIT)
	   INSERT INTO @EvaluatedTable SELECT Value FROM [dbo].[split]( @Evaluated ,',')
       -- SOLAMENTE SI ES SI O NO
       IF((SELECT COUNT(Val) FROM @EvaluatedTable) = 1 )
       BEGIN
          IF((SELECT Val FROM @EvaluatedTable WHERE Val = 1) IS NOT NULL) -- Si  es Si
           BEGIN 
               SET @QUERY_JOIN = ' JOIN [dbo].[AuditorEvaluationPlan] AEP ON (AEP.[IdInteraction] = PCT.[ID] AND AEP.[Completed]=1 ) '
           END
          ELSE  --Si es No
           BEGIN
               SET @QUERY_JOIN = ' LEFT JOIN [dbo].[AuditorEvaluationPlan] AEP ON (AEP.[IdInteraction] = PCT.[ID] AND AEP.[Completed]=1 ) '
               SET @FILTERS_OPTIONAL_PART += ' AND AEP.[IdInteraction] IS NULL '
           END
       END
       
	 END*/
	-- FIN FILTRO INTERACCIONES EVALUADAS
	

	-- ARMAMOS LA PARTE DE FILTROS
	SET @FILTERS_FIXED_PART = ' WHERE PCT.[ID] IS NOT NULL ';
    SET @FILTERS_OPTIONAL_PART += '
				AND PCT.[EndDate] BETWEEN ''' + (SELECT CONVERT(VARCHAR(21), @StartDate, 127)) + ''' AND ''' + (SELECT CONVERT(VARCHAR(21), @EndDate, 127)) + ''' 
				AND PCT.[DurationTime] BETWEEN ' + (SELECT CONVERT(VARCHAR(10), @MinDuration)) + ' AND ' + (SELECT CONVERT(VARCHAR(10), @MaxDuration)) + ' ';

	-- CONTINUAMOS CON LOS DEMAS FILTROS RECIBIDOS
	IF (@Campaigns IS NOT NULL AND @Campaigns != '') 
		SET @FILTERS_OPTIONAL_PART += 'AND PCT.[Campaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','','')) '
	
	IF (@Agents IS NOT NULL AND @Agents != '') 
		SET @FILTERS_OPTIONAL_PART += 'AND PCT.[Agent] IN (SELECT Value FROM [dbo].[split](''' + @Agents + ''','','')) '
	
	IF (@Origins IS NOT NULL AND @Origins != '') 
		SET @FILTERS_OPTIONAL_PART += 'AND PCT.[Direction] IN (SELECT Value FROM [dbo].[split](''' + @Origins + ''','','')) '
	
	IF (@DispositionCodes IS NOT NULL AND @DispositionCodes != '') 
		--SET @FILTERS_OPTIONAL_PART += 'AND PCT.[DispositionCode] IN (SELECT Value FROM [dbo].[split](''' + @DispositionCodes + ''','','')) '
		SET @FILTERS_OPTIONAL_PART += ' 
			AND (
				(CONCAT(PCT.[Campaign], ''_'', ISNULL(CONVERT(VARCHAR(4), PCT.[DispositionCode]), '''')) IN (SELECT Value FROM [dbo].[split](''' + @DispositionCodes + ''','',''))) 
				OR
				(PCT.[Description] IN (SELECT Value FROM [dbo].[split](''' + @DispositionCodes + ''','','')))
			) '
	IF(@LimitIntByAgent > 0)
		SET @FILTERS_OPTIONAL_PART +=  'AND [AgentNumber] <= ' +  CAST(@LimitIntByAgent AS VARCHAR) + ' '

	-- PARA EL CASO EN QUE HORA DE INICIO Y FIN NO SEAN NULL
	IF @StartHour IS NOT NULL AND @EndHour IS NOT NULL
	BEGIN
		IF (CAST(@StartHour AS TIME) <= CAST(@EndHour AS TIME)) 
			SET @FILTERS_OPTIONAL_PART += 'AND CAST(PCT.[EndDate] AS TIME) BETWEEN ''' + @StartHour + ''' AND ''' + @EndHour + ''' '
		IF (CAST(@StartHour AS TIME) > CAST(@EndHour AS TIME)) 
			SET @FILTERS_OPTIONAL_PART += 'AND CAST(PCT.[EndDate] AS TIME) NOT BETWEEN ''' + @EndHour + ''' AND ''' + @StartHour + ''' '
	END
	-- PARA EL CASO EN QUE HORA DE INICIO NO SEA NULL Y FIN SEA NULL
	ELSE IF @StartHour IS NOT NULL AND @EndHour IS NULL
	BEGIN
		SET @FILTERS_OPTIONAL_PART += 'AND CAST(PCT.[EndDate] AS TIME) > ''' + @StartHour + ''' '
	END
	-- PARA EL CASO EN QUE HORA DE INICIO SEA NULL Y FIN NO SEA NULL
	ELSE IF @StartHour IS NULL AND @EndHour IS NOT NULL
	BEGIN
		SET @FILTERS_OPTIONAL_PART += 'AND CAST(PCT.[EndDate] AS TIME) < ''' + @EndHour + ''' '
	END

    IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
    BEGIN
         SET @FILTERS_OPTIONAL_PART = '';
         --SET @QUERY_JOIN = '';
	     SET @FILTERS_EXCLUDING_PART = ' AND PCT.[ID] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) '
		 -- SET @FILTERS_EXCLUDING_PART = ' AND PCT.[ID] LIKE ''%' + @InteractionIds + '%'' '
	END	  
	-- ELIMINAMOS LA TABLA TEMPORAL DESPUES DE EJECUTAR EL QUERY
	IF @LimitIntByAgent > 0
	SET @SQL_EXTRA += ' DROP TABLE [#ProcessCsvTemp]  DROP TABLE  [#EnumProcessCsvTemp] DROP TABLE [#Agents]'
	ELSE
	SET @SQL_EXTRA += ' DROP TABLE [#ProcessCsvTemp] '
	
	-- UNIMOS LAS PARTES DEL QUERY
	--SET @SQL_SELECT += @SQL_FROM + @QUERY_JOIN + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART + @SQL_EXTRA;
    SET @SQL_SELECT += @SQL_FROM  + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART + @SQL_SELECT_AGENTS + @SQL_EXTRA;
	-- EJECUTAMOS LA CONSULTA
	--PRINT @SQL_SELECT
	EXEC (@SQL_SELECT)
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetCustomField] (
	@VCC              VARCHAR(50) = NULL, 
	@ID               VARCHAR(100) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	DECLARE @QUERY VARCHAR(MAX) = ''
	SET @QUERY += 'SELECT  [ID]				''id'',
				[Description]				''description'',
				[Label]						''label'',
				[Type]						''type'' FROM '

    IF(@EvaluationPlanId IS NOT NULL AND @EvaluationPlanId <> '')
    BEGIN 

	SET @QUERY += ' [HistoricCustomField] (NOLOCK)
		            WHERE [EvaluationPlanId ] = '''+@EvaluationPlanId + ''' AND '
    END
    ELSE
    BEGIN 
		SET @QUERY += ' [CustomField] (NOLOCK) WHERE '

    END

	SET @QUERY += ' [VCC] ='''+ @VCC+''' AND [ID] = '''+ @ID+''''
	EXEC(@QUERY)

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetCustomFieldResults] (
	@VCC VARCHAR(50) = NULL,
	@CustomFieldId VARCHAR(100) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON


    DECLARE @QUERY VARCHAR(MAX) = ''
    SET @QUERY += 'SELECT CFR.[ID]			    ''id'',
				CFR.[Description]				''description'',
				CFR.[Order]						''order'' FROM '

    IF(@EvaluationPlanId IS NOT NULL AND @EvaluationPlanId <> '')
    BEGIN 

		SET @QUERY += ' [HistoricCustomFieldResult] CFR (NOLOCK)
			WHERE CFR.[EvaluationPlanId] = '''+@EvaluationPlanId + ''' AND '
    END
    ELSE
    BEGIN 
        SET @QUERY += ' [CustomFieldResult] CFR (NOLOCK) WHERE '
    END

	SET @QUERY += ' CFR.[VCC] ='''+ @VCC+'''
				    AND CFR.[CustomFieldId] ='''+ @CustomFieldId+'''
			        ORDER BY CFR.[Order] ASC'
	EXEC(@QUERY)

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetCustomFields] (
	@VCC VARCHAR(50) = NULL, 
	@SearchCriteria Varchar(50) = NULL, 
	@SearchExpression Varchar(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	SELECT CF.[ID]							'id',
			CF.[Description]				'description',
			CF.[Label]						'label',
			CF.[Type]						'type'
	FROM [CustomField] CF (NOLOCK)
	WHERE [VCC] = @VCC
		AND ISNULL(CF.[ID], '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(CF.[Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
	ORDER BY CF.[ID] ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetCustomFieldsUsage] 
	@VCC VARCHAR(50) = NULL, 				
	@CustomFieldIds VARCHAR(MAX) = NULL
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

    DECLARE @_IDS TABLE(
	 ID VARCHAR(50)
	)

    INSERT INTO @_IDS SELECT [Value] FROM [dbo].[Split](@CustomFieldIds,',')

    SELECT  DISTINCT EFCF.[CustomFieldId] 'customFieldId'
        FROM [dbo].[EvaluationFormatCustomField] EFCF (NOLOCK)
        WHERE EFCF.[VCC] = @VCC 
            AND EFCF.[CustomFieldId]  IN (SELECT [ID] FROM @_IDS)
           
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetDatabases] '38.127.113.216', 1
CREATE PROCEDURE [dbo].[GetDatabases]
(
@Ip Varchar(50), @Type Int
)
AS
BEGIN

	DECLARE @Sql NVARCHAR(MAX);

	IF(@Type = 0)  
	BEGIN  
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[name]							''name''
		FROM [' + @Ip + ']' + '.master.dbo.sysdatabases WITH (NOLOCK)
		WHERE name NOT IN (''master'', ''tempdb'', ''model'', ''msdb'', ''CRMIntegrationData'', ''MMProDat'',''HistoricalData'')'
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE IF(@Type = 1) 
	BEGIN
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[name]							''name''
		FROM master.dbo.sysdatabases (NOLOCK)
		WHERE name NOT IN (''master'', ''tempdb'', ''model'', ''msdb'', ''CRMIntegrationData'', ''MMProDat'',''HistoricalData'')'
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE  
	BEGIN  
		SET @Sql = 'SELECT 1 AS ''result'''  
	END

	EXEC (@Sql)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetDictionaries] (
	@VCC VARCHAR(50) = NULL, 
	@SearchCriteria VARCHAR(50) = NULL, 
	@SearchExpression VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT [Name]					'name',
			[Description]			'description',
			[Language]				'language',
			[CreatedDate]			'createdDate',
			[CreatedByUserId]		'createdByUserId',
			[LastModifiedDate]		'lastModifiedDate',
			[LastModifiedByUserId]	'lastModifiedByUserId'
		FROM [dbo].[Dictionary] (NOLOCK)
		WHERE [VCC] = @VCC
			AND ISNULL([Name], '') LIKE CASE WHEN @SearchCriteria = 'name' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

create PROCEDURE [dbo].[GetDictionary] (
	@VCC					VARCHAR(50) = NULL,
	@DictionaryId			VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	SELECT [Name]					'name',
			[Description]			'description',
			[Language]				'language',
			[CreatedDate]			'createdDate',
			[CreatedByUserId]		'createdByUserId',
			[LastModifiedDate]		'lastModifiedDate',
			[LastModifiedByUserId]	'lastModifiedByUserId'
		FROM [dbo].[Dictionary] (NOLOCK)
		WHERE [VCC] = @VCC
			AND [Name] = @DictionaryId
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetDictionaryUsage]
(
	@VCC VARCHAR(50) = NULL,
	@DictionaryId VARCHAR(50)= NULL
)
AS
BEGIN
	SET NOCOUNT ON

	--
	SELECT [Name] 'processId'
		FROM [dbo].[Process] (NOLOCK)
		WHERE [VCC] = @VCC
			AND [DictionaryId] = @DictionaryId
			AND [Percentage] IS NOT NULL
			AND [Percentage] < 1
		ORDER BY [CreatedDate] DESC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetDictionaryWord] (
	@VCC VARCHAR(50) = NULL, 
	@DictionaryWordId VARCHAR(50) = NULL
) AS
BEGIN
	DECLARE  @SQL              NVARCHAR(MAX)
	DECLARE  @FILTER           VARCHAR(MAX) = ''
	DECLARE  @ORDER_BY         VARCHAR(MAX) = ''
	
	SET NOCOUNT ON

	--
	SELECT [VCC]							'vcc',
			[DictionaryId]					'dictionaryId',
			[ID]							'id',
			[Word]							'word',
			[CreatedDate]					'createdDate',
			[CreatedByUserId]				'createdByUserId',
			[LastModifiedDate]				'lastModifiedDate',
			[LastModifiedByUserId]			'lastModifiedByUserId'
		FROM [dbo].[DictionaryWord] (NOLOCK)
		WHERE [VCC] = @VCC
			AND [ID] = @DictionaryWordId
	
	SET NOCOUNT OFF

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetDictionaryWords] (
	@VCC VARCHAR(50) = NULL, 
	@SearchCriteria VARCHAR(50) = NULL, 
	@SearchExpression VARCHAR(50) = NULL,
	@DictionaryId VARCHAR(50) = NULL
) AS
BEGIN
	DECLARE  @SQL              NVARCHAR(MAX)
	DECLARE  @FILTER           VARCHAR(MAX) = ''
	DECLARE  @ORDER_BY         VARCHAR(MAX) = ''
	
	SET NOCOUNT ON

	SET @SQL = '
	SELECT [VCC]							''vcc'',
			[DictionaryId]					''dictionaryId'',
			[ID]							''id'',
			[word]							''word'',
			[CreatedDate]					''createdDate'',
			[CreatedByUserId]				''createdByUserId'',
			[LastModifiedDate]				''lastModifiedDate'',
			[LastModifiedByUserId]			''lastModifiedByUserId''
		FROM [dbo].[DictionaryWord] (NOLOCK) ' 

	SET @FILTER = '
		WHERE [VCC] = ''' + @VCC + ''' 
			AND [DictionaryId] = ''' + @DictionaryId + ''' '

	IF (@SearchCriteria IS NOT NULL AND @SearchCriteria <> '' AND @SearchExpression IS NOT NULL AND @SearchExpression <> '')
	BEGIN 
	    SET @FILTER += '
			AND ISNULL([Word], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''word'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END'
	END

	SET @ORDER_BY = ' ORDER BY [LastModifiedDate] ASC, [Word] DESC'

	SET @SQL += @FILTER + @ORDER_BY
	SET NOCOUNT OFF

	EXEC(@SQL)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetDictionaryWordSoundLikes] (
	@VCC VARCHAR(50) = NULL, 
	@DictionaryWordIds VARCHAR(MAX) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- DECLARAMOS UNA VARIABLE PARA ARMAR EL QUERY
	DECLARE  @SQL VARCHAR(MAX)

	SET @SQL = '
		SELECT [VCC]							''vcc'',
				[DictionaryWordId]				''dictionaryWordId'',
				[ID]							''id'',
				[SoundLike]						''soundLike'',
				[CreatedDate]					''createdDate'',
				[CreatedByUserId]				''createdByUserId'',
				[LastModifiedDate]				''lastModifiedDate'',
				[LastModifiedByUserId]			''lastModifiedByUserId''
			FROM [dbo].[DictionaryWordSoundLike] (NOLOCK)
			WHERE [VCC] = ''' + @VCC + '''
				AND [DictionaryWordId] IN (''' + @DictionaryWordIds + ''') ' 
	
	-- EJECUTAMOS EL QUERY ARMADO
	EXEC(@SQL)

	SET NOCOUNT OFF

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetDispositionCodesFromEvaluation] 
(
	@VCC					VARCHAR(50) = NULL,
	@UserId					VARCHAR(50) = NULL,
	@Campaigns				VARCHAR(MAX) = NULL,
	@SearchType				VARCHAR(50) = NULL,
	@EfectivityMode			BIT = 0 

) 
AS
BEGIN
	-- DECLARAMOS UN CADENA PARA EL QUERY FINAL
	DECLARE @SQL_SELECT VARCHAR(MAX) = '';
	DECLARE @SQL_FROM VARCHAR(MAX) = '';
	DECLARE @SQL_FROM_EFECTIVITY VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER_EFECTIVITY VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA VARCHAR(MAX) = '';

	-- ARMAMOS EL QUERY PARA HACER LA CONSULTA
	SET @SQL_SELECT += '
		SELECT DISTINCT IEP.[Instance]							''vcc'',
						IEP.[IdCampaign]						''campaignId'', 
						IEP.[ManagementResult]					''dispositionTreePath'' '

	-- ARMAMOS LA PARTE DE TABLAS 
	SET @SQL_FROM += '
			FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
			INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = AEP.[Instance]
					AND IEP.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND IEP.[IdInteraction] = AEP.[IdInteraction] '
	
	SET @SQL_FROM_EFECTIVITY +='
			FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
			INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = AEP.[Instance]
					AND IEP.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND IEP.[IdInteraction] = AEP.[IdInteraction] 
				INNER JOIN ( SELECT DISTINCT [VCC],[InteractionId],[AgentId],[EvaluationPlanId],[AuditorId] FROM [dbo].[EvaluationResultEfectivity] 
						 WHERE [EfectivityStatus] IN( ''' + 'CREATED'+''''+','+ ''''+ 'DRAFT'+''''+ ')' +

			')AS ERE ON 
				(AEP.[Instance] = ERE.[VCC] 
					AND AEP.[IdEvaluationPlan] = ERE.[EvaluationPlanId]
					AND AEP.[IdInteraction] = ERE.[InteractionId])'	


	-- ARMAMOS LA PARTE DE FILTROS
	SET @SQL_FILTER += '
			WHERE AEP.[Instance] = ''' + @VCC + ''' 
				AND IEP.[ManagementResult] IS NOT NULL '
    SET @SQL_FILTER_EFECTIVITY += '
			WHERE AEP.[Instance] = ''' + @VCC + ''' 
				AND IEP.[ManagementResult] IS NOT NULL '

	-- ARMAMOS LA PARTE DE EXTRAS
	SET @SQL_EXTRA = '
			ORDER BY IEP.[IdCampaign] ASC, IEP.[ManagementResult] ASC '

	-- FILTRAMOS SEGUN EL TIPO DE BUSQUEDA
	IF @SearchType = 'PENDING'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES QUE NO HAN SIDO EVALUADAS
		SET @SQL_FILTER += '
			AND AEP.[Completed] = 0 '
	END
	ELSE IF @SearchType = 'HISTORY'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES QUE YA HAN SIDO EVALUADAS
		SET @SQL_FILTER += '
			AND AEP.[Completed] = 1 '
	END

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL ROLE DEL USUARIO
	DECLARE @USER_ROLE VARCHAR(50) 
	SELECT TOP 1 @USER_ROLE = U.[Role] 
		FROM [dbo].[User] U (NOLOCK) 
		WHERE U.[VCC] = @VCC
			AND U.[ID] = @UserId

	-- FILTRAMOS SEGUN EL ROLE DEL USUARIO
	-- IF @RoleId = 'administrator, quality_admin'
	IF @USER_ROLE = 'quality_auditor'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES AUDITADAS POR EL USUARIO
		SET @SQL_FILTER += '
			AND AEP.[IdAuditor] = ''' + @UserId + ''' '
        SET @SQL_FILTER_EFECTIVITY += '
			AND ERE.[AuditorId] = ''' + @UserId + ''' '
	END
	ELSE IF @USER_ROLE = 'quality_agent'
	BEGIN
		-- HACEMOS LA BUSQUEDA SOBRE LAS INTERACCIONES HECHAS POR EL USUARIO
		SET @SQL_FILTER += '
			AND AEP.[IdAgent] = ''' + @UserId + ''' '
	END

	-- FILTRAMOS SEGUN EL VALOR DE CAMPAIGNS
	IF @Campaigns IS NOT NULL AND @Campaigns <> ''
	BEGIN
		SET @SQL_FILTER += '
			AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','','')) '
        SET @SQL_FILTER_EFECTIVITY += '
			AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','','')) '
	END 

	-- UNIMOS LAS PARTES DEL QUERY
	IF (@EfectivityMode =1)
	SET @SQL_SELECT += @SQL_FROM_EFECTIVITY + @SQL_FILTER_EFECTIVITY + @SQL_EXTRA
	ELSE
	SET @SQL_SELECT += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA
	
	-- EJECUTAMOS LA CONSULTA
	--PRINT @SQL_SELECT
	EXEC (@SQL_SELECT)
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetDomain](
	@VCC varchar(50) = NULL, @Domain varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Domain]						'domain',
		[Description]					'description',
		[Client]						'client',
		[HasHttps]						'hasHttps',
		[ContentPort]					'contentPort',
		[Prefix]						'prefix',
		[IsMainDomain]					'isMainDomain',
		[IsContentDomain]				'isContentDomain',
		[ContentDefaultRedirect]		'contentDefaultRedirect',
		[IsDistributedService]			'isDistributedService',
		[AssociatedDistributedService]	'associatedDistributedService',
		[CustomCode]					'customCode',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Domain] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [Domain] = @Domain

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetDomains](
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Domain]						'domain',
		[Description]					'description',
		[Client]						'client',
		[HasHttps]						'hasHttps',
		[ContentPort]					'contentPort',
		[Prefix]						'prefix',
		[IsMainDomain]					'isMainDomain',
		[IsContentDomain]				'isContentDomain',
		[ContentDefaultRedirect]		'contentDefaultRedirect',
		[IsDistributedService]			'isDistributedService',
		[AssociatedDistributedService]	'associatedDistributedService',
		[CustomCode]					'customCode',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Domain] (NOLOCK)
	WHERE [VCC] = @VCC
	AND ISNULL([Domain], '') LIKE CASE WHEN @SearchCriteria = 'domain' THEN '%' + @SearchExpression + '%' ELSE '%' END
	AND ISNULL([Client], '') LIKE CASE WHEN @SearchCriteria = 'client' THEN '%' + @SearchExpression + '%' ELSE '%' END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE  PROCEDURE [dbo].[GetEfectivityPlans] (
@VCC                Varchar(50) = NULL,
@SearchCriteria     Varchar(50) = NULL,
@SearchExpression   Varchar(50) = NULL
)
AS
BEGIN
SET NOCOUNT ON

	--SELECCIONAMOS TODOS LOS PLANES AUTOMATICOS EXISTENTES PARA GENERAR  LA MUESTRA  A PARTIR DEL PLAN DE EVALUACION

    SELECT
			e.[ID]							'id',
			e.[Modality]					'modality',
			e.[DateFin]						'dateFin',
			e.CreatedDate					'createdDate',
			e.CreatedByUserId				'createdByUserId',
			e.LastModifiedDate				'lastModifiedDate',
			e.LastModifiedByUserId			'lastModifiedByUserId',
			e.Description					'description',
			e.IdEvaluationFormat            'idEvaluationFormat',
			e.Enabled						'enabled',
			e.Status						'status',
			e.Evaluation_Mode				'evaluation_mode',
			e.processId    				    'processId',
			ef.templateId    				'templateId',
			COUNT(a.IdInteraction)			'totalEvaluations',		
			COUNT(DISTINCT CASE a.[Completed] WHEN 1 THEN a.[IdInteraction] ELSE NULL END)					'completedEvaluations',
			(
				(
					CAST(
						(
							COUNT(DISTINCT CASE a.[Completed] WHEN 1 THEN a.[IdInteraction] ELSE NULL END)
							+ 
							COUNT(DISTINCT CASE a.[Status] WHEN 0 THEN a.[IdInteraction] ELSE NULL END)
						)
					 AS NUMERIC(10,2)) 
				)
				/
				CASE COUNT(DISTINCT a.[IdInteraction]) WHEN 0 THEN 1 ELSE COUNT(DISTINCT a.[IdInteraction]) END 
			) * 100																							'completedPercentage',
			e.[Percentage]					'percentage',
			e.[StatusDescription]			'statusDescription',
			e.[BatchStatus]					'batchStatus'		
		FROM EvaluationPlan e (NOLOCK)
		LEFT JOIN AuditorEvaluationPlan a (NOLOCK) 
			ON a.[Instance] = e.[Instance]
				AND a.[IdEvaluationPlan] = e.[ID]
		INNER JOIN EvaluationFormat ef (NOLOCK)
			ON ef.[Instance] = e.[Instance]
				AND ef.[ID] = e.[IdEvaluationFormat]
        
		WHERE e.Instance = @VCC
			AND e.Evaluation_Mode IN ('A','X')
			AND e.Status = '7'			
		GROUP BY e.[ID],e.[Modality],e.[DateFin],e.CreatedDate,e.CreatedByUserId,
				 e.LastModifiedDate,e.LastModifiedByUserId,e.Description,e.IdEvaluationFormat,
				 e.Enabled, e.Status, e.Evaluation_Mode,e.processId,ef.templateId, 
				 e.Percentage, e.StatusDescription, e.[BatchStatus]	
		
		
		--SELECCIONAR PLANES DE EFECTIVIDAD GENERADOS PARA LA GRILLA DE ADMINISTRACION 

		;WITH Temp_EfectivityPlan AS (

		SELECT DISTINCT 
		[EfectivityPlanId],[EvaluationPlanId],[InteractionId],[EfectivityStatus] 
		FROM dbo.[EvaluationResultEfectivity] (NOLOCK)
		WHERE [VCC] = @VCC
		),
		Temp_EfectivityPlan_Result  AS (
		SELECT [EfectivityPlanId],[EvaluationPlanId],COUNT(InteractionId)  'total',
		COUNT(DISTINCT CASE WHEN [EfectivityStatus] = 'COMPLETED' THEN [InteractionId] ELSE NULL END)					'completedEvaluations',
		CAST(CAST(COUNT(DISTINCT CASE WHEN [EfectivityStatus] = 'COMPLETED' THEN [InteractionId] ELSE NULL END) AS DECIMAL(10,2))*100
		/
		CAST(COUNT(InteractionId) AS DECIMAL(10,2)) AS DECIMAL)															'completedPercentage'
		
        FROM Temp_EfectivityPlan EP1		

		WHERE  ISNULL(EfectivityPlanId, '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
			
		
		GROUP BY  EfectivityPlanId,EvaluationPlanId
		)		

		SELECT DISTINCT 
		TER.[EfectivityPlanId] 'id',
		TER.[EvaluationPlanId] 'evaluationPlanId',
		TER.[total] 'total',
		TER.[completedEvaluations] 'completedEvaluations',
		TER.[completedPercentage]	'completedPercentage',	
		TE.[CreatedDate] 'createdDate', 
		CASE WHEN TER.[completedPercentage] = 100 THEN 'COMPLETED' 
			 WHEN TER.[completedPercentage] = 0 THEN 'CREATED' ELSE 'IN_PROCESS' END  'statusDescription',
		TE.[AuditorId]	'auditorId'
		
		FROM Temp_EfectivityPlan_Result TER 
		INNER JOIN (SELECT DISTINCT [EfectivityPlanId],[EvaluationPlanId],[CreatedDate],[AuditorId] FROM dbo.[EvaluationResultEfectivity]
					WHERE VCC = @VCC) AS TE ON 
		TER.EfectivityPlanId = TE.EfectivityPlanId
	    AND TER.EvaluationPlanId = TE.EvaluationPlanId
		

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEfectivityPlansAuditor] (
	@Instance VARCHAR(50) = NULL,
	@UserId VARCHAR(50) = NULL,
	@Campaigns VARCHAR(MAX) = NULL,	
	@EvaluationPlans VARCHAR(MAX) = NULL,	
	@Completed BIT = NULL,
	@UserFunction VARCHAR(50) = NULL,	
	@EfectivityPlans VARCHAR(MAX) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	DECLARE @USER_ROLE VARCHAR(50) = ''
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''	
	DECLARE @SQL_FILTER VARCHAR(MAX) = ''	
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''	
	DECLARE @SQL_SELECT_2 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM_2 VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER_2 VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA_2 VARCHAR(MAX) = ''

	--VARIABLES PARA EFECTIVIDAD
	DECLARE @SQL_SELECT_1_EFECTIVITY VARCHAR(MAX) = ''
	DECLARE @SQL_FROM_EFECTIVITY VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER_EFECTIVITY VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA_EFECTIVITY VARCHAR(MAX) = ''
	

	-- ARMAMOS EL SELECT DE LA QUERY
    
	SET @SQL_SELECT_1_EFECTIVITY += '
		SELECT EP.[ID]																								''id'', 
				ERE.[EfectivityPlanId]																				''efectivityPlanId'',
				EP.[Description]																					''description'',				
				EP.[IdEvaluationFormat]																				''idEvaluationFormat'',
				COUNT(DISTINCT ERE.[InteractionId])																	''totalInteraction'',
				COUNT(DISTINCT CASE ERE.[EfectivityStatus]  WHEN ''COMPLETED'' THEN ERE.[InteractionId] ELSE NULL END)					''completedInteraction'',
				COUNT(DISTINCT ERE.[InteractionId]) -COUNT(DISTINCT CASE ERE.[EfectivityStatus]  WHEN ''COMPLETED'' THEN ERE.[InteractionId] ELSE NULL END)  ''pendingInteraction''
				'
				

    
	SET @SQL_FROM_EFECTIVITY +='
			FROM [EvaluationPlan] EP (NOLOCK)
			INNER JOIN [AuditorEvaluationPlan] AEP (NOLOCK)
				ON (AEP.[Instance] = EP.[Instance] 
					AND AEP.[IdEvaluationPlan] = EP.[ID])
			INNER JOIN ( SELECT DISTINCT [VCC],[InteractionId],[AgentId],[EfectivityPlanId],[EvaluationPlanId],[AuditorId],[EfectivityStatus] FROM [dbo].[EvaluationResultEfectivity] 
						 WHERE [VCC] =' +''''+ @Instance +'''' +

			')AS ERE ON 
				(AEP.[Instance] = ERE.[VCC] 
					AND AEP.[IdEvaluationPlan] = ERE.[EvaluationPlanId]
					AND AEP.[IdInteraction] = ERE.[InteractionId])'

	-- HACEMOS EL FILTRADO SEGUN EL ROL DEL USUARIO
	IF @UserFunction = 'auditor'
	BEGIN
		SET @SQL_FROM_EFECTIVITY += '
					AND ERE.[AuditorId] = ''' + @UserId + ''''
	END
	ELSE IF (@UserFunction = 'admin')
	BEGIN	
		SET @SQL_FROM_EFECTIVITY += ')'
	END
	ELSE 
	BEGIN		
		SET @SQL_FROM_EFECTIVITY += '
					AND ERE.[AuditorId] = ''' + @UserId + ''') '
	END 

	
     SET @SQL_FILTER_EFECTIVITY += '
			WHERE EP.[Instance] = ''' + @Instance + '''				
				AND EP.[Enabled] = 1 
				AND (
					(
						EP.[Evaluation_Mode] = ''X'' 
						AND 
						(
							EP.[Status] = 7
							OR
							EP.[Status] = 8
						)
					)
					OR
					(EP.[Evaluation_Mode] = ''A'')
				) '
				
					
				

	IF (@Campaigns IS NOT NULL AND @Campaigns <> '')
	BEGIN
		
		SET @SQL_FILTER_EFECTIVITY += '
				AND EP.[IdEvaluationFormat] IN (
					SELECT CEF.[IdEvaluationFormat] FROM CampaignEvaluationFormat CEF 
						WHERE CEF.[Instance] = ''' + @Instance + '''
							AND CEF.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','',''))) '
	END

	IF (@EvaluationPlans IS NOT NULL AND @EvaluationPlans <> '') SET @SQL_FILTER += 'AND EP.[ID] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationPlans + ''','','')) '
	IF (@EfectivityPlans IS NOT NULL AND @EfectivityPlans <> '') SET @SQL_FILTER_EFECTIVITY += 'AND ERE.[EfectivityPlanId] IN (SELECT Value FROM [dbo].[split](''' + @EfectivityPlans + ''','','')) '
	
	-- VARIABLE PARA FILTRAR SOLO LOS REGISTROS DE AUDITOREVALUATIONPLAN CON [COMPLETED] = 1
	IF (@Completed = 1)
	BEGIN
		SET @SQL_FILTER += 'AND AEP.[Completed] = 1 '
		SET @SQL_FILTER_2 = ' '
	END
	ELSE 
	BEGIN
		-- SI NO LLEGA UN VALOR PARA EVALUATION PLANS, BUSCAMOS SOLO LOS QUE TIENEN INTERACCIONES PENDIENTES
		IF (@EfectivityPlans IS NULL OR @EfectivityPlans = '')
		BEGIN
		
			SET @SQL_FILTER_2 = 'WHERE X.[pendingInteraction] > 0 '
		END
	END
	

	
	SET @SQL_EXTRA_EFECTIVITY += '
			GROUP BY EP.[ID],
						ERE.[EfectivityPlanId],
						EP.[Modality],
						EP.[DateFin],
						EP.[CreatedDate],
						EP.[IdEvaluationFormat],
						EP.[Description]'

		SET @SQL_SELECT_1_EFECTIVITY += @SQL_FROM_EFECTIVITY + @SQL_FILTER_EFECTIVITY + @SQL_EXTRA_EFECTIVITY
		
		SET @SQL_SELECT_2 = 'SELECT * '
		SET @SQL_FROM_2 = 'FROM (' + @SQL_SELECT_1_EFECTIVITY + ') AS X '
	
		SET @SQL_EXTRA_2 = 'ORDER BY X.[Description] ASC'

		SET @SQL_SELECT_2 += @SQL_FROM_2 + @SQL_FILTER_2 + @SQL_EXTRA_2

		--PRINT @SQL_SELECT_2;
		EXEC (@SQL_SELECT_2)

	
	
	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEntities] (
	@VCC VARCHAR(50) = NULL, 
	@AppId VARCHAR(100) = NULL,
	@SearchCriteria VARCHAR(50) = NULL, 
	@SearchExpression VARCHAR(50) = NULL,
	@Entities VARCHAR(MAX)
) AS
BEGIN
	DECLARE @COGNITIVE_ENGINE VARCHAR(5) = NULL
	DECLARE @SQL NVARCHAR(MAX)

	-- DETERMINAMOS EL MOTOR COGNITIVO DEL VCC
	SELECT @COGNITIVE_ENGINE = N.[CognitiveEngine]
		FROM [dbo].[NLU] N (NOLOCK)
		WHERE N.[VCC] = @VCC
			AND N.[AppId] = @AppId

	SET @SQL = '
	SET NOCOUNT ON
	SELECT E.[ID]							    ''id'',
			E.[Name]						    ''name'',
			E.[AppId]							''appId'',
			E.[Type]							''type'',
			E.[CreatedDate]						''createdDate'',
			E.[CreatedByUserId]					''createdByUserId'',
			E.[LastModifiedDate]				''lastModifiedDate'',
			E.[LastModifiedByUserId]			''lastModifiedByUserId'',
			E.[Status]                           ''status'',
			E.[StatusCode]                      ''statusCode''
		FROM [dbo].[Entity] E (NOLOCK)
		WHERE E.[VCC] = ''' + @VCC + ''' 
			AND E.[AppId] = ''' + @AppId + ''' '
	
	-- PARA NLUS DE LUIS, FILTRAMOS SEGUN EL INPUT
	IF @COGNITIVE_ENGINE = 'LUIS'
	BEGIN
		SET @SQL +=	'
			AND E.[ID] IN (''' + @Entities + ''') '
	END

	SET @SQL +=	'
			AND ISNULL(E.[Name], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
		ORDER BY E.[LastModifiedDate] ASC, E.[Name] DESC
	'
	
	SET @SQL += ' SET NOCOUNT OFF'
	
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEntity] (
	@VCC				VARCHAR(50) = NULL,
	@AppId				VARCHAR(50) = NULL,
	@ID					VARCHAR(50) = NULL,
	@Name				VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'vcc',
		[AppId]							'appId',
		[ID]							'id',
		[Name]							'name',
		[Type]							'type',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [dbo].[Entity] E(NOLOCK)
	WHERE E.[VCC] = @VCC
		AND E.[AppId] = @AppId
		AND E.[ID] = CASE WHEN @ID IS NULL THEN E.[ID] ELSE @ID END
		AND E.[Name] = CASE WHEN @Name IS NULL THEN E.[Name] ELSE @Name END 

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEnvironment] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT [VCC]				'vcc',
			[Name]					'name',
			[Type]					'type',
			[Description]			'description',
			[FTPDirectory]			'ftpDirectory',
			[ApiHost]				'apiHost',
			[ApiPort]				'apiPort',
			[Bucket]				'bucket',
			[ApiUser]				'apiUser',
			[ApiPassword]			'apiPassword',
			[DbDisabled]			'dbDisabled',
			[DbServer]				'dbServer',
			[DbPort]				'dbPort',
			[DbUser]				'dbUser',
			[DbPassword]			'dbPassword',
			[PbxDisabled]			'pbxDisabled',
			[PbxServer]			    'pbxServer',
			[PbxPort]			    'pbxPort',
			[PbxUser]			    'pbxUser',
			[PbxPassword]			'pbxPassword',
			[CreatedDate]			'createdDate',
			[CreatedByUserId]		'createdByUserId',
			[LastModifiedDate]		'lastModifiedDate',
			[LastModifiedByUserId]	'lastModifiedByUserId'
		FROM [Environment] (NOLOCK)
		WHERE [VCC] = @VCC
			AND [Name] = @Name
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEnvironmentByName] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]					'name',
		[Type]					'type',
		[Description]			'description',
		[FTPDirectory]			'ftpDirectory',
		[ApiHost]				'apiHost',
		[ApiPort]				'apiPort',
		[ApiUser]				'apiUser',
		[ApiPassword]			'apiPassword',
		[DbDisabled]			'dbDisabled',
		[DbServer]				'dbServer',
		[DbPort]				'dbPort',
		[DbUser]				'dbUser',
		[DbPassword]			'dbPassword',
		[PbxDisabled]			'pbxDisabled',
		[PbxServer]			    'pbxServer',
		[PbxPort]			    'pbxPort',
		[PbxUser]			    'pbxUser',
		[PbxPassword]			'pbxPassword',
		[CreatedDate]			'createdDate',
		[CreatedByUserId]		'createdByUserId',
		[LastModifiedDate]		'lastModifiedDate',
		[LastModifiedByUserId]	'lastModifiedByUserId'
	FROM [Environment] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [Name] = @Name
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEnvironments] (
	@VCC Varchar(50) = NULL, 
	@SearchCriteria Varchar(50) = NULL, 
	@SearchExpression Varchar(50) = NULL,
	@IncludeIntegrated Bit = 1
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]					'name',
		[Type]					'type',
		[Description]			'description',
		[FTPDirectory]			'ftpDirectory',
		[ApiHost]				'apiHost',
		[ApiPort]				'apiPort',
		[ApiUser]				'apiUser',
		[DbDisabled]			'dbDisabled',
		[DbServer]				'dbServer',
		[DbPort]				'dbPort',
		[DbUser]				'dbUser',
		[PbxDisabled]			'pbxDisabled',
		[PbxServer]			    'pbxServer',
		[PbxPort]			    'pbxPort',
		[PbxUser]			    'pbxUser',
		[CreatedDate]			'createdDate',
		[CreatedByUserId]		'createdByUserId',
		[LastModifiedDate]		'lastModifiedDate',
		[LastModifiedByUserId]	'lastModifiedByUserId'
	FROM [Environment] (NOLOCK)
	WHERE [VCC] = @VCC
		--AND [Type] NOT LIKE CASE WHEN @IncludeIntegrated = 0 THEN '%allegro%' ELSE '' END
		AND (
			(
				@IncludeIntegrated = 0
				AND
				[Type] NOT IN ('allegro', 'i6')
			)
			OR 
			@IncludeIntegrated = 1
		)
		AND ISNULL([Name], '') LIKE CASE WHEN @SearchCriteria = 'name' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL([ApiHost], '') LIKE CASE WHEN @SearchCriteria = 'host' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL([Type], '') LIKE CASE WHEN @SearchCriteria = 'type' THEN '%' + @SearchExpression + '%' ELSE '%' END
		
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEnvironmentsUsage]
(
	@VCC				VARCHAR(50),
	@Environments		VARCHAR(MAX)= NULL
)
AS
BEGIN
	SET NOCOUNT ON
    DECLARE @SQL_PLAN VARCHAR(MAX) = ''
	DECLARE @SQL_PROCESS VARCHAR(MAX) = ''
	   -- Validar para EvalutionPlan.
	SET @SQL_PLAN='SELECT DISTINCT [Description]	''plan'' FROM
			[dbo].[EvaluationPlan] WHERE [Instance] = '''+@VCC+''' AND [Status] < 5 AND '
			
	SET @SQL_PLAN+='[EnvironmentId] IN(SELECT Value FROM [dbo].[split](''' + @Environments + ''','','')) '
			
	  -- Validar para Process .
  	SET @SQL_PROCESS='SELECT
			DISTINCT [Name]	''process'' FROM
			[dbo].[Process] WHERE [VCC] = '''+@VCC+'''  AND '
	SET @SQL_PROCESS+='[Status] NOT IN (''COMPLETE'',''FAILED'') AND '
			
	SET @SQL_PROCESS+='[Environment] IN(SELECT Value FROM [dbo].[split](''' + @Environments + ''','','')) '
	
    EXEC(@SQL_PLAN)
    EXEC(@SQL_PROCESS)
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEnvironmentTypeByName]
(
	@VCC		VARCHAR(50),
	@Name		VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]			'name',
		[Type]			'type'
	FROM
		[dbo].[Environment] (NOLOCK)
	WHERE
		[VCC] = @VCC
		AND [Name] = @Name

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluatedInteractions] (
    @Instance Varchar(50) = NULL,
	@IdAuditor VARCHAR(MAX) = NULL,
	@IdAgent Varchar(MAX) = NULL,
   	@IdCampaign VARCHAR(MAX) = NULL,
 	@EvaluationDateFrom VARCHAR(50) = '1980-01-01T00:00:00',
    @EvaluationDateTo VARCHAR(50) = '2050-01-01T23:59:59',
	@EvaluationHourFrom VARCHAR(50) = '00:00:00',
    @EvaluationHourTo VARCHAR(50) = '23:59:59',
	@InteractionMinDuration Int = 0,
    @InteractionMaxDuration Int = 86400,
    @InteractionOrigins Varchar(50) = NULL,
	@InteractionResults Varchar(MAX) = NULL,
	@InteractionTags Varchar(MAX) = NULL,
	@InteractionFormats Varchar(50) = NULL,
	@EvaluationsLimit Int = 0
)
AS
BEGIN
   	SET NOCOUNT ON
	IF (@EvaluationHourFrom = '') SET @EvaluationHourFrom = '00:00:00';
	IF (@EvaluationHourTo = '') SET @EvaluationHourTo = '23:59:59';

	DECLARE @SQL_SELECT_0 VARCHAR(MAX) = '';
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = '';
	DECLARE @SQL_FROM VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA VARCHAR(MAX) = '';
	
	SET @SQL_SELECT_1 += 'SELECT ';
	SET @SQL_SELECT_1 += '
				AEP.[IdInteraction]					''id'',
    			AEP.[IdCampaign]					''idCampaign'',
    			AEP.[IdCampaign]					''descriptionCampaign'',
    			AEP.[InteractionDate]				''dateIni'',
    			AEP.[InteractionDuration]			''interactionTime'',
    			AEP.[InteractionType]				''interactionType'' ';

	SET @SQL_FROM += '
			FROM EvaluationResult ER (NOLOCK) 
			LEFT JOIN [AuditorEvaluationPlan] AEP WITH (NOLOCK) 
				ON (AEP.[Instance] = ER.[Instance] 
					AND AEP.[IdAuditor] = ER.[IdEvaluator] 
					AND AEP.[IdInteraction] = ER.[IdInteraction] 
					AND AEP.[InteractionType] = ER.[InteractionType] 
					AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]) ';

	SET @SQL_FILTER += '
			WHERE ER.[Instance] = ''' + @Instance + '''
				AND ER.[EvaluationDate] >= ''' + @EvaluationDateFrom + '''
				AND ER.[EvaluationDate] <= ''' + @EvaluationDateTo + ''' ';

	IF (@IdAuditor IS NOT NULL AND @IdAuditor != '') SET @SQL_FILTER += 'AND AEP.[IdAuditor] IN (SELECT Value FROM [dbo].[split](''' + @IdAuditor + ''','','')) ';
	IF (@IdAgent IS NOT NULL AND @IdAgent != '') SET @SQL_FILTER += 'AND AEP.[IdAgent] IN (SELECT Value FROM [dbo].[split](''' + @IdAgent + ''','','')) ';
	IF (@IdCampaign IS NOT NULL AND @IdCampaign != '') SET @SQL_FILTER += 'AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @IdCampaign + ''','','')) ';
	IF (CAST(@EvaluationHourFrom AS TIME) <= CAST(@EvaluationHourTo AS TIME)) SET @SQL_FILTER += 'AND CAST(ER.[EvaluationDate] AS TIME) BETWEEN ''' + @EvaluationHourFrom + ''' AND ''' + @EvaluationHourTo + ''' ';
	IF (CAST(@EvaluationHourFrom AS TIME) > CAST(@EvaluationHourTo AS TIME)) SET @SQL_FILTER += 'AND CAST(ER.[EvaluationDate] AS TIME) NOT BETWEEN ''' + @EvaluationHourTo + ''' AND ''' + @EvaluationHourFrom + ''' ';
	IF (@InteractionMinDuration IS NOT NULL) SET @SQL_FILTER += 'AND AEP.[InteractionDuration] >= ' + (SELECT CONVERT(varchar(10), @InteractionMinDuration)) + ' ';
	IF (@InteractionMaxDuration IS NOT NULL) SET @SQL_FILTER += 'AND AEP.[InteractionDuration] <= ' + (SELECT CONVERT(varchar(10), @InteractionMaxDuration))+ ' ';
	IF (@InteractionOrigins IS NOT NULL AND @InteractionOrigins != '') SET @SQL_FILTER += 'AND AEP.[InteractionOrigin] IN (SELECT Value FROM [dbo].[split](''' + @InteractionOrigins + ''','','')) ';
	IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') SET @SQL_FILTER += 'AND AEP.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')) ';
	IF (@InteractionTags IS NOT NULL AND @InteractionTags != '') 
	BEGIN
		SET @SQL_FROM += '
			LEFT JOIN [Tag] T WITH (NOLOCK) 
				ON (T.[VCC] = ER.[Instance] 
					AND T.[Tag] IN (SELECT Value FROM [dbo].[split](''' + @InteractionTags + ''','',''))) ';
		SET @SQL_FILTER += '
				AND T.[AudioId] LIKE CONCAT(AEP.[IdInteraction], ''%'') 
				AND T.[AudioId] IS NOT NULL ';
	END;
	IF (@InteractionFormats IS NOT NULL AND @InteractionFormats != '') SET @SQL_FILTER += 'AND AEP.[InteractionFormat] IN (SELECT Value FROM [dbo].[split](''' + @InteractionFormats + ''','','')) ';
	
	SET @SQL_EXTRA += '
			GROUP BY AEP.IdInteraction, 
						AEP.[IdCampaign], 
						AEP.[IdCampaign], 
						AEP.[InteractionDate], 
						AEP.[InteractionDuration], 
						AEP.[InteractionType], 
						AEP.IdAgent
			ORDER BY ER.EvaluationDate DESC;';

	SET @SQL_SELECT_1 += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA;

	EXEC (@SQL_SELECT_1);

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluatedInteractionsByAuditor] (
   	@Instance Varchar(50) = NULL,
	@IdAuditor VARCHAR(MAX) = NULL,
	@IdAgent Varchar(MAX) = NULL,
   	@IdCampaign VARCHAR(MAX) = NULL,
	@IdEvaluationPlan VARCHAR(MAX) = NULL,
 	@EvaluationDateFrom VARCHAR(50) = '1980-01-01T00:00:00',
    @EvaluationDateTo VARCHAR(50) = '2050-01-01T23:59:59',
	@EvaluationHourFrom VARCHAR(50) = '00:00:00',
    @EvaluationHourTo VARCHAR(50) = '23:59:59',
	@InteractionMinDuration Int = 0,
    @InteractionMaxDuration Int = 86400,
    @InteractionOrigins Varchar(50) = NULL,
	@InteractionResults Varchar(MAX) = NULL,
	@InteractionTags Varchar(MAX) = NULL,
	@InteractionFormats Varchar(50) = NULL,
	@InteractionIds    VARCHAR(MAX) = NULL,
	@EvaluationsLimit Int = 0
)
AS
BEGIN
   	SET NOCOUNT ON
	IF (@EvaluationHourFrom = '') SET @EvaluationHourFrom = '00:00:00';
	IF (@EvaluationHourTo = '') SET @EvaluationHourTo = '23:59:59';

	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = '';
	DECLARE @SQL_FROM VARCHAR(MAX) = '';
	DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA VARCHAR(MAX) = '';
    DECLARE @FILTERS_FIXED_PART VARCHAR(MAX) = '';
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = '';
	
	SET @SQL_SELECT_1 += 'SELECT ';
	SET @SQL_SELECT_1 += '
				AEP.IdInteraction					''id'',
   				AEP.[IdCampaign]					''idCampaign'',
   				AEP.[IdCampaign]					''descriptionCampaign'',
 				AEP.[IdAgent]						''idAgent'',
 				AEP.[InteractionDate]				''dateIni'',
				AEP.[InteractionDateEnd]			''dateFin'',
   				AEP.[InteractionDuration]			''interactionTime'',
   				AEP.[InteractionType]				''interactionType'',
				AEP.[IdAgent]		                ''agentName'',
				AEP.[InteractionOrigin]					''origin'',
			AEP.[InteractionFormat]						''format'',
				AEP.[ManagementResult]					''dispositionCode'',
			AEP.[ContactId]										''address'' ';

	SET @SQL_FROM += '
			FROM EvaluationResult ER (NOLOCK) 
			LEFT JOIN [AuditorEvaluationPlan] AEP (NOLOCK)
				ON (AEP.[Instance] = ER.[Instance] 
					AND AEP.[IdAuditor] = ER.[IdEvaluator] 
					AND AEP.[IdInteraction] = ER.[IdInteraction] 
					AND AEP.[InteractionType] = ER.[InteractionType] 
					AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]) ';

	SET @FILTERS_FIXED_PART = '
			WHERE ER.[Instance] = ''' + @Instance + '''
				AND AEP.[InteractionDate] >= ''' + @EvaluationDateFrom + '''
				AND AEP.[InteractionDate] <= ''' + @EvaluationDateTo + '''
				AND AEP.[Completed] = 1	';

	IF (@IdAuditor IS NOT NULL AND @IdAuditor != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdAuditor] IN (SELECT Value FROM [dbo].[split](''' + @IdAuditor + ''','','')) ';
	IF (@IdAgent IS NOT NULL AND @IdAgent != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdAgent] IN (SELECT Value FROM [dbo].[split](''' + @IdAgent + ''','','')) ';
	IF (@IdCampaign IS NOT NULL AND @IdCampaign != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @IdCampaign + ''','','')) ';
	IF (@IdEvaluationPlan IS NOT NULL AND @IdEvaluationPlan != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdEvaluationPlan] IN (SELECT Value FROM [dbo].[split](''' + @IdEvaluationPlan + ''','','')) ';
	IF (CAST(@EvaluationHourFrom AS TIME) <= CAST(@EvaluationHourTo AS TIME)) SET @FILTERS_OPTIONAL_PART += 'AND CAST(AEP.[InteractionDate] AS TIME) BETWEEN ''' + @EvaluationHourFrom + ''' AND ''' + @EvaluationHourTo + ''' ';
	IF (CAST(@EvaluationHourFrom AS TIME) > CAST(@EvaluationHourTo AS TIME)) SET @FILTERS_OPTIONAL_PART += 'AND CAST(AEP.[InteractionDate] AS TIME) NOT BETWEEN ''' + @EvaluationHourTo + ''' AND ''' + @EvaluationHourFrom + ''' ';
	IF (@InteractionMinDuration IS NOT NULL) SET @FILTERS_OPTIONAL_PART += 'AND AEP.[InteractionDuration] >= ' + (SELECT CONVERT(varchar(10), @InteractionMinDuration)) + ' ';
	IF (@InteractionMaxDuration IS NOT NULL) SET @FILTERS_OPTIONAL_PART += 'AND AEP.[InteractionDuration] <= ' + (SELECT CONVERT(varchar(10), @InteractionMaxDuration))+ ' ';
	IF (@InteractionOrigins IS NOT NULL AND @InteractionOrigins != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[InteractionOrigin] IN (SELECT Value FROM [dbo].[split](''' + @InteractionOrigins + ''','','')) ';
	IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')) ';
	IF (@InteractionTags IS NOT NULL AND @InteractionTags != '') 
	BEGIN
		SET @SQL_FROM += '
			LEFT JOIN [Tag] T (NOLOCK) 
				ON (T.[VCC] = ER.[Instance] 
					AND T.[Tag] IN (SELECT Value FROM [dbo].[split](''' + @InteractionTags + ''','',''))) ';
		SET @FILTERS_OPTIONAL_PART += '
				AND T.[AudioId] LIKE CONCAT(AEP.[IdInteraction], ''%'') 
				AND T.[AudioId] IS NOT NULL ';
	END;
	--IF (@InteractionFormats IS NOT NULL AND @InteractionFormats != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[InteractionFormat] IN (SELECT Value FROM [dbo].[split](''' + @InteractionFormats + ''','','')) ';
	
	IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
    BEGIN
        SET @FILTERS_OPTIONAL_PART = '';
	    SET @FILTERS_EXCLUDING_PART=' AND AEP.IdInteraction IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) ';
	END    
	SET @SQL_EXTRA += '
			GROUP BY AEP.[IdInteraction], 
						AEP.[IdAuditor], 
						AEP.[IdCampaign], 
						AEP.[IdAgent], 
						AEP.[InteractionDate], 
						AEP.[InteractionDateEnd],
						AEP.[InteractionDuration], 
						AEP.[InteractionType], 
						AEP.[InteractionOrigin], 
						AEP.[InteractionFormat], 
						AEP.[ManagementResult],
						AEP.[ContactId]; ';

	SET @SQL_SELECT_1 += @SQL_FROM + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART + @SQL_EXTRA;

	/*select @SQL_SELECT_1;*/
	EXEC (@SQL_SELECT_1);

	/*


       SELECT
   	    e.IdInteraction					'id',
   		e.[IdCampaign]					'idCampaign',
   		e.[IdCampaign]					'descriptionCampaign',
 		e.[IdAgent]						'idAgent',
 		e.[InteractionDate]				'dateIni',
   		e.[InteractionDuration]			'interactionTime',
   		e.[InteractionType]				'interactionType',
        e.IdAgent		                'agentName'
   	FROM EvaluationResult r, AuditorEvaluationPlan e (NOLOCK)
   	WHERE
   	e.Instance = @Instance
   	AND r.IdEvaluator = @IdAuditor
   	AND r.EvaluationDate >= @EvaluationDateFrom
   	AND r.EvaluationDate <= @EvaluationDateTo
	AND e.instance=r.instance
	AND r.IdEvaluator = e.IdAuditor
	AND r.IdInteraction = e.IdInteraction
	AND r.IdEvaluationPlan = e.IdEvaluationPlan
	AND r.[InteractionType] = e.[InteractionType]
   	AND CASE WHEN @IdCampaign = 'GetEvaluatedInteractionsByAuditor' THEN 'GetEvaluatedInteractionsByAuditor' ELSE e.[IdCampaign] END IN (SELECT Value FROM [dbo].[split](@IdCampaign,','))
   	AND CASE WHEN @IdAgents = 'GetEvaluatedInteractionsByAuditor' THEN 'GetEvaluatedInteractionsByAuditor' ELSE e.[IdAgent] END IN (SELECT Value FROM [dbo].[split](@IdAgents,','))
   	group by e.IdInteraction, e.[IdCampaign], e.[IdCampaign], e.[IdAgent], e.[InteractionDate], e.[InteractionDuration], e.[InteractionType], e.IdAgent
	*/
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluatedInteractionsByCampaignsAuditorsAndAgents] (
	@Instance Varchar(50) = NULL,
	@IdAuditor VARCHAR(MAX) = NULL,
	@IdAgent Varchar(MAX) = NULL,
   	@IdCampaign VARCHAR(MAX) = NULL,
 	@EvaluationDateFrom VARCHAR(50) = '1980-01-01T00:00:00',
    @EvaluationDateTo VARCHAR(50) = '2050-01-01T23:59:59',
	@EvaluationHourFrom VARCHAR(50) = '00:00:00',
    @EvaluationHourTo VARCHAR(50) = '23:59:59',
	@InteractionMinDuration Int = 0,
    @InteractionMaxDuration Int = 86400,
    @InteractionOrigins Varchar(50) = NULL,
	@InteractionResults Varchar(MAX) = NULL,
	@EvaluationTypes Varchar(MAX) = NULL,
	@InteractionTags Varchar(MAX) = NULL,
	@InteractionFormats Varchar(50) = NULL,
	@EvaluationsLimit Int = 0,
	@ContactAddress Varchar(300) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	IF (@EvaluationHourFrom = '') SET @EvaluationHourFrom = '00:00:00';
	IF (@EvaluationHourTo = '') SET @EvaluationHourTo = '23:59:59';

	DECLARE @SQL_SELECT_0 VARCHAR(MAX) = '';
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = '';
	DECLARE @SQL_FROM VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA VARCHAR(MAX) = '';

	SET @SQL_SELECT_1 += 'SELECT ';
	SET @SQL_SELECT_1 += '
				AEP.IdInteraction					''id'',
   				AEP.[IdCampaign]					''idCampaign'',
   				AEP.[IdCampaign]					''descriptionCampaign'',
				/*AEP.[IdAuditor]						''idAuditor'',*/
 				AEP.[IdAgent]						''idAgent'',
				AEP.[InteractionDate]				''dateIni'',
   				AEP.[InteractionDuration]			''interactionTime'',
   				AEP.[InteractionType]				''interactionType'',
				AEP.[IdAgent]		                ''agentName'',
				AEP.[InteractionOrigin]				''origin'',
				AEP.[InteractionFormat]				''format'',
				AEP.[ManagementResult]				''dispositionCode'',
				AEP.[ContactAddress]						''address'' ';

	SET @SQL_FROM += '
			FROM EvaluationResult ER (NOLOCK) 
			LEFT JOIN [AuditorEvaluationPlan] AEP WITH (NOLOCK)
				ON (AEP.[Instance] = ER.[Instance] 
					AND AEP.[IdAuditor] = ER.[IdEvaluator] 
					AND AEP.[IdInteraction] = ER.[IdInteraction] 
					AND AEP.[InteractionType] = ER.[InteractionType] 
					AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]) ';

	SET @SQL_FILTER += '
			WHERE ER.[Instance] = ''' + @Instance + '''
				AND AEP.[InteractionDate] >= ''' + @EvaluationDateFrom + '''
				AND AEP.[InteractionDate] <= ''' + @EvaluationDateTo + '''
				AND AEP.[Completed] = 1	';

	IF (@IdAuditor IS NOT NULL AND @IdAuditor != '') SET @SQL_FILTER += 'AND AEP.[IdAuditor] IN (SELECT Value FROM [dbo].[split](''' + @IdAuditor + ''','','')) ';
	IF (@IdAgent IS NOT NULL AND @IdAgent != '') SET @SQL_FILTER += 'AND AEP.[IdAgent] IN (SELECT Value FROM [dbo].[split](''' + @IdAgent + ''','','')) ';
	IF (@IdCampaign IS NOT NULL AND @IdCampaign != '') SET @SQL_FILTER += 'AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @IdCampaign + ''','','')) ';
	IF (CAST(@EvaluationHourFrom AS TIME) <= CAST(@EvaluationHourTo AS TIME)) SET @SQL_FILTER += 'AND CAST(AEP.[InteractionDate] AS TIME) BETWEEN ''' + @EvaluationHourFrom + ''' AND ''' + @EvaluationHourTo + ''' ';
	IF (CAST(@EvaluationHourFrom AS TIME) > CAST(@EvaluationHourTo AS TIME)) SET @SQL_FILTER += 'AND CAST(AEP.[InteractionDate] AS TIME) NOT BETWEEN ''' + @EvaluationHourTo + ''' AND ''' + @EvaluationHourFrom + ''' ';
	IF (@InteractionMinDuration IS NOT NULL) SET @SQL_FILTER += 'AND AEP.[InteractionDuration] >= ' + (SELECT CONVERT(varchar(10), @InteractionMinDuration)) + ' ';
	IF (@InteractionMaxDuration IS NOT NULL) SET @SQL_FILTER += 'AND AEP.[InteractionDuration] <= ' + (SELECT CONVERT(varchar(10), @InteractionMaxDuration))+ ' ';
	IF (@InteractionOrigins IS NOT NULL AND @InteractionOrigins != '') SET @SQL_FILTER += 'AND AEP.[InteractionOrigin] IN (SELECT Value FROM [dbo].[split](''' + @InteractionOrigins + ''','','')) ';
	IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') SET @SQL_FILTER += 'AND AEP.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')) ';
	IF (@EvaluationTypes IS NOT NULL AND @EvaluationTypes != '') 
	BEGIN
		SET @SQL_FROM += '
			LEFT JOIN [EvaluationPlan] EP WITH (NOLOCK) 
				ON (EP.[Instance] = ER.[Instance] 
					AND EP.[ID] = ER.IdEvaluationPlan) ';
		SET @SQL_FILTER += '
				AND EP.[Evaluation_Mode] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationTypes + ''','',''))  ';
	END;
	IF (@InteractionTags IS NOT NULL AND @InteractionTags != '') 
	BEGIN
		SET @SQL_FROM += '
			LEFT JOIN [Tag] T WITH (NOLOCK) 
				ON (T.[VCC] = ER.[Instance] 
					AND T.[Tag] IN (SELECT Value FROM [dbo].[split](''' + @InteractionTags + ''','',''))) ';
		SET @SQL_FILTER += '
				AND T.[AudioId] LIKE CONCAT(AEP.[IdInteraction], ''%'') 
				AND T.[AudioId] IS NOT NULL ';
	END;
	IF (@ContactAddress IS NOT NULL AND @ContactAddress <> '')
	BEGIN
	SET @SQL_FILTER += '
			AND AEP.ContactAddress LIKE ''%' + @ContactAddress + '%'''
				
	END
	
	-- ESTE FILTRO NO APLICA POR EL MOMENTO YA QUE LA COLUMNA FORMAT SE ESTA GUARDANDO COMO NULL
	-- IF (@InteractionFormats IS NOT NULL AND @InteractionFormats != '') SET @SQL_FILTER += 'AND AEP.[InteractionFormat] IN (SELECT Value FROM [dbo].[split](''' + @InteractionFormats + ''','','')) ';

	SET @SQL_EXTRA += '
			GROUP BY AEP.[IdInteraction],        
						AEP.[IdCampaign],        
						/*AEP.[IdAuditor],*/        
						AEP.[IdAgent],        
						AEP.[InteractionDate],        
						AEP.[InteractionDuration],        
						AEP.[InteractionType],
						AEP.[InteractionOrigin],
						AEP.[InteractionFormat],
						AEP.[ManagementResult],
						AEP.[ContactAddress]; ';

	SET @SQL_SELECT_1 += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA;
	
	EXEC (@SQL_SELECT_1);
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationCustomFieldResultsFromAuditorEvaluationPlan] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(100) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@AuditorId VARCHAR(50) = NULL,
	@AgentId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON


			 SELECT ECFR.[VCC]								'vcc',
					ECFR.[EvaluationPlanId]					'evaluationPlanId',
					ECFR.[InteractionId]					'interactionId',
					ECFR.[AuditorId]						'auditorId',
					ECFR.[AgentId]							'agentId',
					ECFR.[CustomFieldId]					'customFieldId',
					CASE WHEN CFR.[ID] IS NULL 
						THEN ECFR.[Result] 
						ELSE CFR.[Description] END			'result'
				FROM [dbo].[EvaluationCustomFieldResult] ECFR (NOLOCK)
				LEFT JOIN [dbo].[HistoricCustomFieldResult] CFR (NOLOCK)
					ON CFR.[VCC] = ECFR.[VCC]
						AND CFR.[CustomFieldId] = ECFR.[CustomFieldId]
						AND CFR.[ID] =ECFR.[Result]
						AND CFR.[EvaluationPlanId] = ECFR.[EvaluationPlanId]
				WHERE ECFR.[VCC] = @VCC
					AND ECFR.[EvaluationPlanId] = @EvaluationPlanId
					AND ECFR.[InteractionId] = @InteractionId
					AND ECFR.[AuditorId] = @AuditorId
					AND (
						ECFR.[AgentId] = @AgentId
						OR
						(
							@AgentId IS NULL
							AND
							ECFR.[AgentId] = ECFR.[AgentId]
						)
					)


	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationFormat] (
	@Instance         Varchar(50) = NULL,
	@ID               Varchar(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DECLARE @QUERY VARCHAR(MAX) = ''
	
	SET @QUERY = 'SELECT
			[ID]									''id'',
			[Description]							''description'',
			[EnvironmentType]						''environmentType'',
			[ExecutionType]							''executionType'',
			[Type]									''type'',
			[MinQualificationAcceptable]			''minQualificationAcceptable'',
			[MinAutomaticQualification]				''minAutomaticQualification'',
			[ManagementFormURL]						''managementFormURL'',
			[Enabled]								''enabled'',
			[templateId]							''templateId'''

	IF(@EvaluationPlanId IS NOT NULL AND @EvaluationPlanId <> '')
	BEGIN
	   SET @QUERY += ' FROM [dbo].[HistoricEvaluationFormat] (NOLOCK)
	                   WHERE [VCC] = '''+@Instance+''' AND [ID] = '''+@ID+''' AND [EvaluationPlanId] = '''+@EvaluationPlanId+''''
	END
    ELSE
    BEGIN 
         SET @QUERY += ', [CreatedDate]							''createdDate'',
			[CreatedByUserId]						''createdByUserId'',
			[LastModifiedDate]						''lastModifiedDate'',
			[LastModifiedByUserId]					''lastModifiedByUserId''
		    FROM [dbo].[EvaluationFormat] (NOLOCK)
		    WHERE [Instance] = '''+@Instance+''' AND [ID] = ''' +@ID+''''
   END

    EXEC(@QUERY)
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationFormatAttachedSignatures] (
	@VCC                VARCHAR(50) = NULL, 
	@EvaluationFormatId VARCHAR(50) = NULL,
	@EvaluationPlanId   VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	
    DECLARE @FIND_SYSTEM BIT = 1
    
    IF @EvaluationPlanId IS NOT NULL AND @EvaluationPlanId <> ''
    BEGIN 
		    	-- LOS FORMATOS CREADOS PREVIAMENTE A LA IMPLEMENTACION DE FIRMAS ADJUNTAS NO RETORNAN RESULTADO
			IF EXISTS(
				SELECT 1
					FROM [dbo].[HistoricEvaluationFormatAttachedSignature] EFAS (NOLOCK)
					WHERE (
						(
							EFAS.[VCC] = @VCC 
							AND 
							EFAS.[EvaluationFormatId] = @EvaluationFormatId 
							AND 
							EFAS.[EvaluationPlanId] = @EvaluationPlanId)
						OR
						(
							EFAS.[VCC] = 'system' 
							AND 
							EFAS.[EvaluationFormatId] IS NULL 
							AND 
							@EvaluationFormatId IS NULL
						)
					)
			)
			BEGIN
		        SET @FIND_SYSTEM = 0
		        
				SELECT --EFAS.[VCC]							'vcc',
						EFAS.[Description]					'description',
						EFAS.[EvaluationFormatId]			'evaluationFormatId',
						EFAS.[Order]						'order'
					FROM [dbo].[HistoricEvaluationFormatAttachedSignature] EFAS (NOLOCK)
					WHERE (
						(
							EFAS.[VCC] = @VCC 
							AND 
							EFAS.[EvaluationFormatId] = @EvaluationFormatId 
							AND 
							EFAS.[EvaluationPlanId] = @EvaluationPlanId
						)
						OR
						(
							EFAS.[VCC] = 'system' 
							AND 
							EFAS.[EvaluationFormatId] IS NULL 
							AND 
							@EvaluationFormatId IS NULL
						)
					)
					ORDER BY EFAS.[Order] ASC
			END

    END 
    ELSE 
    BEGIN 
		    	-- LOS FORMATOS CREADOS PREVIAMENTE A LA IMPLEMENTACION DE FIRMAS ADJUNTAS NO RETORNAN RESULTADO
			IF EXISTS(
				SELECT 1
					FROM [dbo].[EvaluationFormatAttachedSignature] EFAS (NOLOCK)
					WHERE (
						(
							EFAS.[VCC] = @VCC 
							AND 
							EFAS.[EvaluationFormatId] = @EvaluationFormatId
						)
						OR
						(
							EFAS.[VCC] = 'system' 
							AND 
							EFAS.[EvaluationFormatId] IS NULL 
							AND 
							@EvaluationFormatId IS NULL
						)
					)
			)
			BEGIN
		        SET @FIND_SYSTEM = 0
				SELECT --EFAS.[VCC]							'vcc',
						EFAS.[Description]					'description',
						EFAS.[EvaluationFormatId]			'evaluationFormatId',
						EFAS.[Order]						'order'
					FROM [dbo].[EvaluationFormatAttachedSignature] EFAS (NOLOCK)
					WHERE (
						(
							EFAS.[VCC] = @VCC 
							AND 
							EFAS.[EvaluationFormatId] = @EvaluationFormatId
						)
						OR
						(
							EFAS.[VCC] = 'system' 
							AND 
							EFAS.[EvaluationFormatId] IS NULL 
							AND 
							@EvaluationFormatId IS NULL
						)
					)
					ORDER BY EFAS.[Order] ASC
			END

    END
    
    
    IF(@FIND_SYSTEM = 1)
    BEGIN 
    		-- LLAMAMOS POR DEFECTO LOS REGISTROS DE SYSTEM
		SELECT EFAS.[Description]					'description',
				EFAS.[EvaluationFormatId]			'evaluationFormatId',
				EFAS.[Order]						'order'
			FROM [dbo].[EvaluationFormatAttachedSignature] EFAS (NOLOCK)
			WHERE EFAS.[VCC] = 'system' 
				AND EFAS.[EvaluationFormatId] IS NULL
			ORDER BY EFAS.[Order] ASC
    END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationFormatCustomFields] (
	@VCC VARCHAR(50) = NULL, 
	@EvaluationFormatId VARCHAR(50) = NULL,
	@EvaluationPlanId   VARCHAR(50) = NULL
) AS 
BEGIN
	SET NOCOUNT ON
	DECLARE @QUERY VARCHAR(MAX) = ''
	SET @QUERY += ' SELECT EFCF.[VCC]						''vcc'',
					EFCF.[ID]								''id'',
					EFCF.[CustomFieldId]					''customFieldId'',
					EFCF.[Order]							''order'' FROM '

	IF(@EvaluationPlanId IS NOT NULL AND @EvaluationPlanId <> '')
	BEGIN

		SET @QUERY += ' [dbo].[HistoricEvaluationFormatCustomField] EFCF (NOLOCK)
				        WHERE EFCF.[EvaluationPlanId] ='''+@EvaluationPlanId+''' AND '
	END 
	ELSE
	BEGIN 
       SET @QUERY += ' [dbo].[EvaluationFormatCustomField] EFCF (NOLOCK) WHERE '
	END

	SET @QUERY += ' EFCF.[VCC] =  '''+@VCC + ''' AND EFCF.[EvaluationFormatId] ='''+ @EvaluationFormatId+ ''' 
				    ORDER BY EFCF.[Order] ASC '

	EXEC(@QUERY)

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationFormatDescription] (
	@Instance Varchar(50) = NULL, @Description Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]							'id',
		[Description]					'description',
		[Type]							'type',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [EvaluationFormat] (NOLOCK)
	WHERE [Instance] = @Instance
	AND [Description] = @Description

	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationFormatDescriptionsBySearchCriteria] (
	@Instance Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT DISTINCT
		E.[ID]							'id',
		[Description]					'description',
		[Type]							'type',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [EvaluationFormat] E, [AbilityEvaluationFormat] A (NOLOCK)
	WHERE E.ID = A.IdEvaluationFormat
	AND E.[Instance] = @Instance
	AND A.[Instance] = @Instance
	AND ISNULL(A.IdAbility, '') LIKE CASE WHEN @SearchCriteria = 'idAbility' THEN '%' + @SearchExpression + '%' ELSE '%' END
	AND ISNULL(A.IdGroup, '') LIKE CASE WHEN @SearchCriteria = 'idGroup' THEN '%' + @SearchExpression + '%' COLLATE Latin1_General_CI_AI ELSE '%' END
	AND ISNULL(A.IdSubGroup, '') LIKE CASE WHEN @SearchCriteria = 'idSubGroup' THEN '%' + @SearchExpression + '%' COLLATE Latin1_General_CI_AI ELSE '%' END

	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationFormatQualificationAverage] (
	@Instance Varchar(50) = NULL,
	@IdEvaluationFormat Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		EREFQS.[EvaluationResultEvaluationFormatQualificationSum] / EREFQ.[EvaluationResultEvaluationFormatQuantity] 'EvaluationFormatQualificationAverage'
	FROM
		(
			-- EvaluationResultEvaluationFormatQualificationSum
			SELECT
				SUM([QualificationSum]) EvaluationResultEvaluationFormatQualificationSum
				FROM
					(
						SELECT SUM(AEP.[Score])			'QualificationSum'
							FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
							INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
								ON EP.[Instance] = AEP.[Instance]
									AND EP.[ID] = AEP.[IdEvaluationPlan]
							WHERE AEP.[Instance] = @Instance
								AND EP.[IdEvaluationFormat]= @IdEvaluationFormat
								AND [Completed] = 1 
							GROUP BY
								AEP.[Instance],
								AEP.[IdEvaluationPlan],
								AEP.[IdAuditor],
								AEP.[IdAgent],
								AEP.[IdInteraction]
/*
			SELECT
				SUM([QualificationSum]) EvaluationResultEvaluationFormatQualificationSum
			FROM
				(
					SELECT
						SUM([Qualification]) QualificationSum
					FROM
						EvaluationResult
					WHERE
						[Instance] = @Instance
						AND [IdEvaluationFormat]= @IdEvaluationFormat
					GROUP BY
						[Instance],
						[IdEvaluationPlan],
						[IdAuditor],
						[IdAgent],
						[IdInteraction]
*/
				) EvaluationResultEvaluationFormat
		) EREFQS,
		(
			-- EvaluationResultEvaluationFormatQuantity
			SELECT
				COUNT(*) EvaluationResultEvaluationFormatQuantity
			FROM
				(
					SELECT COUNT(*)							'EvaluationResultEvaluationFormatQuantity'
						FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
						INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
							ON EP.[Instance] = AEP.[Instance]
								AND EP.[ID] = AEP.[IdEvaluationPlan]
						WHERE AEP.[Instance] = @Instance
							AND EP.[IdEvaluationFormat] = @IdEvaluationFormat
							AND [Completed] = 1
						GROUP BY
							AEP.[Instance],
							AEP.[IdEvaluationPlan],
							AEP.[IdAuditor],
							AEP.[IdAgent],
							AEP.[IdInteraction]
/*
					SELECT
						[IdEvaluationFormat]
					FROM
						EvaluationResult
					WHERE
						[Instance] = @Instance
						AND [IdEvaluationFormat]= @IdEvaluationFormat
					GROUP BY
						[Instance],
						[IdEvaluationPlan],
						[IdAuditor],
						[IdAgent],
						[IdInteraction],
						[IdEvaluationFormat]
*/
				) EvaluationResultEvaluationFormat
		) EREFQ

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationFormats] (
	@Instance Varchar(50) = NULL, 
	@SearchCriteria Varchar(50) = NULL, 
	@SearchExpression Varchar(50) = NULL, 
	@Disabled bit = NULL
) AS
BEGIN
	SET NOCOUNT ON;

	-- CREAMOS LAS CTE QUE USAREMOS PARA RECUPERAR LOS MODELOS DE LOS FORMATOS
	WITH CTE_FORMAT_NLUS AS (
		SELECT DISTINCT EF.[Instance]
						, EF.[TemplateId]
						, N.[AppID]
						, N.[Name]
						, N.[Culture]
			FROM [dbo].[EvaluationFormat] EF (NOLOCK)
			INNER JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
				ON AEF.[Instance] = EF.[Instance]
					AND AEF.[IdEvaluationFormat] = EF.[ID]
			INNER JOIN [dbo].[Ability] A (NOLOCK)
				ON A.[Instance] = EF.[Instance]
					AND A.[ID] = AEF.[IdAbility]
			INNER JOIN [dbo].[NLU] N (NOLOCK) 
				ON N.[VCC] = EF.[Instance] 
					AND N.[Name] = SUBSTRING(A.[Query], 0, CHARINDEX('|', A.[Query]))
			WHERE EF.[Instance] = @Instance
				-- EF.[templateId] = 'qty-tmpl-c8d4ba51-be20-4364-a677-76367bd0c694'
				AND A.[QueryName] IS NOT NULL
				AND A.[Query] LIKE '%|%'
	)

	SELECT
		E.ID																	'id',
		E.Description															'description',
		E.EnvironmentType														'environmentType',
		E.[ExecutionType]														'executionType',
		E.Type																	'type',
		E.MinQualificationAcceptable											'minQualificationAcceptable',
		STUFF((
			SELECT ',' + CTN_INNER.[Culture]
				FROM CTE_FORMAT_NLUS CTN_INNER (NOLOCK)
				WHERE CTN_INNER.[TemplateId] = e.[templateId]
		FOR XML PATH(''), TYPE).value('.', 'NVARCHAR(MAX)'), 1, 1, '')			'modelLanguages',
		E.CreatedDate															'createdDate',
		E.CreatedByUserId														'createdByUserId',
		E.LastModifiedDate														'lastModifiedDate',
		E.LastModifiedByUserId													'lastModifiedByUserId',
		E.[Enabled]																'enabled',
		E.[templateId]															'templateId'
	FROM [dbo].[EvaluationFormat] E (NOLOCK)
	WHERE E.Instance = @Instance
		AND ISNULL(E.ID, '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(E.Description, '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' COLLATE Latin1_General_CI_AI ELSE '%' END
		AND ISNULL(E.[ExecutionType], '') = CASE WHEN @SearchCriteria = 'executionType' THEN @SearchExpression ELSE E.[ExecutionType] END
		AND (ISNULL(@Disabled, 0) = 1 OR (ISNULL(@Disabled, 0) = 0 AND ISNULL(@Disabled, 0) <> E.[Enabled]))

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationFormatsFromHistoricEvaluationPlans] (
	@VCC VARCHAR(50)
	)
AS
BEGIN
	SET NOCOUNT ON

 	SELECT DISTINCT HEF.[ID]			'id',
					HEF.[Description]	'description' 
		FROM [dbo].[EvaluationResult] ER (NOLOCK)
		INNER JOIN [dbo].[HistoricEvaluationFormat] HEF (NOLOCK)
			ON HEF.[EvaluationPlanId] = ER.[IdEvaluationPlan]
				AND HEF.[VCC] = ER.[Instance] 
				AND HEF.[ID] = ER.[IdEvaluationFormat]
		INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			ON AEP.[Instance] = ER.[Instance]
				AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]
		WHERE HEF.[VCC] = @VCC 
			AND AEP.[Completed] = 1
		ORDER BY HEF.[Description] ASC

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationPlan] (
    @Instance Varchar(50) = NULL, @ID Varchar(50) = NULL
)
AS
BEGIN
SET NOCOUNT ON

	SELECT
		EP.[ID]												'id',
		EP.[Modality]										'modality',
		EP.[DateFin]										'dateFin',
		EP.[IdEvaluationFormat]								'idEvaluationFormat',
		EP.[Description]									'description',
		EP.[Enabled]										'enabled',
		EP.[CreatedDate]									'createdDate',
		EP.[CreatedByUserId]								'createdByUserId',
		EP.[LastModifiedDate]								'lastModifiedDate',
		EP.[LastModifiedByUserId]							'lastModifiedByUserId',
		EP.[Status]											'status',
		EP.[StatusDescription]								'statusDescription',
		EP.[Evaluation_Mode]								'evaluation_mode',
		EP.[ProcessId]										'processId',
		EP.[EvaluationStarted]								'evaluationStarted',
		EP.[FilterByContent]								'filterByContent',
		EP.[FilterByContentPercentage]						'filterByContentPercentage',
		EP.[FilterByContentQuery]							'filterByContentQuery',
		EP.[FilterByContentQueryName]						'filterByContentQueryName',
		EP.[ProcessAndEvaluate]								'processAndEvaluate',
		EP.[TranscriptionLanguage]							'transcriptionLanguage',
		P.[DictionaryId]									'dictionaryId',
		IIF (
			P.[DictionaryId] IS NULL, 
			CONVERT(BIT, 0), 
			CONVERT(BIT, 1)
		)													'useDictionary',
		EP.[EnvironmentId]									'environmentId',
		E.[Type]											'environmentType'
	FROM [EvaluationPlan] EP (NOLOCK)
	LEFT JOIN [Environment] E (NOLOCK)
		ON E.[VCC] = EP.[Instance]
			AND E.[Name] = EP.[EnvironmentId]
	LEFT JOIN [dbo].[Process] P (NOLOCK)
		ON P.[VCC] = EP.[Instance]
			AND P.[Name] = EP.ProcessId
 	WHERE [Instance] = @Instance
		AND [ID] = @ID

SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationPlanAbilities] (
	@VCC VARCHAR(50) = NULL, 
	@EvaluationPlanId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT DISTINCT
	        HA.ID								'id',
			HA.Title							'title',
			HA.Detail							'detail',
			HA.[Type]							'type',
			HA.[CalificationType]				'calificationType',
			HA.[AnalysisType]					'analysisType',
			HA.[Binary]							'binary',
			HA.[NotApply]						'acceptNotApply',
			HA.[IsCritical]						'isCritical',
			HA.[AcceptComments]					'acceptComments',
			HA.[Distribute]						'distributes',
			HA.[Enabled]						'enabled',
			HA.[Query]							'query',
			HA.[Queryname]						'queryName',
			HA.[IsNegative]						'isNegative', 
			N.[Culture]							'culture'
		FROM [dbo].[HistoricAbility] HA (NOLOCK)
		LEFT JOIN [dbo].[NLU] N (NOLOCK)
			ON N.[VCC] = HA.[VCC]
				AND N.[Name] = SUBSTRING(HA.[Query], 0, CHARINDEX('|', HA.[Query]))
		WHERE HA.[VCC] = @VCC
			AND HA.[EvaluationPlanId] = @EvaluationPlanId
		ORDER BY HA.[Title]
		
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlanAbilitiesQueries] (
	@VCC              VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL
) 
AS
BEGIN
	SET NOCOUNT ON
    
    SELECT DISTINCT HAQ.[ID]											'id',
					HAQ.[AbilityId]										'abilityId',
					HAQ.[QueryId]										'queryId',
					HAQ.[QueryContent]									'queryContent',
					HAQ.[AcceptancePercentage]							'acceptancePercentage',
					ISNULL(HAQ.[AcceptancePercentage], 0) * 100			'acceptanceInteger',
					HAQ.[Order]										    'order' 
		FROM [dbo].[HistoricAbilityQuery] HAQ (NOLOCK) 
		WHERE HAQ.[VCC] = @VCC
			AND HAQ.[EvaluationPlanId] = @EvaluationPlanId
		ORDER BY HAQ.[Order] ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlanAbilitiesQueriesEntities] (
	@VCC              VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL
) 
AS
BEGIN
	SET NOCOUNT ON

	SELECT DISTINCT HAQE.[AbilityQueryId]								'abilityQueryId',
					HAQE.[EntityId]										'entityId',
					HAQE.[EntityName]									'entityName',
					HAQE.[AcceptancePercentage]						    'acceptancePercentage',
					ISNULL(HAQE.[AcceptancePercentage], 0) * 100		'acceptanceInteger',
					HAQE.[Order]										'order'
		FROM [dbo].[HistoricAbilityQueryEntity] HAQE (NOLOCK) 
		WHERE HAQE.[VCC] = @VCC
			AND HAQE.[EvaluationPlanId] = @EvaluationPlanId
		ORDER BY HAQE.[Order] ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


CREATE PROCEDURE [dbo].[GetEvaluationPlanAbilityEvaluationFormats] (
	@VCC				VARCHAR(50) = NULL, 
	@EvaluationPlanId   VARCHAR(50) = NULL 
) AS
BEGIN
	SET NOCOUNT ON

	-- 
	SELECT DISTINCT HAEF.[ID]						'id',
					HAEF.[IdEvaluationFormat]		'idEvaluationFormat',
					HAEF.[IdAbility]				'idAbility',
					HA.[Title]						'titleAbility',
					HA.[Detail]						'detailAbility',
					HA.[Type]						'abilityType',
					HA.[AnalysisType]				'abilityAnalysisType',
					HA.[CalificationType]			'abilityCalificationType',
					HAEF.[IdGroup]					'idGroup',
					HG.[Description]                'descriptionGroup',
					HAEF.[IdSubGroup]				'idSubGroup',
					HSG.[Description]               'descriptionSubGroup',
					HAEF.[Penalizes]				'penalizes',
					HAEF.[Score]					'score',
					HAEF.[Order]					'order',
					HA.[NotApply]					'acceptNotApplies',
					HA.[Distribute]					'distribute',
					HA.[AcceptComments]				'acceptComments',
					HA.[Detail]						'descriptionAbility',
					HAEF.[Conditional]				'conditional',
					HA.[Query]						'abilityQuery',
					HA.QueryName					'query',
					HA.[IsNegative]                 'isNegative'
		FROM [dbo].[HistoricAbilityEvaluationFormat] HAEF (NOLOCK)
		INNER JOIN [dbo].[HistoricGroup] HG (NOLOCK)
			ON HG.[VCC] = HAEF.[VCC]
				AND HG.[EvaluationPlanId] = HAEF.[EvaluationPlanId]
				AND HG.[ID] = HAEF.[IdGroup]
		INNER JOIN [dbo].[HistoricSubGroup] HSG (NOLOCK)
			ON HSG.[VCC] = HAEF.[VCC]
				AND HSG.[EvaluationPlanId] = HAEF.[EvaluationPlanId]
				AND HSG.[ID] = HAEF.[IdSubGroup]
		INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
			ON HA.[VCC] = HAEF.[VCC]
				AND HA.[EvaluationPlanId] = HAEF.[EvaluationPlanId]
				AND HA.[ID] = HAEF.[IdAbility]
		WHERE HAEF.[VCC] = @VCC
			AND HAEF.[EvaluationPlanId]= @EvaluationPlanId
		ORDER BY [Order] ASC


	
	-- CONSULTAMOS LOS SCORES RELACIONADOS AL FORMATO
	SELECT HS.[Id]									'id',
			HS.[IdAbility]							'abilityId',
			HS.[Description]						'description',
			HS.[Value]								'value'
		FROM [dbo].[HistoricScore] HS (NOLOCK)
		WHERE HS.[VCC] = @VCC
			AND HS.[EvaluationPlanId] = @EvaluationPlanId
		ORDER BY HS.[Value] DESC



	-- CONSULTAMOS LAS TIPIFICACIONES DE LAS HABILIDADES RELACIONADAS AL FORMATO
	SELECT CAST(HEFAST.[ScoreId] AS INT)			'scoreId',
			HEFAST.[ID]								'id',
			HEFAST.[Description]					'description',
			HEFAST.[Detail]							'detail',
			HEFAST.[Order]							'order'
		FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipification] HEFAST (NOLOCK)
		WHERE HEFAST.[VCC] = @VCC
			AND HEFAST.[EvaluationPlanId] = @EvaluationPlanId
		ORDER BY HEFAST.[Order] ASC



	-- CONSULTAMOS LAS QUERY DE TIPIFICACION DE LAS HABILIDADES RELACIONADAS AL FORMATO
	SELECT CAST(HEFASTQ.[ScoreId] AS INT)			'scoreId',
			HEFASTQ.[TipificationId]				'tipificationId',
			HEFASTQ.[QueryId]						'queryId',
			HEFASTQ.[QueryContent]					'queryContent',
			HEFASTQ.[AcceptancePercentage]			'acceptancePercentage',
			HEFASTQ.[AcceptancePercentage] * 100	'acceptanceInteger'
		FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipificationQuery] HEFASTQ (NOLOCK)
		WHERE HEFASTQ.[VCC] = @VCC
			AND HEFASTQ.[EvaluationPlanId] = @EvaluationPlanId
			


	-- CONSULTAMOS LOS ITEMS DE TIPIFICACIONES DE LAS HABILIDADES RELACIONADAS AL FORMATO
	SELECT CAST(HEFASTI.[ScoreId] AS INT)			'scoreId',
			HEFASTI.[TipificationId]				'tipificationId',
			HEFASTI.[ID]							'id',
			HEFASTI.[SecondLevelItemId]				'secondLevelItemId',
			HEFASTI.[Description]					'description',
			HEFASTI.[Order]							'order'
		FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipificationItem] HEFASTI (NOLOCK)
		WHERE HEFASTI.[VCC] = @VCC
			AND HEFASTI.[EvaluationPlanId] = @EvaluationPlanId
		ORDER BY HEFASTI.[Order] ASC
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationPlanAbilityGenericQueryTask] (
    @VCC Varchar(50) = NULL, 
	@ID Varchar(50) = NULL
)
AS
BEGIN
SET NOCOUNT ON
	/*
	SELECT DISTINCT
			C.Instance			'vcc'
			,C.QueryName		'name'
		FROM [EvaluationPlan] A (NOLOCK)
		INNER JOIN [AbilityEvaluationFormat] B (NOLOCK)  
			ON A.Instance = B.Instance 
				AND A.IdEvaluationFormat = B.IdEvaluationFormat
		INNER JOIN [Ability] C (NOLOCK)  
			ON A.Instance = C.Instance 
				AND B.IdAbility = C.Id 
				AND C.QueryName IS NOT NULL
		INNER JOIN [dbo].[GenericQuery] D (NOLOCK) 
			ON A.Instance = D.vcc 
			AND D.[Name] = C.[QueryName]
		WHERE A.[Instance] = @VCC
			AND A.[ID] = @ID
	*/

	-- ESTE SP RETORNA LAS GENERICQUERY ASOCIADAS A UN PLAN POR MEDIO DEL FORMATO Y SUS HABILIDADES
	SELECT DISTINCT
			A.[Instance]			'vcc'
			,AQ.[QueryId]			'name'
		FROM [dbo].[EvaluationPlan] EP (NOLOCK)
		INNER JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)  
			ON EP.[Instance] = AEF.[Instance] 
				AND EP.[IdEvaluationFormat] = AEF.IdEvaluationFormat
		INNER JOIN [dbo].[Ability] A (NOLOCK)  
			ON A.[Instance] = EP.[Instance]
				AND AEF.[IdAbility] = A.[Id] 
				AND (
					A.[Type] = 'MANUAL_AUTOMATIC'
					OR
					A.[Type] = 'MIXED' -- PROXIMA IMPLEMENTACION
				)
				--AND A.[AnalysisType] = 'SEMANTIC'
		INNER JOIN [dbo].[AbilityQuery] AQ (NOLOCK)
			ON AQ.[VCC] = EP.[Instance]
				AND AQ.[AbilityId] = A.[ID]
		INNER JOIN [dbo].[GenericQuery] GQ (NOLOCK) 
			ON EP.[Instance] = GQ.[vcc] 
			AND GQ.[Name] = AQ.[QueryId]
		WHERE EP.[Instance] = @VCC
			AND EP.[ID] = @ID

SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationPlanAbilityTask] (
    @VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
)
AS
BEGIN
SET NOCOUNT ON

	SELECT DISTINCT
		C.Instance			'vcc'
		,C.QueryName		'queryId'
	FROM [EvaluationPlan] A (NOLOCK)
	INNER JOIN [AbilityEvaluationFormat] B ON A.Instance = B.Instance AND A.IdEvaluationFormat = B.IdEvaluationFormat
	INNER JOIN [Ability] C ON A.Instance = C.Instance AND B.IdAbility = C.Id AND C.QueryName IS NOT NULL
	INNER JOIN [dbo].[GenericQuery] D WITH (NOLOCK) ON A.Instance = D.vcc AND D.[Name] = C.[QueryName]
	WHERE 
	A.[Instance] = @VCC
	AND A.[ID] = @ID

SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlanAdvancedAIIntents] (
	@VCC              VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL
) 
AS
BEGIN
	SET NOCOUNT ON

	-- CONSULTAMOS LA DATA HISTORICA ASOCIADA AL PLAN
	SELECT DISTINCT HAAII.[AppId]										'appId',
					HAAII.[Id]											'id',
					HAAII.[Name]										'name',
					HAAII.[Prompt]										'prompt'
		FROM [dbo].[HistoricAdvancedAIIntent] HAAII (NOLOCK) 
		WHERE HAAII.[VCC] = @VCC
			AND HAAII.[EvaluationPlanId] = @EvaluationPlanId
			
	-- 
	SELECT DISTINCT HAAIE.[AppId]										'appId',
					HAAIE.[IntentId]									'intentId',
					HAAIE.[Id]											'id',
					HAAIE.[Text]										'text'
		FROM [dbo].[HistoricAdvancedAIExample] HAAIE (NOLOCK)
		WHERE HAAIE.[VCC] = @VCC
			AND HAAIE.[EvaluationPlanId] = @EvaluationPlanId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlanByInteractionId] (
	@VCC VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

 	SELECT TOP 1
		EP.[ID]									'id',
		EP.[Modality]							'modality',
		EP.[DateFin]							'dateFin',
		EP.[CreatedDate]						'createdDate',
		EP.[CreatedByUserId]					'createdByUserId',
		EP.[LastModifiedDate]					'lastModifiedDate',
		EP.[LastModifiedByUserId]				'lastModifiedByUserId',
		EP.[Description]						'description',
		EP.[IdEvaluationFormat]					'idEvaluationFormat',
		EP.[EnvironmentId]						'environmentId',
		EP.[Enabled]							'enabled',
		EP.[Status]								'status',
        EP.[Evaluation_Mode]					'evaluation_mode',
        EP.[processId]    						'processId',
        EP.FilterByContent						'filterByContent',
        EP.EvaluatePercentage					'evaluatePercentage',
        EP.ProcessAndEvaluate    				'processAndEvaluate',
		IEP.[RelativePath]						'relativePath',
		IEP.[FileName]							'fileName',	
		IEP.[RelativeFileName]					'relativeFileName'
	FROM EvaluationPlan EP (NOLOCK) 
	INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
		ON IEP.[IdEvaluationPlan] = EP.[ID]
	WHERE EP.[Instance] = @VCC
		AND IEP.[IdInteraction] = @InteractionId
	GROUP BY EP.[ID], EP.[Modality], EP.[DateFin], EP.[CreatedDate], EP.[CreatedByUserId],
			 EP.[LastModifiedDate], EP.[LastModifiedByUserId], EP.[Description], EP.[IdEvaluationFormat], EP.[EnvironmentId], 
			 EP.[Enabled], EP.[Status], EP.[Evaluation_Mode], EP.[processId], EP.[FilterByContent], 
			 EP.[EvaluatePercentage], EP.[ProcessAndEvaluate], IEP.[RelativePath], IEP.[FileName], IEP.[RelativeFileName]
	ORDER BY EP.[CreatedDate] DESC

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--[GetProcessByName] 'inconcert', 'test2'
CREATE PROCEDURE [dbo].[GetEvaluationPlanByName] (
	@VCC Varchar(50) = NULL, 
	@Description Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]						    'id',
		[Description]					'description',
		[Enabled]						'enabled'
	FROM [EvaluationPlan] (NOLOCK)
	WHERE 
	[Instance] = @VCC AND 
	[Description] = @Description
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
create PROCEDURE [dbo].[GetEvaluationPlanByProcessId] (
	@Instance           Varchar(50) = NULL,
	@ProcessId			Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

 	SELECT
		e.[ID]							'id',
		e.[Modality]					'modality',
		e.[DateFin]						'dateFin',
		e.CreatedDate					'createdDate',
		e.CreatedByUserId				'createdByUserId',
		e.LastModifiedDate				'lastModifiedDate',
		e.LastModifiedByUserId			'lastModifiedByUserId',
		e.Description					'description',
		e.IdEvaluationFormat            'idEvaluationFormat',
		e.Enabled						'enabled',
		e.Status						'status',
        e.Evaluation_Mode				'evaluation_mode',
        e.processId    				    'processId',
        e.FilterByContent				'filterByContent',
        e.EvaluatePercentage			'evaluatePercentage',
        e.ProcessAndEvaluate    		'processAndEvaluate'
	FROM
	    EvaluationPlan e (NOLOCK)
	WHERE
		e.Instance = @Instance
		AND e.[ProcessId] = @ProcessId
	GROUP BY e.[ID],e.[Modality],e.[DateFin],e.CreatedDate,e.CreatedByUserId,
			 e.LastModifiedDate,e.LastModifiedByUserId,e.Description,e.IdEvaluationFormat,e.Enabled, e.Status, e.Evaluation_Mode,e.processId,e.FilterByContent, e.EvaluatePercentage, e.ProcessAndEvaluate
 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlanCognitiveTask] (
    @VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
)
AS
BEGIN
SET NOCOUNT ON

	SELECT DISTINCT
			A.Instance			'vcc'
			,A.ProcessId		'processId'
			,A.[Id]				'evaluationPlanId'
			,B.IdInteraction	'id'
			,C.Section			'section'
			,C.Part				'part'
			,F.nluId			'nluId'
			,F.appId			'appId'
			,C.[FileName]		'fileName'
			,F.[cognitiveId]
	FROM [EvaluationPlan] A (NOLOCK)
	INNER JOIN [InteractionEvaluationPlan] B (NOLOCK) 
		ON A.Instance = B.Instance 
			AND A.Id = B.IdEvaluationPlan
	INNER JOIN [dbo].[Audio] C WITH (NOLOCK) 
		ON A.Instance = C.VCC 
			AND B.IdInteraction = C.ID 
	INNER JOIN [AbilityEvaluationFormat] D 
		ON A.Instance = D.Instance 
			AND A.IdEvaluationFormat = D.IdEvaluationFormat
	INNER JOIN [Ability] E 
		ON A.Instance = E.Instance 
			AND D.IdAbility = E.ID 
			AND E.QueryName IS NOT NULL
	INNER JOIN (SELECT DISTINCT
				A.VCC 'vcc'
				,A.[Name] + '|'+ B.[Name] 'cognitiveId'
				,A.[Name] 'nluId'
				,A.AppId 'appId'
				,B.[IntentId] 'intentId',B.Name
			FROM NLU A (NOLOCK)
			INNER JOIN Intent B (NOLOCK) 
				ON A.VCC = B.VCC
					AND B.AppId = A.AppId) F 
		ON A.Instance = F.VCC 
			--AND E.ID = F.cognitiveId
			AND E.Query = F.cognitiveId
			AND E.QueryName = F.intentId
	WHERE A.[Instance] = @VCC
		AND A.[ID] = @ID

SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlanGroups] (
	@VCC VARCHAR(50) = NULL, 
	@EvaluationPlanId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT HG.[ID]							'id',
			HG.[Description]				'description'
		FROM [HistoricGroup] HG (NOLOCK)
		WHERE HG.[VCC] = @VCC
			AND HG.[EvaluationPlanId] = @EvaluationPlanId
		ORDER BY HG.[Description] ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlanInteractionByTypeAndId] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionType VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- RECUPERAMOS EL PRIMER REGISTRO POR ID DE INTERACCION
	SELECT TOP 1 IEP.[Instance]											'vcc',
					IEP.[IdAgent]										'descriptionAgent',
					''													'descriptionAuditor',
					IEP.[IdCampaign]									'descriptionCampaign',
					EP.[Description]									'descriptionEvaluationPlan',
					IEP.[InteractionDateStart]							'dateIni',
					IEP.[InteractionDateEnd]							'dateFin',
					IEP.[IdInteraction]									'id',
					IEP.[IdAgent]										'idAgent',
					IEP.[AgentName]										'agentName',
					''													'idAuditorEvaluationPlan',
					IEP.[IdCampaign]									'idCampaign',
					IEP.[IdEvaluationPlan]								'idEvaluationPlan',
					IEP.[InteractionType]								'interactionType',
					IEP.[InteractionDuration]							'interactionTime',
					A.[DurationLinguisticTime]							'interactionValidTime',
					''													'origin',
					IEP.[InteractionMessages]							'totalMessages',
					CASE
						WHEN IEP.[InteractionType] = 'SMS'
							THEN ISNULL(IEP.[ContactId], IEP.[ContactName])
							ELSE ISNULL(A.[Number], IEP.[ContactId])
					END													'number',
					IEP.[NeedI6Update]									'needI6Update',
					IEP.[Ender]										    'ender',
					IEP.[ManagementResult]								'managementResult',
					IEP.[Ticket]                                        'ticket'
					/*
					IIF (
						E.[Type] = 'i6',
						ISNULL(
							A.[NeedI6Update], 
							CAST(1 AS BIT)	
						),
						CAST(0 AS BIT)
					)													'needI6Update'
					*/

		FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
		LEFT JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
			ON EP.[Instance] = IEP.[Instance]
				AND EP.[ID] = IEP.[IdEvaluationPlan]
		LEFT JOIN [dbo].[Environment] E (NOLOCK) 
			ON E.[VCC] = IEP.[Instance]
				AND E.[Name] = EP.[EnvironmentId]
		LEFT JOIN [dbo].[Audio] A (NOLOCK)
			ON A.[VCC] = IEP.[Instance]
				AND A.[ID] = IEP.[IdInteraction]
				AND A.[Section] = IEP.[InteractionSection]
				AND A.[Part] = IEP.[InteractionPart]
		WHERE IEP.[Instance] = @VCC
			AND IEP.[IdEvaluationPlan] = @EvaluationPlanId
			AND IEP.[InteractionType] = @InteractionType
			AND IEP.[IdInteraction] = @InteractionId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlanModelInteractionTask] (
    @VCC VARCHAR(50) = NULL, 
	@AnalysisType VARCHAR(50) = NULL,
	@ID VARCHAR(50) = NULL -- EVALUATION PLAN ID
)
AS
BEGIN
	SET NOCOUNT ON
	-- ESTE SP TIENE POR FINALIDAD RETORNAR LA LISTA DE INTERACCIONES CRUZADA CON LOS MODELOS QUE SE VAN A EVALUAR

	-- DECLARAMOS UNA PALABRA PARA ALMACENAR EL MOTOR COGNITIVO
	DECLARE @COGNITIVE_ENGINE VARCHAR(50) = NULL

	-- OBTENEMOS LAS ABILITY QUERY Y LAS INSERTAMOS A UNA TABLA TEMPORAL
	SELECT HAQ.* 
		INTO #HISTORIC_ABILITY_QUERY_TMP 
		FROM [dbo].[HistoricAbilityQuery] HAQ (NOLOCK) 
		INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
			ON HA.[VCC] = HAQ.[VCC]
				AND HA.[ID] = HAQ.[AbilityId]
		WHERE HAQ.[VCC] = @VCC
			AND HAQ.[EvaluationPlanId] = @ID
			AND HA.[AnalysisType] = @AnalysisType

	-- DECLARAMOS UNA TABLE PARA ALMACENAR LA DATA DE MODEL
	DECLARE @TBL_MODEL_DATA AS TABLE ([ModelName] VARCHAR(50), [AppId] VARCHAR(100), [IntentId] VARCHAR(50))
	
	-- AGREGAMOS LA DATA DE MODELO E INTENT SEGUN EL TIPO DE MODELO
	
	IF @AnalysisType = 'ADVANCED_AI'
	BEGIN
		INSERT @TBL_MODEL_DATA 
			SELECT AAI.[ID]
					,AAI.[Id]
					,AAII.[Id]					
				FROM #HISTORIC_ABILITY_QUERY_TMP HAQ_TMP (NOLOCK) 
				INNER JOIN [dbo].[AdvancedAIIntent] AAII (NOLOCK)
					ON AAII.[VCC] = HAQ_TMP.[VCC]
						AND AAII.[Id] = HAQ_TMP.[QueryId]
				INNER JOIN [dbo].[AdvancedAI] AAI (NOLOCK)
					ON AAI.[VCC] = HAQ_TMP.[VCC]
						AND AAI.[Id] = AAII.[AppId]
	END
	IF @AnalysisType = 'COGNITIVE'
	BEGIN
		INSERT @TBL_MODEL_DATA 
			SELECT N.[Name]
					,N.[AppId]
					,I.[IntentId]
				FROM #HISTORIC_ABILITY_QUERY_TMP HAQ_TMP (NOLOCK) 
				INNER JOIN [dbo].[Intent] I (NOLOCK)
					ON I.[VCC] = HAQ_TMP.[VCC]
						AND I.[IntentId] = HAQ_TMP.[QueryId]
				INNER JOIN [dbo].[NLU] N (NOLOCK)
					ON N.[VCC] = HAQ_TMP.[VCC]
						AND N.[AppId] = I.[AppId]

		-- SETTEAMOS EL MOTOR COGNITIVO
		SELECT @COGNITIVE_ENGINE = [CognitiveEngine] FROM [dbo].[VCC] (NOLOCK) WHERE [VCC] = @VCC
	END
	ELSE IF @AnalysisType = 'GENERATIVE'
	BEGIN
		INSERT @TBL_MODEL_DATA 
			SELECT G.[Name]
					,G.[AppId]
					,GI.[Id]
				FROM #HISTORIC_ABILITY_QUERY_TMP HAQ_TMP (NOLOCK) 
				INNER JOIN [dbo].[GenerativeIntent] GI (NOLOCK)
					ON GI.[VCC] = HAQ_TMP.[VCC]
						AND GI.[Id] = HAQ_TMP.[QueryId]
				INNER JOIN [dbo].[Generative] G (NOLOCK)
					ON G.[VCC] = HAQ_TMP.[VCC]
						AND G.[AppId] = GI.[AppId]
	END
	-- SELECT * FROM @TBL_MODEL_DATA
	
	-- CREAMOS LA TABLA TEMPORAL FINAL PARA SER CRUZADA CONTRA LAS INTERACCIONES
	SELECT DISTINCT @VCC										'vcc'
					-- ,N.[Name] + '|' + I.[Name]					'cognitiveId'
					,MD.[ModelName]								'modelId' --'nluId'
					,MD.[AppId]									'appId'
					,MD.[IntentId]								'intentId'
					,'ABILITY'									'mainTarget'
					,STUFF(
						(
							SELECT DISTINCT ',' + INNER_HAQ_TMP.[AbilityId]
								FROM #HISTORIC_ABILITY_QUERY_TMP INNER_HAQ_TMP (NOLOCK)
								-- WHERE INNER_HAQ.[QueryContent] = N.[Name] + '|' + I.[Name]
							FOR XML PATH(''), TYPE
						).value('.', 'NVARCHAR(MAX)'), 
						1, 
						1, 
						''
					)											'abilities'
					,CONVERT(BIT, 1) 							'needRequest'
		INTO #EVALUATION_PLAN_MODELS 
		FROM @TBL_MODEL_DATA MD 

	-- ESTE SCRIPT RECUPERA LAS INTERACCIONES TIPO CALL QUE ENTRARAN A LA EVALUACION COGNITIVA
	SELECT DISTINCT
			EP.Instance									'vcc'
			,EP.ProcessId								'processId'
			,'EVALUATION_PLAN'							'batchTypeName'
			,@AnalysisType								'analysisType'
			,AU.Campaign								'campaign'
			,IEP.[InteractionType]						'interactionType'
			,EP.[Id]									'evaluationPlanId'
			,IEP.IdInteraction							'id'
			,AU.Section									'section'
			,AU.Part									'part'
			,NULL										'folder'
			,EPM.[modelId]								'modelId' --nluId
			,EPM.appId									'appId'
			,EPM.[mainTarget]							'mainTarget'
			,AU.[FileName]								'fileName'
			--,EPM.[cognitiveId]						-- CREA MAS REGISTROS DE LOS NECESARIOS AL INICIAR EL TASK
			,@COGNITIVE_ENGINE							'cognitiveEngine'
			,EPM.[abilities]							'abilities'
			,EPM.[needRequest]							'needRequest'
			,AU.[TotalPhrases]							'totalPhrases'
		FROM [EvaluationPlan] EP (NOLOCK)
		INNER JOIN [InteractionEvaluationPlan] IEP (NOLOCK) 
			ON IEP.Instance = EP.Instance 
				AND IEP.IdEvaluationPlan = EP.Id
		INNER JOIN [dbo].[Audio] AU (NOLOCK) 
			ON AU.VCC = EP.Instance
				AND AU.ID = IEP.IdInteraction 
		INNER JOIN #EVALUATION_PLAN_MODELS EPM 
			ON EPM.[vcc] = EP.[Instance]
		WHERE EP.[Instance] = @VCC
			AND EP.[ID] = @ID
			AND AU.[IsTranscribed] = 1
	
	-- ESTE SCRIPT RECUPERA LAS INTERACCIONES TIPO TEXT QUE ENTRARAN A LA EVALUACION COGNITIVA
	SELECT DISTINCT
			EP.Instance									'vcc'
			,EP.ProcessId								'processId'
			,'EVALUATION_PLAN'							'batchTypeName'
			,@AnalysisType								'analysisType'
			,AI.[Campaign]								'campaign'
			,IEP.[InteractionType]						'interactionType'
			,EP.[Id]									'evaluationPlanId'
			,IEP.IdInteraction							'id'
			,NULL										'section'
			,NULL										'part'
			,AI.[EndDate]								'endDate'
			,AI.[Folder]								'folder'
			,AI.[DispositionCode]						'dispositionCode'
			,EPM.[modelId]								'modelId' --nluId
			,EPM.appId									'appId'
			,EPM.[mainTarget]							'mainTarget'
			,AI.[FileName]								'fileName'
			--,F.[cognitiveId]							-- CREA MAS REGISTROS DE LOS NECESARIOS AL INICIAR EL TASK
			,@COGNITIVE_ENGINE							'cognitiveEngine'
			,EPM.[abilities]							'abilities'
			,EPM.[needRequest]							'needRequest'
			,AI.[TotalPhrases]							'totalPhrases'
		FROM [EvaluationPlan] EP (NOLOCK)
		INNER JOIN [InteractionEvaluationPlan] IEP (NOLOCK) 
			ON EP.Instance = IEP.Instance 
				AND EP.Id = IEP.IdEvaluationPlan
		INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK) 
			ON EP.Instance = AI.VCC 
				AND IEP.IdInteraction = AI.ID
				AND IEP.FormattedEndDate = AI.Folder
		INNER JOIN #EVALUATION_PLAN_MODELS EPM 
			ON EPM.[vcc] = EP.[Instance]
		WHERE EP.[Instance] = @VCC
			AND EP.[ID] = @ID
			AND AI.[IsTranscribed] = 1

	-- ELIMINAMOS LA TABLA TEMPORAL
	DROP TABLE #HISTORIC_ABILITY_QUERY_TMP
	DROP TABLE #EVALUATION_PLAN_MODELS

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE  PROCEDURE [dbo].[GetEvaluationPlans] (
@Instance           Varchar(50) = NULL,
@SearchCriteria     Varchar(50) = NULL,
@SearchExpression   Varchar(50) = NULL,
@Disabled			bit = NULL
)
AS
BEGIN
SET NOCOUNT ON

	-- 
	PRINT CONCAT('GETTING [dbo].[AuditorEvaluationPlan] RECORDS', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	;WITH CTE_AEP_DATA AS (
		SELECT AEP.[IdEvaluationPlan]																				'evaluationPlanId',
				COUNT(DISTINCT AEP.[IdInteraction])																	'totalEvaluations',
				--COUNT(DISTINCT CASE AEP.[Completed] WHEN 1 THEN AEP.[IdInteraction] ELSE NULL END)					'completedEvaluations',
				COUNT(DISTINCT IIF(AEP.[Completed] = 1, AEP.[IdInteraction], NULL))									'completedEvaluations',
				COUNT(DISTINCT IIF(AEP.[Status] = 0, AEP.[IdInteraction], NULL))									'pendingEvaluations'
				/*
				(
					(
						CAST(
							--COUNT(DISTINCT CASE AEP.[Completed] WHEN 1 THEN AEP.[IdInteraction] ELSE NULL END)
							COUNT(DISTINCT IIF(AEP.[Completed] = 1, AEP.[IdInteraction], NULL))
							+ 
							--COUNT(DISTINCT CASE AEP.[Status] WHEN 0 THEN AEP.[IdInteraction] ELSE NULL END)
							COUNT(DISTINCT IIF(AEP.[Status] = 0, AEP.[IdInteraction], NULL))
						 AS NUMERIC(18,2)) 
					)
					/
					--CASE COUNT(DISTINCT AEP.[IdInteraction]) WHEN 0 THEN 1 ELSE COUNT(DISTINCT AEP.[IdInteraction]) END 
					IIF (COUNT(DISTINCT AEP.[IdInteraction]) = 0, 1, COUNT(DISTINCT AEP.[IdInteraction]))
				) * 100																								'completedPercentage'
				*/
			FROM [dbo].[EvaluationPlan] EP (NOLOCK)
			INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				ON AEP.[Instance] = EP.[Instance]
					AND AEP.[IdEvaluationPlan] = EP.[ID]
			WHERE EP.[Instance] = @Instance
				AND EP.[CreatedDate] >= DATEADD(YEAR, -1, GETUTCDATE())
			GROUP BY AEP.[IdEvaluationPlan]
	),
	CTE_AEP_SUMMARY AS (
		SELECT [evaluationPlanId],
				 [totalEvaluations],
				 [completedEvaluations],
				 (
					(
						CAST(
							[totalEvaluations]
							+ 
							[pendingEvaluations]
						 AS NUMERIC(18,2)) 
					)
					/
					IIF([totalEvaluations] = 0, 1, [totalEvaluations])
				) * 100																								'completedPercentage'
			FROM CTE_AEP_DATA
	)
		
	-- 
    SELECT
			e.[ID]									'id',
			e.[Modality]							'modality',
			e.[DateFin]								'dateFin',
			e.CreatedDate							'createdDate',
			e.CreatedByUserId						'createdByUserId',
			e.LastModifiedDate						'lastModifiedDate',
			e.LastModifiedByUserId					'lastModifiedByUserId',
			e.Description							'description',
			e.IdEvaluationFormat					'idEvaluationFormat',
			e.Enabled								'enabled',
			e.Status								'status',
			e.Evaluation_Mode						'evaluation_mode',
			e.processId    							'processId',
			ef.templateId    						'templateId',
			CTE_AEP_S.[totalEvaluations]			'totalEvaluations',
			CTE_AEP_S.[completedEvaluations]		'completedEvaluations',
			CTE_AEP_S.[completedPercentage]			'completedPercentage',

			/*
			count(a.IdInteraction)			'totalEvaluations',
			--ISNULL(sum(cast(a.Completed as int)), 0)   'completedEvaluations',
			--ISNULL(ROUND(sum(cast(a.Completed as numeric(5,2)))*100/count(a.IdInteraction),2), 0) 'completedPercentage',
			COUNT(DISTINCT CASE a.[Completed] WHEN 1 THEN a.[IdInteraction] ELSE NULL END)					'completedEvaluations',
			(
				(
					CAST(
						COUNT(DISTINCT CASE a.[Completed] WHEN 1 THEN a.[IdInteraction] ELSE NULL END)
						+ 
						COUNT(DISTINCT CASE a.[Status] WHEN 0 THEN a.[IdInteraction] ELSE NULL END)
					 AS NUMERIC(18,2)) 
				)
				/
				CASE COUNT(DISTINCT a.[IdInteraction]) WHEN 0 THEN 1 ELSE COUNT(DISTINCT a.[IdInteraction]) END 
			) * 100																							'completedPercentage',
			*/
			e.[Percentage]							'percentage',
			e.[StatusDescription]					'statusDescription',
			e.[BatchStatus]							'batchStatus'
		FROM EvaluationPlan e (NOLOCK)
		/*
		LEFT JOIN AuditorEvaluationPlan a (NOLOCK) 
			ON a.[Instance] = e.[Instance]
				AND a.[IdEvaluationPlan] = e.[ID]
		*/
		INNER JOIN CTE_AEP_SUMMARY CTE_AEP_S
			ON CTE_AEP_S.[evaluationPlanId] = e.[ID] 
		INNER JOIN [dbo].[EvaluationFormat] ef (NOLOCK)
			ON ef.[Instance] = e.[Instance]
				AND ef.[ID] = e.[IdEvaluationFormat]
		WHERE e.Instance = @Instance
			AND ISNULL(e.Description, '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND ISNULL(e.Modality, '') LIKE CASE WHEN @SearchCriteria = 'modality' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND ISNULL(@Disabled, 0) <> e.Enabled

		/*
		GROUP BY e.[ID],
					e.[Modality],
					e.[DateFin],
					e.CreatedDate,
					e.CreatedByUserId,
					e.LastModifiedDate,
					e.LastModifiedByUserId,
					e.Description,
					e.IdEvaluationFormat,
					e.Enabled, 
					e.Status, 
					e.Evaluation_Mode,
					e.processId,
					ef.templateId, 
					e.Percentage, 
					e.StatusDescription, 
					e.[BatchStatus]	
		*/

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlansAgent] (
	@Instance Varchar(50) = NULL,
	@IdAgent Varchar(50) = NULL,
	@Campaigns VARCHAR(MAX) = NULL,
	@EndDate Varchar(50) = NULL,
	@EvaluationFormats Varchar(MAX) = NULL,
	@Modalities Varchar(MAX) = NULL,
	@Completed Bit = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = '';
	DECLARE @SQL_FROM VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA VARCHAR(MAX) = '';
	DECLARE @SQL_SELECT_2 VARCHAR(MAX) = '';
	DECLARE @SQL_FROM_2 VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER_2 VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA_2 VARCHAR(MAX) = '';

	SET @SQL_SELECT_1 += '
		SELECT EP.[ID]																	''id'',
				EP.[Modality]															''modality'',
				EP.[DateFin]															''dateFin'',
				EP.[CreatedDate]														''createdDate'',
				EP.[IdEvaluationFormat]													''idEvaluationFormat'',
				EP.[Description]														''description'',
				COUNT(CASE AEP.Completed WHEN 0 THEN 1 ELSE NULL END)					''pendingInteractionCount'',
				COUNT(AEP.Completed)													''completedCount'',
				SUM(AEP.InteractionDuration)											''interactionTimeSum'',
				SUM(CASE AEP.Completed WHEN 0 THEN AEP.InteractionDuration ELSE 0 END)	''pendingInteractionTimeSum'' ';

	SET @SQL_FROM += '
			FROM [EvaluationPlan] EP (NOLOCK)
			LEFT JOIN [AuditorEvaluationPlan] AEP (NOLOCK)
				ON (AEP.[Instance] = ''' + @Instance + '''
					AND AEP.[IdAgent] = ''' + @IdAgent + ''') ';

	SET @SQL_FILTER += '
			WHERE EP.[Instance] = ''' + @Instance + '''
				AND AEP.[IdEvaluationPlan] = EP.[ID]
				AND EP.[DateFin] <= ''' + @EndDate + ''' 
				AND EP.[Enabled] = 1 ';

	IF (@Campaigns IS NOT NULL AND @Campaigns <> '')
	BEGIN
		SET @SQL_FILTER += '
				AND EP.[IdEvaluationFormat] IN (
					SELECT CEF.[IdEvaluationFormat] FROM CampaignEvaluationFormat CEF (NOLOCK)
						WHERE CEF.[Instance] = ''' + @Instance + '''
							AND CEF.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','',''))) '
	END;

	IF (@EvaluationFormats IS NOT NULL AND @EvaluationFormats <> '') SET @SQL_FILTER += 'AND EP.[IdEvaluationFormat] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationFormats + ''','','')) ';
	IF (@Modalities IS NOT NULL AND @Modalities <> '') SET @SQL_FILTER += 'AND EP.[Modality] IN (SELECT Value FROM [dbo].[split](''' + @Modalities + ''','','')) ';
	
	IF (@Completed = 1)
	BEGIN
		SET @SQL_FILTER += 'AND AEP.[Completed] = 1 ';
		SET @SQL_FILTER_2 = ' ';	
	END
	ELSE 
	BEGIN
		SET @SQL_FILTER += 'AND AEP.[Completed] = 0 ';
		SET @SQL_FILTER_2 = 'WHERE X.[pendingInteractionCount] > 0 ';
	END;
	

	SET @SQL_EXTRA += '
			GROUP BY EP.[ID],
						EP.[Modality],
						EP.[DateFin],
						EP.[CreatedDate],
						EP.[IdEvaluationFormat],
						EP.[Description]';

	SET @SQL_SELECT_1 += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA;

	SET @SQL_SELECT_2 = 'SELECT * ';
	SET @SQL_FROM_2 = 'FROM (' + @SQL_SELECT_1 + ') AS X ';
	-- @SQL_FILTER_2 YA FUE DEFINIDO SEGUN SE BUSCAN LAS YA EVALUADAS O NO EVALUADAS
	SET @SQL_EXTRA_2 = 'ORDER BY X.[Description] ASC;';

	SET @SQL_SELECT_2 += @SQL_FROM_2 + @SQL_FILTER_2 + @SQL_EXTRA_2;

	--select @SQL_SELECT_2;
	EXEC (@SQL_SELECT_2);
	SET NOCOUNT OFF
	
END;
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationPlansAuditor] (
	@Instance VARCHAR(50) = NULL,
	@UserId VARCHAR(50) = NULL,
	@Campaigns VARCHAR(MAX) = NULL,
	@EndDate VARCHAR(50) = NULL,
	@EvaluationPlans VARCHAR(MAX) = NULL,
	@EvaluationFormats VARCHAR(MAX) = NULL,
	@Modalities VARCHAR(MAX) = NULL,
	-- @EvaluationTypes VARCHAR(50) = NULL,
	@Completed BIT = NULL,
	@UserFunction VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	DECLARE @USER_ROLE VARCHAR(50) = ''
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''
	DECLARE @SQL_SELECT_2 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM_2 VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER_2 VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA_2 VARCHAR(MAX) = ''

	-- ARMAMOS EL SELECT DE LA QURY
	SET @SQL_SELECT_1 += '
		SELECT EP.[ID]																								''id'', 
				EP.[Description]																					''description'',
				EP.[Modality]																						''modality'',
				EP.[DateFin]																						''dateFin'',      
				EP.[CreatedDate]																					''createdDate'',      
				EP.[IdEvaluationFormat]																				''idEvaluationFormat'',
				COUNT(DISTINCT AEP.[IdInteraction])																	''totalInteraction'',
				COUNT(DISTINCT CASE AEP.[Completed] WHEN 1 THEN AEP.[IdInteraction] ELSE NULL END)					''completedInteraction'',      
				COUNT(DISTINCT CASE AEP.[Status] WHEN 0 THEN AEP.[IdInteraction] ELSE NULL END)						''discardedInteraction'',
				-- COUNT(DISTINCT CASE AEP.[Completed] WHEN 0 THEN AEP.[IdInteraction] ELSE NULL END)				''pendingInteraction'',
				COUNT(DISTINCT
					CASE 
					WHEN AEP.[Completed] = 0 AND AEP.[Status] = 1 THEN AEP.[IdInteraction] 
					ELSE NULL 
					END)																							''pendingInteraction'',
				SUM(DISTINCT CASE AEP.[Completed] WHEN 0 THEN AEP.[InteractionDuration] ELSE 0 END)					''pendingInteractionTime'',
				COUNT(AEP.[IdInteraction])																			''totalEvaluation'',    
				COUNT(CASE AEP.[Completed] WHEN 1 THEN 1 ELSE NULL END)												''completedEvaluation'',      
				COUNT(CASE AEP.[Status] WHEN 0 THEN 1 ELSE NULL END)												''discardedEvaluation'',    
				-- COUNT(CASE AEP.[Completed] WHEN 0 THEN 1 ELSE NULL END)											''pendingEvaluation'',
				COUNT(
					CASE 
					WHEN AEP.[Completed] = 0 AND AEP.[Status] = 1 THEN 1 
					ELSE NULL 
					END)																							''pendingEvaluation'',
				SUM(CASE AEP.[Completed] WHEN 0 THEN AEP.[InteractionDuration] ELSE 0 END)							''pendingEvaluationTime'' '

	SET @SQL_FROM += '
			FROM [EvaluationPlan] EP (NOLOCK)
			INNER JOIN [AuditorEvaluationPlan] AEP (NOLOCK)
				ON (AEP.[Instance] = EP.[Instance] 
					AND AEP.[IdEvaluationPlan] = EP.[ID]'

	-- HACEMOS EL FILTRADO SEGUN EL ROL DEL USUARIO
	IF @UserFunction = 'auditor'
	BEGIN
		SET @SQL_FROM += '
					AND AEP.[IdAuditor] = ''' + @UserId + ''') '
	END
	ELSE IF (@UserFunction = 'admin')
	BEGIN
		SET @SQL_FROM += ')'
	END
	ELSE 
	BEGIN
		SET @SQL_FROM += '
					AND AEP.[IdAuditor] = ''' + @UserId + ''') '
	END 

	SET @SQL_FILTER += '
			WHERE EP.[Instance] = ''' + @Instance + '''
				AND EP.[DateFin] <= ''' + @EndDate + ''' 
				AND EP.[Enabled] = 1 
				AND (
					(
						EP.[Evaluation_Mode] = ''X'' 
						AND 
						(
							EP.[Status] = 7
							OR
							EP.[Status] = 8
						)
					)
					OR
					(EP.[Evaluation_Mode] = ''M'')
				) '

	IF (@Campaigns IS NOT NULL AND @Campaigns <> '')
	BEGIN
		SET @SQL_FILTER += '
				AND EP.[IdEvaluationFormat] IN (
					SELECT CEF.[IdEvaluationFormat] FROM CampaignEvaluationFormat CEF 
						WHERE CEF.[Instance] = ''' + @Instance + '''
							AND CEF.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','',''))) '
	END

	IF (@EvaluationPlans IS NOT NULL AND @EvaluationPlans <> '') SET @SQL_FILTER += 'AND EP.[ID] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationPlans + ''','','')) '
	IF (@EvaluationFormats IS NOT NULL AND @EvaluationFormats <> '') SET @SQL_FILTER += 'AND EP.[IdEvaluationFormat] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationFormats + ''','','')) '
	IF (@Modalities IS NOT NULL AND @Modalities <> '') SET @SQL_FILTER += 'AND EP.[Modality] IN (SELECT Value FROM [dbo].[split](''' + @Modalities + ''','','')) '
	-- IF @EvaluationTypes IS NOT NULL AND @EvaluationTypes <> '' SET @SQL_FILTER += 'AND EP.[Evaluation_Mode] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationTypes + ''','','')) '
	
	-- VARIABLE PARA FILTRAR SOLO LOS REGISTROS DE AUDITOREVALUATIONPLAN CON [COMPLETED] = 1
	IF (@Completed = 1)
	BEGIN
		SET @SQL_FILTER += 'AND AEP.[Completed] = 1 '
		SET @SQL_FILTER_2 = ' '
	END
	ELSE 
	BEGIN
		-- SI NO LLEGA UN VALOR PARA EVALUATION PLANS, BUSCAMOS SOLO LOS QUE TIENEN INTERACCIONES PENDIENTES
		IF (@EvaluationPlans IS NULL OR @EvaluationPlans = '')
		BEGIN
			--SET @SQL_FILTER += 'AND AEP.[Completed] = 0 '
			SET @SQL_FILTER_2 = 'WHERE X.[pendingInteraction] > 0 '
		END
	END
	

	SET @SQL_EXTRA += '
			GROUP BY EP.[ID],
						EP.[Modality],
						EP.[DateFin],
						EP.[CreatedDate],
						EP.[IdEvaluationFormat],
						EP.[Description]'

	SET @SQL_SELECT_1 += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA

	--SET @SQL_SELECT_1 = 'SELECT * FROM (' + @SQL_SELECT_1 + ') AS X WHERE X.[pendingInteractionCount] > 0 ORDER BY X.[Description] ASC; '
	SET @SQL_SELECT_2 = 'SELECT * '
	SET @SQL_FROM_2 = 'FROM (' + @SQL_SELECT_1 + ') AS X '
	-- @SQL_FILTER_2 YA FUE DEFINIDO SEGUN SE BUSCAN LAS YA EVALUADAS O NO EVALUADAS
	SET @SQL_EXTRA_2 = 'ORDER BY X.[Description] ASC'

	SET @SQL_SELECT_2 += @SQL_FROM_2 + @SQL_FILTER_2 + @SQL_EXTRA_2

	-- select @SQL_SELECT_2;
	EXEC (@SQL_SELECT_2)
	
	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationPlanScores] (
	@VCC VARCHAR(50) = NULL, 
	@EvaluationPlanId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- CONSULTAMOS LOS SCORES RELACIONADOS AL PLAN
	SELECT DISTINCT HS.[ID]								'id',
			HS.[IdAbility]						'idAbility',
			HS.[Description]					'description',
			HS.[Value]				            'value'
		FROM [HistoricScore] HS (NOLOCK)
		WHERE [VCC] = @VCC
			AND [EvaluationPlanId] = @EvaluationPlanId
		ORDER BY Value DESC



	-- CONSULTAMOS LAS TIPIFICACIONES DE LAS HABILIDADES RELACIONADAS AL PLAN
	SELECT DISTINCT EFAST.[ScoreId]							'scoreId',
			EFAST.[ID]								'id',
			EFAST.[Description]						'description',
			EFAST.[Detail]							'detail',
			EFAST.[Order]							'order'
		FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipification] EFAST (NOLOCK)
		WHERE EFAST.[VCC] = @VCC
			AND [EvaluationPlanId] = @EvaluationPlanId
		ORDER BY EFAST.[Order] ASC



	-- CONSULTAMOS LAS QUERY DE TIPIFICACION DE LAS HABILIDADES RELACIONADAS AL FORMATO
	SELECT DISTINCT CAST(HEFASTQ.[ScoreId] AS INT)			'scoreId',
			HEFASTQ.[TipificationId]				'tipificationId',
			HEFASTQ.[QueryId]						'queryId',
			HEFASTQ.[QueryContent]					'queryContent',
			HEFASTQ.[AcceptancePercentage]			'acceptancePercentage',
			HEFASTQ.[AcceptancePercentage] * 100	'acceptanceInteger'
		FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipificationQuery] HEFASTQ (NOLOCK)
		WHERE HEFASTQ.[VCC] = @VCC
			AND HEFASTQ.[EvaluationPlanId] = @EvaluationPlanId



	-- CONSULTAMOS LOS ITEMS DE TIPIFICACIONES DE LAS HABILIDADES RELACIONADAS AL PLAN
	SELECT DISTINCT EFASTI.[ScoreId]							'scoreId',
			EFASTI.[TipificationId]					'tipificationId',
			EFASTI.[ID]								'id',
			EFASTI.[SecondLevelItemId]				'secondLevelItemId',
			EFASTI.[Description]					'description',
			EFASTI.[Order]							'order'
		FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipificationItem] EFASTI (NOLOCK)
		WHERE EFASTI.[VCC] = @VCC
			AND [EvaluationPlanId] = @EvaluationPlanId
		ORDER BY EFASTI.[Order] ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlansExpired]
AS
BEGIN
	SET NOCOUNT ON 

	SELECT EP.[ID]										'id',
			AEP.[IdInteraction]							'idInteraction',
			AEP.[IdAuditor]								'idAuditor',
			AEP.[instance]								'vcc',
			AEP.[LastModifiedByUserId]					'lastModifiedByUserId'
		FROM [AuditorEvaluationPlan] AEP (NOLOCK) 
		INNER JOIN [EvaluationPlan] EP (NOLOCK) 
			ON (EP.[Instance] = EP.Instance
				AND EP.[ID] = AEP.IdEvaluationPlan) 
		WHERE EP.[DateFin] < GETUTCDATE()
			AND AEP.[Completed] = 0
			AND AEP.[StatusDescription] NOT IN('INVALID')
			AND EP.[Enabled] = 1
			AND AEP.[Status] = 1
			AND AEP.[IdAuditor] <> 'SPEECH'
			AND EP.[Evaluation_Mode] <> 'A'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlansSummary] (
	@VCC Varchar(50) = NULL,
	@IdAuditor Varchar(50) = NULL,
	@IdAgent Varchar(50) = NULL,
	@Campaigns VARCHAR(MAX) = NULL,
	@EndDate Varchar(50) = NULL,
	@EvaluationFormats Varchar(MAX) = NULL,
	@Modalities Varchar(MAX) = NULL,
	@Completed Bit = 1
)
AS
BEGIN
	SET NOCOUNT ON
	
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = '';
	DECLARE @SQL_FROM VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA VARCHAR(MAX) = '';
	DECLARE @SQL_SELECT_2 VARCHAR(MAX) = '';
	DECLARE @SQL_FROM_2 VARCHAR(MAX) = '';
	DECLARE @SQL_FILTER_2 VARCHAR(MAX) = '';
	DECLARE @SQL_EXTRA_2 VARCHAR(MAX) = '';

	SET @SQL_SELECT_1 += '
		SELECT EP.[ID]																	''id'',
				EP.[Modality]															''modality'',
				EP.[DateFin]															''dateFin'',
				EP.[CreatedDate]														''createdDate'',
				EP.[IdEvaluationFormat]													''idEvaluationFormat'',
				EP.[Description]														''description'',
				COUNT(CASE AEP.Completed WHEN 0 THEN 1 ELSE NULL END)					''pendingInteractionCount'',
				COUNT(AEP.Completed)													''completedCount'',
				SUM(AEP.InteractionDuration)											''interactionTimeSum'',
				SUM(CASE AEP.Completed WHEN 0 THEN AEP.InteractionDuration ELSE 0 END)	''pendingInteractionTimeSum'' ';

	SET @SQL_FROM += '
			FROM [EvaluationPlan] EP (NOLOCK)
			LEFT JOIN [AuditorEvaluationPlan] AEP (NOLOCK)
				ON AEP.[Instance] = EP.[Instance]
					AND AEP.[IdEvaluationPlan] = EP.[ID] '

	IF (@IdAuditor IS NOT NULL AND @IdAuditor <> '') SET @SQL_FROM += 'AND AEP.[IdAuditor] = ''' + @IdAuditor + ''' '

	IF (@IdAgent IS NOT NULL AND @IdAgent <> '') SET @SQL_FROM += 'AND AEP.[IdAgent] = ''' + @IdAgent + ''' '

	SET @SQL_FILTER += '
			WHERE EP.[Instance] = ''' + @VCC + '''
				AND AEP.[IdEvaluationPlan] = EP.[ID]
				AND EP.[Enabled] = 1 ';
				
				
    IF (@EndDate IS NOT NULL AND @EndDate <> '')
    BEGIN 
       SET @SQL_FILTER +=' AND EP.[DateFin] <= ''' + @EndDate + ''' '
    END

	IF (@Campaigns IS NOT NULL AND @Campaigns <> '')
	BEGIN
		SET @SQL_FILTER += '
				AND EP.[IdEvaluationFormat] IN ( 
					SELECT CEF.[IdEvaluationFormat] FROM CampaignEvaluationFormat CEF (NOLOCK)
						WHERE CEF.[Instance] = ''' + @VCC + '''
							AND CEF.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @Campaigns + ''','',''))) '
	END;

	IF (@EvaluationFormats IS NOT NULL AND @EvaluationFormats <> '') SET @SQL_FILTER += 'AND EP.[IdEvaluationFormat] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationFormats + ''','','')) ';
	IF (@Modalities IS NOT NULL AND @Modalities <> '') SET @SQL_FILTER += 'AND EP.[Modality] IN (SELECT Value FROM [dbo].[split](''' + @Modalities + ''','','')) ';
	
	IF (@Completed = 1)
	BEGIN
		SET @SQL_FILTER += 'AND AEP.[Completed] = 1 ';
		SET @SQL_FILTER_2 = ' ';	
	END
	ELSE 
	BEGIN
		SET @SQL_FILTER += 'AND AEP.[Completed] = 0 ';
		SET @SQL_FILTER_2 = 'WHERE X.[pendingInteractionCount] > 0 ';
	END;
	

	SET @SQL_EXTRA += '
			GROUP BY EP.[ID],
						EP.[Modality],
						EP.[DateFin],
						EP.[CreatedDate],
						EP.[IdEvaluationFormat],
						EP.[Description]';

	SET @SQL_SELECT_1 += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA;

	--SET @SQL_SELECT_1 = 'SELECT * FROM (' + @SQL_SELECT_1 + ') AS X WHERE X.[pendingInteractionCount] > 0 ORDER BY X.[Description] ASC; ';
	SET @SQL_SELECT_2 = 'SELECT * ';
	SET @SQL_FROM_2 = 'FROM (' + @SQL_SELECT_1 + ') AS X ';
	-- @SQL_FILTER_2 YA FUE DEFINIDO SEGUN SE BUSCAN LAS YA EVALUADAS O NO EVALUADAS
	SET @SQL_EXTRA_2 = 'ORDER BY X.[Description] ASC;';

	SET @SQL_SELECT_2 += @SQL_FROM_2 + @SQL_FILTER_2 + @SQL_EXTRA_2;


	EXEC (@SQL_SELECT_2);
	
	SET NOCOUNT OFF
	
END;
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlanSubGroups] (
	@VCC VARCHAR(50) = NULL, 
	@EvaluationPlanId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT HSG.[ID]							'id',
			HSG.[Description]				'description'
		FROM [HistoricSubGroup] HSG (NOLOCK)
		WHERE HSG.[VCC] = @VCC
			AND HSG.[EvaluationPlanId] = @EvaluationPlanId
		ORDER BY HSG.[Description] ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationPlanTask] (
    @Instance Varchar(50) = NULL, @ID Varchar(50) = NULL
)
AS
BEGIN
SET NOCOUNT ON

	SELECT
		[ID]						'id',
		[IdEvaluationFormat]		'idEvaluationFormat',
		[FilterByContentQuery]		'filterByContentQuery',
		[FilterByContentQueryName]	'filterByContentQueryName'
	FROM
		[EvaluationPlan] (NOLOCK)
	WHERE
		[Instance] = @Instance
		AND [ID] = @ID

SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationPlanTipificationInteractionTask] (
    @VCC VARCHAR(50) = NULL, 
	@EvaluationPlanId VARCHAR(50) = NULL -- EVALUATION PLAN ID
)
AS
BEGIN
SET NOCOUNT ON
	-- ESTE SP TIENE POR FINALIDAD RETORNAR LA LISTA DE INTERACCIONES CRUZADA CON LOS MODELOS NLU PARA TIPIFICACIONES QUE SE VAN A EVALUAR

	-- DECLARAMOS UNA VARIABLE PARA DEFINIR EL TAMANO DE CAMPO NUEVO EN LA TABLA TEMPORAL
	DECLARE @MAIN_TARGET VARCHAR(50) = 'ABILITY' 
	
	-- BUSCAMOS LOS MODELOS ASOCIADOS AL PLAN Y LAS HABILIDADES QUE LO APLICAN
	SELECT DISTINCT HAQ.VCC										'vcc'
					,N.[Name] + '|' + I.[Name]					'cognitiveId'
					,N.[Name]									'nluId'
					,N.[AppId]									'appId'
					,I.[IntentId]								'intentId'
					,@MAIN_TARGET								'mainTarget'
					,STUFF(
						(
							SELECT DISTINCT ',' + INNER_HAQ.[AbilityId]
								FROM [dbo].[HistoricAbilityQuery] INNER_HAQ (NOLOCK)
								WHERE INNER_HAQ.[VCC] = HAQ.[VCC]
									AND INNER_HAQ.[EvaluationPlanId] = HAQ.[EvaluationPlanId]
									AND INNER_HAQ.[QueryContent] = N.[Name] + '|' + I.[Name]
							FOR XML PATH(''), TYPE
						).value('.', 'NVARCHAR(MAX)'), 
						1, 
						1, 
						''
					)											'abilities'
					,CONVERT(BIT, 1) 							'needRequest'
		INTO #EVALUATION_PLAN_MODELS 
		FROM [HistoricAbilityQuery] HAQ (NOLOCK) 
		INNER JOIN [HistoricAbility] HA (NOLOCK)
			ON HA.[VCC] = HAQ.[VCC]
				AND HA.[EvaluationPlanId] = HAQ.[EvaluationPlanId]
				AND HA.[ID] = HAQ.[AbilityId]
		INNER JOIN [dbo].[Intent] I (NOLOCK)
			ON I.[VCC] = HAQ.[VCC]
				AND I.[IntentId] = HAQ.[QueryId]
		INNER JOIN [dbo].[NLU] N (NOLOCK)
			ON N.[VCC] = HAQ.[VCC]
				AND N.[AppId] = I.[AppId]
		WHERE HAQ.[VCC] = @VCC
			AND HAQ.[EvaluationPlanId] = @EvaluationPlanId
			AND HA.[AnalysisType] = 'COGNITIVE'

	-- DROP TABLE #EVALUATION_PLAN_MODELS 
	-- SELECT * FROM #EVALUATION_PLAN_MODELS 

	-- BUSCAMOS LOS MODELOS ASOCIADOS AL PLAN (TIPIFICACIONES) QUE NO EXISTEN EN LAS HABILIDADES
	INSERT INTO #EVALUATION_PLAN_MODELS 
		SELECT DISTINCT HEFASTQ.VCC								'vcc'
						,N.[Name] + '|' + I.[Name]				'cognitiveId'
						,N.[Name]								'nluId'
						,N.[AppId]								'appId'
						,I.[IntentId]							'intentId'
						,'TIPIFICATION'							'mainTarget'
						,STUFF(
							(
								SELECT DISTINCT ',' + INNER_HS.[IdAbility]
									FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipificationQuery] INNER_HEFASTQ (NOLOCK)
									INNER JOIN [dbo].[HistoricScore] INNER_HS (NOLOCK)
										ON INNER_HS.[VCC] = INNER_HEFASTQ.[VCC]
											AND INNER_HS.[EvaluationPlanId] = INNER_HEFASTQ.[EvaluationPlanId]
											AND INNER_HS.[ID] = INNER_HEFASTQ.[ScoreId]
									WHERE INNER_HEFASTQ.[VCC] = HEFASTQ.[VCC]
										AND INNER_HEFASTQ.[EvaluationPlanId] = HEFASTQ.[EvaluationPlanId]
										AND INNER_HEFASTQ.[QueryContent] = N.[Name] + '|' + I.[Name]
								FOR XML PATH(''), TYPE
							).value('.', 'NVARCHAR(MAX)'), 
							1, 
							1, 
							''
						)										'abilities'
						,CASE WHEN EPM.[vcc] IS NULL 
								THEN CONVERT(BIT, 1)
								ELSE CONVERT(BIT, 0)
							END									'needRequest'
			FROM [dbo].[HistoricEvaluationFormatAbilityScoreTipificationQuery] HEFASTQ (NOLOCK)
			INNER JOIN [dbo].[HistoricScore] HS (NOLOCK)
				ON HS.[VCC] = HEFASTQ.[VCC]
					AND HS.[EvaluationPlanId] = HEFASTQ.[EvaluationPlanId]
					AND HS.[ID] = HEFASTQ.[ScoreId]
			INNER JOIN [dbo].[Intent] I (NOLOCK)
				ON I.[VCC] = HEFASTQ.[VCC]
					AND I.[IntentId] = HEFASTQ.[QueryId]
			INNER JOIN [dbo].[NLU] N (NOLOCK)
				ON N.[VCC] = HEFASTQ.[VCC]
					AND N.[AppId] = I.[AppId]
			LEFT JOIN #EVALUATION_PLAN_MODELS EPM (NOLOCK)
				ON EPM.[VCC] = HEFASTQ.[VCC]
					AND EPM.[appId] = N.[AppId]
					AND EPM.[intentId] = I.[IntentId]
			WHERE HEFASTQ.[VCC] = @VCC
				AND HEFASTQ.[EvaluationPlanId] = @EvaluationPlanId



	-- ESTE SCRIPT RECUPERA LAS INTERACCIONES TIPO CALL QUE ENTRARAN A LA EVALUACION COGNITIVA
	SELECT DISTINCT
			EP.Instance									'vcc'
			,EP.ProcessId								'processId'
			,'EVALUATION_PLAN'							'batchTypeName'
			,AU.Campaign								'campaign'
			,IEP.[InteractionType]						'interactionType'
			,EP.[Id]									'evaluationPlanId'
			,IEP.IdInteraction							'id'
			,AU.Section									'section'
			,AU.Part									'part'
			,NULL										'folder'
			,EPM.nluId									'nluId'
			,EPM.appId									'appId'
			,EPM.[mainTarget]							'mainTarget'
			,AU.[FileName]								'fileName'
			--,EPM.[cognitiveId]						-- CREA MAS REGISTROS DE LOS NECESARIOS AL INICIAR EL TASK
			,V.[CognitiveEngine]						'cognitiveEngine'
			,EPM.[abilities]							'abilities'
			,EPM.[needRequest]							'needRequest'
	FROM [EvaluationPlan] EP (NOLOCK)
	INNER JOIN [InteractionEvaluationPlan] IEP (NOLOCK) 
		ON IEP.Instance = EP.Instance 
			AND IEP.IdEvaluationPlan = EP.Id
	INNER JOIN [dbo].[Audio] AU (NOLOCK) 
		ON AU.VCC = EP.Instance
			AND AU.ID = IEP.IdInteraction 
	INNER JOIN [dbo].[EvaluationResult] ER (NOLOCK)
		ON ER.[Instance] = EP.[Instance]
			AND ER.[IdEvaluationPlan] = EP.[ID]
			AND ER.[IdInteraction] = AU.[ID]
	INNER JOIN #EVALUATION_PLAN_MODELS EPM 
		ON EPM.[vcc] = EP.[Instance]
			--AND EPM.[cognitiveId] = HAQ.[QueryContent] 
			--AND EPM.[intentId] = HAQ.[QueryId] 
	INNER JOIN [dbo].[VCC] V (NOLOCK)
		ON V.[ID] = EP.[Instance]
	WHERE EP.[Instance] = @VCC
		AND EP.[ID] = @EvaluationPlanId
		AND AU.[IsTranscribed] = 1
		AND ER.[Qualification] = 0
		AND EPM.[mainTarget] = 'TIPIFICATION'
	
	-- ESTE SCRIPT RECUPERA LAS INTERACCIONES TIPO TEXT QUE ENTRARAN A LA EVALUACION COGNITIVA
	SELECT DISTINCT
			EP.Instance									'vcc'
			,EP.ProcessId								'processId'
			,'EVALUATION_PLAN'							'batchTypeName'
			,AI.[Campaign]								'campaign'
			,IEP.[InteractionType]						'interactionType'
			,EP.[Id]									'evaluationPlanId'
			,IEP.IdInteraction							'id'
			,NULL										'section'
			,NULL										'part'
			,AI.[EndDate]								'endDate'
			,AI.[Folder]								'folder'
			,AI.[DispositionCode]						'dispositionCode'
			,EPM.nluId									'nluId'
			,EPM.appId									'appId'
			,EPM.[mainTarget]							'mainTarget'
			,AI.[FileName]								'fileName'
			--,F.[cognitiveId]							-- CREA MAS REGISTROS DE LOS NECESARIOS AL INICIAR EL TASK
			,V.[CognitiveEngine]						'cognitiveEngine'
			,EPM.[abilities]							'abilities'
			,EPM.[needRequest]							'needRequest'
	FROM [EvaluationPlan] EP (NOLOCK)
	INNER JOIN [InteractionEvaluationPlan] IEP (NOLOCK) 
		ON EP.Instance = IEP.Instance 
			AND EP.Id = IEP.IdEvaluationPlan
	INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK) 
		ON EP.Instance = AI.VCC 
			AND IEP.IdInteraction = AI.ID
			AND IEP.FormattedEndDate = AI.Folder
	INNER JOIN [dbo].[EvaluationResult] ER (NOLOCK)
		ON ER.[Instance] = EP.[Instance]
			AND ER.[IdEvaluationPlan] = EP.[ID]
			AND ER.[IdInteraction] = AI.[ID]
	INNER JOIN #EVALUATION_PLAN_MODELS EPM 
		ON EPM.[vcc] = EP.[Instance]
			--AND EPM.[cognitiveId] = HAQ.[QueryContent] 
			--AND EPM.[intentId] = HAQ.[QueryId] 
	INNER JOIN [dbo].[VCC] V (NOLOCK)
		ON V.[ID] = EP.[Instance]
	WHERE EP.[Instance] = @VCC
		AND EP.[ID] = @EvaluationPlanId
		AND AI.[IsTranscribed] = 1
		AND ER.[Qualification] = 0
		AND EPM.[mainTarget] = 'TIPIFICATION'

SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationResultEfectivitySummary](
		@VCC VARCHAR(50),
		@EfectivityPlanId VARCHAR(50)
)
AS
BEGIN
		CREATE TABLE #TEMP_EVALUATIONREF(
		VCC	VARCHAR(50),		
		IdInteraction VARCHAR(250),
		IdEvaluationPlan VARCHAR(50),
		ScoreDescription VARCHAR(20),	
		Efectivity	VARCHAR(10),
		Title	VARCHAR(200),
		
		)

		INSERT INTO #TEMP_EVALUATIONREF
			SELECT DISTINCT ERE.[VCC],ER.[IdInteraction],ER.[IdEvaluationPlan],ER.[ScoreDescription],ERE.[Efectivity]
			,A.[Title] 
			FROM dbo.[EvaluationResult] ER (NOLOCK)
			INNER JOIN dbo.[EvaluationResultEfectivity] ERE (NOLOCK)
 
			ON ERE.[VCC] = ER.[Instance]
			AND ERE.[InteractionId] = ER.[IdInteraction]
			AND ERE.[AgentId] = ER.[IdAgent]
			AND ERE.[EvaluationPlanId] = ER.[IdEvaluationPlan] 
			AND ERE.[AbilityEvaluationFormatId] = ER.[IdAbilityEvaluationFormat]		
			INNER JOIN dbo.[Ability] A ON 
			ER.[IdAbility] = A.[ID]
			AND ER.[Instance] = A.[Instance]
			WHERE
			 ER.[Instance] = @VCC		
			 AND ERE.[EfectivityPlanId] = @EfectivityPlanId
		
	

		SELECT DISTINCT IdInteraction  
			INTO #TEMP_EVALUATIONREF_I
			FROM #TEMP_EVALUATIONREF
			WHERE Efectivity IS NOT NULL

		DECLARE @nroInteracciones INTEGER
		SELECT  @nroInteracciones = COUNT(IdInteraction) FROM #TEMP_EVALUATIONREF_I 

		SELECT [VCC] 'vcc',[title] 'ability',
		ISNULL(SUM(CASE WHEN [Efectivity] = 1 THEN 1 ELSE 0 END),0) AS 'correcto' ,
		ISNULL(SUM(CASE WHEN [Efectivity] = 0 THEN 1 ELSE 0 END ),0) AS 'incorrecto' ,
		@nroInteracciones 'nroInteracciones'
		FROM #TEMP_EVALUATIONREF
		GROUP BY  VCC,title


END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationResults] (
	@VCC VARCHAR(50) = NULL, 
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@AuditorId VARCHAR(50) = NULL,
	@EfectivityPlanId VARCHAR(50) = NULL
) 
AS
BEGIN
	SET NOCOUNT ON

	-- SP PARA SOLICITAR LOS RESULTADOS DE LA EVALUACION DE UNA INTERACCION POR VCC, PLAN Y AUDITOR 
	IF(@EfectivityPlanId IS NULL OR @EfectivityPlanId = '' )
	BEGIN
		SELECT ER.[IdAbility]							'idAbility',
				A.[Type]								'abilityType',
				ER.[Qualification]						'qualification',
				ER.[ScoreDescription]					'scoreDescription',
				ER.[Comment]                            'comments',
				ER.[Conditional]                        'conditional',
				ER.[EvaluationDate]						'evaluationDate',
				ER.[AbilityOrder]						'abilityOrder',
				ER.[LastModifiedByUserId]				'lastModifiedByUserId',
				ER.[LastModifiedDate]					'lastModifiedDate'
			FROM [dbo].[EvaluationResult] ER (NOLOCK)
			LEFT JOIN [dbo].[Ability] A (NOLOCK) 
				ON A.[Instance] = ER.[Instance]
					AND A.[ID] = ER.[IdAbility]
			WHERE ER.[Instance] = @VCC 
				AND ER.[IdEvaluationPlan] = @EvaluationPlanId
				AND ER.[IdInteraction] = @InteractionId
				AND ER.[IdAuditor] = @AuditorId
			ORDER BY ER.[AbilityOrder] ASC
	END
	ELSE
	BEGIN
	  
		SELECT ER.[IdAbility]							'idAbility',
			A.[Type]								'abilityType',
			ER.[Qualification]						'qualification',
			ER.[ScoreDescription]					'scoreDescription',
			ER.[Comment]                            'comments',
			ER.[Conditional]                        'conditional',
			ER.[EvaluationDate]						'evaluationDate',
			ER.[AbilityOrder]						'abilityOrder',
			ER.[LastModifiedByUserId]				'lastModifiedByUserId',
			ER.[LastModifiedDate]					'lastModifiedDate',
			ERE.[EfectivityPlanId]					'efectivityPlanId',
			ERE.[Efectivity]						'efectivity',
			ERE.[EfectivityStatus]					'efectivityStatus',
			ERE.[AbilityEvaluationFormatId]			'abilityEvaluationFormatId'
			
		FROM [dbo].[EvaluationResult] ER (NOLOCK)	
		INNER JOIN [dbo].[EvaluationResultEfectivity] ERE (NOLOCK)
			ON  ER.[Instance] = 	ERE.[VCC] AND
				ER.[IdInteraction] = ERE.[InteractionId] AND	
				ER.[IdAgent] =	ERE.[AgentId] AND
				ER.[IdEvaluationPlan] =	ERE.[EvaluationPlanId] AND
				ER.[IdAbilityEvaluationFormat] = ERE.[AbilityEvaluationFormatId] AND				
				ERE.[EfectivityStatus] IN ('CREATED','DRAFT')
		LEFT JOIN [dbo].[Ability] A (NOLOCK) 
			ON A.[Instance] = ER.[Instance]
				AND A.[ID] = ER.[IdAbility]
		WHERE ER.[Instance] = @VCC 
			AND ER.[IdEvaluationPlan] = @EvaluationPlanId
			AND ER.[IdInteraction] = @InteractionId
			AND ERE.[AuditorId] = @AuditorId
			AND ERE.EfectivityPlanId = @EfectivityPlanId
		ORDER BY ER.[AbilityOrder] ASC


	END

	-- BUSCAMOS LOS RESULTADOS DE LAS TIPIFICACIONES DE 1ER NIVEL
	SELECT ET.[AbilityId]							'abilityId',
			ET.[ScoreId]							'scoreId',
			ET.[ID]									'id',
			ET.[IsChecked]							'isChecked', 
			ET.[Comment]							'comment'
		FROM [dbo].[EvaluationTipification] ET (NOLOCK)
		WHERE ET.[VCC] = @VCC 
			AND ET.[EvaluationPlanId] = @EvaluationPlanId
			AND ET.[InteractionId] = @InteractionId
			AND ET.[AuditorId] = @AuditorId

	-- BUSCAMOS LOS RESULTADOS DE LAS TIPIFICACIONES DE 2DO Y 3ER NIVEL
	SELECT ETI.[ScoreId]								'scoreId',
			ETI.[TipificationId]						'tipificationId',
			ETI.[ID]									'id',
			ETI.[IsChecked]								'isChecked'
		FROM [dbo].[EvaluationTipificationItem] ETI (NOLOCK)
		WHERE ETI.[VCC] = @VCC 
			AND ETI.[EvaluationPlanId] = @EvaluationPlanId
			AND ETI.[InteractionId] = @InteractionId
			AND ETI.[AuditorId] = @AuditorId

	-- CONSULTAMOS LOS RESULTADOS DE LOS CAMPOS PERSONALIZADOS
	EXEC [dbo].[GetEvaluationCustomFieldResultsFromAuditorEvaluationPlan] @VCC, @EvaluationPlanId, @InteractionId, @AuditorId, NULL
		
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationResultsByCriteria] (
	--@VCC VARCHAR(50) = NULL,
	@InteractionStartDate				DATETIME = '1980-01-01T00:00:00',
    @InteractionEndDate					DATETIME = '2050-01-01T23:59:59',
	@EvaluationPlanId					VARCHAR(50) = NULL,
	@AgentIds							VARCHAR(MAX) = NULL,
	@OKAbilities						VARCHAR(MAX) = NULL, -- SETTEAR ALL PARA "TODOS HABILIDADES CUMPLEN"
	@KOAbilities						VARCHAR(MAX) = NULL -- SETTEAR ALL PARA "TODOS HABILIDADES NO CUMPLEN"
)
AS
BEGIN
	SET NOCOUNT ON

	-- VALIDAMOS LA DATA NECESARIA
	IF @EvaluationPlanId IS NULL OR @EvaluationPlanId = '' RAISERROR('DATABASE_ERROR_OK_EVALUATION_PLAN_ID_NOT_RECEIVED', 16, 16)
	IF @OKAbilities = 'ALL'
	-- ISNULL(LEN(@OKAbilities), 0) = 0
	BEGIN
		SELECT @OKAbilities = STUFF(
					(
						SELECT DISTINCT ',' + HA.[Title] 
							FROM [dbo].[HistoricAbilityEvaluationFormat] HAEF (NOLOCK)
							INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
								ON HA.[VCC] = HAEF.[VCC]
									AND HA.[EvaluationPlanId] = [HAEF].[EvaluationPlanId]
									AND HA.[ID] = HAEF.[IdAbility]
							WHERE [HAEF].[EvaluationPlanId] = @EvaluationPlanId
						FOR XML PATH(''), TYPE
					).value('.', 'NVARCHAR(MAX)'), 
					1, 
					1, 
					''
		)
	END
	
	IF @KOAbilities = 'ALL'
	-- IF ISNULL(LEN(@KOAbilities), 0) = 0
	BEGIN
		SELECT @KOAbilities = STUFF(
					(
						SELECT DISTINCT ',' + HA.[Title]  
							FROM [dbo].[HistoricAbilityEvaluationFormat] HAEF (NOLOCK)
							INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
								ON HA.[VCC] = HAEF.[VCC]
									AND HA.[EvaluationPlanId] = [HAEF].[EvaluationPlanId]
									AND HA.[ID] = HAEF.[IdAbility]
							WHERE [HAEF].[EvaluationPlanId] = @EvaluationPlanId
						FOR XML PATH(''), TYPE
					).value('.', 'NVARCHAR(MAX)'), 
					1, 
					1, 
					''
		)
	END

	-- DECLARAMOS VARIABLES PARA DATA DE HABILIDADES
	DECLARE @TOTAL_ABILITIES INT = 0
	DECLARE @TOTAL_OK_ABILITIES INT = 0
	DECLARE @TOTAL_KO_ABILITIES INT = 0
	
	-- OBTENMOS LOS VALORES
	If @OKAbilities = '' SET @TOTAL_OK_ABILITIES = 0 ELSE SELECT @TOTAL_OK_ABILITIES = COUNT(Value) FROM [dbo].[split](@OKAbilities,',')
	If @KOAbilities = '' SET @TOTAL_KO_ABILITIES = 0 ELSE SELECT @TOTAL_KO_ABILITIES = COUNT(Value) FROM [dbo].[split](@KOAbilities,',')
	SET @TOTAL_ABILITIES = @TOTAL_OK_ABILITIES + @TOTAL_KO_ABILITIES

	;WITH RAW_RESULTS_CTE AS (
		SELECT ER.* 
			FROM [dbo].[EvaluationResult] ER (NOLOCK) 
			INNER JOIN [dbo].[Ability] A (NOLOCK)
				ON A.[Instance] =  ER.[Instance]
					AND A.[ID] = ER.IdAbility
			WHERE ER.[Instance] = 'bbva'
				AND ER.[IdEvaluationPlan] = @EvaluationPlanId
				AND ER.[InteractionDateFin] BETWEEN @InteractionStartDate AND @InteractionEndDate
				AND (
					(
						ISNULL(LEN(@AgentIds), 0) <> 0
						AND
						ER.[IdAgent] IN (SELECT Value FROM [dbo].[split](@AgentIds,','))
					)
					OR
					(
						ISNULL(LEN(@AgentIds), 0) = 0
					)
				)
				AND (
					(
						-- RETORNA LAS INTERACCIONES QUE TIENEN 100 EN LAS HABILIDADES OK
						ISNULL(LEN(@OKAbilities), 0) <> 0
						AND 
						A.[Title] IN (SELECT Value FROM [dbo].[split](@OKAbilities,','))
						AND 
						ER.[AbilityInnerScore] = 100
					)
					OR
					(
						ISNULL(LEN(@KOAbilities), 0) <> 0
						AND 
						A.[Title] IN (SELECT Value FROM [dbo].[split](@KOAbilities,','))
						AND 
						ER.[AbilityInnerScore] = 0
					)
				)
	),
	FILTERED_RESULTS_CTE AS (
		SELECT * ,
				RN = ROW_NUMBER() OVER(PARTITION BY [IdInteraction] ORDER BY [CreatedDate] ASC)
			FROM RAW_RESULTS_CTE
	),
	UNIQUE_RESULTS_CTE AS (
		SELECT * 
			FROM FILTERED_RESULTS_CTE WHERE [RN] >= @TOTAL_ABILITIES
	)
	SELECT UR_CTE.[IdEvaluationPlan]				'EvaluationPlanId',
			UR_CTE.[IdCampaign]						'CampaignId',
			UR_CTE.[InteractionType]				'InteractionmType',
			UR_CTE.[IdInteraction]					'InteractionId',
			UR_CTE.[IdAgent]						'AgentId',
			UR_CTE.[InteractionDateIni]				'StartDate',
			UR_CTE.[InteractionDateFin]				'EndDate',
			HA.[Title]								'Ability',
			ER.[ScoreDescription]					'AbilityResult'
		FROM UNIQUE_RESULTS_CTE UR_CTE
		INNER JOIN [dbo].[EvaluationResult] ER (NOLOCK)
			ON ER.[Instance] = UR_CTE.[Instance] 
				AND ER.[IdEvaluationPlan] = UR_CTE.[IdEvaluationPlan]
				AND ER.[IdInteraction] =  UR_CTE.[IdInteraction]
		INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
			ON HA.[VCC] = UR_CTE.[Instance] 
				AND HA.[EvaluationPlanId] = UR_CTE.[IdEvaluationPlan]
				AND HA.[ID] = ER.[IdAbility]
	
	/*
	SELECT STUFF(
				(
					SELECT DISTINCT ',' + IdInteraction
						FROM UNIQUE_RESULTS_CTE
					FOR XML PATH(''), TYPE
				).value('.', 'NVARCHAR(MAX)'), 
				1, 
				1, 
				''
	)						'InteractionIds'
	*/
	/*
	EXEC [dbo].[GetEvaluationResultsByCriteria] 
	@EvaluationPlanId = 'plan_transferencia_2202_bbva', 
	@AgentIds = 'xp65048@gss.bbva.com,xp65024@gss.bbva.com,maria.cubas.cruz.contractor@bbva.com', 
	@OKAbilities = 'Amabilidad_empatia,Despedida_query', 
	@KOAbilities = 'Bienvenida'
	*/

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationResultsFromAuditorEvaluationPlan] (
	@Instance Varchar(50) = NULL,
    @EvaluationPlanId Varchar(50) = NULL,
    @InteractionId varchar(250) = NULL,
    @InteractionType varchar(50) = NULL,
    @AuditorId varchar(50) = NULL,
    @AgentId varchar(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON
	-- TODO: ESTE SP ES USADO PARA GENERAR EL PDF DE EXPORTACION, HAY QUE INTEGRARLO CON LA FUNCIONALIDAD FACTORIZADA Y DESAPARECER ESTE SP

	-- EMULAMOS UNA NUEVA TABLA VARIABLE PARA OBTENER LOS REGISTROS UNICOS QUE INSERTAREMOS EN LA TABLA EVALUATION_RESULT
	--  ORDENANDO POR SECTION Y PART Y OBTENER EL NOMBRE DEL AGENTE
	;WITH CTE AS (
		SELECT TOP 1 IEP.[Instance],
						IEP.[IdEvaluationPlan],
						IEP.[IdInteraction],
						IEP.[IdAgent],
						IEP.[AgentName]
			FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
			WHERE IEP.[Instance] = @Instance
				AND IEP.[IdEvaluationPlan] = @EvaluationPlanId
				AND IEP.[IdInteraction] = @InteractionId 
				AND IEP.[InteractionType] = @InteractionType
				AND IEP.[IdAgent] = @AgentId
	)

    SELECT DISTINCT
         EF.Id										'EvaluationFormatId'				-- Id
        ,EF.Description								'EvaluationFormatDescription'		-- Titulo del reporte
        ,ER.Instance								'Instance'							-- Subtitulo del reporte
        ,ER.IdAgent									'AgentId'							-- Agente
        ,ER.IdAgent									'AgentDescription'					-- Agente
		,C.[AgentName]								'agentName'							-- NOMBRE DEL AGENTE
        ,CONCAT(U.Firstname, ' ', U.Lastname)		'CoordinatorFullName'				-- Coordinador
        ,AEP.IdAuditor								'AuditorId'							-- Auditor
        ,CONCAT(UA.Firstname, ' ',UA.Lastname)		'AuditorDescription'				-- Auditor
		,AEP.IdAuditor
        ,ER.IdCampaign								'CampaignDescription'				-- Campana
        ,ER.IdInteraction							'InteractionId'						-- Grabacion
        ,ER.InteractionType							'InteractionType'					-- Tipo interaccion
        ,ER.InteractionDateini						'InteractionDateIni'				-- Fecha inicio de la interaccion
		,ER.[InteractionDatefin]					'InteractionDateFin'				-- Fecha fin de la interaccion
        ,ER.InteractionAddress						'Contacto'							-- Telefono
        ,ER.EvaluationDate							'EvaluationDate'					-- Fecha monitoreo
        ,AEP.Score									'TotalScore'						-- Puntaje total
        ,G.Id										'GroupId'							-- Grupo
        ,G.Description								'Group'								-- Grupo
        ,SG.Id										'SubGroupId'						-- Subgrupo
        ,SG.Description								'SubGroup'							-- Subgrupo
        ,AB.Id										'idAbility'							-- Habilidad
		,AB.Id										'AbilityId'							-- Habilidad
        ,AB.Title									'Ability'							-- Habilidad
		,AB.[Detail]								'AbilityDetail'						-- Habilidad
        ,ER.[Comment]								'Observacion'						-- Observacion de la habilidad
        ,ER.MaxAbilityScore							'Puntaje'							-- Puntaje
        ,ER.ScoreDescription						'Eval'								-- Eval
        ,ER.Qualification							'Nota'								-- Nota
		,ER.[OwnScore]								'ownScore'
		,ER.[InheritedScore]						'inheritedScore'
		,ER.[AbilityInnerScore]						'abilityInnerScore'
        ,ER.NotApplies								'NotApplies'
		,AB.[Distribute]							'distribute'
        ,ER.AbilityOrder							'Order'
        ,ER.Conditional                             'conditionalSelected'
    FROM [dbo].[EvaluationResult] ER (NOLOCK)
    INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
         ON (EP.[Instance] = ER.[Instance]
			AND EP.ID = ER.IdEvaluationPlan)
	INNER JOIN	[dbo].[HistoricEvaluationFormat] EF (NOLOCK)
	--LEFT JOIN [EvaluationFormat] EF (NOLOCK)
		ON (ER.Instance = EF.VCC AND EP.ID = EF.EvaluationPlanId
			AND	ER.IdEvaluationFormat = EF.ID)
	LEFT JOIN [dbo].[User] U (NOLOCK)
		ON (EP.Instance = U.VCC
			AND EP.CreatedByUserId = U.ID)
    --INNER JOIN [User] UA (NOLOCK)
	LEFT JOIN [dbo].[User] UA (NOLOCK)
		ON (ER.Instance = UA.VCC 
			AND ER.IdEvaluator = UA.ID)
    --INNER JOIN [AuditorEvaluationPlan] AEP (NOLOCK)
	LEFT JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
		ON (ER.IdEvaluationPlan	= AEP.IdEvaluationPlan
			AND ER.IdEvaluator = AEP.IdAuditor
			AND ER.IdInteraction = AEP.IdInteraction
			AND ER.InteractionType = AEP.InteractionType
			AND ER.IdAgent = AEP.IdAgent)
    LEFT JOIN [dbo].[HistoricGroup] G (NOLOCK)
		ON (ER.Instance = G.VCC AND EP.ID = G.EvaluationPlanId
			AND ER.IdGroup = G.ID)
    LEFT JOIN [HistoricSubGroup] SG (NOLOCK)
		ON (ER.Instance = SG.VCC AND EP.ID = SG.EvaluationPlanId
			AND ER.IdSubGroup = SG.ID)
    LEFT JOIN [HistoricAbility] AB (NOLOCK)
		ON (ER.Instance = AB.VCC AND EP.ID = AB.EvaluationPlanId
			AND ER.IdAbility = AB.ID)
	LEFT JOIN CTE C (NOLOCK) -- LA TABLA AGENT NO ES DETERMINANTE POR ESO SE AGREGA POR LEFT JOIN
		ON C.[Instance] = ER.[Instance]
			AND C.[IdInteraction] = ER.IdInteraction
			AND C.[IdAgent] = ER.[IdAgent]
    WHERE ER.Instance = @Instance
		AND ER.IdEvaluationPlan	= @EvaluationPlanId
		AND ER.IdInteraction = @InteractionId
		AND ER.InteractionType = @InteractionType
		--AND ER.IdEvaluator = @AuditorId
		AND ER.[IdAuditor] = @AuditorId
		AND ER.IdAgent = @AgentId
		AND AEP.[StatusDescription] NOT IN ('INVALID')
    ORDER BY ER.AbilityOrder ASC

	-- BUSCAMOS LOS RESULTADOS DE LAS TIPIFICACIONES DE 1ER NIVEL
	SELECT ET.[AbilityId]							'abilityId',
			ET.[ScoreId]							'scoreId',
			ET.[ID]									'id',
			HEFAST.[Description]					'tipificationDescription',
			ET.[IsChecked]							'isChecked',
			ET.[Comment]							'comment'
		FROM [dbo].[EvaluationTipification] ET (NOLOCK)
		INNER JOIN [dbo].[HistoricEvaluationFormatAbilityScoreTipification] HEFAST (NOLOCK)
			ON HEFAST.[VCC] = ET.[VCC]
				AND HEFAST.[EvaluationPlanId] = ET.[EvaluationPlanId]
				AND HEFAST.[ScoreId] = ET.[ScoreId]
				AND HEFAST.[ID] = ET.[ID]
		WHERE ET.[VCC] = @Instance 
			AND ET.[EvaluationPlanId] = @EvaluationPlanId
			AND ET.[InteractionId] = @InteractionId
			AND ET.[AuditorId] = @AuditorId
			AND ET.[IsChecked] = 1
		ORDER BY HEFAST.[Order] ASC

	-- BUSCAMOS LOS RESULTADOS DE LAS TIPIFICACIONES DE 2DO Y 3ER NIVEL
	SELECT ETI.[ScoreId]								'scoreId',
			ETI.[TipificationId]						'tipificationId',
			HEFASTI.[SecondLevelItemId]					'secondLevelItemId',
			ETI.[ID]									'id',
			HEFASTI.[Description]						'tipificationItemDescription',
			ETI.[IsChecked]								'isChecked' 
		FROM [dbo].[EvaluationTipificationItem] ETI (NOLOCK)
		INNER JOIN [dbo].[HistoricEvaluationFormatAbilityScoreTipificationItem] HEFASTI (NOLOCK)
			ON HEFASTI.[VCC] = ETI.[VCC]
				AND HEFASTI.[EvaluationPlanId] = ETI.[EvaluationPlanId]
				AND HEFASTI.[ScoreId] = ETI.[ScoreId]
				AND HEFASTI.[ID] = ETI.[ID]
		WHERE ETI.[VCC] = @Instance 
			AND ETI.[EvaluationPlanId] = @EvaluationPlanId
			AND ETI.[InteractionId] = @InteractionId
			AND ETI.[AuditorId] = @AuditorId
			AND ETI.[IsChecked] = 1
		ORDER BY HEFASTI.[Order] ASC

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationReview] (
	@VCC Varchar(50) = NULL, 
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@SupervisorId VARCHAR(50) = NULL,
	@AgentId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	-- SELECCIONAMOS LOS CAMPOS DE LA TABLA EVALUATION-REVIEW
	SELECT ER.[VCC]													'vcc',
			ER.[EvaluationPlanId]									'evaluationPlanId',
			ER.[InteractionId]										'interactionId',
			ER.[SupervisorId]										'supervisorId',
			ER.[Review]												'review'
		FROM [dbo].[EvaluationReview] ER (NOLOCK) 
		WHERE ER.[VCC] = @VCC
			AND ER.[EvaluationPlanId] = @EvaluationPlanId
			AND ER.[InteractionId] = @InteractionId
			AND ER.[SupervisorId] = ISNULL(@SupervisorId, ER.[SupervisorId])

	-- SELECCIONAMOS LOS CAMPOS DE LA TABLA EVALUATION-REVIEW-COMMITMENT
	EXEC [dbo].[GetEvaluationReviewCommitments] @VCC, @EvaluationPlanId, @InteractionId, @SupervisorId, @AgentId
	/*
	SELECT ER.[EvaluationPlanId]									'evaluationPlanId',
			ER.[SupervisorId]										'supervisorId',
			ERC.[LargeInteractionId]								'largeInteractionId',
			ERC.[AgentId]											'agentId',
			ERC.[Commitment]										'commitment',
			ERC.[CreatedDate]										'eventDate',
			ERC.[CreatedDate]										'createdDate'
		FROM [dbo].[EvaluationReviewCommitment] ERC (NOLOCK) 
		INNER JOIN [dbo].[EvaluationReview] ER (NOLOCK)
			ON ER.[VCC] = ERC.[VCC]
				AND ER.[Id] = ERC.[EvaluationReviewId]
		WHERE ER.[VCC] = @VCC
			AND ER.[EvaluationPlanId] = @EvaluationPlanId
			AND ER.[InteractionId] = @InteractionId
			AND ER.[SupervisorId] = ISNULL(@SupervisorId, ER.[SupervisorId])
			AND ERC.[AgentId] = ISNULL(@AgentId, ERC.[AgentId])
		ORDER BY [ERC].[CreatedDate] DESC
	*/
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationReviewCommitments] (
	@VCC Varchar(50) = NULL, 
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@SupervisorId VARCHAR(50) = NULL,
	@AgentId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- SELECCIONAMOS LOS CAMPOS DE LA TABLA EVALUATION-REVIEW-COMMITMENT
	SELECT ERC.[EvaluationPlanId]										'evaluationPlanId',
			ERC.[ID]													'id',
			AEP.[Score]													'score',
			ERC.[SupervisorId]											'supervisorId',
			CONCAT(U2.[FirstName], ' ', ISNULL(U2.[LastName], ''))		'supervisorName',
			ERC.[LargeInteractionId]									'largeInteractionId',
			ERC.[AgentId]												'agentId',
			CONCAT(U1.[FirstName], ' ', ISNULL(U1.[LastName], ''))		'agentName',
			ERC.[Commitment]											'commitment',
			ERC.[CreatedDate]											'eventDate',
			ERC.[CreatedDate]											'createdDate'
		FROM [dbo].[EvaluationReviewCommitment] ERC (NOLOCK) 
		LEFT JOIN [dbo].[User] U1 (NOLOCK)
			ON U1.[VCC] = ERC.[VCC]
				AND U1.[ID] = ERC.[AgentId]
		INNER JOIN [dbo].[User] U2 (NOLOCK)
			ON U2.[VCC] = ERC.[VCC]
				AND U2.[ID] = ERC.[SupervisorId]
		INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
			ON AEP.[Instance] = ERC.[VCC]
				AND AEP.[IdEvaluationPlan] = ERC.[EvaluationPlanId]
				AND AEP.[IdInteraction] = ERC.[InteractionId]
				-- AND AEP.[IdAuditor] = ERC.[SupervisorId]
				AND AEP.[IdAgent] = ERC.[AgentId]
		WHERE ERC.[VCC] = @VCC
			AND ERC.[EvaluationPlanId] =  ISNULL(@EvaluationPlanId, ERC.[EvaluationPlanId])
			AND ERC.[InteractionId] = ISNULL(@InteractionId, ERC.[InteractionId])
			AND ERC.[SupervisorId] = ISNULL(@SupervisorId, ERC.[SupervisorId])
			AND ERC.[AgentId] = ISNULL(@AgentId, ERC.[AgentId])
		ORDER BY [ERC].[CreatedDate] DESC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluations] (
	@Instance Varchar(50) = NULL,
	@IdAuditor VARCHAR(MAX) = NULL,
	@IdAgent Varchar(MAX) = NULL,
   	@IdCampaign VARCHAR(MAX) = NULL,
	@IdEvaluationPlan VARCHAR(MAX) = NULL,
 	@EvaluationDateFrom VARCHAR(50) = '1980-01-01T00:00:00',
    @EvaluationDateTo VARCHAR(50) = '2050-01-01T23:59:59',
	@EvaluationHourFrom VARCHAR(50) = '00:00:00',
    @EvaluationHourTo VARCHAR(50) = '23:59:59',
	@InteractionMinDuration Int = 0,
    @InteractionMaxDuration Int = 86400,
    @InteractionOrigins Varchar(50) = NULL,
	@InteractionResults Varchar(MAX) = NULL,
	@EvaluationTypes Varchar(MAX) = NULL,
	@InteractionTags Varchar(MAX) = NULL,
	@InteractionFormats Varchar(50) = NULL,
	@InteractionCoached VARCHAR(50) = NULL,
	@InteractionIds     VARCHAR(MAX) = NULL,
	@EvaluationsLimit Int = 0,
	@ContactAddress Varchar(300) = NULL,
	@SentimentIds VARCHAR(MAX) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	 -- SENTIMIENTOS
	 DECLARE @FILTERS_SENTIMENTIDS VARCHAR(MAX)
	 DECLARE @FILTER_ANALYSIS_TYPE VARCHAR(MAX)
	 SET @FILTERS_SENTIMENTIDS = ''
	 SET @FILTER_ANALYSIS_TYPE = ''
	IF (@SentimentIds IS NOT NULL AND @SentimentIds <> '')
    BEGIN		
		SET @FILTERS_SENTIMENTIDS += '  AND SAR.[SentimentId] IN (SELECT Value FROM [dbo].[split](''' + @SentimentIds + ''','','')) '										
	END
	 
	 IF (SELECT COUNT(*) FROM [dbo].[split](@InteractionOrigins, ',')) = 2 
		SET  @FILTER_ANALYSIS_TYPE =  ' 
				AND SAR.[AnalysisType] ='+''''+'general'+''''
	 IF (@InteractionOrigins = 'IN') 
		SET  @FILTER_ANALYSIS_TYPE =  ' 
				AND SAR.[AnalysisType] ='+''''+'client'+''''
	 IF (@InteractionOrigins = 'OUT') 
		SET  @FILTER_ANALYSIS_TYPE =  ' 
				AND SAR.[AnalysisType] ='+''''+'agent'+''''
	 -- FIN SENTIMIENTOS 
	
	IF (@EvaluationHourFrom = '') SET @EvaluationHourFrom = '00:00:00';
	IF (@EvaluationHourTo = '') SET @EvaluationHourTo = '23:59:59';
	
	-- SI ORIGINS TIENE DOS VALORES, IGNORAMOS EL FILTRO
	IF (SELECT COUNT(*) FROM [dbo].[split](@InteractionOrigins, ',')) = 2 SET @InteractionOrigins = ''
	
	DECLARE @SQL_TMP VARCHAR(MAX) = ''
	DECLARE @SQL_CTE VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_SELECT VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FROM_FIXED VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FROM_OPTIONAL VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_FIXED VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_OPTIONAL VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_EXCLUDING VARCHAR(MAX) = ''

	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''
	DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_FIXED_PART     VARCHAR(MAX) = ''

	-- INICIAMOS ARMANDO LOS FILTROS PARA LA TABLA CTE PRINCIPAL
	SET @SQL_CTE_FILTERS_FIXED = '
			WHERE ER.[Instance] = ''' + @Instance + '''
				AND ER.[EvaluationDate] BETWEEN ''' + @EvaluationDateFrom + '''AND ''' + @EvaluationDateTo + '''
				AND AEP.[Completed] = 1
				AND (
					(
						EP.[Modality] <> ''REALTIME''
						AND 
						EP.[Status] = CASE EP.[Evaluation_Mode] WHEN ''A'' THEN 7 ELSE EP.[Status] END 
					)
					OR
					EP.[Modality] = ''REALTIME''
				) '

	-- AGREGAMOS EL SCRIPT PARA LOS FILREOS DE HORA MAXIMA Y HORA MINIMA LAS CUALES TIENEN UN COMPORTAMIENTO DIFERENTES SEGUN CUAL ES MAYOR
	IF (CAST(@EvaluationHourFrom AS TIME) <= CAST(@EvaluationHourTo AS TIME)) 
		SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND CAST(ER.[EvaluationDate] AS TIME) BETWEEN ''' + @EvaluationHourFrom + ''' AND ''' + @EvaluationHourTo + ''' '
	IF (CAST(@EvaluationHourFrom AS TIME) > CAST(@EvaluationHourTo AS TIME)) 
		SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND CAST(ER.[EvaluationDate] AS TIME) NOT BETWEEN ''' + @EvaluationHourTo + ''' AND ''' + @EvaluationHourFrom + ''' '

	-- AGREGAMOS EL SCRIPT PARA LOS FILREOS DE DURACION MAXIMA Y DURACION MINIMA 
	IF (@InteractionMinDuration IS NOT NULL) 
		SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND AEP.[InteractionDuration] >= ' + (SELECT CONVERT(varchar(10), @InteractionMinDuration)) + ' '
	IF (@InteractionMaxDuration IS NOT NULL) 
		SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND AEP.[InteractionDuration] <= ' + (SELECT CONVERT(varchar(10), @InteractionMaxDuration))+ ' '
	
	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE ORIGIN
	IF (@InteractionOrigins IS NOT NULL AND @InteractionOrigins != '') 
		SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND AEP.[InteractionOrigin] IN (SELECT Value FROM [dbo].[split](''' + @InteractionOrigins + ''','','')) '

	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE AUDITOR
	IF (@IdAuditor IS NOT NULL AND @IdAuditor <> '') 
	BEGIN
		-- DEBEMOS PASAR LA LISTA DE AUDITORS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
		SELECT * 
			INTO #FILTER_AUDITORS
			FROM (
				SELECT Value FROM [dbo].[split](@IdAuditor, ',')
			) A

		-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
		SET @SQL_CTE_FROM_OPTIONAL += '  
			INNER JOIN #FILTER_AUDITORS FA 
				ON FA.[Value] = ER.[IdAuditor] ' 
	END

	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE AGENT
	IF (@IdAgent IS NOT NULL AND @IdAgent != '') 
	BEGIN
		-- DEBEMOS PASAR LA LISTA DE AUDITORS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
		SELECT * 
			INTO #FILTER_AGENTS
			FROM (
				SELECT Value FROM [dbo].[split](@IdAgent, ',')
			) A

		-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
		SET @SQL_CTE_FROM_OPTIONAL += '  
			INNER JOIN #FILTER_AGENTS FAG 
				ON FAG.[Value] = ER.[IdAgent] ' 
	END

	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE CAMPAIGN
	IF (@IdCampaign IS NOT NULL AND @IdCampaign != '') 
	BEGIN
		-- DEBEMOS PASAR LA LISTA DE AUDITORS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
		SELECT * 
			INTO #FILTER_CAMPAIGNS
			FROM (
				SELECT Value FROM [dbo].[split](@IdCampaign, ',')
			) A
		
		-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
		SET @SQL_CTE_FROM_OPTIONAL += '  
			INNER JOIN #FILTER_CAMPAIGNS FC 
				ON FC.[Value] = ER.[IdCampaign] ' 
	END

	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE EVALUATION PLAN
	IF (@IdEvaluationPlan IS NOT NULL AND @IdEvaluationPlan != '') 
	BEGIN
		-- DEBEMOS PASAR LA LISTA DE AUDITORS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
		SELECT * 
			INTO #FILTER_EVALUATION_PLANS
			FROM (
				SELECT Value FROM [dbo].[split](@IdEvaluationPlan, ',')
			) A 
		
		-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
		SET @SQL_CTE_FROM_OPTIONAL += '  
			INNER JOIN #FILTER_EVALUATION_PLANS FEP 
				ON FEP.[Value] = ER.[IdEvaluationPlan] ' 
	END

	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE EVALUATION PLAN
	IF @ContactAddress IS NOT NULL AND @ContactAddress <> ''
	BEGIN
		SET @SQL_CTE_FROM_OPTIONAL += '
				AND AEP.[ContactAddress] LIKE ''%' + @ContactAddress + '%'' '
	END	

	-- AGREGAMOS EL SCRIPT PARA INTERACTION_ID
	IF @InteractionIds IS NOT NULL AND @InteractionIds <> ''
	BEGIN
		SET @SQL_CTE_FROM_OPTIONAL = ''
		SET @SQL_CTE_FILTERS_OPTIONAL = ''
			
		-- DEBEMOS PASAR LA LISTA DE CAMPAIGNS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
		SELECT * 
			INTO #FILTER_INTERACTIONS_IDS
			FROM (
				SELECT Value FROM [dbo].[split](@InteractionIds, ',')
			) A

		-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
		SET @SQL_CTE_FROM_OPTIONAL += '  
			INNER JOIN #FILTER_INTERACTIONS_IDS FID 
				ON FID.[Value] = AEP.[IdInteraction] ' 
		-- SET @SQL_CTE_FILTERS_EXCLUDING = ' AND ID.[Id] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) '
	END 

	-- ARMAMOS EL QUERY PARA EL CONTENIDO DE LA CTE
	SET @SQL_CTE_SELECT = '
		;WITH CTE_RESULTS AS (
			SELECT ER.[Instance],
					ER.[IdEvaluationPlan],
					ER.[IdInteraction], 
					IEP.[InteractionSection],
					IEP.[InteractionPart],
					IEP.[InteractionDateEnd],
					ER.[IdAgent],
					IEP.[AgentName],
					IEP.[CreatedDate]
					-- SAR.[SentimentId]
				FROM [dbo].[EvaluationResult] ER (NOLOCK) 
				INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
					ON EP.[Instance] = ER.[Instance]
						AND EP.[Id] =  ER.[IdEvaluationPlan]
				INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
					ON (AEP.[Instance] = ER.[Instance] 
						AND AEP.[IdAuditor] = ER.[IdAuditor] 
						AND AEP.[IdInteraction] = ER.[IdInteraction] 
						AND AEP.[InteractionType] = ER.[InteractionType] 
						AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]) 
				INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
					ON IEP.[Instance] = ER.[Instance] 
						AND IEP.[IdEvaluationPlan] = EP.[ID]
						AND IEP.[IdInteraction] = ER.[IdInteraction]
						AND IEP.[IdAgent] = ER.[IdAgent] '
					+ @SQL_CTE_FROM_OPTIONAL
					+ @SQL_CTE_FILTERS_FIXED
					+ @SQL_CTE_FILTERS_OPTIONAL
					+ @SQL_CTE_FILTERS_EXCLUDING + '
		)
		,CTE_WITH_SAR AS ( 
			SELECT CTE_R.*,
					SAR.[SentimentId],
					RN = ROW_NUMBER() OVER(PARTITION BY CTE_R.[Instance], CTE_R.[IdEvaluationPlan], CTE_R.[IdInteraction] ORDER BY CTE_R.[CreatedDate] ASC) 
				FROM CTE_RESULTS CTE_R
				LEFT JOIN [dbo].[SentimentAnalysisResult] SAR (NOLOCK)
					ON CTE_R.[Instance] = SAR.[VCC]
						AND CTE_R.[IdInteraction] = SAR.[InteractionId]
						AND CTE_R.[InteractionSection] = SAR.[InteractionSection]
						AND CTE_R.[InteractionPart] = SAR.[InteractionPart]
                        AND ISNULL(CTE_R.[InteractionDateEnd], 0) = ISNULL(SAR.[EndDate], 0)
                        AND SAR.[MaxScore] = 1
						' + @FILTERS_SENTIMENTIDS + '
						' + @FILTER_ANALYSIS_TYPE + '
		)
		,SUMMARIZED_CTE AS (
			SELECT CTE_S.[Instance],
					CTE_S.[IdEvaluationPlan],
					CTE_S.[IdInteraction], 
					CTE_S.[IdAgent],
					CTE_S.[AgentName],
					CTE_S.[SentimentId],
					IIF (
						ERV.[InteractionId] IS NULL,
						CAST(0 AS BIT),
						CAST(1 AS BIT)
					)										''Review'',
					COUNT(ERC.[Id])							''Commitments''
				FROM CTE_WITH_SAR CTE_S
				LEFT JOIN [dbo].[EvaluationReview] ERV (NOLOCK)
					ON ERV.[VCC] = CTE_S.[Instance]
						AND ERV.[EvaluationPlanId] = CTE_S.[IdEvaluationPlan]
						AND ERV.[InteractionId] = CTE_S.[IdInteraction]
				LEFT JOIN [dbo].[EvaluationReviewCommitment] ERC (NOLOCK)
					ON ERC.[VCC] = CTE_S.[Instance] 
						AND ERC.[EvaluationPlanId] = CTE_S.[IdEvaluationPlan]
						AND ERC.[InteractionId] = CTE_S.[IdInteraction]
						-- AND ERC.[SupervisorId] = AEP.[IdAuditor] 
						AND ERC.[AgentId] = CTE_S.[idAgent]
				WHERE RN = 1
				GROUP BY CTE_S.[Instance],
							CTE_S.[IdEvaluationPlan],
							CTE_S.[IdInteraction], 
							ERV.[InteractionId],
							CTE_S.[IdAgent],
							CTE_S.[AgentName],
							CTE_S.[SentimentId]'

	IF @InteractionCoached IS NOT NULL AND @InteractionCoached != ''
	BEGIN
		IF @InteractionCoached = 'true'
			SET @SQL_CTE_SELECT += '
				HAVING COUNT(ERC.[Id]) >= 1 '
		ELSE 
			SET @SQL_CTE_SELECT += '
				HAVING COUNT(ERC.[Id]) = 0 '
	END

	SET @SQL_CTE_SELECT += '
		)'


	SET @SQL_CTE_SELECT += '
		-- SELECT * FROM CTE'

	-- ARMAMOS EL QUERY PRINCIPAL
	SET @SQL_CTE = @SQL_TMP + @SQL_CTE_SELECT

	-- EMPEZAMOS A ARMAN LA QUERY PRINCIPAL
	SET @SQL_SELECT_1 = @SQL_CTE

	SET @SQL_SELECT_1 += '
		SELECT AEP.Completed								''completed'',
				AEP.[StatusDescription]						''statusDescription'',
				A.[ErrorMessage]							''errorMessage'',
    			AEP.[Score]									''qualification'',
    			AEP.[IdAuditor]								''idAuditor'',
    			U.[FirstName] + '' '' + U.[LastName]		''auditorName'',
    			MAX(ER.[EvaluationDate])					''evaluationDate'',
    			AEP.[IdEvaluationPlan]						''idEvaluationPlan'',
				EP.[Description]							''evaluationPlan'',
				EP.[Evaluation_Mode]						''evaluationType'',
    			AEP.[idAgent]								''idAgent'',
    			S_CTE.[AgentName]							''agentName'',							-- NOMBRE DEL AGENTE
				S_CTE.[IdInteraction]						''idInteraction'',
				AEP.[InteractionDate]						''interactionDate'',
    			AEP.[InteractionType]						''interactionType'',
    			AEP.[IdCampaign]							''idCampaign'',
				E.[Type]									''environmentType'',
				S_CTE.[Review]								''review'',
				S_CTE.[Commitments]							''commitments'',
				S_CTE.[SentimentId]							''sentimentResult'' '

	SET @SQL_FROM += '
			FROM SUMMARIZED_CTE S_CTE (NOLOCK) 
			INNER JOIN dbo.[EvaluationResult] ER (NOLOCK) 
				ON ER.[Instance] = S_CTE.[Instance]
					AND ER.[IdEvaluationPlan] = S_CTE.[IdEvaluationPlan]
					AND ER.[IdInteraction] = S_CTE.[IdInteraction]
					AND ER.[IdAgent] = S_CTE.[IdAgent]
			INNER JOIN dbo.[EvaluationPlan] EP (NOLOCK) 
				ON EP.[Instance] = S_CTE.[Instance] 
					AND EP.[ID] = S_CTE.[IdEvaluationPlan]
			INNER JOIN dbo.[AuditorEvaluationPlan] AEP (NOLOCK) 
				ON (AEP.[Instance] = ER.[Instance] 
					AND AEP.[IdAuditor] = ER.[IdAuditor] 
					AND AEP.[IdInteraction] = ER.[IdInteraction] 
					AND AEP.[InteractionType] = ER.[InteractionType] 
					AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]) 
			LEFT JOIN [dbo].[Audio] A (NOLOCK)
			    ON A.[VCC] = AEP.[Instance]
			        AND A.[ID] = AEP.[IdInteraction]			
			INNER JOIN [dbo].[Environment] E (NOLOCK)
				ON E.[VCC] = ER.[Instance]
					AND E.[Name] = EP.[EnvironmentId]
			INNER JOIN dbo.[User] U (NOLOCK) 
				ON U.[VCC] = ER.[Instance] 
					AND U.[ID] = AEP.[IdAuditor] '

	-- AGREGAMOS EL SCRIPT PARA DISPOSITION CODES
	IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') 
		SET @FILTERS_OPTIONAL_PART += '
				AND (
					(CONCAT(AEP.[IdCampaign], ISNULL(REPLACE(AEP.[ManagementResult], ''/'', ''_''), ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
					OR
					(CONCAT(AEP.[IdCampaign], ''_'', ISNULL(AEP.[ManagementResult], ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
					OR
					(AEP.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
				)'

	-- AGREGAMOS EL SCRIPT PARA EVALUATION TYPE
	IF (@EvaluationTypes IS NOT NULL AND @EvaluationTypes != '') 
	BEGIN
		SET @FILTERS_OPTIONAL_PART += '
				AND EP.[Evaluation_Mode] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationTypes + ''','',''))  '
	END

	-- AGREGAMOS EL SCRIPT PARA SENTIMENT
	IF @SentimentIds IS NOT NULL AND @SentimentIds <> ''
    BEGIN            
	    SET @FILTERS_OPTIONAL_PART=' 
				AND S_CTE.[SentimentId] IN (SELECT Value FROM [dbo].[split](''' + @SentimentIds + ''','','')) '
    END

	-- AGREGAMOS EL SCRIPT PARA TAG
	IF (@InteractionTags IS NOT NULL AND @InteractionTags != '') 
	BEGIN
		SET @SQL_FROM += '
			LEFT JOIN [Tag] T (NOLOCK) 
				ON (T.[VCC] = ER.[Instance] 
					AND T.[Tag] IN (SELECT Value FROM [dbo].[split](''' + @InteractionTags + ''','',''))) '
		SET @FILTERS_OPTIONAL_PART += '
				AND T.[AudioId] LIKE CONCAT(AEP.[IdInteraction], ''%'') 
				AND T.[AudioId] IS NOT NULL '
	END

	SET @SQL_EXTRA += '
			GROUP BY AEP.[Completed],
						AEP.[StatusDescription],
						A.[ErrorMessage],
						AEP.[score],
						AEP.[IdAuditor],
						U.[FirstName] + '' '' + U.[LastName],
						AEP.[IdEvaluationPlan],
						EP.[Description],
						EP.[Evaluation_Mode],
						AEP.[idAgent],
						S_CTE.[AgentName],
						AEP.[InteractionDate],
						AEP.[InteractionType],
						AEP.[IdCampaign], 
						E.[Type],
						S_CTE.[IdInteraction],
						S_CTE.[Review],
						S_CTE.[Commitments],
						S_CTE.[SentimentId]'

	SET @SQL_EXTRA += '
			ORDER BY MAX(ER.[EvaluationDate]) DESC '

	SET @SQL_SELECT_1 += @SQL_FROM  + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART + @SQL_EXTRA 


	-- ELIMINAMOS LAS TABLAS TEMPORALES
	SET @SQL_SELECT_1 += '
			DROP TABLE IF EXISTS #FILTER_AUDITORS
			DROP TABLE IF EXISTS #FILTER_AGENTS
			DROP TABLE IF EXISTS #FILTER_CAMPAIGNS 
			DROP TABLE IF EXISTS #FILTER_EVALUATION_PLANS
			DROP TABLE IF EXISTS #FILTER_INTERACTIONS_IDS '

	-- EJECUTAMOS EL QUERY ARMADO
	-- PRINT @SQL_SELECT_1
	EXEC (@SQL_SELECT_1)
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationsOfInteraction] (
	@Instance Varchar(50) = NULL,
	@IdInteraction VARCHAR(MAX) = NULL,
	@InteractionType VARCHAR(MAX) = NULL,
	@EvaluationDateFrom VARCHAR(MAX) = '1980-01-01',
	@EvaluationDateTo VARCHAR(MAX) = '2050-01-01'
	)
AS
BEGIN
	SET NOCOUNT ON;
	
	-- CREAMOS UNA CTE PARA AGRUPAR LOS RESULTADOS DE CADA INTERACCION POR PLAN
	WITH ER_CTE AS (
		SELECT ER.[Instance]															'vcc',
				ER.[IdEvaluationPlan]													'idEvaluationPlan',
				ER.[InteractionType]													'interactionType',
				ER.[IdInteraction]														'idInteraction',
				SUM(ISNULL(TRY_CAST(ER.[ScoreDescription] AS DECIMAL(18, 2)), 0))		'scoreDescription',
				--SUM(ER.[ScoreDescription])				'scoreDescription',
				--ER.[IdEvaluator]														'idEvaluator', <= DESDE PLANES MIXTOS YA NO SE PUEDE AGRUPAR EN BASE A ESTA COLUMNA
				ER.[IdAuditor]															'idAuditor',
				MAX(ER.[EvaluationDate])												'evaluationDate'
			FROM [dbo].[EvaluationResult] ER (NOLOCK)
			WHERE ER.[Instance] = @Instance
				AND ER.[IdInteraction] = @IdInteraction
				AND ER.[InteractionType] = @InteractionType
				AND ER.[EvaluationDate] BETWEEN @EvaluationDateFrom AND @EvaluationDateTo
			GROUP BY ER.[Instance], ER.[IdEvaluationPlan], ER.[InteractionType], ER.[IdInteraction], ER.[idAuditor]
	) 

	SELECT AEP.[Completed]									'completed',
			AEP.[score]										'score',
			AEP.[IdAuditor]									'idAuditor',
			U.[FirstName] + ' ' + U.[LastName]				'auditorName',
			ER.[EvaluationDate]					            'evaluationDate',
			EP.[ID]											'idEvaluationPlan',
			EP.[Description]								'evaluationPlan',
			AEP.[idAgent]									'idAgent',
			AEP.[IdInteraction]								'idInteraction',
			AEP.[InteractionType]							'interactionType'
		FROM ER_CTE ER (NOLOCK)
		INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
			ON EP.[Instance] = ER.[vcc]
				AND EP.[ID] = ER.[idEvaluationPlan]
		INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			ON AEP.[Instance] = ER.[vcc]
				AND AEP.[IdEvaluationPlan] = ER.[idEvaluationPlan]
				AND AEP.[IdInteraction] = ER.[idInteraction]
				AND AEP.[IdAuditor] = ER.[idAuditor]
				AND AEP.[InteractionType] = ER.[interactionType]
		LEFT JOIN [dbo].[User] U (NOLOCK) -- QUEDA COMO LEFT JOIN EN CASO NO ENCUENTRE EL AUDITOR AUN SERA VISIBLE EL RESULTADO
			ON U.[VCC] = EP.[Instance]
				AND U.[ID] = AEP.[IdAuditor]
		ORDER BY ER.[evaluationDate] DESC

	/*

	SELECT * 
		FROM [dbo].[EvaluationPlan] EP (NOLOCK) 
		LEFT JOIN [dbo].[EvaluationResult] ER (NOLOCK)
			ON ER.[Instance] = EP.[Instance]
				AND ER.[IdEvaluationPlan] = EP.[ID]
		LEFT JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			ON AEP.[Instance] = EP.[Instance]
				AND AEP.[IdEvaluationPlan] = EP.[ID]


				

	SELECT
		p.Completed						'completed',
		p.score							'score',
		p.IdAuditor						'idAuditor',
		a.FirstName + ' ' + a.LastName	'auditorName',
		MAX(e.EvaluationDate)             'evaluationDate',
		p.IdEvaluationPlan				'idEvaluationPlan',
		ep.Description					'evaluationPlan',
		p.idAgent						'idAgent',
		p.idInteraction					'idInteraction',
		p.InteractionType				'interactionType'
	FROM [EvaluationPlan] ep, EvaluationResult e, AuditorEvaluationPlan p, [User] a (NOLOCK)
	WHERE
	p.Instance = @Instance
	AND p.IdInteraction = @IdInteraction
	AND p.instance = e.instance
	AND p.IdInteraction = e.IdInteraction
	AND p.IdEvaluationPlan = e.IdEvaluationPlan
	AND p.IdAuditor = e.IdEvaluator
	AND p.InteractionType = e.InteractionType
	AND e.EvaluationDate >= @EvaluationDateFrom
	AND e.EvaluationDate <= @EvaluationDateTo
	AND a.ID = p.IdAuditor
	AND a.[VCC] = EP.[Instance]
	AND p.InteractionType = @InteractionType
	AND ep.Instance = e.instance
	AND ep.ID = p.IdEvaluationPlan
	GROUP BY p.Completed, 
				p.score, 
				p.IdAuditor, 
				a.FirstName + ' ' + a.LastName, 
				e.EvaluationDate, 
				p.IdEvaluationPlan, 
				ep.Description, 
				p.idAgent, 
				p.idInteraction, 
				p.InteractionType
	ORDER BY e.EvaluationDate DESC
	*/
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationTipifications] (
	@VCC VARCHAR(50) = NULL, 
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@AuditorId VARCHAR(50) = NULL
) 
AS
BEGIN
	SET NOCOUNT ON

	-- BUSCAMOS LOS RESULTADOS DE LAS TIPIFICACIONES DE 1ER NIVEL
	SELECT ET.[ScoreId]								'scoreId',
			ET.[ID]									'id',
			ET.[IsChecked]							'isChecked'
		FROM [dbo].[EvaluationTipification] ET (NOLOCK)
		WHERE ET.[VCC] = @VCC 
			AND ET.[EvaluationPlanId] = @EvaluationPlanId
			AND ET.[InteractionId] = @InteractionId
			AND ET.[AuditorId] = @AuditorId

	-- BUSCAMOS LOS RESULTADOS DE LAS TIPIFICACIONES DE 1DO Y 3ER NIVEL
	SELECT ETI.[ScoreId]							'scoreId',
			ETI.[TipificationId]					'tipificationId',
			ETI.[ID]								'id',
			ETI.[IsChecked]							'isChecked' 
		FROM [dbo].[EvaluationTipificationItem] ETI (NOLOCK)
		WHERE ETI.[VCC] = @VCC 
			AND ETI.[EvaluationPlanId] = @EvaluationPlanId
			AND ETI.[InteractionId] = @InteractionId
			AND ETI.[AuditorId] = @AuditorId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluationTypesInteraction] (
	@VCC			VARCHAR(50),
	@InteractionId	VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		CASE WHEN [MA].[evaluatedManually] > 0 THEN 1 ELSE 0 END AS evaluatedManually,
		CASE WHEN [AU].[evaluatedAutomatically] > 0 THEN 1 ELSE 0 END AS evaluatedAutomatically
	FROM
		(SELECT COUNT(*) AS evaluatedManually FROM EvaluationResult WHERE Instance = @VCC AND @InteractionId = IdInteraction AND IdAuditor <> 'SPEECH') MA,
		(SELECT COUNT(*) AS evaluatedAutomatically FROM EvaluationResult WHERE Instance = @VCC AND @InteractionId = IdInteraction AND IdAuditor = 'SPEECH') AU


	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetException] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL, @Description Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'vcc',
		[ID]							'id',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId',
		[TimeStamp]						'timeStamp'
	FROM [Exception] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [ID] =CASE WHEN @ID IS NOT NULL THEN @ID ELSE [ID] END
	AND [Description] =CASE WHEN @Description IS NOT NULL THEN @Description ELSE [Description] END
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetExceptions] (
	@VCC Varchar(50) = NULL , @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		e.VCC							'vcc',
		e.ID							'id', 
		e.Description					'description',
		e.CreatedDate					'createdDate',
		e.CreatedByUserId				'createdByUserId',
		e.LastModifiedDate  			'lastModifiedDate',
		e.LastModifiedByUserId			'lastModifiedByUserId',
		e.TimeStamp						'timeStamp'
	FROM Exception e (NOLOCK)
	WHERE e.VCC = @VCC
		AND ISNULL(e.ID, '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(e.Description, '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' COLLATE Latin1_General_CI_AI ELSE '%' END
	ORDER BY e.[Description]

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[GetExecuteTemplateProcess] 'allegro', 'Quality_fullaudios_TestTemplate', NULL
CREATE PROCEDURE [dbo].[GetExecuteTemplateProcess]
(
	@VCC					VARCHAR(50),
	@StudyId				VARCHAR(200),
	@NewStructure			VARCHAR(1)
)
AS
BEGIN
	SET NOCOUNT ON

	-- Listar detalles del Study y el Proceso.
	BEGIN
		SELECT
			B.[Name]						'processName',
			A.[Name]						'studyName',
			A.[Description]					'studyDescription'
		FROM [dbo].[Study] A (NOLOCK)
		INNER JOIN Process B ON A.ProcessId = B.[Name]
		WHERE
			A.[VCC] = @VCC AND
			A.[Name] = @StudyId
	END
	BEGIN
		-- Listar datos de interacciones Tablas: Study, ProcessAudio, Audio.
		SELECT
			PA.[ProcessId]					'processId',
			PA.[AudioId]					'interactionId',
			PA.[Section]					'section',
			PA.[Part]						'part'
		FROM [dbo].[Study] S (NOLOCK)
			INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) ON S.[VCC] = PA.[VCC] AND S.[ProcessId] = PA.[ProcessId]
		WHERE
			S.[VCC] = @VCC AND
			S.[Name] = @StudyId
		
	END
	BEGIN
		-- Listar datos de la tabla StudyResultSummary.
		SELECT
			[VCC]							'vcc',
			[StudyId]						'studyId',
			[TemplateId]					'templateId',
			[TemplateThemeId]				'templateThemeId',
			[ID]							'id',
			[Description]					'description',
			[Category1]						'category1',
			[Category2]						'category2',
			[Total]							'total',
			[Duration]						'duration',
			[Silence]						'silence',
			[Confidence]					'confidence',
			[CreatedDate]					'createdDate',
			[CreatedByUserId]				'createdByUserId',
			[LastModifiedDate]				'lastModifiedDate',
			[LastModifiedByUserId]			'lastModifiedByUserId'
		FROM [dbo].[StudyResultSummary] (NOLOCK)
		WHERE
			[VCC] = @VCC AND
			[StudyId] = @StudyId
	END
	IF(@NewStructure IS NULL)
	BEGIN
		-- Listar datos de interacciones versus queries.
		SELECT
			SRSD.[TemplateThemeId]					'templateThemeId',
			SRSD.[TemplateThemeItemId]				'templateThemeItemId',
			GQ.[Query]								'query',
			PA.[AudioId]							'transcriptionId',
			PA.[Section]							'section',
			PA.[Part]								'part',
			A.[Agent]								'agent'
		FROM 
			[dbo].[Study] S (NOLOCK)
			INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) ON S.[VCC] = PA.[VCC] AND S.[ProcessId] = PA.[ProcessId]
			INNER JOIN [dbo].[Audio] A (NOLOCK) ON PA.[VCC] = A.[VCC] AND CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part]) = CONCAT(A.[ID], '_', A.[Section], '_', A.[Part])
			INNER JOIN [dbo].[StudyResultSummaryDetail] SRSD (NOLOCK) ON PA.[VCC] = SRSD.[VCC] AND S.[Name] = SRSD.StudyId AND SRSD.[TranscriptionId] = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])
			INNER JOIN [dbo].[GenericQuery] GQ (NOLOCK) ON SRSD.[VCC] = GQ.[VCC] AND SRSD.[TemplateThemeItemId] = GQ.[Name]
		WHERE
			S.[VCC] IN ('system', @VCC) AND
			S.[Name] = @StudyId
	END
	ELSE
	BEGIN
		-- Listar datos de interacciones versus queries. Parametro NewStructure.
		SELECT
			A.[TemplateThemeId]				'templateThemeId',
			A.[TemplateThemeItemId]			'templateThemeItemId',
			D.[Query]						'query',
			A.[TranscriptionId]				'transcriptionId',
			C.[Section]						'section',
			C.[Part]						'part',
			ISNULL(C.[Agent], '')			'agent'
		FROM 
			[dbo].[StudyResultSummaryDetail] A (NOLOCK)
			INNER JOIN [dbo].[Study] S (NOLOCK) ON S.[VCC] = A.[VCC] AND S.[Name] = A.[StudyId]
			INNER JOIN [dbo].[TemplateThemeItem] B (NOLOCK) ON S.[TemplateId] = B.[TemplateId] AND A.[TemplateThemeId] = B.[TemplateThemeId] AND A.[TemplateThemeItemId] = B.[ID]
			LEFT JOIN [dbo].[Audio] C (NOLOCK) ON A.[TranscriptionId] = CONCAT(C.[ID], '_', C.[Section], '_', C.[Part])
			INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) ON S.[VCC] = PA.[VCC] AND S.[ProcessId] = PA.[ProcessId] AND A.[TranscriptionId] = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])
			INNER JOIN [dbo].[GenericQuery] D (NOLOCK) ON B.[VCC] = D.[VCC] AND  B.[ID] = D.[Name]
		WHERE
			B.[VCC] IN ('system', @VCC) AND
			A.[StudyId] = @StudyId
	END
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE[dbo].[GetExistDictionaryWordsSoundLike] 
	@VCC					VARCHAR(50) = NULL,
	@DictionaryWordId		VARCHAR(50) = NULL,
	@SoundLikes             VARCHAR(MAX) = NULL
AS
BEGIN
	SET NOCOUNT ON;
	SELECT SoundLike FROM [dbo].[DictionaryWordSoundLike] (NOLOCK)
		WHERE [VCC] = @VCC 
			AND [DictionaryWordId] != @DictionaryWordId
			AND [SoundLike] in (SELECT VALUE FROM STRING_SPLIT(@SoundLikes, ','))
   
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetFeelings] 'inconcert','',''
CREATE PROCEDURE [dbo].[GetFeelings] (
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]							'name',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Feeling] (NOLOCK)
	WHERE [VCC] = @VCC
	AND ISNULL([Name], '') LIKE CASE WHEN @SearchCriteria = 'name' THEN '%' + @SearchExpression + '%' ELSE '%' END
	AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
	ORDER BY [Name] asc, [CreatedDate] desc

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetFiltersFromEvaluation] 
(
	@VCC					VARCHAR(50) = NULL,
	@UserId					VARCHAR(50) = NULL,
	@Campaigns				VARCHAR(MAX) = NULL,
	@SearchType				VARCHAR(50) = NULL,
	@EfectivityMode			BIT = 0
) 
AS
BEGIN
	EXEC [dbo].[GetCampaignsFromEvaluation] @VCC, @UserId,@SearchType,@EfectivityMode
	EXEC [dbo].[GetAgentsFromEvaluation] @VCC, @UserId,@Campaigns,@SearchType,@EfectivityMode
	EXEC [dbo].[GetDispositionCodesFromEvaluation]  @VCC, @UserId,@Campaigns,@SearchType,@EfectivityMode
	EXEC [dbo].[GetInteractionTypes] @VCC, @SearchType, null	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetFinishedBatch] 
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE RETORNAR LOS BATCH QUE HAN SIDO FINALIZADOS HACE MAS DE 5 MINUTOS
	SET NOCOUNT ON

	SELECT TOP 25 B.[VCC]					'vcc',
					B.[BatchTypeName]		'batchTypeName',
					B.[BatchTypeId]			'batchTypeId',
					B.[ClosedDate]			'closedDate'
		FROM [dbo].[Batch] B (NOLOCK)
		WHERE B.[IsFinished] = 1 
			AND B.[FinishReported] = 0
			AND B.[LastModifiedDate] < DATEADD(MINUTE, -5, GETUTCDATE())
			AND B.[CreatedDate] > DATEADD(DAY, -5, GETUTCDATE())
			--AND DATEADD(MINUTE, 5, B.[LastModifiedDate]) < GETUTCDATE()
			--AND B.[CreatedDate] > DATEADD(DAY, -5, GETUTCDATE())
		ORDER BY B.[CreatedDate] DESC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGenerative] (
    @VCC VARCHAR(50) = NULL,
    @SearchCriteria VARCHAR(50) = NULL,
    @SearchExpression VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
   
   -- BUSCAMOS EL MODELO POR EL DATO RECIBIDO
	SELECT [VCC]					'vcc',
			[Name]					'name',
			[AppId]					'id',
			[Description]			'description',
			[Language]				'language',
			[IsPublished]			'isPublished',
			[NeedPublish]			'needPublish',
			[LastPublishedDate]		'lastPublishedDateTime',
			[StatusCode]			'status',
			[CreatedDate]			'createdDate',
			[CreatedByUserId]		'createdByUserId',
			[LastModifiedDate]		'lastModifiedDate',
			[LastModifiedByUserId]	'lastModifiedByUserId'
		FROM [dbo].[Generative] (NOLOCK)
		WHERE [VCC] = @VCC
			AND [AppId] = CASE WHEN @SearchCriteria = 'id' THEN @SearchExpression ELSE [AppId] END
			AND [Name] = CASE WHEN @SearchCriteria = 'name' THEN @SearchExpression ELSE [Name] END

	-- SI LA BUSQUEDA SE HIZO POR ID, BUSCAMOS LOS INTENTS Y EXAMPLES
	IF @SearchCriteria = 'id'
	BEGIN
		EXEC [dbo].[GetGenerativeIntents] @VCC, 'appId', @SearchExpression, NULL
		EXEC [dbo].[GetGenerativeExamples] @VCC, 'appId', @SearchExpression
	END

	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGenerativeEngineToken] (
	@VCC VARCHAR(50) = NULL
) AS
BEGIN
	-- OBTENEMOS EL TOKEN USANDO EL VCC
	SELECT [Token]				'token'
		FROM [dbo].[GenerativeEngineToken] (NOLOCK)
		WHERE [VCC] = @VCC

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGenerativeExamples] (
    @VCC VARCHAR(50) = NULL,
    @SearchCriteria VARCHAR(50) = NULL,
    @SearchExpression VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
   
	-- HACEMOS LA BUSQUEDA SEGUN EL CRITERIO RECIBIDO
	SELECT [Id]						'id',
			[AppId]					'appId',
			[IntentId]				'intentId',
			[Text]					'text',
			[StatusCode]			'status'
		FROM [dbo].[GenerativeExample] (NOLOCK)
		WHERE [VCC] = @VCC
			AND [AppId] = CASE WHEN @SearchCriteria = 'appId' THEN @SearchExpression ELSE [AppId] END
			AND [IntentId] = CASE WHEN @SearchCriteria = 'intentId' THEN @SearchExpression ELSE [IntentId] END
		ORDER BY [text] ASC

	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGenerativeIntentItems] (
    @VCC VARCHAR(50) = NULL,
    @SearchCriteria VARCHAR(50) = NULL,
    @SearchExpression VARCHAR(50) = NULL,
    @IntentIds NVARCHAR(MAX) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- HACEMOS LA BUSQUEDA SEGUN EL CRITERIO RECIBIDO
	SELECT G.[AppId]					'appId',
			G.[Name]					'appName',
			G.[Language]				'culture',
			GI.[Id]						'id',
			GI.[Name]					'name'
		FROM [dbo].[GenerativeIntent] GI (NOLOCK)
		INNER JOIN [dbo].[Generative] G (NOLOCK)
			ON G.[VCC] = GI.[VCC]
				AND G.[AppId] = GI.[AppId]
		WHERE GI.[VCC] = @VCC
			AND GI.[AppId] = CASE WHEN @SearchCriteria = 'appId' THEN @SearchExpression ELSE GI.[AppId] END
			AND GI.[Id] = CASE WHEN @SearchCriteria = 'intentId' THEN @SearchExpression ELSE GI.[Id] END
			AND G.[IsPublished] = 1
		ORDER BY G.[Name] ASC, GI.[Name] ASC
		
	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGenerativeIntents] (
    @VCC VARCHAR(50) = NULL,
    @SearchCriteria VARCHAR(50) = NULL,
    @SearchExpression VARCHAR(50) = NULL,
    @IntentIds NVARCHAR(MAX)
)
AS
BEGIN
	SET NOCOUNT ON

	/*
	-- HACEMOS LA BUSQUEDA SEGUN EL CRITERIO RECIBIDO
	SELECT [AppId]					'appId',
			[Id]					'id',
			[Name]					'name',
			[Description]			'description'
		FROM [dbo].[GenerativeIntent] (NOLOCK)
		WHERE [VCC] = @VCC
			AND ISNULL([AppId], '''') = CASE WHEN @SearchCriteria = 'appId' THEN @SearchExpression ELSE [AppId] END
			AND ISNULL([Name], '''') = CASE WHEN @SearchCriteria = 'name' THEN @SearchExpression ELSE [Name] END
		ORDER BY [LastModifiedDate] ASC, [Name] DESC

	SET NOCOUNT OFF
	*/

	
	DECLARE @SQL NVARCHAR(MAX)
	SET @SQL = '
	SET NOCOUNT ON
	SELECT [AppId]								''appId'',
			[Id]								''id'',
			[Name]								''name'',
	        [Description]						''description''
		FROM [dbo].[GenerativeIntent] (NOLOCK) 
		WHERE [VCC] = ''' + @VCC + ''' '

	-- AGREGAMOS LOS FILTROS SEGUN LOS CRITERIOS
	IF @SearchCriteria IS NOT NULL AND @SearchExpression IS NOT NULL
	BEGIN
		IF @SearchCriteria = 'appId' 
			SET @SQL += ' 
			AND ISNULL([AppId], '''') = ''' + @SearchExpression + ''' '
		IF @SearchCriteria = 'name' 
			SET @SQL += ' 
			AND ISNULL([Name], '''') = ''' + @SearchExpression + ''' '
	END

	-- BUSCAMOS POR IDS EN CASO EXISTE ESE INPUT
	IF ISNULL(@IntentIds, '') <> ''
		SET @SQL += ' 
			AND [ID] IN (SELECT Value FROM [dbo].[split](''' + @IntentIds + ''','',''))'

	SET @SQL += '
		ORDER BY [AppId]'

	SET @SQL += ' 
	SET NOCOUNT OFF'
	
	--print @SQL
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGeneratives] (
    @VCC VARCHAR(50) = NULL,
    @SearchCriteria VARCHAR(50) = NULL,
    @SearchExpression VARCHAR(50) = NULL,
    @AppIds NVARCHAR(MAX)
)
AS
BEGIN
	SET NOCOUNT ON
   
	DECLARE  @SQL              NVARCHAR(MAX)
	DECLARE  @FILTER           VARCHAR(MAX) = ''
    DECLARE  @UNION_SQL        VARCHAR(MAX) = ''
    DECLARE  @APPS_FILTER      VARCHAR(MAX) = ''
    DECLARE  @STATUS_FILTER    VARCHAR(MAX) = ''
    DECLARE  @ORDER_BY         VARCHAR(MAX) = ''
	DECLARE  @FILTER_COGNITIVEENGINE VARCHAR(MAX) = ''
    
	SET @SQL = '
    SELECT [VCC]					''vcc'',
			[Name]					''name'',
			[AppId]					''id'',
			[Description]			''description'',
			[IsPublished]			''isPublished'',
			[NeedPublish]			''needPublish'',
			[LastPublishedDate]		''lastPublishedDateTime'',
			[StatusCode]			''status'',
			[CreatedDate]			''createdDate'',
			[CreatedByUserId]		''createdByUserId'',
			[LastModifiedDate]		''lastModifiedDate'',
			[LastModifiedByUserId]	''lastModifiedByUserId''
		FROM [dbo].[Generative] (NOLOCK)
		WHERE [VCC] = ''' + @VCC + ''' '

	IF (
		@SearchCriteria IS NOT NULL AND @SearchCriteria <> '' 
		AND 
		@SearchExpression IS NOT NULL AND @SearchExpression <> ''
	)
    BEGIN
        SET @FILTER += '
			AND ISNULL([Name], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
			AND ISNULL([Description], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''description'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
			AND ISNULL([LastModifiedByUserId], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''LastModifiedByUserId'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END'
    END

    SET @ORDER_BY = ' ORDER BY [LastModifiedDate] ASC, [Name] DESC'

    SET @SQL += @FILTER + @APPS_FILTER + @STATUS_FILTER + @FILTER_COGNITIVEENGINE + @UNION_SQL +  @ORDER_BY
    
	EXEC(@SQL)

	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGenericQueries]
(
	@VCC								VARCHAR(50) = NULL,
	@SearchCriteria						VARCHAR(50) = NULL,
	@SearchExpression					VARCHAR(50) = NULL,
	@StartDate							DATETIME,
	@EndDate							DATETIME
)
AS
BEGIN

	DECLARE @CURRENT_LANGUAGE varchar(2)
	SELECT @CURRENT_LANGUAGE = [language] from [dbo].[VCC] where [VCC] =@VCC
	
	
	DECLARE @SQL NVARCHAR(MAX)
	DECLARE @SearchExpressionFormat NVARCHAR(MAX)
	
	SET @SQL = 'SET NOCOUNT ON'
	SET @SearchExpressionFormat = REPLACE(@SearchExpression,'_','\_')
	
	IF(@SearchExpression = '')
	BEGIN
		SET @SQL += '
			SELECT GQ.[VCC]							''vcc'',
				GQ.[Name]							''name'',
				GQ.[Description]					''description'',
				GQ.[Type]							''type'',
				GQ.[Query]							''query'',
				GQ.[RawData]						''rawData'',
				GQ.[CreatedDate]					''createdDate'',
				GQ.[CreatedByUserId]				''createdByUserId'',
				GQ.[LastModifiedDate]				''lastModifiedDate'',
				GQ.[LastModifiedByUserId]			''lastModifiedByUserId'',
				GQ.[Version]                        ''version'' 
				FROM GENERICQUERY (NOLOCK) GQ
				-- INNER JOIN ( SELECT * FROM GenericQuery (NOLOCK) WHERE VCC =''SYSTEM'' AND [Name] LIKE ' +''''+ @CURRENT_LANGUAGE + '%'+''') AS GS
				INNER JOIN ( 
					SELECT * FROM GenericQuery (NOLOCK) 
						WHERE VCC =''SYSTEM'' 
							AND [Name] LIKE ' + '''' + @CURRENT_LANGUAGE + '%' + '''
							AND ISNULL([Name], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
							AND ISNULL([Description], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''description'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
							AND ISNULL([Query], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''query'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
				) AS GS 
					ON GQ.[Name] = GS.[Name]	

			WHERE  GQ.[VCC]='''+@VCC+''' 

			UNION ALL 
		'
	END
	SET @SQL += '
	SELECT
		[VCC]							''vcc'',
		[Name]							''name'',
		[Description]					''description'',
		[Type]							''type'',
		[Query]							''query'',
		[RawData]						''rawData'',
		[CreatedDate]					''createdDate'',
		[CreatedByUserId]				''createdByUserId'',
		[LastModifiedDate]				''lastModifiedDate'',
		[LastModifiedByUserId]			''lastModifiedByUserId'',
		[Version]                       ''version''
	FROM
		[dbo].[GenericQuery] (NOLOCK)
	WHERE
		[VCC] = ''' + @VCC + '''
		AND ISNULL([Name], '''') LIKE (CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpressionFormat + '%'' ELSE ''%'' END) ESCAPE ''\''
		AND ISNULL([Description], '''') LIKE (CASE WHEN ''' + @SearchCriteria + ''' = ''description'' THEN ''%' + @SearchExpressionFormat + '%'' ELSE ''%'' END) ESCAPE ''\''
		AND ISNULL([Query], '''') LIKE (CASE WHEN ''' + @SearchCriteria + ''' = ''query'' THEN ''%' + @SearchExpressionFormat + '%'' ELSE ''%'' END) ESCAPE ''\''
		AND ISNULL([Type], '''') LIKE (CASE WHEN ''' + @SearchCriteria + ''' = ''execution_type'' THEN ''%' + @SearchExpressionFormat + '%'' ELSE ''%'' END) ESCAPE ''\''
		AND [NAME] NOT IN (SELECT [NAME] FROM [dbo].[GenericQuery] WHERE VCC =''SYSTEM'' AND [Name] NOT LIKE ' +''''+ @CURRENT_LANGUAGE + '%'+''')
		'
	
	IF(@StartDate IS NOT NULL AND @EndDate IS NOT NULL)
	BEGIN
		SET @SQL += '
			--AND CONVERT(VARCHAR(10), [CreatedDate], 120) BETWEEN ''' + CONVERT(VARCHAR(10), @StartDate, 120) + ''' AND ''' + CONVERT(VARCHAR(10), @EndDate, 120) + '''
			AND [CreatedDate] BETWEEN ''' + CONVERT(VARCHAR(10), @StartDate, 120) + ''' AND ''' + CONVERT(VARCHAR(10), @EndDate, 120) + '''
			
		'
	END
	SET @SQL += '
	ORDER BY [CreatedDate] DESC
	'
	SET NOCOUNT OFF
	-- PRINT @SQL
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGenericQuery]
(
	@VCC							VARCHAR(50) = NULL,
	@Name							VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]						'vcc',
		[Name]						'name',
		[Description]				'description',
		[Type]						'type',
		[Query]						'query',
		[RawData]					'rawData',
		[CreatedDate]				'createdDate',
		[CreatedByUserId]			'createdByUserId',
		[LastModifiedDate]			'lastModifiedDate',
		[LastModifiedByUserId]		'lastModifiedByUserId',
		[Version]                   'version'
	FROM
		[dbo].[GenericQuery] (NOLOCK)
	WHERE
		[VCC] = @VCC
		AND [Name] = @Name

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGenericQueryUsage]
(
	@VCC				VARCHAR(50),
	@QueryName			VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	-- Validar para Ability.
	BEGIN
		SELECT
			DISTINCT [Title]	'ability'
		FROM
			[dbo].[Ability]
		WHERE
			[Instance] = @VCC
			AND [QueryName] = @QueryName
	END

	-- Validar para TemplateThemeItem.
	BEGIN
		SELECT
			COUNT(ID) total
		FROM
			[dbo].[TemplateThemeItem]
		WHERE
			[VCC] IN(@VCC, 'system')
			AND [ID] = @QueryName
	END

	-- Validar para EvalutionPlan.
	BEGIN
		SELECT
			DISTINCT [Description]	'description'
		FROM
			[dbo].[EvaluationPlan]
		WHERE
			[Instance] = @VCC
			AND [FilterByContentQueryName] = @QueryName
			AND [Status] < 5
	END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGenericQueryValidate]
(
	@VCC							VARCHAR(50) = NULL,
	@Name							VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT DISTINCT
		[VCC]						'vcc',
		[Name]						'name',
		[Description]				'description',
		[Type]						'type',
		[Query]						'query',
		[RawData]					'rawData',
		[CreatedDate]				'createdDate',
		[CreatedByUserId]			'createdByUserId',
		[LastModifiedDate]			'lastModifiedDate',
		[LastModifiedByUserId]		'lastModifiedByUserId',
		[Version]                   'version'
	FROM
		[dbo].[GenericQuery] (NOLOCK)
	WHERE
		[VCC] = @VCC
		AND [Name] = @Name

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetGenericTemplate] 'inconcert','CE2C05F1-9EED-4D46-9E8F-4CB5E04EC406'
CREATE PROCEDURE [dbo].[GetGenericTemplate] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]						    'name',
		[Type]							'type',
		[Description]					'description',
		[Template]						'template',
		[Category]						'category',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Template] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [Name] = @Name

	EXEC [GetGenericTemplateQueries] @VCC, @Name
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetGenericTemplateQueries] 'inconcert',''
CREATE PROCEDURE [dbo].[GetGenericTemplateQueries] (
	@VCC Varchar(50) = NULL, @TemplateId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[TemplateId]					'templateId',
		[ID]							'id',
		[Query]							'query',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [GenericTemplateQuery] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [TemplateId] = CASE WHEN ISNULL(@TemplateId, '') = '' THEN [TemplateId] ELSE @TemplateId END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGlobalSearchFilters]
(
	@VCC	VARCHAR(50) = NULL
)
AS 
BEGIN
	SET NOCOUNT ON
	
	--LISTAR CAMPANIAS DISPONIBLES EN LAS TRES TABLAS PRINCIPALES
	SELECT DISTINCT [IdCampaign] 'campaignId',[IdCampaign] 'campaign' 
		FROM [dbo].[InteractionEvaluationPlan] IEP 
		INNER JOIN [dbo].[EvaluationPlan] EP 
			ON IEP.[Instance] = EP.[Instance]
				AND IEP.[IdEvaluationPlan] = EP.[Id]
		WHERE IEP.[Instance] = @VCC	
			AND EP.[Evaluation_Mode] = 'M'
			AND [IdCampaign] IS NOT NULL
	UNION
	
	SELECT DISTINCT [Campaign],[Campaign] 
		FROM [dbo].[Audio]
		WHERE [VCC] = @VCC	
			AND [Campaign] IS NOT NULL
	UNION 

	SELECT DISTINCT [Campaign],[Campaign] 
		FROM [dbo].[AsyncInteraction]
		WHERE [VCC] = @VCC	
			AND [Campaign] IS NOT NULL

	--LISTAR AGENTES DISPONIBLES EN LAS TRES TABLAS PRINCIPALES 
	SELECT DISTINCT [IdAgent] 'agentId',[AgentName] 'agent' 
		FROM [dbo].[InteractionEvaluationPlan] IEP 
		INNER JOIN [dbo].[EvaluationPlan] EP 
			ON IEP.[Instance] = EP.[Instance]
				AND IEP.[IdEvaluationPlan] = EP.[Id]
		WHERE IEP.[Instance] = @VCC	
			AND EP.[Evaluation_Mode] = 'M'	
			AND [IdCampaign] IS NOT NULL
			AND [IdAgent] IS NOT NULL
			AND [AgentName] IS NOT NULL

	UNION
	
	SELECT DISTINCT [Agent],[Agent] 
		FROM [dbo].[Audio]
		WHERE [VCC] = @VCC	
			AND [Agent] IS NOT NULL
	UNION 

	SELECT DISTINCT [FirstTakedAgent],[FirstTakedAgent] 
		FROM [dbo].[AsyncInteraction]
		WHERE [VCC] = @VCC	
			AND [FirstTakedAgent] IS NOT NULL

	--LISTAR ORIGEN DE LAS INTERACCIONES
	SELECT 'Client' AS originId, 'Client' origin

	UNION

	SELECT 'Agent' AS originId, 'Agent' origin

	--LISTAR TIPIFICACIONES  
	SELECT DISTINCT [ManagementResult] 'dispositionCodeId',[ManagementResult] 'dispositionCode' 
		FROM [dbo].[InteractionEvaluationPlan] IEP
		INNER JOIN [dbo].[EvaluationPlan] EP 
			ON IEP.[Instance] = EP.[Instance]
			AND IEP.[IdEvaluationPlan] = EP.[Id]
		WHERE IEP.[Instance] = @VCC	
			AND EP.[Evaluation_Mode] = 'M'	
			AND [ManagementResult] IS NOT NULL

	UNION

	SELECT [Description],  [Description]  
		FROM [dbo].[Audio]
		WHERE VCC = @VCC
			AND [Description] IS NOT NULL
	UNION

	SELECT DISTINCT [DispositionCode],[DispositionTreePath] 
		FROM [dbo].[AsyncInteraction]
		WHERE [VCC] = @VCC
			AND  [DispositionCode] IS NOT NULL
			AND [DispositionTreePath] IS NOT  NULL

	--LISTAR CANALES DE LAS INTERACCIONES
	SELECT DISTINCT [InteractionType] 'interactionTypeId',[InteractionType] 'interactionType' 
		FROM dbo.[InteractionEvaluationPlan] IEP
		INNER JOIN [dbo].[EvaluationPlan] EP 
			ON IEP.[Instance] = EP.[Instance]
				AND IEP.[IdEvaluationPlan] = EP.[Id]
		WHERE IEP.[Instance] = @VCC	
			AND EP.[Evaluation_Mode] = 'M'
	
	UNION

	SELECT DISTINCT [Channel], [Channel] 
		FROM  [dbo].[AUDIO]
		WHERE [VCC] = @VCC

	UNION

	SELECT DISTINCT [Channel], [Channel] 
		FROM  [dbo].[AsyncInteraction]
		WHERE [VCC] = @VCC

	
	SET NOCOUNT OFF
	
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGlobalSearchInteractions] (
	@VCC Varchar(50) = NULL,
	@Agents Varchar(MAX) = NULL,
   	@Campaigns VARCHAR(MAX) = NULL,
 	@InteractionDateFrom VARCHAR(50) = '1980-01-01T00:00:00',
    @InteractionDateTo VARCHAR(50) = '2050-01-01T23:59:59',	
    @InteractionOrigins Varchar(50) = NULL,	
	@InteractionTypes Varchar(50) = NULL,
	@InteractionIds     VARCHAR(MAX) = NULL,
	@ContactAddress Varchar(300) = NULL,
	@DispositionCodes VARCHAR(MAX)
	
	
)
AS
BEGIN
	SET NOCOUNT ON	
	
	--LISTAMOS INTERACCIONES ENCONTRADAS CON LOS  FILTROS BASICOS
	
	--INTERACCIONES DE PLANES MANUALES
	--SELECT * FROM InteractionEvaluationPlan
	CREATE TABLE #AllInteractionEvaluationPlan
	(
		[interactionId]				VARCHAR(250),
		[interactionSection]		INT,
		[interactionPart]			INT,	
		[interactionType]			VARCHAR(50),
		[interactionDateStart]		DATETIME,
		[interactionDateEnd]		DATETIME,
		[agentId]					VARCHAR(50),
		[campaignId]				VARCHAR(50),
		[interactionOrigin]			VARCHAR(50),
		[contactAddress]			VARCHAR(50),
		[interactionDuration]		INT,
		[evaluationPlanId]			VARCHAR(50),
		[processId]						VARCHAR(50),
		[evaluation_Mode]			VARCHAR(50),
		RN							INT
		
	)
	IF (@InteractionIds IS NULL OR  @InteractionIds = '')
	BEGIN
		INSERT INTO #AllInteractionEvaluationPlan
			SELECT DISTINCT
					IEP.[IdInteraction]				'interactionId',
					IEP.[InteractionSection]		'interactionSection',
					IEP.[InteractionPart]			'interactionPart',	
					IEP.[InteractionType]			'interactionType',
					IEP.[InteractionDateStart]		'interactionDateStart',
					IEP.[InteractionDateEnd]		'interactionDateEnd',
					IEP.[IdAgent]					'agentId',
					IEP.[IdCampaign]				'campaignId',
					IEP.[InteractionOrigin]			'originId',
					IEP.[ContactAddress]			'contactAddress',
					IEP.[InteractionDuration]		'duration',
					IEP.[IdEvaluationPlan]			'evaluationPlanId',
					PA.[ProcessId]					'processId',
					EP.[Evaluation_Mode]			'evaluation_mode',
					RN = ROW_NUMBER() OVER(PARTITION BY IEP.[Instance], IEP.[IdInteraction] ORDER BY IEP.[CreatedDate] DESC)
			FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
			INNER JOIN dbo.[EvaluationPlan] EP (NOLOCK)
				ON IEP.[Instance] = EP.[Instance]
					AND IEP.[IdEvaluationPlan] = EP.[ID]
			LEFT JOIN [dbo].[ProcessAudio] PA (NOLOCK)
				ON PA.[VCC] = IEP.[Instance]
					AND PA.[AudioId] = IEP.[IdInteraction]
			WHERE IEP.[Instance] = @VCC
				AND	(
						ISNULL(LEN(@Agents), 0) = 0
						OR
						[IdAgent]  IN (SELECT Value FROM [dbo].[split](@Agents, ','))
					)
	
				AND (
						ISNULL(LEN(@Campaigns), 0) = 0
						OR
						[IdCampaign]  IN (SELECT Value FROM [dbo].[split](@Campaigns, ','))
					)
	
				AND (
						ISNULL(LEN(@InteractionOrigins), 0) = 0
						OR
						[InteractionOrigin]  IN (SELECT Value FROM [dbo].[split](@InteractionOrigins, ','))
					)

				AND	(
						ISNULL(LEN(@InteractionTypes), 0) = 0
						OR
						[InteractionType]  IN (SELECT Value FROM [dbo].[split](@InteractionTypes, ','))
					)	

	

				AND (
						ISNULL(LEN(@ContactAddress), 0) = 0
						OR
						[ContactAddress] LIKE '%' + @ContactAddress + '%'
					)
    
				AND (
						ISNULL(LEN(@DispositionCodes), 0) = 0
						OR
						[ManagementResult] IN (SELECT Value FROM [dbo].[split](@DispositionCodes, ','))
					)
				AND (
						IEP.InteractionDateEnd BETWEEN @InteractionDateFrom AND @InteractionDateTo
					)

					
	END
	ELSE 
	BEGIN
		INSERT INTO #AllInteractionEvaluationPlan
			SELECT DISTINCT
			IEP.[IdInteraction]				'interactionId',
			IEP.[InteractionSection]		'interactionSection',
			IEP.[InteractionPart]			'interactionPart',	
			IEP.[InteractionType]			'interactionType',
			IEP.[InteractionDateStart]		'interactionDateStart',
			IEP.[InteractionDateEnd]		'interactionDateEnd',
			IEP.[IdAgent]					'agentId',
			IEP.[IdCampaign]				'campaignId',
			IEP.[InteractionOrigin]			'originId',
			IEP.[ContactAddress]			'contactAddress',
			IEP.[InteractionDuration]		'duration',
			IEP.[IdEvaluationPlan]			'evaluationPlanId',
			NULL							'processId',
			EP.[Evaluation_Mode]			'evaluation_mode',
			
			RN = ROW_NUMBER() OVER(PARTITION BY IEP.[Instance],IEP.[IdInteraction] ORDER BY IEP.[CreatedDate] DESC)

		
			FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				INNER JOIN dbo.[EvaluationPlan] EP (NOLOCK)
				 ON IEP.[Instance] = EP.[Instance]
					AND IEP.[IdEvaluationPlan] = EP.[ID]

			WHERE IEP.[Instance] = @VCC			
					AND [IdInteraction]  IN (SELECT Value FROM [dbo].[split](@InteractionIds, ','))				
		
	END

		DROP TABLE IF EXISTS #DistinctInteractionEvaluationPlan
	    SELECT [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode]
			INTO  #DistinctInteractionEvaluationPlan
			FROM #AllInteractionEvaluationPlan 
			WHERE RN = 1

	
	--INTERACCIONES DE CALL DE PLANES AUTOMATICOS QUE AUN NO EXISTAN EN EL PRIMER LISTADO

	CREATE TABLE #AllInteractionProcess
	(
		[interactionId]				VARCHAR(250),
		[interactionSection]		INT,
		[interactionPart]			INT,	
		[interactionType]			VARCHAR(50),
		[interactionDateStart]		DATETIME,
		[interactionDateEnd]		DATETIME,
		[agentId]					VARCHAR(50),
		[campaignId]				VARCHAR(50),
		[interactionOrigin]			VARCHAR(50),
		[contactAddress]			VARCHAR(50),
		[interactionDuration]		INT,
		[evaluationPlanId]			VARCHAR(50),
		[processId]						VARCHAR(50),
		[evaluation_Mode]			VARCHAR(50),
		RN							INT
		
	)

	IF (@InteractionIds IS NULL OR  @InteractionIds = '')
	BEGIN
		INSERT INTO #AllInteractionProcess
			SELECT DISTINCT
			A.[ID]					'interactionId',
			A.[Section]				'interactionSection',
			A.[Part]				'interactionPart',	
			A.[Channel]				'interactionType',
			A.[StartDate]			'interactionDateStart',
			A.[EndDate]				'interactionDateEnd',
			A.[Agent]				'agentId',
			A.[Campaign]			'campaignId',
			A.[Type]				'originId',
			A.[ContactAddress]		'contactAddress',
			A.[DurationTime]		'duration',
			NULL					'evaluationPlanId',
			PA.[ProcessId]			'processId',
			'A'						'evaluation_mode',
			RN = ROW_NUMBER() OVER(PARTITION BY A.[VCC],A.[ID] ORDER BY A.[CreatedDate] DESC)
	

			FROM dbo.[Audio] A (NOLOCK)
			INNER JOIN dbo.[ProcessAudio] PA (NOLOCK)
				ON  A.[VCC] = PA.[VCC]
					AND A.[ID] = PA.[AudioId] 
					AND A.[Section] = PA.[Section]
					AND A.[Part] = PA.[Part]

			WHERE A.[VCC] = @VCC 
				AND  A.[IsTranscribed] = 1
				AND	(
						ISNULL(LEN(@Agents), 0) = 0
						OR
						[Agent]  IN (SELECT Value FROM [dbo].[split](@Agents, ','))
					)
	
				AND (
						ISNULL(LEN(@Campaigns), 0) = 0
						OR
						[Campaign]  IN (SELECT Value FROM [dbo].[split](@Campaigns, ','))
					)
		
				AND (
						ISNULL(LEN(@InteractionOrigins), 0) = 0
						OR
						[Type]  IN (SELECT Value FROM [dbo].[split](@InteractionOrigins, ','))
					)

				AND	(
						ISNULL(LEN(@InteractionTypes), 0) = 0
						OR
						[channel]  IN (SELECT Value FROM [dbo].[split](@InteractionTypes, ','))
					)	

				--AND (
				--		ISNULL(LEN(@InteractionIds), 0) = 0
				--		OR
				--		[Id]  IN (SELECT Value FROM [dbo].[split](@InteractionIds, ','))
				--	)

				AND (
						ISNULL(LEN(@ContactAddress), 0) = 0
						OR
						[ContactAddress] LIKE '%' + @ContactAddress + '%'
					)
		
				AND (
						ISNULL(LEN(@DispositionCodes), 0) = 0
						OR
						[Description] IN (SELECT Value FROM [dbo].[split](@DispositionCodes, ','))
					)

				AND (
						A.EndDate BETWEEN @InteractionDateFrom AND @InteractionDateTo
					)

				AND  [Id] NOT IN (SELECT interactionId FROM #DistinctInteractionEvaluationPlan)
					
				
				
	END		
	ELSE
	BEGIN
		INSERT INTO #AllInteractionProcess
			SELECT DISTINCT
			A.[ID]					'interactionId',
			A.[Section]				'interactionSection',
			A.[Part]				'interactionPart',	
			A.[Channel]				'interactionType',
			A.[StartDate]			'interactionDateStart',
			A.[EndDate]				'interactionDateEnd',
			A.[Agent]				'agentId',
			A.[Campaign]			'campaignId',
			A.[Type]				'originId',
			A.[ContactAddress]		'contactAddress',
			A.[DurationTime]		'duration',
			NULL					'evaluationPlanId',
			PA.[ProcessId]			'processId',
			'A'						'evaluation_mode',
			RN = ROW_NUMBER() OVER(PARTITION BY A.[VCC],A.[ID] ORDER BY A.[CreatedDate] DESC)
	

			FROM dbo.[Audio] A (NOLOCK)
			INNER JOIN dbo.[ProcessAudio] PA (NOLOCK)
				ON  A.[VCC] = PA.[VCC]
					AND A.[ID] = PA.[AudioId] 
					AND A.[Section] = PA.[Section]
					AND A.[Part] = PA.[Part]

			WHERE A.[VCC] = @VCC
					AND A.[IsTranscribed] = 1
					AND A.[ID]  IN (SELECT Value FROM [dbo].[split](@InteractionIds, ','))	
					AND A.[ID] NOT IN (SELECT interactionId FROM #DistinctInteractionEvaluationPlan)
					
					
				
				
	END

	DROP TABLE IF EXISTS #DistinctInteractionProcess
	    SELECT [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode]
			INTO  #DistinctInteractionProcess
			FROM #AllInteractionProcess WHERE RN = 1	

	--LISTAMOS INTERACCIONES DE TEXTO DE PLANES AUTOMATICOS QUE AUN NO EXISTAN EN UN PLAN DE EVALUACION 
	CREATE TABLE #AllTextInteractionProcess
	(
		[interactionId]				VARCHAR(250),
		[interactionSection]		INT,
		[interactionPart]			INT,	
		[interactionType]			VARCHAR(50),
		[interactionDateStart]		DATETIME,
		[interactionDateEnd]		DATETIME,
		[agentId]					VARCHAR(50),
		[campaignId]				VARCHAR(50),
		[interactionOrigin]			VARCHAR(50),
		[contactAddress]			VARCHAR(50),
		[interactionDuration]		INT,
		[evaluationPlanId]			VARCHAR(50),
		[processId]						VARCHAR(50),
		[evaluation_Mode]			VARCHAR(50),
		RN							INT
		
	)
	IF (@InteractionIds IS NULL OR  @InteractionIds = '')
	BEGIN
		INSERT INTO #AllTextInteractionProcess
			SELECT DISTINCT
			ASY.[ID]				'interactionId',
			0						'interactionSection',
			0						'interactionPart',
			ASY.[Channel]			'interactionType',
			ASY.[StartDate]			'interactionDateStart',
			ASY.[EndDate]			'interactionDateEnd',
			ASY.[FirstTakedAgent]	'agentId',
			ASY.[Campaign]			'campaignId',
			ASY.[Initiative]		'originId',
			ASY.[ContactAddress]	'contactAddress',
			ASY.[DurationTime]		'duration',
			NULL					'evaluationPlanId',
			PASY.[ProcessId]		'processId',
			'A'						'evaluation_mode',
			RN = ROW_NUMBER() OVER(PARTITION BY ASY.[VCC],ASY.[ID] ORDER BY ASY.[CreatedDate] ASC)


			FROM dbo.[AsyncInteraction] ASY (NOLOCK)
			INNER JOIN dbo.[ProcessAsyncInteraction] PASY  (NOLOCK)
				ON ASY.[VCC] = PASY.[VCC]
					AND ASY.[ID] = PASY.[InteractionId]
					AND ASY.[EndDate] = PASY.[EndDate]


			WHERE ASY.[VCC] = @VCC	
				AND ASY.[IsTranscribed] = 1
				AND	(
						ISNULL(LEN(@Agents), 0) = 0
						OR
						[FirstTakedAgent]  IN (SELECT Value FROM [dbo].[split](@Agents, ','))
					)
	
				AND	(
						ISNULL(LEN(@Campaigns), 0) = 0
						OR
						ASY.[Campaign]  IN (SELECT Value FROM [dbo].[split](@Campaigns, ','))
					)
		
				AND (
						ISNULL(LEN(@InteractionOrigins), 0) = 0
						OR
						[Initiative]  IN (SELECT Value FROM [dbo].[split](@InteractionOrigins, ','))
					)

				AND	(
						ISNULL(LEN(@InteractionTypes), 0) = 0
						OR
						[channel]  IN (SELECT Value FROM [dbo].[split](@InteractionTypes, ','))
					)	

				--AND (
				--		ISNULL(LEN(@InteractionIds), 0) = 0
				--		OR
				--		[Id]  IN (SELECT Value FROM [dbo].[split](@InteractionIds, ','))
				--	)

				AND (
						ISNULL(LEN(@ContactAddress), 0) = 0
						OR
						[ContactAddress] LIKE '%' + @ContactAddress + '%'
					)
		
				AND (
						ISNULL(LEN(@DispositionCodes), 0) = 0
						OR
						[DispositionTreePath] IN (SELECT Value FROM [dbo].[split](@DispositionCodes, ','))
					)

				AND ASY.EndDate BETWEEN @InteractionDateFrom AND @InteractionDateTo
					
				AND ASY.[ID] NOT IN (SELECT interactionId FROM #DistinctInteractionEvaluationPlan)

				
	END
	ELSE
	BEGIN
		INSERT INTO #AllTextInteractionProcess
			SELECT DISTINCT
			ASY.[ID]				'interactionId',
			0						'interactionSection',
			0						'interactionPart',
			ASY.[Channel]			'interactionType',
			ASY.[StartDate]			'interactionDateStart',
			ASY.[EndDate]			'interactionDateEnd',
			ASY.[FirstTakedAgent]	'agentId',
			ASY.[Campaign]			'campaignId',
			ASY.[Initiative]		'originId',
			ASY.[ContactAddress]	'contactAddress',
			ASY.[DurationTime]		'duration',
			NULL					'evaluationPlanId',
			PASY.[ProcessId]		'processId',
			'A'						'evaluation_mode',
			RN = ROW_NUMBER() OVER(PARTITION BY ASY.[VCC],ASY.[ID] ORDER BY ASY.[CreatedDate] ASC)


			FROM dbo.[AsyncInteraction] ASY
			INNER JOIN dbo.[ProcessAsyncInteraction] PASY 
				ON ASY.[VCC] = PASY.[VCC]
					AND ASY.[ID] = PASY.[InteractionId]
					AND ASY.[EndDate] = PASY.[EndDate]


			WHERE ASY.[VCC] = @VCC	
					AND ASY.[IsTranscribed] = 1
					AND ASY.[ID]  IN (SELECT Value FROM [dbo].[split](@InteractionIds, ','))	
					AND ASY.[ID] NOT IN (SELECT interactionId FROM #DistinctInteractionEvaluationPlan)
		

	END

	
	
	DROP TABLE IF EXISTS #DistinctTextInteractionProcess
	    SELECT [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode]
			INTO   #DistinctTextInteractionProcess
			FROM #AllTextInteractionProcess WHERE RN = 1	
	
	--LISTADO CONSOLIDADO 
	
	SELECT  [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode] 
	FROM #DistinctInteractionEvaluationPlan
	UNION
	SELECT  [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode] 
	FROM #DistinctInteractionProcess
	UNION
	SELECT  [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode] 
	FROM #DistinctTextInteractionProcess
		
	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetGroup] (
	@Instance Varchar(50) = NULL, @ID Varchar(50) = NULL, @Disabled bit = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]							'id',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId',
		[Enabled]						'enabled'
	FROM [Group] (NOLOCK)
	WHERE [Instance] = @Instance
	AND [ID] = @ID
	-- AND IsNull(@Disabled, 1) = [Enabled]

	SET NOCOUNT OFF
END


GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetGroupDescription] (
	@Instance Varchar(50) = NULL, @Description Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]							'id',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Group] (NOLOCK)
	WHERE [Instance] = @Instance
	AND [Description] = @Description

	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGroups] (
	@Instance Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL, @Disabled bit = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		g.ID							'id',
		g.Description					'description',
		g.CreatedDate					'createdDate',
		g.CreatedByUserId				'createdByUserId',
		g.LastModifiedDate				'lastModifiedDate',
		g.LastModifiedByUserId			'lastModifiedByUserId',
		g.[Enabled]						'enabled'
	FROM [Group] g
	WHERE g.Instance = @Instance
		AND ISNULL(g.ID, '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(g.Description, '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' COLLATE Latin1_General_CI_AI ELSE '%' END
		AND (ISNULL(@Disabled, 0) = 1 OR (ISNULL(@Disabled, 0) = 0 AND ISNULL(@Disabled, 0) <> [Enabled]))
	ORDER BY g.Description ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetGroupsUsage] 
	@Instance      VARCHAR(50) = NULL,
	@GroupIDs VARCHAR(MAX) = NULL
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

    DECLARE @_IDS TABLE(
	 ID VARCHAR(50)
	)

    INSERT INTO @_IDS SELECT [Value] FROM [dbo].[Split](@GroupIDs,',')

	SELECT [IdGroup] 'idGroup'
        FROM [dbo].[AbilityEvaluationFormat] 
		WHERE Instance = @Instance
			AND IdGroup  IN (SELECT [ID] FROM @_IDS)
    
           
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetHistoricAgents] 
(
	@Vcc					VARCHAR(50) = NULL, 
	@ExcludesWorkGroupSup	BIT 
) 
AS
BEGIN
	WITH ResultAgent AS (	
		SELECT 
			[AU].[Agent]  'id',
			[AU].[Agent]  'description'
		FROM [dbo].[Audio] AU (NOLOCK)
		WHERE [AU].[VCC] = @Vcc
		AND (@ExcludesWorkGroupSup = 0
				OR
			(@ExcludesWorkGroupSup = 1 AND [AU].[Agent] NOT IN (SELECT WKS.AgentId FROM dbo.WorkGroupSupervisor WKS (NOLOCK) WHERE WKS.VCC = @Vcc))
			)
		UNION 
		SELECT 
			[IEP].[IdAgent] AS 'id',
			[IEP].[IdAgent] AS 'description'
		FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
		WHERE [IEP].[Instance] = @Vcc
		AND (@ExcludesWorkGroupSup = 0
				OR
			(@ExcludesWorkGroupSup = 1 AND [IEP].[IdAgent] NOT IN  (SELECT WKS.AgentId FROM dbo.WorkGroupSupervisor WKS (NOLOCK) WHERE WKS.VCC = @Vcc))
			)
	)

	SELECT 
		R.id AS 'id', 
		R.description AS 'description' ,
		(
			SELECT STUFF((
				SELECT ',' + [idcampaign] + ''
				FROM (
					SELECT DISTINCT [AU2].[Campaign] AS 'idcampaign'
					FROM [dbo].[Audio] AU2 (NOLOCK)
					WHERE [AU2].[VCC] = @Vcc AND [AU2].[Agent] = R.id

					UNION

					SELECT DISTINCT [IE2].[IdCampaign] AS 'idcampaign'
					FROM [dbo].[InteractionEvaluationPlan] IE2 (NOLOCK)
					WHERE [IE2].[Instance] = @Vcc AND [IE2].[IdAgent] = R.id
				) campaigns
				FOR XML PATH(''), TYPE
			).value('.', 'NVARCHAR(MAX)'), 1, 1, '') 
		) AS 'campaigns'
	FROM ResultAgent R

	
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetHistoricCampaigns]
(@Vcc					VARCHAR(50), 
 @ExcludesWorkGroupSup	BIT 
)
AS
BEGIN	
	
	
WITH ResultCampaing AS (
		SELECT 
			[AU].[Campaign]  'id',
			[AU].[Campaign]  'campaign'
		FROM [dbo].[Audio] AU (NOLOCK)
		WHERE [AU].[VCC] = @Vcc
		AND (@ExcludesWorkGroupSup = 0
				OR
			(@ExcludesWorkGroupSup = 1 AND [AU].[Campaign] NOT IN (SELECT WKS.CampaignId FROM dbo.WorkGroupSupervisor WKS (NOLOCK) WHERE WKS.VCC = @Vcc))
			)
		UNION 
		SELECT 
			[IEP].[IdCampaign] AS 'id',
			[IEP].[IdCampaign] AS 'campaign'
		FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
		WHERE [IEP].[Instance] = @Vcc
		AND (@ExcludesWorkGroupSup = 0
				OR
			(@ExcludesWorkGroupSup = 1 AND [IEP].[IdCampaign] NOT IN  (SELECT WKS.CampaignId FROM dbo.WorkGroupSupervisor WKS (NOLOCK) WHERE WKS.VCC = @Vcc))
			)
	)

	SELECT 
		R.id  'id', 
		R.campaign  'campaign' ,
		(
			SELECT STUFF((
				SELECT ',' + [idagent] + ''
				FROM (
					SELECT DISTINCT [AU2].[Agent]  'idagent'
					FROM [dbo].[Audio] AU2 (NOLOCK)
					WHERE [AU2].[VCC] = @Vcc AND [AU2].[Campaign] = R.id

					UNION

					SELECT DISTINCT [IE2].[IdAgent]  'idagent'
					FROM [dbo].[InteractionEvaluationPlan] IE2 (NOLOCK)
					WHERE [IE2].[Instance] = @Vcc AND [IE2].[IdCampaign] = R.id
				) agents
				FOR XML PATH(''), TYPE
			).value('.', 'NVARCHAR(MAX)'), 1, 1, '') 
		) AS 'agents'
	FROM ResultCampaing R
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetI6AlreadyRegisteredCallInteractions] (@VCC VARCHAR(50) = NULL)
AS
BEGIN
	-- CRUZAMOS LA TABLA TEMPORAL CON LAS INTERACCIONES DE LA API CON LA TABLA AUDIO
	SELECT DISTINCT A.[VCC]														'vcc',
					CONCAT(A.[ID], '_', A.[Section], '_', A.[Part])				'ID',
					A.[ID]														'InteractionId',
					A.[Section]													'Section',
					A.[Part]													'Part',
					'CALL'														'Channel',
					A.[Number]													'Number',
					A.[ContactAddress]											'ContactAdress',
					A.[Gender]													'Gender',
					A.[SilenceTime]												'SilenceTime',
					A.[DurationTime]											'DurationTime',
					A.[DurationTime]											'AttentionTime',
					1															'TotalParts',
					A.[Category1]												'Category1',
					A.[Category2]												'Category2',
					A.[Category3]												'Category3',
					A.[Description]												'Description',
					A.[DispositionCode]											'DispositionCode',
					A.[Campaign]												'Campaign',
					A.[Agent]													'Agent',
					A.[AgentName]												'agentName',
					A.[Type]													'Type',
					A.[StartDate]												'StartDate',
					A.[EndDate]													'EndDate',
					A.[Format]													'Format',
					ISNULL(A.[NeedI6Update], CAST(1 AS BIT))					'needI6Update',
					A.[Ender]													'ender',
					A.[ticket]													'ticket'
		FROM [#InteractionsTemp] IT
		INNER JOIN [dbo].[Audio] A (NOLOCK)
			ON A.[VCC] = IT.[VCC]
				AND  A.[ID] = IT.[InteractionId]
		WHERE IT.[VCC] = @VCC
		DROP TABLE #InteractionsTemp
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetI6AlreadyRegisteredTextInteractions] (@VCC VARCHAR(50) = NULL)
AS
BEGIN
	-- CRUZAMOS LA TABLA TEMPORAL CON LAS INTERACCIONES DE LA API CON LA TABLA ASYNCINTERACTION
	SELECT DISTINCT AI.[VCC]													'vcc',
					AI.[ID]														'id',
					AI.[ID]														'InteractionId',
					AI.[Folder]													'folder',
					AI.[ExportedContent]										'exportedContent',
					AI.[Channel]												'channel',
					AI.[Campaign]												'campaign',
					AI.[StartDate]												'startDate',
					AI.[EndDate]												'endDate',
					AI.[Account]												'account',
					AI.[Initiative]												'initiative',
					AI.[ContactName]											'contactName',
					AI.[ContactAddress]											'contactAdress',
					AI.[ContactId]												'contactId',
					AI.[FirstTakedAgent]										'firstTakedAgent',
					AI.[LastTakedAgent]											'lastTakedAgent',
					AI.[TotalMessages]											'totalMessages',
					AI.[DurationTime]											'durationTime',
					AI.[AttentionTime]											'attentionTime',
					AI.[DispositionCode]										'dispositionCode',
					AI.[DispositionTreePath]									'dispositionTreePath',
					AI.[RawChar]												'rawChar',
					ISNULL(AI.[NeedI6Update], CAST(1 AS BIT))					'needI6Update',
					AI.[Ender]													'ender',
					AI.[ticket]													'ticket'
		FROM [#InteractionsTemp] IT
		INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
			ON AI.[VCC] = IT.[VCC]
				AND  AI.[ID] = IT.[InteractionId]
				AND AI.[Folder] = IT.[Folder]
		WHERE IT.[VCC] = @VCC
		DROP TABLE #InteractionsTemp
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTE SP RETORNA LAS INTERACCIONES ASINCRONAS 
CREATE PROCEDURE [dbo].[GetI6AsyncInteractions]
(
	@VCC					VARCHAR(50) = NULL,
	@IP						VARCHAR(50),						-- IP DEL SERVIDOR PARA LA BASE I6
	@Type					INT,								-- FLAG PARA INDICAR SI EL SERVIDOR I6 SE ENCUENTRA LOCAL O REMOTO
	@InteractionType		VARCHAR(50) = NULL,
	@StartDate				DATETIME = '1980-01-01T00:00:00',
    @EndDate				DATETIME = '2050-01-01T23:59:59',
	@StartHour				VARCHAR(50) = NULL,
	@EndHour				VARCHAR(50) = NULL,
	@Campaigns				VARCHAR(MAX) = NULL,
 	@Agents					VARCHAR(MAX) = NULL,
	@DispositionCodes		VARCHAR(MAX) = NULL,
	@Origins				VARCHAR(MAX) = NULL,
	@InteractionIds         VARCHAR(MAX) = NULL,
    @Evaluated              VARCHAR(MAX) = NULL,
	@UniqueIds				BIT = 0,							-- ESTE FLAG TIENE COMO PROPOSITO RETORNAR LA LISTA CONSOLIDAD POR IDS O EL TOTAL DEL LISTADO
	@ResultType				VARCHAR(50) = NULL,					-- VARIABLE PARA DETERMINAR SI LA BUSQUEDA SERA SOBRE SOBRE DETALLES O ID (DETAILED, SUMMARIZED)
	@ContactAddress			VARCHAR(300) =NULL
    
)
AS
BEGIN
	-- INFO: EN CHAT SE CREAN UN ID EN CADA INTERACCION(EJM: CHATS DE ATENCION EN LAS WEB DE BANCOS), LOS DEMAS CANALES ID + ENDDATE
	
	-- SETTEAMOS VALORES MINIMOS PARA LAS HORAS DE INICIO Y FIN
	IF (@StartHour IS NOT NULL AND @StartHour = '') SET @StartHour = '00:00:00'
	IF (@EndHour IS NOT NULL AND @EndHour = '') SET @EndHour = '23:59:59'

	-- DECLARAMOS UNA VARIABLE PARA IDENTIFICAR SI EL SP ES LLAMADO DEL SISTEMA O POR OTRO SP
	DECLARE @IS_NESTED BIT = 0

	-- DECLARAMOASUN CADENA PARA EL QUERY FINAL
	DECLARE @SQL_SELECT VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''
    DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = '';
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = '';
    DECLARE @FILTERS_FIXED_PART     VARCHAR(MAX) = '';
	DECLARE @FILTER_CONTACT_ADDRESS VARCHAR(MAX) = '';
	
	-- DECLARAMOS VARIABLES PARA NOS NOMBRES DE LAS TABLAS
	DECLARE @HISTORICAL_DATA_I6_BD NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_APPSTORE_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_CALL_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_CHAT_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_FACEBOOK_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_GOOGLEPLAY_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_INSTAGRAM_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_MAIL_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_TWITTER_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_WEBCONTACT_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_WHATSAPP_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_YOUTUBE_TABLE NVARCHAR(100)

	-- DEFINIMOS EL NOMBRE DE LA BD
	SET @HISTORICAL_DATA_I6_BD = '[HistoricalDataI6]' -- BD PARA DESARROLLO
	-- SET @HISTORICAL_DATA_I6_BD = '[HistoricalDataI6]'

	-- EL PARAMETRO TYPE DEFINE SI LA BD HISTORICALDATAI6 SE ENCUENTRA LOCAL O REMOTO
	IF (@Type = 0)  
	BEGIN  
		SET @HISTORICAL_DATA_I6_APPSTORE_TABLE =		'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Appstore]'
		SET @HISTORICAL_DATA_I6_CHAT_TABLE =			'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Call]'
		SET @HISTORICAL_DATA_I6_CHAT_TABLE =			'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Chat]'
		SET @HISTORICAL_DATA_I6_FACEBOOK_TABLE =		'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Facebook]'
		SET @HISTORICAL_DATA_I6_GOOGLEPLAY_TABLE =		'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Googleplay]'
		SET @HISTORICAL_DATA_I6_INSTAGRAM_TABLE =		'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Instagram]'
		SET @HISTORICAL_DATA_I6_MAIL_TABLE =			'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Mail]'
		SET @HISTORICAL_DATA_I6_TWITTER_TABLE =			'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Twitter]'
		SET @HISTORICAL_DATA_I6_WEBCONTACT_TABLE =		'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Webcontact]'
		SET @HISTORICAL_DATA_I6_WHATSAPP_TABLE =		'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Whatsapp]'
		SET @HISTORICAL_DATA_I6_YOUTUBE_TABLE =			'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Youtube]'
	END
	ELSE IF(@Type = 1) 
	BEGIN
		SET @HISTORICAL_DATA_I6_APPSTORE_TABLE =		@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Appstore]'
		SET @HISTORICAL_DATA_I6_CHAT_TABLE =			@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Call]'
		SET @HISTORICAL_DATA_I6_CHAT_TABLE =			@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Chat]'
		SET @HISTORICAL_DATA_I6_FACEBOOK_TABLE =		@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Facebook]'
		SET @HISTORICAL_DATA_I6_GOOGLEPLAY_TABLE =		@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Googleplay]'
		SET @HISTORICAL_DATA_I6_INSTAGRAM_TABLE =		@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Instagram]'
		SET @HISTORICAL_DATA_I6_MAIL_TABLE =			@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Mail]'
		SET @HISTORICAL_DATA_I6_TWITTER_TABLE =			@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Twitter]'
		SET @HISTORICAL_DATA_I6_WEBCONTACT_TABLE =		@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Webcontact]'
		SET @HISTORICAL_DATA_I6_WHATSAPP_TABLE =		@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Whatsapp]'
		SET @HISTORICAL_DATA_I6_YOUTUBE_TABLE =			@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Youtube]'
	END  

	-- ARMAMOS EL QUERY PARA HACER LA CONSULTA
	SET @SQL_SELECT += '
		SELECT '

	-- ARMAMOS LA PARTE BASE DE FILTROS
	SET @FILTERS_FIXED_PART = ' WHERE HDI6.[Vcc] = ''' + @VCC + ''''
	
	SET @FILTERS_OPTIONAL_PART  =' AND HDI6.[EndDate] BETWEEN '''
    SET @FILTERS_OPTIONAL_PART += (SELECT CONVERT(VARCHAR(21), @StartDate, 127)) 
    SET @FILTERS_OPTIONAL_PART  +=''' AND '''  
    SET @FILTERS_OPTIONAL_PART  +=(SELECT CONVERT(VARCHAR(21), @EndDate, 127)) + ''' '
    

	-- DEPENDIENDO SI LA HORA DE FIN ES MAYOR A LA DE INICIO O NO DAMOS FORMA AL FILTRO DE HORA
	IF (CAST(@StartHour AS TIME) <= CAST(@EndHour AS TIME)) SET @FILTERS_OPTIONAL_PART += 'AND CAST(HDI6.[EndDate] AS TIME) BETWEEN ''' + @StartHour + ''' AND ''' + @EndHour + ''' ';
	IF (CAST(@StartHour AS TIME) > CAST(@EndHour AS TIME)) SET @FILTERS_OPTIONAL_PART += 'AND CAST(HDI6.[EndDate] AS TIME) NOT BETWEEN ''' + @EndHour + ''' AND ''' + @StartHour + ''' ';
		
	IF (@Campaigns IS NOT NULL AND @Campaigns != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Campaign] IN (SELECT VALUE FROM [dbo].[split](''' + @Campaigns + ''','','')) '
	
	
	IF(@ContactAddress IS NOT NULL AND @InteractionType='CHAT')
		BEGIN
			SET @FILTER_CONTACT_ADDRESS = 'CHAT'
			SET @FILTERS_OPTIONAL_PART += '
				AND ContactName LIKE ''%' + @ContactAddress + '%''
			'
		END	
	IF(@ContactAddress IS NOT NULL AND @InteractionType='MAIL')
		BEGIN
			SET @FILTER_CONTACT_ADDRESS = 'MAIL'
			SET @FILTERS_OPTIONAL_PART += '
				AND (MailFrom LIKE ''%' + @ContactAddress + '%'' OR MailFrom LIKE ''%' + @ContactAddress + '%'')
			'
		END	
	IF(@ContactAddress IS NOT NULL AND @InteractionType='WEBCONTACT')
		BEGIN
			SET @FILTER_CONTACT_ADDRESS = 'WEBCONTACT'
			SET @FILTERS_OPTIONAL_PART += '
				AND ContactName LIKE ''%' + @ContactAddress + '%''
			'
		END	
	IF(@ContactAddress IS NOT NULL AND  @FILTER_CONTACT_ADDRESS = '')
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND ContactAddress LIKE ''%' + @ContactAddress + '%''
			'
		END	

	
	IF (@DispositionCodes IS NOT NULL AND @DispositionCodes != '') 
		SET @FILTERS_OPTIONAL_PART += 'AND (
								(HDI6.[DispositionTreePath] IN (SELECT VALUE FROM [dbo].[split](''' + @DispositionCodes + ''','',''))) 
								OR
								(HDI6.[DispositionCode] IN (SELECT VALUE FROM [dbo].[split](''' + @DispositionCodes + ''','','')))
							)'	

	-- ARMAMOS EL ORDER BY
	SET @SQL_EXTRA += 'ORDER BY [ID], [EndDate] DESC'

	-- ARMAMOS EL SELECT PARA LOS DIFERENTES INTERACTION TYPES
	IF (@InteractionType = 'APPSTORE')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_APPSTORE_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_APPSTORE
		SET @SQL_SELECT += '
								Vcc														''vcc'', 
								ID														''id'', 
								CONVERT(VARCHAR, EndDate, 23)							''folder'',
								ExportedContent											''exportedContent'', 
								''APPSTORE''											''channel'',
								Campaign												''campaign'', 
								StartDate												''startDate'', 
								EndDate													''endDate'', 
								Account													''account'', 
								Initiative												''initiative'',
								SUBSTRING(ContactName, 0, 50)							''contactName'',
								ContactAddress											''contactAddress'',
								NULL													''contactId'',
								FirstAgent												''firstTakedAgent'', 
								NULL													''lastTakedAgent'', 
								NULL													''isTaken'',
								ISNULL(TRY_CAST(TotalMessages AS INT), 1)				''totalMessages'',
								CASE WHEN ISNUMERIC(DurationTime) = 0 
									THEN (LEFT(DurationTime, 2 ) * 3600 
											+ SUBSTRING(DurationTime, 4, 2) * 60 
											+ SUBSTRING(DurationTime, 7, 2))
									ELSE DurationTime END									''durationTime'', 
								CASE WHEN ISNUMERIC(AttentionTime) = 0 
									THEN (LEFT(AttentionTime, 2 ) * 3600 
											+ SUBSTRING(AttentionTime, 4, 2) * 60 
											+ SUBSTRING(AttentionTime, 7, 2))
									ELSE AttentionTime END									''attentionTime'', 
								DispositionCode											''dispositionCode'', 
								DispositionTreePath										''dispositionTreePath'',
								RawChar													''rawChar'' '

		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += 'AND HDI6.[FirstAgent] IS NOT NULL '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[FirstAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '
		
		-- PARA HACER EL FILTRO POR ORIGEN, REEMPLAZAMOS EN EL INPUT POR LOS VALORES DE LA TABLA
		IF (@Origins IS NOT NULL AND @Origins != '') 
		BEGIN
			SET @Origins = REPLACE(@Origins, 'E', 'INBOUND')
			SET @Origins = REPLACE(@Origins, 'S', 'OUTBOUND')
			SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Initiative] IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
		END
		
	END
	ELSE IF (@InteractionType = 'CALL')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_CALL_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_CHAT
		SET @SQL_SELECT += '
								[Vcc]														''vcc'',
								[Id] + ''_0_0''												''ID'',
								[Id]														''InteractionId'',
								0															''Section'',
								0															''Part'',
								''CALL''													''Channel'',
								[ContactAddress]											''Number'',	
								[ContactAddress]											''contactAddress'',
								NULL														''contactId'',
								[ContactName]												''contactName'',
								''''														''Gender'',
								ISNULL([HoldTime], 0)										''SilenceTime'',
								ISNULL([AttentionTime], 0)									''DurationTime'', -- ESTO SE DEBE CORREGIR
								ISNULL([AttentionTime], 0)									''AttentionTime'', -- SE AGREGO CAMPO PARA AMBOS EXISTAN
								[DispositionTreePath]										''Category1'',
								NULL														''Category2'',
								NULL														''Category3'',
								NULL														''Description'',
								[DispositionCode]											''DispositionCode'',
								[Campaign]													''Campaign'',
								[LastAgent]													''Agent'',
								NULL														''agentName'',
								CASE [Initiative] WHEN ''INBOUND'' THEN ''E''
													WHEN ''Inb.'' THEN ''E''
													WHEN ''OUTBOUND'' THEN ''S''
													WHEN ''Out.'' THEN ''S''
													ELSE NULL END							''Type'',
								CONVERT(VARCHAR, [StartDate], 120)							''StartDate'',
								CONVERT(VARCHAR, [EndDate], 120)							''EndDate'',
								''wav''														''Format'' '

		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += '[IsTaken] = 1 '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[LastAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '

		-- PARA HACER EL FILTRO POR ORIGEN, REEMPLAZAMOS EN EL INPUT POR LOS VALORES DE LA TABLA
		IF (@Origins IS NOT NULL AND @Origins != '') 
		BEGIN
			SET @Origins = REPLACE(@Origins, 'E', 'INBOUND')
			SET @Origins = REPLACE(@Origins, 'S', 'OUTBOUND')
			SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Initiative] IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
		END
	END 
	ELSE IF (@InteractionType = 'CHAT')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_CHAT_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_CHAT
		SET @SQL_SELECT += '
								Vcc															''vcc'', 
								ID															''id'', 
								CONVERT(VARCHAR, EndDate, 23)								''folder'',
								ExportedContent												''exportedContent'', 
								''CHAT''													''channel'',
								Campaign													''campaign'', 
								StartDate													''startDate'', 
								EndDate														''endDate'', 
								Account														''account'', 
								''IN''														''initiative'', 
								-- ContactName												''contactName'', 
								SUBSTRING(ContactName, 0, 50)								''contactName'', 
								NULL														''contactAddress'',
								NULL														''contactId'',
								FirstTakeAgent												''firstTakedAgent'', 
								LastTakeAgent												''lastTakedAgent'', 
								IsTaken														''isTaken'',
								ISNULL(TRY_CAST(ChatMessages AS INT), 1)					''totalMessages'',
								CASE WHEN ISNUMERIC(DurationTime) = 0 
									THEN (LEFT(DurationTime, 2 ) * 3600 
											+ SUBSTRING(DurationTime, 4, 2) * 60 
											+ SUBSTRING(DurationTime, 7, 2))
									ELSE DurationTime END									''durationTime'', 
								CASE WHEN ISNUMERIC(AttentionTime) = 0 
									THEN (LEFT(AttentionTime, 2 ) * 3600 
											+ SUBSTRING(AttentionTime, 4, 2) * 60 
											+ SUBSTRING(AttentionTime, 7, 2))
									ELSE AttentionTime END									''attentionTime'', 
								DispositionCode												''dispositionCode'', 
								DispositionTreePath											''dispositionTreePath'' ,
								RawChar														''rawChar'' '

		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += 'AND (HDI6.[FirstTakeAgent] IS NOT NULL OR HDI6.[LastTakeAgent] IS NOT NULL) '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[LastTakeAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '

		-- PARA HACER EL FILTRO POR ORIGEN DE CHAT, SOLO ES VALIDO CUANDO EL INPUT INCLUYE IN
		IF (@Origins IS NOT NULL AND @Origins != '') SET @FILTERS_OPTIONAL_PART += 'AND ''E'' IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
	END 
	ELSE IF (@InteractionType = 'FACEBOOK')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_FACEBOOK_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_FACEBOOK
		SET @SQL_SELECT += '
								Vcc															''vcc'', 
								ID															''id'', 
								CONVERT(VARCHAR, EndDate, 23)								''folder'',
								ExportedContent												''exportedContent'', 
								''FACEBOOK''												''channel'',
								Campaign													''campaign'', 
								StartDate													''startDate'', 
								EndDate														''endDate'', 
								Account														''account'', 
								CASE Initiative WHEN ''INBOUND'' THEN ''IN''
									WHEN ''OUTBOUND'' THEN ''OUT''
									ELSE NULL END											''initiative'', 
								SUBSTRING(ContactName, 0, 50)								''contactName'', 
								[ContactAddress]											''contactAddress'',
								NULL														''contactId'',								
								FirstAgent													''firstTakedAgent'', 
								NULL														''lastTakedAgent'', 
								NULL														''isTaken'',
								ISNULL(TRY_CAST(TotalMessages AS INT), 1)					''totalMessages'',
								CASE WHEN ISNUMERIC(DurationTime) = 0 
									THEN (LEFT(DurationTime, 2 ) * 3600 
											+ SUBSTRING(DurationTime, 4, 2) * 60 
											+ SUBSTRING(DurationTime, 7, 2))
									ELSE DurationTime END									''durationTime'', 
								CASE WHEN ISNUMERIC(AttentionTime) = 0 
									THEN (LEFT(AttentionTime, 2 ) * 3600 
											+ SUBSTRING(AttentionTime, 4, 2) * 60 
											+ SUBSTRING(AttentionTime, 7, 2))
									ELSE AttentionTime END									''attentionTime'', 
								DispositionCode												''dispositionCode'', 
								DispositionTreePath											''dispositionTreePath'' ,
								RawChar														''rawChar'' '
		
		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += 'AND HDI6.[FirstAgent] IS NOT NULL '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[FirstAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '
		
		-- PARA HACER EL FILTRO POR ORIGEN, REEMPLAZAMOS EN EL INPUT POR LOS VALORES DE LA TABLA
		IF (@Origins IS NOT NULL AND @Origins != '') 
		BEGIN
			SET @Origins = REPLACE(@Origins, 'E', 'INBOUND')
			SET @Origins = REPLACE(@Origins, 'S', 'OUTBOUND')
			SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Initiative] IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
		END
	END 
	ELSE IF (@InteractionType = 'GOOGLEPLAY')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_GOOGLEPLAY_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_GOOGLEPLAY
		SET @SQL_SELECT += '
								Vcc															''vcc'', 
								ID															''id'', 
								CONVERT(VARCHAR, EndDate, 23)								''folder'',
								ExportedContent												''exportedContent'', 
								''GOOGLEPLAY''												''channel'',
								Campaign													''campaign'', 
								StartDate													''startDate'', 
								EndDate														''endDate'', 
								Account														''account'',  
								Initiative													''initiative'', 
								SUBSTRING(ContactName, 0, 50)								''contactName'', 
								[ContactAddress]											''contactAddress'',
								NULL														''contactId'',	
								FirstAgent													''firstTakedAgent'', 
								NULL														''lastTakedAgent'', 
								HasResponse													''isTaken'',
								ISNULL(TRY_CAST(TotalMessages AS INT), 1)					''totalMessages'',
								CASE WHEN ISNUMERIC(DurationTime) = 0 
									THEN (LEFT(DurationTime, 2 ) * 3600 
											+ SUBSTRING(DurationTime, 4, 2) * 60 
											+ SUBSTRING(DurationTime, 7, 2))
									ELSE DurationTime END									''durationTime'', 
								CASE WHEN ISNUMERIC(AttentionTime) = 0 
									THEN (LEFT(AttentionTime, 2 ) * 3600 
											+ SUBSTRING(AttentionTime, 4, 2) * 60 
											+ SUBSTRING(AttentionTime, 7, 2))
									ELSE AttentionTime END									''attentionTime'', 
								DispositionCode												''dispositionCode'', 
								DispositionTreePath											''dispositionTreePath'' ,
								RawChar														''rawChar'' '

		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += 'AND HDI6.[FirstAgent] IS NOT NULL '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[FirstAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '

		-- PARA HACER EL FILTRO POR ORIGEN, REEMPLAZAMOS EN EL INPUT POR LOS VALORES DE LA TABLA
		IF (@Origins IS NOT NULL AND @Origins != '') 
		BEGIN
			SET @Origins = REPLACE(@Origins, 'E', 'INBOUND')
			SET @Origins = REPLACE(@Origins, 'S', 'OUTBOUND')
			SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Initiative] IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
		END
	END
	ELSE IF (@InteractionType = 'INSTAGRAM')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_INSTAGRAM_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_INSTAGRAM
		SET @SQL_SELECT += '
								Vcc															''vcc'', 
								ID															''id'', 
								CONVERT(VARCHAR, EndDate, 23)								''folder'',
								ExportedContent												''exportedContent'', 
								''INSTAGRAM''												''channel'',
								Campaign													''campaign'', 
								StartDate													''startDate'', 
								EndDate														''endDate'', 
								Account														''account'', 
								CASE Initiative WHEN ''INBOUND'' THEN ''IN''
									WHEN ''OUTBOUND'' THEN ''OUT''
									ELSE NULL END											''initiative'', 
								SUBSTRING(ContactName, 0, 50)								''contactName'', 
								ContactAddress												''contactAddress'',
								NULL														''contactId'',	
								FirstAgent													''firstTakedAgent'', 
								NULL														''lastTakedAgent'', 
								HasResponse													''isTaken'',
								ISNULL(TRY_CAST(TotalMessages AS INT), 1)					''totalMessages'',
								CASE WHEN ISNUMERIC(DurationTime) = 0 
									THEN (LEFT(DurationTime, 2 ) * 3600 
											+ SUBSTRING(DurationTime, 4, 2) * 60 
											+ SUBSTRING(DurationTime, 7, 2))
									ELSE DurationTime END									''durationTime'', 
								CASE WHEN ISNUMERIC(AttentionTime) = 0 
									THEN (LEFT(AttentionTime, 2 ) * 3600 
											+ SUBSTRING(AttentionTime, 4, 2) * 60 
											+ SUBSTRING(AttentionTime, 7, 2))
									ELSE AttentionTime END									''attentionTime'', 
								DispositionCode												''dispositionCode'', 
								DispositionTreePath											''dispositionTreePath'' ,
								RawChar														''rawChar'' '

		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += 'AND HDI6.[FirstAgent] IS NOT NULL '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[FirstAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '

		-- PARA HACER EL FILTRO POR ORIGEN, REEMPLAZAMOS EN EL INPUT POR LOS VALORES DE LA TABLA
		IF (@Origins IS NOT NULL AND @Origins != '') 
		BEGIN
			SET @Origins = REPLACE(@Origins, 'E', 'INBOUND')
			SET @Origins = REPLACE(@Origins, 'S', 'OUTBOUND')
			SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Initiative] IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
		END
	END  
	ELSE IF (@InteractionType = 'MAIL')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_MAIL_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_MAIL
		SET @SQL_SELECT += '
								Vcc															''vcc'', 
								ID															''id'', 
								CONVERT(VARCHAR, EndDate, 23)								''folder'',
								ExportedContent												''exportedContent'', 
								''MAIL''													''channel'',
								Campaign													''campaign'', 
								StartDate													''startDate'', 
								EndDate														''endDate'', 
								Account														''account'', 
								CASE Initiative WHEN ''INBOUND'' THEN ''IN''
									WHEN ''OUTBOUND'' THEN ''OUT''
									ELSE NULL END											''initiative'', 
								SUBSTRING(ContactName, 0, 50)								''contactName'', 
								CASE WHEN Initiative =''INBOUND'' THEN MailFrom ELSE MailTo END ''contactAddress'',
								NULL														''contactId'',	
								FirstAgent													''firstTakedAgent'', 
								NULL														''lastTakedAgent'', 
								MailHasResponse												''isTaken'',
								ISNULL(TRY_CAST(MailTotalMessages AS INT), 1)				''totalMessages'',
								CASE WHEN ISNUMERIC(DurationTime) = 0 
									THEN (LEFT(DurationTime, 2 ) * 3600 
											+ SUBSTRING(DurationTime, 4, 2) * 60 
											+ SUBSTRING(DurationTime, 7, 2))
									ELSE DurationTime END									''durationTime'', 
								CASE WHEN ISNUMERIC(AttentionTime) = 0 
									THEN (LEFT(AttentionTime, 2 ) * 3600 
											+ SUBSTRING(AttentionTime, 4, 2) * 60 
											+ SUBSTRING(AttentionTime, 7, 2))
									ELSE AttentionTime END									''attentionTime'', 
								DispositionCode												''dispositionCode'', 
								DispositionTreePath											''dispositionTreePath'' ,
								RawChar														''rawChar'' '
								
		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += 'AND HDI6.[FirstAgent] IS NOT NULL '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[FirstAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '

		-- PARA HACER EL FILTRO POR ORIGEN, REEMPLAZAMOS EN EL INPUT POR LOS VALORES DE LA TABLA
		IF (@Origins IS NOT NULL AND @Origins != '') 
		BEGIN
			SET @Origins = REPLACE(@Origins, 'E', 'INBOUND')
			SET @Origins = REPLACE(@Origins, 'S', 'OUTBOUND')
			SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Initiative] IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
		END
	END  
	ELSE IF (@InteractionType = 'TWITTER')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_TWITTER_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_TWITTER
		SET @SQL_SELECT += '
								Vcc															''vcc'', 
								ID															''id'', 
								CONVERT(VARCHAR, EndDate, 23)								''folder'',
								ExportedContent												''exportedContent'', 
								''TWITTER''													''channel'',
								Campaign													''campaign'', 
								StartDate													''startDate'', 
								EndDate														''endDate'', 
								Account														''account'', 
								CASE Initiative WHEN ''INBOUND'' THEN ''IN''
									WHEN ''OUTBOUND'' THEN ''OUT''
									ELSE NULL END											''initiative'', 
								SUBSTRING(ContactName, 0, 50)								''contactName'', 
								ContactAddress												''contactAddress'',
								NULL														''contactId'',
								FirstAgent													''firstTakedAgent'', 
								NULL														''lastTakedAgent'', 
								HasResponse													''isTaken'',
								ISNULL(TRY_CAST(TotalMessages AS INT), 1)					''totalMessages'',
								CASE WHEN ISNUMERIC(DurationTime) = 0 
									THEN (LEFT(DurationTime, 2 ) * 3600 
											+ SUBSTRING(DurationTime, 4, 2) * 60 
											+ SUBSTRING(DurationTime, 7, 2))
									ELSE DurationTime END									''durationTime'', 
								CASE WHEN ISNUMERIC(AttentionTime) = 0 
									THEN (LEFT(AttentionTime, 2 ) * 3600 
											+ SUBSTRING(AttentionTime, 4, 2) * 60 
											+ SUBSTRING(AttentionTime, 7, 2))
									ELSE AttentionTime END									''attentionTime'', 
								DispositionCode												''dispositionCode'', 
								DispositionTreePath											''dispositionTreePath'' ,
								RawChar														''rawChar'' '
								
		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += 'AND HDI6.[FirstAgent] IS NOT NULL '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[FirstAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '

		-- PARA HACER EL FILTRO POR ORIGEN, REEMPLAZAMOS EN EL INPUT POR LOS VALORES DE LA TABLA
		IF (@Origins IS NOT NULL AND @Origins != '') 
		BEGIN
			SET @Origins = REPLACE(@Origins, 'E', 'INBOUND')
			SET @Origins = REPLACE(@Origins, 'S', 'OUTBOUND')
			SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Initiative] IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
		END
	END  
	ELSE IF (@InteractionType = 'WEBCONTACT')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_WEBCONTACT_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_WEBCONTACT
		SET @SQL_SELECT += '
								Vcc															''vcc'', 
								ID															''id'', 
								CONVERT(VARCHAR, EndDate, 23)								''folder'',
								ExportedContent												''exportedContent'', 
								''WEBCONTACT''												''channel'',
								Campaign													''campaign'', 
								StartDate													''startDate'', 
								EndDate														''endDate'', 
								Account														''account'', 
								''IN''														''initiative'', 
								SUBSTRING(ContactName, 0, 50)								''contactName'', 
								NULL														''contactAddress'',
								NULL														''contactId'',
								FirstAgent													''firstTakedAgent'', 
								NULL														''lastTakedAgent'', 
								HasResponse													''isTaken'',
								ISNULL(TRY_CAST(TotalResponses AS INT), 1)					''totalMessages'',
								CASE WHEN ISNUMERIC(DurationTime) = 0 
									THEN (LEFT(DurationTime, 2 ) * 3600 
											+ SUBSTRING(DurationTime, 4, 2) * 60 
											+ SUBSTRING(DurationTime, 7, 2))
									ELSE DurationTime END									''durationTime'', 
								CASE WHEN ISNUMERIC(AttentionTime) = 0 
									THEN (LEFT(AttentionTime, 2 ) * 3600 
											+ SUBSTRING(AttentionTime, 4, 2) * 60 
											+ SUBSTRING(AttentionTime, 7, 2))
									ELSE AttentionTime END									''attentionTime'', 
								DispositionCode												''dispositionCode'', 
								DispositionTreePath											''dispositionTreePath'' ,
								RawChar														''rawChar'' '

		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += 'AND HDI6.[FirstAgent] IS NOT NULL '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[FirstAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '

		-- PARA HACER EL FILTRO POR ORIGEN DE WEBCONTACT, SOLO ES VALIDO CUANDO EL INPUT INCLUYE IN
		IF (@Origins IS NOT NULL AND @Origins != '') SET @FILTERS_OPTIONAL_PART += 'AND ''E'' IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
	END
	ELSE IF (@InteractionType = 'WHATSAPP')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_WHATSAPP_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_WHATSAPP
		SET @SQL_SELECT += '
								Vcc															''vcc'', 
								ID															''id'', 
								CONVERT(VARCHAR, EndDate, 23)								''folder'',
								ExportedContent												''exportedContent'', 
								''WHATSAPP''												''channel'',
								Campaign													''campaign'', 
								StartDate													''startDate'', 
								EndDate														''endDate'', 
								Account														''account'', 
								CASE Initiative WHEN ''In.'' THEN ''IN''
									WHEN ''Out.'' THEN ''OUT''
									ELSE NULL END											''initiative'', 
								SUBSTRING(ContactName, 0, 50)								''contactName'',
								ContactAddress												''contactAddress'',
								NULL														''contactId'',
								FirstAgent													''firstTakedAgent'', 
								NULL														''lastTakedAgent'', 
								HasResponse													''isTaken'',
								ISNULL(TRY_CAST(WhatsappTotalMessages AS INT), 1)			''totalMessages'',
								CASE WHEN ISNUMERIC(DurationTime) = 0 
									THEN (LEFT(DurationTime, 2 ) * 3600 
											+ SUBSTRING(DurationTime, 4, 2) * 60 
											+ SUBSTRING(DurationTime, 7, 2))
									ELSE DurationTime END									''durationTime'', 
								CASE WHEN ISNUMERIC(AttentionTime) = 0 
									THEN (LEFT(AttentionTime, 2 ) * 3600 
											+ SUBSTRING(AttentionTime, 4, 2) * 60 
											+ SUBSTRING(AttentionTime, 7, 2))
									ELSE AttentionTime END									''attentionTime'', 
								DispositionCode												''dispositionCode'', 
								DispositionTreePath											''dispositionTreePath'' ,
								RawChar															''rawChar'' '
								
		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += 'AND HDI6.[FirstAgent] IS NOT NULL '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[FirstAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '
		
		-- PARA HACER EL FILTRO POR ORIGEN, REEMPLAZAMOS EN EL INPUT POR LOS VALORES DE LA TABLA
		IF (@Origins IS NOT NULL AND @Origins != '') 
		BEGIN
			SET @Origins = REPLACE(@Origins, 'E', 'INBOUND')
			SET @Origins = REPLACE(@Origins, 'S', 'OUTBOUND')
			SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Initiative] IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
		END
	END  
	ELSE IF (@InteractionType = 'YOUTUBE')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_YOUTUBE_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_YOUTUBE
		SET @SQL_SELECT += '
								Vcc															''vcc'', 
								ID															''id'', 
								CONVERT(VARCHAR, EndDate, 23)								''folder'',
								ExportedContent												''exportedContent'', 
								''YOUTUBE''													''channel'',
								Campaign													''campaign'', 
								StartDate													''startDate'', 
								EndDate														''endDate'', 
								Account														''account'', 
								CASE Initiative WHEN ''INBOUND'' THEN ''IN''
									WHEN ''OUTBOUND'' THEN ''OUT''
									ELSE NULL END											''initiative'', 
								SUBSTRING(ContactName, 0, 50)								''contactName'',
								ContactAddress												''contactAddress'',
								NULL														''contactId'',
								FirstAgent													''firstTakedAgent'', 
								NULL														''lastTakedAgent'', 
								HasResponse													''isTaken'',
								ISNULL(TRY_CAST(TotalMessages AS INT), 1)					''totalMessages'',
								CASE WHEN ISNUMERIC(DurationTime) = 0 
									THEN (LEFT(DurationTime, 2 ) * 3600 
											+ SUBSTRING(DurationTime, 4, 2) * 60 
											+ SUBSTRING(DurationTime, 7, 2))
									ELSE DurationTime END									''durationTime'', 
								CASE WHEN ISNUMERIC(AttentionTime) = 0 
									THEN (LEFT(AttentionTime, 2 ) * 3600 
											+ SUBSTRING(AttentionTime, 4, 2) * 60 
											+ SUBSTRING(AttentionTime, 7, 2))
									ELSE AttentionTime END									''attentionTime'', 
								DispositionCode												''dispositionCode'', 
								DispositionTreePath											''dispositionTreePath'' ,
								RawChar														''rawChar'' '
								
		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += 'AND HDI6.[FirstAgent] IS NOT NULL '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[FirstAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '

		-- PARA HACER EL FILTRO POR ORIGEN, REEMPLAZAMOS EN EL INPUT POR LOS VALORES DE LA TABLA
		IF (@Origins IS NOT NULL AND @Origins != '') 
		BEGIN
			SET @Origins = REPLACE(@Origins, 'E', 'INBOUND')
			SET @Origins = REPLACE(@Origins, 'S', 'OUTBOUND')
			SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Initiative] IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
		END
	END

    IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
    BEGIN
        SET @FILTERS_OPTIONAL_PART = '';
	    SET @FILTERS_EXCLUDING_PART=' AND ID IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) ';
    END
		  
	-- AGREGAMOS EL CONTADOR DE FILAR AL SELECT
	SET @SQL_SELECT += '
								,RN = ROW_NUMBER() OVER(PARTITION BY [ID], CONVERT(VARCHAR, EndDate, 23) ORDER BY [EndDate] DESC) '

	-- ARMAMOS LA PARTE DE TABLAS 
	SET @SQL_FROM += '
			FROM ' + @HISTORICAL_DATA_I6_TABLE + ' HDI6 (NOLOCK) '

	-- UNIMOS LAS PARTES DEL QUERY
	SET @SQL_SELECT += @SQL_FROM + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART + @SQL_EXTRA

	-- VERIFICAMOS SI EXISTE LA TABLA TEMPORAL #TMP_HD_INTERACTIONS
	IF OBJECT_ID('TEMPDB..#TMP_HD_INTERACTIONS') IS NULL
	BEGIN
		-- CREAMOS UNA TABLA TEMPORAL PARA INSERTAR LOS REGISTROS DE LA HDI6
		CREATE TABLE #TMP_HD_INTERACTIONS
		(
			vcc VARCHAR(50), 
			id VARCHAR(250),  
			folder VARCHAR(50),
			exportedContent VARCHAR(250),  
			channel VARCHAR(50), 
			campaign VARCHAR(50), 
			startDate DATETIME,
			endDate DATETIME,
			account VARCHAR(50), 
			initiative VARCHAR(50), 
			contactName VARCHAR(250), 
			contactAddress VARCHAR(300),
			conctacId VARCHAR(50),
			firstTakedAgent VARCHAR(50), 
			lastTakedAgent VARCHAR(50), 
			isTaken VARCHAR(50), 
			totalMessages INT,
			durationTime VARCHAR(50),  
			attentionTime VARCHAR(50), 
			dispositionCode VARCHAR(250), 
			dispositionTreePath VARCHAR(250), 
			rawChar INT,
			RN INT
		)

		-- REAFIRMAMOS QUE EL SP NO ESTA ANIDADO EN OTRO
		SET @IS_NESTED = 0

		-- PRINT 'SP NO ANIDADO'
	END
	ELSE
	BEGIN
		-- SI LA TABLA TEMPORAL YA EXISTIA, EL SP ESTA ANIDADO EN OTRO (P.E.: [dbo].[GetAsyncBillingInteractions])
		SET @IS_NESTED = 1

		-- PRINT 'SP ANIDADO'
	END 

	-- INSERTAMOS EL RESULTADO EN LA TABLA TEMPORAL
	print @SQL_SELECT 
	INSERT INTO #TMP_HD_INTERACTIONS
		-- EJECUTAMOS LA CONSULTA
		
		EXEC (@SQL_SELECT)
   
    
    -- VARIABLES PARA TRABAJAR CON LA TABLA TEMPORAL AL FINAL DEL SP
    DECLARE @QUERY                  VARCHAR(MAX) = ''
    DECLARE @QUERY_FILTER           VARCHAR(MAX) = ''
    DECLARE @QUERY_JOIN             VARCHAR(MAX) = ''
	
	-- FILTRO POR INTERACCIONES EVALUADAS O NO
	IF (@Evaluated IS NOT NULL AND @Evaluated <> '')
	    BEGIN 
		DECLARE @EvaluatedTable TABLE (Val BIT)

		INSERT INTO @EvaluatedTable SELECT Value FROM [dbo].[split]( @Evaluated ,',')

		-- SOLAMENTE SI ES SI O NO
		IF ((SELECT COUNT(Val) FROM @EvaluatedTable) = 1 )
		BEGIN
			IF((SELECT Val FROM @EvaluatedTable WHERE Val = 1) IS NOT NULL) -- Si  es Si
			BEGIN 
				SET @QUERY_JOIN = ' JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) ON (AEP.[IdInteraction] = HDI.[id] AND AEP.[Completed]=1 ) '
			END
		ELSE  --Si es No
           BEGIN
               SET @QUERY_JOIN = ' LEFT JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) ON (AEP.[IdInteraction] = HDI.[id] AND AEP.[Completed]=1 ) '
               SET @QUERY_FILTER = ' AND AEP.[IdInteraction] IS NULL '
           END
       END
       
	 END
	-- FIN FILTRO INTERACCIONES EVALUADAS


	

	-- SI EL SP NO ESTA ANIDADO, HACEMOS SELECT AL RESULTADO A MOSTRAR
	IF @IS_NESTED = 0
	BEGIN
		-- HACEMOS EL SELECT SEGUN EL TIPO DE BUSQUEDA SETTEADO
		-- IF @UniqueIds = 1 -- ENFOCADO A INTERACCIONES
		-- BEGIN
			IF @ResultType = 'SUMMARIZED'
			BEGIN
				SET @QUERY = '
					SELECT HDI.[channel]										AS ''interactionType'',
							COUNT(DISTINCT (
								CASE WHEN '''+@InteractionType+''' = ''CALL'' 
								THEN HDI.[id] 
								ELSE HDI.[id] 
								-- ELSE CONCAT(HDI.[id], HDI.[endDate]) 
								END))											AS ''interactions'',
							COUNT(*)											AS ''details'',
							SUM(HDI.[rawChar])									AS ''rawChars''
						FROM #TMP_HD_INTERACTIONS HDI '
				SET @QUERY += @QUERY_JOIN
				SET @QUERY += ' WHERE HDI.[RN] = 1 ' + @QUERY_FILTER + ' GROUP BY HDI.[channel]'
				
			END
			/*
			ELSE IF @ResultType = 'DETAILED'
			BEGIN
				SET @QUERY = '
					SELECT HDI.[vcc]											AS ''vcc'',
							HDI.[channel]										AS ''interactionType'',
							HDI.[campaign]										AS ''campaign'',
							HDI.[id]											AS ''id'',
							HDI.[firstTakedAgent]								AS ''agent'',
							HDI.[initiative]									AS ''origin'',
							SUM(HDI.[rawChar])									AS ''rawChar''
						FROM #TMP_HD_INTERACTIONS HDI '
				SET @QUERY += @QUERY_JOIN
				
				SET @QUERY += ' WHERE HDI.[RN] = 1 ' + @QUERY_FILTER + '
					GROUP BY HDI.[vcc], HDI.[channel], HDI.[campaign], HDI.[id], HDI.[firstTakedAgent], HDI.[initiative]'
			END
			--EJECUTAR CONSULTA
			EXEC (@QUERY)
			
		END
		ELSE -- ENFOCADO A DETALLES
		BEGIN
			IF @ResultType = 'SUMMARIZED'
			BEGIN
			SET @QUERY = '
				SELECT HDI.[channel]											AS ''interactionType'',
						COUNT(DISTINCT (
							CASE WHEN '''+@InteractionType +'''= ''CALL'' 
							THEN HDI.[id] 
							ELSE CONCAT(HDI.[id], HDI.[endDate]) 
							END))												AS ''interactions'',
						COUNT(*)												AS ''details'',
						SUM(HDI.[rawChar])										AS ''rawChar''
					FROM #TMP_HD_INTERACTIONS HDI '
			SET @QUERY += @QUERY_JOIN
					
			SET @QUERY += '	WHERE HDI.[RN] = 1 ' + @QUERY_FILTER + ' GROUP BY HDI.[channel] '
			
			END
			*/
			ELSE IF @ResultType = 'DETAILED'
			BEGIN
			    SET @QUERY = ' SELECT HDI.* FROM #TMP_HD_INTERACTIONS HDI ' + @QUERY_JOIN +  ' WHERE HDI.[RN] = 1 ' + @QUERY_FILTER
			END 
			-- EJECUTAR CONSULTA
			print @QUERY
			EXEC (@QUERY)
		-- END
	END


	/*
	-- DECLARAMOS LAS CTE
	WITH CTE_AVAILABLE_BALANCE_REPORT AS (
		SELECT * FROM #TMP_HD_INTERACTIONS
	)
	select * from CTE_AVAILABLE_BALANCE_REPORT 
	*/
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetI6InteractionAudios] (
	@VCC					VARCHAR(50) = NULL,
	@InteractionId			VARCHAR(250) = NULL
) AS 
BEGIN
	-- RETORNAMOS LOS REGISTROS DE AMBAS TABLAS
	;WITH CTE_ALL_RECORDS AS (
		-- BUSCAMOS LOS AUDIOS DE LA INTERACCION
		SELECT DISTINCT [ID]									'id',
						[Section]								'section',
						[Part]									'part',
						[Agent]									'agentId',
						[Format]								'format'
			FROM [dbo].[Audio] A (NOLOCK)
			WHERE A.[VCC] = @VCC
				AND A.[ID] = @InteractionId
		UNION
		SELECT DISTINCT [IdInteraction]							'id',
						[InteractionSection]					'section',
						[InteractionPart]						'part',
						[IdAgent]								'agentId',
						[InteractionFormat]						'format'
			FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
			WHERE IEP.[Instance] = @VCC	
				AND IEP.[IdInteraction] = @InteractionId
	),
	CTE_SUMMARY AS (
		SELECT *,
				RN = ROW_NUMBER() OVER(PARTITION BY CTE_AR.[id],
													CTE_AR.[section],
													CTE_AR.[part]
											ORDER BY CTE_AR.[id] ASC, 
													CTE_AR.[section] ASC,
													CTE_AR.[part] ASC,
													CTE_AR.[agentId] DESC,
													CTE_AR.[format] DESC
										) 
			 FROM CTE_ALL_RECORDS CTE_AR
	)
		SELECT CTE_S.[id], 
				CTE_S.[section],
				CTE_S.[part],
				CTE_S.[agentId],
				CTE_S.[format] 
			FROM CTE_SUMMARY CTE_S
			WHERE RN = 1

	/*
	-- DECLARAMOS VARIABLES PARA ALMACENAR LAS CANTIDADES DE REGISTROS
	DECLARE @AUDIO_RECORDS INT = 0
	DECLARE @INTERACION_EVALUATION_PLAN_RECORDS INT = 0

	SELECT @AUDIO_RECORDS = COUNT(*)
		FROM [dbo].[Audio] A (NOLOCK)
		WHERE A.[VCC] = @VCC
			AND A.[ID] = @InteractionId
			AND A.[NeedI6Update] = 0

	SELECT @INTERACION_EVALUATION_PLAN_RECORDS = COUNT(DISTINCT CONCAT(IEP.[IdInteraction], '_', IEP.[InteractionSection], '_', IEP.[InteractionPart]))
		FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
		WHERE IEP.[Instance] = @VCC	
			AND IEP.[IdInteraction] = @InteractionId
			AND IEP.[NeedI6Update] = 0

	IF @AUDIO_RECORDS >= @INTERACION_EVALUATION_PLAN_RECORDS
	BEGIN

		-- BUSCAMOS LOS AUDIOS DE LA INTERACCION
		SELECT DISTINCT [ID]									'id',
						[Section]								'section',
						[Part]									'part',
						[Agent]									'agentId',
						[Format]								'format'
			FROM [dbo].[Audio] A (NOLOCK)
			WHERE A.[VCC] = @VCC
				AND A.[ID] = @InteractionId
				AND A.[NeedI6Update] = 0
			ORDER BY A.[Section] ASC,  A.[Part] ASC
	END
	ELSE
	BEGIN
		SELECT DISTINCT [IdInteraction]							'id',
						[InteractionSection]					'section',
						[InteractionPart]						'part',
						[IdAgent]								'agentId',
						[InteractionFormat]						'format'
			FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
			WHERE IEP.[Instance] = @VCC	
				AND IEP.[IdInteraction] = @InteractionId
				AND IEP.[NeedI6Update] = 0
			ORDER by IEP.[InteractionSection] ASC, IEP.[InteractionPart] ASC
	END
	*/
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetI6InteractionMessages] (
	@VCC					VARCHAR(50) = NULL,
	@InteractionId			VARCHAR(250) = NULL
) AS 
BEGIN
	-- RETORNAMOS LOS REGISTROS DE AMBAS TABLAS
	;WITH CTE_ALL_RECORDS AS (
		-- BUSCAMOS LOS REGISTROS DE LA INTERACCION
		SELECT DISTINCT [ID]									'id',
						[Campaign]								'campaign',
						[Folder]								'folder',
						[LastTakedAgent]						'agentId'
			FROM [dbo].[AsyncInteraction] AI (NOLOCK)
			WHERE AI.[VCC] = @VCC
				AND AI.[ID] = @InteractionId
		UNION
		SELECT DISTINCT [IdInteraction]							'id',
						[IdCampaign]							'campaign',
						[FormattedEndDate]						'folder',
						[IdAgent]								'agentId'
			FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
			WHERE IEP.[Instance] = @VCC	
				AND IEP.[IdInteraction] = @InteractionId
	),
	CTE_SUMMARY AS (
		SELECT *,
				RN = ROW_NUMBER() OVER(PARTITION BY CTE_AR.[id],
													CTE_AR.[folder],
													CTE_AR.[campaign],
													CTE_AR.[agentId]
											ORDER BY CTE_AR.[id] ASC, 
														CTE_AR.[folder] ASC,
														CTE_AR.[campaign] DESC,
														CTE_AR.[agentId] DESC
										) 
			 FROM CTE_ALL_RECORDS CTE_AR
	)
		SELECT CTE_S.[id], 
				CTE_S.[folder],
				CTE_S.[campaign],
				CTE_S.[agentId] 
			FROM CTE_SUMMARY CTE_S
			WHERE RN = 1

	/*
	-- DECLARAMOS VARIABLES PARA ALMACENAR LAS CANTIDADES DE REGISTROS
	DECLARE @MESSAGES_RECORDS INT = 0
	DECLARE @INTERACION_EVALUATION_PLAN_RECORDS INT = 0

	SELECT @MESSAGES_RECORDS = COUNT(*)
		FROM [dbo].[AsyncInteraction] AI (NOLOCK)
		WHERE AI.[VCC] = @VCC
			AND AI.[ID] = @InteractionId

	SELECT @INTERACION_EVALUATION_PLAN_RECORDS = COUNT(DISTINCT CONCAT(IEP.[IdInteraction], '_', IEP.[IdCampaign], '_', IEP.[FormattedEndDate]))
		FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
		WHERE IEP.[Instance] = @VCC	
			AND IEP.[IdInteraction] = @InteractionId

	IF @MESSAGES_RECORDS >= @INTERACION_EVALUATION_PLAN_RECORDS
	BEGIN
		-- BUSCAMOS LOS REGISTROS DE LA INTERACCION
		SELECT DISTINCT [ID]									'id',
						[Campaign]								'campaign',
						[Folder]								'folder',
						[LastTakedAgent]						'agentId'
			FROM [dbo].[AsyncInteraction] AI (NOLOCK)
			WHERE AI.[VCC] = @VCC
				AND AI.[ID] = @InteractionId
			ORDER BY AI.[Folder] ASC
	END 
	ELSE 
	BEGIN
		SELECT DISTINCT [IdInteraction]							'id',
						[IdCampaign]							'campaign',
						[FormattedEndDate]						'folder',
						[IdAgent]								'agentId'
			FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
			WHERE IEP.[Instance] = @VCC	
				AND IEP.[IdInteraction] = @InteractionId
			ORDER by IEP.[FormattedEndDate] ASC
	END
	*/
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetI6SyncInteractions]
(
	@VCC					VARCHAR(50) = NULL,
	@IP						VARCHAR(50),						-- IP DEL SERVIDOR PARA LA BASE I6
	@Type					INT,								-- FLAG PARA INDICAR SI EL SERVIDOR I6 SE ENCUENTRA LOCAL O REMOTO
	@InteractionType		VARCHAR(50) = NULL,
	@StartDate				DATETIME = '1980-01-01T00:00:00',
    @EndDate				DATETIME = '2050-01-01T23:59:59',
	@StartHour				VARCHAR(50) = NULL,
	@EndHour				VARCHAR(50) = NULL,
	@Campaigns				VARCHAR(MAX) = NULL,
 	@Agents					VARCHAR(MAX) = NULL,
	@DispositionCodes		VARCHAR(MAX) = NULL,
	@Origins				VARCHAR(MAX) = NULL,
	@MinDuration			INT = NULL,
	@MaxDuration			INT = NULL,
	@InteractionIds         VARCHAR(MAX) = NULL,
    @Evaluated              VARCHAR(MAX) = NULL,
	@UniqueIds				BIT = 0,							-- ESTE FLAG TIENE COMO PROPOSITO RETORNAR LA LISTA CONSOLIDAD POR IDS O EL TOTAL DEL LISTADO
	@ResultType				VARCHAR(50) = NULL,				-- VARIABLE PARA DETERMINAR SI LA BUSQUEDA SERA SOBRE SOBRE DETALLES O ID (DETAILED, SUMMARIZED)
	@ContactAddress			VARCHAR(300) = NULL
    
)
AS
BEGIN
	-- INFO: EN CHAT SE CREAN UN ID EN CADA INTERACCION(EJM: CHATS DE ATENCION EN LAS WEB DE BANCOS), LOS DEMAS CANALES ID + ENDDATE
	
	-- SETTEAMOS VALORES MINIMOS PARA LAS HORAS DE INICIO Y FIN
	IF (@StartHour IS NOT NULL AND @StartHour = '') SET @StartHour = '00:00:00'
	IF (@EndHour IS NOT NULL AND @EndHour = '') SET @EndHour = '23:59:59'

	-- DECLARAMOS UNA VARIABLE PARA IDENTIFICAR SI EL SP ES LLAMADO DEL SISTEMA O POR OTRO SP
	DECLARE @IS_NESTED BIT = 0

	-- DECLARAMOASUN CADENA PARA EL QUERY FINAL
	DECLARE @SQL_SELECT VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''
    DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_FIXED_PART     VARCHAR(MAX) = ''
	
	-- DECLARAMOS VARIABLES PARA NOS NOMBRES DE LAS TABLAS
	DECLARE @HISTORICAL_DATA_I6_BD NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_APPSTORE_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_CALL_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_CHAT_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_FACEBOOK_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_GOOGLEPLAY_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_INSTAGRAM_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_MAIL_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_TWITTER_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_WEBCONTACT_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_WHATSAPP_TABLE NVARCHAR(100)
	DECLARE @HISTORICAL_DATA_I6_YOUTUBE_TABLE NVARCHAR(100)

	-- DEFINIMOS EL NOMBRE DE LA BD
	SET @HISTORICAL_DATA_I6_BD = '[HistoricalDataI6]' -- BD PARA DESARROLLO
	-- SET @HISTORICAL_DATA_I6_BD = '[HistoricalDataI6]'

	-- EL PARAMETRO TYPE DEFINE SI LA BD HISTORICALDATAI6 SE ENCUENTRA LOCAL O REMOTO
	IF (@Type = 0)  
	BEGIN  
		SET @HISTORICAL_DATA_I6_CALL_TABLE =			'[' + @Ip + ']' + '.' + @HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Call]'
	END
	ELSE IF(@Type = 1) 
	BEGIN
		SET @HISTORICAL_DATA_I6_CALL_TABLE =			@HISTORICAL_DATA_I6_BD + '.[dbo].[Detail_Call]'
	END  

	-- ARMAMOS EL QUERY PARA HACER LA CONSULTA
	SET @SQL_SELECT += '
		SELECT '

	-- ARMAMOS LA PARTE BASE DE FILTROS
	SET @FILTERS_FIXED_PART = ' WHERE HDI6.[Vcc] = ''' + @VCC + ''''
	
	SET @FILTERS_OPTIONAL_PART  =' AND HDI6.[EndDate] BETWEEN '''
    SET @FILTERS_OPTIONAL_PART += (SELECT CONVERT(VARCHAR(21), @StartDate, 127)) 
    SET @FILTERS_OPTIONAL_PART  +=''' AND '''  
    SET @FILTERS_OPTIONAL_PART  +=(SELECT CONVERT(VARCHAR(21), @EndDate, 127)) + ''' '
    

	-- DEPENDIENDO SI LA HORA DE FIN ES MAYOR A LA DE INICIO O NO DAMOS FORMA AL FILTRO DE HORA
	IF (CAST(@StartHour AS TIME) <= CAST(@EndHour AS TIME)) SET @FILTERS_OPTIONAL_PART += 'AND CAST(HDI6.[EndDate] AS TIME) BETWEEN ''' + @StartHour + ''' AND ''' + @EndHour + ''' ';
	IF (CAST(@StartHour AS TIME) > CAST(@EndHour AS TIME)) SET @FILTERS_OPTIONAL_PART += 'AND CAST(HDI6.[EndDate] AS TIME) NOT BETWEEN ''' + @EndHour + ''' AND ''' + @StartHour + ''' ';
		
	IF (@Campaigns IS NOT NULL AND @Campaigns != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Campaign] IN (SELECT VALUE FROM [dbo].[split](''' + @Campaigns + ''','','')) '
	
	IF (@DispositionCodes IS NOT NULL AND @DispositionCodes != '') 
		SET @FILTERS_OPTIONAL_PART += 'AND (
								(HDI6.[DispositionTreePath] IN (SELECT VALUE FROM [dbo].[split](''' + @DispositionCodes + ''','',''))) 
								OR
								(HDI6.[DispositionCode] IN (SELECT VALUE FROM [dbo].[split](''' + @DispositionCodes + ''','','')))
							)'

	-- ARMAMOS EL ORDER BY
	SET @SQL_EXTRA += 'ORDER BY [ID], [EndDate] DESC'

	IF (@InteractionType = 'CALL')
	BEGIN
		-- SETTEAMOS LA TABLA DE BUSQUEDA
		SET @HISTORICAL_DATA_I6_TABLE = @HISTORICAL_DATA_I6_CALL_TABLE

		-- ARMAMOS EL SELECT A LA TABLA DETAIL_CHAT
		SET @SQL_SELECT += '
								[Vcc]														''vcc'',
								[Id] + ''_0_0''												''ID'',
								[Id]														''InteractionId'',
								0															''Section'',
								0															''Part'',
								''CALL''													''Channel'',
								[ContactAddress]											''Number'',
								[ContactAddress]											''ContactAddress'',
								''''														''Gender'',
								ISNULL([HoldTime], 0)										''SilenceTime'',
								ISNULL([AudioDuration], 0)									''DurationTime'', -- ESTO SE DEBE CORREGIR
								-- ISNULL([AttentionTime], 0)								''AttentionTime'', -- SE AGREGO CAMPO PARA AMBOS EXISTAN
								ISNULL([AudioDuration], [AttentionTime])					''AttentionTime'', -- SE AGREGO CAMPO PARA TOMAR EL TIEMPO TOTAL DE TODOS LOS PARTS
								ISNULL([AudioRecords], 1)									''TotalParts'',
								NULL														''Category1'',
								NULL														''Category2'',
								NULL														''Category3'',
								[DispositionTreePath]										''Description'',
								[DispositionCode]											''DispositionCode'',
								[Campaign]													''Campaign'',
								[LastAgent]													''Agent'',
								NULL														''agentName'',
								CASE [Initiative] WHEN ''INBOUND'' THEN ''E''
													WHEN ''Inb.'' THEN ''E''
													WHEN ''OUTBOUND'' THEN ''S''
													WHEN ''Out.'' THEN ''S''
													ELSE NULL END							''Type'',
								CONVERT(VARCHAR, [StartDate], 120)							''StartDate'',
								CONVERT(VARCHAR, [EndDate], 120)							''EndDate'',
								NULL														''Format'' '
								
		-- POR LAS RESTRICCIONES DE SPEECH, FILTRAMOS SOLO INTERACCIONES QUE TENGA ID AGENTE DIFERENTE DE NULL
		SET @FILTERS_FIXED_PART += 'AND [IsTaken] = 1 
									AND [AudioRecords] >= 1 '

		IF (@Agents IS NOT NULL AND @Agents != '') SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[LastAgent] IN (SELECT VALUE FROM [dbo].[split](''' + @Agents + ''','','')) '

		-- PARA HACER EL FILTRO POR ORIGEN, REEMPLAZAMOS EN EL INPUT POR LOS VALORES DE LA TABLA
		IF (@Origins IS NOT NULL AND @Origins != '') 
		BEGIN
			SET @Origins = REPLACE(@Origins, 'E', 'INBOUND,Intb.')
			SET @Origins = REPLACE(@Origins, 'S', 'OUTBOUND,Out.')
			SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[Initiative] IN (SELECT VALUE FROM [dbo].[split](''' + @Origins + ''','','')) '
		END

		IF(@MinDuration IS NOT NULL) SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[AudioDuration] >= ' + (SELECT CONVERT(varchar(10), @MinDuration)) + ' '

		IF(@MaxDuration IS NOT NULL) SET @FILTERS_OPTIONAL_PART += 'AND HDI6.[AudioDuration] <= ' + (SELECT CONVERT(varchar(10), @MaxDuration)) + ' '
	END 

	IF @InteractionIds IS NOT NULL AND @InteractionIds <> ''
    BEGIN
        SET @FILTERS_OPTIONAL_PART = '';
	    SET @FILTERS_EXCLUDING_PART=' AND ID IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) ';
    END
	IF @ContactAddress IS NOT NULL AND @ContactAddress <> ''
    BEGIN
        SET @FILTERS_OPTIONAL_PART += '
				AND HDI6.ContactAddress LIKE ''%' + @ContactAddress + '%''
			'
    END
	-- AGREGAMOS EL CONTADOR DE FILAR AL SELECT
	SET @SQL_SELECT += '
								,RN = ROW_NUMBER() OVER(PARTITION BY [ID], CONVERT(VARCHAR, EndDate, 23) ORDER BY [EndDate] DESC) '

	-- ARMAMOS LA PARTE DE TABLAS 
	SET @SQL_FROM += '
			FROM ' + @HISTORICAL_DATA_I6_TABLE + ' HDI6 (NOLOCK) '

	-- UNIMOS LAS PARTES DEL QUERY
	SET @SQL_SELECT += @SQL_FROM + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART + @SQL_EXTRA
	
	-- VERIFICAMOS SI EXISTE LA TABLA TEMPORAL #TMP_HD_INTERACTIONS
	IF OBJECT_ID('TEMPDB..#TMP_HD_INTERACTIONS') IS NULL
	BEGIN
		-- CREAMOS UNA TABLA TEMPORAL PARA INSERTAR LOS REGISTROS DE LA HDI6
		CREATE TABLE #TMP_HD_INTERACTIONS
		(
			[VCC] VARCHAR(50), 
			[ID] VARCHAR(300),  
			[InteractionId] VARCHAR(250), 
			[Section] INT,
			[Part] INT,
			[Channel] VARCHAR(50), 
			[Number] VARCHAR(50),
			[ContactAddress] VARCHAR(50), 
			[Gender] VARCHAR(50), 
			[SilenceTime] DECIMAL(10, 2),
			[DurationTime] DECIMAL(10, 2),
			[AttentionTime] DECIMAL(10, 2),
			[TotalParts] INT,
			[Category1] VARCHAR(50),
			[Category2] VARCHAR(50),
			[Category3] VARCHAR(50),
			[Description] VARCHAR(50),
			[DispositionCode] VARCHAR(250), 
			[Campaign] VARCHAR(50), 
			[Agent] VARCHAR(50), 
			[AgentName] VARCHAR(250), 
			[Type] VARCHAR(50), 
			[StartDate] VARCHAR(100),
			[EndDate] VARCHAR(100),
			[Format] VARCHAR(50),
			RN INT
		)

		-- REAFIRMAMOS QUE EL SP NO ESTA ANIDADO EN OTRO
		SET @IS_NESTED = 0

		-- PRINT 'SP NO ANIDADO'
	END
	ELSE
	BEGIN
		-- SI LA TABLA TEMPORAL YA EXISTIA, EL SP ESTA ANIDADO EN OTRO (P.E.: [dbo].[GetAsyncBillingInteractions])
		SET @IS_NESTED = 1

		-- PRINT 'SP ANIDADO'
	END 

	-- INSERTAMOS EL RESULTADO EN LA TABLA TEMPORAL
	INSERT INTO #TMP_HD_INTERACTIONS
		-- EJECUTAMOS LA CONSULTA
		EXEC (@SQL_SELECT)

	-- VARIABLES PARA TRABAJAR CON LA TABLA TEMPORAL AL FINAL DEL SP
    DECLARE @QUERY                  VARCHAR(MAX) = ''
    DECLARE @QUERY_FILTER           VARCHAR(MAX) = ''
    DECLARE @QUERY_JOIN             VARCHAR(MAX) = ''
	
	-- FILTRO POR INTERACCIONES EVALUADAS O NO
	IF (@Evaluated IS NOT NULL AND @Evaluated <> '')
	    BEGIN 
		DECLARE @EvaluatedTable TABLE (Val BIT)

		INSERT INTO @EvaluatedTable SELECT Value FROM [dbo].[split]( @Evaluated ,',')

		-- SOLAMENTE SI ES SI O NO
		IF ((SELECT COUNT(Val) FROM @EvaluatedTable) = 1 )
		BEGIN
			IF((SELECT Val FROM @EvaluatedTable WHERE Val = 1) IS NOT NULL) -- Si  es Si
			BEGIN 
				SET @QUERY_JOIN = ' JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) ON (AEP.[IdInteraction] = HDI.[id] AND AEP.[Completed]=1 ) '
			END
		ELSE  --Si es No
           BEGIN
               SET @QUERY_JOIN = ' LEFT JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) ON (AEP.[IdInteraction] = HDI.[id] AND AEP.[Completed]=1 ) '
               SET @QUERY_FILTER = ' AND AEP.[IdInteraction] IS NULL '
           END
       END
       
	 END
	-- FIN FILTRO INTERACCIONES EVALUADAS


	
	-- SI EL SP NO ESTA ANIDADO, HACEMOS SELECT AL RESULTADO A MOSTRAR
	IF @IS_NESTED = 0
	BEGIN
		-- HACEMOS EL SELECT SEGUN EL TIPO DE BUSQUEDA SETTEADO
		IF @UniqueIds = 1 -- ENFOCADO A INTERACCIONES
		BEGIN
			IF @ResultType = 'SUMMARIZED'
			BEGIN
				SET @QUERY = '
					SELECT HDI.[channel]										''interactionType'',
							COUNT(DISTINCT HDI.[id])							''interactions'',
							SUM(HDI.[totalParts])								''details'',
							SUM(HDI.[attentionTime])							''attentionTime''
						FROM #TMP_HD_INTERACTIONS HDI '
				SET @QUERY += @QUERY_JOIN
				SET @QUERY += ' WHERE HDI.[RN] = 1 ' + @QUERY_FILTER + ' GROUP BY HDI.[channel]'
				
			END
			ELSE IF @ResultType = 'DETAILED'
			BEGIN
				SET @QUERY = ' SELECT HDI.* FROM #TMP_HD_INTERACTIONS HDI ' + @QUERY_JOIN +  ' WHERE HDI.[RN] = 1 ' + @QUERY_FILTER
				/*
				SET @QUERY = '
					SELECT HDI.[vcc]											''vcc'',
							HDI.[channel]										''interactionType'',
							HDI.[campaign]										''campaign'',
							HDI.[id]											''id'',
							HDI.[firstTakedAgent]								''agent'',
							HDI.[initiative]									''origin''
						FROM #TMP_HD_INTERACTIONS HDI '
				SET @QUERY += @QUERY_JOIN
				
				SET @QUERY += ' WHERE HDI.[RN] = 1 ' + @QUERY_FILTER + '
					GROUP BY HDI.[vcc], HDI.[channel], HDI.[campaign], HDI.[id], HDI.[firstTakedAgent], HDI.[initiative]'
				*/
			END
			--EJECUTAR CONSULTA
			EXEC (@QUERY)
			
		END
		ELSE -- ENFOCADO A DETALLES
		BEGIN
			IF @ResultType = 'SUMMARIZED'
			BEGIN
			SET @QUERY = '
				SELECT HDI.[channel]											AS ''interactionType'',
						COUNT(DISTINCT HDI.[id])								AS ''interactions'',
						SUM(HDI.[totalParts])									AS ''details'',
						SUM(HDI.[attentionTime])							''attentionTime''
					FROM #TMP_HD_INTERACTIONS HDI '
			SET @QUERY += @QUERY_JOIN
			SET @QUERY += '	WHERE HDI.[RN] = 1 ' + @QUERY_FILTER + ' GROUP BY HDI.[channel] '
			
			END
			ELSE IF @ResultType = 'DETAILED'
			BEGIN
			    SET @QUERY = ' SELECT HDI.* FROM #TMP_HD_INTERACTIONS HDI ' + @QUERY_JOIN +  ' WHERE HDI.[RN] = 1 ' + @QUERY_FILTER
			END 
		
			-- EJECUTAR CONSULTA
			EXEC (@QUERY)
		END
	END
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--[dbo].[GetIdathaAuthByRealm] 'system', 'system'
CREATE PROCEDURE [dbo].[GetIdathaAuthByRealm]
(
	@VCC				VARCHAR(50),
	@Realm				VARCHAR(100)
)
AS
BEGIN
	SET NOCOUNT ON
	SELECT
	[VCC]					'vcc',
	[Realm]					'realm',
	[UserName]				'userName',
	[Password]				'password'
	FROM IdathaAuth (NOLOCK)
	WHERE [VCC] = @VCC 
	AND [Realm] = @Realm
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetInstanceByDomain](
	@Domain varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]				'vcc'
	FROM [Domain] (NOLOCK)
	WHERE [Domain] = @Domain  

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- SP SPEECHANALYTICS: RECUPERA LOS VCC DE LA MMPRODAT
CREATE PROCEDURE [dbo].[GetIntegratedVCCs] (
   	@VCC VARCHAR(50) = NULL,
	@IP VARCHAR(50), 
	@Type INT
) 
AS 
BEGIN 
	-- DECLARAMOS CADENAS DE TEXTO PARA EL QUERY FINAL
	DECLARE @SQL_SELECT_0 VARCHAR(MAX) = ''
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''

	-- DECLARAMOS VARIABLES PARA NOS NOMBRES DE LAS TABLAS
	DECLARE @VCC_SERVICE_TABLE NVARCHAR(100)
	DECLARE @SYSTEM_SERVICE_TABLE NVARCHAR(100)

	-- EL PARAMETRO TYPE DEFINE SI LA BD ALLEGRO SE ENCUENTRA LOCAL O REMOTO
	IF (@Type = 0)  
	BEGIN  
		SET @VCC_SERVICE_TABLE = '[' + @Ip + ']' + '.[mmprodat].[dbo].[VCCService]'
		SET @SYSTEM_SERVICE_TABLE = '[' + @Ip + ']' + '.[mmprodat].[dbo].[SystemService]'
	END
	ELSE IF(@Type = 1) 
	BEGIN
		SET @VCC_SERVICE_TABLE = '[mmprodat].[dbo].[VCCService]';
		SET @SYSTEM_SERVICE_TABLE = '[mmprodat].[dbo].[SystemService]';
	END  

	-- ARMAMOS EL QUERY PARA HACER LA CONSULTA
	SET @SQL_SELECT_1 += '
		SELECT '

	SET @SQL_SELECT_1 += '
				VS.[VCC] ''vcc'',
				SUM(CASE WHEN SS.[Type] = ''OMNICHANNELINTEGRATION'' THEN 1 ELSE 0 END) ''hasOmnichannel'' -- 1: OCC OR 0: ALLEGRO ' 

	-- ARMAMOS LA PARTE DE TABLAS 
	SET @SQL_FROM += '
			FROM ' + @VCC_SERVICE_TABLE + ' VS (NOLOCK)
			LEFT JOIN ' + @SYSTEM_SERVICE_TABLE + ' SS (NOLOCK) 
				ON SS.[Id] = VS.[ServiceId] '

	-- ARMAMOS LA PARTE DE FILTROS
	SET @SQL_FILTER += '' 

	-- ARMAMOS LA PARTE DE EXTRAS
	SET @SQL_EXTRA += '
			GROUP BY VS.[VCC]
			ORDER BY VS.[VCC] ASC '

	-- UNIMOS LAS PARTES DEL QUERY
	SET @SQL_SELECT_1 += @SQL_FROM + @SQL_FILTER + @SQL_EXTRA

	-- EJECUTAMOS LA CONSULTA
	EXEC (@SQL_SELECT_1)
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetIntegrationUsers] '38.127.113.216', 1
CREATE PROCEDURE [dbo].[GetIntegrationUsers]
(
@Ip Varchar(50), @Type Int  
)
AS
BEGIN

	DECLARE @Sql NVARCHAR(MAX);

	IF(@Type = 0)  
	BEGIN  
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[VCC]							    ''vcc'',
			[Id]								''id'',
			NULL								''email'',
			[Name] 								''firstname'',
			NULL								''lastname'',
			[Password]							''password'',
			[UserGroup] 						''role'',
			[Country] 							''country'',
			[LanguageId] 						''language'',
			[TimeZoneId] 						''timeZone'',
			NULL								''preferences'',
			0									''isSystemUser'',
			GETUTCDATE()						''createdDate'',
			''admin''							''createdByUserId'',
			GETUTCDATE()						''lastModifiedDate'',
			''admin''							''lastModifiedByUserId''
		FROM  [' + @Ip + ']' + '.mmprodat.dbo.[User] WITH (NOLOCK)
		WHERE
		UserGroup = ''agent''
		OR UserGroup = ''qualityauditor'''
		SET @Sql += ' 
		SET NOCOUNT OFF'
END  
ELSE IF(@Type = 1) 
BEGIN  
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[VCC]							    ''vcc'',
			[Id]								''id'',
			NULL								''email'',
			[Name] 								''firstname'',
			NULL								''lastname'',
			[Password]							''password'',
			[UserGroup] 						''role'',
			[Country] 							''country'',
			[LanguageId] 						''language'',
			[TimeZoneId] 						''timeZone'',
			NULL								''preferences'',
			0									''isSystemUser'',
			GETUTCDATE()						''createdDate'',
			''admin''							''createdByUserId'',
			GETUTCDATE()						''lastModifiedDate'',
			''admin''							''lastModifiedByUserId''
		FROM mmprodat.dbo.[User] (NOLOCK)
		WHERE
		UserGroup = ''agent''
		OR UserGroup = ''qualityauditor'''
		SET @Sql += ' 
		SET NOCOUNT OFF'
END  
ELSE  
BEGIN 
		SET @Sql = 'SELECT 1 AS ''result'''  
END

EXEC (@Sql)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetIntegrationVCCs] '38.127.113.216', 1, '5.5'
CREATE PROCEDURE [dbo].[GetIntegrationVCCs]
(
@Ip Varchar(50), @Type Int, @Ambiente Varchar(50)
)
AS
BEGIN

	DECLARE @Sql NVARCHAR(MAX);

	IF(@Type = 0)  
	BEGIN  
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[Id]							''id'',
			[Description]					''description'',
			NULL							''country'',
			NULL							''language'',
			NULL							''timeZone'',
			GETUTCDATE()					''createdDate'',
			''admin''						''createdByUserId'',
			GETUTCDATE()					''lastModifiedDate'',
			''admin''						''lastModifiedByUserId''
		FROM  [' + @Ip + ']' + '.mmprodat.dbo.[virtualcontactcenter] WITH (NOLOCK)
		WHERE [Id] != ''system'''
		IF(@Ambiente = '5.5')
		BEGIN
			SET @Sql += '
		AND [Id] NOT IN (SELECT DISTINCT Id FROM mmprodat.dbo.[virtualcontactcenter] VCC WITH (NOLOCK)
		INNER JOIN mmprodat..[VCCService] S WITH (NOLOCK) on VCC.Id = S.VCC
		WHERE S.ServiceId LIKE ''%OMNICHANNEL%''
		)
		'
		END
		IF(@Ambiente = 'INTEGRATED')
		BEGIN
			SET @Sql += '
		AND [Id] IN (SELECT DISTINCT Id FROM mmprodat.dbo.[virtualcontactcenter] VCC WITH (NOLOCK)
		INNER JOIN mmprodat..[VCCService] S WITH (NOLOCK) on VCC.Id = S.VCC
		WHERE S.ServiceId LIKE ''%OMNICHANNEL%''
		)
		'
		END
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE IF(@Type = 1) 
	BEGIN
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[Id]							''id'',
			[Description]					''description'',
			NULL							''country'',
			NULL							''language'',
			NULL							''timeZone'',
			GETUTCDATE()					''createdDate'',
			''admin''						''createdByUserId'',
			GETUTCDATE()					''lastModifiedDate'',
			''admin''						''lastModifiedByUserId''
		FROM mmprodat.dbo.[virtualcontactcenter] (NOLOCK)
		WHERE [Id] != ''system'''
		IF(@Ambiente = '5.5')
		BEGIN
			SET @Sql += '
		AND [Id] NOT IN (SELECT DISTINCT Id FROM mmprodat.dbo.[virtualcontactcenter] VCC WITH (NOLOCK)
		INNER JOIN mmprodat..[VCCService] S WITH (NOLOCK) on VCC.Id = S.VCC
		WHERE S.ServiceId LIKE ''%OMNICHANNEL%''
		)
		'
		END
		IF(@Ambiente = 'INTEGRATED')
		BEGIN
			SET @Sql += '
		AND [Id] IN (SELECT DISTINCT Id FROM mmprodat.dbo.[virtualcontactcenter] VCC WITH (NOLOCK)
		INNER JOIN mmprodat..[VCCService] S WITH (NOLOCK) on VCC.Id = S.VCC
		WHERE S.ServiceId LIKE ''%OMNICHANNEL%''
		)
		'
		END
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE  
	BEGIN  
		SET @Sql = 'SELECT 1 AS ''result'''  
	END

	EXEC (@Sql)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetIntent] 'allegro','test0'
CREATE PROCEDURE [dbo].[GetIntent] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]						    'name',
		[IntentId]						'intentId',
		[Color]							'color',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Intent] (NOLOCK)
	WHERE [VCC] = CASE WHEN ISNULL(@VCC, '') = '' THEN [VCC] ELSE @VCC END
	AND (
		[Name] = @Name 
		OR
		[IntentId] = @Name
		)
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetIntentByAppId] (
	@VCC		VARCHAR(50) = NULL,
	@AppId		VARCHAR(50) = NULL,
	@Name		VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]						    'name',
		[IntentId]						'intentId',
		[Color]							'color',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Intent] (NOLOCK)
	WHERE [VCC] = CASE WHEN ISNULL(@VCC, '') = '' THEN [VCC] ELSE @VCC END
	AND [AppId] = @AppId
	AND [Name] = @Name
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetIntents] (
	@VCC Varchar(50) = NULL, 
	@AppId VARCHAR(100) = NULL,
	@SearchCriteria Varchar(50) = NULL, 
	@SearchExpression Varchar(50) = NULL,
	@Intents NVARCHAR(MAX)
	)
AS
BEGIN
	IF NOT EXISTS(SELECT 1 FROM dbo.[Intent] WHERE [VCC] = @VCC AND [AppId] = @AppId)
	SET @VCC = 'system'

	DECLARE @COGNITIVE_ENGINE VARCHAR(5) = NULL
	DECLARE @SQL NVARCHAR(MAX)

	-- DETERMINAMOS EL MOTOR COGNITIVO DEL VCC
	SELECT @COGNITIVE_ENGINE = N.[CognitiveEngine]
		FROM [dbo].[NLU] N (NOLOCK)
		WHERE N.[VCC] = @VCC
			AND N.[AppId] = @AppId
	
	SET @SQL = '
	SET NOCOUNT ON
	SELECT  IT.[Name]						    ''name'',
	        IT.[AppId]                          ''appId'',
			IT.[IntentId]						''intentId'',
			IT.[Color]							''color'',
			IT.[CreatedDate]					''createdDate'',
			IT.[CreatedByUserId]				''createdByUserId'',
			IT.[LastModifiedDate]				''lastModifiedDate'',
			IT.[LastModifiedByUserId]			''lastModifiedByUserId'',
			IT.[Status]                         ''status'',
			IT.[StatusCode]                     ''statusCode'',
			COUNT(U.[IntentId])				    ''totalUtterances''
		FROM [dbo].[Intent] IT (NOLOCK) 
		LEFT JOIN [dbo].[Utterance] U (NOLOCK) 
			ON IT.[VCC] = U.[VCC]
				AND IT.[IntentId] = U.[IntentId]
		WHERE IT.[VCC] = ''' + @VCC + '''
			AND IT.[AppId] = ''' + @AppId + ''' '

	-- PARA NLUS DE LUIS, FILTRAMOS SEGUN EL INPUT
	IF @COGNITIVE_ENGINE = 'LUIS'
	BEGIN
		SET @SQL +=	'
			AND IT.[IntentId] IN (''' + @Intents + ''') '
	END

	IF @SearchCriteria IS NOT NULL AND @SearchExpression IS NOT NULL
	BEGIN
		SET @SQL +=	'
			AND ISNULL(IT.[Name], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END '
	END

	SET @SQL +=	'
		GROUP BY IT.[IntentId]
				,IT.[Name]
				,IT.[AppId]
				,IT.[Color]
				,IT.[CreatedDate]
				,IT.[CreatedByUserId]
				,IT.[LastModifiedDate]
				,IT.[LastModifiedByUserId]
				,IT.[Status]  
				,IT.[StatusCode] 
		ORDER BY IT.[LastModifiedDate] ASC, IT.[Name] DESC '

	SET @SQL += ' 
	SET NOCOUNT OFF'
	--print @SQL
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetInteractionAgents] 
(
	@VCC					VARCHAR(50) = NULL
) 
AS
BEGIN

SELECT DISTINCT Agent, 0 AS status FROM [dbo].[Audio]  (NOLOCK) WHERE VCC=@VCC;
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTE SP RETORNA LAS CAMPANAS ASOCIADAS A LAS INTERACCIONES
CREATE PROCEDURE [dbo].[GetInteractionCampaigns] 
(
	@VCC					VARCHAR(50) = NULL
) 
AS
BEGIN

SELECT DISTINCT Campaign FROM [dbo].[Audio]  (NOLOCK) WHERE VCC=@VCC;
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetInteractionQueryResults] (
	@VCC VARCHAR(50) = NULL,
	@TransactionType VARCHAR(50) = NULL, 
	@TransactionId VARCHAR(50) = NULL, 
	@InteractionType VARCHAR(10) = NULL,  
	@InteractionId VARCHAR(250) = NULL, 
	@Campaign VARCHAR(100) = NULL,  
	@Folder VARCHAR(50) = NULL
) 
AS 
BEGIN 
    SET NOCOUNT ON

	-- HACEMOS LA BUQUEDA SEGUN EL TIPO DE INTERACCION
	IF @InteractionType = 'CALL'
		BEGIN
			SELECT IQR.[InteractionId]				'interactionId',
					IQR.[Section]					'section',
					IQR.[Part]						'part',
					IQR.[InteractionType]			'interactionType',
					IQR.[BatchType]					'transactionType',
					IQR.[BatchTypeId]				'transactionId',
					IQR.[BatchTypeName]				'transactionName',
					IQR.[AnalysisType]				'analysisType',
					IQR.[QueryId]					'queryId',
					IQR.[QueryName]					'queryName',
					IQR.[QueryValue]				'queryValue',
					IQR.[QueryData]					'queryData',
					IQR.[Matches]					'matches',
					IQR.[Details]					'details'
				FROM [dbo].[InteractionQueryResult] IQR (NOLOCK)
				WHERE IQR.[VCC] = @VCC
					AND IQR.[InteractionType] = @InteractionType
					AND (
						IQR.[InteractionId] = @InteractionId		-- GUID BASICO DE 32 CARACTERES APROXIMADAMENTE
						OR
						IQR.[TranscriptionId] = @InteractionId		-- GUID BASICO + SECTION + PART
					)
		END
		ELSE
		BEGIN
			SELECT IQR.[InteractionId]				'interactionId',
					IQR.[Section]					'section',
					IQR.[Part]						'part',
					IQR.[InteractionType]			'interactionType',
					IQR.[BatchType]					'transactionType',
					IQR.[BatchTypeId]				'transactionId',
					IQR.[BatchTypeName]				'transactionName',
					IQR.[AnalysisType]				'analysisType',
					IQR.[QueryId]					'queryId',
					IQR.[QueryName]					'queryName',
					IQR.[QueryValue]				'queryValue',
					IQR.[QueryData]					'queryData',
					IQR.[Matches]					'matches',
					IQR.[Details]					'details'
				FROM [dbo].[InteractionQueryResult] IQR (NOLOCK)
				WHERE IQR.[VCC] = @VCC
					AND IQR.[InteractionType] = @InteractionType
					AND (
						IQR.[InteractionId] = @InteractionId
						OR
						IQR.[TranscriptionId] = CONCAT(@InteractionType, '_', @Campaign, '_', @Folder, '_', REPLACE(@InteractionId, ':', '_'))
					)
		END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetInteractionsByType] (
	@Instance VARCHAR(50) = NULL,
	@InitialDate DATETIME = NULL,
	@EndDate DATETIME = NULL,
	@IdCampaign VARCHAR(MAX) = 'GetInteractionsByType',
	@IdEvaluationFormat VARCHAR(MAX) = 'GetInteractionsByType',
	@IdAgent VARCHAR(MAX) = 'GetInteractionsByType',
	@IdWorkGroup VARCHAR(MAX) = 'GetInteractionsByType',
	@IdAttentionLevel VARCHAR(MAX) = 'GetInteractionsByType',
	@IdEvaluationPlan VARCHAR(MAX) = 'GetInteractionsByType',
	@IdGroup VARCHAR(MAX) = 'GetInteractionsByType',
	@IdSubGroup VARCHAR(MAX) = 'GetInteractionsByType',
	@IdAbility VARCHAR(MAX) = 'GetInteractionsByType',
	@IdAuditor VARCHAR(MAX) = 'GetInteractionsByType'
)
AS
BEGIN
	SET NOCOUNT ON;

	WITH 
		-- DECLARAMOS UNA CTE PARA DEPOSITAR LOS RESULTADOS DE LA PRIMERA CONSULTA PARA TOTAL DE EVALUACIONES Y TOTAL DE EVALUACIONES APROBADAS
		CteEvaluationResultSummary AS (
			SELECT 
					ER.IdCampaign,
					ER.IdInteraction,
					ER.IdAgent,
					ER.MinQualificationAcceptable,
					SUM(ER.Qualification) Qualification,
					AEP.Score Score, 
					ER.IdEvaluator,
					ER.IdEvaluationPlan,
					ER.InteractionType
				FROM
					[EvaluationResult] ER
				LEFT JOIN [AuditorEvaluationPlan] AEP
					ON AEP.[Instance] = ER.[Instance]
						AND AEP.[Instance] = ER.[Instance]
						AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]
						AND AEP.[IdInteraction] = ER.[IdInteraction]
						AND AEP.[IdAuditor] = ER.[IdAuditor]
				WHERE
					ER.Instance = @Instance
					AND ER.EvaluationDate BETWEEN @InitialDate AND @EndDate
					AND CASE WHEN @IdAgent = 'GetInteractionsByType' THEN 'GetInteractionsByType' ELSE ER.IdAgent END IN (SELECT Value FROM [dbo].[split](@IdAgent,','))
					AND CASE WHEN @IdCampaign = 'GetInteractionsByType' THEN 'GetInteractionsByType' ELSE ER.IdCampaign END IN (SELECT Value FROM [dbo].[split](@IdCampaign,','))
					AND CASE WHEN @IdEvaluationFormat = 'GetInteractionsByType' THEN 'GetInteractionsByType' ELSE ER.IdEvaluationFormat END IN (SELECT Value FROM [dbo].[split](@IdEvaluationFormat,','))
					AND CASE WHEN @IdEvaluationPlan = 'GetInteractionsByType' THEN 'GetInteractionsByType' ELSE ER.IdEvaluationPlan END IN (SELECT Value FROM [dbo].[split](@IdEvaluationPlan,','))
					AND ER.IdAuditor LIKE CASE WHEN @IdAuditor = 'GetInteractionsByType' THEN '%' ELSE 'SPEECH' END AND ER.IdAuditor <> CASE WHEN @IdAuditor = 'GetInteractionsByType' THEN 'SPEECH' ELSE 'B_I_G_C_H_E_E_S_E' END
				GROUP BY
					ER.IdCampaign,
					ER.IdInteraction,
					ER.IdAgent,
					ER.MinQualificationAcceptable,
					ER.IdEvaluator,
					ER.IdEvaluationPlan,
					ER.InteractionType,
					AEP.Score
		)

	SELECT
			EvaluationsByType.InteractionType,
			EvaluationsByType.TotalEvaluations,
			ApprovedEvaluationsByType.ApprovedEvaluations
		FROM
			(
				SELECT
					ApprovedEvaluationsByTypeAux.InteractionType,
					SUM(ApprovedEvaluationsByTypeAux.Approved) ApprovedEvaluations
				FROM
					(
						SELECT
							*,
							-- 20191205: AL EVALUAR DIRECTAMENTE SOBRE LA NOTA DE LA INTERACCION, PASAMOS POR ALTO EL CASO DE PENALIZACION EN LA EVALUACION QUE PUEDE 
							--  AFECTAR AL SUBGRUPO, GRUPO O INTERACCION; POR ESO COMPARAMOS A NIVEL DEL SCORE DE LA EVALUACION
							-- CASE WHEN EvaluationResultsGrouped.MinQualificationAcceptable <= EvaluationResultsGrouped.Qualification THEN 1 ELSE 0 END Approved
							CASE WHEN EvaluationResultsGrouped.MinQualificationAcceptable <= EvaluationResultsGrouped.Score THEN 1 ELSE 0 END Approved
						FROM 
							CteEvaluationResultSummary EvaluationResultsGrouped
					) ApprovedEvaluationsByTypeAux
				GROUP BY
					ApprovedEvaluationsByTypeAux.InteractionType
			) ApprovedEvaluationsByType
			,
			(
				SELECT
					EvaluationResultsGroupedAux.InteractionType,
					COUNT(*) TotalEvaluations
				FROM
					CteEvaluationResultSummary EvaluationResultsGroupedAux
				GROUP BY
					EvaluationResultsGroupedAux.InteractionType
			) EvaluationsByType
		WHERE
			ApprovedEvaluationsByType.InteractionType = EvaluationsByType.InteractionType

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--[GetInteractionsCognitive] 'allegro', 'test1', 'testnluprimero'
CREATE PROCEDURE [dbo].[GetInteractionsCognitive]
(
	@VCC				VARCHAR(50),
	@ProcessId			VARCHAR(50),
	@StudyId			VARCHAR(200)
)
AS
BEGIN
	SET NOCOUNT ON

		SELECT DISTINCT
			A.VCC				'vcc'
			,A.ProcessId		'processId'
			,A.[Name]			'studyId'
			,D.AudioId			'id'
			,D.Section			'section'
			,D.Part				'part'
			,B.TemplateId		'templateId'
			,C.nluId			'nluId'
			,C.appId			'appId'
			,E.[FileName]		'fileName'
		FROM [dbo].[Study] A (NOLOCK)
		INNER JOIN [dbo].[TemplateThemeItem] B (NOLOCK) 
			ON A.VCC = B.VCC 
				AND A.[TemplateId] = B.[TemplateId]
		INNER JOIN (SELECT DISTINCT
				A.VCC 'vcc'
				,A.[Name] + '|'+ B.[Name] 'cognitiveId'
				,A.[Name] 'nluId'
				,A.AppId 'appId'
			FROM NLU A (NOLOCK)
			INNER JOIN Intent B (NOLOCK) 
				ON A.VCC = B.VCC
					AND B.AppId = A.AppId) C 
			ON A.VCC = C.VCC 
				AND B.ID = C.cognitiveId
		INNER JOIN [dbo].[ProcessAudio] D (NOLOCK) 
			ON A.VCC = D.VCC 
				AND A.ProcessId = D.ProcessId
		INNER JOIN [dbo].[Audio] E (NOLOCK) 
			ON A.VCC = E.VCC 
				AND D.AudioId = E.ID 
				AND D.Section = E.Section 
				AND D.Part = E.Part
		WHERE A.[VCC] = @VCC
			AND A.[ProcessId] = @ProcessId
			AND A.[Name] = @StudyId


	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetInteractionsEvaluationPlan] (
	@Instance Varchar(50) = NULL, 
	@IdEvaluationPlan Varchar(50) = NULL/*,
	@IdAuditor Varchar(50) = NULL*/
	)
AS
BEGIN
	SET NOCOUNT ON
	SELECT
		IEP.[IdInteraction]					'id',
		IEP.[IdCampaign]					'idCampaign',
		IEP.[IdCampaign]					'descriptionCampaign',
		IEP.[IdAgent]						'idAgent',
		IEP.[IdAgent]						'descriptionAgent',
		IEP.[InteractionDateStart]			'dateIni',
		IEP.[InteractionDateEnd]			'dateFin',
		IEP.[InteractionDuration]			'interactionTime',
		/*IEP.[Evaluated]					'evaluated',*/
		IEP.[InteractionType]				'interactionType',
		IEP.[ManagementResult]				'dispositionCode',
		IEP.[ContactId]						'contactName',
		IEP.[InteractionOrigin]				'origin',
		IEP.[ContactAddress]				'contactAddress'
	FROM [InteractionEvaluationPlan] IEP (NOLOCK)
	WHERE [Instance] = @Instance
		AND [IdEvaluationPlan] = @IdEvaluationPlan
	ORDER BY IEP.[CreatedDate],
				IEP.[InteractionDateEnd] DESC,
				IEP.[IdEvaluationPlan],
				IEP.[IdInteraction]

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetInteractionsEvaluationPlanWithoutFilters] (
	@Instance Varchar(50) = NULL,
	@IdEvaluationPlan Varchar(50) = NULL
)
AS
BEGIN
	-- DEPRECATED: NO SE USA EN NINGUN SERVICIO, SE REMOVERA
	SET NOCOUNT ON

 	SELECT
		ae.[IdInteraction]				'id',
		ae.[IdCampaign]					'idCampaign',
		ae.[IdCampaign]					'descriptionCampaign',
		ae.[IdAgent]					'idAgent',
		ae.[IdAgent]					'descriptionAgent',
		ae.[InteractionDate]			'dateIni',
		ae.[InteractionDateEnd]			'dateFin',
		ae.[InteractionDuration]		'interactionTime',
		ae.[Completed]					'evaluated',
		ae.[InteractionType]			'interactionType',
		null							'dispositionCode',
		null							'address',
		null							'origin',
		ae.[Completed]					'completed'
	FROM
	    AuditorEvaluationPlan ae(NOLOCK)
	WHERE
	    ae.Instance = @Instance
	    AND ae.IdEvaluationPlan=@IdEvaluationPlan

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetInteractionsResult]
(
	@VCC				VARCHAR(50),
	@ProcessId			VARCHAR(50),
	@Channels			VARCHAR(100)
)
AS
BEGIN
	SET NOCOUNT ON
	
	-- SELECCIONAMOS UN QUERY DEPENDIENDO DEL CANAL
	IF @Channels = 'CALL' 
	BEGIN
		-- HACEMOS LA BUSQUEDA PARA LAS INTERACCIONES SINCRONAS(CALL)
		SELECT 'CALL'																'interactionType', 
				COUNT(DISTINCT CONCAT(A.[ID], '_', A.[Section], '_', A.[Part]))		'quantity',
				ISNULL(SUM(A.[DurationTime]), 0)									'timeQuantity',
				COUNT(DISTINCT A.[ID])												'totalSyncInteractions',
				SUM(
					CASE WHEN A.[IsCancel] = 1 THEN A.[DurationTime] ELSE 0 END
				)																	'alreadyTranscribedTime',
				SUM(
					CASE WHEN A.[IsProcess] IS NULL THEN A.[DurationTime] ELSE 0 END 
				)																	'notTranscribedDurationTime',
				SUM(
					CASE WHEN (A.[IsProcess] = 1 AND A.[IsTranscribed] IS NULL AND A.[IsCancel] IS NULL)
					THEN A.[DurationTime] ELSE 0 END
				)																	'inProcessTime',
				SUM(
					CASE WHEN A.[IsCancel] = 1 THEN A.[DurationTime] ELSE 0 END
				)																	'cancelledTime'
				--COUNT(DISTINCT A.[ID])												'timeQuantity'
			FROM [dbo].[Process] P (NOLOCK)
			LEFT JOIN [dbo].[ProcessAudio] PA (NOLOCK)
				ON PA.[ProcessId] = P.[Name]
			LEFT JOIN [dbo].[Audio] A (NOLOCK)
				ON A.[ID] = PA.[AudioId]
					AND A.[Section] = PA.[Section]
					AND A.[Part] = PA.[Part]
			WHERE P.[VCC] = @VCC
				AND P.[Name] = @ProcessId
				AND A.[ID] IS NOT NULL

	END
	ELSE 
	BEGIN
		-- HACEMOS LA BUSQUEDA PARA LAS INTERACCIONES SINCRONAS(OTROS CANALES)
		SELECT PAI.[InteractionType]												'interactionType',
				COUNT(DISTINCT CONCAT(PAI.[InteractionId], PAI.[EndDate]))			'totalInteractionDetails',
				COUNT(DISTINCT PAI.[InteractionId])									'totalInteractions'
			FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK) 
			WHERE PAI.[VCC] = @VCC 
				AND PAI.[ProcessId] = @ProcessId
				AND PAI.[InteractionType] IN (SELECT VALUE FROM [dbo].[split](@Channels, ','))
			GROUP BY PAI.[InteractionType]
	END

	/* -- ESTA FUNCION RETORNA SOLO EL TOTAL Y TIEMPO DE LOS AUDIOS QUE NO HAN SIDO TRANSCRITOS
	SELECT
		COUNT(PA.[AudioId])						'interactionsTotal',
		ISNULL(SUM(A.[DurationTime]), 0)		'timeInSeconds'
	FROM ProcessAudio PA (NOLOCK)
	INNER JOIN Audio A (NOLOCK) 
		ON CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part]) = CONCAT(A.[ID], '_', A.[Section], '_', A.[Part]) 
			AND PA.[VCC] = A.[VCC]
	WHERE
		PA.[VCC] = @VCC
		AND [ProcessId] = @ProcessId
		AND A.[IsTranscribed] IS NULL
	*/
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetInteractionsSearch] (
   	@Instance Varchar(50) = NULL,
   	@IdAgent Varchar(MAX) = NULL,
   	@IdCampaign VARCHAR(MAX) = NULL,
 	@EvaluationDateFrom VARCHAR(MAX) = '1980-01-01T00:00:00',
    @EvaluationDateTo VARCHAR(MAX) = '2050-01-01T23:59:59',
	@EvaluationHourFrom VARCHAR(MAX) = '00:00:00',
    @EvaluationHourTo VARCHAR(MAX) = '23:59:59',
	@InteractionMinDuration Int = 0,
    @InteractionMaxDuration Int = 86400,
    @InteractionOrigins Varchar(50) = NULL,
	@InteractionTags Varchar(MAX) = NULL,
	@InteractionEvaluationTypes Varchar(MAX) = NULL,
	@InteractionResults Varchar(MAX) = NULL,
	@InteractionFormats Varchar(50) = NULL,
	@InteractionsLimit Int = 0,
	@InteractionIds         VARCHAR(MAX) = NULL,
	@InteractionWithSectionPart Bit = 0
)
AS 
BEGIN 
    SET NOCOUNT ON
	IF (@EvaluationHourFrom = '') SET @EvaluationHourFrom = '00:00:00'
	IF (@EvaluationHourTo = '') SET @EvaluationHourTo = '23:59:59'
	
	DECLARE @SQL_SELECT_0 VARCHAR(MAX) = ''
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''
	DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = ''
	DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = ''

	SET @SQL_SELECT_1 += 'SELECT DISTINCT '
	IF (@InteractionsLimit > 0) SET @SQL_SELECT_1 += 'TOP(' + (SELECT CONVERT(varchar(50), @InteractionsLimit)) + ') '
	SET @SQL_SELECT_1 += '
				e.[IdInteraction]												''id'',
				e.[IdCampaign]													''idCampaign'',
   				e.[IdCampaign]													''descriptionCampaign'',
				e.[IdAgent]														''idAgent'',
				e.[IdAgent]														''descriptionAgent'',
 				FORMAT(e.[InteractionDateStart], ''yyyy-MM-ddTHH:mm:ss.000Z'')		''dateIni'',
				e.[InteractionDateEnd]											''dateFin'',
   				e.[InteractionDuration]											''interactionTime'',
   				e.[InteractionType] 											''interactionType'',
				e.[InteractionOrigin] 											''origin'',
				e.[InteractionFormat] 											''format'',
				e.[ManagementResult]											''result'' '

	IF (@InteractionWithSectionPart = 1)
	BEGIN
		SET @SQL_SELECT_0 += '
			SELECT COUNT(DISTINCT CONCAT(e.[IdInteraction], e.[InteractionSection], e.[InteractionPart])) 	''totalRecords'',
				SUM(e.[InteractionDuration])																''totalTime'', 
				SUM(CASE WHEN e.[InteractionDuration] > 60 THEN e.[InteractionDuration] ELSE 60 END)		''roundedTime'' '

		SET @SQL_SELECT_1 += ',
			e.[InteractionSection]											''section'',
			e.[InteractionPart]												''part'' '
	END
	ELSE
	BEGIN
		SET @SQL_SELECT_0 += '
			SELECT COUNT(DISTINCT e.[IdInteraction])														''totalRecords'',
				SUM(e.[InteractionDuration])																''totalTime'', 
				SUM(CASE WHEN e.[InteractionDuration] > 60 THEN e.[InteractionDuration] ELSE 60 END)		''roundedTime'' '

	END
					
	SET @SQL_FROM += 'FROM InteractionEvaluationPlan e (NOLOCK) '

	SET @FILTERS_OPTIONAL_PART += '
			WHERE e.[Instance] = ''' + @Instance + '''
				AND e.[InteractionDateStart] >= ''' + @EvaluationDateFrom + '''
   				AND e.[InteractionDateStart] <= ''' + @EvaluationDateTo + ''' '

	IF (@IdAgent IS NOT NULL AND @IdAgent != '') SET @FILTERS_OPTIONAL_PART += 'AND e.[IdAgent] IN (SELECT Value FROM [dbo].[split](''' + @IdAgent + ''','','')) '
	IF (@IdCampaign IS NOT NULL AND @IdCampaign != '') SET @FILTERS_OPTIONAL_PART += 'AND e.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @IdCampaign + ''','','')) '
	IF (@InteractionOrigins IS NOT NULL AND @InteractionOrigins != '') SET @FILTERS_OPTIONAL_PART += 'AND e.[InteractionOrigin] IN (SELECT Value FROM [dbo].[split](''' + @InteractionOrigins + ''','','')) '
	IF (@InteractionMinDuration IS NOT NULL) SET @FILTERS_OPTIONAL_PART += 'AND e.[InteractionDuration] >= ' + (SELECT CONVERT(varchar(10), @InteractionMinDuration)) + ' '
	IF (@InteractionMaxDuration IS NOT NULL) SET @FILTERS_OPTIONAL_PART += 'AND e.[InteractionDuration] <= ' + (SELECT CONVERT(varchar(10), @InteractionMaxDuration))+ ' '
	IF (CAST(@EvaluationHourFrom AS TIME) <= CAST(@EvaluationHourTo AS TIME)) SET @FILTERS_OPTIONAL_PART += 'AND CAST(e.[InteractionDateStart] AS TIME) BETWEEN ''' + @EvaluationHourFrom + ''' AND ''' + @EvaluationHourTo + ''' '
	IF (CAST(@EvaluationHourFrom AS TIME) > CAST(@EvaluationHourTo AS TIME)) SET @FILTERS_OPTIONAL_PART += 'AND CAST(e.[InteractionDateStart] AS TIME) NOT BETWEEN ''' + @EvaluationHourTo + ''' AND ''' + @EvaluationHourFrom + ''' '
	IF (@InteractionFormats IS NOT NULL AND @InteractionFormats != '') SET @FILTERS_OPTIONAL_PART += 'AND e.[InteractionFormat] IN (SELECT Value FROM [dbo].[split](''' + @InteractionFormats + ''','','')) '
	IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') SET @FILTERS_OPTIONAL_PART += 'AND e.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')) '
	IF (@InteractionTags IS NOT NULL AND @InteractionTags != '') 
	BEGIN
		SET @SQL_FROM += '
			LEFT JOIN [Tag] t (NOLOCK)
				ON (t.[VCC] = ''' + @Instance + '''
					AND t.[Tag] IN (SELECT Value FROM [dbo].[split](''' + @InteractionTags + ''','',''))) '
		SET @FILTERS_OPTIONAL_PART += '
				AND t.[AudioId] LIKE CONCAT(e.[IdInteraction], ''%'') 
				AND t.[AudioId] IS NOT NULL '
	END
	
	IF (@InteractionEvaluationTypes IS NOT NULL AND @InteractionEvaluationTypes != '')
	BEGIN
		SET @SQL_FROM += '
			LEFT JOIN [EvaluationPlan] ep (NOLOCK)
				ON (ep.[Instance] = ''' + @Instance + '''
					AND ep.[ID] = e.[IdEvaluationPlan]) '
		SET @FILTERS_OPTIONAL_PART += 'AND ep.[Evaluation_Mode] IN (SELECT Value FROM [dbo].[split](''' + @InteractionEvaluationTypes + ''','','')) '
	END

	IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
    BEGIN
       SET @FILTERS_OPTIONAL_PART = ''
	   SET @FILTERS_EXCLUDING_PART = ' AND e.[IdInteraction] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) '
    END
	
	SET @SQL_EXTRA += 'ORDER BY ''dateIni'' ASC;'

	SET @SQL_SELECT_0 += @SQL_FROM + @FILTERS_OPTIONAL_PART +  @FILTERS_EXCLUDING_PART
	SET @SQL_SELECT_1 += @SQL_FROM + @FILTERS_OPTIONAL_PART +  @FILTERS_EXCLUDING_PART + @SQL_EXTRA

	EXEC (@SQL_SELECT_0)
	EXEC (@SQL_SELECT_1)
	
SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTE SP RETORN LAS TRANSCRIPCIONES DE UNA INTERACCION SEGUN SU VCC, INTERACTIONTYPE(CHANNEL), INTERACTIONID
CREATE PROCEDURE [dbo].[GetInteractionTranscription] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = null,
	@ProcessId VARCHAR(50) = NULL,
	@InteractionType VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- SI PROCESSID IS NULL RETORNA TODAS LAS TRANSCRIPCIONES DEL ID
	-- SI PROCESSID IS NOT NULL RETORNA TOADAS LAS TRANSCRIPCIONES DE ID RELACIONADAS CON EL PROCESSASYNCINTERACTION

	-- AGREGAMOS LA BUSQUEDA PARA CANALES DE TEXTO
	IF @InteractionType <> 'CALL'
	BEGIN
		-- PARA EL CASO QUE PLAN DE EVALUACION ID ES NULL
		IF @EvaluationPlanId IS NULL
		BEGIN 
			SELECT DISTINCT AI.[VCC]									'vcc',
							AI.[Campaign]								'campaign',
							AI.[Channel]								'interactionType',
							AI.[ID]										'id',
							AI.[ID]										'interactionId',
							AI.[Folder]									'folder',
							AI.[FileName]								'jsonFileName',
							AI.[ViewFileName]							'viewFileName',
							P.[Name]									'processId',
							AI.[TotalMessages]							'totalMessages',
							AI.[NeedI6Update]							'needI6Update'
				FROM [dbo].[AsyncInteraction] AI (NOLOCK)
				LEFT JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
					ON PAI.[VCC] = AI.[VCC]
						AND PAI.[Campaign] = AI.[Campaign]
						AND PAI.[InteractionType] = AI.[Channel]
						AND PAI.[FormattedEndDate] = AI.[Folder]
						AND PAI.[InteractionId] = AI.[ID]
						AND PAI.[ProcessId] = @ProcessId
				LEFT JOIN [dbo].[Process] P (NOLOCK)
					ON P.[VCC] = AI.[VCC]
						AND P.[Name] = PAI.[ProcessId]
				WHERE AI.[VCC] = @VCC
					AND AI.[Channel] = @InteractionType
					AND AI.[ID] = @InteractionId
					AND AI.[IsTranscribed] = 1
					AND (
						(@ProcessId IS NOT NULL AND PAI.[ProcessId] IS NOT NULL)
						OR
						(@ProcessId IS NULL AND PAI.[ProcessId] IS NULL)	
					)
				ORDER BY AI.[Folder] ASC

		END
		ELSE
		BEGIN
			SELECT IEP.[Instance]										'vcc',
					IEP.[IdCampaign]									'campaign',
					IEP.[InteractionType]								'interactionType',
					IEP.[IdInteraction]									'id',
					IEP.[IdInteraction]									'interactionId',
					IEP.[FormattedEndDate]								'folder',
					AI.[FileName]										'jsonFileName',
					AI.[ViewFileName]									'viewFileName',
					EP.[ProcessId]										'processId',
					IEP.[InteractionMessages]							'totalMessages',
					IEP.[NeedI6Update]									'needI6Update'
				FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK) 
					ON EP.[Instance] = IEP.[Instance]
						AND EP.[ID] = IEP.[IdEvaluationPlan]
				LEFT JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
					ON PAI.[VCC] = IEP.[Instance]
						AND PAI.[Campaign] = IEP.[IdCampaign]
						AND PAI.[InteractionType] = IEP.[InteractionType]
						AND PAI.[FormattedEndDate] = IEP.[FormattedEndDate]
						AND PAI.[InteractionId] = IEP.[IdInteraction]
						AND PAI.[ProcessId] = EP.[ProcessId]
				LEFT JOIN [dbo].[Process] P (NOLOCK)
					ON P.[VCC] = IEP.[Instance]
						AND P.[Name] = EP.[ProcessId]
				LEFT JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
					ON AI.[VCC] = IEP.[Instance]
						AND AI.[Campaign] = PAI.[Campaign]
						AND AI.[Channel] = PAI.[InteractionType] 
						AND AI.[Folder] = PAI.[FormattedEndDate]
						AND AI.[ID] = PAI.[InteractionId]
				WHERE IEP.[Instance] = @VCC
					AND IEP.[IdEvaluationPlan] = @EvaluationPlanId
					AND IEP.[InteractionType] = @InteractionType
					AND IEP.[IdInteraction] = @InteractionId 
		END
	END
	ELSE 
	BEGIN
		-- PARTE OPCIONAL PARA INTERACCIONES CALL, AUN NO IMPLEMENTADO
		DECLARE @ID VARCHAR(50) = ''
		IF EXISTS (
			SELECT 1 
				FROM [dbo].[Audio] A (NOLOCK)
				WHERE A.[VCC] = 'occ_test'--@VCC 
					AND (
						(CONCAT(A.[ID], '_', A.[Section], '_', A.[Part]) = '{23c5b766-4d29-11ea-b486-a4420ef40d06}_0_1')
						OR
						(A.[ID] = '{23c5b766-4d29-11ea-b486-a4420ef40d06}_0_1')
					)
		) 
		BEGIN
			SET @ID = (
						SELECT TOP 1 A.[ID]
							FROM [dbo].[Audio] A (NOLOCK)
								WHERE A.[VCC] = 'occ_test'--@VCC 
									AND (
										(CONCAT(A.[ID], '_', A.[Section], '_', A.[Part]) = '{23c5b766-4d29-11ea-b486-a4420ef40d06}_0_1')
										OR
										(A.[ID] = '{23c5b766-4d29-11ea-b486-a4420ef40d06}_0_1')
									)
						)

			SELECT *
				FROM [dbo].[Audio] A (NOLOCK)
				WHERE A.[VCC] = 'occ_test'--@VCC 
					AND A.[ID] = @ID
		END

		SELECT 'SOON...'

	END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetInteractionTypes] (
	@Instance VARCHAR(50) = NULL,
	@SearchCriteria VARCHAR(50) = NULL,
	@SearchExpression VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- VERIFICAMOS IS ESTAMOS BUSCANDO DE EVALUACIONES PENDIENTES
	IF @SearchCriteria IS NOT NULL AND @SearchCriteria = 'PENDING'
	BEGIN
		SELECT DISTINCT AEP.[Instance]					'instance',
						AEP.[InteractionType]			'id',
						AEP.[InteractionType]			'description'
			FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			WHERE AEP.[Instance] = @Instance
				AND AEP.[Completed] = 0
	END
	ELSE
	BEGIN
 		SELECT IT.[Instance]						'instance',
				IT.[ID]							'id',
				IT.[Description]					'description'
			FROM [dbo].[InteractionType] IT (NOLOCK)
			WHERE IT.[Instance] = @Instance
				AND ISNULL(IT.[ID], '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
				AND ISNULL(IT.[Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' COLLATE Latin1_General_CI_AI ELSE '%' END
	END

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetLinguisticProcessStatus] 'inconcert','radion1'
CREATE PROCEDURE [dbo].[GetLinguisticProcessStatus] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		P.[VCC]																													'vcc',
		P.[Name]																												'processId',
		(CASE WHEN P.[Status] IS NULL THEN 'UNKNOWN' ELSE P.[Status] END)														'status',
		COUNT(A.[ID])																											'toTranscribe',
		SUM(CASE WHEN A.[IsTranscribed] = 1 THEN 1 ELSE 0 END)																	'transcribed',
		CAST(ROUND((SUM(CASE WHEN A.[IsTranscribed] = 1 THEN 1 ELSE 0 END) * 1.0/COUNT(*)) * 100, 3) AS NUMERIC(6,3))			'percentage',
		P.[QueryId]																												'queryId'
	FROM [Process] P (NOLOCK)
	LEFT JOIN [ProcessAudio] PA (NOLOCK) ON P.[Name] = PA.[ProcessId]
	LEFT JOIN [Audio] A (NOLOCK) ON PA.[AudioId] = A.[ID] AND PA.[Section] = A.[Section] AND PA.[Part] = A.[Part]
	WHERE P.[VCC] = @VCC
	AND P.[Name] = @Name
	GROUP BY P.[VCC], P.[Name], P.[Status], [QueryId]
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetMainDomain](
	@VCC varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Domain]						'domain',
		[Description]					'description',
		[Client]						'client',
		[HasHttps]						'hasHttps',
		[ContentPort]					'contentPort',
		[Prefix]						'prefix',
		[IsMainDomain]					'isMainDomain',
		[IsContentDomain]				'isContentDomain',
		[ContentDefaultRedirect]		'contentDefaultRedirect',
		[IsDistributedService]			'isDistributedService',
		[AssociatedDistributedService]	'associatedDistributedService',
		[CustomCode]					'customCode',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Domain] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [IsMainDomain] = 1
	ORDER BY [Domain] ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetNeuroLinguisticProgramming] 'inconcert','visual', 'init', 'es'
--exec [GetNeuroLinguisticProgramming] 'default','visual', '', 'es'
CREATE PROCEDURE [dbo].[GetNeuroLinguisticProgramming] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL, @ProcessId Varchar(50) = NULL, @Language Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]						    'name',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [NeuroLinguisticProgramming] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [Name] = @Name

	EXEC [GetWords] @VCC, 'neuroLinguisticProgrammingId', @Name, @ProcessId, @Language
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetNeuroLinguisticProgrammings] 'default','',''
CREATE PROCEDURE [dbo].[GetNeuroLinguisticProgrammings] (
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]							'name',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [NeuroLinguisticProgramming] (NOLOCK)
	WHERE [VCC] = @VCC
	AND ISNULL([Name], '') LIKE CASE WHEN @SearchCriteria = 'name' THEN '%' + @SearchExpression + '%' ELSE '%' END
	AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
	ORDER BY [Name] asc, [CreatedDate] desc 

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetNLU] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	IF EXISTS (SELECT 1 FROM [NLU]
	WHERE [VCC] = CASE WHEN ISNULL(@VCC, '') = '' THEN [VCC] ELSE @VCC END 
	AND [Name] = @Name )
	
	BEGIN 
		SELECT
			[VCC]							'vcc',
			[Name]						    'name',
			[CognitiveEngine]				'cognitiveEngine',
			[Culture]						'culture',
			[Description]					'description',
			[AppId]							'appId',
			[SubscriptionKey]				'subscriptionKey',
			[IsSystemApp]					'isSystemApp',
			[NeedTrain]						'needTrain',
			[LastTrainedDate]				'lastTrainedDateTime',
			[LastPublishedDate]				'lastPublishedDateTime',
			[CreatedDate]					'createdDate',
			[CreatedByUserId]				'createdByUserId',
			[LastModifiedDate]				'lastModifiedDate',
			[LastModifiedByUserId]			'lastModifiedByUserId'
		FROM [NLU] (NOLOCK)
		WHERE [VCC] = CASE WHEN ISNULL(@VCC, '') = '' THEN [VCC] ELSE @VCC END 
		AND [Name] = @Name
	END
	ELSE
	BEGIN
		SELECT
				[VCC]							'vcc',
				[Name]						    'name',
				[CognitiveEngine]				'cognitiveEngine',
				[Culture]						'culture',
				[Description]					'description',
				[AppId]							'appId',
				[SubscriptionKey]				'subscriptionKey',
				[IsSystemApp]					'isSystemApp',
				[NeedTrain]						'needTrain',
				[LastTrainedDate]				'lastTrainedDateTime',
				[LastPublishedDate]				'lastPublishedDateTime',
				[CreatedDate]					'createdDate',
				[CreatedByUserId]				'createdByUserId',
				[LastModifiedDate]				'lastModifiedDate',
				[LastModifiedByUserId]			'lastModifiedByUserId'
			FROM [NLU] (NOLOCK)
			WHERE [VCC] = 'system'
			AND [Name] = @Name
			AND [Name] LIKE 'sentiment-%'
		
	END
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetNLU] NULL,'test2'
CREATE PROCEDURE [dbo].[GetNLUByAppId] (
	@VCC Varchar(50) = NULL, 
	@AppId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	DECLARE @VCC_NEW VARCHAR(50)
	SET @VCC_NEW = @VCC

	IF EXISTS(SELECT 1 FROM [NLU] WHERE [AppId] = @AppId AND [Name] LIKE 'sentiment-%')
	SELECT @VCC_NEW = VCC FROM [NLU] WHERE [AppId] = @AppId AND [Name] LIKE 'sentiment-%'


	SELECT 
			N.[VCC]							'vcc',
			N.[Name]						'name',
			N.[CognitiveEngine]				'cognitiveEngine',
			N.[Culture]						'culture',
			N.[Description]					'description',
			N.[AppId]						'appId',
			N.[SubscriptionKey]				'subscriptionKey',
			N.[IsSystemApp]					'isSystemApp',
			N.[NeedTrain]					'needTrain',
			N.[LastTrainedDate]				'lastTrainedDateTime',
			N.[LastPublishedDate]			'lastPublishedDateTime',
			N.[CreatedDate]					'createdDate',
			N.[CreatedByUserId]				'createdByUserId',
			N.[LastModifiedDate]			'lastModifiedDate',
			N.[LastModifiedByUserId]		'lastModifiedByUserId',
		    N.[Status]                      'status',
		    N.[StatusCode]                  'statusCode',
            N.[NeedPublish]                 'needPublish',
			N.[TrainId]						'trainId',
			N.[TrainVersion]				'trainVersion'
       
	FROM [dbo].[NLU] N (NOLOCK)
	WHERE N.[VCC] = CASE WHEN ISNULL(@VCC_NEW, '') = '' THEN N.[VCC] ELSE @VCC_NEW END 
		AND N.[AppId] = @AppId
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetNluIntentUtteranceToExport]
(
	@VCC	VARCHAR(50),
	@ListMode INT = 0,
    @UseRasaApi BIT = NULL,
	@RatioRasaMigration INT
)
AS
BEGIN
	SET NOCOUNT ON
    --@ListMode = 0 resumen de nlus a exportar VCC A VCC solo los que no son de sistema
    --@ListMode = 1 listado de nlus a exportar VCC A VCC solo los que no son de sistema
    --@ListMode = 2 resumen de nlus a exportar RASA A RASAAPI O VICEVERSA todos los nlu de todo el sistema
    --@ListMode = 3 listado de nlus a exportar RASA A RASAAPI O VICEVERSA todos los nlu de todo el sistema
	
	DECLARE @NLU TABLE(
	
	 [Name] VARCHAR(50),
	 [AppId] VARCHAR(50),
	 [VCC] VARCHAR(50),
	 [CognitiveEngine] VARCHAR(50)
	)
	DECLARE @Intent TABLE (
	ID VARCHAR(50)
	)

    IF (@ListMode = 0 OR @ListMode = 1)
    BEGIN
        INSERT INTO @NLU SELECT [Name],[AppId],VCC,CognitiveEngine FROM [NLU]  (NOLOCK) WHERE [VCC] = @VCC AND [IsSystemApp] = 0 AND [Status] <> 4
        INSERT INTO @Intent SELECT [IntentId] FROM [Intent] (NOLOCK) WHERE [VCC] = @VCC AND [AppId] IN (SELECT [AppId] FROM @NLU)
    END
    ELSE
    BEGIN
        INSERT INTO @NLU SELECT TOP (@RatioRasaMigration)  [Name],[AppId],VCC,CognitiveEngine FROM [NLU]  (NOLOCK) WHERE [VCC] = @VCC  AND [Status] <> 2 AND [CognitiveEngine] = IIF(@UseRasaApi = 1,'RASA','RASAAPI')
			
    END
   

	IF(@ListMode = 0) -- CONTAR TOTAL DE REGISTROS
	   BEGIN

        DECLARE @TotalNlu INT = 0
        DECLARE @TotalIntent INT = 0
        DECLARE @TotalUtterance INT = 0
        DECLARE @TotalEntity INT = 0
        DECLARE @NluNames VARCHAR(MAX) = NULL
                
        SELECT @TotalNlu = COUNT([Name]) FROM [NLU] (NOLOCK) WHERE [VCC] = @VCC AND [Name] IN (SELECT [Name] FROM @NLU)

        SELECT @TotalIntent = COUNT([IntentId]) FROM [Intent] (NOLOCK) WHERE [VCC] = @VCC AND [IntentId] IN (SELECT ID FROM @Intent)

        SELECT @TotalUtterance = COUNT([Id]) FROM [Utterance] (NOLOCK) WHERE [VCC] = @VCC AND [IntentId] IN (SELECT ID FROM @Intent)
                
        SELECT @TotalEntity = COUNT([ID]) FROM [dbo].[Entity] WHERE [VCC] = @VCC AND [AppId] IN (SELECT [AppId] FROM @NLU)

        SELECT @TotalNlu AS TotalNlu,@TotalIntent AS TotalIntent,@TotalUtterance AS TotalUtterance, @TotalEntity AS TotalEntity
                
        SELECT @NluNames = STUFF(( SELECT ',' + [Name] FROM [NLU]  WHERE [VCC] = @VCC AND [IsSystemApp] = 0 FOR XML PATH(''), TYPE).value('.', 'NVARCHAR(MAX)'), 1, 1, '')
            
       
		--SELECT 0 'InUse'
         EXEC [GetNlusUsage] @VCC,@NluNames

	   END
	   IF (@ListMode = 1)
	   BEGIN
	   --NLUS
	       SELECT 
	       [Name]                name,
	       [Name]                originalName,
	       [Culture]             culture, 
	       [Description]         description,
	       [AppId]               appId,
	       [CognitiveEngine]     cognitiveEngine,
	       0                     'status',
	       0                    'totalRetries'
	       FROM [NLU] (NOLOCK) WHERE [VCC] = @VCC AND [Name] IN (SELECT [Name] FROM @NLU)
					--INTENTS
					   SELECT
			[Name]     'name',
			[IntentId] 'intentId',
			[AppId]     'appId',
			CONCAT([AppId],'[::]',[IntentId]) 'key',
			0                     'status',
	        0                    'totalRetries'
			FROM [Intent] (NOLOCK) WHERE [VCC] = @VCC AND [IntentId] IN (SELECT ID FROM @Intent)
					--UTTERANCES  
					  
			 SELECT DISTINCT
			 U.[Id] 'id',
			 U.[AppId]       'appId',
			 I.[Name]    'intent',
			 U.[IntentId] 'intentId',
			 U.[Text] 'name',
			 (SELECT [EntityName] 'entityName',[Start] 'start',[End] 'end' 
			 FROM [UtteranceEntity] (NOLOCK) WHERE [VCC] = @VCC AND [UtteranceId] IN(U.[Id]) FOR JSON AUTO) 'rawEntityLabels',
			 CONCAT(U.[AppId],'[::]',U.[IntentId],'[::]',U.[Id]) 'key',
			 CONCAT(U.[AppId],'[::]',I.[IntentId]) 'intentKey',
			 0                     'status',
			 0                    'totalRetries'
			 
			 FROM [Utterance] U (NOLOCK) 
			 JOIN [Intent] I (NOLOCK)
			 ON (U.VCC = I.VCC AND U.IntentId = I.IntentId)
			 WHERE U.[VCC] = @VCC AND U.[IntentId] IN (SELECT ID FROM @Intent)
			 
			  --ENTITIES
			 SELECT 
			 [AppId] 'appId',
			 [ID] 'id',
			 [Name] 'name',
			 CONCAT([AppId],'[::]',[ID]) 'key',
			  0                     'status',
			 0                    'totalRetries'
			 FROM [dbo].[Entity] WHERE [VCC] = @VCC AND [AppId] IN (SELECT [AppId] FROM @NLU)
	   END
       IF (@ListMode = 2)
       BEGIN
        IF(@VCC ='ALL') --LISTAR RESUMEN PARA TODOS LOS VCCS RASA O RASAAPI
        BEGIN
					
				WITH ALL_NLUS AS (
                    SELECT   N.[VCC],
					N.[CognitiveEngine],
                    (SELECT COUNT(1) FROM NLU WHERE VCC = N.[VCC] AND appid = N.appid) 'totalNlu',
					0 'totalIntent',
                    0 'totalUtterance',
                    0 'totalEntity',
					[status] 'status'


                    FROM  NLU N WHERE 
                    N.[CognitiveEngine] = IIF(@UseRasaApi = 1,'RASA','RASAAPI')
                ),
				TOTAL_NLUS AS(
					SELECT TOP 1 
						AN.[VCC]                'vcc',
						AN.[CognitiveEngine]    'cognitiveEngine',
						SUM(TotalNlu)               'totalNlu',
						SUM(TotalIntent)            'totalIntent',
						SUM(TotalUtterance)         'totalUtterance',
						SUM(TotalEntity)            'totalEntity',
						AN.[status]					'status'
					FROM ALL_NLUS AN
						---INNER JOIN [dbo].[VCC] V ON V.[VCC] = AN.[VCC]
					--WHERE V.[IsMigrating] = 0 
						GROUP BY AN.[VCC],  AN.[CognitiveEngine],AN.[status] )
				SELECT 
					[vcc]				'vcc',
					[cognitiveEngine]	'cognitiveEngine',
					@RatioRasaMigration	'totalNlu',
					[totalIntent]		'totalIntent',
					[totalUtterance]	'totalUtterance',
					[totalEntity]		'totalEntity'
				
				FROM TOTAL_NLUS
				WHERE totalNlu >= @RatioRasaMigration 
					AND [status] <> 2
				UNION
				SELECT 
					[vcc]				'vcc',
					[cognitiveEngine]	'cognitiveEngine',
					[totalNlu]			'totalNlu',
					[totalIntent]		'totalIntent',
					[totalUtterance]	'totalUtterance',
					[totalEntity]		'totalEntity' 
				FROM TOTAL_NLUS
				WHERE totalNlu < @RatioRasaMigration
					AND [status] <> 2
				

            		
        END
        SELECT 0 'InUse'
      END
     IF (@ListMode = 3)
	   BEGIN
	   --NLUS
	       SELECT 
			   [Name]                name,
			   [Name]                originalName,
			   [Culture]             culture, 
			   [Description]         description,
			   [AppId]               appId,
			   [CognitiveEngine]     cognitiveEngine,
			   0                     'status',
			   0                    'totalRetries'
	       FROM [NLU] (NOLOCK) 
		   WHERE [VCC] = @VCC 
			AND [Name] IN (SELECT [Name] FROM @NLU)
		 -- EN LA MIGRACION SOLO ES NECESARIO MIGRAR NLUS
		 --INTENTS
			SELECT
				[Name]     'name',
				[IntentId] 'intentId',
				[AppId]     'appId',
				CONCAT([AppId],'[::]',[IntentId]) 'key',
				0                     'status',
				0                    'totalRetries'
			FROM [Intent] (NOLOCK) 
			WHERE [VCC] = @VCC 
				AND [IntentId] IN (SELECT ID FROM @Intent)

		-- UTTERANCES  
					  
			 SELECT  DISTINCT
				 U.[Id] 'id',
				 U.[AppId]       'appId',
				 I.[Name]    'intent',
				 U.[IntentId] 'intentId',
				 U.[Text] 'name',
				 (SELECT [EntityName] 'entityName',[Start] 'start',[End] 'end' 
				 FROM [UtteranceEntity] (NOLOCK) WHERE [VCC] = @VCC AND [UtteranceId] IN(U.[Id]) FOR JSON AUTO) 'rawEntityLabels',
				 CONCAT(U.[AppId],'[::]',U.[IntentId],'[::]',U.[Id]) 'key',
				 CONCAT(U.[AppId],'[::]',I.[IntentId]) 'intentKey',
				 0                     'status',
				 0                    'totalRetries'
			 
			 FROM [Utterance] U (NOLOCK) 
				INNER JOIN [Intent] I (NOLOCK)
				ON (U.VCC = I.VCC AND U.IntentId = I.IntentId)
			 WHERE U.[VCC] = @VCC  
				AND U.[IntentId] IN (SELECT ID FROM @Intent)
			 
		--ENTITIES
				 SELECT 
					 [AppId] 'appId',
					 [ID] 'id',
					 [Name] 'name',
					 CONCAT([AppId],'[::]',[ID]) 'key',
					 0                     'status',
					 0                    'totalRetries'
				 FROM [dbo].[Entity] 
				 WHERE [VCC] = @VCC 
					AND [AppId] IN (SELECT ID FROM @Intent)
	   END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetNLUs] (
    @VCC Varchar(50) = NULL,
    @SearchCriteria Varchar(50) = NULL,
    @SearchExpression Varchar(50) = NULL,
    @Apps NVARCHAR(MAX),
    @IsSystemApp BIT = NULL,
    @ShowInMigrationProcess BIT = 0,
	@UseRasaApi BIT = NULL
)
AS
BEGIN
    -- EN CASO EL FLAG LLEGUE NULL LO SETTEAMOS COMO 0

    DECLARE  @SQL              NVARCHAR(MAX)
	DECLARE  @SQL_VCC            NVARCHAR(MAX)
    DECLARE  @FILTER           VARCHAR(MAX) = ''
    DECLARE  @UNION_SQL        VARCHAR(MAX) = ''
    DECLARE  @APPS_FILTER      VARCHAR(MAX) = ''
    DECLARE  @STATUS_FILTER    VARCHAR(MAX) = ''
    DECLARE  @ORDER_BY         VARCHAR(MAX) = ''
	DECLARE  @FILTER_COGNITIVEENGINE VARCHAR(MAX) = ''
    SET NOCOUNT ON
    --Status va ha cambiar a 9 o 10 de acuerdo a LastPublishedDate
    --0 PENDIENTE DE MIGRACION
    --1 ESTA DISPONIBLE EN EL MOTOR
    --2 MIGRANDO
    --3 CON ERROR ( EL ERROR LO INDICA STATUSCODE)
    --4 CANCELADO POR DEMASIADOS INTENTOS ERRONEOS
    --8 MIGRADO EL NLU PERO FALTA LOS INTENT, UTTERANCES,ETC
    --9 ESTA DISPONILBE EL MODELO EN EL MOTOR PERO NO ESTA ENTRENADO(RASA) O LUIS(PUBLICADO)
    --10 ESTA DISPONIBLE EN EL MOTOR Y ENTRENADO
    --11 EN PROCESO DE ENTRENAMIENTO (PROCESO INTERNO DE LUIS)
    --12 EN PROCESO DE PUBLICACION (PROCESO INTERNO DE LUIS)
    --13 ENTRENAMIENTO FALLIDO (LUIS RETORNO ESTADO DE ERROR LUEGO DE VARIAS CONSULTAS)
	--14 FINALIZO LA MIGRACION DE RASA CORRECTAMENTE
    --Se establece estos valores porque actualmente Status puede tener valores de 0 a 4, sin embargo puede ampliarse en futuros requerimientos.
    SET @SQL = '
    SELECT [VCC]                  ''vcc'',
       [Name]                 ''name'',
       [CognitiveEngine]      ''cognitiveEngine'',
       [Culture]              ''culture'',
       [Description]          ''description'',
       [AppId]                ''appId'',
       [SubscriptionKey]      ''subscriptionKey'',
       [IsSystemApp]          ''isSystemApp'',
       [NeedTrain]            ''needTrain'',
       [IsTrained]            ''isTrained'',
       [LastTrainedDate]      ''lastTrainedDateTime'',
       [IsPublished]          ''isPublished'',
       [LastPublishedDate]    ''lastPublishedDateTime'',
       [CreatedDate]          ''createdDate'',
       [CreatedByUserId]      ''createdByUserId'',
       [LastModifiedDate]     ''lastModifiedDate'',
       [LastModifiedByUserId] ''lastModifiedByUserId'',
       [TrainVersion]         ''trainVersion'',
       [NeedPublish]          ''needPublish'',
       [TrainId]              ''trainId'',
       CASE
           WHEN [Status] = 0 THEN
               ''PENDING''
           WHEN [Status] = 1 THEN
               CASE
                   WHEN [NeedTrain] = 1 OR [LastTrainedDate] IS NULL THEN
                       ''NEED_TRAIN''
                   WHEN [NeedTrain] = 0 AND ([NeedPublish] = 1 OR [LastPublishedDate] IS NULL) THEN
                       IIF([CognitiveEngine] = ''LUIS'' OR [CognitiveEngine] =''RASAAPI'', ''NEED_PUBLISH'', ''NEED_TRAIN'')
                   ELSE
                       ''AVAILABLE''
                END
           WHEN [Status] = 2 THEN
                ''MIGRATING''
           WHEN [Status] = 3 THEN
               ''FAILED''
           WHEN [Status] = 4 THEN
               ''CANCELLED''
           WHEN [Status] = 8 THEN
               ''INCOMPLETE''
           WHEN [Status] = 9 THEN
                IIF([CognitiveEngine] = ''LUIS'', ''NEED_PUBLISH'', ''NEED_TRAIN'')
           WHEN [Status] = 10 THEN
                ''AVAILABLE''
           WHEN [Status] = 11 THEN
               ''TRAINING''
           WHEN [Status] = 12 THEN
               ''PUBLISHING''
           WHEN [Status] = 13 THEN
               ''TRAINING_FAILED''
           ELSE
                ''UNKNOWN''
        END ''statusCode''
    FROM [NLU] (NOLOCK)
    WHERE '

	-- EN CASO SE ESTE USANDO RASA API 
	IF(@UseRasaApi = 1)
	BEGIN
		SET @FILTER_COGNITIVEENGINE += ' AND [CognitiveEngine] = ''RASAAPI'' '
	END
	IF (@UseRasaApi = 0)
	BEGIN
		SET @FILTER_COGNITIVEENGINE += ' AND [CognitiveEngine] = ''RASA'' '
	END


    -- EN CASO BUSQUEMOS MODELOS DE TODOS LOS VCC
    IF (@VCC = 'ALL')
        SET @SQL_VCC = '
            [VCC] IS NOT NULL '
    ELSE
	BEGIN

		SET @SQL_VCC = '
		[VCC] = ''' + @VCC + ''' '
	END


    -- EN CASO BUSQUEMOS MODELOS DEL SISTEMA NO FILTRAMOS POR NOMBRES
    IF @IsSystemApp = 0 AND @Apps IS NOT NULL AND @Apps <> '' 
       SET @APPS_FILTER  += ' AND [AppId] IN (''' + @Apps + ''') '

	

   
    IF (@IsSystemApp IS NULL)
     SET @FILTER += ''
    ELSE
     SET @FILTER += ' AND [IsSystemApp] = ' + CASE WHEN @IsSystemApp = 1 THEN '1' ELSE '0' END + ' '


    IF(@SearchCriteria IS NOT NULL AND @SearchCriteria <> '' AND @SearchExpression IS NOT NULL AND @SearchExpression <> '')
        BEGIN
            SET @FILTER += '
        AND ISNULL([Name], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
        AND ISNULL([Description], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''description'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
        AND ISNULL([LastModifiedByUserId], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''LastModifiedByUserId'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END'
        END


    SET @ORDER_BY = ' ORDER BY [LastModifiedDate] ASC, [Name] DESC'
	SET @STATUS_FILTER = ' AND [Status] = 1 '

    IF(@ShowInMigrationProcess IS NOT NULL AND @ShowInMigrationProcess = 1)
        BEGIN
            SET @UNION_SQL = ' UNION ' + @SQL + @SQL_VCC + @FILTER + @FILTER_COGNITIVEENGINE + ' AND [Status] <> 1 '
        END
	IF(@SearchCriteria = 'Description' AND @SearchExpression = 'SENTIMENT_LANGUAGE_')
		BEGIN
			DECLARE @SQL_VCC_SYSTEM VARCHAR(MAX)
			SET @SQL_VCC_SYSTEM = '
			[VCC] = ''' + 'system' + ''' '
            SET @UNION_SQL = ' UNION ' + @SQL + @SQL_VCC_SYSTEM + @FILTER + @FILTER_COGNITIVEENGINE + ' AND [Name] NOT IN (SELECT [Name] FROM NLU WHERE VCC =' +''''+@VCC+''''+ 'AND [Name] LIKE' + ''''+'sentiment-'+'%'+''''+')'
        END
	

	SET @SQL +=  @SQL_VCC
    SET @SQL  +=   @FILTER + @APPS_FILTER + @STATUS_FILTER + @FILTER_COGNITIVEENGINE + @UNION_SQL +  @ORDER_BY
    SET NOCOUNT OFF
	PRINT @SQL
    EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- exec [GetNLUsStudy] 'allegro', NULL, NULL,'Test'
CREATE PROCEDURE [dbo].[GetNLUsStudy] (
	@VCC				VARCHAR(50) = NULL,
	@SearchCriteria		VARCHAR(50) = NULL,
	@SearchExpression	VARCHAR(50) = NULL,
	@Names				NVARCHAR(MAX)
)
AS
BEGIN
	DECLARE @SQL NVARCHAR(MAX)
	SET @SQL = '
	SET NOCOUNT ON
	SELECT
		[Name]						    ''name'',
		[Culture]						''culture'',
		[Description]					''description'',
		[AppId]							''appId'',
		[SubscriptionKey]				''subscriptionKey'',
		[CreatedDate]					''createdDate'',
		[CreatedByUserId]				''createdByUserId'',
		[LastModifiedDate]				''lastModifiedDate'',
		[LastModifiedByUserId]			''lastModifiedByUserId''
	FROM [NLU] (NOLOCK)
	WHERE [VCC] = ''' + @VCC + '''
	AND [Name] IN (''' + @Names + ''')
	ORDER BY [LastModifiedDate] asc, [Name] desc
	'
	SET @SQL += ' SET NOCOUNT OFF'
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetNlusUsage]
(
	@VCC	    VARCHAR(50),
	@NluNames		VARCHAR(MAX)= NULL
)
AS
BEGIN
	SET NOCOUNT ON

	DECLARE @NLUS TABLE (
	  ID VARCHAR(200),
	  QueryId VARCHAR(200)
	)
	
		DECLARE @FOUND_IN_TTI BIT = 0
		
	    DECLARE @FOUND_IN_ABILITY BIT = 0
	    
	    DECLARE @IN_USE BIT =0
    
		INSERT INTO @NLUS SELECT CONCAT(N.[Name],'|',I.[Name]),I.[IntentId] FROM [NLU] N (NOLOCK) 
		JOIN [Intent] I (NOLOCK) ON (N.[AppId] = I.[AppId] AND N.[VCC] = I.[VCC]) 
		WHERE N.[VCC] = @VCC AND N.[Name] IN (SELECT [Value] FROM [dbo].[Split](@NluNames,','))

	

	   -- VALIDAR PARA ESTUDIOS
	   IF(EXISTS( SELECT TTI.[ID] FROM  [TemplateThemeItem] TTI (NOLOCK) 
				  INNER JOIN [Template] TT (NOLOCK) ON TTI.[TemplateId] = TT.[Name] AND TTI.[Vcc] = TT.[Vcc]
				  INNER JOIN [STUDY] S (NOLOCK) ON TT.[Name] = S.[TemplateId] AND TT.[Vcc] =S.[Vcc] AND S.[StatusDescription] = 'IN_PROGRESS'
	   WHERE TTI.VCC = @VCC AND TTI.[ID] IN (SELECT [ID] FROM @NLUS)))
	  				
	   BEGIN 
	        SET @FOUND_IN_TTI = 1
	   END


	   --VALIDAR PARA PLAN
	
		 IF(EXISTS( SELECT [QueryId] FROM AbilityQuery AQ (NOLOCK) 
					INNER JOIN AbilityEvaluationFormat AEF (NOLOCK) ON  AQ.AbilityId =  AEF.[IdAbility]
					INNER JOIN EvaluationPlan EP (NOLOCK) ON AEF.[IdEvaluationFormat] = EP.[IdEvaluationFormat] AND EP.StatusDescription = 'IN_PROGRESS'
		 WHERE [VCC] = @VCC AND [QueryId] IN (SELECT [QueryId] FROM @NLUS) )) 
		 BEGIN 
		 SET @FOUND_IN_ABILITY = 1
		 END 
		 
		  IF(@FOUND_IN_TTI=1 OR @FOUND_IN_ABILITY =1 )
		  BEGIN 
		  SET @IN_USE = 1
		  END 
		
	  SELECT @IN_USE AS 'inUse'
	
		SET NOCOUNT OFF
	END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetNotEvaluatedInteractionsAuditor] (
	@Instance Varchar(50) = NULL, 
	@UserId Varchar(50) = NULL,
	@IdCampaigns VARCHAR(MAX) = NULL,
	@IdAgents Varchar(MAX) = NULL,
	@EvaluationPlans Varchar(MAX) = NULL,
	@InteractionTypes Varchar(100) = NULL,
	@InteractionResults Varchar(MAX) = NULL,
	@EvaluationDateFrom VARCHAR(50) = '1980-01-01T00:00:00',
    @EvaluationDateTo VARCHAR(50) = '2050-01-01T23:59:59',
 	@EvaluationPlanDateTo   VARCHAR(MAX) = '2050-01-01T23:59:59',
	@EvaluationStatus BIT = 1,
 	@InteractionIds     VARCHAR(MAX) = NULL,
	@ContactAddress VARCHAR(300) = NULL,
	@IdAuditors VARCHAR(MAX) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	IF @EvaluationDateFrom IS NULL OR LEN(@EvaluationDateFrom) = 0 SET @EvaluationDateFrom = '1980-01-01T00:00:00'
	IF @EvaluationDateTo IS NULL OR LEN(@EvaluationDateTo) = 0 SET @EvaluationDateTo = '2050-01-01T23:59:59'
	IF @EvaluationPlanDateTo IS NULL OR LEN(@EvaluationPlanDateTo) = 0 SET @EvaluationPlanDateTo = '2050-01-01T23:59:59'

	DECLARE @USER_ROLE VARCHAR(50) = ''
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_FIXED_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = ''

	-- IDENTIFICAMOS EL ROL DEL USUARIO
	SELECT @USER_ROLE = U.[Role]
		FROM [dbo].[User] U (NOLOCK)
		WHERE U.[VCC] = @Instance
			AND U.[ID] = @UserId

    -- CREAMOS LAS TABLAS TEMPORALES AUXILIARES
    SELECT A.[ID]															'IdAuditor',
			A.[Description]													'DescriptionAuditor',
			ISNULL(U.[FirstName] + ' ', '') + ISNULL(U.[LastName], '')		'AuditorName'
		INTO #TMP_AUDITOR
		FROM [dbo].[Auditor] A (NOLOCK)
		LEFT JOIN [dbo].[User] U (NOLOCK)
			ON U.[VCC] = A.[Instance]
				AND U.[ID] = A.[ID]
		WHERE A.[Instance] = @Instance

    SELECT [Value]
		INTO #TMP_CHANNEL
		FROM [dbo].[Split](@InteractionTypes, ',')

	-- EMULAMOS UNA NUEVA TABLA EN BASE A AuditorEvaluationPlan DONDE SE REMOVERAN TODOS LOS REGISTROS DUPLICADOS PARA EL PK DE LA TABLA Audio
	SET @SQL_SELECT_1 += '
		WITH CTE AS (
			SELECT AEP.[Instance], 
					AEP.[IdEvaluationPlan], 
					AEP.[IdInteraction], 
					AEP.[IdAuditor], 
					AEP.[Score], 
					IEP.[AgentName],
					AEP.[LastModifiedDate],
					RN = ROW_NUMBER() OVER(PARTITION BY AEP.[Instance], AEP.[IdEvaluationPlan], AEP.[IdInteraction] ORDER BY AEP.[LastModifiedDate] DESC)
			FROM AuditorEvaluationPlan AEP (NOLOCK)
			INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = AEP.[Instance] 
					AND IEP.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND IEP.[IdInteraction] = AEP.[IdInteraction]
					AND ISNULL(IEP.[IdAgent], '''') = ISNULL(AEP.[IdAgent], '''') '
	IF (
		@InteractionTypes IS NOT NULL 
		AND 
		@InteractionTypes != ''
	) 
		SET @SQL_SELECT_1 += '
			INNER JOIN #TMP_CHANNEL TMP_C
				ON TMP_C.[Value] = AEP.[InteractionType] '

	SET @SQL_SELECT_1 += '
			WHERE AEP.[Instance] = ''' + @Instance + ''' 
				AND AEP.[InteractionDate] BETWEEN ''' + @EvaluationDateFrom + ''' AND ''' + @EvaluationDateTo + '''
				AND AEP.[Completed] = 0 
				AND AEP.[IdAuditor] <> ''SPEECH''
				AND AEP.[StatusDescription] NOT IN(''INVALID'') '

	IF @USER_ROLE = 'quality_auditor' 
	BEGIN
		SET @SQL_SELECT_1 += '
				AND	AEP.[IdAuditor] = ''' + @UserId + ''' '
	END
	
	IF @USER_ROLE = 'administrator' 
    BEGIN
        IF (@IdAuditors IS NOT NULL AND @IdAuditors != '') SET @SQL_SELECT_1 += 'AND AEP.[IdAuditor] IN (SELECT Value FROM [dbo].[split](''' + @IdAuditors + ''','','')) ';
    END

	IF (@IdCampaigns IS NOT NULL AND @IdCampaigns != '') SET @SQL_SELECT_1 += 'AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @IdCampaigns + ''','','')) ';
	IF (@IdAgents IS NOT NULL AND @IdAgents != '') SET @SQL_SELECT_1 += 'AND AEP.[IdAgent] IN (SELECT Value FROM [dbo].[split](''' + @IdAgents + ''','','')) ';
	IF (@EvaluationPlans IS NOT NULL AND @EvaluationPlans != '') SET @SQL_SELECT_1 += 'AND AEP.[IdEvaluationPlan] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationPlans + ''','','')) '
	
	IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') 
		SET @SQL_SELECT_1 += 'AND (
								(CONCAT(AEP.[IdCampaign], ISNULL(REPLACE(AEP.[ManagementResult], ''/'', ''_''), ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
								OR
								(CONCAT(AEP.[IdCampaign], ''_'', ISNULL(AEP.[ManagementResult], ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
								OR
								(AEP.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
							)'

	SET @SQL_SELECT_1 += '
		),
		CLEANED_CTE AS (
			SELECT C.[Instance], 
					C.[IdEvaluationPlan], 
					C.[IdInteraction], 
					C.[IdAuditor], 
					C.[Score], 
					C.[AgentName],
					C.[LastModifiedDate]
			FROM [CTE] C
			WHERE C.[RN] = 1
		) '
	-- EL CTE SOLO PUEDE SER INVOCADO UNA VEZ POR ESO LO PONEMOS EN EL SELECT FINAL DONDE EL CONTADOR(RN) ES 1
	
	-- ARMAMOS EL SELECT PRINCIPAL PARA LA BUSQUEDA DE EVALUACIONES PENDIENTES DEL AUDITOR
	SET @SQL_SELECT_1 += '
		SELECT DISTINCT '
	SET @SQL_SELECT_1 += '
				AEP.[IdInteraction]															''id'', 
				AEP.[IdCampaign]															''descriptionCampaign'',
				AEP.[IdAgent]																''idAgent'',
				AEP.[IdAgent]																''descriptionAgent'',
				AEP.[InteractionDate]														''dateIni'',
				AEP.[InteractionDateEnd]													''dateFin'',
				AEP.[InteractionDuration]													''interactionTime'',
				AEP.[Completed]																''evaluated'',
				AEP.[InteractionType]														''interactionType'',
				AEP.[ManagementResult]														''dispositionCode'',
				AEP.[ContactAddress]																''address'',
				AEP.[InteractionOrigin]														''origin'',
				AEP.[IdEvaluationPlan]														''idEvaluationPlan'',
				EP.[ProcessId]																''processId'',
				EP.[Description]															''descriptionEvaluationPlan'',
				EP.[DateFin]																''dateFinEvaluationPlan'', 
				EP.[Modality]																''evaluationPlanModality'',
				EP.[Evaluation_Mode]														''evaluationPlanEvaluationMode'',
				TMP_A.[IdAuditor]															''idAuditor'', 
				TMP_A.[DescriptionAuditor]													''descriptionAuditor'',
				TMP_A.[AuditorName]															''auditorName'',						-- NOMBRE DEL AUDITOR
				C.[AgentName]																''agentName'',							-- NOMBRE DEL AGENTE
				C.[IdAuditor]																''lastEvaluationEvaluator'', 
				C.[Score]																	''lastEvaluationScore'',
				AEP.[Status]																''status'',
				CASE
					WHEN EP.[DateFin] < CAST(GETDATE()  AS DATE)
					THEN 1
					ELSE 0
				END																			''isExpired'',
				AEP.[StatusDescription]														''statusDescription'''

	SET @SQL_FROM += '
			FROM [AuditorEvaluationPlan] AEP (NOLOCK) 
			INNER JOIN [EvaluationPlan] EP (NOLOCK) 
				ON (EP.[Instance] = EP.Instance
					AND EP.[ID] = AEP.IdEvaluationPlan) 
			INNER JOIN CLEANED_CTE C (NOLOCK) 
				ON C.[Instance] = AEP.[Instance] 
					AND C.[IdEvaluationPlan] = AEP.[IdEvaluationPlan] 
					AND C.[IdInteraction] = AEP.[IdInteraction]
			LEFT JOIN #TMP_AUDITOR  TMP_A (NOLOCK) 
				ON TMP_A.[IdAuditor] = AEP.[IdAuditor] '

	SET @FILTERS_FIXED_PART = '
			WHERE AEP.[Instance] = ''' + @Instance + '''
				AND AEP.[InteractionDate] >= ''' + @EvaluationDateFrom + '''
				AND AEP.[InteractionDate] <= ''' + @EvaluationDateTo + ''' 
				AND AEP.[Completed] = 0
				AND AEP.[StatusDescription] NOT IN(''INVALID'')
				AND EP.[Enabled] = 1 
				AND (
					(EP.[Evaluation_Mode] = ''X'' AND EP.[Status] = 8)
					OR
					(EP.[Evaluation_Mode] = ''M'')
				) '

	-- AGREGAMOS EL FILTRO DE ESTADO
	IF @EvaluationStatus = 0
	BEGIN
		SET @FILTERS_FIXED_PART += '
				AND AEP.[Status] = 0 '
	END
	ELSE IF @EvaluationStatus = 1
	BEGIN
		SET @FILTERS_FIXED_PART += '
				AND AEP.[Status] = 1 '
	END

	-- HACEMOS EL FILTRADO SEGUN EL ROL DEL USUARIO
	IF @USER_ROLE = 'quality_auditor'
	BEGIN
		SET @FILTERS_FIXED_PART += '
				AND	AEP.[IdAuditor] = ''' + @UserId + ''' '
	END
	ELSE IF @USER_ROLE = 'administrator' OR @USER_ROLE = 'quality_admin'
	BEGIN
		SET @FILTERS_FIXED_PART += ''
	END
	ELSE 
	BEGIN
		SET @FILTERS_FIXED_PART += '
				AND	AEP.[IdAuditor] = ''' + @UserId + ''' '
	END

	/*
	IF (@IdCampaigns IS NOT NULL AND @IdCampaigns != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @IdCampaigns + ''','','')) ';
	IF (@IdAgents IS NOT NULL AND @IdAgents != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdAgent] IN (SELECT Value FROM [dbo].[split](''' + @IdAgents + ''','','')) ';
	--IF (@EvaluationPlans IS NOT NULL AND @EvaluationPlans != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[IdEvaluationPlan] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationPlans + ''','','')) '
	IF (@InteractionTypes IS NOT NULL AND @InteractionTypes != '') SET @FILTERS_OPTIONAL_PART += 'AND AEP.[InteractionType] IN (SELECT Value FROM [dbo].[split](''' + @InteractionTypes + ''','','')) ';
	-- IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') SET @FILTERS_EXCLUDING_PART += 'AND CONCAT(AEP.[IdCampaign], ''_'', AEP.[ManagementResult]) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')) ';
	IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') 
		SET @FILTERS_OPTIONAL_PART += 'AND (
								(CONCAT(AEP.[IdCampaign], ISNULL(REPLACE(AEP.[ManagementResult], ''/'', ''_''), ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
								OR
								(CONCAT(AEP.[IdCampaign], ''_'', ISNULL(AEP.[ManagementResult], ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
								OR
								(AEP.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
							)'
	*/
	IF (@EvaluationPlanDateTo IS NOT NULL AND @EvaluationPlanDateTo != '') SET @FILTERS_OPTIONAL_PART += 'AND EP.[DateFin] <= ''' + @EvaluationPlanDateTo + '''  ';
	IF(@ContactAddress IS NOT NULL AND @ContactAddress <>'')
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND AEP.ContactAddress LIKE ''%' + @ContactAddress + '%''
			'
	END	
	
	IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
       BEGIN 
            SET @FILTERS_OPTIONAL_PART = '';
	        SET @FILTERS_EXCLUDING_PART=' AND AEP.[IdInteraction] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) ';
       END

	SET @SQL_SELECT_1 += @SQL_FROM + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART;

	--print @SQL_SELECT_1
	EXEC (@SQL_SELECT_1)

	-- ELIMINAMOS LAS TABLAS TEMPORABLES
	DROP TABLE IF EXISTS #TMP_AUDITOR
	DROP TABLE IF EXISTS #TMP_CHANNEL 

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTE SP MUETRA LA LISTA DE EVALUACIONES PENDIENTES PARA UN AUDITOR
CREATE PROCEDURE [dbo].[GetNotEvaluatedInteractionsEfectivityAuditor] (
	@Instance Varchar(50) = NULL, 
	@UserId Varchar(50) = NULL,
	@IdCampaigns VARCHAR(MAX) = NULL,
	@IdAgents Varchar(MAX) = NULL,
	@EvaluationPlans Varchar(MAX) = NULL,
	@InteractionTypes Varchar(100) = NULL,
	@InteractionResults Varchar(MAX) = NULL,
	@EvaluationDateFrom VARCHAR(50) = '1980-01-01T00:00:00',
    @EvaluationDateTo VARCHAR(50) = '2050-01-01T23:59:59',
 	@EvaluationPlanDateTo   VARCHAR(MAX) = '2050-01-01T23:59:59',
	@EvaluationStatus BIT = 1,
 	@InteractionIds     VARCHAR(MAX) = NULL,
	@ContactAddress VARCHAR(300) = NULL,
	@EfectivityPlans Varchar(MAX) = NULL
	
	)
AS
BEGIN
	SET NOCOUNT ON
	DECLARE @USER_ROLE VARCHAR(50) = ''
	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''	
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_FROM_EFECTIVITY VARCHAR(MAX) = ''
	DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_FIXED_PART VARCHAR(MAX) = ''
	DECLARE @FILTERS_FIXED_PART_EFECTIVITY VARCHAR(MAX) = ''
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = ''

	-- IDENTIFICAMOS EL ROL DEL USUARIO
	SELECT @USER_ROLE = U.[Role]
		FROM [dbo].[User] U (NOLOCK)
		WHERE U.[VCC] = @Instance
			AND U.[ID] = @UserId

	-- EMULAMOS UNA NUEVA TABLA EN BASE A AuditorEvaluationPlan DONDE SE REMOVERAN TODOS LOS REGISTROS DUPLICADOS PARA EL PK DE LA TABLA Audio
	SET @SQL_SELECT_1 += '
		WITH CTE AS (
			SELECT AEP.[Instance], 
					AEP.[IdEvaluationPlan], 
					AEP.[IdInteraction], 
					AEP.[IdAuditor], 
					AEP.[Score], 
					IEP.[AgentName],
					AEP.[LastModifiedDate],
					RN = ROW_NUMBER() OVER(PARTITION BY AEP.[Instance], AEP.[IdEvaluationPlan], AEP.[IdInteraction] ORDER BY AEP.[LastModifiedDate] DESC)
			FROM AuditorEvaluationPlan AEP (NOLOCK)
			LEFT JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = AEP.[Instance] 
					AND IEP.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
					AND IEP.[IdInteraction] = AEP.[IdInteraction]
					AND IEP.[IdAgent] = AEP.[IdAgent]
			WHERE AEP.[Instance] = ''' + @Instance + ''' 				
				AND AEP.[IdAuditor] = ''SPEECH''
				AND AEP.[StatusDescription] NOT IN(''INVALID'') '

	
	IF (@IdCampaigns IS NOT NULL AND @IdCampaigns != '') SET @SQL_SELECT_1 += 'AND AEP.[IdCampaign] IN (SELECT Value FROM [dbo].[split](''' + @IdCampaigns + ''','','')) ';
	IF (@IdAgents IS NOT NULL AND @IdAgents != '') SET @SQL_SELECT_1 += 'AND AEP.[IdAgent] IN (SELECT Value FROM [dbo].[split](''' + @IdAgents + ''','','')) ';	
	IF (@EvaluationPlans IS NOT NULL AND @EvaluationPlans != '') SET @SQL_SELECT_1 += 'AND AEP.[IdEvaluationPlan] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationPlans + ''','','')) '
	IF (@InteractionTypes IS NOT NULL AND @InteractionTypes != '') SET @SQL_SELECT_1 += 'AND AEP.[InteractionType] IN (SELECT Value FROM [dbo].[split](''' + @InteractionTypes + ''','','')) ';
	IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') 
		SET @SQL_SELECT_1 += 'AND (
								(CONCAT(AEP.[IdCampaign], ISNULL(REPLACE(AEP.[ManagementResult], ''/'', ''_''), ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
								OR
								(CONCAT(AEP.[IdCampaign], ''_'', ISNULL(AEP.[ManagementResult], ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
								OR
								(AEP.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
							)'

	SET @SQL_SELECT_1 += '
		),
		CLEANED_CTE AS (
			SELECT C.[Instance], 
					C.[IdEvaluationPlan], 
					C.[IdInteraction], 
					C.[IdAuditor], 
					C.[Score], 
					C.[AgentName],
					C.[LastModifiedDate]
			FROM [CTE] C
			WHERE C.[RN] = 1
		) '
	-- EL CTE SOLO PUEDE SER INVOCADO UNA VEZ POR ESO LO PONEMOS EN EL SELECT FINAL DONDE EL CONTADOR(RN) ES 1
	
	-- ARMAMOS EL SELECT PRINCIPAL PARA LA BUSQUEDA DE EVALUACIONES PENDIENTES DEL AUDITOR
	SET @SQL_SELECT_1 += '
		SELECT DISTINCT '
	SET @SQL_SELECT_1 += '
				AEP.[IdInteraction]															''id'', 
				AEP.[IdCampaign]															''descriptionCampaign'',
				AEP.[IdAgent]																''idAgent'',
				AEP.[IdAgent]																''descriptionAgent'',
				AEP.[InteractionDate]														''dateIni'',
				AEP.[InteractionDateEnd]													''dateFin'',
				AEP.[InteractionDuration]													''interactionTime'',
				AEP.[Completed]																''evaluated'',
				AEP.[InteractionType]														''interactionType'',
				AEP.[ManagementResult]														''dispositionCode'',
				AEP.[ContactAddress]																''address'',
				AEP.[InteractionOrigin]														''origin'',
				AEP.[IdEvaluationPlan]														''idEvaluationPlan'',
				EP.[ProcessId]																''processId'',
				EP.[Description]															''descriptionEvaluationPlan'',
				EP.[DateFin]																''dateFinEvaluationPlan'', 
				EP.[Modality]																''evaluationPlanModality'',
				EP.[Evaluation_Mode]														''evaluationPlanEvaluationMode'',
				A.[ID]																		''idAuditor'', 
				A.[Description]																''descriptionAuditor'',
				ISNULL(U.[FirstName] + '' '', '''') + ISNULL(U.[LastName], '''')			''auditorName'',						-- NOMBRE DEL AUDITOR
				C.[AgentName]																''agentName'',							-- NOMBRE DEL AGENTE
				C.[IdAuditor]																''lastEvaluationEvaluator'', 
				C.[Score]																	''lastEvaluationScore'',
				AEP.[Status]																''status'',
				ERE.[EfectivityStatus]														''statusDescription'',
				ERE.[EfectivityPlanId]														''efectivityPlanId''	'
	

	
	SET @SQL_FROM +='
	FROM [AuditorEvaluationPlan] AEP (NOLOCK) 
			INNER JOIN [EvaluationPlan] EP (NOLOCK) 
				ON (EP.[Instance] = EP.Instance
					AND EP.[ID] = AEP.IdEvaluationPlan) 
			INNER JOIN ( SELECT DISTINCT [VCC],[InteractionId],[AgentId],[EvaluationPlanId],[AuditorId],[EfectivityStatus],[EfectivityPlanId] FROM [EvaluationResultEfectivity] 
						 WHERE [EfectivityStatus] IN( ''' + 'CREATED'+''''+','+ ''''+ 'DRAFT'+''''+ ')' +						 

			')AS ERE ON 
				(AEP.[Instance] = ERE.[VCC] 
					AND AEP.[IdEvaluationPlan] = ERE.[EvaluationPlanId]
					AND AEP.[IdInteraction] = ERE.[InteractionId])
			INNER JOIN CLEANED_CTE C (NOLOCK) 
				ON C.[Instance] = AEP.[Instance] 
					AND C.[IdEvaluationPlan] = AEP.[IdEvaluationPlan] 
					AND C.[IdInteraction] = AEP.[IdInteraction]
			LEFT JOIN [dbo].[User] U (NOLOCK)
				ON U.[VCC] = AEP.[Instance]
					AND U.[ID] = ERE.[AuditorId]
			LEFT JOIN [Auditor] A (NOLOCK) 
				ON (A.[ID] = ERE.[AuditorId])
	'
	
		
	
	SET @FILTERS_FIXED_PART = '
	WHERE AEP.[Instance] = ''' + @Instance + ''' 
				--AND AEP.[Completed] = 0
				AND AEP.[InteractionDate] >= ''' + @EvaluationDateFrom + '''
				AND AEP.[InteractionDate] <= ''' + @EvaluationDateTo + '''
				AND AEP.[StatusDescription] NOT IN(''INVALID'')
				AND EP.[Enabled] = 1 
				AND (
					(EP.[Evaluation_Mode] = ''X'' AND EP.[Status] = 7)
					OR
					(EP.[Evaluation_Mode] = ''A'')
				) 
	'

	IF (@EfectivityPlans IS NOT NULL AND @EfectivityPlans != '') SET  @FILTERS_FIXED_PART += 'AND  ERE.[EfectivityPlanId] IN (SELECT Value FROM [dbo].[split](''' + @EfectivityPlans + ''','','')) '
	

	-- AGREGAMOS EL FILTRO DE ESTADO
	IF @EvaluationStatus = 0
	BEGIN
		SET @FILTERS_FIXED_PART += '
				AND AEP.[Status] = 0 '
		SET @FILTERS_FIXED_PART_EFECTIVITY += '
				AND AEP.[Status] = 0 '
	END
	ELSE IF @EvaluationStatus = 1
	BEGIN
		SET @FILTERS_FIXED_PART += '
				AND AEP.[Status] = 1 '
        
		SET @FILTERS_FIXED_PART_EFECTIVITY += '
				AND AEP.[Status] = 1 '

	END

	-- HACEMOS EL FILTRADO SEGUN EL ROL DEL USUARIO
	IF @USER_ROLE = 'quality_auditor' OR @USER_ROLE = 'administrator'
	BEGIN
		SET @FILTERS_FIXED_PART += '
				AND	ERE.[AuditorId] = ''' + @UserId + ''' '      
		
	END
	--ELSE IF @USER_ROLE = 'administrator' OR @USER_ROLE = 'quality_admin'
	--BEGIN
	--	SET @FILTERS_FIXED_PART += ''		
	--END
	--ELSE 
	--BEGIN
	--	SET @FILTERS_FIXED_PART += '
	--			AND	ERE.[AuditorId] = ''' + @UserId + ''' '
		
	--END

	IF (@EvaluationPlanDateTo IS NOT NULL AND @EvaluationPlanDateTo != '') SET @FILTERS_OPTIONAL_PART += 'AND EP.[DateFin] <= ''' + @EvaluationPlanDateTo + '''  ';
	IF(@ContactAddress IS NOT NULL AND @ContactAddress <>'')
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND AEP.ContactAddress LIKE ''%' + @ContactAddress + '%''
			'
	END	
	
	IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
       BEGIN 
            SET @FILTERS_OPTIONAL_PART = '';
	        SET @FILTERS_EXCLUDING_PART=' AND AEP.[IdInteraction] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) ';
       END

	
	
	SET @SQL_SELECT_1 += @SQL_FROM + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART;

	--PRINT @SQL_SELECT_1
	EXEC (@SQL_SELECT_1)

 	SET NOCOUNT OFF
END


GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [GetPendingAudios] 'allegro', 'procesoftp06', 'ftp'
CREATE PROCEDURE [dbo].[GetPendingAudios]
(
	@VCC				VARCHAR(50),
	@ProcessId			VARCHAR(50),
	@EnvironmentType	VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	IF(@EnvironmentType = 'ftp' OR @EnvironmentType = 'sftp')
	BEGIN
		SELECT
			CASE CONCAT(A.[Section], '_', A.[Part]) WHEN '0_0' THEN A.[ID] ELSE A.[ID] + '_' + CONVERT(VARCHAR(10), A.[Section]) + '_' + CONVERT(VARCHAR(10), A.[Part]) END		'ID',
			NULL						'Number',	
			''							'Gender',
			A.[SilenceTime]				'SilenceTime',
			A.[DurationTime]			'DurationTime',
			''							'Category1',
			''							'Category2',
			''							'Category3',
			''							'Description',
			A.[Campaign]				'Campaign',
			A.[Agent]					'Agent',
			A.[Type]					'Type',
			CONVERT(VARCHAR, A.[StartDate], 113)			'StartDate',
			CONVERT(VARCHAR, A.[EndDate], 113)				'EndDate',
			ISNULL(A.[DispositionCode], 0)		'DispositionCode'
		FROM
			[dbo].[ProcessAudio] PA WITH (NOLOCK)
			INNER JOIN [dbo].[Audio] A WITH (NOLOCK) ON CONCAT(A.[ID], '_', A.[Section], '_', A.[Part]) = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])
		WHERE
			A.[VCC] = @VCC AND PA.[ProcessId] = @ProcessId
	END
	ELSE
	BEGIN
		SELECT
			A.[ID] + '_' + CONVERT(VARCHAR(10), A.[Section]) + '_' + CONVERT(VARCHAR(10), A.[Part])		'ID',
			NULL						'Number',	
			''							'Gender',
			A.[SilenceTime]				'SilenceTime',
			A.[DurationTime]			'DurationTime',
			''							'Category1',
			''							'Category2',
			''							'Category3',
			''							'Description',
			A.[Campaign]				'Campaign',
			A.[Agent]					'Agent',
			A.[Type]					'Type',
			CONVERT(VARCHAR, A.[StartDate], 113)			'StartDate',
			CONVERT(VARCHAR, A.[EndDate], 113)				'EndDate',
			ISNULL(A.[DispositionCode], 0)		'DispositionCode'
		FROM
			[dbo].[ProcessAudio] PA WITH (NOLOCK)
			INNER JOIN [dbo].[Audio] A WITH (NOLOCK) ON PA.[AudioId] = A.[ID]
		WHERE
			A.[VCC] = @VCC AND PA.[ProcessId] = @ProcessId
	END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetProcess] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	DECLARE @TOTAL_INTERACTIONS_TO_I6_UPDATE INT = 0
	
	-- CONSULTAMOS CUANTAS INTERACCIONES CALL QUE NECESITAN ACTUALIZACION I6
	PRINT CONCAT('SEARCHING I6_UPDATE NEEDED AUDIO INTERACTIONS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	SET @TOTAL_INTERACTIONS_TO_I6_UPDATE += (
		SELECT COUNT(*)
			FROM [dbo].[Audio] A (NOLOCK)
			WHERE A.[ID] IN (
				SELECT PA.[AudioId] FROM [dbo].[ProcessAudio] (NOLOCK) PA WHERE PA.[ProcessId] = @Name
			)
			AND A.[NeedI6Update] = 1
	)

	-- CONSULTAMOS CUANTAS INTERACCIONES TEXT QUE NECESITAN ACTUALIZACION I6
	PRINT CONCAT('SEARCHING I6_UPDATE TEXT AUDIO INTERACTIONS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	SET @TOTAL_INTERACTIONS_TO_I6_UPDATE += (
		SELECT COUNT(*)
			FROM [dbo].[AsyncInteraction] AI (NOLOCK)
			WHERE AI.[ID] IN (
				SELECT PAI.[InteractionId] FROM [dbo].[ProcessAsyncInteraction] PAI WHERE PAI.[ProcessId] = @Name
			)
			AND AI.[NeedI6Update] = 1
	)

	-- CONSULTAMOS LA DATA DEL PROCESS
	PRINT CONCAT('SEARCHING PROCESS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
	SELECT P.[VCC]										'vcc',
			P.[Name]									'name',
			P.[Description]								'description',
			P.[ClientId]								'clientId',
			P.[Status]									'status',
			P.[Type]									'type',
			P.[StartDate]								'startDate',
			P.[EndDate]									'endDate',
			P.[MinDurationTime]							'minDurationTime',
			P.[MaxDurationTime]							'maxDurationTime',
			P.[Language]								'language',
			P.[DictionaryId]							'dictionaryId',
			IIF (
				P.[DictionaryId] IS NULL, 
				CONVERT(BIT, 0), 
				CONVERT(BIT, 1)
			)											'useDictionary',
			P.[Environment]								'environment',
			E.[Type]									'environmentType',
			P.[QueryId]									'queryId',
			P.[QueryDataId]								'queryDataId',
			P.[QueryDataIdDate]							'queryDataIdDate',
			P.[BatchScheduleId]							'batchScheduleId',
			IIF (
				@TOTAL_INTERACTIONS_TO_I6_UPDATE > 0,
				CONVERT(BIT, 1), 
				CONVERT(BIT, 0)
			)											'needI6Update',
			P.[CreatedDate]								'createdDate',
			P.[CreatedByUserId]							'createdByUserId',
			P.[LastModifiedDate]						'lastModifiedDate',
			P.[LastModifiedByUserId]					'lastModifiedByUserId'
		FROM [dbo].[Process] P (NOLOCK)
		INNER JOIN [dbo].[Environment] E (NOLOCK)
			ON E.[VCC] = P.[VCC]
				AND E.[Name] = P.[Environment]
		WHERE P.[VCC] = @VCC
			AND P.[Name] = @Name

	EXEC [GetProcessFilters] @VCC, @Name
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetProcessAudios] 'allegro','cupo_spain_11'
CREATE PROCEDURE [dbo].[GetProcessAudios] (
	@VCC Varchar(50) = NULL, @ProcessId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]						    'vcc',
		[ProcessId]						'processId',
		[AudioId]						'audioId',
		[Section]						'section',
		[Part]							'part',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [ProcessAudio] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [ProcessId] = @ProcessId
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[GetProcessAudioSummary] 'default', 'in_concert'
CREATE PROCEDURE [dbo].[GetProcessAudioSummary]
(
	@VCC					VARCHAR(50),
	@ProcessId				VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]				'vcc',
		[ProcessId]			'processId',
		[Category1]			'category1',
		[Category2]			'category2',
		[Category3]			'category3',
		[Description]		'description',
		[Total]				'total'
	FROM
		[dbo].[ProcessAudioSummary] (NOLOCK)
	WHERE
		[VCC] = @VCC AND [ProcessId] = @ProcessId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetProcessByName] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Name]						    'name',
		[Description]					'description',
		[ClientId]						'clientId',
		[Status]						'status',
		[StartDate]						'startDate',
		[EndDate]						'endDate',
		[MinDurationTime]				'minDurationTime',
		[MaxDurationTime]				'maxDurationTime',
		[Language]						'language',
		[DictionaryId]					'dictionaryId',
		IIF (
			[DictionaryId] IS NULL, 
			CONVERT(BIT, 0), 
			CONVERT(BIT, 1)
		)								'useDictionary',
		[Environment]					'environment',
		[QueryId]						'queryId',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Process] (NOLOCK)
	WHERE [Name] = @Name
      AND [VCC] = @VCC
	
	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
/*
Los estados son:
NotStarted	=	0% (Default)
Audios		=	20% (Mover audios)
InProcess	=	70% (Procesado en SpeechMatics)
Completed	=	100% (Procesado en IDATHA)
*/
-- [dbo].[GetProcessByStatus] 'default', 'Completed'
CREATE PROCEDURE [dbo].[GetProcessByStatus]
(
	@VCC								VARCHAR(50),
	@Status								VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON
	SELECT
		A.[VCC]							'vcc',
		A.[Description]					'description',
		A.[ClientId]					'clientId',
		A.[Status]						'status',
		A.[StartDate]					'startDate',
		A.[EndDate]						'endDate',
		A.[MinDurationTime]				'minDurationTime',
		A.[MaxDurationTime]				'maxDurationTime',
		A.[Environment]					'environment',
		A.[CreatedDate]					'createdDate',
		A.[CreatedByUserId]				'createdByUserId',
		A.[LastModifiedDate]			'lastModifiedDate',
		A.[LastModifiedByUserId]		'lastModifiedByUserId',
		B.[UserName]					'userName',
		B.[Password]					'password',
		B.[Realm]						'realm'
	FROM [Process] A (NOLOCK)
	INNER JOIN [IdathaAuth] B ON A.[Name] = B.Realm
	WHERE A.[VCC] = @VCC
	AND A.[Status] = @Status
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetProcessChannelByProcessId]
(
	@VCC				VARCHAR(50),
	@ProcessId			VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	-- BUSCAMOS LOS INTERACTIONTYPES(CHANNELS) SELECCIONADOS EN EL PROCESS
	SELECT PF.[Value] 
		FROM [dbo].[ProcessFilter] PF (NOLOCK) 
		WHERE PF.[VCC] = @VCC 
			AND PF.[ProcessId] = @ProcessId 
			AND (PF.[Key] = 'types' OR PF.[Key] = 'channel') 

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetProcesses]
(
	@VCC						VARCHAR(50) = NULL,
	@SearchCriteria				VARCHAR(50) = NULL,
	@SearchExpression			VARCHAR(50) = NULL,
	@StartDate					DATETIME,
	@EndDate					DATETIME,
	@Type						VARCHAR(50),
	@Environment				VARCHAR(50) = NULL,
	@Language					VARCHAR(50) = NULL
)
AS
BEGIN
	DECLARE @SQL NVARCHAR(MAX)
	IF @Type = 'ALL'
	BEGIN 
		SELECT [name],
				[name] as 'description',
				[type],
				[Environment]					'environment',
				[BatchScheduleId] as 'batchScheduleId'
			FROM dbo.[Process] (NOLOCK)
			WHERE [Status] = 'COMPLETE'
				AND [Type] = 'SPEECH'
				AND [VCC] = @VCC
				AND [Environment] = @Environment
				AND [Language] = @Language
		UNION ALL
		SELECT P.[name],
				EP.[Id] as 'description' ,
				[type],
				P.[Environment]					'environment',
				P.[BatchScheduleId] 
			FROM dbo.[Process] P (NOLOCK)
			INNER JOIN  dbo.[EvaluationPlan] (NOLOCK) EP 
				ON P.[VCC] = EP.[Instance]
					AND P.[Name] = EP.[ProcessId]
					AND P.[Status] = 'COMPLETE'
					AND P.[Type] = 'QUALITY'
					AND P.[VCC] = @VCC
					AND P.[Environment] = @Environment
					AND P.[Language] = @Language
					AND EP.[Status] = 7
			INNER JOIN (
			  SELECT [processId], MAX([CreatedDate]) AS CreatedDate
				FROM dbo.[EvaluationPlan] (NOLOCK) 
				WHERE [Instance] =  @VCC
					AND [ProcessId] IS NOT NULL
					AND [Status] = 7
				GROUP BY [ProcessId]
			  ) AS QPEP
			  ON EP.[ProcessId] = QPEP.[ProcessId]
				AND EP.[CreatedDate] = QPEP.[CreatedDate]

	END
	ELSE
	BEGIN
		SET @SQL = '
		SET NOCOUNT ON

		SELECT [VCC]																				''vcc'',
				[Name]																				''name'',
				[Description]																		''description'',
				[ClientId]																			''clientId'',
				[Percentage]																		''percentage'',
				[StatusDescription]																	''statusDescription'',
				(CASE WHEN [Status] IS NULL THEN ''UNKNOWN'' ELSE [Status] END)						''status'',
				[StartDate]																			''startDate'',
				[EndDate]																			''endDate'',
				[MinDurationTime]																	''minDurationTime'',
				[MaxDurationTime]																	''maxDurationTime'',
				[Language]																			''language'',
				[Environment]																		''environment'',
				[BatchScheduleId]																	''batchScheduleId'',
				[CreatedDate]																		''createdDate'',
				[CreatedByUserId]																	''createdByUserId'',
				[LastModifiedDate]																	''lastModifiedDate'',
				[LastModifiedByUserId]																''lastModifiedByUserId''
			FROM [Process] (NOLOCK)
			WHERE [VCC] = ''' + @VCC + '''
				AND ISNULL([Name], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
				AND ISNULL([Description], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''description'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
				AND ISNULL([ClientId], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''clientId'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
				AND ISNULL([Status], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''status'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
				AND ISNULL([Environment], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''environment'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END'

		IF(@StartDate IS NOT NULL AND @EndDate IS NOT NULL)
		BEGIN
			SET @SQL += '
				AND [CreatedDate] BETWEEN ''' + CONVERT(VARCHAR(10), @StartDate, 120) + ''' AND ''' + CONVERT(VARCHAR(10), @EndDate, 120) + '''
			'
		END
		SET @SQL += '
				AND [Type] = ''' + @Type + '''
		ORDER BY [Status] desc, [CreatedDate] desc 

		SET NOCOUNT OFF
		'
		EXEC(@SQL)
	END
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetProcess] 'inconcert','test'
CREATE PROCEDURE [dbo].[GetProcessEvaluationPlanTask] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL, @EvaluationPlanId Varchar(200) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		A.[VCC]						    'vcc',
		A.[Name]						'processId',
		A.[Status]						'status',
		A.[Type]						'type',
		B.[ID]							'evaluationPlanId',
		B.[IdEvaluationFormat]			'evaluationFormatId',
		B.[Status]						'evaluationPlanStatus',
		B.[Evaluation_Mode]				'evaluationPlanType'
	FROM [Process] A (NOLOCK)
	INNER JOIN [EvaluationPlan] B (NOLOCK) 
		ON A.vcc = B.Instance 
			AND  A.[Name] = B.ProcessId 
	WHERE A.[VCC] = @VCC
		AND A.[Name] = @ID
		AND B.Id = @EvaluationPlanId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetProcessFilters] 'inconcert','test'
CREATE PROCEDURE [dbo].[GetProcessFilters] (
	@VCC Varchar(50) = NULL, @ProcessId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ProcessId]						'processId',
		[Key]							'key',
		[Value]							'value',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [ProcessFilter] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [ProcessId] = @ProcessId
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetProcessFiltersData] ( 
	@VCC VARCHAR(50) = NULL, 
	@ProcessId VARCHAR(50) = NULL,
	@EvaluationFormatId VARCHAR(50) = NULL
)
AS
BEGIN
	-- ELIMINAMOS LA TABLA TEMPORAL EN CASO EXISTA
	DROP TABLE IF EXISTS  #ProcessFiltersData 

	-- CREAMOS LA TABLA TEMPORAL PARA LOS RESUMENES
	CREATE TABLE #ProcessFiltersData (
		[Channel] VARCHAR(50),
		[Campaign] VARCHAR (50),
		[AgentId] VARCHAR(50),
		[AgentName] VARCHAR(100),
		[DispositionCode] VARCHAR(50),
		[Category1] VARCHAR(100),
		[Category2] VARCHAR(100),
		[Category3] VARCHAR(100),
		[Description] VARCHAR(100),
		[StartDate] DATETIME,
		[EndDate] DATETIME,
		[DurationTime] NUMERIC(12,3)
	)
	
	-- CONSULTAMOS DATA DE LA TABLA
	--SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'AsyncInteraction' AND COLUMN_NAME IN ('Agent', 'AgentName', 'FirstTakedAgent', 'FirstTakedAgentName', 'LastTakedAgent', 'LastTakedAgentName')
	--SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'Audio' AND COLUMN_NAME IN ('Category1', 'Category2', 'Category3', 'Description', 'DispositionCode')
	
	-- INSERTAMOS LA DATA DE AUDIOS
	IF @EvaluationFormatId IS NULL
	BEGIN
		INSERT INTO #ProcessFiltersData
			SELECT 'CALL',
					A.[Campaign],
					A.[Agent],
					A.[AgentName],
					A.[DispositionCode],
					A.[Category1],
					A.[Category2],
					A.[Category3],
					A.[Description],
					A.[StartDate],
					A.[EndDate],
					A.[DurationLinguisticTime]

				FROM [dbo].[ProcessAudio] PA (NOLOCK) 
				INNER JOIN [Audio] A (NOLOCK)
					ON A.[VCC] = PA.[VCC]
						AND A.[ID] = PA.[AudioId]
						AND A.[Section] = PA.[Section]
						AND A.[Part] = PA.[Part]
				WHERE PA.[VCC] = @VCC
					AND PA.[ProcessId] = @ProcessId
	END
	ELSE
	BEGIN
		INSERT INTO #ProcessFiltersData
			SELECT 'CALL',
					A.[Campaign],
					A.[Agent],
					A.[AgentName],
					A.[DispositionCode],
					A.[Category1],
					A.[Category2],
					A.[Category3],
					A.[Description],
					A.[StartDate],
					A.[EndDate],
					A.[DurationLinguisticTime]
				FROM [dbo].[ProcessAudio] PA (NOLOCK) 
				INNER JOIN [Audio] A (NOLOCK)
					ON A.[VCC] = PA.[VCC]
						AND A.[ID] = PA.[AudioId]
						AND A.[Section] = PA.[Section]
						AND A.[Part] = PA.[Part]
				INNER JOIN [dbo].[CampaignEvaluationFormat] CEF 
					ON A.[VCC] = CEF.[Instance]
						AND A.[Campaign] = CEF.[IdCampaign]
				WHERE PA.[VCC] = @VCC
					AND PA.[ProcessId] = @ProcessId
					AND CEF.[IdEvaluationFormat] = @EvaluationFormatId
		
	END
	
	-- INSERTAMOS LA DATA DE TEXTOS
	IF @EvaluationFormatId IS NULL
	BEGIN
		INSERT INTO #ProcessFiltersData
			SELECT AI.[Channel],
					AI.[Campaign],
					ISNULL(AI.[LastTakedAgent], AI.[FirstTakedAgent]),
					ISNULL(AI.[LastTakedAgentName], AI.[FirstTakedAgentName]),
					AI.[DispositionCode],
					NULL,--AI.[Category1],
					NULL,--AI.[Category2],
					NULL,--AI.[Category3],
					NULL,--AI.[Description],
					AI.[StartDate],
					AI.[EndDate],
					AI.[DurationTime]
				FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK) 
				INNER JOIN [AsyncInteraction] AI (NOLOCK)
					ON AI.[VCC] = PAI.[VCC] 
						AND AI.[ID] = PAI.[InteractionId]
						AND AI.[Folder] = PAI.[FormattedEndDate]
						AND AI.[Campaign] = PAI.[Campaign]
				WHERE PAI.[VCC] = @VCC
					AND PAI.[ProcessId] = @ProcessId
					
	END
	ELSE
	BEGIN
		INSERT INTO #ProcessFiltersData
			SELECT AI.[Channel],
					AI.[Campaign],
					ISNULL(AI.[LastTakedAgent], AI.[FirstTakedAgent]),
					ISNULL(AI.[LastTakedAgentName], AI.[FirstTakedAgentName]),
					AI.[DispositionCode],
					NULL,--AI.[Category1],
					NULL,--AI.[Category2],
					NULL,--AI.[Category3],
					NULL,--AI.[Description]
					AI.[StartDate],
					AI.[EndDate],
					AI.[DurationTime]
				FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK) 
				INNER JOIN [AsyncInteraction] AI (NOLOCK)
					ON AI.[VCC] = PAI.[VCC] 
						AND AI.[ID] = PAI.[InteractionId]
						AND AI.[Folder] = PAI.[FormattedEndDate]
						AND AI.[Campaign] = PAI.[Campaign]
				INNER JOIN [dbo].[CampaignEvaluationFormat] CEF 
					ON AI.[VCC] = CEF.[Instance]
						AND AI.[Campaign] = CEF.[IdCampaign]
				WHERE PAI.[VCC] = @VCC
					AND PAI.[ProcessId] = @ProcessId
					AND CEF.[IdEvaluationFormat] = @EvaluationFormatId
				
		
	END

	-- SELECCIONAMOS AGRUPADO POR FILTRO
	SELECT [Channel] 'channel' 
		FROM #ProcessFiltersData
		WHERE [Channel] IS NOT NULL 
		GROUP BY [Channel] ORDER BY [Channel] ASC

	SELECT [Campaign] 'campaignId'	
		FROM #ProcessFiltersData 
		WHERE [Campaign] IS NOT NULL 
		GROUP BY [Campaign] ORDER BY [Campaign] ASC

	SELECT [AgentId] 'agentId', 
			[AgentName] 'agentName'
		FROM #ProcessFiltersData 
		WHERE [AgentId] IS NOT NULL
		GROUP BY [AgentId], [AgentName] 
		ORDER BY [AgentId], [AgentName] ASC

	SELECT [DispositionCode]	'dispositionCode',
			[Category1]			'description',
			[Category2]			'description2',
			[Category3]			'description3',
			[Description]		'description4'
		FROM #ProcessFiltersData 
		WHERE  [DispositionCode] IS NOT NULL
		GROUP BY [DispositionCode], [Category1], [Category2], [Category3], [Description] 
		--ORDER BY [DispositionCode], [Category1], [Category2], [Category3], [Description]  ASC
	SELECT MIN(StartDate) 'startDate',MAX(EndDate) 'endDate' 
	FROM #ProcessFiltersData
	SELECT [Channel] 'interactionType', COUNT(1) 'totalInteractions' ,COUNT(1) 'totalInteractionDetails',SUM([DurationTime]) 'totalInteractionTime'
	FROM #ProcessFiltersData
	GROUP BY [Channel]
	-- ELIMINAMOS LA TABLA TEMPORAL
	DROP TABLE #ProcessFiltersData 


END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
 CREATE PROCEDURE [dbo].[GetProcessI6InteractionsToUpdate] (
	@VCC VARCHAR(50) = NULL, 
	@ProcessId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	/*
	IF OBJECT_ID(N'tempdb..#I6_INTERACTIONS_TO_UPDATE') IS NOT NULL
	BEGIN
		DROP TABLE #I6_INTERACTIONS_TO_UPDATE
	END
	ELSE 
	BEGIN
		CREATE TABLE #I6_INTERACTIONS_TO_UPDATE(
			InteractionId VARCHAR(250)
		)
	END
	*/

	-- SELECCIONAMOS LOS ID DE INTERACIONES QUE TIENEN AUDIOS QUE REQUIEREN ACTUALIZACION DE I6
	-- INSERT INTO #I6_INTERACTIONS_TO_UPDATE
		SELECT PA.[VCC]								'vcc',
				PA.[ProcessId]						'processId',
				'CALL'								'interactionType',
				A.[ID]								'id',
				P.[Language]						'language',
				A.[NeedI6Update]					'needI6Update'
			FROM [dbo].[ProcessAudio] PA (NOLOCK)
			INNER JOIN [dbo].[Audio] A (NOLOCK)
				ON A.[VCC] = PA.[VCC]
					AND A.[ID] = PA.[AudioId]
					AND A.[Section] = PA.[Section]
					AND A.[Part] = PA.[Part]
			INNER JOIN [dbo].[Process] P (NOLOCK)
				ON P.[Name] = PA.[ProcessId]
			WHERE PA.[VCC] = @VCC
				AND PA.[ProcessId] = @ProcessId
				AND A.[NeedI6Update] = 1
				AND (
					(
						ISNULL(A.[IsProcess], 0) = 0				
						AND 
						ISNULL(A.[IsTranscribed], 0) = 0
					)											-- RETORNAR SOLO LAS INTERACCIONES QUE NO HAYAN SIDO TOMADAS POR OTRO PROCESS
					OR 
					(
						A.[IsProcess] = 1						
						AND 
						A.[IsCancel] = 1
					)
				)												-- RETORNAR SOLO LAS INTERACCIONES QUE HAYAN SIDO TOMADAS Y CANCELADAS POR OTRO PROCESS
			GROUP BY PA.[VCC], PA.[ProcessId], A.[ID], P.[Language], A.[NeedI6Update]
		UNION
		SELECT PAI.[VCC]							'vcc',
				PAI.[ProcessId]						'processId',
				AI.[Channel]						'interactionType',
				AI.[ID]								'id',
				P.[Language]						'language',
				AI.[NeedI6Update]					'needI6Update'
			FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
			INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
				ON AI.[VCC] = PAI.[VCC]
					AND AI.[ID] = PAI.[InteractionId]
			INNER JOIN [dbo].[Process] P (NOLOCK)
				ON P.[Name] = PAI.[ProcessId]
			WHERE PAI.[VCC] = @VCC 
				AND PAI.[ProcessId] = @ProcessId
				AND AI.[NeedI6Update] = 1
				AND (
					(
						ISNULL(AI.[IsProcess], 0) = 0				
						AND 
						ISNULL(AI.[IsTranscribed], 0) = 0
					)											-- RETORNAR SOLO LAS INTERACCIONES QUE NO HAYAN SIDO TOMADAS POR OTRO PROCESS
					OR 
					(
						AI.[IsProcess] = 1						
						AND 
						AI.[IsCancel] = 1
					)
				)												-- RETORNAR SOLO LAS INTERACCIONES QUE HAYAN SIDO TOMADAS Y CANCELADAS POR OTRO PROCESS
			GROUP BY PAI.[VCC], PAI.[ProcessId], AI.[Channel], AI.[ID], P.[Language], AI.[NeedI6Update]
	
	-- ACTUALIZAMOS LOS AUDIOS QUE SE VAN A TRABAJAR EN ESE PROCESS
	UPDATE A
		SET [IsProcess] = 1,	-- Indicar que se esta tomando como procesando para que ningun otro procesamiento en parelelo pueda utilizar el registro.
			[IsCancel] = NULL
		FROM [dbo].[ProcessAudio] PA (NOLOCK)
		INNER JOIN [dbo].[Audio] A (NOLOCK)
			ON A.[VCC] = PA.[VCC]
				AND A.[ID] = PA.[AudioId]
				AND A.[Section] = PA.[Section]
				AND A.[Part] = PA.[Part]
		WHERE PA.[VCC] = @VCC
			AND PA.[ProcessId] = @ProcessId
			AND A.[NeedI6Update] = 1
			AND (
				(
					ISNULL(A.[IsProcess], 0) = 0				
					AND 
					ISNULL(A.[IsTranscribed], 0) = 0
				)											-- RETORNAR SOLO LAS INTERACCIONES QUE NO HAYAN SIDO TOMADAS POR OTRO PROCESS
				OR 
				(
					A.[IsProcess] = 1						
					AND 
					A.[IsCancel] = 1
				)
			)												-- RETORNAR SOLO LAS INTERACCIONES QUE HAYAN SIDO TOMADAS Y CANCELADAS POR OTRO PROCESS

	-- ACTUALIZAMOS LOS TEXTOS QUE SE VAN A TRABAJAR EN ESE PROCESS
	UPDATE AI
		SET [IsProcess] = 1,
			[IsCancel] = NULL
		FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
		INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
			ON AI.[VCC] = PAI.[VCC] 
				AND AI.[ID] = PAI.[InteractionId]
		WHERE PAI.[VCC] = @VCC
			AND PAI.[ProcessId] = @ProcessId
			AND AI.[NeedI6Update] = 1
			AND (
				(
					ISNULL(AI.[IsProcess], 0) = 0				
					AND 
					ISNULL(AI.[IsTranscribed], 0) = 0
				)											-- RETORNAR SOLO LAS INTERACCIONES QUE NO HAYAN SIDO TOMADAS POR OTRO PROCESS
				OR 
				(
					AI.[IsProcess] = 1						
					AND 
					AI.[IsCancel] = 1
				)
			)	
	
	-- SELECT * FROM [dbo].[ProcessAudio] PA (NOLOCK) WHERE [VCC] = 'incollege'
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
 CREATE PROCEDURE [dbo].[GetProcessInteractionsByProcessId]
(
	@VCC		VARCHAR(50) = NULL,
	@ProcessId	VARCHAR(50) = NULL,
	@ResultType VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- ESTA FUNCION RETORNA LAS INTERACCIONES DE TEXTO Y AUDIO RELACIONADA A UN PROCESS

	IF @ResultType = 'RAW' 
	BEGIN
		-- RESULTADO TIPO RAW GENERA UN RESULTADO CON LOS DATOS Y NOMBRES DE COLUMNAS COMO VIENEN DE HD

		SELECT CONCAT(A.[ID], '_', A.[Section], '_', A.[PArt])				'ID',
				A.[ID]														'InteractionId',
				A.[Section]													'Section',
				A.[Part]													'Part',
				'CALL'														'Channel',
				A.[Number]													'Number',
				A.[Gender]													'Gender',
				A.[SilenceTime]												'Silence',
				A.[DurationTime]											'DurationTime',
				A.[DurationTime]											'AtentionTime',
				A.[Category1]												'Category1',
				A.[Category2]												'Category2',
				A.[Category3]												'Category3',
				A.[Description]												'Description',
				A.[DispositionCode]											'DispositionCode',
				A.[Campaign]												'Campaign',
				A.[Agent]													'Agent',
				A.[Number]													'contactName',
				A.[Type]													'Type',
				A.[StartDate]												'StartDate',
				A.[EndDate]													'EndDate',
				NULL														'Format',
				A.[ValidChar]												'validChar',
				A.[TotalPhrases]											'totalPhrases',
				A.[ContactAddress]											'contactAddress',
				A.[NeedI6Update]											'needI6Update',
				A.[Ender]													'ender',
				A.[ticket]													'ticket'
			FROM [dbo].[ProcessAudio] PA (NOLOCK)
			INNER JOIN [dbo].[Audio] A (NOLOCK)
				ON A.[VCC] = PA.[VCC]
					AND A.[ID] = PA.[AudioId]
					AND A.[Section] = PA.[Section]
					AND A.[Part] = PA.[Part]
			WHERE PA.[VCC] = @VCC
				AND PA.[ProcessId] = @ProcessId
				

		SELECT PAI.[VCC]													'vcc', 
				PAI.[InteractionId]											'id', 
				PAI.[FormattedEndDate]										'folder',
				AI.[ExportedContent]										'exportedContent',
				PAI.[InteractionType]										'channel',
				PAI.[Campaign]												'campaign',
				AI.[StartDate]												'startDate',
				AI.[EndDate]												'endDate',
				AI.[Account]												'account',
				AI.[Initiative]												'initiative',
				AI.[ContactName]											'contactName',
				AI.[ContactAddress]											'contactAddress',
				AI.[FirstTakedAgent]										'firstTakedAgent',
				AI.[LastTakedAgent]											'lastTakedAgent', 
				AI.[IsTaken]												'isTaken',
				AI.[TotalMessages]											'totalMessages',
				AI.[DurationTime]											'durationTime',
				AI.[AttentionTime]											'attentionTime',
				AI.[DispositionCode]										'dispositionCode',
				AI.[DispositionTreePath]									'dispositionTreePath',
				CASE WHEN AI.[TotalPhrases] IS NULL 
					THEN AI.[RawChar] ELSE 0 END							'rawChar',
				ISNULL(AI.[TotalPhrases], 0)								'totalPhrases',
				AI.[NeedI6Update]											'needI6Update',
				AI.[Ender]													'ender',
				AI.[ticket]													'ticket'
			FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
			INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
				ON AI.[VCC] = PAI.[VCC]
					AND AI.[ID] = PAI.[InteractionId]
					AND AI.[Channel] = PAI.[InteractionType]
					AND AI.[Folder] = PAI.[FormattedEndDate]
			WHERE PAI.[VCC] = @VCC
				AND PAI.[ProcessId] = @ProcessId
				
	END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetProcessInteractionsHistory](
	@VCC		VARCHAR(50),
	@ProcessId	VARCHAR(50),
	@SecondsPerRequest DECIMAL(18, 5) = 1
) AS
BEGIN
	SET NOCOUNT ON

	BEGIN

		-- RECUPERAMOS LA INFORMACION DE LOS AUDIOS DEL PROCESS QUE YA HAN SIDO TRANSCRITOS 
		SELECT 'CALL'																															'interactionType',
				COUNT(*)																														'totalAudio',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN 1 ELSE 0 END)																			'alreadyTranscribedAudio',
				SUM(CASE WHEN A.[IsCancel] = 1 THEN 1 ELSE 0 END)																				'alreadyCanceledAudio',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN A.[DurationTime] ELSE 0 END)															'alreadyTranscribedDurationTime',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN (CEILING(A.[DurationTime] / 60) * 60) ELSE 0 END)										'alreadyTranscribedRoundedDurationTime',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN A.[DurationLinguisticTime] ELSE 0 END)													'alreadyTranscribedValidDurationTime',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN (CEILING(A.[DurationLinguisticTime] / 60) * 60) ELSE 0 END)							'alreadyTranscribedRoundedValidDurationTime',
				SUM(CASE WHEN (A.[IsProcess] = 1 AND A.[IsTranscribed] IS NULL AND A.[IsCancel] IS NULL) THEN 1 ELSE 0 END)						'inProcessAudio',
				SUM(CASE WHEN (A.[IsProcess] = 1 AND A.[IsTranscribed] IS NULL AND A.[IsCancel] IS NULL) THEN A.[DurationTime] ELSE 0 END)		'inProcessDurationTime',
				SUM(CASE WHEN (A.[IsProcess] IS NULL OR (A.[IsProcess] = 1 AND A.[IsCancel] = 1)) THEN 1 ELSE 0 END)							'notTranscribedAudio',
				SUM(CASE WHEN (A.[IsProcess] IS NULL OR (A.[IsProcess] = 1 AND A.[IsCancel] = 1)) THEN A.[DurationTime] ELSE 0 END)				'notTranscribedDurationTime',
				--SUM(A.[DurationTime] / @SecondsPerRequest)																						'averageModelRequest',
				SUM(CASE WHEN A.[TotalPhrases] IS NULL THEN (A.[DurationTime] / @SecondsPerRequest) ELSE 0 END)									'averageModelRequest',
				ISNULL(SUM(A.[TotalPhrases]), 0)																								'totalCallPhrases',
				0																																'totalTextPhrases'
			FROM [dbo].[ProcessAudio] PA (NOLOCK)
			LEFT JOIN [dbo].[Audio] A (NOLOCK) 
				ON A.[VCC] =  PA.[VCC]
					AND A.[ID] = PA.[AudioId]
					AND A.[Section] = PA.[Section]
					AND A.[Part] = PA.[Part]
			WHERE PA.[VCC] = @VCC
				AND PA.[ProcessID] = @ProcessId
				-- AND A.[IsTranscribed] = 1 
				-- AND ISNULL(A.[IsProcess], 0) = 0

		SELECT PAI.[InteractionType]																											'interactionType',
				COUNT(*)																														'totalAsyncInteraction',
				SUM(CASE WHEN AI.[IsTranscribed] = 1 THEN 1 ELSE 0 END)																			'alreadyTranscribedAsyncInteraction',
				SUM(CASE WHEN AI.[IsCancel] = 1 THEN 1 ELSE 0 END)																				'alreadyCanceledAsyncInteraction',
				SUM(CASE WHEN AI.[IsTranscribed] = 1 THEN AI.[RawChar] ELSE 0 END)																'alreadyTranscribedRawChar',
				SUM(CASE WHEN AI.[IsTranscribed] = 1 THEN AI.[ValidChar] ELSE 0 END)															'alreadyTranscribedValidChar',
				SUM(CASE WHEN (AI.[IsProcess] = 1 AND AI.[IsTranscribed] IS NULL AND AI.[IsCancel] IS NULL) THEN 1 ELSE 0 END)					'inProcessAsyncInteraction',
				SUM(CASE WHEN AI.[IsProcess] = 1 THEN AI.[RawChar] ELSE 0 END)																	'inProcessRawChar',
				SUM(CASE WHEN (AI.[IsProcess] IS NULL OR (AI.[IsProcess] = 1 AND AI.[IsCancel] = 1)) THEN 1 ELSE 0 END)							'notTranscribedAsyncInteraction',
				SUM(CASE WHEN AI.[IsTranscribed] = 1 THEN AI.[RawChar] ELSE 0 END)																'notTranscribedRawChar',
				--SUM(A.[DurationTime] / @SecondsPerRequest)																					'averageModelRequest'
				0																																'averageModelRequest',
				0																																'totalCallPhrases',
				ISNULL(SUM(AI.[TotalPhrases]), 0)																								'totalTextPhrases'
			FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
			LEFT JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
				ON AI.[VCC] = PAI.[VCC]
					AND AI.[Channel] = PAI.[InteractionType]
					AND AI.[ID] = PAI.[InteractionId]
					AND AI.[Folder] = PAI.[FormattedEndDate]
			WHERE PAI.[ProcessId] = @ProcessId
			GROUP BY PAI.[InteractionType]
		
	END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetProcessInteractionsRestore] (
	@VCC VARCHAR(50) = NULL,
	@ProcessId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	BEGIN
		SELECT A.[VCC]									'vcc',
				PA.[ProcessId]							'processId',
				'CALL'									'interactionType',
				A.[ID]									'id',
				A.[Section]								'section',
				A.[Part]								'part',
				A.[StartDate]							'startDate',
				A.[EndDate]								'endDate',
				A.[Campaign]							'campaign',
				A.[DispositionCode]						'dispositionCode',
				A.[Agent]								'agent',
				A.[Type]								'origin',
				A.[RelativePath]						'relativePath',
				A.[FileName]							'fileName',
				A.[Format]								 'format',
				A.[MediaInfoTime]						'mediaInfoTime',
				A.[NeedI6Update]						'needI6Update',
				0										'isDiariced',
				P.[Language]							'language',
				P.[DictionaryId]						'dictionaryId',
				P.Retranscription						'retranscription',
				P.Diarization							'diarization',
				P.ChannelChange							'channelchange',
				E.[Name]								'name',
				E.[Type]								'environmentType',
				E.[Description]							'description',
				E.[Bucket]								'bucket',
				E.[FTPDirectory]						'ftpDirectory',
				E.[ApiHost]								'apiHost',
				E.[ApiPort]								'apiPort',
				E.[ApiUser]								'apiUser',
				E.[ApiPassword]							'apiPassword',
				A.[DurationTime]						'durationTime',
				A.[RelativeFileName]					'relativeFileName'
			FROM [dbo].[ProcessAudio] PA (NOLOCK)
			INNER JOIN [dbo].[Audio] A (NOLOCK) 
				ON A.[VCC] = PA.[VCC]
					AND CONCAT(A.[ID], '_', A.[Section], '_', A.[Part]) = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])
			INNER JOIN [dbo].[Process] P (NOLOCK) 
				ON P.[VCC] = A.[VCC]
					AND PA.ProcessId = P.[Name]
			INNER JOIN [dbo].[Environment] E (NOLOCK) 
				ON E.[VCC] = PA.[VCC]
					AND P.[Environment] = E.[Name]
			WHERE A.[VCC] = @VCC
				AND PA.[ProcessId] = @ProcessId
				AND A.[IsProcess] = 1
				AND A.[IsTranscribed] IS NULL	-- RETORNAR SOLO LAS INTERACCIONES QUE NO HAYAN SIDO TOMADAS POR OTRO PROCESS
				AND A.[IsCancel] IS NULL
		
		SELECT AI.[VCC]														'vcc',
				P.[Name]													'processId',
				AI.[Channel]												'interactionType',
				AI.[ID]														'id',
				AI.[EndDate]												'endDate',
				AI.[Folder]													'folder',
				AI.[Campaign]												'campaign',
				AI.[DispositionCode]										'dispositionCode',
				ISNULL(AI.[LastTakedAgent], AI.[FirstTakedAgent])			'agent',
				AI.[Initiative]												'origin',
				AI.[TotalMessages]											'totalMessages',
				AI.[RawChar]												'rawChar',
				AI.[ExportedContent]										'exportedContent',
				P.[Language]												'language',
				AI.[FileName]                                                'fileName',
				'allegro'													'environmentType',
				P.Retranscription											'retranscription'
		FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
		INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
			ON AI.[ID] = PAI.[InteractionId]
				AND FORMAT (AI.[EndDate], 'yyyy-MM-dd') = FORMAT (PAI.[EndDate], 'yyyy-MM-dd')
		INNER JOIN [dbo].[Process] P (NOLOCK) 
			ON P.[Name] = PAI.[ProcessId] 
		WHERE AI.[VCC] = @VCC
			AND PAI.[ProcessId] = @ProcessId
			AND AI.[IsProcess] = 1
			AND AI.[IsTranscribed] IS NULL	-- RETORNAR SOLO LAS INTERACCIONES QUE NO HAYAN SIDO TOMADAS POR OTRO PROCESS
			AND AI.[IsCancel] IS NULL

	END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetProcessInteractionsTask]
(
	@VCC		VARCHAR(50),
	@ProcessId	VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	BEGIN
		-- ACTUALIZAMOS LOS REGISTROS DE INTERACCIONES SINCRONAS(CALL) Y RETORNAMOS LOS DETALLES DE LAS QUE SE VAN A TRANSCRIBIR

		-- Actualiza en el primer momento que se consulta a estos registros, con el valor 1 en el campo IsProcess. Indica que se encuentra utilizado por un proceso.
		UPDATE [dbo].[Audio] SET
			[dbo].[Audio].[IsProcess] = 1,	-- Indicar que se esta tomando como procesando para que ningun otro procesamiento en parelelo pueda utilizar el registro.
			[dbo].[Audio].[IsCancel] = NULL, -- Limpiamos el campo isCancel para retomar los audios que fallaron en procesos pasados
			[dbo].[Audio].[ErrorMessage] = NULL
		
		-- Retornamos todas las columnas a listar.
		OUTPUT INSERTED.[VCC]							'vcc',
				PA.[ProcessId]							'processId',
				'CALL'									'interactionType',
				INSERTED.[ID]							'id',
				INSERTED.[Section]						'section',
				INSERTED.[Part]							'part',
				INSERTED.[StartDate]					'startDate',
				INSERTED.[EndDate]						'endDate',
				INSERTED.[Campaign]						'campaign',
				INSERTED.[DispositionCode]				'dispositionCode',
				INSERTED.[Agent]						'agent',
				INSERTED.[Type]							'origin',
				INSERTED.[RelativePath]					'relativePath',
				INSERTED.[FileName]						'fileName',
				INSERTED.[Format]                       'format',
				INSERTED.[MediaInfoTime]                'mediaInfoTime',
				INSERTED.[NeedI6Update]					'needI6Update',
				DELETED.[ErrorMessage]					'errorMessage',
				0										'isDiariced',
				P.[Language]							'language',
				P.[DictionaryId]						'dictionaryId',
				P.Retranscription						'retranscription',
				P.Diarization							'diarization',
				P.ChannelChange							'channelchange',
				E.[Name]								'name',
				E.[Type]								'environmentType',
				E.[Description]							'description',
				E.[Bucket]								'bucket',
				E.[FTPDirectory]						'ftpDirectory',
				E.[ApiHost]								'apiHost',
				E.[ApiPort]								'apiPort',
				E.[ApiUser]								'apiUser',
				E.[ApiPassword]							'apiPassword',
				--E.[DbDisabled]							'dbDisabled',
				--E.[DbServer]							'dbServer',
				--E.[DbPort]								'dbPort',
				--E.[DbUser]								'dbUser',
				--E.[DbPassword]							'dbPassword',
				--E.[PbxDisabled]							'pbxDisabled',
				--E.[PbxServer]							'pbxServer',
				--E.[PbxPort]								'pbxPort',
				--E.[PbxUser]								'pbxUser',
				--E.[PbxPassword]							'pbxPassword',
				INSERTED.[DurationTime]					'durationTime',
				INSERTED.[RelativeFileName]				'relativeFileName'
		FROM [dbo].[ProcessAudio] PA (NOLOCK)
		INNER JOIN [dbo].[Audio] A (NOLOCK) 
			ON A.[VCC] = PA.[VCC]
				AND CONCAT(A.[ID], '_', A.[Section], '_', A.[Part]) = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])
		INNER JOIN [dbo].[Process] P (NOLOCK) 
			ON P.[VCC] = A.[VCC]
				AND PA.ProcessId = P.[Name]
		INNER JOIN [dbo].[Environment] E (NOLOCK) 
			ON E.[VCC] = PA.[VCC]
				AND P.[Environment] = E.[Name]
		WHERE A.[VCC] = @VCC
			AND PA.[ProcessId] = @ProcessId
			AND (
				(
					ISNULL(A.[IsProcess], 0) = 0				
					AND 
					ISNULL(A.[IsTranscribed], 0) = 0	-- RETORNAR SOLO LAS INTERACCIONES QUE NO HAYAN SIDO TOMADAS POR OTRO PROCESS
				)
				OR 
				(
					A.[IsProcess] = 1						
					AND 
					A.[IsCancel] = 1
				)
			
				
				
			)					-- RETORNAR SOLO LAS INTERACCIONES QUE HAYAN SIDO TOMADAS Y CANCELADAS POR OTRO PROCESS

		-- ACTUALIZAMOS LOS REGISTROS DE INTERACCIONES ASINCRONAS Y RETORNAMOS LOS DETALLES DE LAS QUE SE VAN A TRANSCRIBIR
		UPDATE [dbo].[AsyncInteraction] 
		SET [dbo].[AsyncInteraction].[IsProcess] = 1,
			[dbo].[AsyncInteraction].[IsCancel] = NULL,
			[dbo].[AsyncInteraction].[ErrorMessage] = NULL
		OUTPUT INSERTED.[VCC]														'vcc',
				P.[Name]															'processId',
				INSERTED.[Channel]													'interactionType',
				INSERTED.[ID]														'id',
				INSERTED.[EndDate]													'endDate',
				INSERTED.[Folder]													'folder',
				INSERTED.[Campaign]													'campaign',
				INSERTED.[DispositionCode]											'dispositionCode',
				ISNULL(INSERTED.[LastTakedAgent], INSERTED.[FirstTakedAgent])		'agent',
				INSERTED.[Initiative]												'origin',
				INSERTED.[TotalMessages]											'totalMessages',
				INSERTED.[RawChar]													'rawChar',
				INSERTED.[ExportedContent]											'exportedContent',
				P.[Language]														'language',
				INSERTED.[FileName]                                                 'fileName',
				'allegro'															'environmentType',
				P.Retranscription													'retranscription'
		FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
		INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
			ON AI.[ID] = PAI.[InteractionId]
				-- AND AI.[EndDate] = PAI.[EndDate]
				AND FORMAT (AI.[EndDate], 'yyyy-MM-dd') = FORMAT (PAI.[EndDate], 'yyyy-MM-dd')
		INNER JOIN [dbo].[Process] P (NOLOCK) 
			ON P.[Name] = PAI.[ProcessId] 
		WHERE AI.[VCC] = @VCC
			AND PAI.[ProcessId] = @ProcessId
			AND (
				(
					ISNULL(AI.[IsProcess], 0) = 0
					AND 
					ISNULL(AI.[IsTranscribed], 0) = 0)	-- RETORNAR SOLO LAS INTERACCIONES QUE NO HAYAN SIDO TOMADAS POR OTRO PROCESS
				OR 
				(
					AI.[IsProcess] = 1
					AND 
					AI.[IsCancel] = 1
				)

				
			)					-- RETORNAR SOLO LAS INTERACCIONES QUE HAYAN SIDO TOMADAS Y CANCELADAS POR OTRO PROCESS
	END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--[dbo].[GetProcessSummary] 'inconcert', '1FC1C683-191F-4A46-A932-31C5D8116FC2'
CREATE PROCEDURE [dbo].[GetProcessSummary]
(
	@VCC					VARCHAR(50),
	@ProcessId				VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	-- PARA NO HACER UNA MODIFICACION DEMASIADO FUERTE AL SP, BUSCAMOS EL TIEMPO PROCESADO POR SEPARADO

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL ACUMULADO DE TIEMPO PROCESADO
	DECLARE @TOTAL_DURATION_LINGUISTIC_TIME INT = 0

	-- CONSULTAMOS EL TOTAL DE TIEMPO PROCESADO DE LOS AUDIOS DEL PROCESO
	SELECT @TOTAL_DURATION_LINGUISTIC_TIME = SUM(A.[DurationLinguisticTime]) 
		FROM [dbo].[ProcessAudio] PA (NOLOCK)
		LEFT JOIN [dbo].[Audio] A (NOLOCK)
			ON A.[ID] = PA.[AudioId] 
				AND A.[Section] = PA.[Section]
				AND A.[Part] = PA.[Part]
		WHERE PA.VCC = @VCC 
				AND PA.[ProcessId] = @ProcessId

	SELECT
		[VCC]																			'vcc',
		[ProcessId]																		'processId',
		[SilenceTime]																	'silenceTime',
		[AverageSilenceTime]															'averageSilenceTime',
		[DurationTime]																	'durationTime',
		CONVERT(varchar, DATEADD(ms, @TOTAL_DURATION_LINGUISTIC_TIME * 1000, 0), 108)	'durationLinguisticTime',
		[AverageDurationTime]															'averageDurationTime',
		[DurationTimeRate]																'durationTimeRate',
		[CreatedDate]																	'createdDate',
		[CreatedByUserId]																'createdByUserId',
		[LastModifiedDate]																'lastModifiedDate',
		[LastModifiedByUserId]															'lastModifiedByUserId'
	FROM [dbo].[ProcessSummary] (NOLOCK)
	WHERE [VCC] = @VCC AND [ProcessId] = @ProcessId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetProcess] 'inconcert','test'
CREATE PROCEDURE [dbo].[GetProcessTask] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]						    'vcc',
		[Name]						    'name',
		[Description]					'description',
		[ClientId]						'clientId',
		[Status]						'status',
		[Type]							'type',
		[StartDate]						'startDate',
		[EndDate]						'endDate',
		[MinDurationTime]				'minDurationTime',
		[MaxDurationTime]				'maxDurationTime',
		[Language]						'language',
		[DictionaryId]					'dictionaryId',
		[Environment]					'environment',
		[QueryId]						'queryId',
		[QueryDataId]					'queryDataId',
		[QueryDataIdDate]				'queryDataIdDate',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Process] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [Name] = @ID

	EXEC [GetProcessFilters] @VCC, @ID
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetProcessUsage]
(
	@VCC				VARCHAR(50),
	@ProcessId			VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	BEGIN
		-- Validar para Proceso. Avisar si esta en progreso.
		SELECT [Name]				'name'
			FROM [dbo].[Process] (NOLOCK)
			WHERE [VCC] = @VCC
				AND [Name] = @ProcessId
				AND [Status] NOT IN('COMPLETE', 'FAILED')
	END

	-- LISTAMOS LOS ESTUDIOS REGULARES
	BEGIN
		-- Validar para Estudio. Avisar si esta asociado a un estudio.
		SELECT DISTINCT [Name]		'name'
			FROM [dbo].[Study] (NOLOCK)
			WHERE [VCC] = @VCC
				AND [ProcessId] = @ProcessId
				AND [Type] = 'SPEECH'
	END

	-- LISTAMOS LOS ESTUDIOS DE PLANTILLA AGENDADA
	BEGIN
		SELECT DISTINCT [Name]		'name'
			FROM [dbo].[Study] (NOLOCK)
			WHERE [VCC] = @VCC
				AND [ProcessId] = @ProcessId
				AND [Type] = 'TEMPLATE'
	END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE  PROCEDURE [dbo].[GetProcessValidDurationTime](
	@VCC VARCHAR(50) = NULL,
	@ProcessId VARCHAR(50)
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE RETORNAR LA CANTIDAD DE SEGUNDOS DE TRANSCRIPCION DE AUDIO CORRESPONDIENTE A UN PROCESS
	SET XACT_ABORT, NOCOUNT ON
	
	SELECT P.[Name] 'name',
			SUM(A.[DurationLinguisticTime]) 'validDurationTime'
		FROM Process P (NOLOCK)
		LEFT JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
			ON PA.[ProcessId] = P.[Name]
		LEFT JOIN [dbo].[Audio] A (NOLOCK) 
			ON A.[ID] = PA.[AudioId]
				AND A.[Section] = PA.[Section]
				AND A.[Part] = PA.[Part]
		WHERE P.[VCC] = @VCC 
			AND P.[Name] = @ProcessId
		GROUP BY P.[Name] 

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetProfileImage] (
	@VCC Varchar(50) = NULL, @Type Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Type]								'type',
		[ID]								'id',
		[ContentType]						'contentType',
		[ContentData]						'contentData'
	FROM [ProfileImage] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [Type] = @Type
	AND [ID] = @ID

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
Create PROCEDURE [dbo].[GetRealtimeEvaluationFormatsFromHistoricEvaluationPlans] (
	@VCC VARCHAR(50)
) AS
BEGIN
	SET NOCOUNT ON

 	SELECT DISTINCT HEF.[ID]			'id',
					HEF.[Description]	'description' 
		FROM [dbo].[EvaluationResult] ER (NOLOCK)
		INNER JOIN [dbo].[HistoricEvaluationFormat] HEF (NOLOCK)
			ON HEF.[EvaluationPlanId] = ER.[IdEvaluationPlan]
				AND HEF.[VCC] = ER.[Instance] 
				AND HEF.[ID] = ER.[IdEvaluationFormat]
		INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			ON AEP.[Instance] = ER.[Instance]
				AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]
		WHERE HEF.[VCC] = @VCC 
			AND HEF.[ExecutionType] = 'REALTIME'
			AND AEP.[Completed] = 1
		ORDER BY HEF.[Description] ASC

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetRealtimeEvaluationPlans] (
	@VCC VARCHAR(50) = NULL, 
	@EvaluationFormatId VARCHAR(50) = NULL, 
	@CampaignId VARCHAR(100) = NULL,
	@IsActive BIT = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- ESTE SP RETORNA LA LISTA DE PLANES REALTIME 
	SELECT [VCC]						'vcc',
			[ID]						'id',
			[EvaluationFormatId]		'evaluationFormatId',
			[CampaignId]				'campaignId'
		FROM [dbo].[RealtimeEvaluationPlan] (NOLOCK)
		WHERE [VCC] = @VCC
			AND [EvaluationFormatId] = @EvaluationFormatId
			AND [CampaignId] = @CampaignId
			AND [IsActive] = CASE WHEN @IsActive IS NOT NULL THEN @IsActive ELSE [IsActive] END
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetReport] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL, @Role Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	IF(@Role IS NULL)
	BEGIN
		SELECT
			[ID]								'id',
			[Description] 						'description',
			[Procedure] 						'procedure',
			[StartDate]							'startDate',
			[EndDate]							'endDate',
			[Frecuency]							'frecuency',
			[HourSpace]							'hourSpace',
			[ExactTime]							'exactTime',
			[Weekday]							'weekday',
			[DayMonth]							'dayMonth',
			[Summary]							'summary',
			[Filter]							'filter',
			[Email]								'email',
			[Type]								'type'
		FROM [Report] (NOLOCK)
		WHERE [VCC] = @VCC
		AND ([Type] IN (SELECT [Product] FROM VCCLicense (NOLOCK)  WHERE [VCC] = @VCC AND [Status] = 1) OR [Type] = 'SHARED')
		AND [ID] = @ID
	END
	ELSE IF (@Role = 'business_administrator')
	BEGIN
		SELECT
			[ID]								'id',
			[Description] 						'description',
			[Procedure] 						'procedure',
			[StartDate]							'startDate',
			[EndDate]							'endDate',
			[Frecuency]							'frecuency',
			[HourSpace]							'hourSpace',
			[ExactTime]							'exactTime',
			[Weekday]							'weekday',
			[DayMonth]							'dayMonth',
			[Summary]							'summary',
			[Filter]							'filter',
			[Email]								'email',
			[Type]								'type'
		FROM [Report] (NOLOCK)
		WHERE [VCC] = @VCC
		AND [Type] IN ('BILLING', 'BUSINESS') 
		AND [ID] = @ID
	END
	ELSE IF @Role = 'provisioning'
	BEGIN 
		SELECT
			[ID]								'id',
			[Description] 						'description',
			[Procedure] 						'procedure',
			[StartDate]							'startDate',
			[EndDate]							'endDate',
			[Frecuency]							'frecuency',
			[HourSpace]							'hourSpace',
			[ExactTime]							'exactTime',
			[Weekday]							'weekday',
			[DayMonth]							'dayMonth',
			[Summary]							'summary',
			[Filter]							'filter',
			[Email]								'email',
			[Type]								'type'
		FROM [Report] (NOLOCK)
		WHERE  [VCC] = @VCC
		AND [Type] IN ('BILLING') 
		AND [ID] = @ID
	END
	
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetReportByDetailAgent] (@VCC varchar(50) = '', @Campaign varchar(100) = '', @Plan varchar(50) = '',
@EvaluationFormat varchar(50), @StartDate varchar(50) = '', @EndDate varchar(50) = '', @IdAuditor varchar(50) = '')
AS
  DECLARE @cols varchar(max)
  DECLARE @query varchar(max)
  
  -- GetReportByDetailAgent 'allegro','entrantes','Plan_2_nov01-0356','nov01-0356','20191001','20191201','isidoro'
  SET NOCOUNT ON

  SELECT
    * INTO #TempEvaluationResult
  FROM (SELECT
    EP.Description AS 'DescriptionPlan',
    EF.Description AS 'DescriptionFormat',
    GP.Description + '-' + SGP.Description + '-' + A.Title AS 'Ability',
    ER.AgentDescription,
    ER.CampaignDescription,
    ER.IdInteraction,
    ER.Qualification,
    Format(ER.EvaluationDate, 'dd/MM/yyyy hh:mm:ss') AS 'EvaluationDate',
    ER.IdAuditor,
	EP.Evaluation_Mode	
  FROM EvaluationResult ER
  LEFT JOIN EvaluationFormat EF
    ON ER.IdEvaluationFormat = EF.ID
  LEFT JOIN Ability A
    ON A.ID = ER.IdAbility
  LEFT JOIN EvaluationPlan EP
    ON EP.ID = ER.IdEvaluationPlan
  LEFT JOIN [Group] GP
    ON GP.ID = ER.IdGroup
  LEFT JOIN SubGroup SGP
    ON SGP.ID = ER.IdSubGroup
  WHERE EF.Description LIKE CASE
    WHEN @EvaluationFormat <> '' THEN @EvaluationFormat
    ELSE EF.Description
  END
  AND ER.Instance LIKE CASE
    WHEN @VCC <> '' THEN @VCC
    ELSE ER.Instance
  END
  AND EP.Description LIKE CASE
    WHEN @Plan <> '' THEN @Plan
    ELSE EP.Description
  END
  AND ER.CampaignDescription LIKE CASE
    WHEN @Campaign <> '' THEN @Campaign
    ELSE ER.CampaignDescription
  END
  AND ER.IdAuditor LIKE CASE
    WHEN @IdAuditor <> '' THEN @IdAuditor
    ELSE ER.IdAuditor
  END
  AND ER.EvaluationDate >=
                          CASE
                            WHEN @StartDate <> '' THEN @StartDate
                            ELSE '19800101'
                          END
  AND ER.EvaluationDate <
                         CASE
                           WHEN @EndDate <> '' THEN @EndDate + ' 23:59'
                           ELSE '20500101'
                         END) a



  SET @cols = STUFF((SELECT DISTINCT
    ',' + QUOTENAME(Ability)
  FROM #TempEvaluationResult
  FOR xml PATH (''), TYPE)
  .value('.', 'varchar(Max)')
  , 1, 1, '')
  --Select @cols
  IF @cols IS NOT NULL
  BEGIN

    DECLARE @type varchar(max)=(Select Top 1 Evaluation_Mode From #TempEvaluationResult)

    DECLARE @Total varchar(max) = STUFF((SELECT DISTINCT
      '+ ISNULL(' + QUOTENAME(Ability) + ',0)'
    FROM #TempEvaluationResult
    FOR xml PATH (''), TYPE)
    .value('.', 'varchar(Max)')
    , 1, 1, '')
   -- SELECT @type
	If @type = 'M'
	BEGIN
    SET @query = ' 
				Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,InteractionDuration as DurationTime, 
				null As Number,InteractionDateStart As StartDate,null As Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join (
				Select Distinct IdInteraction,InteractionDuration,InteractionDateStart From InteractionEvaluationPlan (Nolock)
				) as A On
				A.IdInteraction=p.IdInteraction'
    END
	ELSE 
	BEGIN
    SET @query = ' 
				Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,
				A.DurationTime,A.Number,A.StartDate,A.Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join Audio as A On
				A.ID=p.IdInteraction'
	END
	--Print(@query)
    EXECUTE (@query)
    DROP TABLE #TempEvaluationResult
  END
  ELSE
  BEGIN
    DROP TABLE #TempEvaluationResult
    SELECT
      '' AS DescriptionPlan,
      '' AS DescriptionFormat,
      '' AS AgentDescription,
      '' AS CampaignDescription,
      '' AS IdInteraction,
      '' AS EvaluationDate,
      '' AS IdAuditor
  END
  SET NOCOUNT OFF
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

--ALTER PROCEDURE [dbo].[GetReportBySummaryAgent] (@VCC varchar(50) = '', @Campaign varchar(50) = '', @Plan varchar(50) = '',
--@EvaluationFormat varchar(50), @StartDate varchar(50) = '', @EndDate varchar(50) = '', @IdAuditor varchar(50) = '')
--AS

  --DECLARE @VCC varchar(50) = '' 
  --DECLARE @Campaign varchar(50) = '' 
  --DECLARE @Plan varchar(50) = ''
  --DECLARE @EvaluationFormat varchar(50) ='nov01-0356'
  --DECLARE @StartDate varchar(50) = ''
  --DECLARE @EndDate varchar(50) = ''
  --DECLARE @IdAuditor varchar(50) = ''
  --DECLARE @cols varchar(max)
  --DECLARE @query varchar(max)

  CREATE PROCEDURE [dbo].[GetReportBySummaryAgent] (@VCC varchar(50) = '', @Campaign varchar(100) = '', @Plan varchar(50) = '',
@EvaluationFormat varchar(50) = '', @StartDate varchar(50) = '', @EndDate varchar(50) = '', @IdAuditor varchar(50) = '')
AS
  DECLARE @cols varchar(max)
  DECLARE @query varchar(max)
  --GetReportBySummaryAgent 'allegro','entrantes','Plan_2_nov01-0356','nov01-0356','20191010','20191231',''
  --SET @EvaluationFormat = 'nov01-0356'
  SET NOCOUNT ON
  SELECT
    * INTO #TempEvaluationResult
  FROM (SELECT
    EP.Description AS 'DescriptionPlan',
    EF.Description AS 'DescriptionFormat',
    GP.Description + '-' + SGP.Description + '-' + A.Title AS 'Ability',
    ER.AgentDescription,
    ER.CampaignDescription,
    ER.IdInteraction,
    ER.Qualification,
    Format(ER.EvaluationDate, 'dd/MM/yyyy hh:mm:ss') AS 'EvaluationDate',
    ER.IdAuditor,
	EP.Evaluation_Mode	
  FROM EvaluationResult ER
  LEFT JOIN EvaluationFormat EF
    ON ER.IdEvaluationFormat = EF.ID
  LEFT JOIN Ability A
    ON A.ID = ER.IdAbility
  LEFT JOIN EvaluationPlan EP
    ON EP.ID = ER.IdEvaluationPlan
  LEFT JOIN [Group] GP
    ON GP.ID = ER.IdGroup
  LEFT JOIN SubGroup SGP
    ON SGP.ID = ER.IdSubGroup
  WHERE EF.Description LIKE CASE
    WHEN @EvaluationFormat <> '' THEN @EvaluationFormat
    ELSE EF.Description
  END
  AND ER.Instance LIKE CASE
    WHEN @VCC <> '' THEN @VCC
    ELSE ER.Instance
  END
  AND EP.Description LIKE CASE
    WHEN @Plan <> '' THEN @Plan
    ELSE EP.Description
  END
  AND ER.CampaignDescription LIKE CASE
    WHEN @Campaign <> '' THEN @Campaign
    ELSE ER.CampaignDescription
  END
  AND ER.IdAuditor LIKE CASE
    WHEN @IdAuditor <> '' THEN @IdAuditor
    ELSE ER.IdAuditor
  END
  AND ER.EvaluationDate >=
                          CASE
                            WHEN @StartDate <> '' THEN @StartDate
                            ELSE '19800101'
                          END
  AND ER.EvaluationDate <
                         CASE
                           WHEN @EndDate <> '' THEN @EndDate + ' 23:59'
                           ELSE '20500101'
                         END) a



  SET @cols = STUFF((SELECT DISTINCT
    ',' + QUOTENAME(Ability)
  FROM #TempEvaluationResult
  FOR xml PATH (''), TYPE)
  .value('.', 'varchar(Max)')
  , 1, 1, '')
  --Select @cols
  IF @cols IS NOT NULL
  BEGIN

    DECLARE @type varchar(max)=(Select Top 1 Evaluation_Mode From #TempEvaluationResult)

    DECLARE @Total varchar(max) = STUFF((SELECT DISTINCT
      '+ ISNULL(' + QUOTENAME(Ability) + ',0)'
    FROM #TempEvaluationResult
    FOR xml PATH (''), TYPE)
    .value('.', 'varchar(Max)')
    , 1, 1, '')
	-- ROUND(AVG(CAST(ER.Qualification AS float)), 2) AS 'Qualification',
	DECLARE @colavg varchar(max) = STUFF((SELECT DISTINCT
      ', ROUND(AVG(CAST(q.' + QUOTENAME(Ability) + ' AS float)), 2) As ['+Ability +']'
    FROM #TempEvaluationResult
    FOR xml PATH (''), TYPE)
    .value('.', 'varchar(Max)')
    , 1, 1, '')
    --SELECT @colavg
	If @type = 'M'
	BEGIN
    SET @query = ' 
				Select q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription,count(q.IdInteraction) As Audits,
				sum(q.DurationTime) As DurationTime,ROUND(AVG(CAST(q.Note AS float)), 2) As Note, '+ @colavg +' From
				(Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,InteractionDuration as DurationTime, 
				null As Number,InteractionDateStart As StartDate,null As Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join (
				Select Distinct IdInteraction,InteractionDuration,InteractionDateStart From InteractionEvaluationPlan (Nolock)
				) as A On
				A.IdInteraction=p.IdInteraction)q
				Group By 
				q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription
				'
    END
	ELSE 
	BEGIN
    SET @query = '
	
			    Select q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription,count(q.IdInteraction) As Audits,
				sum(q.DurationTime) As DurationTime,ROUND(AVG(CAST(q.Note AS float)), 2) As Note, '+ @colavg +' From 
				(Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,
				A.DurationTime,A.Number,A.StartDate,A.Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join Audio as A On
				A.ID=p.IdInteraction)q
				Group By 
				q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription'
	END
	Print(@query)
    EXECUTE (@query)
	--Select * From @query

    DROP TABLE #TempEvaluationResult
  END
  ELSE
  BEGIN
    DROP TABLE #TempEvaluationResult
    SELECT
      '' AS DescriptionPlan,
      '' AS DescriptionFormat,
      '' AS AgentDescription,
      '' AS CampaignDescription,
      '' AS IdInteraction,
      '' AS EvaluationDate,
      '' AS IdAuditor
  END
  SET NOCOUNT OFF
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetReportProcedureParameters](
	@Procedure Varchar(200)
	)
AS
BEGIN
	SET NOCOUNT ON

    DECLARE 
        @object_id INT,
        @paramID INT,
        @paramName SYSNAME,
        @definition NVARCHAR(MAX),
        @t NVARCHAR(MAX),
        @loc1 INT,
        @loc2 INT,
        @loc3 INT,
        @loc4 INT,
        @has_default_value BIT;

    SET @object_id = OBJECT_ID(@Procedure);

    IF (@object_id IS NOT NULL)
    BEGIN
    
        SELECT @definition = OBJECT_DEFINITION(@object_id);

        CREATE TABLE #params
        (
            parameter_id        INT PRIMARY KEY,
            has_default_value    BIT NOT NULL DEFAULT (0)
        );

        DECLARE c CURSOR
        LOCAL FORWARD_ONLY STATIC READ_ONLY
        FOR 
            SELECT 
                parameter_id,
                [name]
            FROM
                sys.parameters
            WHERE
                [object_id] = @object_id;

        OPEN c;

        FETCH NEXT FROM c INTO @paramID, @paramName;

        WHILE (@@FETCH_STATUS = 0)
        BEGIN
    
            SELECT
                @t = SUBSTRING
                (
                    @definition,
                    CHARINDEX(@paramName, @definition),
                    4000
                ),
                @has_default_value = 0;
            
            SET @loc1 = COALESCE(NULLIF(CHARINDEX('''', @t), 0), 4000);
            SET @loc2 = COALESCE(NULLIF(CHARINDEX(',', @t), 0), 4000);
            SET @loc3 = NULLIF(CHARINDEX('OUTPUT', @t), 0);
            SET @loc4 = NULLIF(CHARINDEX('AS', @t), 0);
            
            SET @loc1 = CASE WHEN @loc2 < @loc1 THEN @loc2 ELSE @loc1 END;
            SET @loc1 = CASE WHEN @loc3 < @loc1 THEN @loc3 ELSE @loc1 END;
            SET @loc1 = CASE WHEN @loc4 < @loc1 THEN @loc4 ELSE @loc1 END;
            
            IF CHARINDEX('=', LTRIM(RTRIM(SUBSTRING(@t, 1, @loc1)))) > 0
                SET @has_default_value = 1;

            INSERT #params
            (
                parameter_id,
                has_default_value
            )
            SELECT
                @paramID,
                @has_default_value;
            
            FETCH NEXT FROM c INTO @paramID, @paramName;
        END
        
        SELECT 
            sp.[name]						'parameter',
            --sp.[parameter_id]				'ordinal',
            st.[name]						'type',
           --sp.[max_length]				'maxLength',
           --sp.[precision]					'precission',
           --sp.[scale]						'scale',
           --sp.[is_output]					'isOutput',
            p.[has_default_value]			'hasDefaultValue',
            IIF(RFA.[Label] IS NULL,REPLACE(sp.[name],'@',''),RFA.[Label])               'label',
            IIF(RFA.[IsDisabled] IS NULL,CAST(0 AS BIT),RFA.[IsDisabled])                'isDisabled',
            IIF(RFA.[IsDropdown] IS NULL,CAST(0 AS BIT),RFA.[IsDropdown])                'isDropdown',
            IIF(RFA.[IsMultiple] IS NULL,CAST(0 AS BIT),RFA.[IsMultiple])                'isMultiple',
            RFA.[Procedure]                 'procedure',
            RFA.[Arguments]                 '_arguments',
            RFA.[Variables]                 'variables'
        FROM
            sys.parameters sp
        INNER JOIN
            #params p
        ON
            sp.parameter_id = p.parameter_id
        INNER JOIN
            sys.types st
        ON
            sp.user_type_id = st.user_type_id
        LEFT JOIN 
                [dbo].[ReportFieldAttribute] RFA (NOLOCK)
         ON RFA.[Field] =  sp.[name]
        
        WHERE
            sp.[object_id] = @object_id;
        
        CLOSE c;
        DEALLOCATE c;
        DROP TABLE #params;
    END
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetReportProcedures]
AS
BEGIN
	SET NOCOUNT ON

	SELECT [name]
	FROM [sys].[procedures] (NOLOCK)
	WHERE [type] = 'P' 
	AND [name] LIKE 'Report%'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetReports] (
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL, @Role Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	IF(@Role IS NULL)
	BEGIN
		IF @VCC = 'system' 
		BEGIN
			-- VOLCADO PARA LA CREACION DE VCC EN PROVISIONING
			SELECT
				[ID]								'id',
				[Description] 						'description',
				[Procedure] 						'procedure',
				[StartDate]							'startDate',
				[EndDate]							'endDate',
				[Frecuency]							'frecuency',
				[HourSpace]							'hourSpace',
				[ExactTime]							'exactTime',
				[Weekday]							'weekday',
				[DayMonth]							'dayMonth',
				[Summary]							'summary',
				[Filter]							'filter',
				[Email]								'email',
				[Type]								'type'
			FROM [Report] (NOLOCK)
			WHERE [VCC] = @VCC
			AND ISNULL([Id], '') + ' ' + ISNULL([Id], '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND (
				[Type] IN (SELECT [Product] FROM VCCLicense (NOLOCK)  WHERE [VCC] = @VCC AND [Status] = 1)
					OR [Type] IN ('SHARED', 'BILLING', 'BUSINESS')
			)
		END 
		ELSE
		BEGIN
			-- VOLCADO PARA LA VISTA DE REPORTES SPEECH Y QUALITY
			SELECT
				[ID]								'id',
				[Description] 						'description',
				[Procedure] 						'procedure',
				[StartDate]							'startDate',
				[EndDate]							'endDate',
				[Frecuency]							'frecuency',
				[HourSpace]							'hourSpace',
				[ExactTime]							'exactTime',
				[Weekday]							'weekday',
				[DayMonth]							'dayMonth',
				[Summary]							'summary',
				[Filter]							'filter',
				[Email]								'email',
				[Type]								'type'
			FROM [Report] (NOLOCK)
			WHERE [VCC] = @VCC
			AND ISNULL([Id], '') + ' ' + ISNULL([Id], '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND [Type] IN (SELECT [Product] FROM VCCLicense (NOLOCK)  WHERE [VCC] = @VCC AND [Status] = 1)
			OR [Type] = 'SHARED'
			/*
			AND (
				[Type] IN (SELECT [Product] FROM VCCLicense (NOLOCK)  WHERE [VCC] = @VCC AND [Status] = 1)
					OR [Type] = 'SHARED'
			)
			*/
		END
	END
	ELSE IF @Role = 'business_administrator'
	BEGIN
		SELECT [ID]									'id',
				[Description] 						'description',
				[Procedure] 						'procedure',
				[StartDate]							'startDate',
				[EndDate]							'endDate',
				[Frecuency]							'frecuency',
				[HourSpace]							'hourSpace',
				[ExactTime]							'exactTime',
				[Weekday]							'weekday',
				[DayMonth]							'dayMonth',
				[Summary]							'summary',
				[Filter]							'filter',
				[Email]								'email',
				[Type]								'type'
			FROM [Report] (NOLOCK)
			WHERE [VCC] = @VCC
			AND ISNULL([Id], '') + ' ' + ISNULL([Id], '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND [Type] IN ('BUSINESS') 
	END
	ELSE IF @Role = 'provisioning'
	BEGIN 
		-- REPORTES PARA LA VISTA DEL PROVISIONING
		SELECT [ID]									'id',
				[Description] 						'description',
				[Procedure] 						'procedure',
				[StartDate]							'startDate',
				[EndDate]							'endDate',
				[Frecuency]							'frecuency',
				[HourSpace]							'hourSpace',
				[ExactTime]							'exactTime',
				[Weekday]							'weekday',
				[DayMonth]							'dayMonth',
				[Summary]							'summary',
				[Filter]							'filter',
				[Email]								'email',
				[Type]								'type'
			FROM [Report] (NOLOCK)
			WHERE [VCC] = @VCC
			AND ISNULL([Id], '') + ' ' + ISNULL([Id], '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
			AND [Type] IN ('BILLING') 
	END
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetReportsTask]
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]								'instance',
		[ID]								'id',
		[Description] 						'description',
		[Procedure] 						'procedure',
		[StartDate]							'startDate',
		[EndDate]							'endDate',
		[Frecuency]							'frecuency',
		[HourSpace]							'hourSpace',
		[ExactTime]							'exactTime',
		[Weekday]							'weekday',
		[DayMonth]							'dayMonth',
		[Summary]							'summary',
		[Filter]							'filter',
		[Email]								'email',
		[LastModifiedByUserId]				'lastModifiedByUserId'
	FROM [Report] (NOLOCK)
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetRole] 'bpo', NULL
CREATE PROCEDURE [dbo].[GetRole] (
	@VCC Varchar(50) = NULL, @RoleId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	DECLARE @CountLicense INT, @License VARCHAR(50)
	

	SET @CountLicense = (SELECT COUNT(*) FROM VCCLicense (NOLOCK) WHERE [VCC] =@VCC AND [Status] = 1 )	

	SELECT
		[ID]							'id',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId',
		[SystemRole]					'systemRole'
	FROM [Role] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [ID] = CASE WHEN ISNULL(@RoleId, '') = '' THEN [ID] ELSE @RoleId END

	IF(@CountLicense>1)
	BEGIN
		EXEC [GetRolePermission] @VCC, @RoleId
		EXEC [GetRoleRoutePermission] @VCC, @RoleId
		EXEC [GetRoleSinglePermission] @VCC, @RoleId
		EXEC [dbo].[GetRoleCategories] @VCC, @RoleId
	END
	IF(@CountLicense=1)
	BEGIN
		SET @License = (SELECT Product FROM VCCLicense (NOLOCK) WHERE [VCC] =@VCC AND [Status] = 1)
		EXEC [GetRolePermission] @VCC, @RoleId, @License 
		EXEC [GetRoleRoutePermission] @VCC, @RoleId, @License
		EXEC [GetRoleSinglePermission] @VCC, @RoleId, @License
		EXEC [dbo].[GetRoleCategories] @VCC, @RoleId
	END
	IF(@CountLicense<1)
	BEGIN
		EXEC [GetRolePermission] 'empty', @RoleId
		EXEC [GetRoleRoutePermission] 'empty', @RoleId
		EXEC [GetRoleSinglePermission] 'empty', @RoleId
		EXEC [dbo].[GetRoleCategories] 'empty', @RoleId
	END

	
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetRoleCategories] (
	@VCC VARCHAR(50), 
	@RoleId VARCHAR(50) = NULL,
	@License Varchar(50) = NULL
) AS
BEGIN
	-- ESTE SP RETORNA LAS CATEGORIAS RELACIONADAS A UN USUARIO
	
	SELECT RPG.[Category]					'categoryId'
		FROM [dbo].[RolePermission] RP (NOLOCK)
		INNER JOIN [dbo].[RolePermissionsGroup] RPG (NOLOCK)
			ON RPG.[VCC] = RP.[VCC]
				AND RPG.[GroupId] = RP.[GroupId]
		WHERE RP.[VCC] = @VCC
			AND RP.[RoleId] = CASE WHEN ISNULL(@RoleId, '') = '' THEN rp.[RoleId] ELSE @RoleId END
			AND (@License IS NULL OR RPG.[Type] IN (@License, 'SHARED'))
		GROUP BY RP.[VCC], RPG.[Category] 
		ORDER BY RPG.[Category] ASC
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetRolePermission] (
	@VCC Varchar(50) = NULL, @RoleId Varchar(50) = NULL, @License Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		rp.[RoleId]							'roleId',
		rp.[GroupId] 							'groupId'
	FROM [RolePermission] rp(NOLOCK)
	LEFT JOIN RolePermissionsGroup rpg (NOLOCK)
		ON rpg.GroupId = rp.[GroupId]
	WHERE rp.[VCC] = @VCC
		AND rp.[RoleId] = CASE WHEN ISNULL(@RoleId, '') = '' THEN rp.[RoleId] ELSE @RoleId END
		AND ( @License IS NULL OR rpg.Type in( @License,'SHARED') )
		AND rpg.[GroupId] IN (
			SELECT GroupId
				FROM [RolePermission] (NOLOCK)
				WHERE VCC = @VCC
					AND RoleId = CASE WHEN ISNULL(@RoleId, '')='' THEN RoleId ELSE @RoleId END)
	GROUP BY rp.[RoleId], rp.GroupId
	ORDER BY rp.[RoleId] ASC, rp.[GroupId] ASC
	
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetRolePermissionsGroups] (
	@VCC Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON 

	SELECT
		[Category]						    'category',
		[GroupId] 							'groupId',
		[Type]								'type'
	FROM [RolePermissionsGroup] (NOLOCK)
	WHERE [VCC] = @VCC
	AND (
		[Type] IN ((SELECT  [Product] FROM [VCCLicense] (NOLOCK) WHERE VCC =@VCC))
		OR 
		[Type] = 'SHARED'
	)
	GROUP BY [GroupId],[Category],[Type]
	ORDER BY [GroupId] ASC
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetRoleRoutePermission] (
	@VCC Varchar(50) = NULL, @RoleId Varchar(50) = NULL, @License Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT rp.[ID]							    'routeId',
			rp.[Route] 							'route',
			rp.[Level] 							'level',
			rp.[GroupId] 							'groupId'
	FROM [RoutePermission] rp (NOLOCK)
	LEFT JOIN RolePermissionsGroup rpg (NOLOCK)
		ON rpg.GroupId = rp.[GroupId]
	WHERE rp.[VCC] = @VCC
		AND ( @License IS NULL OR rpg.Type in( @License,'SHARED') )
		AND rp.[GroupId] IN (
			SELECT [GroupId]
			FROM [RolePermission] (NOLOCK)
			WHERE [VCC] = @VCC		
			AND [RoleId] = CASE WHEN ISNULL(@RoleId, '') = '' THEN [RoleId] ELSE @RoleId END
		
		)
	GROUP BY rp.[ID], rp.[Route],rp.[Level],rp.GroupId
	ORDER BY rp.[GroupId], rp.[ID] ASC
	

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetRoles] (
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	DECLARE @License VARCHAR(50) = NULL
	IF((SELECT COUNT(*) FROM VCCLicense WHERE VCC =@VCC) = 1)
	BEGIN
	SET @License = (SELECT Product FROM VCCLicense WHERE VCC =@VCC)
	END
	

	SELECT
		[ID]							'id',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId',
		[SystemRole]					'systemRole',
		[Type]							'type'
	FROM [Role] (NOLOCK)
	WHERE [VCC] = @VCC
	AND ( @License IS NULL OR [Type] in( @License,'SHARED') or [Type] is null )
	AND ISNULL([ID], '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
	AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetRoleSinglePermission] (
	@VCC Varchar(50) = NULL, @RoleId Varchar(50) = NULL, @License Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		sp.[ID]							    'singleId',
		sp.[GroupId] 							'groupId',
		sp.[Type] 								'type',
		sp.[Element] 							'element'
	FROM [SinglePermission] sp(NOLOCK)
	LEFT JOIN RolePermissionsGroup rpg on rpg.GroupId = sp.[GroupId]
	WHERE sp.[VCC] = @VCC
	AND ( @License IS NULL OR rpg.Type = @License )
	OR rpg.[Type] = 'SHARED'
	AND sp.[GroupId] IN (
		SELECT [GroupId]
		FROM [RolePermission] (NOLOCK)
		WHERE [VCC] = @VCC
		AND [RoleId] = @RoleId
	)
	GROUP BY sp.[ID], sp.[GroupId] ,sp.[Type], sp.[Element]
	ORDER BY sp.[ID] ASC
	

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetScores] (
	@Instance Varchar(50) = NULL,
	@IdAbility Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- CONSULTAMOS LA DATA DE LOS SCORE
	SELECT [ID]										'id',
			[IdAbility]								'idAbility',
			[Description]							'description',
			[Value]									'value'
		FROM [dbo].[Score] (NOLOCK)
		WHERE [Instance] = @Instance
			AND [IdAbility] = @IdAbility
		ORDER BY [Value] DESC

	-- CONSULTAMOS LAS TIPIFICACIONES DE LOS SCORES
	SELECT CAST(ST.[ScoreId] AS INT)				'scoreId',
			ST.[ID]									'id',
			ST.[Description]						'description',
			ST.[Detail]								'detail',
			ST.[Order]								'order'
		FROM [dbo].[ScoreTipification] ST (NOLOCK)
		INNER JOIN [dbo].[Score] S (NOLOCK)
			ON S.[Instance] = ST.[VCC]
				AND S.[ID] = ST.[ScoreId]
		WHERE S.IdAbility = @IdAbility
		ORDER BY ST.[Order] ASC

	-- CONSULTAMOS LA QUERY DE LAS TIPIFICACIONES DE LOS SCORES
	SELECT CAST(STQ.[ScoreId] AS INT)				'scoreId',
			STQ.[TipificationId]					'tipificationId',
			STQ.[QueryId]							'queryId',
			STQ.[QueryContent]						'queryContent',
			STQ.[AcceptancePercentage]				'acceptancePercentage',
			(STQ.[AcceptancePercentage] * 100)		'acceptanceInteger' 
		FROM [dbo].[ScoreTipificationQuery] STQ (NOLOCK)
		INNER JOIN [dbo].[Score] S (NOLOCK)
			ON S.[Instance] = STQ.[VCC]
				AND S.[ID] = STQ.[ScoreId]
		WHERE S.IdAbility = @IdAbility

	-- CONSULTAMOS LOS ITEMS DE TIPIFICACIONES DE LOS SCORES
	SELECT CAST(STI.[ScoreId] AS INT)				'scoreId',
			STI.[TipificationId]					'tipificationId',
			STI.[ID]								'id',
			STI.[SecondLevelItemId]					'secondLevelItemId',
			STI.[Description]						'description',
			STI.[Order]								'order' 
		FROM [dbo].[ScoreTipificationItem] STI (NOLOCK)
		INNER JOIN [dbo].[Score] S (NOLOCK)
			ON S.[Instance] = STI.[VCC]
				AND S.[ID] = STI.[ScoreId]
		WHERE S.IdAbility = @IdAbility
		ORDER BY STI.[Order] ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetSentimentAnalysisResult] (
	@VCC varchar(50),
	@InteractionId varchar(250),
	@InteractionSection numeric(5,0),
	@InteractionPart numeric(5,0),
	@EndDate varchar(50)
   
	)
AS
BEGIN
	SET NOCOUNT ON

	
	DROP TABLE IF EXISTS #SentimentResultDefault
	CREATE TABLE #SentimentResultDefault(
	analysisType VARCHAR(50),
	intentId VARCHAR(50),
	score NUMERIC(10,5),
	[status] INT
	
	)

	 IF EXISTS(SELECT 1 FROM [dbo].[SentimentAnalysisResult] SAR WHERE SAR.[VCC] = @VCC
				AND SAR.[InteractionId] = @InteractionId
				AND SAR.[InteractionSection] = @InteractionSection
				AND SAR.[InteractionPart] = @InteractionPart
				AND ISNULL(CONVERT(DATE, SAR.[EndDate], 105), '') = ISNULL(@EndDate, '') )
	BEGIN
	INSERT INTO  #SentimentResultDefault
		SELECT DISTINCT 'client', intentid, 0, 0 from intent where vcc = @VCC and [name] like 'sentiment-%'
		UNION
		SELECT DISTINCT 'agent', intentid, 0, 0 from intent where vcc = @VCC and [name] like 'sentiment-%'
		UNION
		SELECT DISTINCT 'general', intentid, 0, 0 from intent where vcc = @VCC and [name] like 'sentiment-%'

	END

	INSERT INTO #SentimentResultDefault
		SELECT [AnalysisType]					'analysisType',
				[SentimentId]					'intentId',
				[Score]							'averageResult',			
				[MaxScore]
			FROM [dbo].[SentimentAnalysisResult] SAR (NOLOCK)
	
			WHERE SAR.[VCC] = @VCC
					AND SAR.[InteractionId] = @InteractionId
					AND SAR.[InteractionSection] = @InteractionSection
					AND SAR.[InteractionPart] = @InteractionPart
					AND ISNULL(CONVERT(DATE, SAR.[EndDate], 105), '') = ISNULL(@EndDate, '')
	
	DROP TABLE IF EXISTS #SentimentResultDefaultMax
	
	SELECT analysisType,intentId,MAX(score) as 'score',MAX([status]) 'status'
		INTO  #SentimentResultDefaultMax
		FROM  #SentimentResultDefault
		GROUP BY analysisType,intentId
	
   
				
	SELECT [AnalysisType]				'analysisType',
		   [intentId]					'intentId',
		   [Score]						'averageResult',
		   CASE WHEN [status] = 0 THEN	'normal' 
				ELSE 'max' 
				END						'status'
			
		FROM #SentimentResultDefaultMax
	
	

		
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE[dbo].[GetSoundLikesFromDictionaryWords] 
	@VCC					VARCHAR(50) = NULL,
	@DictionaryWordId		VARCHAR(50) = NULL,
	@SoundLikes             VARCHAR(MAX) = NULL
AS
BEGIN
	SET NOCOUNT ON;
	SELECT [SoundLike] 'soundLike',
		   [Word] 'word'
		FROM [dbo].[DictionaryWordSoundLike] SL (NOLOCK)
		INNER JOIN [dbo].[DictionaryWord] DW (NOLOCK)
			ON SL.DictionaryWordId = DW.ID
		WHERE SL.[VCC] = @VCC 
			AND [DictionaryWordId] != @DictionaryWordId
			AND [SoundLike] in (SELECT VALUE FROM STRING_SPLIT(@SoundLikes, ','))
   
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetStudies]
(
	@VCC Varchar(50) = NULL,
	@SearchCriteria Varchar(50) = NULL,
	@SearchExpression Varchar(50) = NULL,
	@StartDate DATETIME, 
	@EndDate DATETIME,
	@Type VARCHAR(50)
)
AS
BEGIN
	DECLARE @SQL NVARCHAR(MAX)
	SET @SQL = '
	SET NOCOUNT ON

	SELECT
		[VCC]							''vcc'',
		[ProcessId]						''processId'',
		[TemplateId]					''templateId'',
		[Name]							''name'',
		[Description]					''description'',
		[Type]							''type'',
		[Status]						''status'',
		[Error]							''error'',
		[Percentage]					''percentage'',
		[StatusDescription]				''statusDescription'',
		[LastQueryId]					''lastQueryId'',
		[ErrorMessage]					''errorMessage'',
		[BatchStatus]					''batchStatus'',
		[CreatedDate]					''createdDate'',
		[CreatedByUserId]				''createdByUserId'',
		[LastModifiedDate]				''lastModifiedDate'',
		[LastModifiedByUserId]			''lastModifiedByUserId''
	FROM [dbo].[Study] S (NOLOCK)
	WHERE [VCC] = ''' + @VCC + '''
		AND [Type] = ''' + @Type + '''
		AND ISNULL([Name], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''name'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
		AND ISNULL([Description], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''description'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END
		AND ISNULL([Status], '''') LIKE CASE WHEN ''' + @SearchCriteria + ''' = ''status'' THEN ''%' + @SearchExpression + '%'' ELSE ''%'' END'

	IF(@StartDate IS NOT NULL AND @EndDate IS NOT NULL)
	BEGIN
		SET @SQL += '
			AND [CreatedDate] BETWEEN ''' + CONVERT(VARCHAR(10), @StartDate, 120) + ''' AND ''' + CONVERT(VARCHAR(10), @EndDate, 120) + '''
		'
	END
	SET @SQL += '
	ORDER BY [Status] desc, [CreatedDate] desc 

	SET NOCOUNT OFF
	'
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetStudiesInProcess]
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]				'vcc',
		[ProcessId]			'processId',
		TemplateId			'templateId',
		[studyId]			'studyId',
		SUM(Total)			'total'
	FROM (SELECT S.[VCC]				'vcc',
		S.[ProcessId]					'processId',
		S.TemplateId					'templateId',
		S.[Name]						'studyId',
		COUNT(DISTINCT TTI.[ID])		'total'
	FROM
		[dbo].[Study] S WITH (NOLOCK)
		INNER JOIN [dbo].[TemplateThemeItem] TTI WITH (NOLOCK) ON S.[TemplateId] = TTI.[TemplateId]
		INNER JOIN [dbo].[GenericQuery] GQ WITH (NOLOCK) ON TTI.[ID] = GQ.[Name]
	WHERE S.[Status] < 1
	GROUP BY
		S.[VCC],
		S.[ProcessId],
		S.[Name],
		S.TemplateId
	UNION ALL
	SELECT A.[VCC]			'vcc',
		A.[ProcessId]		'processId',
		A.TemplateId		'templateId',
		A.[studyId]			'studyId',
		COUNT(1)			'total'
	FROM (SELECT DISTINCT
				A.VCC				'vcc'
				,A.ProcessId		'processId'
				,A.[Name]			'studyId'
				,D.AudioId			'id'
				,D.Section			'section'
				,D.Part				'part'
				,B.TemplateId		'templateId'
				,C.nluId			'nluId'
				,C.appId			'appId'
				,E.[FileName]		'fileName'
			FROM [dbo].[Study] A (NOLOCK)
			INNER JOIN [dbo].[TemplateThemeItem] B WITH (NOLOCK) ON A.VCC = B.VCC AND A.[TemplateId] = B.[TemplateId]
			INNER JOIN (SELECT DISTINCT
					A.VCC 'vcc'
					,A.[Name] + '|'+ B.[Name] 'cognitiveId'
					,A.[Name] 'nluId'
					,A.AppId 'appId'
				FROM NLU A (NOLOCK)
				INNER JOIN Intent B (NOLOCK) ON A.VCC = B.VCC) C ON A.VCC = C.VCC AND B.ID = C.cognitiveId
			INNER JOIN [dbo].[ProcessAudio] D WITH (NOLOCK) ON A.VCC = D.VCC AND A.ProcessId = D.ProcessId
			INNER JOIN [dbo].[Audio] E WITH (NOLOCK) ON A.VCC = E.VCC AND D.AudioId = E.ID AND D.Section = E.Section AND D.Part = E.Part
			WHERE A.[Status] < 1) A
		GROUP BY
		A.[VCC],
		A.[ProcessId],
		A.TemplateId,
		A.[studyId]) A
		GROUP BY
		A.[VCC],
		A.[ProcessId],
		A.TemplateId,
		A.[studyId]

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetStudy] 'default',''
CREATE PROCEDURE [dbo].[GetStudy] (
	@VCC Varchar(50) = NULL,		@StudyId Varchar(200) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ProcessId]						'processId',
		[TemplateId]					'templateId',
		[Name]						    'name',
		[Description]					'description',
		[Type]							'type',
		[Status]						'status',
		[LastQueryId]					'lastQueryId',
		[ErrorMessage]					'errorMessage',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Study] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [Name] = @StudyId
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetCategoryByName] 'default','BAnca'
CREATE PROCEDURE [dbo].[GetStudyByName] (
	@VCC Varchar(50) = NULL, @Name Varchar(200) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[processId]						'processId',
		[templateId]					'templateId',
		[Name]						    'name',
		[Description]					'description',
		[Type]							'type',
		[Status]						'status',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Study] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [Name] = @Name
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetStudyGenericQueryTask]
(
	@VCC				VARCHAR(50),
	@ProcessId			VARCHAR(50),
	@StudyId			VARCHAR(200)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT DISTINCT
		S.[VCC]				'vcc',
		S.[ProcessId]		'processId',
		TTI.[ID]			'queryId',
		S.[Name]			'studyId'
	FROM [dbo].[Study] S (NOLOCK)
	INNER JOIN [dbo].[TemplateThemeItem] TTI (NOLOCK) 
		ON TTI.[VCC] = S.[VCC]
			AND S.[TemplateId] = TTI.[TemplateId]
	INNER JOIN [dbo].[GenericQuery] GQ (NOLOCK) 
		ON GQ.[VCC] = S.[VCC]
			AND TTI.[ID] = GQ.[Name]
	WHERE
		S.[VCC] = @VCC
		AND S.[ProcessId] = @ProcessId
		AND S.[Name] = @StudyId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetStudyModelInteractionTask](
	@VCC VARCHAR(50) = NULL,
	@ProcessId VARCHAR(50) = NULL,
	@StudyId VARCHAR(200) = NULL,
	@AnalysisType VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	-- ESTE SP TIENE POR FINALIDAD RETORNAR LA LISTA DE INTERACCIONES CRUZADA CON LOS MODELOS QUE SE VAN A EVALUAR

	-- DECLARAMOS UNA PALABRA PARA ALMACENAR EL MOTOR COGNITIVO
	DECLARE @COGNITIVE_ENGINE VARCHAR(50) = NULL

	-- OBTENEMOS LAS ABILITY QUERY Y LAS INSERTAMOS A UNA TABLA TEMPORAL
	SELECT TTI.* 
		INTO #THEME_ITEM_TMP 
		FROM [dbo].[Study] S (NOLOCK) 
		INNER JOIN [dbo].[TemplateTheme] TT (NOLOCK)
			ON TT.[VCC] = S.[VCC]
				AND TT.[TemplateId] = S.[TemplateId]
		INNER JOIN [dbo].[TemplateThemeItem] TTI (NOLOCK)
			ON TTI.[VCC] = S.[VCC]
				AND TTI.[TemplateThemeId] = TT.[ID]
		WHERE S.[VCC] = @VCC
			AND S.[Name] = @StudyId
			AND TT.[AnalysisType] = @AnalysisType

	-- DECLARAMOS UNA TABLE PARA ALMACENAR LA DATA DE MODEL
	DECLARE @TBL_MODEL_DATA AS TABLE ([ModelName] VARCHAR(50), [AppId] VARCHAR(100), [IntentId] VARCHAR(50))
	
	-- AGREGAMOS LA DATA DE MODELO E INTENT SEGUN EL TIPO DE MODELO
	IF @AnalysisType = 'COGNITIVE'
	BEGIN
		INSERT @TBL_MODEL_DATA 
			SELECT N.[Name]
					,N.[AppId]
					,I.[IntentId]
				FROM #THEME_ITEM_TMP TI_TMP (NOLOCK) 
				INNER JOIN [dbo].[NLU] N (NOLOCK)
					ON N.[VCC] = TI_TMP.[VCC]
				INNER JOIN [dbo].[Intent] I (NOLOCK)
					ON I.[VCC] = TI_TMP.[VCC]
						AND I.[AppId] = N.[AppId]
				WHERE TI_TMP.[ID] = CONCAT(N.[Name], '|', I.[Name])

		-- SETTEAMOS EL MOTOR COGNITIVO
		SELECT @COGNITIVE_ENGINE = [CognitiveEngine] FROM [dbo].[VCC] (NOLOCK) WHERE [VCC] = @VCC
	END
	ELSE IF @AnalysisType = 'GENERATIVE'
	BEGIN
		INSERT @TBL_MODEL_DATA 
			SELECT G.[Name]
					,G.[AppId]
					,GI.[Id]
				FROM #THEME_ITEM_TMP TI_TMP (NOLOCK) 
				INNER JOIN [dbo].[Generative] G (NOLOCK)
					ON G.[VCC] = TI_TMP.[VCC]
				INNER JOIN [dbo].[GenerativeIntent] GI (NOLOCK)
					ON GI.[VCC] = TI_TMP.[VCC]
						AND GI.[AppId] = G.[AppId]
				WHERE TI_TMP.[ID] LIKE CONCAT(G.[Name], '|', GI.[Name])
	END

	-- CREAMOS LA TABLA TEMPORAL FINAL PARA SER CRUZADA CONTRA LAS INTERACCIONES
	SELECT DISTINCT @VCC										'vcc'
					-- ,N.[Name] + '|' + I.[Name]					'cognitiveId'
					,MD.[ModelName]								'modelId' --'nluId'
					,MD.[AppId]									'appId'
					,MD.[IntentId]								'intentId'
					,'ABILITY'									'mainTarget'
					,STUFF(
						(
							SELECT DISTINCT ',' + TI_TMP.[TemplateThemeId]
								FROM #THEME_ITEM_TMP TI_TMP (NOLOCK)
							FOR XML PATH(''), TYPE
						).value('.', 'NVARCHAR(MAX)'), 
						1, 
						1, 
						''
					)											'templateThemes'
					,CONVERT(BIT, 1) 							'needRequest'
		INTO #STUDY_MODELS 
		FROM @TBL_MODEL_DATA MD 

	-- ESTE SCRIPT RECUPERA LAS INTERACCIONES TIPO CALL QUE ENTRARAN A LA EVALUACION COGNITIVA
		SELECT DISTINCT
			A.VCC										'vcc'
			,A.ProcessId								'processId'
			,'STUDY'									'batchTypeName'
			,@AnalysisType								'analysisType'
			,E.[Campaign]								'campaign'
			,'CALL'										'interactionType'
			,A.[Name]									'studyId'
			,D.AudioId									'id'
			,D.Section									'section'
			,D.Part										'part'
			,B.TemplateId								'templateId'
			,SM.[modelId]								'modelId'
			,SM.[appId]									'appId'
			,E.[FileName]								'fileName'
			,@COGNITIVE_ENGINE							'cognitiveEngine'
			,SM.[templateThemes]						'templateThemes'
			,E.[TotalPhrases]							'totalPhrases'
		FROM [dbo].[Study] A (NOLOCK)
		INNER JOIN [dbo].[TemplateThemeItem] B (NOLOCK) 
			ON A.VCC = B.VCC 
				AND A.[TemplateId] = B.[TemplateId]
		INNER JOIN [dbo].[ProcessAudio] D (NOLOCK) 
			ON A.VCC = D.VCC 
				AND A.ProcessId = D.ProcessId
		INNER JOIN [dbo].[Audio] E (NOLOCK) 
			ON A.VCC = E.VCC 
				AND D.AudioId = E.ID 
				AND D.Section = E.Section 
				AND D.Part = E.Part
				AND E.[IsTranscribed] = 1
		INNER JOIN #STUDY_MODELS SM 
			ON SM.[vcc] = A.[VCC]
		WHERE A.[VCC] = @VCC
			AND A.[ProcessId] = @ProcessId
			AND A.[Name] = @StudyId



	-- OBTENEMOS LAS INTERACCIONES CALL
		SELECT DISTINCT
			S.[VCC]													'vcc'
			,S.[ProcessId]											'processId'
			,'STUDY'												'batchTypeName'
			,AI.[Campaign]											'campaign'
			,AI.[Channel]											'interactionType'
			,S.[Name]												'studyId'
			,AI.[ID]												'id'
			,AI.[EndDate]											'endDate'
			,AI.[Folder]											'folder'
			,AI.[DispositionCode]									'dispositionCode'
			,ISNULL(AI.[LastTakedAgent], AI.[FirstTakedAgent])		'agent'
			,AI.[Initiative]										'origin'
			,AI.[TotalMessages]										'totalMessages'
			,AI.[RawChar]											'rawChar'
			,TTI.[TemplateId]										'templateId'
			,SM.[modelId]											'modelId'
			,SM.[appId]												'appId'
			,AI.[FileName]											'fileName'
			,@COGNITIVE_ENGINE										'cognitiveEngine'
			,SM.[templateThemes]									'templateThemes'
			,AI.[TotalPhrases]										'totalPhrases'
		FROM [dbo].[Study] S (NOLOCK)
		INNER JOIN [dbo].[TemplateThemeItem] TTI (NOLOCK) 
			ON S.[VCC] = TTI.[VCC] 
				AND S.[TemplateId] = TTI.[TemplateId]
		INNER JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
			ON PAI.[VCC] = S.[VCC]
				AND PAI.[ProcessId] = S.[ProcessId]
		INNER JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
			ON AI.[VCC] = S.[VCC]
				AND AI.[Campaign] = PAI.[Campaign]
				AND AI.[Channel] = PAI.[InteractionType]
				AND AI.[ID] = PAI.[InteractionId]
				AND AI.[Folder] = PAI.[FormattedEndDate]
		INNER JOIN #STUDY_MODELS SM 
			ON SM.[vcc] = S.[VCC]
		WHERE S.[VCC] = @VCC
			AND S.[ProcessId] = @ProcessId
			AND S.[Name] = @StudyId
			AND AI.[IsTranscribed] = 1

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetStudyQuery]
(
	@VCC				VARCHAR(50),
	@ProcessId			VARCHAR(50),
	@StudyId			VARCHAR(200)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT DISTINCT
		S.[VCC]				'vcc',
		S.[ProcessId]		'processId',
		TTI.[ID]			'queryId',
		S.[Name]			'studyId'
	FROM
		[dbo].[Study] S WITH (NOLOCK)
		INNER JOIN [dbo].[TemplateThemeItem] TTI WITH (NOLOCK) ON S.[TemplateId] = TTI.[TemplateId]
		INNER JOIN [dbo].[GenericQuery] GQ WITH (NOLOCK) ON TTI.[ID] = GQ.[Name]
	WHERE
		S.[VCC] = @VCC
		AND S.[ProcessId] = @ProcessId
		AND S.[Name] = @StudyId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[GetStudyReport] '2019-03-08', '2019-03-20', 'callyou_colombia,Quality_test_custom'
CREATE PROCEDURE [dbo].[GetStudyReport]
(
	@StartDate VARCHAR(10),
	@EndDate VARCHAR(10),
	@Processes VARCHAR(800)
)
AS
BEGIN
	DECLARE @SQL NVARCHAR(MAX)
	SET @SQL = '
	SET NOCOUNT ON

	SELECT
		A.ProcessId AS processId,
		A.[TemplateId] AS templateId,
		A.[Name] AS studyId,
		B.[Category1] AS theme,
		B.[Category2] AS subTheme1,
		B.[Description] AS subTheme2,
		C.[TranscriptionId] AS interactionId,
		D.[Section] AS section,
		D.[Part] AS part,
		D.[Agent] AS agent,
		A.[CreatedDate] AS createdDate
	FROM [dbo].[Study] A (NOLOCK)
	INNER JOIN [dbo].[StudyResultSummary] B (NOLOCK) ON A.[Name] = B.[StudyId]
	INNER JOIN [dbo].[StudyResultSummaryDetail] C (NOLOCK) ON A.[Name] = C.[StudyId]
	INNER JOIN [dbo].[Audio] D (NOLOCK) ON C.[TranscriptionId] = D.[ID]
	WHERE
		A.[CreatedDate] BETWEEN ''' + @StartDate + ' 00:00:00.000'' AND ''' + @EndDate + ' 23:59:59.999'' AND
		A.[ProcessId] IN(SELECT value FROM [dbo].[SplitString](''' + @Processes + ''', '',''))

	SET NOCOUNT OFF
	'
	EXEC sp_executesql @SQL
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetStudyResultSummary]
(
	@VCC					VARCHAR(50),
	@StudyId				VARCHAR(200),
	@NewStructure			VARCHAR(1)
)
AS
BEGIN
	SET NOCOUNT ON

	BEGIN
		SELECT
			B.[Name]						'processName',
			A.[Name]						'studyName',
			A.[Description]					'studyDescription',
			A.[TemplateId]					'templateId'
		FROM [dbo].[Study] A (NOLOCK)
		INNER JOIN Process B ON A.ProcessId = B.[Name]
		WHERE A.[VCC] = @VCC 
			AND A.[Name] = @StudyId
	END
	
	BEGIN
		SELECT SRS.[VCC]							'vcc',
				SRS.[StudyId]						'studyId',
				SRS.[TemplateId]					'templateId',
				SRS.[TemplateThemeId]				'templateThemeId',
				SRS.[ID]							'id',
				SRS.[Description]					'description',
				SRS.[Category1]						'category1',
				SRS.[Category2]						'category2',
				SRS.[Total]							'total',
				SRS.[Duration]						'duration',
				SRS.[Silence]						'silence',
				SRS.[Confidence]					'confidence',
				SRS.[CreatedDate]					'createdDate',
				SRS.[CreatedByUserId]				'createdByUserId',
				SRS.[LastModifiedDate]				'lastModifiedDate',
				SRS.[LastModifiedByUserId]			'lastModifiedByUserId'
			FROM [dbo].[StudyResultSummary] SRS (NOLOCK)
			WHERE SRS.[VCC] = @VCC 
				AND SRS.[StudyId] = @StudyId
	END

	IF(@NewStructure IS NULL)
	BEGIN

		-- CREAMOS UNA TEMPORAL PARA AUDIOS Y ASYNCINTERACTIONS
		CREATE TABLE #INNER_INTERACTION (
			[VCC] VARCHAR(50),
			[Channel] VARCHAR(50),
			[ID] VARCHAR(250),
			[Section] NUMERIC(5,0),
			[Part] NUMERIC(5,0), 
			[Campaign] VARCHAR(100),
			[Agent] VARCHAR(50),
			[IsTranscribed] BIT,
			[IsCancel] BIT,
			[Folder] VARCHAR(100),
			[TranscriptionId] VARCHAR(250),
			
		)

		-- INSERTAMOS LOS Audios 
		INSERT INTO #INNER_INTERACTION
				SELECT AU.[VCC],
					AU.[Channel], 
					AU.[ID], 
					AU.[Section], 
					AU.[Part], 
					AU.[Campaign], 
					AU.[agent], 
					AU.[IsTranscribed], 
					AU.[IsCancel],
					null as 'Folder',
					CONCAT(AU.[ID], '_', AU.[Section], '_', AU.[Part]) AS 'TranscriptionId'
				
				FROM [dbo].[Audio] AU (NOLOCK)
				INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK)
					ON AU.[VCC] = PA.[VCC]
					AND AU.[ID] = PA.[AudioId]
				INNER JOIN [dbo].[Study] ST (NOLOCK)
					ON PA.[ProcessId] = ST.[processId]
				WHERE 
				AU.[VCC] IN ('system', @VCC) AND
				ST.[Name] = @StudyId

		-- INSERTAMOS LOS AsyncInteraction 
		INSERT INTO #INNER_INTERACTION
				SELECT 
				AI.[VCC], 
				AI.[Channel], 
				AI.[ID], 
				null as 'Section',
				null as 'Part',
				AI.[Campaign], 
				AI.[FirstTakedAgent] AS 'Agent', 
				AI.[IsTranscribed], 
				AI.[IsCancel],
				AI.[Folder],
				CONCAT(AI.[Channel], '_', AI.[Campaign], '_', AI.[Folder], '_', REPLACE(AI.[ID], ':', '_')) AS 'TranscriptionId'
			
			FROM [dbo].[AsyncInteraction] AI (NOLOCK)
			INNER JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
				ON AI.[VCC] = PAI.[VCC]
				AND AI.[ID] = PAI.[InteractionId]
			INNER JOIN [dbo].[Study] ST (NOLOCK)
				ON ST.[processId] = PAI.[ProcessId]
			WHERE 
			AI.[VCC] IN ('system', @VCC) AND
			ST.[Name] = @StudyId		

		SELECT
			A.[TemplateThemeId]									'templateThemeId',
			A.[TemplateThemeItemId]								'templateThemeItemId',
			--ISNULL(D.[Query], A.[TemplateThemeItemId])			'query', -- Si es NULL se mostrara valor de TemplateThemeItem campo ID.
			ISNULL(ISNULL(D.[Query], E.[EntityName]),'')			'query', -- Si es NULL se mostrara valor de TemplateThemeItem campo ID.
			C.[Channel]											'interactionType',
			S.[ProcessId]										'processId',
			C.[ID]								                'transcriptionId',
			C.[Section]											'section',
			C.[Part]											'part',
			C.[Folder]											'folder',
			C.[Campaign]										'campaign',
			C.[Agent]		                                    'agent',
			C.[IsTranscribed]		                            'isTranscribed',
			C.[IsCancel]				                        'isCancel'
		FROM [dbo].[StudyResultSummaryDetail] A (NOLOCK)
		INNER JOIN [dbo].[Study] S (NOLOCK) 
			ON S.[VCC] = A.[VCC] 
				AND S.[Name] = A.[StudyId]
		INNER JOIN [dbo].[HistoricTemplateThemeItem] B (NOLOCK) 
			ON S.[TemplateId] = B.[TemplateId] 
				AND A.[TemplateThemeId] = B.[TemplateThemeId] 
				AND A.[TemplateThemeItemId] = B.[ID]
				AND B.[StudyId] = S.[Name]
			
		INNER JOIN [dbo].[#INNER_INTERACTION] C (NOLOCK)  -- (!)SE CAMBIO EL INNER POR LEFT PARA CONSIDER LAS TRANSCRIPCIONES DE CANALES ASINCRINOS
			ON A.[VCC] = C.[VCC]
				AND A.[TranscriptionId] = C.[TranscriptionId]
		LEFT JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
			ON S.[VCC] = PA.[VCC] 
				AND S.[ProcessId] = PA.[ProcessId] 
				AND A.[TranscriptionId] = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])

		--LEFT JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
			-- ON A.[TranscriptionId] = CONCAT(AI.[Channel], '_', AI.[Campaign], '_', AI.[Folder], '_', REPLACE(AI.[ID], ':', '_'))
		LEFT JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
			ON PAI.[VCC] = S.[VCC]
				AND PAI.[ProcessId] = S.[ProcessId]
				AND CONCAT(PAI.[InteractionType], '_', PAI.[Campaign], '_', PAI.[FormattedEndDate], '_', REPLACE(PAI.[InteractionId], ':', '_')) = A.[TranscriptionId]

		LEFT JOIN [dbo].[GenericQuery] D (NOLOCK) 
			ON B.[VCC] = D.[VCC] 
				AND  B.[ID] = D.[Name] -- Se agrega LEFT JOIN para poder mostrar.

		LEFT JOIN [dbo].[TemplateThemeItemEntity] E 
			ON A.TemplateThemeId = E.TemplateThemeId
		WHERE
			B.[VCC] IN ('system', @VCC) AND
			A.[StudyId] = @StudyId

		DROP TABLE #INNER_INTERACTION		
	END
	ELSE
	BEGIN
		-- ESTE BLOQUE NUNCA SE USA, EL UNICO LLAMADO DE LA FUNCION INSERTA DIRECTAMENTE NULL PARA LA VARIABLE @NewStructure
		SELECT
			A.[TemplateThemeId]									'templateThemeId',
			A.[TemplateThemeItemId]								'templateThemeItemId',
			D.[Query]											'query',
			ISNULL(C.[Channel], AI.[Channel])					'interactionType',
			CASE WHEN C.[Channel] IS NOT NULL 
					AND C.[Channel] = 'CALL' 
				THEN A.[TranscriptionId] 
				ELSE AI.[ID] END								'transcriptionId',
			C.[Section]											'section',
			C.[Part]											'part',
			AI.[Folder]											'folder',
			AI.[Campaign]										'campaign',
			--ISNULL(C.[Agent], '')								'agent',
			CASE WHEN C.[Channel] IS NOT NULL 
					AND C.[Channel] = 'CALL' 
				THEN ISNULL(C.[Agent], '') 
				ELSE ISNULL(AI.[FirstTakedAgent], '') END		'agent',
			CASE WHEN C.[Channel] IS NOT NULL 
					AND C.[Channel] = 'CALL' 
				THEN ISNULL(C.[IsTranscribed], '') 
				ELSE ISNULL(AI.[IsTranscribed], '') END			'isTranscribed',
			CASE WHEN C.[Channel] IS NOT NULL
					AND C.[Channel] = 'CALL' 
				THEN ISNULL(C.[IsCancel], '') 
				ELSE ISNULL(AI.[IsCancel], '') END				'isCancel'
			
		FROM 
			[dbo].[StudyResultSummaryDetail] A (NOLOCK)
			INNER JOIN [dbo].[Study] S (NOLOCK) 
				ON S.[VCC] = A.[VCC] 
					AND S.[Name] = A.[StudyId]
			INNER JOIN [dbo].[HistoricTemplateThemeItem] B (NOLOCK) 
				ON S.[TemplateId] = B.[TemplateId] 
					AND A.[TemplateThemeId] = B.[TemplateThemeId] 
					AND A.[TemplateThemeItemId] = B.[ID]
					AND B.[StudyId] = S.[Name]
			
			LEFT JOIN [dbo].[Audio] C (NOLOCK) 
				ON A.[VCC] = C.[VCC]
					AND A.[TranscriptionId] = CONCAT(C.[ID], '_', C.[Section], '_', C.[Part])
			INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
				ON S.[VCC] = PA.[VCC] 
					AND S.[ProcessId] = PA.[ProcessId] 
					AND A.[TranscriptionId] = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])
			
			LEFT JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
				ON A.[TranscriptionId] = CONCAT(AI.[Channel], '_', AI.[Campaign], '_', AI.[Folder], '_', REPLACE(AI.[ID], ':', '_'))
			LEFT JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
				ON PAI.[VCC] = S.[VCC]
					AND PAI.[ProcessId] = S.[ProcessId]
					AND CONCAT(PAI.[InteractionType], '_', PAI.[Campaign], '_', PAI.[FormattedEndDate], '_', REPLACE(PAI.[InteractionId], ':', '_')) = A.[TranscriptionId]

			INNER JOIN [dbo].[GenericQuery] D (NOLOCK) 
				ON B.[VCC] = D.[VCC] 
					AND  B.[ID] = D.[Name]
		WHERE
			B.[VCC] IN ('system', @VCC) AND
			A.[StudyId] = @StudyId
	END
	
	SET NOCOUNT OFF


END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetSubGroup] (
	@Instance Varchar(50) = NULL, @ID Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]							'id',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId',
		[Enabled]						'enabled'
	FROM [SubGroup] (NOLOCK)
	WHERE [Instance] = @Instance
	AND [ID] = @ID

	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetSubGroupDescription] (
	@Instance Varchar(50) = NULL, @Description Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]							'id',
		[Description]					'description',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [SubGroup] (NOLOCK)
	WHERE [Instance] = @Instance
	AND [Description] = @Description

	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetSubGroups] (
	@Instance Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL, @Disabled bit = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		s.ID							'id',
		s.Description					'description',
		s.CreatedDate					'createdDate',
		s.CreatedByUserId				'createdByUserId',
		s.LastModifiedDate				'lastModifiedDate',
		s.LastModifiedByUserId			'lastModifiedByUserId',
		s.[Enabled]						'enabled'
	FROM SubGroup s (NOLOCK)
	WHERE s.Instance = @Instance
		AND ISNULL(s.ID, '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(s.Description, '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' COLLATE Latin1_General_CI_AI ELSE '%' END
		AND (ISNULL(@Disabled, 0) = 1 OR (ISNULL(@Disabled, 0) = 0 AND ISNULL(@Disabled, 0) <> [Enabled]))
	ORDER BY s.Description ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetSubGroupsUsage] 
	@Instance VARCHAR(50) = NULL,
	@SubGroupIDs VARCHAR(MAX) = NULL
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

    DECLARE @_IDS TABLE(
	 ID VARCHAR(50)
	)

    INSERT INTO @_IDS SELECT [Value] FROM [dbo].[Split](@SubGroupIDs,',')

	SELECT [IdSubGroup] 'idSubGroup'
        FROM [dbo].[AbilityEvaluationFormat] 
		WHERE Instance = @Instance 
			AND IdSubGroup  IN (SELECT [ID] FROM @_IDS)
    
           
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetSubscriptionKeys] 'allegro'
CREATE PROCEDURE [dbo].[GetSubscriptionKey] (
	@VCC Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT TOP 1
		[SubscriptionKey]				'subscriptionKey',
		[SubscriptionRegion]			'subscriptionRegion',
		[IsDemo]						'isDemo',
		[SubscriptionName]				'subscriptionName'
	FROM [AzureSubscriptionKey] (NOLOCK)
	WHERE [VCC] = @VCC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetSubscriptionKeys] 'allegro'
CREATE PROCEDURE [dbo].[GetSubscriptionKeys] (
	@VCC Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[SubscriptionKey]				'subscriptionKey',
		[SubscriptionRegion]			'subscriptionRegion',
		[IsDemo]						'isDemo',
		[SubscriptionName]				'subscriptionName'
	FROM [AzureSubscriptionKey] (NOLOCK)
	WHERE [VCC] = @VCC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


create PROCEDURE [dbo].[GetSupervisors] (
	@VCC VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
 	SELECT U.[VCC]															'vcc',
			U.[Id]															'userId',
			CONCAT(U.[Firstname], ' ', ISNULL(U.[Lastname], ''))			'userName',
			NULL															'campaignId',
			NULL															'workGroupId'
		FROM [dbo].[User] U (NOLOCK)
		WHERE U.[VCC] = @VCC
			AND (
				U.[IsLocalSupervisor] = 1
				OR
				U.[IsExternalSupervisor] = 1
			)
 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetSystemWords] 'default','NeuroLinguisticProgrammingId','','E4CC3881-9B1E-420B-977D-7CB473ED9C14', 'es'
--exec [GetSystemWords] 'system','NeuroLinguisticProgrammingId','','init', 'es'
CREATE PROCEDURE [dbo].[GetSystemWords] (
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL,
	@ProcessId Varchar(50) = NULL, @Language Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[NeuroLinguisticProgrammingId]	'neuroLinguisticProgrammingId',
		[Name]							'name',
		[ProcessId]						'processId',
		[Language]						'language',
		[Type]							'type',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Word] (NOLOCK)
	WHERE [VCC] = @VCC
	AND ISNULL([NeuroLinguisticProgrammingId], '') LIKE CASE WHEN @SearchCriteria = 'neuroLinguisticProgrammingId' THEN '%' + @SearchExpression + '%' ELSE '%' END
	AND ([ProcessId] = @ProcessId OR [ProcessId] = 'init')
	AND [Language] = @Language

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetTables] '38.127.113.216', 'GyT', 1
CREATE PROCEDURE [dbo].[GetTables]
(
@Ip Varchar(50), @Database Varchar(50), @Type Int
)
AS
BEGIN

	DECLARE @Sql NVARCHAR(MAX);

	IF(@Type = 0)  
	BEGIN  
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[TABLE_CATALOG]						''table_catalog'',
			[TABLE_NAME]						''table_name''
		FROM [' + @Ip + ']' + '.' + @Database + '.INFORMATION_SCHEMA.TABLES WITH (NOLOCK)
		WHERE TABLE_TYPE = ''BASE TABLE'''
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE IF(@Type = 1) 
	BEGIN
		SET @Sql = 'SET NOCOUNT ON
		SELECT
			[TABLE_CATALOG]						''table_catalog'',
			[TABLE_NAME]						''table_name''
		FROM ' + @Database + '.INFORMATION_SCHEMA.TABLES (NOLOCK)
		WHERE TABLE_TYPE = ''BASE TABLE'''
		SET @Sql += '
		SET NOCOUNT OFF'
	END  
	ELSE  
	BEGIN  
		SET @Sql = 'SELECT 1 AS ''result'''  
	END

	EXEC (@Sql)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

-- [dbo].[GetTags] 'default', '{00c7211a-973a-11e8-87f3-be2929e88805}', 0, 0
CREATE PROCEDURE [dbo].[GetTags]
(
	@VCC						VARCHAR(50),
	@AudioId					VARCHAR(50),
	@Section					NUMERIC(5, 0),
	@Part						NUMERIC(5, 0)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]												'vcc',
		[AudioId]											'audioId',
		[Section]											'section',
		[Part]												'part',
		STR([CurrentTime], 18, 18)							'currentTime',
		[Tag]												'tag',
		[Comment]											'comment',
		[CreatedDate]										'createdDate',
		[CreatedByUserId]									'createdByUserId',
		[LastModifiedDate]									'lastModifiedDate',
		[LastModifiedByUserId]								'lastModifiedByUserId',
		[TimeStamp]											'timeStamp'
	FROM
		[dbo].[Tag] (NOLOCK)
	WHERE
		[VCC] = @VCC AND
		[AudioId] = @AudioId AND
		[Section] = @Section AND
		[Part] = @Part

	SET NOCOUNT OFF
END 
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetTeamPerformanceFilters] (
	@VCC VARCHAR(50) = NULL,
	@BatchType VARCHAR(50) NULL,
	@EvaluationFormatId VARCHAR(50) NULL,
	@TemplateId VARCHAR(50) NULL

)
AS
BEGIN
	SET NOCOUNT ON
	-- OBTENEMOS LOS FORMATOS DE EVALUACION PARA EL VCC 	
	IF @BatchType = 'EVALUATION_PLAN'
	BEGIN
		SELECT '' 'id','' 'description'
		UNION ALL
		SELECT DISTINCT [ID]  'id', 
						[Description] 'description' 
			FROM [dbo].[HistoricEvaluationFormat]
			WHERE [VCC] =  @VCC 

		-- OBTENEMOS LOS PLANES COMPLETADOS
		SELECT DISTINCT EP.[ID]							'id',
						EP.[Description]				'description'
			FROM [dbo].[EvaluationPlan] EP (NOLOCK)
			INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				ON  EP.[Instance] = AEP.[Instance]
					AND	EP.[ID] = AEP.IdEvaluationPlan
			
			WHERE EP.[Instance] = @VCC
				/*
				AND (
						EP.[Status] = 7 AND EP.Evaluation_Mode = 'A'
						OR
						AEP.[Completed] = 1 AND EP.Evaluation_Mode IN ('X','M')
					)
				*/
				AND AEP.[Completed] = 1
				AND EP.[IdEvaluationFormat] = @EvaluationFormatId
				
		  		

	
		-- OBTENEMOS LOS AGENTES 
		SELECT DISTINCT ER.[IdAgent]							'id',
						ER.[AgentDescription]					'description'
			FROM [dbo].[EvaluationResult] ER (NOLOCK)
			INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
				ON EP.[Instance] = ER.[Instance]
					AND ER.[IdEvaluationPlan] = ER.[IdEvaluationPlan]
			INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				ON EP.[Instance] = AEP.[Instance]
					AND EP.[ID] = AEP.[IdEvaluationPlan]
			WHERE ER.[Instance] = @VCC
				AND (
						EP.[Status] = 7 AND EP.Evaluation_Mode = 'A'
						OR
						AEP.[Completed] = 1 AND EP.Evaluation_Mode IN ('X','M')
					)
				AND ER.[IdEvaluationFormat] = @EvaluationFormatId


		-- OBTENEMOS LOS SUPERVISORES DE LOS AGENTES DE LA CONFIGURACION DE GRUPOS DE TRABAJO 
			SELECT DISTINCT WGS.[SupervisorId]							'id',
						WGS.[SupervisorId]					'description'
			FROM [dbo].[EvaluationResult] ER (NOLOCK)
			INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
				ON EP.[Instance] = ER.[Instance]
					AND ER.[IdEvaluationPlan] = ER.[IdEvaluationPlan]
			INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				ON EP.[Instance] = AEP.[Instance]
					AND EP.[ID] = AEP.[IdEvaluationPlan]
			INNER JOIN [dbo].[WorkGroupSupervisor] WGS (NOLOCK)
				ON WGS.[VCC] = ER.[Instance]
					AND WGS.[AgentId] = ER.[IdAgent] 
			WHERE ER.[Instance] = @VCC
				AND (
						EP.[Status] = 7 AND EP.Evaluation_Mode = 'A'
						OR
						AEP.[Completed] = 1 AND EP.Evaluation_Mode IN ('X','M')
					)
				AND ER.[IdEvaluationFormat] = @EvaluationFormatId
		
		END
		ELSE
		BEGIN

		-- LISTAR TEMPLATES
			
		-- OBTENEMOS LOS ESTUDIOS COMPLETADOS
		SELECT DISTINCT S.[Name]						'id',
						S.[Name]						'name'
			FROM [dbo].[StudyResultSummaryDetail] SRSD (NOLOCK)
			INNER JOIN [dbo].[Study] S (NOLOCK)
				ON S.[VCC] = SRSD.[VCC]
					AND S.[Name] = SRSD.[StudyId]
			WHERE S.[VCC] = @VCC
				AND S.[Type] = 'SPEECH'
				AND S.[Percentage] = 1
				AND S.[TemplateId] = @TemplateId

		END
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetTeamPerformanceFromEvaluationPlan] (
	@VCC					VARCHAR(50) = NULL,
	@EvaluationFormatId		VARCHAR(50) = NULL,
	@EvaluationPlanIds		VARCHAR(MAX) = NULL,
	@AgentIds				VARCHAR(MAX) = NULL,
	@AbilityIds				VARCHAR(MAX) = NULL,
	@StartDate				DATETIME,
	@EndDate				DATETIME,
	@drilldownTarget		VARCHAR(50) = NULL,
	@drilldownId			VARCHAR(50) = NULL,
	@SupervisorIds			VARCHAR(MAX) =NULL,
	@ResultBySupervisor		BIT = NULL
)
AS
BEGIN

	DROP TABLE IF EXISTS #SEARCHED_INTERACTIONS
	CREATE TABLE #SEARCHED_INTERACTIONS
	(
		AgentId				VARCHAR(100),
		EvaluationPlanId	VARCHAR(50),
		InteractionId		VARCHAR(250)
	)
	
	IF(@drilldownTarget ='ABILITY')
	BEGIN
		SET @AbilityIds = @drilldownId
	
		INSERT INTO #SEARCHED_INTERACTIONS
			SELECT DISTINCT ER.[IdAgent]															'AgentId',
							ER.[IdEvaluationPlan]													'EvaluationPlanId',
							ER.[IdInteraction]														'InteractionId' 
		
				FROM [dbo].[EvaluationResult] ER (NOLOCK)				
				INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
					ON ER.[Instance] = EP.[Instance]
						AND ER.[IdEvaluationPlan] = EP.[ID]
				INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
					ON EP.[Instance] = AEP.[Instance]
						AND EP.[ID] = AEP.[IdEvaluationPlan]
						AND (
							(
								EP.[Status] = 7 
								AND 
								EP.Evaluation_Mode = 'A'
							)
							OR
							(	
								AEP.[Completed] = 1 
								AND 
								EP.Evaluation_Mode IN ('X','M')
							)
						)
						AND ER.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
						AND ER.[IdInteraction] = AEP.[IdInteraction]
						AND ER.[IdAgent] = AEP.[IdAgent]
						AND ER.[IdAuditor] = AEP.[IdAuditor]
						AND AEP.[Completed] = 1 
						AND AEP.[StatusDescription] NOT IN('INVALID') 
				WHERE ER.[Instance] = @VCC
					AND ER.[InteractionDatefin] BETWEEN @StartDate AND @EndDate
					AND ER.[IdEvaluationFormat] = @EvaluationFormatId
					AND (
						ISNULL(LEN(@EvaluationPlanIds), 0) = 0
						OR					
						ER.[IdEvaluationPlan]  IN (SELECT Value FROM [dbo].[split](@EvaluationPlanIds, ','))
					)	
					AND (
						ISNULL(LEN(@AgentIds), 0) = 0
						OR
						ER.[IdAgent]  IN (SELECT Value FROM [dbo].[split](@AgentIds, ','))
					)
					AND (
						ISNULL(LEN(@AbilityIds), 0) = 0
						OR 					
						(
							ER.[IdAbility] IN (SELECT Value FROM [dbo].[split](@AbilityIds, ','))
							AND
							(
								/* EXCEPCION PARA HABILIDADES CUALITATIVAS Y CUANTITATIVAS, LAS CUALITATIVAS TIENEN [MaxAbilityScore] = 0 */
								/* Y NO PUEDE DISTINGUIRSE CUAL ES EL SCORE SELECCIONADO, POR ESO COMPARAMOS CON [AbilityInnerScore]      */
								(
									ER.[MaxAbilityScore] <> 0
									AND
									ER.[Qualification] = ER.[MaxAbilityScore]
								)
								OR
								(
									ER.[MaxAbilityScore] = 0
									AND
									ER.[AbilityInnerScore] = 100
								)
							)
						)
					)

	END
	ELSE
	BEGIN
		INSERT INTO #SEARCHED_INTERACTIONS
			SELECT DISTINCT ER.[IdAgent]															'AgentId',
							ER.[IdEvaluationPlan]													'EvaluationPlanId',
							ER.[IdInteraction]														'InteractionId' 
				FROM [dbo].[EvaluationResult] ER (NOLOCK)
				INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
					ON ER.[Instance] = EP.[Instance]
						AND ER.[IdEvaluationPlan] = EP.[ID]
				INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
					ON EP.[Instance] = AEP.[Instance]
						AND EP.[ID] = AEP.[IdEvaluationPlan]
						AND (
							(
								EP.[Status] = 7 
								AND 
								EP.Evaluation_Mode = 'A'
							)
							OR
							(
								AEP.[Completed] = 1 
								AND 
								EP.Evaluation_Mode IN ('X','M')
							)
						)
						AND ER.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
						AND ER.[IdInteraction] = AEP.[IdInteraction]
						AND ER.[IdAgent] = AEP.[IdAgent]
						AND ER.[IdAuditor] = AEP.[IdAuditor]
						AND AEP.[Completed] = 1 
						AND AEP.[StatusDescription] NOT IN('INVALID') 

				WHERE ER.[Instance] = @VCC
					AND ER.[InteractionDatefin] BETWEEN @StartDate AND @EndDate
					AND ER.[IdEvaluationFormat] = @EvaluationFormatId
					AND (
						ISNULL(LEN(@EvaluationPlanIds), 0) = 0
						OR					
						ER.[IdEvaluationPlan]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))
						)
					
					AND (
						ISNULL(LEN(@AgentIds), 0) = 0
						OR
						ER.[IdAgent]  IN(SELECT Value FROM [dbo].[split](@AgentIds,','))
						)
			
					AND (
						ISNULL(LEN(@AbilityIds), 0) = 0
						OR 
						(
						ER.[IdAbility] IN (SELECT Value FROM [dbo].[split](@AbilityIds, ','))
					
						)
					)

	END


   
	-- GENERAMOS EL LISTADO DE INTERACCIONES A BUSCAR

	IF (@ResultBySupervisor = 1)
	BEGIN
		DROP TABLE IF EXISTS #WORKGROUPSUPERVISORS
		SELECT DISTINCT SupervisorId,AgentId 
		INTO #WORKGROUPSUPERVISORS
		FROM dbo.[WorkGroupSupervisor]
		WHERE (
				ISNULL(LEN(@SupervisorIds), 0) = 0
				OR
				[SupervisorId]  IN(SELECT Value FROM [dbo].[split](@SupervisorIds,','))
			  )
		
	END
	
	
	--SELECT * FROM #SEARCHED_INTERACTIONS
	-- OBTENEMOS TOTAL DE INTERACCIONES POR AGENTE	
	DROP TABLE IF EXISTS #SEARCHED_INTERACTIONS_TOTALBYAGENT
	SELECT [AgentId],COUNT([InteractionId]) 'Total' 
		INTO #SEARCHED_INTERACTIONS_TOTALBYAGENT
		FROM #SEARCHED_INTERACTIONS 
		GROUP BY [AgentId]
	
	--SELECT * FROM #SEARCHED_INTERACTIONS_TOTALBYAGENT


	
	--OBTENENEMOS SUMATORIA DE NOTAS POR AGENTE
	--PARA EL CASO DE MIXTAS LOS RESULTADOS SE MULTIPLICAN SEGUN EL NUMERO DE AUDITORES ES POR ELL QUE SE DEBE TOMAR 
	
	DROP TABLE IF EXISTS #SEARCHED_INTERACTIONS_NOTAS
	SELECT ER.[IdAgent]					'AgentId',
			ER.[IdEvaluationPlan]		'EvaluationPlanId',	
			ER.[IdInteraction]			'InteractionId', 
			AVG(AEP.[Score])				'NotaTotal'
		INTO  #SEARCHED_INTERACTIONS_NOTAS
		FROM [dbo].[EvaluationResult] ER (NOLOCK)
		INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			ON ER.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
				AND ER.[IdInteraction] = AEP.[IdInteraction]
				AND ER.[IdAgent] = AEP.[IdAgent]
				AND ER.[IdAuditor] = AEP.[IdAuditor]
				AND AEP.[Completed] = 1 
				AND AEP.[StatusDescription] NOT IN('INVALID')
		INNER JOIN #SEARCHED_INTERACTIONS SI
			ON ER.[IdAgent] =SI.[AgentId]
				AND ER.[IdEvaluationPlan] = SI.[EvaluationPlanId]
				AND  ER.[IdInteraction] = SI.[InteractionId]
		WHERE ER.[Instance] = @VCC
			AND ER.[IdEvaluationFormat] = @EvaluationFormatId
			AND (
				ISNULL(LEN(@EvaluationPlanIds), 0) = 0
				OR					
			    ER.[IdEvaluationPlan]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))
			)
		GROUP BY ER.[IdAgent],
					ER.[IdEvaluationPlan],
					ER.[IdInteraction]
					
				
	--SELECT * FROM #SEARCHED_INTERACTIONS_NOTAS
	-- OBTENEMOS EL PROMEDIO DE NOTAS POR AGENTE PARA OBTENER CUARILTES
	DROP TABLE IF EXISTS #SEARCHED_INTERACTIONS_NOTAS_PROM
    SELECT [AgentId], CAST(AVG([NotaTotal]) AS DECIMAL(7,1)) 'PromNotaTotal'
	INTO #SEARCHED_INTERACTIONS_NOTAS_PROM
	FROM #SEARCHED_INTERACTIONS_NOTAS
	GROUP BY [AgentId]		
	
	--SELECT * FROM #SEARCHED_INTERACTIONS_NOTAS_PROM

	IF(@ResultBySupervisor = 1)
	BEGIN
		SELECT WS.[SupervisorId],CAST(AVG(SINP.PromNotaTotal) AS DECIMAL(7,1)) 'PromNotaTotal'
		INTO #SEARCHED_INTERACTIONS_NOTAS_PROM_SUPERVISOR
		FROM #SEARCHED_INTERACTIONS_NOTAS_PROM SINP
		INNER JOIN #WORKGROUPSUPERVISORS WS ON 
		SINP.AgentId = WS.AgentId
		GROUP BY [SupervisorId]
	END

	--SELECT * FROM #SEARCHED_INTERACTIONS_NOTAS_PROM_SUPERVISOR

	--OBTENEMOS CUARTILES
	--OBTENEMOS CUARTILES DE ACUERDO A LOS PROMEDIOS DE NOTAS POR AGENTE
	IF(@ResultBySupervisor = 1)
	BEGIN
		DROP TABLE IF EXISTS #CUARTILES
		SELECT DISTINCT CAST([PromNotaTotal] AS DECIMAL(7,1)) 'PromNotaTotal'
		INTO #CUARTILES_SUP
		FROM #SEARCHED_INTERACTIONS_NOTAS_PROM_SUPERVISOR

		DROP TABLE IF EXISTS #CUARTILES_ENUM_SUP
		SELECT ROW_NUMBER() OVER(ORDER BY [PromNotaTotal] DESC) 'lux',[PromNotaTotal] 
		INTO #CUARTILES_ENUM_SUP
		FROM #CUARTILES_SUP
    
		DECLARE @CUARTIL_EL_SUP INT
		SELECT @CUARTIL_EL_SUP = COUNT(*) FROM #CUARTILES_ENUM_SUP
	
		DROP TABLE IF EXISTS #CUARTILES_FINAL_SUP
		SELECT
		CAST(CE.[PromNotaTotal] AS DECIMAL(7,1)) 'PromNotaTotal',
		IIF (A.PromNotaTotal IS NOT NULL AND @CUARTIL_EL_SUP>=4 , 1,
			IIF (B.[PromNotaTotal] IS NOT NULL AND @CUARTIL_EL_SUP>=4, 2, 
			IIF (C.[PromNotaTotal] IS NOT NULL AND @CUARTIL_EL_SUP>=4, 3, 
			IIF (D.[PromNotaTotal] IS NOT NULL AND @CUARTIL_EL_SUP>=4, 4, 1)))
		) as quartile
		INTO #CUARTILES_FINAL_SUP
		FROM
		#CUARTILES_ENUM_SUP CE
		LEFT OUTER JOIN #CUARTILES_ENUM_SUP A on CE.PromNotaTotal = A.PromNotaTotal and A.lux > (SELECT COUNT(*)*0.00 FROM #CUARTILES_ENUM_SUP) and a.lux <= (SELECT COUNT(*)*0.25 FROM #CUARTILES_ENUM_SUP)
		LEFT OUTER JOIN #CUARTILES_ENUM_SUP B on CE.PromNotaTotal = B.PromNotaTotal and B.lux > (SELECT COUNT(*)*0.25 FROM #CUARTILES_ENUM_SUP) and b.lux <= (SELECT COUNT(*)*0.50 FROM #CUARTILES_ENUM_SUP)
		LEFT OUTER JOIN #CUARTILES_ENUM_SUP C on CE.PromNotaTotal = C.PromNotaTotal and C.lux > (SELECT COUNT(*)*0.50 FROM #CUARTILES_ENUM_SUP) and c.lux <= (SELECT COUNT(*)*0.75 FROM #CUARTILES_ENUM_SUP)
		LEFT OUTER JOIN #CUARTILES_ENUM_SUP D on CE.PromNotaTotal = D.PromNotaTotal and D.lux > (SELECT COUNT(*)*0.75 FROM #CUARTILES_ENUM_SUP)
		END
	ELSE
	BEGIN 
		DROP TABLE IF EXISTS #CUARTILES
		SELECT DISTINCT CAST([PromNotaTotal] AS DECIMAL(7,1)) 'PromNotaTotal'
		INTO #CUARTILES
		FROM #SEARCHED_INTERACTIONS_NOTAS_PROM

		DROP TABLE IF EXISTS #CUARTILES_ENUM
		SELECT ROW_NUMBER() OVER(ORDER BY [PromNotaTotal] DESC) 'lux',[PromNotaTotal] 
		INTO #CUARTILES_ENUM
		FROM #CUARTILES
    
		DECLARE @CUARTIL_EL INT
		SELECT @CUARTIL_EL = COUNT(*) FROM #CUARTILES_ENUM
	
		DROP TABLE IF EXISTS #CUARTILES_FINAL
		SELECT
		CAST(CE.[PromNotaTotal] AS DECIMAL(7,1)) 'PromNotaTotal',
		IIF (A.PromNotaTotal IS NOT NULL AND @CUARTIL_EL>=4 , 1,
			IIF (B.[PromNotaTotal] IS NOT NULL AND @CUARTIL_EL>=4, 2, 
			IIF (C.[PromNotaTotal] IS NOT NULL AND @CUARTIL_EL>=4, 3, 
			IIF (D.[PromNotaTotal] IS NOT NULL AND @CUARTIL_EL>=4, 4, 1)))
		) as quartile
		INTO #CUARTILES_FINAL
		FROM
		#CUARTILES_ENUM CE
		LEFT OUTER JOIN #CUARTILES_ENUM A on CE.PromNotaTotal = A.PromNotaTotal and A.lux > (SELECT COUNT(*)*0.00 FROM #CUARTILES_ENUM) and a.lux <= (SELECT COUNT(*)*0.25 FROM #CUARTILES_ENUM)
		LEFT OUTER JOIN #CUARTILES_ENUM B on CE.PromNotaTotal = B.PromNotaTotal and B.lux > (SELECT COUNT(*)*0.25 FROM #CUARTILES_ENUM) and b.lux <= (SELECT COUNT(*)*0.50 FROM #CUARTILES_ENUM)
		LEFT OUTER JOIN #CUARTILES_ENUM C on CE.PromNotaTotal = C.PromNotaTotal and C.lux > (SELECT COUNT(*)*0.50 FROM #CUARTILES_ENUM) and c.lux <= (SELECT COUNT(*)*0.75 FROM #CUARTILES_ENUM)
		LEFT OUTER JOIN #CUARTILES_ENUM D on CE.PromNotaTotal = D.PromNotaTotal and D.lux > (SELECT COUNT(*)*0.75 FROM #CUARTILES_ENUM)
		
	END
	
	--SELECT * FROM #CUARTILES_FINAL
	--SELECT * FROM #CUARTILES_FINAL_SUP

	
	-- OBTENEMOS LOS TIEMPOS DE CADA AGENTE
	DROP TABLE IF EXISTS #AGENT_AUDIOS_TIME
	SELECT A.[Agent]								'AgentId',
			A.[AgentName]							'AgentName',
			AVG(A.[DurationLinguisticTime])			'AverageLinguisticTime',
			SUM(A.[DurationLinguisticTime])			'TotalLinguisticTime'
		INTO #AGENT_AUDIOS_TIME
		FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
		INNER JOIN #SEARCHED_INTERACTIONS SI
			ON SI.[InteractionId] = IEP.[IdInteraction]
			AND SI.[EvaluationPlanId] = IEP.[IdEvaluationPlan]
		INNER JOIN [dbo].[Audio] A (NOLOCK)
			ON A.[VCC] = IEP.[Instance]
				AND A.[ID] = IEP.[IdInteraction]
				AND A.[Section] = IEP.[InteractionSection]
				AND A.[Part] = IEP.[InteractionPart]
		WHERE IEP.[Instance] = @VCC
			AND (
			 ISNULL(LEN(@EvaluationPlanIds), 0) = 0
			 OR
			 IEP.[IdEvaluationPlan]  IN (SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))

			)
			
		GROUP BY A.[Agent], 
					A.[AgentName]
	
	--SELECT * FROM  #AGENT_AUDIOS_TIME	

	IF (@resultBySupervisor = 1)
	BEGIN
		DROP TABLE IF EXISTS #SUPERVISORS_AUDIOS_TIME
		SELECT WGS.[SupervisorId],AVG(AverageLinguisticTime) 'AverageLinguisticTime',SUM(TotalLinguisticTime) 'TotalLinguisticTime'
			INTO #SUPERVISORS_AUDIOS_TIME
			FROM #AGENT_AUDIOS_TIME AAT
			INNER JOIN #WORKGROUPSUPERVISORS WGS 
				ON AAT.[AgentId] =  WGS.[AgentId]
		GROUP BY WGS.[SupervisorId]

	END

	 --SELECT * FROM  #SUPERVISORS_AUDIOS_TIME
	
	-- OBTENEMOS LAS INTEARACCIONES QUE TUVIERON SENTIMIENTO POSITVO O NEGATIVO
	-- OBTENEMOS LAS INTEARACCIONES QUE TUVIERON SENTIMIENTO POSITVO O NEGATIVO
	SELECT DISTINCT SI.[AgentId],IEP.[IdEvaluationPlan],SR.[InteractionId] ,SR.[SentimentId]
		INTO #SEARCHED_INTERACTIONS_SENTIMENTS
		FROM [dbo].[SentimentAnalysisResult] SR (NOLOCK)
				INNER JOIN [dbo].[InteractionEvaluationPlan] IEP
					ON SR.[InteractionId] = IEP.[IdInteraction]
					AND SR.[InteractionSection] = IEP.[InteractionSection] 
					AND SR.[InteractionPart] = IEP.[InteractionPart]
					AND ISNULL(SR.[EndDate],'') = ISNULL(IEP.[InteractionDateEnd],'')
					AND IEP.[Instance] = @VCC					
					AND (
						 ISNULL(LEN(@EvaluationPlanIds), 0) = 0
						 OR					
						 IEP.[IdEvaluationPlan]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))
						 )
					 AND SR.[AnalysisType] = 'Client'
					 AND SR.[Score] > 0
					 AND SR.[MaxScore] = 1
					 AND SR.[SentimentId] IN ('Positive','Negative') 
				
					INNER JOIN 
					#SEARCHED_INTERACTIONS SI 
					ON 	SR.[InteractionId] = SI.[InteractionId]
						AND IEP.[IdEvaluationPlan] = SI.[EvaluationPlanId]
						AND IEP.[IdAgent] = SI.[AgentId]
			 WHERE SR.VCC = @VCC
				AND SR.[AnalysisType] = 'Client'
				AND SR.[Score] > 0
				AND SR.[SentimentId] IN ('Positive','Negative')

		--SELECT * FROM #SEARCHED_INTERACTIONS_SENTIMENTS
			





	-- OBTENEMOS TOTAL DE INTERACCIONES POR AGENTE 
	DROP TABLE IF EXISTS #TOTALINTERACTION_SR_BY_AGENT
	SELECT [AgentId], COUNT([InteractionId]) 'TotalSentiments'
	INTO #TOTALINTERACTION_SR_BY_AGENT
	FROM  #SEARCHED_INTERACTIONS_SENTIMENTS
	GROUP BY [AgentId]

	
	

	DROP TABLE IF EXISTS #SENTIMENT_RESULT
	SELECT SIR.[AgentId]												'AgentId', 
		SIR.[SentimentId]												'SentimentId',
		COUNT(SIR.[InteractionId])										'Matches' ,
		CAST ((COUNT(SIR.[InteractionId])*100
		/
		CONVERT(DECIMAL(10,2),SRA.TotalSentiments)) AS DECIMAL(7,1))		'MatchesPercentage',
		SRA.[TotalSentiments]											'TotalSentiments',
	    SRAT.[Total]													'Total'
		
		INTO #SENTIMENT_RESULT
		FROM #SEARCHED_INTERACTIONS_SENTIMENTS SIR 
		INNER JOIN  #TOTALINTERACTION_SR_BY_AGENT SRA
		ON SIR.[AgentId] = SRA.[AgentId]
		INNER JOIN #SEARCHED_INTERACTIONS_TOTALBYAGENT SRAT
		ON SRAT.[AgentId] = SIR.[AgentId]
			
		GROUP BY SIR.[AgentId], SIR.[SentimentId], SRA.[TotalSentiments],[Total]	
		
		IF(@resultBySupervisor = 1)
		BEGIN
		DROP TABLE IF EXISTS #SENTIMENT_RESULT_SP
			SELECT	WGS.[SupervisorId],	SentimentId, SUM(Matches) 'Matches',SUM(MatchesPercentage) 'MatchesPercentage',SUM(TotalSentiments) 'TotalSentiments',SUM(Total) 'Total'
			INTO #SENTIMENT_RESULT_SP
			FROM #SENTIMENT_RESULT SR
		    INNER JOIN #WORKGROUPSUPERVISORS WGS 
				ON SR.[AgentId] =  WGS.[AgentId]
			GROUP BY WGS.[SupervisorId],SentimentId
		END
     

	-- OBTENEMOS LAS HABILIDADES PARA NOMBRAR LAS COLUMNAS
		DECLARE @HABILITY_COLUMNS VARCHAR(MAX)
		SET @HABILITY_COLUMNS = STUFF(
				(
					/*
					SELECT DISTINCT ', ROUND(AVG(CAST(q.' + QUOTENAME(Ability) + ' AS float)), 2) As [' + Ability + ' Promedio]'
						FROM #TempEvaluationResult
					*/
					SELECT DISTINCT ',' + QUOTENAME(HA.[Id])
						FROM [dbo].[HistoricAbility] HA (NOLOCK) 
						WHERE HA.[VCC] = @VCC
							AND
                            (
                                ISNULL(LEN(@EvaluationPlanIds), 0) = 0
                                OR
                                HA.[EvaluationPlanId]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))

                            ) 
                            
						FOR xml PATH (''), TYPE
				).value('.', 'VARCHAR(MAX)'), 
				1, 
				1, 
				''
		)

		DECLARE @HABILITY_COLUMNS_ISNULL VARCHAR(MAX)
		SET @HABILITY_COLUMNS_ISNULL = STUFF(
				(
					/*
					SELECT DISTINCT ', ROUND(AVG(CAST(q.' + QUOTENAME(Ability) + ' AS float)), 2) As [' + Ability + ' Promedio]'
						FROM #TempEvaluationResult
					*/
					SELECT DISTINCT ',' +'ISNULL('+ QUOTENAME(HA.[Id]) +',0) AS' + QUOTENAME(HA.[Id]) + ' '
						FROM [dbo].[HistoricAbility] HA (NOLOCK) 
						WHERE HA.[VCC] = @VCC
							AND
                            (
                                ISNULL(LEN(@EvaluationPlanIds), 0) = 0
                                OR
                                HA.[EvaluationPlanId]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))

                            ) 
                            
						FOR xml PATH (''), TYPE
				).value('.', 'VARCHAR(MAX)'), 
				1, 
				1, 
				''
		)
		DECLARE @SENTIMENT_COLUMNS VARCHAR(100)
		DECLARE @ALL_COLUMNS VARCHAR(MAX)
		DECLARE @ALL_COLUMNS_ISNULL VARCHAR(MAX)
		SET @SENTIMENT_COLUMNS =',[Positive],[Negative]'
		SET @ALL_COLUMNS =  @HABILITY_COLUMNS + @SENTIMENT_COLUMNS
		SET @HABILITY_COLUMNS_ISNULL = @HABILITY_COLUMNS_ISNULL  +',ISNULL([Positive],0) AS [Positive],ISNULL([Negative],0) AS [Negative]'

		

		



	-- GENERAMOS EL RESUMEN DE RESULTADOS POR HABILIDAD Y MATCH POSITIVO
	DROP TABLE IF EXISTS #RESULTS_SUMMARY
	SELECT ER.[IdAgent]																	'AgentId',
			ER.[IdAbility]																'AbilityId',
			HA.[Title]																	'AbilityTitle',
			/*
			COUNT(IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL))				'Matches',
			*/
			/* EXCEPCION PARA HABILIDADES CUALITATIVAS Y CUANTITATIVAS, LAS CUALITATIVAS TIENEN [MaxAbilityScore] = 0 */
			/* Y NO PUEDE DISTINGUIRSE CUAL ES EL SCORE SELECCIONADO, POR ESO COMPARAMOS CON [AbilityInnerScore]      */
			/*
			CASE WHEN ER.[MaxAbilityScore] <> 0
					THEN COUNT(IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL))	
					ELSE COUNT(IIF(ER.[AbilityInnerScore] = 100, 1, NULL))
				END																		'Matches',
			*/
			COUNT(
				CASE WHEN ER.[MaxAbilityScore] <> 0
						THEN IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL)
						ELSE IIF(ER.[AbilityInnerScore] = 100, 1, NULL)
					END
			)																			'Matches',
			/*
			CAST(COUNT(IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL))*100
			/
			CONVERT(DECIMAL(4,2),SIT.Total) AS DECIMAL(7,1))							'MatchesPercentage',
			*/
			/* EXCEPCION PARA HABILIDADES CUALITATIVAS Y CUANTITATIVAS, LAS CUALITATIVAS TIENEN [MaxAbilityScore] = 0 */
			/* Y NO PUEDE DISTINGUIRSE CUAL ES EL SCORE SELECCIONADO, POR ESO COMPARAMOS CON [AbilityInnerScore]      */
			/*
			CASE WHEN ER.[MaxAbilityScore] <> 0
					THEN CAST(COUNT(IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL)) * 100
							/
							CONVERT(DECIMAL(4, 2), SIT.Total) AS DECIMAL(7, 1))
					ELSE CAST(COUNT(IIF(ER.[AbilityInnerScore] = 100, 1, NULL)) * 100
							/
							CONVERT(DECIMAL(4, 2), SIT.Total) AS DECIMAL(7, 1))
				END																		'MatchesPercentage',

			*/
			CAST(
				COUNT(
					CASE WHEN ER.[MaxAbilityScore] <> 0
							THEN IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL)
							ELSE IIF(ER.[AbilityInnerScore] = 100, 1, NULL)
						END
				) 
				* 
				100 
				/ 
				CONVERT(DECIMAL(10, 2), SIT.Total) AS DECIMAL(7, 1)
			)																			'MatchesPercentage',
			SIT.Total																	'Total'
			
			
		INTO #RESULTS_SUMMARY
		FROM [dbo].[EvaluationResult] ER (NOLOCK)
		INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			ON ER.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
				AND ER.[IdInteraction] = AEP.[IdInteraction]
				AND ER.[IdAgent] = AEP.[IdAgent]
				AND ER.[IdAuditor] = AEP.[IdAuditor]
				AND AEP.[Completed] = 1 
				AND AEP.[StatusDescription] NOT IN('INVALID')

		INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
			ON HA.[EvaluationPlanId] = ER.[IdEvaluationPlan]
				AND HA.[ID] = ER.[IdAbility]
		INNER JOIN #SEARCHED_INTERACTIONS SI 
			ON SI.[AgentId] = ER.[IdAgent]
				AND SI.[EvaluationPlanId] =ER.[IdEvaluationPlan]
				AND SI.[InteractionId] = ER.[IdInteraction]
       INNER JOIN #SEARCHED_INTERACTIONS_TOTALBYAGENT (NOLOCK) SIT 
		ON ER.[IdAgent] =SIT.[AgentId]			
		WHERE ER.[Instance] = @VCC
			AND (
				ISNULL(LEN(@EvaluationPlanIds), 0) = 0
				OR
				ER.[IdEvaluationPlan]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))
			)
		GROUP BY ER.[IdAgent],
					ER.[IdAbility],
					HA.[Title],
					SIT.[Total]

	 UNION ALL
	 SELECT [AgentId],[SentimentId],[SentimentId],[Matches],[MatchesPercentage],[Total]
		    FROM #SENTIMENT_RESULT

	  IF (@resultBySupervisor = 1)
	  BEGIN
	  
	  	 DROP TABLE IF EXISTS #RESULTS_SUMMARY_SP
		 SELECT WGS.SupervisorId, AbilityId,AbilityTitle,	SUM(Matches) 'Matches',	CAST(CONVERT(DECIMAL(10,2),SUM(Matches) *100/SUM(Total)) AS DECIMAL(7,1)) 'MatchesPercentage',SUM(Total) 'Total'
		 INTO #RESULTS_SUMMARY_SP
		 FROM #RESULTS_SUMMARY RS
		 INNER JOIN #WORKGROUPSUPERVISORS WGS ON 
		 RS.AgentId = WGS.AgentId
		 GROUP BY WGS.SupervisorId, AbilityId,	AbilityTitle
	  END

	  --SELECT * FROM 	#RESULTS_SUMMARY
	  --SELECT * FROM 	#RESULTS_SUMMARY_SP

	 -- OBTENEMOS LAS COLUMANS DINAMICAS(HABILIDADES,SENTIMIENTOS) Y ESTATICAS(TOTAL, AGENTE) PARA EL GRID 
	-- OBTENEMOS LAS COLUMANS DINAMICAS(HABILIDADES,SENTIMIENTOS) Y ESTATICAS(TOTAL, AGENTE) PARA EL GRID 
	
	 SELECT 'HEADER_QUARTILE' 'headerName', 'quartile' 'field', 'quartile' 'customtype'	 
	 UNION ALL
	 SELECT 'HEADER_AGENT' 'headerName','AgentId' 'field', 'agent' 'customtype'
	 UNION ALL
	 SELECT 'HEADER_SUPERVISOR' 'headerName','SupervisorId' 'field', 'supervisor' 'customtype'
	 UNION ALL
	 SELECT 'HEADER_TOTAL_INTERACTIONS' 'headerName', 'TotalInteractions' 'field', 'totalInteractions' 'customtype'
	 UNION ALL
	 SELECT DISTINCT AbilityTitle 'headerName',AbilityId 'field', 
     CASE WHEN HA.IsNegative = 1 AND AC.ID IS NOT NULL  THEN 'negativeAbility' 
          WHEN HA.IsNegative = 0 AND AC.ID IS NOT NULL THEN  'positiveAbility'
          ELSE 'normalAbility' END  'customtype'
	 FROM #RESULTS_SUMMARY RS
		INNER JOIN HistoricAbility HA 
		ON RS.AbilityId = HA.ID 
        LEFT JOIN AbilityScoreCard AC 
        ON HA.ID = AC.ID
        AND AC.DegradeScoreCard = 1 
        

	 WHERE AbilityId NOT IN('Positive','Negative')
	 UNION ALL
	 SELECT 'HEADER_PROM_NOTA_TOTAL' 'headerName','PromNotaTotal' 'field', 'finalnote' 'customtype'
	 UNION ALL
	 SELECT DISTINCT AbilityId 'headerName',AbilityTitle 'field','sentiment' 'customtype' FROM #RESULTS_SUMMARY
	 WHERE AbilityId IN('Positive','Negative')
	 UNION ALL
	 SELECT 'TMO' 'headerName','TMO' 'field', 'TMO' 'customtype'
	



	 

	-- ARMAR DRILLDOWN PARA SETEAR UN NUEVO TOTAL A LA TABLA RESULTANTE
	-- REQUERIMOS LOS DATOS DE AGENTEID, DRILLDOWNTARGET= HABILIDAD O SENTIMIENTO, DRILLDOWNID=IDABILIDAD O IDSENTIMIENTO.
	DECLARE @QUERY VARCHAR(MAX)
	IF (@ResultBySupervisor = 1)
	BEGIN
      
		SET @QUERY  = '
        
		SELECT P.[SupervisorId],
				' + @ALL_COLUMNS + ',
				P.[Total],
				INNER_AAT.[AverageLinguisticTime],
				INNER_AAT.[TotalLinguisticTime]


			INTO #RESULT_SUMMARY_PIVOT
			FROM (
				SELECT [SupervisorId],
						[AbilityId],
						[MatchesPercentage],
						[Total]
					FROM #RESULTS_SUMMARY_SP
			)
			X
			PIVOT (
				MAX([MatchesPercentage]) FOR [AbilityId] IN (' + @ALL_COLUMNS + ')
			) P
			
			LEFT JOIN #SUPERVISORS_AUDIOS_TIME INNER_AAT
				ON   INNER_AAT.[SupervisorId] = P.[SupervisorId]
                
			ORDER BY P.[Total] DESC ' + '

			
			
			SELECT	RSP.[SupervisorId],' + @HABILITY_COLUMNS_ISNULL+ ',					
					SIP.[PromNotaTotal],
					CF.[quartile] AS quartile,
					RSP.[Total] AS TotalInteractions,
					CAST (RSP.[AverageLinguisticTime] AS DECIMAL(7,1)) AS TMO
					
				
			
			FROM  #RESULT_SUMMARY_PIVOT RSP
			INNER JOIN #SEARCHED_INTERACTIONS_NOTAS_PROM_SUPERVISOR SIP
                ON RSP.[SupervisorId] = SIP.[SupervisorId]
               
				
			INNER JOIN #CUARTILES_FINAL_SUP CF ON 
				CF.PromNotaTotal = SIP.PromNotaTotal

			ORDER BY SIP.[PromNotaTotal] DESC
			
	    '
		END 
		ELSE
		BEGIN
			SET @QUERY  = '
        
		SELECT P.[AgentId],
				' + @ALL_COLUMNS + ',
				P.[Total],
				INNER_AAT.[AverageLinguisticTime],
				INNER_AAT.[TotalLinguisticTime]


			INTO #RESULT_SUMMARY_PIVOT
			FROM (
				SELECT [AgentId],
						[AbilityId],
						[MatchesPercentage],
						[Total]
					FROM #RESULTS_SUMMARY
			)
			X
			PIVOT (
				MAX([MatchesPercentage]) FOR [AbilityId] IN (' + @ALL_COLUMNS + ')
			) P
			
			LEFT JOIN #AGENT_AUDIOS_TIME INNER_AAT
				ON   INNER_AAT.[AgentId] = P.[AgentId]
                
			ORDER BY P.[Total] DESC ' + '

			
			
			SELECT	RSP.[AgentId],' + @HABILITY_COLUMNS_ISNULL+ ',					
					SIP.[PromNotaTotal],
					CF.[quartile] AS quartile,
					RSP.[Total] AS TotalInteractions,
					CAST (RSP.[AverageLinguisticTime] AS DECIMAL(7,1)) AS TMO
					
				
			
			FROM  #RESULT_SUMMARY_PIVOT RSP
			INNER JOIN #SEARCHED_INTERACTIONS_NOTAS_PROM SIP
                ON RSP.[AgentId] = SIP.[AgentId]
               
				
			INNER JOIN #CUARTILES_FINAL CF ON 
				CF.PromNotaTotal = SIP.PromNotaTotal

			ORDER BY SIP.[PromNotaTotal] DESC
			
	    '
		END


	-- EJECUTAMOS EL QUERY ARMADO

	
	--PRINT (@QUERY)
	EXECUTE (@QUERY)

	

	DROP TABLE IF EXISTS #AGENT_AUDIOS_TIME
	DROP TABLE IF EXISTS #RESULTS_SUMMARY
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetTemplate] 'inconcert','Template1'
CREATE PROCEDURE [dbo].[GetTemplate] (
	@VCC Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'vcc',
		[Type]							'type',
		[Name]						    'name',
		[Description]					'description',
		[Template]						'template',
		[Category]						'category',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Template] (NOLOCK)
	WHERE [VCC] = CASE WHEN ISNULL(@VCC,'') = '' THEN [VCC] ELSE @VCC END
	AND [Name] = @Name

	EXEC [GetTemplateThemes] @VCC, @Name
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetTemplateByName] 'default','gss_template'
CREATE PROCEDURE [dbo].[GetTemplateByName] (
	@VCC Varchar(50) = NULL, @Type Varchar(50) = NULL, @Name Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Type]							'type',
		[Name]						    'name',
		[Description]					'description',
		[Template]						'template',
		[Category]						'category',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Template] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [Type] = @Type
	AND [Name] = @Name
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetTemplates] 'system','template','name',''
CREATE PROCEDURE [dbo].[GetTemplates] (
	@VCC Varchar(50) = NULL,	
	@Type Varchar(50) = NULL,	
	@SearchCriteria Varchar(50) = NULL,	
	@SearchExpression Varchar(50) = NULL,
	@Language VARCHAR(5) = 'en'
	)
AS
BEGIN
	SET NOCOUNT ON

	-- CREAMOS LAS CTE QUE USAREMOS PARA RECUPERAR LOS MODELOS DE LAS PLANTILLAS
	;WITH CTE_TEMPLATE_NLUS AS (
		SELECT DISTINCT TTI.[VCC]
						, TTI.[TemplateId]
						--, TTI.[TemplateThemeId]
						--, TTI.[ID]
						, N.[AppID]
						, N.[Name]
						, N.[Culture]
			FROM [TemplateThemeItem] TTI (NOLOCK)
			LEFT JOIN [dbo].[GenericQuery] GQ (NOLOCK) 
				ON GQ.[VCC] = TTI.[VCC]
					AND GQ.[Name] = TTI.[ID] 
					AND GQ.[VCC] = TTI.[VCC]
			INNER JOIN [dbo].[NLU] N (NOLOCK) 
				ON N.[VCC] = TTI.[VCC] 
					AND N.[Name] = SUBSTRING(TTI.[ID], 0, CHARINDEX('|', TTI.[ID]))
			WHERE TTI.[VCC] IN (@VCC, 'system')
				AND TTI.[ID] LIKE '%|%'
				AND GQ.[Query] IS NULL
	),
	TEMP_TEMPLATE AS (
		SELECT T.[Type]																	'type',
				T.[Name]																'name',
				T.[Description]															'description',
				T.[Template]															'template',
				T.[Category]															'category',
				STUFF((
					SELECT ',' + CTN_INNER.[Culture]
						FROM CTE_TEMPLATE_NLUS CTN_INNER (NOLOCK)
						WHERE CTN_INNER.[TemplateId] = T.[Name]
				FOR XML PATH(''), TYPE).value('.', 'NVARCHAR(MAX)'), 1, 1, '')			'modelLanguages',
				T.[CreatedDate]															'createdDate',
				T.[CreatedByUserId]														'createdByUserId',
				T.[LastModifiedDate]													'lastModifiedDate',
				T.[LastModifiedByUserId]												'lastModifiedByUserId'
			FROM [dbo].[Template] T (NOLOCK)
			LEFT JOIN [dbo].[BatchSchedule] BS (NOLOCK)
				ON BS.[VCC] = T.[VCC]
					AND BS.[BatchData] = T.[Name]
			WHERE (
					T.[VCC] = @VCC
					OR
					(
						T.[VCC] = 'system'
						AND 
						T.[Name] LIKE @Language + '%'
					)
				)
				AND ISNULL(T.[Name], '') LIKE CASE WHEN @SearchCriteria = 'name' THEN '%' + @SearchExpression + '%' ELSE '%' END
				AND ISNULL(T.[Category], '') LIKE CASE WHEN @SearchCriteria = 'category' THEN '%' + @SearchExpression + '%' ELSE '%' END
				AND ISNULL(T.[Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
				AND ISNULL(T.[Type], '') LIKE CASE WHEN @SearchCriteria = 'type' THEN '%' + @SearchExpression + '%' ELSE '%' END
				AND ( -- BLOQUE PARA OBTENER LAS PLANTILLAS QUE NO TIENEN ASIGNADA UNA AGENDA (PLANTILLA GLOBAL)
					(
						@SearchCriteria = 'schedule'
						AND 
						BS.[Id] IS NULL
					)
					OR
					@SearchCriteria <> 'schedule'
				)
				AND ( -- BLOQUE PARA OBTENER LAS PLANTILLAS QUE TIENEN ASIGNADA UNA AGENDA (PLANTILLA GLOBAL)
					(
						@SearchCriteria = 'scheduled'
						AND 
						BS.[Id] IS NOT NULL
					)
					OR
					@SearchCriteria <> 'scheduled'
				)
				/*
	SELECT
		T.[Type]																'type',
		T.[Name]																'name',
		T.[Description]															'description',
		T.[Template]															'template',
		T.[Category]															'category',
		STUFF((
			SELECT ',' + CTN_INNER.[Culture]
				FROM CTE_TEMPLATE_NLUS CTN_INNER (NOLOCK)
				WHERE CTN_INNER.[TemplateId] = T.[Name]
		FOR XML PATH(''), TYPE).value('.', 'NVARCHAR(MAX)'), 1, 1, '')			'modelLanguages',
		T.[CreatedDate]															'createdDate',
		T.[CreatedByUserId]														'createdByUserId',
		T.[LastModifiedDate]													'lastModifiedDate',
		T.[LastModifiedByUserId]												'lastModifiedByUserId'
	FROM [Template] T (NOLOCK)
	WHERE T.[VCC] IN (@VCC)
		AND T.[Type] = CASE WHEN ISNULL(@Type, '') = '' THEN T.[Type] ELSE @Type END
		AND T.[Type] IN ('custom')
		AND ISNULL(T.[Name], '') LIKE CASE WHEN @SearchCriteria = 'name' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(T.[Category], '') LIKE CASE WHEN @SearchCriteria = 'category' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(T.[Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(T.[Type], '') LIKE CASE WHEN @SearchCriteria = 'type' THEN '%' + @SearchExpression + '%' ELSE '%' END
		
		UNION 
		
		SELECT
		T.[Type]																'type',
		T.[Name]																'name',
		T.[Description]															'description',
		T.[Template]															'template',
		T.[Category]															'category',
		STUFF((
			SELECT ',' + CTN_INNER.[Culture]
				FROM CTE_TEMPLATE_NLUS CTN_INNER (NOLOCK)
				WHERE CTN_INNER.[TemplateId] = T.[Name]
		FOR XML PATH(''), TYPE).value('.', 'NVARCHAR(MAX)'), 1, 1, '')			'modelLanguages',
		T.[CreatedDate]															'createdDate',
		T.[CreatedByUserId]														'createdByUserId',
		T.[LastModifiedDate]													'lastModifiedDate',
		T.[LastModifiedByUserId]												'lastModifiedByUserId'
	FROM [Template] T (NOLOCK)
	WHERE T.[VCC] IN ('system')
		AND T.[Type] = CASE WHEN ISNULL(@Type, '') = '' THEN T.[Type] ELSE @Type END
		AND T.[Type] IN ('system')
		AND T.[Name] LIKE @Language+'%'
		AND ISNULL(T.[Name], '') LIKE CASE WHEN @SearchCriteria = 'name' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(T.[Category], '') LIKE CASE WHEN @SearchCriteria = 'category' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(T.[Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(T.[Type], '') LIKE CASE WHEN @SearchCriteria = 'type' THEN '%' + @SearchExpression + '%' ELSE '%' END
		*/
	)


		SELECT * 
			FROM TEMP_TEMPLATE 
			WHERE [NAME] NOT LIKE 'qty_%' 
				AND [type] <> 'global'
		ORDER BY [Name] asc, [CreatedDate] desc 
		
	--ORDER BY T.[Name] asc, T.[CreatedDate] desc 

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetTemplatesVCC] 'system'
CREATE PROCEDURE [dbo].[GetTemplatesVCC] (
	@VCC Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Type]							'type',
		[Name]							'name',
		[Description]					'description',
		[Template]						'template',
		[Category]						'category',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Template] (NOLOCK)
	WHERE [VCC] = @VCC
	AND ([Type] = 'template' OR
	[Type] = 'generic')
	ORDER BY [Name] asc, [CreatedDate] desc 

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

-- [dbo].[GetTemplateThemeItemById] 'inconcert', '2D6D3989-7F78-4DF9-BB72-61C696CBCCA8'
CREATE PROCEDURE [dbo].[GetTemplateThemeItemById]
(
	@VCC			VARCHAR(50),
	@ID				VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON
	
	SELECT
		[VCC]							'vcc', 
		[TemplateId]					'templateId',
		[TemplateThemeId]				'templateThemeId',
		[ID]							'id',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [dbo].[TemplateThemeItem]
	WHERE
		[VCC] = @VCC AND
		[ID] = @ID

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetTemplateThemeItemEntities] (
	@VCC VARCHAR(50) = NULL, 
	@TemplateThemeItemId VARCHAR(100) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	--
	SELECT [TTIE].[EntityId]									'entityId',
			[TTIE].[EntityName]									'entityName',
			[TTIE].[AcceptancePercentage]						'acceptancePercentage',
			ISNULL([TTIE].[AcceptancePercentage], 0) * 100		'acceptanceInteger',
			[TTIE].[Order]										'order'
		FROM [dbo].[TemplateThemeItemEntity] TTIE (NOLOCK)
		WHERE [TTIE].[VCC] = @VCC
			AND [TTIE].[TemplateThemeItemId] = @TemplateThemeItemId
		ORDER BY [TTIE].[Order] ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetTemplateThemeModels] (
	@VCC VARCHAR(50) = NULL, 
	@AnalysisType VARCHAR(50) = NULL,
	@TemplateId VARCHAR(50) = NULL,
	@TemplateThemeId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- DECLARAMOS UNA VARIABLE PARA DETERMINAR SI ES UN FORMATO O PLANTILLA
	DECLARE @TEMPLATE_TYPE VARCHAR(50) = NULL

	-- BUSCAMOS POR EL ID SI SE TRATA DE UNA PLANTILLA O FORMATO
	IF EXISTS (
		SELECT 1 
			FROM [dbo].[Template] T (NOLOCK)
			INNER JOIN [dbo].[EvaluationFormat] EF (NOLOCK)
				ON EF.[Instance] = T.[VCC]
					AND EF.[templateId] = T.[Name]
			WHERE T.[VCC] = @VCC
				AND T.[Name] = @TemplateId
	)
		SET @TEMPLATE_TYPE = 'EVALUATION_FORMAT'
	ELSE 
		SET @TEMPLATE_TYPE = 'TEMPLATE'

	-- CREAMOS LA TABLA TEMPORAL
	CREATE TABLE #MODEL_DATA_TMP ([VCC] VARCHAR(50), [ModelId] VARCHAR(125), [AppId] VARCHAR(50), [IntentId] VARCHAR(50))

	IF @AnalysisType = 'COGNITIVE' OR @AnalysisType IS NULL
	BEGIN
		PRINT CONCAT('GETTING COGNITIVE MODELS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
		INSERT INTO [dbo].[#MODEL_DATA_TMP]
			SELECT DISTINCT
					N.[VCC]
					,N.[Name] + '|'+ I.[Name]
					,N.[AppId]
					,I.[IntentId]
				FROM [dbo].[NLU] N (NOLOCK)
				INNER JOIN [dbo].[Intent] I (NOLOCK) 
					ON N.[VCC] = I.[VCC]
						AND I.[AppId] = N.[AppId]
				WHERE N.[VCC] = @VCC
	END
	
	IF @AnalysisType = 'GENERATIVE' OR @AnalysisType IS NULL
	BEGIN
		PRINT CONCAT('GETTING GENERATIVE MODELS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
		INSERT INTO [dbo].[#MODEL_DATA_TMP]
			SELECT DISTINCT
					G.[VCC]
					,G.[Name] + '|'+ GI.[Name]
					,G.[AppId]
					,GI.[Id]
				FROM [dbo].[Generative] G (NOLOCK)
				INNER JOIN [dbo].[GenerativeIntent] GI (NOLOCK)
					ON GI.[VCC] = G.[VCC]
						AND GI.[AppId] = G.[AppId]
				WHERE G.[VCC] = @VCC
	END

	IF @AnalysisType = 'ADVANCED_AI' OR @AnalysisType IS NULL
	BEGIN
		PRINT CONCAT('GETTING ADVANCED AI MODELS ', (CONVERT(VARCHAR(24), GETUTCDATE(), 121)))
		INSERT INTO [dbo].[#MODEL_DATA_TMP]
			SELECT DISTINCT
					AAI.[VCC]
					,AAI.[Id] + '|'+ AAII.[Name]
					,AAI.[Id]
					,AAII.[Id]
				FROM [dbo].[AdvancedAI] AAI (NOLOCK)
				INNER JOIN [dbo].[AdvancedAIIntent] AAII (NOLOCK)
					ON AAII.[VCC] = AAI.[VCC]
						AND AAII.[AppId] = AAI.[Id]
				WHERE AAI.[VCC] = @VCC
	END

	-- SELECCIONAMOS LOS ITEMS INMEDIATOS RELACIONADOS AL TEMPLATE
	SELECT DISTINCT A.[VCC]												'vcc',	
					A.[TemplateId]										'templateId',
					A.[TemplateThemeId]									'templateThemeId',
					A.[ID]												'id',
					CASE WHEN @TEMPLATE_TYPE = 'EVALUATION_FORMAT' 
						THEN A.[ID]
						ELSE 
							CASE WHEN MD_TMP.[ModelId] IS NOT NULL 
								THEN MD_TMP.[ModelId]
								ELSE A.[ID]
								END
						END												'query',
					MD_TMP.[IntentId]									'intentId',
					TT.[AnalysisType]									'analysisType',
					@TEMPLATE_TYPE										'templateType',
					A.[AcceptancePercentage]							'acceptancePercentage',
					ISNULL(A.[AcceptancePercentage],0) * 100			'acceptanceInteger',
					A.[CreatedDate]										'createdDate',
					A.[CreatedByUserId]									'createdByUserId',
					A.[LastModifiedDate]								'lastModifiedDate',
					A.[LastModifiedByUserId]							'lastModifiedByUserId'
		INTO #TEMPLATE_MODELS 
		FROM [TemplateThemeItem] A (NOLOCK)
		INNER JOIN [dbo].[TemplateTheme] TT (NOLOCK)
			ON TT.[VCC] = A.[VCC]
				AND TT.[TemplateId] = A.[TemplateId]
				AND TT.[ID] = A.[TemplateThemeId]
		INNER JOIN [dbo].[#MODEL_DATA_TMP] MD_TMP (NOLOCK)
			ON MD_TMP.[VCC] = A.[VCC] 
				AND (
					MD_TMP.[ModelId] = A.[ID]
					OR
					MD_TMP.[IntentId] = A.[ID]
				)
		WHERE A.[VCC] = @VCC
			AND A.[TemplateId] = CASE WHEN ISNULL(@TemplateId, '') = '' THEN A.[TemplateId] ELSE @TemplateId END
			AND A.[TemplateThemeId] = CASE WHEN ISNULL(@TemplateThemeId, '') = '' THEN A.[TemplateThemeId] ELSE @TemplateThemeId END

	-- PARA TEMPLATES DE UN FORMATO, AGREGAMOS LAS QUERY_ID (INTENTS) DE LAS TIPIFICACIONES DE 1ER NIVEL
	IF @TEMPLATE_TYPE = 'EVALUATION_FORMAT'
	BEGIN
		INSERT INTO #TEMPLATE_MODELS 
			SELECT DISTINCT EFASTQ.[VCC],
							EF.[templateId],
							'',
							EFASTQ.[QueryId],
							EFASTQ.[QueryId],
							NULL,
							A.[AnalysisType],
							@TEMPLATE_TYPE,
							EFASTQ.[AcceptancePercentage],
							EFASTQ.[AcceptancePercentage] * 100, 
							NULL,
							NULL,
							NULL,
							NULL
				FROM [dbo].[EvaluationFormatAbilityScoreTipificationQuery] EFASTQ (NOLOCK)
				INNER JOIN [dbo].[EvaluationFormat] EF (NOLOCK)
					ON EF.[Instance] = EFASTQ.[VCC]
						AND EF.[ID] = EFASTQ.[EvaluationFormatId]
				INNER JOIN [dbo].[Score] S (NOLOCK)
					ON S.[Instance] = EFASTQ.[VCC]
						AND S.[ID] = EFASTQ.[ScoreId]
				INNER JOIN [dbo].[Ability] A (NOLOCK)
					ON A.[Instance] = EFASTQ.[VCC]
						AND A.[ID] = S.[IdAbility]
				WHERE EFASTQ.[VCC] = @VCC
					AND EF.[templateId] = @TemplateId

	END
	--select * FROM [dbo].[EvaluationFormatAbilityScoreTipificationQuery] EFASTQ (NOLOCK)
	--SELECT * FROM Score (NOLOCK)
	-- CONSULTAMOS EL TOTAL DE LA TABLA TEMPORAL
	SELECT * FROM #TEMPLATE_MODELS (NOLOCK)

	-- ELIMINAMOS LA TABLA TEMPORAL
	DROP TABLE IF EXISTS [dbo].[#MODEL_DATA_TMP]

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetTemplateThemeItems] 'system',''
CREATE PROCEDURE [dbo].[GetTemplateThemeQueries] (
	@VCC Varchar(50) = NULL, 
	@TemplateId Varchar(50) = NULL,
	@TemplateThemeId Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	SELECT A.vcc,
			A.[TemplateId]									'templateId',
			A.[TemplateThemeId]								'templateThemeId',
			A.[ID]											'id',
			B.[Query]										'query',
			TT.[AnalysisType]								'analysisType',
			A.[AcceptancePercentage]						'acceptancePercentage',
			ISNULL(A.[AcceptancePercentage],0) * 100        'acceptanceInteger',
			A.[CreatedDate]									'createdDate',
			A.[CreatedByUserId]								'createdByUserId',
			A.[LastModifiedDate]							'lastModifiedDate',
			A.[LastModifiedByUserId]						'lastModifiedByUserId'
		FROM [TemplateThemeItem] A (NOLOCK)
		INNER JOIN [dbo].[TemplateTheme] TT (NOLOCK)
			ON TT.[VCC] = A.[VCC]
				AND TT.[TemplateId] = A.[TemplateId]
				AND TT.[ID] = A.[TemplateThemeId]
		INNER JOIN [dbo].[GenericQuery] B (NOLOCK) 
			ON B.[VCC] = A.[VCC]
				AND B.[Name] = A.[ID]
		WHERE A.[VCC] = @VCC
			AND A.[TemplateId] = CASE WHEN ISNULL(@TemplateId, '') = '' THEN A.[TemplateId] ELSE @TemplateId END
			AND A.[TemplateThemeId] = CASE WHEN ISNULL(@TemplateThemeId, '') = '' THEN A.[TemplateThemeId] ELSE @TemplateThemeId END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetTemplateThemes] 'system','C6F633E4-4DC0-409E-980C-590CAB2DAC33'
CREATE PROCEDURE [dbo].[GetTemplateThemes] (
	@VCC Varchar(50) = NULL, @TemplateId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[TemplateId]					'templateId',
		[ID]							'id',
		[Order]							'order',
		[Description]					'description',
		[Category1]						'category1',
		[Category2]						'category2',
		[AnalysisType]                  'analysisType',
		[IsNegative]                    'isNegative',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [TemplateTheme] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [TemplateId] = CASE WHEN ISNULL(@TemplateId, '') = '' THEN [TemplateId] ELSE @TemplateId END
	ORDER BY [Order] asc

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetTenantVCCByVCC] (
	@VCC Varchar(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	SELECT TV.[TenantId] 'tenantId',
			TV.[ClientId] 'clientId',
			TV.[ClientSecret] 'clientSecret',
			TV.[Resource] 'resource'
		FROM [dbo].[TenantVCC] TV (NOLOCK)
		WHERE TV.[VCC] = @VCC

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetTipification] (
	@VCC VARCHAR(50) = NULL, 
	@ID VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- CONSULTAMOS LA DATA PROPIA DE LA TIPIFICACION
	SELECT T.[ID]								'id',
			T.[Description]						'description',
			T.[Detail]							'detail',
			T.[CreatedDate]						'createdDate',
			T.[CreatedByUserId]					'createdByUserId',
			T.[LastModifiedDate]				'lastModifiedDate',
			T.[LastModifiedByUserId]			'lastModifiedByUserId'
		FROM [dbo].[Tipification] T (NOLOCK)
		WHERE T.[VCC] = @VCC
			AND T.[ID] = @ID

	-- CONSULTAMOS LA DATA QUERY DE LA TIPIFICACION
	SELECT TQ.[QueryId]							'queryId',
			TQ.[QueryContent]					'queryContent',
			TQ.[AcceptancePercentage]			'acceptancePercentage',
			(TQ.[AcceptancePercentage] * 100)	'acceptanceInteger'
		FROM [dbo].[TipificationQuery] TQ (NOLOCK)
		WHERE TQ.[VCC] = @VCC
			AND TQ.[TipificationId] = @ID

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetTipificationItems] (
	@VCC VARCHAR(50) = NULL, 				
	@TipificationId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	--
	SELECT TI.[ID]								'id',
			TI.[SecondLevelItemId]				'secondLevelItemId',
			TI.[Description]					'description',
			TI.[Order]							'order'
		FROM [dbo].[TipificationItem] TI (NOLOCK)
		WHERE TI.[VCC] = @VCC 
			AND TI.[TipificationId] = @TipificationId
		ORDER BY TI.[Order] ASC
	
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetTipifications] (
	@VCC VARCHAR(50) = NULL, 
	@SearchCriteria VARCHAR(50) = NULL, 
	@SearchExpression VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT T.[ID]							'id',
			T.[Description]					'description'
		FROM [dbo].[Tipification] T (NOLOCK)
		WHERE T.[VCC] = @VCC
			AND ISNULL(T.[Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
		ORDER BY T.[Description] ASC
		
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE  PROCEDURE [dbo].[GetTopAgents] (
	@Instance VARCHAR(50) = NULL,
	@InitialDate DATETIME = NULL,
	@EndDate DATETIME = NULL,
	@TopQuantity INT = 10,
	@TopBest BIT = 1,
	@IdCampaign VARCHAR(MAX) = 'GetTopAgents',
	@IdEvaluationFormat VARCHAR(MAX) = 'GetTopAgents',
	@IdAgent VARCHAR(MAX) = 'GetTopAgents',
	@IdWorkGroup VARCHAR(MAX) = 'GetTopAgents',
	@IdAttentionLevel VARCHAR(MAX) = 'GetTopAgents',
	@IdEvaluationPlan VARCHAR(MAX) = 'GetTopAgents',
	@IdGroup VARCHAR(MAX) = 'GetTopAgents',
	@IdSubGroup VARCHAR(MAX) = 'GetTopAgents',
	@IdAbility VARCHAR(MAX) = 'GetTopAgents',
	@IdAuditor VARCHAR(MAX) = 'GetTopAgents'
)
AS
BEGIN
	DECLARE @InteractionsEvaluated INT = 0;
	DECLARE @InteractionsApproved INT = 0;

	SET NOCOUNT ON;

	WITH 
	-- DECLARAMOS UNA CTE PARA DEPOSITAR LAS LLAMADAS EVALUADAS CON SU ULTIMA FECHA DE EVALUACION
	CteTotalInteractionsEvaluated AS (
		SELECT DISTINCT ER.[Instance], 
						ER.[IdEvaluationPlan], 
						ER.[IdAuditor], 
						ER.[IdAgent], 
						ER.[IdInteraction],
						AEP.[Score],
						ER.[MinQualificationAcceptable], 
						RN = ROW_NUMBER() OVER(PARTITION BY ER.[Instance], ER.[IdEvaluationPlan], ER.[IdAuditor], ER.[IdAgent], ER.[IdInteraction] ORDER BY ER.[EvaluationDate] DESC)
			FROM EvaluationResult ER (NOLOCK)
			LEFT JOIN AuditorEvaluationPlan AEP (NOLOCK) 
				ON AEP.[Instance] = ER.[Instance]
					AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]
					AND AEP.[IdAgent] = ER.[IdAgent]
					AND AEP.[IdInteraction] = ER.[IdInteraction]
			WHERE ER.[Instance] = @Instance
				AND ER.[TimeStamp] BETWEEN @InitialDate AND @EndDate
				AND CASE WHEN @IdAgent = 'GetTopAgents' THEN 'GetTopAgents' ELSE AEP.[IdAgent] END IN (SELECT Value FROM [dbo].[split](@IdAgent,','))
				AND CASE WHEN @IdCampaign = 'GetTopAgents' THEN 'GetTopAgents' ELSE AEP.[IdCampaign] END IN (SELECT Value FROM [dbo].[split](@IdCampaign,','))
				AND CASE WHEN @IdEvaluationFormat = 'GetTopAgents' THEN 'GetTopAgents' ELSE ER.[IdEvaluationFormat] END IN (SELECT Value FROM [dbo].[split](@IdEvaluationFormat,','))
				AND CASE WHEN @IdEvaluationPlan = 'GetTopAgents' THEN 'GetTopAgents' ELSE AEP.[IdEvaluationPlan] END IN (SELECT Value FROM [dbo].[split](@IdEvaluationPlan,','))
				AND ER.[IdAuditor] LIKE CASE WHEN @IdAuditor = 'GetTopAgents' THEN '%' ELSE 'SPEECH' END AND ER.[IdAuditor] <> CASE WHEN @IdAuditor = 'GetTopAgents' THEN 'SPEECH' ELSE 'B_I_G_C_H_E_E_S_E' END
				AND AEP.[Instance] IS NOT NULL
	)
	-- HACEMOS EL SELECT RESUMEN SOBRE EL RESULTADO DE LA CTE
	SELECT TOP 15 IE.[IdAgent] 'AgentDescription',
					IE.[IdAgent] 'AgentId',
					COUNT(*) 'Evaluated',
					SUM(CASE WHEN IE.[Score] >= IE.[MinQualificationAcceptable] THEN 1 ELSE 0 END) 'Approved', 
					SUM(CASE WHEN IE.[Score] < IE.[MinQualificationAcceptable] THEN 1 ELSE 0 END) 'Disapproved',
					SUM(CASE WHEN IE.[Score] >= IE.[MinQualificationAcceptable] THEN 1 ELSE 0 END) * 100 / COUNT(*) 'ApprovalPercentage',
					SUM(CASE WHEN IE.[Score] < IE.[MinQualificationAcceptable] THEN 1 ELSE 0 END) * 100 / COUNT(*) 'DisapprovalPercentage'
		FROM CteTotalInteractionsEvaluated IE (NOLOCK)
		WHERE IE.[RN] = 1
		GROUP BY IE.[Instance], IE.[IdAgent], IE.[RN]
		ORDER BY 
			-- DEPENDIENDO QUE TOP BUSCAMOS (MEJOR O PEOR) AGREGAMOS EL ORDER BY AL FINAL SOBRE LA COLUMNA DE PORCENTAJE
			CASE @TopBest 
				WHEN 1
					THEN SUM(CASE WHEN IE.[Score] >= IE.[MinQualificationAcceptable] THEN 1 ELSE 0 END) * 100 / COUNT(*)
			END DESC,
			CASE @TopBest
				WHEN 0 
					THEN SUM(CASE WHEN IE.[Score] < IE.[MinQualificationAcceptable] THEN 1 ELSE 0 END) * 100 / COUNT(*)
			END DESC

		


	/*
	WITH 
		-- DECLARAMOS UNA CTE PARA DEPOSITAR LAS LLAMADAS EVALUADAS CON SU ULTIMA FECHA DE EVALUACION
		CteTotalInteractionsEvaluated AS (
			-- Llamadas evaluadas con su ultima calificacion, agente evaluado y boolean indicador de aprobacion
			SELECT
				ER.IdCampaign,
				ER.IdInteraction,
				ER.IdAgent,
				SUM(ER.Qualification) TotalQualification,
				CASE
					WHEN SUM(ER.Qualification) >= ER.MinQualificationAcceptable THEN 1
					ELSE 0
				END InteractionApproved
			FROM
				EvaluationResult ER,
				(
					-- Llamadas evaluadas con su ultima fecha de evaluacion
					SELECT
						IdCampaign,
						IdInteraction,
						IdAgent,
						MAX(EvaluationDate) LastEvaluationDate
					FROM
						EvaluationResult
					WHERE
						EvaluationResult.Instance = @Instance
						AND EvaluationDate BETWEEN @InitialDate AND @EndDate
						AND CASE WHEN @IdAgent = 'GetTopAgents' THEN 'GetTopAgents' ELSE IdAgent END IN (SELECT Value FROM [dbo].[split](@IdAgent,','))
						AND CASE WHEN @IdCampaign = 'GetTopAgents' THEN 'GetTopAgents' ELSE IdCampaign END IN (SELECT Value FROM [dbo].[split](@IdCampaign,','))
						AND CASE WHEN @IdEvaluationFormat = 'GetTopAgents' THEN 'GetTopAgents' ELSE IdEvaluationFormat END IN (SELECT Value FROM [dbo].[split](@IdEvaluationFormat,','))
						AND CASE WHEN @IdEvaluationPlan = 'GetTopAgents' THEN 'GetTopAgents' ELSE IdEvaluationPlan END IN (SELECT Value FROM [dbo].[split](@IdEvaluationPlan,','))
						AND IdAuditor LIKE CASE WHEN @IdAuditor = 'GetTopAgents' THEN '%' ELSE 'SPEECH' END AND IdAuditor <> CASE WHEN @IdAuditor = 'GetTopAgents' THEN 'SPEECH' ELSE 'B_I_G_C_H_E_E_S_E' END
					GROUP BY
						IdCampaign,
						IdInteraction,
						idAgent
				) LastEvaluation
			WHERE
				ER.IdInteraction = LastEvaluation.IdInteraction AND
				ER.EvaluationDate = LastEvaluation.LastEvaluationDate
			GROUP BY
				ER.IdCampaign,
				ER.IdInteraction,
				ER.IdAgent,
				ER.MinQualificationAcceptable
		)

	SELECT TOP (@TopQuantity)
		InteractionsEvaluated.IdAgent AgentId,
		InteractionsEvaluated.IdAgent AgentDescription,
		(InteractionsApproved.InteractionsApproved * 100 / InteractionsEvaluated.InteractionsEvaluated) ApprovalPercentage,
		InteractionsEvaluated.InteractionsEvaluated Evaluated,
		InteractionsApproved.InteractionsApproved Approved
	FROM
		(
			-- Llamadas evaluadas para cada agente
			SELECT
				InteractionTotalQualification.IdAgent,
				Count(*) InteractionsEvaluated
			FROM
				CteTotalInteractionsEvaluated InteractionTotalQualification
			GROUP BY
				InteractionTotalQualification.IdAgent
		) InteractionsEvaluated,
		(
			-- Llamadas aprobadas para cada agente
			SELECT
				InteractionTotalQualification.IdAgent,
				SUM(InteractionTotalQualification.InteractionApproved) InteractionsApproved
			FROM
				CteTotalInteractionsEvaluated InteractionTotalQualification
			GROUP BY
				InteractionTotalQualification.IdAgent
		) InteractionsApproved
	WHERE
		InteractionsEvaluated.IdAgent = InteractionsApproved.IdAgent
	ORDER BY
		CASE @TopBest
			WHEN 1
			THEN (InteractionsApproved.InteractionsApproved * 100 / InteractionsEvaluated.InteractionsEvaluated)
		END DESC,
		CASE @TopBest
			WHEN 0
			THEN (InteractionsApproved.InteractionsApproved * 100 / InteractionsEvaluated.InteractionsEvaluated)
		END ASC
	*/
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[GetTranscription] 'default', 'F040EA5A-E455-49EC-8A8D-4A4AD13D8786', 0, 0, 'FFB04B5FF6E242098CD31F419CC4E4A3'
CREATE PROCEDURE [dbo].[GetTranscription]
(
	@VCC VARCHAR(50) = NULL,
	@ProcessId VARCHAR(50) = NULL,
	@InteractionType VARCHAR(50) = NULL,
	@Section NUMERIC(5, 0),
	@Part NUMERIC(5, 0),
	@Campaign VARCHAR(100) = NULL,
	@Folder VARCHAR(50) = NULL,
	@ID VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- SI INTERACTIONTYPE ES NULL SETTEAMOS COMO CALL
	IF @InteractionType IS NULL SET @InteractionType = 'CALL'

	SELECT
		[VCC]						'vcc',
		[ProcessId]					'processId',
		[InteractionType]			'interactionType',
		[Section]					'section',
		[Part]						'part',
		[Folder]					'folder',
		[Campaign]					'campaign',
		[ID]						'id',
		[Value]						'value',
		[CreatedDate]				'createdDate',
		[CreatedByUserId]			'createdByUserId',
		[LastModifiedDate]			'lastModifiedDate',
		[LastModifiedByUserId]		'lastModifiedByUserId'
	FROM [dbo].[Transcription] (NOLOCK)
	WHERE
		[VCC] = @VCC 
		AND [ProcessId] = @ProcessId 
		AND [InteractionType] = @InteractionType
		AND (
			(
				[InteractionType] = 'CALL'
					AND [Section] = @Section 
					AND [Part] = @Part 
			) OR (
				[InteractionType] <> 'CALL'
					-- AND [Folder] = @Folder
					AND [Campaign] = @Campaign
			)
		)
		AND [ID] = @ID

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetUniqueModelsByIntents]
(
	@VCC					VARCHAR(50),
	@TemplateId				VARCHAR(50),
	@AnalysisType			VARCHAR(50),
	@Intents				VARCHAR(MAX)
)
AS
BEGIN
	SET NOCOUNT ON

	-- DECLARAMOS UNA VARIABLE PARA DETERMINAR SI ES UN FORMATO O PLANTILLA
	DECLARE @TEMPLATE_TYPE VARCHAR(50) = NULL

	-- BUSCAMOS POR EL ID SI SE TRATA DE UNA PLANTILLA O FORMATO
	IF EXISTS (
		SELECT 1 
			FROM [dbo].[Template] T (NOLOCK)
			INNER JOIN [dbo].[EvaluationFormat] EF (NOLOCK)
				ON EF.[Instance] = T.[VCC]
					AND EF.[templateId] = T.[Name]
			WHERE T.[VCC] = @VCC
				AND T.[Name] = @TemplateId
	)
		SET @TEMPLATE_TYPE = 'EVALUATION_FORMAT'
	ELSE 
		SET @TEMPLATE_TYPE = 'TEMPLATE'

	-- CREAMOS LA TABLA TEMPORAL
	CREATE TABLE #MODEL_DATA_TMP ([VCC] VARCHAR(50), [ModelId] VARCHAR(125), [AppId] VARCHAR(50), [IntentId] VARCHAR(50))

	IF @AnalysisType = 'COGNITIVE'
	BEGIN
		INSERT INTO [dbo].[#MODEL_DATA_TMP]
			SELECT DISTINCT
					N.[VCC]
					,N.[Name] + '|' + I.[Name]
					--,N.[Name]								'nluId'
					,N.[AppId]
					,I.[IntentId]
					--,I.[Name]
				FROM [dbo].[NLU] N (NOLOCK)
				INNER JOIN [dbo].[Intent] I (NOLOCK) 
					ON N.[VCC] = I.[VCC]
						AND I.[AppId] = N.[AppId]
				WHERE N.[VCC] = @VCC
					AND (
						I.[IntentId] IN (SELECT Value FROM dbo.[SplitString](@Intents, ','))
						OR
						CONCAT(N.[Name], '|', I.[Name]) IN (SELECT Value FROM dbo.[SplitString](@Intents, ','))
					)
	END
	ELSE IF @AnalysisType = 'GENERATIVE'
	BEGIN
		INSERT INTO [dbo].[#MODEL_DATA_TMP]
			SELECT DISTINCT
					G.[VCC]
					,G.[Name] + '|' + GI.[Name]
					--,G.[Name]								'nluId'
					,G.[AppId]
					,GI.[Id]
					--,GI.[Name]
				FROM [dbo].[Generative] G (NOLOCK)
				INNER JOIN [dbo].[GenerativeIntent] GI (NOLOCK)
					ON GI.[VCC] = G.[VCC]
						AND GI.[AppId] = G.[AppId]
				WHERE G.[VCC] = @VCC
					AND (
						GI.[Id] IN (SELECT Value FROM dbo.[SplitString](@Intents, ','))
						OR
						CONCAT(G.[Name], '|', GI.[Name]) IN (SELECT Value FROM dbo.[SplitString](@Intents, ','))
					)
	END
	ELSE IF @AnalysisType = 'ADVANCED_AI'
	BEGIN
		INSERT INTO [dbo].[#MODEL_DATA_TMP]
			SELECT DISTINCT
					AI.[VCC]
					,AI.[Id] + '|' + AII.[Name]
					--,G.[Name]								'nluId'
					,AI.[Id]
					,AII.[Id]
					--,GI.[Name]
				FROM [dbo].[AdvancedAI] AI (NOLOCK)
				INNER JOIN [dbo].[AdvancedAIIntent] AII (NOLOCK)
					ON AII.[VCC] = AI.[VCC]
						AND AII.[AppId] = AI.[Id]
				WHERE AI.[VCC] = @VCC
					AND (
						AII.[Id] IN (SELECT Value FROM dbo.[SplitString](@Intents, ','))
						OR
						CONCAT(AI.[Id], '|', AII.[Name]) IN (SELECT Value FROM dbo.[SplitString](@Intents, ','))
					)
	END
	
	-- CREAMOS LA TABLA TEMPORAL
	CREATE TABLE #TEMPLATE_MODELS ([appId] VARCHAR(50))

	-- PARA TEMPLATE DE PLANTILLA SOLO RESUMIMOS LO YA OBTENIDO
	IF @TEMPLATE_TYPE = 'TEMPLATE'
	BEGIN
		INSERT INTO [dbo].[#TEMPLATE_MODELS] 
			SELECT DISTINCT MD_TMP.[AppId]
				FROM [dbo].[#MODEL_DATA_TMP] MD_TMP 
	END

	-- PARA TEMPLATES DE UN FORMATO, AGREGAMOS LAS QUERY_ID (INTENTS) DE LAS TIPIFICACIONES DE 1ER NIVEL
	IF @TEMPLATE_TYPE = 'EVALUATION_FORMAT'
	BEGIN
		-- ESTA FUNCION ES USADA SOLO PARA FORMATOS DE EVALUACION
		INSERT INTO [dbo].[#TEMPLATE_MODELS] 
			SELECT DISTINCT MD_TMP.[AppId]
				FROM [dbo].[EvaluationFormat] EF (NOLOCK)
				INNER JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
					ON EF.[Instance] = AEF.[Instance] 
						AND EF.[ID] = AEF.[IdEvaluationFormat]
				INNER JOIN [dbo].[Ability] A (NOLOCK) 
					ON A.[Instance] = EF.[Instance]
						AND A.[ID] = AEF.IdAbility 
						AND A.[AnalysisType] IS NOT NULL 
				INNER JOIN [dbo].[AbilityQuery] AQ (NOLOCK)
					ON AQ.[VCC] = A.[Instance] 
						AND AQ.[AbilityId] = A.[ID] 
				INNER JOIN [dbo].[#MODEL_DATA_TMP] MD_TMP 
					ON MD_TMP.[VCC] = EF.[Instance] 
						AND MD_TMP.[ModelId] = AQ.[QueryContent] 
						AND MD_TMP.[IntentId] = AQ.[QueryId] 
				WHERE EF.[Instance] = @VCC
					AND EF.[templateId] = @TemplateId

		INSERT INTO [dbo].[#TEMPLATE_MODELS] 
			SELECT MD_TMP.[AppId]
				FROM [dbo].[EvaluationFormatAbilityScoreTipificationQuery] EFASTQ (NOLOCK)
				INNER JOIN [dbo].[EvaluationFormat] EF (NOLOCK)
					ON EF.[Instance] = EFASTQ.[VCC]
						AND EF.[ID] = EFASTQ.[EvaluationFormatId]
				INNER JOIN [dbo].[#MODEL_DATA_TMP] MD_TMP (NOLOCK)
					ON MD_TMP.[VCC] = EFASTQ.[VCC]
						AND MD_TMP.[IntentId] = EFASTQ.[QueryId]
				WHERE EF.[templateId] = @TemplateId

	END

	-- CONSULTAMOS EL TOTAL DE LA TABLA TEMPORAL
	SELECT DISTINCT * FROM #TEMPLATE_MODELS (NOLOCK)

	-- ELIMINAMOS LA TABLA TEMPORAL
	DROP TABLE IF EXISTS [dbo].[#MODEL_DATA_TMP]
	DROP TABLE IF EXISTS [dbo].[#TEMPLATE_MODELS]

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

-- [dbo].[GetUniqueTags] 'inconcert'
CREATE PROCEDURE [dbo].[GetUniqueTags]
(
	@VCC						VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]												'vcc',
		[TagName]											'tagName',
		[CreatedDate]										'createdDate',
		[CreatedByUserId]									'createdByUserId',
		[LastModifiedDate]									'lastModifiedDate',
		[LastModifiedByUserId]								'lastModifiedByUserId',
		[TimeStamp]											'timeStamp'
	FROM
		[dbo].[UniqueTag]
	WHERE
		[VCC] = @VCC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetUser] (
	@VCC Varchar(50) = NULL, @UserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]								'id',
		[Email] 							'email',
		[Firstname] 						'firstname',
		[Lastname] 							'lastname',
		[Role] 								'role',
		[Country] 							'country',
		[Language] 							'language',
		[TimeZone] 							'timeZone',
		[Preferences] 						'preferences',
		[IsSystemUser] 						'isSystemUser',
		[IsLocalSupervisor]					'isLocalSupervisor',
		[IsExternalSupervisor]				'isExternalSupervisor',
		[IsExternalAgent]					'isExternalAgent',
		[LimitClients] 						'limitClients',
		[API]								'api',
		[CreatedDate]						'createdDate',
		[CreatedByUserId]					'createdByUserId',
		[LastModifiedDate]					'lastModifiedDate',
		[LastModifiedByUserId]				'lastModifiedByUserId'
	FROM [User] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [ID] = @UserId
	
	EXEC [GetUserClient] @VCC, @UserId
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[GetUserAccessToken] 'default', 'admin'
CREATE PROCEDURE [dbo].[GetUserAccessToken]
(
	@VCC					VARCHAR(50),
	@UserName				VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON
	SELECT
		[VCC]				'VCC',
		[UserName]			'userName',
		[AccessToken]		'accessToken',
		[CreatedDate]		'createdDate'
		FROM [dbo].[UserAccessToken]
	WHERE VCC = @VCC AND UserName = @UserName
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--[dbo].[GetUserAccessTokenByRealm] 'system', 'system'
CREATE PROCEDURE [dbo].[GetUserAccessTokenByRealm]
(
	@VCC					VARCHAR(50),
	@Realm					VARCHAR(100)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
	[VCC]					'vcc',
	[Realm]					'realm',
	[UserName]				'userName',
	[AccessToken]			'accessToken',
	[CreatedDate]			'createdDate'
	FROM UserAccessToken (NOLOCK)
	WHERE [VCC] = @VCC 
	AND [Realm] = @Realm

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetUserClient] (
	@VCC Varchar(50) = NULL, @UserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[UserId]						'userId',
		[ClientId]						'clientId'
	FROM [UserClient] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [UserId] = @UserId

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetUsers] (
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	-- LISTAMOS LOS USUARIOS SEGUN LOS CRITERIOS DE BUSQUEDA Y NO SEAN USUARIOS DEL SISTEMA
	SELECT
		[ID]								'id',
		[Email] 							'email',
		[Firstname] 						'firstname',
		[Lastname] 							'lastname',
		[Role] 								'role',
		[Country] 							'country',
		[Language] 							'language',
		[TimeZone] 							'timeZone',
		[Preferences] 						'preferences',
		[IsSystemUser] 						'isSystemUser',
		[API]								'api',
		[CreatedDate]						'createdDate',
		[CreatedByUserId]					'createdByUserId',
		[LastModifiedDate]					'lastModifiedDate',
		[LastModifiedByUserId]				'lastModifiedByUserId'
	FROM [User] (NOLOCK)
	WHERE [VCC] = @VCC
		AND [IsSystemUser] = 0 
		AND ISNULL([ID], '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL([Firstname], '') + ' ' + ISNULL([Lastname], '') LIKE CASE WHEN @SearchCriteria = 'name' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL([Role], '') LIKE CASE WHEN @SearchCriteria = 'role' THEN '%' + @SearchExpression + '%' ELSE '%' END
	
	SET NOCOUNT OFF
END 
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[GetUtterancesByIntentId] 'allegro', '45799dd8-17e7-49a1-9bd8-cb74f00a788c'
CREATE PROCEDURE [dbo].[GetUtterance]
(
	@VCC			VARCHAR(50),
	@AppId			VARCHAR(50),
	@IntentName		VARCHAR(50),
	@Text			VARCHAR(500)
)
AS
BEGIN
	SET NOCOUNT ON

	DECLARE @INTENT_ID VARCHAR(50) = NULL
	SELECT @INTENT_ID = [IntentId]
		FROM [dbo].[Intent] (NOLOCK) 
		WHERE [VCC] = @VCC
			AND [AppId] = @AppId
			AND [Name] = @IntentName
  
    -- BUSCAMOS LOS UTTERANCES SEGUN LOS PARAMETROS RECIBIDOS
	SELECT U.[VCC]								'vcc',
			U.[AppId]							'appId',
			U.[IntentId]						'intentId',
			I.[Name]							'intentName',
			U.[Id]								'id',
			U.[Text]							'text',
			U.[CreatedDate]						'createdDate',
			U.[CreatedByUserId]					'createdByUserId',
			U.[LastModifiedDate]				'lastModifiedDate',
			U.[LastModifiedByUserId]			'lastModifiedByUserId'
		FROM [dbo].[Utterance] U (NOLOCK)
		INNER JOIN [dbo].[Intent] I (NOLOCK) 
			ON I.[VCC] = U.[VCC]
				AND I.[IntentId] = U.[IntentId]
		WHERE U.[VCC] = @VCC
			AND U.[AppId] = @AppId
			AND U.[IntentId] = @INTENT_ID
			AND U.[Text] = @Text

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[GetUtterancesByIntentId] 'allegro', '45799dd8-17e7-49a1-9bd8-cb74f00a788c'
CREATE PROCEDURE [dbo].[GetUtteranceEntities]
(
	@VCC			VARCHAR(50),
	@UtteranceId	VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	-- BUSCAMOS LOS UTTERANCE ENTITIES SEGUN LOS PARAMETROS RECIBIDOS
	SELECT UE.[VCC]									'vcc',
			UE.[UtteranceId]						'utteranceId',
			UE.[EntityName]							'entityName',
			UE.[Start]								'start',
			UE.[End]								'end',
			UE.[StartTokenIndex]					'startTokenIndex',
			UE.[EndTokenIndex]						'endTokenIndex'
		FROM [dbo].[UtteranceEntity] UE (NOLOCK)
		WHERE UE.[VCC] = @VCC
			AND UE.[UtteranceId] = @UtteranceId
		ORDER BY UE.[Start] ASC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[GetUtterancesByIntentId] 'allegro', '45799dd8-17e7-49a1-9bd8-cb74f00a788c'
CREATE PROCEDURE [GetUtterances]
(
	@VCC			  VARCHAR(50),
	@AppId			  VARCHAR(50),
	@Intent  		  VARCHAR(50),
	@SearchCriteria   VARCHAR(50) = NULL, 
	@SearchExpression VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- BUSCAMOS LOS UTTERANCES SEGUN LOS PARAMETROS RECIBIDOS
	DECLARE @QUERY VARCHAR(MAX) = ''
	SET @QUERY = 'SELECT U.[VCC]			    vcc,
			U.[AppId]							appId,
			U.[IntentId]						intentId,
			I.[Name]							intentName,
			U.[Id]								id,
			U.[Text]							text,
			U.[CreatedDate]						createdDate,
			U.[CreatedByUserId]					createdByUserId,
			U.[LastModifiedDate]				lastModifiedDate,
			U.[LastModifiedByUserId]			lastModifiedByUserId,
			U.[Status]                          status,
			U.[StatusCode]                      statusCode
		FROM [dbo].[Utterance] U (NOLOCK)
		INNER JOIN [dbo].[Intent] I (NOLOCK) 
			ON I.[VCC] = U.[VCC]
				AND I.[AppId] = U.[AppId]
				AND I.[IntentId] = U.[IntentId]
		WHERE U.[VCC] ='
		
		SET @QUERY += ''''
		SET @QUERY += @VCC
		SET @QUERY += ''''
	
	IF(@AppId IS NOT NULL AND @AppId <> '')
	BEGIN 
	     SET @QUERY += ' AND U.[AppId] ='
	     SET @QUERY += ''''
	     SET @QUERY += @AppId
	     SET @QUERY += ''''
	END
	
	IF(@Intent IS NOT NULL AND @Intent <> '')
	BEGIN 
	     SET @QUERY += ' AND I.[Name] ='
	     SET @QUERY += ''''
	     SET @QUERY += @Intent
	     SET @QUERY += ''''
	END
	
	
	IF(@SearchCriteria IS NOT NULL AND @SearchExpression IS NOT NULL AND @SearchExpression <> '')
	BEGIN
		SET @QUERY += ' AND '
		SET @QUERY += @SearchCriteria
		SET @QUERY += ' LIKE '
		SET @QUERY += ''''
		SET @QUERY += '%'
		SET @QUERY += @SearchExpression
		SET @QUERY += '%'
		SET @QUERY += ''''
	END
     EXEC(@QUERY)
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[GetUtterancesByInteraction] 'allegro', '{f2c9e160-8eeb-11e9-8562-b65bb2fb6bcb}', 0, 1
CREATE PROCEDURE [dbo].[GetUtterancesByInteraction]
(
	@VCC			VARCHAR(50),
	@AudioId		VARCHAR(50),
	@Section		NUMERIC(5, 0),
	@Part			NUMERIC(5, 0)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		U.[VCC]							'vcc',
		UT.[AppId]							'appId',
		UT.[IntentId]						'intentId',
		U.[AudioId]						'audioId',
		U.[Section]						'section',
		U.[Part]							'part',
		U.[Speaker]						'speaker',
		U.[AnchorOffset]					'anchorOffset',
		U.[FocusOffset]					'focusOffset',
		U.[CurrentTime]					'currentTime',
		U.[EndCurrentTime]				'endCurrentTime',
		U.[UtteranceId]					'utteranceId',
		UT.[Text]					'textContent',
		U.[CreatedDate]					'createdDate',
		U.[CreatedByUserId]				'createdByUserId',
		U.[LastModifiedDate]				'lastModifiedDate',
		U.[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM
		[dbo].[Underlined] U WITH (NOLOCK) 
		JOIN [dbo].[Utterance] UT (NOLOCK) 
		ON (U.[VCC] = UT.[VCC] AND U.[UtteranceId] = UT.[Id])
	WHERE
		U.[VCC] = @VCC
		AND U.[AudioId] = @AudioId
		AND U.[Section] = @Section
		AND U.[Part] = @Part

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetVCC](
	@ID varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]									'vcc',
		[ID]									'id',
		[TYPE]									'type',
		[Authentication]						'authentication',
		[TranscriptionDefaultLanguage]			'transcriptionDefaultLanguage',
		[Description]							'description',
		[CognitiveEngine]						'cognitiveEngine',
		[EnableAdvancedAI]						'enableAdvancedAI',
		[EnableGenerativeEngine]				'enableGenerativeEngine',
		[EnableRealTime]						'enableRealTime',
		[Country]								'country',
		[Language]								'language',
		[TimeZone]								'timeZone',
		[CreatedDate]							'createdDate',
		[CreatedByUserId]						'createdByUserId',
		[LastModifiedDate]						'lastModifiedDate',
		[LastModifiedByUserId]					'lastModifiedByUserId',
		[IsMigrating]                           'isMigrating',
		[DeleteDataDays]                        'deleteDataDays'
	FROM [VCC] (NOLOCK)
	WHERE [ID] = @ID

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetVccCleanerConfig]
AS
BEGIN
	SET NOCOUNT ON
	
	--OBTENER LA CONFIGURACION DE LOS VCC PARA ELIMINAR LA DATA
	
	SELECT 
	[VCC] 'vcc',
	[DeleteDataDays] 'deleteDataDays' 
	FROM [dbo].[VCC] (NOLOCK) 
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetVCCLicense](
	@VCC varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'vcc',
		[Product]						'product',
		[Status]						'status',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [VCCLicense] (NOLOCK)
	WHERE [VCC] = @VCC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetVCCs] (
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[ID]									'id',
		[Description]							'description',
		[Type]									'type',
		[Authentication]						'authentication',
		[TranscriptionDefaultLanguage]			'transcriptionDefaultLanguage',
		[CognitiveEngine]						'cognitiveEngine',
		[EnableRealTime]						'enableRealTime',
		[Country]								'country',
		[Language]								'language',
		[TimeZone]								'timeZone',
		[CreatedDate]							'createdDate',
		[CreatedByUserId]						'createdByUserId',
		[LastModifiedDate]						'lastModifiedDate',
		[LastModifiedByUserId]					'lastModifiedByUserId',
        [IsMigrating]                           'isMigrating'
	FROM [VCC] (NOLOCK)
	WHERE [VCC] <> 'system'
		AND [VCC] <> 'default'
		AND ISNULL([ID], '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL([Country], '') LIKE CASE WHEN @SearchCriteria = 'country' THEN '%' + @SearchExpression + '%' ELSE '%' END
        AND ISNULL([CognitiveEngine], '') LIKE CASE WHEN @SearchCriteria = 'cognitiveEngine' THEN '%' + @SearchExpression + '%' ELSE '%' END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetVersion]
AS
BEGIN
	
	SET NOCOUNT ON

	SELECT
		[Id]								'id',
		[InstallationId]					'installationId',
		[ServicePack]						'servicePack',
		[BuildVersion]						'buildVersion',
		[IsRegistered]						'isRegistered',
		[TimeStamp]							'timeStamp',
		[ServicePackTimeStamp]				'servicePackTimeStamp'
	FROM
		[dbo].[Version] (NOLOCK)


	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

-- [dbo].[GetWordBlacklist] 'default', '1FC1C683-191F-4A46-A932-31C5D8116FC2'
CREATE PROCEDURE [dbo].[GetWordBlacklist]
(
	@VCC			VARCHAR(50),
	@ProcessId		VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[VCC]							'VCC',
		[ProcessId]						'processId',
		[Word]							'word',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM
		WordBlacklist WITH (NOLOCK)
	WHERE
		[VCC]	= @VCC AND
		[ProcessId] = @ProcessId
		
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [GetWords] 'default','NeuroLinguisticProgrammingId','visual','E4CC3881-9B1E-420B-977D-7CB473ED9C14', 'es'
--exec [GetWords] 'system','NeuroLinguisticProgrammingId','','init', 'es'
CREATE PROCEDURE [dbo].[GetWords] (
	@VCC Varchar(50) = NULL, @SearchCriteria Varchar(50) = NULL, @SearchExpression Varchar(50) = NULL,
	@ProcessId Varchar(50) = NULL, @Language Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[NeuroLinguisticProgrammingId]	'neuroLinguisticProgrammingId',
		[Name]							'name',
		[ProcessId]						'processId',
		[Language]						'language',
		[Type]							'type',
		[CreatedDate]					'createdDate',
		[CreatedByUserId]				'createdByUserId',
		[LastModifiedDate]				'lastModifiedDate',
		[LastModifiedByUserId]			'lastModifiedByUserId'
	FROM [Word] (NOLOCK)
	WHERE [VCC] = @VCC
	AND ISNULL([NeuroLinguisticProgrammingId], '') LIKE CASE WHEN @SearchCriteria = 'neuroLinguisticProgrammingId' THEN '%' + @SearchExpression + '%' ELSE '%' END
	AND [ProcessId] = @ProcessId
	AND [Language] = @Language

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [dbo].[GetWorkGroupsFromSupervisorEP] 'vccocc',''
CREATE PROCEDURE [dbo].[GetWorkGroupsFromSupervisorEP] (
	@VCC Varchar(50) = NULL, 
	@CampaignIds Varchar(MAX) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
    IF @CampaignIds <> ''
    BEGIN 
        SELECT DISTINCT
            [Description]					'description',
            [SupervisorId]                  'supervisorId',
            [CampaignId]                    'campaignId',
            [AgentId]                       'agentId'
        FROM [WorkGroupSupervisor] (NOLOCK)
        WHERE [VCC] = @VCC
            AND ( 
				[CampaignId]  IN (SELECT Value FROM [dbo].[split](@CampaignIds, ','))
				OR [CampaignId] = 'withoutCampaign'
				)
          
	END
    ELSE
    BEGIN
         SELECT DISTINCT
            [Description]					'description',
            [SupervisorId]                  'supervisorId',
            [CampaignId]                    'campaignId',
            [AgentId]                       'agentId'
        FROM [WorkGroupSupervisor] (NOLOCK)
        WHERE [VCC] = @VCC

    END

		
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetWorkGroupsSupervisor] (
	@VCC Varchar(50) = NULL, 
	@SearchCriteria Varchar(50) = NULL, 
	@SearchExpression Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT DISTINCT
		[Description]					'description',
		[SupervisorId]                  'supervisorId',
		[IsHistoric]					'isHistoric'
	FROM [WorkGroupSupervisor] (NOLOCK)
	WHERE [VCC] = @VCC
		AND ISNULL([Description], '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL([SupervisorId], '') LIKE CASE WHEN @SearchCriteria = 'supervisor' THEN '%' + @SearchExpression + '%' ELSE '%' END
		
		
	SET NOCOUNT OFF
END

PRINT N'Update complete.';
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetWorkGroupSupervisor] (
	@VCC Varchar(50) = NULL, @SupervisorId Varchar(250) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[Description]					'description',
		[SupervisorId]					'supervisorId',
		[CampaignID]					'campaignId',
		[agentId]					    'agentId',
		[IsHistoric]					'isHistoric'
		
	FROM [WorkGroupSupervisor] (NOLOCK)
	WHERE [VCC] = @VCC
    AND [SupervisorId] = @SupervisorId
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[InsertUserHistory](
	@VCC Varchar(50) = NULL, @UserId Varchar(50) = NULL, @UserRole Varchar(50) = NULL,
	@Event Varchar(50) = NULL, @EventDate Datetime = NULL, @EventDetail Varchar(200) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	INSERT INTO [UserHistory] (
		[VCC], [UserId], [UserRole],
		[Event], [EventDate], [EventDetail],
		[TimeStamp]
		) 
	VALUES (
		@VCC, @UserId, @UserRole,
		@Event, @EventDate, @EventDetail,
		GETUTCDATE()
	)

    SELECT 1 AS 'result'
    
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[LaunchAudioConverter] (
	@VCC VARCHAR(50) = NULL, 
	@APP_PATH VARCHAR(250) = NULL,
	@INPUT_FILE_PATH VARCHAR(250) = NULL, 
	@OUTPUT_FOLDER_PATH VARCHAR(250) = NULL,
	@REMOTE_REPO_DIARIZATION_PATH VARCHAR(250) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	DECLARE @CMD_MOUNT VARCHAR(250) = 'net use ' + @REMOTE_REPO_DIARIZATION_PATH
	DECLARE @CMD_UMOUNT VARCHAR(250) = 'net use ' + LEFT(@REMOTE_REPO_DIARIZATION_PATH,2) + ' /delete'
	
	-- ASEGURARSE DE QUE LA CARPETA DE RED DEL FTP ES VISIBLE PARA EL USUARIO DE SISTEMA DE SQL SERVER (TOQUE OL)
	IF (@REMOTE_REPO_DIARIZATION_PATH <> 'local')	
	EXEC xp_cmdshell @CMD_MOUNT

	-- CREAMOS UNA VARIABLE PARA ARMAR EL COMANDO
	DECLARE @COPY_TO_LOCAL_COMMAND VARCHAR(1000) = NULL
	DECLARE @COPY_TO_ORIGINAL_COMMAND VARCHAR(1000) = NULL
	DECLARE @DIARIZATE_COMMAND VARCHAR(1000) = NULL
	DECLARE @DELETE_FROM_LOCAL_INPUT VARCHAR(1000) = NULL
	DECLARE @DELETE_FROM_LOCAL_OUTPUT VARCHAR(1000) = NULL

	DECLARE @LOCAL_C_INPUT_FOLDER VARCHAR(200) = @APP_PATH + '/input'
	DECLARE @LOCAL_C_OUTPUT_FOLDER VARCHAR(200) = @APP_PATH + '/output'
	DECLARE @INPUT_FILE_NAME VARCHAR(250) = NULL

	--SET @COMMAND = 'C:/diarization/inConcertDiarization/inConcertDiarization.exe -i G:/Ftp_Volcado/Repositorio/diarization/inConcertDiarization/source/input/ENTRANTE.wav -o G:/Ftp_Volcado/Repositorio/diarization/inConcertDiarization/source/output -d 1'
	-- xp_cmdshell 'C:\diarization\inConcertDiarization\inConcertDiarization.exe -i C:\diarization\inConcertDiarization\source\input\ENTRANTE.wav -o C:\diarization\inConcertDiarization\source\output -d 1';
	
	-- CREAMOS UNA CTE PARA SEPARAR LAS PARTES DE LA RUTA DE INPUT
	;WITH CTE_INPUT_SPLIT AS (
		SELECT Value 'VAL', 
				ROW_NUMBER() OVER (ORDER BY (SELECT 1)) 'RN' 
			--FROM STRING_SPLIT('G:/Ftp_Volcado/Repositorio/diarization/inConcertDiarization/source/input/ENTRANTE.wav','/')
			FROM STRING_SPLIT(@INPUT_FILE_PATH, '/')
	)

	-- GUARDAMOS EN VARIABLE EL NOMBRE DEL ARCHIVO INPUT
	SELECT TOP 1 @INPUT_FILE_NAME = [VAL] 
		FROM CTE_INPUT_SPLIT 
		ORDER BY [RN] DESC

	-- COMPIAMOS EL ARCHIVO A LA CARPEA DE INPUT CUANDO LAS RUTAS SON DIFERENTES
	IF @INPUT_FILE_PATH <> (@LOCAL_C_INPUT_FOLDER + '/' + @INPUT_FILE_NAME)
	BEGIN
		-- COPIAMOS EL ARCHIVO INPUT DE LA RUTA RECIBIDA A LA CARPETA INPUT LOCAL
		SET @COPY_TO_LOCAL_COMMAND = 'copy "' + @INPUT_FILE_PATH + '" "' +  @LOCAL_C_INPUT_FOLDER + '/' + @INPUT_FILE_NAME + '" '
		SET @COPY_TO_LOCAL_COMMAND = REPLACE(@COPY_TO_LOCAL_COMMAND, '/', '\')
	
		-- EJECUTAMOS EL COMANDO
		EXEC('xp_cmdshell ''' +  @COPY_TO_LOCAL_COMMAND + '''')
	END
	ELSE
	BEGIN
		SELECT 1
	END

	-- ARMAMOS EL COMANDO QUE MUEVE A LA CARPETA LOCAL Y DIARIZAMOS
	--SET @COMMAND = @APP_PATH + '/inConcertDiarization.exe -i ' + @INPUT_FILE_PATH + ' -o ' + @OUTPUT_FOLDER_PATH + ' -d 1';
	--SET @DIARIZATE_COMMAND = 'cd ' + @LOCAL_C_FOLDER + ' & ' + @APP_PATH + '/inConcertDiarization.exe -i input/' + @INPUT_FILE_NAME + ' -o output '
	SET @DIARIZATE_COMMAND = 'cd ' + @APP_PATH + ' & inConcertDiarization.exe -i input/' + @INPUT_FILE_NAME + ' -o output -d 1'
	
	-- EJECUTAMOS EL COMANDO
	EXEC('xp_cmdshell ''' +  @DIARIZATE_COMMAND + '''')

	-- COMPIAMOS EL ARCHIVO A LA CARPEA DE INPUT CUANDO LAS RUTAS SON DIFERENTES
	IF @INPUT_FILE_PATH <> (@LOCAL_C_INPUT_FOLDER + '/' + @INPUT_FILE_NAME)
	BEGIN
		-- COPIAMOS EL ARCHIVO OUTPUT DE LA RUTA LOCAL A LA CARPETA OUTPUT RECIBIDA
		SET @COPY_TO_ORIGINAL_COMMAND = 'move "' + @LOCAL_C_OUTPUT_FOLDER + '/' + @INPUT_FILE_NAME + '" "' +  @OUTPUT_FOLDER_PATH + '/' + @INPUT_FILE_NAME + '" '
		SET @COPY_TO_ORIGINAL_COMMAND = REPLACE(@COPY_TO_ORIGINAL_COMMAND, '/', '\')

		-- EJECUTAMOS EL COMANDO
		EXEC('xp_cmdshell ''' +  @COPY_TO_ORIGINAL_COMMAND + '''')

		-- REMOVEMOS LOS ARCHIVOS DE INPUT Y OUTPUT LOCAL
		SET @DELETE_FROM_LOCAL_INPUT = 'del "' + @LOCAL_C_INPUT_FOLDER + '/' + @INPUT_FILE_NAME + '" '
		SET @DELETE_FROM_LOCAL_OUTPUT = 'del "' + @LOCAL_C_OUTPUT_FOLDER + '/' + @INPUT_FILE_NAME + '" '
	
		SET @DELETE_FROM_LOCAL_INPUT = REPLACE(@DELETE_FROM_LOCAL_INPUT, '/', '\')
		SET @DELETE_FROM_LOCAL_OUTPUT = REPLACE(@DELETE_FROM_LOCAL_OUTPUT, '/', '\')
	
		EXEC('xp_cmdshell ''' +  @DELETE_FROM_LOCAL_INPUT + '''')
		--EXEC('xp_cmdshell ''' +  @DELETE_FROM_LOCAL_OUTPUT + '''')
	END

	--EXEC xp_cmdshell 'net use L: /delete /y' -- TOQUE OL DESCARTADO
	EXEC xp_cmdshell @CMD_UMOUNT	
	

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ReportAuditsByAgent] (
-- Debe venir siempre un unico formato evaluacion, si no viene no camina
   @Instance VARCHAR(50) = NULL,
   @IdEvaluationFormat VARCHAR(50) = NULL,
   @IdsCampanias VARCHAR(1000) = NULL,
   @IdAgente INT = NULL,
   @FechaDesde DATE =  NULL,
   @FechaHasta DATE = NULL,
   @IdAuditor VARCHAR(50) = NULL,
   @PlanAuditoria INT = NULL
   )
AS
BEGIN
DECLARE @columns VARCHAR(MAX),
	@columnsName VARCHAR(MAX),
	@columnsFLOAT VARCHAR(MAX),
	@insertVar VARCHAR(MAX),
	@idAgent VARCHAR(50),
	@idInteraction VARCHAR(50),
	@idEvaluationPlan VARCHAR(50),
	@idAgentAnterior VARCHAR(50) = NULL,
	@CreatedDate DATETIME,
	@CreatedDateAnterior DATETIME = NULL,
	@qualification FLOAT,
	@maxAbilityScore FLOAT, 
	@porcentajeAbilidad FLOAT,
	@cantAuditoriasAgente INT = 0,
	@cantAgentes INT = 0,
	@idHabilidad VARCHAR(50),
	@idInteractionAnterior VARCHAR(50),
	@idEvaluationPlanAnterior VARCHAR(50),
	@idAuditorAnterior VARCHAR(50),
	@ConsultaSQLsum VARCHAR(MAX),
	@columnsInsert VARCHAR(MAX),
	@columnsInsertLength INT,
	@columnsValuesLength INT

	SET NOCOUNT ON
	-- Insercion de datos retornados de filtro en tabla temporal.
	CREATE TABLE #EvaluationResult (
		Instance			VARCHAR(50),
		IdAgent				VARCHAR(50),
		CreatedDate			DATETIME,
		Qualification		NUMERIC(5, 2),
		MaxAbilityScore		NUMERIC(5, 2),
		IdAbility			VARCHAR(50),
		AbilityOrder		NUMERIC(4, 0),
		IdAuditor			VARCHAR(50),
		IdInteraction		VARCHAR(50),
		IdEvaluationPlan	VARCHAR(50),
		IdEvaluationFormat	VARCHAR(50)
	)
	INSERT INTO #EvaluationResult
	SELECT
		Instance, IdAgent, CreatedDate, Qualification, MaxAbilityScore, IdAbility, AbilityOrder, IdAuditor, IdInteraction, IdEvaluationPlan, IdEvaluationFormat
	FROM
		EvaluationResult
	WHERE
		Instance = @Instance
		AND CreatedDate BETWEEN CONVERT(VARCHAR, @FechaDesde) + ' 00:00:00.000' AND CONVERT(VARCHAR, @FechaHasta) + ' 23:59:59.999'
		AND IdEvaluationFormat = @IdEvaluationFormat-- TODO ajustar en base a demas filtros que se definan.

	-- Variable para validar cantidad de registros.
	DECLARE @COUNT INT
	SET @COUNT = (SELECT COUNT(IdInteraction) FROM #EvaluationResult)
	PRINT '@COUNT: ' + CONVERT(VARCHAR, @COUNT)
	
	IF(@COUNT > 0)
	BEGIN
		-- Continua con desarrollo de cursor.
		DECLARE curP CURSOR LOCAL FOR
			SELECT
				IdAgent, CreatedDate, Qualification, MaxAbilityScore, IdAbility,IdAuditor, IdInteraction, IdEvaluationPlan
			FROM
				EvaluationResult
			WHERE
				Instance = @Instance
				AND CreatedDate BETWEEN CONVERT(VARCHAR, @FechaDesde) + ' 00:00:00.000' AND CONVERT(VARCHAR, @FechaHasta) + ' 23:59:59.999'
				AND IdEvaluationFormat = @IdEvaluationFormat-- TODO ajustar en base a demas filtros que se definan.
				-- AND IdAgent = 'elin.engman'
			ORDER BY IdAgent, IdEvaluationPlan, IdInteraction, idauditor,  IdAbility

			-- Danny Jhonston:
			-- Inicio: Se anade mapeo a nombre de habilidades.
			SELECT @columns = COALESCE(@columns + ', ', '') + '[' + idAbility + '] VARCHAR(50)',
			@columnsName = COALESCE(@columnsName + ', ', '') + '''' + Title + ''''
			FROM (SELECT DISTINCT ER.idAbility, ER.AbilityOrder, A.Title FROM #EvaluationResult ER
			INNER JOIN Ability A ON ER.IdAbility = A.ID
			WHERE
				ER.Instance = @Instance
				AND ER.CreatedDate BETWEEN CONVERT(VARCHAR, @FechaDesde) + ' 00:00:00.000' AND CONVERT(VARCHAR, @FechaHasta) + ' 23:59:59.999'
				AND ER.IdEvaluationFormat = @IdEvaluationFormat) VW
			ORDER BY IdAbility
			PRINT '@columnsName: ' + @columnsName
			-- Danny Jhonston:
			-- Fin: Se anade mapeo a nombre de habilidades.


			PRINT '@columns: ' + @columns
			PRINT '@columnsName: ' + @columnsName
			----print 'Analiznado InteracionId: ' 
			----print ISNULL(@IdInteraction,'')
			----print 'columns '
			----print @columns

			CREATE TABLE #T1(
				Agente VARCHAR(50)
			)

			IF @columns IS NOT NULL AND @columns <> ''
			BEGIN
				EXEC('ALTER TABLE #T1 ADD ' + @columns)

				PRINT @columns

				----PRINT 'agregadas columnas a t1'
				----PRINT @columnsName

				----PRINT ('Ejecutando SQL ....')
				----PRINT ('INSERT INTO #T1 VALUES (''Agente'','+@columnsName+')')
				-- Agrego en la primer fila los titulos de las columnas

				CREATE TABLE #TableTempColumns(
					[name] VARCHAR(100)
				)
		
				-- Danny Jhonston:
				-- Inicio: Almaceno las columnas de la tabla temporal #T1 a excepcion de la columna Agente.
				INSERT INTO #TableTempColumns
				SELECT [name] FROM tempdb.sys.columns
				WHERE OBJECT_ID = OBJECT_ID('tempdb..#T1')
				-- Danny Jhonston:
				-- Fin: Almaceno las columnas de la tabla temporal #T1 a excepcion de la columna Agente.


				-- Danny Jhonston:
				-- Inicio: Para el error: Column name or number of supplied values does not match table definition, se decide obtener las columnas para anadirlas en la sentencia INSERT INTO #T.
				SET @columnsInsert = (SELECT STUFF((SELECT + ', [' + [name] + ']' FROM
				#TableTempColumns FOR XML PATH('')), 1, 1, ''))
				PRINT '@columnsInsert: ' + @columnsInsert
				-- Danny Jhonston:
				-- Fin: Para el error: Column name or number of supplied values does not match table definition, se decide obtener las columnas para anadirlas en la sentencia INSERT INTO #T.


				-- Agrego en la primera fila los titulos de las columnas.
				PRINT('INSERT INTO #T1 (' + @columnsInsert + ') VALUES (''Agente'',' + @columnsName + ')')
				EXEC('INSERT INTO #T1 (' + @columnsInsert + ') VALUES (''Agente'',' + @columnsName + ')')
	
				CREATE TABLE #PromediosAgente(
					Agente VARCHAR(50)
				)

				EXEC('ALTER TABLE #PromediosAgente ADD ' + @columns) -- Anade columnas.
				INSERT INTO #PromediosAgente (Agente) VALUES ('Promedios sobre X auditorias') -- Anade el texto para la columna.

				CREATE TABLE #PromediosGenerales(
					Agente VARCHAR(50)
				)

				EXEC('ALTER TABLE #PromediosGenerales ADD ' + @columns)
				INSERT INTO #PromediosGenerales (Agente) VALUES ('Promedios Generales')


				OPEN curP
				FETCH NEXT FROM curP INTO @idAgent, @CreatedDate, @qualification, @maxAbilityScore, @idHabilidad, @IdAuditor, @IdInteraction, @IdEvaluationPlan

				PRINT '@IdEvaluationPlan: ' + @IdEvaluationPlan

				SET @idAgentAnterior = @idAgent
				SET @CreatedDateAnterior = @CreatedDate
				SET @insertVar = ''''+@idAgent+''''
				SET @cantAuditoriasAgente = @cantAuditoriasAgente + 1
				SET @IdAuditorAnterior = @IdAuditor
				SET @IdInteractionAnterior = @IdInteraction
				SET @IdEvaluationPlanAnterior = @IdEvaluationPlan
				SET @ConsultaSQLsum = ''
		
				WHILE @@FETCH_STATUS = 0
				BEGIN
					IF @IdAuditorAnterior != @IdAuditor OR @IdInteractionAnterior != @IdInteraction OR @IdEvaluationPlanAnterior != @IdEvaluationPlan
					BEGIN
						--PRINT @columnsName
						--PRINT @insertVar
						PRINT 'insertvar: ' + ISNULL(@insertVar,'NULLL')
						--PRINT 'analizando 2do insert'
						PRINT ' '
						PRINT '@IdAuditor: ' + @IdAuditor + ' @IdEvaluationPlan: ' + @IdEvaluationPlan + ' @idHabilidad: ' + @idHabilidad

						-- Validar que la cantidad de columnas coincida con la cantidad de valores a insertar.
						PRINT ''
						PRINT '********'
						PRINT @columnsInsert
						PRINT @insertVar
						SET @columnsInsertLength = (SELECT COUNT([value]) FROM [dbo].[SplitString](@columnsInsert, ','))
						PRINT @columnsInsertLength

						SET @columnsValuesLength = (SELECT COUNT([value]) FROM [dbo].[SplitString](@insertVar, ','))
						PRINT @columnsValuesLength
						IF(@columnsInsertLength = @columnsValuesLength)
						BEGIN
							PRINT ('INSERT INTO #T1 (' + @columnsInsert + ') VALUES(' + @insertVar +')')
							EXEC ('INSERT INTO #T1 (' + @columnsInsert + ') VALUES('+ @insertVar +')')	
						END
						PRINT '********'
						PRINT ''
						SET @insertVar = '''' + @idAgent + ''''
						SET @cantAuditoriasAgente = @cantAuditoriasAgente + 1
					END
					PRINT @idAgentAnterior + ' - ' + @idAgent
					IF @idAgentAnterior!= @idAgent
					BEGIN
						PRINT 'XXXXXXXX'
						PRINT @idAgent
						-- Danny Jhonston:
						-- Inicio: Actualizar valores sumarizados sobre cantidad de auditorias de agente para obtener promedios.
						SET @ConsultaSQLsum = 'UPDATE #PromediosAgente SET ' + (SELECT STUFF((SELECT + ', [' + [name] + '] = ROUND(CONVERT(FLOAT, [' + [name] + ']) / CONVERT(FLOAT, ' + CONVERT(VARCHAR, (@cantAuditoriasAgente-1)) + '), 2)' FROM
												#TableTempColumns
												WHERE [name] NOT IN('Agente') FOR XML PATH('')), 1, 1, ''))
						PRINT '@ConsultaSQLsum: ' + @ConsultaSQLsum
						EXEC (@ConsultaSQLsum)
						-- Danny Jhonston:
						-- Fin: Actualizar valores sumarizados sobre cantidad de auditorias de agente para obtener promedios.

						IF @cantAuditoriasAgente-1 = 1
						BEGIN
							UPDATE #PromediosAgente SET
							Agente = Replace(Agente, 'X auditorias', CONVERT(VARCHAR, @cantAuditoriasAgente-1)+' auditoria')
						END
						ELSE
						BEGIN
							UPDATE #PromediosAgente SET
							Agente = Replace(Agente, 'X', CONVERT(VARCHAR, @cantAuditoriasAgente-1))
						END
						-- PRINT 'actualizados promedios'
						-- Insertamos sumarizacion agente.

						INSERT #T1 SELECT * FROM #PromediosAgente
						DELETE FROM #PromediosAgente
						INSERT INTO #PromediosAgente (Agente) VALUES ('Promedios sobre X auditorias')
						--PRINT 'insertado promedios'
						SET @cantAuditoriasAgente = 1
					END
				
					SET @CreatedDateAnterior = @CreatedDate
					SET @idAgentAnterior = @idAgent
					SET @IdAuditorAnterior = @IdAuditor
					SET @IdInteractionAnterior = @IdInteraction
					SET @IdEvaluationPlanAnterior = @IdEvaluationPlan
					SET @porcentajeAbilidad = (ISNULL(@qualification, 0) * 100.0) / HistoricalData.dbo.IsZero(ISNULL(@maxAbilityScore, 0), 0)
					PRINT '$$$$$$$$$$$$$$$$$'
					PRINT ''
					PRINT ''
					PRINT '@qualification: ' + CONVERT(VARCHAR, @qualification) + ' @porcentajeAbilidad: ' + CONVERT(VARCHAR, @porcentajeAbilidad) + ' @maxAbilityScore: ' + CONVERT(VARCHAR, @maxAbilityScore)
					PRINT ''
					PRINT ''
					-- PRINT 'actualizando promedios 2'
					DECLARE @ConsultaSQL1 VARCHAR(MAX)

					-- Danny Jhonston:
					-- Inicio: Actualizar porcentaje de habilidad para cada habilidad.
					SET @ConsultaSQL1 = 'UPDATE #PromediosAgente SET [' + @idHabilidad + '] = CAST(ISNULL([' + @idHabilidad + '], 0) AS FLOAT) +' + CONVERT(VARCHAR, @porcentajeAbilidad)			

					--SELECT @ConsultaSQL1 = 'UPDATE #PromediosAgente SET [' + @idHabilidad + '] = ' + 'CONVERT(VARCHAR, (IsNull(CONVERT(FLOAT, [' + @idHabilidad + ']), 0))) *'  
					--SELECT @ConsultaSQL2 = '(( ' + CONVERT(VARCHAR,@cantAuditoriasAgente) + ' + -1) +' + CONVERT(VARCHAR, @porcentajeAbilidad) + ')/ + ' + CONVERT(VARCHAR, @cantAuditoriasAgente)
					--SELECT @ConsultaSQL3 = @ConsultaSQL1 + ' ' + @ConsultaSQL2
					--print '@ConsultaSQL3: ' + @ConsultaSQL3
					--print '@ConsultaSQL1: ' + @ConsultaSQL1
			
					--print '____@idHabilidad: ' + @idHabilidad + ' @porcentajeAbilidad: '+  CONVERT(VARCHAR, @porcentajeAbilidad) + '@cantAuditoriasAgente: ' + CONVERT(VARCHAR,@cantAuditoriasAgente) + ' dummy: ' + CONVERT(VARCHAR,@dummy)
					PRINT ''
					PRINT ''
					PRINT ''
					PRINT '**********|||||||||'
					PRINT('@ConsultaSQL1: ' + @ConsultaSQL1)
					EXEC(@ConsultaSQL1)
					-- Danny Jhonston:
					-- Fin: Actualizar porcentaje de habilidad para cada habilidad.


					SET @insertVar = COALESCE(@insertVar + ', ', '') + '''' + CONVERT(VARCHAR, @porcentajeAbilidad) + ''''
					FETCH NEXT FROM curP INTO @idAgent, @CreatedDate, @qualification, @maxAbilityScore, @idHabilidad, @IdAuditor, @IdInteraction, @IdEvaluationPlan
					PRINT '_______________________' + CONVERT(VARCHAR, @@FETCH_STATUS)

					-- En la ultima iteracion ejecutar sintaxis para promediar.
					IF(@@FETCH_STATUS = -1)
					BEGIN
						-- Danny Jhonston:
						-- Inicio: Actualizar valores sumarizados sobre cantidad de auditorias de agente para obtener promedios.
						SET @ConsultaSQLsum = 'UPDATE #PromediosAgente SET ' + (SELECT STUFF((SELECT + ', [' + [name] + '] = ROUND(CONVERT(FLOAT, [' + [name] + ']) / CONVERT(FLOAT, ' + CONVERT(VARCHAR, (@cantAuditoriasAgente)) + '), 2)' FROM
												#TableTempColumns
												WHERE [name] NOT IN('Agente') FOR XML PATH('')), 1, 1, ''))
						PRINT '@ConsultaSQLsum: ' + @ConsultaSQLsum
						EXEC (@ConsultaSQLsum)
						-- Danny Jhonston:
						-- Fin: Actualizar valores sumarizados sobre cantidad de auditorias de agente para obtener promedios.
					END
				END -- END WHILE

				--PRINT ' @cantAuditoriasAgente: ' + CONVERT(VARCHAR, @cantAuditoriasAgente)

			-- Inicio de segundo row - promedio de notas.
			PRINT 'Insertando en t1 SQL: ' + ('INSERT INTO #T1 (' + @columnsInsert + ') VALUES('+ ISNULL(@insertVar, 'NULL') +')')

			EXEC ('INSERT INTO #T1 (' + @columnsInsert + ') VALUES (' + @insertVar  + ')' )

			-- PRINT 'insertados! en t1 2'
			IF @cantAuditoriasAgente = 1
			BEGIN
				UPDATE #PromediosAgente SET
				Agente = Replace(Agente, 'X auditorias', CONVERT(VARCHAR, @cantAuditoriasAgente) +' auditoria')
			END
			ELSE
			BEGIN
				UPDATE #PromediosAgente SET
				Agente = Replace(Agente, 'X', CONVERT(VARCHAR, @cantAuditoriasAgente))
			END

			INSERT #T1 SELECT * FROM #PromediosAgente
		CLOSE curP
		DEALLOCATE curP

		-- Danny Jhonston:
		-- Inicio: Se anade mapeo a nombre de habilidades para campos del resultSet.
		DECLARE @Headers VARCHAR(MAX)
		SET @Headers = (SELECT STUFF((SELECT + ', [' + A.[ID] + '] AS [' + A.[Title] + ']' FROM
		#TableTempColumns temp
		INNER JOIN Ability A ON temp.[name] = A.ID
		WHERE temp.[name] NOT IN('Agente') FOR XML PATH('')), 1, 1, ''))


		DECLARE @SQL VARCHAR(MAX)
		SET @SQL = '
			SELECT Agente, '
		SET @SQL += @Headers
		SET @SQL += ' FROM #T1
		'
		EXEC (@SQL)
		-- Danny Jhonston:
		-- Fin: Se anade mapeo a nombre de habilidades.
		END
	END
	ELSE
	BEGIN
		SELECT '-1' AS result
	END
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ReportByBatchDetailAgent] (
	@VCC varchar(50) = NULL, 
	@HistoricCampaign varchar(MAX) = NULL , 
	@BatchPlan varchar(MAX) = NULL,
	@BatchFormat varchar(MAX), 
	@StartDate DateTime = NULL , 
	@EndDate DateTime = NULL , 
	@IdAuditor varchar(MAX) = NULL ,
	@IdAgente varchar(MAX) = NULL
) AS
BEGIN

	-- ReportByDetailAgent 'allegro','entrantes','Plan_2_nov01-0356','nov01-0356','20191001','20191201','isidoro','icagente3'
	SET NOCOUNT ON

	DECLARE @cols varchar(max) 
	DECLARE @query varchar(max)

	-- SETTEAMOS EL STARTDATE Y ENDDATE EN CASO SEAN NULL
	IF @StartDate IS NULL SET @StartDate = '2021-01-01 00:00:00'
	IF @EndDate IS NULL SET @EndDate = '2050-01-01 23:59:59' 
	ELSE 
	BEGIN
		SET @EndDate = DateAdd(hh,- (DatePart(hh, @EndDate)) + 23, @EndDate) 
		SET @EndDate = DateAdd(mi,- (DatePart(mi, @EndDate)) + 59, @EndDate) 
		SET @EndDate = DateAdd(ss,- (DatePart(ss, @EndDate)) + 59, @EndDate) 
	END

	-- INSERTAMOS LAS CAMPAIGN A UNA TABLA TEMPORAL
	DECLARE @_CAMPAIGN_IDS TABLE(ID VARCHAR(50))
	INSERT INTO @_CAMPAIGN_IDS SELECT [Value] FROM [dbo].[Split](@HistoricCampaign, ',')

	-- INSERTAMOS LOS PLANS A UNA TABLA TEMPORAL
	DECLARE @_EVALUATION_PLAN_IDS TABLE(ID VARCHAR(50))
	INSERT INTO @_EVALUATION_PLAN_IDS SELECT [Value] FROM [dbo].[Split](@BatchPlan, ',')

	-- INSERTAMOS LOS AUDITORS A UNA TABLA TEMPORAL
	DECLARE @_AUDITOR_IDS TABLE(ID VARCHAR(50))
	INSERT INTO @_AUDITOR_IDS SELECT [Value] FROM [dbo].[Split](@IdAuditor, ',')

	-- INSERTAMOS LOS AGENTS A UNA TABLA TEMPORAL
	DECLARE @_AGENT_IDS TABLE(ID VARCHAR(50))
	INSERT INTO @_AGENT_IDS SELECT [Value] FROM [dbo].[Split](@IdAgente, ',')

    -- SEGMENTAMOS LA DATA ANTES DE HACER LOS JOIN
	SELECT ER.*,
			EP.[Description],
			EP.[Evaluation_Mode]
		INTO #TMP_ER
		FROM [dbo].[EvaluationResult] (NOLOCK) ER
		INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
			ON EP.[Instance] = ER.[Instance]
				AND EP.[ID] = ER.[IdEvaluationPlan]
		WHERE ER.Instance = ISNULL(@VCC, ER.Instance)
			AND ER.[IdEvaluationPlan] IN (SELECT [ID] FROM @_EVALUATION_PLAN_IDS)
			AND ER.EvaluationDate >= ISNULL(@StartDate, '19800101')
			AND ER.EvaluationDate < IIF(@EndDate IS NOT NULL, @EndDate + ' 23:59', '20500101')

        -- AGREGAMOS INDICES A LA TABLA TEMPORAL
        CREATE NONCLUSTERED INDEX [TMP_ER_01] ON #TMP_ER ([Instance], [IdEvaluationPlan], [IdInteraction], [IdAuditor])
        CREATE NONCLUSTERED INDEX [TMP_ER_02] ON #TMP_ER ([Instance], [IdEvaluationFormat], [IdEvaluationPlan])
        CREATE NONCLUSTERED INDEX [TMP_ER_03] ON #TMP_ER ([Instance], [IdAbility], [IdEvaluationPlan])
        CREATE NONCLUSTERED INDEX [TMP_ER_04] ON #TMP_ER ([Instance], [IdEvaluationPlan], [IdGroup])
        CREATE NONCLUSTERED INDEX [TMP_ER_05] ON #TMP_ER ([Instance], [IdEvaluationPlan], [IdSubGroup])


	SELECT * 
		INTO #TempEvaluationResult
		FROM (
			SELECT ER.[Instance]														'VCC',
					ER.[IdEvaluationPlan]												'IdPlan',
					ER.[Description]													'DescriptionPlan',
					EF.[Description]													'DescriptionFormat',
					SUBSTRING((
						GP.[Description]
						+ '-' 
						+ SGP.[Description]
						+ '-' 
						+ A.[Title]
					), 1, 128)															'Ability', --Collate SQL_Latin1_General_CP1253_CI_AI
					ER.[AgentDescription],
					ER.[CampaignDescription],
					ER.[IdInteraction],
					ISNULL(ER.[Qualification], 0)										'Qualification',
					AEP.[Score]															'Score',
					AEP.[AutomaticScore],
					AEP.[ManualScore],
					--Format(ER.EvaluationDate, 'dd/MM/yyyy hh:mm:ss')					AS 'EvaluationDate',
					--CAST(Format(AEP.[TimeStamp], 'yyyy-MM-dd hh:mm:ss') AS DATETIME)					AS 'EvaluationDate',
					--AEP.[TimeStamp] AT TIME ZONE 'SA Pacific Standard Time' AS 'EvaluationDate',
					AEP.[TimeStamp] AS 'EvaluationDate',
					AEP.[InteractionDate]												'StartDate',
					ER.[IdAuditor],
					ER.[Evaluation_Mode],
					AEP.[EvaluationTime],
					RN = ROW_NUMBER() OVER(PARTITION BY EP.[Instance], EP.[Description], ER.[AgentDescription], ER.[CampaignDescription], ER.[IdInteraction] ORDER BY ER.[IdInteraction])
				FROM #TMP_ER ER (NOLOCK)
				INNER JOIN dbo.[HistoricEvaluationFormat] EF
					ON EF.[VCC] = ER.[Instance]
						AND EF.[EvaluationPlanId] = ER.[IdEvaluationPlan] 
						AND EF.ID = ER.[IdEvaluationFormat] 
				INNER JOIN dbo.[HistoricAbility] (NOLOCK) A
					ON A.[VCC] = ER.[Instance]
						AND A.[EvaluationPlanId] = ER.[IdEvaluationPlan]  
						AND A.[ID] = ER.[IdAbility] 
				INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
					ON AEP.[Instance] = ER.[Instance]
						AND AEP.[IdEvaluationPlan] =  ER.[IdEvaluationPlan]
						AND AEP.[IdInteraction] = ER.[IdInteraction]
						AND AEP.[IdAuditor] =  ER.[IdAuditor]
				INNER JOIN [HistoricGroup] (NOLOCK) GP
					ON GP.[VCC] = ER.[Instance]
						AND GP.[EvaluationPlanId] = ER.[IdEvaluationPlan] 
						AND GP.[ID] = ER.[IdGroup] 
				INNER JOIN dbo.[HistoricSubGroup] (NOLOCK) SGP
					ON SGP.[VCC] = ER.[Instance]
						AND SGP.[EvaluationPlanId] = ER.[IdEvaluationPlan] 
						AND SGP.[ID] = ER.[IdSubGroup] 
				WHERE AEP.[TimeStamp] BETWEEN @StartDate AND  @EndDate
					AND AEP.[Completed] = 1
					AND AEP.[StatusDescription] NOT IN('INVALID')
					AND ER.[Instance] LIKE CASE WHEN @VCC IS NOT NULL THEN @VCC ELSE ER.[Instance] END
					AND ER.[AgentDescription] IN (SELECT [ID] FROM @_AGENT_IDS)
					AND ER.[IdAuditor] IN (SELECT [ID] FROM @_AUDITOR_IDS)
					AND ER.[CampaignDescription] IN (SELECT [ID] FROM @_CAMPAIGN_IDS)
					AND EF.[ID] IN (SELECT [Value] FROM [dbo].[Split](ISNULL(@BatchFormat, EF.[ID]), ','))
					--AND EP.[Status] = 7
		) A
		WHERE A.[RN] = 1

	-- print 'Despues del select'

	--select * from #TempEvaluationResult ORDER BY IdInteraction

	SET @cols = STUFF(
		(
			SELECT (',' + QUOTENAME(TMP_ER.[Ability]))
				FROM #TempEvaluationResult TMP_ER
				GROUP BY TMP_ER.[Ability] 
				ORDER BY TMP_ER.[Ability] 
			FOR xml PATH (''), TYPE
		).value('.', 'varchar(Max)')
		, 1, 1, '')

  --Select @cols

  IF @cols IS NOT NULL
  BEGIN
	DECLARE @EXECUTION_TYPE VARCHAR(10) = (SELECT [ExecutionType] FROM [dbo].[EvaluationFormat] (NOLOCK) WHERE [Id] = @BatchFormat)
    DECLARE @type varchar(max)=(Select Top 1 Evaluation_Mode From #TempEvaluationResult)
    DECLARE @Total varchar(max) = STUFF(
		(
			SELECT '+ ISNULL(' + QUOTENAME(TMP_ER.[Ability]) + ',0)'
				FROM #TempEvaluationResult TMP_ER
				GROUP BY TMP_ER.[Ability] 
				ORDER BY TMP_ER.[Ability] 
			FOR xml PATH (''), TYPE
		).value('.', 'varchar(Max)')
		, 1, 1, '')

    If @type = 'M' OR @EXECUTION_TYPE = 'REALTIME'
	BEGIN
		print 'M'
		SET @query = ' 
				SELECT p.[VCC]																	[VCC],
						p.[DescriptionPlan]														[EVALUATION_PLAN],
						p.[DescriptionFormat]													[EVALUATION_FORMAT],
						p.[AgentDescription]													[AGENT],
  						p.[CampaignDescription]													[CAMPAIGN],
						p.[IdInteraction]														[INTERACTION],
						p.[EvaluationDate]														[EVALUATION_DATE],
						p.[IdAuditor]															[AUDITOR],
						[InteractionDuration]													[DURATION_TIME], 
						NULL																	[PHONE],
						--[InteractionDateStart] AT TIME ZONE ''SA Pacific Standard Time''		[START_DATE],
						p.[StartDate]															[START_DATE],
						NULL																	[CATEGORY],
						-- (' + @Total + ')														[COLUMN_TOTAL_SCORE], 
						p.[Score]																[COLUMN_TOTAL_SCORE], 
						p.[AutomaticScore]														[COLUMN_AUTOMATIC_SCORE],
						p.[ManualScore]															[COLUMN_MANUAL_SCORE],
						' + @cols + ',
						RN = ROW_NUMBER() OVER(PARTITION BY p.[VCC], p.[DescriptionPlan], p.[AgentDescription], p.[CampaignDescription], p.[IdInteraction] ORDER BY p.[IdInteraction])
					INTO [#TMP_REPORT]
					FROM 
				( select VCC,
							IdPlan,
							DescriptionPlan,
							DescriptionFormat,
							Ability,
							AgentDescription,
							CampaignDescription,
							IdInteraction,
							EvaluationDate,
							Qualification,
							Score,
							AutomaticScore,
							ManualScore,
							IdAuditor,
							StartDate,
							EvaluationTime
						from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 

				Left Join (
					Select Distinct [IdEvaluationPlan],
									IdInteraction,
									InteractionDuration,
									InteractionDateStart
									,RN = ROW_NUMBER() OVER(PARTITION BY [Instance], [IdEvaluationPlan], [IdInteraction] ORDER BY  [InteractionSection], [InteractionPart] ASC)
						FROM [dbo].[InteractionEvaluationPlan] (Nolock) 
				) as A 
					On (
							A.[IdEvaluationPlan] = p.[DescriptionPlan]
							OR
							A.[IdEvaluationPlan] = p.[IdPlan]
						)
						AND A.IdInteraction = p.IdInteraction 
						AND A.[RN] = 1 
			SELECT * FROM [#TMP_REPORT] WHERE [RN] = 1 '
    END
	ELSE
	BEGIN
		SET @query = ' 
				Select p.[VCC]							[VCC],
						p.[DescriptionPlan]				[EVALUATION_PLAN],
						p.[DescriptionFormat]			[EVALUATION_FORMAT],
						p.[AgentDescription]			[AGENT],
  						p.[CampaignDescription]			[CAMPAIGN] ,
						p.[IdInteraction]				[INTERACTION] ,
						p.[EvaluationDate]				[EVALUATION_DATE],
						p.[IdAuditor]					[AUDITOR],
						A.[DurationTime]				[DURATION_TIME],
						A.[Number]						[PHONE] ,
						p.[StartDate]					[START_DATE] ,
						A.[Category1]					[CATEGORY] ,
						-- (' + @Total + ')				[COLUMN_TOTAL_SCORE], 
						p.[Score]						[COLUMN_TOTAL_SCORE], 
						p.[AutomaticScore]				[COLUMN_AUTOMATIC_SCORE],
						p.[ManualScore]					[COLUMN_MANUAL_SCORE],
						p.[EvaluationTime]				[COLUMN_EVALUATION_TIME],
						' + @cols + ',
						RN = ROW_NUMBER() OVER(PARTITION BY p.[VCC], p.[DescriptionPlan], p.[AgentDescription], p.[CampaignDescription], p.[IdInteraction] ORDER BY p.[IdInteraction]) 
					from ( 
						select VCC,
								DescriptionPlan,
								DescriptionFormat,
								Ability,
								AgentDescription,
								CampaignDescription,
								IdInteraction,
								EvaluationDate,
								Qualification,
								Score,
								AutomaticScore,
								ManualScore,
								IdAuditor,
								StartDate,
								EvaluationTime
							from #TempEvaluationResult
					) x pivot 
					( 
						 max(Qualification)
						for Ability in (' + @cols + ')
					) p 
					LEFT Join Audio as A 
						On A.[VCC] = p.[VCC]
							AND A.ID = p.IdInteraction 
			SELECT * FROM [#TMP_REPORT] WHERE [RN] = 1'
	END

	--Print(@query)
    EXECUTE (@query)
    DROP TABLE IF EXISTS [#TempEvaluationResult]
	DROP TABLE IF EXISTS [#TMP_REPORT]
  END
  ELSE
  BEGIN
    SELECT
						''				As [EVALUATION_PLAN],
						''				As [EVALUATION_FORMAT],
						''				As [AGENT],
  						''			    As [CAMPAIGN] ,
						''			 	As [INTERACTION] ,
						''				As [EVALUATION_DATE],
						''				As [AUDITOR]

  END

	-- ELIMINARMOS LA TABLA TEMPORAL
	DROP TABLE IF EXISTS #TMP_ER

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ReportByBatchSummaryAgent] (
	@VCC varchar(50) = NULL, 
	@HistoricCampaign varchar(MAX) = NULL, 
	@BatchPlan varchar(MAX) = NULL,
	@BatchFormat varchar(MAX) , 
	@StartDate DateTime = NULL , 
	@EndDate DateTime = NULL , 
	@IdAuditor varchar(MAX) = NULL ,
	@IdAgente varchar(MAX) = NULL
) AS
  DECLARE @cols varchar(max)
  DECLARE @query varchar(max)
  --ReportBySummaryAgent 'allegro','entrantes',NULL,'nov01-0356',NULL,NULL,NULL,NULL
  --SET @Format = 'nov01-0356'
  SET NOCOUNT ON

	-- INSERTAMOS LAS CAMPAIGN A UNA TABLA TEMPORAL
	DECLARE @_CAMPAIGN_IDS TABLE(ID VARCHAR(50))
	INSERT INTO @_CAMPAIGN_IDS SELECT [Value] FROM [dbo].[Split](@HistoricCampaign, ',')

	-- INSERTAMOS LOS PLANS A UNA TABLA TEMPORAL
	DECLARE @_EVALUATION_PLAN_IDS TABLE(ID VARCHAR(50))
	INSERT INTO @_EVALUATION_PLAN_IDS SELECT [Value] FROM [dbo].[Split](@BatchPlan, ',')

	-- INSERTAMOS LOS FORMATS A UNA TABLA TEMPORAL
	DECLARE @_EVALUATION_FORMAT_IDS TABLE(ID VARCHAR(50))
	INSERT INTO @_EVALUATION_FORMAT_IDS SELECT [Value] FROM [dbo].[Split](@BatchFormat, ',')

	-- INSERTAMOS LOS AUDITORS A UNA TABLA TEMPORAL
	DECLARE @_AUDITOR_IDS TABLE(ID VARCHAR(50))
	INSERT INTO @_AUDITOR_IDS SELECT [Value] FROM [dbo].[Split](@IdAuditor, ',')

	-- INSERTAMOS LOS AGENTS A UNA TABLA TEMPORAL
	DECLARE @_AGENT_IDS TABLE(ID VARCHAR(50))
	INSERT INTO @_AGENT_IDS SELECT [Value] FROM [dbo].[Split](@IdAgente, ',')

  -- SEGMENTAMOS LA DATA ANTES DE HACER LOS JOIN
  SELECT ER.*,
			EP.[Description],
			EP.[Evaluation_Mode]
	INTO #TMP_ER
	FROM [dbo].[EvaluationResult] (NOLOCK) ER
	INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
		ON EP.[Instance] = ER.[Instance]
			AND EP.[ID] = ER.[IdEvaluationPlan]
	WHERE ER.Instance = ISNULL(@VCC, ER.Instance)
		AND ER.[IdEvaluationPlan] IN(SELECT [Value] FROM [dbo].[Split](IIF(@BatchPlan IS NULL,EP.Description, @BatchPlan),','))
		AND ER.EvaluationDate >= ISNULL(@StartDate, '19800101')
		AND ER.EvaluationDate < IIF(@EndDate IS NOT NULL, @EndDate + ' 23:59', '20500101')
	
	-- AGREGAMOS INDICES A LA TABLA TEMPORAL
	CREATE NONCLUSTERED INDEX [TMP_ER_01] ON #TMP_ER ([Instance], [IdEvaluationPlan], [IdInteraction], [IdAuditor])
	CREATE NONCLUSTERED INDEX [TMP_ER_02] ON #TMP_ER ([Instance], [IdEvaluationFormat], [IdEvaluationPlan])
	CREATE NONCLUSTERED INDEX [TMP_ER_03] ON #TMP_ER ([Instance], [IdAbility], [IdEvaluationPlan])
	CREATE NONCLUSTERED INDEX [TMP_ER_04] ON #TMP_ER ([Instance], [IdEvaluationPlan], [IdGroup])
	CREATE NONCLUSTERED INDEX [TMP_ER_05] ON #TMP_ER ([Instance], [IdEvaluationPlan], [IdSubGroup])
  
  SELECT
    * INTO #TempEvaluationResult
  FROM (
  SELECT
    ER.Description AS 'DescriptionPlan',
    EF.Description AS 'DescriptionFormat',
    SUBSTRING((GP.Description + '-' + SGP.Description + '-' + A.Title), 1, 119) AS 'Ability', -- RESTAMOS 128-9 PORQUE MAS ADELANTA CONCATENA " PROMEDIO"
    ER.AgentDescription,
    ER.CampaignDescription,
    ER.IdInteraction,
    ER.Qualification,
    Format(ER.EvaluationDate, 'dd/MM/yyyy hh:mm:ss') AS 'EvaluationDate',
    ER.IdAuditor,
    ER.Evaluation_Mode,
	AEP.[EvaluationTime]
  FROM #TMP_ER ER (NOLOCK) 
  INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
					ON AEP.[Instance] = ER.[Instance]
						AND AEP.[IdEvaluationPlan] =  ER.[IdEvaluationPlan]
						AND AEP.[IdInteraction] = ER.[IdInteraction]
						AND AEP.[IdAuditor] =  ER.[IdAuditor]
  INNER  JOIN HistoricEvaluationFormat (NOLOCK) EF
    ON EF.[VCC] = ER.[Instance]
		AND ER.IdEvaluationFormat = EF.ID 
		AND ER.IdEvaluationPlan = EF.EvaluationPlanId
  INNER  JOIN HistoricAbility (NOLOCK) A
    ON A.[VCC] = ER.[Instance]
		AND A.ID = ER.IdAbility 
		AND A.EvaluationPlanId = ER.IdEvaluationPlan 
  INNER  JOIN EvaluationPlan (NOLOCK) EP
    ON EP.ID = ER.IdEvaluationPlan
  INNER  JOIN [HistoricGroup] (NOLOCK) GP
	ON GP.[VCC] = ER.[Instance]
		AND GP.ID = ER.IdGroup 
		AND GP.EvaluationPlanId = EP.ID
  INNER  JOIN HistoricSubGroup SGP (NOLOCK)
	ON SGP.[VCC] = ER.[Instance]
		AND SGP.ID = ER.IdSubGroup 
		AND SGP.EvaluationPlanId = EP.ID
	WHERE ( 
			(
				ISNULL(LEN(@BatchPlan), 0) <> 0
				AND
				ER.[IdEvaluationPlan] IN (SELECT [ID] FROM @_EVALUATION_PLAN_IDS)
			)	
			OR
			(
				ISNULL(LEN(@BatchPlan), 0) = 0
			)
		)
		AND ( 
			(
				ISNULL(LEN(@BatchFormat), 0) <> 0
				AND
				ER.[IdEvaluationFormat] IN (SELECT [ID] FROM @_EVALUATION_FORMAT_IDS)
			)	
			OR
			(
				ISNULL(LEN(@BatchFormat), 0) = 0
			)
		)
		AND ( 
			(
				ISNULL(LEN(@HistoricCampaign), 0) <> 0
				AND
				ER.[CampaignDescription] IN (SELECT [ID] FROM @_CAMPAIGN_IDS)
			)	
			OR
			(
				ISNULL(LEN(@HistoricCampaign), 0) = 0
			)
		)
		AND (
			(
				ISNULL(LEN(@IdAgente), 0) <> 0
				AND
				ER.[AgentDescription] IN (SELECT [ID] FROM @_AGENT_IDS)
			)
			OR
			(
				ISNULL(LEN(@IdAgente), 0) = 0
			)
		)
		AND (
			(
				ISNULL(LEN(@IdAuditor), 0) <> 0
				AND
				ER.[IdAuditor] IN (SELECT [ID] FROM @_AUDITOR_IDS)
			)
			OR
			(
				ISNULL(LEN(@IdAuditor), 0) = 0
			)
		)
	) a

	
	SET @cols = STUFF(
		(
			SELECT (',' + QUOTENAME(TMP_ER.[Ability]))
				FROM #TempEvaluationResult TMP_ER
				GROUP BY TMP_ER.[Ability] 
				ORDER BY TMP_ER.[Ability] 
			FOR xml PATH (''), TYPE
		).value('.', 'varchar(Max)')
		, 1, 1, '')
  
  IF @cols IS NOT NULL
  BEGIN
    ------
    DECLARE @type varchar(max) = (SELECT TOP 1
      Evaluation_Mode
    FROM #TempEvaluationResult)
	
	-------
    DECLARE @Total varchar(max) = STUFF(
		(
			SELECT '+ ISNULL(' + QUOTENAME(TMP_ER.[Ability]) + ',0)'
				FROM #TempEvaluationResult TMP_ER
				GROUP BY TMP_ER.[Ability] 
				ORDER BY TMP_ER.[Ability] 
			FOR xml PATH (''), TYPE
		).value('.', 'varchar(Max)')
		, 1, 1, '')    
	
	-- ROUND(AVG(CAST(ER.Qualification AS float)), 2) AS 'Qualification',
    DECLARE @colavg varchar(max) = STUFF(
		(
			SELECT ', ROUND(AVG(CAST(q.' + QUOTENAME(TMP_ER.[Ability]) + ' AS float)), 2) As [' + TMP_ER.[Ability] + ' Promedio]'
				FROM #TempEvaluationResult TMP_ER
				GROUP BY TMP_ER.[Ability] 
				ORDER BY TMP_ER.[Ability] 
			FOR xml PATH (''), TYPE
		).value('.', 'varchar(Max)')
		, 1, 1, '')
	
	--SELECT @colavg
    IF @type = 'M'
    BEGIN
      SET @query = ' 
				Select 
				q.DescriptionPlan As   [EVALUATION_PLAN] ,
				q.DescriptionFormat As [EVALUATION_FORMAT] ,
				q.AgentDescription As  [AGENT] ,
				q.CampaignDescription As [CAMPAIGN] ,
				count(q.IdInteraction) As [AUDITS],
				sum(q.DurationTime) As [DURATION_TIME],
				ROUND(AVG(CAST(q.Note AS float)), 2) As [AVERAGE_SCORE],
				ROUND(AVG(CAST(q.EvaluationTime AS float)), 2) As [AVERAGE_EVALUATION_TIME],
				' + @colavg + ' From
				(Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,p.EvaluationTime,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,InteractionDuration as DurationTime, 
				null As Number,InteractionDateStart As StartDate,null As Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor,
						  EvaluationTime			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join (
				Select Distinct IdInteraction,InteractionDuration,InteractionDateStart From InteractionEvaluationPlan (Nolock)
				) as A On
				A.IdInteraction=p.IdInteraction)q
				Group By 
				q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription
				'
    END
    ELSE
    BEGIN
      SET @query = '
	
			    Select 
				q.DescriptionPlan As   [EVALUATION_PLAN] ,
				q.DescriptionFormat As [EVALUATION_FORMAT] ,
				q.AgentDescription As  [AGENT] ,
				q.CampaignDescription As [CAMPAIGN] ,
			    count(q.IdInteraction) As [AUDITS],
				sum(q.DurationTime) As [DURATION_TIME],
				ROUND(AVG(CAST(q.Note AS float)), 2) As [AVERAGE_SCORE],
				ROUND(AVG(CAST(q.EvaluationTime AS float)), 2) As [AVERAGE_EVALUATION_TIME],
				' + @colavg + '
				From 
				(Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,p.EvaluationTime,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,
				A.DurationTime,A.Number,A.StartDate,A.Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor,
						  EvaluationTime			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join Audio as A On
				A.ID=p.IdInteraction)q
				Group By 
				q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription'
    END
    --Print(@query)
    EXECUTE (@query)
    --Select * From @query
	

    DROP TABLE #TempEvaluationResult
  END
  ELSE
  BEGIN
    DROP TABLE #TempEvaluationResult
    SELECT
						''				As [EVALUATION_PLAN],
						''				As [EVALUATION_FORMAT],
						''				As [AGENT],
  						''			    As [CAMPAIGN] 
  END

	-- ELIMINARMOS LA TABLA TEMPORAL
	DROP TABLE IF EXISTS #TMP_ER
  
  SET NOCOUNT OFF
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ReportByRealtimeDetailAgent] (
	@VCC varchar(50) = NULL, 
	@HistoricCampaign varchar(MAX) = NULL,
	@RealtimeFormat varchar(MAX), 
	@StartDate DateTime = NULL , 
	@EndDate DateTime = NULL , 
	@IdAgente varchar(MAX) = NULL
) AS
BEGIN

	-- ReportByDetailAgent 'allegro','entrantes','Plan_2_nov01-0356','nov01-0356','20191001','20191201','isidoro','icagente3'
	SET NOCOUNT ON

	DECLARE @cols varchar(max) 
	DECLARE @query varchar(max)

	-- SETTEAMOS EL STARTDATE Y ENDDATE EN CASO SEAN NULL
	IF @StartDate IS NULL SET @StartDate = '2021-01-01 00:00:00'
	IF @EndDate IS NULL SET @EndDate = '2050-01-01 23:59:59' 
	ELSE 
	BEGIN
		SET @EndDate = DateAdd(hh,- (DatePart(hh, @EndDate)) + 23, @EndDate) 
		SET @EndDate = DateAdd(mi,- (DatePart(mi, @EndDate)) + 59, @EndDate) 
		SET @EndDate = DateAdd(ss,- (DatePart(ss, @EndDate)) + 59, @EndDate) 
	END

	SELECT * 
		INTO #TempEvaluationResult
		FROM (
			SELECT EP.[Instance]														'VCC',
					EP.[ID]																'IdPlan',
					EP.[Description]													'DescriptionPlan',
					EF.[Description]													'DescriptionFormat',
					A.[Title]															'Ability', --Collate SQL_Latin1_General_CP1253_CI_AI
					ER.[AgentDescription],
					ER.[CampaignDescription],
					ER.[IdInteraction],
					ISNULL(ER.[Qualification], 0)										'Qualification',
					ER.[ScoreDescription]												'Score',
					--AEP.[Score]															'Score',
					--AEP.[AutomaticScore],
					--AEP.[ManualScore],
					AEP.[TimeStamp] AS 'EvaluationDate',
					ER.[IdAuditor],
					EP.[Evaluation_Mode]
				FROM dbo.[EvaluationResult] (NOLOCK) ER
				INNER JOIN dbo.[HistoricEvaluationFormat] EF
					ON EF.[VCC] = ER.[Instance]
						AND EF.[EvaluationPlanId] = ER.[IdEvaluationPlan] 
						AND EF.ID = ER.[IdEvaluationFormat] 
				INNER JOIN dbo.[HistoricAbility] (NOLOCK) A
					ON A.[VCC] = ER.[Instance]
						AND A.[EvaluationPlanId] = ER.[IdEvaluationPlan]  
						AND A.[ID] = ER.[IdAbility] 
				INNER JOIN dbo.[EvaluationPlan] (NOLOCK) EP
					ON EP.[Instance] = ER.[Instance]
						AND EP.[ID] = ER.[IdEvaluationPlan]
				INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
					ON AEP.[Instance] = ER.[Instance]
						AND AEP.[IdEvaluationPlan] =  ER.[IdEvaluationPlan]
						AND AEP.[IdInteraction] = ER.[IdInteraction]
						AND AEP.[IdAuditor] =  ER.[IdAuditor]
				WHERE AEP.[TimeStamp] BETWEEN @StartDate AND  @EndDate
					AND AEP.[Completed] = 1
					AND AEP.[StatusDescription] NOT IN('INVALID')
					AND ER.[Instance] LIKE CASE WHEN @VCC IS NOT NULL THEN @VCC ELSE ER.[Instance] END
					AND ER.[AgentDescription] IN (SELECT [Value] FROM [dbo].[Split](ISNULL(@IdAgente, ER.[AgentDescription]), ','))
					AND ER.[CampaignDescription] IN (SELECT [Value] FROM [dbo].[Split](ISNULL(@HistoricCampaign, ER.[CampaignDescription]), ','))
					AND EF.[ID] IN (SELECT [Value] FROM [dbo].[Split](ISNULL(@RealtimeFormat, EF.[ID]), ','))
					--AND EP.[Status] = 7
		) A

	-- print 'Despues del select'

	--select * from #TempEvaluationResult ORDER BY IdInteraction

	SET @cols = STUFF(
		(
			SELECT (',' + QUOTENAME(TMP_ER.[Ability]))
				FROM #TempEvaluationResult TMP_ER
				GROUP BY TMP_ER.[Ability] 
				ORDER BY TMP_ER.[Ability] 
			FOR xml PATH (''), TYPE
		).value('.', 'varchar(Max)')
		, 1, 1, '')

  --Select @cols

  IF @cols IS NOT NULL
  BEGIN
	DECLARE @EXECUTION_TYPE VARCHAR(10) = (SELECT [ExecutionType] FROM [dbo].[EvaluationFormat] (NOLOCK) WHERE [Id] = @RealtimeFormat)
    DECLARE @type varchar(max)=(Select Top 1 Evaluation_Mode From #TempEvaluationResult)
    DECLARE @Total varchar(max) = STUFF(
		(
			SELECT '+ ISNULL(' + QUOTENAME(TMP_ER.[Ability]) + ',0)'
				FROM #TempEvaluationResult TMP_ER
				GROUP BY TMP_ER.[Ability] 
				ORDER BY TMP_ER.[Ability] 
			FOR xml PATH (''), TYPE
		).value('.', 'varchar(Max)')
		, 1, 1, '')

    SET @query = ' 
				SELECT p.[VCC]							[VCC],
						p.[DescriptionPlan]				[EVALUATION_PLAN],
						p.[DescriptionFormat]			[EVALUATION_FORMAT],
						p.[AgentDescription]			[AGENT],
  						p.[CampaignDescription]			[CAMPAIGN],
						p.[IdInteraction]				[INTERACTION],
						p.[EvaluationDate]				[EVALUATION_DATE],
						p.[IdAuditor]					[AUDITOR],
						[InteractionDuration]			[DURATION_TIME], 
						NULL							[PHONE],
						[InteractionDateStart]			[START_DATE],
						NULL							[CATEGORY],
						-- (' + @Total + ')				[COLUMN_TOTAL_SCORE], 
						-- p.[Score]						[COLUMN_TOTAL_SCORE], 
						-- p.[AutomaticScore]				[COLUMN_AUTOMATIC_SCORE],
						-- p.[ManualScore]					[COLUMN_MANUAL_SCORE],
						' + @cols + ' 
					FROM 
				( select VCC,
							IdPlan,
							DescriptionPlan,
							DescriptionFormat,
							Ability,
							AgentDescription,
							CampaignDescription,
							IdInteraction,
							EvaluationDate,
							Qualification,
							Score,
							--AutomaticScore,
							--ManualScore,
							IdAuditor	
						from #TempEvaluationResult
			   ) x
				pivot 
				( 
					--max(Qualification)
					MAX(SCORE)
					for Ability in (' + @cols + ')
				) p 

				Left Join (
					Select Distinct [IdEvaluationPlan],
									IdInteraction,
									InteractionDuration,
									InteractionDateStart
									,RN = ROW_NUMBER() OVER(PARTITION BY [Instance], [IdEvaluationPlan], [IdInteraction] ORDER BY  [InteractionSection], [InteractionPart] ASC)
						FROM [dbo].[InteractionEvaluationPlan] (Nolock) 
				) as A 
					On (
							A.[IdEvaluationPlan] = p.[DescriptionPlan]
							OR
							A.[IdEvaluationPlan] = p.[IdPlan]
						)
						AND A.IdInteraction = p.IdInteraction 
						AND A.[RN] = 1 '

	--Print(@query)
    EXECUTE (@query)

    DROP TABLE #TempEvaluationResult
  END
  ELSE
  BEGIN
    SELECT
						''				As [EVALUATION_PLAN],
						''				As [EVALUATION_FORMAT],
						''				As [AGENT],
  						''			    As [CAMPAIGN] ,
						''			 	As [INTERACTION] ,
						''				As [EVALUATION_DATE],
						''				As [AUDITOR]

  END

  SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ReportByRealtimeSummaryAgent] (
	@VCC varchar(50) = NULL, 
	@HistoricCampaign varchar(MAX) = NULL, 
	@RealtimeFormat varchar(MAX) , 
	@StartDate DateTime = NULL , 
	@EndDate DateTime = NULL , 
	@IdAuditor varchar(MAX) = NULL ,
	@IdAgente varchar(MAX) = NULL
) AS
  DECLARE @cols varchar(max)
  DECLARE @query varchar(max)
  --ReportBySummaryAgent 'allegro','entrantes',NULL,'nov01-0356',NULL,NULL,NULL,NULL
  --SET @Format = 'nov01-0356'
  SET NOCOUNT ON
  
  SELECT
    * INTO #TempEvaluationResult
  FROM (
  SELECT
    EP.Description AS 'DescriptionPlan',
    EF.Description AS 'DescriptionFormat',
    A.[Title] AS 'Ability',
    ER.AgentDescription,
    ER.CampaignDescription,
    ER.IdInteraction,
    ER.Qualification,
    Format(ER.EvaluationDate, 'dd/MM/yyyy hh:mm:ss') AS 'EvaluationDate',
    ER.IdAuditor,
    EP.Evaluation_Mode,
	AEP.[EvaluationTime]
  FROM EvaluationResult (NOLOCK) ER
  INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
					ON AEP.[Instance] = ER.[Instance]
						AND AEP.[IdEvaluationPlan] =  ER.[IdEvaluationPlan]
						AND AEP.[IdInteraction] = ER.[IdInteraction]
						AND AEP.[IdAuditor] =  ER.[IdAuditor]
  LEFT JOIN HistoricEvaluationFormat (NOLOCK) EF
    ON ER.IdEvaluationFormat = EF.ID AND ER.IdEvaluationPlan = EF.EvaluationPlanId
  LEFT JOIN HistoricAbility (NOLOCK) A
    ON A.ID = ER.IdAbility AND ER.IdEvaluationPlan = A.EvaluationPlanId
  LEFT JOIN EvaluationPlan (NOLOCK) EP
    ON EP.ID = ER.IdEvaluationPlan
  LEFT JOIN [HistoricGroup] (NOLOCK) GP
    ON GP.ID = ER.IdGroup AND EP.ID = GP.EvaluationPlanId
  LEFT JOIN HistoricSubGroup SGP (NOLOCK)
    ON SGP.ID = ER.IdSubGroup AND EP.ID = SGP.EvaluationPlanId
	WHERE ER.Instance LIKE CASE
			WHEN @VCC IS NOT NULL THEN @VCC
			ELSE ER.Instance
			END
		AND ER.[IdEvaluationFormat] IN (SELECT [Value] FROM [dbo].[Split](IIF(@RealtimeFormat IS NULL,EF.Description,@RealtimeFormat),','))
		AND ER.CampaignDescription IN(SELECT [Value] FROM [dbo].[Split](IIF(@HistoricCampaign IS NULL,ER.CampaignDescription,@HistoricCampaign),','))
		AND ER.AgentDescription IN(SELECT [Value] FROM [dbo].[Split](IIF(@IdAgente IS NULL,ER.AgentDescription,@IdAgente),','))
		AND ER.IdAuditor IN(SELECT [Value] FROM [dbo].[Split](IIF(@IdAuditor IS NULL,ER.IdAuditor,@IdAuditor),','))
		AND ER.EvaluationDate >=
                          CASE
                            WHEN @StartDate IS NOT NULL THEN @StartDate
                            ELSE '19800101'
                          END
		AND ER.EvaluationDate <
                         CASE
                           WHEN @EndDate IS NOT NULL THEN @EndDate + ' 23:59'
                           ELSE '20500101'
                         END
	) a

	
	SET @cols = STUFF(
		(
			SELECT (',' + QUOTENAME(TMP_ER.[Ability]))
				FROM #TempEvaluationResult TMP_ER
				GROUP BY TMP_ER.[Ability] 
				ORDER BY TMP_ER.[Ability] 
			FOR xml PATH (''), TYPE
		).value('.', 'varchar(Max)')
		, 1, 1, '')
  
  IF @cols IS NOT NULL
  BEGIN
    ------
    DECLARE @type varchar(max) = (SELECT TOP 1
      Evaluation_Mode
    FROM #TempEvaluationResult)
	
	-------
    DECLARE @Total varchar(max) = STUFF(
		(
			SELECT '+ ISNULL(' + QUOTENAME(TMP_ER.[Ability]) + ',0)'
				FROM #TempEvaluationResult TMP_ER
				GROUP BY TMP_ER.[Ability] 
				ORDER BY TMP_ER.[Ability] 
			FOR xml PATH (''), TYPE
		).value('.', 'varchar(Max)')
		, 1, 1, '')    
	
	-- ROUND(AVG(CAST(ER.Qualification AS float)), 2) AS 'Qualification',
    DECLARE @colavg varchar(max) = STUFF(
		(
			SELECT ', ROUND(AVG(CAST(q.' + QUOTENAME(TMP_ER.[Ability]) + ' AS float)), 2) As [' + TMP_ER.[Ability] + ' Promedio]'
				FROM #TempEvaluationResult TMP_ER
				GROUP BY TMP_ER.[Ability] 
				ORDER BY TMP_ER.[Ability] 
			FOR xml PATH (''), TYPE
		).value('.', 'varchar(Max)')
		, 1, 1, '')
	
	--SELECT @colavg
    IF @type = 'M'
    BEGIN
      SET @query = ' 
				Select 
				q.DescriptionPlan As   [EVALUATION_PLAN] ,
				q.DescriptionFormat As [EVALUATION_FORMAT] ,
				q.AgentDescription As  [AGENT] ,
				q.CampaignDescription As [CAMPAIGN] ,
				count(q.IdInteraction) As [AUDITS],
				sum(q.DurationTime) As [DURATION_TIME],
				ROUND(AVG(CAST(q.Note AS float)), 2) As [AVERAGE_SCORE],
				ROUND(AVG(CAST(q.EvaluationTime AS float)), 2) As [AVERAGE_EVALUATION_TIME],
				' + @colavg + ' From
				(Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,p.EvaluationTime,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,InteractionDuration as DurationTime, 
				null As Number,InteractionDateStart As StartDate,null As Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor,
						  EvaluationTime			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join (
				Select Distinct IdInteraction,InteractionDuration,InteractionDateStart From InteractionEvaluationPlan (Nolock)
				) as A On
				A.IdInteraction=p.IdInteraction)q
				Group By 
				q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription
				'
    END
    ELSE
    BEGIN
      SET @query = '
	
			    Select 
				q.DescriptionPlan As   [EVALUATION_PLAN] ,
				q.DescriptionFormat As [EVALUATION_FORMAT] ,
				q.AgentDescription As  [AGENT] ,
				q.CampaignDescription As [CAMPAIGN] ,
			    count(q.IdInteraction) As [AUDITS],
				sum(q.DurationTime) As [DURATION_TIME],
				ROUND(AVG(CAST(q.Note AS float)), 2) As [AVERAGE_SCORE],
				ROUND(AVG(CAST(q.EvaluationTime AS float)), 2) As [AVERAGE_EVALUATION_TIME],
				' + @colavg + '
				From 
				(Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,p.EvaluationTime,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,
				A.DurationTime,A.Number,A.StartDate,A.Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor,
						  EvaluationTime			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join Audio as A On
				A.ID=p.IdInteraction)q
				Group By 
				q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription'
    END
    --Print(@query)
    EXECUTE (@query)
    --Select * From @query
	

    DROP TABLE #TempEvaluationResult
  END
  ELSE
  BEGIN
    DROP TABLE #TempEvaluationResult
    SELECT
						''				As [EVALUATION_PLAN],
						''				As [EVALUATION_FORMAT],
						''				As [AGENT],
  						''			    As [CAMPAIGN] 
  END
  
  SET NOCOUNT OFF
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[ReportBySummaryFormat] (@VCC varchar(50) = NULL, @Campana varchar(50) = NULL, @Plan varchar(50) = NULL,
@Formato varchar(50) , @FechaInicial varchar(50) = NULL, @FechaFinal varchar(50) = NULL , @IdAuditor varchar(50) = NULL ,@IdAgente varchar(50) = NULL)
AS
  DECLARE @cols varchar(max)
  DECLARE @query varchar(max)
  --ReportBySummaryFormat 'allegro','entrantes',NULL,'nov01-0356',NULL,NULL,NULL,'icagente2'
  --SET @Formato = 'nov01-0356'
  SET NOCOUNT ON
  SELECT
    * INTO #TempEvaluationResult
  FROM (SELECT
    EP.Description AS 'DescriptionPlan',
    EF.Description AS 'DescriptionFormat',
    GP.Description + '-' + SGP.Description + '-' + A.Title AS 'Ability',
    ER.AgentDescription,
    ER.CampaignDescription,
    ER.IdInteraction,
    ER.Qualification,
    Format(ER.EvaluationDate, 'dd/MM/yyyy hh:mm:ss') AS 'EvaluationDate',
    ER.IdAuditor,
    EP.Evaluation_Mode
  FROM EvaluationResult (NOLOCK) ER
  LEFT JOIN EvaluationFormat (NOLOCK) EF
    ON ER.IdEvaluationFormat = EF.ID
  LEFT JOIN Ability (NOLOCK) A
    ON A.ID = ER.IdAbility
  LEFT JOIN EvaluationPlan (NOLOCK) EP
    ON EP.ID = ER.IdEvaluationPlan
  LEFT JOIN [Group] (NOLOCK) GP
    ON GP.ID = ER.IdGroup
  LEFT JOIN SubGroup SGP (NOLOCK)
    ON SGP.ID = ER.IdSubGroup
  WHERE EF.Description LIKE CASE
    WHEN @Formato IS NOT NULL THEN @Formato
    ELSE EF.Description
  END
  AND ER.Instance LIKE CASE
    WHEN @VCC IS NOT NULL THEN @VCC
    ELSE ER.Instance
  END
  AND EP.Description LIKE CASE
    WHEN @Plan IS NOT NULL THEN @Plan
    ELSE EP.Description
  END
  AND ER.CampaignDescription LIKE CASE
    WHEN @Campana IS NOT NULL THEN @Campana
    ELSE ER.CampaignDescription
  END
  AND ER.AgentDescription LIKE CASE
    WHEN @IdAgente IS NOT NULL THEN @IdAgente
    ELSE ER.AgentDescription
  END
  AND ER.IdAuditor LIKE CASE
    WHEN @IdAuditor IS NOT NULL THEN @IdAuditor
    ELSE ER.IdAuditor
  END
  AND ER.EvaluationDate >=
                          CASE
                            WHEN @FechaInicial IS NOT NULL THEN @FechaInicial
                            ELSE '19800101'
                          END
  AND ER.EvaluationDate <
                         CASE
                           WHEN @FechaFinal IS NOT NULL THEN @FechaFinal + ' 23:59'
                           ELSE '20500101'
                         END) a



  SET @cols = STUFF((SELECT DISTINCT
    ',' + QUOTENAME(Ability)
  FROM #TempEvaluationResult
  FOR xml PATH (''), TYPE)
  .value('.', 'varchar(Max)')
  , 1, 1, '')
  --Select @cols
  IF @cols IS NOT NULL
  BEGIN
    ------
    DECLARE @type varchar(max) = (SELECT TOP 1
      Evaluation_Mode
    FROM #TempEvaluationResult)
	-------
    DECLARE @Total varchar(max) = STUFF((SELECT DISTINCT
      '+ ISNULL(' + QUOTENAME(Ability) + ',0)'
    FROM #TempEvaluationResult
    FOR xml PATH (''), TYPE)
    .value('.', 'varchar(Max)')
    , 1, 1, '')    
	-- ROUND(AVG(CAST(ER.Qualification AS float)), 2) AS 'Qualification',
    DECLARE @colavg varchar(max) = STUFF((SELECT DISTINCT
      ', ROUND(AVG(CAST(q.' + QUOTENAME(Ability) + ' AS float)), 2) As [' + Ability + ' Promedio]'
    FROM #TempEvaluationResult
    FOR xml PATH (''), TYPE)
    .value('.', 'varchar(Max)')
    , 1, 1, '')

    --SELECT @colavg
    IF @type = 'M'
    BEGIN
      SET @query = ' 
				Select q.DescriptionPlan As [Plan] ,q.DescriptionFormat As Formato ,q.AgentDescription As Agente ,q.CampaignDescription As Campana ,count(q.IdInteraction) As Auditorias,
				sum(q.DurationTime) As [Tiempo Duracion],ROUND(AVG(CAST(q.Note AS float)), 2) As [Nota Promedio], ' + @colavg + ' From
				(Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,InteractionDuration as DurationTime, 
				null As Number,InteractionDateStart As StartDate,null As Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join (
				Select Distinct IdInteraction,InteractionDuration,InteractionDateStart From InteractionEvaluationPlan (Nolock)
				) as A On
				A.IdInteraction=p.IdInteraction)q
				Group By 
				q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription
				'
    END
    ELSE
    BEGIN
      SET @query = '
	
			    Select q.DescriptionPlan As Plan ,q.DescriptionFormat As Formato ,q.AgentDescription As Agente,q.CampaignDescription As [Campana,count(q.IdInteraction) As Auditorias,
				sum(q.DurationTime) As [Tiempo Duraccion],ROUND(AVG(CAST(q.Note AS float)), 2) As [Nota Promedio], ' + @colavg + ' From 
				(Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,
				A.DurationTime,A.Number,A.StartDate,A.Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join Audio as A On
				A.ID=p.IdInteraction)q
				Group By 
				q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription'
    END
    --Print(@query)
    EXECUTE (@query)
    --Select * From @query

    DROP TABLE #TempEvaluationResult
  END
  ELSE
  BEGIN
    DROP TABLE #TempEvaluationResult
    SELECT
      '' AS [Plan],
      '' AS Formato,
      '' AS Agente,
      '' AS [Campana]
  END
  SET NOCOUNT OFF 
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ReportDummy] (
-- Debe venir siempre un unico formato evaluacion, si no viene no camina
   @Plan  varchar(MAX) = NULL
)
AS
BEGIN
	Select top 5 * from EvaluationPlan (nolocK) where ID = @Plan
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ReportTestEvaluationResult]
(
	@VCC		VARCHAR(50),
	@IdAgent	VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	SELECT
		[IdAgent]					'Agente',
		[IdInteraction]				'Id Interaccion',
		[Qualification]				'Nota',
		[EvaluationDate]			'Fecha de Evaluacion'
	FROM 
		[dbo].[EvaluationResult]
	WHERE
		[Instance] = @VCC
		AND [IdAgent] = @IdAgent

	SET NOCOUNT OFF

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[RestoreEvaluationResultDraftBackup] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@AuditorId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	-- EL PRESENTE SP TIENE LA FINALIDAD DE ELIMINAR LOS REGISTROS DE RESPALDO PARA BORRADOR

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- ELIMINAMOS LOS RESULTS DE LA EVALUACION
		DELETE FROM [dbo].[EvaluationResult]
			WHERE [Instance] = @VCC
				AND [IdEvaluationPlan] = @EvaluationPlanId
				AND [IdInteraction] = @InteractionId
				AND [IdAuditor] = @AuditorId

		-- DECLARAMOS UNA VARIABLE PARA VERIFICAR EL TIPO DEL PLAN
		DECLARE @IS_MIXED_EVALUATION_PLAN BIT = 0

		-- VERIFICAMOS QUE EL PLAN DE EVALUACION ES DEL TIPO MIXTO
		SELECT @IS_MIXED_EVALUATION_PLAN = IIF (EP.[Evaluation_Mode] = 'X', 1, 0) 
			FROM [dbo].[EvaluationPlan] EP (NOLOCK) 
			WHERE EP.[Instance] = @VCC
				AND EP.[ID] = @EvaluationPlanId

		-- DECLARAMOS UNA VARIABLE PARA VERIFICAR SI LA AUDITORIA YA ESTA GUARDADA COMO BORRADOR
		DECLARE @EXISTS_DRAFT_DATA BIT = 0

		IF EXISTS (
			SELECT 1 
				FROM [dbo].[EvaluationResult] (NOLOCK)
				WHERE [Instance] = @VCC
					AND [IdEvaluationPlan] = @EvaluationPlanId
					AND [IdInteraction] = @InteractionId
					AND [IdEvaluator] = 'SPEECH'
					AND [IdAuditor] = @AuditorId + '-DRAFT'
		)
			SET @EXISTS_DRAFT_DATA = 1

		IF @IS_MIXED_EVALUATION_PLAN = 1 AND @EXISTS_DRAFT_DATA = 1
		BEGIN
			-- RESTAURAMOS LOS RESULTADOS DE LA PARTE AUTOMATICA
			UPDATE [dbo].[EvaluationResult]
				SET [IdAuditor] = @AuditorId
				WHERE [Instance] = @VCC
					AND [IdEvaluationPlan] = @EvaluationPlanId
					AND [IdInteraction] = @InteractionId
					AND [IdEvaluator] = 'SPEECH'
					AND [IdAuditor] = @AuditorId + '-DRAFT'
		END 

		-- ACTUALIZAMOS EL ESTADO DE LA AUDITORIA
		UPDATE [dbo].[AuditorEvaluationPlan] 
			SET [StatusDescription] = 'PENDING'
			WHERE [Instance] = @VCC
				AND [IdEvaluationPlan] = @EvaluationPlanId
				AND [IdInteraction] = @InteractionId
				AND [IdAuditor] = @AuditorId 

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[RollbackVcc] (
	@VCC VARCHAR(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	

		DELETE FROM [dbo].[VCC] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[Role] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[RolePermission] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[RoutePermission] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[RolePermissionsGroup] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[Feeling] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[NeuroLinguisticProgramming] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[Word] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[Category] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[Environment] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[CategoryPermissions] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[TenantVCC] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[NLU] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[Intent] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[Utterance] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[Dictionary] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[InteractionType] WHERE [Instance] = @VCC
		DELETE FROM [dbo].[GenericQuery] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[User] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[UserClient] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[Report] WHERE [VCC] = @VCC

		DELETE FROM [dbo].[VCCLicense] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[AzureSubscriptionKey] WHERE [VCC] = @VCC
		DELETE FROM [dbo].[AzureSubscription] WHERE [VCC] = @VCC
		
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAbility] (
	@Instance Varchar(50) = NULL, 				
	@ID Varchar(50) = NULL,
	@Title Varchar(100) = NULL,					
	@Detail Varchar(500) = NULL,
	@Type VARCHAR(25) = NULL, -- MANUAL/MANUAL_AUTOMATIC/MIXED
	@CalificationType VARCHAR(25) = NULL, -- QUALITATIVE/QUANTITATIVE
	@ExecutionType VARCHAR(25) = NULL, -- BATCH/REALTIME
	@Binary Bit = NULL,
	@NotApply Bit = NULL,
	@Distribute BIT = NULL,
	@IsCritical BIT = NULL,
	@AcceptComments Bit = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL,
	@Enabled bit = NULL,
	@AnalysisType VARCHAR(25) = NULL,
	@Language VARCHAR(50) = NULL,
	@IsNegative BIT = NULL,
    @DegradeScoreCard BIT = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	-- FLAG PARA EL DEGRADADO DE SCORE CARD
    UPDATE [AbilityScoreCard]
        SET [DegradeScoreCard] = @DegradeScoreCard
        WHERE [VCC] = @Instance 
			AND [ID] = @ID
    IF @@ROWCOUNT = 0
		INSERT INTO [AbilityScoreCard] (
			[VCC], [ID], [DegradeScoreCard]
		) VALUES(
			@Instance, @ID, @DegradeScoreCard
		)

	-- EN CASO NOT_APPLY SEA FALSE, SETTEAMOS DISTRIBUTE COMO FALSE
	IF @NotApply = 0 SET @Distribute = 0
	IF @Distribute IS NULL SET @Distribute = 0
	
	-- 
	UPDATE [Ability]
		SET [Title] = @Title,
			[Detail] = @Detail,
			[Type] = @Type,
			[CalificationType] = @CalificationType,
			[ExecutionType] = @ExecutionType,
			[Binary] = @Binary,
			[NotApply] = @NotApply, 
			[Distribute] = @Distribute, 
			[IsCritical] = @IsCritical,
			[AcceptComments] = @AcceptComments,
			[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
			[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
			[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[Enabled] = CASE WHEN @Enabled IS NULL THEN 1 ELSE @Enabled END,
			[TimeStamp] = GETUTCDATE(),
			[AnalysisType] = @AnalysisType,
			[Language] = @Language,
			[IsNegative] = @IsNegative
		WHERE [Instance] = @Instance
			AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [Ability] (
			[Instance], [ID],
			[Title], [Detail], 
			[Type], [CalificationType],
			[ExecutionType], [Binary], 
			[NotApply], [Distribute], 
			[IsCritical], [AcceptComments],
			[CreatedDate],  [CreatedByUserId], 
			[LastModifiedDate], [LastModifiedByUserId],
			[Enabled], [TimeStamp], 
			[AnalysisType], [Language], 
			[IsNegative]
			)
		VALUES (
			@Instance, @ID,
			@Title, @Detail, 
			@Type, @CalificationType,
			@ExecutionType, @Binary, 
			@NotApply, @Distribute, 
			@IsCritical, @AcceptComments, 
			@CreatedDate, @CreatedByUserId, 
			@LastModifiedDate, @LastModifiedByUserId,
			1, GETUTCDATE(), 
			@AnalysisType, @Language, 
			@IsNegative
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveAbilityEvaluationFormat] (
	@Instance VARCHAR(50) = NULL,
	@IdEvaluationFormat VARCHAR(50) = NULL,	
	@IdAbility VARCHAR(50) = NULL,
	@IdGroup VARCHAR(50) = NULL,
	@IdSubGroup VARCHAR(50) = NULL, 
	@Score NUMERIC(5,2) = 0, 
	@Penalizes varchar(3) = 'NO', 
	@Order NUMERIC(3) = 0,
	@Conditional VARCHAR(50) = NULL,
	@HelpComment VARCHAR(2250) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	INSERT INTO [dbo].[AbilityEvaluationFormat] (
		[Instance], [IdEvaluationFormat], [IdAbility],
		[IdGroup], [IdSubGroup], [Score], 
		[Penalizes], [Order], [Conditional],
		[HelpComment], [TimeStamp]
		)
	VALUES (
		@Instance, @IdEvaluationFormat, @IdAbility, 
		@IdGroup, @IdSubGroup, @Score, 
		@Penalizes, @Order, @Conditional,
		@HelpComment, GETUTCDATE()
	)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAbilityQuery] (
	@VCC							VARCHAR(50) = NULL,
	@ID								VARCHAR(50) = NULL,
	@AbilityId						VARCHAR(50) = NULL,
	@QueryId						VARCHAR(50) = NULL,
	@QueryContent					VARCHAR(1000) = NULL,
	@AcceptancePercentage			DECIMAL(18, 2) = NULL,
	@Order							INT = NULL,
	@CreatedByUserId				VARCHAR(50) = NULL,
	@LastModifiedByUserId			VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	UPDATE [AbilityQuery]
		SET [AcceptancePercentage] = @AcceptancePercentage,
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [ID] = @ID

	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [AbilityQuery] (
			[VCC], [ID], [AbilityId], [QueryId], [QueryContent],
			[AcceptancePercentage], [Order], [CreatedDate], [CreatedByUserId], [LastModifiedDate], 
			[LastModifiedByUserId], [TimeStamp]
		)
		VALUES (
			@VCC, @ID, @AbilityId, @QueryId, @QueryContent, 
			@AcceptancePercentage, @Order, GETUTCDATE(), @CreatedByUserId, GETUTCDATE(), 
			@LastModifiedByUserId, GETUTCDATE()
		)
	END

	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAbilityQueryEntity] (
	@VCC							VARCHAR(50) = NULL,
	@AbilityQueryId					VARCHAR(50) = NULL,
	@EntityId						VARCHAR(50) = NULL,
	@EntityName						VARCHAR(50) = NULL,
	@AcceptancePercentage			DECIMAL(18, 2) = NULL,
	@Order							INT = NULL,
	@CreatedByUserId				VARCHAR(50) = NULL,
	@LastModifiedByUserId			VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	UPDATE [AbilityQueryEntity]
		SET [AcceptancePercentage] = @AcceptancePercentage,
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [AbilityQueryId] = @AbilityQueryId
			AND [EntityId] = @EntityId

	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [AbilityQueryEntity] (
			[VCC], [AbilityQueryId], [EntityId], [EntityName], [AcceptancePercentage], 
			[Order], [CreatedDate], [CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId], 
			[TimeStamp]
		)
		VALUES (
			@VCC, @AbilityQueryId, @EntityId, @EntityName, @AcceptancePercentage, 
			@Order, GETUTCDATE(), @CreatedByUserId, GETUTCDATE(), @LastModifiedByUserId, 
			GETUTCDATE()
		)
	END

	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAdvancedAI] (
    @VCC VARCHAR(50) = NULL,
    @ID VARCHAR(50) = NULL,
    @Description VARCHAR(250) = NULL,
	@StatusCode VARCHAR(50) = NULL,
    @CurrentUser VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON
	
	-- ACTUALIZAMOS LA AAI EN CASO EXISTA
	UPDATE [dbo].[AdvancedAI]
		SET [Description] = @Description,
			[StatusCode] =  CASE WHEN @StatusCode IS NULL THEN [StatusCode] ELSE @StatusCode END,
			[LastModifiedByUserId] = @CurrentUser,
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
		  AND [ID] = @ID

	-- VERIFICAMOS SI SE ACTUALIZO ALGUN REGISTRO
    IF @@ROWCOUNT = 0
    BEGIN
		-- INSERTAMOS EL MODELO
        INSERT INTO [dbo].[AdvancedAI] (
            [VCC], [ID], [Description], 
			[StatusCode], [CreatedByUserId], [CreatedDate], 
			[TimeStamp]
        )
		VALUES (
            @VCC, @ID, @Description, 
			@StatusCode, @CurrentUser, GETUTCDATE(), 
			GETUTCDATE()
        )
    END

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAdvancedAIExample] (
    @VCC VARCHAR(50) = NULL,
	@AppId VARCHAR(50) = NULL,
	@IntentId VARCHAR(50) = NULL,
	@Id VARCHAR(50) = NULL,
    @Text VARCHAR(250) = NULL,
    @CurrentUser VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON
	
	-- ACTUALIZAMOS EL INTENT EN CASO EXISTA
	UPDATE [AdvancedAIExample]
		SET [Text] = @Text,
			[LastModifiedByUserId] = @CurrentUser,
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
		  AND [AppId] = @AppId
		  AND [IntentId] = @IntentId
		  AND [Id] = @Id

	-- VERIFICAMOS SI SE ACTUALIZO ALGUN REGISTRO
    IF @@ROWCOUNT = 0
    BEGIN
		-- INSERTAMOS EL MODELO
        INSERT INTO [AdvancedAIExample] (
            [VCC], [AppId], [IntentId], [Id], [Text], 
			[CreatedByUserId], [CreatedDate], [TimeStamp]
        )
		VALUES (
            @VCC, @AppId, @IntentId, @Id, @Text, 
			@CurrentUser, GETUTCDATE(), GETUTCDATE()
        )
    END

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAdvancedAIIntent] (
    @VCC VARCHAR(50) = NULL,
	@AppId VARCHAR(50) = NULL,
	@Id VARCHAR(50) = NULL,
    @Name VARCHAR(50) = NULL,
    @Prompt VARCHAR(1000) = NULL,
    @CurrentUser VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON
	
	-- ACTUALIZAMOS EL INTENT EN CASO EXISTA
	UPDATE [AdvancedAIIntent]
		SET [Prompt] = @Prompt,
			[LastModifiedByUserId] = @CurrentUser,
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
		  AND [AppId] = @AppId
		  AND [Id] = @Id

	-- VERIFICAMOS SI SE ACTUALIZO ALGUN REGISTRO
    IF @@ROWCOUNT = 0
    BEGIN
		-- INSERTAMOS EL MODELO
        INSERT INTO [AdvancedAIIntent] (
            [VCC], [AppId], [Id], [Name], [Prompt], 
			[CreatedByUserId], [CreatedDate], [TimeStamp]
        )
		VALUES (
            @VCC, @AppId, @Id, @Name, @Prompt, 
			@CurrentUser, GETUTCDATE(), GETUTCDATE()
        )
    END

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [SaveAllegroGroup] 'inconcert', 'test5', 0, 'test5', '38.127.113.216', 0
CREATE PROCEDURE [dbo].[SaveAllegroGroup] (
	@VCC Varchar(50) = NULL, @Id Varchar(50) = NULL, @IsSystemGroup Varchar(50) = NULL, @Description Varchar(50) = NULL, @Ip Varchar(50) = NULL, @Type Int  
	)
AS
BEGIN

	DECLARE @Sql NVARCHAR(MAX);
	
	IF(@Type = 0)
	BEGIN  
		SET @Sql = '
		SET NOCOUNT ON
		IF EXISTS(
		SELECT [Id], [VCC] FROM [' + @Ip + ']' + '.mmprodat.dbo.[Group] WITH (NOLOCK)
		WHERE [VCC] = ''' + @VCC + '''
		AND [Id] = ''' + @Id + '''
		)
		BEGIN
			SELECT 1 AS ''result''
		END
		ELSE
		BEGIN
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[Group] ([Id], [IsSystemGroup], [Description], [VCC]) VALUES (''' + @Id + ''',''' + @IsSystemGroup + ''',''' + @Description + ''',''' + @VCC + ''');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''deleteflowdesigner'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''deletescriptinglauncher'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''deletewebdesignerlauncher'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''designwebdesigner'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''saveflowdesigner'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''savescriptinglauncher'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''savewebdesignerlauncher'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewentityscriptingdesigner'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewentityscriptinglauncher'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewentitywebdesignerlauncher'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewflowdesigner'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewscriptingdesigner'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewscriptinglauncher'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewwebdesignerlauncher'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewwebdesigner'',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupApplication] ([VCC], [GroupId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''designer'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupApplication] ([VCC], [GroupId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''webbrowser'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupApplication] ([VCC], [GroupId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''webdesigner'');
			INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[GroupApplication] ([VCC], [GroupId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''dashboard'');
			SELECT 1 AS ''result''
		END	'
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE IF(@Type = 1) 
	BEGIN
		SET @Sql = '
		SET NOCOUNT ON
		IF EXISTS(
		SELECT [Id], [VCC] FROM mmprodat.dbo.[Group] (NOLOCK)
		WHERE [VCC] = ''' + @VCC + '''
		AND [Id] = ''' + @Id + '''
		)
		BEGIN
			SELECT 1 AS ''result''
		END
		ELSE
		BEGIN
			INSERT INTO mmprodat.dbo.[Group] ([Id], [IsSystemGroup], [Description], [VCC]) VALUES (''' + @Id + ''',''' + @IsSystemGroup + ''',''' + @Description + ''',''' + @VCC + ''');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''deleteflowdesigner'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''deletescriptinglauncher'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''deletewebdesignerlauncher'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''designwebdesigner'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''saveflowdesigner'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''savescriptinglauncher'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''savewebdesignerlauncher'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewentityscriptingdesigner'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewentityscriptinglauncher'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewentitywebdesignerlauncher'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewflowdesigner'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewscriptingdesigner'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewscriptinglauncher'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewwebdesignerlauncher'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupPermission] ([VCC], [GroupId], [ActionId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''viewwebdesigner'',''designer'');
			INSERT INTO mmprodat.dbo.[GroupApplication] ([VCC], [GroupId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''designer'');
			INSERT INTO mmprodat.dbo.[GroupApplication] ([VCC], [GroupId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''webbrowser'');
			INSERT INTO mmprodat.dbo.[GroupApplication] ([VCC], [GroupId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''webdesigner'');
			INSERT INTO mmprodat.dbo.[GroupApplication] ([VCC], [GroupId], [ApplicationId]) VALUES (''' + @VCC + ''',''' + @Id + ''',''dashboard'');
			SELECT 1 AS ''result''
		END	'
		SET @Sql += ' 
		SET NOCOUNT OFF'
	END  
	ELSE  
	BEGIN  
		SET @Sql = 'SELECT 1 AS ''result'''  
	END

	EXEC (@Sql)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
--exec [SaveAllegroUser] 'agent', 0, '54', 'jeampierre2', 'ENABLED', 0, 'jeampierre9', 'es', 'PET', 'ac6c61c1e7b8b6a5a5f850d9c10c95f5', 'inconcert', 'system', '38.127.113.216', '0'  
CREATE PROCEDURE [dbo].[SaveAllegroUser] (    
	 @UserGroup Varchar(50) = NULL,   
	 @ValidationOrigin Varchar(50) = NULL,   
	 @Country Varchar(100) = NULL,    
	 @Id Varchar(50) = NULL,     
	 @State Varchar(20) = NULL,       
	 @IsSystemUser Varchar(50) = NULL,    
	 @Name Varchar(100) = NULL,     
	 @LanguageId Varchar(100) = NULL,     
	 @TimeZoneId Varchar(50) = NULL,    
	 @Password Varchar(100) = NULL,   
	 @VCC Varchar(50) = NULL,       
	 @VccGroup Varchar(50) = NULL,    
	 @Ip Varchar(50) = NULL, 
	 @Type Int  
 )    
AS    
BEGIN    
    
 DECLARE @Sql NVARCHAR(MAX);  
  
 IF(@Type = 0)  
 BEGIN  
	  SET @Sql = '    
	SET NOCOUNT ON    
	IF EXISTS(    
		SELECT [Id], [VCC] FROM [' + @Ip + ']' + '.mmprodat.dbo.[User] (NOLOCK)    
			WHERE [VCC] = ''' + @VCC + '''    
				AND [Id] = ''' + @Id + '''    
	)    
	BEGIN    
		SELECT 1 AS ''result''    
	END    
	ELSE    
	BEGIN    
		INSERT INTO [' + @Ip + ']' + '.mmprodat.dbo.[User] (    
			[UserGroup], [ValidationOrigin], [Country],     
			[Id], [State], [IsSystemUser],     
			[Name], [LanguageId], [TimeZoneId],     
			[Password], [VCC], [VccGroup]    
		)    
		VALUES (''' + @UserGroup + ''',''' + @ValidationOrigin + ''',''' +@Country + ''',''' +    
			@Id + ''',''' + @State + ''',''' + @IsSystemUser + ''',''' +    
			@Name + ''',''' + @LanguageId + ''',''' + @TimeZoneId + ''',''' +    
			@Password + ''',''' + @VCC + ''',''' + @VccGroup + '''   
		)    
		SELECT 1 AS ''result''
	END '        
	
	SET @Sql += ' 
	SET NOCOUNT OFF'

 END  
 ELSE IF(@Type = 1)  
 BEGIN  
	  SET @Sql = '    
	  SET NOCOUNT ON    
	  IF EXISTS(    
	  SELECT [Id], [VCC] FROM mmprodat.dbo.[User] (NOLOCK)    
	  WHERE [VCC] = ''' + @VCC + '''    
	  AND [Id] = ''' + @Id + '''    
	  )    
	  BEGIN    
	   SELECT 1 AS ''result''    
	  END    
	  ELSE    
	  BEGIN    
	   INSERT INTO mmprodat.dbo.[User] (  
	   [UserGroup], [ValidationOrigin], [Country],     
	   [Id], [State], [IsSystemUser],     
	   [Name], [LanguageId], [TimeZoneId],     
	   [Password], [VCC], [VccGroup]    
	   )    
	   VALUES ('''    
		+ @UserGroup + ''',''' + @ValidationOrigin + ''',''' +@Country + ''',''' +    
		@Id + ''',''' + @State + ''',''' + @IsSystemUser + ''',''' +    
		@Name + ''',''' + @LanguageId + ''',''' + @TimeZoneId + ''',''' +    
		@Password + ''',''' + @VCC + ''',''' + @VccGroup + '''    
	   )    
	   SELECT 1 AS ''result''    
	  END '        
	  SET @Sql += ' 
	  SET NOCOUNT OFF'    
 END  
 ELSE  
 BEGIN  
	 SET @Sql = 'SELECT 1 AS ''result'''  
 END

 EXEC (@Sql)

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAPIUser] (
	@VCC VARCHAR(50) = NULL, 					@User VARCHAR(50) = NULL, 				@CreatedDate DATETIME = NULL,			
	@CreatedByUserId VARCHAR(50) = NULL,    	@LastModifiedDate DATETIME = NULL,		@LastModifiedByUserId VARCHAR(50) = NULL,
	@IsLocked BIT = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [UserAPI]
	SET [IsLocked] = @IsLocked,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
	AND [User] = @User
    
    IF @@ROWCOUNT = 0
		INSERT INTO [UserAPI] (
			[VCC], [User], [IsLocked],
			[CreatedDate], [CreatedByUserId],
			[LastModifiedDate], [LastModifiedByUserId],
			[TimeStamp]
			)
		VALUES (
			@VCC, @User, @IsLocked,
			@CreatedDate, @CreatedByUserId,
			@LastModifiedDate, @LastModifiedByUserId,
			GETUTCDATE()
		)
		
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAudio] (
	@VCC Varchar(50) = NULL,						@ID Varchar(100) = NULL,					@Section Varchar(100) = NULL,
	@Part Varchar(100) = NULL,						@ProcessId Varchar(50) = NULL,				@Number Varchar(50) = NULL,
	@Gender Varchar(50) = NULL,						@SilenceTime Numeric(12,3) = NULL,			@DurationTime Numeric(12,3) = NULL,
	@Category1 Varchar(100) = NULL,					@Category2 Varchar(100) = NULL,				@Category3 Varchar(100) = NULL,
	@Description Varchar(100) = NULL,				@Campaign Varchar(100) = NULL,				@Agent Varchar(50) = NULL,
	@Type Varchar(50) = NULL,					   @Init Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	DECLARE @Process VARCHAR(50) = CASE WHEN @ProcessId IS NULL THEN '' ELSE @ProcessId END
	
	UPDATE [Audio]
	SET [Number] = @Number,
		[Gender] = @Gender,
		[SilenceTime] = @SilenceTime,
		[DurationTime] = @DurationTime,
		[Category1] = @Category1,
		[Category2] = @Category2,
		[Category3] = @Category3,
		[Description] = @Description,
		[Campaign] = @Campaign,
		[Agent] = @Agent,
		[Type] = @Type,
		[Init] = CASE WHEN @Init IS NULL THEN [Init] ELSE @Init END
	WHERE [VCC] = @VCC
    -- AND ISNULL([ProcessId], '') = @Process
	AND [ID] = @ID
	AND [Section] = @Section 
	AND [Part] = @Part
	
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[SaveAudioLinguisticProcess] 'inconcert', 'procesoallegro12345', '011AAB7E-FB67-482A-838F-67CDB31264E4', 0, 1, 228.000, true
CREATE PROCEDURE [dbo].[SaveAudioLinguisticProcess]
(
	@VCC						VARCHAR(50),
	@ID							VARCHAR(50),
	@Section					NUMERIC(5, 0),
	@Part						NUMERIC(5, 0),
	@ProcessId					VARCHAR(50),
	@DurationLinguisticTime		NUMERIC(12, 0),
	@IsTranscribed				INT,
	@LastModifiedDate			DATETIME,
	@LastModifiedByUserId		VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [dbo].[Audio] SET
	[DurationLinguisticTime] = @DurationLinguisticTime,
	[IsTranscribed] = @IsTranscribed,
	[LastModifiedDate] = @LastModifiedDate,
	[LastModifiedByUserId] = @LastModifiedByUserId
	WHERE
	[VCC] = @VCC AND
	[ID] = @ID AND
	[Section] = ISNULL(@Section, 0) AND
	[Part] = ISNULL(@Part, 0)

	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

-- [dbo].[SaveAuditor] 'allegro', 'danny.jhonston', '', null, 'analytic', '2018-01-01 00:00:00.000', null
CREATE PROCEDURE [dbo].[SaveAuditor]
(
	@Instance				VARCHAR(50),
	@ID						VARCHAR(50),
	@Description			VARCHAR(100),
	@CreatedDate			DATETIME,
	@CreatedByUserId		VARCHAR(50),
	@LastModifiedDate		DATETIME,
	@LastModifiedByUserId	VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [dbo].[Auditor] SET
	[ID] = @ID,
	[Description] = @Description,
	[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
	[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
	[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
	[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
	[TimeStamp] = GETUTCDATE()
	WHERE
		[Instance] = @Instance
		AND [ID] = @ID

	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [dbo].[Auditor](
			[Instance],
			[ID],
			[Description],
			[CreatedDate],
			[CreatedByUserId],
			[LastModifiedDate],
			[LastModifiedByUserId],
			[TimeStamp]
		)
		VALUES(
			@Instance,
			@ID,
			@Description,
			@CreatedDate,
			@CreatedByUserId,
			@LastModifiedDate,
			@LastModifiedByUserId,
			GETUTCDATE()
		)
	END

	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAuditorEvaluationPlan] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(100) = NULL,
	@CampaignId VARCHAR(50) = NULL,
	@AgentId VARCHAR(50) = NULL,
	@AuditorId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(50) = NULL,
	@InteractionType VARCHAR(50) = NULL,
	@InteractionStartDate DATETIME = NULL,
	@InteractionEndDate DATETIME = NULL,
	@InteractionDuration INT = NULL,
	@InteractionManagementResult VARCHAR(250) = NULL,
	@InteractionOrigin VARCHAR(10) = NULL,
	@InteractionFormat VARCHAR(25) = NULL,
	@InteractionContactId VARCHAR(50) = NULL,
	--@InteractionContactAddress VARCHAR(50) = NULL,
	--@InteractionContactName VARCHAR(50) = NULL,
	--@Status BIT = NULL,
	@StatusDescription VARCHAR(50) = NULL,
	@Completed BIT = NULL,
	@Score INT = NULL,
	@CurrentUser VARCHAR(50) = NULL
) AS
BEGIN
	-- ESTE SP INSERTA UN NUEVO REGISTRO EN LA TABLA [dbo].[AuditorEvaluationPlan] (PRINCIPLAMENTE PARA REALTIME)
	SET NOCOUNT ON

	UPDATE [dbo].[AuditorEvaluationPlan] 
		SET [ManagementResult] = @InteractionManagementResult,
			[InteractionFormat] = @InteractionFormat,
			[ContactId] = @InteractionContactId/*,
			[ContactAddress] = @InteractionContactAddress,
			[ContactName] = @InteractionContactName*/
		WHERE [Instance] = @VCC
			AND [IdEvaluationPlan] = @EvaluationPlanId
			AND [IdInteraction] = @InteractionId
			AND [IdAuditor] = @AuditorId

	-- SI NO SE ACTUALIZO NINGUN REGISTRO LO INSERTAMOS
	IF @@ROWCOUNT = 0
	BEGIN
		-- INSERTMAOS LA DATA EN LA TABLA AEP
		INSERT INTO [dbo].[AuditorEvaluationPlan] (
			[Instance], [IdEvaluationPlan], [idCampaign], [IdAgent], [IdAuditor],
			[IdInteraction], [InteractionType], [InteractionDate], [InteractionDateEnd], [InteractionDuration],
			[ManagementResult], [InteractionOrigin], [InteractionFormat], [ContactId], /*[ContactAddress], 
			[ContactName],*/ [Status], [StatusDescription], [Completed], [Score],
			[CreatedByUserId], [CreatedDate], [TimeStamp]
		)
		VALUES (
			@VCC, @EvaluationPlanId, @CampaignId , @AgentId, @AuditorId,
			@InteractionId, @InteractionType, @InteractionStartDate, @InteractionEndDate, @InteractionDuration,
			@InteractionManagementResult, @InteractionOrigin, @InteractionFormat, @InteractionContactId, /*@InteractionContactAddress,
			@InteractionContactName,*/ 1, @StatusDescription, @Completed, @Score,
			@CurrentUser, GETUTCDATE(), GETUTCDATE()
		)
	END

	SELECT 1 AS 'result'
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveAuditorEvPlanResult] (
	@Instance Varchar(50) = NULL,
	@IdEvaluationPlan Varchar(50) = NULL,
	@idAuditor Varchar(50) = NULL,
	@idInteraction Varchar(50) = NULL,
	@interactionType Varchar(50) = NULL,
	@idAgent Varchar(50) = NULL,
	@score numeric(5,2) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [dbo].[AuditorEvaluationPlan]
	set [score] = @score, 
		[Completed] = 1,
		[StatusDescription] = 'COMPLETED',
		[LastModifiedDate] = GETUTCDATE(),
		[TimeStamp] = GETUTCDATE()
	WHERE
		[idEvaluationPlan] = @IdEvaluationPlan
		AND [Instance] = @Instance
		AND [IdAuditor] = @idAuditor
		AND [IdAgent] = @idAgent
		AND [IdInteraction] = @idInteraction
		AND [InteractionType] = @interactionType
		AND [Completed] <> 1

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveAzureSubscription] (
	@VCC Varchar(50) = NULL, 				
	@AzureSubscription Varchar(50) = NULL,				
	@ResourceGroup Varchar(50) = NULL,			
	@AccountName Varchar(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EL REGISTRO CORRESPONDIENTE AL VCC
	UPDATE [AzureSubscription]
		SET [AzureSubscription] = @AzureSubscription,
			[ResourceGroup] = @ResourceGroup,
			[AccountName] = @AccountName
		WHERE [VCC] = @VCC

	-- SI NO SE PUDO ACTUALIZAR REGISTRAMOS EL NUEVO REGISTRO
    IF @@ROWCOUNT = 0
		INSERT INTO [AzureSubscription] ([VCC], [AzureSubscription], [ResourceGroup], [AccountName])
		VALUES (@VCC, @AzureSubscription, @ResourceGroup, @AccountName)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveAzureSubscriptionKey] (
	@VCC Varchar(50) = NULL, 				
	@SubscriptionKey Varchar(50) = NULL,				
	@SubscriptionRegion Varchar(50) = NULL,			
	@SubscriptionName Varchar(50) = NULL,
	@IsDemo Bit = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [AzureSubscriptionKey]
	SET [SubscriptionKey] = @SubscriptionKey,
		[SubscriptionRegion] = SubscriptionRegion,
		[SubscriptionName] = @SubscriptionName,
		[IsDemo] = @IsDemo
	WHERE [VCC] = @VCC
		AND [SubscriptionName] = @SubscriptionName

    IF @@ROWCOUNT = 0
		INSERT INTO [AzureSubscriptionKey] (
			[VCC],[SubscriptionKey],
			[SubscriptionRegion],
			[SubscriptionName],
			[IsDemo]
			)
		VALUES (
			@VCC,@SubscriptionKey,
			@SubscriptionRegion, 
			@SubscriptionName,
			@IsDemo
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveBatchError] (
	@VCC VARCHAR(50) = NULL,
	@SystemService VARCHAR(50) = NULL,
	@BatchTypeId VARCHAR(50) = NULL,
	@BatchTypeName VARCHAR(50) = NULL,
	@Implementation VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@Query VARCHAR(250) = NULL,
	@ErrorFunction VARCHAR(100) = NULL,
	@ErrorMessage VARCHAR(250) = NULL,
	@ErrorDetail VARCHAR(MAX) = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE REGISTRAR LOS ERRORES QUE PUEDEN OCURRIR DURANTE LA EJECUCION DE LAS COLAS
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- VALIDAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @VCC IS NULL 
			OR @SystemService IS NULL 
			OR @BatchTypeId IS NULL 
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		-- VERIFICAMOS QUE EL BATCH TYPE SEA UN TIPO CONOCIDO DEL SISTEMA
		IF @SystemService <> 'TRANSCRIPTION' AND @SystemService <> 'SEMANTIC_ANALYSIS' AND @SystemService <> 'COGNITIVE_ANALYSIS'
			-- LANZAMOS MENSAJE AVISANDO QUE HAN SOLICITADO UNA OPERACION DESCONOCIDA
			RAISERROR('DATABASE_ERROR_UNKNOWN_SYSTEM_SERVICE', 16, 16)

		-- DECLARAMOS UNA VARIABLE PARA TIPO DE BATCH CORRESPONDIENTE AL BATCHTYPEID
		DECLARE @BATCH_TYPE VARCHAR(50) = NULL

		-- IDENTIFICAMOS EL TIPO BATCH SEGUN EL BATCHTYPEID Y SERVICIO DEL SISTEMA
		IF @SystemService = 'TRANSCRIPTION'
		BEGIN
			-- VERIFICAMOS SI ES DEL TIPO PROCESS
			IF EXISTS (SELECT 1 FROM [dbo].[Process] P (NOLOCK) WHERE [VCC] = @VCC AND [Name] = @BatchTypeId)
				SET @BATCH_TYPE = 'TRANSCRIPTION'

			-- VERIFICAMOS SI ES DEL TIPO EVALUATION_PLAN
			IF EXISTS (SELECT 1 FROM [dbo].[EvaluationPlan] P (NOLOCK) WHERE [Instance] = @VCC AND [ID] = @BatchTypeId)
				SET @BATCH_TYPE = 'EVALUATION_PLAN'
		END
		ELSE IF @SystemService = 'SEMANTIC_ANALYSIS'
		BEGIN
			-- VERIFICAMOS SI ES DEL TIPO STUDY
			IF EXISTS (SELECT 1 FROM [dbo].[Study] P (NOLOCK) WHERE [VCC] = @VCC AND [Name] = @BatchTypeId)
				SET @BATCH_TYPE = 'STUDY'

			-- VERIFICAMOS SI ES DEL TIPO EVALUATION_PLAN
			IF EXISTS (SELECT 1 FROM [dbo].[EvaluationPlan] P (NOLOCK) WHERE [Instance] = @VCC AND [ID] = @BatchTypeId)
				SET @BATCH_TYPE = 'EVALUATION_PLAN'
		END
		ELSE IF @SystemService = 'COGNITIVE_ANALYSIS'
		BEGIN
			-- VERIFICAMOS SI ES DEL TIPO STUDY
			IF EXISTS (SELECT 1 FROM [dbo].[Study] P (NOLOCK) WHERE [VCC] = @VCC AND [Name] = @BatchTypeId)
				SET @BATCH_TYPE = 'STUDY'

			-- VERIFICAMOS SI ES DEL TIPO EVALUATION_PLAN
			IF EXISTS (SELECT 1 FROM [dbo].[EvaluationPlan] P (NOLOCK) WHERE [Instance] = @VCC AND [ID] = @BatchTypeId)
				SET @BATCH_TYPE = 'EVALUATION_PLAN'
		END
		
		-- VERIFICAMOS QUE EL BATCH TYPE SEA UN TIPO CONOCIDO DEL SISTEMA
		IF @BATCH_TYPE <> 'TRANSCRIPTION' AND @BATCH_TYPE <> 'STUDY' AND @BATCH_TYPE <> 'EVALUATION_PLAN'
			-- LANZAMOS MENSAJE AVISANDO QUE HAN SOLICITADO UNA OPERACION DESCONOCIDA
			RAISERROR('DATABASE_ERROR_UNKNOWN_BATCH_TYPE', 16, 16)

		-- SOLO EN CASO DE PLANES @BatchTypeId Y @BatchTypeName SON DIFERENTES
		IF @BatchTypeName IS NULL
			SELECT @BatchTypeName = EP.[Description]
				FROM [dbo].[EvaluationPlan] EP (NOLOCK)
				 WHERE [Instance] = @VCC AND [ID] = @BatchTypeId

		-- INSERTAMOS EL REGISTRO EN LA TABLA DE ERROR
		INSERT INTO [dbo].[BatchError] (
				[VCC], [BatchType], [BatchTypeId], [BatchTypeName], [Implementation],
				[InteractionId], [Query], [ErrorFunction], [ErrorMessage], [ErrorDetail],
				[CreatedDate], [TimeStamp]
			) VALUES (
				@VCC, @BATCH_TYPE, @BatchTypeId, @BatchTypeName, @Implementation,
				@InteractionId, @Query, @ErrorFunction, @ErrorMessage, @ErrorDetail,
				GETUTCDATE(), GETUTCDATE()
			)

	END TRY
    BEGIN CATCH
        --IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveBatchSchedule] (
	@VCC VARCHAR(50) = NULL,
	@Id VARCHAR(50) = NULL,
	@BatchType VARCHAR(50) = NULL, -- TRANSCRIPTION/STUDY(?)/EVALUATION_PLAN
	@Frequency VARCHAR(50) = NULL,
	@Weekday INT = NULL,
	@Monthday INT = NULL,
	@DaysInterval INT = NULL,
	@HoursInterval INT = NULL,
	@DaysToExpire INT = NULL,
	@ExecutionHour VARCHAR(50) = NULL,
	@FirstExecutionDate DATETIME = NULL,
	@ExecutionDelay INT = NULL,
	@StartDate DATETIME = NULL,
	@EndDate DATETIME = NULL,
	@BatchData VARCHAR(MAX) = NULL,
	@ProcessFiltersId VARCHAR(50) = NULL,
	@Status VARCHAR(50) = NULL, -- CREATED/RUNNING/PAUSED/FINISHED
	@UserId VARCHAR(50) = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE REGISTRAR LOS ERRORES QUE PUEDEN OCURRIR DURANTE LA EJECUCION DE LAS COLAS
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- VALIDAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @VCC IS NULL 
			OR @Id IS NULL 
			OR @Frequency IS NULL 
			OR @StartDate IS NULL 
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		--
		UPDATE [dbo].[BatchSchedule]
			SET [LastExecutionDate] = NULL,
				[Status] = ISNULL(@Status, [Status]),
				[LastModifiedDate] = GETUTCDATE(),
				[LastModifiedByUserId] = @UserId,
				[TimeStamp] = GETUTCDATE()
			WHERE [VCC] = @VCC
				AND [Id] = @Id
				AND [BatchType] = @BatchType

		-- SI NO SE ACTUALIZO NINGUN REGISTRO, LO INSERTAMOS
		IF @@ROWCOUNT = 0
		BEGIN
			--select * from BatchSchedule (nolock)
			-- INSERTAMOS EL REGISTRO EN LA TABLA DE ERROR
			INSERT INTO [dbo].[BatchSchedule] (
					[VCC], [Id], [BatchType], [Frequency], [Weekday],
					[Monthday], [DaysInterval], [HoursInterval], [DaysToExpire], [StartDate], 
					[EndDate], [FirstExecutionDate], [LastExecutionDate], [FinalExecutionDate], [ExecutionDelay], 
					[BatchData], [ProcessFiltersId], [Status], [CreatedDate], [CreatedByUserId], 
					[TimeStamp]
				) VALUES (
					@VCC, @Id, @BatchType, @Frequency, @Weekday,
					@Monthday, @DaysInterval, @HoursInterval, @DaysToExpire, @StartDate, 
					@EndDate, @FirstExecutionDate, NULL, NULL, @ExecutionDelay, 
					@BatchData, @ProcessFiltersId, 'CREATED', GETUTCDATE(), @UserId, 
					GETUTCDATE()
				)
		END

		SELECT 1 AS 'result'
    
	END TRY
    BEGIN CATCH
        --IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION
		
        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveBatchScheduleExecution] (
	@VCC VARCHAR(50) = NULL,
	@Id VARCHAR(50) = NULL,
	@BatchScheduleId VARCHAR(50) = NULL, 
	@BatchId VARCHAR(50) = NULL,
	@ExecutionDate DATETIME = NULL,
	@Order VARCHAR(50) = NULL,
	@Status VARCHAR(50) = NULL,
	@UserId VARCHAR(50) = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE REGISTRAR LOS ERRORES QUE PUEDEN OCURRIR DURANTE LA EJECUCION DE LAS COLAS
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- VALIDAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @VCC IS NULL 
			OR @Id IS NULL 
			OR @BatchScheduleId IS NULL 
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		--
		UPDATE [dbo].[BatchScheduleExecution]
			SET [BatchId] = @BatchId,
				[Status] = @Status,
				[LastModifiedDate] = GETUTCDATE(),
				[LastModifiedByUserId] = @UserId,
				[TimeStamp] = GETUTCDATE()
			WHERE [VCC] = @VCC
				AND [Id] = @Id

		-- SI NO SE ACTUALIZO NINGUN REGISTRO, LO INSERTAMOS
		IF @@ROWCOUNT = 0
		BEGIN
			--select * from BatchSchedule (nolock)
			-- INSERTAMOS EL REGISTRO EN LA TABLA DE ERROR
			INSERT INTO [dbo].[BatchScheduleExecution] (
					[VCC], [Id], [BatchScheduleId], [BatchId], [ExecutionDate], 
					[Order], [Status], [CreatedDate], [CreatedByUserId], [TimeStamp]
				) VALUES (
					@VCC, @Id, @BatchScheduleId, @BatchId, @ExecutionDate, 
					@Order, @Status, GETUTCDATE(), @UserId, GETUTCDATE()
				)
		END

		-- EN CASO EL STATUS SEA BATCH CREADO O INICIADO, ACTUALIZAMOS EL REGISTRO DE LA AGENDA
		IF @Status = 'REGISTERED_ON_REDIS' OR @Status = 'BATCH_CREATED' OR @Status = 'BATCH_STARTED'
			UPDATE BS 
				SET BS.[LastExecutionDate] = BSE.[ExecutionDate],
					BS.[Status] = 'RUNNING',
					BS.[LastModifiedDate] = GETUTCDATE(),
					BS.[LastModifiedByUserId] = @UserId,
					BS.[TimeStamp] = GETUTCDATE()
				FROM [dbo].[BatchSchedule] BS 
				INNER JOIN [dbo].[BatchScheduleExecution] BSE
					ON BSE.[VCC] = BS.[VCC]	
						AND BSE.[BatchScheduleId] = BS.[Id]
				WHERE BS.[VCC] = @VCC 
					AND BSE.[Id] = @Id
						

	END TRY
    BEGIN CATCH
        --IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- ESTA FUNCION CAMBIA EL ESTADO DE UN BATCH DEL SISTEMA
CREATE PROCEDURE [dbo].[SaveBatchStatus] (
	@VCC VARCHAR(50) = NULL,
	@BatchTypeId VARCHAR(100) = NULL,
	@SystemService VARCHAR(50) = NULL,
	@Status VARCHAR(50) = NULL
	)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		
		-- VALIDAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @VCC IS NULL 
			OR @BatchTypeId IS NULL 
			OR @Status IS NULL
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		-- VERIFICAMOS QUE EL BATCH TYPE SEA UN TIPO CONOCIDO DEL SISTEMA
		IF @SystemService NOT IN ('TRANSCRIPTION', 'SEMANTIC_ANALYSIS', 'COGNITIVE_ANALYSIS', 'GENERATIVE_ANALYSIS', 'ADVANCED_AI_ANALYSIS')
			-- LANZAMOS MENSAJE AVISANDO QUE HAN SOLICITADO UNA OPERACION DESCONOCIDA
			RAISERROR('DATABASE_ERROR_UNKNOWN_SYSTEM_SERVICE', 16, 16)

		-- DECLARAMOS UNA VARIABLE PARA TIPO DE BATCH CORRESPONDIENTE AL BATCHTYPEID
		DECLARE @BATCH_TYPE_NAME VARCHAR(50) = NULL

		-- IDENTIFICAMOS EL TIPO BATCH SEGUN EL BATCHTYPEID Y SERVICIO DEL SISTEMA
		IF @SystemService = 'TRANSCRIPTION'
		BEGIN
			-- VERIFICAMOS SI ES DEL TIPO PROCESS
			IF EXISTS (SELECT 1 FROM [dbo].[Process] P WHERE [VCC] = @VCC AND [Name] = @BatchTypeId)
				SET @BATCH_TYPE_NAME = 'TRANSCRIPTION'

			-- VERIFICAMOS SI ES DEL TIPO EVALUATION_PLAN
			IF EXISTS (SELECT 1 FROM [dbo].[EvaluationPlan] P WHERE [Instance] = @VCC AND [ID] = @BatchTypeId)
				SET @BATCH_TYPE_NAME = 'EVALUATION_PLAN'
		END
		ELSE IF @SystemService IN ('SEMANTIC_ANALYSIS', 'COGNITIVE_ANALYSIS', 'GENERATIVE_ANALYSIS', 'ADVANCED_AI_ANALYSIS')
		BEGIN
			-- VERIFICAMOS SI ES DEL TIPO STUDY
			IF EXISTS (SELECT 1 FROM [dbo].[Study] P WHERE [VCC] = @VCC AND [Name] = @BatchTypeId)
				SET @BATCH_TYPE_NAME = 'STUDY'

			-- VERIFICAMOS SI ES DEL TIPO EVALUATION_PLAN
			IF EXISTS (SELECT 1 FROM [dbo].[EvaluationPlan] P WHERE [Instance] = @VCC AND [ID] = @BatchTypeId)
				SET @BATCH_TYPE_NAME = 'EVALUATION_PLAN'
		END
		
		-- VERIFICAMOS QUE EL BATCH TYPE SEA UN TIPO CONOCIDO DEL SISTEMA
		IF @BATCH_TYPE_NAME <> 'TRANSCRIPTION' AND @BATCH_TYPE_NAME <> 'STUDY' AND @BATCH_TYPE_NAME <> 'EVALUATION_PLAN'
			-- LANZAMOS MENSAJE AVISANDO QUE HAN SOLICITADO UNA OPERACION DESCONOCIDA
			RAISERROR('DATABASE_ERROR_UNKNOWN_BATCH_TYPE', 16, 16)

		-- ACTUALIZAMOS EL BATCH SEGUN EL SERVICIO DEL SISTEMA
		IF @BATCH_TYPE_NAME = 'TRANSCRIPTION'
		BEGIN
			-- ACTUALIZAMOS EL REGISTRO DE PROCESS
			UPDATE [dbo].[Process] 
				SET [StatusDescription] = @Status, 
					[Status] = @Status 
				WHERE [VCC] = @VCC
					AND [Name] = @BatchTypeId
		END 
		ELSE IF @BATCH_TYPE_NAME = 'STUDY'
		BEGIN
			-- ACTUALIZAMOS EL REGISTRO DE STUDY
			UPDATE [dbo].[Study] 
				SET [StatusDescription] = @Status, 
					[BatchStatus] = @Status,
					[LastModifiedDate] = CURRENT_TIMESTAMP,
					[TimeStamp] = CURRENT_TIMESTAMP 
				WHERE [VCC] = @VCC
					AND [Name] = @BatchTypeId
		END
		ELSE IF @BATCH_TYPE_NAME = 'EVALUATION_PLAN'
		BEGIN
			-- ACTUALIZAMOS EL REGISTRO DE EVALUATION_PLAN
			UPDATE [dbo].[EvaluationPlan] 
				SET [StatusDescription] = @Status, 
					-- [Status] = @Status, 
					[LastModifiedDate] = CURRENT_TIMESTAMP,
					[TimeStamp] = CURRENT_TIMESTAMP
				WHERE [Instance] = @VCC
					AND [ID] = @BatchTypeId
		END

		-- VERIFICAMOS SI SE ACTUALIZO EL REGISTRO
		IF @@ROWCOUNT = 0
			RAISERROR('DATABASE_ERROR_BATCH_SUMMARY_STATUS_UPDATE_FAILED', 16, 16)

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

	END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveCampaignEvaluationFormat] (
	@Instance Varchar(50) = NULL, 				@IdEvaluationFormat Varchar(50) = NULL,
	@IdCampaign Varchar(100) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	INSERT INTO [CampaignEvaluationFormat] (
		[Instance], [IdEvaluationFormat], [IdCampaign],
		[TimeStamp]
		)
	VALUES (
		@Instance, @IdEvaluationFormat, @IdCampaign,
		GETUTCDATE()
	)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


CREATE PROCEDURE [dbo].[SaveCategory] (
	@VCC Varchar(50) = NULL, 					@Name Varchar(50) = NULL,
	@Description Varchar(50) = NULL,			@CreatedDate Datetime = NULL,               @CreatedByUserId Varchar(50) = NULL,		
	@LastModifiedDate Datetime = NULL,			@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [Category]
	SET [Description] = @Description,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
    AND [Name] = @Name

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [Category] (
			[VCC], [Name], [Description],
			[CreatedDate], [CreatedByUserId], [LastModifiedDate], 
			[LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
			@VCC, @Name, @Description,
			@CreatedDate, @CreatedByUserId, @LastModifiedDate, 
			@LastModifiedByUserId, GETUTCDATE()
		)
	END
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


CREATE PROCEDURE [dbo].[SaveCategoryPermission] (
	@VCC Varchar(50) = NULL, 					@Category Varchar(50) = NULL,					@Position Int = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [CategoryPermissions]
	SET [Category] = @Category,
		[Position] = CASE WHEN @Position IS NULL THEN [Position] ELSE @Position END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [CategoryPermissions] (
			[VCC], [Category], [Position], [TimeStamp]
			)
		VALUES (
			@VCC, @Category, @Position, GETUTCDATE()
		)
	END
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveCleanerSummary] (
@Task         VARCHAR(15),
@Status       BIT,
@Description  VARCHAR(250)
)
AS
BEGIN
	SET NOCOUNT ON
	
	INSERT INTO [dbo].[CleanerSummary]([Task],[Status],[Description],[CreatedAt]) VALUES(@Task,@Status,@Description,GETUTCDATE())
	
	SELECT 1 AS 'result'
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveClient] (
	@VCC Varchar(50) = NULL, 					@ID Varchar(50) = NULL,						@Description Varchar(50) = NULL,
	@Country Varchar(50) = NULL,				@Contact Varchar(50) = NULL,				@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,		@LastModifiedDate Datetime = NULL,			@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [Client]
	SET [Description] = @Description,
		[Country] = @Country,
		[Contact] = @Contact,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
    AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [Client] (
			[VCC], [ID], [Description],
			[Country], [Contact], [CreatedDate],
			[CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId],
			[TimeStamp]
			)
		VALUES (
			@VCC, @ID, @Description,
			@Country, @Contact, @CreatedDate,
			@CreatedByUserId, @LastModifiedDate, @LastModifiedByUserId,
			GETUTCDATE()
		)
		
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


CREATE PROCEDURE [dbo].[SaveCognitiveEngineMigrateStatus]
(
	@VCC	          VARCHAR(50),
	@Model            VARCHAR(50),
	@Key              VARCHAR(100),
	@Status           INT,
	@StatusCode       VARCHAR(50),
	@NlusToMigrate	  VARCHAR(MAX) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	--ACTUALIZAR EL ESTADO DE MIGRACION
	DECLARE @SQL VARCHAR(MAX)
	SET @SQL = ''

    IF(@Model = '*')
    BEGIN 
          UPDATE [dbo].[NLU] SET [StatusCode] = @StatusCode,[Status] = @Status WHERE [VCC] = @VCC AND [IsSystemApp] = 0 AND [Status] <> 4
          UPDATE I SET I.[StatusCode] = @StatusCode, I.[Status] = @Status FROM [dbo].[Intent] I JOIN [dbo].[NLU] N 
          ON (I.[VCC] = N.[VCC] AND I.[AppId] = N.[AppId])  WHERE I.[VCC] = @VCC AND N.[IsSystemApp] = 0 AND I.[Status] <> 4
          UPDATE [dbo].[Entity] SET [StatusCode] = @StatusCode, [Status] = @Status WHERE [VCC] = @VCC AND [Status] <> 4
          
          UPDATE U SET U.[StatusCode] = @StatusCode, U.[Status] = @Status FROM [dbo].[Utterance] U JOIN [dbo].[NLU] N 
          ON (U.[VCC] = N.[VCC] AND U.[AppId] = N.[AppId]) WHERE U.[VCC] = @VCC AND N.[IsSystemApp] = 0 AND U.[Status] <> 4
    END--END MODEL * 
    ELSE 
    BEGIN --NLU
    	 IF(@Model = 'nlu')
			  BEGIN
			  	-- STATUS 4 SIGNIFICA QUE ESTA CANCELADO POR VARIOS INTENTOS ERRONEOS
				 IF(@Key = '*')
				 UPDATE [dbo].[NLU] SET [StatusCode] = @StatusCode,[Status] = @Status WHERE [VCC] = @VCC AND [IsSystemApp] = 0 AND [Status] <> 4
				 ELSE
				 BEGIN 
				  IF(@Key = 'AVAILABLE')
				    --ACTUALIZA A DISPONIBLE TODOS LOS QUE MIGRARON
				    UPDATE [dbo].[NLU] SET [StatusCode] = @StatusCode,[Status] = @Status  WHERE [VCC] = @VCC AND [Status] = 8 AND [IsSystemApp] = 0
				  IF(@Key = 'AVAILABLE_TM')
				    --ACTUALIZA A DISPONIBLE TODOS LOS QUE MIGRARON
				    UPDATE [dbo].[NLU] SET [StatusCode] = @StatusCode,[Status] = @Status  WHERE [VCC] = @VCC
				   IF(@Key = 'TOTAL_MIGRATION')
				   BEGIN
				    --ACTUALIZA A DISPONIBLE TODOS LOS QUE MIGRARON
					SET @SQL= ' UPDATE [dbo].[NLU] SET [StatusCode] = ' + '''' + @StatusCode + '''' + ',[Status] = '
					+ CAST(@Status AS VARCHAR) + '  WHERE [VCC] = ' + '''' +@VCC+ '''' + 'AND CognitiveEngine IN (' + '''' +'RASA' + '''' +',' + ''''+'RASAAPI'+ ''''+ ')'
					+ 'AND [AppId] In (' + '''' + @NlusToMigrate +'''' +')'
					--PRINT @SQL
					EXEC (@SQL)
				   -- UPDATE [dbo].[NLU] SET [StatusCode] = @StatusCode,[Status] = @Status  WHERE [VCC] = @VCC AND CognitiveEngine IN ('RASA','RASAAPI')
				   END
				  ELSE
				    UPDATE [dbo].[NLU] SET [StatusCode] = @StatusCode,[Status] = @Status  WHERE [VCC] = @VCC AND [Name] = @key AND [IsSystemApp] = 0
				 END
				 
			  END
		  ELSE 
		   BEGIN --INTENT
		   
			  IF(@Model = 'intent')
			    BEGIN 
			    
			     IF(@Key = '*')
				  UPDATE I SET I.[StatusCode] = @StatusCode, I.[Status] = @Status FROM [dbo].[Intent] I JOIN [dbo].[NLU] N 
                  ON (I.[VCC] = N.[VCC] AND I.[AppId] = N.[AppId])  WHERE I.[VCC] = @VCC AND N.[IsSystemApp] = 0 AND I.[Status] <> 4
				 ELSE
				 UPDATE [dbo].[Intent] SET [StatusCode] = @StatusCode, [Status] = @Status  WHERE [VCC] = @VCC AND [IntentId] = @key 
				 
			     END
			   ELSE
			     BEGIN --ENTITY
			       
				    IF(@Model = 'entity')
				    BEGIN 
				    
					     IF(@Key = '*')
						 UPDATE [dbo].[Entity] SET [StatusCode] = @StatusCode, [Status] = @Status WHERE [VCC] = @VCC AND [Status] <> 4
						 ELSE
						 UPDATE [dbo].[Entity] SET [StatusCode] = @StatusCode, [Status] = @Status  WHERE [VCC] = @VCC AND [ID] = @key 
						 
			        END--END ENTITY
			        ELSE 
                        BEGIN --UTTERANCE
	                        IF(@Model = 'utterance')
	                        BEGIN
	                         IF(@Key = '*')
							  UPDATE U SET U.[StatusCode] = @StatusCode, U.[Status] = @Status FROM [dbo].[Utterance] U JOIN [dbo].[NLU] N 
							  ON (U.[VCC] = N.[VCC] AND U.[AppId] = N.[AppId]) WHERE U.[VCC] = @VCC AND N.[IsSystemApp] = 0 AND U.[Status] <> 4
							 ELSE
							 UPDATE [dbo].[Utterance] SET [StatusCode] = @StatusCode, [Status] = @Status  WHERE [VCC] = @VCC AND [Id] = @key 
	                       
	                         END 
                      
						 
	
			             END--UTTERANCE
			       
			         
			    END--ENTITY
			    
		  
		  END -- INTENT
    END--NLU

	SELECT 1 AS result

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveCustomField] (
	@VCC VARCHAR(50) = NULL, 				
	@ID VARCHAR(100) = NULL,
	@Description VARCHAR(250) = NULL,					
	@Type VARCHAR(25) = NULL,
	@Label VARCHAR(50) = NULL,
	@CreatedByUserId VARCHAR(50) = NULL,
	@LastModifiedByUserId VARCHAR(50) = NULL
) AS
BEGIN
	SET XACT_ABORT, NOCOUNT ON;

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- VERIFICAMOS SI EL CUSTOMFIELD ESTA RELACIONADO A ALGUN FORMATO EN USO
		IF EXISTS (
			SELECT DISTINCT --efcf.CustomFieldId,
							EP.[ID],
							EP.[Description]
				FROM [dbo].[EvaluationFormatCustomField] EFCF (NOLOCK)
				LEFT JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
					ON EP.[IdEvaluationFormat] = EFCF.[EvaluationFormatId]
				WHERE EFCF.[VCC] = @VCC 
					AND EFCF.[CustomFieldId] = @ID
					AND EP.[Instance] IS NOT NULL
		)	
			RAISERROR('DATABABASE_ERROR_CUSTOM_FIELD_ON_USE', 16, 16)

		UPDATE [CustomField]
			SET [ID] = @ID,
				[Description] = @Description,
				[Type] = @Type,
				[Label] = @Label,
				[LastModifiedDate] = GETUTCDATE(),
				[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
				[TimeStamp] = GETUTCDATE()
			WHERE [VCC] = @VCC
				AND [ID] = @ID

		IF @@ROWCOUNT = 0
			INSERT INTO [CustomField] (
				[VCC], 
				[ID],
				[Description], 
				[Type], 
				[Label],
				[CreatedDate],
				[CreatedByUserId],
				[TimeStamp]
			)
			VALUES (
				@VCC, 
				@ID,
				@Description, 
				@Type, 
				@Label,
				GETUTCDATE(),
				@CreatedByUserId,
				GETUTCDATE()
			)

		SELECT 1 AS 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

    SET NOCOUNT OFF
END


GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveCustomFieldResult] (
	@VCC VARCHAR(50) = NULL, 				
	@ID VARCHAR(100) = NULL,
	@Description VARCHAR(150) = NULL,
	@Order INT = NULL,
	@CustomFieldId VARCHAR(100) = NULL,
	@CreatedByUserId VARCHAR(50) = NULL,
	@LastModifiedByUserId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	UPDATE [CustomFieldResult]
		SET [Description] = @Description,
			[Order] = @Order,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [ID] = @ID
			AND [CustomFieldId] =  @CustomFieldId

    IF @@ROWCOUNT = 0
		INSERT INTO [CustomFieldResult] (
			[VCC], 
			[ID],
			[Description], 
			[Order],
			[CustomFieldId],
			[CreatedDate],
			[CreatedByUserId],
			[TimeStamp]
		)
		VALUES (
			@VCC, 
			@ID,
			@Description, 
			@Order,
			@CustomFieldId,
			GETUTCDATE(),
			@CreatedByUserId,
			GETUTCDATE()
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveDictionary] (
	@VCC					VARCHAR(50) = NULL,
	@Name					VARCHAR(50) = NULL,
	@Description			VARCHAR(250) = NULL,
	@Language				VARCHAR(5) = NULL,
	@IsSystemDictionary		BIT = NULL,
	@SaveUser				VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INSERTAMOS LA NUEVO DICCIONARIO
	INSERT INTO [dbo].[Dictionary] (
			[VCC], [Name], [Description], [Language], [IsSystemDictionary], 
			[CreatedDate], [CreatedByUserId], [TimeStamp]
		)
		VALUES (
			@VCC, @Name, @Description, @Language, @IsSystemDictionary, 
			GETUTCDATE(), @SaveUser, GETUTCDATE()
		)
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveDictionaryWord] (
	@VCC					VARCHAR(50) = NULL,
	@DictionaryId			VARCHAR(50) = NULL,
	@ID						VARCHAR(50) = NULL,
	@Word					VARCHAR(50) = NULL,
	@SaveUser				VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- VERIFICAMOS SI EL ID EXISTE EN LOS PARAMETROS DE ENTRADA
		IF @ID IS NULL
		BEGIN
			-- VALIDAMOS SI LA PALABRA YA EXISTE EN LA TABLA
			IF EXISTS (
				SELECT * 
					FROM [dbo].[DictionaryWord] (NOLOCK) 
					WHERE [VCC] = @VCC 
						AND [DictionaryId] = @DictionaryId
						AND [Word] = @Word
			)
				RAISERROR('DATABASE_ERROR_DICTIONARY_WORD_ALREADY_EXISTS', 16, 16)

			-- VALIDAMOS SI EL DICCIONARIO A ALCANZO EL LIMITE DE 1000 PALABRAS
			IF 1000 <= (
				SELECT COUNT(*) 
					FROM [dbo].[DictionaryWord] (NOLOCK) 
					WHERE [VCC] = @VCC 
						AND [DictionaryId] = @DictionaryId
			)
				RAISERROR('DATABASE_ERROR_DICTIONARY_ALREADY_HAVE_1000_WORDS', 16, 16)
			
			-- INSERTAMOS LA NUEVA PALABRA
			INSERT INTO [dbo].[DictionaryWord] (
					[VCC], [DictionaryId], [ID], [Word], [CreatedDate], [CreatedByUserId], 
					[TimeStamp]
				)
				VALUES (
					@VCC, @DictionaryId, NEWID(), @Word, GETUTCDATE(), @SaveUser,
					GETUTCDATE()
				)
		END
		ELSE
		BEGIN
			-- ACTUALIZAMOS LA PALABRA POR EL ID
			UPDATE [dbo].[DictionaryWord]
				SET [Word] = @Word,
					[LastModifiedByUserId] = @SaveUser,
					[LastModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE()
				WHERE [VCC] = @VCC
					AND [DictionaryId] = @DictionaryId
					AND [ID] = @ID
		END
		
		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveDictionaryWordSoundLike] (
	@VCC					VARCHAR(50) = NULL,
	@DictionaryWordId		VARCHAR(50) = NULL,
	@ID						VARCHAR(50) = NULL,
	@SoundLike				VARCHAR(50) = NULL,
	@SaveUser				VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- VALIDAMOS SI LA PALABRA YA EXISTE EN LA TABLA
		IF EXISTS (
			SELECT * 
				FROM [dbo].[DictionaryWordSoundLike] (NOLOCK) 
				WHERE [VCC] = @VCC 
					AND [DictionaryWordId] = @DictionaryWordId
					AND [SoundLike] = @SoundLike
		)
			RAISERROR('DATABASE_ERROR_DICTIONARY_WORD_SOUND_LIKE_ALREADY_EXISTS', 16, 16)

		-- INSERTAMOS LA NUEVA PALABRA
		INSERT INTO [dbo].[DictionaryWordSoundLike] (
				[VCC], [DictionaryWordId], [ID], [SoundLike], [CreatedDate], [CreatedByUserId], 
				[TimeStamp]
			)
			VALUES (
				@VCC, @DictionaryWordId, NEWID(), @SoundLike, GETUTCDATE(), @SaveUser,
				GETUTCDATE()
			)
		
		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveDomain] (
	@VCC Varchar(50) = NULL, 					    @Domain Varchar(50) = NULL,						@Description Varchar(200) = NULL,
	@Client Varchar(50) = NULL,						@HasHttps Bit = NULL,							@ContentPort Varchar(50) = NULL,
	@Prefix Varchar(100) = NULL,					@IsMainDomain Bit = NULL,						@IsContentDomain Bit = NULL,
	@ContentDefaultRedirect Varchar(50) = NULL,		@IsDistributedService Bit = NULL,				@AssociatedDistributedService Varchar(50) = NULL,
	@CustomCode Varchar(MAX) = NULL,				@CreatedDate Datetime = NULL,					@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,				@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [Domain]
	SET [Description] = @Description,
		[Client] = @Client,
		[HasHttps] = @HasHttps,
		[ContentPort] = @ContentPort,
		[Prefix] = @Prefix,
		[IsMainDomain] = @IsMainDomain,
		[IsContentDomain] = @IsContentDomain,
		[ContentDefaultRedirect] = @ContentDefaultRedirect,
		[IsDistributedService] = @IsDistributedService,
		[AssociatedDistributedService] = @AssociatedDistributedService,
		[CustomCode] = @CustomCode,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
    AND [Domain] = @Domain

    IF @@ROWCOUNT = 0
		INSERT INTO [Domain] (
			[VCC], [Domain], [Description],
			[Client], [HasHttps], [ContentPort],
			[Prefix], [IsMainDomain], [IsContentDomain],
			[ContentDefaultRedirect], [IsDistributedService], [AssociatedDistributedService],
			[CustomCode], [CreatedDate], [CreatedByUserId],
			[LastModifiedDate], [LastModifiedByUserId],
			[TimeStamp]
			)
		VALUES (
			@VCC, @Domain, @Description,
			@Client, @HasHttps, @ContentPort,
			@Prefix, @IsMainDomain, @IsContentDomain,
			@ContentDefaultRedirect, @IsDistributedService, @AssociatedDistributedService,
			@CustomCode, @CreatedDate, @CreatedByUserId,
			@LastModifiedDate, @LastModifiedByUserId, GETUTCDATE()
		)
		
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEntity] (
    @VCC					VARCHAR(50) = NULL,
    @AppId					VARCHAR(50) = NULL,
    @ID						VARCHAR(50) = NULL,
    @Name					VARCHAR(50) = NULL,
    @Type					VARCHAR(50) = NULL,
    @CreatedByUserId		VARCHAR(50) = NULL,
    @LastModifiedByUserId	VARCHAR(50) = NULL
) AS
BEGIN
    SET NOCOUNT ON

    UPDATE [Entity]
    SET [Name] = @Name,
        [Type] = @Type,
        [LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
        [LastModifiedDate] = GETUTCDATE(),
        [TimeStamp] = GETUTCDATE()
    WHERE [VCC] = @VCC
      AND [AppId] = @AppId
      AND [ID] = @ID
      AND [Name] = @Name

    IF @@ROWCOUNT = 0
        BEGIN
            INSERT INTO [Entity] (
                [VCC], [AppId], [ID], [Name], [Type],
                [CreatedDate], [CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId], [TimeStamp],[Status],
                [StatusCode]
            )
            VALUES (
                       @VCC, @AppId, @ID, @Name, @Type,
                       GETUTCDATE(), @CreatedByUserId, GETUTCDATE(), @LastModifiedByUserId, GETUTCDATE(),1,'AVAILABLE'
                   )
        END

    --Marco el NLU como pendiente de entrenamiento
    UPDATE
        [dbo].[NLU]
    SET
        [NeedTrain] = 1
    WHERE
            [VCC] = @VCC AND [AppId] = @AppId

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveEnvironment] (
	@VCC VARCHAR(50) = NULL,
	@Name VARCHAR(50) = NULL,
	@Type VARCHAR(50) = NULL,
	@Description VARCHAR(200) = NULL,
	@Bucket VARCHAR(50) = NULL,
	@FTPDirectory VARCHAR(MAX) = NULL,
	@ApiHost VARCHAR(50) = NULL,
	@ApiPort VARCHAR(50) = NULL,
	@ApiUser VARCHAR(50) = NULL,
	@ApiPassword VARCHAR(50) = NULL, 		
	@DbDisabled Bit = NULL,
	@DbServer VARCHAR(50) = NULL,
	@DbPort VARCHAR(50) = NULL,				
	@DbUser VARCHAR(50) = NULL,
	@DbPassword VARCHAR(50) = NULL,
	@PbxDisabled BIT = NULL, 				
	@PbxServer VARCHAR(50) = NULL,
	@PbxPort VARCHAR(50) = NULL,
	@PbxUser VARCHAR(50) = NULL,			
	@PbxPassword VARCHAR(50) = NULL,
	@CreatedDate DATETIME = NULL,
	@CreatedByUserId VARCHAR(50) = NULL, 	
	@LastModifiedDate DATETIME = NULL,
	@LastModifiedByUserId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [Environment]
		SET [Type] = @Type,
			[Description] = @Description,
			[Bucket] = @Bucket,
			[FTPDirectory] = @FTPDirectory,
			[ApiHost] = @ApiHost,
			[ApiPort] = @ApiPort,
			[ApiUser] = @ApiUser,
			[ApiPassword] = CASE WHEN @ApiPassword IS NULL THEN [ApiPassword] ELSE @ApiPassword END,
			[DbDisabled] = @DbDisabled,
			[DbServer] = @DbServer,
			[DbPort] = @DbPort,
			[DbUser] = @DbUser,
			[DbPassword] = CASE WHEN @DbPassword IS NULL THEN [DbPassword] ELSE @DbPassword END,
			[PbxDisabled] = @PbxDisabled,
			[PbxServer] = @PbxServer,
			[PbxPort] = @PbxPort,
			[PbxUser] = @PbxUser,
			[PbxPassword] = CASE WHEN @PbxPassword IS NULL THEN [PbxPassword] ELSE @PbxPassword END,
			[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
			[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
			[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [Name] = @Name

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [Environment] (
			[VCC], [Name], [Type], [Bucket], [FTPDirectory],
			[Description], [ApiHost], [ApiPort],
			[ApiUser], [ApiPassword], [DbDisabled],
			[DbServer],[DbPort],[DbUser], 
			[DbPassword], [PbxDisabled], [PbxServer],
			[PbxPort], [PbxUser],[PbxPassword],
			[CreatedDate], [CreatedByUserId], 
			[LastModifiedDate],[LastModifiedByUserId], 
			[TimeStamp]
			)
		VALUES (
			@VCC, @Name, @Type, @Bucket, @FTPDirectory,
			@Description, @ApiHost, @ApiPort,
			@ApiUser, @ApiPassword, @DbDisabled, 
			@DbServer,@DbPort,@DbUser, 
			@DbPassword, @PbxDisabled, @PbxServer,
			@PbxPort, @PbxUser, @PbxPassword,
			@CreatedDate, @CreatedByUserId, 
			@LastModifiedDate, @LastModifiedByUserId, 
			GETUTCDATE()
		)
	END

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationAttachedSignature] (
	@VCC VARCHAR(50) = NULL, 
	@EvaluationFormatId VARCHAR(50) = NULL,
	@Description VARCHAR(100) = NULL,
	@Order Int = NULL,
	@CreatedByUserId VARCHAR(50) = NULL,
	@LastModifiedByUserId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	UPDATE [EvaluationFormatAttachedSignature]
		SET [Order] = @Order,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [EvaluationFormatId] = @EvaluationFormatId
			AND [Order] = @Order

    IF @@ROWCOUNT = 0
		INSERT INTO [EvaluationFormatAttachedSignature] (
			[VCC], 
			[EvaluationFormatId],
			[Description],
			[Order], 
			[CreatedDate],
			[CreatedByUserId],
			[TimeStamp]
		)
		VALUES (
			@VCC, 
			@EvaluationFormatId,
			@Description, 
			@Order,
			GETUTCDATE(),
			@CreatedByUserId,
			GETUTCDATE()
		)

	SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationBaseResult] (
	@VCC VARCHAR(50) = NULL,
	@EVALUATION_PLAN_ID VARCHAR(100) = NULL,
	@ABILITY_TYPE VARCHAR(50) = NULL
) AS

BEGIN
	-- ALERTA: NO ESTA PROBBADO PARA PLAN MANUAL

	-- ESTE SP INSERTA LOS RESULT BASE DE UN PLAN AUTOMATICO/MIXTO
	SET NOCOUNT ON

	-- SETTEAMOS EL TYPE ABILITY COMO AUTOMATIC PO DEFECTO
	IF @ABILITY_TYPE IS NULL SET @ABILITY_TYPE = 'AUTOMATIC'

	-- EMULAMOS UNA NUEVA TABLA VARIABLE PARA OBTENER LOS REGISTROS UNICOS QUE INSERTAREMOS EN LA TABLA EVALUATION_RESULT
	--  ORDENANDO POR SECTION Y PART

	DROP TABLE IF EXISTS #TMP_TRANSCRIBED_CALL_INTERACTIONS
	DROP TABLE IF EXISTS #TMP_RESULTS_FOR_EVALUATION 
	DROP TABLE IF EXISTS #TMP_EVALUATIONS_RESULTS

	-- CREAMOS UNA TABLA TEMPORAL PARA ALMACENAR LAS INTERACCIONES CALL YA TRANSRITAS
	SELECT A.[ID]
		INTO #TMP_TRANSCRIBED_CALL_INTERACTIONS
		FROM [dbo].[Audio] A (NOLOCK)
		INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK)
			ON PA.[VCC] = A.[VCC]
				AND PA.[AudioId] = A.[ID] 
		INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
			ON EP.[Instance] = A.[VCC]
				AND EP.[ProcessId] = PA.[ProcessId]
		WHERE A.[VCC] = @VCC 
			AND A.[IsTranscribed] = 1
			AND EP.[ID] = @EVALUATION_PLAN_ID

	-- SELECT * FROM #TMP_TRANSCRIBED_CALL_INTERACTIONS
	
	-- CREAMOS UNA TABLA TEMPORAL PARA ALMACENAR LAS INTERACCIONES TEXT YA TRANSRITAS
	SELECT AI.[ID]
		INTO #TMP_TRANSCRIBED_TEXT_INTERACTIONS
		FROM [dbo].[AsyncInteraction] AI (NOLOCK)
		INNER JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
			ON PAI.[VCC] = AI.[VCC]
				AND PAI.[InteractionId] = AI.[ID]
				AND PAI.[FormattedEndDate] = AI.[Folder]
		INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
			ON EP.[Instance] = AI.[VCC]
				AND EP.[ProcessId] = PAI.[ProcessId]
		WHERE AI.[VCC] = @VCC 
			AND AI.[IsTranscribed] = 1
			AND EP.[ID] = @EVALUATION_PLAN_ID

	-- CREAMOS UNA TABLA TEMPORAL PARA ALMACENAR LOS EVALUATION_RESULT YA REGISTRADOS
	SELECT ER.*
		INTO #TMP_EVALUATIONS_RESULTS
		FROM [dbo].[EvaluationResult] ER (NOLOCK) 
		WHERE ER.[Instance] = @VCC 
			AND ER.[IdEvaluationPlan] = @EVALUATION_PLAN_ID

	-- CREAMOS UN INDICE PARA LA TEMPORAL
	CREATE NONCLUSTERED INDEX [TMP_EVALUATION_RESULTS_01] ON #TMP_EVALUATIONS_RESULTS ([Instance], [IdEvaluationPlan], [IdInteraction], [IdAbility], [IdAgent], [IdAbilityEvaluationFormat])

	-- SELECT * FROM #TMP_TRANSCRIBED_TEXT_INTERACTIONS
	
	-- GENERAMOS REGISTROS PARA TODAS LAS INTERACCIONES
	;WITH VALID_INTERACTIONS_CTE AS (
		SELECT DISTINCT [ID] FROM #TMP_TRANSCRIBED_CALL_INTERACTIONS
		UNION ALL
		SELECT DISTINCT [ID] FROM #TMP_TRANSCRIBED_TEXT_INTERACTIONS
	)
	, INTERACTIONS_ALL_CTE AS (
		SELECT IEP.[Instance],
				IEP.[IdEvaluationPlan],
				IEP.[IdCampaign],
				IEP.[IdInteraction],
				IEP.[IdAgent],
				IEP.[InteractionDateStart],
				IEP.[InteractionDateEnd],
				IEP.[InteractionType],
				IEP.[ContactId],
				RN = ROW_NUMBER() OVER(PARTITION BY IEP.[Instance],
													IEP.[IdEvaluationPlan],				
													IEP.[IdCampaign],
													IEP.[IdInteraction],
													IEP.[IdAgent],
													--IEP.[InteractionDateStart], -- DUPLICA REGISTROS CUANDO HAY MAS DE 1 SECTION Y PART
													--IEP.[InteractionDateEnd], -- DUPLICA REGISTROS CUANDO HAY MAS DE 1 SECTION Y PART
													IEP.[InteractionType]
													--IEP.[ContactId] -- DUPLICA REGISTROS CUANDO HAY MAS DE 1 SECTION Y PART
										ORDER BY IEP.[InteractionSection] ASC, IEP.[InteractionPart] ASC)
			FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
			INNER JOIN VALID_INTERACTIONS_CTE IA_CTE 
				ON IA_CTE.[ID] = IEP.[IdInteraction]
			WHERE IEP.[Instance] = @VCC
				AND IEP.[IdEvaluationPlan] = @EVALUATION_PLAN_ID
	)  -- SELECT * FROM INTERACTIONS_ALL_CTE RETURN
	, INTERACTIONS_SUMMARY_CTE AS (
		SELECT I_A_CTE.*
			FROM INTERACTIONS_ALL_CTE I_A_CTE
			WHERE I_A_CTE.RN = 1
	) -- SELECT * FROM INTERACTIONS_SUMMARY_CTE RETURN
	, INTERACTIONS_FIRST_FILTER_CTE AS (
		SELECT * 
			FROM INTERACTIONS_SUMMARY_CTE IS_CTE
			WHERE (
					IS_CTE.[InteractionType] = 'CALL' 
					AND 
					IS_CTE.[IdInteraction] IN (
						SELECT [ID] FROM #TMP_TRANSCRIBED_CALL_INTERACTIONS
					)
				)
				OR
				IS_CTE.[InteractionType] <> 'CALL' 
	)  -- SELECT * FROM INTERACTIONS_FIRST_FILTER_CTE RETURN /*
	-- TODO: EL SECOND_FILTER ES SOBRE LAS INTERACCIONES TEXT TRANSCRITAS
	, MAIN_CTE AS (
		SELECT EP.[Instance]										'vcc',
				EP.[ID]												'IdEvaluationPlan',
				EP.[IdEvaluationFormat]								'IdEvaluationFormat',
				HEF.[MinQualificationAcceptable]					'MinQualificationAcceptable',
				IFF_CTE.[IdInteraction]								'IdInteraction',
				IFF_CTE.[IdCampaign]								'IdCampaign',
				IFF_CTE.[IdAgent]									'IdAgent',
				EP.[ProcessId]										'ProcessId',
				HAEF.[IdGroup]										'IdGroup',
				HAEF.[IdSubGroup]									'IdSubGroup',
				HAEF.[IdAbility]									'IdAbility',
				HAEF.[Score]										'MaxAbilityScore',
				HS.[Value]											'AbilityInnerScore',
				HS.[Description]									'ScoreDescription',
				IFF_CTE.[IdCampaign]								'CampaignDescription',
				IFF_CTE.[IdAgent]									'AgentDescription',
				IFF_CTE.[InteractionDateStart]						'InteractionDateini',
				IFF_CTE.[InteractionDateEnd]						'InteractionDatefin',
				IFF_CTE.[ContactId]									'InteractionAddress',
				HAEF.[Order]										'AbilityOrder',
				IFF_CTE.[InteractionType]							'InteractionType',
				HAEF.[ID]											'IdAbilityEvaluationFormat',
				HAEF.[Conditional]									'Conditional'
			FROM [dbo].[EvaluationPlan] EP (NOLOCK)
			INNER JOIN [dbo].[HistoricEvaluationFormat] HEF (NOLOCK)
				ON HEF.[VCC] = EP.[Instance]
					AND HEF.[EvaluationPlanId] = EP.[ID]
			INNER JOIN [dbo].[HistoricAbilityEvaluationFormat] HAEF (NOLOCK)
				ON HAEF.[VCC] = EP.[Instance]
					AND HAEF.[EvaluationPlanId] = EP.[ID]
			INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
				ON HA.[VCC] = EP.[Instance]
					AND HA.[EvaluationPlanId] = EP.[ID]
					AND HA.[ID] = HAEF.[IdAbility]
			INNER JOIN [dbo].[HistoricScore] HS (NOLOCK)
				ON HS.[VCC] = EP.[Instance]
					AND HS.[EvaluationPlanId] = EP.[ID]
					AND HS.[IdAbility] = HA.[ID]
					AND HS.[Value] = IIF(HA.[IsNegative] = 1, 100, 0)
			INNER JOIN INTERACTIONS_FIRST_FILTER_CTE IFF_CTE 
				ON IFF_CTE.[Instance] = EP.[Instance]
					AND IFF_CTE.[IdEvaluationPlan] = EP.[ID]
			LEFT JOIN #TMP_EVALUATIONS_RESULTS TMP_ER 
				ON TMP_ER.[Instance] = EP.[Instance]
					AND TMP_ER.[IdEvaluationPlan] = EP.[ID]
					AND TMP_ER.[IdInteraction] = IFF_CTE.[IdInteraction]
					AND TMP_ER.[IdAbility] = HA.[ID]
					AND ISNULL(TMP_ER.[IdAgent], '') = ISNULL(IFF_CTE.[IdAgent], '')
					AND TMP_ER.[IdAbilityEvaluationFormat] = HAEF.[ID]
			WHERE TMP_ER.[Instance] IS NULL
				AND EP.[Instance] = @VCC
				AND IFF_CTE.[IdInteraction] IS NOT NULL
			    AND EP.[ID] = @EVALUATION_PLAN_ID
				AND (
					(
						@ABILITY_TYPE = 'AUTOMATIC' 
						AND 
						HA.[Type] = 'MANUAL_AUTOMATIC'
					)
					OR
					(
						@ABILITY_TYPE = 'MANUAL' 
						AND 
						HA.[Type] = 'MANUAL'
					)
				)
	)  -- SELECT * FROM MAIN_CTE RETURN
	
		
	-- INSERTAMOS TODOS LOS REGISTROS EN UNA TABLA TEMPORAL
	SELECT * -- 0 'DEBUG'
		INTO #TMP_RESULTS_FOR_EVALUATION
		FROM MAIN_CTE
	
	-- ELIMINAMOS LOS REGISTROS PARA INTERACCIONES TEXT NO TRANSCRITAS
	/*
	DELETE FROM TMP_RESULTS_FOR_EVALUATION
		WHERE [InteractionType] <> 'CALL' 
			AND [IdInteraction] NOT IN (
				SELECT [ID] FROM #TMP_NOT_TRANSCRIBED_TEXT_INTERACTIONS
			)
			*/

	-- TENEMOS LOS REGISTROS PARA INSERTAR EN EVALUATIONRESULT
	-- SELECT * FROM #TMP_RESULTS_FOR_EVALUATION RETURN

	-- INSERTAMOS SOLO LAS PRIMERAS OCURRENCIAS PARA CADA INTERACCION
	INSERT INTO [dbo].[EvaluationResult] (
		[Instance], [IdCampaign], [IdInteraction],
		[IdAgent], [IdEvaluationFormat], [MinQualificationAcceptable],
		[IdGroup], [IdSubGroup], [IdAbility],
		[Qualification], [OwnScore], [AbilityInnerScore],
		[IdEvaluator], [EvaluationDate], [MaxAbilityScore],
		[IdEvaluationPlan], [CreatedDate], [CreatedByUserId],
		[LastModifiedDate], [LastModifiedByUserId], [TimeStamp],
		[Comment], [NotApplies], [ScoreDescription],
		[CampaignDescription], [AgentDescription], [InteractionDateini],
		[InteractionDatefin], [InteractionAddress], [AbilityOrder],
		[InteractionType], [IdAbilityEvaluationFormat],  [IdAuditor],
		[Conditional]
	)
		SELECT TMP_RFE.[vcc], TMP_RFE.[IdCampaign], TMP_RFE.[IdInteraction],
				ISNULL(TMP_RFE.[IdAgent], ''), TMP_RFE.[IdEvaluationFormat], TMP_RFE.[MinQualificationAcceptable],
				TMP_RFE.[IdGroup], TMP_RFE.[IdSubGroup], TMP_RFE.[IdAbility],
				(TMP_RFE.[MaxAbilityScore] * TMP_RFE.[AbilityInnerScore] / 100), (TMP_RFE.[MaxAbilityScore] * TMP_RFE.[AbilityInnerScore] / 100), TMP_RFE.[AbilityInnerScore],
				'SPEECH', GETUTCDATE(), TMP_RFE.[MaxAbilityScore],
				TMP_RFE.[IdEvaluationPlan], GETUTCDATE(), 'SPEECH',
				--'100', TMP_RFE.[IdEvaluationPlan], GETUTCDATE(),
				GETUTCDATE(), 'SPEECH', GETUTCDATE(),
				NULL, '0', TMP_RFE.[ScoreDescription],
				TMP_RFE.[CampaignDescription], TMP_RFE.[AgentDescription], TMP_RFE.[InteractionDateini],
				TMP_RFE.[InteractionDatefin], TMP_RFE.[InteractionAddress], TMP_RFE.[AbilityOrder],
				TMP_RFE.[InteractionType], TMP_RFE.[IdAbilityEvaluationFormat], 'SPEECH',
				TMP_RFE.[Conditional]
			FROM #TMP_RESULTS_FOR_EVALUATION TMP_RFE

	-- ELIMINAMOS LAS TABLAS TEMPORALES
	DROP TABLE #TMP_TRANSCRIBED_CALL_INTERACTIONS
	DROP TABLE #TMP_RESULTS_FOR_EVALUATION
	DROP TABLE #TMP_EVALUATIONS_RESULTS

	SET NOCOUNT OFF

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationCustomFieldResult] (
	@VCC VARCHAR(50) = NULL, 				
	@EvaluationPlanId VARCHAR(100) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@AuditorId VARCHAR(50) = NULL,					
	@AgentId VARCHAR(50) = NULL,
	@CustomFieldId VARCHAR(100) = NULL,
	@Result VARCHAR(150) = NULL,
	@CreatedByUserId VARCHAR(50) = NULL,
	@LastModifiedByUserId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	UPDATE [EvaluationCustomFieldResult]
		SET [Result] = @Result,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [EvaluationPlanId] = @EvaluationPlanId
			AND [InteractionId] = @InteractionId
			AND [AuditorId] = @AuditorId
			AND [AgentId] = @AgentId
			AND [CustomFieldId] = @CustomFieldId

    IF @@ROWCOUNT = 0
		INSERT INTO [EvaluationCustomFieldResult] (
			[VCC], 
			[EvaluationPlanId],
			[InteractionId], 
			[AuditorId], 
			[AgentId],
			[CustomFieldId],
			[Result],
			[CreatedDate],
			[CreatedByUserId],
			[TimeStamp]
		)
		VALUES (
			@VCC, 
			@EvaluationPlanId,
			@InteractionId,
			@AuditorId,
			@AgentId,
			@CustomFieldId,
			@Result,
			GETUTCDATE(),
			@CreatedByUserId,
			GETUTCDATE()
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE procedure [dbo].[SaveEvaluationEfectivitySample](
@VCC varchar(50),
@EfectivityPlanId varchar(50),
@EvaluationPlanId varchar(50),
@AuditorId varchar(50),
@SampleSize int
)
AS
BEGIN
	    
		IF (@SampleSize = -1)
		BEGIN
		UPDATE [dbo].[EvaluationResultEfectivity] 
			SET [AuditorId] = @AuditorId,
				[LastModifiedDate] = GETUTCDATE()
			WHERE 
			[VCC] = @VCC
			AND [EfectivityPlanId] = @EfectivityPlanId
			AND [EvaluationPlanId] = @EvaluationPlanId

			SELECT 1 AS 'result'
		END	

		ELSE
		BEGIN
			IF NOT EXISTS (SELECT 1 FROM [dbo].[EvaluationResultEfectivity] (NOLOCK) WHERE [VCC] = @VCC
			AND [EfectivityPlanId] = @EfectivityPlanId
			AND [EvaluationPlanId] = @EvaluationPlanId  )
			BEGIN 
				CREATE TABLE #RANDOM_INTERACTION(
				[IdInteraction] VARCHAR(250)
				)
				INSERT INTO #RANDOM_INTERACTION
				SELECT TOP (@SampleSize) [IEP].[IdInteraction]
										FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK) 
										WHERE IEP.[Instance] = @VCC
											  AND [IEP].[IdEvaluationPlan] = @EvaluationPlanId
											 -- AND IEP.[InteractionType] = @CURRENT_CHANNEL
										GROUP BY [IEP].[IdInteraction] 
										ORDER BY NEWID()

		

				INSERT INTO [dbo].[EvaluationResultEfectivity]						
				SELECT ER.[Instance],@EfectivityPlanId,ER.[IdEvaluationPlan],ER.[IdInteraction],ER.[IdAgent],ER.[IdAbilityEvaluationFormat],@AuditorId ,null,'CREATED',GETUTCDATE(),GETUTCDATE() 
				FROM [dbo].[EvaluationResult] ER (NOLOCK)
				INNER JOIN #RANDOM_INTERACTION RI ON 
				ER.IdInteraction = RI.IdInteraction 
				WHERE ER.Instance = @VCC AND
				ER.IdEvaluationPlan = @EvaluationPlanId

				SELECT 1 AS 'result'

			END
			ELSE
			BEGIN
			SELECT 0 AS 'result' FROM [dbo].[EvaluationResultEfectivity] WHERE [VCC] = @VCC
			AND [EfectivityPlanId] = @EfectivityPlanId
			AND [EvaluationPlanId] = @EvaluationPlanId
				
			END

			
		END

		
			
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveEvaluationFormat] (
	@Instance Varchar(50) = NULL, 
	@ID Varchar(50) = NULL,
	@Description Varchar(50) = NULL,
	@Type Varchar(50) = NULL,
	@EnvironmentType VARCHAR(50) = NULL,
	@ExecutionType VARCHAR(25) = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL,
	@MinQualificationAcceptable numeric(5,2),
	@MinAutomaticQualification NUMERIC(5, 2),
	@ManagementFormURL varchar(1000) = NULL,
	@Enabled bit = NULL, 
	@templateId Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [EvaluationFormat] SET
		[Description] = @Description,
		[Type] = @Type,
		[EnvironmentType] = @EnvironmentType,
		[MinQualificationAcceptable] = @MinQualificationAcceptable,
		[MinAutomaticQualification] = @MinAutomaticQualification,
		[ManagementFormURL] = @ManagementFormURL,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[Enabled] = CASE WHEN @Enabled IS NULL THEN 1 ELSE @Enabled END,
		[TimeStamp] = GETUTCDATE(),
		[templateId] = @templateId
	WHERE [Instance] = @Instance
	AND [ID] = @ID
	
	-- SI NO SE ACTUALIZO NINGUN REGISTRO LO INSERTAMOS
	IF @@ROWCOUNT = 0
	BEGIN
		-- INSERTAMOS LA DATA EN LA TABLA EF
		INSERT INTO [EvaluationFormat] (
			[Instance],[ID], [Description], [Type], 
			[ExecutionType], [EnvironmentType], [MinQualificationAcceptable], [MinAutomaticQualification], 
			[ManagementFormURL],
			[CreatedDate], [CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId],
			[Enabled], [TimeStamp], [templateId]
			)
		VALUES (
			@Instance, @ID, @Description, @Type, 
			@ExecutionType, @EnvironmentType, @MinQualificationAcceptable, @MinAutomaticQualification, 
			@ManagementFormURL,
			@CreatedDate, @CreatedByUserId, @LastModifiedDate, @LastModifiedByUserId,
			@Enabled, GETUTCDATE(), @templateId
		)

		-- EN CASO SE TRATE DE REALTIME, CREAMOS EL PLAN DE EVALUACION CORRESPONDIENTE
		IF @ExecutionType = 'REALTIME'
		BEGIN
			-- DEFINIR REGLAS PARA PLAN DE EVALUACION RT
			SELECT 'RT'
		END
	END

	SELECT CAST(Scope_Identity() AS INT) as 'idEvaluationFormat'
	

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationFormatAbilityScoreTipification] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationFormatId VARCHAR(50) = NULL,
	@ScoreId VARCHAR(50) = NULL,
	@ID VARCHAR(50) = NULL,
	@Description VARCHAR(250) = NULL,
	@Detail VARCHAR(500) = NULL,
	@Order INT = NULL,
	@UserId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EN CASO EXISTA
	UPDATE [EvaluationFormatAbilityScoreTipification]
		SET [Description] = @Description,
			[Detail] = @Detail,
			[Order] = @Order,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedByUserId] = @UserId
		WHERE [VCC] = @VCC
			AND [EvaluationFormatId] = @EvaluationFormatId
			AND [ScoreId] = @ScoreId
			AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [EvaluationFormatAbilityScoreTipification] (
			[VCC], 
			[EvaluationFormatId],
			[ScoreId],
			[ID],
			[Description], 
			[Detail],
			[Order],
			[CreatedDate],
			[CreatedByUserId]
			)
		VALUES (
			@VCC,
			@EvaluationFormatId,
			@ScoreId,
			@ID,
			@Description, 
			@Detail,
			@Order,
			GETUTCDATE(),
			@UserId
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveEvaluationFormatAbilityScoreTipificationItem] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationFormatId VARCHAR(50) = NULL,
	@ScoreId VARCHAR(50) = NULL,
	@ID VARCHAR(50) = NULL,
	@TipificationId VARCHAR(50) = NULL,
	@SecondLevelItemId VARCHAR(50) = NULL,
	@Description VARCHAR(250) = NULL,
	@Order INT = NULL,
	@UserId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EN CASO EXISTA
	UPDATE [EvaluationFormatAbilityScoreTipificationItem]
		SET [Description] = @Description,
			[Order] = @Order,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedByUserId] = @UserId
		WHERE [VCC] = @VCC
			AND [EvaluationFormatId] = @EvaluationFormatId
			AND [ScoreId] = @ScoreId
			AND [TipificationId] = @TipificationId
			AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [EvaluationFormatAbilityScoreTipificationItem] (
			[VCC], 
			[EvaluationFormatId],
			[ScoreId],
			[TipificationId], 
			[ID],
			[SecondLevelItemId],
			[Description], 
			[Order], 
			[CreatedDate],
			[CreatedByUserId]
			)
		VALUES (
			@VCC, 
			@EvaluationFormatId,
			@ScoreId,
			@TipificationId,
			@ID,
			@SecondLevelItemId,
			@Description, 
			@Order,
			GETUTCDATE(),
			@UserId
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationFormatAbilityScoreTipificationQuery] (
	@VCC							VARCHAR(50) = NULL,
	@EvaluationFormatId				VARCHAR(50) = NULL,
	@ScoreId						VARCHAR(50) = NULL,
	@TipificationId					VARCHAR(50) = NULL,
	@QueryId						VARCHAR(50) = NULL,
	@QueryContent					VARCHAR(1000) = NULL,
	@AcceptancePercentage			DECIMAL(18, 2) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EL REGISTRO EN CASO YA EXISTA
	UPDATE [EvaluationFormatAbilityScoreTipificationQuery]
		SET [QueryId] = @QueryId,
			[QueryContent] = @QueryContent,
			[AcceptancePercentage] = @AcceptancePercentage,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [EvaluationFormatId] = @EvaluationFormatId
 			AND [ScoreId] = @ScoreId
			AND [TipificationId] = @TipificationId

	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [EvaluationFormatAbilityScoreTipificationQuery] (
			[VCC], [EvaluationFormatId], [ScoreId], [TipificationId], [QueryId], 
			[QueryContent], [AcceptancePercentage], [TimeStamp]
		)
		VALUES (
			@VCC, @EvaluationFormatId, @ScoreId, @TipificationId, @QueryId, 
			@QueryContent, @AcceptancePercentage, GETUTCDATE()
		)
	END
	-- select * from [EvaluationFormatAbilityScoreTipificationQuery]
	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationFormatAttachedSignature] (
	@VCC VARCHAR(50) = NULL, 
	@EvaluationFormatId VARCHAR(50) = NULL,
	@Description VARCHAR(100) = NULL,
	@Order Int = NULL,
	@CreatedByUserId VARCHAR(50) = NULL,
	@LastModifiedByUserId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	UPDATE [EvaluationFormatAttachedSignature]
		SET [Order] = @Order,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [EvaluationFormatId] = @EvaluationFormatId
			AND [Order] = @Order

    IF @@ROWCOUNT = 0
		INSERT INTO [EvaluationFormatAttachedSignature] (
			[VCC], 
			[EvaluationFormatId],
			[Description],
			[Order], 
			[CreatedDate],
			[CreatedByUserId],
			[TimeStamp]
		)
		VALUES (
			@VCC, 
			@EvaluationFormatId,
			@Description, 
			@Order,
			GETUTCDATE(),
			@CreatedByUserId,
			GETUTCDATE()
		)

	SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationFormatCustomField] (
	@VCC VARCHAR(50) = NULL, 
	@ID VARCHAR(50) = NULL,
	@EvaluationFormatId VARCHAR(50) = NULL,
	@CustomFieldId VARCHAR(100) = NULL,
	@Order Int = NULL,
	@CreatedByUserId VARCHAR(50) = NULL,
	@LastModifiedByUserId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	UPDATE [EvaluationFormatCustomField]
		SET [Order] = @Order,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [EvaluationFormatId] = @EvaluationFormatId
			AND [CustomFieldId] = @CustomFieldId

    IF @@ROWCOUNT = 0
		INSERT INTO [EvaluationFormatCustomField] (
			[VCC], 
			[ID],
			[EvaluationFormatId],
			[CustomFieldId],
			[Order], 
			[CreatedDate],
			[CreatedByUserId],
			[TimeStamp]
		)
		VALUES (
			@VCC, 
			@ID,
			@EvaluationFormatId,
			@CustomFieldId, 
			@Order,
			GETUTCDATE(),
			@CreatedByUserId,
			GETUTCDATE()
		)

	SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationFormatScoreTipificationQuery] (
	@VCC							VARCHAR(50) = NULL,
	@EvaluationFormatId				VARCHAR(50) = NULL,
	@ScoreId						VARCHAR(50) = NULL,
	@TipificationId					VARCHAR(50) = NULL,
	@QueryId						VARCHAR(50) = NULL,
	@QueryContent					VARCHAR(1000) = NULL,
	@AcceptancePercentage			DECIMAL(18, 2) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EL REGISTRO EN CASO YA EXISTA
	UPDATE [EvaluationFormatScoreTipificationQuery]
		SET [QueryId] = @QueryId,
			[QueryContent] = @QueryContent,
			[AcceptancePercentage] = @AcceptancePercentage,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [EvaluationFormatId] = @EvaluationFormatId
			AND [ScoreId] = @ScoreId
			AND [TipificationId] = @TipificationId

	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [EvaluationFormatScoreTipificationQuery] (
			[VCC], [EvaluationFormatId], [ScoreId], [TipificationId], [QueryId], 
			[QueryContent], [AcceptancePercentage], [TimeStamp]
		)
		VALUES (
			@VCC, @EvaluationFormatId, @ScoreId, @TipificationId, @QueryId, 
			@QueryContent, @AcceptancePercentage, GETUTCDATE()
		)
	END

	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationPlan] (
	@Instance Varchar(50) = NULL,
	@ID Varchar(50) = NULL,
	@Modality Varchar(50) = NULL,
	@DateFin Datetime = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL,
	@IdEvaluationFormat Varchar(50) = NULL,
	@TranscriptionLanguage VARCHAR(25) = NULL,
	@EnvironmentId VARCHAR(100) = NULL,
	@Description Varchar(200) = NULL,
	@Enabled Bit = NULL,

	@FilterByContent BIT = NULL,
	@FilterByContentPercentage NUMERIC(3,0) = NULL,
	@EvaluatePercentage NUMERIC(3,0) = NULL,
	@ProcessAndEvaluate BIT = NULL,
	@FilterByContentQueryName VARCHAR(50) = NULL,
	@FilterByContentQuery VARCHAR(1000) = NULL,
	@EvaluationStarted BIT = NULL,
	@ProcessId VARCHAR(50) = NULL,
	@Status	NUMERIC(2,0) = NULL,
	@Evaluation_Mode VARCHAR(10) = NULL,
	@BatchScheduleId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [EvaluationPlan]
	SET
		[Modality] = @Modality,
		[DateFin] = @DateFin,
		[IdEvaluationFormat] = @IdEvaluationFormat,
		[TranscriptionLanguage] = @TranscriptionLanguage,
		[EnvironmentId] = @EnvironmentId,
		[Description] =@Description,
		[Enabled] = CASE WHEN @Enabled IS NULL THEN [Enabled] ELSE @Enabled END,
		[EvaluatePercentage] = @EvaluatePercentage,
		[ProcessAndEvaluate] = @ProcessAndEvaluate,
		[FilterByContent] = CASE WHEN @FilterByContent IS NULL THEN [FilterByContent] ELSE @FilterByContent END,
		[FilterByContentPercentage] = CASE WHEN @FilterByContentPercentage IS NULL THEN [FilterByContentPercentage] ELSE @FilterByContentPercentage END,
		[FilterByContentQueryName] = CASE WHEN @FilterByContentQueryName IS NULL THEN [FilterByContentQueryName] ELSE @FilterByContentQueryName END,
		[FilterByContentQuery] = CASE WHEN @FilterByContentQuery IS NULL THEN [FilterByContentQuery] ELSE @FilterByContentQuery END,
		
		[EvaluationStarted] = @EvaluationStarted,
		[ProcessId] = @ProcessId,
		[Status] = @Status,
		[Total] = CASE WHEN [Status] = 5 AND @Status = 1 THEN NULL ELSE [Total] END,
		[Progress] = CASE WHEN [Status] = 5 AND @Status = 1 THEN NULL ELSE [Progress] END,
		[Error] = CASE WHEN [Status] = 5 AND @Status = 1 THEN NULL ELSE [Error] END,
		[Percentage] = CASE WHEN [Status] = 5 AND @Status = 1 THEN NULL ELSE [Percentage] END,
		[Evaluation_Mode] = @Evaluation_Mode,
		
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[LastModifiedDate] = GETUTCDATE(),
		[TimeStamp] = GETUTCDATE()
	WHERE [Instance] = @Instance
    AND [ID] = @ID

    IF @@ROWCOUNT = 0
    BEGIN 
		INSERT INTO [EvaluationPlan] (
			[Instance], [ID], [Modality], [DateFin], 
			[IdEvaluationFormat], [TranscriptionLanguage], [EnvironmentId], [Description], 
			[Enabled], [FilterByContent], [FilterByContentPercentage], [EvaluatePercentage], 
			[ProcessAndEvaluate], [FilterByContentQueryName], [FilterByContentQuery], [EvaluationStarted], 
			[ProcessId], [Status], [Evaluation_Mode], [BatchScheduleId],
				
			[CreatedByUserId], [CreatedDate], [TimeStamp]
		)
		VALUES (
			@Instance, @ID, @Modality, @DateFin, 
			@IdEvaluationFormat, @TranscriptionLanguage, @EnvironmentId, @Description, 
			1, @FilterByContent, @FilterByContentPercentage, @EvaluatePercentage, 
			@ProcessAndEvaluate, @FilterByContentQueryName, @FilterByContentQuery, @EvaluationStarted, 
			@ProcessId, @Status, @Evaluation_Mode, @BatchScheduleId,
				
			@CreatedByUserId, GETUTCDATE(), GETUTCDATE()
		)
			   
		EXEC [dbo].[SaveHistoricEvaluationPlanData] @Instance,@ID,@IdEvaluationFormat
    END

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
/* Procedure para marcar el plan con cognitive como completado */
CREATE PROCEDURE [dbo].[SaveEvaluationPlanAbilityStatusTask] (
    @VCC Varchar(50) = NULL, @EvaluationPlanId Varchar(50) = NULL
)
AS
BEGIN
SET NOCOUNT ON

	UPDATE EvaluationPlan 
		SET [Status] = 7 
		WHERE [Instance] = @VCC
			AND [ID] = @EvaluationPlanId;

SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveEvaluationPlanCompleted] (
	@Instance Varchar(50) = NULL, 
	@IdEvaluationPlan Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [dbo].[EvaluationPlan]
		SET [Status] = 7,
			[Percentage] = 1,
			[StatusDescription] = CASE WHEN ISNULL([ERROR], 0) = 0 THEN 'COMPLETED' ELSE 'COMPLETED_WITH_ERROR' END
		WHERE NOT EXISTS (
			SELECT AEP.[idEvaluationPLan]
				FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				WHERE AEP.[idEvaluationPlan] = @IdEvaluationPlan
					AND AEP.[Instance] = @Instance
					AND AEP.[Completed] <> 1
					AND AEP.[Status] = 1
			)
			AND [Instance] = @Instance
			AND [ID] = @IdEvaluationPlan

	-- ACTUALIZAMOS EL STATUS DESCRIPTION CUANDO EXISTEN DESCARTADOS
	/*
	UPDATE [dbo].[EvaluationPlan]
		SET [StatusDescription] = 'COMPLETED_WITH_DISCARDED'
		WHERE EXISTS (
				SELECT AEP.[idEvaluationPLan]
					FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
					WHERE AEP.[idEvaluationPlan] = @IdEvaluationPlan
						AND AEP.[Instance] = @Instance
						AND AEP.[Status] = 0
			)
			AND [Instance] = @Instance
			AND [ID] = @IdEvaluationPlan
			AND (
				[Status] = 7
				OR
				[Status] = 8
			)
	*/

	-- ACTUALIZAMOS EL STATUS DESCRIPTION CUANDO EXISTEN DESCARTADOS
	UPDATE EP 
		SET EP.[StatusDescription] = 'COMPLETED_WITH_DISCARDED',
			--EP.[Total] = INNER_AEP.[Total],
			--EP.[Progress] = INNER_AEP.[Completed] + INNER_AEP.[Discarded]
			EP.[Percentage] = (INNER_AEP.[Completed] + INNER_AEP.[Discarded]) / INNER_AEP.[Total]
		FROM [dbo].[EvaluationPlan] EP 
		INNER JOIN (
			SELECT AEP.[Instance],
					AEP.[idEvaluationPlan],
					COUNT(AEP.[IdInteraction])														'Total',
					COUNT(CASE AEP.[Completed] WHEN 1 THEN 1 ELSE NULL END)							'Completed',
					COUNT(CASE AEP.[Status] WHEN 0 THEN 1 ELSE NULL END)							'Discarded',
					COUNT(CASE 
							WHEN AEP.[Completed] = 0 AND AEP.[Status] = 1 THEN 1 
							ELSE NULL 
							END
					)																				'Pending'
				FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				WHERE AEP.[Instance] = @Instance
					AND AEP.[idEvaluationPlan] = @IdEvaluationPlan
				GROUP BY AEP.[Instance], AEP.[idEvaluationPlan]
		) INNER_AEP
			ON INNER_AEP.[Instance] = EP.[Instance]
				AND INNER_AEP.[IdEvaluationPlan] = EP.[ID] 
		WHERE EP.[Instance] = @Instance
			AND [ID] = @IdEvaluationPlan
			AND (
				(
					EP.[Status] = 7
					AND 
					INNER_AEP.[Discarded] > 0
				)
				OR
				(
					EP.[Status] = 8
					AND
					INNER_AEP.[Pending] = 0
					AND 
					INNER_AEP.[Total] = INNER_AEP.[Discarded]
				)
			)

	-- ACTUALIZAMOS EL PLAN MANUA DE CREADO A EVALUANDO
	UPDATE [dbo].[EvaluationPlan]
		SET [Status] = 6
		WHERE EXISTS (
			SELECT AEP.[idEvaluationPLan]
				FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				WHERE AEP.[idEvaluationPlan] = @IdEvaluationPlan
					AND AEP.[Instance] = @Instance
					AND (
						AEP.[Completed] = 1
						OR 
						AEP.[Status] = 0
					)
			)
			AND [Instance] = @Instance
			AND [ID] = @IdEvaluationPlan
			AND [Status] = 1
			AND [Evaluation_Mode] = 'M'

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveEvaluationResult] (
	@Instance Varchar(50) = NULL,
	@Id Varchar(250) = NULL,
	@IdCampaign Varchar(100) = NULL,
	@IdInteraction Varchar(250) = NULL,
	@IdAgent Varchar(50) = NULL,
	@IdEvaluationFormat Varchar(50) = NULL,
	@MinQualificationAcceptable numeric(5,2) = 0,
	@IdGroup Varchar(50) = NULL,
	@IdSubGroup Varchar(50) = NULL,
	@IdAbility Varchar(50) = NULL,
	@Qualification numeric(5,2) = NULL,
	@OwnScore NUMERIC(5,2) = NULL,
	@InheritedScore NUMERIC(5,2) = NULL,
	@IdEvaluator Varchar(50) = NULL,
	@EvaluationDate Datetime = NULL,
	@MaxAbilityScore numeric(5,2) = NULL,
	@AbilityInnerScore NUMERIC(5,2) = NULL,
	@IdEvaluationPlan Varchar(50) = NULL,
	@Comment Varchar(1000) = NULL,
	@NotApplies bit = NULL,
	@ScoreDescription varchar(100) = NULL,
	@CampaignDescription varchar(100) = NULL,
	@AgentDescription varchar(100) = NULL,
	@InteractionDateini [datetime] = NULL,
	@InteractionDatefin [datetime] = NULL,
	@InteractionAddress VARCHAR(200) = NULL,
	@InteractionType varchar(50) = NULL,
	@AbilityOrder numeric(4,0) = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL,
	@Conditional varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- SETTEAMOS UN VALOR POR DEFECTO A LA NOTA MINIMA
	IF @MinQualificationAcceptable IS NULL
		SET @MinQualificationAcceptable = 0

	DECLARE @IdAbilityEvaluationFormat Varchar(50) = NULL

	-- RECUPERAMOS EL IDABILITYEVALUATIONFORMAT
	SELECT @IdAbilityEvaluationFormat = ID
		FROM [AbilityEvaluationFormat] (NOLOCK)
		WHERE [Instance] = @Instance
			AND [IdEvaluationFormat] = @IdEvaluationFormat
			AND [IdGroup] = @IdGroup
			AND [IdSubGroup] = @IdSubGroup
			AND [IdAbility] = @IdAbility
			AND isNull([Conditional],'') = isNull(@Conditional,'')

	-- BUSCAMOS SI YA EXISTE UN REGISTRO EN LA BASE DE DATOS PARA LA EVALUACION
	DECLARE @SAVED_QUALIFICATION NUMERIC(5, 2) = NULL
	DECLARE @SAVED_COMMENT       VARCHAR(1000) = NULL

	SELECT @SAVED_QUALIFICATION = ER.[Qualification],
	       @SAVED_COMMENT = ER.[Comment]
		FROM [dbo].[EvaluationResult] ER (NOLOCK)
		WHERE ER.[Instance] = @Instance
			AND ER.[IdEvaluationPlan] = @IdEvaluationPlan
			AND ER.[IdEvaluationFormat] = @IdEvaluationFormat
			AND ER.[IdAbility] = @IdAbility
			AND ER.[IdAbilityEvaluationFormat] = @IdAbilityEvaluationFormat
			AND ER.[IdInteraction] = @IdInteraction
			AND ER.[IdAuditor] = @IdEvaluator

	-- 
	IF @SAVED_QUALIFICATION IS NULL 
	BEGIN
	   
		-- INSERTAMOS EL NUEVO RESULT
		INSERT INTO [dbo].[EvaluationResult]
			   ([Instance]
			   ,[IdCampaign]
			   ,[IdInteraction]
			   ,[IdAgent]
			   ,[IdEvaluationFormat]
			   ,[MinQualificationAcceptable]
			   ,[IdGroup]
			   ,[IdSubGroup]
			   ,[IdAbility]
			   ,[Qualification]
			   ,[OwnScore]
			   ,[InheritedScore]
			   ,[IdEvaluator]
			   ,[IdAuditor]
			   ,[EvaluationDate]
			   ,[MaxAbilityScore]
			   ,[AbilityInnerScore]
			   ,[IdEvaluationPlan]
			   ,[Comment]
			   ,[NotApplies]
			   ,[ScoreDescription]
				,[CampaignDescription]
				,[AgentDescription]
				,[InteractionDateini]
				,[InteractionDatefin]
				,[InteractionAddress]
				,[InteractionType]
				,[AbilityOrder]
			   ,[CreatedDate]
			   ,[CreatedByUserId]
			   ,[LastModifiedDate]
			   ,[LastModifiedByUserId]
			   ,[IdAbilityEvaluationFormat]
			   ,[Conditional]
			   ,[TimeStamp]
			   )
		 VALUES
			   (
			   @Instance,
			   @IdCampaign,
				@IdInteraction,
				@IdAgent,
				@IdEvaluationFormat,
				@MinQualificationAcceptable ,
				@IdGroup, @IdSubGroup, @IdAbility,
				@Qualification,
				@OwnScore,
				@InheritedScore,
				@IdEvaluator,
				@IdEvaluator,
				@EvaluationDate,
				@MaxAbilityScore,
				@AbilityInnerScore,
				@IdEvaluationPlan,
				@Comment,
				@NotApplies,
				@ScoreDescription,
				@CampaignDescription,
				@AgentDescription,
				@InteractionDateini,
				@InteractionDatefin,
				@InteractionAddress,
				@InteractionType,
				@AbilityOrder,
				@CreatedDate,@CreatedByUserId,@LastModifiedDate,@LastModifiedByUserId,
				@IdAbilityEvaluationFormat,
				@Conditional,
			   GETUTCDATE())

	END
	ELSE
	BEGIN

		-- PRINT CONCAT('SAVED: ', @SAVED_QUALIFICATION)
		-- PRINT CONCAT('INPUT: ', @Qualification)

		-- ACTUALIZAMOS LA CALIFICACION EN CASO SEA DIFERENTE
		UPDATE [dbo].[EvaluationResult] 
			SET [Qualification] = @Qualification,
				[OwnScore] = @OwnScore,
				[InheritedScore] = @InheritedScore,
				[AbilityInnerScore] = @AbilityInnerScore,
				[ScoreDescription] = @ScoreDescription,
				[IdEvaluator] = @IdEvaluator,
				[LastModifiedByUserId] = @LastModifiedByUserId,
				[LastModifiedDate] = @LastModifiedDate,
				[TimeStamp] = GETUTCDATE(),
				[NotApplies] = @NotApplies
			WHERE [Instance] = @Instance
				AND [IdEvaluationPlan] = @IdEvaluationPlan
				AND [IdEvaluationFormat] = @IdEvaluationFormat
				AND [IdAbility] = @IdAbility
				AND [IdAbilityEvaluationFormat] = @IdAbilityEvaluationFormat
				AND [IdInteraction] = @IdInteraction
				AND [IdAuditor] = @IdEvaluator
		
		--ACTUALIZAR EL COMENTARIO
		IF @Comment <> ISNULL(@SAVED_COMMENT,'') 
		BEGIN 
			UPDATE [dbo].[EvaluationResult] 
					SET	[Comment] = @Comment
					WHERE [Instance] = @Instance
						AND [IdEvaluationPlan] = @IdEvaluationPlan
						AND [IdEvaluationFormat] = @IdEvaluationFormat
						AND [IdAbility] = @IdAbility
						AND [IdAbilityEvaluationFormat] = @IdAbilityEvaluationFormat
						AND [IdInteraction] = @IdInteraction
						AND [IdAuditor] = @IdEvaluator

		END
		
		
	END

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationResultDraftBackup] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionType VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@AuditorId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	-- EL PRESENTE SP TIENE LA FINALIDAD DE RESPALDAR LOS RESULTADOS GENERADOS EN LA PARTE AUTOMATICA DEL PLAN MIXTO

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- DECLARAMOS UNA VARIABLE PARA VERIFICAR EL TIPO DEL PLAN
		DECLARE @IS_MIXED_EVALUATION_PLAN BIT = 0

		-- VERIFICAMOS QUE EL PLAN DE EVALUACION ES DEL TIPO MIXTO
		SELECT @IS_MIXED_EVALUATION_PLAN = IIF (EP.[Evaluation_Mode] = 'X', 1, 0) 
			FROM [dbo].[EvaluationPlan] EP (NOLOCK) 
			WHERE EP.[Instance] = @VCC
				AND EP.[ID] = @EvaluationPlanId

		-- DECLARAMOS UNA VARIABLE PARA VERIFICAR SI LA AUDITORIA YA ESTA GUARDADA COMO BORRADOR
		DECLARE @EXISTS_DRAFT_DATA BIT = 0

		IF EXISTS (
			SELECT 1 
				FROM [dbo].[EvaluationResult] (NOLOCK)
				WHERE [Instance] = @VCC
					AND [IdEvaluationPlan] = @EvaluationPlanId
					AND [InteractionType] = @InteractionType
					AND [IdInteraction] = @InteractionId
					AND [IdEvaluator] = 'SPEECH'
					AND [IdAuditor] = @AuditorId + '-DRAFT'
		)
			SET @EXISTS_DRAFT_DATA = 1

		-- SI ES UN PLAN MIXTO REALIZAMOS EL RESPALDO DE RESULTADOS
		IF @IS_MIXED_EVALUATION_PLAN = 1 AND @EXISTS_DRAFT_DATA = 0
		BEGIN
			-- INSERTAMOS EL NUEVO RESULT DE RESPALDO (SE BORRARA AL CERRAR LA AUDITORIA)
			INSERT INTO [dbo].[EvaluationResult] (
				[Instance] ,[IdCampaign] ,[IdInteraction] ,[IdAgent] ,[IdEvaluationFormat]
				,[MinQualificationAcceptable] ,[IdGroup] ,[IdSubGroup] ,[IdAbility] ,[Qualification]
				,[OwnScore] ,[InheritedScore] ,[IdEvaluator] ,[IdAuditor] ,[EvaluationDate]
				,[MaxAbilityScore] ,[AbilityInnerScore] ,[IdEvaluationPlan] ,[Comment] ,[NotApplies]
				,[ScoreDescription] ,[CampaignDescription] ,[AgentDescription] ,[InteractionDateini] ,[InteractionDatefin]
				,[InteractionAddress] ,[InteractionType] ,[AbilityOrder] ,[CreatedDate] ,[CreatedByUserId]
				,[LastModifiedDate] ,[LastModifiedByUserId] ,[IdAbilityEvaluationFormat] ,[Conditional] ,[TimeStamp]
			)
				SELECT ER.[Instance] ,ER.[IdCampaign] ,ER.[IdInteraction] ,ER.[IdAgent] ,ER.[IdEvaluationFormat]
						,ER.[MinQualificationAcceptable] ,ER.[IdGroup] ,ER.[IdSubGroup] ,ER.[IdAbility] ,ER.[Qualification]
						,ER.[OwnScore] ,ER.[InheritedScore] ,ER.[IdEvaluator] ,ER.[IdAuditor] + '-DRAFT' ,ER.[EvaluationDate]
						,ER.[MaxAbilityScore] ,ER.[AbilityInnerScore] ,ER.[IdEvaluationPlan] ,ER.[Comment] ,ER.[NotApplies]
						,ER.[ScoreDescription] ,ER.[CampaignDescription] ,ER.[AgentDescription] ,ER.[InteractionDateini] ,ER.[InteractionDatefin]
						,ER.[InteractionAddress] ,ER.[InteractionType] ,ER.[AbilityOrder] ,ER.[CreatedDate] ,ER.[CreatedByUserId]
						,ER.[LastModifiedDate] ,ER.[LastModifiedByUserId] ,ER.[IdAbilityEvaluationFormat] ,ER.[Conditional] ,ER.[TimeStamp]
					FROM [dbo].[EvaluationResult] ER (NOLOCK)
					WHERE ER.[Instance] = @VCC
						AND ER.[IdEvaluationPlan] = @EvaluationPlanId
						AND ER.[InteractionType] = @InteractionType
						AND ER.[IdInteraction] = @InteractionId
						AND ER.[IdEvaluator] = 'SPEECH'
						AND ER.[IdAuditor] = @AuditorId
		END   

		SELECT 1 AS 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveEvaluationResultEfectivity] (
	@VCC Varchar(50) = NULL,
	@EfectivityPlanId Varchar(50) = NULL,
	@EvaluationPlanId Varchar(50) = NULL,
	@InteractionId Varchar(250) = NULL,	
	@AbilityEvaluationFormatId Varchar(50) = NULL,	
	@Efectivity Varchar(50) = NULL,
	@EfectivityStatus Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON


	IF EXISTS(SELECT 1 FROM dbo.[EvaluationResultEfectivity](NOLOCK)
			  WHERE VCC = @VCC 
			  AND [EfectivityPlanId] = @EfectivityPlanId
			  AND [EvaluationPlanId] = @EvaluationPlanId
			  AND [InteractionId] = @InteractionId			  
			  AND [AbilityEvaluationFormatId] = @AbilityEvaluationFormatId
			  )
	BEGIN
	UPDATE [dbo].[EvaluationResultEfectivity] 
			SET [Efectivity] = @Efectivity,								   								
				[EfectivityStatus] = @EfectivityStatus,
				[LastModifiedDate] = GETUTCDATE()
				
			 WHERE [VCC] = @VCC 
			  AND [EfectivityPlanId] = @EfectivityPlanId
			  AND [EvaluationPlanId] = @EvaluationPlanId
			  AND [InteractionId] = @InteractionId			  
			  AND [AbilityEvaluationFormatId] = @AbilityEvaluationFormatId
    END 

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationReview] (
	@VCC VARCHAR(50) = NULL, 
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@SupervisorId VARCHAR(50) = NULL,
	@Review VARCHAR(500) = NULL,
	@SaveUserId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EL REGISTRO CORRESPONDIENTE A LA REVISION
	UPDATE [dbo].[EvaluationReview]
		SET [Review] = @Review,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedUserId] = @SaveUserId,
			[Timestamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [EvaluationPlanId] = @EvaluationPlanId
			AND [InteractionId] = @InteractionId
			AND [SupervisorId] = @SupervisorId

	-- SI NO SE ACTUALIZO NINGUN REGISTRO, LO INSERTAMOS
	IF @@ROWCOUNT = 0
	BEGIN
		-- 
		INSERT INTO [dbo].[EvaluationReview] (
				[VCC],
				[EvaluationPlanId],
				[InteractionId],
				[SupervisorId],
				[Review],
				[CreatedDate],
				[CreatedUserId],
				[Timestamp]
			) VALUES (
				@VCC,
				@EvaluationPlanId,
				@InteractionId,
				@SupervisorId,
				@Review,
				GETUTCDATE(),
				@SaveUserId,
				GETUTCDATE()
			)
	END

	-- RETORNAMOS EL REGISTRO INSERTADO/ACTUALIZADO
    SELECT ER.[VCC]													'vcc',
			ER.[EvaluationPlanId]									'evaluationPlanId',
			ER.[InteractionId]										'interactionId',
			ER.[SupervisorId]										'supervisorId',
			ER.[Review]												'review'
		FROM [dbo].[EvaluationReview] ER (NOLOCK) 
		WHERE ER.[VCC] = @VCC
			AND ER.[EvaluationPlanId] = @EvaluationPlanId
			AND ER.[InteractionId] = @InteractionId
			AND ER.[SupervisorId] = ISNULL(@SupervisorId, ER.[SupervisorId])
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationReviewCommitment] (
	@VCC VARCHAR(50) = NULL, 
	@Id VARCHAR(50) = NULL,
	@LargeInteractionId VARCHAR(255) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@SupervisorId VARCHAR(50) = NULL,
	@AgentId VARCHAR(50) = NULL,
	@Commitment VARCHAR(500) = NULL,
	@SaveUserId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EL REGISTRO CORRESPONDIENTE A LA REVISION
	UPDATE [dbo].[EvaluationReviewCommitment]
		SET [Commitment] = @Commitment,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedUserId] = @SaveUserId,
			[Timestamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [EvaluationPlanId] = @EvaluationPlanId
			AND [InteractionId] = @InteractionId
			AND [SupervisorId] = @SupervisorId
			AND [ID] = @Id
			
	-- SI NO SE ACTUALIZO NINGUN REGISTRO, LO INSERTAMOS
	IF @@ROWCOUNT = 0
	BEGIN
		-- 
		INSERT INTO [dbo].[EvaluationReviewCommitment] (
				[VCC],
				[LargeInteractionId],
				[EvaluationPlanId],
				[InteractionId],
				[SupervisorId],
				[AgentId],
				[Commitment],
				[CreatedDate],
				[CreatedUserId],
				[Timestamp]
			) VALUES (
				@VCC,
				@LargeInteractionId,
				@EvaluationPlanId,
				@InteractionId,
				@SupervisorId,
				@AgentId,
				@Commitment,
				GETUTCDATE(),
				@SaveUserId,
				GETUTCDATE()
			)
	END

	-- 
    SELECT 1 AS 'result'
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationStatus]
(
	@VCC							VARCHAR(50) = NULL,
	@EvaluationPlanId				VARCHAR(50) = NULL,
	@InteractionId					VARCHAR(250) = NULL,
	@AuditorId						VARCHAR(50) = NULL,
	@Status							BIT = NULL,
	@Multiple						BIT = NULL,
	@LastModifiedByUserId			VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	--
	UPDATE [dbo].[AuditorEvaluationPlan]	
		SET [Status] = @Status,
			[LastModifiedByUserId] = @LastModifiedByUserId,
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [Instance] = @VCC
			AND [IdEvaluationPlan] = @EvaluationPlanId
			AND [IdInteraction] = @InteractionId
			AND [IdAuditor] = CASE @Multiple WHEN 0 THEN @AuditorId ELSE [IdAuditor] END

	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationTipification] (
	@VCC VARCHAR(50) = NULL,
	@AbilityId VARCHAR(50) = NULL,
	@ScoreId VARCHAR(50) = NULL,
	@ID VARCHAR(50) = NULL,
	@IsChecked BIT = 0,
	@Comment VARCHAR(500) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@AuditorId VARCHAR(50) = NULL,
	@UserId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	-- EL PRESENTE SP TIENE LA FINALIDAD DE RESPALDAR LOS RESULTADOS GENERADOS EN LA PARTE AUTOMATICA DEL PLAN MIXTO

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- ACTUALIZAMOS EL REGISTRO EN CASO EXISTA
		UPDATE [dbo].[EvaluationTipification]
			SET [IsChecked] = @IsChecked,
				[Comment] = @Comment,
				[AuditorId] = @AuditorId
			WHERE [VCC] = @VCC 
				AND [EvaluationPlanId] = @EvaluationPlanId
				AND [InteractionId] = @InteractionId
				AND [AuditorId] = @AuditorId
				AND [AbilityId] = @AbilityId
				AND [ScoreId] = @ScoreId
				AND [ID] = @ID

		-- EN CASO NO HAYA HABIDO NINGUNA ACTUALIZACION, INSERTAMOS EL REGISTRO
		IF @@ROWCOUNT = 0
		BEGIN
			INSERT INTO [dbo].[EvaluationTipification] (
				[VCC], [AbilityId], [ScoreId], [ID], [IsChecked], 
				[Comment], [CreatedDate], [CreatedByUserId], [EvaluationPlanId], [InteractionId], 
				[AuditorId]
			)
			VALUES (
				@VCC, @AbilityId, @ScoreId, @ID, @IsChecked, 
				@Comment, GETUTCDATE(), @UserId, @EvaluationPlanId, @InteractionId, 
				@AuditorId
			)

		END
		
		SELECT 1 AS 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	
    SET NOCOUNT OFF
ENDGO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationTipificationBaseResult] (
	@VCC VARCHAR(50) = NULL,
	@EVALUATION_PLAN_ID VARCHAR(100) = NULL
) AS
BEGIN
	-- ESTE SP INSERTA LOS RESULT BASE DE UN PLAN AUTOMATICO/MIXTO
	SET NOCOUNT ON

	-- EMULAMOS UNA NUEVA TABLA VARIABLE PARA OBTENER LOS REGISTROS UNICOS QUE INSERTAREMOS EN LA TABLA EVALUATION_RESULT
	--  ORDENANDO POR SECTION Y PART
	;WITH CTE_TIPIFICATION_QUERY AS (
		SELECT DISTINCT HA.[ID]							'AbilityId',
							HEFASTQ.*
			FROM [dbo].[HistoricAbility] HA (NOLOCK)
			INNER JOIN [dbo].[HistoricScore] HS (NOLOCK)
				ON HS.[VCC] = HA.[VCC]
					AND HS.[EvaluationPlanId] = HA.[EvaluationPlanId]
					AND HS.[IdAbility] = HA.[ID]
					/*
			INNER JOIN [dbo].[HistoricEvaluationFormatAbilityScoreTipification] HEFAST (NOLOCK)
				ON HEFAST.[VCC] = HA.[VCC]
					AND HEFAST.[EvaluationPlanId] = HS.[EvaluationPlanId]
					AND HEFAST.[ScoreId] = HS.[ID]
					*/
			INNER JOIN [dbo].[HistoricEvaluationFormatAbilityScoreTipificationQuery] HEFASTQ (NOLOCK)
				ON HEFASTQ.[VCC] = HA.[VCC]
					AND HEFASTQ.[EvaluationPlanId] = HS.[EvaluationPlanId]
					AND HEFASTQ.[ScoreId] = HS.[ID]
					--AND HEFASTQ.[TipificationId] = HEFAST.[ID]
			WHERE HA.[VCC] = @VCC
				AND HA.[EvaluationPlanId] = @EVALUATION_PLAN_ID
	) 
		
		-- GENERAMOS LOS REGISTROS DE TIPIFICACION PARA LAS INTERACCIONES CON 0 DE CALIFICACION Y NO ESTEN YA REGISTRADAS
		INSERT INTO [dbo].[EvaluationTipification] (
			[VCC], [AbilityId], [ScoreId], [ID], [EvaluationPlanId], 
			[InteractionId], [AuditorId], [CreatedDate], [CreatedByUserId]
		) 
			SELECT CTE_TQ.[VCC], CTE_TQ.[AbilityId], CTE_TQ.[ScoreId], CTE_TQ.[TipificationId], CTE_TQ.[EvaluationPlanId],
					AEP.[IdInteraction], 'SPEECH', GETUTCDATE(), 'system'
				FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
				LEFT JOIN CTE_TIPIFICATION_QUERY CTE_TQ (NOLOCK)
					ON CTE_TQ.[VCC] = AEP.[Instance]
				LEFT JOIN [EvaluationTipification] ET (NOLOCK)
					ON ET.[VCC] = CTE_TQ.[VCC]
						AND ET.[AbilityId] = CTE_TQ.[AbilityId]
						AND ET.[ScoreId] = CTE_TQ.[ScoreId]
						AND ET.[ID] = CTE_TQ.[TipificationId]
						AND ET.[EvaluationPlanId] = CTE_TQ.[EvaluationPlanId]
						AND ET.[InteractionId] = AEP.[IdInteraction]
				WHERE AEP.[Instance] = @VCC
					AND AEP.[IdEvaluationPlan] = @EVALUATION_PLAN_ID
					AND AEP.[Score] = 0
					AND ET.[VCC] IS NULL
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationTipificationItem] (
	@VCC VARCHAR(50) = NULL,
	@ScoreId VARCHAR(50) = NULL,
	@TipificationId VARCHAR(50) = NULL,
	@ID VARCHAR(50) = NULL,
	@IsChecked BIT = 0,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@AuditorId VARCHAR(50) = NULL,
	@UserId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	-- EL PRESENTE SP TIENE LA FINALIDAD DE RESPALDAR LOS RESULTADOS GENERADOS EN LA PARTE AUTOMATICA DEL PLAN MIXTO

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION
		
		-- ACTUALIZAMOS EL REGISTRO EN CASO EXISTA
		UPDATE [dbo].[EvaluationTipificationItem]
			SET [IsChecked] = @IsChecked,
				[AuditorId] = @AuditorId
			WHERE [VCC] = @VCC 
				AND [EvaluationPlanId] = @EvaluationPlanId
				AND [InteractionId] = @InteractionId
				AND [AuditorId] = @AuditorId
				AND [ScoreId] = @ScoreId
				AND [TipificationId] = @TipificationId
				AND [ID] = @ID

		-- EN CASO NO HAYA HABIDO NINGUNA ACTUALIZACION, INSERTAMOS EL REGISTRO
		IF @@ROWCOUNT = 0
		BEGIN
			INSERT INTO [dbo].[EvaluationTipificationItem] (
				[VCC], [ScoreId], [TipificationId], [ID], [IsChecked], 
				[CreatedDate], [CreatedByUserId], [EvaluationPlanId], [InteractionId], [AuditorId]
			)
			VALUES (
				@VCC, @ScoreId, @TipificationId, @ID, @IsChecked, 
				GETUTCDATE(), @UserId, @EvaluationPlanId, @InteractionId, @AuditorId
			)

		END
		
		SELECT 1 AS 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveException] (
	@VCC Varchar(50) = NULL, 				    @ID Varchar(50) = NULL,				            @Description Varchar(50) = NULL,
	@CreatedDate Datetime = NULL,				@CreatedByUserId Varchar(50) = NULL,			@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [Exception]
	SET
		[Description] = @Description,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
    AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [Exception] (
			[VCC],[ID],
			[Description], [CreatedDate],
			[CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId],
			[TimeStamp]
			)
		VALUES (
			@VCC,@ID,
			@Description, @CreatedDate,
			@CreatedByUserId, @LastModifiedDate, @LastModifiedByUserId,
			GETUTCDATE()
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveFeeling] (
	@VCC Varchar(50) = NULL, 					@Name Varchar(50) = NULL,
	@Description Varchar(50) = NULL,			@CreatedDate Datetime = NULL,               @CreatedByUserId Varchar(50) = NULL,		
	@LastModifiedDate Datetime = NULL,			@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [Feeling]
	SET [Description] = @Description,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
    AND [Name] = @Name

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [Feeling] (
			[VCC], [Name], [Description],
			[CreatedDate], [CreatedByUserId], [LastModifiedDate], 
			[LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
			@VCC, @Name, @Description,
			@CreatedDate, @CreatedByUserId, @LastModifiedDate, 
			@LastModifiedByUserId, GETUTCDATE()
		)
	END

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveGenerative] (
    @VCC VARCHAR(50) = NULL,
    @Name VARCHAR(50) = NULL,
    @AppId VARCHAR(100) = NULL,
    @Description VARCHAR(50) = NULL,
	@Language VARCHAR(5) = NULL,
    @IsPublished BIT = NULL,
    @NeedPublish BIT = NULL,
    @LastPublishedDate DATETIME = NULL,
	@StatusCode VARCHAR(50) = NULL,
    @CurrentUser VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON
	
	-- ACTUALIZAMOS EL MODELO EN CASO EXISTA
	UPDATE [Generative]
		SET [Description] = @Description,
			[IsPublished] = CASE WHEN @IsPublished IS NULL THEN [IsPublished] ELSE @IsPublished END,
			[NeedPublish] = CASE WHEN @NeedPublish IS NULL THEN [NeedPublish] ELSE @NeedPublish END,
			[LastPublishedDate] = CASE WHEN @LastPublishedDate IS NULL THEN [LastPublishedDate] ELSE @LastPublishedDate END,
			[StatusCode] =  CASE WHEN @StatusCode IS NULL THEN [StatusCode] ELSE @StatusCode END,
			[LastModifiedByUserId] = @CurrentUser,
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
		  AND [AppId] = @AppId

	-- VERIFICAMOS SI SE ACTUALIZO ALGUN REGISTRO
    IF @@ROWCOUNT = 0
    BEGIN
		-- INSERTAMOS EL MODELO
        INSERT INTO [Generative] (
            [VCC], [Name], [AppId], [Description], [Language], 
			[IsPublished], [NeedPublish], [StatusCode], [CreatedByUserId], [CreatedDate], 
			[TimeStamp]
        )
		VALUES (
            @VCC, @Name, @AppId, @Description, @Language,
			0, 0, @StatusCode, @CurrentUser, GETUTCDATE(), 
			GETUTCDATE()
        )
    END

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveGenerativeEngineUsage] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@AppId VARCHAR(50) = NULL,
	@Tokens INT = 0
) AS
BEGIN
	-- GUARDAMOS EL REGISTRO DE CONSUMO
	INSERT INTO [dbo].[GenerativeEngineUsage] ([VCC], [EvaluationPlanId], [InteractionId], [AppId], [Tokens], [TimeStamp])
		VALUES (@VCC, @EvaluationPlanId, @InteractionId, @AppId, @Tokens, GETUTCDATE())

    SELECT 1 AS 'result'
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveGenerativeExample] (
    @VCC VARCHAR(50) = NULL,
	@AppId VARCHAR(50) = NULL,
	@IntentId VARCHAR(50) = NULL,
	@Id VARCHAR(50) = NULL,
    @Text VARCHAR(250) = NULL,
    @CurrentUser VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON
	
	-- ACTUALIZAMOS EL INTENT EN CASO EXISTA
	UPDATE [GenerativeExample]
		SET [Text] = @Text,
			[LastModifiedByUserId] = @CurrentUser,
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
		  AND [AppId] = @AppId
		  AND [IntentId] = @IntentId
		  AND [Id] = @Id

	-- VERIFICAMOS SI SE ACTUALIZO ALGUN REGISTRO
    IF @@ROWCOUNT = 0
    BEGIN
		-- INSERTAMOS EL MODELO
        INSERT INTO [GenerativeExample] (
            [VCC], [AppId], [IntentId], [Id], [Text], 
			[CreatedByUserId], [CreatedDate], [TimeStamp]
        )
		VALUES (
            @VCC, @AppId, @IntentId, @Id, @Text, 
			@CurrentUser, GETUTCDATE(), GETUTCDATE()
        )
    END

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveGenerativeExampleId] (
    @VCC VARCHAR(50) = NULL,
	@AppId VARCHAR(50) = NULL,
	@IntentId VARCHAR(50) = NULL,
    @OldId VARCHAR(50) = NULL,
    @NewId VARCHAR(50) = NULL,
	@CurrentUser VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
   
	-- ACTUALIZAMOS LOS EJEMPLOS
	UPDATE [dbo].[GenerativeExample] SET [Id] = @NewId, [LastModifiedByUserId] = @CurrentUser, [LastModifiedDate] = GETUTCDATE(), [TimeStamp] = GETUTCDATE() WHERE [AppId] = @AppId AND [IntentId] = @IntentId AND [Id] = @OldId
	
	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveGenerativeId] (
    @VCC VARCHAR(50) = NULL,
    @OldId VARCHAR(50) = NULL,
    @NewId VARCHAR(50) = NULL,
	@CurrentUser VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
   
	-- ACTUALIZAMOS EL MODELO, INTENCIONES Y EJEMPLOS
	UPDATE [dbo].[Generative] 
		SET [AppId] = @NewId, 
			[NeedPublish] = 1,
			[StatusCode] = 'NEED_PUBLISH',
			[LastModifiedByUserId] = @CurrentUser, 
			[LastModifiedDate] = GETUTCDATE(), 
			[TimeStamp] = GETUTCDATE() 
		WHERE [AppId] = @OldId
	UPDATE [dbo].[GenerativeIntent] SET [AppId] = @NewId, [LastModifiedByUserId] = @CurrentUser, [LastModifiedDate] = GETUTCDATE(), [TimeStamp] = GETUTCDATE() WHERE [AppId] = @OldId
	UPDATE [dbo].[GenerativeExample] SET [AppId] = @NewId, [LastModifiedByUserId] = @CurrentUser, [LastModifiedDate] = GETUTCDATE(), [TimeStamp] = GETUTCDATE() WHERE [AppId] = @OldId

	SELECT 1 AS 'result'

	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveGenerativeIntent] (
    @VCC VARCHAR(50) = NULL,
	@AppId VARCHAR(50) = NULL,
	@Id VARCHAR(50) = NULL,
    @Name VARCHAR(50) = NULL,
    @Description VARCHAR(250) = NULL,
    @CurrentUser VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON
	
	-- ACTUALIZAMOS EL INTENT EN CASO EXISTA
	UPDATE [GenerativeIntent]
		SET [Description] = @Description,
			[LastModifiedByUserId] = @CurrentUser,
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
		  AND [AppId] = @AppId
		  AND [Id] = @Id

	-- VERIFICAMOS SI SE ACTUALIZO ALGUN REGISTRO
    IF @@ROWCOUNT = 0
    BEGIN
		-- INSERTAMOS EL MODELO
        INSERT INTO [GenerativeIntent] (
            [VCC], [AppId], [Id], [Name], [Description], 
			[CreatedByUserId], [CreatedDate], [TimeStamp]
        )
		VALUES (
            @VCC, @AppId, @Id, @Name, @Description, 
			@CurrentUser, GETUTCDATE(), GETUTCDATE()
        )
    END

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveGenerativeIntentId] (
    @VCC VARCHAR(50) = NULL,
	@AppId VARCHAR(50) = NULL,
    @OldId VARCHAR(50) = NULL,
    @NewId VARCHAR(50) = NULL,
	@CurrentUser VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
   
	-- ACTUALIZAMOS EL MODELO, INTENCIONES Y EJEMPLOS
	UPDATE [dbo].[GenerativeIntent] SET [Id] = @NewId, [LastModifiedByUserId] = @CurrentUser, [LastModifiedDate] = GETUTCDATE(), [TimeStamp] = GETUTCDATE() WHERE [AppId] = @AppId AND [Id] = @OldId
	UPDATE [dbo].[GenerativeExample] SET [IntentId] = @NewId, [LastModifiedByUserId] = @CurrentUser, [LastModifiedDate] = GETUTCDATE(), [TimeStamp] = GETUTCDATE() WHERE [AppId] = @AppId AND [IntentId] = @OldId
	
	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

-- [dbo].[SaveGenericQuery] 'inconcert', ''
CREATE PROCEDURE [dbo].[SaveGenericQuery] (
	@VCC							VARCHAR(50) = NULL,
	@Name							VARCHAR(50) = NULL,
	@Description					VARCHAR(100) NULL,
	@Type							VARCHAR(25) NULL,
	@Query							VARCHAR(MAX) = NULL,
	@RawData						VARCHAR(MAX) = NULL,
	@CreatedDate					DATETIME = NULL,
	@CreatedByUserId				VARCHAR(50) = NULL, 	
	@LastModifiedDate				DATETIME = NULL,
	@LastModifiedByUserId			VARCHAR(50) = NULL,
	@Version                        SMALLINT = 1
	)
AS
BEGIN
	SET NOCOUNT ON
	--SI LA QUERY SE ENCUENTRA USADA EN HABILIDADES DE FORMATO O TEMAS SE DEBEN ACTUALIZAR TODAS 
	  
	 UPDATE [AbilityQuery] SET
		[QueryContent] = @Query
	 WHERE 
		QueryId = @Name
		AND [VCC] = @VCC

	UPDATE [GenericQuery] SET
		[Name] = @Name,
		[Description] = @Description,
		[Type] = @Type,
		[Query] = @Query,
		[RawData] = @RawData,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE
		[VCC] = @VCC
		AND [Name] = @Name

	

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [GenericQuery]
		(
			[VCC],
			[Name],
			[Description],
			[Type],
			[Query],
			[RawData],
			[CreatedDate],
			[CreatedByUserId],
			[LastModifiedDate], 
			[LastModifiedByUserId],
			[Version],
			[TimeStamp]
		)
		VALUES
		(
			@VCC,
			@Name,
			@Description,
			@Type,
			@Query,
			@RawData,
			@CreatedDate,
			@CreatedByUserId,
			@LastModifiedDate, 
			@LastModifiedByUserId,
			@Version,
			GETUTCDATE()
		)
	END

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveGenericTemplateQuery] (
	@VCC Varchar(50) = NULL, 				@TemplateId Varchar(50) = NULL, 			@ID Varchar(50) = NULL,
	@Query Varchar(1000) = NULL,			@CreatedDate Datetime = NULL,				@CreatedByUserId Varchar(50) = NULL, 	
	@LastModifiedDate Datetime = NULL,		@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [GenericTemplateQuery]
	SET [Query] = @Query,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
	AND [TemplateId] = @TemplateId
	AND [ID] = @ID

    IF @@ROWCOUNT = 0
	BEGIN
		SET @ID = NEWID()
		INSERT INTO [GenericTemplateQuery] (
				[VCC], [TemplateId], [ID], [Query],
				[CreatedDate], [CreatedByUserId], [LastModifiedDate], 
				[LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
				@VCC, @TemplateId, @ID, @Query,
				@CreatedDate, @CreatedByUserId, @LastModifiedDate, 
				@LastModifiedByUserId, GETUTCDATE()
			)
	END

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveGroup] (
	@Instance Varchar(50) = NULL, 				@ID Varchar(50) = NULL,				@Description Varchar(50) = NULL,
	@CreatedDate Datetime = NULL,				@CreatedByUserId Varchar(50) = NULL,			@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL,	@Enabled bit = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [Group]
	SET
		[Description] = @Description,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[Enabled] = CASE WHEN @Enabled IS NULL THEN 1 ELSE @Enabled END,
		[TimeStamp] = GETUTCDATE()
	WHERE [Instance] = @Instance
    AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [Group] (
			[Instance],[ID],
			[Description], [CreatedDate],
			[CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId],
			[Enabled], [TimeStamp]
			)
		VALUES (
			@Instance,@ID,
			@Description, @CreatedDate,
			@CreatedByUserId, @LastModifiedDate, @LastModifiedByUserId,
			1, GETUTCDATE()
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveHistoricEvaluationPlanData] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50)=NULL,
	@FormatId VARCHAR(100) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- DECLARAMOS UNA VARIABLE PARA LA MODALIDAD DEL PLAN
	DECLARE @EVALUATION_MODE VARCHAR(50) = NULL 
	SELECT @EVALUATION_MODE = [Evaluation_Mode] FROM [dbo].[EvaluationPlan] (NOLOCK) WHERE [ID] = @EvaluationPlanId

	-- REGISTRAMOS LA DATA DEL FORMATO DE EVALUACION
	INSERT INTO [dbo].[HistoricEvaluationFormat] 
		SELECT EF.[Instance],
				EF.[ID],
				EF.[Description],
				EF.[type],
				EF.[EnvironmentType],
				EF.[ExecutionType],
				EF.[MinAutomaticQualification],
				EF.[MinQualificationAcceptable],
				EF.[ManagementFormURL],
				EF.[Enabled],
				EF.[templateId],
				@EvaluationPlanId
            FROM [dbo].[EvaluationFormat] EF (NOLOCK) 
            WHERE [ID] = @FormatId 
				AND [Instance] = @VCC


	-- 
	INSERT INTO [dbo].[HistoricAbilityEvaluationFormat]
		SELECT DISTINCT A.[Instance],
							A.[ID] ,
							A.[IdEvaluationFormat],
							A.[IdAbility],
							A.[IdGroup],
							A.[IdSubGroup],
							A.[Penalizes],
							A.[Score],
							A.[Order],
							A.[Conditional],
							@EvaluationPlanId
			FROM  [dbo].[AbilityEvaluationFormat] A (NOLOCK)
			JOIN [dbo].[EvaluationFormat] EF (NOLOCK)
				ON EF.[Instance] = A.[Instance] 
					AND EF.[ID] = A.[IdEvaluationFormat]
			WHERE EF.[ID] = @FormatId AND EF.[Instance] = @VCC

	--
	INSERT INTO [dbo].[HistoricEvaluationFormatCustomField]
		SELECT ECF.[VCC],
				ECF.[ID],
				ECF.[EvaluationFormatId],
				ECF.[CustomFieldId],
				ECF.[Order],
				@EvaluationPlanId
			FROM [dbo].[EvaluationFormatCustomField] ECF (NOLOCK)
			JOIN [dbo].[EvaluationFormat] EF (NOLOCK) 
				ON ECF.[VCC] = EF.[Instance] 
				AND ECF.[EvaluationFormatId] = EF.[ID]
			WHERE EF.[ID] = @FormatId AND EF.[Instance] = @VCC
			GROUP BY ECF.[VCC],
						ECF.[ID],
						ECF.[EvaluationFormatId],
						ECF.[CustomFieldId],
						ECF.[Order]

	-- 
	INSERT INTO [dbo].[HistoricCustomField]
		SELECT C.[VCC],
				 C.[ID],
				 C.[Description],
				 C.[Type],
				 C.[Label],
				 @EvaluationPlanId
			FROM [dbo].[CustomField] C (NOLOCK)
			JOIN [dbo].[EvaluationFormatCustomField] FC (NOLOCK)
				ON C.[VCC] = FC.[VCC] 
					AND C.[ID] = FC.[CustomFieldId]
			WHERE FC.[EvaluationFormatId] = @FormatId 
				AND FC.[VCC] = @VCC
			GROUP BY C.[VCC],
						C.[ID],
						C.[Description],
						C.[Type],
						C.[Label]


	--
	INSERT INTO [dbo].[HistoricCustomFieldResult]
		SELECT CR.[VCC],
				CR.[ID],
				CR.[Description],
				CR.[Order],
				CR.[CustomFieldId],
				@EvaluationPlanId
			FROM [dbo].[CustomFieldResult] CR (NOLOCK)
			JOIN [dbo].[CustomField] C (NOLOCK)
				ON CR.[VCC] = C.[VCC] 
					AND CR.[CustomFieldId] = C.[ID]
			JOIN [dbo].[EvaluationFormatCustomField] FC (NOLOCK)
				ON C.[VCC] = FC.[VCC] 
					AND C.[ID] = FC.[CustomFieldId]
			WHERE FC.[EvaluationFormatId] = @FormatId 
				AND FC.[VCC] = @VCC
			GROUP BY CR.[VCC],
						CR.[ID],
						CR.[Description],
						CR.[Order],
						CR.[CustomFieldId]
	
	
	--
	INSERT INTO [dbo].[HistoricEvaluationFormatAttachedSignature]
		SELECT AFS.[VCC],
				AFS.[Description],
				AFS.[Order],
				AFS.[EvaluationFormatId],
				@EvaluationPlanId
			FROM [dbo].[EvaluationFormatAttachedSignature] AFS (NOLOCK)
            WHERE AFS.[EvaluationFormatId] = @FormatId 
				AND AFS.[VCC] = @VCC


	--
	INSERT INTO [dbo].[HistoricCampaignEvaluationFormat]
		SELECT CEF.[Instance],
				CEF.[IdEvaluationFormat],
				CEF.[IdCampaign],
				@EvaluationPlanId
			FROM [dbo].[CampaignEvaluationFormat] CEF (NOLOCK) 
			WHERE CEF.[IdEvaluationFormat] = @FormatId 
				AND CEF.[Instance] = @VCC
			GROUP BY CEF.[Instance],
						CEF.[IdEvaluationFormat],
						CEF.[IdCampaign]


	--
	INSERT INTO [dbo].[HistoricGroup] 
		SELECT G.[Instance] ,
				G.[ID],
				G.[Description] ,
				G.[Enabled],
				@EvaluationPlanId
			FROM [dbo].[Group] G (NOLOCK)
			JOIN [dbo].[AbilityEvaluationFormat] A (NOLOCK)
				ON  A.[Instance] = G.[Instance] 
					AND A.[IdGroup] = G.[ID]
			WHERE A.[IdEvaluationFormat] = @FormatId 
				AND A.[Instance] = @VCC
			GROUP BY G.[Instance] ,
						G.[ID],
						G.[Description] ,
						G.[Enabled] 
	


	--
	INSERT INTO [dbo].[HistoricSubGroup] 
		SELECT G.[Instance] ,
				G.[ID],
				G.[Description] ,
				G.[Enabled],
				@EvaluationPlanId
			FROM [dbo].[SubGroup] G (NOLOCK)
			JOIN [dbo].[AbilityEvaluationFormat] A (NOLOCK)
				ON  A.[Instance] = G.[Instance] 
					AND A.[IdSubGroup] = G.[ID]
			WHERE A.[IdEvaluationFormat] = @FormatId 
				AND A.[Instance] = @VCC
			GROUP BY G.[Instance],
						G.[ID],
						G.[Description] ,
						G.[Enabled] 


	-- INSERTAMOS LA DATA DE LAS HABILIDADES
	INSERT INTO [dbo].[HistoricAbility] 
		SELECT A.[Instance],
				A.[ID],
				A.[Title],
				A.[Detail],
				A.[Type],
				A.[CalificationType],
				A.[ExecutionType],
				A.[Binary],
				A.[NotApply],
				A.[Distribute],
				A.[IsCritical],
				A.[AcceptComments],
				A.[Enabled],
				A.[AnalysisType],
				A.[Query],
				A.[QueryName],
				A.[Language],
				A.[IsNegative],
				@EvaluationPlanId
			FROM [dbo].[Ability] A (NOLOCK)
			JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
				ON  A.[Instance] = AEF.[Instance] 
					AND AEF.[IdAbility] = A.[ID]
			WHERE AEF.[IdEvaluationFormat] = @FormatId 
				AND A.[Instance] = @VCC

	-- REGISTRAMOS LOS SCORES ASOCIADOS A LAS HABILIDADES DEL FORMATO DEL PLAN
	INSERT INTO [dbo].[HistoricScore] 
		SELECT DISTINCT S.[Instance], 
						S.[ID], 
						S.[IdAbility], 
						S.[Description], 
						S.[Value], 
						EP.[ID]
			FROM [dbo].[EvaluationPlan] EP (NOLOCK)
			LEFT JOIN [dbo].[AbilityEvaluationFormat] AEP (NOLOCK)
				ON AEP.[Instance] = EP.[Instance]
					AND AEP.[IdEvaluationFormat] = EP.[IdEvaluationFormat]
			LEFT JOIN [dbo].[Score] S (NOLOCK)
				ON S.[Instance] = EP.[Instance]
					AND S.[IdAbility] = AEP.[IdAbility]
			WHERE EP.[Instance] = @VCC
			   AND EP.[ID] = @EvaluationPlanId
				
	-- REGISTRAMOS LOS SCORE_TIPIFICATIONS ASOCIADOS A LAS HABILIDADES DEL FORMATO DEL PLAN
	INSERT INTO [dbo].[HistoricEvaluationFormatAbilityScoreTipification]
		SELECT [VCC],
				[EvaluationFormatId],
				[ScoreId],
				[ID],
				[Description],
				[Detail],
				[Order],
				GETUTCDATE(),
				NULL,
				@EvaluationPlanId
			FROM [dbo].[EvaluationFormatAbilityScoreTipification] (NOLOCK)
			WHERE [VCC] = @VCC
				AND [EvaluationFormatId] = @FormatId
				
	-- REGISTRAMOS LOS SCORE_TIPIFICATION_QUERIES ASOCIADOS A LAS HABILIDADES DEL FORMATO DEL PLAN
	INSERT INTO [dbo].[HistoricEvaluationFormatAbilityScoreTipificationQuery]
		SELECT [VCC],
				[EvaluationFormatId],
				[ScoreId],
				[TipificationId],
				[QueryId],
				[QueryContent],
				[AcceptancePercentage],
				GETUTCDATE(),
				NULL,
				@EvaluationPlanId
			FROM [dbo].[EvaluationFormatAbilityScoreTipificationQuery] (NOLOCK)
			WHERE [VCC] = @VCC
				AND [EvaluationFormatId] = @FormatId
				-- select * FROM [dbo].[EvaluationFormatAbilityScoreTipificationQuery] (NOLOCK)
	
	
	-- REGISTRAMOS TIPIFICACIONES DE 2DO Y TERCER NIVEL CUANDO EL PLAN NO ES AUTOMATIC
	IF @EVALUATION_MODE <> 'A' 
	BEGIN
		-- REGISTRAMOS LOS SCORE_TIPIFICATIONS_ITEMS ASOCIADOS A LAS HABILIDADES DEL FORMATO DEL PLAN
		INSERT INTO [dbo].[HistoricEvaluationFormatAbilityScoreTipificationItem]
			SELECT [VCC],
					[EvaluationFormatId],
					[ScoreId],
					[TipificationId],
					[ID],
					[SecondLevelItemId],
					[Description],
					[Order],
					GETUTCDATE(),
					NULL,
					@EvaluationPlanId
				FROM [dbo].[EvaluationFormatAbilityScoreTipificationItem] (NOLOCK)
				WHERE [VCC] = @VCC
					AND [EvaluationFormatId] = @FormatId
	END
				
	-- 
	INSERT INTO [dbo].[HistoricAbilityQuery]
		SELECT AQ.[ID],
				AQ.[VCC],
				AQ.[AbilityId],
				AQ.[QueryId],
				AQ.[QueryContent],
				AQ.[AcceptancePercentage],
				AQ.[Order],
				@EvaluationPlanId
			FROM [dbo].[AbilityQuery] AQ (NOLOCK)
			JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
			ON  AQ.[VCC] = AEF.[Instance] AND AEF.[IdAbility] = AQ.[AbilityId]
			WHERE AEF.[IdEvaluationFormat] = @FormatId AND AQ.[VCC] = @VCC
			GROUP BY 
				AQ.[ID],
				AQ.[VCC],
				AQ.[AbilityId],
				AQ.[QueryId],
				AQ.[QueryContent],
				AQ.[AcceptancePercentage],
				AQ.[Order]

	--
	INSERT INTO [dbo].[HistoricAbilityQueryEntity]
		SELECT AQE.[VCC],
				AQE.[AbilityQueryId],
				AQE.[EntityId],
				AQE.[EntityName],
				AQE.[AcceptancePercentage],
				AQE.[Order],
				@EvaluationPlanId
			FROM [dbo].[AbilityQueryEntity] AQE (NOLOCK)
			JOIN [dbo].[AbilityQuery] AQ (NOLOCK)
				ON AQ.[VCC] = AQE.[VCC] 
					AND AQE.[AbilityQueryId] = AQ.[ID]
			JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
				ON  AQ.[VCC] = AEF.[Instance] 
					AND AEF.[IdAbility] = AQ.[AbilityId]
			WHERE AQ.[VCC] = @VCC 
				AND AEF.[IdEvaluationFormat] = @FormatId
			GROUP BY AQE.[VCC],
						AQE.[AbilityQueryId],
						AQE.[EntityId],
						AQE.[EntityName],
						AQE.[AcceptancePercentage],
						AQE.[Order]
						SELECT * FROM [dbo].[HistoricAdvancedAI]
	
	-- REGISTRAMOS LA DATA DE LOS INTENTS DE MODELOS DE IA AVANZADA ASOCIADOS AL PLAN
	INSERT INTO [dbo].[HistoricAdvancedAIIntent]
		SELECT AAII.[VCC],
				AAII.[AppId],
				AAII.[ID],
				AAII.[Name],
				AAII.[Prompt],
				@EvaluationPlanId
			FROM [dbo].[AdvancedAIIntent] AAII (NOLOCK)
			WHERE AAII.[VCC] = @VCC 
				AND AAII.[ID] IN (
					SELECT DISTINCT HAQ.[QueryId] 
						FROM [dbo].[HistoricABilityQuery] HAQ (NOLOCK)
						WHERE HAQ.[EvaluationPlanId] = @EvaluationPlanId
				)

	-- REGISTRAMOS LA DATA DE LOS MODELOS DE IA AVANZADA ASOCIADOS AL PLAN
	INSERT INTO [dbo].[HistoricAdvancedAI]
		SELECT AI.[VCC],
				AI.[ID],
				AI.[Description],
				AI.[StatusCode],
				@EvaluationPlanId
			FROM [dbo].[AdvancedAI] AI (NOLOCK)
			WHERE AI.[VCC] = @VCC
				AND AI.[ID] IN (
					SELECT DISTINCT HAAII.[AppId]
						FROM [dbo].[HistoricAdvancedAIIntent] HAAII (NOLOCK)
						WHERE HAAII.[EvaluationPlanId] = @EvaluationPlanId
				)

	-- REGISTRAMOS LA DATA DE LOS EXAMPLES DE MODELOS DE IA AVANZADA ASOCIADOS AL PLAN
	INSERT INTO [dbo].[HistoricAdvancedAIExample]
		SELECT AAIE.[VCC],
				AAIE.[AppId],
				AAIE.[IntentId],
				AAIE.[Id],
				AAIE.[Text],
				@EvaluationPlanId
			FROM [dbo].[AdvancedAIExample] AAIE (NOLOCK)
			WHERE AAIE.[VCC] = @VCC
				AND AAIE.[IntentId] IN (
					SELECT DISTINCT HAAII.[Id]
						FROM [dbo].[HistoricAdvancedAIIntent] HAAII (NOLOCK)
						WHERE HAAII.[EvaluationPlanId] = @EvaluationPlanId
				)

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveHistoricStudyData] (
	@VCC VARCHAR(50) =        NULL,
	@StudyId    VARCHAR(50) = NULL,
	@TemplateId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	  
	-- INSERTAMOS LA DATA DE LA PLANTILLA
	INSERT INTO [dbo].[HistoricTemplate] 
		SELECT T.[VCC],
				T.[Name],
				T.[Type],
				T.[Description],
				T.[Template],
				T.[Category],
				@StudyId
			FROM [dbo].[Template] T (NOLOCK)
			WHERE T.[Name] = @TemplateId 
				AND T.[VCC] = @VCC

	-- INSERTAMOS LA DATA DE LOS TEMAS
	INSERT INTO [dbo].[HistoricTemplateTheme]
		SELECT TT.[VCC],
				TT.[TemplateId],
				TT.[ID],
				TT.[Order],
				TT.[Description],
				TT.[Category1],
				TT.[Category2],
				TT.[AnalysisType],
				TT.[IsNegative],
				@StudyId
			FROM [dbo].[TemplateTheme] TT (NOLOCK)
			WHERE TT.[TemplateId] = @TemplateId 
				AND TT.[VCC] = @VCC

	-- INSERTAMOS LA DATA DE LOS THEMEITEMS
	INSERT INTO [dbo].[HistoricTemplateThemeItem]
		SELECT TTI.[VCC],
				TTI.[TemplateId],
				TTI.[TemplateThemeId],
				TTI.[ID],
				TTI.[AcceptancePercentage],
				@StudyId
			FROM [dbo].[TemplateThemeItem] TTI (NOLOCK)
			WHERE TTI.[TemplateId] = @TemplateId 
				AND TTI.[VCC] = @VCC

	-- INSERTAMOS LA DATA DE THEMEITEMENTITY
	INSERT INTO [dbo].[HistoricTemplateThemeItemEntity]
		SELECT TTIE.[VCC],
				TTIE.[TemplateId],
				TTIE.[TemplateThemeId],
				TTIE.[TemplateThemeItemId],
				TTIE.[EntityId],
				TTIE.[EntityName],
				TTIE.[AcceptancePercentage],
				TTIE.[Order],
				@StudyId
			FROM [dbo].[TemplateThemeItemEntity] TTIE (NOLOCK)
			WHERE TTIE.[TemplateId] = @TemplateId 
				AND TTIE.[VCC] = @VCC

	-- INSERTAMOS LA DATA DE LOS QUERY (SE USA?)
	INSERT INTO [dbo].[HistoricGenericTemplateQuery]
		SELECT GTQ.[VCC],
				GTQ.[TemplateId],
				GTQ.[ID],
				GTQ.[Query],
				@StudyId
			FROM [dbo].[GenericTemplateQuery] GTQ (NOLOCK)
            WHERE GTQ.[TemplateId] = @TemplateId 
				AND GTQ.[VCC] = @VCC

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveI6InteractionAsyncInteraction] (
	@VCC					VARCHAR(50) = NULL,
	@InteractionId			VARCHAR(250) = NULL,
	@Folder					VARCHAR(50) = NULL,
	@ExportedContent		VARCHAR(250) = NULL,
	@Channel				VARCHAR(25) = NULL,
	@StartDate				DATETIME = NULL,
	@EndDate				DATETIME = NULL,
	@DurationTime			INT = NULL,
	@AgentId				VARCHAR(50) = NULL,
	@Campaign				VARCHAR(100) = NULL,
	@Account				VARCHAR(50) = NULL,
	@Origin					VARCHAR(25) = NULL,
	@DispositionTreePath	VARCHAR(50) = NULL,
	@DispositionCode		VARCHAR(50) = NULL,
	@ContactId				VARCHAR(50) = NULL,
	@ContactName			VARCHAR(300) = NULL,
	@ContactAddress			VARCHAR(300) = NULL,
	@TotalMessages			INT = NULL,
	@ProcessId				VARCHAR(50) = NULL,
	@Ender				    VARCHAR(300) = NULL,
	@Ticket				    VARCHAR(300) = NULL
) AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR UNA INTERACCION DE I6
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
	--select * from AsyncInteraction (nolock)
	--select * from InteractionEvaluationPlan (nolock)
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- DECLARAMOS UNA VARIABLE PARA AYUDAR A LA ACTUALIZACION
		DECLARE @BD_AI_ID VARCHAR(250) = NULL
		DECLARE @NEED_I6_UPDATE BIT = 1
		DECLARE @AGENT_NAME VARCHAR(100) = NULL -- NOMBRE DEL AGENTE
		DECLARE @EVALUATION_PLAN_ID VARCHAR(50) = NULL -- ID DEL PLAN EN CASO EXISTA

		-- ASIGNAMOS EL VALOR ACTUALIZADO DEL FLAG
		SELECT @BD_AI_ID = AI.[Id],
				@NEED_I6_UPDATE = AI.[NeedI6Update],
                @AgentId = IIF(@AgentId IS NULL OR @AgentId = '', AI.[LastTakedAgent], @AgentId)
			FROM [dbo].[AsyncInteraction] AI (NOLOCK)
			WHERE AI.[VCC] = @VCC
				AND AI.[ID] = @InteractionId
				AND AI.[Folder] = @Folder

		-- BUSCAMOS EL ID DEL PLAN EN CASO EXISTA
		SELECT @EVALUATION_PLAN_ID = EP.[ID]
			FROM [dbo].[EvaluationPlan] EP (NOLOCK)
			WHERE EP.[Instance] = @VCC 
				AND EP.[ProcessId] = @ProcessId
				AND EP.[Status] < 5
		
		-- ASIGNAMOS A LA VARIABLE EL NOMBRE DEL AGENTE
		IF @AgentId IS NULL OR @AgentId = ''
        BEGIN
            IF @EVALUATION_PLAN_ID IS NOT NULL
                    SELECT @AgentId = [IdAgent]
                            FROM [dbo].[AuditorEvaluationPlan] (NOLOCK)
                            WHERE [IdEvaluationPlan] = @EVALUATION_PLAN_ID
                                    AND [IdInteraction] = @InteractionId
            -- PRINT(@AgentId)
        END
		SELECT @AGENT_NAME = A.[Name]
			FROM [dbo].[Agent] A (NOLOCK)
			WHERE A.[VCC] = @VCC
				AND A.[AgentId] = @AgentId

		-- ASIGNAMOS EL NAME EN CASO SEA NULL
		SET @AGENT_NAME = IIF((@AgentId IS NOT NULL AND @AgentId <> ''), @AgentId, NULL)
		
		-- EN CASO REQUIERAN ACTUALIZACION POR UNICA VEZ
		IF @NEED_I6_UPDATE = 1 AND @ProcessId IS NOT NULL
		BEGIN
			print @BD_AI_ID
			-- VERIFICAMOS IS EXISTE
			IF @BD_AI_ID IS NOT NULL
			BEGIN
				PRINT 'UPDATE'
				-- ACTUALIZAMOS LA TABLA DE LA INTERACCION EN LA TABLA ASYNC_INTERACTION
				UPDATE [dbo].[AsyncInteraction]
					SET [ExportedContent] = @ExportedContent,
						[Campaign]= @Campaign,
						[StartDate] = @StartDate,
						[EndDate] = @EndDate,
						[Account] = @Account,
						[Initiative] = @Origin,
						[LastTakedAgent] = @AgentId,
						[LastTakedAgentName] = @AGENT_NAME,
						[DurationTime] = @DurationTime,
						[DispositionTreePath] = @DispositionTreePath,
						[DispositionCode] = @DispositionCode,
						[ContactId] = @ContactId,
						[ContactName] = @ContactName,
						[ContactAddress] = @ContactAddress,
						[TotalMessages] = @TotalMessages,
						[NeedI6Update] = 0,
						[IsProcess] = NULL,
						[IsCancel] = NULL,
						[LastModifiedByUserId] = 'system',
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE(),
						[Ender] = @ender,
						[Ticket] = @Ticket
					WHERE [VCC] = @VCC
						AND [ID] = @InteractionId
						AND [Folder] = @Folder
						AND [NeedI6Update] = 1
						AND (
							[IsTranscribed] IS NULL 
							OR
							[IsTranscribed] <> 1
						)

				-- ACTUALIZAMOS EL CAMPAIGN Y ENDDATE EN LA INTERACCION EN LA TABLA PROCESSAUDIO
				UPDATE [dbo].[ProcessAsyncInteraction]
					SET [Campaign] = @Campaign,
						[EndDate] = @EndDate,
						[LastModifiedByUserId] = 'system',
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [VCC] = @VCC
						AND [InteractionId] = @InteractionId
						AND [FormattedEndDate] = @Folder 
						AND [ProcessId] = @ProcessId
			END
			ELSE
			BEGIN
				-- SELECT * FROM AsyncInteraction (NOLOCK)
				-- SELECT * FROM ProcessAsyncInteraction (NOLOCK)
				-- INSERTAMOS EL REGISTRO EN ASYNC_INTERACTION
				INSERT INTO [dbo].[AsyncInteraction] (
					[VCC], [ID], [Folder], [ExportedContent], [Channel], 
					[Campaign], [StartDate], [EndDate], [Account], [Initiative], 
					[ContactId], [ContactName], [ContactAddress], [LastTakedAgent], [LastTakedAgentName], 
					[TotalMessages], [DurationTime], [AttentionTime], [DispositionCode], [DispositionTreePath], 
					[NeedI6Update], [LastModifiedByUserId], [LastModifiedDate], [TimeStamp],
					[Ender], [Ticket]
				) VALUES (
					@VCC, @InteractionId, @Folder, @ExportedContent, @Channel, 
					@Campaign, @StartDate, @EndDate, @Account, @Origin, 
					@ContactId, @ContactName, @ContactAddress, @AgentId, @AGENT_NAME, 
					@TotalMessages, @DurationTime, @DurationTime, @DispositionCode, @DispositionTreePath, 
					0, 'system', GETUTCDATE(), GETUTCDATE(),
					@Ender, @Ticket
				)

				-- INSERTAMOS EL REGISTRO EN PROCESS_ASYNC_INTERACTION
				INSERT INTO [dbo].[ProcessAsyncInteraction] (
					[VCC], [ProcessId], [InteractionType], [Campaign], [InteractionId],
					[EndDate], [FormattedEndDate], [CreatedDate], [CreatedByUserId], [TimeStamp]
				) VALUES (
					@VCC, @ProcessId, @Channel, @Campaign, @InteractionId, 
					@EndDate, @Folder, GETUTCDATE(), 'system', GETUTCDATE()
				)
			END
		END

		-- ACTUALIZAMOS LOS REGISTROS DE PLAN
		EXEC [dbo].[SaveI6InteractionAsyncInteractionForEvaluationPlan] 
			@VCC, @InteractionId, @Folder, @ExportedContent, @Channel,  
			@StartDate, @EndDate, @DurationTime, @AgentId, @AGENT_NAME, 
			@Campaign, @Account, @Origin, @DispositionTreePath, @DispositionCode, 
			@ContactId, @ContactName, @ContactAddress, @TotalMessages, 
			@EVALUATION_PLAN_ID, @Ender, @Ticket

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveI6InteractionAsyncInteractionForEvaluationPlan] (
	@VCC					VARCHAR(50) = NULL,
	@InteractionId			VARCHAR(250) = NULL,
	@Folder					VARCHAR(50) = NULL,
	@ExportedContent		VARCHAR(250) = NULL,
	@Channel				VARCHAR(25) = NULL,
	@StartDate				DATETIME = NULL,
	@EndDate				DATETIME = NULL,
	@DurationTime			INT = NULL,
	@AgentId				VARCHAR(50) = NULL,
	@AgentName				VARCHAR(100) = NULL,
	@Campaign				VARCHAR(100) = NULL,
	@Account				VARCHAR(50) = NULL,
	@Origin					VARCHAR(25) = NULL,
	@DispositionTreePath	VARCHAR(50) = NULL,
	@DispositionCode		VARCHAR(50) = NULL,
	@ContactId				VARCHAR(50) = NULL,
	@ContactName			VARCHAR(300) = NULL,
	@ContactAddress			VARCHAR(300) = NULL,
	@TotalMessages			INT = NULL,
	@EvaluationPlanId		VARCHAR(50) = NULL,
	@Ender					VARCHAR(300) = NULL,
	@Ticket					VARCHAR(300) = NULL
) AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR UNA INTERACCION DE I6
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- DECLARAMOS UN CURSOR PARA RECORRER PLANES
        DECLARE RECURRENT_CURSOR CURSOR 
        LOCAL STATIC READ_ONLY FORWARD_ONLY
        FOR 
        	-- SELECCIONAMOS LOS PLANES DONDE ESTA REGISTRADA LA INTERACCION
			SELECT [IdEvaluationPlan]
				FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				INNER JOIN[dbo].[EvaluationPlan] EP (NOLOCK)
					ON EP.[Instance] = IEP.[Instance]
						AND EP.[ID] = IEP.[IdEvaluationPlan]
				WHERE IEP.[Instance] = @VCC
					AND IEP.[IdInteraction] = @InteractionId
					AND IEP.[NeedI6Update] = 1
					AND (
						IEP.[IdEvaluationPlan] = @EvaluationPlanId
						OR 
						@EvaluationPlanId IS NULL
					)
					AND EP.[Status] <= 6
		
		-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS RECORRIENDO
        DECLARE @CURRENT_EVALUATION_PLAN VARCHAR(50) = ''

		-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
        OPEN RECURRENT_CURSOR

        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_EVALUATION_PLAN
        WHILE @@FETCH_STATUS = 0
        BEGIN 
			-- PARA EL PLAN ACTUAL BUSCAMOS SI EXISTE INTERACCION CON EL FOLDER CORRESPONDIENTE
			DECLARE @EXISTS_IEP BIT = 0
			SELECT @EXISTS_IEP = IIF(IEP.[IdInteraction] IS NOT NULL, CAST(1 AS BIT), CAST(0 AS BIT))
				FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK) 
				WHERE IEP.[Instance] = @VCC
					AND IEP.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN
					AND IEP.[IdInteraction] = @InteractionId
					AND IEP.[FormattedEndDate] = @Folder

			-- SI LA INTERACCION-FOLDER EXISTE, LA ACTUALIZAMOS
			IF @EXISTS_IEP = 1
			BEGIN
				-- ACTUALIZAMOS LA INTERACCION
				UPDATE [dbo].[InteractionEvaluationPlan] 
					SET [IdCampaign] = @Campaign,
						[IdAgent] = @AgentId,
						[AgentName] = IIF (ISNULL([AgentName], '') = '', @AgentName, [AgentName]),
						[InteractionMessages] = @TotalMessages,
						[InteractionDateStart] = @StartDate,
						[InteractionDateEnd] = @EndDate,
						[InteractionOrigin] = IIF(@Origin = 'IN', 'E', 'S'),
						[ManagementResult] = @DispositionCode,
						[ExportedContent] = @ExportedContent,
						[ContactId] = @ContactId,
						[ContactName] = @ContactName,
						[ContactAddress] = @ContactAddress,
						[NeedI6Update] = 0,
						[LastModifiedByUserId] = 'system',
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE(),
						[Ender] = @Ender,
						[Ticket] = @Ticket
					WHERE [Instance] = @VCC
						AND [IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN
						AND [IdInteraction] = @InteractionId
						AND [FormattedEndDate] = @Folder
						AND [NeedI6Update] = 1
			END
			-- SI LA INTERACCION-FOLDER NO EXISTE, LA INSERTAMOS
			ELSE
			BEGIN
				-- INSERTAMOS LA INTERACCION  SELECT * FROM [dbo].[InteractionEvaluationPlan]
				INSERT INTO [dbo].[InteractionEvaluationPlan] (
					[Instance], [IdEvaluationPlan], [InteractionType], [FormattedEndDate], [IdCampaign],
					[IdInteraction], [IdAgent], [AgentName], [InteractionMessages], [InteractionDateStart], 
					[InteractionDateEnd], [InteractionDuration], [InteractionOrigin], [ManagementResult], [ExportedContent],
					[ContactId], [ContactName], [ContactAddress], [NeedI6Update], [CreatedByUserId], 
					[CreatedDate], [TimeStamp], [Ender], [Ticket]
				) VALUES (
					@VCC, @CURRENT_EVALUATION_PLAN, @Channel, @Folder, @Campaign,
					@InteractionId, @AgentId, @AgentName, @TotalMessages, @StartDate, 
					@EndDate, @DurationTime, IIF(@Origin = 'IN', 'E', 'S'), @DispositionCode, @ExportedContent,
					@ContactId, @ContactName, @ContactAddress, CAST(0 AS BIT), 'system', 
					GETUTCDATE(), GETUTCDATE(), @Ender, @Ticket
				)
			END

			-- DECLARAMOS UN CURSOR PARA RECORRER AUDITORES
			DECLARE RECURRENT_CURSOR_02 CURSOR 
			LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
        		-- SELECCIONAMOS LOS AUDITORES QUE TIENEN ASIGNADA LA INTERACCION
				SELECT AEP.[IdAuditor]
					FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
					WHERE AEP.[Instance] = @VCC 
						AND AEP.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN
						AND AEP.[IdInteraction] = @InteractionId

			-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS RECORRIENDO
			DECLARE @CURRENT_AUDITOR VARCHAR(50) = ''

			-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
			OPEN RECURRENT_CURSOR_02

			FETCH NEXT FROM RECURRENT_CURSOR_02 INTO @CURRENT_AUDITOR
			WHILE @@FETCH_STATUS = 0
			BEGIN 
				-- PARA EL PLAN ACTUAL BUSCAMOS SI EXISTE INTERACCION CON EL FOLDER CORRESPONDIENTE
				DECLARE @EXISTS_AEP BIT = 0
				SELECT @EXISTS_AEP = IIF(AEP.[IdInteraction] IS NOT NULL, CAST(1 AS BIT), CAST(0 AS BIT))
					FROM [dbo].[InteractionEvaluationPlan] AEP (NOLOCK) 
					WHERE AEP.[Instance] = @VCC
						AND AEP.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN
						AND AEP.[IdInteraction] = @InteractionId
						AND AEP.[FormattedEndDate] = @Folder

				-- SI LA INTERACCION-FOLDER EXISTE, LA ACTUALIZAMOS
				IF @EXISTS_IEP = 1
				BEGIN
					-- ACTUALIZAMOS EL REGISTRO DE AUDITOREVALUATIONPLAN
					UPDATE [dbo].[AuditorEvaluationPlan] 
						SET [IdCampaign] = @Campaign,
							[IdAgent] = @AgentId,
							[InteractionDate] = @StartDate,
							[InteractionDateEnd] = @EndDate,
							[InteractionOrigin] = @Origin,
							[ContactId] = @ContactId,
							[ContactName] = @ContactName,
							[ContactAddress] = @ContactAddress,
							[ManagementResult] = @DispositionCode,
							[LastModifiedByUserId] = 'system',
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE()
						WHERE [Instance] = @VCC
							AND [IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN
							AND [IdInteraction] = @InteractionId
							AND [IdAuditor] = @CURRENT_AUDITOR
				END
				/* NO ES NECESARIO INSERTAR
				ELSE
				BEGIN
					-- INSERTAMOS EL REGISTRO A AUDITOREVALUATIONPLA
					INSERT INTO [dbo].[AuditorEvaluationPlan] (
						[Instance], [IdEvaluationPlan], [IdAuditor], [IdInteraction], [Completed], 
						[Score], [IdCampaign], [IdAgent], [InteractionDate], [InteractionType], 
						[InteractionDuration], [InteractionOrigin], [InteractionDateEnd], [ManagementResult], [ContactId], 
						[ContactName], [ContactAddress], [Status], [StatusDescription],
						[CreatedByUserId], [CreatedDate], [TimeStamp]
					) VALUES (
						@VCC, @CURRENT_EVALUATION_PLAN, @CURRENT_AUDITOR, @InteractionId, 0,
						0, @Campaign, @AgentId, @StartDate, @Channel, 
						@DurationTime, @Origin, @EndDate, @DispositionTreePath + @DispositionCode, @ContactId,
						@ContactName, @ContactAddress, 1, 'NEW',
						'system', GETUTCDATE(), GETUTCDATE()
					)
				END
				*/

				FETCH NEXT FROM RECURRENT_CURSOR_02 INTO @CURRENT_AUDITOR
			END
			CLOSE RECURRENT_CURSOR_02
			DEALLOCATE RECURRENT_CURSOR_02

		    FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_EVALUATION_PLAN
        END
        CLOSE RECURRENT_CURSOR
        DEALLOCATE RECURRENT_CURSOR

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveI6InteractionAudio] (
	@VCC					VARCHAR(50) = NULL,
	@InteractionId			VARCHAR(250) = NULL,
	@OldSection				INT = NULL,
	@OldPart				INT = NULL,
	@NewSection				INT = NULL,
	@NewPart				INT = NULL,
	@StartDate				DATETIME = NULL,
	@EndDate				DATETIME = NULL,
	@DurationTime			INT = NULL,
	@AgentId				VARCHAR(50) = NULL,
	@Format					VARCHAR(10) = NULL,
	@Campaign				VARCHAR(100) = NULL,
	@Origin					VARCHAR(25) = NULL,
	@Category1				VARCHAR(100) = NULL,
	@Category2				VARCHAR(100) = NULL,
	@Category3				VARCHAR(100) = NULL,
	@Description			VARCHAR(100) = NULL,
	@Filename				VARCHAR(200) = NULL,
	@ContactId				VARCHAR(50) = NULL,
	@ContactName			VARCHAR(300) = NULL,
	@ContactAddress			VARCHAR(300) = NULL,
	@ProcessId				VARCHAR(50) = NULL,
	@Ender				    VARCHAR(300) = NULL,
	@Ticket					VARCHAR(300) = NULL
) AS 
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR UNA INTERACCION DE I6
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- DECLARAMOS UNA VARIABLE PARA AYUDAR A LA ACTUALIZACION
		DECLARE @BD_A_ID VARCHAR(250) = NULL
		DECLARE @NEED_I6_UPDATE BIT = 1
		DECLARE @AGENT_NAME VARCHAR(100) = NULL -- NOMBRE DEL AGENTE
		DECLARE @EVALUATION_PLAN_ID VARCHAR(50) = NULL -- ID DEL PLAN EN CASO EXISTA

		-- ASIGNAMOS EL VALOR ACTUALIZADO DEL FLAG
		SELECT @BD_A_ID = A.[ID],
				@NEED_I6_UPDATE = A.[NeedI6Update],
				@AgentId = IIF(@AgentId IS NULL OR @AgentId = '', A.[Agent], @AgentId)
			FROM [dbo].[Audio] A (NOLOCK)
			WHERE A.[VCC] = @VCC
				AND A.[ID] = @InteractionId
				AND A.[Section] = @OldSection
				AND A.[Part] = @OldPart

		-- BUSCAMOS EL ID DEL PLAN EN CASO EXISTA
		SELECT @EVALUATION_PLAN_ID = EP.[ID]
			FROM [dbo].[EvaluationPlan] EP (NOLOCK)
			WHERE EP.[Instance] = @VCC 
				AND EP.[ProcessId] = @ProcessId
				AND EP.[Status] = 1

		-- ASIGNAMOS A LA VARIABLE EL NOMBRE DEL AGENTE
		IF @AgentId IS NULL OR @AgentId = ''
		BEGIN
			IF @EVALUATION_PLAN_ID IS NOT NULL
				SELECT @AgentId = [IdAgent]
					FROM [dbo].[AuditorEvaluationPlan] (NOLOCK)
					WHERE [IdEvaluationPlan] = @EVALUATION_PLAN_ID
						AND [IdInteraction] = @InteractionId

			PRINT(@AgentId)
		END
		SELECT @AGENT_NAME = A.[Name]
			FROM [dbo].[Agent] A (NOLOCK)
			WHERE A.[VCC] = @VCC
				AND A.[AgentId] = @AgentId

		-- EN CASO REQUIERAN ACTUALIZACION POR UNICA VEZ
		IF @NEED_I6_UPDATE = 1 AND @ProcessId IS NOT NULL
		BEGIN
			-- VERIFICAMOS IS EXISTE
			IF @BD_A_ID IS NOT NULL
			BEGIN
				-- ACTUALIZAMOS EL SECTION Y PART EN LA INTERACCION EN LA TABLA AUDIO
				UPDATE [dbo].[Audio] 
					SET [Section] = @NewSection,
						[Part] = @NewPart,
						[Agent] = IIF (@AgentId IS NULL OR @AgentId = '', [Agent], @AgentId),
						[AgentName] = IIF (@AGENT_NAME IS NULL OR @AGENT_NAME = '', [AgentName], @AGENT_NAME),
						[Format] = @Format,
						[Campaign]= @Campaign,
						[Type] = @Origin,
						[Category1] = @Category1,
						[Category2] = @Category2,
						[Category3] = @Category3,
						[Description] = @Description,
						[FileName] = @Filename,
						[ContactId] = @ContactId,
						[ContactName] = @ContactName,
						[ContactAddress] = @ContactAddress,
						[NeedI6Update] = 0,
						[IsProcess] = NULL,
						[IsCancel] = NULL,
						[LastModifiedByUserId] = 'system',
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE(),
						[Ender] = @ender,
						[Ticket] = @Ticket
					WHERE [VCC] = @VCC
						AND [ID] = @InteractionId
						AND [Section] = @OldSection
						AND [Part] = @OldPart
						AND [NeedI6Update] = 1
						AND (
							[IsTranscribed] IS NULL 
							OR
							[IsTranscribed] <> 1
						)

				IF @@ROWCOUNT > 0
				BEGIN
					-- ACTUALIZAMOS EL SECTION Y PART EN LA INTERACCION EN LA TABLA PROCESSAUDIO
					UPDATE [dbo].[ProcessAudio]
						SET [Section] = @NewSection,
							[Part] = @NewPart
						WHERE [VCC] = @VCC
							AND [AudioId] = @InteractionId
							AND [Section] = @OldSection
							AND [Part] = @OldPart 
				END
			END
			ELSE
			BEGIN
				-- INSERTAMOS EL NUEVO AUDIO CUANDO NO RECIBIMOS SECTION Y PART
				INSERT INTO [dbo].[Audio] (
					[VCC], [Id], [Section], [Part], [Channel],
					[Number], [Format], [DurationTime], [Category1], [Category2],
					[Category3], [Description], [Filename], [Campaign], [Agent], 
					[AgentName], [Type], [StartDate], [EndDate], [ContactId], 
					[ContactName], [ContactAddress], [NeedI6Update], [CreatedByUserId], [CreatedDate], 
					[TimeStamp], [Ender], [Ticket]
				)
				VALUES (
					@VCC, @InteractionId, @NewSection, @NewPart, 'CALL',
					@ContactAddress, @Format, @DurationTime, @Category1, @Category2,
					@Category3, @Description, @Filename, @Campaign, @AgentId, 
					@AGENT_NAME, @Origin, @StartDate, @EndDate, @ContactId, 
					@ContactName, @ContactAddress, 0, 'system', GETUTCDATE(), 
					GETUTCDATE(), @Ender, @Ticket
				)

				INSERT INTO [dbo].[ProcessAudio] (
					[VCC], [ProcessId], [AudioId], [Section], [Part],
					[CreatedDate], [CreatedByUserId], [TimeStamp]
				)
				VALUES (
					@VCC, @ProcessId, @InteractionId, @NewSection, @NewPart,
					GETUTCDATE(), 'system', GETUTCDATE()
				)
			END 
		END

		-- ACTUALIZAMOS LOS REGISTROS DE PLAN
		EXEC [dbo].[SaveI6InteractionSyncInteractionForEvaluationPlan] 
			@VCC, @InteractionId, @OldSection, @OldPart, @NewSection,
			@NewPart, @StartDate, @EndDate, @DurationTime, @Format,
			@AgentId, @AGENT_NAME, @Campaign, @Origin, @Category1,
			@Category2, @Category3, @Description, @ContactId, @ContactName,
			@ContactAddress, @EVALUATION_PLAN_ID, @Ender, @Ticket
			
		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveI6InteractionSyncInteractionForEvaluationPlan] (
	@VCC					VARCHAR(50) = NULL,
	@InteractionId			VARCHAR(250) = NULL,
	@OldSection				INT = NULL,
	@OldPart				INT = NULL,
	@NewSection				INT = NULL,
	@NewPart				INT = NULL,
	@StartDate				DATETIME = NULL,
	@EndDate				DATETIME = NULL,
	@DurationTime			INT = NULL,
	@Format					VARCHAR(10) = NULL,
	@AgentId				VARCHAR(50) = NULL,
	@AgentName				VARCHAR(100) = NULL,
	@Campaign				VARCHAR(100) = NULL,
	@Origin					VARCHAR(25) = NULL,
	@Category1				VARCHAR(100) = NULL,
	@Category2				VARCHAR(100) = NULL,
	@Category3				VARCHAR(100) = NULL,
	@Description			VARCHAR(100) = NULL,
	@ContactId				VARCHAR(50) = NULL,
	@ContactName			VARCHAR(300) = NULL,
	@ContactAddress			VARCHAR(300) = NULL,
	@EvaluationPlanId		VARCHAR(50) = NULL,
	@Ender				    VARCHAR(300) = NULL,
	@Ticket					VARCHAR(300) = NULL
) AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR UNA INTERACCION DE I6
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		--DECLARAMOS UNA VARIABLE PARA DEFINIR EL RESULT
		DECLARE @RESULT_MANAGEMENT VARCHAR(250) = @Category1 + @Category2 + @Category3 + @Description

		-- DECLARAMOS UN CURSOR PARA RECORRER PLANES
        DECLARE RECURRENT_CURSOR CURSOR 
        LOCAL STATIC READ_ONLY FORWARD_ONLY
        FOR 
        	-- SELECCIONAMOS LOS PLANES DONDE ESTA REGISTRADA LA INTERACCION
			SELECT DISTINCT [IdEvaluationPlan]
				FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				INNER JOIN[dbo].[EvaluationPlan] EP (NOLOCK)
					ON EP.[Instance] = IEP.[Instance]
						AND EP.[ID] = IEP.[IdEvaluationPlan]
				WHERE IEP.[Instance] = @VCC
					AND IEP.[IdInteraction] = @InteractionId
					AND IEP.[NeedI6Update] = 1
					AND (
						IEP.[IdEvaluationPlan] = @EvaluationPlanId
						OR 
						@EvaluationPlanId IS NULL
					)
					AND EP.[Status] <= 6

		-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS RECORRIENDO
        DECLARE @CURRENT_EVALUATION_PLAN VARCHAR(50) = ''

		-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
        OPEN RECURRENT_CURSOR

        FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_EVALUATION_PLAN
        WHILE @@FETCH_STATUS = 0
        BEGIN 
			-- PARA EL PLAN ACTUAL BUSCAMOS SI EXISTE INTERACCION CON EL OLD-SECTION Y OLD-PART CORRESPONDIENTE
			DECLARE @EXISTS_IEP BIT = 0
			SELECT @EXISTS_IEP = IIF(IEP.[IdInteraction] IS NOT NULL, CAST(1 AS BIT), CAST(0 AS BIT))
				FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK) 
				WHERE IEP.[Instance] = @VCC
					AND IEP.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN
					AND IEP.[IdInteraction] = @InteractionId
					AND IEP.[InteractionSection] = @OldSection
					AND IEP.[InteractionPart] = @OldPart

			-- SI LA INTERACCION-SECTION-PART EXISTE, LA ACTUALIZAMOS
			IF @EXISTS_IEP = 1
			BEGIN
				-- ACTUALIZAMOS LA INTERACCION
				UPDATE [dbo].[InteractionEvaluationPlan] 
					SET [IdCampaign] = @Campaign,
						[InteractionSection] = @NewSection,
						[InteractionPart] = @NewPart,
						-- SI YA EXISTE VALOR EN LOS 2 CAMPOS, ES MEJOR CONSERVARLOS
						-- [IdAgent] = IIF (ISNULL(@AgentId, '') = '', [IdAgent], @AgentId),
						-- [AgentName] = IIF (ISNULL(@AgentName, '') = '', [AgentName], @AgentName),
						[IdAgent] = IIF (ISNULL([IdAgent], '') = '', @AgentId, [IdAgent]),
						[AgentName] = IIF (ISNULL([AgentName], '') = '', @AgentName, [AgentName]),
						[InteractionDateStart] = @StartDate,
						[InteractionDateEnd] = @EndDate,
						[InteractionOrigin] = @Origin,
						[InteractionFormat] = @Format,
						[ManagementResult] = IIF (ISNULL(@RESULT_MANAGEMENT, '') = '', [ManagementResult], @RESULT_MANAGEMENT), 
						[ContactId] = @ContactId,
						[ContactName] = @ContactName,
						[ContactAddress] = @ContactAddress,
						[NeedI6Update] = 0,
						[LastModifiedByUserId] = 'system',
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE(),
						[Ender] = @Ender,
						[Ticket] = @Ticket
					WHERE [Instance] = @VCC
						AND [IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN
						AND [IdInteraction] = @InteractionId
						AND [InteractionSection] = @OldSection
						AND [InteractionPart] = @OldPart
						AND [NeedI6Update] = 1
			END
			-- SI LA INTERACCION-SECTION-PART NO EXISTE, LA INSERTAMOS
			ELSE
			BEGIN
				-- INSERTAMOS LA INTERACCION  SELECT * FROM [dbo].[InteractionEvaluationPlan]
				INSERT INTO [dbo].[InteractionEvaluationPlan] (
					[Instance], [IdEvaluationPlan], [InteractionType], [IdCampaign], [IdInteraction], 
					[InteractionSection], [InteractionPart], [IdAgent], [AgentName], [InteractionDateStart], 
					[InteractionDateEnd], [InteractionDuration], [InteractionOrigin], [InteractionFormat], [ManagementResult], 
					[ContactId], [ContactName], [ContactAddress], [NeedI6Update], [CreatedByUserId], 
					[CreatedDate], [TimeStamp], [Ender], [Ticket]
				) VALUES (
					@VCC, @CURRENT_EVALUATION_PLAN, 'CALL', @Campaign, @InteractionId, 
					@NewSection, @NewPart, @AgentId, @AgentName, @StartDate, 
					@EndDate, @DurationTime, @Origin, @Format, @Category1 + @Category2 + @Category3 + @Description, 
					@ContactId, @ContactName, @ContactAddress, CAST(0 AS BIT), 'system', 
					GETUTCDATE(), GETUTCDATE(), @Ender, @Ticket
				)
			END

			-- DECLARAMOS UN CURSOR PARA RECORRER AUDITORES
			DECLARE RECURRENT_CURSOR_02 CURSOR 
			LOCAL STATIC READ_ONLY FORWARD_ONLY
			FOR 
        		-- SELECCIONAMOS LOS AUDITORES QUE TIENEN ASIGNADA LA INTERACCION
				SELECT AEP.[IdAuditor]
					FROM [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
					WHERE AEP.[Instance] = @VCC 
						AND AEP.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN
						AND AEP.[IdInteraction] = @InteractionId

			-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL CAMPO ACTUAL QUE ESTAMOS RECORRIENDO
			DECLARE @CURRENT_AUDITOR VARCHAR(50) = ''

			-- ABRIMOS Y RECORREMOS EL ARRAY DE CAMPOS A ACTUALIZAR
			OPEN RECURRENT_CURSOR_02

			FETCH NEXT FROM RECURRENT_CURSOR_02 INTO @CURRENT_AUDITOR
			WHILE @@FETCH_STATUS = 0
			BEGIN 
				-- PARA EL PLAN ACTUAL BUSCAMOS SI EXISTE INTERACCION CON EL FOLDER CORRESPONDIENTE
				DECLARE @EXISTS_AEP BIT = 0
				SELECT @EXISTS_AEP = IIF(AEP.[IdInteraction] IS NOT NULL, CAST(1 AS BIT), CAST(0 AS BIT))
					FROM [dbo].[InteractionEvaluationPlan] AEP (NOLOCK) 
					WHERE AEP.[Instance] = @VCC
						AND AEP.[IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN
						AND AEP.[IdInteraction] = @InteractionId

				-- SI LA INTERACCION-FOLDER EXISTE, LA ACTUALIZAMOS
				IF @EXISTS_AEP = 1
				BEGIN
					-- ACTUALIZAMOS EL REGISTRO DE AUDITOREVALUATIONPLAN
					UPDATE [dbo].[AuditorEvaluationPlan] 
						SET [IdCampaign] = @Campaign,
							[IdAgent] = IIF (ISNULL(@AgentId, '') = '', [IdAgent], @AgentId),
							[InteractionFormat] = @Format,
							[ManagementResult] = IIF (ISNULL(@RESULT_MANAGEMENT, '') = '', [ManagementResult], @RESULT_MANAGEMENT), 
							[ContactId] = @ContactId,
							[ContactName] = @ContactName,
							[ContactAddress] = @ContactAddress,
							[LastModifiedByUserId] = 'system',
							[LastModifiedDate] = GETUTCDATE(),
							[TimeStamp] = GETUTCDATE()
						WHERE [Instance] = @VCC
							AND [IdEvaluationPlan] = @CURRENT_EVALUATION_PLAN
							AND [IdInteraction] = @InteractionId
							AND [IdAuditor] = @CURRENT_AUDITOR
							AND [Completed] = 0
				END
				ELSE
				BEGIN
					-- INSERTAMOS EL REGISTRO A AUDITOREVALUATIONPLA
					INSERT INTO [dbo].[AuditorEvaluationPlan] (
						[Instance], [IdEvaluationPlan], [IdAuditor], [IdInteraction], [Completed], 
						[Score], [IdCampaign], [IdAgent], [InteractionDate], [InteractionType], 
						[InteractionDuration], [InteractionOrigin], [InteractionDateEnd], [ManagementResult], [ContactId], 
						[ContactName], [ContactAddress], [Status], [StatusDescription],
						[CreatedByUserId], [CreatedDate], [TimeStamp]
					) VALUES (
						@VCC, @CURRENT_EVALUATION_PLAN, @CURRENT_AUDITOR, @InteractionId, 0,
						0, @Campaign, @AgentId, @StartDate, 'CALL', 
						@DurationTime, @Origin, @EndDate, @Category1 + @Category2 + @Category3 + @Description, @ContactId,
						@ContactName, @ContactAddress, 1, 'NEW',
						'system', GETUTCDATE(), GETUTCDATE()
					)
				END

				FETCH NEXT FROM RECURRENT_CURSOR_02 INTO @CURRENT_AUDITOR
			END
			CLOSE RECURRENT_CURSOR_02
			DEALLOCATE RECURRENT_CURSOR_02

			FETCH NEXT FROM RECURRENT_CURSOR INTO @CURRENT_EVALUATION_PLAN
        END
        CLOSE RECURRENT_CURSOR
        DEALLOCATE RECURRENT_CURSOR

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[SaveIdathaAuth] 'default', 'admin', 'KkmydH2Gx', 'in_concert'
CREATE PROCEDURE [dbo].[SaveIdathaAuth]
(
	@VCC					VARCHAR(50),
	@UserName				VARCHAR(50),
	@Password				VARCHAR(200),
	@Realm					VARCHAR(100)
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [dbo].[IdathaAuth] SET
		[Password] = @Password
	WHERE [VCC] = @VCC
	AND [Realm] = @Realm

	IF @@ROWCOUNT = 0

	INSERT INTO [dbo].[IdathaAuth](
		[VCC], [Realm], [Password], [UserName]
	)
	VALUES(
		@VCC, @Realm, @Password, @UserName
	)
	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveIntegratedUser] (
	@VCC Varchar(50) = NULL, 					@ID Varchar(50) = NULL, 				@Email Varchar(50) = NULL,
	@Firstname Varchar(50) = NULL, 				@Lastname Varchar(50) = NULL, 			@Password Varchar(50) = NULL,
	@Role Varchar(50) = NULL, 					@Country Varchar(50) = NULL, 			@Language Varchar(50) = NULL,
	@TimeZone Varchar(50) = NULL,				@Preferences Varchar(200) = NULL, 		@IsSystemUser Bit = NULL,
	@LimitClients Bit = NULL,					@CreatedDate Datetime = NULL,			@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,			@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	IF EXISTS(
	SELECT [VCC], [ID] FROM [User] (NOLOCK)
	WHERE [VCC] = @VCC
	AND [ID] = @ID
	)
	BEGIN
		SELECT 1 AS 'result'
	END 
    ELSE
	BEGIN

		IF EXISTS (SELECT [Country] FROM CountryParser (NOLOCK) WHERE [CountryId] = @Country)
		BEGIN
			SET @Country = (SELECT [Country] FROM CountryParser (NOLOCK) WHERE [CountryId] = @Country)
		END
		ELSE
		BEGIN
			SET @Country = NULL;
		END

		IF EXISTS (SELECT [TimeZone] FROM TimeZoneParser (NOLOCK) WHERE [TimeZoneId] = @TimeZone)
		BEGIN
			SET @TimeZone = (SELECT [TimeZone] FROM TimeZoneParser (NOLOCK) WHERE [TimeZoneId] = @TimeZone)
		END
		ELSE
		BEGIN
			SET @TimeZone = NULL;
		END

		INSERT INTO [User] (
			[VCC], [ID], [Email],
			[Firstname], [Lastname], [Password],
			[Role], [Country], [Language],
			[TimeZone], [Preferences], [IsSystemUser],
			[LimitClients], [CreatedDate], [CreatedByUserId],
			[LastModifiedDate], [LastModifiedByUserId],
			[TimeStamp]
			)
		VALUES (
			@VCC, @ID, @Email,
			@Firstname, @Lastname, @Password,
			@Role, @Country, @Language,
			@TimeZone, @Preferences, @IsSystemUser,
			@LimitClients, @CreatedDate, @CreatedByUserId,
			@LastModifiedDate, @LastModifiedByUserId,
			GETUTCDATE()
		)
		SELECT 1 AS 'result'
	END	
   
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveIntent] (
    @VCC					VARCHAR(50) = NULL,
    @Name					VARCHAR(50) = NULL,
    @IntentId				VARCHAR(50) = NULL,
    @AppId					VARCHAR(50) = NULL,
    @Color					VARCHAR(50) = NULL,
    @CreatedDate			DATETIME = NULL,
    @CreatedByUserId		VARCHAR(50) = NULL,
    @LastModifiedDate		DATETIME = NULL,
    @LastModifiedByUserId	VARCHAR(50) = NULL
)
AS
BEGIN
    SET NOCOUNT ON

    UPDATE [Intent]
    SET [Name] = @Name,
        [CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
        [CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
        [LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
        [LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
        [TimeStamp] = GETUTCDATE()
    WHERE [VCC] = @VCC
      AND [Name] = @Name
      AND [AppId] = @AppId

    IF @@ROWCOUNT = 0
        BEGIN
            INSERT INTO [Intent] (
                [VCC], [Name], [IntentId], [AppId],
                [Color], [CreatedDate], [CreatedByUserId],
                [LastModifiedDate], [LastModifiedByUserId], [TimeStamp],[Status],[StatusCode]
            )
            VALUES (
                       @VCC, @Name, @IntentId, @AppId,
                       @Color, @CreatedDate, @CreatedByUserId,
                       @LastModifiedDate, @LastModifiedByUserId, GETUTCDATE(),1,'AVAILABLE'
                   )
        END

    --Marco el NLU como pendiente de entrenamiento
    UPDATE
        [dbo].[NLU]
    SET
        [NeedTrain] = 1
    WHERE
            [VCC] = @VCC AND [AppId] = @AppId

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveInteractionEvaluationPlan] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionType VARCHAR(10) = NULL,
	@FormattedEndDate VARCHAR(50) = NULL,
	@CampaignId VARCHAR(100) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@InteractionSection INT = NULL,
	@InteractionPart INT = NULL,
	@AgentId VARCHAR(50) = NULL,
	@AgentName VARCHAR(250) = NULL,
	@InteractionMessages INT = NULL,
	@InteractionDateStart DATETIME = NULL,
	@InteractionDateEnd DATETIME = NULL,
	@InteractionDuration NUMERIC(8, 0) = NULL,
	@InteractionOrigin VARCHAR(10) = NULL,
	@InteractionFormat VARCHAR(5) = NULL,
	@ManagementResult VARCHAR(250) = NULL,
	@ExportedContent VARCHAR(250) = NULL,
	@RelativePath VARCHAR(500) = NULL,
	@RelativeFileName VARCHAR(1000) = NULL,
	@FileName VARCHAR(200) = NULL,
	@ContactId VARCHAR(100) = NULL,
	@ContactName VARCHAR(300) = NULL,
	@ContactAddress VARCHAR(300) = NULL,
	@NeedI6Update BIT = NULL,
	@Ender VARCHAR(300) = NULL,
	@Ticket VARCHAR(300) = NULL,
	@CurrentUserId VARCHAR(50) = NULL
) AS
BEGIN
	-- ESTE SP INSERTA UN NUEVO REGISTRO EN LA TABLA [dbo].[Audio] (PRINCIPLAMENTE PARA REALTIME)
	SET NOCOUNT ON

	UPDATE [dbo].[InteractionEvaluationPlan] 
		SET [FormattedEndDate] = @FormattedEndDate, 
			[AgentName] = @AgentName,
			[InteractionMessages] = @InteractionMessages, 
			[InteractionDateStart] = @InteractionDateStart, 
			[InteractionDateEnd] = @InteractionDateEnd, 
			[InteractionDuration] = @InteractionDuration, 
			[InteractionFormat] = @InteractionFormat, 
			[ManagementResult] = @ManagementResult, 
			[ExportedContent] = @ExportedContent, 
			[RelativePath] = @RelativePath, 
			[RelativeFileName] = @RelativeFileName, 
			[FileName] = @FileName, 
			[ContactId] = @ContactId, 
			[ContactName] = @ContactName, 
			[ContactAddress] = @ContactAddress, 
			[Ender] = @Ender, 
			[Ticket] = @Ticket, 
			[LastModifiedByUserId] = @CurrentUserId, 
			[LastModifiedDate] = GETUTCDATE(), 
			[TimeStamp] = GETUTCDATE()
		WHERE [Instance] = @VCC
			AND [IdEvaluationPlan] = @EvaluationPlanId
			AND [IdInteraction] = @InteractionId
			AND [InteractionSection] = @InteractionSection
			AND [InteractionSection] = @InteractionPart

	-- SI NO SE ACTUALIZO NINGUN REGISTRO LO INSERTAMOS
	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [dbo].[InteractionEvaluationPlan] (
			[Instance], [IdEvaluationPlan], [InteractionType], [FormattedEndDate], [IdCampaign],
			[IdInteraction], [InteractionSection], [InteractionPart], [IdAgent], [AgentName],
			[InteractionMessages], [InteractionDateStart], [InteractionDateEnd], [InteractionDuration], [InteractionOrigin],
			[InteractionFormat], [ManagementResult], [ExportedContent], [RelativePath], [RelativeFileName], 
			[FileName], [ContactId], [ContactName], [ContactAddress], [NeedI6Update], 
			[Ender], [Ticket], 
			[CreatedByUserId], [CreatedDate], [TimeStamp]
		)
		VALUES (
			@VCC, @EvaluationPlanId, @InteractionType, @FormattedEndDate, @CampaignId,
			@InteractionId, @InteractionSection, @InteractionPart, @AgentId, @AgentName,
			@InteractionMessages, @InteractionDateStart, @InteractionDateEnd, @InteractionDuration, @InteractionOrigin,
			@InteractionFormat, @ManagementResult, @ExportedContent, @RelativePath, @RelativeFileName,
			@FileName, @ContactId, @ContactName, @ContactAddress, @NeedI6Update,
			@Ender, @Ticket,
			@CurrentUserId, GETUTCDATE(), GETUTCDATE()
		)
	END

	SELECT 1 AS 'result'
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveInteractionEvaluationPlanResult] (
	@VCC VARCHAR(50),
	@IdEvaluationPlan VARCHAR(50),
	@IdAuditor VARCHAR(50),
	@IdInteraction VARCHAR(250),
	@InteractionType VARCHAR(50),
	@Score NUMERIC(5,2),
	@AutomaticScore NUMERIC(5,2),
	@ManualScore NUMERIC(5,2),
	@EvaluationTime INT,
	@IsDraft BIT
) AS
BEGIN
	SET NOCOUNT ON

	-- EN CASO LLEGUE NULL ASIGNAMOS UN VALOR
	IF @IsDraft IS NULL
		SET @IsDraft = 0

 	UPDATE [dbo].[AuditorEvaluationPlan] 
		SET [Completed] = IIF (@IsDraft = 1, 0, 1), 
			[Score] = @Score, 
			[AutomaticScore] = @AutomaticScore,
			[ManualScore] = @ManualScore,
			[EvaluationTime] = @EvaluationTime,
			[StatusDescription] = IIF (@IsDraft = 1, 'DRAFT', 'COMPLETED'),
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [Instance] = @VCC
			AND [IdEvaluationPlan] = @IdEvaluationPlan
			AND [IdInteraction] = @IdInteraction
			AND [InteractionType] = @InteractionType
			AND [IdAuditor] = @IdAuditor

	-- ACTUALIZAMOS LAS NOTAS FINALES TENIENDO EN CUENTA LAS PENALIZACIONES
	IF @IsDraft = 0
	BEGIN
		/****************************************************************************************/
		/*INICIO SECCION PARA ACTUALIZAR LA CALIFICACION TOTAL DE LA EVALUACION C/S PENALIZACION*/
		/****************************************************************************************/

		DECLARE @EvaluationPlans VARCHAR(MAX) = @IdEvaluationPlan
	
		-- EMULAMOS UNA NUEVA TABLA VARIABLE PARA DETERMINAR LAS INTERACCIONES QUE TIENEN HABILIDAD(ES) PENALIZADA(S) POR NOTA CERO
		--  ORDENANDO POR PRIORIDAD (INTERACCION 1, GRUPO 2 Y SUBGRUPO 3)
		;WITH CTEFP AS (
			SELECT DISTINCT 
					ER.[Instance],
					ER.[IdInteraction],
					ER.[IdEvaluationPlan],
					ER.[IdEvaluationFormat],
					ER.[IdAbility],
					CASE WHEN ER.[AbilityInnerScore] = 0 AND ER.[NotApplies] = 0 AND HAEF.[Penalizes] <> 'NO' THEN 1 ELSE 0 END 'Penalized',
					HAEF.[Penalizes],
					CASE HAEF.[Penalizes]
						WHEN 'INT' THEN ER.[IdInteraction] 
						WHEN 'GRP' THEN ER.[IdGroup] 
						WHEN 'SGR' THEN ER.[IdSubGroup] 
					END 'PenalizedOn',
					RN = ROW_NUMBER() OVER(PARTITION BY ER.[IdInteraction], CASE WHEN ER.[AbilityInnerScore] = 0 THEN 1 ELSE 0 END 
						ORDER BY CASE HAEF.[Penalizes]
						WHEN 'INT' THEN 1 
						WHEN 'GRP' THEN 2 
						WHEN 'SGR' THEN 3 
					END)
				FROM [dbo].[EvaluationResult] ER (NOLOCK) 
				LEFT JOIN [HistoricAbilityEvaluationFormat] HAEF (NOLOCK) 
					ON HAEF.[VCC] = ER.[Instance]
						AND HAEF.[EvaluationPlanId] = ER.[IdEvaluationPlan]
						AND HAEF.[IdEvaluationFormat] = ER.[IdEvaluationFormat]
						AND HAEF.[IdAbility] = ER.[IdAbility]
				WHERE ER.[IdEvaluationPlan] IN (SELECT Value FROM [dbo].[split](@EvaluationPlans,',')) 
					AND ER.[IdInteraction] = @IdInteraction
					AND ER.[IdAuditor] = @IdAuditor
					AND ER.[AbilityInnerScore] = 0
					AND ER.[NotApplies] = 0
					AND HAEF.[Penalizes] <> 'NO'
				-- WHERE ER.[IdEvaluationPlan] IN (SELECT DISTINCT EPA.[EvaluationPlanId] FROM [DummyEvaluationPlanAbility] EPA (NOLOCK) WHERE EPA.[EvaluationPlanId] IS NOT NULL)
		),
		CTEFS AS (
			SELECT ER.[Instance],
					ER.[IdEvaluationPlan],
					ER.[IdInteraction],
					ER.[IdAuditor],
					SUM(ER.[Qualification])															'TOTAL_SCORE',
					SUM(IIF(HA.[Type] = 'MANUAL_AUTOMATIC', ER.[Qualification], 0))					'TOTAL_AUTOMATIC_SCORE',
					SUM(IIF(HA.[Type] = 'MANUAL', ER.[Qualification], 0))							'TOTAL_MANUAL_SCORE',
					--SUM(ER.[Qualification] * CASE WHEN FP2.[Penalized] IS NULL THEN 1 ELSE 0 END) 'PENALIZED_SCORE',
					SUM(ER.[Qualification] * (
						CASE WHEN (
							CASE FP2.[Penalizes]
								WHEN 'INT' THEN ER.[IdInteraction] 
								WHEN 'GRP' THEN ER.[IdGroup] 
								WHEN 'SGR' THEN ER.[IdSubGroup] 
							END = FP2.[PenalizedOn]
							AND FP2.[Penalized] = 1) THEN 0 ELSE 1 END
					))																				'PENALIZED_SCORE',
					SUM(IIF(HA.[Type] = 'MANUAL_AUTOMATIC',
						ER.[Qualification] * (
						CASE WHEN (
							CASE FP2.[Penalizes]
								WHEN 'INT' THEN ER.[IdInteraction] 
								WHEN 'GRP' THEN ER.[IdGroup] 
								WHEN 'SGR' THEN ER.[IdSubGroup] 
							END = FP2.[PenalizedOn]
							AND FP2.[Penalized] = 1
						) THEN 0 ELSE 1 END),
						0) 
					)																				'PENALIZED_AUTOMATIC_SCORE',
					SUM(IIF(HA.[Type] = 'MANUAL',
						ER.[Qualification] * (
						CASE WHEN (
							CASE FP2.[Penalizes]
								WHEN 'INT' THEN ER.[IdInteraction] 
								WHEN 'GRP' THEN ER.[IdGroup] 
								WHEN 'SGR' THEN ER.[IdSubGroup] 
							END = FP2.[PenalizedOn]
							AND FP2.[Penalized] = 1
						) THEN 0 ELSE 1 END),
						0) 
					)																				'PENALIZED_MANUAL_SCORE'
				FROM [dbo].[EvaluationResult] ER (NOLOCK) 
				LEFT JOIN (
					-- HACEMOS UN CRUCE CON LA TABLA VARIABLE Y TOMAMOS SOLO UNA DE LAS PENALIZACIONES POR INTERACCION, EMPEZANDO 
					--  POR LA MAS PRIORITARIA (FP.[RN])
					SELECT DISTINCT FP.[Instance],
									FP.[IdEvaluationPlan], 
									FP.[IdInteraction], 
									FP.[Penalized],
									FP.[Penalizes], 
									FP.[PenalizedOn]
						FROM CTEFP FP (NOLOCK)
						WHERE FP.[Penalizes] <> 'NO'
							AND FP.[RN] = 1
				) FP2
					ON FP2.[Instance] = ER.[Instance]
						AND ER.[IdEvaluationPlan] = FP2.[IdEvaluationPlan]
						AND ER.[IdInteraction] = FP2.[IdInteraction]
						/*
						AND CASE FP2.[Penalizes]
								WHEN 'INT' THEN ER.[IdInteraction] 
								WHEN 'GRP' THEN ER.[IdGroup] 
								WHEN 'SGR' THEN ER.[IdSubGroup] 
							END = FP2.[PenalizedOn]
						AND FP2.[Penalized] = 1
						*/
				INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
					ON HA.[VCC] = ER.[Instance]
						AND HA.[EvaluationPlanId] = ER.[IdEvaluationPlan]
						AND HA.[ID] = ER.[IdAbility]
				--WHERE ER.[Instance] = 'multisalud'
				WHERE -- ER.[Instance] = FP2.[Instance] -- ESTA LINEA CAUSA QUE NO SE ACTUALICEN LOS SCORES LAS EVALUACIONES QUE NO PENALIZAN
					-- AND 
					ER.[IdEvaluationPlan] IN (SELECT Value FROM [dbo].[split](@EvaluationPlans, ','))
					AND ER.[IdInteraction] = @IdInteraction
					AND ER.[IdAuditor] = @IdAuditor
					--AND ER.[IdEvaluationPlan] IN (SELECT DISTINCT EvaluationPlanId FROM DummyEvaluationPlanAbility)
				GROUP BY ER.[Instance],
					ER.[IdEvaluationPlan],
					ER.[IdInteraction],
					ER.[IdAuditor]
		)
		--SELECT * FROM CTEFS

		-- ACTUALIZAMOS LOS REGISTROS DE EVALUACION CON LA NOTA PENALIZADA OBTENIDA EN LOS PASOS ANTERIORES
		UPDATE AEP 
			SET AEP.[Score] = FS.[PENALIZED_SCORE],
				AEP.[AutomaticScore] = FS.[PENALIZED_AUTOMATIC_SCORE],
				AEP.[ManualScore] = FS.[PENALIZED_MANUAL_SCORE],
				AEP.[LastModifiedDate] = GETUTCDATE(),
				AEP.[TimeStamp] = GETUTCDATE()
			FROM AuditorEvaluationPlan AEP (NOLOCK)
			LEFT JOIN CTEFS FS (NOLOCK) 
				ON FS.[Instance] = AEP.[Instance] 
					AND FS.[IdEvaluationPlan] = AEP.[IdEvaluationPlan] 
					AND FS.[IdInteraction] = AEP.[IdInteraction]
					AND FS.[IdAuditor] = AEP.[IdAuditor]
			WHERE FS.[Instance] IS NOT NULL

		/*************************************************************************************/
		/*FIN SECCION PARA ACTUALIZAR LA CALIFICACION TOTAL DE LA EVALUACION C/S PENALIZACION*/
		/*************************************************************************************/
	END

    SELECT 1 AS 'result'

 	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveInteractionEvaluationPlanSample] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@EvaluatePercentage DECIMAL(18, 2) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- ESTE SP REDUCE LAS INTERACCIONES DE UN PLAN AL PORCENTAJE DADO EN EL INPUT
	/*
	DECLARE @VCC VARCHAR(50) = 'occ_test'
	DECLARE @EvaluationPlanId VARCHAR(50)  = '50c7da87-c79d-4f05-b66d-4f94d76f8e55' 
	DECLARE @EvaluatePercentage DECIMAL(18, 2) = 0.20
	*/

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL NUMERO TOTAL DE INTERACCIONES
	DECLARE @CHANNEL_COUNTER INT = 0
	DECLARE @CURRENT_TOTAL_INTERACTIONS INT = 0
	DECLARE @NEW_TOTAL_INTERACTIONS DECIMAL(18, 2) = 0

	-- ALMACENAMOS EL TOTAL DE INTERACCIONES QUE EXISTEN ACTUALMENTE
	SELECT @CURRENT_TOTAL_INTERACTIONS = COUNT(DISTINCT IEP.[IdInteraction])
		FROM [InteractionEvaluationPlan] IEP (NOLOCK)
		WHERE IEP.[Instance] = @VCC
			AND IEP.[IdEvaluationPlan] = @EvaluationPlanId

	-- SETTEAMOS EL NUEVO TOTAL DE INTERACCIONES
	SET @NEW_TOTAL_INTERACTIONS =  CASE WHEN ((@CURRENT_TOTAL_INTERACTIONS * @EvaluatePercentage) % 1) >= 0.5 
									THEN CEILING(@CURRENT_TOTAL_INTERACTIONS * @EvaluatePercentage) 
									ELSE FLOOR(@CURRENT_TOTAL_INTERACTIONS * @EvaluatePercentage) 
									END

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS CANALES Y NUMERO DE INTERACCIONES 
	DECLARE MY_CURSOR CURSOR 
	LOCAL STATIC READ_ONLY FORWARD_ONLY -- SCROLL 
		FOR 
		-- CONSULTAMOS LA CANTIDAD DE INTERACCIONES QUE EXISTEN EN EL PLAN
		SELECT IEP.[InteractionType]													'CHANNEL',
				COUNT(DISTINCT IEP.[IdInteraction])										'CHANNEL_INTERACTIONS',
				CEILING(COUNT(DISTINCT IEP.[IdInteraction]) * @EvaluatePercentage)		'NEW_CHANNEL_INTERACTIONS'
			FROM [InteractionEvaluationPlan] IEP (NOLOCK)
			WHERE IEP.[Instance] = @VCC
				AND IEP.[IdEvaluationPlan] = @EvaluationPlanId
			GROUP BY IEP.[InteractionType]
			ORDER BY COUNT(DISTINCT IEP.[IdInteraction]) ASC

	-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL BACTH ACTUAL QUE ESTAMOS RECORRIENDO
	DECLARE @CURRENT_CHANNEL VARCHAR(50) = ''
	DECLARE @CURRENT_CHANNEL_INTERACTIONS INT = 0
	DECLARE @NEW_CHANNEL_INTERACTIONS INT = 0
	
	-- ABRIMOS Y RECORREMOS EL ARRAY DE CANALES
	OPEN MY_CURSOR
	FETCH NEXT FROM MY_CURSOR INTO @CURRENT_CHANNEL, @CURRENT_CHANNEL_INTERACTIONS, @NEW_CHANNEL_INTERACTIONS
	WHILE @@FETCH_STATUS = 0
	BEGIN 
		-- AUMENTAMOS EL CONTADOR DE CANALES
		SET @CHANNEL_COUNTER = @CHANNEL_COUNTER + 1

		-- VERIFICAMOS SI AUN QUEDAN CUPOS DISPONIBLES
		IF @NEW_TOTAL_INTERACTIONS > 0
		BEGIN
			-- VERIFICMAOS SI LA CANTIDAD PARA EL CANAL ES MAYOR A LA CANTIDAD DE CUPOS DISPONIBLES
			IF @NEW_TOTAL_INTERACTIONS <= @NEW_CHANNEL_INTERACTIONS 
			BEGIN
				-- SI LOS CUPOS DISPONIBLES SON MENORES A LA CANTIDAD PARA EL CANAL ACTUAL, LA CANTIDAD DISPONIBLE PASA A SER LA CANTIDAD PARA EL CANAL
				SET @NEW_CHANNEL_INTERACTIONS = @NEW_TOTAL_INTERACTIONS

			END
			ELSE
			BEGIN
				-- SI EL CONTADOR ES IGUAL A @@CURSOR_ROWS  ESTAMOS EN EL ULTIMO ITEM DEL CURSOR
				IF @CHANNEL_COUNTER = @@CURSOR_ROWS 
				BEGIN 
					-- ACTUALIZAMOS LA CANTIDAD PARA EL CANAL ACTUAL
					SET @NEW_CHANNEL_INTERACTIONS = @NEW_TOTAL_INTERACTIONS
					-- SELECT @CURRENT_CHANNEL, '=> AJUSTE =>', @NEW_CHANNEL_INTERACTIONS 
				END
			END
		END
		ELSE
		BEGIN
			-- SI NO QUEDAN CUPOS PENDIENTES, NO SE CONSERVARA NINGUN REGISTROS DEL CANAL
			SET @NEW_CHANNEL_INTERACTIONS = 0
		END
		
		---- ELIMINAMOS LAS INTERACCIONES QUE NO COINCIDEN CON LAS INTERACCIONES DE LA MUESTRA
		DELETE FROM [dbo].[InteractionEvaluationPlan]
			WHERE [Instance] = @VCC
				AND [IdEvaluationPlan] = @EvaluationPlanId
				AND [InteractionType] = @CURRENT_CHANNEL
				AND [IdInteraction] NOT IN (
					-- RETORNAMOS X INTERACCIONES ALEATORIAS POR ID DE INTERACCION
					SELECT TOP (@NEW_CHANNEL_INTERACTIONS) [IEP].[IdInteraction] 
						FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK) 
						WHERE IEP.[Instance] = @VCC
							AND IEP.[IdEvaluationPlan] = @EvaluationPlanId
							AND IEP.[InteractionType] = @CURRENT_CHANNEL
						GROUP BY [IEP].[IdInteraction] 
						ORDER BY NEWID()
				)
	
		-- FINALIZAMOS EL BUCLE RESTANDO A LOS CUPOS LA CANTIDAD USADA PARA EL CANAL
		SET @NEW_TOTAL_INTERACTIONS  = @NEW_TOTAL_INTERACTIONS - @NEW_CHANNEL_INTERACTIONS
		
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_CHANNEL, @CURRENT_CHANNEL_INTERACTIONS, @NEW_CHANNEL_INTERACTIONS
	END
	CLOSE MY_CURSOR
	DEALLOCATE MY_CURSOR

	-- RETORNAMOS UNA RESPUESTA
	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveInteractionType] (
	@Instance Varchar(50) = NULL, 				@ID Varchar(50) = NULL,				@Description Varchar(100) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

		INSERT INTO [InteractionType] (
			[Instance],[ID],
			[Description]
			)
		VALUES (
			@Instance,@ID,
			@Description
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveNeuroLinguisticProgramming] (
	@VCC Varchar(50) = NULL, 					@Name Varchar(50) = NULL,
	@Description Varchar(50) = NULL,			@CreatedDate Datetime = NULL,               @CreatedByUserId Varchar(50) = NULL,		
	@LastModifiedDate Datetime = NULL,			@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [NeuroLinguisticProgramming]
	SET [Description] = @Description,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
    AND [Name] = @Name

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [NeuroLinguisticProgramming] (
			[VCC], [Name], [Description],
			[CreatedDate], [CreatedByUserId], [LastModifiedDate], 
			[LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
			@VCC, @Name, @Description,
			@CreatedDate, @CreatedByUserId, @LastModifiedDate, 
			@LastModifiedByUserId, GETUTCDATE()
		)
	END
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveNLU] (
    @VCC Varchar(50) = NULL,
    @Name Varchar(50) = NULL,
    @Culture Varchar(50) = NULL,
    @Description Varchar(50) = NULL,
    @AppId Varchar(100) = NULL,
    @SubscriptionKey Varchar(50) = NULL,
    @IsSystemApp BIT = 0,
    @IsPublished BIT = NULL,
    @LastPublishedDate DATETIME = NULL,
    @IsTrained BIT = NULL,
    @NeedTrain BIT = NULL,
    @NeedPublish BIT = NULL,
    @LastTrainedDate DATETIME = NULL,
    @CreatedDate Datetime = NULL,
    @CreatedByUserId Varchar(50) = NULL,
    @LastModifiedDate Datetime = NULL,
    @LastModifiedByUserId Varchar(50) = NULL,
    @TrainVersion Varchar(250) = NULL,
    @TrainId Varchar(250) = NULL,
    @UseRasaApi Bit = NULL
)
AS
BEGIN
    SET NOCOUNT ON
	DECLARE @NeedReplicateNlu BIT = 0
	DECLARE @SystemAppId VARCHAR(50)
   

	-- VERIFICAMOS SI ES PARA UN NLU DE SENTIMIENTO PERSONALIZADO
	IF NOT EXISTS( SELECT 1 FROM [NLU] WHERE [Name] LIKE 'sentiment-%' AND AppId = @AppId AND VCC =@VCC)
	BEGIN
	SET @NeedReplicateNlu = 1
	SELECT @SystemAppId = AppId FROM [NLU] WHERE [VCC] = 'system' AND [Name] = @Name
	END
		
    -- SI LA VARIABLE LLEGA NULL, LA SETTEAMOS COMO 0
    IF @IsSystemApp IS NULL
        SET @IsSystemApp = 0

    UPDATE [NLU]
    SET [Culture] = @Culture,
        [Description] = @Description,
        [AppId] = @AppId,
        [IsPublished] = CASE WHEN @IsPublished IS NULL THEN [IsPublished] ELSE @IsPublished END,
        [LastPublishedDate] = CASE WHEN @LastPublishedDate IS NULL THEN [LastPublishedDate] ELSE @LastPublishedDate END,
        [IsSystemApp] = CASE WHEN @IsSystemApp IS NULL THEN [IsSystemApp] ELSE @IsSystemApp END,
        [SubscriptionKey] = CASE WHEN @SubscriptionKey IS NULL THEN [SubscriptionKey] ELSE @SubscriptionKey END,
        [IsTrained] = CASE WHEN @IsTrained IS NULL THEN [IsTrained] ELSE @IsTrained END,
        [NeedTrain] = CASE WHEN @NeedTrain IS NULL THEN [NeedTrain] ELSE @NeedTrain END,
        [NeedPublish] = CASE WHEN @NeedPublish IS NULL THEN [NeedPublish] ELSE @NeedPublish END,
        [LastTrainedDate] = CASE WHEN @LastTrainedDate IS NULL THEN [LastTrainedDate] ELSE @LastTrainedDate END,
        -- [CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
        [LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
        [LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
        [TimeStamp] = GETUTCDATE(),
        [TrainVersion] = @TrainVersion,
        [TrainId] = @TrainId
    WHERE [VCC] = @VCC
      AND [Name] = @Name

    IF @@ROWCOUNT = 0
        BEGIN
			
            INSERT INTO [NLU] (
                [VCC], [Name], [Culture],
                [Description], [AppId], [SubscriptionKey],
                [IsSystemApp], [CreatedDate], [CreatedByUserId],
                [LastModifiedDate], [LastModifiedByUserId], [TimeStamp],[CognitiveEngine]
            )
            VALUES (
                       @VCC, @Name, @Culture,
                       @Description, @AppId, @SubscriptionKey,
                       @IsSystemApp, GETUTCDATE(), @CreatedByUserId,
                       @LastModifiedDate, @LastModifiedByUserId, GETUTCDATE(),
                       IIF(@IsSystemApp = 1, IIF(@UseRasaApi =1,'RASAAPI','RASA'), 
                       IIF(@UseRasaApi=1,'RASAAPI',(SELECT [CognitiveEngine] FROM [dbo].[VCC] WHERE [VCC] = @VCC)))
                   )

			IF(@NeedReplicateNlu = 1)
			BEGIN
				--GENERAMOS INTENCIONES DEL NUEVO NLU
                UPDATE NLU 
                SET NeedTrain = 1 
                WHERE AppId = @AppId
                
				INSERT INTO [Intent]([VCC],[Name],[IntentId],[AppId],[Color],[CreatedDate],[CreatedByUserId],
							[LastModifiedDate],[LastModifiedByUserId],[TimeStamp],[Status],[StatusCode])
				SELECT @VCC,[Name],[IntentId],@AppId,[Color],GETUTCDATE(),'system',GETUTCDATE(),'system',GETUTCDATE(),[Status],[StatusCode] 
					FROM Intent 
					WHERE [AppId] = @SystemAppId
				
				INSERT INTO [Utterance]([VCC],[AppId],[IntentId],[Id],[Text],[CreatedDate],[CreatedByUserId],
							[LastModifiedDate],[LastModifiedByUserId],[Status],[StatusCode])
				SELECT @VCC,@AppId,[IntentId],NEWID(),[Text],GETUTCDATE(),'system',GETUTCDATE(),'system',[Status],[StatusCode] 
					FROM Utterance 
					WHERE [AppId] = @SystemAppId
			END
        END

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveProcess] (
	@VCC Varchar(50) = NULL,
	@Name Varchar(50) = NULL,
	@Description Varchar(50) = NULL,
	@ClientId Varchar(50) = NULL,
	@Type VARCHAR(50),
	@Status Varchar(50) = NULL,
	@StartDate Datetime = NULL,
	@EndDate Datetime = NULL,
	@MinDurationTime Numeric(18, 2) = NULL,
	@MaxDurationTime Numeric(18, 2) = NULL,
	@Language VARCHAR(25) = NULL,
	@DictionaryId VARCHAR(50) = NULL,
	@Environment Varchar(50) = NULL,
	@QueryId Varchar(50) = NULL,
	@QueryDataId Varchar(50) = NULL,
	@QueryDataIdDate Datetime = NULL,
	@BatchScheduleId VARCHAR(50) = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL,
	@Retranscription BIT = 0,
	@Diarization BIT = 0,
	@ChannelChange BIT = 0
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [Process]
	SET [Description] = @Description,
		[ClientId] = @ClientId,
		[Status] = @Status,
		[Type] = @Type,
		[StartDate] = @StartDate,
		[EndDate] = @EndDate,
		[MinDurationTime] = @MinDurationTime,
		[MaxDurationTime] = @MaxDurationTime,
		[Language] = @Language,
		[DictionaryId] = @DictionaryId,
		[Environment] = @Environment,
		[QueryId] = @QueryId,
		[QueryDataId] = @QueryDataId,
		[QueryDataIdDate] = @QueryDataIdDate,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
		
	WHERE [VCC] = @VCC
    AND [Name] = @Name

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [Process] (
			[VCC], [Name], [Description],
			[ClientId], [Status], [Type], [StartDate],
			[EndDate], [MinDurationTime], [MaxDurationTime], 
			[Language], [DictionaryId], [Environment], 
			[QueryId], [QueryDataId], [QueryDataIdDate],
			[BatchScheduleId], 
			[CreatedDate], [CreatedByUserId], [LastModifiedDate], 
			[LastModifiedByUserId], [TimeStamp], [Retranscription], [Diarization], [ChannelChange]
			)
		VALUES (
			@VCC, @Name, @Description,
			@ClientId, @Status, @Type, @StartDate,
			@EndDate, @MinDurationTime, @MaxDurationTime, 
			@Language, @DictionaryId, @Environment, 
			@QueryId, @QueryDataId, @QueryDataIdDate, 
			@BatchScheduleId,
			@CreatedDate, @CreatedByUserId, @LastModifiedDate, 
			@LastModifiedByUserId, GETUTCDATE(), @Retranscription, @Diarization, @ChannelChange
		)
	END

	-- TODO: QUE PASA SI UN AUDIO DEL PROCESS CON RETRANSCRIPCION YA SE ENCUENTRA EN UN PROCESS CORRIENDO
	--En caso de retranscripcion regresar todos los audios a su estado inicial 
	IF @Retranscription = 1
	BEGIN
		UPDATE A
			SET A.[IsProcess] = NULL,
				A.[IsCancel] = NULL,
				A.[IsTranscribed] = NULL,
				A.[init] = NULL,
				A.[IsDiariced] = NULL,
				A.[ChannelsChanged] = NULL
			FROM Audio A inner join ProcessAudio PA on A.id = PA.AudioId
			WHERE PA.[VCC] = @VCC 
				AND PA.[ProcessId] = @Name 


		UPDATE AI
			SET AI.[IsProcess] = NULL,
				AI.[IsCancel] = NULL,
				AI.[IsTranscribed] = NULL
			FROM [dbo].[AsyncInteraction] AI 
			INNER JOIN [dbo].[ProcessAsyncInteraction] PAI 
				ON AI.[id] = PAI.[InteractionId]
			WHERE PAI.[VCC] = @VCC 
				AND PAI.[ProcessId] = @Name
	END

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveProcessFilter] (
	@VCC Varchar(50) = NULL, 				@ProcessId Varchar(50) = NULL,				@Key Varchar(50) = NULL,
	@Value Varchar(250) = NULL,				@CreatedDate Datetime = NULL,				@CreatedByUserId Varchar(50) = NULL,		
	@LastModifiedDate Datetime = NULL,		@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [ProcessFilter]
	SET [Key] = @Key,
		[Value] = @Value,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
    AND [ProcessId] = @ProcessId
	AND [Key] = @Key
	AND [Value] = @Value

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [ProcessFilter] (
			[VCC], [ProcessId], [Key],
			[Value], [CreatedDate], 
			[CreatedByUserId], [LastModifiedDate], 
			[LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
			@VCC, @ProcessId, @Key,
			@Value, @CreatedDate,
			@CreatedByUserId, @LastModifiedDate, 
			@LastModifiedByUserId, GETUTCDATE()
		)
	END

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[SaveProcessStatus] 'iconcert', 'callyou_colombia', 'COMPLETE', '2019-03-15 00:00:00.000', 'analytic'
CREATE PROCEDURE [dbo].[SaveProcessStatus]
(
	@VCC						VARCHAR(50),
	@Name						VARCHAR(50),
	@Status						VARCHAR(50),
	@LastModifiedDate			DATETIME,
	@LastModifiedByUserId		VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [dbo].[Process] SET
	[Status] = @Status,
	[LastModifiedDate] = @LastModifiedDate,
	[LastModifiedByUserId] = @LastModifiedByUserId,
	[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC AND
	[Name] = @Name

	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveProcessSummary] (
	@VCC Varchar(50) = NULL, 					@ProcessId Varchar(50) = NULL,				@SilenceTime Varchar(50) = NULL,
	@AverageSilenceTime Varchar(50) = NULL,		@DurationTime Varchar(50) = NULL,			@AverageDurationTime Varchar(50) = NULL,
	@DurationTimeRate Varchar(50) = NULL,		@CreatedDate Datetime = NULL,				@CreatedByUserId Varchar(50) = NULL,		
	@LastModifiedDate Datetime = NULL,			@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [ProcessSummary]
	SET [SilenceTime] = @SilenceTime,
		[AverageSilenceTime] = @AverageSilenceTime,
		[DurationTime] = @DurationTime,
		[AverageDurationTime] = @AverageDurationTime,
		[DurationTimeRate] = @DurationTimeRate,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
    AND [ProcessId] = @ProcessId

    IF @@ROWCOUNT = 0
		INSERT INTO [ProcessSummary] (
			[VCC], [ProcessId], [SilenceTime], [AverageSilenceTime], [DurationTime], [AverageDurationTime],
			[DurationTimeRate], [CreatedDate], [CreatedByUserId],
			[LastModifiedDate], [LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
			@VCC, @ProcessId, @SilenceTime, @AverageSilenceTime, @DurationTime, @AverageDurationTime,
			@DurationTimeRate, @CreatedDate, @CreatedByUserId, 
			@LastModifiedDate, @LastModifiedByUserId, GETUTCDATE()
		)
		
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveRealtimeEvaluationPlan] (
	@VCC VARCHAR(50) = NULL,
	@ID VARCHAR(50) = NULL,
	@CampaignId VARCHAR(100) = NULL,
	@Description VARCHAR(200) = NULL,
	@Modality VARCHAR(50) = NULL,
	@EvaluationFormatId VARCHAR(50) = NULL,
	@Enabled BIT = NULL,
	@StatusDescription VARCHAR(25) = 'CREATED',
	@Status	NUMERIC(2,0) = 1,
	@EvaluationMode VARCHAR(10) = NULL,
	@CurrentUser VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- OBTENEMOS EL NAME DEL AMBIENTE POR DEFAULT
	DECLARE @ENVIRONMENT_ID VARCHAR(50) = NULL
	SELECT @ENVIRONMENT_ID = WBL.[Word] 
		FROM [dbo].[WordsByLanguage] WBL (NOLOCK) 
		INNER JOIN [dbo].[VCC] V (NOLOCK)
			ON V.[VCC] = @VCC
		WHERE WBL.[Language] = V.[Language]
			AND WBL.[Label] = 'ENVIRONMENT' 
	
	-- CREAMOS EL REGISTRO PARA LA TABLA RealtimeEvaluationPlan
	INSERT INTO [dbo].[RealtimeEvaluationPlan] (
		[VCC], [ID], [EvaluationFormatId], [CampaignId], [IsActive], 
		[CreatedDate], [CreatedByUserId], [TimeStamp]
	)
	VALUES (
		@VCC, @ID, @EvaluationFormatId, @CampaignId, 1, 
		GETUTCDATE(), @CurrentUser, GETUTCDATE() 
	)
	
	-- CREAMOS EL REGISTRO PARA LA TABLA EvaluationPlan
	INSERT INTO [dbo].[EvaluationPlan] (
		[Instance], [ID], [Description], [Modality], [DateFin], 
		[IdEvaluationFormat], [EnvironmentId], [Enabled], [StatusDescription], [Status], 
		[Evaluation_Mode], [CreatedDate], [CreatedByUserId], [TimeStamp]
	)
	VALUES (
		@VCC, @ID, @Description, @Modality, '2050-01-01 00:00:00', 
		@EvaluationFormatId, @ENVIRONMENT_ID, @Enabled, @StatusDescription, 1,
		@EvaluationMode, GETUTCDATE(), @CurrentUser, GETUTCDATE() 
	)

	-- INSERTAMOS LA DATA HISTORICA
	EXEC [dbo].[SaveHistoricEvaluationPlanData] @VCC, @ID, @EvaluationFormatId

	SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveReport](
	@VCC Varchar(50) = NULL,				@ID Varchar(50) = NULL,					@Description Varchar(300) = NULL,
	@Procedure Varchar(50) = NULL,				@StartDate Datetime = NULL,				@EndDate Datetime = NULL,
	@Frecuency Varchar(50) = NULL,				@HourSpace Integer = NULL,				@ExactTime Varchar(50) = NULL,
	@Weekday Varchar(50) = NULL,				@DayMonth Varchar(50) = NULL,			@Summary Varchar(300) = NULL,
	@Filter Varchar(1000) = NULL,				@Email Varchar(1000) = NULL,			@CreatedDate Datetime = NULL,			
	@CreatedByUserId Varchar(50) = NULL,		@LastModifiedDate Datetime = NULL, 		@LastModifiedByUserId Varchar(50) = NULL,
	@Type Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [Report]
	SET [Description]	= @Description,
		[Procedure]		= @Procedure,
		[StartDate]		= @StartDate,
		[EndDate]		= @EndDate,
		[Frecuency]		= @Frecuency,
		[HourSpace]		= @HourSpace,
		[ExactTime]		= @ExactTime,
		[Weekday]		= @Weekday,
		[DayMonth]		= @DayMonth,
		[Summary]		= @Summary,
		[Filter]		= @Filter,
		[Email]			= @Email,
		[Type]			= @Type,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
	AND [ID] = @ID
    
    IF @@ROWCOUNT = 0
		INSERT INTO [Report] (
			[VCC], [ID], [Description],[Type],
            [Procedure], [StartDate],[EndDate],
            [Frecuency],[HourSpace],[ExactTime],
            [Weekday],[DayMonth],[Summary],
            [Filter], [Email], [CreatedDate],
            [CreatedByUserId],[LastModifiedDate],[LastModifiedByUserId],
            [TimeStamp]
        )
		VALUES (
			@VCC, @ID, @Description,@Type,
            @Procedure, @StartDate,@EndDate,
            @Frecuency, @HourSpace, @ExactTime,
            @Weekday, @DayMonth, @Summary,
            @Filter, @Email, @CreatedDate,
            @CreatedByUserId, @LastModifiedDate, @LastModifiedByUserId,
            GETUTCDATE()
		)
		
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveRole] (
	@VCC Varchar(50) = NULL, 			@ID Varchar(50) = NULL, 				@Description Varchar(50) = NULL, @SystemRole Bit = NULL, @Type Varchar(50)=NULL,
	@CreatedDate Datetime = NULL,			@CreatedByUserId Varchar(50) = NULL,	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [Role]
	SET [Description] = @Description,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
	AND [ID] = @ID
    
    IF @@ROWCOUNT = 0
		INSERT INTO [Role] (
			[VCC], [ID], [Description],[SystemRole],[Type],
			[CreatedDate], [CreatedByUserId], [LastModifiedDate],
			[LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
			@VCC, @ID, @Description,@SystemRole, @Type,
			@CreatedDate, @CreatedByUserId, @LastModifiedDate,
			@LastModifiedByUserId, GETUTCDATE()
		)
		
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveRolePermission] (
	@VCC Varchar(50) = NULL, @RoleId Varchar(50) = NULL, @GroupId Varchar(200) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [RolePermission] SET
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
	AND [RoleId] = @RoleId
	AND [GroupId] = @GroupId
	
    IF @@ROWCOUNT = 0
		INSERT INTO [RolePermission] (
			[VCC], [RoleId], [GroupId],
			[TimeStamp]
			)
		VALUES (
			@VCC, @RoleId, @GroupId,
			GETUTCDATE()	
		)

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveRolePermissionsGroup] (
	@VCC Varchar(50) = NULL, @GroupId Varchar(200) = NULL, @Category Varchar(50) = NULL, @Type Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [RolePermissionsGroup] SET
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
	AND [GroupId] = @GroupId
	
    IF @@ROWCOUNT = 0
		INSERT INTO [RolePermissionsGroup] (
			[VCC], [GroupId], [Category], [Type], [TimeStamp]
			)
		VALUES (
			@VCC, @GroupId, @Category, @Type, GETUTCDATE()	
		)

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveRoutePermission] (
	@VCC Varchar(50) = NULL, @ID Varchar(50) = NULL, @GroupId Varchar(200) = NULL, 
	@Route Varchar(250) = NULL,	@Level Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [RoutePermission] SET
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
	AND [ID] = @ID
	AND [GroupId] = @GroupId
	
    IF @@ROWCOUNT = 0
		INSERT INTO [RoutePermission] (
			[VCC], [ID], [GroupId],
			[Route], [Level], [TimeStamp]
			)
		VALUES (
			@VCC, @ID, @GroupId,
			@Route, @Level, GETUTCDATE()	
		)

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveScore] (
	@Instance Varchar(50) = NULL, 				@ID Varchar(50) = NULL,						@IdAbility Varchar(50) = NULL,		
	@Description Varchar(100) = NULL,						@Value Numeric(5,1) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	INSERT INTO [Score] (
		[Instance], [IdAbility], [Description], [Value],
		[TimeStamp]
		)
	VALUES (
		@Instance, @IdAbility, @Description, @Value,
		GETUTCDATE()
	)

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveScoreTipification] (
	@VCC VARCHAR(50) = NULL, 				
	@ScoreId VARCHAR(50) = NULL,
	@ID VARCHAR(50) = NULL,
	@Description VARCHAR(250) = NULL,					
	@Detail VARCHAR(500) = NULL,
	@Order INT = NULL,
	@UserId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EN CASO EXISTA
	UPDATE [ScoreTipification]
		SET [Description] = @Description,
			[Detail] = @Detail,
			[Order] = @Order,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedByUserId] = @UserId
		WHERE [VCC] = @VCC
			AND [ScoreId] = @ScoreId
			AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [ScoreTipification] (
			[VCC], 
			[ScoreId],
			[ID],
			[Description], 
			[Detail],
			[Order],
			[CreatedDate],
			[CreatedByUserId]
			)
		VALUES (
			@VCC,
			@ScoreId,
			@ID,
			@Description, 
			@Detail,
			@Order,
			GETUTCDATE(),
			@UserId
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveScoreTipificationItem] (
	@VCC VARCHAR(50) = NULL, 	
	@ScoreId VARCHAR(50) = NULL,
	@ID VARCHAR(50) = NULL,
	@TipificationId VARCHAR(50) = NULL,
	@SecondLevelItemId VARCHAR(50) = NULL,
	@Description VARCHAR(250) = NULL,					
	@Order INT = NULL,
	@UserId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EN CASO EXISTA
	UPDATE [ScoreTipificationItem]
		SET [Description] = @Description,
			[Order] = @Order,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedByUserId] = @UserId
		WHERE [VCC] = @VCC
			AND [ScoreId] = @ScoreId
			AND [TipificationId] = @TipificationId
			AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [ScoreTipificationItem] (
			[VCC], 
			[ScoreId],
			[TipificationId], 
			[ID],
			[SecondLevelItemId],
			[Description], 
			[Order], 
			[CreatedDate],
			[CreatedByUserId]
			)
		VALUES (
			@VCC, 
			@ScoreId,
			@TipificationId,
			@ID,
			@SecondLevelItemId,
			@Description, 
			@Order,
			GETUTCDATE(),
			@UserId
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveScoreTipificationQuery] (
	@VCC							VARCHAR(50) = NULL,
	@ScoreId							VARCHAR(50) = NULL,
	@TipificationId					VARCHAR(50) = NULL,
	@QueryId						VARCHAR(50) = NULL,
	@QueryContent					VARCHAR(1000) = NULL,
	@AcceptancePercentage			DECIMAL(18, 2) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EL REGISTRO EN CASO YA EXISTA
	UPDATE [ScoreTipificationQuery]
		SET [QueryId] = @QueryId,
			[QueryContent] = @QueryContent,
			[AcceptancePercentage] = @AcceptancePercentage,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [ScoreId] = @ScoreId
			AND [TipificationId] = @TipificationId

	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [ScoreTipificationQuery] (
			[VCC], [ScoreId], [TipificationId], [QueryId], 
			[QueryContent], [AcceptancePercentage], [TimeStamp]
		)
		VALUES (
			@VCC, @ScoreId, @TipificationId, @QueryId, 
			@QueryContent, @AcceptancePercentage, GETUTCDATE()
		)
	END

	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveSentimentAnalysisResult] (
	@VCC varchar(50),
	@InteractionId varchar(250),
	@InteractionSection numeric(5,0),
	@InteractionPart numeric(5,0),
	@InteractionType varchar(50),
	@InteractionInterval numeric(5,0),
	@AnalysisType varchar(50),
	@SentimentId varchar(50),
	@AppId varchar(50),
	@Score numeric(10,5),
	@CreatedDate datetime,
	@CreatedByUserId varchar(50),
	@LastModifiedDate datetime,
	@LastModifiedByUserId varchar(50),
	@EndDate varchar(50),
    @Status varchar(50)
	)
AS
BEGIN
	SET NOCOUNT ON
	UPDATE [SentimentAnalysisResult]
	SET [VCC] = @VCC,
		[InteractionId] = @InteractionId,
		[InteractionSection] = @InteractionSection,
		[InteractionPart] = @InteractionPart,
		[InteractionType] = @InteractionType,
		[InteractionInterval] = @InteractionInterval,
		[AnalysisType] = @AnalysisType,
		[SentimentId] = @SentimentId,
		[AppId] = @AppId,
		[Score] = @Score,		
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = GETUTCDATE(),
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE(),
		[EndDate] = CASE WHEN @InteractionType = 'CALL' THEN '' ELSE  @EndDate END, 
        [MaxScore] = CASE WHEN @Status = 'Normal' THEN 0 ELSE 1 END
		
	WHERE [VCC] = @VCC
    AND [InteractionId] = @InteractionId
	AND [InteractionSection] = @InteractionSection
	AND	[InteractionPart] = @InteractionPart
	AND [InteractionType] = @InteractionType
	AND [InteractionInterval] = @InteractionInterval
	AND [AnalysisType] = @AnalysisType
	AND [SentimentId] = @SentimentId
	AND [AppId] = @AppId
	AND [EndDate] = @EndDate
	

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [SentimentAnalysisResult] (
			[VCC], 
			[InteractionId],
			[InteractionSection],
			[InteractionPart],
			[InteractionType],
			[InteractionInterval],
			[AnalysisType],
			[SentimentId],
			[AppId],
			[Score],
			[CreatedDate],
			[CreatedByUserId],
			[LastModifiedDate],
			[LastModifiedByUserId],
			[TimeStamp],
			[EndDate],
            [MaxScore]

			)
		VALUES (
			@VCC,
			@InteractionId,
			@InteractionSection,
			@InteractionPart,
			@InteractionType,
			@InteractionInterval,
			@AnalysisType,
			@SentimentId,
			@AppId,
			@Score,
			GETUTCDATE(),
			@CreatedByUserId,
			GETUTCDATE(),
			@LastModifiedByUserId,
			GETUTCDATE(),
			CASE WHEN @InteractionType = 'CALL' THEN '' ELSE  @EndDate END,
            CASE WHEN @Status = 'Normal' THEN 0 ELSE 1 END
		)
	END
	
	

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
ENDGO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveSinglePermission] (
	@VCC Varchar(50) = NULL, @Type Varchar(50) = NULL, @Element Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	BEGIN TRY

        DECLARE @Result Int = 1

        UPDATE [SinglePermission] SET
            [TimeStamp] = GETUTCDATE()
        WHERE [VCC] = @VCC
        AND [GroupId] = 'SINGLE_PERM_' + @Type + '_' + @Element
        AND [Type] = @Type
        AND [Element] = @Element

        IF @@ROWCOUNT = 0
            INSERT INTO [SinglePermission] (
                [VCC], [ID], [GroupId],
                [Type], [Element],
                [TimeStamp]
            )
            SELECT TOP 1 
                @VCC, CONVERT(VARCHAR(50), CAST([ID] AS Int) + 1), 'SINGLE_PERM_' + @Type + '_' + @Element,
                @Type, @Element,
                GETUTCDATE()
            FROM [SinglePermission] (NOLOCK)
            ORDER BY CAST([ID] AS Int) DESC

        UPDATE [RolePermissionsGroup] SET
            [TimeStamp] = GETUTCDATE()
        WHERE [VCC] = @VCC
        AND [GroupId] = 'SINGLE_PERM_' + @Type + '_' + @Element
        AND [Category] = 'CATEG_GROUP_' + @Type

        IF @@ROWCOUNT = 0
            INSERT INTO [RolePermissionsGroup] (
                [VCC], [GroupId], [Category],
                [TimeStamp]
            )
            VALUES (
                @VCC, 'SINGLE_PERM_' + @Type + '_' + @Element, 'CATEG_GROUP_' + @Type,
                GETUTCDATE()
            )

	END TRY
	BEGIN CATCH

        SET @Result = 0

	END CATCH

    SELECT @Result AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveSpeechAnalyticAuditorUser] (
	@VCC Varchar(50) = NULL
) AS
BEGIN
    SET NOCOUNT ON

	-- DECLARAMOS UN FLAG PARA INDICAR SI EXISTE EL USUARIO SPEECH
	DECLARE @SPEECH_EXISTS BIT = 0
	
	-- VERIFICAMOS SI EL USUARIO SPEECH EXISTE EN EL VCC INGRESADO
	IF EXISTS (
		SELECT 1
			FROM [dbo].[User] U (NOLOCK)
			WHERE U.[VCC] = @VCC
				AND U.[ID] = 'speech'
	)
		SET @SPEECH_EXISTS = 1

	-- VALIDAMOS EL VALOR DEL FLAG
	IF @SPEECH_EXISTS = 0
	BEGIN
		-- INSERTAMOS EL USUARIO SPEECH EN EL VCC
		INSERT [dbo].[User] (
			[VCC], [ID], [Email], [Firstname], [Lastname], 
			[Password], [Role], [Country], [Language], [TimeZone], 
			[Preferences], [IsSystemUser], [LimitClients], [CreatedDate], [CreatedByUserId], 
			[LastModifiedDate], [LastModifiedByUserId], [TimeStamp]
		) VALUES (
			@VCC, N'speech', N'speech@inconcertcc.com', N'SPEECH', N'ANALYTIC', 
			N'289128e9a91391c0198705308a88ea5c', N'quality_auditor', N'PE', N'es', N'America/Mexico_City', N'{}', 
			1, 1, CAST(N'2016-08-09T17:36:45.647' AS DateTime), N'admin', GETUTCDATE(), 
			N'admin', GETUTCDATE()
		)
	END

	-- RETORNAMOS UN VALOR PARA EL RESULTADO
	SELECT 1 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveSQLStudyDetail]
(
	@VCC					VARCHAR(50),
	@StudyId				VARCHAR(200),
	@TemplateId				VARCHAR(50),
	@SQL					VARCHAR(MAX),
	@Restart				INT

)
AS
BEGIN
	SET NOCOUNT ON

	-- Si llega el parametro que indica que debe reprocesar estudio.
	IF(@Restart = 1)
	BEGIN
		-- Restablecer tablas:
		-- Study
		-- StudyDetail
		-- StudyResultSummary
		-- StudyResultSummaryDetail
		BEGIN
			UPDATE [dbo].[Study] SET
			[Status] = NULL
			WHERE VCC = @VCC
			AND [Name] = @StudyId
		END

		BEGIN
			DELETE FROM [dbo].[StudyDetail]
			WHERE VCC = @VCC
			AND StudyId = @StudyId
		END

		BEGIN
			DELETE FROM [dbo].[StudyResultSummary]
			WHERE VCC = @VCC
			AND StudyId = @StudyId
		END

		BEGIN
			DELETE FROM [dbo].[StudyResultSummaryDetail]
			WHERE VCC = @VCC
			AND StudyId = @StudyId
		END
	END

	-- CONTINUAMOS LA EJECUCION NORMAL
	BEGIN
		EXEC(@SQL)
	END
	BEGIN
	    -- AGREGAMOS [IsMatched] = 0 PARA INTENTS NEGATIVOS
		INSERT INTO [dbo].[StudyResultSummary] (
			[VCC], 
			[StudyId], 
			[TemplateId], 
			[TemplateThemeId], 
			[ID], 
			[Description], 
			[Category1], 
			[Category2], 
			[Total], 
			[Duration], 
			[Silence], 
			[Confidence], 
			[CreatedDate], 
			[CreatedByUserId], 
			[LastModifiedDate], 
			[LastModifiedByUserId],
			[IsMatched], 
			[TimeStamp]
		)
			SELECT B.[VCC], 
					B.[StudyId], 
					A.[TemplateId], 
					A.[ID], 
					CONVERT(VARCHAR(50), NEWID()), 
					A.[Description], 
					A.[Category1], 
					A.[Category2], 
					COUNT(B.[Total]), 
					SUM(B.[Duration]), 
					SUM(B.[Silence]), 
					AVG(B.[Confidence]), 
					A.[CreatedDate], 
					A.[CreatedByUserId], 
					A.[LastModifiedDate], 
					A.[LastModifiedByUserId],
					0, 
					GETUTCDATE()
		FROM [dbo].[TemplateTheme] A (NOLOCK)
		INNER JOIN [dbo].[StudyDetail] B (NOLOCK) 
			ON A.ID = B.TemplateThemeId
		INNER JOIN [dbo].[Study] S (NOLOCK) 
			ON S.[VCC] = B.[VCC] 
				AND S.[Name] = B.[StudyId]
		LEFT JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
			ON S.[VCC] = PA.[VCC] 
				AND S.[ProcessId] = PA.[ProcessId] 
				AND B.[TranscriptionId] = CONCAT(PA.[AudioId], '_', PA.[Section], '_', PA.[Part])
		WHERE
			A.VCC IN ('system', @VCC)
			AND B.StudyId = @StudyId
			AND A.TemplateId = @TemplateId
		GROUP BY B.[VCC], 
					B.[StudyId], 
					A.[TemplateId], 
					A.[ID], 
					A.[Description], 
					A.[Category1], 
					A.[Category2], 
					A.[CreatedDate], 
					A.[CreatedByUserId], 
					A.[LastModifiedDate], 
					A.[LastModifiedByUserId], 
					A.[TimeStamp]
	END
	BEGIN
		UPDATE [dbo].[Study] 
			SET [Status] = 1
			WHERE VCC = @VCC
				AND [Name] = @StudyId
	END
	
	-- RETORNAMOS EL RESULTADO
	SELECT 1 AS 'result'

	SET NOCOUNT OFF

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveStudy] (
	@VCC Varchar(50) = NULL,
	@ProcessId Varchar(50) = NULL,
	@TemplateId Varchar(50) = NULL,
	@Name Varchar(200) = NULL,
	@Description Varchar(50) = NULL,
	@Status Float = NULL,
	@Type VARCHAR(50) = NULL,
	@BatchScheduleId VARCHAR(50) = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [Study]
		SET [Name] = @Name,
			[Description] = @Description,
			[Status] = @Status,
			[Type] = @Type,
			[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
			[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
			[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [ProcessId] = @ProcessId
			AND [TemplateId] =@TemplateId
			AND [Name] = @Name

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [Study] (
			[VCC], [ProcessId], [TemplateId], 
			[Name], [Description], [Status],
			[Type],	[BatchScheduleId], [CreatedDate], [CreatedByUserId], 
			[LastModifiedDate], [LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
			@VCC, @ProcessId, @TemplateId,
			@Name, @Description, @Status,
			@Type, @BatchScheduleId, @CreatedDate, @CreatedByUserId, 
			@LastModifiedDate, @LastModifiedByUserId, GETUTCDATE()
		)
		EXEC [dbo].[SaveHistoricStudyData] @VCC,@Name,@TemplateId
	END
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveStudyBaseResult] (
	@VCC VARCHAR(50) = NULL,
	@STUDY_ID VARCHAR(100) = NULL
) AS
BEGIN
	-- ESTE SP INSERTA LOS RESULT BASE DE UN ESTUDIO
	SET NOCOUNT ON
	--DECLARE 	@VCC VARCHAR(50) = 'incollege'
	--DECLARE @STUDY_ID VARCHAR(100) = 'positive-study-2'


/************************ SECTOR PARA INSERTAR EN StudyResultSummaryDetail INTERACCIONES DE TIPO CALL **************/
	;WITH CTE_CALL AS (
		SELECT S.[TemplateId],
				TTI.[TemplateThemeId]		'TemplateThemeId',
				TTI.[ID]					'TemplateThemeItemId',
              CONCAT(
				PA.[AudioId],
				'_',
				ISNULL(PA.[Section], '0'),
				'_',
				ISNULL(PA.[Part], '0'))		'TranscriptionId'
			FROM [dbo].[Study] S (NOLOCK) 
			JOIN [dbo].[TemplateThemeItem] TTI (NOLOCK) 
				ON (
					S.[VCC] = TTI.[VCC] 
					AND S.[TemplateId] = TTI.[TemplateId]
				)
			JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
				ON (
					PA.[VCC] = S.[VCC] 
					AND PA.[ProcessId] = S.[ProcessId]
				)
			JOIN [dbo].[Audio] A (NOLOCK) 
				 ON (
					A.[VCC] = S.[VCC] 
					AND A.[ID] = PA.[AudioId]
				)
			LEFT JOIN [dbo].[StudyResultSummaryDetail] SRSD (NOLOCK)
				ON (
					S.[VCC] = SRSD.[VCC]
					AND S.[Name] = SRSD.[StudyId]
					AND TTI.[TemplateThemeId] = SRSD.[TemplateThemeId]
					AND TTI.[ID] = SRSD.[TemplateThemeItemId]
					AND CONCAT(PA.[AudioId], '_', ISNULL(PA.[Section], '0'), '_', ISNULL(PA.[Part], '0')) = SRSD.TranscriptionId
				)
			WHERE S.[VCC] = @VCC 
				AND S.[Name] = @STUDY_ID 
				AND A.[Channel] ='CALL' 
				AND SRSD.TemplateThemeId IS NULL
			GROUP BY S.[TemplateId],
						TTI.[TemplateThemeId],
						TTI.[ID],
						PA.[AudioId],
						PA.[Section],
						PA.[Part]
    )

        
	INSERT INTO [dbo].[StudyResultSummaryDetail] (
        [VCC],
        [StudyId],
        [TemplateThemeId],
        [TemplateThemeItemId],
        [TranscriptionId],
        [CreatedDate],
        [CreatedByUserId],
        [TimeStamp],
        [IsMatched]
    )
        SELECT @VCC,
				@STUDY_ID,
				[TemplateThemeId],
				[TemplateThemeItemId],
				[TranscriptionId], 
				GETUTCDATE(),
				'system',
				GETUTCDATE(),
				0 --0 POR DEFECTO PARA ACTUALIZAR MAS ADELANTE A 1 EN EL BULK LOS QUE HICIERON MATCH
			FROM CTE_CALL

/************************ SECTOR PARA INSERTAR StudyResultSummaryDetail INTERACCIONES ASINCRONAS ******************/

	;WITH CTE_ASYNC AS (
		SELECT S.[TemplateId],
				TTI.[TemplateThemeId]			'TemplateThemeId',
				TTI.[ID]						'TemplateThemeItemId',
				CONCAT(
					PAI.[InteractionType], 
					'_', 
					PAI.[Campaign], 
					'_', 
					PAI.[FormattedEndDate], 
					'_', 
					REPLACE(PAI.[InteractionId], ':', '_')
				)								'TranscriptionId'
			FROM  [dbo].[Study] S (NOLOCK) 
			JOIN [dbo].[TemplateThemeItem] TTI (NOLOCK) 
				ON (
					S.[VCC] = TTI.[VCC] 
					AND S.[TemplateId] = TTI.[TemplateId]
				)
			JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK) 
				ON (
					PAI.[VCC] = S.[VCC] 
					AND PAI.[ProcessId] = S.[ProcessId]
					)
			LEFT JOIN [dbo].[StudyResultSummaryDetail] SRSD (NOLOCK)
				ON (
					S.[VCC] = SRSD.[VCC] 
					AND S.[Name] = SRSD.[StudyId] 
					AND TTI.[TemplateThemeId] = SRSD.[TemplateThemeId] 
					AND TTI.[ID] = SRSD.[TemplateThemeItemId] 
					AND CONCAT(PAI.[InteractionType], '_', PAI.[Campaign], '_', PAI.[FormattedEndDate], '_', REPLACE(PAI.[InteractionId], ':', '_')) = SRSD.TranscriptionId
				)
			WHERE S.[VCC] = @VCC 
				AND S.[Name] = @STUDY_ID 
				AND SRSD.TemplateThemeId IS NULL
			GROUP BY S.[TemplateId],
						TTI.[TemplateThemeId],
						TTI.[ID],
						PAI.[InteractionType],
						PAI.[Campaign],
						PAI.[FormattedEndDate],
						PAI.[InteractionId]
	)

    INSERT INTO [dbo].[StudyResultSummaryDetail] (
		[VCC],
		[StudyId],
		[TemplateThemeId],
		[TemplateThemeItemId],
		[TranscriptionId],
		[CreatedDate],
		[CreatedByUserId],
		[TimeStamp],
		[IsMatched]
    )
		SELECT @VCC,
				@STUDY_ID,
				[TemplateThemeId],
				[TemplateThemeItemId],
				[TranscriptionId], 
				GETUTCDATE(),
				'system',
				GETUTCDATE(),
				0 --0 POR DEFECTO PARA ACTUALIZAR MAS ADELANTE A 1 EN EL BULK LOS QUE HICIERON MATCH
			FROM CTE_ASYNC

/************************ SECTOR PARA INSERTAR StudyResultSummary INTERACCIONES DEL CANAL CALL **************/
	;WITH CTE_SRS AS (
		 SELECT S.[TemplateId],
				  TT.[ID]				'TemplateThemeId',
				  TT.[Description],
				  TT.[Category1],
				  TT.[Category2]
			FROM [dbo].[Study] S (NOLOCK)
			JOIN [dbo].[TemplateTheme] TT (NOLOCK) 
				ON (
					S.[VCC] = TT.[VCC] 
					AND S.[TemplateId] = TT.[TemplateId]
				)
			JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
				ON (
					PA.[VCC] = S.[VCC] 
					AND PA.[ProcessId] = S.[ProcessId]
				   )
			JOIN [dbo].[Audio] A (NOLOCK) 
				ON (
					A.[VCC] = S.[VCC] 
					AND A.[ID] = PA.[AudioId]
				  )
			LEFT JOIN [dbo].[StudyResultSummary] SRS (NOLOCK)
				ON (
					SRS.[VCC] = S.[VCC] 
					AND SRS.[StudyId] = S.[Name] 
					AND SRS.[TemplateThemeId] = TT.[ID]
				)
			WHERE S.[VCC] = @VCC 
				AND S.[Name] = @STUDY_ID 
				AND A.[Channel] ='CALL' 
				AND SRS.[TemplateThemeId] IS NULL
			GROUP BY S.[TemplateId],
						TT.[ID],
						TT.[Description],
						TT.[Category1], 
						TT.[Category2]
	)

    INSERT INTO [dbo].[StudyResultSummary](
		[VCC],
		[StudyId],
		[TemplateId],
		[TemplateThemeId],
		[ID],
		[Description],
		[Category1],
		[Category2],
		[Total],
		[Duration],
		[Silence],
		[OtherDuration],
		[OtherSilence],
		[Confidence],
		[CreatedDate],
		[CreatedByUserId],
		[LastModifiedDate],
		[LastModifiedByUserId],
		[TimeStamp],
		[IsMatched]
    )
		SELECT @VCC,@STUDY_ID,
				[TemplateId],
				[TemplateThemeId],
				CONVERT(VARCHAR(50), NEWID()),
				[Description],
				[Category1],
				[Category2],
				1,
				NULL,--Duration
				NULL,--Silence
				NULL,--OtherDuration
				NULL,--OtherSilence,
				NULL,--Confidence
				GETUTCDATE(),
				'system',
				GETUTCDATE(),
				'system',
				GETUTCDATE(),
				0
			FROM CTE_SRS


 /******************  SECTOR PARA INSERTAR StudyResultSummary PARA INTERACCIONES  ASINCRONAS   ****************************/


	;WITH CTE_SRS_ASYNC AS (
		SELECT S.[TemplateId],
				TT.[ID] 'TemplateThemeId',
				TT.[Description],
				TT.[Category1],
				TT.[Category2]
			FROM [dbo].[Study] S (NOLOCK)
			JOIN [dbo].[TemplateTheme]  TT (NOLOCK) 
				ON (
					S.[VCC] = TT.[VCC] AND
					S.[TemplateId] = TT.[TemplateId]
				)
			JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK) 
				ON (
					PAI.[VCC] = S.[VCC] AND 
					PAI.[ProcessId] = S.[ProcessId]
				)
			LEFT JOIN [dbo].[StudyResultSummary] SRS (NOLOCK)
				ON (
					SRS.[VCC] = S.[VCC] AND 
					SRS.[StudyId] = S.[Name] AND 
					SRS.[TemplateThemeId] = TT.[ID]
				)
			WHERE S.[VCC] = @VCC 
				AND S.[Name] = @STUDY_ID 
				AND SRS.[TemplateThemeId] IS NULL
			GROUP BY S.[TemplateId],
						TT.[ID],
						TT.[Description], 
						TT.[Category1], 
						TT.[Category2]
	)

	INSERT INTO [dbo].[StudyResultSummary](
		[VCC],
		[StudyId],
		[TemplateId],
		[TemplateThemeId],
		[ID],
		[Description],
		[Category1],
		[Category2],
		[Total],
		[Duration],
		[Silence],
		[OtherDuration],
		[OtherSilence],
		[Confidence],
		[CreatedDate],
		[CreatedByUserId],
		[LastModifiedDate],
		[LastModifiedByUserId],
		[TimeStamp],
		[IsMatched]
	)
		SELECT @VCC,@STUDY_ID,
				[TemplateId],
				[TemplateThemeId],
				CONVERT(VARCHAR(50), NEWID()),
				[Description],
				[Category1],
				[Category2],
				1,
				NULL,--Duration
				NULL,--Silence
				NULL,--OtherDuration
				NULL,--OtherSilence,
				NULL,--Confidence
				GETUTCDATE(),
				'system',
				GETUTCDATE(),
				'system',
				GETUTCDATE(),
				0 --0 POR DEFECTO PARA ACTUALIZAR MAS ADELANTE A 1 EN EL BULK LOS QUE HICIERON MATCH
			FROM CTE_SRS_ASYNC

/**********************  INSERTAR EN  [StudyDetail]  PARA INERACCIONES DE TIPO CALL *********************/
	
	;WITH CTE_SD_CALL AS (
	     SELECT S.[TemplateId],
				TTI.[TemplateThemeId]					'TemplateThemeId',
				TTI.[ID]								'TemplateThemeItemId',
				CONCAT(
					PA.[AudioId],
					'_',
					ISNULL(PA.[Section], '0'),
					'_',
					ISNULL(PA.[Part], '0'))				'TranscriptionId'
			FROM  [dbo].[Study] S (NOLOCK) 
			JOIN [dbo].[TemplateThemeItem] TTI (NOLOCK) 
				ON (
						S.[VCC] = TTI.[VCC] 
						AND S.[TemplateId] = TTI.[TemplateId]
					)
			JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
				ON (
					PA.[VCC] = S.[VCC] 
					AND PA.[ProcessId] = S.[ProcessId]
				)
			JOIN [dbo].[Audio] A (NOLOCK) 
				 ON (
					A.[VCC] = S.[VCC] 
					AND A.[ID] = PA.[AudioId]
				)
			LEFT JOIN [dbo].[StudyDetail] SD (NOLOCK)
				 ON (
					S.[VCC] = SD.[VCC]
					AND S.[Name] = SD.[StudyId]
					AND TTI.[TemplateThemeId] = SD.[TemplateThemeId]
					AND TTI.[ID] = SD.[TemplateThemeItemId]
					AND CONCAT(PA.[AudioId],'_',ISNULL(PA.[Section], '0'),'_',ISNULL(PA.[Part], '0')) = SD.TranscriptionId
                )
			WHERE S.[VCC] = @VCC 
				AND S.[Name] = @STUDY_ID 
				AND A.[Channel] ='CALL' 
				AND SD.TemplateThemeId IS NULL
			GROUP BY S.[TemplateId],
						TTI.[TemplateThemeId],
						TTI.[ID],
						PA.[AudioId],
						PA.[Section],
						PA.[Part]
    )
	
		INSERT INTO [dbo].[StudyDetail] (
			[VCC]
			,[StudyId]
			,[TemplateThemeId]
			,[TemplateThemeItemId]
			,[TranscriptionId]
			,[Total]
			,[Duration]
			,[Silence]
			,[OtherDuration]
			,[OtherSilence]
			,[Confidence]
			,[CreatedDate]
			,[CreatedByUserId]
			,[LastModifiedDate]
			,[LastModifiedByUserId]
			,[TimeStamp]
			,[IsMatched]
		) 
			SELECT @VCC,
					 @STUDY_ID,
					 [TemplateThemeId],
					 [TemplateThemeItemId],
					 [TranscriptionId],
					 1,--Total
					 NULL,--Duration
					 NULL,--Silence
					 NULL,--OtherDuration
					 NULL,--OtherSilence,
					 NULL,--Confidence
					 GETUTCDATE(),
					'system',
					 GETUTCDATE(),
					'system',
					 GETUTCDATE(),
					 0 --IsMatched
				FROM CTE_SD_CALL




	/**********************  INSERTAR EN  [StudyDetail]  PARA INERACCIONES DE ASINCRONAS *********************/
	


	;WITH CTE_SD_ASYNC AS (
		SELECT S.[TemplateId],
				 TTI.[TemplateThemeId]			'TemplateThemeId',
				 TTI.[ID]						'TemplateThemeItemId',
				 CONCAT(
					PAI.[InteractionType], 
					'_', 
					PAI.[Campaign], 
					'_', 
					PAI.[FormattedEndDate], 
					'_', 
					REPLACE(PAI.[InteractionId], ':', '_')) 'TranscriptionId'
		FROM  [dbo].[Study] S (NOLOCK) 
		JOIN [dbo].[TemplateThemeItem] TTI (NOLOCK) 
			ON (
				S.[VCC] = TTI.[VCC] 
				AND S.[TemplateId] = TTI.[TemplateId]
			)
		JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK) 
			ON (
				PAI.[VCC] = S.[VCC] 
				AND PAI.[ProcessId] = S.[ProcessId]
			)
		LEFT JOIN [dbo].[StudyDetail] SD (NOLOCK)
			ON (
				S.[VCC] = SD.[VCC] 
				AND S.[Name] = SD.[StudyId] 
				AND TTI.[TemplateThemeId] = SD.[TemplateThemeId] 
				AND TTI.[ID] = SD.[TemplateThemeItemId] 
				AND CONCAT(PAI.[InteractionType], '_', PAI.[Campaign], '_', PAI.[FormattedEndDate], '_', REPLACE(PAI.[InteractionId], ':', '_')) = SD.TranscriptionId
			)
		WHERE S.[VCC] = @VCC 
			AND S.[Name] = @STUDY_ID 
			AND SD.TemplateThemeId IS NULL
		GROUP BY S.[TemplateId],
					TTI.[TemplateThemeId],
					TTI.[ID],
					PAI.[InteractionType],
					PAI.[Campaign],
					PAI.[FormattedEndDate],
					PAI.[InteractionId]
	)
	
	INSERT INTO [dbo].[StudyDetail] (
		[VCC]
		,[StudyId]
		,[TemplateThemeId]
		,[TemplateThemeItemId]
		,[TranscriptionId]
		,[Total]
		,[Duration]
		,[Silence]
		,[OtherDuration]
		,[OtherSilence]
		,[Confidence]
		,[CreatedDate]
		,[CreatedByUserId]
		,[LastModifiedDate]
		,[LastModifiedByUserId]
		,[TimeStamp]
		,[IsMatched]
		) 
		SELECT @VCC,
				 @STUDY_ID,
				 [TemplateThemeId],
				 [TemplateThemeItemId],
				 [TranscriptionId],
				 1,--Total
				 NULL,--Duration
				 NULL,--Silence
				 NULL,--OtherDuration
				 NULL,--OtherSilence,
				 NULL,--Confidence
				 GETUTCDATE(),
				'system',
				 GETUTCDATE(),
				'system',
				 GETUTCDATE(),
				 0 --IsMatched
			FROM CTE_SD_ASYNC


	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

-- [dbo].[SaveStudyStatus] 'inconcert', 'procesoallegro33', 'test_custom', 'Estudio_Prueba', 0, 'D234FD3D-6F5E-499C-942E-1AD5B92EE368', ''
CREATE PROCEDURE [dbo].[SaveStudyStatus]
(
	@VCC					VARCHAR(50),
	@ProcessId				VARCHAR(50),
	@TemplateId				VARCHAR(50),
	@StudyId				VARCHAR(200),
	@Status					FLOAT,
	@LastQueryId			VARCHAR(50),
	@ErrorMessage			VARCHAR(800),
	@LastModifiedDate		DATETIME,
	@LastModifiedByUserId	VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [dbo].[Study] SET
			[Status] = @Status,
			[LastQueryId] = @LastQueryId,
			[ErrorMessage] = CASE WHEN @ErrorMessage = '' THEN NULL ELSE @ErrorMessage END,
			[LastModifiedDate] = @LastModifiedDate,
			[LastModifiedByUserId] = @LastModifiedByUserId,
			[TimeStamp] = GETUTCDATE()
		WHERE
			[VCC] = @VCC
			AND [ProcessId] = @ProcessId
			AND [TemplateId] = @TemplateId
			AND [Name] = @StudyId
	
	-- DECLARAMOS UNA VARIABLE PARA IDENTIFICAR SI EL ESTUDIO PERTENECE A UNA AGENDA
	DECLARE @SCHEDULE_ID VARCHAR(50) = NULL

	-- OBTENEMOS EL ID DE AGENDA DEL ESTUDIO
	SELECT @SCHEDULE_ID = S.[BatchScheduleId]
		FROM [dbo].[Study] S (NOLOCK)
		WHERE S.[VCC] = @VCC 
			AND S.[Name] = @StudyId

	-- SI EL ESTUDIO PERTENECE A UNA AGENDA ACTUALIZAMOS
	IF @SCHEDULE_ID IS NOT NULL
	BEGIN
		-- ACTUALIZAMOS LA EJECUCION
		UPDATE [dbo].[BatchScheduleExecution]
			SET [Status] = 'BATCH_STARTED'
			WHERE [VCC] = @VCC
				AND [BatchScheduleId] = @SCHEDULE_ID
				AND [BatchId] = @StudyId
	END


	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveSubGroup] (
	@Instance Varchar(50) = NULL, 				@ID Varchar(50) = NULL,						@Description Varchar(50) = NULL,
	@CreatedDate Datetime = NULL,				@CreatedByUserId Varchar(50) = NULL,			@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL,	@Enabled bit = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [SubGroup]
	SET [Description] = @Description,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[Enabled] = CASE WHEN @Enabled IS NULL THEN 1 ELSE @Enabled END,
		[TimeStamp] = GETUTCDATE()
	WHERE [Instance] = @Instance
    AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [SubGroup] (
			[Instance], [ID],
			[Description], [CreatedDate],
			[CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId],
			[Enabled], [TimeStamp]
			)
		VALUES (
			@Instance, @ID,
			@Description, @CreatedDate,
			@CreatedByUserId, @LastModifiedDate, @LastModifiedByUserId,
			1, GETUTCDATE()
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveTag]
(
	@VCC						VARCHAR(50),
	@AudioId					VARCHAR(50),
	@Section					NUMERIC(5, 0),
	@Part						NUMERIC(5, 0),
	@Tag						VARCHAR(50),
	@CurrentTime				FLOAT,
	@Comment					VARCHAR(200),
	@CreatedDate				DATETIME,
	@CreatedByUserId			VARCHAR(50),
	@LastModifiedDate			DATETIME,
	@LastModifiedByUserId		VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [dbo].[Tag] SET
		[Comment] = @Comment,
		[LastModifiedDate] = @LastModifiedDate,
		[LastModifiedByUserId] = @LastModifiedByUserId,
		[TimeStamp] = GETUTCDATE()
	WHERE
		[VCC] = @VCC AND
		[AudioId] = @AudioId AND
		[Section] = @Section AND
		[Part] = @Part AND
		[Tag] = @Tag AND
		[CurrentTime] = STR(@CurrentTime, 18, 18)

	IF @@ROWCOUNT = 0
	BEGIN
		BEGIN
			IF EXISTS (
			SELECT * FROM UniqueTag (NOLOCK)
			WHERE [VCC] = @VCC
			AND [TagName] = @Tag
			)
			BEGIN
				SELECT 1 AS 'result'
			END
			ELSE
			BEGIN
				INSERT INTO [dbo].[UniqueTag](
					[VCC],
					[TagName],
					[CreatedDate],
					[CreatedByUserId],
					[TimeStamp]
				)
				VALUES(
					@VCC,
					@Tag,
					@CreatedDate,
					@CreatedByUserId,
					GETUTCDATE()
				)
			END
		END
		BEGIN
			INSERT INTO [dbo].[Tag](
				[VCC],
				[AudioId],
				[Section],
				[Part],
				[Tag],
				[CurrentTime],
				[Comment],
				[CreatedDate],
				[CreatedByUserId],
				[TimeStamp]
			)
			VALUES(
				@VCC,
				@AudioId,
				@Section,
				@Part,
				@Tag,
				STR(@CurrentTime, 18, 18),
				@Comment,
				@CreatedDate,
				@CreatedByUserId,
				GETUTCDATE()
			)
		END
	END

	 SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveTemplate] (
	@VCC Varchar(50) = NULL, 					@Name Varchar(50) = NULL,					@Type Varchar(50) = NULL,
	@Description Varchar(50) = NULL,			@Template Varchar(50) = NULL,
	@Category Varchar(50) = NULL,				@CreatedDate Datetime = NULL,				@CreatedByUserId Varchar(50) = NULL,		
	@LastModifiedDate Datetime = NULL,			@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [Template]
	SET [Description] = @Description,
		[Category] = @Category,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
    AND [Name] = @Name

    IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [Template] (
			[VCC], [Name], [Type], [Description], [Template], [Category],
			[CreatedDate], [CreatedByUserId], [LastModifiedDate], 
			[LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
			@VCC, @Name, @Type, @Description, @Template, @Category,
			@CreatedDate, @CreatedByUserId, @LastModifiedDate, 
			@LastModifiedByUserId, GETUTCDATE()
		)
	END
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveTemplateTheme] (
	@VCC Varchar(50) = NULL,
	@TemplateId Varchar(50) = NULL,
	@ID Varchar(200) = NULL,
	@Order Int = NULL,
	@Description Varchar(50) = NULL,
	@Category1 Varchar(50) = NULL,
	@Category2 Varchar(50) = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL, 
	@AnalysisType VARCHAR(15) = NULL, 
	@IsNegative BIT = 0
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [TemplateTheme]
		SET [Order] = @Order,
			[Description] = @Description,
			[Category1] = @Category1,
			[Category2] = @Category2,
			[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
			[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
			[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[TimeStamp] = GETUTCDATE(),[AnalysisType] = @AnalysisType, [IsNegative] = @IsNegative
		WHERE [VCC] = @VCC
			AND [TemplateId] = @TemplateId
			AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [TemplateTheme] (
			[VCC], [TemplateId], [ID], [Order],
			[Description], [Category1], [Category2],
			[CreatedDate], [CreatedByUserId], [LastModifiedDate], 
			[LastModifiedByUserId], [TimeStamp],[AnalysisType],[IsNegative]
			)
		VALUES (
			@VCC, @TemplateId, @ID, @Order,
			@Description, @Category1, @Category2,
			@CreatedDate, @CreatedByUserId, @LastModifiedDate, 
			@LastModifiedByUserId, GETUTCDATE(),@AnalysisType,@IsNegative
		)

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveTemplateThemeItem] (
	@VCC Varchar(50) = NULL,
	@TemplateId Varchar(50) = NULL,
	@TemplateThemeId Varchar(200) = NULL,
	@ID Varchar(50) = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL,
	@AcceptancePercentage DECIMAL(5,2) = 0
)
AS
BEGIN
	SET NOCOUNT ON

    IF @@ROWCOUNT = 0
		INSERT INTO [TemplateThemeItem] (
			[VCC], [TemplateId], [TemplateThemeId], [ID],
			[CreatedDate], [CreatedByUserId], [LastModifiedDate], 
			[LastModifiedByUserId],[AcceptancePercentage], [TimeStamp]
			)
		VALUES (
			@VCC, @TemplateId, @TemplateThemeId, @ID,
			@CreatedDate, @CreatedByUserId, @LastModifiedDate, 
			@LastModifiedByUserId,CAST(@AcceptancePercentage AS DECIMAL(5,2)), GETUTCDATE()
		)
	
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveTemplateThemeItemEntity] (
	@VCC							VARCHAR(50) = NULL,
	@TemplateId						VARCHAR(50) = NULL,
	@TemplateThemeId				VARCHAR(50) = NULL,
	@TemplateThemeItemId			VARCHAR(50) = NULL,
	@EntityId						VARCHAR(50) = NULL,
	@EntityName						VARCHAR(50) = NULL,
	@AcceptancePercentage			DECIMAL(18, 2) = NULL,
	@Order							INT = NULL,
	@CreatedByUserId				VARCHAR(50) = NULL,
	@LastModifiedByUserId			VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	UPDATE [TemplateThemeItemEntity]
		SET [AcceptancePercentage] = @AcceptancePercentage,
			[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
			[LastModifiedDate] = GETUTCDATE(),
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [TemplateId] = @TemplateId
			AND [TemplateThemeId] = @TemplateThemeId
			AND [TemplateThemeItemId] = @TemplateThemeItemId
			AND [EntityId] = @EntityId

	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [TemplateThemeItemEntity] (
			[VCC], [TemplateId], [TemplateThemeId], [TemplateThemeItemId], [EntityId], 
			[EntityName], [AcceptancePercentage], [Order], [CreatedDate], [CreatedByUserId], 
			[LastModifiedDate], [LastModifiedByUserId], [TimeStamp]
		)
		VALUES (
			@VCC, @TemplateId, @TemplateThemeId, @TemplateThemeItemId, @EntityId, 
			@EntityName, @AcceptancePercentage, @Order, GETUTCDATE(), @CreatedByUserId, 
			GETUTCDATE(), @LastModifiedByUserId, GETUTCDATE()
		)
	END
	
	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveTenantVCC] (
	@VCC Varchar(50) = NULL, 				
	@TenantId Varchar(50) = NULL,				
	@ClientId Varchar(50) = NULL,			
	@ClientSecret Varchar(50) = NULL,
	@Resource varchar(50) = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedByUserId Varchar(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EL REGISTRO CORRESPONDIENTE AL VCC
	UPDATE [TenantVCC]
		SET [TenantId] = @TenantId,
			[ClientId] = @ClientId,
			[ClientSecret] = @ClientSecret,
			[Resource] = @Resource,
			[LastModifiedByUserId] = @LastModifiedByUserId,
			[LastModifiedDate] = CURRENT_TIMESTAMP,
			[TimeStamp] = CURRENT_TIMESTAMP
		WHERE [VCC] = @VCC

	-- SI NO SE PUDO ACTUALIZAR REGISTRAMOS EL NUEVO REGISTRO
    IF @@ROWCOUNT = 0
		INSERT INTO [TenantVCC] ([VCC], [TenantId], [ClientId], [ClientSecret], [Resource], [CreatedDate], [CreatedByUserId], [TimeStamp])
		VALUES (@VCC, @TenantId, @ClientId, @ClientSecret, @Resource, CURRENT_TIMESTAMP, @CreatedByUserId, CURRENT_TIMESTAMP)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveTipification] (
	@VCC VARCHAR(50) = NULL, 				
	@ID VARCHAR(50) = NULL,
	@Description VARCHAR(250) = NULL,					
	@Detail VARCHAR(500) = NULL,
	@UserId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EN CASO EXISTA
	UPDATE [Tipification]
		SET [Description] = @Description,
			[Detail] = @Detail,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedByUserId] = @UserId
		WHERE [VCC] = @VCC
			AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [Tipification] (
			[VCC], 
			[ID],
			[Description], 
			[Detail], 
			[CreatedDate],
			[CreatedByUserId]
			)
		VALUES (
			@VCC, 
			@ID,
			@Description, 
			@Detail, 
			GETUTCDATE(),
			@UserId
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveTipificationItem] (
	@VCC VARCHAR(50) = NULL, 				
	@ID VARCHAR(50) = NULL,
	@TipificationId VARCHAR(50) = NULL,
	@SecondLevelItemId VARCHAR(50) = NULL,
	@Description VARCHAR(250) = NULL,					
	@Order INT = NULL,
	@UserId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EN CASO EXISTA
	UPDATE [TipificationItem]
		SET [Description] = @Description,
			[Order] = @Order,
			[LastModifiedDate] = GETUTCDATE(),
			[LastModifiedByUserId] = @UserId
		WHERE [VCC] = @VCC
			AND [ID] = @ID

    IF @@ROWCOUNT = 0
		INSERT INTO [TipificationItem] (
			[VCC], 
			[TipificationId], 
			[ID],
			[SecondLevelItemId],
			[Description], 
			[Order], 
			[CreatedDate],
			[CreatedByUserId]
			)
		VALUES (
			@VCC, 
			@TipificationId,
			@ID,
			@SecondLevelItemId,
			@Description, 
			@Order,
			GETUTCDATE(),
			@UserId
		)

    SELECT 1 AS 'result'

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveTipificationQuery] (
	@VCC							VARCHAR(50) = NULL,
	@TipificationId					VARCHAR(50) = NULL,
	@QueryId						VARCHAR(50) = NULL,
	@QueryContent					VARCHAR(1000) = NULL,
	@AcceptancePercentage			DECIMAL(18, 2) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- ACTUALIZAMOS EL REGISTRO EN CASO YA EXISTA
	UPDATE [TipificationQuery]
		SET [QueryId] = @QueryId,
			[QueryContent] = @QueryContent,
			[AcceptancePercentage] = @AcceptancePercentage,
			[TimeStamp] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [TipificationId] = @TipificationId

	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [TipificationQuery] (
			[VCC], [TipificationId], [QueryId], 
			[QueryContent], [AcceptancePercentage], [TimeStamp]
		)
		VALUES (
			@VCC, @TipificationId, @QueryId, 
			@QueryContent, @AcceptancePercentage, GETUTCDATE()
		)
	END

	SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [SaveTranscription] 'default', 'F040EA5A-E455-49EC-8A8D-4A4AD13D8786', 'FFB04B5FF6E242098CD31F419CC4E4A3', 0, 0, 2, 'admin'
CREATE PROCEDURE [dbo].[SaveTranscription]
(
	@VCC VARCHAR(50) = NULL,
	@ProcessId VARCHAR(50) = NULL,
	@InteractionType VARCHAR(50) = NULL,
	@Section NUMERIC(5, 0),
	@Part NUMERIC(5, 0),
	@Campaign VARCHAR(100) = NULL,
	@Folder VARCHAR(50) = NULL,
	@ID VARCHAR(50) = NULL,
	@Value INT,
	@UserId VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON

	-- SI EL INTERACTIONTYPE ES NULL SETTEAMOS CALL POR DEFECTO
	IF @InteractionType IS NULL SET @InteractionType = 'CALL'

	UPDATE [dbo].[Transcription] SET
	[Value] = @Value,
	[LastModifiedDate] = GETUTCDATE(),
	[LastModifiedByUserId] = @UserId
	WHERE
		[VCC] = @VCC 
		AND [ProcessId] = @ProcessId 
		AND [InteractionType] = @InteractionType
		AND (
			(
				[InteractionType] = 'CALL'
					AND [Section] = @Section 
					AND [Part] = @Part 
			) OR (
				[InteractionType] <> 'CALL'
					AND [Folder] = @Folder
					AND [Campaign] = @Campaign
			)
		)
		AND [ID] = @ID

	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [dbo].[Transcription](
			[VCC],
			[ProcessId],
			[InteractionType],
			[Section],
			[Part],
			[Campaign],
			[Folder],
			[ID],
			[Value],
			[CreatedByUserId]
		)
		VALUES(
			@VCC,
			@ProcessId,
			@InteractionType,
			@Section,
			@Part,
			@Campaign,
			@Folder,
			@ID,
			@Value,
			@UserId
		)
	END

	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[SaveUtterance] 'allegro', '4bad9174-ea84-45ee-ab2f-b17c73c43734', '45799dd8-17e7-49a1-9bd8-cb74f00a788c', '{f2c9e160-8eeb-11e9-8562-b65bb2fb6bcb}', 0, 1, 'in', 10, 14, 1.23, 1.38, 0, 'hola', '2019-08-02 15:30:00.000', 'analytic', NULL, NULL
CREATE PROCEDURE [dbo].[SaveUnderlined]
(
	@VCC						VARCHAR(50),
	@AppId						VARCHAR(50),
	@IntentId					VARCHAR(50),
	@AudioId					VARCHAR(50),
	@Section					NUMERIC(5, 0),
	@Part						NUMERIC(5, 0),
	@Speaker					VARCHAR(3),
	@AnchorOffset				INT,
	@FocusOffset				INT,
	@CurrentTime				FLOAT,
	@EndCurrentTime				FLOAT,
	@UtteranceId				VARCHAR(50),
	@TextContent				VARCHAR(500),
	@CreatedDate				DATETIME,
	@CreatedByUserId			VARCHAR(50),
	@LastModifiedDate			DATETIME,
	@LastModifiedByUserId		VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	-- EN CASO LLEGUE COMO NULL LO IGUALAMOS A 0
	IF @Section IS NULL BEGIN SET @Section = 0 END 

	UPDATE [dbo].Underlined SET
		[VCC] = @VCC,
		[AudioId] = @AudioId,
		[Section] = @Section,
		[Part] = @Part,
		[Speaker] = @Speaker,
		[AnchorOffset] = @AnchorOffset,
		[FocusOffset] = @FocusOffset,
		[CurrentTime] = @CurrentTime,
		[EndCurrentTime] = @EndCurrentTime,
		[UtteranceId] = @UtteranceId,
		[CreatedDate] = @CreatedDate,
		[CreatedByUserId] = @CreatedByUserId,
		[LastModifiedDate] = @LastModifiedDate,
		[LastModifiedByUserId] = @LastModifiedByUserId,
		[TimeStamp] = GETUTCDATE()
	WHERE
		[VCC] = @VCC
	    AND [UtteranceId] = @UtteranceId
		AND [AudioId] = @AudioId
		AND [Section] = @Section
		AND [Part] = @Part

	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [dbo].[Underlined] (
			[VCC],
			[AudioId],
			[Section],
			[Part],
			[Speaker],
			[AnchorOffset],
			[FocusOffset],
			[CurrentTime],
			[EndCurrentTime],
			[UtteranceId],
			[CreatedDate],
			[CreatedByUserId],
			[LastModifiedDate],
			[LastModifiedByUserId],
			[TimeStamp]
		)
		VALUES(
			@VCC,
			@AudioId,
			@Section,
			@Part,
			@Speaker,
			@AnchorOffset,
			@FocusOffset,
			@CurrentTime,
			@EndCurrentTime,
			@UtteranceId,
			@CreatedDate,
			@CreatedByUserId,
			@LastModifiedDate,
			@LastModifiedByUserId,
			GETUTCDATE()
		)
	END

	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveUser] (
	@VCC Varchar(50) = NULL, 					@ID Varchar(50) = NULL, 				@Email Varchar(50) = NULL,
	@Firstname Varchar(50) = NULL, 				@Lastname Varchar(50) = NULL, 			@Password Varchar(50) = NULL,
	@Role Varchar(50) = NULL, 					@Country Varchar(50) = NULL, 			@Language Varchar(50) = NULL,
	@TimeZone Varchar(50) = NULL,				@Preferences Varchar(200) = NULL, 		@IsSystemUser Bit = NULL,
	@IsLocalSupervisor BIT = 0,
	@IsExternalSupervisor BIT = 0,
	@IsExternalAgent BIT =0,				
	@LimitClients Bit = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [User]
	SET [Email] = @Email,
		[Firstname] = @Firstname,
		[Lastname] = @Lastname,
		[Password] = CASE WHEN @Password IS NULL OR @Password = '' THEN [Password] ELSE @Password END,
		[Role] = @Role,
		[Country] = @Country,
		[Language] = @Language,
		[TimeZone] = @TimeZone,
		[Preferences] = @Preferences,
		[IsSystemUser] = @IsSystemUser,
		[IsLocalSupervisor] = @IsLocalSupervisor,
		[IsExternalSupervisor] = @IsExternalSupervisor,
		[IsExternalAgent] = @IsExternalAgent,
		[LimitClients] = @LimitClients,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
	AND [ID] = @ID
    
    IF @@ROWCOUNT = 0
		INSERT INTO [User] (
			[VCC], [ID], [Email],
			[Firstname], [Lastname], [Password],
			[Role], [Country], [Language],
			[TimeZone], [Preferences], [IsSystemUser],
			[IsLocalSupervisor], 
			[IsExternalSupervisor], 
			[IsExternalAgent], 
			[LimitClients], 
			[CreatedDate], 
			[CreatedByUserId],
			[LastModifiedDate], 
			[LastModifiedByUserId],
			[TimeStamp]
			)
		VALUES (
			@VCC, @ID, @Email,
			@Firstname, @Lastname, @Password,
			@Role, @Country, @Language,
			@TimeZone, @Preferences, @IsSystemUser,
			@IsLocalSupervisor,
			@IsExternalSupervisor, 
			@IsExternalAgent, 
			@LimitClients, 
			@CreatedDate, @CreatedByUserId,
			@LastModifiedDate, @LastModifiedByUserId,
			GETUTCDATE()
		)
		
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
-- [dbo].[SaveUserAccessToken] 'default', 'admin', 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpYXQiOjE1MjgzMjQwNDEsIm5iZiI6MTUyODMyNDA0MSwianRpIjoiZDVhY2E2NzAtNjgzMC00MGIzLTkwMDktZGY0NWFkNmY3YzNlIiwiZXhwIjoxNTI4OTI4ODQxLCJpZGVudGl0eSI6ImluX2NvbmNlcnQiLCJmcmVzaCI6ZmFsc2UsInR5cGUiOiJhY2Nlc3MifQ.84IHnM3YimnSHXUvxLR9iUBPrlwFaQ2Fs5AtmBY7td8', 'in_concert', '2018-06-07 15:47:56.363'
CREATE PROCEDURE [dbo].[SaveUserAccessToken]
(
	@VCC						VARCHAR(50),
	@UserName					VARCHAR(50),
	@AccessToken				VARCHAR(2000),
	@Realm						VARCHAR(100),
	@CreatedDate				DATETIME
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [dbo].[UserAccessToken] SET
		[AccessToken] = @AccessToken,
		[CreatedDate] = @CreatedDate
	WHERE [VCC] = @VCC
	AND [UserName] = @UserName

	IF @@ROWCOUNT = 0
	BEGIN
		INSERT INTO [dbo].[UserAccessToken](
			[VCC], [UserName], [AccessToken], [Realm], [CreatedDate]
		)
		VALUES(
			@VCC, @UserName, @AccessToken, @Realm, @CreatedDate
		)
	END

	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveUserClient] (
	@VCC Varchar(50) = NULL, 			@UserId Varchar(50) = NULL, 		@ClientId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [UserClient]
	SET [TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
	AND [UserId] = @UserId
	AND [ClientId] = @ClientId

    IF @@ROWCOUNT = 0
		INSERT INTO [UserClient] (
			[VCC], [UserId], [ClientId],
			[TimeStamp]
			)
		VALUES (
			@VCC, @UserId, @ClientId,
			GETUTCDATE()
		)

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveUtterance]
(
	@VCC						VARCHAR(50),
	@AppId						VARCHAR(50),
	@IntentId					VARCHAR(50),
	@IntentName					VARCHAR(50),
	@Id							VARCHAR(50),
	@Text						VARCHAR(500),
	@UserId			            VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON
	
	DECLARE @INTENT_ID VARCHAR(50) = NULL
	
	-- VERIFICAMOS SI HEMOS RECIBIDO INTENT ID
	IF @IntentId IS NULL OR @IntentId = ''
	BEGIN
		SELECT @INTENT_ID = [IntentId]
			FROM [dbo].[Intent] (NOLOCK) 
			WHERE [VCC] = @VCC
				AND [AppId] = @AppId
				AND [Name] = @IntentName
	END
	ELSE
	BEGIN
		SET @INTENT_ID = @IntentId
	END
  
    IF(NOT EXISTS (
		SELECT [Id] 
			FROM [dbo].[Utterance] (NOLOCK) 
			WHERE VCC = @VCC 
				AND [AppId] = @AppId
				AND [IntentId] = @INTENT_ID
				AND [Id] = @Id 
	)) 
	BEGIN
		INSERT INTO [dbo].[Utterance] (
				[VCC],
				[AppId],
				[IntentId],
				[Id],
				[Text],
				[CreatedByUserId],
				[Status],
				[StatusCode]
			)
			VALUES(
				@VCC,
				@AppId,
				@INTENT_ID,
				@Id,
				LOWER(@Text),
				@UserId,
				1,
				'AVAILABLE'
			)
	END

    SELECT [Id] AS 'utteranceId'
		FROM [dbo].[Utterance] (NOLOCK)
	    WHERE [VCC] = @VCC
			AND [AppId] = @AppId
			AND [IntentId] = @INTENT_ID
			AND [Id] = @Id

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveUtteranceEntity] (
    @VCC				VARCHAR(50) = NULL,
    @UtteranceId		VARCHAR(50) = NULL,
    @EntityName			VARCHAR(50) = NULL,
    @Start				INT = NULL,
    @End				INT = NULL,
    @EndTokenIndex		INT = NULL,
    @StartTokenIndex	INT = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	-- VERIFICAMOS SI EL UTTERANCE ENTITY EXISTE
	IF NOT EXISTS(
		SELECT [EntityName] 
			FROM  [dbo].[UtteranceEntity] (NOLOCK) 
			WHERE [VCC] = @VCC 
				AND [UtteranceId] = @UtteranceId 
				AND [EntityName] = @EntityName 
				AND (
					(
						[Start] = @Start
						AND 
						[End] = @End
					)
					OR
					(
						[StartTokenIndex] = @StartTokenIndex
						AND
						[EndTokenIndex] = @EndTokenIndex
					)
				)
	)
	BEGIN
	    INSERT INTO [dbo].[UtteranceEntity] (
			[VCC], [UtteranceId], [EntityName], 
			[Start], [End], [EndTokenIndex], 
			[StartTokenIndex], [CreatedDate]
		) 
		VALUES (
			@VCC, @UtteranceId, @EntityName, 
			@Start, @End, @EndTokenIndex, 
			@StartTokenIndex, GETUTCDATE()
		)
	END
		
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveVcc] (
	@VCC VARCHAR(50) = NULL, 
	@ID VARCHAR(50) = NULL,
	@Description Varchar(50) = NULL,
	@Type VARCHAR(50) = NULL,
	@TranscriptionDefaultLanguage VARCHAR(25) = NULL,
	@CognitiveEngine VARCHAR(25) = NULL,
	@Country Varchar(50) = NULL,
	@Language Varchar(50) = NULL,
	@TimeZone Varchar(50) = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId   VARCHAR(50) = NULL,
	@DeleteDataDays         INT = 360,
	@ChangedCognitiveEngine BIT = 0,
	@UseRasaApi				BIT = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	DECLARE @IsMigrating BIT = 0
	IF(@ChangedCognitiveEngine IS NOT NULL AND @ChangedCognitiveEngine = 1)
	   SET @IsMigrating = 1
	   
	IF(EXISTS(SELECT [ID] FROM [dbo].[VCC] (NOLOCK) WHERE [VCC] = @VCC))
	BEGIN
			UPDATE [VCC]
				SET [Description] = @Description,
					[Type] = @Type,
					[Country] = @Country,
					[TranscriptionDefaultLanguage] = @TranscriptionDefaultLanguage,
					[CognitiveEngine] = @CognitiveEngine,
					[Language] = @Language,
					[TimeZone] = @TimeZone,
					[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
					[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
					[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
					[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
					[IsMigrating] = IIF(@IsMigrating = 1,1,0),
					[DeleteDataDays] = @DeleteDataDays,
					[TimeStamp] = GETUTCDATE()
				WHERE [VCC] = @VCC
					AND [ID] = @ID
		

		-- insertamos los permisos que faltan 

			DECLARE @RowCount INT,
					@RowCountRoutePermision INT

			DECLARE @ROLE_PERMISSIONS TABLE(
					[RoleId]  VARCHAR(50),
					[GroupId] VARCHAR(200)
				)			 
		
			INSERT INTO @ROLE_PERMISSIONS 
				SELECT 					
				RPT.RoleId, 
				RPT.GroupId
				FROM dbo.RolePermission RPT
				INNER JOIN dbo.RolePermissionsGroup RPG ON RPG.GroupId = RPT.GroupId AND RPG.VCC = RPT.VCC
				WHERE RPT.VCC = 'system'
					AND NOT EXISTS(
									SELECT * 
									FROM dbo.RolePermission RP 
									WHERE RP.GroupId = RPT.GroupId 
										AND RP.VCC = @VCC
											)

			DECLARE @ROUTE_PERMISSIONS TABLE(
				 [Id]      VARCHAR(50),
				 [Route]   VARCHAR(250),
				 [Level]   VARCHAR(50),
				 [GroupId] VARCHAR(200)
				)

			INSERT INTO @ROUTE_PERMISSIONS
						SELECT 
						NEWID(),  
						RPT.Route, 
						RPT.Level,
						RPG.GroupId
						FROM dbo.RoutePermission RPT
						INNER JOIN dbo.RolePermissionsGroup RPG ON RPG.GroupId = RPT.GroupId AND RPG.VCC = RPT.VCC
						WHERE RPT.VCC = 'system'
						AND NOT EXISTS(
										SELECT * 
										FROM dbo.RoutePermission RP 
										WHERE RP.GroupId = RPT.GroupId 
											AND RP.VCC = @VCC
										)


			SELECT @RowCount = COUNT(*) FROM @ROLE_PERMISSIONS

			IF @RowCount > 0
			BEGIN

			INSERT INTO [dbo].[RolePermission](
						[VCC],
						[RoleId],
						[GroupId],
						[TimeStamp]
						)
						SELECT	
						@VCC, 
						[RoleId],
						[GroupId],
						GETUTCDATE()
						FROM @ROLE_PERMISSIONS
			END		
		 
			SELECT @RowCountRoutePermision = COUNT(*)  FROM @ROUTE_PERMISSIONS

			IF @RowCountRoutePermision > 0
			BEGIN
				INSERT INTO dbo.RoutePermission (
							VCC,
							ID,
							GroupId,
							Route,
							Level,
							TimeStamp
							)
							SELECT
							@VCC,
							[Id],
							[GroupId],
							[Route],  
							[Level],  
							GETUTCDATE()
							FROM @ROUTE_PERMISSIONS						
			END
   END
   ELSE
   BEGIN
   			   EXEC [dbo].[CreateVcc] @VCC,
									  @ID, 
									  @Description, 
									  @Type, 
									  @TranscriptionDefaultLanguage,
									  @CognitiveEngine, 
									  @Country, 
									  @Language, 
									  @TimeZone, 
									  @CreatedDate,  
									  @CreatedByUserId, 
									  @LastModifiedDate, 
									  @LastModifiedByUserId, 
									  @DeleteDataDays,
									  @UseRasaApi
   END

		
		
	--SI HA CAMBIADO LOS NLUS ACTUALIZAR EL ESTADO DE LOS MODELOS A NO DISPONIBLE
	IF(@IsMigrating = 1)
	BEGIN 
	  EXEC SaveCognitiveEngineMigrateStatus @VCC,'*','*',0,'NOT_AVAILABLE'
	END



    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveVccLicense] (
	@VCC Varchar(50) = NULL, 					@Product Varchar(50) = NULL,				@Status bit = NULL,
	@CreatedDate Datetime = NULL,				@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,			@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	UPDATE [VCCLicense]
	SET [Product] = @Product,
		[Status] = @Status,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END
	WHERE [VCC] = @VCC AND [Product]=@Product

    IF @@ROWCOUNT = 0
		INSERT INTO [VCCLicense] (
			[VCC], [Product], [Status], [CreatedDate],
			[CreatedByUserId], [LastModifiedDate], [LastModifiedByUserId]
			)
		VALUES (
			@VCC, @Product, @Status, @CreatedDate,
			@CreatedByUserId, @LastModifiedDate, @LastModifiedByUserId
		)
		
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveVccPermissions] (
	@VCC VARCHAR(50) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	-- DECLARAMOS UNA VARIABLE PARA EL VCC BASE
	DECLARE @VCC_SYSTEM VARCHAR(50) = 'system'
	
	-- ELIMINAMOS LOS PERMISOS ACTUALES
	DELETE FROM [dbo].[RolePermission] WHERE [VCC] = @VCC
	DELETE FROM [dbo].[RolePermissionsGroup] WHERE [VCC] = @VCC
	DELETE FROM [dbo].[RoutePermission] WHERE [VCC] = @VCC

	-- DECLARAMOS VARIABLES AUXILIARES
	DECLARE @CountLicense INT = 0
	DECLARE @License VARCHAR(50)
	
	-- DECLARAMOS TABLAS COMO VARIABLES PARA ACUMULAR LOS REGISTRO A INSERTAR
	DECLARE @ROLE_PERMISSIONS TABLE([ID] VARCHAR(50), [GroupId] VARCHAR(200))
	DECLARE @ROUTE_PERMISSIONS TABLE([ID] VARCHAR(50), [Route] VARCHAR(250), [Level] VARCHAR(50), [GroupId] VARCHAR(200))

	-- CONTAMOS LAS LICENCIAS PARA EL VCC
	SET @CountLicense = (SELECT COUNT(*) FROM [dbo].[VCCLicense] (NOLOCK) WHERE [VCC] = @VCC AND [Status] = 1)
	
	IF @CountLicense > 1
	BEGIN
	    INSERT INTO @ROLE_PERMISSIONS EXEC [dbo].[GetRolePermission] @VCC_SYSTEM, ''
		INSERT INTO @ROUTE_PERMISSIONS EXEC [dbo].[GetRoleRoutePermission] @VCC_SYSTEM
	END

	IF @CountLicense = 1
	BEGIN
	    SET @License = (SELECT Product FROM [dbo].[VCCLicense] (NOLOCK) WHERE [VCC] = @VCC AND [Status] = 1)
	    INSERT INTO @ROLE_PERMISSIONS EXEC [dbo].[GetRolePermission] @VCC_SYSTEM,'', @License
		INSERT INTO @ROUTE_PERMISSIONS EXEC [dbo].[GetRoleRoutePermission] @VCC_SYSTEM,'', @License
	END
	
	-- INSERTAMOS LOS REGISTRO DE ROL
	INSERT INTO [dbo].[RolePermission] (
			    [VCC], 
				[RoleId], 
				[GroupId],
			    [TimeStamp]
	) 
		SELECT @VCC,
				[ID],
				[GroupId],
			    GETUTCDATE()
			FROM @ROLE_PERMISSIONS

	-- INSERTAMOS LOS REGISTROS DE RUTA
	INSERT INTO [dbo].[RoutePermission] (
			[VCC], 
			[ID], 
			[GroupId],
			[Route], 
			[Level], 
			[TimeStamp]
	)
		SELECT @VCC,
				[ID],
				[GroupId],
				[Route],
				[Level],
				GETUTCDATE()
			FROM @ROUTE_PERMISSIONS


    --INSERTAR GRUPOS DE PERMISOS PARA LOS ROLES
	DECLARE @ROLE_PERMISSIONS_GROUP TABLE([Category] VARCHAR(50), [GroupId] VARCHAR(200), [Type] VARCHAR(50))

	INSERT INTO @ROLE_PERMISSIONS_GROUP EXEC [dbo].[GetRolePermissionsGroups] @VCC_SYSTEM

	-- INSERTAMOS LOS GRUPOS DE PERMISOS
	INSERT INTO [dbo].[RolePermissionsGroup] (
			                              [VCC], 
										  [GroupId], 
										  [Category], 
										  [Type], 
										  [TimeStamp]
										  )
		                                 SELECT @VCC, 
												 [GroupId],
												 [Category],
												 [Type], 
												 GETUTCDATE()	
											FROM @ROLE_PERMISSIONS_GROUP


    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
Create PROCEDURE [dbo].[SaveVersionRegistration] (
	@InstallationId VARCHAR(100) = NULL,
	@BuildVersion VARCHAR(25) = NULL
) AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LA VERSION DE BUILD DE UNA PLATAFORMA
	SET XACT_ABORT, NOCOUNT ON

	--
	UPDATE [dbo].[Version] 
		SET [IsRegistered] = 1,
			[TimeStamp] = GETUTCDATE()
		WHERE [BuildVersion] = @BuildVersion
			AND [InstallationId] = @InstallationId

	--
	SELECT 1 'result'

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveWord] (
	@VCC Varchar(50) = NULL, 				@NeuroLinguisticProgrammingId Varchar(50) = NULL, 		@Name Varchar(50) = NULL,
	@ProcessId Varchar(50) = NULL,			@Language Varchar(50) = NULL,							@Type Varchar(50) = NULL,				
	@CreatedDate Datetime = NULL,			@CreatedByUserId Varchar(50) = NULL, 					@LastModifiedDate Datetime = NULL, 		
	@LastModifiedByUserId Varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [Word]
	SET [Type] = @Type,
		[CreatedDate] = CASE WHEN @CreatedDate IS NULL THEN [CreatedDate] ELSE @CreatedDate END,
		[CreatedByUserId] = CASE WHEN @CreatedByUserId IS NULL THEN [CreatedByUserId] ELSE @CreatedByUserId END,
		[LastModifiedDate] = CASE WHEN @LastModifiedDate IS NULL THEN [LastModifiedDate] ELSE @LastModifiedDate END,
		[LastModifiedByUserId] = CASE WHEN @LastModifiedByUserId IS NULL THEN [LastModifiedByUserId] ELSE @LastModifiedByUserId END,
		[TimeStamp] = GETUTCDATE()
	WHERE [VCC] = @VCC
	AND [NeuroLinguisticProgrammingId] = @NeuroLinguisticProgrammingId
	AND [Name] = @Name
	AND [ProcessId] = @ProcessId

    IF @@ROWCOUNT = 0
		INSERT INTO [Word] (
			[VCC], [NeuroLinguisticProgrammingId], [Name], [ProcessId], [Language],
			[Type], [CreatedDate], [CreatedByUserId],
			[LastModifiedDate], [LastModifiedByUserId], [TimeStamp]
			)
		VALUES (
			@VCC, @NeuroLinguisticProgrammingId, @Name, @ProcessId, @Language,
			@Type, @CreatedDate, @CreatedByUserId, 
			@LastModifiedDate, @LastModifiedByUserId, GETUTCDATE()
		)

    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveWorkGroupSupervisor] (
	@VCC Varchar(50) = NULL,
	@Description Varchar(50) = NULL,
	@SupervisorId Varchar(250) = NULL,
	@CampaignId Varchar(250) = NULL,
	@AgentId Varchar(250) = NULL,
	@CreatedDate Datetime = NULL,
	@CreatedByUserId Varchar(50) = NULL,
	@LastModifiedDate Datetime = NULL,
	@LastModifiedByUserId Varchar(50) = NULL,
	@IsHistoric BIT = NULL
)
AS
BEGIN
	SET NOCOUNT ON

		INSERT INTO [WorkGroupSupervisor] (
			[VCC], [Description], [SupervisorId], 
			[CampaignId], [AgentId],[CreatedDate], [CreatedByUserId], 
			[LastModifiedDate], [LastModifiedByUserId], [TimeStamp], [IsHistoric]
			)
		VALUES (
			@VCC, @Description, @SupervisorId,
			@CampaignId, @AgentId,@CreatedDate, @CreatedByUserId, 
			@LastModifiedDate, @LastModifiedByUserId, GETUTCDATE(), @IsHistoric
		)
		
	
    SELECT 1 AS 'result'
    
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE Procedure [dbo].[ScanSp] (@NomTb VarChar(255) = '')
As
Select @NomTb = '%'+ ltrim(@NomTb)
Select @NomTb = rtrim(@NomTb) + '%' 
Select SubString(Name, 9, 4) Numero, Name from sysobjects 
         Where type = 'P' and Name Like @NomTb
         Order By SubString(Name, 9, 4)
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET ANSI_NULLS ON 
GO

CREATE Procedure [dbo].[ScanTb] (@NomTb VarChar(255) = '')
As

Select @NomTb = '%'+ ltrim(@NomTb)
Select @NomTb = rtrim(@NomTb) + '%' 
Select SubString(Name, 9, 4) Numero, Name from sysobjects 
         Where type = 'U' and Name Like @NomTb
         Order By 
 SubString(Name, 9, 4)
GO
SET ANSI_NULLS OFF 
GO

GO

SET ANSI_NULLS ON 
GO

CREATE Procedure [dbo].[ScanTxt] @ProcName VarChar(255) = '%' , @Texto VarChar(255) = '%'

AS
select * 
  from sysobjects o
 where exists (select * 
                 from syscomments c
		where o.id = c.id 
	          and Text Like '%' + @Texto + '%')                                                                                                                                                                                                                    
   and name like   '%' + @ProcName + '%'
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[UpdateEntityId]
(
	@VCC	    VARCHAR(50),
	@AppId      VARCHAR(50),
	@EntityID   VARCHAR(50),
	@NewEntityId VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON
	
	--Actualizar los registros con el nuevo AppId
	
	UPDATE [dbo].[Entity] SET [ID] = @NewEntityId WHERE [VCC] = @VCC AND [AppId] = @AppId AND [ID] = @EntityID

	SELECT 1 AS result

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[UpdateIntentId]
(
	@VCC	     VARCHAR(50),
	@AppId       VARCHAR(50),
	@IntentId    VARCHAR(50),
	@NewIntentId VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON
	
	--Actualizar los registros con el nuevo IntentId
	

	UPDATE [dbo].[Intent] SET [IntentId] = @NewIntentId WHERE [VCC] = @VCC AND [AppId] = @AppId AND [IntentId] = @IntentId

	UPDATE [dbo].[Utterance] 
		SET [IntentId] = @NewIntentId 
		WHERE [VCC] = @VCC 
			AND [AppId] = @AppId 
			AND [IntentId] = @IntentId

	SELECT 1 AS result

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[UpdateNluAppId]
(
	@VCC	    VARCHAR(50),
	@AppId      VARCHAR(50),
	@NewAppId   VARCHAR(50),
	@Culture    VARCHAR(10) = '',
	@CognitiveEngine  VARCHAR(50) = '',
    @UseRasaApi BIT = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	
	--ACTUALIZAR LOS REGISTROS CON EL NUEVO APPID
	IF (@CognitiveEngine = 'NEW_RASA_ENGINE' AND @UseRasaApi = 1)
	SET @CognitiveEngine  = 'RASAAPI'
	
    IF (@CognitiveEngine = 'NEW_RASA_ENGINE' AND @UseRasaApi = 0)
	SET @CognitiveEngine  = 'RASA'

	--CONSULTAR SI AL MENOS TIENE UNA INTENCION Y FRASES PARA ENVIAR A ENTRENAR
	DECLARE @MinUtteranceNumber INT
	SELECT  @MinUtteranceNumber = COUNT(*) FROM dbo.[Utterance] U
	INNER JOIN dbo.[Intent] I ON 
	U.IntentId = I.IntentId
	AND U.VCC =I.VCC
	WHERE U.VCC = @VCC 
	AND U.AppId = @AppId
	
	UPDATE [dbo].[NLU] 
	SET [AppId] = @NewAppId,
	[Culture] = @Culture,
	[LastPublishedDate] = NULL,
	[LastTrainedDate] = NULL,
	[LastModifiedDate] = GETUTCDATE(),
	[IsPublished] = 0,
	[IsTrained] = 0,
	[NeedTrain] = IIF(@MinUtteranceNumber >= 1, 1, 0),
	[NeedPublish] = 0,
	[TrainVersion] = null,
	[TrainId]= null,
	[CognitiveEngine] = @CognitiveEngine
	WHERE [VCC] = @VCC AND [AppId] = @AppId

	UPDATE [dbo].[Intent] SET [AppId] = @NewAppId WHERE [VCC] = @VCC AND [AppId] = @AppId

	UPDATE [dbo].[Utterance] SET [AppId] = @NewAppId WHERE [VCC] = @VCC AND [AppId] = @AppId
	
	UPDATE [dbo].[Entity] SET [AppId] = @NewAppId WHERE [VCC] = @VCC AND [AppId] = @AppId

	SELECT 1 AS result

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[UpdateNLUStatus]
(
    @VCC					VARCHAR(50) = NULL,
    @AppId					VARCHAR(50) = NULL,
    @NewStatus              INT = NULL
)
AS
BEGIN
    SET NOCOUNT ON

    UPDATE [dbo].[NLU] SET [Status] = @NewStatus WHERE [VCC] = @VCC AND [AppId] = @AppId
    SELECT 1 AS result

    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[UpdateStatusEvaluationPlan]
(
	@VCC	    VARCHAR(50),
	@IdEvaluationPlan      VARCHAR(50),
	@Status   VARCHAR(50),
	@StatusDescription VARCHAR(50) = null
)
AS
BEGIN
	SET NOCOUNT ON
	
	--Actualizar los registros con el nuevo status
	
	UPDATE [dbo].[EvaluationPlan] 
	SET [status] = @Status, 
		[StatusDescription] = IsNull(@StatusDescription, StatusDescription)
	WHERE [Instance] = @VCC 
	AND [ID] = @IdEvaluationPlan
	
	SELECT 1 AS result

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[UpdateUtteranceId]
(
	@VCC	        VARCHAR(50),
	@AppId          VARCHAR(50),
	@IntentId       VARCHAR(50),
	@UtteranceId    VARCHAR(50),
	@NewUtteranceId VARCHAR(50)
)
AS
BEGIN
	SET NOCOUNT ON

	--Actualizar los registros con el nuevo UtteranceId
	UPDATE [dbo].[Utterance] SET [Id] = @NewUtteranceId , [Status] = 1, [StatusCode] = 'AVAILABLE'
	WHERE [VCC] = @VCC AND [AppId] = @AppId AND [IntentId] = @IntentId AND [Id] = @UtteranceId

	UPDATE [dbo].[UtteranceEntity] SET [UtteranceId] = @NewUtteranceId WHERE [VCC] = @VCC AND [UtteranceId] = @UtteranceId

	UPDATE [dbo].[Underlined] SET [UtteranceId] = @NewUtteranceId WHERE [VCC] = @VCC AND [UtteranceId] = @UtteranceId

	SELECT 1 AS result

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[UpdateVccStatus]
(
	@VCC	          VARCHAR(50),
	@IsMigrating      BIT = 0
)
AS
BEGIN
	SET NOCOUNT ON

	UPDATE [dbo].[VCC] SET [IsMigrating] = @IsMigrating WHERE [VCC] = @VCC
	SELECT 1 AS result

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[ValidateUserLogin] (
	@VCC varchar(50) = NULL, @UserId varchar(50) = NULL, @Password varchar(50) = NULL
	)
AS
BEGIN
	SET NOCOUNT ON
	
	IF EXISTS (
		SELECT [ID]
		FROM [User] (NOLOCK) 
		WHERE [VCC] = @VCC
		AND [ID] = @UserId
		AND [Password] = @Password
	)
		SELECT 1 AS 'result'
	ELSE
		SELECT 0 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
